package Task;

use strict;
use warnings;
use IO::Socket qw(SHUT_WR);
use JSON;
use Xormon;
use Data::Dumper;

# Task class serving as a base class for different types of tasks
# Attributes:
#   data   - A hash reference containing task-specific data.
#   client - The client object, used for sending responses.

sub new {
    my ($class, %args) = @_;
    my $self = {
        data => $args{data},
        client => $args{client},
    };
    bless $self, $class;
    return $self;
}


sub execute {
    my ($self) = @_;
    # Abstract method - must be implemented by subclasses
    die "Method 'execute' must be implemented in subclass " . ref($self);    
}

sub log {
   my ($self, $message) = @_;   
   Xormon::log($message);
}

sub shutdown {
    my ($self) = @_;
     # Check if the client socket is defined and still connected
    if (defined $self->{client} && $self->{client}->connected) {
        $self->{client}->shutdown(SHUT_WR);
        print "Socket shutdown successfully.\n";
    } else {
        print "Client socket is not connected or is undefined.\n";
    }
}
# A shared method to send a response to the client and close the connection
# Parameters:
#   message - The message to be sent to the client.
sub send_response {
    my ($self, $message) = @_;

    # Check if the client socket is defined and still connected
    if (defined $self->{client} && $self->{client}->connected) {
        $self->{client}->send(encode_json($message));
        print "Response sent successfully.\n";
    } else {
        print "Cannot send response. Client socket is not connected or is undefined.\n";
    }    
}

1;
