"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const process_1 = require("process");
const Upgrade_1 = require("./service/configuration/Upgrade");
const dataSource_1 = require("./config/dataSource");
const typeorm_transactional_1 = require("typeorm-transactional");
if (!process.argv[2]) {
    console.error("The archive path is not specified");
    (0, process_1.exit)(1);
}
const archivePath = process.argv[2];
async function upgrade() {
    const fs = require('fs');
    const fse = require('fs-extra');
    try {
        if (!await fs.existsSync(archivePath)) {
            console.error("The archive does not exist");
            (0, process_1.exit)(1);
        }
        let extension = '';
        if (archivePath.endsWith(".tar.gz")) {
            extension = '.tar.gz';
        }
        else if (archivePath.endsWith(".tar")) {
            extension = '.tar';
        }
        else {
            console.error("The archive does not have a .tar or .tar.gz extension");
            (0, process_1.exit)(1);
        }
        let cliUpgradePath = Upgrade_1.Upgrade.TMP_PATH + "/cli-upgrade" + extension;
        if (fs.existsSync(cliUpgradePath)) {
            fse.removeSync(cliUpgradePath);
        }
        if (!await fs.existsSync(Upgrade_1.Upgrade.TMP_PATH)) {
            await fs.mkdirSync(Upgrade_1.Upgrade.TMP_PATH);
        }
        await fse.copySync(archivePath, cliUpgradePath, { overwrite: true });
        if ((0, dataSource_1.getDataSource)() === undefined) {
            await (0, dataSource_1.createDataSource)();
            (0, typeorm_transactional_1.initializeTransactionalContext)({ storageDriver: typeorm_transactional_1.StorageDriver.AUTO });
        }
        const upgraded = await Upgrade_1.Upgrade.install("cli-upgrade" + extension, true);
        if (upgraded.success) {
            console.log("The upgrade was successful");
        }
        else {
            console.log(upgraded.message);
        }
    }
    catch (e) {
        console.error("Error during upgrade: " + e);
        (0, process_1.exit)(1);
    }
}
upgrade();
//# sourceMappingURL=cli-upgrade.js.map