import { Item } from './architecture.dto';
import { BasicResponseDTO } from './basicResponse.dto';
import { MenuItem } from './menuResponse.dto';
export declare class ItemStructure {
    class: string;
    hw_type?: string;
    subsystem: string;
    groups: ItemGroup[];
}
export declare class SavedItemStructure extends ItemStructure {
    groups: SavedItemGroup[];
}
export declare class ItemGroup {
    item_ids?: string[];
    regex?: string[];
    tags?: string[];
    parents?: {
        intersection: boolean;
        tags?: ParentTag[];
        item_ids?: string[];
        regex?: ParentRegex[];
    };
}
export declare class ItemGroupTag {
    tag_id: string;
    tag: string;
}
export declare class ItemGroupParentTag extends ItemGroupTag {
    subsystem: string;
}
export declare class ItemGroupParents {
    intersection: boolean;
    items?: Item[];
    regex?: ParentRegex[];
    tags?: ItemGroupParentTag[];
}
export declare class ItemGroupWithLabels {
    items?: Item[];
    regex?: string[];
    tags?: ItemGroupTag[];
    parents?: ItemGroupParents;
}
export declare class SavedItemGroup extends ItemGroup {
    item_group_id: string;
}
export declare class ParentTag {
    tag: string;
    subsystem: string;
}
export declare class ParentRegex {
    regex: string;
    subsystem: string;
}
export declare class CustomGroupItem {
    item_id: string;
    label: string;
    parents?: CustomGroupItemParent[];
}
export declare class CustomGroupItemParent {
    item_id: string;
    label: string;
}
export interface CustomGroupSearchResult {
    metadata: {
        start: number;
        end: number;
        max: number;
        page: number;
        info?: string;
    };
    items: CustomGroupItem[];
}
export declare class CustomGroupSearchDTO extends BasicResponseDTO {
    data: CustomGroupSearchResult;
}
export declare class CustomGroupItemDTO extends BasicResponseDTO {
    data: CustomGroupItem[];
}
export declare class CreateCustomGroupDTO extends ItemStructure {
    label: string;
    custom_group_folder_id: string;
    group_id?: number;
}
export declare class UpdateCustomGroupDTO extends SavedItemStructure {
    label: string;
    custom_group_id: string;
    custom_group_folder_id: string;
    group_id?: number;
}
export declare class CustomGroupFolder {
    custom_group_folder_id?: string;
    label: string;
    parent?: string;
    path?: string;
}
export declare class CustomGroupFolderTree {
    custom_group_folder_id?: string;
    label: string;
    children?: CustomGroupFolderTree[];
}
export declare class CreateCustomGroupFolder {
    label: string;
    parent?: string;
    group_id?: number;
}
export declare class CustomGroup {
    custom_group_id?: string;
    class: string;
    label: string;
    hw_type: string;
    subsystem: string;
    groups: CustomGroupItemGroup[];
    custom_group_folder_id?: string;
    group_id?: number;
}
export declare class CustomGroupItemGroup {
    items?: Item[];
    regex?: string[];
    tags?: {
        tag_id: string;
        tag: string;
    }[];
    parents: {
        intersection: boolean;
        items?: Item[];
        regex?: ParentRegex[];
        tags?: {
            tag_id: string;
            tag: string;
            subsystem: string;
        }[];
    };
}
export declare class GroupCustomGroupFolderTree {
    group_id: number;
    label: string;
    children: CustomGroupFolderTree[];
}
export declare class CustomGroupFoldersResponseDTO extends BasicResponseDTO {
    data: CustomGroupFolderTree[];
}
export declare class GroupCustomGroupFoldersTreeDTO extends BasicResponseDTO {
    data: GroupCustomGroupFolderTree[];
}
export declare class CustomGroupFolderResponseDTO extends BasicResponseDTO {
    data: CustomGroupFolder;
}
export declare class CustomGroupsResponseDTO extends BasicResponseDTO {
    data: CustomGroup[];
}
export declare class CustomGroupResponseDTO extends BasicResponseDTO {
    data: CustomGroup;
}
export declare class GroupCustomGroupFolder extends MenuItem {
    group_id: number;
}
export declare class GroupCustomGroupFoldersDTO extends BasicResponseDTO {
    data: GroupCustomGroupFolder[];
}
export declare class CustomGroupFavoriteResponseDTO extends BasicResponseDTO {
    data: {
        favorite: boolean;
    };
}
