import { PROTO, SnmpAuthProto, SnmpLevel, SnmpPrivProto, SnmpVersion } from "./hostcfgData.types";
export type IHostcfgDataDescription = {
    [key: string]: {
        required: boolean;
        type: "string" | "string[]" | "boolean" | "number" | "number[]" | "format[]";
        description?: string;
        possibleValues?: string[] | number[];
        default?: boolean | string | number | string[];
        format?: IHostcfgDataDescription;
    };
};
export interface IBaseDeviceProps {
    vendor: string;
    device: string;
    created: number;
    updated: number | null;
    description: string;
    managementUrl: string;
    disabled: boolean;
    isProxy?: boolean;
    radio?: string;
    sourceAddress?: string;
    licType?: string;
}
export declare class BaseDeviceProps implements IBaseDeviceProps {
    vendor: string;
    device: string;
    created: number;
    updated: number | null;
    description: string;
    managementUrl: string;
    disabled: boolean;
    radio: string;
    sourceAddress?: string;
    licType?: string;
    type?: string;
}
export declare const BaseDeviceDescription: IHostcfgDataDescription;
export declare class PortBaseDeviceProps extends BaseDeviceProps {
    host?: string;
    port: number;
}
export declare const PortBaseDeviceDescription: IHostcfgDataDescription;
export declare class ProtoPortBaseDevice extends PortBaseDeviceProps {
    protocol: PROTO;
}
export declare const ProtoPortBaseDeviceDescription: IHostcfgDataDescription;
export declare class AuthBaseProps extends ProtoPortBaseDevice {
    username: string;
    password: string;
}
export declare const AuthBaseDescription: IHostcfgDataDescription;
export declare class VMProps extends AuthBaseProps {
    backup_host: string;
    domain: string;
}
export declare const VMDescription: IHostcfgDataDescription;
export interface IRestHost {
    apiHost: string;
}
export interface IRestUserPass {
    apiUsername: string;
    apiPassword: string;
}
export interface IRestPort {
    apiPort: number;
}
export interface RestProps extends IRestHost, IRestUserPass, IRestPort {
}
export declare class RestDeviceProps extends BaseDeviceProps implements RestProps {
    apiHost: string;
    apiPort: number;
    apiUsername: string;
    apiPassword: string;
}
export declare const RestDeviceDescription: IHostcfgDataDescription;
export declare class ProtoRestDeviceProps extends RestDeviceProps {
    proto: PROTO;
}
export declare const ProtoRestDeviceDescription: IHostcfgDataDescription;
export interface ISshUsername {
    sshUsername: string;
}
export interface ISshPassword {
    sshPassword: string;
}
export interface IIsKeyFile {
    isKeyFile: boolean;
}
export interface ISshKeyFile extends IIsKeyFile {
    sshKeyFile: string;
}
export interface ISshPort {
    sshPort: number;
}
export interface ISshProps extends ISshUsername, ISshPort {
    sshHost: string;
}
export declare class SshDeviceProps extends BaseDeviceProps implements ISshProps {
    sshHost: string;
    sshPort: number;
    sshUsername: string;
}
export declare const SshDeviceDescription: IHostcfgDataDescription;
export declare class ExtensiveSshDeviceProps extends SshDeviceProps {
    isKeyFile: boolean;
    sshPassword: string;
    sshKeyFile: string;
}
export declare const ExtensiveSshDeviceDescription: IHostcfgDataDescription;
export declare class SnmpDeviceProps extends BaseDeviceProps {
    snmpHost: string;
    snmpPort: number;
    snmpVersion: SnmpVersion;
    snmpCommunity: string;
    snmpSecUser: string;
    snmpSecLevel: SnmpLevel;
    snmpPrivProtocol: SnmpPrivProto;
    snmpAuthProtocol: SnmpAuthProto;
    snmpPrivPassword: string;
    snmpAuthPassword: string;
}
export declare const SnmpDeviceDescription: IHostcfgDataDescription;
export declare class SnmpNetworkDeviceProps extends SnmpDeviceProps {
    vfId: number;
    fabric_id: string;
    fabric_label: string;
}
export declare const SnmpNetworkDeviceDescription: IHostcfgDataDescription;
export declare class SnmpStorageDeviceProps extends SnmpDeviceProps {
    snmpContextName: string;
}
export declare const SnmpStorageDeviceDescription: IHostcfgDataDescription;
export interface IOldDeviceProps {
    uuid: string;
    proto: string;
    password: string;
    username: string;
    updated: string;
    created: string;
    api_port: string | number;
    auth_ssh: boolean;
    auth_api: boolean;
    host: string;
    hostalias: string;
    devicealias: string;
    disabled: boolean;
    radio: string;
}
export declare class OldDeviceProps implements IOldDeviceProps {
    uuid: string;
    hostalias: string;
    proto: PROTO;
    password: string;
    username: string;
    updated: string;
    created: string;
    api_port: number;
    auth_ssh: boolean;
    auth_api: boolean;
    host: string;
    devicealias: string;
    "api-port": string;
    disabled: boolean;
    radio: string;
}
export declare class OldVmProps extends OldDeviceProps {
    domain: string;
    backup_host: string;
}
export declare class OldRestDeviceProps extends OldDeviceProps {
    "api-user": string;
    "api-port": string;
    "snmp-auth-pass": string;
    "api-password": string;
    "api-password2": string;
    "api-user2": string;
    "snmp-version": string;
    "snmp-port": string;
    "snmp-priv-pass": string;
    "horcm-file": string;
    "snmp-user": string;
    "snmp-host": string;
    "snmp-community": string;
    "host-node": string;
    "api-port-hcm": string;
    device: string;
}
export declare class OldApiAuthDeviceProps extends OldDeviceProps {
    "api-user": string;
    "api-password": string;
    "api-port": string;
}
export declare class OldSshDeviceProps extends OldRestDeviceProps {
    "ssh-user": string;
    "ssh-password": string;
    "ssh-key-id": string;
    "ssh-password-enable": number;
    "ssh-port": string | number;
}
export declare class OldSnmpDeviceProps extends OldDeviceProps {
    "snmp-priv-protocol": string;
    "snmp-auth-pass": string;
    "snmp-version": string;
    "snmp-port": string;
    "snmp-priv-pass": string;
    "snmp-user": string;
    "snmp-host": string;
    "snmp-community": string;
    "fabric": string;
    "snmp-context": string;
    "snmp-sec-level": string;
}
export declare class UniCliDeviceProps extends BaseDeviceProps {
    host: string;
    username: string;
    password: string;
}
export declare const UniCliDeviceDescription: IHostcfgDataDescription;
export declare class OldUniCliDeviceProps extends OldDeviceProps {
    "api-user": string;
    "api-password": string;
}
export declare class TokenRestDeviceProps extends RestDeviceProps {
    apiToken: string;
    isToken: boolean;
}
export declare class TokenRestDevicePropsOld extends OldRestDeviceProps {
    "api-token": string;
    auth_token: number;
}
export declare const TokenRestDeviceDescription: IHostcfgDataDescription;
export declare class TwoNodeRestDeviceProps extends RestDeviceProps {
    node1Host: string;
    node2Host: string;
}
export declare const TwoNodeRestDeviceDescription: IHostcfgDataDescription;
export declare class TwoNodeRestDevicePropsOld extends OldRestDeviceProps {
    host1: string;
    host2: string;
}
export declare class AbstractDBProps extends PortBaseDeviceProps {
    host?: string;
    instance: string;
    username: string;
    password: string;
    use_whitelist: boolean;
    dbs: string[];
}
export declare class AbstractOldDBProps extends OldDeviceProps {
    host: string;
    port: number;
    instance: string;
    username: string;
    password: string;
    use_whitelist: boolean;
    dbs: string[];
}
export declare const AbstractDBDescription: IHostcfgDataDescription;
