"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setPublicUrlToSourceMaps = exports.setPublicUrlToUI = exports.getPublicUrl = exports.isNewerVersion = void 0;
const LPAR_PATH = '/home/lpar2rrd/lpar2rrd';
const UI_PATH = 'client';
const isNewerVersion = (oldVer, newVer) => {
    const oldParts = oldVer.split('.');
    const newParts = newVer.split('.');
    for (let i = 0; i < newParts.length; i++) {
        const a = ~~newParts[i];
        const b = ~~oldParts[i];
        if (a > b)
            return true;
        if (a < b)
            return false;
    }
    return false;
};
exports.isNewerVersion = isNewerVersion;
const getPublicUrl = async () => {
    let publicUrl = '';
    try {
        const fs = require('fs');
        if (await fs.existsSync(LPAR_PATH)) {
            if (await fs.existsSync(LPAR_PATH + '/etc/.magic')) {
                const envFile = await fs.readFileSync(LPAR_PATH + '/etc/.magic', { encoding: 'utf8', flag: 'r' });
                if (envFile.includes("VI_IMAGE")) {
                    if (await fs.existsSync(LPAR_PATH + '/etc/version.txt')) {
                        const versionFile = await fs.readFileSync(LPAR_PATH + '/etc/version.txt', { encoding: 'utf8', flag: 'r' });
                        const lines = versionFile.trim().split('\n');
                        const lastLine = lines[lines.length - 1];
                        const version = lastLine.slice(0, 4);
                        if ((0, exports.isNewerVersion)('7.59', version)) {
                            return 'xormon-ng';
                        }
                    }
                }
            }
        }
    }
    catch (e) {
        console.error('Failed to check lpar data', e);
    }
    return publicUrl !== '' ? publicUrl : null;
};
exports.getPublicUrl = getPublicUrl;
const setPublicUrlToUI = async (url) => {
    try {
        const fs = await Promise.resolve().then(() => require('fs'));
        let html = await fs.readFileSync(UI_PATH + "/index.html", { encoding: 'utf8', flag: 'r' });
        html = html.replace(">var publicUrl<", `>var publicUrl=${url ? `'/${url}'` : url};<`);
        html = html.replace("var publicUrl;", `var publicUrl=${url ? `'/${url}'` : url};`);
        if (url !== undefined && url !== null) {
            if (!html.includes(`${url}/static/`)) {
                html = html.replace(/static\//g, `${url}/static/`);
            }
            if (!html.includes(`${url}/assets/`)) {
                html = html.replace(/assets\//g, `${url}/assets/`);
            }
            if (!html.includes(`/${url}/`)) {
                html = html.replaceAll(`/index-`, `/${url}/index-`);
                html = html.replaceAll(`/style-`, `/${url}/style-`);
            }
        }
        await fs.writeFileSync(UI_PATH + "/index.html", html);
        if (url !== undefined && url !== null) {
            if (await fs.existsSync(UI_PATH + "/static/css")) {
                const cssFiles = await fs.readdirSync(UI_PATH + "/static/css");
                for (const cssFile of cssFiles) {
                    let file = await fs.readFileSync(UI_PATH + "/static/css/" + cssFile, { encoding: 'utf8', flag: 'r' });
                    if (!file.includes(`${url}/static/`) && !file.includes(`${url}\\/static/`)) {
                        file = file.replace(/static\//g, `${url}/static/`);
                        await fs.writeFileSync(UI_PATH + "/static/css/" + cssFile, file);
                    }
                }
            }
            if (await fs.existsSync(UI_PATH + "/static/js")) {
                const jsFiles = await fs.readdirSync(UI_PATH + "/static/js");
                for (const jsFile of jsFiles) {
                    let file = await fs.readFileSync(UI_PATH + "/static/js/" + jsFile, { encoding: 'utf8', flag: 'r' });
                    if (!file.includes(`${url}/static/`) && !file.includes(`${url}\\/static/`)) {
                        file = file.replace(/static\//g, `${url}\\/static/`);
                        await fs.writeFileSync(UI_PATH + "/static/js/" + jsFile, file);
                    }
                }
            }
            if (fs.existsSync(UI_PATH + "/assets")) {
                const allFiles = fs.readdirSync(UI_PATH + "/assets");
                for (const jFile of allFiles) {
                    if (!/.*\.(js|ts|css)$/.test(jFile.toLowerCase()))
                        continue;
                    let file = fs.readFileSync(UI_PATH + "/assets/" + jFile, { encoding: 'utf8', flag: 'r' });
                    if (!file.includes(`${url}/assets/`) && !file.includes(`${url}\\/assets/`)) {
                        file = file.replace(/assets\//g, `${url}\\/assets/`);
                        fs.writeFileSync(UI_PATH + "/assets/" + jFile, file);
                    }
                }
            }
        }
    }
    catch (err) {
        console.error('Failed to set public URL for UI', err);
    }
};
exports.setPublicUrlToUI = setPublicUrlToUI;
const setPublicUrlToSourceMaps = async (url) => {
    try {
        const fs = await Promise.resolve().then(() => require('fs'));
        if (url != null) {
            if (fs.existsSync(UI_PATH + "/assets")) {
                const allFiles = fs.readdirSync(UI_PATH + "/assets");
                for (const jFile of allFiles) {
                    if (!/.*\.(map)$/.test(jFile.toLowerCase()))
                        continue;
                    let file = fs.readFileSync(UI_PATH + "/assets/" + jFile, { encoding: 'utf8', flag: 'r' });
                    if (file.includes(`"sourceRoot": "${url}"`))
                        break;
                    file = file.replace(/assets\//g, `${url}\\/assets/`);
                    file = file.replace('"sources":', `"sourceRoot": "${url}",\n"sources":`);
                    fs.writeFileSync(UI_PATH + "/assets/" + jFile, file);
                }
            }
        }
        else if (fs.existsSync(UI_PATH + "/assets")) {
            const allFiles = fs.readdirSync(UI_PATH + "/assets");
            for (const jFile of allFiles) {
                if (!/.*\.map$/.test(jFile.toLowerCase()))
                    continue;
                let file = fs.readFileSync(UI_PATH + "/assets/" + jFile, { encoding: 'utf8', flag: 'r' });
                if (!file.includes(`"sourceRoot": "",`)) {
                    file = file.replace('"sources":', `"sourceRoot": "",\n"sources":`);
                    fs.writeFileSync(UI_PATH + "/assets/" + jFile, file);
                }
                else {
                    break;
                }
            }
        }
    }
    catch (err) {
        console.error('Failed to set public URL for UI source maps', err);
    }
};
exports.setPublicUrlToSourceMaps = setPublicUrlToSourceMaps;
//# sourceMappingURL=publicUrl.js.map