"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDataSource = exports.createDataSource = exports.databaseConfiguration = void 0;
const typeorm_1 = require("typeorm");
const process_1 = require("process");
const Crypto_1 = require("../service/configuration/Crypto");
const typeorm_transactional_1 = require("typeorm-transactional");
const databaseConfiguration = async () => {
    require('dotenv').config({ override: true });
    let dbPassword = await Crypto_1.Crypto.decrypt(process_1.env['DB_PASSWORD']);
    if (!dbPassword) {
        dbPassword = process_1.env['DB_PASSWORD'];
    }
    return {
        type: 'postgres',
        host: process_1.env['DB_HOST'],
        port: +process_1.env['DB_PORT'],
        username: process_1.env['DB_USERNAME'],
        password: dbPassword,
        database: process_1.env['DB_DATABASE'],
        entities: [`${__dirname}/../**/**.entity{.ts,.js}`],
        subscribers: [`${__dirname}/../**/*.subscriber{.ts,.js}`],
        migrations: [`${__dirname}/../**/*.migration{.ts,.js}`],
        synchronize: false,
        poolSize: 10,
        extra: {
            max: 10
        }
    };
};
exports.databaseConfiguration = databaseConfiguration;
const createDataSource = async () => {
    global.dataSource = (0, typeorm_transactional_1.getDataSourceByName)('default') || await ((0, typeorm_transactional_1.addTransactionalDataSource)(new typeorm_1.DataSource({
        ...await (0, exports.databaseConfiguration)(),
        type: 'postgres',
    }))).initialize();
};
exports.createDataSource = createDataSource;
const getDataSource = () => { return global.dataSource; };
exports.getDataSource = getDataSource;
//# sourceMappingURL=dataSource.js.map