"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const process_1 = require("process");
const Crypto_1 = require("./service/configuration/Crypto");
require('dotenv').config();
if (!process_1.env['DB_HOST']) {
    (0, process_1.exit)(1);
}
let manualTables = [];
process.argv.forEach(function (val, index, array) {
    if (index >= 2) {
        manualTables.push(val);
    }
});
const views = ['1d', '6h', '1h', '5m', '1m'];
async function startDestroy() {
    const { Pool, Client } = require('pg');
    const xmPool = new Pool({
        user: process_1.env['DB_USERNAME'],
        host: process_1.env['DB_HOST'],
        password: await Crypto_1.Crypto.decrypt(process_1.env['DB_PASSWORD'], process_1.env['APP_SECRET']),
        port: process_1.env['DB_PORT'],
        database: process_1.env['DB_DATABASE']
    });
    const getTables = async () => {
        const result = await xmPool.query("SELECT tablename FROM pg_catalog.pg_tables WHERE schemaname = 'metrics' ORDER BY tablename ASC;");
        console.log(result);
        if (result.rows) {
            let tables = result.rows.map((obj) => {
                return obj.tablename;
            });
            return tables;
        }
        else {
            return [];
        }
    };
    const now = () => {
        return new Date().toISOString();
    };
    const destroyTable = async (table) => {
        console.log("[" + now() + "] [DropTable] " + table + " start");
        try {
            await xmPool.query("DROP TABLE " + table + " CASCADE");
        }
        catch (e) {
            console.error("[" + now() + "] [DropTableError: " + e);
        }
        console.log("[" + now() + "] [DropTable] " + table + " end");
    };
    const destroyView = async (view) => {
        console.log("[" + now() + "] [DropView] " + view + " start");
        try {
            await xmPool.query("DROP MATERIALIZED VIEW " + view + " CASCADE");
        }
        catch (e) {
            console.error("[" + now() + "] [DropViewError: " + e);
        }
        console.log("[" + now() + "] [DropView] " + view + " end");
    };
    const tables = manualTables.length >= 1 ? manualTables : await getTables();
    for (const tableIndex in tables) {
        const table = tables[tableIndex];
        for (const view of views) {
            await destroyView('metrics.' + table + "_" + view);
        }
        await destroyTable('metrics.' + table);
    }
}
startDestroy();
//# sourceMappingURL=drop-perf-table.js.map