"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const process_1 = require("process");
const dataSource_1 = require("./config/dataSource");
const App_1 = require("./service/configuration/App");
const path_1 = require("path");
const TableGenerator_1 = require("./service/metrics/TableGenerator");
const typeorm_transactional_1 = require("typeorm-transactional");
const IMPORT_TMP_PATH = (0, path_1.resolve)(App_1.App.TMP_FILES_PATH + '/import');
if (!process.argv[2]) {
    console.error("The archive path is not specified");
    (0, process_1.exit)(1);
}
const archivePath = process.argv[2];
async function importData() {
    const fs = require('fs');
    const fse = require('fs-extra');
    const tar = require('tar');
    let dataSource;
    if ((0, dataSource_1.getDataSource)() === undefined) {
        await (0, dataSource_1.createDataSource)();
        (0, typeorm_transactional_1.initializeTransactionalContext)({ storageDriver: typeorm_transactional_1.StorageDriver.AUTO });
    }
    dataSource = (0, dataSource_1.getDataSource)();
    try {
        if (!await fs.existsSync(archivePath)) {
            console.error("The archive does not exists");
            (0, process_1.exit)(1);
        }
        if (!archivePath.includes(".tar.gz")) {
            console.error("The archive does not have a tar.gz extension");
            (0, process_1.exit)(1);
        }
        if (!await fs.existsSync(App_1.App.TMP_FILES_PATH)) {
            await fs.mkdirSync(App_1.App.TMP_FILES_PATH);
        }
        if (await fs.existsSync(IMPORT_TMP_PATH)) {
            await fs.rmSync(IMPORT_TMP_PATH, { recursive: true, force: true });
        }
        await fs.mkdirSync(IMPORT_TMP_PATH);
        await fse.copySync(archivePath, App_1.App.TMP_FILES_PATH + "/import.tar.gz", { overwrite: true });
        const tarFile = App_1.App.TMP_FILES_PATH + "/import.tar.gz";
        await tar.extract({
            file: tarFile,
            cwd: IMPORT_TMP_PATH,
            sync: true
        });
        let importFiles = await fs.readdirSync(IMPORT_TMP_PATH);
        for (const importFolder of importFiles) {
            const stats = await fs.lstatSync(IMPORT_TMP_PATH + "/" + importFolder);
            if (stats.isDirectory()) {
                const subFiles = fs.readdirSync(IMPORT_TMP_PATH + "/" + importFolder);
                for (const importFile of subFiles) {
                    const subStats = await fs.lstatSync(IMPORT_TMP_PATH + "/" + importFolder + "/" + importFile);
                    if (!subStats.isDirectory()) {
                        await fs.copyFileSync(IMPORT_TMP_PATH + "/" + importFolder + "/" + importFile, IMPORT_TMP_PATH + "/" + importFile);
                    }
                }
                await fs.rmSync(IMPORT_TMP_PATH + "/" + importFolder, { recursive: true, force: true });
            }
            else {
                continue;
            }
        }
        importFiles = fs.readdirSync(IMPORT_TMP_PATH);
        const objectItemsFile = importFiles.find(file => file === 'object_items.json' || file === 'db_object_items.json');
        if (objectItemsFile) {
            const index = importFiles.indexOf(objectItemsFile);
            if (index > -1) {
                importFiles.splice(index, 1);
            }
            importFiles.unshift(objectItemsFile);
        }
        for (const importFile of importFiles) {
            try {
                if (!importFile.includes(".json")) {
                    fs.unlinkSync(IMPORT_TMP_PATH + "/" + importFile);
                    continue;
                }
                if (importFile.startsWith("metrics_")) {
                    let table = importFile.replace(".json", "");
                    const entity = dataSource.manager.connection.entityMetadatas.find(ent => ent.tableName === table);
                    if (!entity) {
                        console.error(`[${table}] Entity not found`);
                        continue;
                    }
                    await TableGenerator_1.TableGenerator.createPerformanceTableFromEntity(entity);
                    const data = JSON.parse(await fs.readFileSync(IMPORT_TMP_PATH + '/' + importFile, { encoding: 'utf8', flag: 'r' }));
                    const now = new Date();
                    const date = now.getUTCDate();
                    const month = now.getUTCMonth();
                    const year = now.getUTCFullYear();
                    for (const timePoint of data) {
                        const time = new Date(timePoint.time);
                        time.setUTCDate(date);
                        time.setUTCMonth(month);
                        time.setUTCFullYear(year);
                        timePoint.time = time.toISOString();
                    }
                    await dataSource.transaction(async (transactionalEntityManager) => {
                        const chunk = 500;
                        for (let index = 0; index < data.length; index += chunk) {
                            const toSaveChunk = data.slice(index, index + chunk);
                            await transactionalEntityManager.getRepository("metrics." + table).createQueryBuilder().insert().values(toSaveChunk).orIgnore().execute();
                        }
                    });
                    fs.unlinkSync(IMPORT_TMP_PATH + "/" + importFile);
                    console.log(`[${table}] imported`);
                }
                else if (importFile.startsWith("db_")) {
                    const table = importFile.replace("db_", "").replace(".json", "");
                    const entity = dataSource.manager.connection.entityMetadatas.find(ent => ent.tableName === table);
                    if (!entity) {
                        console.error(`[${table}] Entity not found`);
                        await fs.unlinkSync(IMPORT_TMP_PATH + "/" + importFile);
                        continue;
                    }
                    const content = await fs.readFileSync(IMPORT_TMP_PATH + '/' + importFile, { encoding: 'utf8', flag: 'r' });
                    const data = JSON.parse(content);
                    await dataSource.transaction(async (transactionalEntityManager) => {
                        const chunk = 1000;
                        for (let index = 0; index < data.length; index += chunk) {
                            const toSaveChunk = data.slice(index, index + chunk);
                            await transactionalEntityManager.getRepository(table).createQueryBuilder().insert().values(toSaveChunk).orIgnore().execute();
                        }
                    });
                    await fs.unlinkSync(IMPORT_TMP_PATH + "/" + importFile);
                    console.log(`[${table}] imported`);
                }
            }
            catch (e) {
                console.error("Import file " + importFile + " failed: " + e);
            }
        }
    }
    catch (e) {
        console.error("Error during import: " + e);
        (0, process_1.exit)(1);
    }
    (0, process_1.exit)(0);
}
importData();
//# sourceMappingURL=import-data.js.map