"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MenuAccessInterceptor = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../config/dataSource");
const aclResponse_constants_1 = require("../common/constants/aclResponse.constants");
const menuResponse_dto_1 = require("../common/menuResponse.dto");
const agentMapping_entity_1 = require("../entity/agentMapping.entity");
const As400_1 = require("../service/menu/menu/As400");
const Linux_1 = require("../service/menu/menu/Linux");
const user_entity_1 = require("../entity/user.entity");
const storageGroup_entity_1 = require("../entity/storageGroup.entity");
const ACLCheck_1 = require("../service/acl/ACLCheck");
const User_1 = require("../service/users/User");
const operators_1 = require("rxjs/operators");
const typeorm_2 = require("typeorm");
let MenuAccessInterceptor = class MenuAccessInterceptor {
    constructor(usersRepository, agentMappingRepository) {
        this.usersRepository = usersRepository;
        this.agentMappingRepository = agentMappingRepository;
    }
    async intercept(context, next) {
        const request = context.switchToHttp().getRequest();
        const response = context.switchToHttp().getResponse();
        const caller = context.getHandler().name;
        const user = await User_1.User.getUserByRequest(request, true);
        const groupIds = user.groups.map(group => group.group_id);
        const url = response.req.url;
        const tmpUrl = new URL('https://do.not.care' + url);
        let itemId = tmpUrl.searchParams.get('item_id');
        if (itemId && itemId.includes(":::")) {
            itemId = itemId.split(":::")[0];
        }
        let agent, page;
        if (caller === 'getPage' && itemId) {
            agent = await this.agentMappingRepository.findOne({ where: { item_id: itemId } });
            if (agent && agent !== null) {
                let agentPage;
                if (agent.agent_id.startsWith('as400_')) {
                    agentPage = new As400_1.As400.ServerPage(agent.agent_id, request);
                }
                else {
                    agentPage = new Linux_1.LinuxServerPage(agent.agent_id, request);
                }
                if (agentPage) {
                    page = await agentPage.getPage();
                }
            }
        }
        return next
            .handle()
            .pipe((0, operators_1.map)(async (data) => {
            if (caller === 'getPage' && itemId && data.data.tabs && page && page !== null) {
                let tabTitles = data.data.tabs.map(tab => tab.title);
                for (let agentTab of page.tabs) {
                    if (tabTitles.includes(agentTab.title)) {
                        agentTab.title = agentTab.title + ' OS';
                    }
                }
                data.data.tabs.push(...page.tabs);
            }
            if (groupIds.includes(1)) {
                return data;
            }
            let isMenuTree = false;
            let path = url.replace(menuResponse_dto_1.MenuLink.API_URL_PAGE, '');
            if (caller === 'getPath') {
                isMenuTree = true;
                path = url.replace('/api/menu/v1/getpath/', '');
            }
            let tmpPath = path.split('/');
            let hwType = tmpPath[0];
            let subsystem = (tmpPath.length > 1) ? tmpPath[1] : null;
            if (caller === 'getTab') {
                return data;
            }
            else if (caller === 'getPage' || caller === 'getPath') {
                let itemStatus;
                if (itemId) {
                    let itemIdIsStorageGroupId;
                    try {
                        itemIdIsStorageGroupId = await (0, dataSource_1.getDataSource)().getRepository(storageGroup_entity_1.StorageGroup).findOne({
                            where: {
                                storage_group_id: itemId,
                                group_id: (0, typeorm_2.In)(groupIds),
                            }
                        });
                    }
                    catch (err) {
                        if (!(err instanceof typeorm_2.QueryFailedError)) {
                            throw err;
                        }
                        itemIdIsStorageGroupId = false;
                    }
                    if (itemIdIsStorageGroupId) {
                        itemStatus = aclResponse_constants_1.ITEM_STATUS.granted;
                    }
                    else if (hwType && hwType === 'sgroup') {
                        itemStatus = await ACLCheck_1.ACLCheck.getClassStatus(groupIds, 'storage', false);
                        if (itemStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                            itemStatus = await ACLCheck_1.ACLCheck.getItemStatus(groupIds, itemId, false);
                        }
                    }
                    else {
                        itemStatus = await ACLCheck_1.ACLCheck.getItemStatus(groupIds, itemId, false);
                    }
                }
                else if (hwType && hwType === 'sgroup') {
                    itemStatus = await ACLCheck_1.ACLCheck.getClassStatus(groupIds, 'storage', false);
                }
                else if (hwType && (hwType === 'san' || hwType === 'lan' || hwType === 'storage')) {
                    itemStatus = await ACLCheck_1.ACLCheck.getClassStatus(groupIds, hwType, false);
                }
                else if (hwType) {
                    itemStatus = await ACLCheck_1.ACLCheck.getHwTypeStatus(groupIds, hwType, false);
                }
                if ((caller === 'getPage') && (itemStatus !== aclResponse_constants_1.ITEM_STATUS.denied)) {
                    let tabs = data.data.tabs;
                    for (let tab of tabs) {
                        for (let box of tab.content) {
                            if (box.type === 'FourGraphBox') {
                                let filteredIds = (agent && agent !== null) ? box.uuids : await ACLCheck_1.ACLCheck.filterItemsByRulesTopDown(groupIds, box.uuids);
                                box.uuids = filteredIds;
                            }
                        }
                    }
                    return data;
                }
                else if ((caller === 'getPath') && (itemStatus === aclResponse_constants_1.ITEM_STATUS.granted)) {
                    return data;
                }
                else if (itemStatus === aclResponse_constants_1.ITEM_STATUS.denied) {
                    return {
                        statusCode: common_1.HttpStatus.FORBIDDEN,
                        message: "Access denied",
                        error: 'Access denied'
                    };
                }
            }
            else if (hwType && hwType === 'sgroups') {
                let itemStatus = await ACLCheck_1.ACLCheck.getClassStatus(groupIds, 'storage', false);
                if (itemStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                    return data;
                }
            }
            const menuItems = data.data;
            let filteredItems = [];
            if (!isMenuTree) {
                filteredItems = await ACLCheck_1.ACLCheck.filterMenuItems(groupIds, menuItems, url);
            }
            else {
                filteredItems = await ACLCheck_1.ACLCheck.filterMenuItems(groupIds, menuItems);
                let currentFolder = filteredItems;
                let hasChildren = false;
                do {
                    hasChildren = false;
                    for (let item of currentFolder) {
                        if (item.children && item.children.length > 0) {
                            item.children = await ACLCheck_1.ACLCheck.filterMenuItems(groupIds, item.children, item.url);
                            currentFolder = item.children;
                            hasChildren = true;
                            break;
                        }
                    }
                } while (currentFolder && hasChildren);
            }
            return {
                statusCode: data.statusCode,
                message: data.message,
                data: filteredItems
            };
        }));
    }
};
exports.MenuAccessInterceptor = MenuAccessInterceptor;
exports.MenuAccessInterceptor = MenuAccessInterceptor = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(user_entity_1.User)),
    __param(1, (0, typeorm_1.InjectRepository)(agentMapping_entity_1.AgentMapping)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository])
], MenuAccessInterceptor);
//# sourceMappingURL=menuAccess.interceptor.js.map