"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@nestjs/core");
const express_1 = require("express");
const app_module_1 = require("./app.module");
const config_1 = require("@nestjs/config");
const dataSource_1 = require("./dataSource");
const Settings_1 = require("../service/configuration/Settings");
const App_1 = require("../service/configuration/App");
async function bootstrap() {
    await (0, dataSource_1.createDataSource)();
    const fs = require('fs');
    let httpsOptions;
    let useDefaultCert = true;
    const settings = await Settings_1.Settings.get({
        prefix: 'CERT'
    });
    if (settings['CERT_KEY'] && settings['CERT_KEY'].length >= 1) {
        const keyFile = App_1.App.ROOT_PATH + "/server-nest/" + settings['CERT_KEY'];
        const crtFile = App_1.App.ROOT_PATH + "/server-nest/" + settings['CERT_CRT'];
        if (fs.existsSync(keyFile) && fs.existsSync(crtFile)) {
            httpsOptions = {
                key: fs.readFileSync(keyFile),
                cert: fs.readFileSync(crtFile),
            };
            useDefaultCert = false;
        }
    }
    if (useDefaultCert) {
        httpsOptions = {
            key: fs.readFileSync(App_1.App.ROOT_PATH + "/server-nest/" + 'ssl/xormon.key'),
            cert: fs.readFileSync(App_1.App.ROOT_PATH + "/server-nest/" + 'ssl/xormon.crt'),
        };
    }
    const app = await core_1.NestFactory.create(app_module_1.AppModule, httpsOptions ? {
        httpsOptions,
    } : undefined);
    app.use((0, express_1.json)({ limit: '100mb' }));
    app.use((0, express_1.urlencoded)({ extended: true, limit: '100mb' }));
    const configService = app.get(config_1.ConfigService);
    const port = +configService.get('METRICS_SERVER_PORT') || 3030;
    await app.listen(port);
}
bootstrap();
//# sourceMappingURL=main.js.map