"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DatabaseCleanup1711444660073 = void 0;
const DatabaseHelper_1 = require("../../config/DatabaseHelper");
const dataSource_1 = require("../../config/dataSource");
const microservice_entity_1 = require("../../entity/microservice.entity");
const PM2_1 = require("../../service/configuration/PM2");
const typeorm_1 = require("typeorm");
class DatabaseCleanup1711444660073 {
    constructor() {
        this.name = 'DatabaseCleanup1711444660073';
    }
    async up(queryRunner) {
        await (0, dataSource_1.createDataSource)();
        const dataSource = (0, dataSource_1.getDataSource)();
        const newInstall = !(await DatabaseHelper_1.DatabaseHelper.tableExists("users", dataSource));
        const schema_check = await dataSource.manager.connection.query("SELECT schema_name FROM information_schema.schemata WHERE schema_name = 'metrics';");
        if (!newInstall && schema_check) {
            let microservices = await dataSource.getRepository(microservice_entity_1.Microservice).find({
                select: {
                    microservice: true
                },
                where: [
                    { onStartup: true },
                    { microservice: (0, typeorm_1.In)(['linux']) }
                ]
            });
            for (const microservice of microservices) {
                try {
                    await PM2_1.PM2.stopService(microservice.microservice);
                }
                catch (e) {
                    console.error(`[PM2] ${microservice.microservice} error: ${e.message}`);
                }
            }
            await queryRunner.connection.query(`
            SELECT alter_job(job_id, scheduled => false)
            FROM timescaledb_information.jobs
            WHERE proc_name IN ('policy_retention', 'policy_compression', 'policy_refresh_continuous_aggregate')
        `);
            const entities = dataSource.entityMetadatas;
            for (const entity of entities) {
                if (entity.schema === "metrics") {
                    const tableName = entity.tableName;
                    try {
                        await DatabaseHelper_1.DatabaseHelper.deletePerfTable(tableName, dataSource);
                    }
                    catch (e) {
                        throw new Error(e);
                    }
                }
            }
            await queryRunner.query(`DROP SCHEMA metrics CASCADE;`);
            await queryRunner.query(`CREATE SCHEMA IF NOT EXISTS metrics;`);
        }
        let defaultUser;
        if (!newInstall) {
            const xormonUser = await queryRunner.query(`SELECT * FROM public.users WHERE user_id = 1`);
            if (xormonUser && xormonUser[0]) {
                defaultUser = xormonUser[0];
            }
        }
        if (!defaultUser) {
            defaultUser = {
                user_id: 1,
                username: 'xormon',
                email: null,
                password: '$2b$10$GTliGfYOL7cUmvLpd6qTB.6x8UNTymyHrvLTncLoBmM/7Y5p4WsXi',
                active: true,
                timezone: 'Etc/UTC',
                configuration: {},
            };
        }
        let newMicroservicesTable = [];
        if (!newInstall) {
            const microservicesOnStartup = await queryRunner.query(`SELECT * FROM public.microservices`);
            for (const microserviceOnStartup of microservicesOnStartup) {
                newMicroservicesTable.push({
                    microservice: microserviceOnStartup.microservice,
                    script: microserviceOnStartup.script,
                    host: microserviceOnStartup.host,
                    port: microserviceOnStartup.port,
                    hw_type: microserviceOnStartup.hw_type,
                    onStartup: microserviceOnStartup ? microserviceOnStartup.onStartup : false,
                    needConf: microserviceOnStartup.needConf,
                    updated_at: new Date(),
                    label: microserviceOnStartup.label,
                });
            }
        }
        const tablesToDrop = [
            'acl',
            'admin_groups',
            'agent_mappings',
            'aggregation',
            'alerting_hw_emails',
            'alerting_mail_groups',
            'alerting_launches',
            'alerting',
            'audits',
            'backend',
            'bucketing',
            'custom_task_mail_group_relations',
            'custom_task',
            'dashboard_graphs',
            'dashboard_groups',
            'dashboards',
            'downsampling',
            'exports',
            'group_ldap_groups',
            'health_status_history',
            'health_status_mirror',
            'health_status_mirror_history',
            'health_status',
            'heatmap_tier_items',
            'heatmap_tiers',
            'heatmaps',
            'hostcfg_relations',
            'hostcfg_status',
            'item_group_parent_tags',
            'item_group_parents',
            'item_group_regex_parents',
            'item_group_tags',
            'item_group_items',
            'item_properties',
            'item_relations',
            'last_timestamps',
            'ldap_groups',
            'metrics_1d',
            'metrics_1h',
            'metrics_5m',
            'metrics_6h',
            'metrics',
            'microservice_data',
            'microservice_job_status',
            'microservice_jobs',
            'microservice_variables',
            'microservices',
            'readings',
            'report_files',
            'report_items',
            'reports_mail_groups',
            'reports',
            'sessions',
            'storage_group_devices',
            'storage_group_regex',
            'storage_groups',
            'storage_mappings',
            'subsystem_metrics',
            'subsystems',
            'tag_items',
            'tags',
            'target_destination_hostcfgs',
            'target_destinations',
            'tops',
            'user_api_keys',
            'users_groups',
            'versions',
            'alerting_groups',
            'audit_events',
            'consistency_groups',
            'dashboard_folders',
            'groups',
            'item_groups',
            'mail_groups',
            'object_items',
            'custom_groups',
            'custom_group_folders',
            'users',
            'ldaps',
        ];
        await queryRunner.query(`TRUNCATE TABLE public.migrations;`);
        for (const table of tablesToDrop) {
            await queryRunner.query(`DROP TABLE IF EXISTS public.${table} CASCADE`);
        }
        await queryRunner.query(`CREATE TABLE public.audit_events (
      audit_event_id serial4 NOT NULL,
      "label" varchar NOT NULL,
      "text" varchar NOT NULL,
      CONSTRAINT "PK_7cd7fcc65c3309cbb1dd51f3327" PRIMARY KEY (audit_event_id)
    );
    `);
        await queryRunner.query(`
    INSERT INTO public.audit_events (audit_event_id, "label", "text") VALUES
      (1, 'Log In', 'User logged in'),
      (2, 'Log out', 'User logged out'),
      (3, 'Hostcfg add', 'User added hostcfg'),
      (4, 'Hostcfg edit', 'User eddited hostcfg'),
      (5, 'Hostcfg delete', 'User deleted hostcfg'),
      (6, 'User add', 'User added new user'),
      (7, 'User edit', 'User edited user'),
      (8, 'User delete', 'User deleted user'),
      (9, 'Group add', 'User added group'),
      (10, 'Group edit', 'User edited group'),
      (11, 'Group delete', 'User deleted group'),
      (12, 'Settings updated', 'User updated settings'),
      (13, 'Settings deleted', 'User deleted settings'),
      (14, 'Proxy devices add', 'User added proxy'),
      (15, 'Proxy devices update', 'User updated proxy'),
      (16, 'Proxy devices delete', 'User deleted proxy'),
      (17, 'Microservice start', 'User started microservice'),
      (18, 'Microservice stop', 'User stopped microservice'),
      (19, 'Variable add', 'User added variable'),
      (20, 'Variable update', 'User updated variable'),
      (21, 'Variable delete', 'User deleted variable'),
      (22, 'Imported Devices', 'User Imported Device Configurations');
    `);
        await queryRunner.query(`CREATE TABLE public.users (
      user_id serial4 NOT NULL,
      username varchar NOT NULL,
      email varchar NULL,
      "password" varchar NOT NULL,
      active bool NOT NULL,
      created_at timestamptz NOT NULL,
      updated_at timestamptz NOT NULL,
      logged_at timestamptz NULL,
      "configuration" jsonb NULL,
      ldap_id uuid NULL,
      timezone varchar DEFAULT 'Etc/UTC'::character varying NOT NULL,
      CONSTRAINT "PK_96aac72f1574b88752e9fb00089" PRIMARY KEY (user_id)
    );`);
        await queryRunner.query(`INSERT INTO public.users (user_id, username, email, "password", active, "configuration", timezone, created_at, updated_at) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9)`, [
            defaultUser.user_id,
            defaultUser.username,
            defaultUser.email,
            defaultUser.password,
            defaultUser.active,
            defaultUser.configuration,
            defaultUser.timezone,
            new Date().toISOString(),
            new Date().toISOString()
        ]);
        if (newInstall) {
            await queryRunner.query(`
            CREATE TABLE public.classes (
                "class" varchar NOT NULL,
                "label" varchar NOT NULL,
                class_order int4 NULL,
                CONSTRAINT "PK_073307fe24a1eaabd138574eb10" PRIMARY KEY (class)
            );
        `);
            await queryRunner.query(`
            CREATE TABLE public.hw_types (
                hw_type varchar NOT NULL,
                "label" varchar NOT NULL,
                "class" varchar NULL DEFAULT 'virtualization'::character varying,
                hw_type_order int4 NULL,
                CONSTRAINT "PK_b83bd925b1927d185bebac068dd" PRIMARY KEY (hw_type)
            );

            ALTER TABLE public.hw_types ADD CONSTRAINT "FK_415b80f0ceb1ac8eabec1418cc0" FOREIGN KEY ("class") REFERENCES public.classes("class") ON DELETE CASCADE;
        `);
            await queryRunner.query(`
            CREATE TABLE public.hostcfg (
                hostcfg_id varchar NOT NULL,
                "label" varchar NOT NULL,
                hw_type varchar NOT NULL,
                "data" jsonb NOT NULL,
                created_at timestamptz NOT NULL DEFAULT now(),
                updated_at timestamptz NOT NULL DEFAULT now(),
                disabled bool NOT NULL DEFAULT false,
                ignore_health_status bool NULL DEFAULT false,
                ignore_health_status_reason varchar NULL,
                target bool NOT NULL DEFAULT false,
                CONSTRAINT "PK_173e6c7efac7a86090905c81847" PRIMARY KEY (hostcfg_id)
            );

            ALTER TABLE public.hostcfg ADD CONSTRAINT "FK_c62802fd2c01144f94fa16ad3db" FOREIGN KEY (hw_type) REFERENCES public.hw_types(hw_type) ON DELETE CASCADE;
        `);
            await queryRunner.query(`
            CREATE TABLE public.settings (
                "key" varchar NOT NULL,
                value varchar NULL,
                created_at timestamptz NOT NULL DEFAULT now(),
                updated_at timestamptz NOT NULL DEFAULT now(),
                CONSTRAINT "PK_2a62e3fe17e7f6f24e162d7431b" PRIMARY KEY (key)
            );
        `);
            await queryRunner.query(`
            INSERT INTO public.settings ("key",value,created_at,updated_at) VALUES
                ('ALERTING_REPEAT_TIME','60','2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('ALERTING_PEAK_TIME','30','2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('ALERTING_INCLUDE_GRAPHS','25','2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('ALERTING_NAGIOS','0','2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('ALERTING_SCRIPT',NULL,'2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('ALERTING_XORMON_URL',NULL,'2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('ALERTING_ALARMONE_URL',NULL,'2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('GLOB_TOKEN_EXPIRATION','30','2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('GLOB_SNMP_COMMUNITY',NULL,'2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('GLOB_SNMP_TRAP_HOST',NULL,'2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('SAN_CISCO_OPERATIONAL_STATUS','{"adminDown":"yellow","adminStateConfigChange":"yellow","authzPending":"yellow","badFramesRcvdFromLink":"yellow","bitErrRTThresExceeded":"red","bundleMisCfg":"yellow","bundleStandby":"yellow","cdpInfoUnavailable":"yellow","cellularModemUnattached":"yellow"}','2022-12-05 12:02:28.100679+01','2022-12-05 12:02:28.100679+01'),
                ('GLOB_TOP_ITEMS','100','2024-02-29 14:08:07.139901+01','2024-02-29 14:08:07.139901+01'),
                ('SMTP_HOST',NULL,'2024-02-29 14:08:07.142504+01','2024-02-29 14:08:07.142504+01'),
                ('SMTP_PORT',NULL,'2024-02-29 14:08:07.142504+01','2024-02-29 14:08:07.142504+01'),
                ('SMTP_USER',NULL,'2024-02-29 14:08:07.142504+01','2024-02-29 14:08:07.142504+01'),
                ('SMTP_PASS',NULL,'2024-02-29 14:08:07.142504+01','2024-02-29 14:08:07.142504+01'),
                ('SMTP_ENCRYPTION',NULL,'2024-02-29 14:08:07.142504+01','2024-02-29 14:08:07.142504+01'),
                ('GLOB_PUBLIC_URL',NULL,'2024-02-29 14:08:07.151228+01','2024-02-29 14:08:07.151228+01'),
                ('SMTP_MAIL_FROM','xormon@localhost','2024-02-29 14:08:07.039969+01','2024-02-29 14:08:07.039969+01'),
                ('GLOB_VMWARE_SDK','0','2024-02-29 14:08:15.122009+01','2024-02-29 14:08:15.122009+01');
        `);
        }
        await queryRunner.query(`CREATE TABLE public.microservices (
      microservice varchar NOT NULL,
      script varchar NOT NULL,
      host varchar NOT NULL,
      port varchar NOT NULL,
      hw_type varchar NULL,
      "onStartup" bool DEFAULT false NOT NULL,
      "needConf" bool DEFAULT false NOT NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      "label" varchar DEFAULT '"Unlabeled"'::character varying NOT NULL,
      CONSTRAINT "PK_05ecbaa79debe30d2e829b61730" PRIMARY KEY (microservice),
      CONSTRAINT "FK_b6cdbd55aef16fd39b4431ba2d1" FOREIGN KEY (hw_type) REFERENCES public.hw_types(hw_type) ON DELETE CASCADE
    );
    `);
        for (const microservice of newMicroservicesTable) {
            await queryRunner.query(`INSERT INTO public.microservices (microservice, script, host, port, hw_type, "onStartup", "needConf", updated_at, "label") VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9)`, [
                microservice.microservice,
                microservice.script,
                microservice.host,
                microservice.port,
                microservice.hw_type,
                microservice.onStartup,
                microservice.needConf,
                microservice.updated_at,
                microservice.label,
            ]);
        }
    }
    async down(queryRunner) { }
}
exports.DatabaseCleanup1711444660073 = DatabaseCleanup1711444660073;
//# sourceMappingURL=1711444660073-DatabaseCleanup.migration.js.map