"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InitDatabaseVersion11711445104900 = void 0;
const Settings_1 = require("../../service/configuration/Settings");
class InitDatabaseVersion11711445104900 {
    constructor() {
        this.name = 'InitDatabaseVersion11711445104900';
    }
    async up(queryRunner) {
        const enums = {
            'backend_type_enum': `CREATE TYPE backend_type_enum AS ENUM ('master', 'slave');`,
            'ldap_protocol_v1': `CREATE TYPE public.ldap_protocol_v1 AS ENUM ('ldap','ldaps');`,
            'acl_type_enum': `CREATE TYPE public.acl_type_enum AS ENUM ('granted','class','hwtype','id','subsys');`,
            'admin_groups_type_enum': `CREATE TYPE public.admin_groups_type_enum AS ENUM ('granted','class','hwtype');`,
            'top_interval_v1': `CREATE TYPE public.top_interval_v1 AS ENUM ('day','week','month','year');`,
            'dashboard_type_v1': `CREATE TYPE public.dashboard_type_v1 AS ENUM ('personal','global');`,
            'dashboard_interval_v1': `CREATE TYPE public.dashboard_interval_v1 AS ENUM ('day','week','month','year');`,
            'export_format_v1': `CREATE TYPE public.export_format_v1 AS ENUM ('json','csv','xlsx');`,
            'export_type_v1': `CREATE TYPE public.export_type_v1 AS ENUM ('timeserie','configuration','alerting','mapping','top','metrics','timeseries');`,
            'hs_v1': `CREATE TYPE public.hs_v1 AS ENUM ('ok','warning','error','unknown');`,
            'origin_v1': `CREATE TYPE public.origin_v1 AS ENUM ('child','parent','undefined');`,
            'status_v1': `CREATE TYPE public.status_v1 AS ENUM ('successful','error','unauthorized','running','unknown');`,
            'report_item_type_enum': `CREATE TYPE public.report_item_type_enum AS ENUM ('metric','custom_group','configuration','health_status','text','top');`,
            'report_item_format_enum': `CREATE TYPE public.report_item_format_enum AS ENUM ('graph','table','text');`
        };
        const enumExists = async (enumName) => {
            return (await queryRunner.query(`SELECT 1 as exists FROM pg_type WHERE typname = '${enumName}'`))[0] !== undefined;
        };
        for (const _enum in enums) {
            if (!await enumExists(_enum)) {
                await queryRunner.query(enums[_enum]);
            }
        }
        await queryRunner.query(`CREATE TABLE public.aggregation (
      "table" varchar NOT NULL,
      "time" timestamptz NOT NULL,
      CONSTRAINT "PK_0f2319940d9acbb446776b613e8" PRIMARY KEY ("table")
    );`);
        await queryRunner.query(`

    CREATE TABLE public.backend (
      "uuid" varchar NOT NULL,
      started int4 NOT NULL,
      echoed int4 NOT NULL,
      "type" public."backend_type_enum" DEFAULT 'slave'::backend_type_enum NOT NULL,
      ip varchar NULL,
      "version" varchar NULL,
      CONSTRAINT "PK_a5128c7d697572a9354eda08cdd" PRIMARY KEY (uuid)
    );`);
        await queryRunner.query(`CREATE TABLE public.bucketing (
      hw_type varchar NOT NULL,
      subsystem varchar NOT NULL,
      "time" timestamp NOT NULL,
      CONSTRAINT "PK_c623163d92d7458c8c065ba20fa" PRIMARY KEY (hw_type, subsystem)
    );`);
        await queryRunner.query(`CREATE TABLE public.consistency_groups (
      consistency_group_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      "mode" varchar NULL,
      status varchar NULL,
      syncing_lvl varchar NULL,
      "role" varchar NULL,
      CONSTRAINT "PK_67f98dde880049bbce16727e577" PRIMARY KEY (consistency_group_id)
    );`);
        await queryRunner.query(`CREATE TABLE public.custom_task (
      custom_task_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "name" varchar NOT NULL,
      file_path varchar NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      last_run_at timestamptz NULL,
      CONSTRAINT "PK_7045517f2ec0583edacf5ecb12f" PRIMARY KEY (custom_task_id)
    );`);
        await queryRunner.query(`CREATE TABLE public.downsampling (
      hw_type varchar NOT NULL,
      subsystem varchar NOT NULL,
      "time" timestamp NOT NULL,
      CONSTRAINT "PK_f520e76037349275dc97e67c6a2" PRIMARY KEY (hw_type, subsystem)
    );`);
        await queryRunner.query(`CREATE TABLE public."groups" (
      group_id serial4 NOT NULL,
      "name" varchar NOT NULL,
      description varchar NULL,
      CONSTRAINT "PK_7cfd923277f6ef9f89b04c60436" PRIMARY KEY (group_id)
    );
    `);
        await queryRunner.query(`INSERT INTO public."groups" (group_id, "name", description) VALUES (1, 'Admin', 'Admin');`);
        await queryRunner.query(`CREATE TABLE public.heatmap_tiers (
      heatmap_tier_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      limit_pool int4 NOT NULL,
      limit_volume int4 NOT NULL,
      read_intervals jsonb NOT NULL,
      write_intervals jsonb NOT NULL,
      CONSTRAINT "PK_26cd746aa380f3e5b33c3b2cac9" PRIMARY KEY (heatmap_tier_id)
    );`);
        await queryRunner.query(`CREATE TABLE public.last_timestamps (
      item_id varchar NOT NULL,
      hw_type varchar NOT NULL,
      subsystem varchar NOT NULL,
      updated_at timestamp NOT NULL,
      CONSTRAINT last_timestamps_pk PRIMARY KEY (item_id)
    );`);
        await queryRunner.query(`CREATE TABLE public.ldaps (
      "label" varchar NOT NULL,
      full_dn_auth boolean NULL,
      url _text NOT NULL,
      base_dn varchar NOT NULL,
      group_base varchar NULL,
      group_object_class varchar NOT NULL,
      group_filter_attribute varchar NOT NULL,
      user_base varchar NULL,
      user_object_class varchar NOT NULL,
      user_filter_attribute varchar NOT NULL,
      required_user_group varchar NULL,
      ldap_protocol public."ldap_protocol_v1" DEFAULT 'ldap'::ldap_protocol_v1 NOT NULL,
      ldap_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      user_prefix varchar NULL,
      user_postfix varchar NULL,
      CONSTRAINT "PK_0dec8b1363e8c928a81f2823544" PRIMARY KEY (ldap_id)
    );`);
        await queryRunner.query(`
        ALTER TABLE public.users
        ADD CONSTRAINT "FK_cf32ed857491eb97b29d85e3f32"
        FOREIGN KEY (ldap_id)
        REFERENCES public.ldaps(ldap_id)
        ON DELETE CASCADE;
      `);
        await queryRunner.query(`CREATE TABLE public.sessions (
      sid varchar NOT NULL,
      sess json NOT NULL,
      expire timestamp(6) NOT NULL,
      CONSTRAINT session_pkey PRIMARY KEY (sid)
    );`);
        await queryRunner.query(`CREATE INDEX "IDX_session_expire" ON public.sessions USING btree (expire);`);
        const lKey = await Settings_1.Settings.generateLKey();
        await queryRunner.query(`
      INSERT INTO public.settings ("key",value) VALUES
        ('GLOB_LKEY','${lKey}');
    `);
        await queryRunner.query(`CREATE TABLE public.target_destinations (
      target_destination_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      destination varchar NOT NULL,
      "token" varchar NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      port int4 NOT NULL,
      url varchar NULL,
      CONSTRAINT "PK_37ed14fbd1798e691fca02259dc" PRIMARY KEY (target_destination_id)
    );`);
        await queryRunner.query(`CREATE TABLE public.versions (
      upgraded_at timestamp DEFAULT now() NOT NULL,
      id serial4 NOT NULL,
      "version" varchar NOT NULL,
      CONSTRAINT "PK_921e9a820c96cc2cd7d4b3a107b" PRIMARY KEY (id)
    );`);
        await queryRunner.query(`CREATE TABLE public.acl (
      rule_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "type" public."acl_type_enum" NOT NULL,
      value0 varchar NULL,
      value1 varchar NULL,
      value2 varchar NULL,
      "groupGroupId" int4 NULL,
      CONSTRAINT "PK_7826db84867d4d8ccbe265b2604" PRIMARY KEY (rule_id),
      CONSTRAINT "FK_4fd01957abfed5a6f414daede1f" FOREIGN KEY ("groupGroupId") REFERENCES public."groups"(group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.admin_groups (
      rule_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "type" public."admin_groups_type_enum" NOT NULL,
      value varchar NULL,
      "groupGroupId" int4 NULL,
      CONSTRAINT "PK_39032856e1180f4bf2ddc7759b0" PRIMARY KEY (rule_id),
      CONSTRAINT "FK_d3abfd7630936ad6204a4c88783" FOREIGN KEY ("groupGroupId") REFERENCES public."groups"(group_id)
    );`);
        await queryRunner.query(`CREATE TABLE public.ldap_groups (
      ldap_group varchar NOT NULL,
      group_id int4 NULL,
      ldap_id uuid NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      CONSTRAINT "PK_d798de434ac54163e9fa7eaff89" PRIMARY KEY (ldap_group, ldap_id),
      CONSTRAINT "FK_1e44bb70f3a53d6b3ec13a75b2e" FOREIGN KEY (group_id) REFERENCES public."groups"(group_id) ON DELETE SET NULL,
      CONSTRAINT "FK_d97e74ef4ad7dbf58e7076ffd5f" FOREIGN KEY (ldap_id) REFERENCES public.ldaps(ldap_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.object_items (
      item_id varchar NOT NULL,
      "label" varchar NOT NULL,
      object_id varchar NOT NULL,
      hw_type varchar NOT NULL,
      subsystem varchar NOT NULL,
      has_agent bool DEFAULT false NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      alias varchar NULL,
      CONSTRAINT "PK_4433d14fb839775f030587b5279" PRIMARY KEY (item_id),
      CONSTRAINT "FK_ead64e75ab2eb1495ea94fe74ee" FOREIGN KEY (hw_type) REFERENCES public.hw_types(hw_type) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE INDEX "IDX_64fd58138682fbb4492cf9faef" ON public.object_items USING btree (hw_type, subsystem);`);
        await queryRunner.query(`CREATE TABLE public.storage_mappings (
      item_id varchar NOT NULL,
      wwn varchar NOT NULL,
      target bool DEFAULT false NOT NULL,
      CONSTRAINT "PK_fb10e712dce2cdc510498c88f04" PRIMARY KEY (item_id, wwn),
      CONSTRAINT "FK_7768b4bbec9346e646870c3ec26" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.subsystems (
      hw_type varchar NOT NULL,
      subsystem varchar NOT NULL,
      "label" varchar NULL,
      properties jsonb DEFAULT '{}'::jsonb NOT NULL,
      CONSTRAINT "PK_7e9ece02a4139098f3b4930500e" PRIMARY KEY (hw_type, subsystem),
      CONSTRAINT "FK_6f79908fc7f66e6f1164eec39ed" FOREIGN KEY (hw_type) REFERENCES public.hw_types(hw_type) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.tops (
      item_id varchar NOT NULL,
      metric varchar NOT NULL,
      "interval" public."top_interval_v1" NOT NULL,
      avg float4 NOT NULL,
      max float4 NOT NULL,
      updated_at timestamp DEFAULT now() NOT NULL,
      CONSTRAINT "PK_e716d2bd3a918c411d342807eb5" PRIMARY KEY (item_id, metric, "interval"),
      CONSTRAINT "FK_9041a99ffd988ed50f7ad2fa528" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.users_groups (
      user_id int4 NOT NULL,
      group_id int4 NOT NULL,
      CONSTRAINT "PK_4c4a5923bbe3439dfc72dcfef4c" PRIMARY KEY (user_id, group_id),
      CONSTRAINT "FK_3f4a7469c59e1c47a02a4f9ac50" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE,
      CONSTRAINT "FK_d665a3539878a2669c5ff26966c" FOREIGN KEY (group_id) REFERENCES public."groups"(group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE INDEX "IDX_3f4a7469c59e1c47a02a4f9ac5" ON public.users_groups USING btree (user_id);`);
        await queryRunner.query(`CREATE INDEX "IDX_d665a3539878a2669c5ff26966" ON public.users_groups USING btree (group_id);`);
        await queryRunner.query(`INSERT INTO public.users_groups (user_id, group_id) VALUES (1, 1);`);
        await queryRunner.query(`CREATE TABLE public.agent_mappings (
      item_id varchar NOT NULL,
      agent_id varchar NOT NULL,
      CONSTRAINT "PK_89ff61acd1b3e0a657f0b13e374" PRIMARY KEY (item_id, agent_id),
      CONSTRAINT "FK_1faefedb3d3daabd3f9502347df" FOREIGN KEY (agent_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE,
      CONSTRAINT "FK_87c7b5514ab301767bb016ed105" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.audits (
      audit_id serial4 NOT NULL,
      user_id int4 NOT NULL,
      "time" timestamptz DEFAULT now() NOT NULL,
      "event" int4 NOT NULL,
      affected varchar NULL,
      ip varchar NULL,
      CONSTRAINT "PK_ef298e005292e91adee4b3aa5e7" PRIMARY KEY (audit_id),
      CONSTRAINT "FK_71dbdcdf7b96628ee77e50b6ca1" FOREIGN KEY ("event") REFERENCES public.audit_events(audit_event_id) ON DELETE CASCADE,
      CONSTRAINT "FK_96b1bc5c4ff586cff3f85bc91fb" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.custom_group_folders (
      custom_group_folder_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      "parentCustomGroupFolderId" uuid NULL,
      mpath varchar DEFAULT ''::character varying NULL,
      user_id int4 NULL,
      CONSTRAINT "PK_ce3ff23962515e0087eae07ec5b" PRIMARY KEY (custom_group_folder_id),
      CONSTRAINT "FK_228f49089413202569da4545d92" FOREIGN KEY ("parentCustomGroupFolderId") REFERENCES public.custom_group_folders(custom_group_folder_id) ON DELETE CASCADE,
      CONSTRAINT "FK_579aac55db9a7f49f6ad36ac87d" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.custom_groups (
      custom_group_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      hw_type varchar NULL,
      subsystem varchar NOT NULL,
      custom_group_folder_id uuid NULL,
      user_id int4 NULL,
      internal bool DEFAULT false NOT NULL,
      "class" varchar NULL,
      CONSTRAINT "PK_a3a4c859c20f95f70c588d8e486" PRIMARY KEY (custom_group_id),
      CONSTRAINT "FK_1346a542511c51f794d4fbc0418" FOREIGN KEY (hw_type) REFERENCES public.hw_types(hw_type) ON DELETE CASCADE,
      CONSTRAINT "FK_59140ea4407dbf22c9f4ea0d44c" FOREIGN KEY (custom_group_folder_id) REFERENCES public.custom_group_folders(custom_group_folder_id) ON DELETE CASCADE,
      CONSTRAINT "FK_662464d7f23fd7a76fa7f23052d" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE,
      CONSTRAINT "FK_7ec5b32ed5e21c886de97d777e7" FOREIGN KEY ("class") REFERENCES public.classes("class") ON DELETE CASCADE
    );
    `);
        await queryRunner.query(`CREATE TABLE public.dashboard_folders (
      dashboard_folder_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      mpath varchar DEFAULT ''::character varying NULL,
      "parentDashboardFolderId" uuid NULL,
      "type" public."dashboard_type_v1" DEFAULT 'personal'::dashboard_type_v1 NOT NULL,
      user_id int4 NULL,
      CONSTRAINT "PK_e834e3fe94dfa56e0541cbcd899" PRIMARY KEY (dashboard_folder_id),
      CONSTRAINT "FK_aeffd02a7692e56ee7a179ee024" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE,
      CONSTRAINT "FK_cc8633d896874d22acbb3fb77e9" FOREIGN KEY ("parentDashboardFolderId") REFERENCES public.dashboard_folders(dashboard_folder_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.dashboards (
      dashboard_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      parent uuid NULL,
      "type" public."dashboard_type_v1" DEFAULT 'personal'::dashboard_type_v1 NOT NULL,
      "default" bool DEFAULT false NOT NULL,
      user_id int4 NULL,
      width int2 NULL,
      CONSTRAINT "PK_491f4eaecd2828241aa9b1b4452" PRIMARY KEY (dashboard_id),
      CONSTRAINT "FK_03de56d990ad3d68ae274b2518f" FOREIGN KEY (parent) REFERENCES public.dashboard_folders(dashboard_folder_id) ON DELETE CASCADE,
      CONSTRAINT "FK_6a4f2a44f108ed840a13c1c6fee" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.group_ldap_groups (
      group_id int4 NOT NULL,
      ldap_id uuid NOT NULL,
      ldap_group varchar NOT NULL,
      CONSTRAINT "PK_08afe497f4482732cae09f53084" PRIMARY KEY (group_id, ldap_id, ldap_group),
      CONSTRAINT "FK_1cb2e3a2472aa39a45b97c9d350" FOREIGN KEY (ldap_id,ldap_group) REFERENCES public.ldap_groups(ldap_id,ldap_group) ON DELETE CASCADE,
      CONSTRAINT "FK_3fb9505db557c277d2d495b3803" FOREIGN KEY (group_id) REFERENCES public."groups"(group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.health_status (
      item_id varchar NOT NULL,
      status public."hs_v1" DEFAULT 'unknown'::hs_v1 NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      "data" varchar NULL,
      CONSTRAINT "PK_99e3a748292ea5203b62e0b69b6" PRIMARY KEY (item_id),
      CONSTRAINT "FK_99e3a748292ea5203b62e0b69b6" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.health_status_history (
      health_status_history_id serial4 NOT NULL,
      item_id varchar NOT NULL,
      status public."hs_v1" DEFAULT 'unknown'::hs_v1 NOT NULL,
      reason varchar NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      ended_at timestamptz NULL,
      CONSTRAINT "PK_e60f87b2d1350445e9b60dc7dfb" PRIMARY KEY (health_status_history_id),
      CONSTRAINT "FK_ba60236d9a64e0a200aa211583c" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.health_status_mirror (
      source_item varchar NOT NULL,
      target_item varchar NOT NULL,
      consistency_group_id uuid NULL,
      "type" varchar NOT NULL,
      state varchar NOT NULL,
      freeze_time int4 NULL,
      created_at timestamp DEFAULT now() NOT NULL,
      updated_at timestamp NULL,
      CONSTRAINT "PK_57a009886f97c8cc3d7608eb40a" PRIMARY KEY (source_item, target_item),
      CONSTRAINT "FK_HEALTH_STATUS_MIRROR_ON_CONSISTENCY_GROUP" FOREIGN KEY (consistency_group_id) REFERENCES public.consistency_groups(consistency_group_id),
      CONSTRAINT "FK_HEALTH_STATUS_MIRROR_ON_SOURCE_OBJECT_ITEM" FOREIGN KEY (source_item) REFERENCES public.object_items(item_id),
      CONSTRAINT "FK_HEALTH_STATUS_MIRROR_ON_TARGET_OBJECT_ITEM" FOREIGN KEY (target_item) REFERENCES public.object_items(item_id)
    );`);
        await queryRunner.query(`CREATE TABLE public.health_status_mirror_history (
      health_status_mirror_history_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      source_item varchar NOT NULL,
      target_item varchar NOT NULL,
      state varchar NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      ended_at timestamptz NULL,
      CONSTRAINT "PK_349b1e4bfa237a61a15a66a05ae" PRIMARY KEY (health_status_mirror_history_id),
      CONSTRAINT "FK_3c67400034d6f889e996bd3993d" FOREIGN KEY (source_item) REFERENCES public.object_items(item_id) ON DELETE CASCADE,
      CONSTRAINT "FK_aa2e2a9cc28b9b3db0a15bcc4af" FOREIGN KEY (target_item) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.heatmap_tier_items (
      heatmap_tier_id uuid NOT NULL,
      item_id varchar NOT NULL,
      CONSTRAINT "PK_fd5ba10bc9b6bc309131f303deb" PRIMARY KEY (item_id),
      CONSTRAINT "UQ_fd5ba10bc9b6bc309131f303deb" UNIQUE (item_id),
      CONSTRAINT "FK_6361bc083872cf993397bdc1192" FOREIGN KEY (heatmap_tier_id) REFERENCES public.heatmap_tiers(heatmap_tier_id) ON DELETE CASCADE,
      CONSTRAINT "FK_fd5ba10bc9b6bc309131f303deb" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.heatmaps (
      item_id varchar NOT NULL,
      metric varchar NOT NULL,
      value float8 NULL,
      "label" varchar NULL,
      device varchar NULL,
      subsystem varchar NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      CONSTRAINT "PK_e1ae9edea7c366000ae25c3a772" PRIMARY KEY (item_id, metric),
      CONSTRAINT "FK_d292809606a669761924c2ce98d" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.hostcfg_relations (
      hostcfg_id varchar NOT NULL,
      item_id varchar NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      CONSTRAINT "PK_45eb27880248f189548ccee93fa" PRIMARY KEY (hostcfg_id, item_id),
      CONSTRAINT "FK_5e5d081665f50db0e3f04650b2b" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE,
      CONSTRAINT "FK_80cc78d30b6874218d08720b160" FOREIGN KEY (hostcfg_id) REFERENCES public.hostcfg(hostcfg_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.hostcfg_status (
      hostcfg_id varchar NOT NULL,
      "connection" bool NULL,
      "data" bool NULL,
      message varchar NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      last_data timestamptz NULL,
      perf_triggered bool DEFAULT false NOT NULL,
      CONSTRAINT "PK_1a22892401284c3ba6c922f235c" PRIMARY KEY (hostcfg_id),
      CONSTRAINT "FK_1a22892401284c3ba6c922f235c" FOREIGN KEY (hostcfg_id) REFERENCES public.hostcfg(hostcfg_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.item_groups (
      item_group_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      custom_group_id uuid NULL,
      regex _text NULL,
      "intersection" bool DEFAULT false NOT NULL,
      CONSTRAINT "PK_1ae85f6f708b9f1a0394607dd44" PRIMARY KEY (item_group_id),
      CONSTRAINT "FK_d19383451abfe7845ec418ea8a2" FOREIGN KEY (custom_group_id) REFERENCES public.custom_groups(custom_group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.item_properties (
      item_id varchar NOT NULL,
      property_name varchar NOT NULL,
      property_value varchar NULL,
      CONSTRAINT "PK_54fa3c3a5cb0619c1a8cb24a13e" PRIMARY KEY (item_id, property_name),
      CONSTRAINT "FK_3278f31c56fea0c4ae769f5167e" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.item_relations (
      parent varchar NOT NULL,
      child varchar NOT NULL,
      origin public."origin_v1" DEFAULT 'undefined'::origin_v1 NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      CONSTRAINT "PK_5e60bf1241502d75f8e480b8b89" PRIMARY KEY (parent, child),
      CONSTRAINT "FK_142c8dd549586c659ab97ab8c07" FOREIGN KEY (child) REFERENCES public.object_items(item_id) ON DELETE CASCADE,
      CONSTRAINT "FK_e469d55961845e794cc353079d8" FOREIGN KEY (parent) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.mail_groups (
      mail_group_id serial4 NOT NULL,
      "label" varchar NOT NULL,
      mails jsonb NOT NULL,
      user_id int4 NULL,
      CONSTRAINT "PK_15163a240942016faa962851ae4" PRIMARY KEY (mail_group_id),
      CONSTRAINT "FK_e62dd184bb551741cef938cb39d" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.microservice_data (
      microservice varchar NOT NULL,
      value jsonb NOT NULL,
      hostcfg_id varchar NOT NULL,
      CONSTRAINT "FK_2f4e567a06cd19b383257fe8e08" FOREIGN KEY (microservice) REFERENCES public.microservices(microservice) ON DELETE CASCADE,
      CONSTRAINT "FK_f75e84439001726e3fb22290da3" FOREIGN KEY (hostcfg_id) REFERENCES public.hostcfg(hostcfg_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.microservice_job_status (
      microservice varchar NOT NULL,
      job varchar NOT NULL,
      identification varchar NOT NULL,
      "start" int4 NULL,
      "end" int4 NULL,
      status public."status_v1" DEFAULT 'unknown'::status_v1 NOT NULL,
      pid int4 NOT NULL,
      CONSTRAINT "PK_baec9deaf5bf3408e45060982f5" PRIMARY KEY (microservice, job, identification, pid),
      CONSTRAINT "FK_5710124c157855eb31d7a28eb31" FOREIGN KEY (microservice) REFERENCES public.microservices(microservice) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.microservice_jobs (
      microservice varchar NOT NULL,
      job varchar NOT NULL,
      "interval" int4 NOT NULL,
      timeout int4 DEFAULT 20 NOT NULL,
      concurrence int4 DEFAULT 3 NOT NULL,
      CONSTRAINT "PK_4d2e505d5c28ede6ce3b4d0eb9d" PRIMARY KEY (microservice, job),
      CONSTRAINT "FK_2f0d32632bf7c82de59acaee1b0" FOREIGN KEY (microservice) REFERENCES public.microservices(microservice) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.microservice_variables (
      microservice_variables_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "key" varchar NOT NULL,
      value varchar NOT NULL,
      hostcfg_id varchar NULL,
      hw_type varchar NULL,
      CONSTRAINT "PK_MICROSERVICE_VARIABLES_PRIMARY" PRIMARY KEY (microservice_variables_id),
      CONSTRAINT "FK_MICROSERVICE_VARIALBES_ON_HOSTCFG" FOREIGN KEY (hostcfg_id) REFERENCES public.hostcfg(hostcfg_id),
      CONSTRAINT "FK_MICROSERVICE_VARIALBES_ON_HWTYPES" FOREIGN KEY (hw_type) REFERENCES public.hw_types(hw_type)
    );`);
        await queryRunner.query(`CREATE TABLE public.reports (
      report_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      recurrence jsonb NULL,
      time_range jsonb NULL,
      disabled bool DEFAULT false NOT NULL,
      on_generate_create_report_file bool DEFAULT true NOT NULL,
      last_run_at timestamptz NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      updated_at timestamptz DEFAULT now() NOT NULL,
      user_id int4 NULL,
      CONSTRAINT "PK__reports__report_id" PRIMARY KEY (report_id),
      CONSTRAINT "FK__users__user_id" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE ON UPDATE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.reports_mail_groups (
      report_id uuid NOT NULL,
      mail_group_id int4 NOT NULL,
      CONSTRAINT "PK__reports_mail_groups__report_id__mail_group_id" PRIMARY KEY (report_id, mail_group_id),
      CONSTRAINT "FK__reports_mail_groups__mail_group_id" FOREIGN KEY (mail_group_id) REFERENCES public.mail_groups(mail_group_id) ON DELETE CASCADE ON UPDATE CASCADE,
      CONSTRAINT "FK__reports_mail_groups__report_id" FOREIGN KEY (report_id) REFERENCES public.reports(report_id) ON DELETE CASCADE ON UPDATE CASCADE
    );`);
        await queryRunner.query(`CREATE INDEX "IDX__reports_mail_groups__mail_group_id" ON public.reports_mail_groups USING btree (mail_group_id);`);
        await queryRunner.query(`CREATE INDEX "IDX__reports_mail_groups__report_id" ON public.reports_mail_groups USING btree (report_id);`);
        await queryRunner.query(`CREATE TABLE public.storage_groups (
      storage_group_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      description varchar NULL,
      user_id int4 NULL,
      CONSTRAINT "PK_f4652d8760638d848e0b4dbdfbe" PRIMARY KEY (storage_group_id),
      CONSTRAINT "FK_d2031e77a5b70b75c108db1ae25" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.tags (
      tag_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      tag varchar NOT NULL,
      created_at timestamp DEFAULT now() NOT NULL,
      updated_at timestamp DEFAULT now() NOT NULL,
      user_id int4 NULL,
      CONSTRAINT "PK_06a35221325edeb80ad2ec1ff85" PRIMARY KEY (tag_id),
      CONSTRAINT "FK_74603743868d1e4f4fc2c0225b6" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE UNIQUE INDEX "IDX_db66121dc39534bfc85341711d" ON public.tags USING btree (tag);`);
        await queryRunner.query(`CREATE TABLE public.target_destination_hostcfgs (
      target_destination_id uuid NOT NULL,
      hostcfg_id varchar NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      CONSTRAINT "PK_54f86b7fc338e5bb86c97b2906c" PRIMARY KEY (target_destination_id, hostcfg_id),
      CONSTRAINT "FK_3f0910ad8b85fef617897fc8900" FOREIGN KEY (target_destination_id) REFERENCES public.target_destinations(target_destination_id) ON DELETE CASCADE,
      CONSTRAINT "FK_4eb9fd7a7774082657946409c74" FOREIGN KEY (hostcfg_id) REFERENCES public.hostcfg(hostcfg_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.user_api_keys (
      api_key varchar NOT NULL,
      user_id int4 NOT NULL,
      expiration_at timestamptz NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      "isTemporary" bool DEFAULT false NOT NULL,
      "label" varchar DEFAULT 'unlabeled'::character(1) NULL,
      CONSTRAINT "PK_63c0369bd14c0dbc46fecc24368" PRIMARY KEY (api_key, user_id),
      CONSTRAINT "FK_42f98f737ddeea944b7b184685c" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.alerting_groups (
      alerting_group_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      subsystem varchar NULL,
      "class" varchar NULL,
      hw_type varchar NULL,
      item_group_id uuid NULL,
      custom_group_id uuid NULL,
      CONSTRAINT "PK_fd0ba0b1e29dbdc57f1eff505c1" PRIMARY KEY (alerting_group_id),
      CONSTRAINT "FK_15bfd2e010e966178866c3588ad" FOREIGN KEY (item_group_id) REFERENCES public.item_groups(item_group_id) ON DELETE CASCADE,
      CONSTRAINT "FK_6c32faf3dedb68b6df71cd34878" FOREIGN KEY (custom_group_id) REFERENCES public.custom_groups(custom_group_id) ON DELETE CASCADE,
      CONSTRAINT "FK_be50b71e3d513e76f0a0d12e492" FOREIGN KEY (hw_type) REFERENCES public.hw_types(hw_type) ON DELETE CASCADE,
      CONSTRAINT "FK_d9fbdd5be041f43cbd5f274b458" FOREIGN KEY ("class") REFERENCES public.classes("class") ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.alerting_hw_emails (
      mail_group_id int4 NOT NULL,
      CONSTRAINT "PK_4c7d66a6c33dbb3c60adf08ee4b" PRIMARY KEY (mail_group_id),
      CONSTRAINT "FK_4c7d66a6c33dbb3c60adf08ee4b" FOREIGN KEY (mail_group_id) REFERENCES public.mail_groups(mail_group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.custom_task_mail_group_relations (
      custom_task_id uuid NOT NULL,
      mail_group_id serial4 NOT NULL,
      CONSTRAINT "FK_2c09ee7141c811a2696d2b01f85" FOREIGN KEY (custom_task_id) REFERENCES public.custom_task(custom_task_id) ON DELETE CASCADE,
      CONSTRAINT "FK_d24756e88d229687a5b4a1de824" FOREIGN KEY (mail_group_id) REFERENCES public.mail_groups(mail_group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.dashboard_groups (
      dashboard_group_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      "label" varchar NOT NULL,
      width int4 NOT NULL,
      height int4 NOT NULL,
      dashboard_id uuid NOT NULL,
      x int4 NOT NULL,
      y int4 NOT NULL,
      CONSTRAINT "PK_2dbd5df6aa9f30fad4ff16a3641" PRIMARY KEY (dashboard_group_id),
      CONSTRAINT "FK_467edc37283acce6f39ca0e4469" FOREIGN KEY (dashboard_id) REFERENCES public.dashboards(dashboard_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.exports (
      "label" varchar DEFAULT 'unlabeled'::character varying NOT NULL,
      "format" public."export_format_v1" DEFAULT 'csv'::export_format_v1 NOT NULL,
      "type" public."export_type_v1" DEFAULT 'timeserie'::export_type_v1 NOT NULL,
      "data" jsonb NOT NULL,
      last_run timestamptz NULL,
      created timestamp DEFAULT now() NOT NULL,
      export_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      recurrence jsonb NULL,
      user_id int4 NULL,
      "class" varchar NULL,
      hw_type varchar NULL,
      subsystem varchar NULL,
      item_group_id uuid NULL,
      custom_group_id uuid NULL,
      CONSTRAINT "PK_4cc1611c90029d9e53586a19410" PRIMARY KEY (export_id),
      CONSTRAINT "FK_37f22f05f793b6617551219c3da" FOREIGN KEY ("class") REFERENCES public.classes("class") ON DELETE CASCADE,
      CONSTRAINT "FK_39849003634bba2e9d3d43f1e65" FOREIGN KEY (hw_type) REFERENCES public.hw_types(hw_type) ON DELETE CASCADE,
      CONSTRAINT "FK_9436a31587064c89086706f4077" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE,
      CONSTRAINT "FK_d6e2ec918191bffec45c5e693ad" FOREIGN KEY (item_group_id) REFERENCES public.item_groups(item_group_id) ON DELETE CASCADE,
      CONSTRAINT "FK_d6e63e7b2e63cbee04517663bf9" FOREIGN KEY (custom_group_id) REFERENCES public.custom_groups(custom_group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.item_group_items (
      item_group_id uuid NOT NULL,
      item_id varchar NOT NULL,
      CONSTRAINT "PK_53ffecbdd94c2cc8ec3e3816760" PRIMARY KEY (item_group_id, item_id),
      CONSTRAINT "FK_56486e5b1818faeec9671e092d5" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE,
      CONSTRAINT "FK_606a06a0c5ad16245cc6559341c" FOREIGN KEY (item_group_id) REFERENCES public.item_groups(item_group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.item_group_parent_tags (
      item_group_id uuid NOT NULL,
      tag_id uuid NOT NULL,
      CONSTRAINT "PK_617cad35b0df69d6a6a7b0cb1ce" PRIMARY KEY (item_group_id, tag_id),
      CONSTRAINT "FK_c868e355c66f37f203a08742009" FOREIGN KEY (tag_id) REFERENCES public.tags(tag_id) ON DELETE CASCADE,
      CONSTRAINT "FK_d95b2b6b526953e8c14dc4227ab" FOREIGN KEY (item_group_id) REFERENCES public.item_groups(item_group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.item_group_parents (
      item_group_id uuid NOT NULL,
      item_id varchar NOT NULL,
      CONSTRAINT "PK_8e098e5d7a9d0c83b5856002bb1" PRIMARY KEY (item_group_id, item_id),
      CONSTRAINT "FK_18f674c82b6e4faf84411a11347" FOREIGN KEY (item_group_id) REFERENCES public.item_groups(item_group_id) ON DELETE CASCADE,
      CONSTRAINT "FK_7c364ee183bed9e60195d583bb7" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.item_group_regex_parents (
      item_group_regex_parent_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      item_group_id uuid NOT NULL,
      subsystem varchar NOT NULL,
      regex varchar NOT NULL,
      CONSTRAINT "PK_a09cdade0732641a22e7f67e82a" PRIMARY KEY (item_group_regex_parent_id, item_group_id),
      CONSTRAINT "FK_b800102004f1dc3551ebefdf37c" FOREIGN KEY (item_group_id) REFERENCES public.item_groups(item_group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.item_group_tags (
      item_group_id uuid NOT NULL,
      tag_id uuid NOT NULL,
      CONSTRAINT "PK_65f4446fb78e33606dcd9a7fe23" PRIMARY KEY (item_group_id, tag_id),
      CONSTRAINT "FK_29aee9193addc263fd86d1b1df1" FOREIGN KEY (tag_id) REFERENCES public.tags(tag_id) ON DELETE CASCADE,
      CONSTRAINT "FK_9285dd8f4a1cb608fc9182205cf" FOREIGN KEY (item_group_id) REFERENCES public.item_groups(item_group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.report_files (
      report_file_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      report_id uuid NOT NULL,
      file_name varchar NOT NULL,
      relative_path varchar NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      CONSTRAINT "PK__report_files__report_file_id" PRIMARY KEY (report_file_id),
      CONSTRAINT "FK__report_files__report_id" FOREIGN KEY (report_id) REFERENCES public.reports(report_id) ON DELETE CASCADE ON UPDATE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.report_items (
      report_item_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      report_id uuid NOT NULL,
      "format" public."report_item_format_enum" NOT NULL,
      "type" public."report_item_type_enum" NOT NULL,
      "data" jsonb DEFAULT '{}'::jsonb NOT NULL,
      item_index int4 NULL,
      "class" varchar NULL,
      hw_type varchar NULL,
      subsystem varchar NULL,
      item_group_id uuid NULL,
      custom_group_id uuid NULL,
      CONSTRAINT "PK__report_items__report_item_id" PRIMARY KEY (report_item_id),
      CONSTRAINT "FK_3a07cdd516faf0b46b2c4678ab3" FOREIGN KEY (custom_group_id) REFERENCES public.custom_groups(custom_group_id) ON DELETE CASCADE,
      CONSTRAINT "FK__report_items__report_id" FOREIGN KEY (report_id) REFERENCES public.reports(report_id) ON DELETE CASCADE ON UPDATE CASCADE,
      CONSTRAINT "FK_a166101ea8975145d80e1835c39" FOREIGN KEY (item_group_id) REFERENCES public.item_groups(item_group_id) ON DELETE CASCADE,
      CONSTRAINT "FK_cb30fc6081e2b218044eb68e112" FOREIGN KEY ("class") REFERENCES public.classes("class") ON DELETE CASCADE,
      CONSTRAINT "FK_d8bc2169964c45374f3dac3bc20" FOREIGN KEY (hw_type) REFERENCES public.hw_types(hw_type) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.storage_group_devices (
      storage_group_id uuid NOT NULL,
      item_id varchar NOT NULL,
      created_at timestamptz DEFAULT now() NOT NULL,
      CONSTRAINT "PK_86dec5e75c6054506ecaa6fcef1" PRIMARY KEY (storage_group_id, item_id),
      CONSTRAINT "FK_431ce80a5c035c4d07711d20910" FOREIGN KEY (storage_group_id) REFERENCES public.storage_groups(storage_group_id) ON DELETE CASCADE,
      CONSTRAINT "FK_4fa919f65c214c50eb054ec3551" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.storage_group_regex (
      storage_group_id uuid NOT NULL,
      regex varchar NOT NULL,
      created_at timestamp DEFAULT now() NOT NULL,
      CONSTRAINT "PK_c29bd97e39bd7d8f724c078fc3c" PRIMARY KEY (storage_group_id, regex),
      CONSTRAINT "FK_9257a59c05f8a7fdc8a0a78aa8c" FOREIGN KEY (storage_group_id) REFERENCES public.storage_groups(storage_group_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.tag_items (
      item_id varchar NOT NULL,
      tag_id uuid NOT NULL,
      created_at timestamp DEFAULT now() NOT NULL,
      CONSTRAINT "PK_00d9c8a6cb3dde39c8f5d67d6bb" PRIMARY KEY (item_id, tag_id),
      CONSTRAINT "FK_386f3e442f3c8d961dda5122f09" FOREIGN KEY (tag_id) REFERENCES public.tags(tag_id) ON DELETE CASCADE,
      CONSTRAINT "FK_66c708c8d492a48abed92676070" FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`
    CREATE TABLE public.alerting (
      metric varchar NOT NULL,
      peak int4 NULL,
      repeat int4 NULL,
      exclude_start int4 NULL,
      exclude_end int4 NULL,
      triggered int4 NULL,
      alerting_group_id uuid NOT NULL,
      "limit" float4 NOT NULL,
      "level" varchar DEFAULT 'undefined'::character varying NOT NULL,
      user_id int4 NOT NULL,
      alerting_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      checked_until timestamptz NULL,
      CONSTRAINT "PK_13c603988484d45a8cb4374b241" PRIMARY KEY (alerting_id),
      CONSTRAINT "FK_51e464e3cbbe31c9e210c88d7d9" FOREIGN KEY (alerting_group_id) REFERENCES public.alerting_groups(alerting_group_id) ON DELETE CASCADE,
      CONSTRAINT "FK_6bafada7c9bf1296383a24f4763" FOREIGN KEY (user_id) REFERENCES public.users(user_id) ON DELETE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.alerting_launches (
      alerting_id uuid NOT NULL,
      item_id varchar NOT NULL,
      last_triggered_at timestamptz NOT NULL,
      CONSTRAINT pk__alerting_launches__alerting_id__item_id PRIMARY KEY (alerting_id, item_id),
      CONSTRAINT fk__alerting_launches__alerting_id FOREIGN KEY (alerting_id) REFERENCES public.alerting(alerting_id) ON DELETE CASCADE ON UPDATE CASCADE,
      CONSTRAINT fk__alerting_launches__item_id FOREIGN KEY (item_id) REFERENCES public.object_items(item_id) ON DELETE CASCADE ON UPDATE CASCADE
    );`);
        await queryRunner.query(`CREATE TABLE public.alerting_mail_groups (
      mail_group_id int4 NOT NULL,
      alerting_id uuid NOT NULL,
      CONSTRAINT "PK_c53d53cafe7af5b164c04a82438" PRIMARY KEY (mail_group_id, alerting_id),
      CONSTRAINT "FK_09c7424ce18067a5d903de03779" FOREIGN KEY (alerting_id) REFERENCES public.alerting(alerting_id) ON DELETE CASCADE ON UPDATE CASCADE,
      CONSTRAINT "FK_300193140d6afb57e15f516b6fe" FOREIGN KEY (mail_group_id) REFERENCES public.mail_groups(mail_group_id) ON DELETE CASCADE ON UPDATE CASCADE
    );`);
        await queryRunner.query(`CREATE INDEX "IDX_09c7424ce18067a5d903de0377" ON public.alerting_mail_groups USING btree (alerting_id);`);
        await queryRunner.query(`CREATE INDEX "IDX_300193140d6afb57e15f516b6f" ON public.alerting_mail_groups USING btree (mail_group_id);`);
        await queryRunner.query(`CREATE TABLE public.dashboard_graphs (
      dashboard_graph_id uuid DEFAULT uuid_generate_v4() NOT NULL,
      width int4 NOT NULL,
      height int4 NOT NULL,
      dashboard_group_id uuid NOT NULL,
      url varchar NOT NULL,
      tab varchar NOT NULL,
      x int4 NOT NULL,
      "interval" public."dashboard_interval_v1" DEFAULT 'year'::dashboard_interval_v1 NOT NULL,
      y int4 NOT NULL,
      CONSTRAINT "PK_c7e4c1fe591c6cea7391393988e" PRIMARY KEY (dashboard_graph_id),
      CONSTRAINT "FK_6fd63325184735407c885238d18" FOREIGN KEY (dashboard_group_id) REFERENCES public.dashboard_groups(dashboard_group_id) ON DELETE CASCADE
    );`);
        const dashboard = await queryRunner.query(`
      SELECT * from public.dashboards order by dashboard_id limit 1
    `);
        if (dashboard && dashboard[0]) {
            await queryRunner.query(`
        UPDATE public.dashboards
        SET "default" = true
        WHERE dashboard_id = '${dashboard[0]['dashboard_id']}'
      `);
        }
        else {
            await queryRunner.query(`
        INSERT INTO public.dashboards
        (dashboard_id, "label", parent, "type", "default", user_id)
        VALUES('8ad47092-15d6-469b-9bb6-e44316af225b', 'DefaultDashboard', null, 'global'::dashboard_type_v1, true, 1);

        INSERT INTO public.dashboard_groups
        (dashboard_group_id, "label", width, height, dashboard_id, x, y)
        VALUES('f137510c-71db-456d-b903-5e195ba62252', 'DefaultGroup', 12, 6, '8ad47092-15d6-469b-9bb6-e44316af225b', 0, 0);
      `);
        }
    }
    async down(queryRunner) { }
}
exports.InitDatabaseVersion11711445104900 = InitDatabaseVersion11711445104900;
//# sourceMappingURL=1711445104900-InitDatabaseVersion1.migration.js.map