/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
import { PROTO } from "@xormon/common/hostcfgs/hostcfgData.types";
import { WriteStream } from "fs";
import * as http from "http";
export declare enum RequestMethod {
    POST = "POST",
    GET = "GET",
    PUT = "PUT",
    DELETE = "DELETE",
    PATCH = "PATCH"
}
export type RequestOptions = {
    url: string;
    method: RequestMethod;
    protocol: PROTO;
    body?: string;
    headers?: http.OutgoingHttpHeaders;
    responseEncoding?: BufferEncoding;
    timeout?: number;
};
export type RequestResponse = {
    statusCode?: number;
    body?: string;
    headers?: http.IncomingHttpHeaders;
    error?: any;
};
export declare class RequestBuilder<BodyType, ResponseType = RequestResponse> {
    private proto;
    private url;
    private method;
    private protocol;
    private options;
    private stream;
    private body;
    private responseEncoding;
    constructor(proto: PROTO, url: string, method: RequestMethod);
    addAgent(): this;
    setRequestTimeout(timeoutInMs: number): this;
    setStream(stream: WriteStream): this;
    setHeaders(headers: http.OutgoingHttpHeaders): this;
    addBody(body: BodyType): this;
    setRequestEncoding(encoding: BufferEncoding): this;
    private static defineBaseRequestEvents;
    static simpleRequest(options: RequestOptions, useAgent?: boolean): Promise<RequestResponse>;
    processRequest(): Promise<RequestResponse>;
}
