"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ACLService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const aclResponse_dto_1 = require("../../common/aclResponse.dto");
const aclResponse_constants_1 = require("../../common/constants/aclResponse.constants");
const menuResponse_dto_1 = require("../../common/menuResponse.dto");
const dataSource_1 = require("../../config/dataSource");
const acl_entity_1 = require("../../entity/acl.entity");
const adminGroup_entity_1 = require("../../entity/adminGroup.entity");
const class_entity_1 = require("../../entity/class.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const Architecture_1 = require("../architecture/Architecture");
const typeorm_2 = require("typeorm");
const ACLCheck_1 = require("./ACLCheck");
let ACLService = class ACLService {
    constructor(aclRepository, objectItemsRepository, classesRepository) {
        this.aclRepository = aclRepository;
        this.objectItemsRepository = objectItemsRepository;
        this.classesRepository = classesRepository;
    }
    async getRules(group_id) {
        if (group_id) {
            return await this.aclRepository.find({ relations: { group: true }, where: { group: { group_id: group_id } } });
        }
        else {
            return await this.aclRepository.find({ relations: { group: true } });
        }
    }
    async saveRules(group_id, rules) {
        let saved_rules = [];
        const existing = await this.getRules(group_id);
        for (let rule of rules.data) {
            if (existing.some((r) => {
                const props = ['type', 'value0', 'value1', 'value2'];
                console.dir({ r, rule, same: props.every((p) => (r[p] ?? undefined) === (rule[p] ?? undefined)) });
                return props.every((p) => (r[p] ?? undefined) === (rule[p] ?? undefined));
            })) {
                continue;
            }
            let saved = await this.aclRepository.save({ group: { group_id: group_id }, ...rule });
            saved_rules.push(saved);
        }
        return saved_rules;
    }
    async saveItemRule(menuItem, group_id) {
        let saved;
        if (menuItem.isLeaf) {
            const item = await this.objectItemsRepository.findOneBy({ item_id: menuItem.itemId });
            menuItem.hwType = item.hw_type;
            menuItem.subsystem = item.subsystem;
        }
        if (!menuItem.hwType && !menuItem.subsystem && !menuItem.itemId) {
            const itemClass = await this.getItemClass(menuItem);
            saved = await this.aclRepository.save({ group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.CLASS, value0: itemClass });
        }
        else if (menuItem.hwType && !menuItem.subsystem && !menuItem.itemId) {
            saved = await this.aclRepository.save({ group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.HWTYPE, value0: menuItem.hwType });
        }
        else if (menuItem.hwType && menuItem.subsystem && !menuItem.itemId) {
            if (menuItem.url) {
                let url = new URL('https://do.not.care' + menuItem.url);
                let ids = url.searchParams.get('ids');
                saved = await this.aclRepository.save({ group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.SUBSYS, value0: menuItem.hwType, value1: menuItem.subsystem, value2: ids });
            }
            else {
                saved = await this.aclRepository.save({ group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.SUBSYS, value0: menuItem.hwType, value1: menuItem.subsystem });
            }
        }
        else if (menuItem.hwType && menuItem.subsystem && menuItem.itemId) {
            saved = await this.aclRepository.save({ group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.ID, value0: menuItem.hwType, value1: menuItem.itemId });
        }
        return saved;
    }
    async clearRules() {
        return await this.aclRepository.clear();
    }
    async deleteRules(group_id) {
        const rules = await this.aclRepository.find({ relations: { group: true }, where: { group: { group_id: group_id } } });
        ;
        rules.forEach(async (rule) => await this.aclRepository.delete({ rule_id: rule.rule_id }));
    }
    async deleteItemRules(menuItem, group_id) {
        const promises = [];
        if (menuItem.isLeaf && menuItem.itemId) {
            const item = await this.objectItemsRepository.findOneBy({ item_id: menuItem.itemId });
            menuItem.hwType = item.hw_type;
            menuItem.subsystem = item.subsystem;
        }
        const itemClass = await this.getItemClass(menuItem);
        if (!menuItem.hwType && !menuItem.subsystem && !menuItem.itemId) {
            const rule = await this.aclRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.CLASS, value0: itemClass } });
            if (rule && rule.length > 0) {
                rule.forEach((rule) => promises.push(this.aclRepository.delete({ rule_id: rule.rule_id })));
            }
            const hwTypes = (await Architecture_1.Architecture.getHwTypes(false, itemClass)).map(hwType => hwType.hw_type);
            for (let hwType of hwTypes) {
                const rule = await this.aclRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, value0: hwType } });
                if (rule && rule.length > 0) {
                    rule.forEach((rule) => promises.push(this.aclRepository.delete({ rule_id: rule.rule_id })));
                }
            }
        }
        else if (menuItem.hwType && !menuItem.subsystem && !menuItem.itemId) {
            const rule = await this.aclRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, value0: menuItem.hwType } });
            if (rule && rule.length > 0) {
                rule.forEach((rule) => promises.push(this.aclRepository.delete({ rule_id: rule.rule_id })));
            }
        }
        else if (menuItem.hwType && menuItem.subsystem && !menuItem.itemId) {
            const ruleSubsys = await this.aclRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.SUBSYS, value0: menuItem.hwType, value1: menuItem.subsystem } });
            if (ruleSubsys && ruleSubsys.length > 0) {
                ruleSubsys.forEach((ruleSubsys) => promises.push(this.aclRepository.delete({ rule_id: ruleSubsys.rule_id })));
            }
            const items = (await this.objectItemsRepository.findBy({ subsystem: menuItem.subsystem })).map(item => item.item_id);
            const ruleItems = await this.aclRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.ID, value0: menuItem.hwType, value1: (0, typeorm_2.In)(items) } });
            if (ruleItems && ruleItems.length > 0) {
                ruleItems.forEach((ruleItems) => promises.push(this.aclRepository.delete({ rule_id: ruleItems.rule_id })));
            }
        }
        else if (menuItem.itemId) {
            const rule = await this.aclRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.ID, value0: menuItem.hwType, value1: menuItem.itemId } });
            if (rule && rule.length > 0) {
                rule.forEach((rule) => promises.push(this.aclRepository.delete({ rule_id: rule.rule_id })));
            }
        }
        else {
        }
        await Promise.allSettled(promises);
        return true;
    }
    async createAdminRolesRules(group_id) {
        const adminRolesRepository = (0, dataSource_1.getDataSource)().getRepository(adminGroup_entity_1.AdminGroup);
        const adminRoleRules = await adminRolesRepository.find({ relations: { group: true }, where: { group: { group_id: group_id } } });
        if (adminRoleRules && adminRoleRules.length > 0) {
            let aclRules = [];
            for (let rule of adminRoleRules) {
                if (rule.type === aclResponse_constants_1.RULE_TYPE.CLASS || rule.type === aclResponse_constants_1.RULE_TYPE.HWTYPE) {
                    aclRules.push({
                        type: rule.type,
                        value0: rule.value
                    });
                }
            }
            await this.saveRules(group_id, { statusCode: common_1.HttpStatus.OK, data: aclRules });
        }
    }
    async getHwTypeClass(hwType) {
        const classEntry = await Architecture_1.Architecture.getHwType(hwType);
        const className = classEntry.class.toString();
        return className;
    }
    async getItemClass(menuItem) {
        let itemClass;
        if (menuItem.hwType) {
            itemClass = await this.getHwTypeClass(menuItem.hwType);
        }
        else {
            const itemClassEntry = await this.classesRepository.findOneBy({ label: menuItem.title });
            itemClass = itemClassEntry._class.toString();
        }
        return itemClass;
    }
    async getStatus(menuItem, group_id, strict = false) {
        return await ACLCheck_1.ACLCheck.getStatus(menuItem, group_id, strict);
    }
    async getVirtualizationSubsys(hwType) {
        let menuItems = [];
        if (hwType === 'power') {
            const vmFolder = new menuResponse_dto_1.MenuFolder('LPAR', hwType + '/vm', hwType, 'vm');
            menuItems.push(vmFolder);
        }
        else if (hwType === 'vmware') {
            const folder = new menuResponse_dto_1.MenuFolder('VCenter', hwType + '/vcenter', hwType, 'vcenter');
            menuItems.push(folder);
            const clusterFolder = new menuResponse_dto_1.MenuFolder('Cluster', hwType + '/cluster', hwType, 'cluster');
            menuItems.push(clusterFolder);
            const vmFolder = new menuResponse_dto_1.MenuFolder('VM', hwType + '/vm', hwType, 'vm');
            menuItems.push(vmFolder);
        }
        else if (hwType === 'nutanix') {
            const folder = new menuResponse_dto_1.MenuFolder('Host', hwType + '/host', hwType, 'host');
            menuItems.push(folder);
            const vmFolder = new menuResponse_dto_1.MenuFolder('VM', hwType + '/vm', hwType, 'vm');
            menuItems.push(vmFolder);
        }
        else if (hwType === 'proxmox') {
            const folder = new menuResponse_dto_1.MenuFolder('Cluster', hwType + '/cluster', hwType, 'cluster');
            menuItems.push(folder);
            const vmFolder = new menuResponse_dto_1.MenuFolder('VM', hwType + '/vm', hwType, 'vm');
            menuItems.push(vmFolder);
        }
        return menuItems;
    }
    async getVirtualizationSubsysItems(hwType, subsystem) {
        let menuItems = [];
        let items = await this.objectItemsRepository.find({
            where: {
                hw_type: hwType,
                subsystem: subsystem
            }
        });
        for (let item of items.sort((a, b) => (a.label.localeCompare(b.label) < b.label.localeCompare(a.label) ? -1 : 1))) {
            let path = hwType + '/' + subsystem + '?item_id=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, subsystem, item.item_id));
        }
        return menuItems;
    }
    async getStorageDeviceSubsysItems(hwType, item_ids) {
        let menuItems = [];
        let subsystem;
        if (hwType === 'ecs' || hwType === 'storagegrid' || hwType === 'ddnintelliflash') {
            return menuItems;
        }
        else if (hwType === 'hcp') {
            subsystem = 'namespace';
        }
        else if (hwType === 'isilon' || hwType === 'storeonce' || hwType === 'datadomain' || hwType === 'rubrik' || hwType === 'cohesity' || hwType === 'ibmcoss') {
            subsystem = 'pool';
        }
        else {
            subsystem = 'volume';
        }
        let itemId = item_ids;
        let items = await this.objectItemsRepository.find({
            where: {
                hw_type: hwType,
                subsystem: subsystem,
                parents: {
                    parent: itemId.toString()
                }
            },
            relations: {
                parents: true
            }
        });
        items.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let item of items) {
            let path = hwType.toLowerCase() + '/' + subsystem + '?item_id=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, subsystem, item.item_id));
        }
        return menuItems;
    }
    async filterFolder(menuItems) {
        let aclMenuItems = [];
        let filteredSubsys = await this.filterSubsystems(menuItems);
        for (let item of filteredSubsys) {
            let aclItem;
            if (item.isLeaf) {
                if (!item.itemId) {
                    continue;
                }
                if (item.itemId) {
                    const itemClassification = await ACLCheck_1.ACLCheck.getItemClassification(item.itemId);
                    if (itemClassification.subsystem !== item.pageType && itemClassification.subsystem.toLowerCase().indexOf('folder') === -1) {
                        continue;
                    }
                }
                aclItem = new aclResponse_dto_1.AclMenuLink(item.title, '', aclResponse_constants_1.ITEM_STATUS.denied, item.itemId);
            }
            else {
                aclItem = new aclResponse_dto_1.AclMenuFolder(item.title, '', aclResponse_constants_1.ITEM_STATUS.denied, undefined, item.hwType, item.subsystem, item.itemId);
            }
            aclItem.url = item.url.replace('/api/menu/v1/folder/', '/api/acl/v1/tree/');
            ;
            aclItem.key = aclItem.getKey();
            aclMenuItems.push(aclItem);
        }
        return aclMenuItems;
    }
    async filterSubsystems(menuItems) {
        let filteredItems = [];
        for (let item of menuItems) {
            if (item.hwType && item.subsystem) {
                const itemClass = await this.getItemClass(item);
                if ((itemClass === 'database' && item.subsystem !== 'oracledb') || itemClass === 'cloud') {
                    item.isLeaf = true;
                    filteredItems.push(item);
                    continue;
                }
                else if (itemClass === 'container') {
                }
                else if (itemClass === 'san' || itemClass === 'lan') {
                    if (item.subsystem === 'device' || item.subsystem === 'fabric') {
                        filteredItems.push(item);
                    }
                    continue;
                }
                else if (itemClass === 'storage') {
                    if (item.subsystem === 'device' || item.subsystem === 'volume' || item.subsystem === 'pool') {
                        filteredItems.push(item);
                    }
                    else if (item.hwType === 'hcp' && item.subsystem === 'namespace') {
                        filteredItems.push(item);
                    }
                    continue;
                }
                else if (itemClass === 'virtualization') {
                    if (item.hwType === 'power') {
                        if (item.subsystem === 'server' || item.subsystem === 'vm') {
                            filteredItems.push(item);
                        }
                        continue;
                    }
                }
            }
            filteredItems.push(item);
        }
        return filteredItems;
    }
    async convertItemPaths(menuItems) {
        let aclMenuItems = [];
        for (let item of menuItems) {
            item.url = item.url.replace('/api/menu/v1/folder/', '/api/acl/v1/tree/');
            aclMenuItems.push(item);
        }
        return aclMenuItems;
    }
    async assignStatus(menuItems, group_id) {
        let aclMenuItems = [];
        for (let item of menuItems) {
            let aclItem = item;
            aclItem.status = await this.getStatus(item, group_id, false);
            aclMenuItems.push(aclItem);
        }
        return aclMenuItems;
    }
    async assignStatusDeviceFolder(menuItems, group_id, _class) {
        let aclMenuItems = [];
        let folderStatus = await ACLCheck_1.ACLCheck.getClassStatus([group_id], _class, false);
        for (let item of menuItems) {
            let aclItem = item;
            aclItem.disabled = false;
            if (folderStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                aclItem.status = await this.getStatus(item, group_id, false);
            }
            else {
                aclItem.status = folderStatus;
                if (folderStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                    aclItem.disabled = true;
                }
            }
            if (aclItem.subsystem === 'device' && (_class === 'san' || _class === 'lan')) {
                aclItem.isLeaf = true;
            }
            aclMenuItems.push(aclItem);
        }
        return aclMenuItems;
    }
    async assignStatusSubsysFolder(menuItems, group_id, hw_type, subsystem, item_ids) {
        let aclMenuItems = [];
        let ids = item_ids ? item_ids.split(',') : [];
        let folderStatus = subsystem
            ? await ACLCheck_1.ACLCheck.getSubsysStatus([group_id], hw_type, subsystem, ids, false)
            : await ACLCheck_1.ACLCheck.getHwTypeStatus([group_id], hw_type, false);
        let strict = (hw_type === 'power' && subsystem !== undefined) ? true : false;
        let filteredIds = [];
        if (folderStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
            let itemIds = menuItems.filter(item => (item.itemId && item.itemId !== undefined)).map(item => item.itemId);
            filteredIds = await ACLCheck_1.ACLCheck.filterItemsByRulesTopDown([group_id], itemIds);
        }
        for (let item of menuItems) {
            let aclItem = item;
            aclItem.disabled = false;
            if (folderStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                if (item.itemId && item.itemId !== undefined) {
                    if (filteredIds.includes(item.itemId)) {
                        aclItem.status = aclResponse_constants_1.ITEM_STATUS.granted;
                    }
                    else {
                        if (item.isLeaf) {
                            aclItem.status = aclResponse_constants_1.ITEM_STATUS.denied;
                        }
                        else {
                            aclItem.status = await this.getStatus(item, group_id, strict);
                        }
                    }
                }
                else {
                    aclItem.status = await this.getStatus(item, group_id, strict);
                }
            }
            else {
                aclItem.status = folderStatus;
                if (folderStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                    aclItem.disabled = true;
                }
            }
            aclMenuItems.push(aclItem);
        }
        return aclMenuItems;
    }
    async isDisabled(group_id, technology_class, hw_type) {
        let disabled = false;
        const rulesRepository = (0, dataSource_1.getDataSource)().getRepository(adminGroup_entity_1.AdminGroup);
        const classRules = await rulesRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.CLASS, value: technology_class } });
        if (classRules && classRules.length > 0) {
            disabled = true;
        }
        if (hw_type !== undefined) {
            const hwTypeRules = await rulesRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, type: aclResponse_constants_1.RULE_TYPE.HWTYPE, value: hw_type } });
            if (hwTypeRules && hwTypeRules.length > 0) {
                disabled = true;
            }
        }
        return disabled;
    }
    async updateRules(group_id, AclTree) {
        for (let item of AclTree.data) {
            const newStatus = item.status;
            const oldStatus = await this.getStatus(item, group_id, false);
            if (newStatus === aclResponse_constants_1.ITEM_STATUS.denied && oldStatus !== aclResponse_constants_1.ITEM_STATUS.denied) {
                await this.deleteItemRules(item, group_id);
            }
            else if (newStatus === aclResponse_constants_1.ITEM_STATUS.granted && oldStatus !== aclResponse_constants_1.ITEM_STATUS.granted) {
                if (oldStatus !== aclResponse_constants_1.ITEM_STATUS.denied) {
                    let subTree = new aclResponse_dto_1.AclMenuResponseDTO();
                    subTree.data = item.children;
                    this.updateRules(group_id, subTree);
                }
                const rule = await this.saveItemRule(item, group_id);
            }
            else if (newStatus === aclResponse_constants_1.ITEM_STATUS.partial) {
                let subTree = new aclResponse_dto_1.AclMenuResponseDTO();
                subTree.data = item.children;
                this.updateRules(group_id, subTree);
            }
        }
        return true;
    }
    async updateRuleChanges(group_id, changes) {
        for (const [key, status] of Object.entries(changes.data)) {
            const str = Buffer.from(key, 'base64').toString('utf8');
            const item = JSON.parse(str);
            if (status) {
                await this.saveItemRule(item, group_id);
            }
            else {
                await this.deleteItemRules(item, group_id);
            }
        }
        await this.createAdminRolesRules(group_id);
        return true;
    }
};
exports.ACLService = ACLService;
exports.ACLService = ACLService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(acl_entity_1.ACL)),
    __param(1, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(2, (0, typeorm_1.InjectRepository)(class_entity_1.Class)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], ACLService);
//# sourceMappingURL=acl.service.js.map