"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ItemRegexWrapper = void 0;
const dataSource_1 = require("../../config/dataSource");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const tagValueItem_entity_1 = require("../../entity/tagValueItem.entity");
const Classes_1 = require("../metrics/Classes");
const typeorm_1 = require("typeorm");
class ItemRegexWrapper {
    static async getItemsByRegexParentsTag(parentTags, hwTypeOrClass, subsystem, regex, intersection = false, withDeleted = false) {
        if (regex.length === 0) {
            return [];
        }
        if (!parentTags || parentTags.length === 0) {
            return await this.getItemsByRegex(hwTypeOrClass, subsystem, regex, withDeleted);
        }
        const regexFilter = await this.isAnyAlias()
            ? '((object_items.alias ~ ANY(array[:...regex]) AND object_items.alias is not null) OR (object_items.label ~ ANY(array[:...regex]) AND object_items.alias is null))'
            : 'object_items.label ~ ANY(array[:...regex])';
        if (intersection) {
            let queryBuilder = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
                .createQueryBuilder('object_items');
            let c = 1;
            for (const parentTag of parentTags) {
                queryBuilder = queryBuilder.leftJoin("object_items.parents", `parents${c}`)
                    .leftJoin(`parents${c}._parent`, `parent${c}`)
                    .leftJoin(`parent${c}.tag_value_items`, `tag_value_items${c}`);
                c++;
            }
            if (['storage', 'san', 'virtualization', 'lan'].includes(hwTypeOrClass)) {
                const hwTypes = this.getClassHwTypes(hwTypeOrClass);
                queryBuilder = queryBuilder.where('object_items.hw_type IN (:...hwTypes) AND object_items.subsystem = :subsystem AND ' + regexFilter, {
                    hwTypes: hwTypes,
                    subsystem: subsystem,
                    regex: regex
                });
            }
            else {
                queryBuilder = queryBuilder.where('object_items.hw_type = :hwType AND object_items.subsystem = :subsystem AND ' + regexFilter, {
                    hwType: hwTypeOrClass,
                    subsystem: subsystem,
                    regex: regex
                });
            }
            queryBuilder = queryBuilder.andWhere(new typeorm_1.Brackets((qb) => {
                let i = 1;
                for (const parentTag of parentTags) {
                    if (i === 1) {
                        qb.where(`tag_value_items${i}.tag_value_id = :tag${i} AND parent${i}.subsystem = :subsystem${i}`, {
                            [`tag${i}`]: parentTag.tag,
                            [`subsystem${i}`]: parentTag.subsystem,
                        });
                    }
                    else {
                        qb.andWhere(`tag_value_items${i}.tag_value_id = :tag${i} AND parent${i}.subsystem = :subsystem${i}`, {
                            [`tag${i}`]: parentTag.tag,
                            [`subsystem${i}`]: parentTag.subsystem,
                        });
                    }
                    i++;
                }
            }));
            return await queryBuilder.orderBy('object_items.label', 'ASC')
                .getMany();
        }
        else {
            let parents = [];
            const tagValueItems = await (0, dataSource_1.getDataSource)().getRepository(tagValueItem_entity_1.TagValueItem).find({
                relations: {
                    _item: true
                },
                where: parentTags.map(pTag => {
                    return {
                        tag_value_id: pTag.tag,
                        _item: {
                            subsystem: pTag.subsystem
                        }
                    };
                })
            });
            parents = tagValueItems.map(vItem => vItem.item_id);
            if (parents.length === 0) {
                return [];
            }
            return this.getItemsByRegexParents(parents, hwTypeOrClass, subsystem, regex);
        }
    }
    static async getItemsByRegexParentsRegex(parentsRegex, hwTypeOrClass, subsystem, regex, intersection = false, withDeleted = false) {
        if (regex.length === 0) {
            return [];
        }
        if (!parentsRegex || parentsRegex.length === 0) {
            return await this.getItemsByRegex(hwTypeOrClass, subsystem, regex, withDeleted);
        }
        const regexFilter = await this.isAnyAlias()
            ? '((object_items.alias ~ ANY(array[:...regex]) AND object_items.alias is not null) OR (object_items.label ~ ANY(array[:...regex]) AND object_items.alias is null))'
            : 'object_items.label ~ ANY(array[:...regex])';
        if (intersection) {
            let queryBuilder = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
                .createQueryBuilder('object_items')
                .leftJoin("object_items.parents", "parents");
            let c = 1;
            for (const parentRegex of parentsRegex) {
                queryBuilder = queryBuilder.leftJoin("object_items.parents", `parents${c}`)
                    .leftJoin(`parents${c}._parent`, `parent${c}`);
                c++;
            }
            if (['storage', 'san', 'virtualization', 'lan'].includes(hwTypeOrClass)) {
                const hwTypes = this.getClassHwTypes(hwTypeOrClass);
                queryBuilder = queryBuilder.where('object_items.hw_type IN (:...hwTypes) AND object_items.subsystem = :subsystem AND ' + regexFilter, {
                    hwTypes: hwTypes,
                    subsystem: subsystem,
                    regex: regex
                });
            }
            else {
                queryBuilder = queryBuilder.where('object_items.hw_type = :hwType AND object_items.subsystem = :subsystem AND ' + regexFilter, {
                    hwType: hwTypeOrClass,
                    subsystem: subsystem,
                    regex: regex
                });
            }
            const useAlias = await this.isAnyAlias();
            queryBuilder = queryBuilder.andWhere(new typeorm_1.Brackets((qb) => {
                let i = 1;
                for (const parentRegex of parentsRegex) {
                    const parentRegexFilter = useAlias
                        ? `((parent${i}.label ~ :regex${i} and parent${i}.alias is null) OR (parent${i}.alias ~ :regex${i} and parent${i}.alias is not null))`
                        : `parent${i}.label ~ :regex${i}`;
                    if (i === 1) {
                        qb.where(`parent${i}.subsystem = :subsystem${i} AND ${parentRegexFilter}`, {
                            [`subsystem${i}`]: parentRegex.subsystem,
                            [`regex${i}`]: parentRegex.regex,
                        });
                    }
                    else {
                        qb.andWhere(`parent${i}.subsystem = :subsystem${i} AND ${parentRegexFilter}`, {
                            [`subsystem${i}`]: parentRegex.subsystem,
                            [`regex${i}`]: parentRegex.regex,
                        });
                    }
                    i++;
                }
            }));
            queryBuilder.orderBy('object_items.label', 'ASC');
            if (withDeleted) {
                queryBuilder.withDeleted();
            }
            return await queryBuilder.getMany();
        }
        else {
            let parents = [];
            for (const parentRegex of parentsRegex) {
                const items = await this.getItemsByRegex(hwTypeOrClass, parentRegex.subsystem, [parentRegex.regex], withDeleted);
                parents = parents.concat(items.map(item => item.item_id));
            }
            if (parents.length === 0) {
                return [];
            }
            return this.getItemsByRegexParents(parents, hwTypeOrClass, subsystem, regex, withDeleted);
        }
    }
    static async getItemsByRegexParents(parents, hwTypeOrClass, subsystem, regex, intersection = false, withDeleted = false) {
        if (regex.length === 0) {
            return [];
        }
        if (!parents || parents.length === 0) {
            return await this.getItemsByRegex(hwTypeOrClass, subsystem, regex, withDeleted);
        }
        const regexFilter = await this.isAnyAlias()
            ? '((object_items.alias ~ ANY(array[:...regex]) AND object_items.alias is not null) OR (object_items.label ~ ANY(array[:...regex]) AND object_items.alias is null))'
            : 'object_items.label ~ ANY(array[:...regex])';
        let items = [];
        if (['storage', 'san', 'virtualization', 'lan'].includes(hwTypeOrClass)) {
            const hwTypes = this.getClassHwTypes(hwTypeOrClass);
            let queryBuilder = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
                .createQueryBuilder('object_items')
                .leftJoinAndSelect("object_items.parents", "parents")
                .where('parents.parent IN (:...parents) AND object_items.hw_type IN (:...hwTypes) AND object_items.subsystem = :subsystem AND ' + regexFilter, {
                hwTypes: hwTypes,
                subsystem: subsystem,
                parents: parents,
                regex: regex
            })
                .orderBy('object_items.label', 'ASC');
            if (withDeleted) {
                queryBuilder.withDeleted();
            }
            items = await queryBuilder.getMany();
        }
        else {
            let queryBuilder = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem)
                .createQueryBuilder('object_items')
                .leftJoinAndSelect("object_items.parents", "parents")
                .where('parents.parent IN (:...parents) AND object_items.hw_type = :hwType AND object_items.subsystem = :subsystem AND ' + regexFilter, {
                hwType: hwTypeOrClass,
                subsystem: subsystem,
                parents: parents,
                regex: regex
            })
                .orderBy('object_items.label', 'ASC');
            if (withDeleted) {
                queryBuilder.withDeleted();
            }
            items = await queryBuilder.getMany();
        }
        if (intersection) {
            items = items.filter(item => item.parents.length === parents.length);
        }
        return items;
    }
    static async getItemsByRegex(hwTypeOrClass, subsystem, regex, withDeleted = false) {
        if (regex.length === 0) {
            return [];
        }
        const regexFilter = await this.isAnyAlias()
            ? '((object_items.alias ~ ANY(array[:...regex]) AND object_items.alias is not null) OR (object_items.label ~ ANY(array[:...regex]) AND object_items.alias is null))'
            : 'object_items.label ~ ANY(array[:...regex])';
        let hwTypes;
        if (['storage', 'san', 'virtualization', 'lan'].includes(hwTypeOrClass)) {
            hwTypes = this.getClassHwTypes(hwTypeOrClass);
        }
        else {
            hwTypes = [hwTypeOrClass];
        }
        let queryBuilder = (0, dataSource_1.getDataSource)()
            .getRepository(objectItem_entity_1.ObjectItem)
            .createQueryBuilder('object_items')
            .where('object_items.hw_type IN (:...hwTypes) AND object_items.subsystem = :subsystem AND ' + regexFilter, {
            hwTypes: hwTypes,
            subsystem: subsystem,
            regex: regex,
        })
            .orderBy('object_items.label', 'ASC');
        if (withDeleted) {
            queryBuilder.withDeleted();
        }
        return await queryBuilder.getMany();
    }
    static async getItemsByRegexDevice(device_id, subsystem, regex, withDeleted = false) {
        if (regex.length === 0) {
            return [];
        }
        const regexFilter = await this.isAnyAlias()
            ? '((object_items.alias ~ ANY(array[:...regex]) AND object_items.alias is not null) OR (object_items.label ~ ANY(array[:...regex]) AND object_items.alias is null))'
            : 'object_items.label ~ ANY(array[:...regex])';
        let queryBuilder = (0, dataSource_1.getDataSource)()
            .getRepository(objectItem_entity_1.ObjectItem)
            .createQueryBuilder('object_items')
            .leftJoinAndSelect("item_relations", "item_relations", "item_relations.child = object_items.item_id")
            .where('item_relations.parent = :device_id AND object_items.subsystem = :subsystem AND ' + regexFilter, {
            regex: regex,
            device_id: device_id,
            subsystem: subsystem,
        })
            .orderBy('object_items.label', 'ASC');
        if (withDeleted) {
            queryBuilder.withDeleted();
        }
        return await queryBuilder.getMany();
    }
    static getClassHwTypes(_class) {
        switch (_class) {
            case 'storage': {
                return Object.keys(Classes_1.Storage.technology);
            }
            case 'san': {
                return Object.keys(Classes_1.San.technology);
            }
            case 'virtualization': {
                return Object.keys(Classes_1.Virtualization.technology);
            }
            case 'lan': {
                return Object.keys(Classes_1.Lan.technology);
            }
            default: {
                return [];
            }
        }
    }
    static async isAnyAlias() {
        return await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findOneBy({ alias: (0, typeorm_1.Not)((0, typeorm_1.IsNull)()) }) !== null;
    }
}
exports.ItemRegexWrapper = ItemRegexWrapper;
//# sourceMappingURL=ItemRegexWrapper.js.map