/// <reference types="multer" />
import { ArchitectureResponseDTO, ClassesResponseDTO, HwTypesResponseDTO, ItemAliasesResponseDTO, ItemGroupMetricRequest, ItemGroupPreview, ItemIds, ItemsResponseDTO, ItemsSearchResponseDTO, LabelsResponseDTO, MetricsResponseDTO, RegexPreviewDTO, SubsystemsResponseDTO } from '@xormon/common/architecture.dto';
import { ArchitectureService } from './architecture.service';
import { Alias } from '@xormon/common/alias.dto';
import { AliasService } from './alias.service';
import { BasicResponseDTO } from '@xormon/common/basicResponse.dto';
import { Request } from 'express';
import { DeviceGroupObjectItemsDTO } from '@xormon/common/deviceGroupHwAlerting.dto';
export declare class ArchitectureController {
    private readonly architectureService;
    private readonly aliasService;
    constructor(architectureService: ArchitectureService, aliasService: AliasService);
    getClasses(active?: boolean, skip_acl?: boolean): Promise<ClassesResponseDTO>;
    hw_types(active?: boolean, className?: string, type?: string, skip_acl?: boolean, admin_groups?: boolean): Promise<HwTypesResponseDTO>;
    getSubsystems(active?: boolean, className?: string, hwType?: string, type?: string, skip_acl?: boolean): Promise<SubsystemsResponseDTO>;
    subsystems(hwType: string, active?: boolean, type?: string, skip_acl?: boolean): Promise<SubsystemsResponseDTO>;
    itemsByHwTypeAndSubsystem(hw_type: string, subsystem: string): Promise<ItemsResponseDTO>;
    private getParentLabels;
    parentsByHwTypeAndSubsystem(hw_type: string, subsystem: string): Promise<SubsystemsResponseDTO>;
    items(subsystem?: string, hw_type?: string, _class?: string, parents?: string[], withParents?: boolean, withRootParent?: boolean, childSubsystem?: string, strict_acl?: boolean, withoutMapping?: boolean, withDeleted?: boolean): Promise<ItemsResponseDTO>;
    search(subsystem?: string, hw_type?: string, _class?: string, parents?: string[], search?: string, page?: number, limit?: number): Promise<ItemsSearchResponseDTO>;
    itemsByIds(items: ItemIds): Promise<LabelsResponseDTO>;
    getMetrics(): Promise<ArchitectureResponseDTO>;
    metricsByHwTypeAndSubsystem(hw_type: string, subsystem: string, type?: string): Promise<MetricsResponseDTO>;
    regex(regex: RegexPreviewDTO, withParents?: boolean, withRootParent?: boolean, strict_acl?: boolean, withDeleted?: boolean): Promise<ItemsResponseDTO>;
    multiRegex(regexes: RegexPreviewDTO[], withParents?: boolean, strict_acl?: boolean): Promise<ItemsResponseDTO>;
    getAliases(): Promise<ItemAliasesResponseDTO>;
    saveAliases(aliases: Alias[]): Promise<ItemsResponseDTO>;
    importAliases(file: Express.Multer.File): Promise<ItemsResponseDTO>;
    exportAliases(): Promise<string>;
    deleteAliases(): Promise<BasicResponseDTO>;
    deleteAlias(itemId: string): Promise<BasicResponseDTO>;
    itemGroupMetrics(itemGroupMetricRequest: ItemGroupMetricRequest, type?: 'alerting' | 'customGroup' | 'exporter' | 'reporter'): Promise<MetricsResponseDTO>;
    preview(preview: ItemGroupPreview, withParents?: boolean, withRootParent?: boolean): Promise<ItemsResponseDTO>;
    multiPreview(previews: ItemGroupPreview[], withParents?: boolean, withRootParent?: boolean): Promise<ItemsResponseDTO>;
    getDeviceGroupItems(request: Request, _class: string, hw_type?: string, regex?: boolean): Promise<DeviceGroupObjectItemsDTO>;
    getDeviceGroupItemsRegex(request: Request, regexPreview: RegexPreviewDTO): Promise<DeviceGroupObjectItemsDTO>;
}
