"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.App = void 0;
const dataSource_1 = require("../../config/dataSource");
const backend_entity_1 = require("../../entity/backend.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const path_1 = require("path");
const uuid_1 = require("uuid");
const PM2_1 = require("./PM2");
const License_1 = require("../../config/License");
const microserviceVariables_entity_1 = require("../../entity/microserviceVariables.entity");
const typeorm_1 = require("typeorm");
const Settings_1 = require("./Settings");
const audit_entity_1 = require("../../entity/audit.entity");
const auditEvent_entity_1 = require("../../entity/auditEvent.entity");
const version_entity_1 = require("../../entity/version.entity");
const hwTypeKey_constants_1 = require("../../common/constants/hwTypeKey.constants");
const publicUrl_1 = require("../../common/publicUrl");
require('dotenv').config();
const fs = require('fs');
class App {
    static isDemo() {
        return this.checkEnv('APP_DEMO', "true");
    }
    static isProduction() {
        return this.checkEnv('NODE_ENV', "production");
    }
    static async isDebug() {
        const envCheck = this.checkEnv('APP_DEBUG', "1");
        if (envCheck !== undefined) {
            return envCheck;
        }
        const msVarCheckValue = await this.checkGlobalMsVariables("APP_DEBUG");
        return msVarCheckValue === "true" ? true : msVarCheckValue === "1" ? true : false;
    }
    static isDockerVersion() {
        return this.checkEnv('DOCKER', "1");
    }
    static async isFree() {
        return !(await License_1.License.hasAnyValid());
    }
    static async isMaster() {
        return await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).findOneBy({ type: backend_entity_1.backendType.MASTER, uuid: this.getId() }) !== null;
    }
    static async isExistsMaster() {
        return await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).findOneBy({ type: backend_entity_1.backendType.MASTER }) !== null;
    }
    static async isFirstLogin() {
        return await (0, dataSource_1.getDataSource)().getRepository(audit_entity_1.Audit).findOneBy({ user_id: 1, event: auditEvent_entity_1.auditEventId.LOGIN }) === null;
    }
    static async setMaster() {
        const version = this.getVersion();
        const master = await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).findOneBy({ type: backend_entity_1.backendType.MASTER });
        if (master) {
            if ((0, publicUrl_1.isNewerVersion)(version, master.version)) {
                console.error("[SetMaster] error: version of this node is lower than master node " + master.ip);
                return;
            }
            await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).save({
                ...master,
                type: backend_entity_1.backendType.SLAVE
            });
        }
        console.log("[SetMaster]: switch master node");
        const node = await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).findOneBy({ uuid: this.getId() });
        if (node) {
            await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).save({
                ...node,
                type: backend_entity_1.backendType.MASTER
            });
        }
        else {
            await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).save({
                uuid: this.getId(),
                started: Math.floor(Date.now() / 1000),
                echoed: Math.floor(Date.now() / 1000),
                ip: this.getIp(),
                version: version,
                type: backend_entity_1.backendType.MASTER
            });
        }
        console.log("[SetMaster]: restarting...");
        this.restart();
    }
    static async restart() {
        await PM2_1.PM2.restartService('backend');
    }
    static async registerNode() {
        const node = await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).findOneBy({ uuid: this.getId() });
        if (!node) {
            let type = backend_entity_1.backendType.SLAVE;
            const master = await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).findOneBy({ type: backend_entity_1.backendType.MASTER });
            if (master) {
                if (!await _a.isDebug()) {
                    if ((0, publicUrl_1.isNewerVersion)(master.version, this.getVersion())) {
                        console.log("[RegisterNode]: found older version " + master.ip);
                        await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).update(master.uuid, {
                            ...master,
                            type: backend_entity_1.backendType.SLAVE
                        });
                        type = backend_entity_1.backendType.MASTER;
                    }
                }
            }
            else {
                type = backend_entity_1.backendType.MASTER;
            }
            await (0, dataSource_1.getDataSource)().getRepository(backend_entity_1.Backend).save({
                uuid: this.getId(),
                started: Math.floor(Date.now() / 1000),
                echoed: Math.floor(Date.now() / 1000),
                ip: this.getIp(),
                version: this.getVersion(),
                type: type
            });
        }
    }
    static getSecret() {
        const secret = process.env['APP_SECRET'];
        if (secret == undefined) {
            return this.DEFAULT_APP_SECRET;
        }
        else {
            return secret;
        }
    }
    static getIp() {
        const interfaces = require('os').networkInterfaces();
        for (const devName in interfaces) {
            const iface = interfaces[devName];
            for (let i = 0; i < iface.length; i++) {
                const alias = iface[i];
                if (alias.family === 'IPv4' && alias.address !== '127.0.0.1' && !alias.internal)
                    return alias.address;
            }
        }
        return '0.0.0.0';
    }
    static getHostname() {
        const os = require('os');
        return os.hostname();
    }
    static getId() {
        return (0, uuid_1.v5)(this.getIp(), this.DEFAULT_NAMESPACE);
    }
    static getVersion() {
        try {
            const packageJson = require(this.ROOT_PATH + '/server-nest/package.json');
            return packageJson.version;
        }
        catch (e) {
            console.error(e);
            return "?";
        }
    }
    static async checkVersionInDatabase() {
        const version = this.getVersion();
        if (version !== "?") {
            const versionInDatabase = await (0, dataSource_1.getDataSource)().getRepository(version_entity_1.Version).findOne({
                where: {
                    version: (0, typeorm_1.Not)((0, typeorm_1.IsNull)())
                },
                order: {
                    upgraded: 'DESC'
                }
            });
            if (!versionInDatabase || version !== versionInDatabase.version) {
                await (0, dataSource_1.getDataSource)().getRepository(version_entity_1.Version).insert({
                    version: version
                });
            }
        }
    }
    static async getTimescaleVersion() {
        const version = await (0, dataSource_1.getDataSource)().query("SELECT default_version, installed_version FROM pg_available_extensions where name = 'timescaledb';");
        return version[0]['installed_version'];
    }
    static allowedHwTypes() {
        return [
            hwTypeKey_constants_1.HwTypeKey.VMWARE, hwTypeKey_constants_1.HwTypeKey.PROXMOX, hwTypeKey_constants_1.HwTypeKey.NUTANIX, hwTypeKey_constants_1.HwTypeKey.POWER, hwTypeKey_constants_1.HwTypeKey.POSTGRES, hwTypeKey_constants_1.HwTypeKey.SQLSERVER, hwTypeKey_constants_1.HwTypeKey._3PAR, hwTypeKey_constants_1.HwTypeKey.COMPELLENT, hwTypeKey_constants_1.HwTypeKey.DATADOMAIN, hwTypeKey_constants_1.HwTypeKey.DOTHILL, hwTypeKey_constants_1.HwTypeKey.ESERIES, hwTypeKey_constants_1.HwTypeKey.IBMCOSS, hwTypeKey_constants_1.HwTypeKey.ISILON, hwTypeKey_constants_1.HwTypeKey.POWERFLEX, hwTypeKey_constants_1.HwTypeKey.RUBRIK, hwTypeKey_constants_1.HwTypeKey.STOREONCE, hwTypeKey_constants_1.HwTypeKey.SWIZ, hwTypeKey_constants_1.HwTypeKey.QNAP, hwTypeKey_constants_1.HwTypeKey.ORACLEZFS, hwTypeKey_constants_1.HwTypeKey.SANBRCD, hwTypeKey_constants_1.HwTypeKey.SANCISCO, hwTypeKey_constants_1.HwTypeKey.KUBERNETES, hwTypeKey_constants_1.HwTypeKey.ORACLEDB, hwTypeKey_constants_1.HwTypeKey.VMAX, hwTypeKey_constants_1.HwTypeKey.PURE, hwTypeKey_constants_1.HwTypeKey.XIV, hwTypeKey_constants_1.HwTypeKey.DB2, hwTypeKey_constants_1.HwTypeKey.NIMBLE, hwTypeKey_constants_1.HwTypeKey.NETAPP, hwTypeKey_constants_1.HwTypeKey.STORAGEGRID, hwTypeKey_constants_1.HwTypeKey.INFINIBOX, hwTypeKey_constants_1.HwTypeKey.VSPG, hwTypeKey_constants_1.HwTypeKey.COHESITY, hwTypeKey_constants_1.HwTypeKey.OCEANSTOR, hwTypeKey_constants_1.HwTypeKey.XTREMIO, hwTypeKey_constants_1.HwTypeKey.POWERSTORE, hwTypeKey_constants_1.HwTypeKey.ECS, hwTypeKey_constants_1.HwTypeKey.AWS, hwTypeKey_constants_1.HwTypeKey.AZURE, hwTypeKey_constants_1.HwTypeKey.GCLOUD, hwTypeKey_constants_1.HwTypeKey.LINUX, hwTypeKey_constants_1.HwTypeKey.ORACLEVM, hwTypeKey_constants_1.HwTypeKey.OVIRT, hwTypeKey_constants_1.HwTypeKey.WINDOWS, hwTypeKey_constants_1.HwTypeKey.DOCKER, hwTypeKey_constants_1.HwTypeKey.OPENSHIFT, hwTypeKey_constants_1.HwTypeKey.HCP, hwTypeKey_constants_1.HwTypeKey.DDNINTELLIFLASH, hwTypeKey_constants_1.HwTypeKey.LANCISCO, hwTypeKey_constants_1.HwTypeKey.LANOTHERS, hwTypeKey_constants_1.HwTypeKey.OCEANSTORPACIFIC, hwTypeKey_constants_1.HwTypeKey.OCEANSTORPACIFICNAS, hwTypeKey_constants_1.HwTypeKey.OCEANSTORNAS, hwTypeKey_constants_1.HwTypeKey.SANNAV, hwTypeKey_constants_1.HwTypeKey.UNITY, hwTypeKey_constants_1.HwTypeKey.SYNOLOGY, hwTypeKey_constants_1.HwTypeKey.SOLARIS, hwTypeKey_constants_1.HwTypeKey.DATACORE
        ];
    }
    static async getApplianceVersion() {
        const xorux_version_file = '/etc/xorux_version';
        try {
            if (fs.existsSync(xorux_version_file)) {
                const versionFile = await fs.readFileSync(xorux_version_file, { encoding: 'utf8', flag: 'r' });
                const lines = versionFile.trim().split('\n');
                const version = lines[lines.length - 1];
                return version;
            }
        }
        catch (e) {
            console.error(e);
        }
        return null;
    }
    static async getActiveHwTypes() {
        return (await (0, dataSource_1.getDataSource)()
            .getRepository(objectItem_entity_1.ObjectItem)
            .createQueryBuilder()
            .select(['hw_type'])
            .distinct()
            .getRawMany())
            .map(item => item.hw_type);
    }
    static checkEnv(env, value) {
        const envValue = process.env[env];
        if (envValue == undefined) {
            return undefined;
        }
        else {
            return process.env[env] === value;
        }
    }
    static async checkGlobalMsVariables(key, hw_type = null, hostcfg_id = null) {
        return (await (0, dataSource_1.getDataSource)()
            .getRepository(microserviceVariables_entity_1.MicroserviceVariables)
            .findOneBy({ key: key, hw_type: hw_type ? hw_type : (0, typeorm_1.IsNull)(), hostcfg_id: hostcfg_id ? hostcfg_id : (0, typeorm_1.IsNull)() }))?.value;
    }
    static async getCert() {
        const settings = await Settings_1.Settings.get({
            prefix: 'CERT'
        });
        if (settings['CERT_KEY'] && settings['CERT_KEY'].length >= 1) {
            return {
                crt: settings['CERT_CRT'],
                key: settings['CERT_KEY']
            };
        }
        else {
            return {
                crt: this.ROOT_PATH + "/server-nest/ssl/xormon.crt",
                key: this.ROOT_PATH + "/server-nest/ssl/xormon.key"
            };
        }
    }
    static async sleep(ms) {
        return new Promise((resolve) => setTimeout(resolve, ms));
    }
}
exports.App = App;
_a = App;
App.ROOT_PATH = (0, path_1.resolve)('../');
App.APPLIANCE_LOGS_PATH = '/var/log/xormon';
App.LOGS_PATH = fs.existsSync(_a.APPLIANCE_LOGS_PATH) ? _a.APPLIANCE_LOGS_PATH : _a.ROOT_PATH + "/server-nest/logs";
App.CUSTOM_SCRIPT_PATH = (0, path_1.resolve)('../server-nest/dist/custom');
App.TMP_FILES_PATH = (0, path_1.resolve)('./files/tmp');
App.CERT_FILES_PATH = (0, path_1.resolve)('./files/cert');
App.LDAP_CERT_FILES_PATH = (0, path_1.resolve)('./files/ldap');
App.DEFAULT_APP_SECRET = "5514eb3d2bcc2720a116c91687739ff6";
App.DEFAULT_NAMESPACE = "47bf86ed-9639-44b8-b648-06915b2ab68b";
//# sourceMappingURL=App.js.map