export interface PM2ServiceDefinition {
    name: string;
    script: string;
    cwd: string;
    error_file: string;
    out_file: string;
    interpreter?: string;
    env?: {};
}
export interface PM2ServiceInfo {
    active: boolean;
    pid: number;
    version: string;
    monitor: {
        memory_byte: number;
        cpu_percent: number;
    };
}
export interface PM2Raw {
    name: string;
    pid: number;
    pm_id: number;
    monit: {
        memory: number;
        cpu: number;
    };
    pm2_env: {
        status: string;
        instances: number;
        pm_uptime: number;
        version: string;
    };
}
export interface PM2Message {
    id: string;
    type: string;
    data: PM2MessageData | PM2TaskMessageData;
    topic: string;
}
export interface PM2MessageData {
    action: string;
}
export interface PM2TaskMessageData extends PM2MessageData {
    taskPayload: any;
    taskType: string;
    number_of_items?: number;
}
export declare class PM2 {
    private static pm2;
    static describeService(name: string): Promise<PM2ServiceInfo>;
    static startService(service: string): Promise<unknown>;
    static stopService(service: string): Promise<unknown>;
    static restartService(service: string): Promise<unknown>;
    static listServices(): Promise<unknown>;
    static describeServiceCall(name: string): Promise<PM2Raw[]>;
    static registerService(data: JSON | PM2ServiceDefinition): Promise<unknown>;
    static getProcessId(name: string): Promise<number[]>;
    static isRunning(): boolean;
    static isMasterBackend(): boolean;
    static shouldILive(): Promise<boolean>;
    static commitSuicide(): Promise<void>;
    static sendMessage(processName: string, data: PM2MessageData | PM2TaskMessageData): Promise<void>;
    static parseMessage(message: PM2Message): PM2MessageData;
    static parseMessageTask(message: PM2Message): PM2TaskMessageData;
    static scale(service: 'backend' | 'metrics-server' | 'tasks-server', count: number | string): Promise<void>;
}
