"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Settings = void 0;
const dataSource_1 = require("../../config/dataSource");
const setting_entity_1 = require("../../entity/setting.entity");
const typeorm_1 = require("typeorm");
const Crypto_1 = require("./Crypto");
class Settings {
    static async get(args) {
        let items = {};
        let queryParams = [];
        if (args.keys) {
            queryParams.push({
                key: (0, typeorm_1.In)(args.keys)
            });
        }
        if (args.prefix) {
            queryParams.push({
                key: (0, typeorm_1.Like)(`${args.prefix}_%`)
            });
        }
        const settings = await (0, dataSource_1.getDataSource)().getRepository(setting_entity_1.Setting).findBy(queryParams);
        for (const setting of settings) {
            items[setting.key] = setting.value;
        }
        return items;
    }
    static async getWithTime(args) {
        let items = {};
        let queryParams = [];
        if (args.keys) {
            queryParams.push({
                key: (0, typeorm_1.In)(args.keys)
            });
        }
        if (args.prefix) {
            queryParams.push({
                key: (0, typeorm_1.Like)(`${args.prefix}_%`)
            });
        }
        const settings = await (0, dataSource_1.getDataSource)().getRepository(setting_entity_1.Setting).findBy(queryParams);
        for (const setting of settings) {
            items[setting.key] = {
                value: setting.value,
                updated: setting.updatedAt.toISOString()
            };
        }
        return items;
    }
    static async all() {
        let items = {};
        const settings = await (0, dataSource_1.getDataSource)().getRepository(setting_entity_1.Setting).find();
        for (const setting of settings) {
            items[setting.key] = setting.value;
        }
        return items;
    }
    static async save(items) {
        let settings = [];
        for (const key in items) {
            settings.push({
                key: key,
                value: items[key]
            });
        }
        const save = await (0, dataSource_1.getDataSource)().getRepository(setting_entity_1.Setting).save(settings);
        return save.length === Object.keys(items).length;
    }
    static async delete(keys) {
        const filtered = keys.filter(key => !this.protected.includes(key));
        const deleted = await (0, dataSource_1.getDataSource)().getRepository(setting_entity_1.Setting).delete({ key: (0, typeorm_1.In)(filtered) });
        return deleted.affected === keys.length;
    }
    static async generateLKey() {
        const text = `{"technology": "*","expirationDate": "${new Date(Date.now() + 60 * 24 * 60 * 60 * 1000).toISOString().split('T')[0]}","maxDeviceCount": -1,"publicAPI": true}`;
        return await Crypto_1.Crypto.encrypt(text);
    }
}
exports.Settings = Settings;
Settings.protected = ['GLOB_TOKEN_EXPIRATION'];
Settings.hided = ['SMTP_PASS'];
//# sourceMappingURL=Settings.js.map