"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigurationController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const platform_express_1 = require("@nestjs/platform-express");
const swagger_1 = require("@nestjs/swagger");
const architecture_dto_1 = require("../../common/architecture.dto");
const configuration_dto_1 = require("../../common/configuration.dto");
const microserviceJob_constants_1 = require("../../common/constants/microserviceJob.constants");
const fabric_dto_1 = require("../../common/fabric.dto");
const hostCfgResponse_dto_1 = require("../../common/hostCfgResponse.dto");
const mailGroup_dto_1 = require("../../common/mailGroup.dto");
const microserviceJob_dto_1 = require("../../common/microserviceJob.dto");
const microserviceVariables_dto_1 = require("../../common/microserviceVariables.dto");
const settings_dto_1 = require("../../common/settings.dto");
const targetDestination_dto_1 = require("../../common/targetDestination.dto");
const License_1 = require("../../config/License");
const HttpResponse_1 = require("../../dto/HttpResponse");
const auditEvent_entity_1 = require("../../entity/auditEvent.entity");
const itemRelation_entity_1 = require("../../entity/itemRelation.entity");
const xormon_exception_1 = require("../../exception/xormon.exception");
const adminRoles_guard_1 = require("../../guard/adminRoles.guard");
const adminGroups_interceptor_1 = require("../../interceptor/adminGroups.interceptor");
const mail_service_1 = require("../../mail/mail.service");
const User_1 = require("../users/User");
const https = require("https");
const path_1 = require("path");
const rawbody = require("raw-body");
const uuid_1 = require("uuid");
const MetricsWrapper_1 = require("../metrics/MetricsWrapper");
const metricsTable_service_1 = require("../metrics/metricsTable.service");
const Audit_1 = require("./Audit");
const Settings_1 = require("./Settings");
const Upgrade_1 = require("./Upgrade");
const VmWareSDK_1 = require("./VmWareSDK");
const configuration_service_1 = require("./configuration.service");
const fabric_service_1 = require("./fabric.service");
const hostCfg_service_1 = require("./hostCfg.service");
const mailgroup_service_1 = require("./mailgroup.service");
const microserviceVariable_service_1 = require("./microserviceVariable.service");
const tokenRequest_dto_1 = require("../../common/tokenRequest.dto");
const ItemGroupWrapper_1 = require("../custom_group/ItemGroupWrapper");
const Linux_1 = require("../menu/menu/Linux");
const pageResponse_constants_1 = require("../../common/constants/pageResponse.constants");
const PM2_1 = require("./PM2");
const App_1 = require("./App");
const menuGroup_service_1 = require("./menuGroup.service");
const menuGroup_dto_1 = require("../../common/menuGroup.dto");
const Timescaledb_1 = require("../menu/menu/Timescaledb");
const Item_1 = require("./Item");
const request_builder_1 = require("../../request/request.builder");
const hostcfgData_types_1 = require("../../common/hostcfgs/hostcfgData.types");
let ConfigurationController = class ConfigurationController {
    constructor(configurationService, metricsTableService, mailService, microserviceVariablesService, hostCfgService, fabricService, mailGroupService, menugroupService) {
        this.configurationService = configurationService;
        this.metricsTableService = metricsTableService;
        this.mailService = mailService;
        this.microserviceVariablesService = microserviceVariablesService;
        this.hostCfgService = hostCfgService;
        this.fabricService = fabricService;
        this.mailGroupService = mailGroupService;
        this.menugroupService = menugroupService;
        this.TMP_PATH = (0, path_1.resolve)('./files/tmp');
    }
    async saveConfigrationJSON(data) {
        let items = [];
        for (let uuid in data) {
            for (let item in data[uuid]) {
                let itemProperty = {
                    item_id: uuid,
                    property_name: item,
                    property_value: data[uuid][item],
                };
                items.push(itemProperty);
            }
        }
        let inserted = 0;
        try {
            inserted = await this.configurationService.saveItemProperties(items);
        }
        catch (error) {
            const itemIds = items
                .map((item) => item.item_id)
                .filter(function (elem, index, self) {
                return index === self.indexOf(elem);
            });
            const existedItemIds = await this.configurationService.getItemIds();
            const missing = itemIds.filter(item => existedItemIds.indexOf(item) < 0);
            if (missing.length >= 1) {
                console.error('Missing item_id in object_items [IDs: ' + missing.toString() + ']: ' + error);
            }
            else {
                console.error('Configuration store error [IDs: ' + missing.toString() + ']: ' + error);
            }
            throw new xormon_exception_1.XormonException("Error during save, bad data", common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: 'saved ' + inserted + ' rows',
        };
    }
    async saveConfigrationCSV(req) {
        if (req.readable) {
            let items = [];
            const raw = await rawbody(req);
            const csv = raw.toString().trim().split('\n');
            const legend = csv[0].split(';');
            for (let i = 1; i < csv.length; i++) {
                const line = csv[i].split(';');
                for (let x = 1; x < line.length; x++) {
                    if (legend[x].length === 0) {
                        continue;
                    }
                    let itemProperty = {
                        item_id: line[0],
                        property_name: legend[x],
                        property_value: line[x],
                    };
                    items.push(itemProperty);
                }
            }
            let inserted = 0;
            try {
                inserted = await this.configurationService.saveItemProperties(items);
            }
            catch (error) {
                const itemIds = items
                    .map((item) => item.item_id)
                    .filter(function (elem, index, self) {
                    return index === self.indexOf(elem);
                });
                console.error('Configuration store error [IDs: ' + itemIds.toString() + ']: ' + error);
                throw new xormon_exception_1.XormonException("Error during save, bad data", common_1.HttpStatus.BAD_REQUEST);
            }
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'saved ' + inserted + ' rows',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error during save, bad request body", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getConfigurationByUUID(params, res) {
        if (params.uuid) {
            let data = await this.configurationService.getItemProperties(params.uuid, null);
            if (data.length === 0) {
                return HttpResponse_1.HttpResponse.prototype.error(res, 'no data found');
            }
            let configuration = new Object();
            for (let i = 0; i < data.length; i++) {
                const item = data[i];
                if (configuration[item.item_id]) {
                    configuration[item.item_id][item.property_name] = item.property_value;
                }
                else {
                    configuration[item.item_id] = {
                        [item.property_name]: item.property_value,
                    };
                }
            }
            return HttpResponse_1.HttpResponse.prototype.json(res, configuration);
        }
        else {
            throw new xormon_exception_1.XormonException("Bad request", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async findProperties(find) {
        const properties = await this.configurationService.getItemsProperties(find.uuids, find.properties);
        let itemsProperties = {};
        for (const property of properties) {
            if (itemsProperties[property.item_id]) {
                itemsProperties[property.item_id][property.property_name] = property.property_value;
            }
            else {
                itemsProperties[property.item_id] = {
                    [property.property_name]: property.property_value
                };
            }
        }
        return {
            data: itemsProperties
        };
    }
    async getConfigurationProperties(hwType, subsystem) {
        let props = [];
        const properties = await MetricsWrapper_1.MetricsWrapper.getConfigurationFindClass(hwType, subsystem);
        for (const propertyKey in properties) {
            props.push({
                property_name: propertyKey,
                label: properties[propertyKey] && properties[propertyKey].label ? properties[propertyKey].label : propertyKey
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: props
        };
    }
    async getConfiguration(params, res) {
        if (params.uuid && params.property_name) {
            let data = await this.configurationService.getItemProperties(params.uuid, params.property_name);
            if (data.length === 0) {
                return HttpResponse_1.HttpResponse.prototype.error(res, 'no data found');
            }
            let configuration = new Object();
            for (let i = 0; i < data.length; i++) {
                const item = data[i];
                if (configuration[item.item_id]) {
                    configuration[item.item_id][item.property_name] = item.property_value;
                }
                else {
                    configuration[item.item_id] = {
                        [item.property_name]: item.property_value,
                    };
                }
            }
            return HttpResponse_1.HttpResponse.prototype.json(res, configuration);
        }
        else {
            throw new xormon_exception_1.XormonException("Bad request", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getConfigurationKeys(uuid) {
        if (uuid) {
            const uuids = uuid.includes(",") ? uuid.split(",") : [uuid];
            const keys = await this.configurationService.getPropertiesKey(uuids);
            return {
                statusCode: common_1.HttpStatus.OK,
                data: keys
            };
        }
        else {
            throw new xormon_exception_1.XormonException("No uuids sent", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async saveArchitecture(architecture, agent = 0) {
        if (architecture && architecture.length >= 1) {
            const now = new Date();
            let objectItems = [];
            let itemRelations = [];
            let agentMapping = [];
            let hostcfgRelations = [];
            let storageMapping = [];
            for (const arch of architecture) {
                objectItems.push({
                    item_id: arch.item_id,
                    label: arch.label,
                    object_id: arch.hw_type.toLowerCase(),
                    hw_type: arch.hw_type.toLowerCase(),
                    subsystem: arch.subsystem.toLowerCase(),
                    updatedAt: now,
                });
                if (arch.parents && arch.parents.length >= 1) {
                    for (const parent of arch.parents) {
                        itemRelations.push({
                            parent: parent,
                            child: arch.item_id,
                            origin: itemRelation_entity_1.RelationOrigin.child,
                            updatedAt: now,
                        });
                    }
                }
                if (arch.children && arch.children.length >= 1) {
                    for (const child of arch.children) {
                        itemRelations.push({
                            parent: arch.item_id,
                            child: child,
                            origin: itemRelation_entity_1.RelationOrigin.parent,
                            updatedAt: now,
                        });
                    }
                }
                if (arch.agent_mapping) {
                    agentMapping.push({
                        agent_id: arch.item_id,
                        item_id: arch.agent_mapping,
                    });
                }
                if (arch.hostcfg_id) {
                    hostcfgRelations.push({
                        hostcfg_id: arch.hostcfg_id,
                        item_id: arch.item_id,
                        updatedAt: now,
                    });
                }
                if (arch.wwn) {
                    storageMapping.push({
                        item_id: arch.item_id,
                        wwn: arch.wwn,
                    });
                }
                if (arch.wwns) {
                    for (const wwn of arch.wwns) {
                        storageMapping.push({
                            item_id: arch.item_id,
                            wwn: wwn,
                        });
                    }
                }
                if (arch.target_wwn) {
                    storageMapping.push({
                        item_id: arch.item_id,
                        wwn: arch.target_wwn,
                        target: true
                    });
                }
                if (arch.target_wwns) {
                    for (const wwn of arch.target_wwns) {
                        storageMapping.push({
                            item_id: arch.item_id,
                            wwn: wwn,
                            target: true
                        });
                    }
                }
            }
            let itemIds = objectItems.map(function (item) { return item.item_id; });
            let duplicates = itemIds.filter((n, i) => itemIds.indexOf(n) !== i);
            if (duplicates.length > 0) {
            }
            else {
            }
            let inserted_arch = 0;
            try {
                if (agent == 1) {
                    inserted_arch = await this.configurationService.saveArchitectureAgent(objectItems);
                }
                else {
                    inserted_arch = await this.configurationService.saveArchitecture(objectItems);
                }
            }
            catch (e) {
                let itemIds = objectItems.map(function (item) { return item.item_id; });
                let duplicates = itemIds.filter((n, i) => itemIds.indexOf(n) !== i);
                if (duplicates.length > 0) {
                    console.error("Architecture duplicate ids " + duplicates.toString());
                }
                else {
                    console.error("Architecture error ");
                }
            }
            const inserted_rel = await this.configurationService.saveItemRelations(itemRelations);
            this.configurationService.saveAgentMapping(agentMapping);
            this.hostCfgService.saveHostcfgRelations(hostcfgRelations);
            this.configurationService.saveStorageMapping(storageMapping);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'inserted ' + inserted_arch + ' items and ' + inserted_rel + ' relations',
            };
        }
    }
    async getTargetDestinations() {
        const targetDestinations = await this.configurationService.getTargetDestinations();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: targetDestinations.map(destination => {
                return {
                    target_destination_id: destination.target_destination_id,
                    destination: destination.url ? destination.url : destination.destination,
                    url: destination.url,
                    token: destination.token,
                    port: destination.port,
                    hostcfgs: destination._target_destination_hostcfgs.length >= 1
                        ? destination._target_destination_hostcfgs.map(hostcfg => {
                            return {
                                hostcfg_id: hostcfg._hostcfg.hostcfg_id,
                                label: hostcfg._hostcfg.label
                            };
                        })
                        : null
                };
            }),
        };
    }
    async saveTargetDestinations(request, destination) {
        const saved = await this.configurationService.saveTargetDestination(destination);
        if (saved) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.PROXY_ADD, "Cnetral site: " + saved.destination + " - " + await this.configurationService.joinDestinationDevices(destination));
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    target_destination_id: saved.target_destination_id,
                    destination: saved.url ? saved.url : saved.destination,
                    url: saved.url,
                    port: saved.port,
                    token: saved.token,
                    hostcfgs: saved._target_destination_hostcfgs.length >= 1
                        ? saved._target_destination_hostcfgs.map(hostcfg => {
                            return {
                                hostcfg_id: hostcfg._hostcfg.hostcfg_id,
                                label: hostcfg._hostcfg.label
                            };
                        })
                        : null
                }
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error during save", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateTargetDestinations(request, target_destination_id, destination) {
        const saved = await this.configurationService.updateTargetDestination(target_destination_id, destination);
        if (saved) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.PROXY_UPDATE, "Cnetral site: " + saved.destination + " - " + await this.configurationService.joinDestinationDevices(destination));
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Target destination updated'
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error during update", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteTargetDestinations(request, target_destination_id) {
        const targetDestination = await this.configurationService.getTargetDestination(target_destination_id);
        const cfgIds = targetDestination._target_destination_hostcfgs.map((targetCfg) => targetCfg.hostcfg_id);
        let failedToDelete = false;
        if (targetDestination.url) {
            const baseDestiUrl = targetDestination.url.split("/dashboard")[0];
            for (const id of cfgIds) {
                console.log("deleting cfg on target destination with hostcfg_id:", id);
                try {
                    const response = await new request_builder_1.RequestBuilder(hostcfgData_types_1.PROTO.HTTPS, `${baseDestiUrl}/api/configuration/v1/hostcfg/${id}`, request_builder_1.RequestMethod.DELETE)
                        .addAgent()
                        .setHeaders({ Authorization: "Bearer " + targetDestination.token })
                        .setRequestTimeout(5000)
                        .processRequest();
                }
                catch (error) {
                    console.log(error);
                    failedToDelete = true;
                }
            }
        }
        const target = await this.configurationService.getTargetDestination(target_destination_id);
        const joinedDevices = await this.configurationService.joinDestinationDevices(target_destination_id);
        const deleted = await this.configurationService.deleteTargetDestination(target_destination_id);
        if (deleted) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.PROXY_DELETE, "Cnetral site: " + target.destination + " - " + joinedDevices);
            if (failedToDelete) {
                return {
                    statusCode: common_1.HttpStatus.OK,
                    message: 'Target destination deleted',
                    error: "Failed to delete some device configurations on central destination, you might need to delete them manually."
                };
            }
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Target destination deleted'
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error during delete", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async fetchDestinationToken(tokenAuthDTO) {
        const tokenBody = {
            username: tokenAuthDTO.username,
            password: tokenAuthDTO.password
        };
        const options = {
            agent: new https.Agent({
                rejectUnauthorized: false,
            }),
            method: "POST",
            timeout: 20000,
            headers: {
                'Content-Type': 'application/json'
            }
        };
        function doRequest(url, options, body) {
            return new Promise((resolve, reject) => {
                const req = https.request(url, options, (res) => {
                    res.setEncoding('utf8');
                    let responseBody = '';
                    if (res.statusCode !== 201) {
                        if (res.statusCode === 401) {
                            reject(new xormon_exception_1.XormonException("Wrong credentials", common_1.HttpStatus.UNAUTHORIZED));
                        }
                        reject(new xormon_exception_1.XormonException(res.statusMessage, res.statusCode));
                    }
                    res.on('data', (chunk) => {
                        responseBody += chunk;
                    });
                    res.on('end', () => {
                        resolve(JSON.parse(responseBody));
                    });
                });
                req.on('timeout', () => {
                    reject(new xormon_exception_1.XormonException("Target destination unreachable", common_1.HttpStatus.REQUEST_TIMEOUT));
                });
                req.on('error', (err) => {
                    reject(err);
                });
                req.write(JSON.stringify(body));
                req.end();
            });
        }
        try {
            return await doRequest(tokenAuthDTO.url, options, tokenBody);
        }
        catch (error) {
            console.error("[PROXY] ", error);
            if (error.code && error.code === "EHOSTUNREACH") {
                throw new xormon_exception_1.XormonException("No route to host", common_1.HttpStatus.SERVICE_UNAVAILABLE);
            }
            if (error.code && error.code === "ERR_INVALID_URL") {
                throw new xormon_exception_1.XormonException("No route to host", common_1.HttpStatus.SERVICE_UNAVAILABLE);
            }
            if (error instanceof xormon_exception_1.XormonException) {
                if (error.getStatus() === common_1.HttpStatus.UNAUTHORIZED) {
                    throw new xormon_exception_1.XormonException("Wrong credentials", common_1.HttpStatus.I_AM_A_TEAPOT);
                }
                throw error;
            }
            throw new xormon_exception_1.XormonException("Couldn't reach destination", common_1.HttpStatus.SERVICE_UNAVAILABLE);
        }
    }
    async getHostCfg(_class, hw_type) {
        let hostcfg = await this.hostCfgService.getHostCfgData(_class, hw_type);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: (await License_1.License.checkHostcfgLimits(hostcfg))
                .sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }))
        };
    }
    async saveHostCfg(request, hostcfg, source) {
        const saved = await this.hostCfgService.saveHostcfgData(hostcfg, source);
        if (saved.hostcfg_id) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.HOSTCFG_ADD, hostcfg.label + ' [' + hostcfg.hw_type + ']');
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'successfully saved',
                data: saved,
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while saving hostcfg", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateHostCfg(request, headers, hostcfg_id, hostcfg) {
        const [updated, old] = await this.hostCfgService.updateHostcfgData(hostcfg_id, hostcfg);
        if (updated) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.HOSTCFG_EDIT, hostcfg.label === old.label ? hostcfg.label + ' [' + hostcfg.hw_type + ']' : "Old alias: " + old.label + ", new alias: " + hostcfg.label + " [" + hostcfg.hw_type + "]");
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'hostcfg successfully updated',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while updating hostcfg", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteHostCfg(request, headers, hostcfg_id) {
        let hostcfg = await this.hostCfgService.getHostCfg(hostcfg_id);
        if (!hostcfg || !hostcfg_id) {
            throw new xormon_exception_1.XormonException("Hostcfg not found", common_1.HttpStatus.NOT_FOUND);
        }
        let deleted = undefined;
        try {
            await Item_1.Item.removeDeviceInfo(hostcfg_id);
            deleted = await this.hostCfgService.deleteHostCfg(hostcfg);
        }
        catch (e) {
            throw new xormon_exception_1.XormonException("An error occurred while deleting hostcfg", common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        if (deleted) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.HOSTCFG_DELETE, hostcfg.label + ' [' + hostcfg.hw_type + ']');
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'deleted ' + hostcfg.label,
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while deleting hostcfg", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async disableHostCfg(request, hostcfgs) {
        const updated = await this.hostCfgService.disableHostCfgs(hostcfgs);
        if (updated.length > 0 && hostcfgs.length > 0) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'hostcfgs successfully disabled',
                data: updated
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while disabling hostcfgs", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateHostcfgLabels(hostcfgs) {
        const updated = await this.hostCfgService.updateHostcfgLabel(hostcfgs);
        if (updated) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'hostcfg successfully updated',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while updating hostcfg labels", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteObjectItemWithRelations(request, headers, item_id) {
        try {
            await Item_1.Item.removeDeviceInfo(item_id);
            return {
                statusCode: common_1.HttpStatus.OK,
            };
        }
        catch {
            throw new xormon_exception_1.XormonException("An error occurred while deleting object item", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getOSAgentStatus(request) {
        let agents = await this.configurationService.getOSAgentStatus(request);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: agents && agents.length === 0 ? [] : agents
        };
    }
    async getFabrics() {
        const fabrics = await this.fabricService.getFabrics();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: fabrics.map(fabric => {
                return {
                    fabric_id: fabric.item_id,
                    label: fabric.label
                };
            }),
        };
    }
    async saveFabric(fabric) {
        const saved = await this.fabricService.saveFabric({
            item_id: (0, uuid_1.v4)(),
            object_id: 'sanbrcd',
            label: fabric.label,
            hw_type: 'sanbrcd',
            subsystem: 'fabric'
        });
        if (saved) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'successfully saved',
                data: {
                    fabric_id: saved.item_id,
                    label: saved.label
                }
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while saving fabric", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateFabric(fabric_id, fabric) {
        const updated = await this.fabricService.saveFabric({
            object_id: 'san',
            item_id: fabric_id,
            label: fabric.label,
            hw_type: 'sanbrcd',
            subsystem: 'fabric'
        });
        if (updated) {
            const hostcfgs = await this.hostCfgService.getHostCfgByHWType('sanbrcd');
            hostcfgs.forEach(async (hostcfg) => {
                if (hostcfg['data']['fabric_id'] && hostcfg['data']['fabric_id'] === fabric_id) {
                    hostcfg['data']['fabric_label'] = fabric.label;
                    await this.hostCfgService.updateHostCfg(hostcfg.hostcfg_id, hostcfg);
                }
            });
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'successfully saved',
                data: {
                    fabric_id: updated.item_id,
                    label: updated.label
                }
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while updating fabric", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteFabric(fabric_id) {
        try {
            const hostcfgs = await this.hostCfgService.getHostCfgByHWType('sanbrcd');
            hostcfgs.forEach(async (hostcfg) => {
                if (hostcfg['data']['fabric_id'] && hostcfg['data']['fabric_id'] === fabric_id) {
                    hostcfg['data']['fabric_id'] = null;
                    if (hostcfg['data']['fabric_label']) {
                        hostcfg['data']['fabric_label'] = null;
                    }
                    await this.hostCfgService.updateHostCfg(hostcfg.hostcfg_id, hostcfg);
                }
            });
            const deleted = await this.fabricService.deleteFabric(fabric_id);
            if (deleted) {
                return {
                    statusCode: common_1.HttpStatus.CREATED,
                    message: 'successfully deleted'
                };
            }
            else {
                throw new xormon_exception_1.XormonException("An error occurred while deleting fabric", common_1.HttpStatus.BAD_REQUEST);
            }
        }
        catch (e) {
            throw new xormon_exception_1.XormonException(e, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getMenugroups(type) {
        const menuGroups = await this.menugroupService.getMenugroups(type);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuGroups.map(group => {
                return {
                    group_id: group.item_id,
                    label: group.label
                };
            }),
        };
    }
    async saveMenugroup(menuGroup) {
        const saved = await this.fabricService.saveFabric({
            item_id: (0, uuid_1.v4)(),
            object_id: 'oracledb',
            label: menuGroup.label,
            hw_type: 'oracledb',
            subsystem: menuGroup.type
        });
        if (saved) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'successfully saved',
                data: {
                    group_id: saved.item_id,
                    label: saved.label
                }
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while saving menugroup", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateMenuGroup(group_id, menuGroup) {
        const getOne = await this.menugroupService.getMenuGroup(group_id);
        const updated = await this.menugroupService.saveMenuGroups({
            object_id: 'oracledb',
            item_id: group_id,
            label: menuGroup.label,
            hw_type: 'oracledb',
            subsystem: getOne.subsystem
        });
        if (updated) {
            const hostcfgs = await this.hostCfgService.getHostCfgByHWType('oracledb');
            hostcfgs.forEach(async (hostcfg) => {
                if (hostcfg['data']['main_group_id'] && hostcfg['data']['main_group_id'] === group_id) {
                    hostcfg['data']['main_group_id'] = updated.object_id;
                    hostcfg['data']['main_group'] = menuGroup.label;
                    await this.hostCfgService.updateHostCfg(hostcfg.hostcfg_id, hostcfg);
                }
                else if (hostcfg["data"]["sub_group_id"] && hostcfg["data"]["sub_group_id"] === group_id) {
                    hostcfg['data']['sub_group_id'] = updated.object_id;
                    hostcfg['data']['sub_group'] = menuGroup.label;
                    await this.hostCfgService.updateHostCfg(hostcfg.hostcfg_id, hostcfg);
                }
            });
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'successfully saved',
                data: {
                    group_id: updated.item_id,
                    label: updated.label
                }
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while updating menugroup", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteMenugroup(group_id) {
        try {
            const hostcfgs = await this.hostCfgService.getHostCfgByHWType('oracledb');
            hostcfgs.forEach(async (hostcfg) => {
                if (hostcfg['data']['main_group_id'] && hostcfg['data']['main_group_id'] === group_id) {
                    hostcfg['data']['main_group_id'] = null;
                    if (hostcfg['data']['main_group']) {
                        hostcfg['data']['main_group'] = null;
                    }
                    if (hostcfg["data"]["sub_group_id"]) {
                        hostcfg['data']['sub_group_id'] = null;
                    }
                    if (hostcfg["data"]["sub_group"]) {
                        hostcfg['data']['sub_group'] = null;
                    }
                    await this.hostCfgService.updateHostCfg(hostcfg.hostcfg_id, hostcfg);
                }
                else if (hostcfg['data']['sub_group_id'] && hostcfg['data']['sub_group_id'] === group_id) {
                    if (hostcfg["data"]["sub_group_id"]) {
                        hostcfg['data']['sub_group_id'] = null;
                    }
                    if (hostcfg["data"]["sub_group"]) {
                        hostcfg['data']['sub_group'] = null;
                    }
                    await this.hostCfgService.updateHostCfg(hostcfg.hostcfg_id, hostcfg);
                }
            });
            const deleted = await this.menugroupService.deleteMenuGroup(group_id);
            if (deleted) {
                return {
                    statusCode: common_1.HttpStatus.CREATED,
                    message: 'successfully deleted'
                };
            }
            else {
                throw new xormon_exception_1.XormonException("An error occurred while deleting menugroup", common_1.HttpStatus.BAD_REQUEST);
            }
        }
        catch (e) {
            throw new xormon_exception_1.XormonException(e, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async uploadVmwareSdk(file) {
        try {
            const fs = require('fs');
            if (!fs.existsSync(this.TMP_PATH)) {
                await fs.mkdirSync(this.TMP_PATH);
            }
            await fs.writeFileSync(this.TMP_PATH + "/vmware-sdk.tar.gz", file.buffer);
            const installed = await VmWareSDK_1.VmWareSDK.install();
            if (installed) {
                await Settings_1.Settings.save({ 'GLOB_VMWARE_SDK': '1' });
                return {
                    statusCode: common_1.HttpStatus.CREATED,
                    message: 'vmware sdk installed successfully'
                };
            }
            else {
                throw new xormon_exception_1.XormonException("An error occurred while installing the SDK", common_1.HttpStatus.BAD_REQUEST);
            }
        }
        catch (e) {
            console.error("[VmWareSDKUploadError]: " + e);
            throw new xormon_exception_1.XormonException("The VMware SDK archive is not valid", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async upgrade(file) {
        try {
            const fs = require('fs');
            console.log(file.originalname);
            if (!fs.existsSync(this.TMP_PATH)) {
                await fs.mkdirSync(this.TMP_PATH);
            }
            const filename = "upgrade";
            let extension = '';
            if (file.originalname.endsWith(".tar.gz")) {
                extension = '.tar.gz';
            }
            else if (file.originalname.endsWith(".tar")) {
                extension = '.tar';
            }
            else {
                throw new xormon_exception_1.XormonException("File extension not supported", common_1.HttpStatus.BAD_REQUEST);
            }
            await fs.writeFileSync(this.TMP_PATH + "/" + filename + extension, file.buffer);
            const upgraded = await Upgrade_1.Upgrade.install(filename + extension);
            if (upgraded.success) {
                return {
                    statusCode: common_1.HttpStatus.CREATED,
                    message: upgraded.message
                };
            }
            else {
                throw new xormon_exception_1.XormonException("An error occured during upgrade: " + upgraded.message, common_1.HttpStatus.BAD_REQUEST);
            }
        }
        catch (error) {
            console.error("[UpgradeError]: " + error);
            if (error instanceof xormon_exception_1.XormonException) {
                throw error;
            }
            else {
                throw new xormon_exception_1.XormonException("An error occured during upgrade: " + error, common_1.HttpStatus.BAD_REQUEST);
            }
        }
    }
    async saveLicense(file) {
        const fs = require('fs');
        let licenseCheck;
        try {
            licenseCheck = await License_1.License.check(file.buffer.toString());
        }
        catch (e) {
            console.error("[LicenseUploadError]: " + e);
            throw new xormon_exception_1.XormonException("License is not valid", common_1.HttpStatus.BAD_REQUEST);
        }
        if (licenseCheck.verified) {
            await fs.writeFileSync(License_1.License.LICENSE_PATH + "/xormon.lic", file.buffer);
            if (process.env['APP_FREE']) {
                process.env['APP_FREE'] = "false";
            }
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'successfully uploaded',
                data: {
                    expirationDate: licenseCheck.expirationDate,
                    technologies: licenseCheck.technologies
                }
            };
        }
        else if (licenseCheck.expirationDate && new Date(licenseCheck.expirationDate).getTime() < new Date().getTime()) {
            throw new xormon_exception_1.XormonException("License is already expired", common_1.HttpStatus.BAD_REQUEST);
        }
        else {
            throw new xormon_exception_1.XormonException("License is not valid", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async saveMicroservicesData(data) {
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: '',
        };
    }
    async getMicroservicesData(microservice, key) {
        return {
            data: {},
        };
    }
    async saveMicroservicesJob(job) {
        if (!Object.values(microserviceJob_constants_1.JOB_STATUS).includes(job.status)) {
            job.status = microserviceJob_constants_1.JOB_STATUS.unknown;
        }
        try {
            let status = await this.configurationService.getMicroservicesJobStatus(job.microservice, job.job, job.identification, job.pid);
            if (status && job.status) {
                await this.configurationService.saveMicroserviceJobStatus({
                    microservice: job.microservice,
                    job: job.job,
                    identification: job.identification,
                    pid: job.pid,
                    status: job.status,
                    start: job.start ? job.start : undefined,
                    end: job.end ? job.end : undefined
                });
            }
            else {
                if (job.start && !job.end) {
                    await this.configurationService.saveMicroserviceJobStatus({
                        microservice: job.microservice,
                        job: job.job,
                        identification: job.identification,
                        pid: job.pid,
                        status: 'running',
                        start: job.start,
                        end: null
                    });
                }
            }
        }
        catch (e) {
            console.error(`[${job.microservice}] [${job.identification}] save status error: ${e}`);
            throw new xormon_exception_1.XormonException("Bad job params", common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: 'job updated',
        };
    }
    async getMicroservicesJobStatus(microservice) {
        let jobs = microservice
            ? await this.configurationService.getMicroservicesJobsStatus(microservice)
            : await this.configurationService.getMicroservicesJobsStatus();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: jobs,
        };
    }
    async getSettings(filter, prefix) {
        let filters = filter ? filter.split(',') : undefined;
        let items = filters || prefix ? await Settings_1.Settings.get({
            keys: filters,
            prefix: prefix
        }) : await Settings_1.Settings.all();
        for (const key in items) {
            if (Settings_1.Settings.hided.includes(key)) {
                items[key] = undefined;
            }
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: items
        };
    }
    async updateSettings(request, headers, settings) {
        const update = await Settings_1.Settings.save(settings);
        if (update) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.SETTINGS_UPDATE, Object.keys(settings).toString());
            if (Object.keys(settings).includes("SMTP_HOST")) {
                PM2_1.PM2.sendMessage('backend', {
                    action: "refresh-transport"
                });
                PM2_1.PM2.sendMessage('metrics-server', {
                    action: "refresh-transport"
                });
                PM2_1.PM2.sendMessage('tasks-server', {
                    action: "refresh-transport"
                });
                if (settings['SMTP_HOST'] === "" || settings['SMTP_HOST'] === null) {
                    this.mailService.destroyTransport();
                }
                else {
                    this.mailService.refreshTransport();
                }
            }
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Settings updated successfully',
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Settings not updated', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteSettings(request, headers, keys) {
        const deleted = await Settings_1.Settings.delete(keys);
        if (deleted) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.SETTINGS_DELETE, keys.toString());
            if (keys.includes("SMTP_HOST")) {
                this.mailService.refreshTransport();
            }
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Settings deleted successfully',
            };
        }
        else {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Some keys are not deleted (non-existent or protected)',
            };
        }
    }
    async saveCertificate(request, files) {
        if (files.crt[0] && files.crt[0]) {
            try {
                const { X509Certificate, createPrivateKey } = require('crypto');
                const crt = new X509Certificate(files.crt[0].buffer);
                if (new Date(crt.validTo).getTime() < Date.now()) {
                    throw new xormon_exception_1.XormonException("Certificate has expired", common_1.HttpStatus.BAD_REQUEST);
                }
                const key = createPrivateKey(files.key[0].buffer);
                const value = crt.checkPrivateKey(key);
                if (!value) {
                    throw new xormon_exception_1.XormonException("Certificate is not signed by the given public key", common_1.HttpStatus.BAD_REQUEST);
                }
                const fs = require('fs');
                if (!fs.existsSync(App_1.App.CERT_FILES_PATH)) {
                    await fs.mkdirSync(App_1.App.CERT_FILES_PATH);
                }
                let keyFileName = files.key[0].originalname;
                let crtFileName = files.crt[0].originalname;
                await fs.writeFileSync(App_1.App.CERT_FILES_PATH + "/" + keyFileName, files.key[0].buffer);
                await fs.writeFileSync(App_1.App.CERT_FILES_PATH + "/" + crtFileName, files.crt[0].buffer);
                await Settings_1.Settings.save({
                    CERT_KEY: App_1.App.CERT_FILES_PATH + "/" + keyFileName,
                    CERT_CRT: App_1.App.CERT_FILES_PATH + "/" + crtFileName
                });
                PM2_1.PM2.restartService('backend');
            }
            catch (e) {
                if (e.response) {
                    console.error(`[UploadCertificate] Error: ${e.response.error}`);
                    throw e;
                }
                else {
                    throw new xormon_exception_1.XormonException("An error has occured (Invalid certificate files?)", common_1.HttpStatus.BAD_REQUEST);
                }
            }
        }
        else {
            throw new xormon_exception_1.XormonException("Empty request", common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: "Certificate successfully uploaded"
        };
    }
    async mailTest(body) {
        try {
            await this.mailService.verifyTransport();
            await this.mailService.send([body.mail], "Test e-mail", "Test email from the XORMON monitoring system");
        }
        catch (e) {
            let err = e.toString();
            if (e.library && e.reason) {
                err = `[${e.library}]: ${e.reason}`;
            }
            throw new xormon_exception_1.XormonException(err, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'test email sent',
        };
    }
    async getClasses(active) {
        let classes = active ? await this.configurationService.getClasses(active) : await this.configurationService.getClasses();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: classes,
        };
    }
    async getHwTypes(active) {
        let hw_types = active ? await this.configurationService.getHwTypes(active) : await this.configurationService.getHwTypes();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: hw_types,
        };
    }
    async getSubsystems(hw_type) {
        let subsystems = hw_type ? await this.configurationService.getSubsystems(hw_type) : await this.configurationService.getSubsystems();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: subsystems,
        };
    }
    async getMailGroups(request) {
        const user = await User_1.User.getUserByRequest(request);
        const mailGroups = await this.mailGroupService.getMailGroups(user.user_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: mailGroups,
        };
    }
    async createMailGroup(request, mailGroup) {
        const user = await User_1.User.getUserByRequest(request);
        const createdMailGroup = await this.mailGroupService.saveMailGroup(mailGroup, user.user_id);
        if (createdMailGroup) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'mail group created',
                data: createdMailGroup,
            };
        }
        else {
            throw new xormon_exception_1.XormonException('error during mail group creation', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteMailGroup(request, mail_group_id) {
        const user = await User_1.User.getUserByRequest(request);
        if (!this.mailGroupService.checkMailGroupOwner(mail_group_id, user.user_id)) {
            throw new common_1.ForbiddenException();
        }
        const deleted = this.mailGroupService.deleteMailGroup(mail_group_id);
        if (deleted) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'mail group deleted',
            };
        }
        else {
            throw new xormon_exception_1.XormonException('error during mail group deleting', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateMailGroup(request, mail_group_id, mailGroup) {
        const user = await User_1.User.getUserByRequest(request);
        if (!this.mailGroupService.checkMailGroupOwner(mail_group_id, user.user_id)) {
            throw new common_1.ForbiddenException();
        }
        const updated = await this.mailGroupService.updateMailGroup(mail_group_id, mailGroup);
        if (updated) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'mail group updated',
            };
        }
        else {
            throw new xormon_exception_1.XormonException('error on mail group update', common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
    async test() {
        let array = ['fiber', 'generic', 'genericphysical', 'genericvirtual', 'hea', 'mempool', 'pool', 'sea', 'server', 'sharedstoragepool', 'sriovlogical', 'sriovphysical', 'virtualethernet', 'vm'];
        array.forEach(element => {
            this.metricsTableService.reGenerate('power', element);
        });
        return {
            statusCode: common_1.HttpStatus.OK,
        };
    }
    async getMicroserviceVariable(hw_type, hostcfg_id) {
        if (hostcfg_id && hw_type) {
            throw new xormon_exception_1.XormonException('error only one query parameter needs to be specified', common_1.HttpStatus.BAD_REQUEST);
        }
        try {
            const microserviceVariables = {
                microserviceVariables: await this.microserviceVariablesService.getByTypeOrConfig(hw_type, hostcfg_id)
            };
            return {
                statusCode: common_1.HttpStatus.OK,
                message: "succesfuly fetched microservice variable",
                data: microserviceVariables,
            };
        }
        catch (error) {
            throw new xormon_exception_1.XormonException('failed at fetching microservice variables', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async createMicroserviceVariable(request, createMicroserviceVariablesDTO) {
        let microserviceVariables = [];
        const auditLog = this.microserviceVariablesService.auditLog();
        for (const variable of createMicroserviceVariablesDTO.microserviceVariables) {
            if (!variable.key) {
                throw new xormon_exception_1.XormonException('error missing required field the key', common_1.HttpStatus.BAD_REQUEST);
            }
            if (!variable.value) {
                throw new xormon_exception_1.XormonException('error missing required field value in one of the values', common_1.HttpStatus.BAD_REQUEST);
            }
            if (variable.hostcfg_id && variable.hw_type) {
                throw new xormon_exception_1.XormonException('error both hostcfg_id and hw_type are specified in one of the values', common_1.HttpStatus.BAD_REQUEST);
            }
            try {
                const existsForKey = await this.microserviceVariablesService.checkIfKeyExistsForTypeOrConf(variable);
                if (existsForKey) {
                    throw new xormon_exception_1.XormonException('error variable for specified key or key and configuration or key and hw_type already exists', common_1.HttpStatus.BAD_REQUEST);
                }
            }
            catch (error) {
                throw new xormon_exception_1.XormonException('error variable for specified key or key and configuration or key and hw_type already exists', common_1.HttpStatus.BAD_REQUEST);
            }
            try {
                const microserviceVariable = await this.microserviceVariablesService.createMicroserviceVariable(variable);
                microserviceVariables.push(microserviceVariable);
                Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.VARIABLE_ADD, await auditLog(microserviceVariable));
            }
            catch (error) {
                throw new xormon_exception_1.XormonException(error, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: `sucessfuly created microservice variable`,
            data: { microserviceVariables: microserviceVariables },
        };
    }
    async deleteMicroserviceVariable(request, deleteIds) {
        try {
            const variables = await this.microserviceVariablesService.getByIds(deleteIds.ids);
            const deleted = await this.microserviceVariablesService.deleteById(deleteIds.ids);
            const auditLog = this.microserviceVariablesService.auditLog();
            if (!deleted) {
                throw new xormon_exception_1.XormonException(`error failed at deleting microservice variable`, common_1.HttpStatus.BAD_REQUEST);
            }
            for (const variable of variables) {
                Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.VARIALBE_DELETE, await auditLog(variable));
            }
        }
        catch (error) {
            throw new xormon_exception_1.XormonException(`error failed at deleting microservice variables`, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            message: `sucessfuly deleted microservice variables`,
        };
    }
    async updateMicroserviceVariable(request, microserviceVariableDTO, microservice_variables_id) {
        try {
            const exists = await this.microserviceVariablesService.findById(microservice_variables_id);
            if (!exists) {
                throw new xormon_exception_1.XormonException(`error cannot update, microservice varialbe with id: ${microservice_variables_id} does not exist.`, common_1.HttpStatus.BAD_REQUEST);
            }
        }
        catch {
            throw new xormon_exception_1.XormonException(`error cannot update, microservice varialbe with id: ${microservice_variables_id} does not exist.`, common_1.HttpStatus.BAD_REQUEST);
        }
        try {
            const updated = await this.microserviceVariablesService.updateMicroserviceVariable(microservice_variables_id, microserviceVariableDTO);
            if (updated) {
                if (updated.value !== microserviceVariableDTO.value || updated.key !== microserviceVariableDTO.key) {
                    Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.VARIABLE_UPDATE, await this.microserviceVariablesService.auditLog()(microserviceVariableDTO));
                }
                return {
                    statusCode: common_1.HttpStatus.OK,
                    message: `sucessfuly updated microservice variable`,
                    microserviceVariableDTO: microserviceVariableDTO
                };
            }
        }
        catch (error) {
            if (error.response && error.response.error === "key pair already exists") {
                throw new xormon_exception_1.XormonException(`error failed at updating microservice variable with id: ${microservice_variables_id}, key already exists`, common_1.HttpStatus.BAD_REQUEST);
            }
            throw new xormon_exception_1.XormonException(`error failed at updating microservice variable with id: ${microservice_variables_id}`, common_1.HttpStatus.BAD_REQUEST);
        }
        throw new xormon_exception_1.XormonException(`error failed at updating microservice variable`, common_1.HttpStatus.BAD_REQUEST);
    }
    async itemGroupProperties(itemGroupConfigurationRequest) {
        return {
            statusCode: common_1.HttpStatus.OK,
            data: await ItemGroupWrapper_1.ItemGroupWrapper.getPropertiesByData(itemGroupConfigurationRequest, itemGroupConfigurationRequest.subsystem, itemGroupConfigurationRequest.hw_type, itemGroupConfigurationRequest.class),
        };
    }
    async selfMonitoring(request) {
        const item = await (0, Linux_1.getSelfMonitoringAgent)();
        if (!item) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    title: "Self monitoring",
                    tabs: [{
                            title: "Self monitoring",
                            content: [{
                                    type: pageResponse_constants_1.BOX_TYPE.TextBox,
                                    text: "No data found"
                                }]
                        }]
                }
            };
        }
        let data = await (new Linux_1.LinuxServerPage(item.item_id, request)).getPage();
        let ndata = await (new Timescaledb_1.TimescaledbSelfmonitoringPage('selfmonitdb')).getPage();
        data.tabs.push(...ndata.tabs);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: data
        };
    }
};
exports.ConfigurationController = ConfigurationController;
__decorate([
    openapi.ApiOperation({ description: "stores configuration data from JSON format" }),
    (0, swagger_1.ApiBody)({
        schema: {
            type: 'object',
            example: {
                'f6969d67-453d-4337-970c-ccc72f5d8c43': {
                    name: 'testItem',
                    num_cpu: 6,
                    ip: '10.0.0.2',
                },
            },
        },
    }),
    (0, common_1.Post)('v1/properties'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveConfigrationJSON", null);
__decorate([
    openapi.ApiOperation({ description: "stores configuration data from CSV format" }),
    (0, swagger_1.ApiBody)({
        schema: {
            type: 'string',
            example: 'uuid;name;num_cpu;ip;\n\
        f6969d67-453d-4337-970c-ccc72f5d8c43;testItem;6;10.0.0.2;\n\
        4aa4351f-3b05-420f-9060-5814740fe5ac;testItem2;6;10.0.0.3;',
        },
    }),
    (0, swagger_1.ApiHeader)({ name: 'Content-Type', description: 'text/plain' }),
    (0, common_1.Post)('v2/properties'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveConfigrationCSV", null);
__decorate([
    openapi.ApiOperation({ description: "returns configuration data based on the UUID" }),
    (0, common_1.Get)('v1/properties/:uuid'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Param)()),
    __param(1, (0, common_1.Response)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getConfigurationByUUID", null);
__decorate([
    (0, swagger_1.ApiBody)({
        schema: {
            type: 'object',
            example: {
                data: {
                    'f6969d67-453d-4337-970c-ccc72f5d8c43': {
                        name: 'testItem',
                        num_cpu: 6,
                        ip: '10.0.0.2',
                    },
                }
            },
        },
    }),
    (0, common_1.Post)('v1/properties/find'),
    openapi.ApiResponse({ status: 201, type: require("../../common/configuration.dto").ItemPropertiesDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [configuration_dto_1.FindProperties]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "findProperties", null);
__decorate([
    openapi.ApiOperation({ description: "returns configuration props based on hw_type &\u00A0subsystem" }),
    (0, common_1.Get)('v1/props/:hw_type/:subsystem'),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").PropsResponseDTO }),
    __param(0, (0, common_1.Param)('hw_type')),
    __param(1, (0, common_1.Param)('subsystem')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getConfigurationProperties", null);
__decorate([
    openapi.ApiOperation({ description: "returns configuration data based on the UUID and property name" }),
    (0, common_1.Get)('v1/properties/:uuid/:property_name'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Param)()),
    __param(1, (0, common_1.Response)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getConfiguration", null);
__decorate([
    openapi.ApiOperation({ description: "returns configuration keys" }),
    (0, common_1.Get)('v1/properties_key/:uuid'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").ArrayResponseDTO }),
    __param(0, (0, common_1.Param)('uuid')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getConfigurationKeys", null);
__decorate([
    openapi.ApiOperation({ description: "save architecture" }),
    (0, common_1.Post)('v1/architecture'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Query)('agent')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array, Number]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveArchitecture", null);
__decorate([
    openapi.ApiOperation({ description: "returns target destinations" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Get)('v1/destinations'),
    openapi.ApiResponse({ status: 200, type: require("../../common/targetDestination.dto").TargetDestinationsResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getTargetDestinations", null);
__decorate([
    openapi.ApiOperation({ description: "create target destination" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Post)('v1/destinations'),
    openapi.ApiResponse({ status: 201, type: require("../../common/targetDestination.dto").TargetDestinationResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, targetDestination_dto_1.SaveTargetDestination]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveTargetDestinations", null);
__decorate([
    openapi.ApiOperation({ description: "update target destination" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Put)('v1/destination/:target_destination_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('target_destination_id')),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, targetDestination_dto_1.SaveTargetDestination]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "updateTargetDestinations", null);
__decorate([
    openapi.ApiOperation({ description: "delete target destination" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Delete)('v1/destination/:target_destination_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('target_destination_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "deleteTargetDestinations", null);
__decorate([
    openapi.ApiOperation({ description: "fetches central site proxy token, has to be done this way to bypass browser cors policy" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Post)("v1/destination/token"),
    openapi.ApiResponse({ status: 201, type: require("../../common/tokenResponse.dto").TokenResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [tokenRequest_dto_1.TokenRequestWithURLDTO]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "fetchDestinationToken", null);
__decorate([
    openapi.ApiOperation({ description: "returns hostcfg" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Get)('v1/hostcfg'),
    openapi.ApiResponse({ status: 200, type: require("../../common/hostCfgResponse.dto").HostCfgsResponseDTO }),
    __param(0, (0, common_1.Query)('class')),
    __param(1, (0, common_1.Query)('hw_type')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getHostCfg", null);
__decorate([
    openapi.ApiOperation({ description: "save hostcfg" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Post)('v1/hostcfg'),
    openapi.ApiResponse({ status: 201, type: require("../../common/hostCfgResponse.dto").HostCfgResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Query)('source')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, hostCfgResponse_dto_1.HostCfgDTO, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveHostCfg", null);
__decorate([
    openapi.ApiOperation({ description: "update hostcfg" }),
    (0, common_1.Put)('v1/hostcfg/:hostcfg_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('hostcfg_id')),
    __param(3, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, String, hostCfgResponse_dto_1.HostCfgDTO]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "updateHostCfg", null);
__decorate([
    openapi.ApiOperation({ description: "delete hostcfg" }),
    (0, common_1.Delete)('v1/hostcfg/:hostcfg_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('hostcfg_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "deleteHostCfg", null);
__decorate([
    openapi.ApiOperation({ description: "disable hostcfgs" }),
    (0, common_1.Post)('v1/hostcfg/disable'),
    openapi.ApiResponse({ status: 201, type: require("../../common/hostCfgResponse.dto").HostCfgsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Array]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "disableHostCfg", null);
__decorate([
    (0, common_1.Post)('v1/hostcfg/labels'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "updateHostcfgLabels", null);
__decorate([
    openapi.ApiOperation({ description: "delete object item and its relations" }),
    (0, common_1.Delete)('v1/object_item/:item_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('item_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "deleteObjectItemWithRelations", null);
__decorate([
    (0, swagger_1.ApiBody)({
        schema: {
            type: 'array',
            example: [
                {
                    server: 'xormon',
                    version: '7.60-5',
                    timestamp: 1711022821,
                    ip_address: '192.168.0.1',
                    url: '/api/menu/v1/page/linux/server?item_id=uuid-uuid',
                    uuid: 'uuid-uuid'
                },
            ]
        },
    }),
    (0, common_1.Get)('v1/os_agent'),
    openapi.ApiResponse({ status: 200, type: require("../../common/osAgent.dto").AgentStatusDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getOSAgentStatus", null);
__decorate([
    openapi.ApiOperation({ description: "returns fabric" }),
    (0, common_1.Get)('v1/fabric'),
    openapi.ApiResponse({ status: 200, type: require("../../common/fabric.dto").FabricsResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getFabrics", null);
__decorate([
    openapi.ApiOperation({ description: "save fabric" }),
    (0, common_1.Post)('v1/fabric'),
    openapi.ApiResponse({ status: 201, type: require("../../common/fabric.dto").FabricResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [fabric_dto_1.SaveFabric]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveFabric", null);
__decorate([
    openapi.ApiOperation({ description: "update fabric" }),
    (0, common_1.Put)('v1/fabric/:fabric_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/fabric.dto").FabricResponseDTO }),
    __param(0, (0, common_1.Param)('fabric_id')),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, fabric_dto_1.Fabric]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "updateFabric", null);
__decorate([
    openapi.ApiOperation({ description: "delete fabric" }),
    (0, common_1.Delete)('v1/fabric/:fabric_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)('fabric_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "deleteFabric", null);
__decorate([
    (0, common_1.Get)('v1/menugroup/:type'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuGroup.dto").MenuGroupResponseDTO }),
    __param(0, (0, common_1.Param)("type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getMenugroups", null);
__decorate([
    openapi.ApiOperation({ description: "save menugroup" }),
    (0, common_1.Post)('v1/menugroup'),
    openapi.ApiResponse({ status: 201, type: require("../../common/menuGroup.dto").MenuGroupResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [menuGroup_dto_1.MenuGroupDTO]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveMenugroup", null);
__decorate([
    openapi.ApiOperation({ description: "update menuGroup" }),
    (0, common_1.Put)('v1/menugroup/:group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuGroup.dto").MenuGroupResponseDTO }),
    __param(0, (0, common_1.Param)('group_id')),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, menuGroup_dto_1.MenuGroupDTO]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "updateMenuGroup", null);
__decorate([
    openapi.ApiOperation({ description: "delete menugroup" }),
    (0, common_1.Delete)('v1/menugroup/:group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)('group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "deleteMenugroup", null);
__decorate([
    openapi.ApiOperation({ description: "upload vmware sdk" }),
    (0, common_1.Post)('v1/vmware-sdk'),
    (0, common_1.UseInterceptors)((0, platform_express_1.FileInterceptor)('file')),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.UploadedFile)(new common_1.ParseFilePipe({
        validators: [
            new common_1.FileTypeValidator({ fileType: '.gz' }),
        ]
    }))),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "uploadVmwareSdk", null);
__decorate([
    openapi.ApiOperation({ description: "upgrade" }),
    (0, common_1.Post)('v1/upgrade'),
    (0, common_1.UseInterceptors)((0, platform_express_1.FileInterceptor)('file')),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.UploadedFile)(new common_1.ParseFilePipe({
        validators: [
            new common_1.FileTypeValidator({ fileType: '.(gz|tar)' }),
        ]
    }))),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "upgrade", null);
__decorate([
    openapi.ApiOperation({ description: "save license" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Post)('v1/license'),
    (0, common_1.UseInterceptors)((0, platform_express_1.FileInterceptor)('file')),
    openapi.ApiResponse({ status: 201, type: require("../../common/license.dto").LicenseResponseDTO }),
    __param(0, (0, common_1.UploadedFile)(new common_1.ParseFilePipe({
        validators: [
            new common_1.FileTypeValidator({ fileType: 'lic' }),
        ]
    }))),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveLicense", null);
__decorate([
    openapi.ApiOperation({ description: "@deprecated", deprecated: true }),
    (0, common_1.Post)('v1/microservices/data'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveMicroservicesData", null);
__decorate([
    openapi.ApiOperation({ description: "@deprecated", deprecated: true }),
    (0, common_1.Get)('v1/microservices/data/:microservice'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Param)('microservice')),
    __param(1, (0, common_1.Query)('key')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getMicroservicesData", null);
__decorate([
    openapi.ApiOperation({ description: "save microservices job status" }),
    (0, common_1.Post)('v1/microservices/job/status'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [microserviceJob_dto_1.MicroserviceJobDTO]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveMicroservicesJob", null);
__decorate([
    openapi.ApiOperation({ description: "get microservices job status" }),
    (0, common_1.Get)('v1/microservices/job/status'),
    openapi.ApiResponse({ status: 200, type: require("../../common/microserviceJob.dto").MicroserviceJobsResponseDTO }),
    __param(0, (0, common_1.Query)('microservice')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getMicroservicesJobStatus", null);
__decorate([
    openapi.ApiOperation({ description: "returns settings" }),
    (0, common_1.Get)('v1/settings'),
    openapi.ApiResponse({ status: 200, type: require("../../common/settings.dto").SettingsResponseDTO }),
    __param(0, (0, common_1.Query)('filter')),
    __param(1, (0, common_1.Query)('prefix')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getSettings", null);
__decorate([
    openapi.ApiOperation({ description: "update settings" }),
    (0, swagger_1.ApiBody)({
        schema: {
            example: {
                key1: "value1",
                key2: "value2"
            },
        },
    }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Put)('v1/settings'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "updateSettings", null);
__decorate([
    openapi.ApiOperation({ description: "delete settings" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Delete)('v1/settings'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Array]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "deleteSettings", null);
__decorate([
    (0, common_1.Post)('v1/certificate'),
    (0, common_1.UseInterceptors)((0, platform_express_1.FileFieldsInterceptor)([
        { name: 'crt', maxCount: 1 },
        { name: 'key', maxCount: 1 },
    ])),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.UploadedFiles)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "saveCertificate", null);
__decorate([
    openapi.ApiOperation({ description: "test mail settings" }),
    (0, common_1.Post)('v1/mailtest'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [settings_dto_1.MailTestDTO]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "mailTest", null);
__decorate([
    openapi.ApiOperation({ description: "returns classes" }),
    (0, common_1.Get)('v1/classes'),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ClassesResponseDTO }),
    __param(0, (0, common_1.Query)('active')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getClasses", null);
__decorate([
    openapi.ApiOperation({ description: "returns hw_types" }),
    (0, common_1.UseInterceptors)(adminGroups_interceptor_1.AdminGroupsInterceptor),
    (0, common_1.Get)('v1/hw_types'),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").HwTypesResponseDTO }),
    __param(0, (0, common_1.Query)('active')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getHwTypes", null);
__decorate([
    openapi.ApiOperation({ description: "returns subsystems" }),
    (0, common_1.Get)('v1/subsystems'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Query)('hw_type')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getSubsystems", null);
__decorate([
    openapi.ApiOperation({ description: "return email groups" }),
    (0, common_1.Get)('v1/mail_groups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/mailGroup.dto").MailGroupsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getMailGroups", null);
__decorate([
    openapi.ApiOperation({ description: "create email group" }),
    (0, common_1.Post)('v1/mail_groups'),
    openapi.ApiResponse({ status: 201, type: require("../../common/mailGroup.dto").MailGroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, mailGroup_dto_1.CreateMailGroupDTO]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "createMailGroup", null);
__decorate([
    openapi.ApiOperation({ description: "delete email group" }),
    (0, common_1.Delete)('v1/mail_groups/:mail_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('mail_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Number]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "deleteMailGroup", null);
__decorate([
    openapi.ApiOperation({ description: "update email group" }),
    (0, common_1.Put)('v1/mail_groups/:mail_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('mail_group_id')),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Number, mailGroup_dto_1.MailGroup]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "updateMailGroup", null);
__decorate([
    openapi.ApiOperation({ description: "test, do not use" }),
    (0, common_1.Get)('v1/test'),
    openapi.ApiResponse({ status: 200, type: Object }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "test", null);
__decorate([
    openapi.ApiOperation({ description: "return microserviceVariables" }),
    (0, common_1.Get)('v1/microservices/variables'),
    openapi.ApiResponse({ status: 200, type: require("../../common/microserviceVariables.dto").MicroserviceVariablesResponse }),
    __param(0, (0, common_1.Query)("hw_type")),
    __param(1, (0, common_1.Query)("hostcfg_id")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "getMicroserviceVariable", null);
__decorate([
    openapi.ApiOperation({ description: "create microserviceVariable" }),
    (0, common_1.Post)('v1/microservices/variables'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, microserviceVariables_dto_1.MicroserviceVariablesDTO]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "createMicroserviceVariable", null);
__decorate([
    openapi.ApiOperation({ description: "delete microserviceVariable" }),
    (0, common_1.Delete)('v1/microservices/variables'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "deleteMicroserviceVariable", null);
__decorate([
    openapi.ApiOperation({ description: "update microserviceVariable" }),
    (0, common_1.Put)('v1/microservices/variables/:id'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Param)("id")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, microserviceVariables_dto_1.MicroserviceVariableDTO, String]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "updateMicroserviceVariable", null);
__decorate([
    openapi.ApiOperation({ description: "item group configuration properties" }),
    (0, common_1.Post)('v1/props'),
    (0, common_1.HttpCode)(200),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").PropsResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [architecture_dto_1.ItemGroupConfigurationRequest]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "itemGroupProperties", null);
__decorate([
    openapi.ApiOperation({ description: "self monitoring" }),
    (0, common_1.Get)('v1/monitoring'),
    (0, common_1.HttpCode)(200),
    openapi.ApiResponse({ status: 200, type: require("../../common/pageResponse.dto").PageResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ConfigurationController.prototype, "selfMonitoring", null);
exports.ConfigurationController = ConfigurationController = __decorate([
    (0, swagger_1.ApiTags)('Configuration'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/configuration'),
    __metadata("design:paramtypes", [configuration_service_1.ConfigurationService,
        metricsTable_service_1.MetricsTableService,
        mail_service_1.MailService,
        microserviceVariable_service_1.MicroserviceVariablesService,
        hostCfg_service_1.HostCfgService,
        fabric_service_1.FabricService,
        mailgroup_service_1.MailGroupService,
        menuGroup_service_1.MenuGroupService])
], ConfigurationController);
//# sourceMappingURL=configuration.controller.js.map