"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomGroupController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const customGroup_dto_1 = require("../../common/customGroup.dto");
const pageResponse_dto_1 = require("../../common/pageResponse.dto");
const License_1 = require("../../config/License");
const xormon_exception_1 = require("../../exception/xormon.exception");
const architecture_service_1 = require("../architecture/architecture.service");
const Definitions_1 = require("../metrics/Definitions");
const MetricsWrapper_1 = require("../metrics/MetricsWrapper");
const User_1 = require("../users/User");
const CustomGroupWrapper_1 = require("./CustomGroupWrapper");
const custom_group_service_1 = require("./custom_group.service");
const Stor_1 = require("../menu/menu/Stor");
const class_constants_1 = require("../../common/constants/class.constants");
const Linux_1 = require("../menu/menu/Linux");
const pageResponse_constants_1 = require("../../common/constants/pageResponse.constants");
const ItemGroupWrapper_1 = require("./ItemGroupWrapper");
let CustomGroupController = class CustomGroupController {
    constructor(customGroupService, architectureService) {
        this.customGroupService = customGroupService;
        this.architectureService = architectureService;
    }
    async search(hw_type, subsystem, search, page, limit) {
        limit = limit ? limit : 10;
        let itemsWithName = [];
        let items = await this.customGroupService.search({ hwType: hw_type, subsystem: subsystem, search: search, page: page });
        const metadata = {
            start: page ? page * limit : 0,
            end: page ? (page * limit + limit - 1 >= items.length ? items.length : page * limit + limit - 1) : limit - 1,
            max: items.length - 1,
            page: page ? +page : 0,
        };
        if (metadata.start > metadata.max) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    metadata: {
                        start: metadata.start,
                        end: metadata.end,
                        max: metadata.max,
                        page: metadata.page,
                        info: 'the page number has a higher start index than the max index',
                    },
                    items: [],
                },
            };
        }
        if (items.length >= 1) {
            items = items.slice(metadata.start, metadata.end + 1);
            const parents = await this.customGroupService.findParents(items.map((obj) => {
                return obj.item_id;
            }));
            for (const item of items) {
                itemsWithName.push({
                    item_id: item.item_id,
                    label: item.label,
                    parents: parents[item.item_id] ? parents[item.item_id] : [],
                });
            }
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: {
                metadata: metadata,
                items: itemsWithName,
            },
        };
    }
    async getFolders(request) {
        const user = await User_1.User.getUserByRequest(request);
        const folders = await this.customGroupService.getFoldersTree(user.user_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: folders,
        };
    }
    async getGrouoFolders(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        let tree = [];
        for (const group of user.groups) {
            tree.push({
                label: group.name,
                group_id: group.group_id,
                children: await this.customGroupService.getFoldersTree(undefined, [group.group_id])
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: tree,
        };
    }
    async saveFolder(request, custom_group_folder) {
        const user = await User_1.User.getUserByRequest(request);
        const saved = await this.customGroupService.saveFolder(custom_group_folder, user.user_id);
        if (saved) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    custom_group_folder_id: saved.custom_group_folder_id,
                    label: saved.label,
                },
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Custom group folder not saved", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getFolder(request, custom_group_folder_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupFolderOwner(custom_group_folder_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const folder = await this.customGroupService.getFolder(custom_group_folder_id);
        if (folder) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    custom_group_folder_id: folder.custom_group_folder_id,
                    label: folder.label,
                    path: folder.parent ? this.customGroupService.getFolderParentPath(folder).split('/').reverse().join('/') : '/',
                },
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Custom group folder not found ", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateFolder(request, custom_group_folder_id, custom_group_folder) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupFolderOwner(custom_group_folder_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const update = await this.customGroupService.updateCustomGroupFolder(custom_group_folder_id, custom_group_folder);
        if (update) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'custom group folder updated successfully',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Custom group folder update failed", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteFolder(request, custom_group_folder_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupFolderOwner(custom_group_folder_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const deleted = await this.customGroupService.deleteFolder(custom_group_folder_id);
        if (deleted) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'custom group folder deleted successfully',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Custom group deletion failed", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getCustomGroups(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        const allCustomGroups = await this.customGroupService.getAllCustomGroups(user.user_id, user.groups.map(g => g.group_id));
        const customGroups = allCustomGroups.map((cg) => {
            delete cg.custom_group_folder;
            return {
                custom_group_folder_id: cg.custom_group_folder_id ? cg.custom_group_folder_id : null,
                ...cg,
                groups: ItemGroupWrapper_1.ItemGroupWrapper.formatItemGroups(cg.groups)
            };
        });
        return {
            statusCode: common_1.HttpStatus.OK,
            data: customGroups.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' })),
        };
    }
    async saveCustomGroups(request, custom_group) {
        if (await CustomGroupWrapper_1.CustomGroupWrapper.hasItemStructureAnyItems(custom_group) === false) {
            throw new xormon_exception_1.XormonException("cannot save custom groups without items", common_1.HttpStatus.UNPROCESSABLE_ENTITY);
        }
        const user = await User_1.User.getUserByRequest(request);
        const saved = await this.customGroupService.saveCustomGroup(custom_group, user.user_id);
        if (saved) {
            const cg = await this.customGroupService.getCustomGroup(saved.custom_group_id);
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'custom group saved successfully',
                data: {
                    ...saved,
                    groups: ItemGroupWrapper_1.ItemGroupWrapper.formatItemGroups(cg.groups)
                },
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Custom group save failed", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getCustomGroup(request, custom_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const customGroup = await this.customGroupService.getCustomGroup(custom_group_id);
        if (customGroup) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    custom_group_folder_id: customGroup.custom_group_folder_id,
                    ...customGroup,
                    groups: ItemGroupWrapper_1.ItemGroupWrapper.formatItemGroups(customGroup.groups)
                },
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Custom group not found", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateCustomGroups(request, custom_group_id, custom_group) {
        if (await CustomGroupWrapper_1.CustomGroupWrapper.hasItemStructureAnyItems(custom_group) === false) {
            throw new xormon_exception_1.XormonException("cannot update custom groups without items", common_1.HttpStatus.UNPROCESSABLE_ENTITY);
        }
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const updated = await this.customGroupService.updateCustomGroup(custom_group_id, custom_group);
        if (updated) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'custom group updated successfully',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Custom group update failed", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteCustomGroups(request, custom_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const deleted = await this.customGroupService.deleteCustomGroup(custom_group_id);
        if (deleted) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'custom group deleted successfully',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Custom group deletion failed", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getCustomGroupMetrics(request, custom_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const customGroup = await this.customGroupService.getCustomGroup(custom_group_id);
        if (!customGroup) {
            throw new xormon_exception_1.XormonException("Custom group not found", common_1.HttpStatus.BAD_REQUEST);
        }
        const metrics = await CustomGroupWrapper_1.CustomGroupWrapper.getMetrics(custom_group_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: metrics.map(metric => {
                return {
                    metric: metric.metric,
                    label: metric.longLabel
                        ? metric.longLabel
                        : metric.label
                            ? metric.label
                            : metric.metric,
                    unit: metric.unit,
                    defaultPrefix: metric.defaultPrefix,
                    defaultPrecision: metric.defaultPrecision,
                    shortcut: metric.shortcut
                };
            }),
        };
    }
    async getCustomGroupItems(request, custom_group_id, withParents) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        let items = await CustomGroupWrapper_1.CustomGroupWrapper.getItems(custom_group_id);
        if (withParents) {
            const itemsWithParents = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(await this.architectureService.getItemsByIds(items.map(item => item.item_id), true)));
            items = itemsWithParents.map(itemWithParents => {
                return {
                    item_id: itemWithParents.item_id,
                    label: itemWithParents.label,
                    parents: [
                        itemWithParents.root_parent
                    ]
                };
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: items,
        };
    }
    async getCustomGroupFavorite(request, custom_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: {
                favorite: await this.customGroupService.checkFavoriteCustomGroup(custom_group_id, user.user_id)
            },
        };
    }
    async setCustomGroupFavorite(request, custom_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        if (await this.customGroupService.setFavoriteCustomGroup(custom_group_id, user.user_id)) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Successfully set',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Favorite status was not set", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteCustomGroupFavorite(request, custom_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        if (await this.customGroupService.deleteFavoriteCustomGroup(custom_group_id, user.user_id)) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Successfully deleted',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Favorite status was not deleted", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getCustomGroupsMenu(request) {
        const user = await User_1.User.getUserByRequest(request);
        let items = [];
        const customGroups = await this.customGroupService.getCustomGroups(null, user.user_id);
        customGroups.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const customGroup of customGroups) {
            items.push({
                isLeaf: true,
                itemId: customGroup.custom_group_id,
                pageType: 'custom_group',
                url: '/api/custom_group/v1/menu/item/' + customGroup.custom_group_id,
                title: customGroup.label,
            });
        }
        const customGroupFolders = await this.customGroupService.getRootFolders(user.user_id);
        customGroupFolders.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const customGroupFolder of customGroupFolders) {
            items.push({
                isLeaf: false,
                itemId: customGroupFolder.custom_group_folder_id,
                pageType: 'custom_group_folder',
                url: '/api/custom_group/v1/menu/folder/' + customGroupFolder.custom_group_folder_id,
                title: customGroupFolder.label,
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: items,
        };
    }
    async getGroupsMenu(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        let groupCustomGroupFolders = [];
        const customGroups = await this.customGroupService.getCustomGroups(null, undefined, user.groups.map(g => g.group_id));
        customGroups.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        const customGroupFolders = await this.customGroupService.getRootFolders(undefined, user.groups.map(g => g.group_id));
        customGroupFolders.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const group of user.groups) {
            groupCustomGroupFolders.push({
                group_id: group.group_id,
                title: group.name,
                children: customGroups.filter(cg => cg.group_id === group.group_id).map(customGroup => {
                    return {
                        isLeaf: true,
                        itemId: customGroup.custom_group_id,
                        pageType: 'custom_group',
                        url: '/api/custom_group/v1/menu/item/' + customGroup.custom_group_id,
                        title: customGroup.label,
                    };
                }).concat(customGroupFolders.filter(cgF => cgF.group_id === group.group_id).map(customGroupFolder => {
                    return {
                        isLeaf: false,
                        itemId: customGroupFolder.custom_group_folder_id,
                        pageType: 'custom_group_folder',
                        url: '/api/custom_group/v1/menu/folder/' + customGroupFolder.custom_group_folder_id,
                        title: customGroupFolder.label,
                    };
                }))
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: groupCustomGroupFolders,
        };
    }
    async getCustomGroupMenu(request, custom_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const customGroup = await this.customGroupService.getCustomGroup(custom_group_id);
        if (!customGroup) {
            throw new common_1.NotFoundException();
        }
        let tabs = [];
        let items = await CustomGroupWrapper_1.CustomGroupWrapper.getItems(custom_group_id, customGroup.class == class_constants_1.ClassKey.VIRTUALIZATION && !customGroup.hw_type);
        let metrics = await CustomGroupWrapper_1.CustomGroupWrapper.getMetrics(custom_group_id, true);
        if (customGroup.class == class_constants_1.ClassKey.STORAGE && ['pool', 'volume', 'host', 'node', 'device'].includes(customGroup.subsystem)) {
            const storageCustomGroupMetrics = (0, Stor_1.getStorageCustomGroupMetrics)();
            metrics = storageCustomGroupMetrics.filter(metric => metrics.find(cgMetric => cgMetric.metric === metric.metric));
        }
        const customGroupInfo = await CustomGroupWrapper_1.CustomGroupWrapper.getInfoAndParent(customGroup);
        let isFree = true;
        let licensedTechnologies = undefined;
        if (customGroupInfo.isClassDefined && customGroup.class == 'virtualization') {
            licensedTechnologies = await License_1.License.getTechnologies();
        }
        else {
            isFree = customGroupInfo.isClassDefined
                ? !(await License_1.License.checkTechnology(customGroup.class))
                : !(await License_1.License.checkTechnology(customGroup.hw_type));
        }
        let lastValues;
        if (isFree) {
            const metricsWrapper = new MetricsWrapper_1.MetricsWrapper(metrics.map(m => m.metric), items.map(i => i.item_id));
            lastValues = await metricsWrapper.getLastDailyValuesFromTable();
        }
        for (const metric of metrics) {
            tabs.push(await CustomGroupWrapper_1.CustomGroupWrapper.genTab(items, metric, customGroupInfo.parentType, customGroupInfo.subsystemLabel, isFree, undefined, lastValues, licensedTechnologies));
        }
        if (customGroup.class == class_constants_1.ClassKey.VIRTUALIZATION && customGroup.hw_type === 'power') {
            const linuxMapping = await (0, Linux_1.getLinuxMapping)();
            const mappedIds = linuxMapping.map(item => item.item_id);
            const itemsWithAgent = items.filter(item => mappedIds.includes(item.item_id));
            const itemsWithAgentIds = itemsWithAgent.map(item => item.item_id);
            if (itemsWithAgent.length > 0) {
                const agentIds = linuxMapping.filter(linuxVm => itemsWithAgentIds.includes(linuxVm.item_id)).map(mapping => mapping.agent_id);
                const metrics = (0, Linux_1.getLinuxCustomGroupMetrics)();
                for (const subsystem of (0, Linux_1.getLinuxCustomGroupSubsystems)()) {
                    const subsystemItems = subsystem.subsystem === 'server'
                        ? await (0, Linux_1.getLinuxServers)(agentIds)
                        : await (0, Linux_1.getLinuxMappedChildren)(agentIds, subsystem.subsystem);
                    const subsystemMetrics = metrics[subsystem.subsystem];
                    for (const subsystemMetric of subsystemMetrics) {
                        tabs.push(await CustomGroupWrapper_1.CustomGroupWrapper.genTab(subsystemItems, subsystemMetric, subsystem.subsystem !== 'server' ? {
                            label: customGroupInfo.subsystemLabel,
                            subsystem: 'server'
                        } : undefined, subsystem.subsystem !== 'server' ? subsystem.label : customGroupInfo.subsystemLabel, isFree));
                    }
                }
            }
        }
        const heatmapTab = await CustomGroupWrapper_1.CustomGroupWrapper.genHeatmapTab(customGroup, items, request);
        if (heatmapTab) {
            tabs.push(heatmapTab);
        }
        const page = new pageResponse_dto_1.Page(customGroup.label, tabs);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: page,
        };
    }
    async getCustomGroupDashboard(request, custom_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const customGroup = await this.customGroupService.getCustomGroup(custom_group_id);
        if (!customGroup) {
            throw new common_1.NotFoundException();
        }
        let items = await CustomGroupWrapper_1.CustomGroupWrapper.getItems(custom_group_id, customGroup.class == class_constants_1.ClassKey.VIRTUALIZATION && !customGroup.hw_type);
        let tabs = [];
        const heatmapTab = await CustomGroupWrapper_1.CustomGroupWrapper.genHeatmapTab(customGroup, items, request);
        if (heatmapTab) {
            tabs = heatmapTab.subTabs;
        }
        else {
            tabs.push({
                title: 'Heatmap',
                content: [{
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        text: "No heatmap found"
                    }],
            });
        }
        const page = new pageResponse_dto_1.Page(customGroup.label, tabs);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: page,
        };
    }
    async getCustomGroupFolderMenu(request, custom_group_folder_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupFolderOwner(custom_group_folder_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        let items = [];
        const folder = await this.customGroupService.getFolder(custom_group_folder_id);
        const customGroups = await this.customGroupService.getCustomGroups(folder);
        customGroups.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const customGroup of customGroups) {
            items.push({
                isLeaf: true,
                itemId: customGroup.custom_group_id,
                pageType: 'custom_group',
                url: '/api/custom_group/v1/menu/item/' + customGroup.custom_group_id,
                title: customGroup.label,
            });
        }
        const customGroupFolders = await this.customGroupService.getFolders(folder);
        customGroupFolders.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const customGroupFolder of customGroupFolders) {
            items.push({
                isLeaf: false,
                itemId: customGroupFolder.custom_group_folder_id,
                pageType: 'custom_group_folder',
                url: '/api/custom_group/v1/menu/folder/' + customGroupFolder.custom_group_folder_id,
                title: customGroupFolder.label,
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: items,
        };
    }
    async getPath(request, custom_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!this.customGroupService.checkCustomGroupOwner(custom_group_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        let menuItems = [];
        const customGroup = await this.customGroupService.getCustomGroup(custom_group_id);
        if (!customGroup.custom_group_folder_id) {
            return this.getCustomGroupsMenu(request);
        }
        const customGroupFolder = await this.customGroupService.getFolder(customGroup.custom_group_folder_id);
        const parentsIds = await this.customGroupService.getParentsIds(customGroupFolder);
        menuItems = await this.findCustomGroup(request, (await this.getCustomGroupsMenu(request)).data, parentsIds);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async findCustomGroup(request, menuItems, parents) {
        for (const menuItem of menuItems) {
            if (menuItem.pageType === "custom_group_folder" && parents.includes(menuItem.itemId)) {
                menuItem.children = await this.findCustomGroup(request, (await this.getCustomGroupFolderMenu(request, menuItem.itemId)).data, parents);
            }
        }
        return menuItems;
    }
    async getCustomGroupDashboardFolder(request) {
        const user = await User_1.User.getUserByRequest(request);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: await this.customGroupService.getCustomGroupDashboardFolder(user.user_id),
        };
    }
};
exports.CustomGroupController = CustomGroupController;
__decorate([
    openapi.ApiOperation({ description: "regex preview\nitem search" }),
    (0, common_1.Get)('v1/search'),
    openapi.ApiResponse({ status: 200, type: require("../../common/customGroup.dto").CustomGroupSearchDTO }),
    __param(0, (0, common_1.Query)('hw_type')),
    __param(1, (0, common_1.Query)('subsystem')),
    __param(2, (0, common_1.Query)('search')),
    __param(3, (0, common_1.Query)('page')),
    __param(4, (0, common_1.Query)('limit')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String, Number, Number]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "search", null);
__decorate([
    openapi.ApiOperation({ description: "return all user custom group folders" }),
    (0, common_1.Get)('v1/folders'),
    openapi.ApiResponse({ status: 200, type: require("../../common/customGroup.dto").CustomGroupFoldersResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getFolders", null);
__decorate([
    openapi.ApiOperation({ description: "return all group custom group folders" }),
    (0, common_1.Get)('v1/folders/groups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/customGroup.dto").GroupCustomGroupFoldersTreeDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getGrouoFolders", null);
__decorate([
    openapi.ApiOperation({ description: "return all custom group folders\nsave custom group folder" }),
    (0, common_1.Post)('v1/folders'),
    openapi.ApiResponse({ status: 201, type: require("../../common/customGroup.dto").CustomGroupFolderResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, customGroup_dto_1.CreateCustomGroupFolder]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "saveFolder", null);
__decorate([
    openapi.ApiOperation({ description: "return custom group folder" }),
    (0, common_1.Get)('v1/folders/:custom_group_folder_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/customGroup.dto").CustomGroupFolderResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_folder_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getFolder", null);
__decorate([
    openapi.ApiOperation({ description: "update custom group folder" }),
    (0, common_1.Put)('v1/folders/:custom_group_folder_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_folder_id')),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, customGroup_dto_1.CreateCustomGroupFolder]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "updateFolder", null);
__decorate([
    openapi.ApiOperation({ description: "delete custom group folder" }),
    (0, common_1.Delete)('v1/folders/:custom_group_folder_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_folder_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "deleteFolder", null);
__decorate([
    openapi.ApiOperation({ description: "return all custom groups" }),
    (0, common_1.Get)('v1/custom_groups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/customGroup.dto").CustomGroupsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroups", null);
__decorate([
    openapi.ApiOperation({ description: "save custom group" }),
    (0, common_1.Post)('v1/custom_groups'),
    openapi.ApiResponse({ status: 201, type: require("../../common/customGroup.dto").CustomGroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, customGroup_dto_1.CreateCustomGroupDTO]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "saveCustomGroups", null);
__decorate([
    openapi.ApiOperation({ description: "get custom group" }),
    (0, common_1.Get)('v1/custom_groups/:custom_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/customGroup.dto").CustomGroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroup", null);
__decorate([
    openapi.ApiOperation({ description: "update custom groups" }),
    (0, common_1.Put)('v1/custom_groups/:custom_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, customGroup_dto_1.UpdateCustomGroupDTO]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "updateCustomGroups", null);
__decorate([
    openapi.ApiOperation({ description: "delete custom groups" }),
    (0, common_1.Delete)('v1/custom_groups/:custom_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "deleteCustomGroups", null);
__decorate([
    openapi.ApiOperation({ description: "Returns cg metrics" }),
    (0, common_1.Get)('v1/custom_groups/:custom_group_id/metrics'),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").MetricsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroupMetrics", null);
__decorate([
    openapi.ApiOperation({ description: "return custom group items" }),
    (0, common_1.Get)('v1/custom_groups/:custom_group_id/items'),
    openapi.ApiResponse({ status: 200, type: require("../../common/customGroup.dto").CustomGroupItemDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __param(2, (0, common_1.Query)('withParents')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, Boolean]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroupItems", null);
__decorate([
    (0, common_1.Get)('v1/custom_groups/:custom_group_id/favorite'),
    openapi.ApiResponse({ status: 200, type: require("../../common/customGroup.dto").CustomGroupFavoriteResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroupFavorite", null);
__decorate([
    (0, common_1.Put)('v1/custom_groups/:custom_group_id/favorite'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "setCustomGroupFavorite", null);
__decorate([
    (0, common_1.Delete)('v1/custom_groups/:custom_group_id/favorite'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "deleteCustomGroupFavorite", null);
__decorate([
    (0, common_1.Get)('v1/menu'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroupsMenu", null);
__decorate([
    (0, common_1.Get)('v1/menu/groups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/customGroup.dto").GroupCustomGroupFoldersDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getGroupsMenu", null);
__decorate([
    (0, common_1.Get)('v1/menu/item/:custom_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/pageResponse.dto").PageResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroupMenu", null);
__decorate([
    (0, common_1.Get)('v1/menu/item/:custom_group_id/dashboard'),
    openapi.ApiResponse({ status: 200, type: require("../../common/pageResponse.dto").PageResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroupDashboard", null);
__decorate([
    (0, common_1.Get)('v1/menu/folder/:custom_group_folder_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_folder_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroupFolderMenu", null);
__decorate([
    openapi.ApiOperation({ description: "getPath" }),
    (0, common_1.Get)('v1/menu/getpath/:custom_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('custom_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getPath", null);
__decorate([
    (0, common_1.Get)('v1/menu/dashboard'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CustomGroupController.prototype, "getCustomGroupDashboardFolder", null);
exports.CustomGroupController = CustomGroupController = __decorate([
    (0, swagger_1.ApiTags)('Custom Group'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/custom_group'),
    __metadata("design:paramtypes", [custom_group_service_1.CustomGroupService,
        architecture_service_1.ArchitectureService])
], CustomGroupController);
//# sourceMappingURL=custom_group.controller.js.map