"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomGroupService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../../config/dataSource");
const customGroup_entity_1 = require("../../entity/customGroup.entity");
const customGroupFolder_entity_1 = require("../../entity/customGroupFolder.entity");
const itemGroup_entity_1 = require("../../entity/itemGroup.entity");
const itemGroupItem_entity_1 = require("../../entity/itemGroupItem.entity");
const itemGroupParent_entity_1 = require("../../entity/itemGroupParent.entity");
const itemGroupParentTag_entity_1 = require("../../entity/itemGroupParentTag.entity");
const itemGroupRegexParent_entity_1 = require("../../entity/itemGroupRegexParent.entity");
const itemGroupTag_entity_1 = require("../../entity/itemGroupTag.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const Classes_1 = require("../metrics/Classes");
const typeorm_2 = require("typeorm");
const uuid_1 = require("uuid");
const CustomGroupWrapper_1 = require("./CustomGroupWrapper");
const customGroupFavorite_entity_1 = require("../../entity/customGroupFavorite.entity");
const User_1 = require("../users/User");
let CustomGroupService = class CustomGroupService {
    constructor(customGroupsRepository, customGroupFoldersRepository, objectItemsRepository, customGroupFavoriteRepository) {
        this.customGroupsRepository = customGroupsRepository;
        this.customGroupFoldersRepository = customGroupFoldersRepository;
        this.objectItemsRepository = objectItemsRepository;
        this.customGroupFavoriteRepository = customGroupFavoriteRepository;
    }
    async getCustomGroups(folder, userId, groupIds) {
        return await this.customGroupsRepository.findBy({
            custom_group_folder_id: folder ? folder.custom_group_folder_id : (0, typeorm_2.IsNull)(),
            user_id: userId && !groupIds ? userId : undefined,
            group_id: groupIds ? (0, typeorm_2.In)(groupIds) : folder ? undefined : (0, typeorm_2.IsNull)(),
        });
    }
    async getAllFolders() {
        const trees = await this.customGroupFoldersRepository.findTrees();
        let folders = [];
        for (const tree of trees) {
            folders = folders.concat(this.getFolderChildren(tree, null, ''));
        }
        return folders;
    }
    getFolderChildren(folder, parent, parentPath) {
        let folders = [];
        parentPath += parentPath.length == 0 ? folder.label : '/' + folder.label;
        if (folder.children && folder.children.length >= 1) {
            for (const child of folder.children) {
                folders = folders.concat(this.getFolderChildren(child, folder.custom_group_folder_id, parentPath));
            }
        }
        folders.push({
            label: folder.label,
            custom_group_folder_id: folder.custom_group_folder_id,
            parent: parent,
            path: parentPath,
        });
        return folders;
    }
    getFolderParentPath(folder) {
        let parentPath = '/' + folder.label;
        if (folder.parent) {
            parentPath += this.getFolderParentPath(folder.parent);
        }
        return parentPath;
    }
    async updateCustomGroupFolder(custom_group_folder_id, custom_group_folder) {
        let updated = null;
        if (custom_group_folder.parent) {
            const parent = await this.customGroupFoldersRepository.findOneBy({ custom_group_folder_id: custom_group_folder.parent });
            if (parent) {
                updated = await this.customGroupFoldersRepository.save({
                    custom_group_folder_id: custom_group_folder_id,
                    label: custom_group_folder.label,
                    parent: parent,
                });
                return updated.custom_group_folder_id ? true : false;
            }
        }
        updated = await this.customGroupFoldersRepository.save({
            custom_group_folder_id: custom_group_folder_id,
            label: custom_group_folder.label,
        });
        return updated.custom_group_folder_id ? true : false;
    }
    async getFolders(parent) {
        const folder = await (0, dataSource_1.getDataSource)().getTreeRepository(customGroupFolder_entity_1.CustomGroupFolder).findDescendantsTree(parent);
        return folder.children;
    }
    async getFoldersTree(user_id, groupIds) {
        let trees = await (0, dataSource_1.getDataSource)().getTreeRepository(customGroupFolder_entity_1.CustomGroupFolder).findTrees();
        if (groupIds) {
            trees = trees.filter((tree) => groupIds.includes(tree.group_id));
        }
        else if (user_id) {
            trees = trees.filter((tree) => tree.user_id === user_id && tree.group_id == null);
        }
        return this.filterFolderTree(trees);
    }
    filterFolderTree(folders) {
        for (const folder of folders) {
            folder.user_id = undefined;
            folder.group_id = undefined;
            if (folder.children && folder.children.length >= 1) {
                folder.children = this.filterFolderTree(folder.children);
            }
        }
        return folders;
    }
    async getRootFolders(userId, groupIds) {
        const roots = await (0, dataSource_1.getDataSource)().getTreeRepository(customGroupFolder_entity_1.CustomGroupFolder).findRoots();
        if (userId && !groupIds) {
            return roots.filter((root) => root.user_id === userId && root.group_id == null);
        }
        else if (groupIds) {
            return roots.filter((root) => groupIds.includes(root.group_id));
        }
        return await (0, dataSource_1.getDataSource)().getTreeRepository(customGroupFolder_entity_1.CustomGroupFolder).findRoots();
    }
    async getFolder(custom_group_folder_id) {
        const folder = await this.customGroupFoldersRepository.findOneBy({ custom_group_folder_id: custom_group_folder_id });
        return await (0, dataSource_1.getDataSource)().getTreeRepository(customGroupFolder_entity_1.CustomGroupFolder).findAncestorsTree(folder);
    }
    async saveFolder(folder, userId) {
        if (folder.parent) {
            const parent = await this.getFolder(folder.parent);
            if (parent) {
                return await this.customGroupFoldersRepository.save({
                    parent: parent,
                    label: folder.label,
                    user_id: userId ? userId : null,
                    group_id: parent.group_id ? parent.group_id : undefined,
                });
            }
        }
        return await this.customGroupFoldersRepository.save({
            label: folder.label,
            user_id: userId ? userId : null,
            group_id: folder.group_id ? folder.group_id : null,
        });
    }
    async deleteFolder2(custom_group_folder_id) {
        const deleted = await this.customGroupFoldersRepository.delete({ custom_group_folder_id: custom_group_folder_id });
        return deleted.affected === 1 ? true : false;
    }
    async deleteFolder(custom_group_folder_id) {
        const folder = await this.customGroupFoldersRepository.findOneBy({ custom_group_folder_id: custom_group_folder_id });
        const deleted = await this.customGroupFoldersRepository.remove(folder);
        return deleted ? true : false;
    }
    async getParentsFolders(parent) {
        return await (0, dataSource_1.getDataSource)().getTreeRepository(customGroupFolder_entity_1.CustomGroupFolder).findAncestorsTree(parent);
    }
    async getParentsIds(parent) {
        return this.findInParent(await this.getParentsFolders(parent));
    }
    findInParent(parent) {
        let ids = [parent.custom_group_folder_id];
        if (parent.parent) {
            ids = ids.concat(this.findInParent(parent.parent));
        }
        return ids;
    }
    async getPath(custom_group_folder) {
        const folder = await this.getParentsFolders(custom_group_folder);
        if (folder) {
            let path = await this.findNextPath(folder);
            return '/' + path.split('/').reverse().join('/');
        }
        else {
            return '/';
        }
    }
    async findNextPath(folder) {
        let path = folder.label;
        if (folder.parent) {
            path += '/' + (await this.findNextPath(folder.parent));
        }
        return path;
    }
    async getItems(item_ids) {
        return await this.objectItemsRepository.findByIds(item_ids);
    }
    async getAllCustomGroups(userId, groupIds) {
        return await this.customGroupsRepository.find({
            relations: {
                custom_group_folder: true,
                groups: {
                    items: {
                        _item: true,
                    },
                    parents: {
                        _item: true,
                    },
                    regex_parents: true,
                    tags: {
                        _tag: true,
                    },
                    parent_tags: {
                        _tag: true,
                    },
                },
            },
            where: [{
                    user_id: userId,
                    group_id: (0, typeorm_2.IsNull)()
                }, {
                    group_id: (0, typeorm_2.In)(groupIds)
                }]
        });
    }
    async saveCustomGroup(custom_group, user_id) {
        let savedCustomGroup;
        await (0, dataSource_1.getDataSource)().transaction(async (transactionalEntityManager) => {
            const cgId = (0, uuid_1.v4)();
            if (custom_group.custom_group_folder_id) {
                const folder = await transactionalEntityManager
                    .getRepository(customGroupFolder_entity_1.CustomGroupFolder)
                    .findOneBy({ custom_group_folder_id: custom_group.custom_group_folder_id });
                if (folder) {
                    savedCustomGroup = await transactionalEntityManager.getRepository(customGroup_entity_1.CustomGroup).save({
                        custom_group_id: cgId,
                        custom_group_folder: folder,
                        class: custom_group.class,
                        hw_type: custom_group.hw_type,
                        subsystem: custom_group.subsystem,
                        label: custom_group.label,
                        user_id: user_id ? user_id : null,
                        group_id: folder.group_id ? folder.group_id : null,
                    });
                }
                else {
                    console.warn('[SaveCustomGroup] CG Folder ' + custom_group.custom_group_folder_id + ' not found');
                }
            }
            else {
                savedCustomGroup = await transactionalEntityManager.getRepository(customGroup_entity_1.CustomGroup).save({
                    custom_group_id: cgId,
                    class: custom_group.class,
                    hw_type: custom_group.hw_type,
                    subsystem: custom_group.subsystem,
                    label: custom_group.label,
                    user_id: user_id ? user_id : null,
                    group_id: custom_group.group_id ? custom_group.group_id : null,
                });
            }
            for (const group of custom_group.groups) {
                const igId = (0, uuid_1.v4)();
                await transactionalEntityManager.getRepository(itemGroup_entity_1.ItemGroup).save({
                    custom_group_id: cgId,
                    item_group_id: igId,
                    regex: group.regex,
                    intersection: group.parents
                        ? group.parents.intersection === true
                        : false
                });
                const cgItems = [];
                const parentItems = [];
                const parentRegexItems = [];
                const tags = [];
                const parentTags = [];
                if (group.item_ids) {
                    group.item_ids.forEach((itemId) => {
                        cgItems.push({
                            item_id: itemId,
                            item_group_id: igId,
                        });
                    });
                }
                if (group.tags) {
                    group.tags.forEach((tag) => {
                        tags.push({
                            tag_value_id: tag,
                            item_group_id: igId
                        });
                    });
                }
                if (group.parents && group.parents.item_ids) {
                    group.parents.item_ids.forEach((itemId) => {
                        parentItems.push({
                            item_id: itemId,
                            item_group_id: igId,
                        });
                    });
                }
                if (group.parents && group.parents.regex) {
                    group.parents.regex.forEach((regex) => {
                        parentRegexItems.push({
                            regex: regex.regex,
                            subsystem: regex.subsystem,
                            item_group_id: igId,
                        });
                    });
                }
                if (group.parents && group.parents.tags) {
                    group.parents.tags.forEach((tag) => {
                        parentTags.push({
                            tag_value_id: tag.tag,
                            subsystem: tag.subsystem,
                            item_group_id: igId
                        });
                    });
                }
                if (cgItems.length >= 1) {
                    transactionalEntityManager.getRepository(itemGroupItem_entity_1.ItemGroupItem).save(cgItems);
                }
                if (parentItems.length >= 1) {
                    transactionalEntityManager.getRepository(itemGroupParent_entity_1.ItemGroupParent).save(parentItems);
                }
                if (parentRegexItems.length >= 1) {
                    transactionalEntityManager.getRepository(itemGroupRegexParent_entity_1.ItemGroupRegexParent).save(parentRegexItems);
                }
                if (tags.length >= 1) {
                    transactionalEntityManager.getRepository(itemGroupTag_entity_1.ItemGroupTag).save(tags);
                }
                if (parentTags.length >= 1) {
                    transactionalEntityManager.getRepository(itemGroupParentTag_entity_1.ItemGroupParentTag).save(parentTags);
                }
            }
        });
        return savedCustomGroup;
    }
    async updateCustomGroup(custom_group_id, custom_group) {
        let updated = false;
        try {
            await (0, dataSource_1.getDataSource)().transaction(async (transactionalEntityManager) => {
                if (custom_group.custom_group_folder_id) {
                    const folder = await transactionalEntityManager
                        .getRepository(customGroupFolder_entity_1.CustomGroupFolder)
                        .findOneBy({ custom_group_folder_id: custom_group.custom_group_folder_id });
                    delete custom_group.custom_group_folder_id;
                    if (folder) {
                        const saved = await transactionalEntityManager.getRepository(customGroup_entity_1.CustomGroup).update(custom_group_id, {
                            custom_group_folder: folder,
                            class: custom_group.class,
                            hw_type: custom_group.hw_type,
                            subsystem: custom_group.subsystem,
                            label: custom_group.label,
                            group_id: folder.group_id ? folder.group_id : null,
                        });
                        if (saved.affected === 1) {
                            updated = true;
                        }
                    }
                }
                else if (custom_group.hasOwnProperty('custom_group_folder_id') && custom_group.custom_group_folder_id === null) {
                    delete custom_group.custom_group_folder_id;
                    const saved = await transactionalEntityManager.getRepository(customGroup_entity_1.CustomGroup).update(custom_group_id, {
                        custom_group_folder: null,
                        class: custom_group.class,
                        hw_type: custom_group.hw_type,
                        subsystem: custom_group.subsystem,
                        label: custom_group.label,
                        group_id: custom_group.group_id ? custom_group.group_id : null,
                    });
                    if (saved.affected === 1) {
                        updated = true;
                    }
                }
                else {
                    const saved = await transactionalEntityManager.getRepository(customGroup_entity_1.CustomGroup).update(custom_group_id, {
                        class: custom_group.class,
                        hw_type: custom_group.hw_type,
                        subsystem: custom_group.subsystem,
                        label: custom_group.label,
                        group_id: custom_group.group_id ? custom_group.group_id : null,
                    });
                    if (saved.affected === 1) {
                        updated = true;
                    }
                }
                await transactionalEntityManager.getRepository(itemGroup_entity_1.ItemGroup).delete({ custom_group_id: custom_group_id });
                for (const group of custom_group.groups) {
                    const igId = (0, uuid_1.v4)();
                    let itemIds = [];
                    let parentIds = [];
                    let parentRegexes = [];
                    let tags = [];
                    let parentTags = [];
                    if (group.item_ids) {
                        group.item_ids.forEach((itemId) => itemIds.push(itemId));
                    }
                    if (group.tags) {
                        group.tags.forEach((tag) => tags.push(tag));
                    }
                    if (group.parents && group.parents.item_ids) {
                        group.parents.item_ids.forEach((itemId) => parentIds.push(itemId));
                    }
                    if (group.parents && group.parents.tags) {
                        group.parents.tags.forEach(tag => parentTags.push({
                            tag: tag.tag,
                            subsystem: tag.subsystem
                        }));
                    }
                    if (group.parents && group.parents.regex) {
                        group.parents.regex.forEach((regex) => parentRegexes.push({
                            item_group_id: igId,
                            subsystem: regex.subsystem,
                            regex: regex.regex,
                        }));
                    }
                    await transactionalEntityManager.getRepository(itemGroup_entity_1.ItemGroup).save({
                        item_group_id: igId,
                        custom_group_id: custom_group_id,
                        regex: group.regex,
                        intersection: group.parents
                            ? group.parents.intersection === true
                            : false
                    });
                    if (itemIds) {
                        await transactionalEntityManager.getRepository(itemGroupItem_entity_1.ItemGroupItem).save(itemIds.map((itemId) => {
                            return {
                                item_group_id: igId,
                                item_id: itemId,
                            };
                        }));
                    }
                    if (tags) {
                        await transactionalEntityManager.getRepository(itemGroupTag_entity_1.ItemGroupTag).save(tags.map((tag) => {
                            return {
                                item_group_id: igId,
                                tag_value_id: tag,
                            };
                        }));
                    }
                    if (parentIds) {
                        await transactionalEntityManager.getRepository(itemGroupParent_entity_1.ItemGroupParent).save(parentIds.map((itemId) => {
                            return {
                                item_group_id: igId,
                                item_id: itemId,
                            };
                        }));
                    }
                    if (parentRegexes) {
                        await transactionalEntityManager.getRepository(itemGroupRegexParent_entity_1.ItemGroupRegexParent).save(parentRegexes);
                    }
                    if (parentTags) {
                        await transactionalEntityManager.getRepository(itemGroupParentTag_entity_1.ItemGroupParentTag).save(parentTags.map((tag) => {
                            return {
                                item_group_id: igId,
                                tag_value_id: tag.tag,
                                subsystem: tag.subsystem
                            };
                        }));
                    }
                }
            });
        }
        catch (e) {
            console.error(e);
            return false;
        }
        return updated;
    }
    async getCustomGroup(custom_group_id) {
        return await this.customGroupsRepository.findOne({
            where: {
                custom_group_id: custom_group_id,
            },
            relations: {
                custom_group_folder: true,
                groups: {
                    items: {
                        _item: true,
                    },
                    parents: {
                        _item: true,
                    },
                    regex_parents: true,
                    tags: {
                        _tag: true,
                    },
                    parent_tags: {
                        _tag: true,
                    },
                },
            },
        });
    }
    async deleteCustomGroup(custom_group_id) {
        const deleted = await this.customGroupsRepository.delete({ custom_group_id: custom_group_id });
        return deleted.affected === 1 ? true : false;
    }
    async getObjectItemsByIds(item_ids) {
        return await this.objectItemsRepository.findByIds(item_ids);
    }
    async search(params) {
        let items = this.objectItemsRepository.createQueryBuilder();
        let filtered = 0;
        if (params.hwType) {
            if (params.hwType === 'storage') {
                const hwTypes = Object.keys(Classes_1.Storage.technology);
                items =
                    filtered == 0
                        ? items.where('hw_type IN (:...hwTypes)', { hwTypes: hwTypes })
                        : items.andWhere('hw_type IN (:...hwTypes)', { hwTypes: hwTypes });
            }
            else if (params.hwType === 'san') {
                const hwTypes = Object.keys(Classes_1.San.technology);
                items =
                    filtered == 0
                        ? items.where('hw_type IN (:...hwTypes)', { hwTypes: hwTypes })
                        : items.andWhere('hw_type IN (:...hwTypes)', { hwTypes: hwTypes });
            }
            else {
                items =
                    filtered == 0
                        ? items.where('hw_type = :hw_type', { hw_type: params.hwType })
                        : items.andWhere('hw_type = :hw_type', { hw_type: params.hwType });
            }
            filtered++;
        }
        if (params.subsystem) {
            items =
                filtered == 0
                    ? items.where('subsystem = :subsystem', { subsystem: params.subsystem })
                    : items.andWhere('subsystem = :subsystem', { subsystem: params.subsystem });
            filtered++;
        }
        if (params.search) {
            items =
                filtered == 0
                    ? items.where('label LIKE :label', { label: `%${params.search}%` })
                    : items.andWhere('label LIKE :label', { label: `%${params.search}%` });
            filtered++;
        }
        return await items.orderBy('label', 'ASC').getMany();
    }
    async findParents(item_ids) {
        let dbArray = "('', ";
        for (const [i, value] of item_ids.entries()) {
            if (i === item_ids.length - 1) {
                dbArray += "'" + value + "')";
            }
            else {
                dbArray += "'" + value + "',";
            }
        }
        const parents = await (0, dataSource_1.getDataSource)().query('select ir.child as child, oi.item_id as parent, oi.label from item_relations ir inner join object_items oi on ir.parent = oi.item_id WHERE ir.child IN ' +
            dbArray);
        let parentDict = {};
        for (const parent of parents) {
            if (parentDict[parent.child]) {
                parentDict[parent.child].push({
                    item_id: parent.parent,
                    label: parent.label,
                });
            }
            else {
                parentDict[parent.child] = [];
                parentDict[parent.child].push({
                    item_id: parent.parent,
                    label: parent.label,
                });
            }
        }
        return parentDict;
    }
    async checkCustomGroupOwner(customGroupId, userId, groupIds) {
        let where = [
            {
                custom_group_id: customGroupId,
                user_id: userId,
            },
        ];
        if (groupIds) {
            where.push({
                custom_group_id: customGroupId,
                group_id: (0, typeorm_2.In)(groupIds),
            });
        }
        return ((await this.customGroupsRepository.findOne({
            where: where,
        })) !== null);
    }
    async checkCustomGroupFolderOwner(customGroupFolderId, userId, groupIds) {
        let where = [
            {
                custom_group_folder_id: customGroupFolderId,
                user_id: userId,
            },
        ];
        if (groupIds) {
            where.push({
                custom_group_folder_id: customGroupFolderId,
                group_id: (0, typeorm_2.In)(groupIds),
            });
        }
        return ((await this.customGroupFoldersRepository.findOne({
            where: where,
        })) !== null);
    }
    async getDashboardCustomGroups(userId, groupIds) {
        const customGroups = await this.customGroupsRepository.find({
            where: [
                {
                    user_id: userId,
                    favorites: {
                        user_id: userId,
                    },
                },
                {
                    group_id: (0, typeorm_2.In)(groupIds),
                    favorites: {
                        user_id: userId,
                    },
                },
            ],
            relations: {
                favorites: true,
            },
        });
        return customGroups.filter((cg) => CustomGroupWrapper_1.CustomGroupWrapper.checkCustomGroupHeatmap(cg.class, cg.subsystem));
    }
    async checkFavoriteCustomGroup(customGroupId, userId) {
        return ((await this.customGroupsRepository.findOne({
            where: {
                custom_group_id: customGroupId,
                favorites: {
                    user_id: userId,
                },
            },
            relations: {
                favorites: true,
            },
        })) !== null);
    }
    async setFavoriteCustomGroup(customGroupId, userId) {
        return (await this.customGroupFavoriteRepository.save({
            custom_group_id: customGroupId,
            user_id: userId
        })).user_id !== undefined;
    }
    async deleteFavoriteCustomGroup(customGroupId, userId) {
        return (await this.customGroupFavoriteRepository.delete({
            custom_group_id: customGroupId,
            user_id: userId
        })).affected === 1;
    }
    async getCustomGroupDashboardFolder(userId) {
        const user = await User_1.User.getUserById(userId, true);
        let items = [];
        const customGroups = await this.getDashboardCustomGroups(user.user_id, user.groups.map(g => g.group_id));
        customGroups.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const customGroup of customGroups) {
            items.push({
                isLeaf: true,
                itemId: customGroup.custom_group_id,
                pageType: 'heatmap',
                url: '/api/custom_group/v1/menu/item/' + customGroup.custom_group_id + "/dashboard",
                title: customGroup.label,
            });
        }
        return items;
    }
};
exports.CustomGroupService = CustomGroupService;
exports.CustomGroupService = CustomGroupService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(customGroup_entity_1.CustomGroup)),
    __param(1, (0, typeorm_1.InjectRepository)(customGroupFolder_entity_1.CustomGroupFolder)),
    __param(2, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(3, (0, typeorm_1.InjectRepository)(customGroupFavorite_entity_1.CustomGroupFavorite)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.TreeRepository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], CustomGroupService);
//# sourceMappingURL=custom_group.service.js.map