"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExporterController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const exporter_constants_1 = require("../../common/constants/exporter.constants");
const exporter_dto_1 = require("../../common/exporter.dto");
const Metric_1 = require("../metrics/Metric");
const custom_group_service_1 = require("../custom_group/custom_group.service");
const User_1 = require("../users/User");
const Exporter_1 = require("./Exporter");
const ExporterFormatter_1 = require("./ExporterFormatter");
const exporter_service_1 = require("./exporter.service");
const ItemGroupWrapper_1 = require("../custom_group/ItemGroupWrapper");
const CustomGroupWrapper_1 = require("../custom_group/CustomGroupWrapper");
const License_1 = require("../../config/License");
const xormon_exception_1 = require("../../exception/xormon.exception");
const Item_1 = require("../configuration/Item");
let ExporterController = class ExporterController {
    constructor(exporterService, customGroupService) {
        this.exporterService = exporterService;
        this.customGroupService = customGroupService;
    }
    async exports(request) {
        const user = await User_1.User.getUserByRequest(request);
        const exports = await this.exporterService.getExports(user.user_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: exports.map((hist) => {
                return {
                    export_id: hist.export_id,
                    label: hist.label,
                    format: hist.format,
                    type: hist.type,
                    last_run: hist.last_run,
                    recurrence: hist.recurrence,
                    created: hist.created.toISOString(),
                };
            }),
        };
    }
    async saveExport(request, res, exporter) {
        const user = await User_1.User.getUserByRequest(request);
        const isFree = !(await License_1.License.hasAnyValid());
        const exports = await this.exporterService.getExports(user.user_id);
        if (isFree) {
            if (exports.length >= 1) {
                throw new xormon_exception_1.XormonException("You can only have 1 export in free version", common_1.HttpStatus.PAYMENT_REQUIRED);
            }
        }
        const technology = exporter.hw_type
            ? exporter.hw_type
            : exporter.class
                ? exporter.class
                : undefined;
        if (technology && !(await License_1.License.checkTechnology(technology)) && exports.length >= 1) {
            throw new xormon_exception_1.XormonException("You do not have an active license for the given technology", common_1.HttpStatus.PAYMENT_REQUIRED);
        }
        if (exporter.data === null || exporter.data === undefined) {
            exporter.data = {};
        }
        const saved = await this.exporterService.saveExport(exporter, undefined, user.user_id);
        if (exporter.recurrence) {
            res.send({
                statusCode: common_1.HttpStatus.CREATED,
                message: 'exporter successfully saved',
            });
        }
        else {
            const savedWithRelations = await this.exporterService.getExport(saved.export_id);
            let items = [];
            if (savedWithRelations.item_group_id) {
                items = await ItemGroupWrapper_1.ItemGroupWrapper.getItems(savedWithRelations.item_group_id, 'export');
            }
            if (exporter.type === exporter_constants_1.EXPORT_TYPE.timeseries) {
                return await this.timeseries(request, res, {
                    ...savedWithRelations.data,
                    uuids: items ? items.map(item => item.item_id) : undefined
                }, exporter.format);
            }
            else if (exporter.type === exporter_constants_1.EXPORT_TYPE.mapping) {
                return await this.mapping(request, res, exporter, exporter.format);
            }
            else if (exporter.type === exporter_constants_1.EXPORT_TYPE.configuration) {
                return await this.configuration(request, res, {
                    ...exporter,
                    properties: exporter.data.properties,
                    uuids: items ? items.map(item => item.item_id) : undefined
                }, exporter.format);
            }
            else {
                throw new common_1.HttpException({
                    statusCode: common_1.HttpStatus.BAD_REQUEST,
                    message: 'bad request',
                    error: 'bad request',
                }, common_1.HttpStatus.BAD_REQUEST);
            }
        }
    }
    async updateExport(request, export_id, exporter) {
        const user = await User_1.User.getUserByRequest(request);
        if (!this.exporterService.checkExportOwner(export_id, user.user_id)) {
            throw new common_1.ForbiddenException();
        }
        await this.exporterService.saveExport(exporter, export_id, user.user_id);
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: 'exporter successfully updated',
        };
    }
    async deleteExport(request, export_id) {
        const user = await User_1.User.getUserByRequest(request);
        if (!this.exporterService.checkExportOwner(export_id, user.user_id)) {
            throw new common_1.ForbiddenException();
        }
        const deleted = this.exporterService.deleteExport(export_id);
        if (deleted) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'exporter successfully deleted',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Error occurred while deleting the export',
                error: 'Export not deleted',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async export(request, res, export_id) {
        const user = await User_1.User.getUserByRequest(request);
        if (!this.exporterService.checkExportOwner(export_id, user.user_id)) {
            throw new common_1.ForbiddenException();
        }
        const detail = await this.exporterService.getExport(export_id);
        if (!detail) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Export not found',
                error: 'Export not found',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        let items = undefined;
        if (detail.item_group_id) {
            items = await ItemGroupWrapper_1.ItemGroupWrapper.getItems(detail.item_group_id, 'export');
        }
        else if (detail.custom_group_id) {
            items = await CustomGroupWrapper_1.CustomGroupWrapper.getItems(detail.custom_group_id);
        }
        switch (detail.type) {
            case exporter_constants_1.EXPORT_TYPE.timeseries: {
                const data = detail.data;
                if (detail.recurrence) {
                    const interval = Exporter_1.Exporter.getTimeseriesInterval(data, detail.recurrence.freq);
                    if (interval.start && interval.end) {
                        data.start = interval.start;
                        data.end = interval.end;
                    }
                }
                await this.exporterService.saveUsedTime(export_id);
                return await this.timeseries(request, res, {
                    ...detail.data,
                    uuids: items ? items.map(item => item.item_id) : undefined
                }, detail.format);
            }
            case exporter_constants_1.EXPORT_TYPE.mapping: {
                await this.exporterService.saveUsedTime(export_id);
                return await this.mapping(request, res, {
                    ...detail.data,
                    uuids: items ? items.map(item => item.item_id) : undefined,
                    subsystem: detail.subsystem
                }, detail.format);
            }
            case exporter_constants_1.EXPORT_TYPE.configuration: {
                await this.exporterService.saveUsedTime(export_id);
                return await this.configuration(request, res, {
                    ...detail.data,
                    uuids: items ? items.map(item => item.item_id) : undefined
                }, detail.format);
            }
        }
        throw new common_1.HttpException({
            statusCode: common_1.HttpStatus.BAD_REQUEST,
            message: 'bad request',
            error: 'bad request',
        }, common_1.HttpStatus.BAD_REQUEST);
    }
    async exportDetail(request, export_id) {
        const user = await User_1.User.getUserByRequest(request);
        if (!this.exporterService.checkExportOwner(export_id, user.user_id)) {
            throw new common_1.ForbiddenException();
        }
        const detail = await this.exporterService.getExport(export_id, true);
        if (!detail) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'export not found',
                error: 'export not found',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: {
                export_id: detail.export_id,
                label: detail.label,
                format: detail.format,
                type: detail.type,
                class: detail.class,
                hw_type: detail.hw_type,
                subsystem: detail.subsystem,
                data: detail.data,
                recurrence: detail.recurrence,
                last_run: detail.last_run ? detail.last_run : null,
                items: detail._item_group && detail._item_group.items && detail._item_group.items.length >= 1
                    ? detail._item_group.items.map(item => {
                        return {
                            item_id: item.item_id,
                            label: item._item.label
                        };
                    })
                    : null,
                regex: detail._item_group && detail._item_group.regex && detail._item_group.regex.length >= 1
                    ? detail._item_group.regex
                    : null,
                tags: detail._item_group && detail._item_group.tags && detail._item_group.tags.length >= 1
                    ? detail._item_group.tags.map(tag => {
                        return {
                            tag_id: tag.tag_value_id,
                            tag: tag._tag.value
                        };
                    })
                    : null,
                parents: detail._item_group
                    ? {
                        intersection: detail._item_group.intersection,
                        items: detail._item_group.parents && detail._item_group.parents.length >= 1
                            ? detail._item_group.parents.map(item => {
                                return {
                                    item_id: item.item_id,
                                    label: item._item.label
                                };
                            })
                            : null,
                        regex: detail._item_group.regex_parents && detail._item_group.regex_parents.length >= 1
                            ? detail._item_group.regex_parents.map(regex => {
                                return {
                                    subsystem: regex.subsystem,
                                    regex: regex.regex
                                };
                            })
                            : null,
                        tags: detail._item_group.parent_tags && detail._item_group.parent_tags.length >= 1
                            ? detail._item_group.parent_tags.map(tag => {
                                return {
                                    tag_id: tag.tag_value_id,
                                    tag: tag._tag.value,
                                    subsystem: tag.subsystem
                                };
                            })
                            : null,
                    } : null,
                custom_group_id: detail.custom_group_id ? detail.custom_group_id : null
            },
        };
    }
    async timeseries(request, res, exportRequest, format) {
        if (!Exporter_1.Exporter.isValid(exportRequest)) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'filter is missing',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        const exportData = await Exporter_1.Exporter.timeseries(exportRequest, request);
        const formatter = new ExporterFormatter_1.ExporterFormatter(exportData);
        formatter.setStartEnd({ start: exportRequest.start, end: exportRequest.end });
        return await formatter.exportTimeseriesToResponse(format, res);
    }
    async mapping(request, res, exportRequest, format) {
        const data = await Exporter_1.Exporter.mapping(exportRequest, request, format, res);
        if (format !== 'xlsx') {
            res.send(data);
        }
    }
    async metrics(res, request, format) {
        if (!request.hw_type || !request.subsystem) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'some parameters is missing',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        let metrics = [];
        const rows = await Metric_1.Metric.getMetrics(request.hw_type, request.subsystem);
        for (const row of rows) {
            metrics.push({
                label: row.label,
                metric: row.metric,
                unit: row.unit,
                shortcut: row.shortcut
            });
        }
        if (!format || format === exporter_constants_1.EXPORT_FORMAT.json) {
            res.send(metrics);
        }
        else if (format === exporter_constants_1.EXPORT_FORMAT.csv) {
            let response = 'label;metric;unit;\n';
            for (const metric of metrics) {
                response += metric.label + ';' + metric.metric + ";" + metric.unit + ";\n";
            }
            res.send(response);
        }
        else if (format === exporter_constants_1.EXPORT_FORMAT.xlsx) {
            const ExcelJS = require('exceljs');
            const workbook = new ExcelJS.Workbook();
            const sheet = workbook.addWorksheet("Export");
            let header = ['label', 'metric', 'unit'];
            sheet.addRow(header);
            for (const metric of metrics) {
                let row = [metric.label, metric.metric, metric.unit];
                sheet.addRow(row);
            }
            const fileName = 'export.xlsx';
            res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
            res.setHeader('Content-Disposition', 'attachment; filename=' + fileName);
            await workbook.xlsx.write(res);
            res.end();
        }
        else {
            res.send(null);
        }
    }
    async configuration(request, res, exportRequest, format) {
        const data = await Exporter_1.Exporter.configuration(exportRequest, request, format, res);
        if (format !== 'xlsx') {
            res.send(data);
        }
    }
    async capacity(request, res, exportRequest, format) {
        const data = await Exporter_1.Exporter.capacity(exportRequest, request, format, res);
        if (format !== 'xlsx') {
            res.send(data);
        }
    }
    async values(request, exportRequest) {
        return Exporter_1.Exporter.valuesToCsv(exportRequest, request);
    }
    async alerting(format) { }
    async top(res, request, format) {
        if (!request.uuids || !request.metric || !request.interval) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'some parameters is missing',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        const tops = await this.exporterService.getTop(request.uuids, request.metric, request.interval);
        const metricDefinitions = await Item_1.Item.getMetrics(request.uuids);
        const metricDefinition = metricDefinitions.find(metricObj => metricObj.metric === request.metric);
        if (!format || format === exporter_constants_1.EXPORT_FORMAT.json) {
            let jsonTops = {};
            for (const top of tops) {
                jsonTops[top.item_id] = {
                    label: top._item.label,
                    avg: top.avg,
                    max: top.max,
                };
            }
            res.send(jsonTops);
        }
        else if (format === exporter_constants_1.EXPORT_FORMAT.csv) {
            let csvTops = 'item_id;label;avg;max;';
            if (metricDefinition) {
                csvTops = `item_id;label;avg [${metricDefinition.shortcut}];max [${metricDefinition.shortcut}];`;
            }
            for (const top of tops) {
                csvTops += `\n${top.item_id};${top._item.label};${top.avg};${top.max};`;
            }
            res.send(csvTops);
        }
        else if (format === exporter_constants_1.EXPORT_FORMAT.xlsx) {
            const ExcelJS = require('exceljs');
            const workbook = new ExcelJS.Workbook();
            const sheet = workbook.addWorksheet('Export');
            const row = sheet.getRow(1);
            row.getCell('A').value = 'Item ID';
            row.getCell('B').value = 'Label';
            if (metricDefinition) {
                row.getCell('C').value = `Avg [${metricDefinition.shortcut}]`;
                row.getCell('D').value = `Max [${metricDefinition.shortcut}]`;
            }
            else {
                row.getCell('C').value = 'Avg';
                row.getCell('D').value = 'Max';
            }
            let i = 2;
            for (const top of tops) {
                const row = sheet.getRow(i);
                row.getCell('A').value = top.item_id;
                row.getCell('B').value = top._item.label;
                row.getCell('C').value = top.avg;
                row.getCell('D').value = top.max;
                i++;
            }
            const fileName = 'export.xlsx';
            res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
            res.setHeader('Content-Disposition', 'attachment; filename=' + fileName);
            await workbook.xlsx.write(res);
        }
        else {
            res.send(null);
        }
    }
};
exports.ExporterController = ExporterController;
__decorate([
    openapi.ApiOperation({ description: "Returns exports history" }),
    (0, common_1.Get)('v1/exports'),
    openapi.ApiResponse({ status: 200, type: require("../../common/exporter.dto").ExporterHistoryResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "exports", null);
__decorate([
    openapi.ApiOperation({ description: "Save new export" }),
    (0, common_1.Post)('v1/exports'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, exporter_dto_1.SaveExport]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "saveExport", null);
__decorate([
    openapi.ApiOperation({ description: "Update export" }),
    (0, common_1.Put)('v1/exports/:export_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('export_id')),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, exporter_dto_1.SaveExport]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "updateExport", null);
__decorate([
    openapi.ApiOperation({ description: "Delete export by id" }),
    (0, common_1.Delete)('v1/exports/:export_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('export_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "deleteExport", null);
__decorate([
    openapi.ApiOperation({ description: "Run the export" }),
    (0, common_1.Get)('v1/exports/:export_id'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Param)('export_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "export", null);
__decorate([
    openapi.ApiOperation({ description: "Returns export detail" }),
    (0, common_1.Get)('v1/exports/:export_id/detail'),
    openapi.ApiResponse({ status: 200, type: require("../../common/exporter.dto").ExporterDetailResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('export_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "exportDetail", null);
__decorate([
    openapi.ApiOperation({ description: "Export timeseries data" }),
    (0, common_1.Post)('v1/timeseries'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Body)()),
    __param(3, (0, common_1.Query)('format')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, exporter_dto_1.ExportTimeseriesRequestDTO, String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "timeseries", null);
__decorate([
    openapi.ApiOperation({ description: "Export timeseries data" }),
    (0, common_1.Post)('v1/mapping'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Body)()),
    __param(3, (0, common_1.Query)('format')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, exporter_dto_1.ExportMappingRequestDTO, String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "mapping", null);
__decorate([
    (0, common_1.Post)('v1/metrics'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Res)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Query)('format')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, exporter_dto_1.ExportMetricsRequestDTO, String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "metrics", null);
__decorate([
    (0, common_1.Post)('v1/configuration'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Body)()),
    __param(3, (0, common_1.Query)('format')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, exporter_dto_1.ExportConfigurationRequestDTO, String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "configuration", null);
__decorate([
    (0, common_1.Post)('v1/capacity'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Res)()),
    __param(2, (0, common_1.Body)()),
    __param(3, (0, common_1.Query)('format')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, exporter_dto_1.ExportCapacityRequestDTO, String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "capacity", null);
__decorate([
    (0, common_1.Post)('v1/values'),
    openapi.ApiResponse({ status: 201, type: String }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, exporter_dto_1.ExportValuesRequestDTO]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "values", null);
__decorate([
    (0, common_1.Post)('v1/alerting'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Param)('format')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "alerting", null);
__decorate([
    openapi.ApiOperation({ description: "Export TOP data" }),
    (0, common_1.Post)('v1/top'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Res)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Query)('format')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, exporter_dto_1.ExportTopRequestDTO, String]),
    __metadata("design:returntype", Promise)
], ExporterController.prototype, "top", null);
exports.ExporterController = ExporterController = __decorate([
    (0, swagger_1.ApiTags)('Exporter'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/exporter'),
    __metadata("design:paramtypes", [exporter_service_1.ExporterService,
        custom_group_service_1.CustomGroupService])
], ExporterController);
//# sourceMappingURL=exporter.controller.js.map