"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExporterService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const export_entity_1 = require("../../entity/export.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const top_entity_1 = require("../../entity/top.entity");
const typeorm_2 = require("typeorm");
const ItemGroupWrapper_1 = require("../custom_group/ItemGroupWrapper");
const xormon_exception_1 = require("../../exception/xormon.exception");
const enums_1 = require("@nestjs/common/enums");
const itemGroup_entity_1 = require("../../entity/itemGroup.entity");
let ExporterService = class ExporterService {
    constructor(objectItemsRepository, topRepository, exporterRepository, itemGroupRepository) {
        this.objectItemsRepository = objectItemsRepository;
        this.topRepository = topRepository;
        this.exporterRepository = exporterRepository;
        this.itemGroupRepository = itemGroupRepository;
    }
    async findItem() {
        return await this.objectItemsRepository.find();
    }
    async getItem(itemId) {
        return await this.objectItemsRepository.findOneBy({ item_id: itemId });
    }
    async findInItems(itemIds) {
        return await this.objectItemsRepository.findOneBy({ item_id: (0, typeorm_2.In)(itemIds) });
    }
    arrList(arr) {
        let dbArray = '(\'\', ';
        for (const [i, value] of arr.entries()) {
            if (i === arr.length - 1) {
                dbArray += "'" + value + "')";
            }
            else {
                dbArray += "'" + value + "',";
            }
        }
        return dbArray;
    }
    toLocaleDateTime(datetime) {
        const tzoffset = new Date(datetime).getTimezoneOffset() * 60000;
        return new Date(datetime - tzoffset).toISOString().slice(0, -1);
    }
    async getTop(uuids, metric, interval) {
        return await this.topRepository.find({ where: { item_id: (0, typeorm_2.In)(uuids), metric: metric, interval: interval }, relations: { _item: true } });
    }
    async getExports(user_id) {
        return await this.exporterRepository.find({ order: { last_run: 'DESC' }, where: { user_id: user_id ? user_id : undefined }, relations: { _item_group: true } });
    }
    async getExport(export_id, detailed = false) {
        return await this.exporterRepository.findOne({
            where: {
                export_id: export_id
            }, relations: {
                _item_group: detailed
                    ? {
                        items: {
                            _item: true
                        },
                        tags: {
                            _tag: true
                        },
                        parents: {
                            _item: true
                        },
                        regex_parents: true,
                        parent_tags: {
                            _tag: true
                        }
                    } : true
            }
        });
    }
    async deleteExport(export_id) {
        const exporter = await this.exporterRepository.findOneBy({ export_id: export_id });
        const deleted = await this.exporterRepository.delete(export_id);
        if (exporter && exporter.item_group_id) {
            await this.itemGroupRepository.delete({ item_group_id: exporter.item_group_id });
        }
        return deleted.affected === 1 ? true : false;
    }
    async saveExport(exporter, export_id, user_id) {
        let exp = undefined;
        if (exporter.data === undefined) {
            exporter.data = {};
        }
        try {
            let oldItemGroupId = undefined;
            if (export_id) {
                const beforeUpdate = await this.exporterRepository.findOneBy({ export_id: export_id });
                if (beforeUpdate && beforeUpdate.item_group_id) {
                    oldItemGroupId = beforeUpdate.item_group_id;
                }
            }
            const itemGroupId = !exporter.custom_group_id ? await ItemGroupWrapper_1.ItemGroupWrapper.saveItemGroup(exporter, oldItemGroupId) : null;
            exp = await this.exporterRepository.save({ ...exporter, export_id: export_id ? export_id : undefined, item_group_id: itemGroupId, user_id: user_id ? user_id : null });
        }
        catch (e) {
            console.error(e);
            throw new xormon_exception_1.XormonException("Error while saving", enums_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        return exp;
    }
    async saveUsedTime(export_id) {
        this.exporterRepository.save({
            export_id: export_id,
            last_run: new Date().toISOString(),
        });
    }
    async checkExportOwner(export_id, user_id) {
        return await this.exporterRepository.findOne({
            where: {
                export_id: export_id,
                user_id: user_id
            }
        }) !== null;
    }
};
exports.ExporterService = ExporterService;
exports.ExporterService = ExporterService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(1, (0, typeorm_1.InjectRepository)(top_entity_1.Top)),
    __param(2, (0, typeorm_1.InjectRepository)(export_entity_1.Export)),
    __param(3, (0, typeorm_1.InjectRepository)(itemGroup_entity_1.ItemGroup)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], ExporterService);
//# sourceMappingURL=exporter.service.js.map