"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceImporter = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const child_process_1 = require("child_process");
const hostcfg_entity_1 = require("../../entity/hostcfg.entity");
const dataSource_1 = require("../../config/dataSource");
const typeorm_1 = require("typeorm");
const Microservice_1 = require("../microservices/Microservice");
const rl = require("readline");
const promises_1 = require("fs/promises");
const common_1 = require("@nestjs/common");
const xormon_exception_1 = require("../../exception/xormon.exception");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const crypto_1 = require("crypto");
const deviceProps_1 = require("../../common/hostcfgs/deviceProps");
const ecs_data_1 = require("../../common/hostcfgs/ecs.data");
const dothill_data_1 = require("../../common/hostcfgs/dothill.data");
const hcp_data_1 = require("../../common/hostcfgs/hcp.data");
const datacore_data_1 = require("../../common/hostcfgs/datacore.data");
const compellent_data_1 = require("../../common/hostcfgs/compellent.data");
const vmax_data_1 = require("../../common/hostcfgs/vmax.data");
const vplex_data_1 = require("../../common/hostcfgs/vplex.data");
const xtremio_data_1 = require("../../common/hostcfgs/xtremio.data");
const falconstor_data_1 = require("../../common/hostcfgs/falconstor.data");
const rubrik_data_1 = require("../../common/hostcfgs/rubrik.data");
const gpfs_data_1 = require("../../common/hostcfgs/gpfs.data");
const azure_data_1 = require("../../common/hostcfgs/azure.data");
const aws_data_1 = require("../../common/hostcfgs/aws.data");
const kubernetes_data_1 = require("../../common/hostcfgs/kubernetes.data");
const hostcfgData_types_1 = require("../../common/hostcfgs/hostcfgData.types");
const oracledb_data_1 = require("../../common/hostcfgs/oracledb.data");
const ceph_data_1 = require("../../common/hostcfgs/ceph.data");
const nutanix_data_1 = require("../../common/hostcfgs/nutanix.data");
const eternus_data_1 = require("../../common/hostcfgs/eternus.data");
const gcloud_data_1 = require("../../common/hostcfgs/gcloud.data");
const oraclevm_data_1 = require("../../common/hostcfgs/oraclevm.data");
const power_data_1 = require("../../common/hostcfgs/power.data");
const proxmox_data_1 = require("../../common/hostcfgs/proxmox.data");
const vmware_data_1 = require("../../common/hostcfgs/vmware.data");
const vnx_data_1 = require("../../common/hostcfgs/vnx.data");
const _3par_data_1 = require("../../common/hostcfgs/3par.data");
const ds8k_data_1 = require("../../common/hostcfgs/ds8k.data");
const fusioncompute_data_1 = require("../../common/hostcfgs/fusioncompute.data");
const macrosan_data_1 = require("../../common/hostcfgs/macrosan.data");
const nimble_data_1 = require("../../common/hostcfgs/nimble.data");
const openshift_data_1 = require("../../common/hostcfgs/openshift.data");
const raidix_data_1 = require("../../common/hostcfgs/raidix.data");
const xenserver_data_1 = require("../../common/hostcfgs/xenserver.data");
const xiv_data_1 = require("../../common/hostcfgs/xiv.data");
const qnap_data_1 = require("../../common/hostcfgs/qnap.data");
const swiz_data_1 = require("../../common/hostcfgs/swiz.data");
const vspg_data_1 = require("../../common/hostcfgs/vspg.data");
const hwTypeKey_constants_1 = require("../../common/constants/hwTypeKey.constants");
const request_builder_1 = require("../../request/request.builder");
const vendorMap = {
    "Dell EMC²": "DELL_EMC",
    "Synology": "SYNOLOGY",
    "QNAP Systems": "QNAP_SYSTEMS",
    "Brocade": "BROADCOM"
};
const deviceMap = {
    "CLARiiON CX4": "CLARION",
    "Elastic Cloud Storage": "ELASTIC",
    "PowerVault MD3000": "POWERVAULT_MD3K",
    "SC Series (Compellent)": "SCS_COMPELLENT",
    "VNX for file (Celerra)": "VNX_CELERRA",
    "Cloud Object Storage System": "COSS",
    "DS3000/4000/5000 & DSC3700": "DS_3_4_5_DSC_37",
    "DS8000": "DS8K",
    "FlashSystem A9000": "FS_A9K",
    "FlashSystem V9000/V8400": "FS_V9K",
    "FlashSystem 900/840": "FS_900",
    "FlashSystem 9100": "FS_9100",
    "FlashSystem 9000, 7000, 5000": "FS_9K_7K_5K",
    "FlashSystem 5/7/9XXX": "FS_5K_7K_9K",
    "Spectrum Accelerate (XIV)": "SPECTRUM_ACCELERATE",
    "Spectrum Virtualize (SVC)": "SPECTRUM_VIRTUALIZE",
    "VSP 5000": "VSP_5K",
    "LeftHand": "LEFT_HAND",
    "MSA P2000": "MSA_P2K",
    "StoreOnce": "STORE_ONCE",
    "StoreVirtual": "STORE_VIRTUAL",
    "3PAR": "PAR3",
    "OceanStor": "OCEAN_STOR",
    "Storage S Series (S2200/S3200)": "STORAGE_S_SERIES",
    "Storage V Series (V5030/V3700)": "STORAGE_V_SERIES",
    "ThinkSystem DE series": "TS_DE_SERIES",
    "ThinkSystem DS series": "TS_DS_SERIES",
    "E/EF series": "EF_SERIES",
    "SolidFire": "SOLID_FIRE",
    "StorageGrid": "STORAGE_GRID",
    "FlasArray": "FLASH_ARRAY",
    "FlashBlade": "FLASH_BLADE",
    "Infinibox": "INFINI_BOX",
    "InfiniGuard": "INFINI_GUARD"
};
const sanVendors = {
    "BROADCOM": {
        "BROCADE": "BROADCOM__BROCADE"
    },
    "CISCO": {
        "CISCO": "CISCO__CISCO"
    }
};
const lanVendors = {
    "CISCO": {
        "CISCO_LAN__CISCO": "CISCO_LAN__CISCO"
    },
    "NON_CISCO": {
        "OTHERS_LAN__OTHER": "OTHERS_LAN__OTHER"
    }
};
const storageVendors = {
    "DELL_EMC": {
        "CLARION": "DELL_EMC__CLARION",
        "DATADOMAIN": "DELL_EMC__DATADOMAIN",
        "ELASTIC": "DELL_EMC__ELASTIC",
        "EQUALOGIC": "DELL_EMC__EQUALOGIC",
        "ISILON": "DELL_EMC__ISILON",
        "POWERFLEX": "DELL_EMC__POWERFLEX",
        "POWERMAX": "DELL_EMC__POWERMAX",
        "POWERSCALE": "DELL_EMC__POWERSCALE",
        "POWERSTORE": "DELL_EMC__POWERSTORE",
        "POWERVAULT_MD3K": "DELL_EMC__POWERVAULT_MD3K",
        "POWERVAULT_ME4": "DELL_EMC__POWERVAULT_ME4",
        "SCS_COMPELLENT": "DELL_EMC__SCS_COMPELLENT",
        "UNITY": "DELL_EMC__UNITY",
        "VMAX": "DELL_EMC__VMAX",
        "VNX_BLOCK": "DELL_EMC__VNX_BLOCK",
        "VNXE": "DELL_EMC__VNXE",
        "VNX_CELERRA": "DELL_EMC__VNX_CELERRA",
        "VPLEX": "DELL_EMC__VPLEX",
        "XTREMIO": "DELL_EMC__XTREMIO"
    },
    "IBM": {
        "COSS": "IBM__COSS",
        "DS_3_4_5_DSC_37": "IBM__DS_3_4_5_DSC_37",
        "DS8K": "IBM__DS8K",
        "FS_A9K": "IBM__FS_A9K",
        "FS_V9K": "IBM__FS_V9K",
        "FS_900": "IBM__FS_900",
        "FS_9100": "IBM__FS_9100",
        "SPECTRUM_ACCELERATE": "IBM__SPECTRUM_ACCELERATE",
        "SPECTRUM_VIRTUALIZE": "IBM__SPECTRUM_VIRTUALIZE",
        "STORWIZE_FAMILY": "IBM__STORWIZE_FAMILY",
        "SPECTRUM_SCALE": "IBM__SPECTRUM_SCALE",
        "FS_5K_7K_9K": 'IBM__FS_5K_7K_9K',
        "FS_9K_7K_5K": 'IBM__FS_9K_7K_5K'
    },
    "HITACHI": {
        "ASM": "HITACHI__ASM",
        "HCP": "HITACHI__HCP",
        "HNAS": "HITACHI__HNAS",
        "HUS": "HITACHI__HUS",
        "HUS_VM": "HITACHI__HUS_VM",
        "VSP_E": "HITACHI__VSP_E",
        "VSP_F": "HITACHI__VSP_F",
        "VSP_G": "HITACHI__VSP_G",
        "VSP": "HITACHI__VSP",
        "VSP_5K": "HITACHI__VSP_5K"
    },
    "HPE": {
        "LEFT_HAND": "HPE__LEFT_HAND",
        "MSA_P2K": "HPE__MSA_P2K",
        "NIMBLE": "HPE__NIMBLE",
        "PRIMERA": "HPE__PRIMERA",
        "STORE_ONCE": "HPE__STORE_ONCE",
        "STORE_VIRTUAL": "HPE__STORE_VIRTUAL",
        "XP7": "HPE__XP7",
        "PAR3": "HPE__PAR3",
    },
    "NETAPP": {
        "EF_SERIES": "NETAPP__EF_SERIES",
        "FAS_CDOT": "NETAPP__FAS_CDOT",
        "FAS_CDOT_(REST_API)": "NETAPP__FAS_CDOT",
        "SOLID_FIRE": "NETAPP__SOLID_FIRE",
        "STORAGE_GRID": "NETAPP__STORAGE_GRID",
        "E/EF_SERIES": "NETAPP__EF_SERIES",
        "ASA_A_SERIES": "NETAPP__ASA_ASERIES",
    },
    "LENOVO": {
        "STORAGE_S_SERIES": "LENOVO__STORAGE_S_SERIES",
        "STORAGE_V_SERIES": "LENOVO__STORAGE_V_SERIES",
        "TS_DE_SERIES": "LENOVO__TS_DE_SERIES",
        "TS_DS_SERIES": "LENOVO__TS_DS_SERIES"
    },
    "PROMISE_TECHNO": {
        "VTRAK": "PROMISE_TECHNO__VTRAK"
    },
    "PURE_STORAGE": {
        "FLASH_ARRAY": "PURE_STORAGE__FLASH_ARRAY",
        "FLASH_BLADE": "PURE_STORAGE__FLASH_BLADE",
        "FLASHARRAY": "PURE_STORAGE__FLASH_ARRAY",
        "FLASHBLADE": "PURE_STORAGE__FLASH_BLADE"
    },
    "HUAWEI": {
        "DORADO": "HUAWEI__DORADO",
        "OCEAN_STOR": "HUAWEI__OCEAN_STOR",
        "OCEAN_DISK": "HUAWEI__OCEAN_DISK",
        "OCEAN_PROTECT": "HUAWEI__OCEAN_PROTECT"
    },
    "INFINIDAT": {
        "INFINI_BOX": "INFINIDAT__INFINI_BOX",
        "INFINI_GUARD": "INFINIDAT__INFINI_GUARD"
    },
    "CEPH": {
        "CEPH": "CEPH__CEPH"
    },
    "DATA_CORE": {
        "SAN_SYMPHONY": "DATA_CORE__SAN_SYMPHONY"
    },
    "DOTHILL": {
        "ASSURED_SAN": "DOTHILL__ASSURED_SAN"
    },
    "FALCON_STOR": {
        "FREE_STOR": "FALCON_STOR__FREE_STOR"
    },
    "FUJITSU": {
        "ETERNUM": "FUJITSU__ETERNUM"
    },
    "INFORTREND": {
        "EONSTOR": "INFORTREND__EONSTOR"
    },
    "INSPUR": {
        "AS_55_53_26_22": "INSPUR__AS_55_53_26_22"
    },
    "IXSYSTEMS": {
        "FREE_NAS": "IXSYSTEMS__FREE_NAS"
    },
    "MACRO_SAN": {
        "MACRO_SAN": "MACRO_SAN__MACRO_SAN"
    },
    "MACROSAN": {
        "MACROSAN": "MACRO_SAN__MACRO_SAN"
    },
    "ORACLE": {
        "ZFS": "ORACLE__ZFS"
    },
    "QNAP_SYSTEMS": {
        "QNAP": "QNAP_SYSTEMS__QNAP"
    },
    "QUANTUM": {
        "STOR_NEXT": "QUANTUM__STOR_NEXT"
    },
    "QUMULO": {
        "QUMULO": "QUMULO__QUMULO"
    },
    "RUBRIK": {
        "RUBRIK_CDM": "RUBRIK__RUBRIK_CDM"
    },
    "SYNOLOGY": {
        "SYNOLOGY": "SYNOLOGY__SYNOLOGY"
    },
    "VERITAS": {
        "NET_BACKUP": "VERITAS__NET_BACKUP"
    },
    "COHESITY": {
        "DATAPROTECT": "COHESITY__DATAPROTECT"
    }
};
class AbstractDataTransformer {
    transformBasicProps(data, type) {
        let newData = new type();
        for (const key of Object.keys(newData)) {
            if (data[key] !== undefined) {
                if (key === 'port')
                    newData[key] = parseInt(data[key]);
                if (passKeys.includes(key)) {
                    newData[key] = reveal(data[key]);
                }
                else {
                    newData[key] = data[key];
                }
            }
        }
        if (data.devicealias) {
            newData["alias"] = data.devicealias;
        }
        if (data.hostalias) {
            newData["alias"] = data.hostalias;
        }
        return this.postTransform(newData);
    }
    postTransform(data) {
        if (vendorMap[data.vendor]) {
            data.vendor = vendorMap[data.vendor];
        }
        if (data["port"]) {
            data["port"] = parseInt(data['port']);
        }
        if (data["api_port"]) {
            data["api_port"] = parseInt(data['api_port']);
        }
        if (data.device && data.vendor) {
            let oldDeviceName = data.device;
            let device = data.device.toUpperCase().replaceAll(/[ -]/gi, "_");
            let vendor = data.vendor.toUpperCase().replaceAll(/[ -]/gi, "_");
            console.log(vendor);
            if (sanVendors[vendor] && data.device !== "Cisco LAN switch") {
                data.device = vendor + "__" + Object.keys(sanVendors[vendor])[0];
            }
            else if (storageVendors[vendor]) {
                let devices = Object.keys(storageVendors[vendor]);
                if (devices.length === 1) {
                    data.device = storageVendors[vendor][devices[0]];
                }
                else if (storageVendors[vendor][device]) {
                    data.device = storageVendors[vendor][device];
                }
                else if (deviceMap[oldDeviceName]) {
                    data.device = storageVendors[vendor][deviceMap[oldDeviceName]];
                }
                else {
                    data.device = vendor + "__" + device;
                }
            }
            else if (lanVendors[vendor]) {
                data.device = Object.keys(lanVendors[vendor])[0];
            }
            else {
                data.device = "REMOVE";
            }
        }
        return data;
    }
    transform(data, type = null) {
        let newData = this.transformData(data, type);
        for (const key of Object.keys(newData)) {
            if (newData[key] === undefined || newData[key] === '' || newData[key] === null) {
                delete newData[key];
            }
        }
        if (newData["node1Host"] && newData["host"] instanceof Array) {
            delete newData["host"];
        }
        if ((newData["node1Host"] || newData["svpHost"]) && newData["apiHost"]) {
            delete newData["apiHost"];
        }
        return newData;
    }
}
class BasicDeviceTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        return super.transformBasicProps(data, type);
    }
}
class SnmpDeviceTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.snmpAuthPassword = reveal(data["snmp-auth-pass"]);
        newData.snmpAuthProtocol = data["snmp-auth-protocol"];
        newData.snmpCommunity = data["snmp-community"] === "" ? "public" : data["snmp-community"];
        newData.snmpHost = data["snmp-host"];
        newData.snmpPrivPassword = reveal(data["snmp-priv-pass"]);
        newData.snmpPrivProtocol = data["snmp-priv-protocol"];
        newData.snmpSecUser = data["snmp-user"];
        newData.snmpVersion = data["snmp-version"];
        newData.snmpSecLevel = data["snmp-sec-level"];
        if (data["snmp-port"]) {
            newData.snmpPort = parseInt(data["snmp-port"]);
        }
        return newData;
    }
}
class SnmpNetworkDeviceTransformer extends SnmpDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        if (newData.snmpVersion === "3") {
            newData.snmpSecUser = data["snmp-community"] === "" ? "snmpuser1" : data["snmp-community"];
        }
        return newData;
    }
    async addFabric(data, type) {
        let newData = this.transformData(data, type);
        if (data["fabric"]) {
            const objectItemsRepository = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
            const check = await objectItemsRepository.findOneBy({
                hw_type: 'sanbrcd',
                subsystem: 'fabric',
                label: data["fabric"]
            });
            if (!check) {
                const newFabric = await objectItemsRepository.save({
                    hw_type: 'sanbrcd',
                    subsystem: 'fabric',
                    label: data["fabric"],
                    item_id: (0, crypto_1.createHash)('md5').update(data["fabric"]).digest("hex"),
                    object_id: "sanbrcd"
                });
                newData.fabric_id = newFabric.item_id;
                newData.fabric_label = data["fabric"];
            }
            else {
                newData.fabric_id = check.item_id;
                newData.fabric_label = data["fabric"];
            }
        }
        return newData;
    }
}
class SnmpStorageDeviceTransformer extends SnmpDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        if (data["snmp-context"]) {
            newData.snmpContextName = data["snmp-context"];
        }
        return newData;
    }
}
class UniCliDeviceTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.password = reveal(data["api-password"]);
        newData.username = data["api-user"];
        return newData;
    }
}
class ECSTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.vdc = data.vdc.map(v => { let ported = ecs_data_1.OldVdc.portVdc(v); return { ...ported, password: reveal(ported.password) }; });
        return newData;
    }
}
class DothillTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.password = reveal(data["api-password"]);
        newData.username = data["api-user"];
        newData.node1Host = data.host1;
        newData.node2Host = data.host2;
        if (data.radio && parseInt(data.radio) === 0) {
            newData.interface = 'SMI';
        }
        else {
            newData.interface = 'WBI';
        }
        return newData;
    }
}
class RestDeviceTransformer extends AbstractDataTransformer {
    transformData(data, type = null) {
        let newData = super.transformBasicProps(data, type);
        if (data["api-password2"] && data["api-user2"]) {
            newData.apiPassword = reveal(data["api-password2"]);
            newData.apiUsername = data["api-user2"];
        }
        else {
            newData.apiPassword = reveal(data["api-password"]);
            newData.apiUsername = data["api-user"];
        }
        newData.apiPort = parseInt(data["api-port"]);
        newData.apiHost = data.host ? data.host : "";
        return newData;
    }
}
class HcpDeviceTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.snmpAuthPassword = reveal(data["snmp-auth-pass"]);
        newData.snmpAuthProtocol = data["snmp-priv-protocol"];
        newData.snmpCommunity = data["snmp-community"];
        newData.snmpHost = data["snmp-host"];
        newData.snmpPrivPassword = reveal(data["snmp-priv-pass"]);
        newData.snmpPrivProtocol = data["snmp-priv-protocol"];
        newData.snmpSecUser = data["snmp-user"];
        newData.snmpVersion = data["snmp-version"];
        if (data["snmp-port"]) {
            newData.snmpPort = parseInt(data["snmp-port"]);
        }
        newData.tenants = data.tenant;
        newData.snmpContextName = data["snmp-context"];
        return newData;
    }
}
class SshDeviceTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.sshPort = data["ssh-port"] ? parseInt(data["ssh-port"] + '') : 22;
        newData.sshHost = data.host;
        newData.sshUsername = data["ssh-user"];
        newData.sshKeyFile = data["ssh-key-id"];
        newData.sshPassword = data["ssh-password"] ? reveal(data["ssh-password"]) : "";
        if (data["ssh-password-enable"]) {
        }
        else {
            newData.isKeyFile = true;
        }
        return newData;
    }
    transform(data, type) {
        let newData = super.transform(data, type);
        if (newData.sshKeyFile === undefined) {
            newData.sshKeyFile = "";
        }
        return newData;
    }
}
class SWIZDeviceTransformer extends SshDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        delete newData.sshHost;
        if (data["sample-rate"] && data["sample-rate"].length > 0) {
            newData.sampleRate = parseInt(data["sample-rate"]);
        }
        return newData;
    }
}
class QnapTransformer extends SshDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.qnapName = data["array-name"];
        return newData;
    }
}
class VSPGDataTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.node1Host = data["snmp-host"];
        newData.node2Host = data["host-node"];
        newData.svpHost = data.host;
        return newData;
    }
}
class DataCoreTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.nodeHost = data.host1;
        newData.proxyHost = data.host2;
        return newData;
    }
}
class CompellentTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        if (data.devicealias) {
            newData.storageName = data.devicealias;
        }
        else if (data.hostalias) {
            newData.storageName = data.hostalias;
        }
        return newData;
    }
}
class VmaxTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.symmetrix = data["symmetrix-id"];
        return newData;
    }
}
class VplexTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.sshHost = data.host1;
        newData.cluster = data["cluster-name"];
        if (data.host2) {
            newData.apiHost = data.host2;
            newData.isApi = true;
        }
        return newData;
    }
}
class XtremTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.cluster = data["cluster-name"];
        return newData;
    }
}
class FalconStorTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        if (data.domains) {
            newData.domains = data.domains.split(';').slice(0, -1);
        }
        else if (data.domain) {
            newData.domains = [data.domain];
        }
        return newData;
    }
}
class TokenRestTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        if (data["api-token"] && data["api-token"].length > 0) {
            newData.apiToken = data["api-token"];
        }
        newData.isToken = data.auth_token ? true : false;
        return newData;
    }
}
class RubrikDeviceTransformer extends TokenRestTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.apiHost1 = data.host1 ? data.host1 : "";
        newData.apiHost2 = data.host2 ? data.host2 : "";
        newData.apiHost3 = data.host3 ? data.host3 : "";
        delete newData.apiHost;
        delete data.host;
        return newData;
    }
}
class TwoNodesTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.node1Host = data.host1;
        newData.node2Host = data.host2;
        return newData;
    }
}
class GPFSTransformer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.nodes = data.mgr;
        return newData;
    }
}
class InfiniGuardTransofrmer extends RestDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.systemName = data["system-name"];
        return newData;
    }
}
class AzureTransformer extends AbstractDataTransformer {
    transformData(data) {
        let newData = super.transformBasicProps(data, azure_data_1.AzureProps);
        newData.apiPort = data.api_port;
        return newData;
    }
}
class AmazonTransformer extends AbstractDataTransformer {
    transformData(data) {
        let newData = super.transformBasicProps(data, aws_data_1.AwsProps);
        newData.port = data.api_port;
        return newData;
    }
}
class KubernetesTransformer extends AbstractDataTransformer {
    transformBasicProps(data, type) {
        return super.transformBasicProps(data, type);
    }
    transformData(data) {
        let newData = this.transformBasicProps(data, kubernetes_data_1.KubernetesProps);
        newData.namespacesAvailable = data.available_namespaces;
        newData.namespacesSelected = data.namespaces;
        newData.skipContainer = data.container;
        return newData;
    }
}
class AbstractDBTransformer extends AbstractDataTransformer {
    transformData(data, type = null) {
        let newData = super.transformBasicProps(data, type);
        return newData;
    }
}
class OracleDBTransformer extends AbstractDBTransformer {
    transformData(data) {
        let newData = super.transformData(data, oracledb_data_1.OracledbProps);
        newData.type = data.type.split("_").join("");
        if (data.hosts) {
            newData.hosts = data.hosts
                ? data.hosts.map((host, idx) => data.services ? new hostcfgData_types_1.HostService(host, data.services[idx][0], data.services[idx].slice(1)) : new hostcfgData_types_1.HostService(host))
                : [new hostcfgData_types_1.HostService()];
        }
        else {
            newData.hosts = [
                new hostcfgData_types_1.HostService(data.host, data.services ? data.services[0][0] : data.instance, data.services ? data.services[0].slice(1) : [])
            ];
        }
        if (data.dataguard) {
            newData.dataguard = data.dataguard.map(dg => hostcfgData_types_1.DataGuardOld.getAsHostService(dg));
        }
        return newData;
    }
}
class ApiPortTransformer extends AbstractDataTransformer {
    transformData(data, type = null) {
        let newData = super.transformBasicProps(data, type);
        if (data.api_port) {
            newData.port = parseInt('' + data.api_port);
        }
        else if (data["api-port"]) {
            newData.port = parseInt('' + data["api-port"]);
        }
        return newData;
    }
}
class CephDeviceTransformer extends ApiPortTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.cephs = data.mgr;
        return newData;
    }
}
class ProtoApiTransformer extends ApiPortTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        if (data.proto !== undefined)
            newData.protocol = data.proto;
        else
            newData.protocol = data['protocol'];
        return newData;
    }
}
class NutanixTransformer extends ProtoApiTransformer {
    transformData(data) {
        return super.transformData(data, nutanix_data_1.NutanixProps);
    }
}
class OracleVMTransformer extends ApiPortTransformer {
    transformData(data) {
        return super.transformData(data, oraclevm_data_1.OraclevmProps);
    }
}
class PowerTransformer extends ProtoApiTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        if (data.auth_ssh) {
            newData.disabled = true;
        }
        return newData;
    }
}
class VNXTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.spaHost = data.host1;
        newData.spbHost = data.host2;
        return newData;
    }
}
class EternusDeviceTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.sshUsername = data["ssh-user"];
        newData.node1Host = data.host1;
        newData.node2Host = data.host2;
        return newData;
    }
}
class FusionComputeDeviceTransformer extends ProtoApiTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.userType = parseInt(data.usertype);
        return newData;
    }
}
class DS8KDeviceTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.hmc1 = data.host1;
        newData.hmc2 = data.host2;
        if (data["dscli-path"]) {
            newData.dscliPath = data["dscli-path"];
        }
        return newData;
    }
}
class XIVDeviceTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.storageUsername = data["api-user"];
        newData.password = reveal(data["api-password"]);
        return newData;
    }
}
class MacroSANTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        if (!data.sp || data.sp.length === 0) {
            let sp = new hostcfgData_types_1.Sp();
            sp.host = data.host;
            sp.ftpPassword = reveal(data["ftp-password"]);
            sp.ftpPath = data["ftp-path"];
            sp.ftpUsername = data["ftp-user"];
            sp.sshPassword = reveal(data["ssh-password"]);
            sp.sshPort = data["ssh-port"] ? parseInt(data["ssh-port"] + '') : 22;
            ;
            sp.sshUsername = data["ssh-user"];
        }
        else {
            newData.sps = data.sp.map(sp => {
                let d = hostcfgData_types_1.OldSp.portSp(sp);
                return {
                    ...d,
                    ftpPassword: reveal(d.ftpPassword),
                    sshPassword: reveal(d.sshPassword)
                };
            });
        }
        return newData;
    }
}
class XenserverTransformer extends ProtoApiTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        delete newData.port;
        newData.apiPort = parseInt(data.api_port + "");
        newData.sshPort = parseInt(data.ssh_port + "");
        newData.sshKeyFile = data.ssh_key_id;
        return newData;
    }
}
class Par3DeviceTransform extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        if (data.radio) {
            switch (data.radio) {
                case _3par_data_1.Par3OldRadio.CLI:
                    newData.cliApiSshRadio = _3par_data_1.Par3Radio.CLI;
                    break;
                case _3par_data_1.Par3OldRadio.SSH:
                    newData.cliApiSshRadio = _3par_data_1.Par3Radio.SSH;
                    break;
                case _3par_data_1.Par3OldRadio.REST:
                    newData.cliApiSshRadio = _3par_data_1.Par3Radio.WSAPI;
                    break;
            }
        }
        else if (data.auth_ssh) {
            newData.cliApiSshRadio = _3par_data_1.Par3Radio.SSH;
        }
        else if (data.auth_api) {
            newData.cliApiSshRadio = _3par_data_1.Par3Radio.WSAPI;
        }
        else {
            newData.cliApiSshRadio = _3par_data_1.Par3Radio.CLI;
        }
        if (data.api_port) {
            newData.apiPort = parseInt(data.api_port + "");
        }
        if (data["ssh-port"]) {
            newData.sshPort = parseInt(data["ssh-port"]);
        }
        newData.sshKeyFile = data["ssh-key-id"];
        newData.sshUsername = data["ssh-user"];
        newData.apiUsername = data["api-user"];
        newData.apiPassword = data["api-password"] ? reveal(data["api-password"]) : "";
        newData.cliDir = data["clidir-gui"];
        newData.cliPassword = data["api-password"] ? reveal(data["api-password"]) : "";
        newData.cliUsername = data["api-user"];
        return newData;
    }
}
class NimbleDeviceTransformer extends SshDeviceTransformer {
    transformData(data, type) {
        let newData = super.transformData(data, type);
        newData.arrayName = data["array-name"];
        return newData;
    }
}
class GCloudDeviceTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.credentials = data.credentials instanceof Array ? data.credentials : [data.credentials];
        return newData;
    }
}
class OpenShiftTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.token = data.token;
        newData.protocol = data.protocol ? data.protocol : data.proto;
        newData.projectsAvailable = data.available_namespaces;
        newData.projectsSelected = data.namespaces;
        newData.skipContainer = data.container;
        return newData;
    }
}
class RaidixTransformer extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        newData.sshPassword = reveal(data["ssh-password"]);
        if (data["ssh-port1"])
            newData.sshPort1 = parseInt(data["ssh-port1"]);
        if (data["ssh-port2"])
            newData.sshPort2 = parseInt(data["ssh-port2"]);
        if (data["ssh-host1"])
            newData.host1 = data["ssh-host1"];
        if (data["ssh-host2"])
            newData.host2 = data["ssh-host2"];
        if (data["ssh-key-id"])
            newData.sshKeyId = data["ssh-key-id"];
        if (data["ssh-user1"])
            newData.sshUser1 = data["ssh-user1"];
        if (data["ssh-user2"])
            newData.sshUser2 = data["ssh-user2"];
        return newData;
    }
}
class TransformOtherDevices extends AbstractDataTransformer {
    transformData(data, type) {
        let newData = super.transformBasicProps(data, type);
        for (const key of Object.keys(data)) {
            if (!bannedWords.includes(key)) {
                let newKey = renamePropertyName(renamePropertyName(key, "-"), "_");
                newData[newKey] = data[key];
            }
        }
        return newData;
    }
}
const protoApiAuthOld = [new ProtoApiTransformer, deviceProps_1.AuthBaseProps, deviceProps_1.OldDeviceProps];
const transformAdbAdbOld = [new AbstractDBTransformer, deviceProps_1.AbstractDBProps, deviceProps_1.AbstractOldDBProps];
const transformRest = [new RestDeviceTransformer, deviceProps_1.RestDeviceProps, deviceProps_1.OldRestDeviceProps];
const transformTokenRest = [new TokenRestTransformer, deviceProps_1.TokenRestDeviceProps, deviceProps_1.TokenRestDevicePropsOld];
const transformSnmpOnly = [new SnmpDeviceTransformer, deviceProps_1.SnmpDeviceProps, deviceProps_1.OldSnmpDeviceProps];
const transformSnmpFabric = [new SnmpNetworkDeviceTransformer, deviceProps_1.SnmpNetworkDeviceProps, deviceProps_1.OldSnmpDeviceProps];
const transformSnmpStorage = [new SnmpStorageDeviceTransformer, deviceProps_1.SnmpStorageDeviceProps, deviceProps_1.OldSnmpDeviceProps];
const twoNodesRestTransformer = [new TwoNodesTransformer, deviceProps_1.TwoNodeRestDeviceProps, deviceProps_1.TwoNodeRestDevicePropsOld];
const transformProtoRest = [new RestDeviceTransformer, deviceProps_1.ProtoRestDeviceProps, deviceProps_1.OldRestDeviceProps];
const transformBasic = [new BasicDeviceTransformer, deviceProps_1.BaseDeviceProps, deviceProps_1.OldDeviceProps];
const transformSSH = [new SshDeviceTransformer, deviceProps_1.ExtensiveSshDeviceProps, deviceProps_1.OldSshDeviceProps];
const transformUniCli = [new UniCliDeviceTransformer, deviceProps_1.UniCliDeviceProps, deviceProps_1.OldUniCliDeviceProps];
const hwTypeClassMap = {
    [hwTypeKey_constants_1.HwTypeKey.VMWARE]: [new ProtoApiTransformer, vmware_data_1.VmwareProps, deviceProps_1.OldVmProps],
    [hwTypeKey_constants_1.HwTypeKey.PROXMOX]: [new ProtoApiTransformer, proxmox_data_1.ProxmoxProps, deviceProps_1.OldVmProps],
    [hwTypeKey_constants_1.HwTypeKey.NUTANIX]: [new NutanixTransformer, nutanix_data_1.NutanixProps, deviceProps_1.OldDeviceProps],
    [hwTypeKey_constants_1.HwTypeKey.POWER]: [new PowerTransformer, power_data_1.PowerProps, power_data_1.PowerPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.OVIRT]: protoApiAuthOld,
    [hwTypeKey_constants_1.HwTypeKey.XENSERVER]: [new XenserverTransformer, xenserver_data_1.XenserverProps, xenserver_data_1.XenserverPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.ORACLEVM]: protoApiAuthOld,
    [hwTypeKey_constants_1.HwTypeKey.CLOUDSTACK]: protoApiAuthOld,
    [hwTypeKey_constants_1.HwTypeKey.FUSIONCOMPUTE]: [new FusionComputeDeviceTransformer, fusioncompute_data_1.FusioncomputeProps, fusioncompute_data_1.FusioncomputePropsOld],
    [hwTypeKey_constants_1.HwTypeKey.POSTGRES]: transformAdbAdbOld,
    [hwTypeKey_constants_1.HwTypeKey.SQLSERVER]: transformAdbAdbOld,
    [hwTypeKey_constants_1.HwTypeKey.ORACLEDB]: [new OracleDBTransformer, oracledb_data_1.OracledbProps, oracledb_data_1.OracledbPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.DB2]: transformAdbAdbOld,
    [hwTypeKey_constants_1.HwTypeKey._3PAR]: [new Par3DeviceTransform, _3par_data_1.Par3Props, _3par_data_1.Par3PropsOld],
    [hwTypeKey_constants_1.HwTypeKey.COMPELLENT]: [new CompellentTransformer, compellent_data_1.CompellentDeviceProps, deviceProps_1.OldRestDeviceProps],
    [hwTypeKey_constants_1.HwTypeKey.DATADOMAIN]: transformSSH,
    [hwTypeKey_constants_1.HwTypeKey.DOTHILL]: [new DothillTransformer, dothill_data_1.DothillProps, dothill_data_1.DothillOldProps],
    [hwTypeKey_constants_1.HwTypeKey.ESERIES]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.IBMCOSS]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.ISILON]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.POWERFLEX]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.RUBRIK]: [new RubrikDeviceTransformer, rubrik_data_1.RubrikProps, rubrik_data_1.RubrikPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.STOREONCE]: transformProtoRest,
    [hwTypeKey_constants_1.HwTypeKey.SWIZ]: [new SWIZDeviceTransformer, swiz_data_1.SwizProps, swiz_data_1.SwizPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.QNAP]: [new QnapTransformer, qnap_data_1.QnapProps, qnap_data_1.QnapPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.ORACLEZFS]: twoNodesRestTransformer,
    [hwTypeKey_constants_1.HwTypeKey.COHESITY]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.VMAX]: [new VmaxTransformer, vmax_data_1.VmaxProps, vmax_data_1.VmaxPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.GPFS]: [new GPFSTransformer, gpfs_data_1.GpfsProps, gpfs_data_1.GpfsPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.NETBACKUP]: transformTokenRest,
    [hwTypeKey_constants_1.HwTypeKey.MACROSAN]: [new MacroSANTransformer, macrosan_data_1.MacrosanProps, macrosan_data_1.MacroSANOldProps],
    [hwTypeKey_constants_1.HwTypeKey.CEPH]: [new CephDeviceTransformer, ceph_data_1.CephProps, ceph_data_1.CephPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.DATACORE]: [new DataCoreTransformer, datacore_data_1.DatacoreProps, datacore_data_1.DatacorePropsOld],
    [hwTypeKey_constants_1.HwTypeKey.DS5K]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.DS8K]: [new DS8KDeviceTransformer, ds8k_data_1.Ds8kProps, ds8k_data_1.Ds8kPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.ECS]: [new ECSTransformer, ecs_data_1.EcsProps, ecs_data_1.EcsPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.ETERNUS]: [new EternusDeviceTransformer, eternus_data_1.EternusProps, eternus_data_1.EternusPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.FALCONSTOR]: [new FalconStorTransformer, falconstor_data_1.FalconstorProps, falconstor_data_1.FalconstorPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.FREENAS]: transformProtoRest,
    [hwTypeKey_constants_1.HwTypeKey.HCP]: [new HcpDeviceTransformer, hcp_data_1.HcpProps, hcp_data_1.HcpOldProps],
    [hwTypeKey_constants_1.HwTypeKey.HNAS]: transformSnmpStorage,
    [hwTypeKey_constants_1.HwTypeKey.HUS]: transformBasic,
    [hwTypeKey_constants_1.HwTypeKey.INFINIBOX]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.INFORTREND]: transformSnmpStorage,
    [hwTypeKey_constants_1.HwTypeKey.NETAPP]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.NIMBLE]: [new NimbleDeviceTransformer, nimble_data_1.NimbleProps, nimble_data_1.NimblePropsOld],
    [hwTypeKey_constants_1.HwTypeKey.OCEANSTOR]: twoNodesRestTransformer,
    [hwTypeKey_constants_1.HwTypeKey.PURE]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.PUREBLADE]: transformTokenRest,
    [hwTypeKey_constants_1.HwTypeKey.QUMULO]: transformTokenRest,
    [hwTypeKey_constants_1.HwTypeKey.RAIDIX]: [new RaidixTransformer, raidix_data_1.RaidixProps, raidix_data_1.RaidixPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.SOLIDFIRE]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.STOREVIRTUAL]: transformSSH,
    [hwTypeKey_constants_1.HwTypeKey.SYNOLOGY]: transformSnmpStorage,
    [hwTypeKey_constants_1.HwTypeKey.VNX]: [new VNXTransformer, vnx_data_1.VnxProps, vnx_data_1.VnxPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.VNXE]: transformUniCli,
    [hwTypeKey_constants_1.HwTypeKey.VNXF]: transformSSH,
    [hwTypeKey_constants_1.HwTypeKey.VPLEX]: [new VplexTransformer, vplex_data_1.VplexProps, vplex_data_1.VplexPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.VSPG]: [new VSPGDataTransformer, vspg_data_1.VspgProps, vspg_data_1.VspgPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.UNITY]: transformUniCli,
    [hwTypeKey_constants_1.HwTypeKey.XIV]: [new XIVDeviceTransformer, xiv_data_1.XivProps, xiv_data_1.XivPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.XTREMIO]: [new XtremTransformer, xtremio_data_1.XtremioProps, xtremio_data_1.XtremioPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.STORAGEGRID]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.POWERSTORE]: transformRest,
    [hwTypeKey_constants_1.HwTypeKey.SANBRCD]: transformSnmpFabric,
    [hwTypeKey_constants_1.HwTypeKey.SANCISCO]: transformSnmpOnly,
    [hwTypeKey_constants_1.HwTypeKey.LANCISCO]: transformSnmpOnly,
    [hwTypeKey_constants_1.HwTypeKey.LANOTHERS]: transformSnmpOnly,
    [hwTypeKey_constants_1.HwTypeKey.KUBERNETES]: [new KubernetesTransformer, kubernetes_data_1.KubernetesProps, kubernetes_data_1.KubernetesPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.OPENSHIFT]: [new OpenShiftTransformer, openshift_data_1.OpenshiftProps, openshift_data_1.OpenshiftPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.AZURE]: [new AzureTransformer, azure_data_1.AzureProps, azure_data_1.AzurePropsOld],
    [hwTypeKey_constants_1.HwTypeKey.AWS]: [new AmazonTransformer, aws_data_1.AwsProps, aws_data_1.AwsPropsOld],
    [hwTypeKey_constants_1.HwTypeKey.GCLOUD]: [new GCloudDeviceTransformer, gcloud_data_1.GcloudProps, gcloud_data_1.GcloudPropsOld]
};
const hwTypeMap = {
    BRCD: "sanbrcd",
    "SAN-BRCD": "sanbrcd",
    CISCO: "sancisco",
    "SAN-CISCO": "sancisco",
    "Hyper-V": "windows",
    "IBM Power Systems": "power",
    "ibm power systems": "power",
    "postgresql": "postgres",
    "PostgreSQL": "postgres",
    "rhv (ovirt)": "ovirt",
    "RHV (oVirt)": "ovirt",
    "ontaprapi": "netapp",
    "ONTAPRAPI": "netapp",
    "DS5K": "eseries",
    "ds5k": "eseries",
    "LAN-CISCO": "lancisco",
    "lan-cisco": "lancisco",
    "LAN-NON-CISCO": "lanothers",
    "lan-non-cisco": "lanothers"
};
const bannedWords = ["hwtype", "uuid", "devicealias"];
const passKeys = ['password', 'Password', 'apiPassword', 'snmpPrivPassword', 'snmpAuthPassword', 'sshPassword', 'cliPassword', 'ftpPassword', 'api-password', 'aws_secret_access_key'];
const renameKeyBlackList = ['backup_host', 'use_whitelist'];
function renamePropertyName(propertyName, separator = "-") {
    return renameKeyBlackList.includes(propertyName) ? propertyName : propertyName.split(separator).map((str, idx) => idx !== 0 ? str[0].toUpperCase() + str.substring(1) : str).join("");
}
;
class DeviceImporter {
    constructor() {
        this.doStor = false;
        this.doLpar = false;
        this.unsupportedDevices = [];
        this.alreadyExistingConfiguration = [];
        this.insertedDevices = [];
        this.devicesToSave = [];
    }
    async init(metricsTableService) {
        this.metricsTableService = metricsTableService;
        this.hwTypes = Microservice_1.Microservice.list().map(m => m.microservice);
        this.hostCfgRepository = (0, dataSource_1.getDataSource)().getRepository(hostcfg_entity_1.HostCfg);
    }
    logInformation(unsupportedHwTypes, existingCfgs, inserted) {
        if (unsupportedHwTypes && unsupportedHwTypes.length > 0) {
            console.log("=============================================================================");
            console.log("UNSUPPORTED DEVICES");
            unsupportedHwTypes.forEach(device => {
                console.log("Label: ", device.label, ", ", "Device: ", device.data.device, ", ", "Vendor: ", device.data.vendor, ", ", "HwType: ", device.hw_type);
            });
            console.log("=============================================================================");
        }
        if (existingCfgs && existingCfgs.length > 0) {
            console.log("=============================================================================");
            console.log("ALREADY IMPORTED DEVICES");
            existingCfgs.forEach(device => {
                console.log("Label: ", device.label, ", ", "Device: ", device.data.device, ", ", "Vendor: ", device.data.vendor, ", ", "HwType: ", device.hw_type);
            });
            console.log("=============================================================================");
        }
        let hwTypes = new Set();
        if (inserted && inserted.length > 0) {
            console.log("=============================================================================");
            console.log("NEW IMPORTED DEVICES");
            inserted.forEach(device => {
                console.log("Label: ", device.label, ", ", "Device: ", device.data.device, ", ", "Vendor: ", device.data.vendor, ", ", "HwType: ", device.hw_type);
                hwTypes.add(device.hw_type);
            });
            console.log("=============================================================================");
        }
        return hwTypes;
    }
    async importByType(type) {
        try {
            const typeData = await this.decoder.decodeFile();
            if (!typeData.error) {
                let filteredItemsHwType = [];
                let filteredItemsExistsId = [];
                let devices = (await this.parseByType(typeData, type.type)).filter((device) => {
                    let inHwTypes = this.hwTypes.includes(device.hw_type);
                    if (!inHwTypes)
                        filteredItemsHwType.push(device);
                    if (device.data.device && device.data.device === "REMOVE") {
                        filteredItemsHwType.push(device);
                        return false;
                    }
                    return inHwTypes;
                });
                let ids = (await this.hostCfgRepository.find({
                    where: {
                        hostcfg_id: (0, typeorm_1.In)(devices.map(device => device.hostcfg_id)),
                    }
                })).map(device => device.hostcfg_id);
                let labels = (await this.hostCfgRepository.find({
                    where: {
                        label: (0, typeorm_1.In)(devices.map(d => d.label))
                    }
                })).map(d => d.label);
                devices = devices.filter(device => {
                    let exists = !ids.includes(device.hostcfg_id);
                    if (!exists)
                        filteredItemsExistsId.push(device);
                    return exists;
                });
                devices = devices.filter(device => {
                    let exists = !labels.includes(device.label);
                    if (!exists)
                        filteredItemsExistsId.push(device);
                    return exists;
                });
                let uuidExists = {};
                devices = devices.filter((device) => {
                    if (uuidExists[device.hostcfg_id])
                        return false;
                    uuidExists[device.hostcfg_id] = true;
                    return true;
                });
                this.devicesToSave.push(...devices);
                this.unsupportedDevices.push(...filteredItemsHwType);
                this.alreadyExistingConfiguration.push(...filteredItemsExistsId);
            }
            else {
                console.error(typeData.message);
            }
        }
        catch (error) {
            console.error(error);
            if (error.statusCode === common_1.HttpStatus.UNAUTHORIZED || error.status === common_1.HttpStatus.UNAUTHORIZED) {
                throw new xormon_exception_1.XormonException(error.response.error + " " + type.type, common_1.HttpStatus.I_AM_A_TEAPOT);
            }
            if (error.status === common_1.HttpStatus.NOT_FOUND || error.statusCode === common_1.HttpStatus.NOT_FOUND) {
                throw new xormon_exception_1.XormonException(error.response.error + " " + type.type, common_1.HttpStatus.NOT_FOUND);
            }
            if (error.message.includes("URL")) {
                throw new xormon_exception_1.XormonException("Invalid URL " + type.type, common_1.HttpStatus.BAD_REQUEST);
            }
            if (error.statusCode === 408 || error.status === 408) {
                throw new xormon_exception_1.XormonException(error.response.error + " " + type.type, common_1.HttpStatus.REQUEST_TIMEOUT);
            }
            if (error.message.includes("Bad session key")) {
                throw new xormon_exception_1.XormonException("Bad session key " + type.type, common_1.HttpStatus.NOT_ACCEPTABLE);
            }
            throw new xormon_exception_1.XormonException(error.response.error + " " + type.type, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
    ;
    async parseByType(data, type) {
        const devices = [];
        const obj = type === "stor" ? data.class : data.platforms;
        if (type === "stor") {
            for (const key of Object.keys(obj)) {
                devices.push(...await this.goThroughAliases(obj[key].alias, key));
            }
        }
        else {
            for (const key of Object.keys(obj)) {
                devices.push(...await this.goThroughAliases(obj[key].aliases, "servers", key));
            }
        }
        return devices;
    }
    ;
    async goThroughAliases(aliases, _class, hw_type = null, vendor = null) {
        const devices = [];
        for (const alias of Object.keys(aliases)) {
            const currentDevice = aliases[alias];
            const newHostCfg = new hostcfg_entity_1.HostCfg();
            newHostCfg.hostcfg_id = currentDevice["uuid"];
            if (hw_type) {
                newHostCfg.hw_type = hwTypeMap[hw_type] ? hwTypeMap[hw_type] : hw_type.toLowerCase();
            }
            else {
                newHostCfg.hw_type = hwTypeMap[currentDevice["hwtype"]] ? hwTypeMap[currentDevice["hwtype"]] : currentDevice["hwtype"].toLowerCase();
            }
            newHostCfg.label = alias;
            newHostCfg.data = await this.transformByHwType(currentDevice, newHostCfg.hw_type);
            if (hw_type) {
                newHostCfg.data.device = hw_type;
                newHostCfg.data.vendor = hw_type;
            }
            newHostCfg.disabled = newHostCfg.data.disabled;
            delete newHostCfg.data.disabled;
            devices.push(newHostCfg);
        }
        return devices;
    }
    async transformByHwType(data, hwType) {
        if (hwTypeClassMap[hwType]) {
            let [transformer, props, oldProps] = hwTypeClassMap[hwType];
            if (transformer instanceof SnmpNetworkDeviceTransformer) {
                return await transformer.addFabric(data, props);
            }
            return transformer.transform(data, props);
        }
        else {
            return new TransformOtherDevices().transform(data, deviceProps_1.BaseDeviceProps);
        }
    }
    async importRemoteDevices() {
        if (this.doLpar) {
            this.decoder = new ConfigurationDownloadDecoder(this.lparType.url, this.lparType.username, this.lparType.password, this.lparType.passphrase, this.lparType.type, this.lparType.fileName);
            await this.importByType(this.lparType);
        }
        if (this.doStor) {
            this.decoder = new ConfigurationDownloadDecoder(this.storType.url, this.storType.username, this.storType.password, this.storType.passphrase, this.storType.type, this.storType.fileName);
            await this.importByType(this.storType);
        }
        if (this.devicesToSave.length) {
            try {
                for (const device of this.devicesToSave) {
                    if (device.data["radio"]) {
                        if (device.data["radio"].includes("cli")) {
                            device.disabled = true;
                        }
                    }
                    await this.hostCfgRepository.save(device);
                }
                this.insertedDevices.push(...this.devicesToSave);
                let hwTypes = this.logInformation(this.unsupportedDevices, this.alreadyExistingConfiguration, this.insertedDevices);
                for (const hwType of hwTypes.values()) {
                    await Microservice_1.Microservice.checkAndStart(hwType);
                }
                this.metricsTableService.generate();
            }
            catch (error) {
                console.log(error);
            }
        }
    }
    async startFromREST(lparAuth, storAuth) {
        if (lparAuth) {
            this.lparType = lparAuth;
            this.doLpar = true;
        }
        if (storAuth) {
            this.storType = storAuth;
            this.doStor = true;
        }
        await this.importRemoteDevices();
        return {
            unsupportedDevices: this.unsupportedDevices,
            alreadyExistingConfiguration: this.alreadyExistingConfiguration,
            insertedDevices: this.insertedDevices,
        };
    }
    async gatherAuth(type, rl) {
        const url = await new Promise(resolve => {
            rl.question(`Type the url adress where your file is located\n`, resolve);
        });
        const username = await new Promise(resolve => {
            rl.question(`Type the username of your ${type}2rrd account\n`, resolve);
        });
        const password = await new Promise(resolve => {
            rl.question(`Type the password of your ${type}2rrd account\n`, resolve);
        });
        const passphrase = await new Promise(resolve => {
            rl.question(`Type the passphrase you used to encrypt your ${type}2rrd configuration file\n`, resolve);
        });
        return {
            url: url,
            username: username,
            password: password,
            type: type,
            passphrase: passphrase,
            fileName: ""
        };
    }
    async gatherInformation() {
        let rlInterface = rl.createInterface({ input: process.stdin, output: process.stdout });
        const typeChoice = (await new Promise(resolve => {
            rlInterface.question("Do you want to import devices from both LPAR2RRD and STOR2RRD or only from one?: [BOTH / LPAR / STOR]\n", resolve);
        })).toUpperCase();
        if (typeChoice !== "STOR" && typeChoice !== "LPAR" && typeChoice !== "BOTH") {
            console.log(`Your choice [ ${typeChoice} ] is not one of the valid choices: [BOTH / LPAR / STOR]`);
            console.log("Exiting import");
            return;
        }
        if (typeChoice === "STOR" || typeChoice === "BOTH") {
            this.storType = await this.gatherAuth("stor", rlInterface);
            this.doStor = true;
        }
        if (typeChoice === "LPAR" || typeChoice === "BOTH") {
            this.lparType = await this.gatherAuth("lpar", rlInterface);
            this.doLpar = true;
        }
        await this.importRemoteDevices();
    }
}
exports.DeviceImporter = DeviceImporter;
function decodeChars(inBytes, inIndex, outBytes, outIndex) {
    let c1 = inBytes[inIndex].charCodeAt(0);
    let c2 = inBytes[inIndex + 1].charCodeAt(0);
    let c3 = inBytes[inIndex + 2].charCodeAt(0);
    let c4 = inBytes[inIndex + 3].charCodeAt(0);
    let b1 = (c1 - 32 & 0x3F) << 2 | (c2 - 32 & 0x3F) >> 4;
    let b2 = (c2 - 32 & 0x3F) << 4 | (c3 - 32 & 0x3F) >> 2;
    let b3 = (c3 - 32 & 0x3F) << 6 | c4 - 32 & 0x3F;
    outBytes[outIndex] = String.fromCharCode(b1 & 0xFF);
    if (b2 & 0xFF) {
        outBytes[outIndex + 1] = String.fromCharCode(b2 & 0xFF);
    }
    if (b3 & 0xFF) {
        outBytes[outIndex + 2] = String.fromCharCode(b3 & 0xFF);
    }
}
function reveal(inString) {
    if (typeof inString === "boolean" && inString === true)
        return "!EXISTS!";
    if (!inString || inString === "null") {
        return "";
    }
    let outString = Buffer.from(inString, "base64").toString();
    let stop = false;
    let inIndex = 0;
    let outIndex = 0;
    let totalLen = 0;
    let inBytes = outString.split("");
    let buffLen = inBytes.length;
    let outBytes = new Array(buffLen);
    do {
        if (inIndex < buffLen) {
            let n = inBytes[inIndex].charCodeAt(0) - 32 & 0x3F;
            ++inIndex;
            if (n > 45) {
                throw 'Invalid Data';
            }
            if (n < 45) {
                stop = true;
            }
            totalLen += n;
            while (n > 0) {
                decodeChars(inBytes, inIndex, outBytes, outIndex);
                outIndex += 3;
                inIndex += 4;
                n -= 3;
            }
            ++inIndex;
        }
        else {
            stop = true;
        }
    } while (!stop);
    return outBytes.join("").trim();
}
class ConfigurationDownloadDecoder {
    constructor(url, username, password, passphrase, type, fileName) {
        this.url = url;
        this.username = username;
        this.password = password;
        this.passphrase = passphrase;
        this.type = type;
        this.fileName = fileName;
        this.decodedFilePath = "";
        this.encodedFilePath = "";
        this.fileType = "gpg";
        this.passphrase = this.sanitize(passphrase);
        this.type = this.sanitize(type);
        this.proto = this.parseUrl();
        this.fileType = fileName.split(".").at(-1);
        if (!(0, fs_1.existsSync)(ConfigurationDownloadDecoder.TMP_PATH)) {
            (0, fs_1.mkdirSync)(ConfigurationDownloadDecoder.TMP_PATH, { recursive: true });
        }
    }
    sanitize(input) {
        if (!input) {
            return "";
        }
        const map = {
            '&': '&amp;',
            '<': '&lt;',
            '>': '&gt;',
            '"': '&quot;',
            "'": '&#x27;',
            "/": '&#x2F;',
        };
        const reg = /[&<>"'/]/ig;
        return input.replace(reg, (match) => (map[match]));
    }
    isUrlHttp() {
        return this.url.slice(0, 4) === "http";
    }
    parseUrl() {
        let proto = "unknown";
        if (this.isUrlHttp()) {
            proto = this.url[4] === "s" ? "https" : "http";
        }
        let urlArray = this.url.split("/?");
        this.url = urlArray[0];
        this.url = this.url[this.url.length - 1] === "/" ? this.url + this.fileName : this.url + "/" + this.fileName;
        return proto;
    }
    async get(stream, proto) {
        const fullURL = this.proto !== "unknown" ? this.url : `${proto}://${this.url}`;
        return (await new request_builder_1.RequestBuilder((proto === "unknown" || proto === "http" ? "http" : "https"), fullURL, request_builder_1.RequestMethod.GET)
            .addAgent()
            .setHeaders({ Authorization: "Basic " + btoa(`${this.username}:${this.password}`) })
            .setStream(stream)
            .setRequestTimeout(10000)
            .processRequest()).statusCode;
    }
    async httpsOrHttp() {
        const stream = (0, fs_1.createWriteStream)(ConfigurationDownloadDecoder.TMP_PATH + `/${this.type}_01.${this.fileType}`);
        try {
            if (this.proto === "http") {
                throw new Error("http one");
            }
            const ret = await this.get(stream, "https");
            if (!stream.closed) {
                stream.close();
            }
            return ret;
        }
        catch (error) {
            let err;
            if (error.statusCode) {
                err = error.statusCode;
            }
            else {
                err = error;
            }
            try {
                if (this.proto !== "http") {
                    console.log(err);
                    console.error("Failed import when using https. Status code: " + err);
                    throw err;
                }
                if (!stream.closed) {
                    stream.close();
                }
                this.cleanUp();
                const newStream = (0, fs_1.createWriteStream)(ConfigurationDownloadDecoder.TMP_PATH + `/${this.type}_01.${this.fileType}`);
                const ret = await this.get(newStream, "http");
                if (!newStream.closed) {
                    newStream.close();
                }
                return ret;
            }
            catch (err) {
                let secErr;
                if (err.statusCode) {
                    secErr = err.statusCode;
                }
                else {
                    secErr = err;
                }
                console.error("Failed import when using " + this.proto + ". Status code: " + secErr);
                if (secErr === common_1.HttpStatus.NOT_FOUND) {
                    throw new xormon_exception_1.XormonException("Wrong url address for configuration file or file was already deleted", common_1.HttpStatus.NOT_FOUND);
                }
                if (secErr === common_1.HttpStatus.UNAUTHORIZED) {
                    throw new xormon_exception_1.XormonException("Wrong credentials", common_1.HttpStatus.UNAUTHORIZED);
                }
                if (secErr === common_1.HttpStatus.REQUEST_TIMEOUT) {
                    throw new xormon_exception_1.XormonException("Connection timed out", common_1.HttpStatus.REQUEST_TIMEOUT);
                }
                throw new xormon_exception_1.XormonException(secErr, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }
    }
    async downloadFile() {
        const status = await this.httpsOrHttp();
        if (status === common_1.HttpStatus.NOT_FOUND) {
            throw new xormon_exception_1.XormonException("Wrong url address for configuration file or file was already deleted", common_1.HttpStatus.NOT_FOUND);
        }
        else if (status === common_1.HttpStatus.UNAUTHORIZED) {
            throw new xormon_exception_1.XormonException("Wrong credentials", common_1.HttpStatus.UNAUTHORIZED);
        }
        else if (status === common_1.HttpStatus.REQUEST_TIMEOUT) {
            throw new xormon_exception_1.XormonException("Request timed out", common_1.HttpStatus.UNAUTHORIZED);
        }
        else if (status === common_1.HttpStatus.INTERNAL_SERVER_ERROR) {
            throw new xormon_exception_1.XormonException("Server error", common_1.HttpStatus.UNAUTHORIZED);
        }
        return ConfigurationDownloadDecoder.TMP_PATH + `/${this.type}_01.${this.fileType}`;
    }
    async cleanUp() {
        if ((0, fs_1.existsSync)(this.encodedFilePath)) {
            await (0, promises_1.unlink)(this.encodedFilePath);
        }
        if ((0, fs_1.existsSync)(this.decodedFilePath)) {
            await (0, promises_1.unlink)(this.decodedFilePath);
        }
    }
    async decodeFile() {
        const filePath = await this.downloadFile();
        if (this.fileType === "gpg") {
            (0, child_process_1.execSync)(`gpg --passphrase "${this.passphrase}" --batch --ignore-mdc-error -d -o ${ConfigurationDownloadDecoder.TMP_PATH}/${this.type}.json ${filePath}`);
            this.encodedFilePath = filePath;
            this.decodedFilePath = `${ConfigurationDownloadDecoder.TMP_PATH}/${this.type}.json`;
        }
        else {
            this.decodedFilePath = filePath;
        }
        let configurationsJSON = JSON.parse((0, fs_1.readFileSync)(this.decodedFilePath).toString());
        await this.cleanUp();
        return configurationsJSON;
    }
}
ConfigurationDownloadDecoder.TMP_PATH = (0, path_1.resolve)("./files/tmp");
//# sourceMappingURL=import.js.map