"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LdapController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const ldap_dto_1 = require("../../common/ldap.dto");
const ldap_service_1 = require("./ldap.service");
const xormon_exception_1 = require("../../exception/xormon.exception");
const platform_express_1 = require("@nestjs/platform-express");
const ldap_cert_service_1 = require("./ldap.cert.service");
const typeorm_transactional_1 = require("typeorm-transactional");
let LdapController = class LdapController {
    constructor(ldapService, ldapCertService) {
        this.ldapService = ldapService;
        this.ldapCertService = ldapCertService;
        this.logger = ldap_service_1.LdapService.logger;
    }
    async listLdaps() {
        const ldaps = await this.ldapService.getLdaps();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: ldaps.map(ldap => {
                const groups = ldap._groups.map(group => {
                    return group.ldap_group;
                });
                delete ldap._groups;
                return {
                    ...ldap,
                    groups: groups
                };
            })
        };
    }
    async saveLdap(saveLdap) {
        let data;
        try {
            if (saveLdap.disabled) {
                const ldap = await this.ldapService.saveLdap(saveLdap);
                return {
                    statusCode: common_1.HttpStatus.CREATED,
                    data: {
                        ...ldap,
                        groups: []
                    }
                };
            }
            data = await this.ldapService.testLdapAllGroups(saveLdap);
            const saved = await this.ldapService.saveLdap(saveLdap);
            if (saved) {
                if (data && data.length >= 1) {
                    await this.ldapService.saveLdapGroups(data.map(item => {
                        return {
                            ldap_id: saved.ldap_id,
                            ldap_group: item.dn
                        };
                    }));
                }
                const ldap = await this.ldapService.getLdap(saved.ldap_id);
                const groups = ldap._groups.map(group => group.ldap_group);
                delete ldap._groups;
                return {
                    statusCode: common_1.HttpStatus.CREATED,
                    data: {
                        ...ldap,
                        groups: groups
                    }
                };
            }
            else {
                throw new xormon_exception_1.XormonException("Error occurred while saving the ldap", common_1.HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }
        catch (e) {
            this.logger.error("Failed to save LDAP cfg", e);
            throw e;
        }
    }
    async getLdap(ldap_id) {
        const ldap = await this.ldapService.getLdap(ldap_id);
        const groups = ldap._groups.map(group => group.ldap_group);
        delete ldap._groups;
        return {
            statusCode: common_1.HttpStatus.OK,
            data: {
                ...ldap,
                groups: groups
            }
        };
    }
    async updateLdap(ldap_id, ldapSave) {
        let data;
        ldapSave.ldap_id = ldap_id;
        try {
            if (ldapSave.disabled) {
                const ldap = await this.ldapService.saveLdap(ldapSave);
                return {
                    statusCode: common_1.HttpStatus.CREATED,
                    data: {
                        ...ldap,
                        groups: []
                    }
                };
            }
            data = await this.ldapService.testLdapAllGroups(ldapSave);
            const old = await this.ldapService.getLdap(ldap_id);
            const ldapGroups = [];
            const ldap = {
                ...ldapSave, ldap_id: ldap_id, full_dn_auth: ldapSave.full_dn_auth, group_base: ldapSave.group_base, group_filter_type: ldapSave.group_filter_type, user_base: ldapSave.user_base,
                required_user_group: ldapSave.required_user_group, disabled: ldapSave.disabled,
                _groups: ldapGroups
            };
            for (const lgo of data) {
                const found = old._groups.find(lg => lg.ldap_group === lgo.dn);
                if (found) {
                    ldapGroups.push(found);
                }
                else {
                    ldapGroups.push({ ldap_id, _ldap: ldap, ldap_group: lgo.dn });
                }
            }
            const saved = await this.ldapService.saveLdap(ldap);
            if (saved) {
                const ldap = await this.ldapService.getLdap(saved.ldap_id);
                const groups = ldap._groups.map(group => group.ldap_group);
                delete ldap._groups;
                return {
                    statusCode: common_1.HttpStatus.CREATED,
                    data: {
                        ...ldap,
                        groups: groups
                    }
                };
            }
            else {
                throw new xormon_exception_1.XormonException("Error occurred while updating the ldap", common_1.HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }
        catch (e) {
            this.logger.error("Failed to update LDAP cfg", e);
            throw e;
        }
    }
    async deleteLdap(ldap_id) {
        if (await this.ldapService.deleteLdap(ldap_id)) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: "Ldap deleted successfully"
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'error occurred while deleting the ldap',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async listCert() {
        return {
            statusCode: common_1.HttpStatus.OK,
            data: await this.ldapCertService.getLdapCertificates()
        };
    }
    async saveCert(file) {
        if (!file.buffer) {
            throw new xormon_exception_1.XormonException("Empty file", common_1.HttpStatus.BAD_REQUEST);
        }
        let fileName = file.originalname;
        let checkFileName = true;
        let i = 1;
        while (checkFileName) {
            if (await this.ldapCertService.checkLdapCertificatePath(this.ldapCertService.createLdapCertificatePath(fileName))) {
                const fileNameSplit = file.originalname.split(".");
                fileName = `${fileNameSplit[0]}-${i}.${fileNameSplit[1]}`;
                i++;
            }
            else {
                checkFileName = false;
            }
        }
        if (await this.ldapCertService.saveLdapCertificate(file.buffer.toString(), fileName)) {
            return {
                message: "The LDAP certificate was uploaded successfully",
                statusCode: common_1.HttpStatus.CREATED
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while saving the LDAP certificate", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteCert(ldapCertificateId) {
        if (!await this.ldapCertService.getLdapCertificate(ldapCertificateId)) {
            throw new xormon_exception_1.XormonException("LDAP not found", common_1.HttpStatus.NOT_FOUND);
        }
        if (await this.ldapCertService.deleteLdapCertificates(ldapCertificateId)) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: "The LDAP certificate was deleted successfully"
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while deleting the LDAP certificate", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async testLdap(ldapData) {
        let data, error;
        try {
            data = await this.ldapService.testLdapAllGroups(ldapData);
        }
        catch (e) {
            error = e;
            this.logger.error("LDAP test failed", e);
        }
        if (error) {
            throw new xormon_exception_1.XormonException("Error occurred while testing the ldap cfg", common_1.HttpStatus.BAD_REQUEST, error);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: data.map(item => {
                return {
                    cn: item.cn ? item.cn : null,
                    dn: item.dn ? item.dn : null,
                    objectClass: item.objectClass ? item.objectClass : null
                };
            })
        };
    }
};
exports.LdapController = LdapController;
__decorate([
    (0, common_1.Get)('v1/'),
    openapi.ApiResponse({ status: 200, type: require("../../common/ldap.dto").LdapsResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], LdapController.prototype, "listLdaps", null);
__decorate([
    (0, common_1.Post)('v1/'),
    openapi.ApiResponse({ status: 201, type: require("../../common/ldap.dto").LdapResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [ldap_dto_1.LdapSave]),
    __metadata("design:returntype", Promise)
], LdapController.prototype, "saveLdap", null);
__decorate([
    (0, common_1.Get)('v1/:ldap_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/ldap.dto").LdapResponseDTO }),
    __param(0, (0, common_1.Param)('ldap_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], LdapController.prototype, "getLdap", null);
__decorate([
    (0, typeorm_transactional_1.Transactional)(),
    (0, common_1.Put)('v1/:ldap_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/ldap.dto").LdapResponseDTO }),
    __param(0, (0, common_1.Param)('ldap_id')),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, ldap_dto_1.LdapSave]),
    __metadata("design:returntype", Promise)
], LdapController.prototype, "updateLdap", null);
__decorate([
    (0, common_1.Delete)('v1/:ldap_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)('ldap_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], LdapController.prototype, "deleteLdap", null);
__decorate([
    (0, common_1.Get)('v2/certificates'),
    openapi.ApiResponse({ status: 200, type: require("../../common/ldap.dto").LdapCertificatesResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], LdapController.prototype, "listCert", null);
__decorate([
    (0, common_1.Post)('v2/certificates'),
    (0, common_1.UseInterceptors)((0, platform_express_1.FileInterceptor)('file')),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.UploadedFile)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], LdapController.prototype, "saveCert", null);
__decorate([
    (0, common_1.Delete)('v2/certificate/:ldap_certificate_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)('ldap_certificate_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], LdapController.prototype, "deleteCert", null);
__decorate([
    (0, common_1.Post)('v2/test'),
    openapi.ApiResponse({ status: 201, type: require("../../common/ldap.dto").LdapGroupsResponse }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [ldap_dto_1.LdapSave]),
    __metadata("design:returntype", Promise)
], LdapController.prototype, "testLdap", null);
exports.LdapController = LdapController = __decorate([
    (0, swagger_1.ApiTags)('LDAP'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/ldap'),
    __metadata("design:paramtypes", [ldap_service_1.LdapService, ldap_cert_service_1.LdapCertService])
], LdapController);
//# sourceMappingURL=ldap.controller.js.map