"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoggerFactory = exports.logFormat = void 0;
const winston_1 = require("winston");
const App_1 = require("../configuration/App");
const safe_stable_stringify_1 = require("safe-stable-stringify");
exports.logFormat = winston_1.format.printf(info => {
    const { level, message, timestamp, ...rest } = info;
    const splat = rest[Symbol.for('splat')];
    let extra = '';
    if (info instanceof Error) {
        extra += '\n' + info.stack;
    }
    if (splat?.length) {
        for (const item of splat) {
            extra += '\n' + (0, safe_stable_stringify_1.stringify)(item);
            if (item instanceof Error) {
                extra += '\n' + item.stack;
            }
        }
    }
    return `[${timestamp}] ${level}: ${message} ${extra}`;
});
const LoggerFactory = (file) => {
    return (0, winston_1.createLogger)({
        format: winston_1.format.combine(winston_1.format.timestamp(), winston_1.format.json(), winston_1.format.simple(), exports.logFormat),
        transports: [
            new winston_1.transports.File({ filename: `${App_1.App.LOGS_PATH}/${file}-error.log`, level: 'error' }),
            new winston_1.transports.File({
                filename: `${App_1.App.LOGS_PATH}/${file}.log`,
                level: process.env['LOG_' + file.toUpperCase()] || process.env['LOG_ALL'] || 'info'
            }),
        ],
    });
};
exports.LoggerFactory = LoggerFactory;
//# sourceMappingURL=WinstonLogger.js.map