"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var LogService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogService = void 0;
const common_1 = require("@nestjs/common");
const log_constants_1 = require("../../common/constants/log.constants");
const App_1 = require("../configuration/App");
const microservice_entity_1 = require("../../entity/microservice.entity");
const typeorm_1 = require("typeorm");
const typeorm_2 = require("@nestjs/typeorm");
const PM2_1 = require("../configuration/PM2");
let LogService = LogService_1 = class LogService {
    constructor(microservicesRepository) {
        this.microservicesRepository = microservicesRepository;
    }
    async listLogs() {
        let logs = [];
        const fs = require('fs');
        const files = await fs.readdirSync(App_1.App.LOGS_PATH);
        for (const file of files) {
            if (file.includes('-error-')) {
                const split = file.split('-error-', 3);
                const number = split[1].replace('.log', '');
                if (file.includes("backend")) {
                    logs.push({
                        label: 'Xormon ' + number,
                        file: file,
                        url: '/api/log/' + file.replace('.log', ''),
                        type: log_constants_1.LOG_TYPE.backend,
                        path: App_1.App.LOGS_PATH + '/' + file
                    });
                }
                else if (file.includes("metrics")) {
                    logs.push({
                        label: 'Metrics server ' + number,
                        file: file,
                        url: '/api/log/' + file.replace('.log', ''),
                        type: log_constants_1.LOG_TYPE.backend,
                        path: App_1.App.LOGS_PATH + '/' + file
                    });
                }
            }
        }
        logs.push({
            label: 'Tasks server',
            file: 'tasks-server-error.log',
            url: '/api/log/tasks-server-error',
            type: log_constants_1.LOG_TYPE.backend,
            path: App_1.App.LOGS_PATH + '/tasks-server-error.log'
        });
        logs.push({
            label: 'Alerting',
            file: 'alerting-error.log',
            url: '/api/log/alerting-error',
            type: log_constants_1.LOG_TYPE.alerting,
            path: App_1.App.LOGS_PATH + '/alerting-error.log'
        });
        logs.push({
            label: 'Exporter',
            file: 'exporter-error.log',
            url: '/api/log/exporter-error',
            type: log_constants_1.LOG_TYPE.exporter,
            path: App_1.App.LOGS_PATH + '/exporter-error.log'
        });
        if (await fs.existsSync(App_1.App.LOGS_PATH + "/mailer-error.log")) {
            logs.push({
                label: 'Mailer',
                file: 'mailer-error.log',
                url: '/api/log/mailer-error',
                type: log_constants_1.LOG_TYPE.mailer,
                path: App_1.App.LOGS_PATH + "/mailer-error.log"
            });
        }
        if (await fs.existsSync(App_1.App.LOGS_PATH + "/alias-import-error.log")) {
            logs.push({
                label: 'Alias import',
                file: 'alias-import-error.log',
                url: '/api/log/alias-import-error',
                type: log_constants_1.LOG_TYPE['alias-import'],
                path: App_1.App.LOGS_PATH + "/alias-import-error.log"
            });
        }
        logs.push({
            label: 'Microservices',
            file: 'microservices-error.log',
            url: '/api/log/microservices-error',
            type: log_constants_1.LOG_TYPE.microservice,
            path: App_1.App.LOGS_PATH + "/microservices-error.log"
        });
        const microservices = await PM2_1.PM2.listServices();
        const servicesFromDB = await this.microservicesRepository.find();
        for (const key in microservices) {
            if (microservices[key]['name'] == 'backend' || microservices[key]['name'] == 'metrics-server' || microservices[key]['name'] == 'tasks-server' || microservices[key]['name'] == 'analyzer') {
                continue;
            }
            let serviceFromDB = servicesFromDB.find(ser => ser.microservice === microservices[key]['name']);
            logs.push({
                label: serviceFromDB ? serviceFromDB.label : microservices[key]['name'],
                file: microservices[key]['name'] + '-error.log',
                url: '/api/log/' + microservices[key]['name'] + '-error',
                type: log_constants_1.LOG_TYPE.microservice,
                path: App_1.App.LOGS_PATH + "/" + microservices[key]['name'] + '-error.log',
            });
        }
        return logs;
    }
    async openLog(file, lines = 100) {
        let logger = new common_1.Logger(LogService_1.name);
        const readLastLines = require('read-last-lines');
        try {
            let log = await readLastLines.read(App_1.App.LOGS_PATH + '/' + file + '.log', lines);
            return log.split('\n').reverse().join('\n');
        }
        catch (error) {
            logger.warn(error);
            return null;
        }
    }
};
exports.LogService = LogService;
exports.LogService = LogService = LogService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_2.InjectRepository)(microservice_entity_1.Microservice)),
    __metadata("design:paramtypes", [typeorm_1.Repository])
], LogService);
//# sourceMappingURL=log.service.js.map