"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MenuHandler = void 0;
const dataSource_1 = require("../../config/dataSource");
const Menu = require("./menu/Menu");
const hwType_entity_1 = require("../../entity/hwType.entity");
class MenuHandler {
    constructor(hwType, request) {
        this.hwType = hwType;
        this.request = request;
        this.debugMode = false;
        this.namespace = false;
        this.hwTypeHandlerName = this.formatHwTypeName(hwType);
        if (Menu[this.hwTypeHandlerName] !== undefined) {
            this.namespace = true;
        }
    }
    setDebug() {
        this.debugMode = true;
    }
    async getContent(pageType, itemId, tab) {
        itemId = itemId && itemId !== '' ? itemId : null;
        let pageTypeFormatted = this.formatTypeName(pageType);
        let page;
        try {
            if (this.isExist('Page', pageTypeFormatted)) {
                if (this.debugMode) {
                    if (this.namespace) {
                        console.log(`Handler: [${this.hwTypeHandlerName}] ${pageTypeFormatted + 'Page'}`);
                    }
                    else {
                        console.log(`Handler: ${this.hwTypeHandlerName + pageTypeFormatted + 'Page'}`);
                    }
                }
                page = this.namespace
                    ? new Menu[this.hwTypeHandlerName][pageTypeFormatted + 'Page'](itemId, this.request)
                    : new Menu[this.hwTypeHandlerName + pageTypeFormatted + 'Page'](itemId, this.request);
            }
            else {
                if (this.debugMode) {
                    console.log("Default handler not found");
                }
                const classCheck = await this.isStorageOrSan();
                if (classCheck) {
                    if (this.debugMode) {
                        console.log("Class handler found");
                    }
                    page = this.getClassPageHandler(classCheck, pageType, itemId);
                }
            }
            if (page === undefined) {
                console.error(`[MenuHandlerError] [Handler: ${this.hwTypeHandlerName + pageTypeFormatted + 'Page'}] No handler found`);
            }
            else {
                if (tab) {
                    return await page.getTab(tab);
                }
                else {
                    return await page.getPage();
                }
            }
        }
        catch (e) {
            console.error(`[MenuHandlerError] [HwTypeHandlerName: ${this.hwTypeHandlerName} PagerType: ${pageTypeFormatted}] error: ${e}`);
        }
    }
    ;
    async getPage(pageType, itemId) {
        return await this.getContent(pageType, itemId);
    }
    ;
    async getTab(pageType, itemId, tab) {
        return await this.getContent(pageType, itemId, tab);
    }
    ;
    async getFolder(folderType, ids, acl = false) {
        let itemId = ids !== null && ids !== undefined ? ids.split(',')[0] : null;
        let folderTypeFormatted = this.formatTypeName(folderType);
        let items = [];
        let folder;
        try {
            if (this.isExist('Folder', folderTypeFormatted)) {
                if (this.debugMode) {
                    if (this.namespace) {
                        console.log(`Handler: [${this.hwTypeHandlerName}] ${folderTypeFormatted + 'Folder'}`);
                    }
                    else {
                        console.log(`Handler: ${this.hwTypeHandlerName + folderTypeFormatted + 'Folder'}`);
                    }
                }
                folder = this.namespace
                    ? new Menu[this.hwTypeHandlerName][folderTypeFormatted + 'Folder'](itemId, this.request)
                    : new Menu[this.hwTypeHandlerName + folderTypeFormatted + 'Folder'](itemId, this.request);
            }
            else {
                if (this.debugMode) {
                    console.log("Default handler not found");
                }
                const classCheck = await this.isStorageOrSan();
                if (classCheck) {
                    if (this.debugMode) {
                        console.log("Class handler found");
                    }
                    folder = this.getClassFolderHandler(classCheck, folderType, itemId, acl);
                }
            }
            if (folder === undefined) {
                console.error(`[MenuHandlerError] [Handler: ${this.hwTypeHandlerName + folderTypeFormatted + 'Folder'}] No handler found`);
            }
            else {
                if (acl && typeof folder.getChildrenAcl === 'function') {
                    items = await folder.getChildrenAcl();
                }
                else {
                    items = await folder.getChildren();
                }
            }
        }
        catch (e) {
            console.error(`[MenuHandlerError] [HwTypeHandlerName: ${this.hwTypeHandlerName} FolderType: ${folderTypeFormatted}] error: ${e}`);
        }
        return items;
    }
    ;
    getClassFolderHandler(_class, folderType, itemId, acl = false) {
        const classHandler = _class === 'san' ? 'San' : _class === 'lan' ? 'Lan' : 'Stor';
        if (folderType === 'device') {
            if (this.debugMode) {
                if (this.namespace) {
                    console.log(`Handler: [${classHandler}] DeviceFolder`);
                }
                else {
                    console.log(`Handler: ${classHandler}DeviceFolder`);
                }
            }
            return this.namespace
                ? new Menu[classHandler]['DeviceFolder'](this.hwType, folderType, itemId, this.request)
                : new Menu[classHandler + 'DeviceFolder'](this.hwType, folderType, itemId, this.request);
        }
        else if (classHandler === 'San' && folderType === 'fabric') {
            if (this.debugMode) {
                if (this.namespace) {
                    console.log(`Handler: [San] FabricFolder`);
                }
                else {
                    console.log(`Handler: SanFabricFolder`);
                }
            }
            return this.namespace
                ? new Menu['San']['FabricFolder'](this.hwType, folderType, itemId, this.request)
                : new Menu['SanFabricFolder'](this.hwType, folderType, itemId, this.request);
        }
        else if (_class === "lan" && folderType === "group") {
            if (this.debugMode) {
                if (this.namespace) {
                    console.log(`Handler: [Lan] GroupFolder`);
                }
                else {
                    console.log(`Handler: LanGroupFolder`);
                }
            }
            return this.namespace
                ? new Menu['Lan']['GroupFolder'](this.hwType, folderType, itemId, this.request)
                : new Menu['LanGroupFolder'](this.hwType, folderType, itemId, this.request);
        }
        else {
            if (this.debugMode) {
                if (this.namespace) {
                    console.log(`Handler: [${classHandler}] SubsystemItemsFolder`);
                }
                else {
                    console.log(`Handler: ${classHandler}SubsystemItemsFolder`);
                }
            }
            return this.namespace
                ? new Menu[classHandler]['SubsystemItemsFolder'](this.hwType, folderType, itemId, this.request)
                : new Menu[classHandler + 'SubsystemItemsFolder'](this.hwType, folderType, itemId, this.request);
        }
    }
    getClassPageHandler(_class, pageType, itemId) {
        const classHandler = _class === 'san' ? 'San' : _class === 'lan' ? 'Lan' : 'Stor';
        const pageTypeName = this.formatTypeName(pageType);
        if (Menu[classHandler + '' + pageTypeName + 'Page']) {
            if (this.debugMode) {
                console.log(`Handler: ${classHandler}${pageTypeName}Page`);
            }
            return new Menu[classHandler + '' + pageTypeName + 'Page'](itemId, this.request);
        }
        else if (Menu[classHandler] && Menu[classHandler][pageTypeName + 'Page']) {
            if (this.debugMode) {
                console.log(`Handler: [${classHandler}] ${pageTypeName}Page`);
            }
            return new Menu[classHandler][pageTypeName + 'Page'](itemId, this.request);
        }
        else {
            if (!Menu[classHandler] || !Menu[classHandler]['ItemPage']) {
                this.namespace = false;
            }
            if (this.debugMode) {
                if (this.namespace) {
                    console.log(`Handler: [${classHandler}] ItemPage, pageType: ${pageType}`);
                }
                else {
                    console.log(`Handler: ${classHandler}ItemPage, pageType: ${pageType}`);
                }
            }
            return this.namespace
                ? new Menu[classHandler]['ItemPage'](this.hwType, pageType, itemId, this.request)
                : new Menu[classHandler + 'ItemPage'](this.hwType, pageType, itemId, this.request);
        }
    }
    async getPath(pageType, itemId) {
        itemId = itemId !== '' ? itemId : null;
        let pageTypeFormatted = this.formatTypeName(pageType);
        let page;
        try {
            if (this.isExist('Page', pageTypeFormatted)) {
                if (this.debugMode) {
                    if (this.namespace) {
                        console.log(`Handler: [${this.hwTypeHandlerName}] ${pageTypeFormatted + 'Page'}`);
                    }
                    else {
                        console.log(`Handler: ${this.hwTypeHandlerName + pageTypeFormatted + 'Page'}`);
                    }
                }
                page = this.namespace
                    ? new Menu[this.hwTypeHandlerName][pageTypeFormatted + 'Page'](itemId, this.request)
                    : new Menu[this.hwTypeHandlerName + pageTypeFormatted + 'Page'](itemId, this.request);
            }
            else {
                if (this.debugMode) {
                    console.log("Default handler not found");
                }
                const classCheck = await this.isStorageOrSan();
                if (classCheck) {
                    if (this.debugMode) {
                        console.log("Class handler found");
                    }
                    page = this.getClassPageHandler(classCheck, pageType, itemId);
                }
            }
            if (page === undefined) {
                console.error(`[MenuHandlerError] [Handler: ${this.hwTypeHandlerName + pageTypeFormatted + 'Page'}] No handler found`);
            }
            else {
                let path = await page.getPath();
                return path;
            }
        }
        catch (e) {
            console.error(`[MenuHandlerError] [HwTypeHandlerName: ${this.hwTypeHandlerName} PagerType: ${pageTypeFormatted}] error: ${e}`);
        }
    }
    ;
    async getRootFolder(acl = false) {
        let items = [];
        let folder;
        try {
            if (this.isRootFolderExist()) {
                if (this.debugMode) {
                    if (this.namespace) {
                        console.log(`Handler: [${this.hwTypeHandlerName}] RootFolder}`);
                    }
                    else {
                        console.log(`Handler: ${this.hwTypeHandlerName + 'RootFolder'}`);
                    }
                }
                folder = this.namespace
                    ? new Menu[this.hwTypeHandlerName]['RootFolder']()
                    : new Menu[this.hwTypeHandlerName + 'RootFolder']();
            }
            else {
                const _class = await this.isStorageOrSan();
                if (_class) {
                    if (this.debugMode) {
                        console.log(`Handler: ${_class === 'storage' ? 'Stor' : _class === 'lan' ? 'Lan' : 'San' + 'RootFolder'}`);
                    }
                    folder = _class === 'storage'
                        ? folder = new Menu['StorRootFolder'](this.hwType, this.request)
                        : _class === 'lan'
                            ? folder = new Menu['LanRootFolder'](this.hwType, null, this.request)
                            : folder = new Menu['SanRootFolder'](this.hwType, this.request);
                }
                else {
                    console.error(`[MenuHandlerError] [Handler: ${this.hwTypeHandlerName + 'RootFolder'}] No handler found`);
                    return [];
                }
            }
            if (folder === undefined) {
                console.error(`[MenuHandlerError] [Handler: ${this.hwTypeHandlerName + 'RootFolder'}] No handler found`);
                return [];
            }
            else {
                if (acl && typeof folder.getChildrenAcl === 'function') {
                    items = await folder.getChildrenAcl();
                }
                else {
                    items = await folder.getChildren(this.request);
                }
            }
        }
        catch (e) {
            console.error(`[MenuHandlerError] [Handler: ${this.hwTypeHandlerName + 'RootFolder'}] error: ${e}`);
        }
        return items;
    }
    ;
    isExist(type, pageOrFolder) {
        if (Menu[this.hwTypeHandlerName + pageOrFolder + type] !== undefined) {
            this.namespace = false;
        }
        return this.namespace
            ? Menu[this.hwTypeHandlerName] !== undefined && Menu[this.hwTypeHandlerName][pageOrFolder + type] !== undefined
            : Menu[this.hwTypeHandlerName + pageOrFolder + type] !== undefined;
    }
    isRootFolderExist() {
        return this.namespace
            ? Menu[this.hwTypeHandlerName] !== undefined && Menu[this.hwTypeHandlerName]['RootFolder'] !== undefined
            : Menu[this.hwTypeHandlerName + 'RootFolder'] !== undefined;
    }
    async isStorageOrSan() {
        if (['sgroup', 'sgroups', 'storages', 'storage'].includes(this.hwType)) {
            return 'storage';
        }
        const checkHwType = await (0, dataSource_1.getDataSource)().getRepository(hwType_entity_1.HwType).findOneBy({
            hw_type: this.hwType
        });
        return checkHwType && ['class', 'storage', 'san', 'lan'].includes(checkHwType.class) ? checkHwType.class : undefined;
    }
    formatHwTypeName(hwType) {
        let hwTypeName = hwType.charAt(0).toUpperCase() + hwType.slice(1);
        ;
        if (!isNaN(+hwTypeName.charAt(0))) {
            hwTypeName = "_" + hwTypeName;
        }
        return hwTypeName;
    }
    formatTypeName(type) {
        return type.charAt(0).toUpperCase() + type.slice(1).replace(/-/g, '').replace(/_/g, '');
    }
    static addLeaf(menuItems, itemIds, leaf, found = 0) {
        for (const menuItem of menuItems) {
            if (itemIds.includes(menuItem.itemId)) {
                if (itemIds.length === (found - 1)) {
                    menuItem.children = leaf;
                    return menuItems;
                }
                else {
                    found++;
                    menuItem.children = this.addLeaf(menuItem.children, itemIds, leaf, found);
                }
            }
        }
    }
}
exports.MenuHandler = MenuHandler;
//# sourceMappingURL=MenuHandler.js.map