"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MenuService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const hostcfg_entity_1 = require("../../entity/hostcfg.entity");
const hwType_entity_1 = require("../../entity/hwType.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const storageGroup_entity_1 = require("../../entity/storageGroup.entity");
const subsystem_entity_1 = require("../../entity/subsystem.entity");
const Architecture_1 = require("../architecture/Architecture");
const typeorm_2 = require("typeorm");
const Menu = require("./menu/Menu");
const Menu_1 = require("./menu/Menu");
let MenuService = class MenuService {
    constructor(objectItemsRepository, hwTypesRepository, hostCfgRepository, subsystemsRepository, storageGroupsRepository) {
        this.objectItemsRepository = objectItemsRepository;
        this.hwTypesRepository = hwTypesRepository;
        this.hostCfgRepository = hostCfgRepository;
        this.subsystemsRepository = subsystemsRepository;
        this.storageGroupsRepository = storageGroupsRepository;
    }
    async getRoot(technology_class) {
        return technology_class ? await Architecture_1.Architecture.getHwTypes(false, technology_class) : await Architecture_1.Architecture.getHwTypes();
    }
    async checkHost(hw_type) {
        let check = await this.hostCfgRepository.findBy({ hw_type: hw_type });
        if (check.length > 0) {
            return true;
        }
        else {
            return false;
        }
    }
    async checkItem(hw_type) {
        let check = await this.objectItemsRepository.findOneBy({
            hw_type: hw_type,
            item_id: hw_type === 'linux'
                ? (0, typeorm_2.Not)((0, typeorm_2.In)(await Menu.getLinuxExcludedIds()))
                : undefined,
            subsystem: hw_type === 'linux' ? 'server' : undefined
        });
        if (check) {
            return true;
        }
        else {
            return false;
        }
    }
    sortMenuItems(items) {
        items.sort((a, b) => !a.isLeaf ? 1 : a.title.localeCompare(b.title, undefined, { numeric: true, sensitivity: 'base' }));
        items.forEach(item => {
            if (item.children) {
                item.children = this.sortMenuItems(item.children);
            }
        });
        return items;
    }
    async getObjectItems(hw_type, subsystem) {
        return await this.objectItemsRepository.findBy({ hw_type: hw_type, subsystem: subsystem });
    }
    async getDevices(_class = 'storage') {
        return await this.objectItemsRepository.find({
            relations: {
                hw_type_entity: true
            },
            where: {
                subsystem: 'device',
                hw_type_entity: {
                    class: _class
                }
            }
        });
    }
    async getSanRoot() {
        return await (new Menu_1.SanRootFolder('san')).getChildren();
    }
    async getLanRoot(request) {
        return await (new Menu_1.LanRootFolder('lan', "", request)).getChildren();
    }
    generatePagePath(technology, type) {
        return './page/' + technology + '/' + type + '.json';
    }
    async getHwTypeItems(hw_type, acl = false, request) {
        let hwTypeName = hw_type.charAt(0).toUpperCase() + hw_type.slice(1);
        let folder = Menu[hwTypeName + 'RootFolder']
            ? new Menu[hwTypeName + 'RootFolder']()
            : new Menu['StorRootFolder'](hwTypeName, request);
        let items;
        if (acl && typeof folder.getChildrenAcl === 'function') {
            items = await folder.getChildrenAcl();
        }
        else {
            items = await folder.getChildren();
        }
        return items;
    }
    async getSubsystemItems(hw_type, subsystem, ids, acl = false, request) {
        let hwTypeName = hw_type.charAt(0).toUpperCase() + hw_type.slice(1);
        let subsystemName = subsystem.charAt(0).toUpperCase() + subsystem.slice(1).replace(/-/g, '').replace(/_/g, '');
        let itemId = ids !== null && ids !== undefined ? ids.split(',').slice(-1) : null;
        let folder;
        try {
            if (Menu[hwTypeName + subsystemName + 'Folder']) {
                folder = new Menu[hwTypeName + subsystemName + 'Folder'](itemId);
            }
            else {
                const storageHwTypes = await Architecture_1.Architecture.getHwTypes(false, 'storage');
                if (storageHwTypes.map(hwType => hwType.hw_type).includes(hw_type)) {
                    if (subsystemName === 'Device') {
                        folder = new Menu['Stor' + subsystemName + 'Folder'](hw_type, subsystemName, itemId, request);
                    }
                    else {
                        folder = new Menu['StorSubsystemItemsFolder'](hw_type, subsystem, itemId);
                    }
                }
                const sanHwTypes = await Architecture_1.Architecture.getHwTypes(false, 'san');
                if (sanHwTypes.map(hwType => hwType.hw_type).includes(hw_type)) {
                    if (subsystemName === 'Device' || subsystemName === 'Fabric') {
                        folder = new Menu['San' + subsystemName + 'Folder'](hw_type, subsystemName, itemId);
                    }
                    else {
                        folder = new Menu['SanSubsystemItemsFolder'](hw_type, subsystem, itemId);
                    }
                }
            }
            if (!folder) {
                console.error("[Menu] getSubsystemItems error: folder definition not found (" + hwTypeName + " " + subsystemName + " " + itemId + ")");
            }
            let items;
            if (acl && typeof folder.getChildrenAcl === 'function') {
                items = await folder.getChildrenAcl();
            }
            else {
                items = await folder.getChildren();
            }
            return items;
        }
        catch (e) {
            console.error("[Menu] getSubsystemItems error (" + hwTypeName + " " + subsystemName + " " + itemId + "): " + e);
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.INTERNAL_SERVER_ERROR,
                message: 'get items error',
                error: 'get items error',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
    async getPage(hw_type, type, item_id, request) {
        let hwTypeName = hw_type.charAt(0).toUpperCase() + hw_type.slice(1);
        let typeName = type.charAt(0).toUpperCase() + type.slice(1).replace(/-/g, '');
        let itemId = item_id !== null ? item_id : null;
        let page;
        try {
            if (!isNaN(+hwTypeName.charAt(0)) && Menu['_' + hwTypeName + typeName + 'Page']) {
                page = new Menu['_' + hwTypeName + typeName + 'Page'](itemId, request);
            }
            else if (Menu[hwTypeName + typeName + 'Page']) {
                page = new Menu[hwTypeName + typeName + 'Page'](itemId, request);
            }
            else {
                const storageHwTypes = await Architecture_1.Architecture.getHwTypes(false, 'storage');
                if (storageHwTypes.map(hwType => hwType.hw_type).includes(hw_type) || ['sgroup', 'sgroups', 'storages', 'storage'].includes(hw_type)) {
                    if (Menu['Stor' + typeName + 'Page']) {
                        page = new Menu['Stor' + typeName + 'Page'](hwTypeName, type, itemId, request);
                    }
                    else {
                        page = new Menu['StorItemPage'](hwTypeName, type, itemId);
                    }
                }
                const sanHwTypes = await Architecture_1.Architecture.getHwTypes(false, 'san');
                if (sanHwTypes.map(hwType => hwType.hw_type).includes(hw_type)) {
                    if (Menu['San' + typeName + 'Page']) {
                        page = new Menu['San' + typeName + 'Page'](hwTypeName, type, itemId);
                    }
                    else {
                        page = new Menu['SanItemPage'](hwTypeName, type, itemId);
                    }
                }
            }
            let outPage = await page.getPage();
            return outPage;
        }
        catch (e) {
            console.error("[Menu] getPage error (" + hwTypeName + " " + typeName + " " + itemId + "): " + e);
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.INTERNAL_SERVER_ERROR,
                message: 'get page error',
                error: 'get page error',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
    async getPath(hw_type, type, item_id) {
        let hwTypeName = hw_type.charAt(0).toUpperCase() + hw_type.slice(1);
        let typeName = type.charAt(0).toUpperCase() + type.slice(1).replace(/-/g, '');
        let itemId = item_id !== null ? item_id : null;
        let page;
        try {
            if (Menu[hwTypeName + typeName + 'Page']) {
                page = new Menu[hwTypeName + typeName + 'Page'](itemId);
            }
            else {
                const storageHwTypes = await Architecture_1.Architecture.getHwTypes(false, 'storage');
                if (storageHwTypes.map(hwType => hwType.hw_type).includes(hw_type)) {
                    if (Menu['Stor' + typeName + 'Page']) {
                        page = new Menu['Stor' + typeName + 'Page'](hwTypeName, type, itemId);
                    }
                    else {
                        page = new Menu['StorItemPage'](hwTypeName, type, itemId);
                    }
                }
                const sanHwTypes = await Architecture_1.Architecture.getHwTypes(false, 'san');
                if (sanHwTypes.map(hwType => hwType.hw_type).includes(hw_type)) {
                    if (Menu['San' + typeName + 'Page']) {
                        page = new Menu['San' + typeName + 'Page'](hwTypeName, type, itemId);
                    }
                    else {
                        page = new Menu['SanItemPage'](hwTypeName, type, itemId);
                    }
                }
            }
            let items = await page.getPath();
            return items;
        }
        catch (e) {
            console.error("[Menu] getPath error (" + hwTypeName + " " + typeName + " " + itemId + "): " + e);
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.INTERNAL_SERVER_ERROR,
                message: 'get path error',
                error: 'get path error',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
};
exports.MenuService = MenuService;
exports.MenuService = MenuService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(1, (0, typeorm_1.InjectRepository)(hwType_entity_1.HwType)),
    __param(2, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __param(3, (0, typeorm_1.InjectRepository)(subsystem_entity_1.Subsystem)),
    __param(4, (0, typeorm_1.InjectRepository)(storageGroup_entity_1.StorageGroup)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], MenuService);
//# sourceMappingURL=menu.service.js.map