"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.As400 = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const MenuDefinitions_1 = require("./MenuDefinitions");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const pageResponse_constants_2 = require("../../../common/constants/pageResponse.constants");
const ItemPropertiesWrapper_1 = require("../../configuration/ItemPropertiesWrapper");
const class_constants_1 = require("../../../common/constants/class.constants");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
var As400;
(function (As400) {
    class Folder extends MenuDefinitions_1.DefaultFolder {
        constructor() {
            super(...arguments);
            this.hwType = 'as400';
        }
    }
    class RootFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            let servers = await this.objectItemsRepository.findBy({
                hw_type: this.hwType,
                subsystem: 'server',
            });
            servers.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let server of servers) {
                let path = this.hwType + '/server?item_id=' + server.item_id;
                menuItems.push(new menuResponse_dto_1.MenuLink(server.label, path, 'server', server.item_id));
            }
            return menuItems;
        }
    }
    As400.RootFolder = RootFolder;
    class As400Page extends MenuDefinitions_1.DefaultPage {
        constructor() {
            super(...arguments);
            this.hwType = 'as400';
        }
    }
    class ServerPage extends As400Page {
        constructor() {
            super(...arguments);
            this.subsystem = 'server';
        }
        async getPage() {
            let item = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: this.subsystem,
                item_id: this.itemId,
            });
            let title = 'Container ' + (item ? item.label : this.itemId);
            let tabs = [];
            let jobs = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'job',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            const jobname = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'jobname');
            const jobuser = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'job_user');
            const jobtype = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'jobtype');
            const jobfunction = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'jobfunction');
            const jobstatus = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'jobstatus');
            if (jobs.length >= 1) {
                tabs.push({
                    title: 'WRKACTJOB',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: "JOB CPU"
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.MultiGraphBox,
                            graphs: [
                                {
                                    metrics: [
                                        new As400Metrics.Metric('job', 'percent_used')
                                            .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1)
                                    ],
                                    uuids: jobs.map(job => job.item_id),
                                    aggregated: true,
                                    interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                                    label: 'PID',
                                    properties: [
                                        {
                                            label: "Jobname",
                                            values: jobname
                                        },
                                        {
                                            label: "User",
                                            values: jobuser
                                        },
                                        {
                                            label: "Type",
                                            values: jobtype
                                        },
                                        {
                                            label: "Function",
                                            values: jobfunction
                                        },
                                        {
                                            label: "Status",
                                            values: jobstatus
                                        }
                                    ]
                                },
                                {
                                    metrics: [
                                        new As400Metrics.Metric('job', 'percent_used')
                                            .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1)
                                    ],
                                    uuids: jobs.map(job => job.item_id),
                                    aggregated: true,
                                    interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                                    label: 'PID',
                                    properties: [
                                        {
                                            label: "Jobname",
                                            values: jobname
                                        },
                                        {
                                            label: "User",
                                            values: jobuser
                                        },
                                        {
                                            label: "Type",
                                            values: jobtype
                                        },
                                        {
                                            label: "Function",
                                            values: jobfunction
                                        },
                                        {
                                            label: "Status",
                                            values: jobstatus
                                        }
                                    ]
                                },
                            ],
                        },
                    ],
                });
                tabs.push({
                    title: 'CPUTOP',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.MultiGraphBox,
                            graphs: [
                                {
                                    metrics: [
                                        new As400Metrics.Metric('job', 'proctime_total')
                                            .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1)
                                    ],
                                    uuids: jobs.map(job => job.item_id),
                                    aggregated: true,
                                    interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                                    label: 'PID',
                                    properties: [
                                        {
                                            label: "Jobname",
                                            values: jobname
                                        },
                                        {
                                            label: "User",
                                            values: jobuser
                                        },
                                        {
                                            label: "Type",
                                            values: jobtype
                                        },
                                        {
                                            label: "Function",
                                            values: jobfunction
                                        },
                                        {
                                            label: "Status",
                                            values: jobstatus
                                        }
                                    ]
                                },
                                {
                                    metrics: [
                                        new As400Metrics.Metric('job', 'proctime_total')
                                            .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1)
                                    ],
                                    uuids: jobs.map(job => job.item_id),
                                    aggregated: true,
                                    interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                                    label: 'PID',
                                    properties: [
                                        {
                                            label: "Jobname",
                                            values: jobname
                                        },
                                        {
                                            label: "User",
                                            values: jobuser
                                        },
                                        {
                                            label: "Type",
                                            values: jobtype
                                        },
                                        {
                                            label: "Function",
                                            values: jobfunction
                                        },
                                        {
                                            label: "Status",
                                            values: jobstatus
                                        }
                                    ]
                                },
                            ],
                        },
                    ],
                });
                tabs.push({
                    title: 'IOTOP',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.MultiGraphBox,
                            graphs: [
                                {
                                    metrics: [
                                        new As400Metrics.Metric('job', 'diskIO_total')
                                            .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1)
                                    ],
                                    uuids: jobs.map(job => job.item_id),
                                    aggregated: true,
                                    interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                                    label: 'PID',
                                    properties: [
                                        {
                                            label: "Jobname",
                                            values: jobname
                                        },
                                        {
                                            label: "User",
                                            values: jobuser
                                        },
                                        {
                                            label: "Type",
                                            values: jobtype
                                        },
                                        {
                                            label: "Function",
                                            values: jobfunction
                                        },
                                        {
                                            label: "Status",
                                            values: jobstatus
                                        }
                                    ]
                                },
                                {
                                    metrics: [
                                        new As400Metrics.Metric('job', 'diskIO_total')
                                            .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1)
                                    ],
                                    uuids: jobs.map(job => job.item_id),
                                    aggregated: true,
                                    interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                                    label: 'PID',
                                    properties: [
                                        {
                                            label: "Jobname",
                                            values: jobname
                                        },
                                        {
                                            label: "User",
                                            values: jobuser
                                        },
                                        {
                                            label: "Type",
                                            values: jobtype
                                        },
                                        {
                                            label: "Function",
                                            values: jobfunction
                                        },
                                        {
                                            label: "Status",
                                            values: jobstatus
                                        }
                                    ]
                                },
                            ],
                        },
                    ],
                });
            }
            let asps = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'asp',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            const diskserial = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(asps.map(asp => asp.item_id), 'disk_serial');
            if (asps.length >= 1) {
                tabs.push({
                    title: 'DISKTOP',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: "Disk Busy"
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('asp', 'disk_busy')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: asps.map(asp => asp.item_id),
                            aggregated: true,
                            label: 'ASP',
                            properties: [
                                {
                                    label: "Disk serial",
                                    values: diskserial
                                }
                            ]
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: "Disk IO Aggregated"
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('asp', 'disk_io_aggreg')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: asps.map(asp => asp.item_id),
                            aggregated: true,
                            label: 'ASP',
                            properties: [
                                {
                                    label: "Disk serial",
                                    values: diskserial
                                }
                            ]
                        },
                    ],
                });
            }
            tabs.push({
                title: 'Jobs',
                content: [
                    {
                        type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new As400Metrics.Metric('server', 'active_threads_in_system')
                                .getGraph(pageResponse_constants_2.CHART_TYPE.line, false, 0),
                            new As400Metrics.Metric('server', 'jobs_in_system')
                                .getGraph(pageResponse_constants_2.CHART_TYPE.line, false, 0),
                            new As400Metrics.Metric('server', 'active_jobs_in_system')
                                .getGraph(pageResponse_constants_2.CHART_TYPE.line, false, 0),
                        ],
                        uuids: [this.itemId],
                        aggregated: false,
                    },
                ],
            });
            let spools = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'shrpool',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            const pagingoption = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(spools.map(spool => spool.item_id), 'paging_option');
            if (spools.length >= 1) {
                tabs.push({
                    title: 'Pool Size',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('shrpool', 'size')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: spools.map(spool => spool.item_id),
                            aggregated: true,
                            label: 'Name',
                            properties: [
                                {
                                    label: "type",
                                    values: pagingoption
                                }
                            ]
                        },
                    ],
                });
                tabs.push({
                    title: 'Threads',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('shrpool', 'threads')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: spools.map(spool => spool.item_id),
                            aggregated: true,
                        },
                    ],
                });
                tabs.push({
                    title: 'Faults',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('shrpool', 'database_faults')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                                new As400Metrics.Metric('shrpool', 'non_database_faults')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: spools.map(spool => spool.item_id),
                            aggregated: true,
                        },
                    ],
                });
                tabs.push({
                    title: 'Pages',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('shrpool', 'database_pages')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                                new As400Metrics.Metric('shrpool', 'non_database_pages')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: spools.map(spool => spool.item_id),
                            aggregated: true,
                        },
                    ],
                });
            }
            if (asps.length >= 1) {
                tabs.push({
                    title: 'ASP Used',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('asp', 'mem_alloc')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: asps.map(asp => asp.item_id),
                            aggregated: true,
                        },
                    ],
                });
                tabs.push({
                    title: 'ASP Free',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('asp', 'mem_free')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: asps.map(asp => asp.item_id),
                            aggregated: true,
                        },
                    ],
                });
                tabs.push({
                    title: 'ASP Data',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('asp', 'data_to_memory')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, -1),
                                new As400Metrics.Metric('asp', 'data_from_memory')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: asps.map(asp => asp.item_id),
                            aggregated: true,
                        },
                    ],
                });
                tabs.push({
                    title: 'ASP IO',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('asp', 'io_to_memory')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, -1),
                                new As400Metrics.Metric('asp', 'io_from_memory')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1),
                            ],
                            uuids: asps.map(asp => asp.item_id),
                            aggregated: true,
                        },
                    ],
                });
                tabs.push({
                    title: 'ASP Latency',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('asp', 'latency_time')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.line, false, 1),
                            ],
                            uuids: asps.map(asp => asp.item_id),
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: "DSK SERVICE"
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('asp', 'svc_time')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.line, false, 1),
                            ],
                            uuids: asps.map(asp => asp.item_id),
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: "DSK WAIT"
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('asp', 'wait_time')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.line, false, 1),
                            ],
                            uuids: asps.map(asp => asp.item_id),
                            aggregated: true,
                        },
                    ],
                });
            }
            let lans = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'lan',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            if (lans.length >= 1) {
                tabs.push({
                    title: 'LAN',
                    content: [
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('lan', 'total_bytes_recv')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, -1),
                                new As400Metrics.Metric('lan', 'total_bytes_sent')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1)
                            ],
                            uuids: lans.map(lan => lan.item_id),
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: "Packets IFCB"
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('lan', 'total_packets_recv')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, -1),
                                new As400Metrics.Metric('lan', 'total_packets_sent')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1)
                            ],
                            uuids: lans.map(lan => lan.item_id),
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: "Packets Discarded IFCB"
                        },
                        {
                            type: pageResponse_constants_2.BOX_TYPE.FourGraphBox,
                            metrics: [
                                new As400Metrics.Metric('lan', 'total_inbound_packets_discarded')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, -1),
                                new As400Metrics.Metric('lan', 'total_outbound_packets_discarded')
                                    .getAggrGraph(pageResponse_constants_2.CHART_TYPE.area, true, 1)
                            ],
                            uuids: lans.map(lan => lan.item_id),
                            aggregated: true,
                        },
                    ],
                });
            }
            return new pageResponse_dto_1.Page(title, tabs);
        }
    }
    As400.ServerPage = ServerPage;
    class As400Metrics {
    }
    As400Metrics.Metric = class extends pageResponse_dto_1.Metric {
        constructor(subsystem, metric) {
            super();
            this.metric = metric;
            const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.AS400, subsystem, metric);
            if (definition) {
                Object.assign(this, definition);
            }
        }
        getGraph(type, stacked, order, color) {
            this.type = type;
            this.stacked = stacked ? stacked : false;
            this.order = order ? order : 0;
            this.color = color ? color : undefined;
            return this;
        }
        getAggrGraph(type, stacked, order, aggregate) {
            this.type = type;
            this.stacked = stacked ? stacked : false;
            this.order = order ? order : 0;
            this.aggregate = aggregate ? aggregate : pageResponse_constants_2.METRIC_AGGREGATE.items;
            return this;
        }
    };
    As400.As400Metrics = As400Metrics;
})(As400 || (exports.As400 = As400 = {}));
//# sourceMappingURL=As400.js.map