"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AwsMetrics = exports.AwsConfigurationPage = exports.AwsTopPage = exports.AwsHeatmapPage = exports.AwsLambdatotalPage = exports.AwsEbstotalPage = exports.AwsEc2totalPage = exports.AwsLambdaPage = exports.AwsEbsPage = exports.AwsEc2Page = exports.AwsItemPage = exports.AwsPage = exports.AwsLambdaFolder = exports.AwsEbsFolder = exports.AwsEc2Folder = exports.AwsRegionFolder = exports.AwsRootFolder = exports.AwsSubsystemFolder = exports.AwsFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const Heatmap_1 = require("./Heatmap");
const Table_1 = require("./Table");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const ACLCheck_1 = require("../../acl/ACLCheck");
const URL_1 = require("./URL");
const Top_1 = require("./Top");
const ConfigurationTable_1 = require("./ConfigurationTable");
const AbstractMenu_1 = require("./AbstractMenu");
const Color_1 = require("../../metrics/Color");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
class AwsFolder extends AbstractMenu_1.AbstractFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'aws';
    }
}
exports.AwsFolder = AwsFolder;
class AwsSubsystemFolder extends AwsFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.AwsSubsystemFolder = AwsSubsystemFolder;
class AwsRootFolder extends AwsFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration', 'configuration'));
        menuItems.push(new menuResponse_dto_1.MenuLink('VM TOP', this.hwType + '/top', 'top'));
        let regions = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'region',
            }, relations: {
                children: true
            }
        });
        for (let region of regions) {
            if (region.children.length >= 1) {
                let path = this.hwType + '/region?ids=' + region.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(region.label, path, this.hwType, 'region', region.item_id, AwsRegionFolder));
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.AwsRootFolder = AwsRootFolder;
class AwsRegionFolder extends AwsSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        const subsystems = [{
                "label": "Elastic Compute Cloud",
                "subsystem": "ec2",
                "total": "EC2 Totals",
                "folder": AwsEc2Folder
            }, {
                "label": "Elastic Block Store",
                "subsystem": "ebs",
                "total": "EBS Totals",
                "folder": AwsEbsFolder
            }, {
                "label": "Lambda",
                "subsystem": "lambda",
                "total": "Lambda Totals",
                "folder": AwsLambdaFolder
            }];
        const folders = [];
        for (const subsystem of subsystems) {
            if (this.objectItemsRepository.findOneBy({
                hw_type: hwTypeKey_constants_1.HwTypeKey.AWS,
                subsystem: subsystem.subsystem
            })) {
                menuItems.push(new menuResponse_dto_1.MenuLink(subsystem.total, this.hwType + '/' + subsystem.subsystem + '-total?item_id=' + this.itemId, 'ec2'));
                let path = this.hwType + '/' + subsystem.subsystem + '?ids=' + this.itemId.toString();
                folders.push(new menuResponse_dto_1.MenuFolder(subsystem.label, path, this.hwType, subsystem.subsystem, this.itemId.toString(), subsystem.folder));
            }
        }
        menuItems = menuItems.concat(folders);
        return menuItems;
    }
}
exports.AwsRegionFolder = AwsRegionFolder;
class AwsEc2Folder extends AwsSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let children = await this.itemRelationsRepository.find({
            relations: {
                _child: true
            },
            where: {
                parent: this.itemId.toString(),
                _child: {
                    subsystem: 'ec2'
                }
            }
        });
        const items = [];
        for (const child of children) {
            items.push(child._child);
        }
        for (let item of items) {
            let path = this.hwType + '/ec2?item_id=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, 'ec2', item.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.AwsEc2Folder = AwsEc2Folder;
class AwsEbsFolder extends AwsSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let children = await this.itemRelationsRepository.find({
            relations: {
                _child: true
            },
            where: {
                parent: this.itemId.toString(),
                _child: {
                    subsystem: 'ebs'
                }
            }
        });
        const items = [];
        for (const child of children) {
            items.push(child._child);
        }
        for (let item of items) {
            let path = this.hwType + '/ebs?item_id=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, 'ebs', item.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.AwsEbsFolder = AwsEbsFolder;
class AwsLambdaFolder extends AwsSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let children = await this.itemRelationsRepository.find({
            relations: {
                _child: true
            },
            where: {
                parent: this.itemId.toString(),
                _child: {
                    subsystem: 'lambda'
                }
            }
        });
        const items = [];
        for (const child of children) {
            items.push(child._child);
        }
        for (let item of items) {
            let path = this.hwType + '/lambda?item_id=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, 'lambda', item.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.AwsLambdaFolder = AwsLambdaFolder;
class AwsPage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.hwType = 'aws';
        this.rootFolder = AwsRootFolder;
    }
}
exports.AwsPage = AwsPage;
class AwsItemPage extends AwsPage {
}
exports.AwsItemPage = AwsItemPage;
class AwsEc2Page extends AwsItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'ec2';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'EC2 ' + (item ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU%',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new AwsMetrics.Metric('ec2', 'CPUUtilization').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ec2', 'DiskReadBytes').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.metric.read),
                        new AwsMetrics.Metric('ec2', 'DiskWriteBytes').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.metric.write),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ec2', 'DiskReadOps').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.metric.read),
                        new AwsMetrics.Metric('ec2', 'DiskWriteOps').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.metric.write),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ec2', 'NetworkIn').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.metric.read),
                        new AwsMetrics.Metric('ec2', 'NetworkOut').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.metric.write),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AwsEc2Page = AwsEc2Page;
class AwsEbsPage extends AwsItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'ebs';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'EBS ' + (item ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ebs', 'VolumeReadBytes').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.metric.read),
                        new AwsMetrics.Metric('ebs', 'VolumeWriteBytes').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.metric.write),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ebs', 'VolumeReadOps').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.metric.read),
                        new AwsMetrics.Metric('ebs', 'VolumeWriteOps').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.metric.write),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AwsEbsPage = AwsEbsPage;
class AwsLambdaPage extends AwsItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'lambda';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Lambda ' + (item ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'Invocations',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'Invocations').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Errors',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'Errors').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Throttles',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'Throttles').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Duration',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'Duration').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'ConcurrentExecutions',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'ConcurrentExecutions').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AwsLambdaPage = AwsLambdaPage;
class AwsEc2totalPage extends AwsItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'ec2';
        this.withItemId = true;
    }
    async getPage() {
        let items = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: this.subsystem,
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        let title = 'EC2 Total';
        let tabs = [];
        tabs.push({
            title: 'CPU%',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [new AwsMetrics.Metric('ec2', 'CPUUtilization').getGraph(pageResponse_constants_1.CHART_TYPE.line, false)],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ec2', 'DiskReadBytes').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new AwsMetrics.Metric('ec2', 'DiskWriteBytes').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ec2', 'DiskReadOps').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new AwsMetrics.Metric('ec2', 'DiskWriteOps').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ec2', 'NetworkIn').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new AwsMetrics.Metric('ec2', 'NetworkOut').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AwsEc2totalPage = AwsEc2totalPage;
class AwsEbstotalPage extends AwsItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'ebs';
        this.withItemId = true;
    }
    async getPage() {
        let items = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: this.subsystem,
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        let title = 'EBS Total';
        let tabs = [];
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ebs', 'VolumeReadBytes').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new AwsMetrics.Metric('ebs', 'VolumeWriteBytes').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('ebs', 'VolumeReadOps').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1),
                        new AwsMetrics.Metric('ebs', 'VolumeWriteOps').getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AwsEbstotalPage = AwsEbstotalPage;
class AwsLambdatotalPage extends AwsItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'lambda';
        this.withItemId = true;
    }
    async getPage() {
        let items = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: this.subsystem,
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        let title = 'Lambda Total';
        let tabs = [];
        tabs.push({
            title: 'Invocations',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'Invocations').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Errors',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'Errors').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Throttles',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'Throttles').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Duration',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'Duration').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'ConcurrentExecutions',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new AwsMetrics.Metric('lambda', 'ConcurrentExecutions').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0),
                    ],
                    uuids: items.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AwsLambdatotalPage = AwsLambdatotalPage;
class AwsHeatmapPage extends AwsPage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        tabs.push({
            title: 'EC2',
            content: Object.values(await new Heatmap_1.Heatmap('aws', 'ec2', true)
                .computedHeatmapBox(class_constants_1.ClassKey.CLOUD, this.request)),
        });
        let table = new Table_1.Table()
            .addHeader({ value: 'Region' })
            .addHeader({ value: 'Elastic Compute Cloud' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let ec2s = await this.objectItemsRepository.find({
            where: {
                hw_type: 'aws',
                subsystem: 'ec2'
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        ec2s = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, ec2s);
        for (const ec2 of ec2s) {
            const region = ec2.parents.find(parent => parent._parent.subsystem === 'region');
            const cpu = ec2.heatmaps.find(heatmap => heatmap.metric === 'CPUUtilization');
            table.addRow([
                { order: 0, value: region ? region._parent.label : 'undefined' },
                { order: 1, value: ec2.label, url: URL_1.URL.generate('aws', 'ec2', ec2.item_id) },
                { order: 2, value: cpu ? cpu.value : null }
            ]);
        }
        table.sortByIndex(2);
        tabs.push({
            title: 'Table',
            content: [table.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AwsHeatmapPage = AwsHeatmapPage;
class AwsTopPage extends AwsPage {
    constructor() {
        super(...arguments);
        this.title = 'EC2 TOP';
    }
    async getPage() {
        const top = new Top_1.Top();
        top.setSubsystem('aws', 'ec2');
        top.setParentSubsystem('region');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.AwsTopPage = AwsTopPage;
class AwsConfigurationPage extends AwsPage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let title = 'Configuration';
        let tabs = [];
        let ec2s = await this.objectItemsRepository.find({
            where: {
                hw_type: 'aws',
                subsystem: 'ec2'
            }
        });
        const configurationTableEC2 = new ConfigurationTable_1.ConfigurationTable(ec2s.map(ec2 => ec2.item_id), ['PrivateIpAddress', 'PublicDnsName', 'InstanceType', 'VirtualizationType', 'LaunchTime'], ['region']);
        configurationTableEC2.setItemLabel("Elastic Compute Cloud");
        configurationTableEC2.setAcl(this.request);
        tabs.push({
            title: 'EC2',
            content: [await configurationTableEC2.generate()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AwsConfigurationPage = AwsConfigurationPage;
class AwsMetrics {
}
exports.AwsMetrics = AwsMetrics;
AwsMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(subsystem, metric) {
        super();
        this.metric = metric;
        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.CLOUD, hwTypeKey_constants_1.HwTypeKey.AWS, subsystem, metric);
        if (definition) {
            this.unit = definition.unit;
            this.shortcut = definition.shortcut;
            this.label = definition.label;
            this.defaultPrecision = definition.defaultPrecision ? definition.defaultPrecision : undefined;
            this.defaultPrefix = definition.defaultPrefix ? definition.defaultPrefix : undefined;
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
    }
};
//# sourceMappingURL=Aws.js.map