"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CloudstackPrimarystoragePage = exports.CloudstackVolumePage = exports.CloudstackInstancePage = exports.CloudstackHostPage = exports.CloudstackSingleItemPage = exports.CloudstackPrimarystoragetotalsPage = exports.CloudstackVolumetotalsPage = exports.CloudstackInstancetotalsPage = exports.CloudstackHosttotalsPage = exports.CloudstackCloudTotalsPage = exports.CloudstackConfigurationPage = exports.CloudstackPage = exports.CloudstackMetrics = exports.CloudstackPrimarystorageFolder = exports.CloudstackVolumeFolder = exports.CloudstackInstanceFolder = exports.CloudstackHostFolder = exports.CloudstackChildSubsystemFolder = exports.CloudstackCloudFolder = exports.CloudstackRootFolder = exports.CloudstackSubsystemFolder = exports.CloudstackFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
class CloudstackFolder {
    constructor() {
        this.hwType = 'cloudstack';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
}
exports.CloudstackFolder = CloudstackFolder;
class CloudstackSubsystemFolder extends CloudstackFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.CloudstackSubsystemFolder = CloudstackSubsystemFolder;
class CloudstackRootFolder extends CloudstackFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Alerts', this.hwType + '/alerts'));
        let subsystem_cloud = 'cloud';
        let clouds = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: subsystem_cloud });
        for (var item of clouds) {
            let path = this.hwType + '/' + subsystem_cloud + '?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType, subsystem_cloud, item.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.CloudstackRootFolder = CloudstackRootFolder;
class CloudstackCloudFolder extends CloudstackSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'cloud';
    }
    async getChildren() {
        let menuItems = [];
        let subsystems = ['Host', 'Instance', 'Volume', 'Primary Storage'];
        for (let childSubsystem of subsystems) {
            menuItems.push(new menuResponse_dto_1.MenuLink(childSubsystem + ' Totals', this.hwType + '/' + childSubsystem.toLowerCase() + '-totals' + '?item_id=' + this.itemId, childSubsystem.toLowerCase() + '-totals'));
        }
        for (let childSubsystem of subsystems) {
            let childSubsystemItems = await this.objectItemsRepository.findBy({
                hw_type: this.hwType,
                subsystem: childSubsystem,
            });
            if (childSubsystemItems !== null) {
                let path = this.hwType + '/' + childSubsystem.toLowerCase() + (this.itemId !== null ? '?ids=' + this.itemId : '');
                menuItems.push(new menuResponse_dto_1.MenuFolder(childSubsystem, path, this.hwType, childSubsystem.toLowerCase()));
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.CloudstackCloudFolder = CloudstackCloudFolder;
class CloudstackChildSubsystemFolder extends CloudstackSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let items = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: this.subsystem });
        if (this.itemId) {
            let parentChildren = await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } });
            let childrenIds = parentChildren.map((relation) => relation.child);
            for (let item of items) {
                if (childrenIds.includes(item.item_id)) {
                    let path = this.hwType + '/' + this.subsystem + '?item_id=' + item.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, this.subsystem, item.item_id));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.CloudstackChildSubsystemFolder = CloudstackChildSubsystemFolder;
class CloudstackHostFolder extends CloudstackChildSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'host';
    }
}
exports.CloudstackHostFolder = CloudstackHostFolder;
class CloudstackInstanceFolder extends CloudstackChildSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'instance';
    }
}
exports.CloudstackInstanceFolder = CloudstackInstanceFolder;
class CloudstackVolumeFolder extends CloudstackChildSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'volume';
    }
}
exports.CloudstackVolumeFolder = CloudstackVolumeFolder;
class CloudstackPrimarystorageFolder extends CloudstackChildSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'primarystorage';
    }
}
exports.CloudstackPrimarystorageFolder = CloudstackPrimarystorageFolder;
class CloudstackMetrics {
}
exports.CloudstackMetrics = CloudstackMetrics;
CloudstackMetrics.InstanceCpuMetric = class extends pageResponse_dto_1.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpuused';
        this.order = 0;
    }
};
CloudstackMetrics.InstanceFreememMetric = class extends pageResponse_dto_1.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'memoryintfreekbs';
        this.order = 0;
    }
};
CloudstackMetrics.InstanceMaxmemMetric = class extends pageResponse_dto_1.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'memory';
        this.order = 1;
    }
};
class CloudstackPage {
    constructor() {
        this.hwType = 'cloudstack';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
    async getPath() {
        let root = new CloudstackRootFolder();
        let menuItems = await root.getChildren();
        return menuItems;
    }
}
exports.CloudstackPage = CloudstackPage;
class CloudstackConfigurationPage extends CloudstackPage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let title = 'Configuration';
        let tabs = [];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.CloudstackConfigurationPage = CloudstackConfigurationPage;
class CloudstackCloudTotalsPage extends CloudstackPage {
    constructor(itemId) {
        super();
        this.subsystem = 'cloud';
        this.itemId = itemId;
    }
    async getPath() {
        let menuItems = await super.getPath();
        for (let folder of menuItems) {
            if (folder.hasOwnProperty('subsystem') && folder.subsystem == this.subsystem && folder.itemId == this.itemId) {
                let subfolder = new CloudstackCloudFolder(this.itemId);
                folder.children = await subfolder.getChildren();
                break;
            }
        }
        return menuItems;
    }
}
exports.CloudstackCloudTotalsPage = CloudstackCloudTotalsPage;
class CloudstackHosttotalsPage extends CloudstackCloudTotalsPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Host Totals ' + (item !== null ? item.label : this.itemId);
        let child_hosts = [];
        let hosts = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: 'host' });
        for (let host of hosts) {
            child_hosts.push(host.item_id);
        }
        let items = [];
        let itemRelations = await this.itemRelationsRepository.findBy({ parent: this.itemId, child: (0, typeorm_1.In)(child_hosts) });
        for (let relation of itemRelations) {
            items.push(relation.child);
        }
        let tabs = [];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.CloudstackHosttotalsPage = CloudstackHosttotalsPage;
class CloudstackInstancetotalsPage extends CloudstackCloudTotalsPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Instance Totals ' + (item !== null ? item.label : this.itemId);
        let child_instances = [];
        let instances = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: 'instance' });
        for (let instance of instances) {
            child_instances.push(instance.item_id);
        }
        let items = [];
        let itemRelations = await this.itemRelationsRepository.findBy({ parent: this.itemId, child: (0, typeorm_1.In)(child_instances) });
        for (let relation of itemRelations) {
            items.push(relation.child);
        }
        let tabs = [];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.CloudstackInstancetotalsPage = CloudstackInstancetotalsPage;
class CloudstackVolumetotalsPage extends CloudstackCloudTotalsPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Volume Totals ' + (item !== null ? item.label : this.itemId);
        let child_volumes = [];
        let volumes = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: 'volume' });
        for (let volume of volumes) {
            child_volumes.push(volume.item_id);
        }
        let items = [];
        let itemRelations = await this.itemRelationsRepository.findBy({ parent: this.itemId, child: (0, typeorm_1.In)(child_volumes) });
        for (let relation of itemRelations) {
            items.push(relation.child);
        }
        let tabs = [];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.CloudstackVolumetotalsPage = CloudstackVolumetotalsPage;
class CloudstackPrimarystoragetotalsPage extends CloudstackCloudTotalsPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Primary Storage Totals ' + (item !== null ? item.label : this.itemId);
        let child_primarystorages = [];
        let primarystorages = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: 'primarystorage' });
        for (var primarystorage of primarystorages) {
            child_primarystorages.push(primarystorage.item_id);
        }
        let items = [];
        let itemRelations = await this.itemRelationsRepository.findBy({ parent: this.itemId, child: (0, typeorm_1.In)(child_primarystorages) });
        for (var relation of itemRelations) {
            items.push(relation.child);
        }
        let tabs = [];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.CloudstackPrimarystoragetotalsPage = CloudstackPrimarystoragetotalsPage;
class CloudstackSingleItemPage extends CloudstackPage {
    constructor(itemId) {
        super();
        this.parentSubsystem = 'cloud';
        this.itemId = itemId;
    }
    async getPath() {
        let parent = await (0, dataSource_1.getDataSource)().createQueryBuilder()
            .select('ir.parent', 'parent')
            .from('ItemRelation', 'ir')
            .innerJoin('ObjectItem', 'oi', 'ir.parent = oi.item_id')
            .where('ir.child = :child', { child: this.itemId })
            .andWhere('oi.hw_type = :hw_type', { hw_type: this.hwType })
            .andWhere('oi.subsystem = :subsystem', { subsystem: this.parentSubsystem })
            .getRawOne();
        this.parentId = parent['parent'];
        let menuItems = await super.getPath();
        for (var item of menuItems) {
            if (item.hasOwnProperty('subsystem') && item.subsystem == this.parentSubsystem && item.itemId == this.parentId) {
                let subfolder = new CloudstackCloudFolder(this.parentId);
                item.children = await subfolder.getChildren();
                break;
            }
        }
        return menuItems;
    }
}
exports.CloudstackSingleItemPage = CloudstackSingleItemPage;
class CloudstackHostPage extends CloudstackSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'host';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Host ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        for (var item of menuItems) {
            if (item.hasOwnProperty('subsystem') && item.subsystem == this.parentSubsystem && item.itemId == this.parentId) {
                for (var subitem of item.children) {
                    if (subitem.hasOwnProperty('subsystem') && subitem.subsystem == this.subsystem) {
                        let subfolder = new CloudstackHostFolder();
                        subitem.children = await subfolder.getChildren();
                        break;
                    }
                }
            }
        }
        return menuItems;
    }
}
exports.CloudstackHostPage = CloudstackHostPage;
class CloudstackInstancePage extends CloudstackSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'instance';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Instance ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        for (var item of menuItems) {
            if (item.hasOwnProperty('subsystem') && item.subsystem == this.parentSubsystem && item.itemId == this.parentId) {
                for (var subitem of item.children) {
                    if (subitem.hasOwnProperty('subsystem') && subitem.subsystem == this.subsystem) {
                        let subfolder = new CloudstackInstanceFolder();
                        subitem.children = await subfolder.getChildren();
                        break;
                    }
                }
            }
        }
        return menuItems;
    }
}
exports.CloudstackInstancePage = CloudstackInstancePage;
class CloudstackVolumePage extends CloudstackSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'volume';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Volume ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        for (var item of menuItems) {
            if (item.hasOwnProperty('subsystem') && item.subsystem == this.parentSubsystem && item.itemId == this.parentId) {
                for (var subitem of item.children) {
                    if (subitem.hasOwnProperty('subsystem') && subitem.subsystem == this.subsystem) {
                        let subfolder = new CloudstackVolumeFolder();
                        subitem.children = await subfolder.getChildren();
                        break;
                    }
                }
            }
        }
        return menuItems;
    }
}
exports.CloudstackVolumePage = CloudstackVolumePage;
class CloudstackPrimarystoragePage extends CloudstackSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'primarystorage';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Primary Storage ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
    async getPath() {
        let menuItems = await super.getPath();
        for (var item of menuItems) {
            if (item.hasOwnProperty('subsystem') && item.subsystem == this.parentSubsystem && item.itemId == this.parentId) {
                for (var subitem of item.children) {
                    if (subitem.hasOwnProperty('subsystem') && subitem.subsystem == this.subsystem) {
                        let subfolder = new CloudstackPrimarystorageFolder();
                        subitem.children = await subfolder.getChildren();
                        break;
                    }
                }
            }
        }
        return menuItems;
    }
}
exports.CloudstackPrimarystoragePage = CloudstackPrimarystoragePage;
//# sourceMappingURL=Cloudstack.js.map