"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ddnintelliflash = void 0;
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const MenuDefinitions_1 = require("./MenuDefinitions");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const Storage_1 = require("../../metrics/class/Storage");
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const Stor_1 = require("./Stor");
const ConfigurationTable_1 = require("./ConfigurationTable");
const HealthStatus_1 = require("./HealthStatus");
const healthStatus_entity_1 = require("../../../entity/healthStatus.entity");
const Table_1 = require("./Table");
const typeorm_1 = require("typeorm");
const healthStatus_constants_1 = require("../../../common/constants/healthStatus.constants");
const User_1 = require("../../users/User");
const CapacityTableBuilder_1 = require("./CapacityTableBuilder");
const ACLCheck_1 = require("../../acl/ACLCheck");
var Ddnintelliflash;
(function (Ddnintelliflash) {
    class Folder extends MenuDefinitions_1.DefaultFolder {
        constructor() {
            super(...arguments);
            this.hwType = 'ddnintelliflash';
        }
    }
    class DeviceFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            menuItems.push(new menuResponse_dto_1.MenuLink('Total', this.hwType + '/device?item_id=' + this.itemId, 'device', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration?item_id=' + this.itemId, 'configuration', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Capacity', this.hwType + '/capacity?item_id=' + this.itemId, 'capacity', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Health Status', this.hwType + '/health-status?item_id=' + this.itemId, 'health-status', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuFolder('Node', this.hwType + '/node?ids=' + this.itemId, this.hwType, 'node', this.itemId, NodeFolder));
            menuItems.push(new menuResponse_dto_1.MenuFolder('SFA', this.hwType + '/sfa-systems?ids=' + this.itemId, this.hwType, 'sfa-systems', this.itemId, SfasystemsFolder));
            menuItems.push(new menuResponse_dto_1.MenuFolder('EXAScaler', this.hwType + '/exascalers?ids=' + this.itemId, this.hwType, 'exascalers', this.itemId, ExascalersFolder));
            return menuItems;
        }
    }
    Ddnintelliflash.DeviceFolder = DeviceFolder;
    class DeviceFolderRoot extends Folder {
        async getChildren() {
            let menuItems = [];
            let root_subsystem = 'device';
            let devices = await this.objectItemsRepository.findBy({
                hw_type: this.hwType,
                subsystem: root_subsystem,
            });
            if (this.request) {
                const user = await User_1.User.getUserByRequest(this.request, true);
                const ids = await ACLCheck_1.ACLCheck.filterItems(user.groups.map(group => group.group_id), devices.map(device => device.item_id));
                devices = devices.filter(device => ids.includes(device.item_id));
            }
            devices.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let item of devices) {
                let path = this.hwType.toLowerCase() + '/device?ids=' + item.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType.toLowerCase(), 'device', item.item_id, DeviceFolder));
            }
            return menuItems;
        }
    }
    Ddnintelliflash.DeviceFolderRoot = DeviceFolderRoot;
    class SubsystemItemsFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            const items = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                },
                where: {
                    parents: {
                        parent: this.itemId.toString()
                    },
                    subsystem: this.subsystem,
                    hw_type: this.hwType
                }
            });
            items.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let item of items) {
                menuItems.push(new menuResponse_dto_1.MenuLink(item.label, `${this.hwType}/${this.subsystem}?item_id=${item.item_id}`, this.subsystem, item.item_id));
            }
            return menuItems;
        }
    }
    Ddnintelliflash.SubsystemItemsFolder = SubsystemItemsFolder;
    class NodeitemsFolder extends SubsystemItemsFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'node';
        }
    }
    Ddnintelliflash.NodeitemsFolder = NodeitemsFolder;
    class NodeFolder extends Folder {
        constructor() {
            super(...arguments);
            this.subsystem = 'node';
        }
        async getChildren() {
            let menuItems = [];
            menuItems.push(new menuResponse_dto_1.MenuLink('Capacity', `${this.hwType}/${this.subsystem}-capacity?item_id=${this.itemId}`, 'capacity', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('CPU', `${this.hwType}/${this.subsystem}-cpu?item_id=${this.itemId}`, 'cpu', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Memory', `${this.hwType}/${this.subsystem}-memory?item_id=${this.itemId}`, 'memory', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuFolder('Items', this.hwType + `/${this.subsystem}-items?ids=` + this.itemId, this.hwType, this.subsystem, this.itemId, NodeitemsFolder));
            return menuItems;
        }
    }
    Ddnintelliflash.NodeFolder = NodeFolder;
    class SfasystemsFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            menuItems.push(new menuResponse_dto_1.MenuLink('IO', `${this.hwType}/sfa-io?item_id=${this.itemId}`, 'io', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Data', `${this.hwType}/sfa-data?item_id=${this.itemId}`, 'data', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Latency', `${this.hwType}/sfa-latency?item_id=${this.itemId}`, 'latency', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Block size', `${this.hwType}/sfa-block-size?item_id=${this.itemId}`, 'block-size', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Health Status', `${this.hwType}/shealth-status?item_id=${this.itemId}`, 'health-status', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuFolder('Items', this.hwType + '/sfa-systems-items?ids=' + this.itemId, this.hwType, 'sfa-systems-items', this.itemId, SfasystemsitemsFolder));
            return menuItems;
        }
    }
    Ddnintelliflash.SfasystemsFolder = SfasystemsFolder;
    class ExascalersFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            menuItems.push(new menuResponse_dto_1.MenuLink('IO', `${this.hwType}/exascaler-io?item_id=${this.itemId}`, 'exascaler-io', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Data', `${this.hwType}/exascaler-data?item_id=${this.itemId}`, 'exascaler-data', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Capacity', `${this.hwType}/exascaler-capacity?item_id=${this.itemId}`, 'exascaler-capacity', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('CPU', `${this.hwType}/exascaler-cpu?item_id=${this.itemId}`, 'exascaler-cpu', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Memory', `${this.hwType}/exascaler-memory?item_id=${this.itemId}`, 'exascaler-memory', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Block Size', `${this.hwType}/exascaler-block-size?item_id=${this.itemId}`, 'exascaler-block-size', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Health Status', `${this.hwType}/exascalershealth-status?item_id=${this.itemId}`, 'exascalers-health-status', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuFolder('Items', this.hwType + '/exascaler-items?ids=' + this.itemId, this.hwType, 'exascaler-items', this.itemId, ExascaleritemsFolder));
            return menuItems;
        }
    }
    Ddnintelliflash.ExascalersFolder = ExascalersFolder;
    class SfasystemsitemsFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            const items = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                }, where: {
                    parents: {
                        parent: this.itemId
                    },
                    subsystem: 'sfa',
                }
            });
            for (const item of items) {
                menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, this.hwType + '/sfa?ids=' + item.item_id, this.hwType, 'sfa', item.item_id, SfaFolder));
            }
            return menuItems;
        }
    }
    Ddnintelliflash.SfasystemsitemsFolder = SfasystemsitemsFolder;
    class ExascaleritemsFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            const items = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                }, where: {
                    parents: {
                        parent: this.itemId
                    },
                    subsystem: 'exascaler',
                }
            });
            for (const item of items) {
                menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, this.hwType + '/exascaler?ids=' + item.item_id, this.hwType, 'exascaler', this.itemId, ExascalerFolder));
            }
            return menuItems;
        }
    }
    Ddnintelliflash.ExascaleritemsFolder = ExascaleritemsFolder;
    class SfaFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            menuItems.push(new menuResponse_dto_1.MenuLink('Totals', `${this.hwType}/sfa?item_id=${this.itemId}`, 'sfa', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', `${this.hwType}/sfa-configuration?item_id=${this.itemId}`, 'sfa-configuration', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Health Status', `${this.hwType}/sfa-health-status?item_id=${this.itemId}`, 'sfa-health-status', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Capacity', `${this.hwType}/sfa-capacity?item_id=${this.itemId}`, 'sfa-capacity', this.itemId));
            const subsytems = [
                { label: 'Pool', path: 'sfa-pool', folder: SfapoolFolder },
                { label: 'Volume', path: 'sfa-volume', folder: SfavolumeFolder },
                { label: 'Drive', path: 'sfa-drive', folder: SfadriveFolder },
            ];
            for (const subsystem of subsytems) {
                menuItems.push(new menuResponse_dto_1.MenuFolder(subsystem.label, this.hwType + '/' + subsystem.path + '?ids=' + this.itemId, this.hwType, subsystem.path, this.itemId, subsystem.folder));
            }
            return menuItems;
        }
    }
    Ddnintelliflash.SfaFolder = SfaFolder;
    class ExascalerFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            menuItems.push(new menuResponse_dto_1.MenuLink('Totals', `${this.hwType}/exascaler?item_id=${this.itemId}`, 'exascaler', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', `${this.hwType}/exascaler-configuration?item_id=${this.itemId}`, 'exascaler-configuration', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Health Status', `${this.hwType}/exascaler-health-status?item_id=${this.itemId}`, 'exascaler-health-status', this.itemId));
            menuItems.push(new menuResponse_dto_1.MenuLink('Capacity', `${this.hwType}/exascalers-capacity?item_id=${this.itemId}`, 'exascalers-capacity', this.itemId));
            const subsytems = [
                { label: 'FS', path: 'exascaler-fs', folder: ExascalerfsFolder },
                { label: 'OSS', path: 'exascaler-oss', folder: ExascalerossFolder },
                { label: 'OST', path: 'exascaler-ost', folder: ExascalerostFolder },
            ];
            for (const subsystem of subsytems) {
                menuItems.push(new menuResponse_dto_1.MenuFolder(subsystem.label, this.hwType + '/' + subsystem.path + '?ids=' + this.itemId, this.hwType, subsystem.path, this.itemId, subsystem.folder));
            }
            return menuItems;
        }
    }
    Ddnintelliflash.ExascalerFolder = ExascalerFolder;
    class SubsystemFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            if (Storage_1.Storage.technology[this.hwType]['subsystems'][this.subsystem]) {
                const subsystemDefinition = Storage_1.Storage.technology[this.hwType]['subsystems'][this.subsystem];
                if (subsystemDefinition['totals']) {
                    for (const pageType in subsystemDefinition['totals']) {
                        const page = subsystemDefinition['totals'][pageType];
                        menuItems.push(new menuResponse_dto_1.MenuLink(page.label, `${this.hwType}/sfa-${this.subsystem}-${pageType}?item_id=${this.itemId}`, `sfa-${this.subsystem}-${pageType}`, this.itemId));
                    }
                }
            }
            menuItems.push(new menuResponse_dto_1.MenuFolder('Items', this.hwType + '/sfa-' + this.subsystem + '-items?ids=' + this.itemId, this.hwType, this.subsystem, this.itemId, sfaItemsFolders[this.subsystem]));
            return menuItems;
        }
    }
    class ClusterFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            if (Storage_1.Storage.technology[this.hwType]['subsystems'][this.subsystem]) {
                const subsystemDefinition = Storage_1.Storage.technology[this.hwType]['subsystems'][this.subsystem];
                if (subsystemDefinition['totals']) {
                    for (const pageType in subsystemDefinition['totals']) {
                        const page = subsystemDefinition['totals'][pageType];
                        menuItems.push(new menuResponse_dto_1.MenuLink(page.label, `${this.hwType}/exascaler-${this.subsystem}-${pageType}?item_id=${this.itemId}`, `exascaler-${this.subsystem}-${pageType}`, this.itemId));
                    }
                }
            }
            menuItems.push(new menuResponse_dto_1.MenuFolder('Items', this.hwType + '/exascaler-' + this.subsystem + '-items?ids=' + this.itemId, this.hwType, this.subsystem, this.itemId, exascalerItemsFolders[this.subsystem]));
            return menuItems;
        }
    }
    class SfapoolFolder extends SubsystemFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'pool';
        }
    }
    Ddnintelliflash.SfapoolFolder = SfapoolFolder;
    class SfavolumeFolder extends SubsystemFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'volume';
        }
    }
    Ddnintelliflash.SfavolumeFolder = SfavolumeFolder;
    class SfadriveFolder extends SubsystemFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'drive';
        }
    }
    Ddnintelliflash.SfadriveFolder = SfadriveFolder;
    class SfapoolitemsFolder extends SubsystemItemsFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'pool';
        }
    }
    Ddnintelliflash.SfapoolitemsFolder = SfapoolitemsFolder;
    class SfavolumeitemsFolder extends SubsystemItemsFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'volume';
        }
    }
    Ddnintelliflash.SfavolumeitemsFolder = SfavolumeitemsFolder;
    class SfadriveitemsFolder extends SubsystemItemsFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'drive';
        }
    }
    Ddnintelliflash.SfadriveitemsFolder = SfadriveitemsFolder;
    const sfaItemsFolders = {
        pool: SfapoolitemsFolder,
        volume: SfavolumeitemsFolder,
        drive: SfadriveitemsFolder
    };
    class DDNPage extends MenuDefinitions_1.DefaultPage {
        constructor() {
            super(...arguments);
            this.hwType = 'ddnintelliflash';
            this.withItemId = true;
            this.rootFolder = DeviceFolderRoot;
        }
    }
    Ddnintelliflash.DDNPage = DDNPage;
    class ConfigurationPage extends DDNPage {
        async getPage() {
            let tabs = [];
            const configurationPage = await new Stor_1.StorConfigurationPage(this.itemId, this.request).getPage();
            if (configurationPage && configurationPage.tabs[0]) {
                tabs.push(configurationPage.tabs[0]);
            }
            if (configurationPage && configurationPage.tabs[1]) {
                tabs.push(configurationPage.tabs[1]);
            }
            const sfas = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                }, where: {
                    parents: {
                        parent: this.itemId
                    },
                    subsystem: 'sfa'
                }
            });
            const sfaIds = sfas.map(sfa => sfa.item_id);
            const configurationSfaTable = new ConfigurationTable_1.ConfigurationTable(sfaIds, ['model', 'bios', 'bmc_version', 'version', 'ctla', 'ctlb', 'e_platform', 'sfx_capable', 'sfx_enabled']);
            configurationSfaTable.setItemLabel("Name");
            configurationSfaTable.setAcl(this.request);
            tabs.push({
                title: 'SFA',
                content: [await configurationSfaTable.generate()],
            });
            const clusters = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                }, where: {
                    parents: {
                        parent: this.itemId
                    },
                    subsystem: 'exascaler'
                }
            });
            const clusterIds = clusters.map(cluster => cluster.item_id);
            const configurationClusterTable = new ConfigurationTable_1.ConfigurationTable(clusterIds, ['ip', 'version']);
            configurationClusterTable.setItemLabel("Name");
            configurationClusterTable.setAcl(this.request);
            tabs.push({
                title: 'Exascaler',
                content: [await configurationClusterTable.generate()],
            });
            return new pageResponse_dto_1.Page("Configuration", tabs);
        }
    }
    Ddnintelliflash.ConfigurationPage = ConfigurationPage;
    class HealthstatusPage extends DDNPage {
        async getPage() {
            let title = 'Health Status';
            let tabs = [];
            let healthStatusBox = await (new HealthStatus_1.HealthStatus()).getStatusByIds([this.itemId]);
            let subsystemsHealthStatusBox = await (new HealthStatus_1.HealthStatus()).getDeviceSubsystems(this.itemId, ['sfa']);
            let exascalerHealthStatusBox = await (new HealthStatus_1.HealthStatus()).getDeviceSubsystems(this.itemId, ['exascaler']);
            const sfasText = {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                text: "SFAs"
            };
            const exascalerText = {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                text: "Exascalers"
            };
            tabs.push({
                title: 'Health Status',
                content: [
                    healthStatusBox,
                    sfasText,
                    subsystemsHealthStatusBox,
                    exascalerText,
                    exascalerHealthStatusBox
                ],
            });
            const page = new pageResponse_dto_1.Page(title, tabs);
            return page;
        }
    }
    Ddnintelliflash.HealthstatusPage = HealthstatusPage;
    class ShealthstatusPage extends DDNPage {
        async getPage() {
            let tabs = [];
            let subsystemsHealthStatusBox = await (new HealthStatus_1.HealthStatus()).getDeviceSubsystems(this.itemId, ['sfa']);
            tabs.push({
                title: 'Health Status',
                content: [
                    subsystemsHealthStatusBox
                ],
            });
            const page = new pageResponse_dto_1.Page("Health Status", tabs);
            return page;
        }
    }
    Ddnintelliflash.ShealthstatusPage = ShealthstatusPage;
    class ExascalershealthstatusPage extends DDNPage {
        async getPage() {
            let tabs = [];
            let subsystemsHealthStatusBox = await (new HealthStatus_1.HealthStatus()).getDeviceSubsystems(this.itemId, ['exascaler']);
            tabs.push({
                title: 'Health Status',
                content: [
                    subsystemsHealthStatusBox
                ],
            });
            const page = new pageResponse_dto_1.Page("Health Status", tabs);
            return page;
        }
    }
    Ddnintelliflash.ExascalershealthstatusPage = ExascalershealthstatusPage;
    class SfacapacityPage extends DDNPage {
        constructor() {
            super(...arguments);
            this.title = 'Capacity';
        }
        async getPage() {
            const user = await User_1.User.getUserByRequest(this.request);
            const timezone = user ? user.timezone : 'Etc/UTC';
            let tabs = [];
            const pools = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                }, where: {
                    parents: {
                        parent: this.itemId
                    },
                    subsystem: 'pool'
                }
            });
            const poolIds = pools.map(pool => pool.item_id);
            tabs.push({
                title: 'Pool Capacity',
                content: [
                    await new CapacityTableBuilder_1.CapacityTableBuilder("pool", this.request)
                        .setItemLabel("Pool")
                        .addItems(poolIds)
                        .addDefaultMetrics()
                        .addUsed()
                        .addTotal()
                        .generateBox()
                ],
            });
            return new pageResponse_dto_1.Page("Capacity", tabs);
        }
    }
    Ddnintelliflash.SfacapacityPage = SfacapacityPage;
    class ExascalerscapacityPage extends DDNPage {
        constructor() {
            super(...arguments);
            this.title = 'Capacity';
        }
        async getPage() {
            const user = await User_1.User.getUserByRequest(this.request);
            const timezone = user ? user.timezone : 'Etc/UTC';
            const fss = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                }, where: {
                    parents: {
                        parent: this.itemId
                    },
                    subsystem: 'fs'
                }
            });
            const fsIds = fss.map(fs => fs.item_id);
            let tabs = [];
            tabs.push({
                title: 'FS',
                content: [
                    await new CapacityTableBuilder_1.CapacityTableBuilder("fs", this.request)
                        .setItemLabel("FS")
                        .addItems(fsIds)
                        .addDefaultMetrics()
                        .addUsed()
                        .addTotal()
                        .generateBox()
                ]
            });
            return new pageResponse_dto_1.Page("Capacity", tabs);
        }
    }
    Ddnintelliflash.ExascalerscapacityPage = ExascalerscapacityPage;
    class CapacityPage extends DDNPage {
        constructor() {
            super(...arguments);
            this.title = 'Capacity';
        }
        async getPage() {
            const user = await User_1.User.getUserByRequest(this.request);
            const timezone = user ? user.timezone : 'Etc/UTC';
            const exascalers = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                }, where: {
                    parents: {
                        parent: this.itemId
                    },
                    subsystem: 'exascaler'
                }
            });
            const exascalerIds = exascalers.map(exascaler => exascaler.item_id);
            let tabs = [];
            tabs.push({
                title: 'Exascaler',
                content: [
                    await new CapacityTableBuilder_1.CapacityTableBuilder("exascaler", this.request)
                        .setItemLabel("Exascaler")
                        .addItems(exascalerIds)
                        .addDefaultMetrics()
                        .addUsed()
                        .addTotal()
                        .generateBox()
                ]
            });
            return new pageResponse_dto_1.Page("Capacity", tabs);
        }
    }
    Ddnintelliflash.CapacityPage = CapacityPage;
    class SfaconfigurationPage extends DDNPage {
        async getPage() {
            let tabs = [];
            const configurationTable = new ConfigurationTable_1.ConfigurationTable([this.itemId], ['model', 'bios', 'bmc_version', 'version', 'ctla', 'ctlb', 'e_platform', 'sfx_capable', 'sfx_enabled']);
            configurationTable.setItemLabel("Name");
            configurationTable.setAcl(this.request);
            tabs.push({
                title: 'Configuration',
                content: [await configurationTable.generate()],
            });
            return new pageResponse_dto_1.Page("Configuration", tabs);
        }
    }
    Ddnintelliflash.SfaconfigurationPage = SfaconfigurationPage;
    class ExascalerconfigurationPage extends DDNPage {
        async getPage() {
            let tabs = [];
            console.log(this.itemId);
            const configurationTable = new ConfigurationTable_1.ConfigurationTable([this.itemId], ['ip', 'version']);
            configurationTable.setItemLabel("Name");
            configurationTable.setAcl(this.request);
            tabs.push({
                title: 'Configuration',
                content: [await configurationTable.generate()],
            });
            return new pageResponse_dto_1.Page("Configuration", tabs);
        }
    }
    Ddnintelliflash.ExascalerconfigurationPage = ExascalerconfigurationPage;
    class SfahealthstatusPage extends DDNPage {
        async getPage() {
            let title = 'Health Status';
            let tabs = [];
            let data = await this.itemPropertiesRepository.find({ where: { item_id: this.itemId, property_name: (0, typeorm_1.In)(['status', 'label', 'status_data']) } });
            const itemProperties = await this.itemPropertiesRepository.find({ where: { item_id: this.itemId, property_name: 'events' } });
            let label;
            let status_data;
            let status = healthStatus_constants_1.STATUS.UNKNOWN;
            let state;
            for (const items of data) {
                if (items.property_name == 'status') {
                    state = items.property_value;
                }
                if (items.property_name == 'label') {
                    label = items.property_value;
                }
                if (items.property_name == 'status_data') {
                    status_data = items.property_value;
                }
            }
            if (state === 'error') {
                status = healthStatus_constants_1.STATUS.CRITICAL;
            }
            if (state === 'ok') {
                status = healthStatus_constants_1.STATUS.OK;
            }
            if (state === 'warning') {
                status = healthStatus_constants_1.STATUS.WARNING;
            }
            let hsTable = new Table_1.Table()
                .addHeader({ value: 'Status', styleClass: pageResponse_constants_1.TABLE_STYLE_CLASS.healthStatus })
                .addHeader({ value: 'SFA' })
                .addHeader({ value: 'Status reason', trim: true, trimLength: 80 });
            hsTable.addRow([
                { order: 0, value: status, bgStatus: status },
                { order: 1, value: label },
                { order: 2, value: status_data }
            ]);
            const eventLogText = {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                text: "Event log",
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2
            };
            let eventLogTable = new Table_1.Table()
                .addHeader({ value: 'Id' })
                .addHeader({ value: 'Unique id' })
                .addHeader({ value: 'Type' })
                .addHeader({ value: 'Severity' })
                .addHeader({ value: 'First occured time' })
                .addHeader({ value: 'Last occured time' })
                .addHeader({ value: 'Description' });
            if (itemProperties) {
                for (const items of itemProperties) {
                    const data = (JSON.parse(items.property_value));
                    for (const row of data) {
                        eventLogTable.addRow([
                            { order: 0, value: row.id },
                            { order: 1, value: row.unique_id },
                            { order: 2, value: row.event_type },
                            { order: 3, value: row.severity },
                            { order: 4, value: new Date(row.first_occured_time) },
                            { order: 5, value: new Date(row.last_occured_time) },
                            { order: 6, value: row.description }
                        ]);
                    }
                }
            }
            else {
                eventLogTable.addRow([
                    { order: 0, value: '' },
                    { order: 1, value: '' },
                    { order: 2, value: '' },
                    { order: 3, value: '' },
                    { order: 4, value: '' },
                    { order: 5, value: '' },
                    { order: 6, value: '' },
                ]);
            }
            const componentsText = {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                text: "Components",
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2
            };
            let subsystemsHealthStatusBox = await (new HealthStatus_1.HealthStatus()).getDeviceSubsystems(this.itemId);
            tabs.push({
                title: 'Health Status',
                content: [
                    hsTable.generateBox(),
                    eventLogText,
                    eventLogTable.generateBox(),
                    componentsText,
                    subsystemsHealthStatusBox
                ],
            });
            const page = new pageResponse_dto_1.Page(title, tabs);
            return page;
        }
        transformEntityStatus(property_name) {
            if (property_name === healthStatus_entity_1.Status.ERROR) {
                return healthStatus_constants_1.STATUS.CRITICAL;
            }
            else {
                return healthStatus_constants_1.STATUS[property_name.toUpperCase()];
            }
        }
    }
    Ddnintelliflash.SfahealthstatusPage = SfahealthstatusPage;
    class ExascalerhealthstatusPage extends DDNPage {
        async getPage() {
            let title = 'Health Status';
            let tabs = [];
            let data = await this.itemPropertiesRepository.find({ where: { item_id: this.itemId, property_name: (0, typeorm_1.In)(['status', 'label', 'status_data']) } });
            const itemProperties = await this.itemPropertiesRepository.find({ where: { item_id: this.itemId, property_name: 'events' } });
            let label;
            let status_data;
            let status = healthStatus_constants_1.STATUS.UNKNOWN;
            let state;
            for (const items of data) {
                if (items.property_name == 'status') {
                    state = items.property_value;
                }
                if (items.property_name == 'label') {
                    label = items.property_value;
                }
                if (items.property_name == 'status_data') {
                    status_data = items.property_value;
                }
            }
            if (state === 'error') {
                status = healthStatus_constants_1.STATUS.CRITICAL;
            }
            if (state === 'ok') {
                status = healthStatus_constants_1.STATUS.OK;
            }
            if (state === 'warning') {
                status = healthStatus_constants_1.STATUS.WARNING;
            }
            let hsTable = new Table_1.Table()
                .addHeader({ value: 'Status', styleClass: pageResponse_constants_1.TABLE_STYLE_CLASS.healthStatus })
                .addHeader({ value: 'ES' })
                .addHeader({ value: 'Status reason', trim: true, trimLength: 80 });
            hsTable.addRow([
                { order: 0, value: status, bgStatus: status },
                { order: 1, value: label },
                { order: 2, value: status_data }
            ]);
            const eventLogText = {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                text: "Event log",
                styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2
            };
            let eventLogTable = new Table_1.Table()
                .addHeader({ value: 'Id' })
                .addHeader({ value: 'Unique id' })
                .addHeader({ value: 'Type' })
                .addHeader({ value: 'Severity' })
                .addHeader({ value: 'First occured time' })
                .addHeader({ value: 'Last occured time' })
                .addHeader({ value: 'Description' });
            if (itemProperties) {
                for (const items of itemProperties) {
                    const data = (JSON.parse(items.property_value));
                    for (const row of data) {
                        eventLogTable.addRow([
                            { order: 0, value: row.id },
                            { order: 1, value: row.unique_id },
                            { order: 2, value: row.event_type },
                            { order: 3, value: row.severity },
                            { order: 4, value: new Date(row.first_occured_time) },
                            { order: 5, value: new Date(row.last_occured_time) },
                            { order: 6, value: row.description }
                        ]);
                    }
                }
            }
            else {
                eventLogTable.addRow([
                    { order: 0, value: '' },
                    { order: 1, value: '' },
                    { order: 2, value: '' },
                    { order: 3, value: '' },
                    { order: 4, value: '' },
                    { order: 5, value: '' },
                    { order: 6, value: '' },
                ]);
            }
            tabs.push({
                title: 'Health Status',
                content: [
                    hsTable.generateBox(),
                    eventLogText,
                    eventLogTable.generateBox(),
                ],
            });
            const page = new pageResponse_dto_1.Page(title, tabs);
            return page;
        }
        transformEntityStatus(property_name) {
            if (property_name === healthStatus_entity_1.Status.ERROR) {
                return healthStatus_constants_1.STATUS.CRITICAL;
            }
            else {
                return healthStatus_constants_1.STATUS[property_name.toUpperCase()];
            }
        }
    }
    Ddnintelliflash.ExascalerhealthstatusPage = ExascalerhealthstatusPage;
    class ItemPage extends DDNPage {
        constructor() {
            super(...arguments);
            this.withItemId = true;
        }
        async getPage() {
            let tabs = [];
            const item = await this.objectItemsRepository.findOneBy({
                item_id: this.itemId
            });
            if (!item) {
                return new pageResponse_dto_1.Page("Not found", tabs);
            }
            if (Storage_1.Storage.technology[this.hwType]['subsystems'][this.subsystem]) {
                const subsystemDefinition = Storage_1.Storage.technology[this.hwType]['subsystems'][this.subsystem];
                if (subsystemDefinition['single']) {
                    for (const tabIndex in subsystemDefinition['single']) {
                        const tab = subsystemDefinition['single'][tabIndex];
                        if (!tab || !tab.label) {
                            console.warn("Bad tab" + tabIndex);
                            continue;
                        }
                        tabs.push({
                            title: tab.label,
                            content: [{
                                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                                    metrics: tab.metrics,
                                    uuids: [this.itemId],
                                    aggregated: false
                                }],
                        });
                    }
                }
            }
            return new pageResponse_dto_1.Page(item.label, tabs);
        }
    }
    Ddnintelliflash.ItemPage = ItemPage;
    class DevicePage extends DDNPage {
        async getPage() {
            const page = new Stor_1.StorDevicePage(this.itemId, this.request);
            return page.getPage();
        }
    }
    Ddnintelliflash.DevicePage = DevicePage;
    class NodePage extends ItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'node';
        }
    }
    Ddnintelliflash.NodePage = NodePage;
    class PoolPage extends ItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'pool';
        }
    }
    Ddnintelliflash.PoolPage = PoolPage;
    class DrivePage extends ItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'drive';
        }
    }
    Ddnintelliflash.DrivePage = DrivePage;
    class SfaPage extends ItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'sfa';
        }
    }
    Ddnintelliflash.SfaPage = SfaPage;
    class VolumePage extends ItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'volume';
        }
    }
    Ddnintelliflash.VolumePage = VolumePage;
    class ExascalerPage extends ItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'exascaler';
        }
    }
    Ddnintelliflash.ExascalerPage = ExascalerPage;
    class FsPage extends ItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'fs';
        }
    }
    Ddnintelliflash.FsPage = FsPage;
    class OssPage extends ItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'oss';
        }
    }
    Ddnintelliflash.OssPage = OssPage;
    class OstPage extends ItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'ost';
        }
    }
    Ddnintelliflash.OstPage = OstPage;
    class TotalsItemPage extends DDNPage {
        async getPage() {
            let tabs = [];
            const items = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                }, where: {
                    parents: {
                        parent: this.itemId
                    },
                    subsystem: this.subsystem
                }
            });
            const itemIds = items.map(i => i.item_id);
            let label = "unlabeled";
            if (Storage_1.Storage.technology[this.hwType]['subsystems'][this.subsystem]) {
                const subsystemDefinition = Storage_1.Storage.technology[this.hwType]['subsystems'][this.subsystem];
                if (subsystemDefinition['totals'] && subsystemDefinition['totals'][this.page]) {
                    if (subsystemDefinition['totals'][this.page].label) {
                        label = subsystemDefinition['totals'][this.page].label;
                    }
                    for (const tabIndex in subsystemDefinition['totals'][this.page]['tabs']) {
                        const tab = subsystemDefinition['totals'][this.page]['tabs'][tabIndex];
                        const aggregated = subsystemDefinition['totals'][this.page]['single'] ? false : true;
                        tabs.push({
                            title: tab.label,
                            content: [{
                                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                                    metrics: tab.metrics,
                                    uuids: itemIds,
                                    aggregated: aggregated
                                }],
                        });
                    }
                }
            }
            return new pageResponse_dto_1.Page(label, tabs);
        }
    }
    class MemoryPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'device';
            this.page = 'memory';
        }
    }
    Ddnintelliflash.MemoryPage = MemoryPage;
    class CpuPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'device';
            this.page = 'cpu';
        }
    }
    Ddnintelliflash.CpuPage = CpuPage;
    class IoPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'device';
            this.page = 'io';
        }
    }
    Ddnintelliflash.IoPage = IoPage;
    class DataPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'device';
            this.page = 'data';
        }
    }
    Ddnintelliflash.DataPage = DataPage;
    class LatencyPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'device';
            this.page = 'latency';
        }
    }
    Ddnintelliflash.LatencyPage = LatencyPage;
    class SfadataPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'sfa';
            this.page = 'data';
        }
    }
    Ddnintelliflash.SfadataPage = SfadataPage;
    class SfaioPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'sfa';
            this.page = 'io';
        }
    }
    Ddnintelliflash.SfaioPage = SfaioPage;
    class SfalatencyPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'sfa';
            this.page = 'resp_t';
        }
    }
    Ddnintelliflash.SfalatencyPage = SfalatencyPage;
    class SfablocksizePage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'sfa';
            this.page = 'block_size';
        }
    }
    Ddnintelliflash.SfablocksizePage = SfablocksizePage;
    class SfavolumeioPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'volume';
            this.page = 'io';
        }
    }
    Ddnintelliflash.SfavolumeioPage = SfavolumeioPage;
    class SfapoolioPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'pool';
            this.page = 'io';
        }
    }
    Ddnintelliflash.SfapoolioPage = SfapoolioPage;
    class SfadriveioPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'drive';
            this.page = 'io';
        }
    }
    Ddnintelliflash.SfadriveioPage = SfadriveioPage;
    class SfavolumedataPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'volume';
            this.page = 'data';
        }
    }
    Ddnintelliflash.SfavolumedataPage = SfavolumedataPage;
    class SfapooldataPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'pool';
            this.page = 'data';
        }
    }
    Ddnintelliflash.SfapooldataPage = SfapooldataPage;
    class SfadrivedataPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'drive';
            this.page = 'data';
        }
    }
    Ddnintelliflash.SfadrivedataPage = SfadrivedataPage;
    class SfavolumeblocksizePage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'volume';
            this.page = 'block_size';
        }
    }
    Ddnintelliflash.SfavolumeblocksizePage = SfavolumeblocksizePage;
    class SfapoolblocksizePage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'pool';
            this.page = 'block_size';
        }
    }
    Ddnintelliflash.SfapoolblocksizePage = SfapoolblocksizePage;
    class SfadriveblocksizePage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'drive';
            this.page = 'block_size';
        }
    }
    Ddnintelliflash.SfadriveblocksizePage = SfadriveblocksizePage;
    class SfavolumeresptPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'volume';
            this.page = 'resp_t';
        }
    }
    Ddnintelliflash.SfavolumeresptPage = SfavolumeresptPage;
    class SfapoolresptPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'pool';
            this.page = 'resp_t';
        }
    }
    Ddnintelliflash.SfapoolresptPage = SfapoolresptPage;
    class SfadriveresptPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'drive';
            this.page = 'resp_t';
        }
    }
    Ddnintelliflash.SfadriveresptPage = SfadriveresptPage;
    class SfavolumecapacityPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'volume';
            this.page = 'capacity';
        }
    }
    Ddnintelliflash.SfavolumecapacityPage = SfavolumecapacityPage;
    class SfapoolcapacityPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'pool';
            this.page = 'capacity';
        }
    }
    Ddnintelliflash.SfapoolcapacityPage = SfapoolcapacityPage;
    class SfadrivecapacityPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'drive';
            this.page = 'capacity';
        }
    }
    Ddnintelliflash.SfadrivecapacityPage = SfadrivecapacityPage;
    class ExascalerioPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'exascaler';
            this.page = 'io';
        }
    }
    Ddnintelliflash.ExascalerioPage = ExascalerioPage;
    class ExascalerdataPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'exascaler';
            this.page = 'data';
        }
    }
    Ddnintelliflash.ExascalerdataPage = ExascalerdataPage;
    class ExascalermemoryPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'exascaler';
            this.page = 'memory';
        }
    }
    Ddnintelliflash.ExascalermemoryPage = ExascalermemoryPage;
    class ExascalercpuPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'exascaler';
            this.page = 'cpu';
        }
    }
    Ddnintelliflash.ExascalercpuPage = ExascalercpuPage;
    class ExascalerblocksizePage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'exascaler';
            this.page = 'block_size';
        }
    }
    Ddnintelliflash.ExascalerblocksizePage = ExascalerblocksizePage;
    class ExascalercapacityPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'exascaler';
            this.page = 'capacity';
        }
    }
    Ddnintelliflash.ExascalercapacityPage = ExascalercapacityPage;
    class ExascalerossFolder extends ClusterFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'oss';
        }
    }
    Ddnintelliflash.ExascalerossFolder = ExascalerossFolder;
    class ExascalerostFolder extends ClusterFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'ost';
        }
    }
    Ddnintelliflash.ExascalerostFolder = ExascalerostFolder;
    class ExascalerfsFolder extends ClusterFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'fs';
        }
    }
    Ddnintelliflash.ExascalerfsFolder = ExascalerfsFolder;
    class ExascalerossitemsFolder extends SubsystemItemsFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'oss';
        }
    }
    Ddnintelliflash.ExascalerossitemsFolder = ExascalerossitemsFolder;
    class ExascalerostitemsFolder extends SubsystemItemsFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'ost';
        }
    }
    Ddnintelliflash.ExascalerostitemsFolder = ExascalerostitemsFolder;
    class ExascalerfsitemsFolder extends SubsystemItemsFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'fs';
        }
    }
    Ddnintelliflash.ExascalerfsitemsFolder = ExascalerfsitemsFolder;
    const exascalerItemsFolders = {
        oss: ExascalerossitemsFolder,
        ost: ExascalerostitemsFolder,
        fs: ExascalerfsitemsFolder
    };
    class ExascalerfsioPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'fs';
            this.page = 'io';
        }
    }
    Ddnintelliflash.ExascalerfsioPage = ExascalerfsioPage;
    class ExascalerfsdataPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'fs';
            this.page = 'data';
        }
    }
    Ddnintelliflash.ExascalerfsdataPage = ExascalerfsdataPage;
    class ExascalerfscapacityPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'fs';
            this.page = 'capacity';
        }
    }
    Ddnintelliflash.ExascalerfscapacityPage = ExascalerfscapacityPage;
    class ExascalerfsblocksizePage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'fs';
            this.page = 'block_size';
        }
    }
    Ddnintelliflash.ExascalerfsblocksizePage = ExascalerfsblocksizePage;
    class ExascalerossioPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'oss';
            this.page = 'io';
        }
    }
    Ddnintelliflash.ExascalerossioPage = ExascalerossioPage;
    class ExascalerossdataPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'oss';
            this.page = 'data';
        }
    }
    Ddnintelliflash.ExascalerossdataPage = ExascalerossdataPage;
    class ExascalerossmemoryPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'oss';
            this.page = 'memory';
        }
    }
    Ddnintelliflash.ExascalerossmemoryPage = ExascalerossmemoryPage;
    class ExascalerosscpuPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'oss';
            this.page = 'cpu';
        }
    }
    Ddnintelliflash.ExascalerosscpuPage = ExascalerosscpuPage;
    class ExascalerossblocksizePage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'oss';
            this.page = 'block_size';
        }
    }
    Ddnintelliflash.ExascalerossblocksizePage = ExascalerossblocksizePage;
    class ExascalerosscapacityPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'oss';
            this.page = 'capacity';
        }
    }
    Ddnintelliflash.ExascalerosscapacityPage = ExascalerosscapacityPage;
    class ExascalerostioPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'ost';
            this.page = 'io';
        }
    }
    Ddnintelliflash.ExascalerostioPage = ExascalerostioPage;
    class ExascalerostdataPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'ost';
            this.page = 'data';
        }
    }
    Ddnintelliflash.ExascalerostdataPage = ExascalerostdataPage;
    class ExascalerostblocksizePage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'ost';
            this.page = 'block_size';
        }
    }
    Ddnintelliflash.ExascalerostblocksizePage = ExascalerostblocksizePage;
    class ExascalerostcapacityPage extends TotalsItemPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'ost';
            this.page = 'capacity';
        }
    }
    Ddnintelliflash.ExascalerostcapacityPage = ExascalerostcapacityPage;
})(Ddnintelliflash || (exports.Ddnintelliflash = Ddnintelliflash = {}));
//# sourceMappingURL=Ddnintelliflash.js.map