"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Docker = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const Color_1 = require("../../metrics/Color");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const Heatmap_1 = require("./Heatmap");
const Table_1 = require("./Table");
const URL_1 = require("./URL");
const MenuDefinitions_1 = require("./MenuDefinitions");
const GraphBuilder_1 = require("./GraphBuilder");
var Docker;
(function (Docker) {
    class Folder extends MenuDefinitions_1.DefaultFolder {
        constructor() {
            super(...arguments);
            this.hwType = 'docker';
        }
    }
    class RootFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/totals', 'totals'));
            menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
            let hosts = await this.objectItemsRepository.findBy({
                hw_type: this.hwType,
                subsystem: 'host',
            });
            hosts.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let host of hosts) {
                let path = this.hwType + '/host?ids=' + host.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(host.label, path, this.hwType, 'host', host.item_id, HostFolder));
            }
            return menuItems;
        }
        async getChildrenAcl() {
            let menuItems = [];
            let hosts = await this.objectItemsRepository.findBy({
                hw_type: this.hwType,
                subsystem: 'host',
            });
            hosts.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let host of hosts) {
                let path = this.hwType + '/host?ids=' + host.item_id;
                menuItems.push(new menuResponse_dto_1.MenuLink(host.label, path, 'host', host.item_id));
            }
            return menuItems;
        }
    }
    Docker.RootFolder = RootFolder;
    class HostFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            const containerCheck = await this.itemRelationsRepository.findOne({
                relations: {
                    _child: true
                }, where: {
                    parent: this.itemId.toString(),
                    _child: {
                        subsystem: 'container'
                    }
                }
            });
            const volumesCheck = await this.itemRelationsRepository.findOne({
                relations: {
                    _child: true
                }, where: {
                    parent: this.itemId.toString(),
                    _child: {
                        subsystem: 'volume'
                    }
                }
            });
            if (containerCheck || volumesCheck) {
                let path = this.hwType + '/host?item_id=' + this.itemId;
                menuItems.push(new menuResponse_dto_1.MenuLink('Totals', path, 'host', this.itemId, true));
            }
            if (containerCheck) {
                let path = this.hwType + '/container-folder?ids=' + this.itemId;
                menuItems.push(new menuResponse_dto_1.MenuFolder('Containers', path, this.hwType, 'host', this.itemId, ContainerfolderFolder));
            }
            if (volumesCheck) {
                let path = this.hwType + '/volume-folder?ids=' + this.itemId;
                menuItems.push(new menuResponse_dto_1.MenuFolder('Volumes', path, this.hwType, 'host', this.itemId, VolumefolderFolder));
            }
            return menuItems;
        }
    }
    Docker.HostFolder = HostFolder;
    class ContainerfolderFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            const containers = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                },
                where: {
                    parents: {
                        parent: this.itemId.toString()
                    },
                    subsystem: 'container',
                    hw_type: 'docker'
                }
            });
            containers.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let container of containers) {
                let path = this.hwType + '/container?item_id=' + container.item_id;
                menuItems.push(new menuResponse_dto_1.MenuLink(container.label, path, 'container', container.item_id));
            }
            return menuItems;
        }
    }
    Docker.ContainerfolderFolder = ContainerfolderFolder;
    class VolumefolderFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            const volumes = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                },
                where: {
                    parents: {
                        parent: this.itemId.toString()
                    },
                    subsystem: 'volume',
                    hw_type: 'docker'
                }
            });
            volumes.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let volume of volumes) {
                let path = this.hwType + '/volume?item_id=' + volume.item_id;
                menuItems.push(new menuResponse_dto_1.MenuLink(volume.label, path, 'volume', volume.item_id));
            }
            return menuItems;
        }
    }
    Docker.VolumefolderFolder = VolumefolderFolder;
    class DockerPage extends MenuDefinitions_1.DefaultPage {
        constructor() {
            super(...arguments);
            this.hwType = 'docker';
            this.rootFolder = RootFolder;
        }
    }
    class DockerTotalsPage extends DockerPage {
        constructor() {
            super(...arguments);
            this.withItemId = true;
        }
    }
    class HeatmapPage extends DockerPage {
        constructor() {
            super(...arguments);
            this.title = 'Heatmap';
        }
        async getPage() {
            let title = 'Heatmap';
            let tabs = [];
            tabs.push({
                title: 'Container',
                content: Object.values(await new Heatmap_1.Heatmap('docker', 'container', true)
                    .computedHeatmapBox(class_constants_1.ClassKey.CONTAINER)),
            });
            let containerTable = new Table_1.Table()
                .addHeader({ value: 'Container' })
                .addHeader({ value: 'Host' })
                .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
            const containers = await this.objectItemsRepository.find({
                where: {
                    hw_type: 'docker',
                    subsystem: 'container'
                },
                relations: {
                    heatmaps: true,
                    parents: {
                        _parent: true
                    }
                }
            });
            for (const container of containers) {
                const host = container.parents.find(parent => parent._parent.subsystem === 'host');
                const cpu = container.heatmaps.find(heatmap => heatmap.metric === 'cpu_usage');
                containerTable.addRow([
                    { order: 0, value: container.label, url: URL_1.URL.generate('docker', 'container', container.item_id) },
                    { order: 1, value: host ? host._parent.label : 'undefined' },
                    { order: 2, value: cpu ? cpu.value : null },
                ]);
            }
            containerTable.sortByIndex(2);
            containerTable.setBoxExport("docker-heatmap");
            tabs.push({
                title: 'Container table',
                content: [containerTable.generateBox()],
            });
            return new pageResponse_dto_1.Page(title, tabs);
        }
    }
    Docker.HeatmapPage = HeatmapPage;
    class HostPage extends DockerTotalsPage {
        constructor() {
            super(...arguments);
            this.title = 'Totals';
        }
        async getPage() {
            let title = 'Totals';
            let tabs = [];
            const containers = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                },
                where: {
                    parents: {
                        parent: this.itemId.toString()
                    },
                    subsystem: 'container',
                    hw_type: 'docker'
                }
            });
            tabs.push({
                title: 'CPU%',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('cpu_usage', pageResponse_constants_1.CHART_TYPE.line, false)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Memory',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('memory_used', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Read',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('read_bytes', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Write',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('write_bytes', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Read IO',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('read_io', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Write IO',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('write_io', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'LAN In',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('rx_bytes', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'LAN Out',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('tx_bytes', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Size',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('size_root_fs', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Size RW',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('size_rw', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            const volumes = await this.objectItemsRepository.find({
                relations: {
                    parents: true
                },
                where: {
                    parents: {
                        parent: this.itemId.toString()
                    },
                    subsystem: 'volume',
                    hw_type: 'docker'
                }
            });
            tabs.push({
                title: 'Volumes',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'volume')
                        .addItems(volumes.map(item => item.item_id))
                        .addMetric('size', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Volume')
                        .generateGraph()
                ],
            });
            return new pageResponse_dto_1.Page(title, tabs);
        }
    }
    Docker.HostPage = HostPage;
    class TotalsPage extends DockerTotalsPage {
        constructor() {
            super(...arguments);
            this.title = 'Totals';
        }
        async getPage() {
            const unstackedMetricsToNames = {
                'memory_used': 'Memory',
                'read_bytes': 'Read',
                'write_bytes': 'Write',
                'read_io': 'Read IO',
                'write_io': 'Write IO',
                'rx_bytes': 'LAN In',
                'tx_bytes': 'LAN Out',
                'size_root_fs': 'Size',
                'size_rw': 'Size RW',
            };
            let title = 'Totals';
            let tabs = [];
            const containers = await this.objectItemsRepository.find({
                where: {
                    hw_type: 'docker',
                    subsystem: 'container'
                }
            });
            tabs.push({
                title: "CPU%",
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems(containers.map(item => item.item_id))
                        .addMetric('cpu_usage', pageResponse_constants_1.CHART_TYPE.line, false)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Container')
                        .generateGraph()
                ],
            });
            for (const metric in unstackedMetricsToNames) {
                tabs.push({
                    title: unstackedMetricsToNames[metric],
                    content: [
                        new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                            .addItems(containers.map(item => item.item_id))
                            .addMetric(metric, pageResponse_constants_1.CHART_TYPE.area, true)
                            .setAggregated()
                            .setParent({
                            label: 'Host',
                            subsystem: 'host'
                        })
                            .setLabel('Container')
                            .generateGraph()
                    ],
                });
            }
            const volumes = await this.objectItemsRepository.find({
                where: {
                    hw_type: 'docker',
                    subsystem: 'volume'
                }
            });
            tabs.push({
                title: 'Volumes',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'volume')
                        .addItems(volumes.map(item => item.item_id))
                        .addMetric('size', pageResponse_constants_1.CHART_TYPE.area, true)
                        .setAggregated()
                        .setParent({
                        label: 'Host',
                        subsystem: 'host'
                    })
                        .setLabel('Volume')
                        .generateGraph()
                ],
            });
            return new pageResponse_dto_1.Page(title, tabs);
        }
        ;
    }
    Docker.TotalsPage = TotalsPage;
    class ContainerPage extends DockerPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'container';
        }
        async getPage() {
            let item = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: this.subsystem,
                item_id: this.itemId,
            });
            let title = 'Container ' + (item ? item.label : this.itemId);
            let tabs = [];
            tabs.push({
                title: 'CPU%',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems([this.itemId])
                        .addMetric('cpu_usage', pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Memory',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems([this.itemId])
                        .addMetric('memory_used', pageResponse_constants_1.CHART_TYPE.area, true, 0, Color_1.Color.pallete.red)
                        .addMetric('memory_free', pageResponse_constants_1.CHART_TYPE.area, true, 1, Color_1.Color.pallete.green)
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Data',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems([this.itemId])
                        .addMetric('read_bytes', pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark)
                        .addMetric('write_bytes', pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.blueDark)
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'IO',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems([this.itemId])
                        .addMetric('read_io', pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark)
                        .addMetric('write_io', pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.blueDark)
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'LAN',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems([this.itemId])
                        .addMetric('rx_bytes', pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark)
                        .addMetric('tx_bytes', pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.blueDark)
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Size',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems([this.itemId])
                        .addMetric('size_root_fs', pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                        .generateGraph()
                ],
            });
            tabs.push({
                title: 'Size RW',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'container')
                        .addItems([this.itemId])
                        .addMetric('size_rw', pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                        .generateGraph()
                ],
            });
            const volumes = await this.itemRelationsRepository.findBy({ parent: this.itemId });
            if (volumes.length >= 1) {
                tabs.push({
                    title: 'Volumes',
                    content: [
                        new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'volume')
                            .addItems(volumes.map(v => v.child))
                            .addMetric('size', pageResponse_constants_1.CHART_TYPE.area, true, 0)
                            .setAggregated()
                            .generateGraph()
                    ],
                });
            }
            return new pageResponse_dto_1.Page(title, tabs);
        }
    }
    Docker.ContainerPage = ContainerPage;
    class VolumePage extends DockerPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'volume';
        }
        async getPage() {
            let item = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: this.subsystem,
                item_id: this.itemId,
            });
            let title = 'Volume ' + (item ? item.label : this.itemId);
            let tabs = [];
            tabs.push({
                title: 'Volumes',
                content: [
                    new GraphBuilder_1.GraphBuilder(class_constants_1.ClassKey.CONTAINER, hwTypeKey_constants_1.HwTypeKey.DOCKER, 'volume')
                        .addItems([this.itemId.toString()])
                        .addMetric('size', pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                        .setAggregated()
                        .generateGraph()
                ],
            });
            return new pageResponse_dto_1.Page(title, tabs);
        }
    }
    Docker.VolumePage = VolumePage;
})(Docker || (exports.Docker = Docker = {}));
//# sourceMappingURL=Docker.js.map