"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HcpCapacityPage = exports.HcpHealthstatusPage = exports.HcpConfigurationPage = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const Stor_1 = require("./Stor");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const HealthStatus_1 = require("./HealthStatus");
const Table_1 = require("./Table");
const CapacityTableBuilder_1 = require("./CapacityTableBuilder");
class HcpConfigurationPage extends Stor_1.StorConfigurationPage {
    async getPage() {
        const page = await super.getPage();
        page.tabs = [page.tabs[0]];
        const itemProperties = await this.itemPropertiesRepository.find({
            where: {
                item_id: this.itemId,
                property_name: 'nodes',
            }
        });
        const nodeTable = new Table_1.Table();
        const nodeKeys = ['Number', 'IP address', 'Management IP', 'IPv6 address', 'Management IPv6', 'Date time', 'Boot time', 'Hardware type', 'Model', 'SAN status'];
        nodeKeys.forEach(value => {
            nodeTable.addHeader({ value });
        });
        if (itemProperties) {
            for (const items of itemProperties) {
                const data = (JSON.parse(items.property_value));
                if (data.length) {
                    for (const row of data) {
                        const rowData = [];
                        nodeKeys.forEach((prop, order) => {
                            rowData.push({ order, value: row[prop] });
                        });
                        nodeTable.addRow(rowData);
                    }
                }
            }
        }
        this.page = new pageResponse_dto_1.Page(page.title, [{
                ...page.tabs[0],
                content: [
                    ...page.tabs[0].content,
                    makeHeader("Node"),
                    nodeTable.generateBox()
                ]
            }]);
        return this.page;
    }
}
exports.HcpConfigurationPage = HcpConfigurationPage;
class HcpHealthstatusPage extends Stor_1.StorHealthstatusPage {
    async getPage() {
        const healthStatus = new HealthStatus_1.HealthStatus().getStatusByIds([this.itemId], 'Device');
        const title = 'Health Status';
        const itemPropertiesHs = await this.itemPropertiesRepository.find({
            where: {
                item_id: this.itemId,
                property_name: 'hs',
            }
        });
        const hsDetailTable = new Table_1.Table();
        if (itemPropertiesHs) {
            const hsDetailKeys = ['Pool Status', 'Components Status', 'Irreparable Objects Present', 'Unavailable Objects Present'];
            for (const items of itemPropertiesHs) {
                const data = (JSON.parse(items.property_value));
                const row = [];
                hsDetailKeys.forEach(key => {
                    hsDetailTable.addHeader({ value: key });
                    row.push({ order: 0, value: data[key] });
                });
                hsDetailTable.addRow(row);
            }
        }
        const nodeHsDetailKeys = ['Label', 'Status', 'NIC Failure', 'NIC Degraded', 'SAN Status', 'Volume Status'];
        const nodeHsTable = new Table_1.Table();
        nodeHsDetailKeys.forEach(value => {
            nodeHsTable.addHeader({ value });
        });
        const itemPropertiesNodeHs = await this.itemPropertiesRepository.find({
            where: {
                item_id: this.itemId,
                property_name: 'node_hs',
            }
        });
        if (itemPropertiesNodeHs) {
            for (const items of itemPropertiesNodeHs) {
                const data = (JSON.parse(items.property_value));
                if (data.length) {
                    for (const row of data) {
                        const tableRow = [];
                        nodeHsDetailKeys.forEach((key, order) => {
                            tableRow.push({ order, value: row[key] });
                        });
                        nodeHsTable.addRow(tableRow);
                    }
                }
            }
        }
        const alertsTable = new Table_1.Table()
            .addHeader({ value: 'Date' })
            .addHeader({ value: 'Level' })
            .addHeader({ value: 'Description' });
        const itemPropertiesAlert = await this.itemPropertiesRepository.find({
            where: {
                item_id: this.itemId,
                property_name: 'alerts',
            }
        });
        if (itemPropertiesAlert) {
            for (const items of itemPropertiesAlert) {
                const data = (JSON.parse(items.property_value));
                if (data.length) {
                    for (const row of data) {
                        alertsTable.addRow([
                            { order: 0, value: row.timestamp },
                            { order: 1, value: row.severity },
                            { order: 2, value: row.description },
                        ]);
                    }
                }
                else {
                    alertsTable.addRow([
                        { order: 0, value: '-' },
                        { order: 1, value: '-' },
                        { order: 2, value: '-' },
                    ]);
                }
            }
        }
        const tabs = [{
                title,
                content: [
                    await healthStatus,
                    makeHeader("Detail"),
                    hsDetailTable.generatePivotBox(),
                    makeHeader("Node"),
                    nodeHsTable.generateBox(),
                    makeHeader("Alerts"),
                    alertsTable.generateBox(),
                ],
            }];
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.HcpHealthstatusPage = HcpHealthstatusPage;
function makeHeader(text) {
    return {
        type: pageResponse_constants_1.BOX_TYPE.TextBox,
        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
        text
    };
}
class HcpCapacityPage extends Stor_1.StorCapacityPage {
    async getPage() {
        const tabs = [];
        const subsystems = [
            ['node', 'Node', {
                    total: 'capacity_total',
                    used: 'capacity_used',
                    free: 'capacity_free',
                }],
            ['volume', 'Volume', {
                    total: 'capacity_total',
                    used: 'capacity_used',
                    free: 'capacity_free',
                }, 'node'],
            ['tenant', 'Tenant', {
                    total: 'hard_quota',
                    used: 'size',
                    free: 'capacity_free',
                }],
            ['namespace', 'Namespace', {
                    total: 'hard_quota',
                    used: 'size',
                    free: 'capacity_free',
                }, 'tenant'],
        ];
        for (const [subsystem, tabTitle, metrics, parentSub] of subsystems) {
            let items = [];
            if (parentSub) {
                const parents = await this.objectItemsRepository.find({
                    where: {
                        subsystem: parentSub,
                        parents: {
                            parent: this.itemId
                        }
                    }
                });
                for (const parent of parents) {
                    const children = await this.objectItemsRepository.find({
                        where: {
                            subsystem,
                            parents: {
                                parent: parent.item_id
                            }
                        }
                    });
                    children.forEach(child => {
                        items.push({ ...child, parent });
                    });
                }
            }
            else {
                items = await this.objectItemsRepository.find({
                    where: {
                        subsystem,
                        parents: {
                            parent: this.itemId
                        }
                    }
                });
            }
            tabs.push({
                title: tabTitle,
                content: await new CapacityTableBuilder_1.CapacityTableBuilder(subsystem, this.request)
                    .setItemLabel(tabTitle)
                    .addItems(items.map(it => it.item_id))
                    .addMetric(metrics.total, "Total")
                    .addMetric(metrics.used, "Used")
                    .addMetric(metrics.free, "Free")
                    .addUsed({ total: metrics.total, used: metrics.used })
                    .addTotal()
                    .addPrediction(true, metrics.used, metrics.total)
                    .generateContentWithPredictions(),
            });
        }
        ;
        this.page = new pageResponse_dto_1.Page("Capacity", tabs);
        return this.page;
    }
}
exports.HcpCapacityPage = HcpCapacityPage;
//# sourceMappingURL=Hcp.js.map