"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Heatmap = exports.HeatmapStorage = exports.HeatmapStorageCapacity = exports.HeatmapLan = exports.HeatmapSan = exports.CrossHeatmap = exports.HeatmapMetric = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const dataSource_1 = require("../../../config/dataSource");
const heatmap_entity_1 = require("../../../entity/heatmap.entity");
const heatmapTier_entity_1 = require("../../../entity/heatmapTier.entity");
const heatmapTierItem_entity_1 = require("../../../entity/heatmapTierItem.entity");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const ACLCheck_1 = require("../../acl/ACLCheck");
const Definitions_1 = require("../../metrics/Definitions");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const typeorm_1 = require("typeorm");
const Table_1 = require("./Table");
const URL_1 = require("./URL");
const Color_1 = require("../../metrics/Color");
const Linux_1 = require("./Linux");
const MenuDefinitions_1 = require("./MenuDefinitions");
const User_1 = require("../../users/User");
const class_constants_1 = require("../../../common/constants/class.constants");
class HeatmapMetric {
}
exports.HeatmapMetric = HeatmapMetric;
function formatDate(date) {
    let formattedDate = new Intl.DateTimeFormat('sv-SE', {
        year: 'numeric',
        month: 'numeric',
        day: 'numeric',
        hour: 'numeric',
        minute: 'numeric'
    }).format(new Date(date).getTime());
    return formattedDate;
}
class CrossHeatmap {
    constructor(request) {
        this.request = request;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.lastUpdateAt = null;
    }
    async gen() {
        const tabs = [];
        let heatmapItems = await this.objectItemsRepository.find({
            where: {
                hw_type: (0, typeorm_1.In)([
                    'power', 'vmware', 'nutanix', 'proxmox', 'oraclevm', 'linux', 'ovirt', 'solaris'
                ]),
                subsystem: (0, typeorm_1.In)([
                    'vm', 'server', 'esxi', 'node', 'host', 'ldom', 'cdom'
                ]),
                item_id: (0, typeorm_1.Not)((0, typeorm_1.In)(await (0, Linux_1.getLinuxExcludedIds)()))
            },
            relations: {
                heatmaps: true
            },
            order: {
                heatmaps: {
                    metric: 'ASC'
                }
            }
        });
        if (this.request) {
            heatmapItems = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, heatmapItems);
        }
        const itemIds = heatmapItems.map(item => item.item_id);
        const itemsWithRelations = await this.objectItemsRepository.find({
            relations: {
                parents: {
                    _parent: true
                }
            },
            where: {
                item_id: (0, typeorm_1.In)(itemIds)
            }
        });
        const itemsWithParents = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(itemsWithRelations));
        let tabLabel = {};
        let metric = {};
        let mapping = {};
        for (const heatmapItem of heatmapItems) {
            mapping[heatmapItem.item_id] = {
                hw_type: heatmapItem.hw_type,
                subsystem: heatmapItem.subsystem
            };
            const itemWithParents = itemsWithParents.find(item => item.item_id === heatmapItem.item_id);
            for (const heatmapData of heatmapItem.heatmaps) {
                if (heatmapData.value === undefined || heatmapData.value === null) {
                    continue;
                }
                if (heatmapData.updatedAt > this.lastUpdateAt) {
                    this.lastUpdateAt = heatmapData.updatedAt;
                }
                if (!tabLabel[heatmapData.metric]) {
                    if (heatmapData.metric.includes("cpu")) {
                        tabLabel[heatmapData.metric] = "CPU";
                    }
                    else if (heatmapData.metric.includes("mem")) {
                        tabLabel[heatmapData.metric] = "Memory";
                    }
                }
                if (metric[heatmapData.metric]) {
                    metric[heatmapData.metric]['items'].push({
                        item_id: heatmapItem.item_id,
                        label: itemWithParents && itemWithParents.root_parent
                            ? `[${itemWithParents.root_parent.label}]: ${heatmapItem.label}`
                            : heatmapItem.label,
                        percent: heatmapData.value,
                        url: URL_1.URL.generate(heatmapItem.hw_type, heatmapItem.subsystem, heatmapItem.item_id, undefined, tabLabel[heatmapData.metric])
                    });
                }
                else {
                    metric[heatmapData.metric] = {
                        label: null,
                        metric: heatmapData.metric,
                        items: [{
                                item_id: heatmapItem.item_id,
                                label: itemWithParents && itemWithParents.root_parent
                                    ? `[${itemWithParents.root_parent.label}]: ${heatmapItem.label}`
                                    : heatmapItem.label,
                                percent: heatmapData.value,
                                url: URL_1.URL.generate(heatmapItem.hw_type, heatmapItem.subsystem, heatmapItem.item_id, undefined, tabLabel[heatmapData.metric])
                            }]
                    };
                }
            }
        }
        const createHeatmapTable = (data) => {
            let vmTable = new Table_1.Table()
                .addHeader({ value: 'Item' })
                .addHeader({ value: 'Parent' })
                .addHeader({ value: data.label.includes("CPU") ? 'CPU' : "Memory", unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
            for (const item of data.items) {
                let parent;
                let label;
                const labelSplit = item.label.split("]:");
                if (labelSplit[1]) {
                    parent = labelSplit[0].replace("[", "");
                    label = labelSplit[1];
                }
                else {
                    label = item.label;
                }
                vmTable.addRow([
                    { order: 0, value: label, url: item.url },
                    { order: 1, value: parent },
                    { order: 2, value: item.percent }
                ]);
            }
            return vmTable.generateBox();
        };
        let vmCpuHeatmap = {
            label: 'VM CPU%',
            metric: 'cpu_utilization_percent',
            items: []
        };
        for (const metricItem in metric) {
            if (['cpu_utilization_percent', 'cpu_util', 'utilizedProcUnitsDeductIdlePercent', 'cpu_utilization'].includes(metric[metricItem].metric)) {
                vmCpuHeatmap.items.push(...(metric[metricItem].items.filter(item => ['vm', 'ldom'].includes(mapping[item.item_id].subsystem) || (mapping[item.item_id].subsystem === 'server' && mapping[item.item_id].hw_type === 'linux'))));
            }
        }
        vmCpuHeatmap.items.sort((a, b) => (a.percent < b.percent || a.percent === null || Number.isNaN(a.percent)) ? 1 : ((b.percent < a.percent || b.percent === null || Number.isNaN(b.percent)) ? -1 : 0));
        if (vmCpuHeatmap.items && vmCpuHeatmap.items.length >= 1) {
            tabs.push({
                title: 'VM CPU',
                content: [],
                subTabs: [
                    {
                        title: 'Heatmap',
                        content: [
                            {
                                type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
                                metrics: [vmCpuHeatmap],
                            },
                            {
                                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                                text: "Last update at " + (this.lastUpdateAt ? formatDate(this.lastUpdateAt) : "unknown")
                            },
                        ]
                    }, {
                        title: 'Table',
                        content: [
                            createHeatmapTable(vmCpuHeatmap)
                        ]
                    }
                ]
            });
        }
        let vmMemHeatmap = {
            label: 'VM Mem',
            metric: 'mem_util',
            items: []
        };
        for (const metricItem in metric) {
            if (['memory_proc', 'memory', 'memory_usage', 'mem_util', 'mem_percent'].includes(metric[metricItem].metric)) {
                vmMemHeatmap.items.push(...(metric[metricItem].items.filter(item => ['vm'].includes(mapping[item.item_id].subsystem) || (mapping[item.item_id].subsystem === 'server' && mapping[item.item_id].hw_type === 'linux'))));
            }
        }
        vmMemHeatmap.items.sort((a, b) => (a.percent < b.percent || a.percent === null || Number.isNaN(a.percent)) ? 1 : ((b.percent < a.percent || b.percent === null || Number.isNaN(b.percent)) ? -1 : 0));
        if (vmMemHeatmap.items && vmMemHeatmap.items.length >= 1) {
            tabs.push({
                title: 'VM Mem',
                content: [],
                subTabs: [
                    {
                        title: 'Heatmap',
                        content: [
                            {
                                type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
                                metrics: [vmMemHeatmap],
                            },
                            {
                                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                                text: "Last update at " + (this.lastUpdateAt ? formatDate(this.lastUpdateAt) : "unknown")
                            },
                        ]
                    }, {
                        title: 'Table',
                        content: [
                            createHeatmapTable(vmMemHeatmap)
                        ]
                    }
                ]
            });
        }
        let serverCpuHeatmap = {
            label: 'Server CPU%',
            metric: 'cpu_utilization_percent',
            items: []
        };
        for (const metricItem in metric) {
            if (['cpu_utilization_percent', 'utilizedProcUnitsDeductIdlePercent', 'cpu_usage_perc', 'cpu_utilization'].includes(metric[metricItem].metric)) {
                serverCpuHeatmap.items.push(...(metric[metricItem].items.filter(item => ['server', 'node', 'host', 'esxi', 'cdom'].includes(mapping[item.item_id].subsystem) && mapping[item.item_id].hw_type !== 'linux')));
            }
        }
        serverCpuHeatmap.items.sort((a, b) => (a.percent < b.percent || a.percent === null) ? 1 : ((b.percent < a.percent || b.percent === null) ? -1 : 0));
        if (serverCpuHeatmap.items && serverCpuHeatmap.items.length >= 1) {
            tabs.push({
                title: 'Server CPU',
                content: [],
                subTabs: [
                    {
                        title: 'Heatmap',
                        content: [
                            {
                                type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
                                metrics: [serverCpuHeatmap],
                            },
                            {
                                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                                text: "Last update at " + (this.lastUpdateAt ? formatDate(this.lastUpdateAt) : "unknown")
                            },
                        ]
                    }, {
                        title: 'Table',
                        content: [
                            createHeatmapTable(serverCpuHeatmap)
                        ]
                    }
                ]
            });
        }
        const checkAnyItem = async (_class, subsystem) => {
            return await this.objectItemsRepository.findOne({
                relations: {
                    hw_type_entity: true
                }, where: {
                    hw_type_entity: {
                        class: _class
                    },
                    subsystem: subsystem
                }
            }) !== null;
        };
        if (await checkAnyItem('storage', 'pool')) {
            tabs.push({
                title: 'Storage Pool',
                lazyContent: (0, MenuDefinitions_1.createLazyContentURL)('storage', 'heatmap', 'pool')
            });
            tabs.push({
                title: 'Storage Volume',
                lazyContent: (0, MenuDefinitions_1.createLazyContentURL)('storage', 'heatmap', 'volume')
            });
        }
        if (await checkAnyItem('san', 'device')) {
            tabs.push({
                title: 'SAN In',
                lazyContent: (0, MenuDefinitions_1.createLazyContentURL)('san', 'heatmap', 'in')
            });
            tabs.push({
                title: 'SAN Out',
                lazyContent: (0, MenuDefinitions_1.createLazyContentURL)('san', 'heatmap', 'out')
            });
        }
        if (await checkAnyItem('lan', 'device')) {
            tabs.push({
                title: 'LAN In',
                lazyContent: (0, MenuDefinitions_1.createLazyContentURL)('lan', 'heatmap', 'in')
            });
            tabs.push({
                title: 'LAN Out',
                lazyContent: (0, MenuDefinitions_1.createLazyContentURL)('lan', 'heatmap', 'out')
            });
        }
        return {
            title: "Global Heatmap",
            tabs: tabs
        };
    }
}
exports.CrossHeatmap = CrossHeatmap;
class HeatmapSan {
    constructor(request, itemIds) {
        this.request = request;
        this.itemIds = itemIds;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.lastUpdateAt = null;
    }
    async table(metric) {
        let portTable = new Table_1.Table()
            .addHeader({ value: 'Port' })
            .addHeader({ value: 'Switch' });
        const withDataIn = metric === undefined || metric === 'data_in_util';
        const withDataOut = metric === undefined || metric === 'data_out_util';
        if (withDataIn) {
            portTable.addHeader({ value: 'Data in util', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        }
        if (withDataOut) {
            portTable.addHeader({ value: 'Data out util', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        }
        const sanIds = this.generatedBox.metrics && this.generatedBox.metrics[0]
            ? this.generatedBox.metrics[0].items.map(i => i.item_id)
            : [];
        let ports = await this.objectItemsRepository.find({
            where: {
                item_id: (0, typeorm_1.In)(sanIds),
                parents: {
                    _parent: {
                        subsystem: 'device',
                    }
                },
            },
            relations: {
                parents: {
                    _parent: true
                }
            }
        });
        const dataInUtil = this.generatedBox.metrics.find(m => m.metric === 'data_in_util');
        const dataOutUtil = this.generatedBox.metrics.find(m => m.metric === 'data_out_util');
        for (const port of ports) {
            const device = port.parents.find(parent => parent._parent.subsystem === 'device');
            const data_in_util = dataInUtil.items.find(i => i.item_id === port.item_id);
            const data_out_util = dataOutUtil.items.find(i => i.item_id === port.item_id);
            let row = [
                { order: 0, value: port.label, url: URL_1.URL.generate(port.hw_type, 'sanport', port.item_id) },
                { order: 1, value: device ? device._parent.label : 'undefined' }
            ];
            let i = 2;
            if (withDataIn) {
                row.push({ order: i, value: data_in_util ? data_in_util.percent : null });
                i++;
            }
            if (withDataOut) {
                row.push({ order: i, value: data_out_util ? data_out_util.percent : null });
            }
            portTable.addRow(row);
        }
        portTable.sortByIndex(2);
        return portTable.generateBox();
    }
    async gen() {
        let metrics = [];
        let ports = await this.objectItemsRepository.find({
            relations: {
                hw_type_entity: true,
                heatmaps: true,
                properties: true,
                parents: {
                    _parent: true
                }
            },
            where: {
                hw_type_entity: {
                    class: 'san'
                },
                subsystem: 'sanport',
                item_id: this.itemIds ? (0, typeorm_1.In)(this.itemIds) : undefined
            }
        });
        ports = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, ports);
        const labels = {
            'data_in_util': 'Data In',
            'data_out_util': 'Data Out'
        };
        let metric = {};
        for (const heatmapItem of ports) {
            const device = heatmapItem.parents.find(parent => parent._parent.subsystem === 'device');
            if (heatmapItem.properties.find(property => property.property_name === 'operational_status' && (property.property_value === 'up' || property.property_value === 'online' || property.property_value === 'Online'))) {
                for (const heatmapData of heatmapItem.heatmaps) {
                    if (heatmapData.updatedAt > this.lastUpdateAt) {
                        this.lastUpdateAt = heatmapData.updatedAt;
                    }
                    if (metric[heatmapData.metric]) {
                        metric[heatmapData.metric]['items'].push({
                            item_id: heatmapItem.item_id,
                            label: device ? device._parent.label + ': ' + heatmapItem.label : heatmapItem.label,
                            percent: heatmapData.value,
                            url: URL_1.URL.generate(heatmapItem.hw_type, heatmapItem.subsystem, heatmapItem.item_id)
                        });
                    }
                    else {
                        metric[heatmapData.metric] = {
                            label: labels[heatmapData.metric] ? labels[heatmapData.metric] : heatmapData.metric,
                            metric: heatmapData.metric,
                            items: [{
                                    item_id: heatmapItem.item_id,
                                    label: device ? device._parent.label + ': ' + heatmapItem.label : heatmapItem.label,
                                    percent: heatmapData.value,
                                    url: URL_1.URL.generate(heatmapItem.hw_type, heatmapItem.subsystem, heatmapItem.item_id)
                                }]
                        };
                    }
                }
            }
        }
        for (const metricItem in metric) {
            metrics.push(metric[metricItem]);
        }
        for (const metricItem of metrics) {
            metricItem.items.sort((a, b) => (a.percent < b.percent) ? 1 : ((b.percent < a.percent) ? -1 : 0));
        }
        this.generatedBox = {
            type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
            metrics: metrics,
        };
        return {
            heatmapBox: this.generatedBox,
            textBox: {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                text: "Last update at " + (this.lastUpdateAt ? formatDate(this.lastUpdateAt) : "unknown")
            },
        };
    }
}
exports.HeatmapSan = HeatmapSan;
class HeatmapLan {
    constructor(request, itemIds) {
        this.request = request;
        this.itemIds = itemIds;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.lastUpdateAt = null;
    }
    async table(metric) {
        let portTable = new Table_1.Table()
            .addHeader({ value: 'Port' })
            .addHeader({ value: 'Switch' });
        const withDataIn = metric === undefined || metric === 'data_in_util';
        const withDataOut = metric === undefined || metric === 'data_out_util';
        if (withDataIn) {
            portTable.addHeader({ value: 'Data in util', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        }
        if (withDataOut) {
            portTable.addHeader({ value: 'Data out util', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        }
        const lanIds = this.generatedBox.metrics && this.generatedBox.metrics[0]
            ? this.generatedBox.metrics[0].items.map(i => i.item_id)
            : [];
        let ports = await this.objectItemsRepository.find({
            where: {
                item_id: (0, typeorm_1.In)(lanIds),
                parents: {
                    _parent: {
                        subsystem: 'device',
                    }
                },
            },
            relations: {
                parents: {
                    _parent: true
                }
            }
        });
        const dataInUtil = this.generatedBox.metrics.find(m => m.metric === 'data_in_util');
        const dataOutUtil = this.generatedBox.metrics.find(m => m.metric === 'data_out_util');
        for (const port of ports) {
            const device = port.parents.find(parent => parent._parent.subsystem === 'device');
            const data_in_util = dataInUtil.items.find(i => i.item_id === port.item_id);
            const data_out_util = dataOutUtil.items.find(i => i.item_id === port.item_id);
            let row = [
                { order: 0, value: port.label, url: URL_1.URL.generate(port.hw_type, port.subsystem, port.item_id) },
                { order: 1, value: device ? device._parent.label : 'undefined' }
            ];
            let i = 2;
            if (withDataIn) {
                row.push({ order: i, value: data_in_util ? data_in_util.percent : null });
                i++;
            }
            if (withDataOut) {
                row.push({ order: i, value: data_out_util ? data_out_util.percent : null });
            }
            portTable.addRow(row);
        }
        portTable.sortByIndex(2);
        return portTable.generateBox();
    }
    async gen() {
        let metrics = [];
        let ports = await this.objectItemsRepository.find({
            relations: {
                hw_type_entity: true,
                heatmaps: true,
                properties: true,
                parents: {
                    _parent: true
                },
                server_group_items: {
                    _server_group: true
                }
            },
            where: {
                hw_type_entity: {
                    class: 'lan'
                },
                subsystem: 'physical',
                item_id: this.itemIds ? (0, typeorm_1.In)(this.itemIds) : undefined
            }
        });
        const user = await User_1.User.getUserByRequest(this.request);
        const url = new URL_1.URL();
        ports = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, ports);
        const labels = {
            'data_in_util': 'Data In',
            'data_out_util': 'Data Out'
        };
        let metric = {};
        for (const heatmapItem of ports) {
            const device = heatmapItem.parents.find(parent => parent._parent.subsystem === 'device');
            if (heatmapItem.properties.find(property => property.property_name === 'working_state' && property.property_value === 'used')) {
                for (const heatmapData of heatmapItem.heatmaps) {
                    if (heatmapData.updatedAt > this.lastUpdateAt) {
                        this.lastUpdateAt = heatmapData.updatedAt;
                    }
                    if (metric[heatmapData.metric]) {
                        metric[heatmapData.metric]['items'].push({
                            item_id: heatmapItem.item_id,
                            label: device ? device._parent.label + ': ' + heatmapItem.label : heatmapItem.label,
                            percent: heatmapData.value,
                            url: url.generateFromObject(heatmapItem, null, null, user)
                        });
                    }
                    else {
                        metric[heatmapData.metric] = {
                            label: labels[heatmapData.metric] ? labels[heatmapData.metric] : heatmapData.metric,
                            metric: heatmapData.metric,
                            items: [{
                                    item_id: heatmapItem.item_id,
                                    label: device ? device._parent.label + ': ' + heatmapItem.label : heatmapItem.label,
                                    percent: heatmapData.value,
                                    url: url.generateFromObject(heatmapItem, null, null, user)
                                }]
                        };
                    }
                }
            }
        }
        for (const metricItem in metric) {
            metrics.push(metric[metricItem]);
        }
        for (const metricItem of metrics) {
            metricItem.items.sort((a, b) => (a.percent < b.percent) ? 1 : ((b.percent < a.percent) ? -1 : 0));
        }
        this.generatedBox = {
            type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
            metrics: metrics,
        };
        return {
            heatmapBox: this.generatedBox,
            textBox: {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                text: "Last update at " + (this.lastUpdateAt ? formatDate(this.lastUpdateAt) : "unknown")
            },
        };
    }
}
exports.HeatmapLan = HeatmapLan;
class HeatmapStorageCapacity {
    constructor(request) {
        this.request = request;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.lastUpdateAt = null;
    }
    async gen(deviceId) {
        let metrics = [];
        let items = await this.objectItemsRepository.find({
            relations: {
                hw_type_entity: true,
                heatmaps: true,
                parents: {
                    _parent: true
                }
            },
            where: {
                hw_type_entity: {
                    class: 'storage'
                },
                heatmaps: {
                    metric: 'capacity_used_proc'
                },
                subsystem: (0, typeorm_1.In)(['pool', 'site']),
                parents: deviceId ? {
                    parent: deviceId
                } : undefined
            }
        });
        if (this.request) {
            items = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, items);
        }
        let metric = {};
        for (const heatmapItem of items) {
            const device = heatmapItem.parents.find(parent => parent._parent.subsystem === 'device');
            for (const heatmapData of heatmapItem.heatmaps) {
                if (heatmapData.updatedAt > this.lastUpdateAt) {
                    this.lastUpdateAt = heatmapData.updatedAt;
                }
                if (metric[heatmapData.metric]) {
                    metric[heatmapData.metric]['items'].push({
                        item_id: heatmapItem.item_id,
                        label: device ? '[' + device._parent.label + '] ' + heatmapItem.label : heatmapItem.label,
                        percent: heatmapData.value,
                        url: URL_1.URL.generate(heatmapItem.hw_type, heatmapItem.subsystem, heatmapItem.item_id, undefined, 'Capacity')
                    });
                }
                else {
                    metric[heatmapData.metric] = {
                        label: 'Capacity Used',
                        metric: heatmapData.metric,
                        items: [{
                                item_id: heatmapItem.item_id,
                                label: device ? '[' + device._parent.label + '] ' + heatmapItem.label : heatmapItem.label,
                                percent: heatmapData.value,
                                url: URL_1.URL.generate(heatmapItem.hw_type, heatmapItem.subsystem, heatmapItem.item_id, undefined, 'Capacity')
                            }]
                    };
                }
            }
        }
        for (const metricItem in metric) {
            metrics.push(metric[metricItem]);
        }
        for (const metricItem of metrics) {
            metricItem.items.sort((a, b) => (a.percent < b.percent) ? 1 : ((b.percent < a.percent) ? -1 : 0));
        }
        return {
            heatmapBox: {
                type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
                metrics: metrics,
            },
            textBox: {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                text: "Last update at " + (this.lastUpdateAt ? formatDate(this.lastUpdateAt) : "unknown")
            },
        };
    }
}
exports.HeatmapStorageCapacity = HeatmapStorageCapacity;
class HeatmapStorage {
    constructor(request, itemIds) {
        this.request = request;
        this.itemIds = itemIds;
        this.heatmapRepository = (0, dataSource_1.getDataSource)().getRepository(heatmap_entity_1.Heatmap);
        this.heatmapTierRepository = (0, dataSource_1.getDataSource)().getRepository(heatmapTier_entity_1.HeatmapTier);
        this.heatmapTierItemRepository = (0, dataSource_1.getDataSource)().getRepository(heatmapTierItem_entity_1.HeatmapTierItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.volumes = [];
        this.pools = [];
        this.lastUpdateAt = null;
    }
    async loadItems(subsystem, parentId) {
        if (subsystem === 'volume') {
            this.volumes = await this.objectItemsRepository.find({
                relations: {
                    heatmaps: true,
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    item_id: this.itemIds ? (0, typeorm_1.In)(this.itemIds) : undefined,
                    subsystem: subsystem,
                    parents: {
                        _parent: {
                            subsystem: (0, typeorm_1.In)(['pool', 'device'])
                        }
                    },
                    heatmaps: {
                        updatedAt: (0, typeorm_1.MoreThan)(new Date(Date.now() - (90 * 60 * 1000)))
                    }
                }
            });
            if (parentId) {
                this.volumes = this.volumes.filter(volume => volume.parents.find(relation => relation.parent === parentId));
            }
            if (this.request) {
                const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, this.volumes.map(item => item.item_id));
                this.volumes = this.volumes.filter(item => ids.includes(item.item_id));
            }
        }
        else {
            this.pools = await this.objectItemsRepository.find({
                relations: {
                    heatmaps: true,
                    parents: {
                        _parent: true
                    }
                },
                where: {
                    subsystem: subsystem,
                    item_id: this.itemIds ? (0, typeorm_1.In)(this.itemIds) : undefined,
                    parents: {
                        parent: parentId
                            ? parentId
                            : undefined,
                        _parent: {
                            subsystem: 'device'
                        }
                    },
                    heatmaps: {
                        updatedAt: (0, typeorm_1.MoreThan)(new Date(Date.now() - (90 * 60 * 1000)))
                    }
                }
            });
            if (this.request) {
                const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, this.pools.map(item => item.item_id));
                this.pools = this.pools.filter(item => ids.includes(item.item_id));
            }
        }
    }
    getColor(intervals, value) {
        intervals.sort((a, b) => {
            return a.limit - b.limit;
        });
        let color = Color_1.Color.pallete.gray;
        const greaterIndex = intervals.findIndex(interval => interval.limit > value);
        if (greaterIndex !== undefined && greaterIndex !== -1) {
            if (greaterIndex > 0) {
                color = intervals[greaterIndex].color;
            }
            else if (greaterIndex === 0) {
                color = intervals[0].color;
            }
        }
        else {
            color = intervals[intervals.length - 1].color;
        }
        return color;
    }
    async tableBox(subsystem) {
        let items = [];
        let heatBox;
        if (subsystem === 'volume') {
            items = this.volumes;
            heatBox = this.volumeData;
        }
        else {
            items = this.pools;
            heatBox = this.poolData;
        }
        let columns;
        if (this.deviceId) {
            const device = await this.objectItemsRepository.findOneBy({ item_id: this.deviceId });
            columns = MetricsWrapper_1.MetricsWrapper.getMetricsFromEntity(device.hw_type, subsystem);
        }
        const tierRows = await this.heatmapTierItemRepository.find({
            relations: {
                _item: true,
                _heatmap_tier: true
            }
        });
        let headers = [
            { metric: 'resp_t_w', header: { value: 'Latency Write', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.millisecond, shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond } },
            { metric: 'write_io', header: { value: 'IO Write', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.IOPS, shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS } },
            { metric: 'resp_t_r', header: { value: 'Latency Read', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.millisecond, shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond } },
            { metric: 'read_io', header: { value: 'IO Read', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.IOPS, shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS } },
            { metric: 'resp_t', header: { value: 'Latency', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.millisecond, shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond } },
            { metric: 'io_rate', header: { value: 'IO', number: true, unit: subsystemMetricsResponse_constants_1.UNIT.IOPS, shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS } }
        ];
        let table = new Table_1.Table()
            .addHeader({ value: subsystem.charAt(0).toUpperCase() + subsystem.slice(1) })
            .addHeader({ value: 'Storage' })
            .addHeader({ value: 'Tier' });
        let h = 0;
        for (const header of headers) {
            if (!columns || columns.includes(header.metric)) {
                table.addHeader(header.header);
                h++;
            }
        }
        let tierItemsRead = {};
        let tierItemsWrite = {};
        let tierItemsTotal = {};
        for (const item of items) {
            let poolId = '';
            if (subsystem === 'volume') {
                const pool = item.parents.find(parent => parent._parent.subsystem === 'pool');
                if (pool) {
                    poolId = pool._parent.item_id;
                }
            }
            else {
                poolId = item.item_id;
            }
            const tier = tierRows.find(tierItem => tierItem.item_id === poolId);
            if (!tier) {
                continue;
            }
            const heatItemTier = heatBox.tiers[tier._heatmap_tier.label];
            if (!heatItemTier) {
                console.warn(`[StorageHeatmapTable] [${tier._heatmap_tier.label}] not found`);
                continue;
            }
            const heatItemTierItem = heatItemTier.items.find(tierItem => tierItem.item_id === item.item_id);
            tierItemsRead[item.item_id] = {
                tier_label: tier._heatmap_tier.label,
                heatmap_tier_id: tier.heatmap_tier_id,
                color: heatItemTierItem && heatItemTierItem.read ? heatItemTierItem.read.color : undefined,
                value: item.heatmaps.find(heatItem => heatItem.metric === 'resp_t_r')?.value
            };
            tierItemsWrite[item.item_id] = {
                tier_label: tier._heatmap_tier.label,
                heatmap_tier_id: tier.heatmap_tier_id,
                color: heatItemTierItem && heatItemTierItem.write ? heatItemTierItem.write.color : undefined,
                value: item.heatmaps.find(heatItem => heatItem.metric === 'resp_t_w')?.value
            };
            if (!tierItemsWrite[item.item_id].color || !tierItemsRead[item.item_id].color)
                tierItemsTotal[item.item_id] = {
                    tier_label: tier._heatmap_tier.label,
                    heatmap_tier_id: tier.heatmap_tier_id,
                    color: heatItemTierItem && heatItemTierItem.total ? heatItemTierItem.total.color : undefined,
                    value: item.heatmaps.find(heatItem => heatItem.metric === 'resp_t')?.value
                };
        }
        const heatMetrics = ['resp_t', 'resp_t_w', 'resp_t_r', 'io_rate', 'write_io', 'read_io'];
        for (const item of items) {
            let heatItems = {};
            for (const heatMetric of heatMetrics) {
                const heatmapRow = item.heatmaps.find(heatItem => heatItem.metric === heatMetric);
                if (heatmapRow && Date.now() - heatmapRow.updatedAt.getTime() < (90 * 60 * 1000)) {
                    heatItems[heatMetric] = item.heatmaps.find(heatItem => heatItem.metric === heatMetric);
                }
            }
            const storage = item.parents.find(parent => parent._parent.subsystem === 'device');
            let tableRow = [
                { order: 0, value: item.label, url: URL_1.URL.generate(item.hw_type, item.subsystem, item.item_id) },
                { order: 1, value: storage ? storage._parent.label : '' },
                { order: 2, value: tierItemsRead[item.item_id] ? tierItemsRead[item.item_id].tier_label : '' }
            ];
            const metricRows = [
                { metric: 'resp_t_w', row: { value: heatItems['resp_t_w'] ? heatItems['resp_t_w']['value'] : null, bgColor: tierItemsWrite[item.item_id] ? tierItemsWrite[item.item_id].color : undefined } },
                { metric: 'write_io', row: { value: heatItems['write_io'] ? heatItems['write_io']['value'] : null } },
                { metric: 'resp_t_r', row: { value: heatItems['resp_t_r'] ? heatItems['resp_t_r']['value'] : null, bgColor: tierItemsRead[item.item_id] ? tierItemsRead[item.item_id].color : undefined } },
                { metric: 'read_io', row: { value: heatItems['read_io'] ? heatItems['read_io']['value'] : null } },
                { metric: 'resp_t', row: { value: heatItems['resp_t'] ? heatItems['resp_t']['value'] : null, bgColor: tierItemsTotal[item.item_id] ? tierItemsTotal[item.item_id].color : undefined } },
                { metric: 'io_rate', row: { value: heatItems['io_rate'] ? heatItems['io_rate']['value'] : null } }
            ];
            let i = 3;
            for (const row of metricRows) {
                if (!columns || columns.includes(row.metric)) {
                    tableRow.push({ order: i, ...row.row });
                    i++;
                }
            }
            table.addRow(tableRow);
        }
        if (h === 6) {
            table.sortByIndex(7);
        }
        else {
            table.sortByIndex(3);
        }
        table.setBoxExport("storage-heatmap");
        return table.generateBox();
    }
    async heatmapBox(subsystem, deviceId) {
        await this.loadItems(subsystem, deviceId ? deviceId : undefined);
        const data = await this.heatmapStorageBox(subsystem);
        if (subsystem === 'volume') {
            this.volumeData = data.heatmapBox;
        }
        else {
            this.poolData = data.heatmapBox;
        }
        if (deviceId) {
            this.deviceId = deviceId;
        }
        return data;
    }
    async heatmapStorageBox(subsystem) {
        let poolVolumesMapping = {};
        if (this.pools.length === 0 && this.volumes.length === 0) {
            await this.loadItems(subsystem);
        }
        let objectItems = [];
        if (subsystem === 'volume') {
            objectItems = this.volumes;
            for (const volume of objectItems) {
                const poolId = volume.parents.find(parent => parent._parent.subsystem === 'pool')?.parent;
                if (poolId) {
                    if (!poolVolumesMapping[poolId]) {
                        poolVolumesMapping[poolId] = [volume.item_id];
                    }
                    else {
                        poolVolumesMapping[poolId].push(volume.item_id);
                    }
                }
            }
        }
        else {
            objectItems = this.pools;
        }
        let heatmapItems = [];
        for (const objectItem of objectItems) {
            for (const heatmapRow of objectItem.heatmaps) {
                if (heatmapRow.updatedAt > this.lastUpdateAt) {
                    this.lastUpdateAt = heatmapRow.updatedAt;
                }
                heatmapItems.push(heatmapRow);
            }
        }
        const heatmapTiers = await this.heatmapTierRepository.find({
            relations: {
                _heatmap_tier_items: true
            },
            where: {
                _heatmap_tier_items: {
                    item_id: subsystem === 'volume' ? (0, typeorm_1.In)(Object.keys(poolVolumesMapping)) : (0, typeorm_1.In)(objectItems.map((item) => item.item_id))
                }
            }
        });
        let data = {};
        const processHeatmapItem = (objectItem, heatmapMetrics, read_intervals, write_intervals, limit) => {
            let storageHeatmapItem = {
                item_id: objectItem.item_id,
                device: heatmapMetrics[0].device,
                label: objectItem.label,
                url: URL_1.URL.generate(objectItem.hw_type, objectItem.subsystem, objectItem.item_id)
            };
            const resp_t_r = heatmapMetrics.find(heatmapMetric => heatmapMetric.metric === 'resp_t_r');
            const read_io = heatmapMetrics.find(heatmapMetric => heatmapMetric.metric === 'read_io');
            if (resp_t_r && read_io && read_io.value > limit) {
                storageHeatmapItem.read = {
                    line: `${resp_t_r.device}: ${objectItem.label}: read ${resp_t_r.value} ms, ${read_io.value} IOPS`,
                    color: this.getColor(read_intervals, resp_t_r.value),
                    value: resp_t_r.value
                };
            }
            const resp_t_w = heatmapMetrics.find(heatmapMetric => heatmapMetric.metric === 'resp_t_w');
            const write_io = heatmapMetrics.find(heatmapMetric => heatmapMetric.metric === 'write_io');
            if (resp_t_w && write_io && write_io.value > limit) {
                storageHeatmapItem.write = {
                    line: `${resp_t_w.device}: ${objectItem.label}: write ${resp_t_w.value} ms, ${write_io.value} IOPS`,
                    color: this.getColor(write_intervals, resp_t_w.value),
                    value: resp_t_w.value
                };
            }
            if (!resp_t_w && !resp_t_r) {
                const resp_t = heatmapMetrics.find(heatmapMetric => heatmapMetric.metric === 'resp_t');
                const io_rate = heatmapMetrics.find(heatmapMetric => heatmapMetric.metric === 'io_rate');
                if (resp_t && io_rate && io_rate.value > limit) {
                    storageHeatmapItem.total = {
                        line: `${resp_t.device}: ${objectItem.label}: write ${resp_t.value} ms, ${io_rate.value} IOPS`,
                        color: this.getColor(read_intervals, resp_t.value),
                        value: resp_t.value
                    };
                }
            }
            return storageHeatmapItem;
        };
        for (const heatmapTier of heatmapTiers) {
            data[heatmapTier.label] = {
                read_intervals: heatmapTier.read_intervals,
                write_intervals: heatmapTier.write_intervals,
                items: []
            };
            for (const heatmapTierItem of heatmapTier._heatmap_tier_items) {
                if (subsystem === 'volume') {
                    for (const volumeId of poolVolumesMapping[heatmapTierItem.item_id]) {
                        const objectItem = objectItems.find((item) => item.item_id === volumeId);
                        const heatmapMetrics = heatmapItems.filter(item => item.item_id === volumeId);
                        if (heatmapMetrics.length >= 1) {
                            const storageHeatmapItem = processHeatmapItem(objectItem, heatmapMetrics, heatmapTier.read_intervals, heatmapTier.write_intervals, heatmapTier.limit_volume);
                            if (storageHeatmapItem.read || storageHeatmapItem.write || storageHeatmapItem.total) {
                                data[heatmapTier.label].items.push(storageHeatmapItem);
                            }
                        }
                    }
                }
                else {
                    const objectItem = objectItems.find((item) => item.item_id === heatmapTierItem.item_id);
                    const heatmapMetrics = heatmapItems.filter(item => item.item_id === heatmapTierItem.item_id);
                    if (heatmapMetrics.length >= 1) {
                        const storageHeatmapItem = processHeatmapItem(objectItem, heatmapMetrics, heatmapTier.read_intervals, heatmapTier.write_intervals, heatmapTier.limit_pool);
                        if (storageHeatmapItem.read || storageHeatmapItem.write || storageHeatmapItem.total) {
                            data[heatmapTier.label].items.push(storageHeatmapItem);
                        }
                    }
                }
                data[heatmapTier.label].items.forEach(i => {
                    if (i.total?.value !== undefined) {
                        i.sortValue = i.total?.value;
                    }
                    else if (i.write?.value !== undefined) {
                        i.sortValue = i.write?.value;
                    }
                    else if (i.read?.value !== undefined) {
                        i.sortValue = i.read?.value;
                    }
                    else {
                        i.sortValue = 0;
                    }
                });
                data[heatmapTier.label].items.sort((a, b) => (a.sortValue < b.sortValue) ? 1 : ((b.sortValue < a.sortValue) ? -1 : 0));
            }
        }
        return {
            heatmapBox: {
                type: pageResponse_constants_1.BOX_TYPE.StorageHeatmapBox,
                tiers: data,
            },
            textBox: {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                text: "Last update at " + (this.lastUpdateAt ? formatDate(this.lastUpdateAt) : "unknown")
            },
        };
    }
    async getTierColor(tierItems, interval, subsystem) {
        try {
            const heatmapTiers = await (0, dataSource_1.getDataSource)().getRepository(heatmapTier_entity_1.HeatmapTier).find();
            const heatmapTierColors = {};
            let heatmapTiersMapped = {};
            for (const heatmapTier of heatmapTiers) {
                heatmapTiersMapped[heatmapTier.heatmap_tier_id] = heatmapTier;
                heatmapTierColors[heatmapTier.heatmap_tier_id] = heatmapTier[interval];
                heatmapTierColors[heatmapTier.heatmap_tier_id].sort((a, b) => a.limit - b.limit);
            }
            const limitMetric = "write_intervals" ? 'write_io' : 'read_io';
            const maxAvgValues = await new MetricsWrapper_1.MetricsWrapper([limitMetric], Object.keys(tierItems)).getAvgAndMaxByTime((Date.now() / 1000) - (90 * 60), Date.now() / 1000);
            for (const itemId in tierItems) {
                if (tierItems[itemId].value && tierItems[itemId].heatmap_tier_id && heatmapTierColors[tierItems[itemId].heatmap_tier_id]) {
                    const limit = heatmapTiersMapped[tierItems[itemId].heatmap_tier_id][`limit_${subsystem}`];
                    if (maxAvgValues[itemId] && maxAvgValues[itemId][limitMetric]['avg'] !== undefined && maxAvgValues[itemId][limitMetric]['avg'] < limit) {
                        tierItems[itemId].color = Color_1.Color.pallete.gray;
                    }
                    else {
                        const greaterIndex = heatmapTierColors[tierItems[itemId].heatmap_tier_id].findIndex(interval => interval.limit > tierItems[itemId].value);
                        if (greaterIndex && greaterIndex !== -1) {
                            if (greaterIndex > 0) {
                                tierItems[itemId].color = heatmapTierColors[tierItems[itemId].heatmap_tier_id][greaterIndex - 1].color;
                            }
                            else {
                                tierItems[itemId].color = heatmapTierColors[tierItems[itemId].heatmap_tier_id][greaterIndex].color;
                            }
                        }
                        else {
                            tierItems[itemId].color = heatmapTierColors[tierItems[itemId].heatmap_tier_id][heatmapTierColors[tierItems[itemId].heatmap_tier_id].length - 1].color;
                        }
                    }
                }
            }
            return tierItems;
        }
        catch (e) {
            console.error(e);
            return tierItems;
        }
    }
}
exports.HeatmapStorage = HeatmapStorage;
class Heatmap {
    constructor(hw_type, subsystem, table = false, itemIds) {
        this.metrics = [];
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.lastUpdateAt = null;
        this.hw_type = hw_type;
        this.subsystem = subsystem;
        this.table = table;
        if (itemIds) {
            this.itemIds = itemIds;
        }
    }
    addMetric(metric) {
        this.metrics.push(metric);
        return this;
    }
    async generateUrl(it, request, tabLabel) {
        if (it.server_group_items && it.server_group_items.length) {
            const group = it.server_group_items.find(sg => sg._server_group.user_id === request.session['userId']);
            return URL_1.URL.generate(it.hw_type, it.subsystem, group ? it.item_id + ":::" + group.server_group_id : it.item_id, undefined, tabLabel);
        }
        return URL_1.URL.generate(it.hw_type, it.subsystem, it.item_id, undefined, tabLabel);
    }
    async computedHeatmapBox(_class, request) {
        let metrics = [];
        const hwTypes = this.hw_type === 'power' && this.subsystem === 'vm'
            ? ['linux', 'power']
            : [this.hw_type];
        const subsystems = this.hw_type === 'power' && this.subsystem === 'vm'
            ? ['vm', 'server']
            : [this.subsystem];
        let heatmapItems = await this.objectItemsRepository.find({
            where: this.itemIds ? {
                item_id: (0, typeorm_1.In)(this.itemIds)
            } : {
                hw_type: (0, typeorm_1.In)(hwTypes),
                subsystem: (0, typeorm_1.In)(subsystems),
                item_id: this.hw_type === 'linux'
                    ? (0, typeorm_1.Not)((0, typeorm_1.In)(await (0, Linux_1.getLinuxExcludedIds)()))
                    : undefined
            },
            relations: {
                heatmaps: true,
                server_group_items: this.hw_type === "linux" ? {
                    _server_group: true
                } : undefined
            },
            order: {
                heatmaps: {
                    metric: 'ASC'
                }
            }
        });
        if (this.hw_type === 'power' && this.subsystem === 'vm') {
            let heatmapItemsMapped = [];
            let linuxMapping = {};
            const linuxMappingItems = await (0, Linux_1.getLinuxMapping)();
            for (const linuxMappingItem of linuxMappingItems) {
                linuxMapping[linuxMappingItem.agent_id] = linuxMappingItem.item_id;
            }
            for (const heatmapItem of heatmapItems) {
                if (heatmapItem.hw_type === 'linux') {
                    if (linuxMapping[heatmapItem.item_id] && heatmapItem.item_id.startsWith("power_")) {
                        heatmapItemsMapped.push({
                            ...heatmapItem,
                            item_id: linuxMapping[heatmapItem.item_id],
                            subsystem: 'vm',
                            hw_type: 'power'
                        });
                    }
                }
                else {
                    heatmapItemsMapped.push(heatmapItem);
                }
            }
            heatmapItems = heatmapItemsMapped;
        }
        if (request) {
            heatmapItems = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(request, heatmapItems);
        }
        const itemsWithRelations = await this.objectItemsRepository.find({
            relations: {
                parents: {
                    _parent: true
                }
            },
            where: {
                item_id: (0, typeorm_1.In)(heatmapItems.map(i => i.item_id))
            }
        });
        this.itemsWithParents = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(itemsWithRelations));
        let metric = {};
        const labels = !this.hw_type && _class == class_constants_1.ClassKey.VIRTUALIZATION
            ? { 'cpu_utilization_percent': 'CPU' }
            : MetricsWrapper_1.MetricsWrapper.getHeatmapLabels(class_constants_1.classDefinitionLabel[_class], this.hw_type, this.subsystem);
        for (const metricKey in labels) {
            metric[metricKey] = {
                label: labels[metricKey],
                metric: metricKey,
                items: []
            };
        }
        for (const heatmapItem of heatmapItems) {
            for (const heatmapData of heatmapItem.heatmaps) {
                if (heatmapData.value === undefined || heatmapData.value === null) {
                    continue;
                }
                if (heatmapData.updatedAt > this.lastUpdateAt) {
                    this.lastUpdateAt = heatmapData.updatedAt;
                }
                let parent;
                const itemWithParents = this.itemsWithParents.find(item => item.item_id === heatmapItem.item_id);
                if (this.parentSubsystem) {
                    const itemParent = itemWithParents.parents.find(p => p.subsystem === this.parentSubsystem);
                    if (itemParent) {
                        parent = itemParent.label;
                    }
                }
                else if (itemWithParents.root_parent) {
                    parent = itemWithParents.root_parent.label;
                }
                if (metric[heatmapData.metric]) {
                    metric[heatmapData.metric]['items'].push({
                        item_id: heatmapItem.item_id,
                        label: parent ? `[${parent}]: ${heatmapItem.label}` : heatmapItem.label,
                        percent: heatmapData.value,
                        url: await this.generateUrl(heatmapItem, request, metric[heatmapData.metric]?.label)
                    });
                }
                else {
                    if (labels[heatmapData.metric]) {
                        metric[heatmapData.metric] = {
                            label: labels[heatmapData.metric],
                            metric: heatmapData.metric,
                            items: [{
                                    item_id: heatmapItem.item_id,
                                    label: parent ? `[${parent}]: ${heatmapItem.label}` : heatmapItem.label,
                                    percent: heatmapData.value,
                                    url: await this.generateUrl(heatmapItem, request, metric[heatmapData.metric]?.label)
                                }]
                        };
                    }
                }
            }
        }
        for (const metricItem in metric) {
            metrics.push(metric[metricItem]);
        }
        for (const metricItem of metrics) {
            metricItem.items.sort((a, b) => (a.percent < b.percent || a.percent === null || Number.isNaN(a.percent)) ? 1 : ((b.percent < a.percent || b.percent === null || Number.isNaN(b.percent)) ? -1 : 0));
        }
        this.box = {
            type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
            metrics: metrics,
        };
        return {
            heatmapBox: {
                type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
                metrics: metrics,
            },
            textBox: {
                type: pageResponse_constants_1.BOX_TYPE.TextBox,
                text: "Last update at " + (this.lastUpdateAt ? formatDate(this.lastUpdateAt) : "unknown")
            },
        };
    }
    addParent(subsystem) {
        this.parentSubsystem = subsystem;
    }
    genTable() {
        let table = new Table_1.Table()
            .addHeader({ value: 'Parent' })
            .addHeader({ value: 'VM' });
        for (const metric of this.box.metrics) {
            table.addHeader({ value: metric.label, unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        }
        let items = {};
        for (const data of this.box.metrics) {
            for (const item of data.items) {
                if (!items[item.item_id]) {
                    items[item.item_id] = {
                        label: item.label.includes("]: ")
                            ? item.label.split("]: ")[1]
                            : item.label,
                        url: item.url,
                        metrics: [{
                                label: data.label,
                                metric: data.metric,
                                value: item.percent
                            }]
                    };
                }
                else {
                    items[item.item_id].metrics.push({
                        label: data.label,
                        metric: data.metric,
                        value: item.percent
                    });
                }
            }
        }
        for (const itemId in items) {
            let parent;
            const itemWithParents = this.itemsWithParents.find(item => item.item_id === itemId);
            if (this.parentSubsystem) {
                const itemParent = itemWithParents.parents.find(p => p.subsystem === this.parentSubsystem);
                if (itemParent) {
                    parent = itemParent;
                }
            }
            else if (itemWithParents.root_parent) {
                parent = itemWithParents.root_parent;
            }
            let row = [
                { order: 0, value: items[itemId].label, url: items[itemId].url },
                { order: 1, value: parent ? parent.label : '', url: parent ? URL_1.URL.generate(parent.hw_type, parent.subsystem, parent.item_id) : undefined }
            ];
            let i = 2;
            for (const metric of this.box.metrics) {
                const iMetric = items[itemId].metrics.find(m => m.metric === metric.metric);
                row.push({
                    order: i, value: iMetric ? iMetric.value : null
                });
            }
            table.addRow(row);
        }
        table.sortByIndex(2);
        table.setBoxExport("heatmap");
        return table.generateBox();
    }
    async virtHeatmapBox(request) {
        return this.computedHeatmapBox(class_constants_1.ClassKey.VIRTUALIZATION, request);
    }
    async heatmapBox() {
        let metrics = [];
        for (const metric of this.metrics) {
            if (metric.metricUsed && metric.metricTotal) {
                metrics.push({
                    metric: metric.label,
                    items: await this.getMetricByUsedAndTotal(metric.metricUsed, metric.metricTotal),
                });
            }
            else if (metric.metricFree && metric.metricTotal) {
                metrics.push({
                    metric: metric.label,
                    items: await this.getMetricByFreeAndTotal(metric.metricFree, metric.metricTotal),
                });
            }
            else if (metric.metric) {
                metrics.push({
                    metric: metric.label,
                    items: await this.getMetric(metric.metric),
                });
            }
        }
        return {
            type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
            metrics: metrics,
        };
    }
    async getData(isProxy) {
        let metrics = [];
        for (const metric of this.metrics) {
            if (metric.metricUsed && metric.metricTotal) {
                metrics.push({
                    metric: metric.label,
                    items: await this.getMetricByUsedAndTotal(metric.metricUsed, metric.metricTotal, isProxy),
                });
            }
            else if (metric.metricFree && metric.metricTotal) {
                metrics.push({
                    metric: metric.label,
                    items: await this.getMetricByFreeAndTotal(metric.metricFree, metric.metricTotal, isProxy),
                });
            }
            else if (metric.metric) {
                metrics.push({
                    metric: metric.label,
                    items: await this.getMetric(metric.metric, isProxy),
                });
            }
        }
        return metrics;
    }
    async getList() {
        let items = {};
        for (const metric of this.metrics) {
            if (metric.metricUsed && metric.metricTotal) {
                items[metric.label] = await this.getMetricByUsedAndTotalSimpleItem(metric.metricUsed, metric.metricTotal);
            }
            else if (metric.metric) {
                items[metric.label] = await this.getMetricSimpleItem(metric.metric);
            }
        }
        return items;
    }
    async getMetricByUsedAndTotal(metricUsed, metricTotal, isProxy) {
        let items = [];
        const itemsFromDB = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hw_type,
                subsystem: this.subsystem
            },
            relations: {
                hostcfg_relations: isProxy ? {
                    _hostcfg: {
                        destinations: {
                            _target_destination: true
                        }
                    }
                } : undefined
            }
        });
        const itemsIds = itemsFromDB.map((item) => item.item_id);
        const values = await (new MetricsWrapper_1.MetricsWrapper([metricUsed, metricTotal], itemsIds)).getLastHourValueFromTable();
        for (const itemFromDB of itemsFromDB) {
            const percent = values[itemFromDB.item_id] ? values[itemFromDB.item_id][metricUsed] / (values[itemFromDB.item_id][metricTotal] / 100) : undefined;
            items.push({
                item_id: itemFromDB.item_id,
                label: itemFromDB.label,
                percent: percent !== undefined ? percent : null,
                url: URL_1.URL.generate(this.hw_type, this.subsystem, itemFromDB.item_id),
                hostcfg_relations: itemFromDB.hostcfg_relations ? itemFromDB.hostcfg_relations : []
            });
        }
        return items;
    }
    async getMetricByFreeAndTotal(metricFree, metricTotal, isProxy) {
        let items = [];
        const itemsFromDB = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hw_type,
                subsystem: this.subsystem
            },
            relations: {
                hostcfg_relations: {
                    _hostcfg: {
                        destinations: {
                            _target_destination: true
                        }
                    }
                }
            }
        });
        const itemsIds = itemsFromDB.map((item) => item.item_id);
        const values = await (new MetricsWrapper_1.MetricsWrapper([metricFree, metricTotal], itemsIds)).getLastHourValueFromTable();
        for (const itemFromDB of itemsFromDB) {
            const percent = values[itemFromDB.item_id] ? (values[itemFromDB.item_id][metricTotal] - values[itemFromDB.item_id][metricFree]) / (values[itemFromDB.item_id][metricTotal] / 100) : undefined;
            items.push({
                item_id: itemFromDB.item_id,
                label: itemFromDB.label,
                percent: percent !== undefined ? percent : null,
                url: URL_1.URL.generate(this.hw_type, this.subsystem, itemFromDB.item_id),
                hostcfg_relations: itemFromDB.hostcfg_relations ? itemFromDB.hostcfg_relations : []
            });
        }
        return items;
    }
    async getMetricByUsedAndTotalSimpleItem(metricUsed, metricTotal) {
        const itemsFromDB = await this.objectItemsRepository.findBy({ hw_type: this.hw_type, subsystem: this.subsystem });
        const itemsIds = itemsFromDB.map((item) => item.item_id);
        const values = await (new MetricsWrapper_1.MetricsWrapper([metricUsed, metricTotal], itemsIds)).getLastHourValueFromTable();
        let metrics = {};
        for (const itemFromDB of itemsFromDB) {
            const percent = values[itemFromDB.item_id] !== undefined ? values[itemFromDB.item_id][metricUsed] / (values[itemFromDB.item_id][metricTotal] / 100) : undefined;
            metrics[itemFromDB.item_id] = percent ? percent : null;
        }
        return metrics;
    }
    async getMetric(metric, isProxy) {
        let items = [];
        const itemsFromDB = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hw_type,
                subsystem: this.subsystem
            },
            relations: {
                hostcfg_relations: isProxy ? {
                    _hostcfg: {
                        destinations: {
                            _target_destination: true
                        }
                    }
                } : undefined
            }
        });
        const itemsIds = itemsFromDB.map((item) => item.item_id);
        const values = await (new MetricsWrapper_1.MetricsWrapper([metric], itemsIds)).getLastHourValueFromTable();
        for (const itemFromDB of itemsFromDB) {
            const percent = values[itemFromDB.item_id] ? values[itemFromDB.item_id][metric] !== null ? +values[itemFromDB.item_id][metric] : null : undefined;
            items.push({
                item_id: itemFromDB.item_id,
                label: itemFromDB.label,
                percent: percent !== undefined ? percent : null,
                url: URL_1.URL.generate(this.hw_type, this.subsystem, itemFromDB.item_id),
                hostcfg_relations: itemFromDB.hostcfg_relations ? itemFromDB.hostcfg_relations : []
            });
        }
        return items;
    }
    async getMetricSimpleItem(metric) {
        const itemsFromDB = await this.objectItemsRepository.findBy({ hw_type: this.hw_type, subsystem: this.subsystem });
        const itemsIds = itemsFromDB.map((item) => item.item_id);
        const values = await (new MetricsWrapper_1.MetricsWrapper([metric], itemsIds)).getLastHourValueFromTable();
        let metrics = {};
        for (const itemFromDB of itemsFromDB) {
            const percent = +values[itemFromDB.item_id][metric];
            metrics[itemFromDB.item_id] = percent !== undefined ? percent : null;
        }
        return metrics;
    }
}
exports.Heatmap = Heatmap;
//# sourceMappingURL=Heatmap.js.map