"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PerformanceTable = void 0;
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const dataSource_1 = require("../../../config/dataSource");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const typeorm_1 = require("typeorm");
const TableHeaderGroup_1 = require("./TableHeaderGroup");
const Color_1 = require("../../metrics/Color");
const Definitions_1 = require("../../metrics/Definitions");
class PerformanceTable {
    constructor(pTGroups, startEnd, itemLabel) {
        this.groups = [];
        this.itemIds = [];
        this.isHeated = false;
        this.groups = pTGroups;
        this.startEnd = startEnd;
        if (itemLabel) {
            this.itemLabel = itemLabel;
        }
    }
    static async createPerformanceTableFromTemplate(label, headers, items) {
        const table = new TableHeaderGroup_1.TableHeaderGroup();
        table.addHeader({ value: label });
        let metrics = [];
        for (const header of headers) {
            for (const subheader of header.subheader) {
                table.addHeader({ value: subheader.label, group: header.label, number: true });
                metrics.push(subheader.metric);
            }
            if (!header.metricType) {
                header.metricType = Definitions_1.metricTypes.capacity;
            }
            table.addGroup({
                group: header.label,
                unit: header.metricType.unit,
                shortcut: header.metricType.shortcut,
                prefix: header.metricType.defaultPrefix,
                number: true,
                decimal: header.metricType.defaultPrecision,
                trim: false
            });
        }
        const lastDailyValues = await (new MetricsWrapper_1.MetricsWrapper(metrics, items.map(item => item.item_id))).getLastDailyValuesFromTable();
        for (const item of items) {
            const row = [{ order: 0, value: item.label }];
            let i = 1;
            for (const header of headers) {
                for (const subheader of header.subheader) {
                    row.push({
                        order: i,
                        value: lastDailyValues[item.item_id] && lastDailyValues[item.item_id][subheader.metric] !== undefined
                            ? lastDailyValues[item.item_id][subheader.metric]
                            : null
                    });
                    i++;
                }
            }
            table.addRow(row);
        }
        return table.generateBox();
    }
    setHeated() {
        this.isHeated = true;
    }
    addItemIds(itemIds) {
        this.itemIds = itemIds;
    }
    async genTable() {
        const table = new TableHeaderGroup_1.TableHeaderGroup();
        let relations;
        if (this.isHeated) {
            relations = {
                heatmap_tier_item: {
                    _heatmap_tier: true
                }
            };
        }
        const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_1.In)(this.itemIds)
            },
            relations: relations ? relations : undefined
        });
        const maxAvgMetrics = this.groups.map(group => group.metric);
        if (maxAvgMetrics.find(metric => metric.includes("resp_t_r"))) {
            maxAvgMetrics.push('read_io');
        }
        if (maxAvgMetrics.find(metric => metric.includes("resp_t_w"))) {
            maxAvgMetrics.push('write_io');
        }
        if (maxAvgMetrics.find(metric => metric.includes("resp_t"))) {
            maxAvgMetrics.push('io_rate');
        }
        const maxAvgValues = await new MetricsWrapper_1.MetricsWrapper(maxAvgMetrics, this.itemIds).getAvgAndMaxByTime(this.startEnd.start, this.startEnd.end);
        table.addHeader({ value: this.itemLabel ? this.itemLabel : items[0] ? items[0].subsystem : 'Item' });
        for (const group of this.groups) {
            table.addHeader({ value: 'avg', group: group.group, number: true })
                .addHeader({ value: 'max', group: group.group, number: true });
            table.addGroup({
                group: group.group,
                unit: group.metricType.unit,
                shortcut: group.metricType.shortcut,
                prefix: group.metricType.defaultPrefix,
                number: true,
                decimal: group.metricType.defaultPrecision,
                trim: false
            });
        }
        for (const item of items) {
            const row = [{ order: 0, value: item.label }];
            let i = 1;
            for (const group of this.groups) {
                let bgColor;
                if (['resp_t_r', 'resp_t_w', 'resp_t'].includes(group.metric) && item.heatmap_tier_item?._heatmap_tier && maxAvgValues[item.item_id][group.metric] && maxAvgValues[item.item_id][group.metric]['avg'] !== undefined) {
                    let intervals;
                    let underLimit = false;
                    if (group.metric === 'resp_t' && (!maxAvgValues[item.item_id]['read_io'] && !maxAvgValues[item.item_id]['write_io'])) {
                        intervals = item.heatmap_tier_item?._heatmap_tier.read_intervals;
                        const limit = item.heatmap_tier_item?._heatmap_tier.limit_pool;
                        if (maxAvgValues[item.item_id]['io_rate'] && maxAvgValues[item.item_id]['io_rate']['avg'] < limit) {
                            underLimit = true;
                        }
                    }
                    else if (group.metric === 'resp_t_r' && maxAvgValues[item.item_id]['read_io']) {
                        intervals = item.heatmap_tier_item?._heatmap_tier.read_intervals;
                        const limit = item.heatmap_tier_item?._heatmap_tier.limit_pool;
                        if (maxAvgValues[item.item_id]['read_io']['avg'] !== undefined && maxAvgValues[item.item_id]['read_io']['avg'] < limit) {
                            underLimit = true;
                        }
                    }
                    else if (group.metric === 'resp_t_w' && maxAvgValues[item.item_id]['write_io']) {
                        intervals = item.heatmap_tier_item?._heatmap_tier.write_intervals;
                        const limit = item.heatmap_tier_item?._heatmap_tier.limit_pool;
                        if (maxAvgValues[item.item_id]['write_io']['avg'] !== undefined && maxAvgValues[item.item_id]['write_io']['avg'] < limit) {
                            underLimit = true;
                        }
                    }
                    if (underLimit) {
                        bgColor = Color_1.Color.pallete.gray;
                    }
                    else if (intervals !== undefined) {
                        intervals.sort((a, b) => a.limit - b.limit);
                        const greaterIndex = intervals.findIndex(interval => interval.limit > maxAvgValues[item.item_id][group.metric]['avg']);
                        if (greaterIndex !== undefined && greaterIndex !== -1) {
                            if (greaterIndex > 0) {
                                bgColor = intervals[greaterIndex].color;
                            }
                            else if (greaterIndex === 0) {
                                bgColor = intervals[0].color;
                            }
                        }
                        else {
                            bgColor = intervals[intervals.length - 1].color;
                        }
                    }
                }
                row.push({
                    order: i,
                    value: maxAvgValues[item.item_id][group.metric]
                        ? group.metricType.unit === subsystemMetricsResponse_constants_1.UNIT.IOPS
                            ? Math.round(maxAvgValues[item.item_id][group.metric]['avg'])
                            : maxAvgValues[item.item_id][group.metric]['avg']
                        : null,
                    bgColor: bgColor ? bgColor : undefined
                });
                i++;
                row.push({
                    order: i,
                    value: maxAvgValues[item.item_id][group.metric]
                        ? group.metricType.unit === subsystemMetricsResponse_constants_1.UNIT.IOPS
                            ? Math.round(maxAvgValues[item.item_id][group.metric]['max'])
                            : maxAvgValues[item.item_id][group.metric]['max']
                        : null
                });
                i++;
            }
            table.addRow(row);
        }
        return table.generateBox();
    }
}
exports.PerformanceTable = PerformanceTable;
//# sourceMappingURL=PerformanceTable.js.map