"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RubrikMetrics = exports.RubrikPoolFolder = exports.RubrikPoolcapacityPage = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const Stor_1 = require("./Stor");
const Color_1 = require("../../metrics/Color");
class RubrikPoolcapacityPage extends Stor_1.StorPage {
    constructor(itemId) {
        console.log(itemId);
        super('rubrik');
        this.title = 'Pool Capacity';
        this.itemId = itemId;
    }
    async getPage() {
        let title = 'Pool Capacity';
        let tabs = [];
        if (this.itemId) {
            const itemIds = (await this.objectItemsRepository.find({
                relations: {
                    parents: true
                },
                where: {
                    parents: {
                        parent: this.itemId.toString()
                    },
                    hw_type: 'rubrik',
                    subsystem: 'sla_domain'
                }
            })).map(item => item.item_id);
            tabs.push({
                title: 'SLA',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new RubrikMetrics.Metric('sla_domain', 'capacity_used')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, Color_1.Color.pallete.red),
                        ],
                        uuids: itemIds,
                        aggregated: true,
                    }
                ],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.RubrikPoolcapacityPage = RubrikPoolcapacityPage;
class RubrikPoolFolder extends Stor_1.StorSubsystemFolder {
    constructor(itemId) {
        super('rubrik', 'pool', itemId);
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('IO', this.hwType + '/' + this.subsystem + '-io?item_id=' + this.itemId, 'io'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Data', this.hwType + '/' + this.subsystem + '-data?item_id=' + this.itemId, 'data'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Replication', this.hwType + '/' + this.subsystem + '-replication?item_id=' + this.itemId, 'replication'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Capacity', this.hwType + '/' + this.subsystem + '-capacity?item_id=' + this.itemId, 'capacity'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Block size', this.hwType + '/' + this.subsystem + '-block-size?item_id=' + this.itemId, 'block-size'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Archive', this.hwType + '/' + this.subsystem + '-archive?item_id=' + this.itemId, 'archive'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Ingest', this.hwType + '/' + this.subsystem + '-ingest?item_id=' + this.itemId, 'ingest'));
        return menuItems;
    }
}
exports.RubrikPoolFolder = RubrikPoolFolder;
class RubrikMetrics {
}
exports.RubrikMetrics = RubrikMetrics;
RubrikMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(subsystem, metric) {
        super();
        this.metric = metric;
        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.STORAGE, hwTypeKey_constants_1.HwTypeKey.RUBRIK, subsystem, metric);
        if (definition) {
            Object.assign(this, definition);
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
    }
};
//# sourceMappingURL=Rubrik.js.map