"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SanMetrics = exports.SanHealthstatusPage = exports.SanHeatmapPage = exports.SanIslPage = exports.SanSantopologyPage = exports.SanConfigurationPage = exports.SanCpumemoryPage = exports.SanFabricPage = exports.SanFramePage = exports.SanDataPage = exports.SanTotalsPage = exports.SanSanportcpumemoryPage = exports.SanItemPage = exports.SanSingleItemPage = exports.SimpleSanPage = exports.SanPage = exports.SanSubsystemItemsFolder = exports.SanDeviceFolder = exports.SanFabricFolder = exports.SanRootFolder = exports.SanSubsystemFolder = exports.SanFolder = exports.sanGlobalTopologyBox = exports.sanDeviceTopologyBox = exports.sanGlobalPages = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const Classes_1 = require("../../metrics/Classes");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const typeorm_1 = require("typeorm");
const Heatmap_1 = require("./Heatmap");
const Table_1 = require("./Table");
const URL_1 = require("./URL");
const HealthStatus_1 = require("./HealthStatus");
const ConfigurationTable_1 = require("./ConfigurationTable");
const Color_1 = require("../../metrics/Color");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const License_1 = require("../../../config/License");
const microserviceVariables_entity_1 = require("../../../entity/microserviceVariables.entity");
const AbstractMenu_1 = require("./AbstractMenu");
const Topology_1 = require("./Topology");
const common_1 = require("@nestjs/common");
const class_constants_1 = require("../../../common/constants/class.constants");
const cpu_mem_help = {
    url: "https://xormon.com/SAN-CPU-Memory.php",
    text: "How it works?",
};
const sanGlobalPages = () => {
    let menuItems = [];
    menuItems.push(new menuResponse_dto_1.MenuLink('Data', 'san/data', 'data', undefined, true));
    menuItems.push(new menuResponse_dto_1.MenuLink('Frame', 'san/frame', 'frame'));
    menuItems.push(new menuResponse_dto_1.MenuLink('System', 'san/cpumemory', 'cpumemory'));
    menuItems.push(new menuResponse_dto_1.MenuLink('Fabric', 'san/fabric', 'fabric'));
    menuItems.push(new menuResponse_dto_1.MenuLink('ISL', 'san/isl', 'isl'));
    menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', 'san/heatmap', 'heatmap'));
    menuItems.push(new menuResponse_dto_1.MenuLink('Health Status', 'san/health-status', 'healthstatus'));
    menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', 'san/configuration', 'configuration'));
    menuItems.push(new menuResponse_dto_1.MenuLink('SAN Topology', 'san/santopology', 'santopology'));
    return menuItems;
};
exports.sanGlobalPages = sanGlobalPages;
const sanDeviceTopologyBox = async (itemId) => {
    const isFree = !((await License_1.License.checkTechnology('storage')) || (await License_1.License.checkTechnology('san')));
    if (isFree) {
        return {
            type: pageResponse_constants_1.BOX_TYPE.HtmlTextBox,
            html: `To view the SAN topology, you must have an active license for storage or SAN. Documentation: <a href="https://xormon.com/SAN_Topology.php">SAN Topology</a>`
        };
    }
    let topologies = [
        {
            label: "Storage - Switch",
            type: "switch-storage",
            item_id: itemId
        }
    ];
    return {
        type: pageResponse_constants_1.BOX_TYPE.TopologyRequestBox,
        topologies: topologies
    };
};
exports.sanDeviceTopologyBox = sanDeviceTopologyBox;
const sanGlobalTopologyBox = async () => {
    const isFree = !((await License_1.License.checkTechnology('storage')) || (await License_1.License.checkTechnology('san')));
    if (isFree) {
        return {
            type: pageResponse_constants_1.BOX_TYPE.HtmlTextBox,
            html: `To view the SAN topology, you must have an active license for Storage or SAN. Documentation: <a href="https://xormon.com/SAN_Topology.php">SAN Topology</a>`
        };
    }
    let topologies = [
        {
            label: "Storage - Switch",
            type: "switch-storage"
        },
        {
            label: "Host - Switch",
            type: "switch-host"
        }
    ];
    const fabrics = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findBy({ hw_type: (0, typeorm_1.In)(['sanbrcd', 'sannav']), subsystem: 'fabric' });
    for (const fabric of fabrics) {
        topologies.push({
            label: `Fabric: ${fabric.label}`,
            type: 'fabric',
            item_id: fabric.item_id
        });
    }
    return {
        type: pageResponse_constants_1.BOX_TYPE.TopologyRequestBox,
        topologies: topologies
    };
};
exports.sanGlobalTopologyBox = sanGlobalTopologyBox;
class SanFolder extends AbstractMenu_1.StorAbstractFolder {
}
exports.SanFolder = SanFolder;
class SanSubsystemFolder extends SanFolder {
    constructor(hwType, subsystem, itemId) {
        super(hwType, subsystem, "");
        if (itemId !== null) {
            if (Array.isArray(itemId)) {
                if (itemId.length === 1) {
                    this.itemId = itemId[0];
                }
                else {
                    this.itemIds = itemId;
                }
            }
            else {
                this.itemId = itemId;
            }
        }
        if (subsystem !== null) {
            this.subsystem = subsystem.toString();
        }
    }
}
exports.SanSubsystemFolder = SanSubsystemFolder;
class SanRootFolder extends SanFolder {
    async getChildren() {
        let menuItems = [];
        let devices = await this.objectItemsRepository.find({
            relations: {
                hw_type_entity: true,
                parents: {
                    _parent: true
                }
            },
            where: {
                subsystem: 'device',
                hw_type_entity: {
                    class: 'san'
                }
            }
        });
        devices.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        if (devices.length >= 1) {
            menuItems = (0, exports.sanGlobalPages)();
        }
        const fabrics = await this.objectItemsRepository.find({
            where: {
                hw_type_entity: {
                    class: 'san'
                },
                subsystem: 'fabric'
            }, relations: {
                hw_type_entity: true,
                children: true
            }
        });
        fabrics.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const fabric of fabrics) {
            if (fabric.hw_type !== hwTypeKey_constants_1.HwTypeKey.SANCISCO && fabric.children && fabric.children.length >= 1) {
                let path = fabric.hw_type.toLowerCase() + '/fabric?ids=' + fabric.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(fabric.label, path, fabric.hw_type.toLowerCase(), 'fabric', fabric.item_id, SanFabricFolder));
            }
        }
        for (const device of devices) {
            if (device.parents.find(parent => parent._parent.subsystem === 'fabric') === undefined) {
                let path = device.hw_type.toLowerCase() + '/device?ids=' + device.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(device.label, path, device.hw_type.toLowerCase(), 'device', device.item_id, SanDeviceFolder));
            }
        }
        return menuItems;
    }
}
exports.SanRootFolder = SanRootFolder;
class SanFabricFolder extends SanSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'fabric';
    }
    async getChildren() {
        let menuItems = [];
        let devices = await this.objectItemsRepository.find({
            where: {
                subsystem: 'device',
                parents: {
                    parent: this.itemId.toString()
                }
            },
            relations: {
                parents: true
            }
        });
        devices.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let item of devices) {
            let path = this.hwType.toLowerCase() + '/device?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType.toLowerCase(), 'device', item.item_id, SanDeviceFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.SanFabricFolder = SanFabricFolder;
class SanDeviceFolder extends SanSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'device';
    }
    async getChildren() {
        if (!Classes_1.San.technology[this.hwType]) {
            return [];
        }
        let menuItems = await (new SanSubsystemItemsFolder(this.hwType, 'sanport', this.itemId)).getChildren();
        this.children = menuItems;
        return menuItems;
    }
}
exports.SanDeviceFolder = SanDeviceFolder;
class SanSubsystemItemsFolder extends SanSubsystemFolder {
    constructor(hwType, subsystem, itemId) {
        super(hwType, subsystem, itemId);
        this.subsystem = subsystem.toString();
    }
    async getChildren() {
        let menuItems = [];
        if (Classes_1.San.technology[this.hwType]['subsystems'][this.subsystem] ||
            Classes_1.San.technology[this.hwType]['subsystems'][this.subsystem.replace('-items', '').replace('items', '')]) {
            if (this.subsystem.includes('items')) {
                const subsys = this.subsystem.replace('-items', '').replace('items', '');
                let parentChildren = await this.itemRelationsRepository.find({
                    relations: {
                        _child: {
                            properties: true
                        }
                    },
                    where: {
                        parent: this.itemId.toString(),
                        _child: {
                            hw_type: this.hwType,
                            subsystem: subsys
                        }
                    }
                });
                let items = [];
                let unusedItems = [];
                for (const child of parentChildren) {
                    if (child._child.properties.find(property => property.property_name === 'working_state' && property.property_value === 'unused')) {
                        unusedItems.push(child._child);
                    }
                    else {
                        items.push(child._child);
                    }
                }
                items.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
                for (let item of items) {
                    let path = this.hwType.toLowerCase() + '/' + subsys + '?item_id=' + item.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, subsys, item.item_id));
                }
                if (unusedItems.length >= 1) {
                    let path = this.hwType.toLowerCase() + '/unused-ports?ids=' + this.itemId.toString();
                    menuItems.push(new menuResponse_dto_1.MenuFolder('Unused', path, this.hwType.toLowerCase(), 'unused-ports', this.itemId.toString(), SanSubsystemItemsFolder));
                }
                this.children = menuItems;
            }
            else {
                let i = 0;
                for (const page in Classes_1.San.technology[this.hwType]['subsystems'][this.subsystem]['totals']) {
                    const item = Classes_1.San.technology[this.hwType]['subsystems'][this.subsystem]['totals'][page];
                    menuItems.push(new menuResponse_dto_1.MenuLink(item['label'], this.hwType + '/' + this.subsystem + '-' + page + '?item_id=' + this.itemId, this.subsystem + '-totals', undefined, i === 0 ? true : undefined));
                    i++;
                }
                menuItems.push(new menuResponse_dto_1.MenuLink('Health Status', this.hwType + '/health-status?item_id=' + this.itemId, 'healthstatus', this.itemId));
                menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration?item_id=' + this.itemId, 'configuration', this.itemId));
                let path = this.hwType + '/' + this.subsystem + '-items' + (this.itemId !== null ? '?ids=' + this.itemId : '');
                menuItems.push(new menuResponse_dto_1.MenuFolder('Port', path, this.hwType, this.subsystem, this.itemId, SanSubsystemItemsFolder));
                this.children = menuItems;
            }
        }
        else if (this.subsystem === 'unused-ports') {
            let parentChildren = await this.itemRelationsRepository.find({
                relations: {
                    _child: {
                        properties: true
                    }
                },
                where: {
                    parent: this.itemId.toString(),
                    _child: {
                        hw_type: this.hwType,
                        subsystem: 'sanport',
                        properties: {
                            property_name: 'working_state',
                            property_value: 'unused'
                        }
                    }
                }
            });
            let items = [];
            for (const child of parentChildren) {
                items.push(child._child);
            }
            items.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let item of items) {
                let path = this.hwType.toLowerCase() + '/sanport?item_id=' + item.item_id;
                menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, 'sanport', item.item_id));
            }
        }
        return menuItems;
    }
}
exports.SanSubsystemItemsFolder = SanSubsystemItemsFolder;
class SanPage extends AbstractMenu_1.StorAbstractPage {
    constructor() {
        super(...arguments);
        this.withItemId = true;
        this.rootFolder = SanRootFolder;
    }
}
exports.SanPage = SanPage;
class SimpleSanPage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.withItemId = true;
        this.rootFolder = SanRootFolder;
    }
}
exports.SimpleSanPage = SimpleSanPage;
class SanSingleItemPage extends SanPage {
    constructor() {
        super(...arguments);
        this.parentSubsystem = 'device';
    }
}
exports.SanSingleItemPage = SanSingleItemPage;
class SanItemPage extends SanSingleItemPage {
    constructor(hwType, pageType, itemId, request) {
        super(hwType, pageType, itemId, request);
        this.pageType = pageType;
    }
    async getPage() {
        this.subsystem = this.pageType.split('-')[0];
        this.pageType = this.pageType.split('-')[1];
        let sanItems = [];
        let tabs = [];
        let title;
        if (this.pageType) {
            let items = await this.objectItemsRepository.findBy({
                hw_type: this.hwType,
                subsystem: this.subsystem,
            });
            let parentChildren = await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } });
            let childrenIds = parentChildren.map((relation) => relation.child);
            for (let item of items) {
                if (childrenIds.includes(item.item_id)) {
                    sanItems.push(item.item_id);
                }
            }
            title = this.subsystem + ' totals';
            for (const tabIndex in Classes_1.San.technology[this.hwType]['subsystems'][this.subsystem]['totals'][this.pageType]['tabs']) {
                const tab = Classes_1.San.technology[this.hwType]['subsystems'][this.subsystem]['totals'][this.pageType]['tabs'][tabIndex];
                tabs.push({
                    title: tab.label,
                    content: [
                        {
                            type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                            metrics: tab.metrics,
                            uuids: sanItems,
                            aggregated: true,
                        },
                    ],
                });
            }
        }
        else {
            let item = await this.objectItemsRepository.findOneBy({
                item_id: this.itemId,
            });
            sanItems.push(item.item_id);
            if (Classes_1.San.technology[this.hwType]['subsystems'][this.subsystem]['single']) {
                for (const tabIndex in Classes_1.San.technology[this.hwType]['subsystems'][this.subsystem]['single']) {
                    const tab = Classes_1.San.technology[this.hwType]['subsystems'][this.subsystem]['single'][tabIndex];
                    if (!tab) {
                        console.error("Cannot find tab definition for " + tabIndex);
                        continue;
                    }
                    tabs.push({
                        title: tab.label ? tab.label : 'unlabeled',
                        content: [
                            {
                                type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                                metrics: tab.metrics,
                                uuids: sanItems,
                                aggregated: false,
                            },
                        ],
                    });
                }
            }
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SanItemPage = SanItemPage;
class SanSanportcpumemoryPage extends SimpleSanPage {
    constructor() {
        super(...arguments);
        this.title = 'System';
    }
    async getPage() {
        let title = 'System';
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('cpu_system')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
            help: cpu_mem_help,
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('memory_used_percent')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SanSanportcpumemoryPage = SanSanportcpumemoryPage;
class SanTotalsPage extends SimpleSanPage {
    constructor(itemId, request, type) {
        super(itemId, request);
        this.title = 'Data';
        this.type = type;
    }
    async getPage() {
        let title = 'Data';
        let tabs = [];
        const devicesIds = (await this.objectItemsRepository.find({
            relations: {
                hw_type_entity: true
            },
            where: {
                subsystem: 'device',
                hw_type_entity: {
                    class: 'san'
                }
            }
        })).map((device) => device.item_id);
        if (this.type === 'data') {
            tabs.push({
                title: 'In',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('data_in')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            tabs.push({
                title: 'Out',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('data_out')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
        }
        else if (this.type === 'frame') {
            tabs.push({
                title: 'In',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('frames_in')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            tabs.push({
                title: 'Out',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('frames_out')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            tabs.push({
                title: 'BB Credits',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('bb_credits')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            tabs.push({
                title: 'CRC errors',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('crc_errors')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            tabs.push({
                title: 'PCS errors',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('pcs_errors')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            tabs.push({
                title: 'ENC out',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('enc_out')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            tabs.push({
                title: 'Disc C3 frames',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('disc_c3')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            tabs.push({
                title: 'Link fail',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('link_fail')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            tabs.push({
                title: 'Loss sync',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('loss_sync')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
            let portTable = new Table_1.Table()
                .addHeader({ value: 'Switch' })
                .addHeader({ value: 'Port' })
                .addHeader({ value: 'SFP Power in', unit: subsystemMetricsResponse_constants_1.UNIT.sfp_power_in, number: true })
                .addHeader({ value: 'SFP Power out', unit: subsystemMetricsResponse_constants_1.UNIT.sfp_power_out, number: true });
            const ports = await this.objectItemsRepository.find({
                where: {
                    parents: {
                        _parent: {
                            subsystem: 'device',
                            hw_type: (0, typeorm_1.In)(['sanbrcd', 'sannav']),
                        }
                    },
                    hw_type: (0, typeorm_1.In)(['sanbrcd', 'sannav']),
                    subsystem: 'sanport'
                },
                relations: {
                    parents: {
                        _parent: true
                    }
                }
            });
            for (const port of ports) {
                const device = port.parents.find(parent => parent._parent.subsystem === 'device');
                const lastValues = await (new MetricsWrapper_1.MetricsWrapper(['sfp_power_in', 'sfp_power_out'], [port.item_id])).getLastHourValueFromTable();
                if (lastValues[port.item_id]) {
                    const value_in = lastValues[port.item_id]['sfp_power_in'];
                    const value_out = lastValues[port.item_id]['sfp_power_out'];
                    if (value_in || value_out) {
                        portTable.addRow([
                            { order: 0, value: device ? device._parent.label : 'undefined' },
                            { order: 1, value: port.label, url: URL_1.URL.generate(port.hw_type, 'sanport', port.item_id) },
                            { order: 2, value: value_in ? value_in : null },
                            { order: 3, value: value_out ? value_out : null },
                        ]);
                    }
                }
            }
            portTable.sortByIndex(3, true);
            portTable.sortByIndex(2, true);
            tabs.push({
                title: 'SFP Power',
                content: [portTable.generateBox()],
            });
        }
        else if (this.type === 'cpumemory') {
            tabs.push({
                title: 'CPU',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('cpu_system')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
                help: cpu_mem_help,
            });
            tabs.push({
                title: 'Memory',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new SanMetrics.Metric('memory_used_percent')
                                .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                        ],
                        uuids: devicesIds,
                        aggregated: true,
                    }
                ],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SanTotalsPage = SanTotalsPage;
class SanDataPage extends SanTotalsPage {
    constructor(itemId, request) {
        super(itemId, request, 'data');
    }
}
exports.SanDataPage = SanDataPage;
class SanFramePage extends SanTotalsPage {
    constructor(itemId, request) {
        super(itemId, request, 'frame');
    }
}
exports.SanFramePage = SanFramePage;
class SanFabricPage extends SimpleSanPage {
    async getPage() {
        let title = 'Fabric totals';
        let tabs = [];
        const fabrics = await this.objectItemsRepository.findBy({
            subsystem: 'fabric',
            hw_type: (0, typeorm_1.In)(['sanbrcd', 'sancisco', 'sannav'])
        });
        const fabricIds = fabrics.map(item => item.item_id);
        tabs.push({
            title: 'In',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('data_in')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'Out',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('data_out')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'Frames in',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('frames_in')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'Frames out',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('frames_out')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'BB Credits',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('bb_credits')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'CRC errors',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('crc_errors')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'PCS errors',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('pcs_errors')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'ENC out',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('enc_out')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'Disc C3 frames',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('disc_c3')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'Link fail',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('link_fail')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'Loss sync',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('loss_sync')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true)
                    ],
                    uuids: fabricIds,
                    aggregated: true,
                }
            ],
        });
        let fabricTable = new Table_1.Table()
            .addHeader({ value: 'Fabric Name' })
            .addHeader({ value: 'Switch Name' });
        const brcdSwitches = await this.objectItemsRepository.find({
            where: {
                parents: {
                    _parent: {
                        subsystem: 'fabric',
                        hw_type: (0, typeorm_1.In)(['sanbrcd', 'sannav']),
                    }
                },
                hw_type: (0, typeorm_1.In)(['sanbrcd', 'sannav']),
                subsystem: 'device'
            },
            relations: {
                parents: {
                    _parent: true
                }
            }
        });
        for (const device of brcdSwitches) {
            const fabric = device.parents.find(parent => parent._parent.subsystem === 'fabric');
            fabricTable.addRow([
                { order: 0, value: fabric ? fabric._parent.label : 'undefined' },
                { order: 2, value: device.label, url: URL_1.URL.generate(device.hw_type, 'device', device.item_id) },
            ]);
        }
        fabricTable.sortByIndex(0);
        let vsanTable = new Table_1.Table()
            .addHeader({ value: 'vSAN Name' })
            .addHeader({ value: 'Switch Name' })
            .addHeader({ value: 'Port Name' });
        const ciscoPorts = await this.objectItemsRepository.find({
            where: {
                parents: {
                    _parent: {
                        subsystem: (0, typeorm_1.In)(['fabric', 'device']),
                        hw_type: (0, typeorm_1.In)(['sancisco']),
                    }
                },
                hw_type: (0, typeorm_1.In)(['sancisco']),
                subsystem: 'sanport'
            },
            relations: {
                parents: {
                    _parent: true
                }
            }
        });
        for (const port of ciscoPorts) {
            const vsan = port.parents.find(parent => parent._parent.subsystem === 'fabric');
            const device = port.parents.find(parent => parent._parent.subsystem === 'device');
            vsanTable.addRow([
                { order: 0, value: vsan ? vsan._parent.label : 'undefined' },
                { order: 1, value: device ? device._parent.label : 'undefined' },
                { order: 2, value: port.label, url: URL_1.URL.generate(port.hw_type, 'device', port.item_id) },
            ]);
        }
        vsanTable.sortByIndex(0);
        tabs.push({
            title: 'Configuration',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.TextBox,
                    styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                    text: 'Fabrics'
                },
                fabricTable.generateBox(),
                {
                    type: pageResponse_constants_1.BOX_TYPE.TextBox,
                    styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                    text: 'vSAN'
                },
                vsanTable.generateBox(),
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SanFabricPage = SanFabricPage;
class SanCpumemoryPage extends SanTotalsPage {
    constructor(itemId, request) {
        super(itemId, request, 'cpumemory');
    }
}
exports.SanCpumemoryPage = SanCpumemoryPage;
class SanConfigurationPage extends SimpleSanPage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let title = 'Configuration';
        let tabs = [];
        if (this.itemId) {
            const device = await this.objectItemsRepository.findOneBy({
                item_id: this.itemId
            });
            const ports = await this.objectItemsRepository.findBy({
                subsystem: 'sanport',
                parents: {
                    parent: this.itemId
                }
            });
            if (device.hw_type === 'sanbrcd') {
                const switchesTable = new ConfigurationTable_1.ConfigurationTable([device.item_id], ["chassis_name", "ip", "wwn", "model", "speed", "version", "serial", "total_ports", "available_ports", "free_ports", "unused_ports"]);
                switchesTable.setItemLabel('Switch Name');
                switchesTable.setAcl(this.request);
                const portsTable = new ConfigurationTable_1.ConfigurationTable(ports.map(port => port.item_id), ["alias", "wwn", "host_wwn", "speed", "working_state", "operational_status", "physical_status", "encrypt", "compression"], ["device"]);
                portsTable.setItemLabel('Port');
                portsTable.sortByColumn(0, 'desc', true);
                portsTable.setAcl(this.request);
                tabs.push({
                    title: 'Switch',
                    content: [
                        await switchesTable.generate(),
                        {
                            type: pageResponse_constants_1.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: 'Ports'
                        },
                        await portsTable.generate()
                    ],
                });
            }
            else if (device.hw_type === 'sannav') {
                const switchesTable = new ConfigurationTable_1.ConfigurationTable([device.item_id], ["fabric_name", "domain_id", "chassis_name", "ip", "wwn", "model", "days_up", "speed", "version", "serial", "total_ports", "available_ports", "free_ports", "unused_ports", "isl_ports", "licensed_ports"]);
                switchesTable.setItemLabel('Switch Name');
                switchesTable.setAcl(this.request);
                const portsTable = new ConfigurationTable_1.ConfigurationTable(ports.map(port => port.item_id), ["alias", "wwn", "host_wwn", "zone_alias", "target_port", "isl", "speed", "working_state", "operational_status", "physical_status", "license"], ["device"]);
                portsTable.setItemLabel('Port');
                portsTable.sortByColumn(0, 'desc', true);
                portsTable.setAcl(this.request);
                tabs.push({
                    title: 'Switch',
                    content: [
                        await switchesTable.generate(),
                        {
                            type: pageResponse_constants_1.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: 'Ports'
                        },
                        await portsTable.generate()
                    ],
                });
            }
            else if (device.hw_type === 'sancisco') {
                const switchesTable = new ConfigurationTable_1.ConfigurationTable([device.item_id], ["chassis_name", "ip", "wwn", "model", "speed", "version", "serial", "total_ports", "available_ports", "free_ports", "unused_ports", "licensed_ports"]);
                switchesTable.setItemLabel('Switch Name');
                switchesTable.setAcl(this.request);
                const portsTable = new ConfigurationTable_1.ConfigurationTable(ports.map(port => port.item_id), ["alias", "wwn", "host_wwn", "vsan_name", "speed", "working_state", "operational_status", "operational_status_reason", "physical_status"], ["device"]);
                portsTable.setItemLabel('Port');
                portsTable.sortByColumn(0, 'desc', true);
                portsTable.setAcl(this.request);
                tabs.push({
                    title: 'Switch',
                    content: [
                        await switchesTable.generate(),
                        {
                            type: pageResponse_constants_1.BOX_TYPE.TextBox,
                            styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                            text: 'Ports'
                        },
                        await portsTable.generate()
                    ],
                });
            }
        }
        else {
            let switchesTable = new Table_1.Table()
                .addHeader({ value: 'Fabric name' })
                .addHeader({ value: 'Domain ID' })
                .addHeader({ value: 'Switch Name' })
                .addHeader({ value: 'Chassis Name' })
                .addHeader({ value: 'IP Address' })
                .addHeader({ value: 'World Wide Name' })
                .addHeader({ value: 'Model' })
                .addHeader({ value: 'Speed' })
                .addHeader({ value: 'OS Version' })
                .addHeader({ value: 'Serial Number' })
                .addHeader({ value: 'Total Ports' })
                .addHeader({ value: 'Total available Ports' })
                .addHeader({ value: 'Free Ports' })
                .addHeader({ value: 'Unused Ports' })
                .addHeader({ value: 'Licenced Ports' });
            const devicesWithProperties = await this.objectItemsRepository.find({
                relations: {
                    hw_type_entity: true,
                    properties: true
                },
                where: {
                    subsystem: 'device',
                    hw_type_entity: {
                        class: 'san'
                    }
                }
            });
            for (const device of devicesWithProperties) {
                let props = {};
                for (const property of device.properties) {
                    props[property.property_name] = property.property_value;
                }
                switchesTable.addRow([
                    { order: 0, value: props['fabric_name'] ? props['fabric_name'] : ' ' },
                    { order: 1, value: props['domain_id'] ? props['domain_id'] : ' ' },
                    { order: 2, value: device.label, url: URL_1.URL.generate(device.hw_type, 'device', device.item_id) },
                    { order: 3, value: props['chassis_name'] ? props['chassis_name'] : ' ' },
                    { order: 4, value: props['ip'] ? props['ip'] : ' ' },
                    { order: 5, value: props['wwn'] ? props['wwn'] : ' ' },
                    { order: 6, value: props['model'] ? props['model'] : ' ' },
                    { order: 7, value: props['speed'] ? props['speed'] : ' ' },
                    { order: 8, value: props['version'] ? props['version'] : ' ' },
                    { order: 9, value: props['serial'] ? props['serial'] : ' ' },
                    { order: 10, value: props['total_ports'] ? props['total_ports'] : ' ' },
                    { order: 11, value: props['available_ports'] ? props['available_ports'] : ' ' },
                    { order: 12, value: props['free_ports'] ? props['free_ports'] : ' ' },
                    { order: 13, value: props['unused_ports'] ? props['unused_ports'] : ' ' },
                    { order: 14, value: props['licensed_ports'] ? props['licensed_ports'] : ' ' },
                ]);
            }
            switchesTable.addExport(devicesWithProperties.map(i => i.item_id), ["fabric_name", "domain_id", "chassis_name", "ip", "wwn", "model", "speed", "version", "serial", "total_ports", "available_ports", "free_ports", "unused_ports", "licensed_ports"]);
            tabs.push({
                title: 'Configuration',
                content: [switchesTable.generateBox()],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SanConfigurationPage = SanConfigurationPage;
class SanSantopologyPage extends SimpleSanPage {
    constructor() {
        super(...arguments);
        this.title = 'SAN Topology';
    }
    async getPage() {
        let title = 'SAN Topology';
        let tabs = [];
        const topologyBox = await (0, exports.sanGlobalTopologyBox)();
        tabs.push({
            title: 'SAN Topology',
            content: [topologyBox],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SanSantopologyPage = SanSantopologyPage;
class SanIslPage extends SimpleSanPage {
    constructor() {
        super(...arguments);
        this.title = 'ISL';
    }
    async getPage() {
        let title = 'ISL';
        let tabs = [];
        const topology = await Topology_1.Topology.getPage({
            type: 'switch-switch'
        }, this.request);
        let allChildren = [];
        let portIds = [];
        let mapping = {};
        let sourceSwitch = {};
        let destinationSwitch = {};
        const topologyBox = topology.content[0];
        for (const device of topologyBox.groups[0].children) {
            allChildren = allChildren.concat(device.children.map(ch => { return { ...ch, device: device.label, device_id: device.id, device_url: device.url }; }));
            for (const port of device.children) {
                const connection = topologyBox.connections.find(c => c.sourceWwn === port.wwn);
                if (connection) {
                    const destination = allChildren.find(ch => ch.wwn === connection.destinationWwn);
                    if (destination) {
                        portIds.push(port.id);
                        mapping[port.id] = {
                            value: destination.label,
                            url: destination.url
                        };
                        sourceSwitch[port.id] = {
                            value: device.label,
                            url: device.url
                        };
                        destinationSwitch[port.id] = {
                            value: destination.device,
                            url: destination.device_url
                        };
                    }
                }
            }
        }
        tabs.push({
            title: 'In',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('data_in')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Source Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch,
                            search: true
                        },
                        {
                            label: "Target Port",
                            values: mapping,
                            search: true
                        },
                        {
                            label: "Target Switch",
                            values: destinationSwitch,
                            search: true
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'Out',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('data_out')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'Frames in',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('frames_in')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'Frames out',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('frames_out')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'BB Credits',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('bb_credits')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'CRC errors',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('crc_errors')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'PCS errors',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('pcs_errors')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'ENC out',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('enc_out')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'Disc C3 frames',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('disc_c3')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'Link fail',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('link_fail')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        tabs.push({
            title: 'Loss sync',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new SanMetrics.Metric('loss_sync')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false)
                    ],
                    uuids: portIds,
                    aggregated: true,
                    label: "Port",
                    properties: [
                        {
                            label: "Source Switch",
                            values: sourceSwitch
                        },
                        {
                            label: "Destination Port",
                            values: mapping
                        },
                        {
                            label: "Destination Switch",
                            values: destinationSwitch
                        }
                    ]
                }
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SanIslPage = SanIslPage;
class SanHeatmapPage extends SimpleSanPage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getTab(tab) {
        if (tab === 'in') {
            const heatmap = new Heatmap_1.HeatmapSan(this.request);
            const heatmapBox = await heatmap.gen();
            const dataIn = heatmapBox.heatmapBox.metrics.find(m => m.metric === 'data_in_util');
            return {
                title: 'SAN In',
                content: [],
                subTabs: [
                    {
                        title: 'Heatmap',
                        content: [{
                                type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
                                metrics: dataIn ? [dataIn] : []
                            }, heatmapBox.textBox]
                    }, {
                        title: 'Table',
                        content: dataIn ? [await heatmap.table("data_in_util")] : []
                    }
                ]
            };
        }
        else if (tab === 'out') {
            const heatmap = new Heatmap_1.HeatmapSan(this.request);
            const heatmapBox = await heatmap.gen();
            const dataOut = heatmapBox.heatmapBox.metrics.find(m => m.metric === 'data_out_util');
            return {
                title: 'SAN Out',
                content: [],
                subTabs: [
                    {
                        title: 'Heatmap',
                        content: [{
                                type: pageResponse_constants_1.BOX_TYPE.HeatmapBox,
                                metrics: dataOut ? [dataOut] : []
                            }, heatmapBox.textBox]
                    }, {
                        title: 'Table',
                        content: dataOut ? [await heatmap.table("data_out_util")] : []
                    }
                ]
            };
        }
        else {
            throw new common_1.NotFoundException();
        }
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        const heatmap = new Heatmap_1.HeatmapSan(this.request);
        tabs.push({
            title: 'Heatmap',
            content: Object.values(await heatmap.gen()),
        });
        tabs.push({
            title: 'Ports',
            content: [
                await heatmap.table()
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SanHeatmapPage = SanHeatmapPage;
class SanHealthstatusPage extends SimpleSanPage {
    constructor() {
        super(...arguments);
        this.title = 'Health Status';
    }
    async getPage() {
        let title = 'Health Status';
        let tabs = [];
        let portHealthStatus = {
            type: pageResponse_constants_1.BOX_TYPE.PortHealthStatusBox,
            hwTypes: []
        };
        let customStatuses = [];
        const variables = await (0, dataSource_1.getDataSource)().getRepository(microserviceVariables_entity_1.MicroserviceVariables).find({
            where: {
                hw_type: 'sancisco',
                key: (0, typeorm_1.Like)(`%OSR_%`)
            }
        });
        for (const variable of variables) {
            const label = variable.key.replace("OSR_", "");
            customStatuses.push({
                label: label,
                color: variable.value
            });
        }
        const hsColors = {
            ['yellow']: 'yellow',
            ['green']: 'green',
            ['red']: 'red',
            ['gray']: 'gray'
        };
        let osrColorMapping = {};
        let colorMapping = {
            ['inSync']: hsColors.green,
            ['validating']: hsColors.green,
            ['online']: hsColors.green,
            ['up']: hsColors.green,
            ['none']: hsColors.green,
            ['noLight']: hsColors.yellow,
            ['noSync']: hsColors.yellow,
            ['lockRef']: hsColors.yellow,
            ['offline']: hsColors.yellow,
            ['down']: hsColors.yellow,
            ['testing']: hsColors.yellow,
            ['linkFailure']: hsColors.yellow,
            ['laserFault']: hsColors.red,
            ['portFault']: hsColors.red,
            ['diagFault']: hsColors.red,
            ['invalidModule']: hsColors.red,
            ['faulty']: hsColors.red,
            ['lowerLayerDown']: hsColors.red,
            ['noCard']: hsColors.gray,
            ['noTransceiver']: hsColors.gray,
            ['noSigDet']: hsColors.gray,
            ['unknown']: hsColors.gray,
            ['notPresent']: hsColors.gray,
            ['fcotNotPresent']: hsColors.gray,
            ['In_Sync']: hsColors.green,
            ['Online']: hsColors.green,
            ['Mod_Val']: hsColors.green,
            ['No_Card']: hsColors.gray,
            ['No_Module']: hsColors.gray,
            ['No_SigDet']: hsColors.gray,
            ['No_Light']: hsColors.yellow,
            ['No_Sync']: hsColors.yellow,
            ['Lock_Ref']: hsColors.yellow,
            ['Offline']: hsColors.yellow,
            ['Disabled']: hsColors.yellow,
            ['Testing']: hsColors.yellow,
            ['Transient']: hsColors.yellow,
            ['Faulty']: hsColors.red,
            ['Laser_Flt']: hsColors.red,
            ['Port_Flt']: hsColors.red,
            ['Hard_Flt']: hsColors.red,
            ['Mod_Inv']: hsColors.red,
            ['Mod_Uns']: hsColors.red,
        };
        const operationalStatusReason = [
            { label: "adminDown", color: "yellow" },
            { label: "adminStateConfigChange", color: "yellow" },
            { label: "authzPending", color: "yellow" },
            { label: "badFramesRcvdFromLink", color: "yellow" },
            { label: "bitErrRTThresExceeded", color: "red" },
            { label: "bundleMisCfg", color: "yellow" },
            { label: "bundleStandby", color: "yellow" },
            { label: "cdpInfoUnavailable", color: "yellow" },
            { label: "cellularModemUnattached", color: "yellow" },
            { label: "channelAdminDown", color: "yellow" },
            { label: "channelConfigurationInProgress", color: "yellow" },
            { label: "channelErrDisabled", color: "yellow" },
            { label: "channelOperSuspended", color: "yellow" },
            { label: "corePortMct", color: "yellow" },
            { label: "dcxCompatMismatch", color: "yellow" },
            { label: "dcxHundredPdusRcvdNoAck", color: "yellow" },
            { label: "dcxMultipleMsapIds", color: "yellow" },
            { label: "deniedDueToPortBinding", color: "yellow" },
            { label: "domainAddrAssignFailureIsolation", color: "yellow" },
            { label: "domainIdConfigMismatch", color: "yellow" },
            { label: "domainIdsInvalid", color: "yellow" },
            { label: "domainInvalidRCFReceived", color: "yellow" },
            { label: "domainManagerDisabled", color: "yellow" },
            { label: "domainMaxReTxFailure", color: "yellow" },
            { label: "domainNotAllowedIsolated", color: "yellow" },
            { label: "domainOther", color: "yellow" },
            { label: "domainOtherSideEportIsolation", color: "yellow" },
            { label: "domainOverlapIsolation", color: "yellow" },
            { label: "dot1xSecViolationErrDisabled", color: "yellow" },
            { label: "dupTunnelConfigDetected", color: "yellow" },
            { label: "ePortProhibited", color: "red" },
            { label: "ecSuspendedLoop", color: "yellow" },
            { label: "elpFailureAllZeroPeerWWNRcvd", color: "yellow" },
            { label: "elpFailureClassFParamErr", color: "yellow" },
            { label: "elpFailureClassNParamErr", color: "yellow" },
            { label: "elpFailureInvalidFlowCTLParam", color: "yellow" },
            { label: "elpFailureInvalidPayloadSize", color: "yellow" },
            { label: "elpFailureInvalidPortName", color: "yellow" },
            { label: "elpFailureInvalidSwitchName", color: "yellow" },
            { label: "elpFailureInvalidTxBBCredit", color: "yellow" },
            { label: "elpFailureIsolation", color: "yellow" },
            { label: "elpFailureLoopbackDetected", color: "yellow" },
            { label: "elpFailureRatovEdtovMismatch", color: "yellow" },
            { label: "elpFailureRevMismatch", color: "yellow" },
            { label: "elpFailureUnknownFlowCTLCode", color: "yellow" },
            { label: "emptyEchoUDLD", color: "yellow" },
            { label: "enmLoopDetected", color: "yellow" },
            { label: "enmPinFailLinkDown", color: "yellow" },
            { label: "enmSatIncompatibleUplink", color: "yellow" },
            { label: "eppFailure", color: "yellow" },
            { label: "errDisableHandShkFailure", color: "yellow" },
            { label: "errDisabledBPDUGuard", color: "yellow" },
            { label: "errDisabledIpConflict", color: "yellow" },
            { label: "errDisabledLacpMiscfg", color: "yellow" },
            { label: "errorDisabled", color: "yellow" },
            { label: "errorDisabledReInitLimitReached", color: "yellow" },
            { label: "escFailureIsolation", color: "yellow" },
            { label: "ethIntfInvalidBinding", color: "yellow" },
            { label: "ethIntfNotLicensed", color: "gray" },
            { label: "ethL2VlanDown", color: "yellow" },
            { label: "externallyDisabled", color: "yellow" },
            { label: "fabricBindingDBMismatch", color: "yellow" },
            { label: "fabricBindingDomainInvalid", color: "yellow" },
            { label: "fabricBindingNoRspFromPeer", color: "yellow" },
            { label: "fabricBindingSWWNNotFound", color: "yellow" },
            { label: "fabricNameInvalid", color: "yellow" },
            { label: "fcRedirectIsolation", color: "yellow" },
            { label: "fcidAllocationFailed", color: "yellow" },
            { label: "fcipPortAdminCfgChange", color: "gray" },
            { label: "fcipPortKeepAliveTimerExpire", color: "yellow" },
            { label: "fcipPortMaxReTx", color: "yellow" },
            { label: "fcipPortPersistTimerExpire", color: "yellow" },
            { label: "fcipPortSrcAdminDown", color: "gray" },
            { label: "fcipPortSrcLinkDown", color: "red" },
            { label: "fcipSrcModuleNotOnline", color: "gray" },
            { label: "fcipSrcPortRemoved", color: "gray" },
            { label: "fcotChksumErr", color: "red" },
            { label: "fcotClkRateMismatch", color: "yellow" },
            { label: "fcotNotPresent", color: "gray" },
            { label: "fcotValidationFailedAtDriver", color: "yellow" },
            { label: "fcotVendorNotSupported", color: "gray" },
            { label: "fcspAuthenfailure", color: "red" },
            { label: "fexSfpInvalid", color: "yellow" },
            { label: "ficonBeingEnabled", color: "yellow" },
            { label: "ficonDupPortNum", color: "yellow" },
            { label: "ficonInorderNotActive", color: "yellow" },
            { label: "ficonInvalidPortNum", color: "yellow" },
            { label: "ficonNoPortSwapLogicalPort", color: "yellow" },
            { label: "ficonNoPortnumber", color: "yellow" },
            { label: "ficonNotEnabled", color: "yellow" },
            { label: "ficonVsanDown", color: "yellow" },
            { label: "firstPortNotUp", color: "yellow" },
            { label: "firstPortUpAsEport", color: "yellow" },
            { label: "flowControlMismatch", color: "yellow" },
            { label: "hotStdbyInBundle", color: "yellow" },
            { label: "hwFailure", color: "red" },
            { label: "hwProgrammingFailed", color: "yellow" },
            { label: "ifSifLimitExceeded", color: "yellow" },
            { label: "inactiveM1PortFpathActiveVlan", color: "yellow" },
            { label: "incomAdminRxBBCreditPerBuf", color: "yellow" },
            { label: "incompatibleAdminMode", color: "yellow" },
            { label: "incompatibleAdminRxBBCredit", color: "yellow" },
            { label: "incompatibleAdminRxBufferSize", color: "yellow" },
            { label: "incompatibleAdminSpeed", color: "yellow" },
            { label: "incompleteConfig", color: "yellow" },
            { label: "incompleteTunnelCfg", color: "yellow" },
            { label: "initializing", color: "yellow" },
            { label: "interfaceRemoved", color: "gray" },
            { label: "invalidAttachment", color: "yellow" },
            { label: "invalidConfig", color: "yellow" },
            { label: "invalidEncapsulation", color: "yellow" },
            { label: "invalidFabricBindExh", color: "yellow" },
            { label: "invalidOtherSidePrincEFPReqRecd", color: "yellow" },
            { label: "invalidSatFabricIf", color: "yellow" },
            { label: "ipQosDcbxpCompatFailure", color: "yellow" },
            { label: "ipQosMgrPolicyAppFailure", color: "yellow" },
            { label: "ipSecHndshkInProgress", color: "yellow" },
            { label: "isolateBundleLimitExceeded", color: "yellow" },
            { label: "isolatedBundle", color: "yellow" },
            { label: "l3NotReady", color: "yellow" },
            { label: "linkFailCreditLoss", color: "red" },
            { label: "linkFailCreditLossB2B", color: "red" },
            { label: "linkFailDebounceTimeout", color: "red" },
            { label: "linkFailLIPF8Rcvd", color: "red" },
            { label: "linkFailLIPRcvdB2B", color: "red" },
            { label: "linkFailLRRcvdB2B", color: "red" },
            { label: "linkFailLineCardPortShutdown", color: "red" },
            { label: "linkFailLinkReset", color: "red" },
            { label: "linkFailLossOfSignal", color: "red" },
            { label: "linkFailLossOfSync", color: "red" },
            { label: "linkFailNOSRcvd", color: "red" },
            { label: "linkFailOLSRcvd", color: "red" },
            { label: "linkFailOPNYRETB2B", color: "red" },
            { label: "linkFailOPNYTMOB2B", color: "red" },
            { label: "linkFailPortInitFail", color: "red" },
            { label: "linkFailPortUnusable", color: "red" },
            { label: "linkFailRxQOverFlow", color: "red" },
            { label: "linkFailTooManyINTR", color: "red" },
            { label: "linkFailure", color: "yellow" },
            { label: "localRcf", color: "yellow" },
            { label: "loopbackDiagFailure", color: "red" },
            { label: "loopbackIsolation", color: "yellow" },
            { label: "lstGrpUplinkDown", color: "yellow" },
            { label: "modemLineDeasserted", color: "yellow" },
            { label: "moduleOffline", color: "gray" },
            { label: "moduleRemoved", color: "gray" },
            { label: "neighborMismatchUDLD", color: "yellow" },
            { label: "noCommonVsanIsolation", color: "yellow" },
            { label: "noFcoe", color: "yellow" },
            { label: "noRemoteChassis", color: "yellow" },
            { label: "nonCiscoHbaVfTag", color: "yellow" },
            { label: "nonCorePortMct", color: "yellow" },
            { label: "nonExistentVlan", color: "yellow" },
            { label: "nonParticipating", color: "yellow" },
            { label: "nonStickyExternallyDisabled", color: "yellow" },
            { label: "none", color: "green" },
            { label: "npivNotEnabledInUpstream", color: "yellow" },
            { label: "offline", color: "red" },
            { label: "ohmsExtLBTest", color: "yellow" },
            { label: "other", color: "gray" },
            { label: "outOfGlblRxBuffers", color: "yellow" },
            { label: "outOfService", color: "yellow" },
            { label: "parentDown", color: "yellow" },
            { label: "parentPortDown", color: "yellow" },
            { label: "pauseRateLimitErrDisabled", color: "yellow" },
            { label: "peerFCIPPortClosedConnection", color: "yellow" },
            { label: "peerFCIPPortResetConnection", color: "yellow" },
            { label: "peerNotSupportBundle", color: "yellow" },
            { label: "phyIntfDown", color: "red" },
            { label: "pmonFailure", color: "yellow" },
            { label: "portActLicenseNotAvailable", color: "yellow" },
            { label: "portAuthFailed", color: "yellow" },
            { label: "portBindFailure", color: "yellow" },
            { label: "portBlocked", color: "yellow" },
            { label: "portBringupIsolation", color: "yellow" },
            { label: "portCapabilitiesUnknown", color: "yellow" },
            { label: "portChannelMembersDown", color: "yellow" },
            { label: "portConnectorTypeErr", color: "yellow" },
            { label: "portDisabled", color: "gray" },
            { label: "portFabricBindFailure", color: "yellow" },
            { label: "portGracefulShutdown", color: "gray" },
            { label: "portGuardBitErrRate", color: "red" },
            { label: "portGuardCreditLoss", color: "red" },
            { label: "portGuardLinkReset", color: "red" },
            { label: "portGuardSigLoss", color: "red" },
            { label: "portGuardSyncLoss", color: "red" },
            { label: "portGuardTrustSecViolation", color: "yellow" },
            { label: "portVsanMismatchIsolation", color: "yellow" },
            { label: "pppAuthFailed", color: "yellow" },
            { label: "pppPeerNotResponding", color: "yellow" },
            { label: "preferredPathIsolation", color: "yellow" },
            { label: "primaryVLANDown", color: "yellow" },
            { label: "profileNotFound", color: "yellow" },
            { label: "protoPortSuspend", color: "yellow" },
            { label: "pvlanNativeVlanErr", color: "yellow" },
            { label: "rcfInProgress", color: "yellow" },
            { label: "remotePortInL2vpnDown", color: "yellow" },
            { label: "routerMacFailure", color: "yellow" },
            { label: "satFabricIfDown", color: "yellow" },
            { label: "satIncompatTopo", color: "yellow" },
            { label: "satNotConfigured", color: "yellow" },
            { label: "sdmIsolation", color: "yellow" },
            { label: "sdpTimeout", color: "yellow" },
            { label: "setPortStateFailed", color: "yellow" },
            { label: "sfpAbsent", color: "gray" },
            { label: "sfpEthCompliantErr", color: "yellow" },
            { label: "sfpFcCompliantErr", color: "red" },
            { label: "sfpInvalid", color: "yellow" },
            { label: "sfpReadError", color: "yellow" },
            { label: "sfpSpeedMismatch", color: "yellow" },
            { label: "sifAdminDown", color: "gray" },
            { label: "sifHoldDown", color: "yellow" },
            { label: "sifNotBound", color: "yellow" },
            { label: "speedMismatch", color: "yellow" },
            { label: "srcPortNotBound", color: "yellow" },
            { label: "stickyDnLinkFailure", color: "yellow" },
            { label: "stickyDownOnLinkFailure", color: "yellow" },
            { label: "stpInconsVpcPeerDisabled", color: "yellow" },
            { label: "stpNotFwdingInFcoeMappedVlan", color: "yellow" },
            { label: "subGroupIdNotAssigned", color: "yellow" },
            { label: "suspendByMtu", color: "yellow" },
            { label: "suspendedByMode", color: "yellow" },
            { label: "suspendedBySpeed", color: "yellow" },
            { label: "suspendedByVpc", color: "yellow" },
            { label: "suspendedByWWN", color: "yellow" },
            { label: "suspendedDueToMinLinks", color: "yellow" },
            { label: "suspendedDueToNoLacpPdus", color: "yellow" },
            { label: "swFailure", color: "red" },
            { label: "tooManyInvalidFLOGIs", color: "yellow" },
            { label: "tooManyLinkFlapsErr", color: "yellow" },
            { label: "tovMismatch", color: "yellow" },
            { label: "trackedPortDown", color: "yellow" },
            { label: "trunkNotFullyActive", color: "yellow" },
            { label: "tunnelDstNotConfigured", color: "yellow" },
            { label: "tunnelDstUnreachable", color: "yellow" },
            { label: "tunnelModeNotConfigured", color: "yellow" },
            { label: "tunnelSrcDown", color: "yellow" },
            { label: "tunnelSrcNotConfigured", color: "yellow" },
            { label: "tunnelUnableToResolveDstIP", color: "yellow" },
            { label: "tunnelUnableToResolveSrcIP", color: "yellow" },
            { label: "tunnelVrfDown", color: "yellow" },
            { label: "twoSwitchesWithSameWWN", color: "yellow" },
            { label: "txRxLoopUDLD", color: "yellow" },
            { label: "udldAggModeLinkFailure", color: "yellow" },
            { label: "unavailableNPVPrefUpstreamPort", color: "yellow" },
            { label: "unavailableNPVUpstreamPort", color: "yellow" },
            { label: "unidirectionalUDLD", color: "yellow" },
            { label: "unsupportedTransceiver", color: "yellow" },
            { label: "unsupportedTransceiverMd5DigestNotSame", color: "yellow" },
            { label: "upgradeInProgress", color: "yellow" },
            { label: "vdcMode", color: "yellow" },
            { label: "vemUnlicensed", color: "yellow" },
            { label: "vfTaggingCapErr", color: "yellow" },
            { label: "vfcBindingInvalid", color: "yellow" },
            { label: "vicDisableReceived", color: "yellow" },
            { label: "vicEnableNotReceived", color: "yellow" },
            { label: "virtualIvrDomainOverlapIsolation", color: "yellow" },
            { label: "vlanAllowedList", color: "yellow" },
            { label: "vlanAllowedListAdd", color: "yellow" },
            { label: "vlanAllowedListOverride", color: "yellow" },
            { label: "vlanAllowedRemove", color: "yellow" },
            { label: "vlanCfgDelete", color: "yellow" },
            { label: "vlanCfgStateChange", color: "yellow" },
            { label: "vlanDown", color: "yellow" },
            { label: "vlanInvalidType", color: "yellow" },
            { label: "vlanNotFcoeEnabled", color: "yellow" },
            { label: "vlanVsanMappingNotEnabled", color: "yellow" },
            { label: "vpcCfgInProgress", color: "yellow" },
            { label: "vpcNoRspFromPeer", color: "yellow" },
            { label: "vpcPeerLinkDown", color: "yellow" },
            { label: "vpcPeerUpgrade", color: "yellow" },
            { label: "vrfForwardReferencing", color: "yellow" },
            { label: "vrfMismatch", color: "yellow" },
            { label: "vrfUnusable", color: "yellow" },
            { label: "vsanInactive", color: "yellow" },
            { label: "vsanMismatchIsolation", color: "yellow" },
            { label: "vsanMismatchWithUpstreamSwPort", color: "yellow" },
            { label: "vsanNotFound", color: "yellow" },
            { label: "vsanSuspended", color: "yellow" },
            { label: "waitForFlogi", color: "yellow" },
            { label: "xcvrAbsent", color: "yellow" },
            { label: "xcvrAuthFailed", color: "yellow" },
            { label: "xcvrInitializing", color: "yellow" },
            { label: "xcvrInvalid", color: "yellow" },
            { label: "zoneMergeFailureIsolation", color: "yellow" },
            { label: "zoneRemoteNoRespIsolation", color: "yellow" }
        ];
        for (const status of operationalStatusReason) {
            osrColorMapping[status.label] = status.color;
        }
        for (const customStatus of customStatuses) {
            osrColorMapping[customStatus.label] = customStatus.color;
            const fStatus = operationalStatusReason.find(osr => osr.label == customStatus.label);
            if (fStatus) {
                fStatus.color = customStatus.color;
            }
            else {
                operationalStatusReason.push(customStatus);
            }
        }
        const legends = {
            'sanbrcd': [
                {
                    'type': 'physicalStatus',
                    'label': "Physical status",
                    'statuses': [
                        { 'color': colorMapping['inSync'], 'label': 'inSync', 'text': 'The module is receiving light and is in sync.' },
                        { 'color': colorMapping['validating'], 'label': 'validating', 'text': 'The module is being validated.' },
                        { 'color': colorMapping['noCard'], 'label': 'noCard', 'text': 'No card is present in this switch slot.' },
                        { 'color': colorMapping['noTransceiver'], 'label': 'noTransceiver', 'text': 'No Transceiver module in this port. (Transceiver is the generic name for GBIC, SFP, and so on' },
                        { 'color': colorMapping['noSigDet'], 'label': 'noSigDet', 'text': 'No Signal is detected in the port.' },
                        { 'color': colorMapping['unknown'], 'label': 'unknown', 'text': 'The module is unknown.' },
                        { 'color': colorMapping['noLight'], 'label': 'noLight', 'text': 'The module is not receiving light.' },
                        { 'color': colorMapping['noSync'], 'label': 'noSync', 'text': 'The module is receiving light but is out of sync.' },
                        { 'color': colorMapping['lockRef'], 'label': 'lockRef', 'text': 'Port is locking to the reference signal.' },
                        { 'color': colorMapping['laserFault'], 'label': 'laserFault', 'text': 'The module is signaling a laser fault. (defective GBIC)' },
                        { 'color': colorMapping['portFault'], 'label': 'portFault', 'text': 'The port is marked faulty. (defective GBIC, cable, or device)' },
                        { 'color': colorMapping['diagFault'], 'label': 'diagFault', 'text': 'The port failed diagnostics. (defective G_Port or FL_Port card or motherboard)' },
                        { 'color': colorMapping['invalidModule'], 'label': 'lockRef', 'text': 'The module is invalid.' },
                    ]
                },
                {
                    'type': 'operationalStatus',
                    'label': "Operational status",
                    'statuses': [
                        { 'color': colorMapping['online'], 'label': 'online', 'text': 'User frames can be passed.' },
                        { 'color': colorMapping['unknown'], 'label': 'unknown', 'text': 'The port module is physically absent.' },
                        { 'color': colorMapping['offline'], 'label': 'offline', 'text': 'No user frames can be passed.' },
                        { 'color': colorMapping['faulty'], 'label': 'faulty', 'text': 'The port module is physically faulty.' },
                    ]
                }
            ],
            'sancisco': [
                {
                    'type': 'adminStatus',
                    'label': "Admin status",
                    "text": "The desired state of the interface.",
                    'statuses': [
                        { 'color': colorMapping['up'], 'label': 'up' },
                        { 'color': colorMapping['down'], 'label': 'down' },
                        { 'color': colorMapping['testing'], 'label': 'testing' },
                    ]
                },
                {
                    'type': 'operationalStatus',
                    'label': "Operational status",
                    "text": "The current operational state of the interface.",
                    'statuses': [
                        { 'color': colorMapping['up'], 'label': 'up' },
                        { 'color': colorMapping['notPresent'], 'label': 'notPresent' },
                        { 'color': colorMapping['unknown'], 'label': 'unknown' },
                        { 'color': colorMapping['down'], 'label': 'down' },
                        { 'color': colorMapping['testing'], 'label': 'testing' },
                        { 'color': colorMapping['lowerLayerDown'], 'label': 'lowerLayerDown' }
                    ]
                },
                {
                    'type': 'operationalStatusReason',
                    'label': "Operational status reason",
                    'text': "The cause of current operational state of the port.",
                    'statuses': operationalStatusReason
                }
            ],
            'sannav': [
                {
                    'type': 'physicalStatus',
                    'label': "Physical status",
                    'statuses': [
                        { 'color': colorMapping['In_Sync'], 'label': 'In_Sync', 'text': 'The module is receiving light and in sync.' },
                        { 'color': colorMapping['Online'], 'label': 'Online', 'text': 'The port is up and running.' },
                        { 'color': colorMapping['Mod_Val'], 'label': 'Mod_Val', 'text': 'Module validation in process.' },
                        { 'color': colorMapping['No_Card'], 'label': 'No_Card', 'text': 'No interface card present.' },
                        { 'color': colorMapping['No_Module'], 'label': 'No_Module', 'text': 'No module (GBIC or other) present.' },
                        { 'color': colorMapping['No_SigDet'], 'label': 'No_SigDet', 'text': 'No signal is detected on the port. For 16Gb/s-capable interchassis link (ICL) ports, this state replaces the No_Light indicator. It indicates that a quad small form-factor pluggable (QSFP) has been installed but is not connected with a cable.' },
                        { 'color': colorMapping['No_Light'], 'label': 'No_Light', 'text': 'The module is not receiving light. This state is not applicable to 16Gb/s-capable interchassis link (ICL) ports.' },
                        { 'color': colorMapping['No_Sync'], 'label': 'No_Sync', 'text': 'The module is receiving light but is out of sync.' },
                        { 'color': colorMapping['Lock_Ref'], 'label': 'Lock_Ref', 'text': 'The port is locking to the reference signal.' },
                        { 'color': colorMapping['Offline'], 'label': 'Offline', 'text': 'A port connection is not established (for virtual ports only).' },
                        { 'color': colorMapping['Disabled'], 'label': 'Disabled', 'text': 'The port is disabled.' },
                        { 'color': colorMapping['Testing'], 'label': 'Testing', 'text': 'The port is running diagnostics.' },
                        { 'color': colorMapping['Transient'], 'label': 'Transient', 'text': 'The port is disabled and re-enabled by MAPS to recover from the bottleneck condition caused by the target device.' },
                        { 'color': colorMapping['Laser_Flt'], 'label': 'Laser_Flt', 'text': 'The module is signaling a laser fault.' },
                        { 'color': colorMapping['Port_Flt'], 'label': 'Port_Flt', 'text': 'The port is marked faulty.' },
                        { 'color': colorMapping['Hard_Flt'], 'label': 'Hard_Flt', 'text': 'The port is hard faulted.' },
                        { 'color': colorMapping['Mod_Inv'], 'label': 'Mod_Inv', 'text': 'Module speed mismatch or incompatible SFP.' },
                        { 'color': colorMapping['Mod_Uns'], 'label': 'Mod_Uns', 'text': 'Module unsupported.' },
                    ]
                },
                {
                    'type': 'operationalStatus',
                    'label': "Operational status",
                    'statuses': [
                        { 'color': colorMapping['Online'], 'label': 'Online', 'text': 'User frames can be passed.' },
                        { 'color': colorMapping['Unknown'], 'label': 'Unknown', 'text': 'The port module is physically absent.' },
                        { 'color': colorMapping['Offline'], 'label': 'Offline', 'text': 'No user frames can be passed.' },
                        { 'color': colorMapping['Faulty'], 'label': 'Faulty', 'text': 'The port module is physically faulty.' },
                    ]
                }
            ],
        };
        let devices = [];
        let devicePorts = [];
        if (this.itemId) {
            devices = await this.objectItemsRepository.find({
                where: {
                    item_id: this.itemId,
                }
            });
        }
        else {
            devices = await this.objectItemsRepository.find({
                relations: {
                    hw_type_entity: true,
                    health_status: true,
                    hostcfg_relations: {
                        _hostcfg: true
                    }
                },
                where: {
                    hw_type_entity: {
                        class: 'san'
                    },
                    subsystem: 'device',
                }
            });
            const hs = new HealthStatus_1.HealthStatus().setAcl(this.request);
            let healthStatus = hs.generateBasicTable(devices, "Switch");
            let history = hs.getHistory(class_constants_1.ClassKey.SAN, "device");
            tabs.push({
                title: 'Health Status',
                content: [await healthStatus],
            });
            tabs.push({
                title: 'Device History',
                content: [await history],
            });
        }
        devicePorts = await this.objectItemsRepository.find({
            relations: {
                properties: true,
                parents: true
            },
            where: {
                subsystem: 'sanport',
                properties: {
                    property_name: (0, typeorm_1.In)(['operational_status', 'physical_status', 'operational_status_reason', 'admin_status'])
                },
                parents: {
                    parent: (0, typeorm_1.In)(devices.map(i => i.item_id))
                }
            }
        });
        let cisco = {
            label: 'Cisco SAN switches',
            devices: [],
            legends: legends['sancisco']
        };
        let sanbrcd = {
            label: 'Brocade SAN switches',
            devices: [],
            legends: legends['sanbrcd']
        };
        let sannav = {
            label: 'Brocade SANnav switches',
            devices: [],
            legends: legends['sannav']
        };
        for (const device of devices) {
            let dPorts = devicePorts.filter(p => p.parents[0]?.parent === device.item_id);
            if (dPorts.length === 0) {
                continue;
            }
            let ports = [];
            for (const port of dPorts) {
                if (port.properties.length >= 2) {
                    let propertiesMap = {};
                    for (const property of port.properties) {
                        propertiesMap[property.property_name] = property.property_value;
                    }
                    ports.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
                    ports.push({
                        label: port.label,
                        url: URL_1.URL.generate(port.hw_type, port.subsystem, port.item_id),
                        physicalStatus: (port.hw_type === 'sanbrcd' || port.hw_type === 'sannav') && propertiesMap['physical_status'] ? propertiesMap['physical_status'] : undefined,
                        physicalStatusColor: (port.hw_type === 'sanbrcd' || port.hw_type === 'sannav') && propertiesMap['physical_status'] ? colorMapping[propertiesMap['physical_status']] : undefined,
                        operationalStatus: propertiesMap['operational_status'] ? propertiesMap['operational_status'] : undefined,
                        operationalStatusColor: propertiesMap['operational_status'] ? colorMapping[propertiesMap['operational_status']] : undefined,
                        operationalStatusReason: propertiesMap['operational_status_reason'] ? propertiesMap['operational_status_reason'] : undefined,
                        operationalStatusReasonColor: propertiesMap['operational_status_reason'] ? osrColorMapping[propertiesMap['operational_status_reason']] : undefined,
                        adminStatus: port.hw_type === 'sancisco' && propertiesMap['physical_status'] ? propertiesMap['physical_status'] : undefined,
                        adminStatusColor: port.hw_type === 'sancisco' && propertiesMap['physical_status'] ? colorMapping[propertiesMap['physical_status']] : undefined,
                    });
                }
            }
            if (device.hw_type === 'sanbrcd') {
                sanbrcd['devices'].push({
                    label: device.label,
                    ports: ports
                });
            }
            else if (device.hw_type === 'sancisco') {
                cisco['devices'].push({
                    label: device.label,
                    ports: ports
                });
            }
            else if (device.hw_type === 'sannav') {
                sannav['devices'].push({
                    label: device.label,
                    ports: ports
                });
            }
        }
        if (sanbrcd.devices.length >= 1) {
            portHealthStatus.hwTypes.push(sanbrcd);
        }
        if (cisco.devices.length >= 1) {
            portHealthStatus.hwTypes.push(cisco);
        }
        if (sannav.devices.length >= 1) {
            portHealthStatus.hwTypes.push(sannav);
        }
        if (this.itemId) {
            for (const device of devices) {
                if (device.hw_type === 'sanbrcd') {
                    const itemProperties = await this.itemPropertiesRepository.find({ where: { item_id: this.itemId, property_name: 'events' } });
                    const eventsText = {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "Events"
                    };
                    let eventsTable = new Table_1.Table()
                        .addHeader({ value: 'Date/Time' })
                        .addHeader({ value: 'Severity' })
                        .addHeader({ value: 'Description' });
                    if (itemProperties) {
                        for (const items of itemProperties) {
                            const data = (JSON.parse(items.property_value));
                            for (const row of data) {
                                eventsTable.addRow([
                                    { order: 0, value: row.time },
                                    { order: 1, value: row.severity },
                                    { order: 2, value: row.description },
                                ]);
                            }
                        }
                    }
                    else {
                        eventsTable.addRow([
                            { order: 0, value: '' },
                            { order: 1, value: '' },
                            { order: 2, value: '' },
                        ]);
                    }
                    portHealthStatus.tabs = [
                        {
                            title: "Events",
                            content: [
                                eventsText,
                                eventsTable.generateBox(),
                            ],
                        }
                    ];
                }
            }
        }
        tabs.push({
            title: 'Details',
            content: [portHealthStatus],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.SanHealthstatusPage = SanHealthstatusPage;
class SanMetrics {
}
exports.SanMetrics = SanMetrics;
SanMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(metric) {
        super();
        this.metric = metric;
        if (Classes_1.San.metrics[metric]) {
            const definition = Classes_1.San.metrics[metric];
            this.unit = definition.unit;
            this.shortcut = definition.shortcut;
            this.label = definition.label;
            this.defaultPrecision = definition.defaultPrecision ? definition.defaultPrecision : undefined;
            this.defaultPrefix = definition.defaultPrefix ? definition.defaultPrefix : undefined;
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
    }
};
//# sourceMappingURL=San.js.map