"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TabFactory = exports.defaultPicker = exports.MPDefault = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const GraphBuilder_1 = require("./GraphBuilder");
const Color_1 = require("../../metrics/Color");
var MPDefault;
(function (MPDefault) {
    MPDefault["AREA"] = "area";
    MPDefault["LINE"] = "line";
    MPDefault["AREA_STACKED"] = "area_stacked";
    MPDefault["LINE_STACKED"] = "line_stacked";
    MPDefault["READ_LINE_AGGR"] = "r_line_aggr";
    MPDefault["WRITE_LINE_AGGR"] = "w_line_aggr";
    MPDefault["READ_LINE"] = "r_line";
    MPDefault["WRITE_LINE"] = "w_line";
    MPDefault["LINE_AGGR"] = "line_aggr";
    MPDefault["AREA_AGGR"] = "area_aggr";
})(MPDefault || (exports.MPDefault = MPDefault = {}));
exports.defaultPicker = {
    [MPDefault.AREA]: {
        type: pageResponse_constants_1.CHART_TYPE.area,
        order: 0,
        stacked: false
    },
    [MPDefault.LINE]: {
        type: pageResponse_constants_1.CHART_TYPE.line,
        order: 0,
        stacked: false
    },
    [MPDefault.AREA_STACKED]: {
        type: pageResponse_constants_1.CHART_TYPE.area,
        order: 0,
        stacked: true
    },
    [MPDefault.LINE_STACKED]: {
        type: pageResponse_constants_1.CHART_TYPE.line,
        order: 0,
        stacked: true
    },
    [MPDefault.READ_LINE_AGGR]: {
        type: pageResponse_constants_1.CHART_TYPE.line,
        order: -1,
        stacked: false,
        color: Color_1.Color.metric.read,
        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items
    },
    [MPDefault.WRITE_LINE_AGGR]: {
        type: pageResponse_constants_1.CHART_TYPE.line,
        order: 0,
        stacked: false,
        color: Color_1.Color.metric.write,
        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items
    },
    [MPDefault.READ_LINE]: {
        type: pageResponse_constants_1.CHART_TYPE.line,
        order: -1,
        stacked: false,
        color: Color_1.Color.metric.read,
    },
    [MPDefault.WRITE_LINE]: {
        type: pageResponse_constants_1.CHART_TYPE.line,
        order: 0,
        stacked: false,
        color: Color_1.Color.metric.write,
    },
    [MPDefault.LINE_AGGR]: {
        type: pageResponse_constants_1.CHART_TYPE.line,
        order: 0,
        stacked: false,
        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items
    },
    [MPDefault.AREA_AGGR]: {
        type: pageResponse_constants_1.CHART_TYPE.line,
        order: 0,
        stacked: false,
        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items
    },
};
class TabFactory {
    constructor(_class, hwType, subsystem, itemIds) {
        this._class = _class;
        this.hwType = hwType;
        this.subsystem = subsystem;
        this.itemIds = itemIds;
        this.tabs = [];
        this.createTab = (title, mp, label, parent, setAggregated) => {
            const gb = new GraphBuilder_1.GraphBuilder(this._class, this.hwType, this.subsystem)
                .addItems(this.itemIds);
            if (label)
                gb.setLabel(label);
            if (parent)
                gb.setParent(parent);
            if (setAggregated)
                gb.setAggregated();
            if (Array.isArray(mp)) {
                const metrics = mp.map(this.setDefaults);
                for (const p of metrics) {
                    gb.addMetric(p.metric, p.type, p.stacked, p.order, p.color);
                }
            }
            else {
                const metric = this.setDefaults(mp);
                gb.addMetric(metric.metric, metric.type, metric.stacked, metric.order, metric.color);
            }
            return {
                title: title,
                content: [
                    gb.generateGraph()
                ]
            };
        };
    }
    addTab(title, mp, label, parent, setAggregated) {
        this.tabs.push(this.createTab(title, mp, label, parent, setAggregated));
    }
    getTabs() {
        return this.tabs;
    }
    setDefaults(mp) {
        if (mp.default) {
            return { ...exports.defaultPicker[mp.default], ...mp };
        }
        else if (!(mp.color || mp.order === 0 || mp.order || mp.stacked !== null || mp.stacked !== undefined)) {
            return { ...exports.defaultPicker[MPDefault.LINE] };
        }
        return mp;
    }
    setItems(itemIds) {
        this.itemIds = itemIds;
    }
    setSubsystem(subsystem) {
        this.subsystem = subsystem;
    }
}
exports.TabFactory = TabFactory;
//# sourceMappingURL=TabFactory.js.map