"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TableHeaderGroup = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
class HeaderDefinition {
}
class TableHeaderGroup {
    constructor() {
        this.header = [];
        this.rows = [];
        this.groups = [];
        this.lastIndex = 0;
        this.orientation = 'horizontal';
    }
    setOrientation(orientation) {
        this.orientation = orientation;
    }
    addHeader(header) {
        this.header.push({
            order: this.lastIndex,
            value: header.value,
            group: header.group ? header.group : undefined,
            number: header.number ? header.number : undefined
        });
        this.lastIndex++;
        return this;
    }
    addGroup(group) {
        this.groups.push(group);
        return this;
    }
    sortByIndex(i) {
        this.rows.sort((a, b) => (a[i].value < b[i].value) ? 1 : ((b[i].value < a[i].value) ? -1 : 0));
    }
    addRow(data) {
        if (data.length === this.header.length) {
            this.rows.push(data.map(column => {
                const value = Array.isArray(column.value)
                    ? column.value
                    : [column.value];
                const url = Array.isArray(column.url)
                    ? column.url
                    : [column.url];
                return {
                    ...column,
                    value: value,
                    url: url
                };
            }));
        }
        return this;
    }
    addRows(data) {
        for (const row of data) {
            if (row.length === this.header.length) {
                this.rows.push(row.map(column => {
                    const value = Array.isArray(column.value)
                        ? column.value
                        : [column.value];
                    const url = Array.isArray(column.url)
                        ? column.url
                        : [column.url];
                    return {
                        ...column,
                        value: value,
                        url: url
                    };
                }));
            }
        }
        return this;
    }
    generateBox() {
        return {
            type: pageResponse_constants_1.BOX_TYPE.TableHeaderGroupBox,
            header: this.header,
            rows: this.rows,
            groups: this.groups,
            orientation: this.orientation
        };
    }
}
exports.TableHeaderGroup = TableHeaderGroup;
//# sourceMappingURL=TableHeaderGroup.js.map