"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _a, _Topology_getSanSwitches;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Topology = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const dataSource_1 = require("../../../config/dataSource");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
const URL_1 = require("./URL");
const ACLCheck_1 = require("../../acl/ACLCheck");
class Topology {
    static async getPage(topologyRequest, request) {
        if (_a[topologyRequest.type]) {
            return await _a[topologyRequest.type](topologyRequest, request);
        }
        return {
            title: "Topology",
            content: []
        };
    }
    static async 'fabric'(topologyRequest, request) {
        return await _a['switch-switch'](topologyRequest, request);
    }
    static async 'switch-switch'(topologyRequest, request) {
        let tab = {
            title: "Switch Switch",
            content: []
        };
        let devices = await __classPrivateFieldGet(this, _a, "m", _Topology_getSanSwitches).call(this, topologyRequest.item_id);
        if (request) {
            devices = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(request, devices);
        }
        let wwnConnections = [];
        let items = [];
        let sourceWwns = [];
        let tagetWWns = [];
        for (const device of devices) {
            let children = [];
            for (const relation of device.children) {
                const port = relation._child;
                const myWwn = (port.mappings.find(wwn => wwn.target === false))?.wwn;
                const targetWwn = (port.mappings.find(wwn => wwn.target === true))?.wwn;
                if (myWwn) {
                    sourceWwns.push(myWwn);
                    if (targetWwn) {
                        tagetWWns.push(targetWwn);
                    }
                    if (targetWwn) {
                        if (port.heatmaps.length >= 2) {
                            const data_in_util = port.heatmaps.find(heatmap => heatmap.metric === 'data_in_util');
                            const data_out_util = port.heatmaps.find(heatmap => heatmap.metric === 'data_out_util');
                            if (data_out_util && data_in_util) {
                                wwnConnections.push({
                                    sourceWwn: myWwn,
                                    destinationWwn: targetWwn,
                                    sourceType: 'switch',
                                    destinationType: 'switch',
                                    percent: +((data_out_util.value + data_in_util.value) / 2).toFixed(2),
                                    text: ''
                                });
                            }
                        }
                        else {
                            wwnConnections.push({
                                sourceWwn: myWwn,
                                destinationWwn: targetWwn,
                                sourceType: 'switch',
                                destinationType: 'switch',
                                percent: 0,
                                text: ''
                            });
                        }
                    }
                    children.push({
                        label: port.label,
                        id: port.item_id,
                        url: new URL_1.URL().generateFromObject(port),
                        wwn: myWwn
                    });
                }
            }
            items.push({
                label: device.label,
                id: device.item_id,
                url: new URL_1.URL().generateFromObject(device),
                children: children
            });
        }
        wwnConnections = wwnConnections.filter(conn => sourceWwns.includes(conn.destinationWwn));
        let connectedPorts = wwnConnections.map(conn => conn.sourceWwn).concat(wwnConnections.map(conn => conn.destinationWwn));
        let filteredItems = [];
        for (const item of items) {
            item.children = item.children.filter(item => connectedPorts.includes(item.wwn));
            if (item.children.length >= 1) {
                filteredItems.push(item);
            }
        }
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TopologyBox,
            groups: [
                {
                    label: "SAN",
                    children: filteredItems,
                    type: 'switch'
                }
            ],
            connections: wwnConnections
        });
        return tab;
    }
    static async 'switch-storage'(topologyRequest, request) {
        let tab = {
            title: "Storage Switch",
            content: []
        };
        let devices = await __classPrivateFieldGet(this, _a, "m", _Topology_getSanSwitches).call(this);
        if (request) {
            devices = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(request, devices);
        }
        let wwnConnections = [];
        let leftItems = [];
        let wwns = [];
        for (const device of devices) {
            let children = [];
            for (const relation of device.children) {
                const port = relation._child;
                const myWwn = port.mappings.find(wwn => wwn.target === false);
                if (myWwn) {
                    for (const mapping of port.mappings) {
                        if (mapping.target === false) {
                            continue;
                        }
                        wwns.push(mapping.wwn);
                        if (port.heatmaps.length >= 2) {
                            const data_in_util = port.heatmaps.find(heatmap => heatmap.metric === 'data_in_util');
                            const data_out_util = port.heatmaps.find(heatmap => heatmap.metric === 'data_out_util');
                            if (data_out_util && data_in_util) {
                                wwnConnections.push({
                                    sourceWwn: myWwn.wwn,
                                    destinationWwn: mapping.wwn,
                                    sourceType: 'san',
                                    destinationType: 'storage',
                                    percent: +((data_out_util.value + data_in_util.value) / 2).toFixed(2),
                                    text: ''
                                });
                            }
                        }
                        else {
                            wwnConnections.push({
                                sourceWwn: myWwn.wwn,
                                destinationWwn: mapping.wwn,
                                sourceType: 'san',
                                destinationType: 'storage',
                                percent: 0,
                                text: ''
                            });
                        }
                    }
                    children.push({
                        label: port.label,
                        id: port.item_id,
                        url: new URL_1.URL().generateFromObject(port),
                        wwn: myWwn.wwn
                    });
                }
            }
            leftItems.push({
                label: device.label,
                id: device.item_id,
                url: new URL_1.URL().generateFromObject(device),
                children: children
            });
        }
        let mappedDevices = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                hw_type_entity: true,
                children: {
                    _child: {
                        mappings: true
                    }
                }
            },
            where: {
                item_id: topologyRequest.item_id
                    ? topologyRequest.item_id
                    : undefined,
                hw_type_entity: {
                    class: 'storage'
                },
                subsystem: 'device',
                children: {
                    _child: {
                        subsystem: (0, typeorm_1.In)(['port', 'fcport', 'feport']),
                        mappings: {
                            wwn: (0, typeorm_1.In)(wwns),
                            target: false
                        }
                    }
                }
            }
        });
        if (request) {
            mappedDevices = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(request, mappedDevices);
        }
        let deviceWwns = [];
        let rightItems = [];
        for (const device of mappedDevices) {
            let children = [];
            for (const child of device.children) {
                const wwn = child._child.mappings[0] ? child._child.mappings[0].wwn : undefined;
                deviceWwns.push(wwn);
                children.push({
                    label: child._child.label,
                    id: child._child.item_id,
                    wwn: wwn,
                    url: new URL_1.URL().generateFromObject(child._child)
                });
            }
            rightItems.push({
                label: device.label,
                id: device.item_id,
                url: new URL_1.URL().generateFromObject(device),
                children: children
            });
        }
        wwnConnections = wwnConnections.filter(conn => deviceWwns.includes(conn.destinationWwn));
        let connectedPorts = wwnConnections.map(conn => conn.sourceWwn);
        let filteredLeftItems = [];
        for (const leftItem of leftItems) {
            leftItem.children = leftItem.children.filter(item => connectedPorts.includes(item.wwn));
            if (leftItem.children.length >= 1) {
                filteredLeftItems.push(leftItem);
            }
        }
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TopologyBox,
            groups: [
                {
                    label: "SAN",
                    children: filteredLeftItems,
                    type: 'san'
                },
                {
                    label: "Storage",
                    children: rightItems,
                    type: 'storage'
                }
            ],
            connections: wwnConnections
        });
        return tab;
    }
    static async 'switch-host'(topologyRequest, request) {
        let tab = {
            title: "Host - Switch",
            content: []
        };
        let devices = await __classPrivateFieldGet(this, _a, "m", _Topology_getSanSwitches).call(this);
        if (request) {
            devices = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(request, devices);
        }
        let wwnConnections = [];
        let leftItems = [];
        let wwns = [];
        for (const device of devices) {
            let children = [];
            for (const relation of device.children) {
                const port = relation._child;
                const myWwn = port.mappings.find(wwn => wwn.target === false);
                if (myWwn) {
                    for (const mapping of port.mappings) {
                        if (mapping.target === false) {
                            continue;
                        }
                        wwns.push(mapping.wwn);
                        if (port.heatmaps.length >= 2) {
                            const data_in_util = port.heatmaps.find(heatmap => heatmap.metric === 'data_in_util');
                            const data_out_util = port.heatmaps.find(heatmap => heatmap.metric === 'data_out_util');
                            if (data_out_util && data_in_util) {
                                wwnConnections.push({
                                    sourceWwn: myWwn.wwn,
                                    destinationWwn: mapping.wwn,
                                    sourceType: 'san',
                                    destinationType: 'host',
                                    percent: +((data_out_util.value + data_in_util.value) / 2).toFixed(2),
                                    text: ''
                                });
                            }
                        }
                        else {
                            wwnConnections.push({
                                sourceWwn: myWwn.wwn,
                                destinationWwn: mapping.wwn,
                                sourceType: 'san',
                                destinationType: 'host',
                                percent: 0,
                                text: ''
                            });
                        }
                    }
                    children.push({
                        label: port.label,
                        id: port.item_id,
                        url: new URL_1.URL().generateFromObject(port),
                        wwn: myWwn.wwn
                    });
                }
            }
            leftItems.push({
                label: device.label,
                id: device.item_id,
                url: new URL_1.URL().generateFromObject(device),
                children: children
            });
        }
        let mappedDevices = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                hw_type_entity: true,
                children: {
                    _child: {
                        mappings: true
                    }
                }
            },
            where: {
                item_id: topologyRequest.item_id
                    ? topologyRequest.item_id
                    : undefined,
                hw_type_entity: {
                    class: 'virtualization'
                },
                subsystem: (0, typeorm_1.In)(['host', 'node', 'server']),
                hw_type: topologyRequest.hw_type
                    ? topologyRequest.hw_type
                    : undefined,
                children: {
                    _child: {
                        subsystem: (0, typeorm_1.In)(['virtualfibrenpiv']),
                        mappings: {
                            wwn: (0, typeorm_1.In)(wwns.concat(wwns.map(w => w.toLowerCase()))),
                            target: false
                        }
                    }
                }
            }
        });
        if (request) {
            mappedDevices = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(request, mappedDevices);
        }
        let deviceWwns = [];
        let rightItems = [];
        for (const device of mappedDevices) {
            let children = [];
            for (const child of device.children) {
                const wwn = child._child.mappings[0] ? child._child.mappings[0].wwn.toUpperCase() : undefined;
                deviceWwns.push(wwn);
                children.push({
                    label: child._child.label,
                    id: child._child.item_id,
                    wwn: wwn,
                    url: new URL_1.URL().generateFromObject(child._child)
                });
            }
            rightItems.push({
                label: device.label,
                id: device.item_id,
                url: new URL_1.URL().generateFromObject(device),
                children: children
            });
        }
        wwnConnections = wwnConnections.filter(conn => deviceWwns.includes(conn.destinationWwn));
        let connectedPorts = wwnConnections.map(conn => conn.sourceWwn);
        let filteredLeftItems = [];
        for (const leftItem of leftItems) {
            leftItem.children = leftItem.children.filter(item => connectedPorts.includes(item.wwn));
            if (leftItem.children.length >= 1) {
                filteredLeftItems.push(leftItem);
            }
        }
        tab.content.push({
            type: pageResponse_constants_1.BOX_TYPE.TopologyBox,
            groups: [
                {
                    label: "SAN",
                    children: filteredLeftItems,
                    type: 'san'
                },
                {
                    label: "Host",
                    children: rightItems,
                    type: 'host'
                }
            ],
            connections: wwnConnections
        });
        return tab;
    }
}
exports.Topology = Topology;
_a = Topology, _Topology_getSanSwitches = async function _Topology_getSanSwitches(fabric_id) {
    return await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
        relations: {
            hw_type_entity: true,
            children: {
                _child: {
                    mappings: true,
                    heatmaps: true
                }
            },
            parents: fabric_id ? true : false
        },
        where: {
            subsystem: 'device',
            hw_type_entity: {
                class: 'san'
            },
            children: {
                _child: {
                    subsystem: 'sanport',
                    mappings: true
                }
            },
            parents: fabric_id
                ? {
                    parent: fabric_id
                } : undefined
        }
    });
};
//# sourceMappingURL=Topology.js.map