"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VmwareConfigurationPage = exports.VmwareHeatmapPage = exports.VmwareOverviewPage = exports.VmwareDatastoretopPage = exports.VmwareVcentertopPage = exports.VmwareVmtopPage = exports.VmwareMetrics = exports.VmwareClustertotalsPage = exports.VmwareVcentertotalsPage = exports.VmwareResourcepoolPage = exports.VmwareEsxiviewPage = exports.VmwareEsxilanPage = exports.VmwareEsxidiskPage = exports.VmwareEsximemoryPage = exports.VmwareEsxicpuPage = exports.VmwareEsxiPage = exports.VmwareDatastorePage = exports.VmwareVmPage = exports.VmwareRpfolderFolder = exports.VmwareResourcepoolFolder = exports.VmwareDsfolderFolder = exports.VmwareDatacenterFolder = exports.VmwareVmfolderFolder = exports.VmwareVmFolder = exports.VmwareEsxiitemFolder = exports.VmwareEsxiFolder = exports.VmwareClusterFolder = exports.VmwareVcenterFolder = exports.VmwareRootFolder = exports.VmwareSingleItemPage = exports.VmwareSubsystemFolder = exports.VmwarePage = exports.VmwareFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemProperty_entity_1 = require("../../../entity/itemProperty.entity");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const ACLCheck_1 = require("../../acl/ACLCheck");
const Color_1 = require("../../metrics/Color");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const typeorm_1 = require("typeorm");
const ConfigurationTable_1 = require("./ConfigurationTable");
const CustomTop_1 = require("./CustomTop");
const Heatmap_1 = require("./Heatmap");
const Overview_1 = require("./Overview");
const Table_1 = require("./Table");
const Top_1 = require("./Top");
const URL_1 = require("./URL");
const Prediction_1 = require("../../metrics/Prediction");
const cpu_peak_help = {
    url: "https://xormon.com/max_vrs_average.php",
    text: "How it works?",
};
class VmwareFolder {
    constructor() {
        this.hwType = 'vmware';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
    async getPathFolder(parentIds) {
        const children = await this.getChildren();
        for (const child of children) {
            const child_id = parentIds.find((v) => v === child.itemId);
            if (child_id) {
                if (child._new) {
                    child.children = await new child._new(child_id)["getPathFolder"](parentIds);
                }
            }
        }
        return children;
    }
    async getChildren() {
        return null;
    }
}
exports.VmwareFolder = VmwareFolder;
class VmwarePage {
    constructor(itemId, request) {
        this.itemId = itemId;
        this.request = request;
        this.hwType = 'vmware';
        this.withItemId = false;
        this.otherParents = false;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        this.itemPropertiesRepository = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
    }
    ;
    async getPath() {
        const parents = await this.objectItemsRepository.find({
            relations: {
                children: true
            },
            where: {
                children: {
                    child: this.itemId
                }
            }
        });
        if (this.otherParents) {
            const otherParents = await this.objectItemsRepository.find({
                relations: {
                    children: true
                },
                where: {
                    children: {
                        child: (0, typeorm_1.In)(parents.map(item => item.item_id))
                    },
                    hw_type: this.hwType
                }
            });
            parents.push(...otherParents);
        }
        const parentIds = parents.map(item => item.item_id);
        if (this.withItemId) {
            parentIds.push(this.itemId);
        }
        const path = await new VmwareRootFolder().getPathFolder(parentIds);
        return path;
    }
}
exports.VmwarePage = VmwarePage;
class VmwareSubsystemFolder extends VmwareFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.VmwareSubsystemFolder = VmwareSubsystemFolder;
class VmwareSingleItemPage extends VmwarePage {
}
exports.VmwareSingleItemPage = VmwareSingleItemPage;
class VmwareRootFolder extends VmwareFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration', 'configuration'));
        menuItems.push(new menuResponse_dto_1.MenuLink('TOP Datastore', this.hwType + '/datastore-top', 'datastore-top'));
        menuItems.push(new menuResponse_dto_1.MenuLink('TOP VM', this.hwType + '/vm-top', 'vm-top'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Overview', this.hwType + '/overview', 'overview'));
        let vCenters = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: 'vcenter'
        });
        vCenters.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let vCenter of vCenters) {
            let path = this.hwType + '/vcenter?ids=' + vCenter.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(vCenter.label, path, this.hwType, 'vcenter', vCenter.item_id, VmwareVcenterFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        let vCenters = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: 'vcenter'
        });
        vCenters.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let vCenter of vCenters) {
            let path = this.hwType + '/vcenter?ids=' + vCenter.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(vCenter.label, path, this.hwType, 'vcenter', vCenter.item_id, VmwareVcenterFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareRootFolder = VmwareRootFolder;
class VmwareVcenterFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/vcenter-totals?item_id=' + this.itemId, 'vcenter-totals', this.itemId, true));
        menuItems.push(new menuResponse_dto_1.MenuLink('TOP VM', this.hwType + '/vcenter-top?item_id=' + this.itemId, 'vcenter-top', this.itemId));
        let childrenIds = (await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } })).map((relation) => relation.child);
        let children = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: (0, typeorm_1.In)(['cluster', 'datacenter']),
                item_id: (0, typeorm_1.In)(childrenIds)
            }, order: {
                subsystem: 'ASC',
                label: 'ASC'
            }
        });
        const prefix = {
            cluster: "Cluster",
            datacenter: "Datastores"
        };
        const folder = {
            cluster: VmwareClusterFolder,
            datacenter: VmwareDatacenterFolder
        };
        for (const child of children) {
            let path = this.hwType + '/' + child.subsystem + '?ids=' + child.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(prefix[child.subsystem] + ": " + child.label, path, this.hwType, child.subsystem, child.item_id, folder[child.subsystem]));
        }
        let esxiCheck = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'esxi',
                item_id: (0, typeorm_1.In)(childrenIds),
                parents: true
            },
            relations: {
                parents: true
            }
        })).find(esxi => esxi.parents.length === 1);
        if (esxiCheck) {
            let path = this.hwType + '/esxi' + '?ids=' + this.itemId.toString();
            menuItems.push(new menuResponse_dto_1.MenuFolder("ESXi", path, this.hwType, 'esxi', this.itemId.toString(), VmwareEsxiFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        let childrenIds = (await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } })).map((relation) => relation.child);
        let children = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'cluster',
                item_id: (0, typeorm_1.In)(childrenIds)
            }, order: {
                subsystem: 'ASC',
                label: 'ASC'
            }
        });
        for (const child of children) {
            let path = this.hwType + '/cluster?ids=' + child.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder("Cluster: " + child.label, path, this.hwType, 'cluster', child.item_id, VmwareClusterFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareVcenterFolder = VmwareVcenterFolder;
class VmwareClusterFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/cluster-totals?item_id=' + this.itemId, 'cluster-totals', this.itemId, true));
        let childrenIds = (await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } })).map((relation) => relation.child);
        let resourcePoolCheck = await this.objectItemsRepository.findOne({
            where: {
                hw_type: this.hwType,
                subsystem: 'respool',
                item_id: (0, typeorm_1.In)(childrenIds)
            }
        });
        if (resourcePoolCheck) {
            let path = this.hwType + '/resource-pool' + (this.itemId !== null ? '?ids=' + this.itemId : '');
            menuItems.push(new menuResponse_dto_1.MenuFolder("Resource Pool", path, this.hwType, 'resource-pool', this.itemId, VmwareResourcepoolFolder));
        }
        let esxiCheck = await this.objectItemsRepository.findOne({
            where: {
                hw_type: this.hwType,
                subsystem: 'esxi',
                item_id: (0, typeorm_1.In)(childrenIds)
            }
        });
        if (esxiCheck) {
            let path = this.hwType + '/esxi' + (this.itemId !== null ? '?ids=' + this.itemId : '');
            menuItems.push(new menuResponse_dto_1.MenuFolder("ESXi", path, this.hwType, 'esxi', this.itemId, VmwareEsxiFolder));
        }
        let path = this.hwType + '/vm' + (this.itemId !== null ? '?ids=' + this.itemId : '');
        menuItems.push(new menuResponse_dto_1.MenuFolder("VM", path, this.hwType, 'vm', this.itemId, VmwareVmFolder));
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        let path = this.hwType + '/vm' + (this.itemId !== null ? '?ids=' + this.itemId : '');
        menuItems.push(new menuResponse_dto_1.MenuFolder('VM', path, this.hwType, 'vm', this.itemId, VmwareVmFolder));
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareClusterFolder = VmwareClusterFolder;
class VmwareEsxiFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let parent = await this.objectItemsRepository.findOneBy({
            item_id: this.itemId.toString()
        });
        let esxis = [];
        if (parent.subsystem === 'vcenter') {
            esxis = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'esxi',
                }, order: {
                    subsystem: 'DESC'
                },
                relations: {
                    parents: {
                        _parent: true
                    }
                }
            });
            esxis = esxis.filter(esxi => esxi.parents.find(esxiParent => esxiParent._parent.item_id === parent.item_id) && esxi.parents.length === 1);
        }
        else {
            esxis = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'esxi',
                    parents: {
                        _parent: {
                            item_id: this.itemId.toString()
                        }
                    }
                }, order: {
                    subsystem: 'DESC'
                },
                relations: {
                    parents: {
                        _parent: true
                    }
                }
            });
        }
        esxis.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const esxi of esxis) {
            let path = this.hwType + '/esxi-item?ids=' + esxi.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(esxi.label, path, this.hwType, 'esxi-item', esxi.item_id, VmwareEsxiitemFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareEsxiFolder = VmwareEsxiFolder;
class VmwareEsxiitemFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('CPU', this.hwType + '/esxi-cpu?item_id=' + this.itemId, 'esxi-cpu'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Memory', this.hwType + '/esxi-memory?item_id=' + this.itemId, 'esxi-memory'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Disk', this.hwType + '/esxi-disk?item_id=' + this.itemId, 'esxi-disk'));
        menuItems.push(new menuResponse_dto_1.MenuLink('LAN', this.hwType + '/esxi-lan?item_id=' + this.itemId, 'esxi-lan'));
        menuItems.push(new menuResponse_dto_1.MenuLink('VIEW', this.hwType + '/esxi-view?item_id=' + this.itemId, 'esxi-view'));
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareEsxiitemFolder = VmwareEsxiitemFolder;
class VmwareVmFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let clusterWithFolders = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        parents: true
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'vm_folder',
                    }
                }
            }
        });
        let folderIds = [];
        if (clusterWithFolders) {
            clusterWithFolders.children.sort((a, b) => {
                return a._child.label > b._child.label ? 1 : -1;
            });
            folderIds = clusterWithFolders.children.map(relation => relation.child);
            for (const childIndex in clusterWithFolders.children) {
                const vmFolder = clusterWithFolders.children[childIndex]._child;
                if (!vmFolder.parents.find(vmFolderParent => folderIds.includes(vmFolderParent.parent))) {
                    let path = this.hwType + '/vm_folder?ids=' + vmFolder.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuFolder(vmFolder.label, path, this.hwType, 'vm_folder', vmFolder.item_id, VmwareVmfolderFolder));
                }
            }
        }
        let clusterWithVms = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        parents: {
                            _parent: true
                        }
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'vm'
                    }
                }
            }
        });
        if (clusterWithVms) {
            clusterWithVms.children.sort((a, b) => a._child.label.localeCompare(b._child.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (const childIndex in clusterWithVms.children) {
                const vm = clusterWithVms.children[childIndex]._child;
                if (!vm.parents.find(vmParent => vmParent._parent.subsystem === "vm_folder")) {
                    menuItems.push(new menuResponse_dto_1.MenuLink(vm.label, this.hwType + '/vm?item_id=' + vm.item_id, 'vm', vm.item_id));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        menuItems = await this.getChildren();
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareVmFolder = VmwareVmFolder;
class VmwareVmfolderFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let vmFolderWithSubfolders = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        parents: {
                            _parent: true
                        }
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'vm_folder'
                    }
                }
            }
        });
        let folderIds = [];
        if (vmFolderWithSubfolders) {
            folderIds = vmFolderWithSubfolders.children.map(child => {
                return child._child.item_id;
            });
            vmFolderWithSubfolders.children.sort((a, b) => a._child.label.localeCompare(b._child.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (const childIndex in vmFolderWithSubfolders.children) {
                const vmFolder = vmFolderWithSubfolders.children[childIndex]._child;
                if (!vmFolder.parents.find(vmFolderParent => folderIds.includes(vmFolderParent.parent))) {
                    let path = this.hwType + '/vm_folder?ids=' + vmFolder.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuFolder(vmFolder.label, path, this.hwType, 'vm_folder', vmFolder.item_id, VmwareVmfolderFolder));
                }
            }
        }
        let clusterWithVms = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        parents: true
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'vm'
                    }
                }
            }
        });
        if (clusterWithVms) {
            clusterWithVms.children.sort((a, b) => a._child.label.localeCompare(b._child.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (const childIndex in clusterWithVms.children) {
                const vm = clusterWithVms.children[childIndex]._child;
                if (!vm.parents.find(vmParent => folderIds.includes(vmParent.parent))) {
                    menuItems.push(new menuResponse_dto_1.MenuLink(vm.label, this.hwType + '/vm?item_id=' + vm.item_id, 'vm', vm.item_id));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        menuItems = await this.getChildren();
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareVmfolderFolder = VmwareVmfolderFolder;
class VmwareDatacenterFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let datacenterWithFolders = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        parents: {
                            _parent: true
                        }
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'ds_folder',
                    }
                }
            }
        });
        let folderIds = [];
        if (datacenterWithFolders) {
            folderIds = datacenterWithFolders.children.map(child => {
                return child._child.item_id;
            });
            datacenterWithFolders.children.sort((a, b) => {
                return a._child.label > b._child.label ? 1 : -1;
            });
            for (const childIndex in datacenterWithFolders.children) {
                const vmFolder = datacenterWithFolders.children[childIndex]._child;
                if (!vmFolder.parents.find(vmFolderParent => vmFolderParent._parent.subsystem === 'ds_folder')) {
                    let path = this.hwType + '/ds-folder?ids=' + vmFolder.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuFolder(vmFolder.label, path, this.hwType, 'ds-folder', vmFolder.item_id, VmwareDsfolderFolder));
                }
            }
        }
        let datacenterWithDatastores = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        parents: true
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'datastore'
                    }
                }
            }
        });
        if (datacenterWithDatastores) {
            datacenterWithDatastores.children.sort((a, b) => {
                return a._child.label > b._child.label ? 1 : -1;
            });
            for (const childIndex in datacenterWithDatastores.children) {
                const datastore = datacenterWithDatastores.children[childIndex]._child;
                if (!datastore.parents.find(dsParent => folderIds.includes(dsParent.parent))) {
                    menuItems.push(new menuResponse_dto_1.MenuLink(datastore.label, this.hwType + '/datastore?item_id=' + datastore.item_id, 'datastore'));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareDatacenterFolder = VmwareDatacenterFolder;
class VmwareDsfolderFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let dsFolderWithSubfolders = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        parents: {
                            _parent: true
                        }
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'ds_folder'
                    }
                }
            }
        });
        let folderIds = [];
        if (dsFolderWithSubfolders) {
            folderIds = dsFolderWithSubfolders.children.map(child => {
                return child._child.item_id;
            });
            dsFolderWithSubfolders.children.sort((a, b) => {
                return a._child.label > b._child.label ? 1 : -1;
            });
            for (const childIndex in dsFolderWithSubfolders.children) {
                const dsFolder = dsFolderWithSubfolders.children[childIndex]._child;
                let path = this.hwType + '/ds-folder?ids=' + dsFolder.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(dsFolder.label, path, this.hwType, 'ds-folder', dsFolder.item_id, VmwareDsfolderFolder));
            }
        }
        let clusterWithDatastores = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        parents: true
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'datastore'
                    }
                }
            }
        });
        if (clusterWithDatastores) {
            clusterWithDatastores.children.sort((a, b) => {
                return a._child.label > b._child.label ? 1 : -1;
            });
            for (const childIndex in clusterWithDatastores.children) {
                const datastore = clusterWithDatastores.children[childIndex]._child;
                if (!datastore.parents.find(dsParent => folderIds.includes(dsParent.parent))) {
                    menuItems.push(new menuResponse_dto_1.MenuLink(datastore.label, this.hwType + '/datastore?item_id=' + datastore.item_id, 'datastore'));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareDsfolderFolder = VmwareDsfolderFolder;
class VmwareResourcepoolFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let clusterWithResourcePools = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        parents: {
                            _parent: true
                        },
                        children: {
                            _child: true
                        }
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'respool'
                    }
                }
            }
        });
        if (clusterWithResourcePools) {
            clusterWithResourcePools.children.sort((a, b) => a._child.label.localeCompare(b._child.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (const childIndex in clusterWithResourcePools.children) {
                const rp = clusterWithResourcePools.children[childIndex]._child;
                if (rp.parents.find(parent => parent._parent.subsystem === 'respool')) {
                    continue;
                }
                if (rp.children.length >= 1 && rp.children.find(child => child._child.subsystem === 'respool')) {
                    let path = this.hwType + '/rp-folder?ids=' + rp.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuFolder(rp.label, path, this.hwType, 'rp-folder', rp.item_id, VmwareRpfolderFolder));
                }
                else {
                    menuItems.push(new menuResponse_dto_1.MenuLink(rp.label, this.hwType + '/resource-pool?item_id=' + rp.item_id, 'resource-pool'));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareResourcepoolFolder = VmwareResourcepoolFolder;
class VmwareRpfolderFolder extends VmwareSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let rpFolderWithSubfolders = await this.objectItemsRepository.findOne({
            relations: {
                children: {
                    _child: {
                        children: {
                            _child: true
                        },
                        parents: true
                    }
                }
            },
            where: {
                item_id: this.itemId.toString(),
                children: {
                    _child: {
                        subsystem: 'respool'
                    }
                }
            }
        });
        let rpFoldersIds = [];
        if (rpFolderWithSubfolders) {
            menuItems.push(new menuResponse_dto_1.MenuLink("Total", this.hwType + '/resource-pool?item_id=' + this.itemId.toString(), 'resourcepool', this.itemId.toString(), true));
            rpFoldersIds = rpFolderWithSubfolders.children.map(child => child._child.item_id);
            rpFolderWithSubfolders.children.sort((a, b) => a._child.label.localeCompare(b._child.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (const childIndex in rpFolderWithSubfolders.children) {
                const rp = rpFolderWithSubfolders.children[childIndex]._child;
                if (rp.parents.find(parent => rpFoldersIds.includes(parent.parent))) {
                    continue;
                }
                if (rp.children.length >= 1 && rp.children.find(child => child._child.subsystem === 'respool')) {
                    let path = this.hwType + '/rp-folder?ids=' + rp.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuFolder(rp.label, path, this.hwType, 'rp-folder', rp.item_id, VmwareRpfolderFolder));
                }
                else {
                    menuItems.push(new menuResponse_dto_1.MenuLink(rp.label, this.hwType + '/resource-pool?item_id=' + rp.item_id, 'resource-pool', rp.item_id));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.VmwareRpfolderFolder = VmwareRpfolderFolder;
class VmwareVmPage extends VmwareSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOne({
            where: {
                hw_type: this.hwType,
                subsystem: this.subsystem,
                item_id: this.itemId,
            }, relations: {
                properties: true
            }
        });
        let title = 'VM ' + (item !== null ? item.label : this.itemId);
        const metricsWrapper = await (new MetricsWrapper_1.MetricsWrapper(['cpu_alloc'], [this.itemId])).getLastDailyValuesFromTable();
        const maxCpuHz = metricsWrapper[this.itemId]?.['cpu_alloc'];
        const maxCpuCores = item.properties.find(property => property.property_name === 'vCPU')?.property_value;
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_utilization_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('vm', 'vcpu')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.black)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new VmwareMetrics.Metric('vm', 'cpu_utilization_cores')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_vm',
                    interval: 'month',
                    max: +maxCpuCores
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new VmwareMetrics.Metric('vm', 'cpu_utilization_cores')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_vm',
                    interval: 'year',
                    max: +maxCpuCores
                }),
            ],
        });
        tabs.push({
            title: 'CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_utilization_percent')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('vm', 'vm_cpu_ready')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.brownDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new VmwareMetrics.Metric('vm', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_vm',
                    interval: 'month',
                    max: 100
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new VmwareMetrics.Metric('vm', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_vm',
                    interval: 'year',
                    max: 100
                }),
            ],
        });
        tabs.push({
            title: 'CPU Hz',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_alloc')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, false, 0, Color_1.Color.pallete.green),
                        new VmwareMetrics.Metric('vm', 'cpu_usage')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('vm', 'cpu_usage_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'memory_granted')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new VmwareMetrics.Metric('vm', 'memory_baloon')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                        new VmwareMetrics.Metric('vm', 'memory_active')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Disk',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'disk_read')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('vm', 'disk_write')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        let datastores = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'datastore_vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore_vm', 'iops_read')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1),
                        new VmwareMetrics.Metric('datastore_vm', 'iops_write')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0)
                    ],
                    uuids: datastores.map(ds => ds.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'network_received')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('vm', 'network_transmitted')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Swap',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'memory_swapout')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('vm', 'memory_swapin')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Comp',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'memory_decompres')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('vm', 'memory_compres')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'CPU Ready',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_ready')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        let vm_cpu = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'esxi_vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'vMotion',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi_vm', 'cpu_usage_proc')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0)
                    ],
                    uuids: vm_cpu.map(vm => vm.item_id),
                    aggregated: true,
                    parent: {
                        label: 'ESXi',
                        subsystem: 'esxi'
                    }
                },
            ],
        });
        tabs.push({
            title: 'Space Provisioned',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore_vm', 'provisioned_space')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: datastores.map(ds => ds.item_id),
                    aggregated: true,
                    parent: {
                        label: 'Datastore',
                        subsystem: 'datastore'
                    },
                    label: 'VM'
                },
            ],
        });
        tabs.push({
            title: 'Space Used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore_vm', 'used_space')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: datastores.map(ds => ds.item_id),
                    aggregated: true,
                    parent: {
                        label: 'Datastore',
                        subsystem: 'datastore'
                    },
                    label: 'VM'
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareVmPage = VmwareVmPage;
class VmwareDatastorePage extends VmwareSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'datastore';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Datastore ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'Space',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'disk_capacity')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, false, 0, Color_1.Color.pallete.green),
                        new VmwareMetrics.Metric('datastore', 'disk_used')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, false, 1, Color_1.Color.pallete.red),
                        new VmwareMetrics.Metric('datastore', 'disk_provisioned')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        tabs.push({
            title: 'Used Space %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'disk_percent_used')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'read')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('datastore', 'write')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'read_avg')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('datastore', 'write_avg')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        let datastores = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'datastore_vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'IO/VM',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore_vm', 'iops_read')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1, pageResponse_constants_1.METRIC_AGGREGATE.items),
                        new VmwareMetrics.Metric('datastore_vm', 'iops_write')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: datastores.map(ds => ds.item_id),
                    aggregated: true,
                    parent: {
                        label: 'Datastore',
                        subsystem: 'datastore'
                    },
                    label: 'VM'
                },
            ],
        });
        tabs.push({
            title: 'Space/VM Provisioned',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore_vm', 'provisioned_space')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: datastores.map(ds => ds.item_id),
                    aggregated: true,
                    parent: {
                        label: 'Datastore',
                        subsystem: 'datastore'
                    },
                    label: 'VM'
                },
            ],
        });
        tabs.push({
            title: 'Space/VM Used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore_vm', 'used_space')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: datastores.map(ds => ds.item_id),
                    aggregated: true,
                    parent: {
                        label: 'Datastore',
                        subsystem: 'datastore'
                    },
                    label: 'VM'
                },
            ],
        });
        tabs.push({
            title: 'Latency',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'read_latency')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('datastore', 'write_latency')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        const configurationTable = new ConfigurationTable_1.ConfigurationTable(vms.map(vm => vm.item_id), ['powerState', 'Provisioned_Space_GB', 'Used_Space_GB']);
        configurationTable.setItemLabel("VM name");
        configurationTable.setAcl(this.request);
        const itemProperties = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'mounted' } });
        let esxisTable = new Table_1.Table()
            .addHeader({ value: 'Mounted ESXi name' })
            .addHeader({ value: 'accessible' })
            .addHeader({ value: 'access mode' });
        if (itemProperties) {
            let mountedHosts = JSON.parse(itemProperties.property_value);
            for (const itemId in mountedHosts) {
                const host = mountedHosts[itemId];
                esxisTable.addRow([
                    { order: 0, value: host.label },
                    { order: 1, value: host.accessible },
                    { order: 2, value: host.access_mode },
                ]);
            }
        }
        tabs.push({
            title: 'VM list',
            content: [
                esxisTable.generateBox(),
                await configurationTable.generate()
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareDatastorePage = VmwareDatastorePage;
class VmwareEsxiPage extends VmwareSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'esxi';
        this.withItemId = true;
    }
}
exports.VmwareEsxiPage = VmwareEsxiPage;
class VmwareEsxicpuPage extends VmwareEsxiPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'ESXi ' + (item !== null ? item.label : this.itemId);
        const metricsWrapper = await (new MetricsWrapper_1.MetricsWrapper(['cpu_alloc'], [this.itemId])).getLastDailyValuesFromTable();
        const maxCpu = metricsWrapper[this.itemId]?.['cpu_alloc'];
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'cpu_alloc')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new VmwareMetrics.Metric('esxi', 'cpu_usage')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.red),
                        new VmwareMetrics.Metric('esxi', 'cpu_usage_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, Color_1.Color.pallete.lightblue)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new VmwareMetrics.Metric('esxi', 'cpu_usage').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_cpu',
                    interval: 'month',
                    max: maxCpu
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new VmwareMetrics.Metric('esxi', 'cpu_usage').getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_cpu',
                    interval: 'year',
                    max: maxCpu
                }),
            ],
            help: cpu_peak_help,
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'VM aggregated',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(item => item.item_id),
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'Power',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'power_usage')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        const configurationTable = new ConfigurationTable_1.ConfigurationTable(vms.map(vm => vm.item_id), ['vCPU', 'Reserved_MHz', 'Limit_MHz', 'Shares', 'Shares_value', 'Provisioned_Space_GB', 'Used_Space_GB', 'IpAddress', 'powerState', 'toolsStatus', 'OS']);
        configurationTable.setAcl(this.request);
        configurationTable.setItemLabel("VM");
        tabs.push({
            title: 'Configuration',
            content: [await configurationTable.generate()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareEsxicpuPage = VmwareEsxicpuPage;
class VmwareEsximemoryPage extends VmwareEsxiPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'ESXi ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'Allocation',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'memory_host_size')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new VmwareMetrics.Metric('esxi', 'memory_granted')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.greenDark),
                        new VmwareMetrics.Metric('esxi', 'memory_active')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('esxi', 'memory_baloon')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 3, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'Aggregated',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'memory_granted')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(item => item.item_id),
                    aggregated: true,
                }
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareEsximemoryPage = VmwareEsximemoryPage;
class VmwareEsxidiskPage extends VmwareEsxiPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'ESXi ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'Vmdiskrw',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'disk_read')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('esxi', 'disk_write')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        const configurationTable = new ConfigurationTable_1.ConfigurationTable(vms.map(vm => vm.item_id), ['Provisioned_Space_GB', 'Used_Space_GB']);
        configurationTable.setItemLabel("VM");
        configurationTable.setAcl(this.request);
        tabs.push({
            title: 'Space',
            content: [await configurationTable.generate()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareEsxidiskPage = VmwareEsxidiskPage;
class VmwareEsxilanPage extends VmwareEsxiPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'ESXi ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'network_received')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('vm', 'network_transmitted')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareEsxilanPage = VmwareEsxilanPage;
class VmwareEsxiviewPage extends VmwareEsxiPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'ESXi ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        const intervals = [{
                label: 'Daily',
                interval: pageResponse_constants_1.GRAPH_INTERVAL.day
            }, {
                label: 'Weekly',
                interval: pageResponse_constants_1.GRAPH_INTERVAL.week
            }, {
                label: 'Monthly',
                interval: pageResponse_constants_1.GRAPH_INTERVAL.month
            }, {
                label: 'Yearly',
                interval: pageResponse_constants_1.GRAPH_INTERVAL.year
            }];
        const vms = await this.objectItemsRepository.find({
            where: {
                hw_type: hwTypeKey_constants_1.HwTypeKey.VMWARE,
                subsystem: 'vm',
                parents: {
                    parent: item.item_id
                }
            },
            relations: {
                parents: true
            }
        });
        const vmIds = vms.map(item => item.item_id);
        for (const interval of intervals) {
            let vmsGraphs = [];
            for (const vm of vms) {
                vmsGraphs.push({
                    interval: interval.interval,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_alloc')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new VmwareMetrics.Metric('vm', 'cpu_usage')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [vm.item_id],
                    aggregated: false,
                    graphLabel: vm.label
                });
            }
            tabs.push({
                title: interval.label,
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                        graphs: [
                            {
                                interval: interval.interval,
                                metrics: [
                                    new VmwareMetrics.Metric('esxi', 'cpu_alloc')
                                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                                    new VmwareMetrics.Metric('esxi', 'cpu_usage')
                                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark)
                                ],
                                uuids: [item.item_id],
                                aggregated: false,
                            },
                            {
                                interval: interval.interval,
                                metrics: [
                                    new VmwareMetrics.Metric('esxi', 'memory_host_size')
                                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                                    new VmwareMetrics.Metric('esxi', 'memory_granted')
                                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.greenDark),
                                    new VmwareMetrics.Metric('esxi', 'memory_active')
                                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, Color_1.Color.pallete.redDark),
                                    new VmwareMetrics.Metric('esxi', 'memory_baloon')
                                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 3, Color_1.Color.pallete.blueDark)
                                ],
                                uuids: [item.item_id],
                                aggregated: false,
                            },
                            {
                                interval: interval.interval,
                                metrics: [
                                    new VmwareMetrics.Metric('vm', 'cpu_usage')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                                ],
                                uuids: vmIds,
                                aggregated: true,
                            },
                            {
                                interval: interval.interval,
                                metrics: [
                                    new VmwareMetrics.Metric('vm', 'memory_granted')
                                        .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0),
                                ],
                                uuids: vmIds,
                                aggregated: true,
                            }
                        ].concat(vmsGraphs),
                    },
                ],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareEsxiviewPage = VmwareEsxiviewPage;
class VmwareResourcepoolPage extends VmwareSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'respool';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Resource Pool ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('respool', 'cpu_reservation')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, false, 0, Color_1.Color.pallete.green),
                        new VmwareMetrics.Metric('respool', 'cpu_usage')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('respool', 'cpu_limit')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, Color_1.Color.pallete.black)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                }
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'CPU VM',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(item => item.item_id),
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('respool', 'memory_reservation')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.purple),
                        new VmwareMetrics.Metric('respool', 'memory_granted')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.greenDark),
                        new VmwareMetrics.Metric('respool', 'memory_consumed')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, Color_1.Color.pallete.blueDark),
                        new VmwareMetrics.Metric('respool', 'memory_active')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 3, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('respool', 'memory_baloon')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 4, Color_1.Color.pallete.yellowDark),
                        new VmwareMetrics.Metric('respool', 'memory_swap')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 5, Color_1.Color.pallete.orangeDark),
                        new VmwareMetrics.Metric('respool', 'memory_limit')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 6, Color_1.Color.pallete.black)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareResourcepoolPage = VmwareResourcepoolPage;
class VmwareVcentertotalsPage extends VmwareSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'vcenter';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'vCenter ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        let esxis = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'esxi',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'Server',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'cpu_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: esxis.map(esxi => esxi.item_id),
                    aggregated: true,
                    label: 'ESXi',
                    parent: {
                        label: 'Cluster',
                        subsystem: 'cluster'
                    }
                },
            ],
        });
        tabs.push({
            title: 'Server %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: esxis.map(esxi => esxi.item_id),
                    aggregated: true,
                    label: 'ESXi',
                    parent: {
                        label: 'Cluster',
                        subsystem: 'cluster'
                    }
                },
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'CPU VM',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Cluster',
                        subsystem: 'cluster'
                    }
                },
            ],
        });
        tabs.push({
            title: 'CPU VM %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Cluster',
                        subsystem: 'cluster'
                    }
                },
            ],
        });
        tabs.push({
            title: 'Memory VM Active',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'memory_active')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Cluster',
                        subsystem: 'cluster'
                    }
                },
            ], help: {
                url: 'https://xormon.com/VMware-VM-active-memory.php',
                text: 'Active Memory'
            },
        });
        tabs.push({
            title: 'Memory VM Granted',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'memory_granted')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Cluster',
                        subsystem: 'cluster'
                    }
                },
            ],
        });
        let clusters = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'cluster',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'CPU Clusters',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('cluster', 'cpu_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: clusters.map(cluster => cluster.item_id),
                    aggregated: true,
                    label: 'Cluster'
                },
            ],
        });
        tabs.push({
            title: 'CPU Clusters %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('cluster', 'cpu_usage_proc')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: clusters.map(cluster => cluster.item_id),
                    aggregated: true,
                    label: 'Cluster'
                },
            ],
        });
        tabs.push({
            title: 'Power',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'power_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: esxis.map(esxi => esxi.item_id),
                    aggregated: true,
                    label: 'ESXi'
                },
            ],
        });
        let vcenter = await this.objectItemsRepository.findOne({
            where: {
                hw_type: 'vmware',
                subsystem: 'vcenter',
                item_id: this.itemId
            }
        });
        const configurationTableVcenter = new ConfigurationTable_1.ConfigurationTable([vcenter.item_id], ['host', 'vcenter_ghz', 'vcenter_overallCpuUsage_ghz', 'vcenter_memorySize_gb', 'vcenter_overallMemoryUsage_gb', 'fullName_top', 'vcenter_last_update', 'clusters', 'esxis', 'vcenter_vms_count']);
        configurationTableVcenter.setItemLabel("Alias");
        configurationTableVcenter.setAcl(this.request);
        const configurationTableEsxis = new ConfigurationTable_1.ConfigurationTable(esxis.map(esxi => esxi.item_id), ['vcente_name', 'cluste_name', 'host_ghz', 'overallCpuUsage_ghz', 'host_memorySize_gb', 'overallMemoryUsage_gb', 'hw_numCpuCores', 'hw_numCpuThreads', 'sw_fullName', 'esxi_uptime_days', 'hw_vendor', 'hw_model', 'hw_cpuModel']);
        configurationTableEsxis.setItemLabel("ESXi");
        configurationTableEsxis.setAcl(this.request);
        tabs.push({
            title: 'Configuration',
            content: [
                await configurationTableVcenter.generate(),
                await configurationTableEsxis.generate(),
            ],
        });
        tabs.push({
            title: 'vMotion TOP',
            content: [await getVmotionTop(this.request, this.itemId)],
        });
        let dstrvms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'datastore_vm',
                parents: {
                    parent: this.itemId,
                }
            },
            relations: {
                parents: true,
                properties: true
            }
        });
        dstrvms = dstrvms.filter(datastore_vm => datastore_vm.properties && datastore_vm.properties.length);
        const configurationTable = new ConfigurationTable_1.ConfigurationTable(dstrvms.map(datastore_vm => datastore_vm.item_id), ['provisioned', 'used'], ['vm', 'vcenter', 'cluster']);
        configurationTable.setItemLabel("Datastore name");
        configurationTable.setAcl(this.request);
        tabs.push({
            title: 'Datastore list',
            content: [
                await configurationTable.generate()
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareVcentertotalsPage = VmwareVcentertotalsPage;
class VmwareClustertotalsPage extends VmwareSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'cluster';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Cluster ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('cluster', 'cpu_utilization_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                        new VmwareMetrics.Metric('cluster', 'total_cluster_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.black)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                    label: 'Cluster'
                },
            ],
        });
        tabs.push({
            title: 'CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('cluster', 'cpu_usage_proc')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                    label: 'Cluster'
                },
            ],
        });
        tabs.push({
            title: 'CPU Hz',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('cluster', 'cpu_total')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new VmwareMetrics.Metric('cluster', 'cpu_total_eff')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                        new VmwareMetrics.Metric('cluster', 'cpu_usage')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                    label: 'Cluster'
                },
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'CPU VM',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'ESXi',
                        subsystem: 'esxi'
                    }
                },
            ],
        });
        tabs.push({
            title: 'CPU VM %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'ESXi',
                        subsystem: 'esxi'
                    }
                },
            ],
        });
        tabs.push({
            title: 'Memory VM Active',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'memory_active')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Cluster',
                        subsystem: 'cluster'
                    }
                },
            ], help: {
                url: 'https://xormon.com/VMware-VM-active-memory.php',
                text: 'Active Memory'
            },
        });
        tabs.push({
            title: 'Memory VM Granted',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'memory_granted')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Cluster',
                        subsystem: 'cluster'
                    }
                },
            ],
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('cluster', 'memory_total')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new VmwareMetrics.Metric('cluster', 'memory_granted')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.greenDark),
                        new VmwareMetrics.Metric('cluster', 'memory_consumed')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 2, Color_1.Color.pallete.blueDark),
                        new VmwareMetrics.Metric('cluster', 'memory_active')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 3, Color_1.Color.pallete.orangeDark),
                        new VmwareMetrics.Metric('cluster', 'memory_baloon')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 4, Color_1.Color.pallete.yellowDark),
                        new VmwareMetrics.Metric('cluster', 'memory_swap')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 5, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                    label: 'Cluster'
                },
            ],
        });
        let esxis = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'esxi',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'Server',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'cpu_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: esxis.map(esxi => esxi.item_id),
                    aggregated: true,
                    label: 'ESXi'
                },
            ],
        });
        tabs.push({
            title: 'Server %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: esxis.map(esxi => esxi.item_id),
                    aggregated: true,
                    label: 'ESXi'
                },
            ],
        });
        tabs.push({
            title: 'CPU Ready',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'cpu_ready')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'ESXi',
                        subsystem: 'esxi'
                    }
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('vm', 'network_received')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, false, -1, Color_1.Color.pallete.red),
                        new VmwareMetrics.Metric('vm', 'network_transmitted')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, false, 1, Color_1.Color.pallete.blue)
                    ],
                    uuids: esxis.map(esxi => esxi.item_id),
                    aggregated: true,
                    label: 'ESXi'
                }
            ],
        });
        tabs.push({
            title: 'Power',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('esxi', 'power_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: esxis.map(esxi => esxi.item_id),
                    aggregated: true,
                    label: 'ESXi'
                },
            ],
        });
        const configurationTable = new ConfigurationTable_1.ConfigurationTable(vms.map(vm => vm.item_id), ['Provisioned_Space_GB', 'Used_Space_GB']);
        configurationTable.setItemLabel("VM");
        configurationTable.setAcl(this.request);
        configurationTable.sortByColumn(2, 'desc');
        tabs.push({
            title: 'VM Space',
            content: [await configurationTable.generate()],
        });
        let rspool = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'respool',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        const configurationTableRsp = new ConfigurationTable_1.ConfigurationTable(rspool.map(respool => respool.item_id), ['cpu_reservation', 'cpu_limit', 'cpu_alloc_type', 'cpu_shares', 'cpu_value', 'mem_reservation', 'mem_limit', 'mem_alloc_type', 'mem_shares', 'mem_value']);
        configurationTableRsp.setItemLabel("Resource Pool");
        configurationTableRsp.setAcl(this.request);
        tabs.push({
            title: 'Configuration',
            content: [await configurationTableRsp.generate()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareClustertotalsPage = VmwareClustertotalsPage;
class VmwareMetrics {
}
exports.VmwareMetrics = VmwareMetrics;
VmwareMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(subsystem, metric) {
        super();
        this.metric = metric;
        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.VMWARE, subsystem, metric);
        if (definition) {
            Object.assign(this, definition);
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
    }
    getAggrGraph(type, stacked, order, aggregate) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.aggregate = aggregate ? aggregate : pageResponse_constants_1.METRIC_AGGREGATE.items;
        return this;
    }
};
class VmwareVmtopPage extends VmwarePage {
    constructor() {
        super(...arguments);
        this.title = 'VM TOP';
        this.withItemId = true;
    }
    async getPage() {
        const top = new Top_1.Top();
        await top.setSubsystem('vmware', 'vm');
        await top.setParentSubsystem('vcenter');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.VmwareVmtopPage = VmwareVmtopPage;
class VmwareVcentertopPage extends VmwarePage {
    constructor() {
        super(...arguments);
        this.title = 'VM TOP';
    }
    async getPage() {
        const items = await this.objectItemsRepository.find({
            where: {
                parents: {
                    parent: this.itemId
                },
                subsystem: 'vm',
                hw_type: 'vmware'
            }, relations: {
                parents: true
            }
        });
        const top = new Top_1.Top();
        top.setItemIds(items.map(item => item.item_id));
        top.setSubsystem('vmware', 'vm');
        top.setParentSubsystem('cluster');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.VmwareVcentertopPage = VmwareVcentertopPage;
class VmwareDatastoretopPage extends VmwarePage {
    constructor() {
        super(...arguments);
        this.title = 'Datastore TOP';
    }
    async getPage() {
        let tabs = [];
        tabs.push({
            title: 'TOP',
            content: [
                CustomTop_1.CustomTop.generateRequestBox('vmware', 'datastore', ['read_avg', 'write_avg', 'read', 'write', 'read_latency', 'write_latency', 'disk_used', 'disk_provisioned'], ['vcenter'], 'Datastores TOP')
            ],
        });
        const datastoresIds = (await this.objectItemsRepository.findBy({
            hw_type: hwTypeKey_constants_1.HwTypeKey.VMWARE,
            subsystem: 'datastore'
        })).map(item => item.item_id);
        tabs.push({
            title: 'IO Read',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'read_avg')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true),
                    ],
                    uuids: datastoresIds,
                    aggregated: true,
                    label: 'Datastore',
                    parent: {
                        label: 'vCenter',
                        subsystem: 'vcenter'
                    },
                    sort: {
                        metric: 'read_avg',
                        order: 'max'
                    }
                }
            ],
        });
        tabs.push({
            title: 'IO Write',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'write_avg')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true),
                    ],
                    uuids: datastoresIds,
                    aggregated: true,
                    label: 'Datastore',
                    parent: {
                        label: 'vCenter',
                        subsystem: 'vcenter'
                    },
                    sort: {
                        metric: 'write_avg',
                        order: 'max'
                    }
                }
            ],
        });
        tabs.push({
            title: 'Data Read',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'read')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true),
                    ],
                    uuids: datastoresIds,
                    aggregated: true,
                    label: 'Datastore',
                    parent: {
                        label: 'vCenter',
                        subsystem: 'vcenter'
                    },
                    sort: {
                        metric: 'read',
                        order: 'max'
                    }
                }
            ],
        });
        tabs.push({
            title: 'Data Write',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'write')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true),
                    ],
                    uuids: datastoresIds,
                    aggregated: true,
                    label: 'Datastore',
                    parent: {
                        label: 'vCenter',
                        subsystem: 'vcenter'
                    },
                    sort: {
                        metric: 'write',
                        order: 'max'
                    }
                }
            ],
        });
        tabs.push({
            title: 'Used Space',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'disk_used')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true),
                    ],
                    uuids: datastoresIds,
                    aggregated: true,
                    label: 'Datastore',
                    parent: {
                        label: 'vCenter',
                        subsystem: 'vcenter'
                    },
                    sort: {
                        metric: 'disk_used',
                        order: 'max'
                    }
                }
            ],
        });
        tabs.push({
            title: 'Used Space %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new VmwareMetrics.Metric('datastore', 'disk_percent_used')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items),
                    ],
                    uuids: datastoresIds,
                    aggregated: true,
                    label: 'Datastore',
                    parent: {
                        label: 'vCenter',
                        subsystem: 'vcenter'
                    },
                    sort: {
                        metric: 'disk_percent_used',
                        order: 'max'
                    }
                }
            ],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.VmwareDatastoretopPage = VmwareDatastoretopPage;
class VmwareOverviewPage extends VmwarePage {
    constructor() {
        super(...arguments);
        this.title = 'Overview';
    }
    async getPage() {
        let title = 'Overview';
        let tabs = [];
        const overview = new Overview_1.Overview('vmware');
        let clusters = await this.objectItemsRepository.find({
            where: {
                hw_type: 'vmware',
                subsystem: 'cluster'
            }
        });
        clusters = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, clusters);
        overview.addCategory('Cluster', clusters.map(item => {
            return {
                label: item.label,
                item_id: item.item_id
            };
        }));
        tabs.push({
            title: 'Cluster',
            content: [
                overview.form()
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareOverviewPage = VmwareOverviewPage;
class VmwareHeatmapPage extends VmwarePage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        tabs.push({
            title: 'VM',
            content: Object.values(await new Heatmap_1.Heatmap('vmware', 'vm', true)
                .virtHeatmapBox(this.request))
        });
        let vmTable = new Table_1.Table()
            .addHeader({ value: 'VM' })
            .addHeader({ value: 'Cluster' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true })
            .addHeader({ value: 'Memory', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: 'vmware',
                subsystem: 'vm',
                parents: {
                    _parent: {
                        subsystem: 'cluster'
                    }
                }
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, vms.map(item => item.item_id));
            vms = vms.filter(item => ids.includes(item.item_id));
        }
        for (const vm of vms) {
            const cluster = vm.parents.find(parent => parent._parent.subsystem === 'cluster');
            const cpu = vm.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization_percent');
            const memory = vm.heatmaps.find(heatmap => heatmap.metric === 'memory_proc');
            vmTable.addRow([
                { order: 0, value: vm.label, url: URL_1.URL.generate('vmware', 'vm', vm.item_id) },
                { order: 1, value: cluster._parent.label },
                { order: 2, value: cpu ? cpu.value : null },
                { order: 3, value: memory ? memory.value : null },
            ]);
        }
        vmTable.sortByIndex(2);
        tabs.push({
            title: 'VM table',
            content: [vmTable.generateBox()],
        });
        tabs.push({
            title: 'Server',
            content: Object.values(await new Heatmap_1.Heatmap('vmware', 'esxi', true)
                .virtHeatmapBox(this.request))
        });
        let serverTable = new Table_1.Table()
            .addHeader({ value: 'Server' })
            .addHeader({ value: 'Cluster' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: 'vmware',
                subsystem: 'esxi'
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, servers.map(item => item.item_id));
            servers = servers.filter(item => ids.includes(item.item_id));
        }
        for (const server of servers) {
            const cpu = server.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization_percent');
            const cluster = server.parents.find(parent => parent._parent.subsystem === 'cluster');
            serverTable.addRow([
                { order: 0, value: server.label ? server.label : null, url: server ? URL_1.URL.generate('vmware', 'esxi', server.item_id) : undefined },
                { order: 1, value: cluster ? cluster._parent.label : null },
                { order: 2, value: cpu ? cpu.value : null },
            ]);
        }
        serverTable.sortByIndex(2);
        tabs.push({
            title: 'Server table',
            content: [serverTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareHeatmapPage = VmwareHeatmapPage;
class VmwareConfigurationPage extends VmwarePage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let title = 'Configuration';
        let tabs = [];
        let vcenter = await this.objectItemsRepository.find({
            where: {
                hw_type: 'vmware',
                subsystem: 'vcenter'
            }
        });
        const configurationTableVcenter = new ConfigurationTable_1.ConfigurationTable(vcenter.map(vcenter => vcenter.item_id), ['host', 'vcenter_ghz', 'vcenter_overallCpuUsage_ghz', 'vcenter_memorySize_gb', 'vcenter_overallMemoryUsage_gb', 'fullName_top', 'vcenter_last_update', 'clusters', 'esxis', 'vcenter_vms_count']);
        configurationTableVcenter.setItemLabel("Alias");
        configurationTableVcenter.setAcl(this.request);
        tabs.push({
            title: 'vCenter',
            content: [await configurationTableVcenter.generate()],
        });
        let esxis = await this.objectItemsRepository.find({
            where: {
                hw_type: 'vmware',
                subsystem: 'esxi'
            }
        });
        const configurationTableEsxi = new ConfigurationTable_1.ConfigurationTable(esxis.map(esxi => esxi.item_id), ['host_ghz', 'overallCpuUsage_ghz', 'host_memorySize_gb', 'overallMemoryUsage_gb', 'hw_numCpuCores', 'hw_numCpuThreads', 'sw_fullName', 'esxi_uptime_days', 'hw_vendor', 'hw_model', 'hw_cpuModel'], ['vcenter', 'cluster']);
        configurationTableEsxi.setItemLabel("ESXi");
        configurationTableEsxi.setAcl(this.request);
        tabs.push({
            title: 'ESXi',
            content: [await configurationTableEsxi.generate()],
        });
        const vms = await this.objectItemsRepository.findBy({
            subsystem: 'vm',
            hw_type: 'vmware'
        });
        const vmTable = new ConfigurationTable_1.ConfigurationTable(vms.map(vm => vm.item_id), undefined, ['vcenter', 'cluster']);
        vmTable.setItemLabel("VM");
        vmTable.setAcl(this.request);
        tabs.push({
            title: 'VM',
            content: [await vmTable.generate()],
        });
        let datastore = await this.objectItemsRepository.find({
            where: {
                hw_type: 'vmware',
                subsystem: 'datastore'
            }
        });
        const configurationTableDS = new ConfigurationTable_1.ConfigurationTable(datastore.map(datastore => datastore.item_id), ['ds_used', 'ds_freeSpace', 'ds_provisioned', 'disk_uids'], ['vcenter']);
        configurationTableDS.setItemLabel("datastore");
        configurationTableDS.setAcl(this.request);
        tabs.push({
            title: 'Datastore',
            content: [await configurationTableDS.generate()],
        });
        tabs.push({
            title: 'vMotion TOP',
            content: [await getVmotionTop(this.request)],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.VmwareConfigurationPage = VmwareConfigurationPage;
const getVmotionTop = async (request, vcenter_id) => {
    let vms = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
        relations: {
            children: {
                _child: true
            },
            parents: {
                _parent: true
            }
        },
        where: {
            hw_type: hwTypeKey_constants_1.HwTypeKey.VMWARE,
            subsystem: 'vm',
            children: {
                _child: {
                    subsystem: 'esxi_vm'
                }
            },
            parents: {
                _parent: {
                    subsystem: 'vcenter',
                    item_id: vcenter_id ? vcenter_id : undefined
                }
            }
        }
    });
    vms = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(request, vms);
    const esxiVmIds = [];
    vms.forEach(vm => {
        vm.children.forEach(child => {
            esxiVmIds.push(child.child);
        });
    });
    const now = new Date().getTime() / 1000;
    const totalData = await new MetricsWrapper_1.MetricsWrapper(['start_or_vmotion'], esxiVmIds).getSumByTime(now - (365 * 24 * 60 * 60), now - (24 * 60 * 60));
    const dayData = await new MetricsWrapper_1.MetricsWrapper(['start_or_vmotion'], esxiVmIds).getSumByTime(now - (24 * 60 * 60), now);
    const weekData = await new MetricsWrapper_1.MetricsWrapper(['start_or_vmotion'], esxiVmIds).getSumByTime(now - (7 * 24 * 60 * 60), now);
    const monthData = await new MetricsWrapper_1.MetricsWrapper(['start_or_vmotion'], esxiVmIds).getSumByTime(now - (30 * 24 * 60 * 60), now);
    let table = new Table_1.Table()
        .addHeader({ value: 'vCenter' })
        .addHeader({ value: 'VM' })
        .addHeader({ value: 'vMotion total', number: true })
        .addHeader({ value: 'vMotion last day', number: true })
        .addHeader({ value: 'vMotion last week', number: true })
        .addHeader({ value: 'vMotion last month', number: true });
    for (const vm of vms) {
        const vcenter = vm.parents.find(parent => parent._parent.subsystem === 'vcenter');
        let total = 0;
        let day = 0;
        let week = 0;
        let month = 0;
        vm.children.forEach(child => {
            const id = child.child;
            if (totalData[id] && totalData[id]['start_or_vmotion']) {
                total += totalData[id]['start_or_vmotion'];
            }
            if (dayData[id] && dayData[id]['start_or_vmotion']) {
                day += dayData[id]['start_or_vmotion'];
            }
            if (weekData[id] && weekData[id]['start_or_vmotion']) {
                week += weekData[id]['start_or_vmotion'];
            }
            if (monthData[id] && monthData[id]['start_or_vmotion']) {
                month += monthData[id]['start_or_vmotion'];
            }
        });
        table.addRow([
            { order: 0, value: vcenter ? vcenter._parent.label : '' },
            { order: 1, value: vm.label, url: URL_1.URL.generate('vmware', 'vm', vm.item_id) },
            { order: 2, value: total + day },
            { order: 3, value: day },
            { order: 4, value: week },
            { order: 5, value: month },
        ]);
    }
    table.sortByIndex(2);
    return table.generateBox();
};
//# sourceMappingURL=Vmware.js.map