"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApplicationTasksPage = exports.ApplicationPage = exports.DatabaseHwtypePage = exports.DatabaseTotalPage = exports.DatabasePage = exports.DatabaseTechnologiesFolder = exports.DatabaseRootFolder = exports.ApplicationRootFolder = exports.DatabaseFolder = exports.ApplicationFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
class XormonFolder {
    constructor() {
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
}
class ApplicationFolder extends XormonFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'application';
    }
}
exports.ApplicationFolder = ApplicationFolder;
class DatabaseFolder extends XormonFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'database';
    }
}
exports.DatabaseFolder = DatabaseFolder;
class ApplicationRootFolder extends ApplicationFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Tasks', this.hwType + '/tasks', 'tasks'));
        this.children = menuItems;
        return menuItems;
    }
}
exports.ApplicationRootFolder = ApplicationRootFolder;
class DatabaseRootFolder extends DatabaseFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Total', this.hwType + '/total', 'total'));
        menuItems.push(new menuResponse_dto_1.MenuFolder('Technologies', this.hwType + '/technologies?ids=xxx', this.hwType, 'technologies', 'xxx'));
        this.children = menuItems;
        return menuItems;
    }
}
exports.DatabaseRootFolder = DatabaseRootFolder;
class DatabaseTechnologiesFolder extends DatabaseFolder {
    async getChildren() {
        let menuItems = [];
        const hwTypes = await this.objectItemsRepository.find({
            where: {
                hw_type: 'database',
                subsystem: 'hw_type',
                label: (0, typeorm_1.Not)((0, typeorm_1.In)(['xormon', 'Application', 'Database']))
            }
        });
        hwTypes.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const hwType of hwTypes) {
            menuItems.push(new menuResponse_dto_1.MenuLink(hwType.label, this.hwType + '/hw-type?item_id=' + hwType.item_id, 'hw-type', hwType.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.DatabaseTechnologiesFolder = DatabaseTechnologiesFolder;
class DatabasePage {
    constructor() {
        this.hwType = 'database';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
    async getPath() {
        let root = new DatabaseRootFolder();
        let menuItems = await root.getChildren();
        return menuItems;
    }
}
exports.DatabasePage = DatabasePage;
class DatabaseTotalPage extends DatabasePage {
    async getPage() {
        const item = await this.objectItemsRepository.findOneBy({ hw_type: 'database', subsystem: 'schema' });
        const hwTypes = await this.objectItemsRepository.find({
            where: {
                hw_type: 'database',
                subsystem: 'hw_type'
            }
        });
        let title = 'Database size';
        const metric = {
            metric: 'size',
            unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
            shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
            label: 'Size',
            type: 'area',
            defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
            defaultPrecision: 2,
            stacked: false,
            order: 0
        };
        let tabs = [];
        tabs.push({
            title: 'Size',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                    graphs: [
                        {
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.month,
                            metrics: [metric],
                            uuids: [item.item_id],
                            aggregated: false,
                        },
                        {
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.year,
                            metrics: [metric],
                            uuids: [item.item_id],
                            aggregated: false,
                        },
                    ],
                },
                {
                    type: pageResponse_constants_1.BOX_TYPE.SingleGraphBox,
                    interval: pageResponse_constants_1.GRAPH_INTERVAL.year,
                    metrics: [{
                            ...metric,
                            stacked: true,
                        }],
                    uuids: hwTypes.map(item => item.item_id),
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.DatabaseTotalPage = DatabaseTotalPage;
class DatabaseHwtypePage extends DatabasePage {
    constructor(itemId) {
        super();
        this.itemId = itemId;
    }
    async getPage() {
        const item = await this.objectItemsRepository.findOne({
            where: {
                item_id: this.itemId,
                children: {
                    _child: {
                        subsystem: 'subsystem'
                    }
                }
            },
            relations: {
                children: {
                    _child: true
                }
            }
        });
        let title = 'Technology size';
        const metric = {
            metric: 'size',
            unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
            shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
            label: 'Size',
            type: 'area',
            defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
            defaultPrecision: 2,
            stacked: false,
            order: 0
        };
        let tabs = [];
        tabs.push({
            title: 'Size',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                    graphs: [
                        {
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.month,
                            metrics: [metric],
                            uuids: [item.item_id],
                            aggregated: false,
                        },
                        {
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.year,
                            metrics: [metric],
                            uuids: [item.item_id],
                            aggregated: false,
                        },
                    ],
                },
                {
                    type: pageResponse_constants_1.BOX_TYPE.SingleGraphBox,
                    interval: pageResponse_constants_1.GRAPH_INTERVAL.year,
                    metrics: [{
                            ...metric,
                            stacked: true,
                        }],
                    uuids: item.children.map(child => child.child),
                    aggregated: true,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.DatabaseHwtypePage = DatabaseHwtypePage;
class ApplicationPage {
    constructor() {
        this.hwType = 'application';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
    async getPath() {
        let root = new ApplicationRootFolder();
        let menuItems = await root.getChildren();
        return menuItems;
    }
}
exports.ApplicationPage = ApplicationPage;
class ApplicationTasksPage extends ApplicationPage {
    async getPage() {
        const itemIds = (await this.objectItemsRepository.findBy({
            hw_type: 'application',
            subsystem: 'task'
        })).map(item => item.item_id);
        let title = 'Tasks monitoring';
        const metric = {
            metric: 'duration',
            unit: subsystemMetricsResponse_constants_1.UNIT.second,
            shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.second,
            label: 'Duration',
            type: 'line',
            stacked: false,
            order: 0
        };
        let tabs = [];
        tabs.push({
            title: 'Tasks Duration',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                    graphs: [
                        {
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                            metrics: [metric],
                            uuids: itemIds,
                            aggregated: true,
                            minInterval: 0
                        },
                        {
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.month,
                            metrics: [metric],
                            uuids: itemIds,
                            aggregated: true,
                            minInterval: 0
                        },
                        {
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.year,
                            metrics: [metric],
                            uuids: itemIds,
                            aggregated: true,
                            minInterval: 0
                        },
                    ],
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ApplicationTasksPage = ApplicationTasksPage;
//# sourceMappingURL=Xormon.js.map