import { ChartType, MetricAggregate, IMetricType, UnitPrefix } from '@xormon/common/pageResponse.dto';
import { Unit } from '@xormon/common/subsystemMetricsResponse.dto';
import { IMetrics, ITabsDefinitions } from './class/Storage';
import { HwTypeKey } from '@xormon/common/constants/hwTypeKey.constants';
export * from './class/Cloud';
export * from './class/Container';
export * from './class/Database';
export * from './class/San';
export * from './class/Lan';
export * from './class/Storage';
export * from './class/Virtualization';
export interface IStatusDefinition {
    explicit?: string;
    calculated?: string[];
}
export interface IClassDefinition {
    label: string;
    order: number;
    technology: ITechnology<IDefaultSubsystem>;
    metrics?: IMetrics;
    tabs?: ITabsDefinitions;
}
export interface IDefaultSubsystem {
    label?: string;
    parent?: string;
    aliases?: boolean;
    top?: {
        [group: string]: string[];
    };
    heatmap?: HeatmapItem[];
    excludedTools?: Array<'alerting' | 'reporter' | 'customGroup' | 'exporter' | 'all'>;
    excluded?: {
        all?: string[];
        alerting?: string[];
        customGroup?: string[];
        expoter?: string[];
        reporter?: string[];
    };
    configuration?: IConfiguration;
    metrics?: {
        [metric: string]: IMetric;
    };
    status?: IStatusDefinition;
}
export interface IConfiguration {
    [key: string]: {
        label: string;
        unit?: Unit;
        defaultPrefix?: UnitPrefix;
        decimal?: number;
        shortcut?: string;
        type?: 'number' | 'string';
    };
}
export interface HeatmapItem {
    label: string;
    key: string;
    percent?: string;
    used?: string;
    total?: string;
    free?: string;
}
export type ITechnology<Subsystem> = {
    [key in HwTypeKey]?: ITechnologyHwType<Subsystem>;
};
export interface ITechnologyHwType<Subsystem> {
    order?: number;
    label?: string;
    heatmap?: boolean;
    pages?: string[];
    subsystems?: {
        [subsystem: string]: Subsystem;
    };
    status?: IStatusDefinition;
}
export interface IFullMetric extends IMetric {
    type: ChartType;
    stacked: boolean;
    order?: number;
    aggregate?: MetricAggregate;
}
export type { IMetricType };
export interface IMetric extends IMetricType {
    metric: string;
    label: string;
    longLabel?: string;
    top?: boolean;
    color?: string;
}
