"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectItemToArchItem = exports.addRootParent = exports.getStatusDefinitions = exports.findStatusSubsystemsPerHwtype = exports.findStatusSubsystem = exports.findRootSubsystem = exports.isRootSubsystem = exports.rootSubsystems = exports.metricTypes = exports.MetricTypeKey = void 0;
const pageResponse_constants_1 = require("../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../common/constants/subsystemMetricsResponse.constants");
const hwTypeKey_constants_1 = require("../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../common/constants/class.constants");
var MetricTypeKey;
(function (MetricTypeKey) {
    MetricTypeKey["MEMORY"] = "memory";
    MetricTypeKey["CAPACITY"] = "capacity";
    MetricTypeKey["CAPACITY_VOLUME"] = "capacity_volume";
    MetricTypeKey["CPU_CORE"] = "cpu_core";
    MetricTypeKey["CPU_GHZ"] = "cpu_ghz";
    MetricTypeKey["CPU_PERCENT"] = "cpu_percent";
    MetricTypeKey["IOPS"] = "iops";
    MetricTypeKey["LATENCY"] = "latency";
    MetricTypeKey["THROUGHPUT"] = "throughput";
    MetricTypeKey["THROUGHPUT_TOTAL"] = "throughput_total";
    MetricTypeKey["PACKETS_PER_SECOND"] = "packets_per_second";
    MetricTypeKey["PERCENT"] = "percent";
    MetricTypeKey["BLOCK_SIZE"] = "block_size";
    MetricTypeKey["SESSION"] = "session";
    MetricTypeKey["POWER"] = "power";
    MetricTypeKey["TEMPERATURE"] = "temperature";
})(MetricTypeKey || (exports.MetricTypeKey = MetricTypeKey = {}));
exports.metricTypes = {
    memory: {
        metricType: 'memory',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
        defaultPrecision: 0,
    },
    capacity: {
        metricType: 'capacity',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.tebi,
        defaultPrecision: 1
    },
    capacity_volume: {
        metricType: 'capacity_volume',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
        defaultPrecision: 2
    },
    cpu_core: {
        metricType: 'cpu_core',
        unitLabel: 'CPU core',
        unit: subsystemMetricsResponse_constants_1.UNIT.cpu_core,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.cpu_core,
        defaultPrecision: 1
    },
    cpu_ghz: {
        metricType: 'cpu_ghz',
        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.hertz,
        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.giga,
        defaultPrecision: 1
    },
    cpu_percent: {
        metricType: 'cpu_percent',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        defaultPrecision: 0,
        minY: 100
    },
    iops: {
        metricType: 'iops',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        unitLabel: 'IOPS',
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
        defaultPrecision: 0
    },
    latency: {
        metricType: 'latency',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
        defaultPrecision: 2
    },
    throughput_total: {
        metricType: 'throughput_total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        unitLabel: 'Bytes/sec',
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mega,
        defaultPrecision: 0
    },
    throughput: {
        metricType: 'throughput',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        unitLabel: 'Bytes/sec',
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mega,
        defaultPrecision: 2
    },
    packets_per_second: {
        metricType: 'packets_per_second',
        unit: subsystemMetricsResponse_constants_1.UNIT.packets_per_second,
        unitLabel: 'Packets/sec',
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.packets_per_second,
        defaultPrecision: 0
    },
    percent: {
        metricType: 'percent',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        defaultPrecision: 0,
        minY: 100
    },
    block_size: {
        metricType: 'block_size',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.kibi,
        defaultPrecision: 1
    },
    session: {
        metricType: 'sessions',
        unit: subsystemMetricsResponse_constants_1.UNIT.sessions,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sessions,
        defaultPrecision: 0
    },
    power: {
        metricType: 'power',
        unit: subsystemMetricsResponse_constants_1.UNIT.watt,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.watt,
        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.kilo,
        defaultPrecision: 1,
    },
    temperature: {
        metricType: 'temperature',
        unit: subsystemMetricsResponse_constants_1.UNIT.temperature,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.temperature,
        defaultPrecision: 1,
    },
};
exports.rootSubsystems = {
    [hwTypeKey_constants_1.HwTypeKey.POWER]: 'server',
    [hwTypeKey_constants_1.HwTypeKey.PROXMOX]: 'cluster',
    [hwTypeKey_constants_1.HwTypeKey.NUTANIX]: 'cluster',
    [hwTypeKey_constants_1.HwTypeKey.VMWARE]: 'vcenter',
    [hwTypeKey_constants_1.HwTypeKey.WINDOWS]: 'domain',
    [hwTypeKey_constants_1.HwTypeKey.ORACLEVM]: 'server',
    [hwTypeKey_constants_1.HwTypeKey.LINUX]: 'server',
    [hwTypeKey_constants_1.HwTypeKey.DOCKER]: 'host',
    [hwTypeKey_constants_1.HwTypeKey.KUBERNETES]: 'cluster',
    [hwTypeKey_constants_1.HwTypeKey.OPENSHIFT]: 'cluster',
    [hwTypeKey_constants_1.HwTypeKey.AS400]: 'server',
    [hwTypeKey_constants_1.HwTypeKey.OVIRT]: 'datacenter',
    [hwTypeKey_constants_1.HwTypeKey.SOLARIS]: 'cdom',
};
const isRootSubsystem = (hwType, subsystem) => {
    if (subsystem === 'device') {
        return true;
    }
    return exports.rootSubsystems[hwType] && exports.rootSubsystems[hwType][subsystem];
};
exports.isRootSubsystem = isRootSubsystem;
const findRootSubsystem = (_class, hwType) => {
    if (_class === class_constants_1.ClassKey.STORAGE || _class === class_constants_1.ClassKey.SAN || _class === class_constants_1.ClassKey.LAN) {
        return 'device';
    }
    else {
        if (Object.values(hwTypeKey_constants_1.HwTypeKey).includes(hwType)) {
            return exports.rootSubsystems[hwType];
        }
        else {
            return null;
        }
    }
};
exports.findRootSubsystem = findRootSubsystem;
const findStatusSubsystem = (_class, hwType) => {
    const found = (0, exports.getStatusDefinitions)(_class, hwType);
    return found ? found[hwType].explicit : null;
};
exports.findStatusSubsystem = findStatusSubsystem;
const findStatusSubsystemsPerHwtype = (_class, hwType) => {
    let statusDefinitions = (0, exports.getStatusDefinitions)(_class, hwType);
    let statusSubsystems = {};
    for (const hwtype of Object.keys(statusDefinitions)) {
        if (statusDefinitions[hwtype].explicit) {
            statusSubsystems[hwtype] = statusDefinitions[hwtype].calculated && statusDefinitions[hwtype].calculated.length ? [statusDefinitions[hwtype].explicit, ...statusDefinitions[hwtype].calculated] : [statusDefinitions[hwtype].explicit];
        }
    }
    return statusSubsystems;
};
exports.findStatusSubsystemsPerHwtype = findStatusSubsystemsPerHwtype;
const getStatusDefinitions = (incClass, incHwType) => {
    let statusDefinitions = {};
    let _ClassesDef = getClassDefinitions();
    let classArray = Object.keys(_ClassesDef);
    if (incClass) {
        classArray = [incClass.charAt(0).toUpperCase() + incClass.slice(1)];
    }
    for (const _class of classArray) {
        let hwTypesDef = _ClassesDef[_class]['technology'];
        if (incHwType) {
            if (!_ClassesDef[_class]['technology'][incHwType]) {
                continue;
            }
            hwTypesDef = {};
            hwTypesDef[incHwType] = _ClassesDef[_class]['technology'][incHwType];
        }
        for (const hwType in hwTypesDef) {
            let definition = {
                explicit: undefined,
                calculated: undefined,
            };
            if (hwTypesDef[hwType].status) {
                definition = hwTypesDef[hwType].status;
            }
            if (!definition.explicit) {
                if (_class == 'Storage' || _class == 'San' || _class == 'Lan') {
                    definition.explicit = 'device';
                }
                else {
                    definition.explicit = exports.rootSubsystems[hwType] ? exports.rootSubsystems[hwType] : null;
                }
            }
            statusDefinitions[hwType] = definition;
        }
        if (incHwType) {
            break;
        }
    }
    return statusDefinitions;
};
exports.getStatusDefinitions = getStatusDefinitions;
const getClassDefinitions = () => {
    const Cloud = require('./class/Cloud');
    const Container = require('./class/Container');
    const Database = require('./class/Database');
    const San = require('./class/San');
    const Lan = require('./class/Lan');
    const Storage = require('./class/Storage');
    const Virtualization = require('./class/Virtualization');
    return { ...Cloud, ...Container, ...Database, ...San, ...Lan, ...Storage, ...Virtualization };
};
const addRootParent = (items) => {
    for (const item of items) {
        if (item.parents && item.parents.find(parent => parent.subsystem === 'device')) {
            item.root_parent = item.parents.find(parent => parent.subsystem === 'device');
        }
        else if (item.parents && exports.rootSubsystems[item.hw_type] && item.subsystem !== exports.rootSubsystems[item.hw_type]) {
            const rootParent = item.parents.find(parent => parent.subsystem === exports.rootSubsystems[item.hw_type]);
            if (rootParent) {
                item.root_parent = rootParent;
            }
        }
    }
    return items;
};
exports.addRootParent = addRootParent;
const ObjectItemToArchItem = (items) => {
    return items.map(item => {
        return {
            item_id: item.item_id,
            label: item.label,
            subsystem: item.subsystem,
            hw_type: item.hw_type,
            class: item.hw_type_entity ? item.hw_type_entity.class : undefined,
            parents: hasParents(item.parents)
                ? (0, exports.ObjectItemToArchItem)(item.parents.map(parent => parent._parent))
                : undefined
        };
    });
};
exports.ObjectItemToArchItem = ObjectItemToArchItem;
const hasParents = (relations) => {
    return relations && relations.length && relations[0]._parent !== null && relations[0]._parent !== undefined;
};
//# sourceMappingURL=Definitions.js.map