import * as _Class from './Classes';
import { IMetric } from './Classes';
import { ClassKey } from "@const/class.constants";
export interface MetricSimpleItem {
    [key: string]: number;
}
export interface MetricItem {
    [key: string]: {
        [key: string]: number;
    };
}
export interface ParentType {
    label: string;
    subsystem: string;
    hasPage: boolean;
}
interface MetricTimeseriesItem {
    [key: string]: {
        [key: string]: number;
    };
}
export declare class Metric {
    static readonly INTERVAL_1_M = 1;
    static readonly INTERVAL_5_M = 5;
    static readonly INTERVAL_1_H = 60;
    static readonly INTERVAL_6_H = 360;
    static readonly INTERVAL_1_D = 1440;
    static readonly SCHEMA = "metrics.";
    static readonly LOCF: number;
    private uuids;
    private itemGroups;
    private metric;
    private hw_type;
    private subsystem;
    private class;
    private initialized;
    private exists;
    private entityManager;
    constructor(metric: string, items?: string[]);
    isMultiStorage(): boolean;
    isStorage(): boolean;
    getHwType(): string;
    getSubsystem(): string;
    getClass(): string;
    init(): Promise<boolean>;
    addItem(item_id: string): void;
    findHwTypeAndSubsystem(): Promise<boolean>;
    getDataFromTable(start: number, end: number): Promise<any>;
    getDataFromBucket(start: number, end: number): Promise<any>;
    getGapFill(func: string, interval: number, start: string, end: string): Promise<any>;
    getMetricTimeseriesItem(func: string, interval: number, start: string, end: string, locale?: boolean): Promise<MetricTimeseriesItem>;
    getWithOperator(operator: string, value: number, start: number, end: number): Promise<MetricSimpleItem>;
    getAverage(interval: number): Promise<MetricSimpleItem>;
    getAverageFromTable(interval: number): Promise<MetricSimpleItem>;
    getAverageAndMaxFromTable(interval: number): Promise<MetricItem>;
    getLastValue(): Promise<MetricSimpleItem>;
    getLastValueFromTable(): Promise<MetricSimpleItem>;
    static getInterval(start: number, end: number): number;
    toLocaleDateTime(datetime: number): string;
    static getLocf(metric: any, hwtype?: any, subsystem?: any): number;
    static getMetrics(hwType: string, subsystem: string, exclude?: boolean): Promise<_Class.IMetric[]>;
    static getHeatmapLabels(_class: ClassKey, hwType: string, subsystem: string): {
        [key: string]: string;
    };
    static getMetric(_class: ClassKey, hwType: string, subsystem: string, metric: string): Partial<IMetric>;
    static getMetricFindClass(hwType: string, subsystem: string, metric: string): Promise<Partial<IMetric>>;
    static getParent(_class: ClassKey, hwType: string, subsystem: string): ParentType;
    static getParentInfo(_class: ClassKey, hwType: string, parent: string): ParentType;
}
export {};
