"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricsTransform = void 0;
const dataSource_1 = require("../../config/dataSource");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
class MetricsTransform {
    constructor(itemIdsToTransform) {
        this.itemIdsToTransform = [];
        this.itemIdsTransformed = [];
        this.mappedItems = {};
        this.itemIdsToTransform = itemIdsToTransform;
    }
    getIdsToTransform() {
        return this.itemIdsToTransform;
    }
    getIdsTransformed() {
        return this.itemIdsTransformed;
    }
    static async check(hwType, subsystem) {
        if (MetricsTransform[hwType] && MetricsTransform[hwType][subsystem]) {
            return true;
        }
        return false;
    }
    static async checkId(itemId) {
        const itemToCheck = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findOneBy({ item_id: itemId });
        if (itemToCheck &&
            this.mapping[itemToCheck.hw_type] &&
            this.mapping[itemToCheck.hw_type][itemToCheck.subsystem] &&
            this.check(itemToCheck.hw_type, itemToCheck.subsystem)) {
            return true;
        }
        return false;
    }
    async transformIds() {
        const items = await (0, dataSource_1.getDataSource)()
            .getRepository(objectItem_entity_1.ObjectItem)
            .findBy({ item_id: (0, typeorm_1.In)(this.itemIdsToTransform) });
        let finalIds = [];
        for (const item of items) {
            if (MetricsTransform.mapping[item.hw_type][item.subsystem]) {
                if (!this.mappedItems[item.hw_type]) {
                    this.mappedItems[item.hw_type] = {
                        [item.subsystem]: [item.item_id],
                    };
                }
                else if (!this.mappedItems[item.hw_type][item.subsystem]) {
                    this.mappedItems[item.hw_type][item.subsystem] = [item.item_id];
                }
                else {
                    this.mappedItems[item.hw_type][item.subsystem].push(item.item_id);
                }
            }
        }
        for (const hwType in this.mappedItems) {
            for (const subsystem in this.mappedItems[hwType]) {
                const transfomedItems = await (0, dataSource_1.getDataSource)()
                    .getRepository(objectItem_entity_1.ObjectItem)
                    .find({
                    relations: {
                        parents: {
                            _parent: true,
                        },
                    },
                    where: {
                        parents: {
                            parent: (0, typeorm_1.In)(this.mappedItems[hwType][subsystem]),
                        },
                        subsystem: MetricsTransform.mapping[hwType][subsystem],
                        hw_type: hwType,
                    },
                });
                finalIds = finalIds.concat(transfomedItems.map((item) => item.item_id));
            }
        }
        this.itemIdsTransformed = finalIds;
    }
    async transformData(items, metadata, interval) {
        let fabricItems = [];
        for (const hwType in this.mappedItems) {
            for (const subsystem in this.mappedItems[hwType]) {
                if (MetricsTransform[hwType] && MetricsTransform[hwType][subsystem]) {
                    fabricItems = fabricItems.concat(await MetricsTransform[hwType][subsystem](items, metadata, interval));
                }
            }
        }
        return fabricItems;
    }
    static transformFabric(devices, items, metadata, interval) {
        let itemMap = {};
        let fabricLabels = {};
        for (const device of devices) {
            const parent = device.parents ? device.parents[0].parent : undefined;
            if (parent) {
                itemMap[device.item_id] = parent;
                fabricLabels[parent] = device.parents[0]._parent.label;
            }
        }
        let fabricItems = [];
        for (const itemId in items) {
            if (!itemMap[itemId]) {
                continue;
            }
            const fabricItem = fabricItems.find((fabricItem) => fabricItem.metadata.uuid === itemMap[itemId]);
            if (fabricItem) {
                for (const i in items[itemId].timeseries) {
                    fabricItem.timeseries[i] += items[itemId].timeseries[i];
                }
            }
            else {
                fabricItems.push({
                    metadata: {
                        ...metadata[itemId],
                        uuid: itemMap[itemId],
                        name: fabricLabels[itemMap[itemId]],
                        interval: interval,
                    },
                    timeseries: items[itemId].timeseries,
                });
            }
        }
        for (const fabricItem of fabricItems) {
            const filtered = fabricItem.timeseries.filter((e) => {
                return e !== null;
            });
            const sum = fabricItem.timeseries.reduce((a, b) => a + b, 0);
            fabricItem.metadata['start'] = fabricItem.metadata['start'] / 1000;
            fabricItem.metadata['end'] = fabricItem.metadata['end'] / 1000;
            fabricItem.metadata['min'] = Math.min(...filtered);
            fabricItem.metadata['max'] = Math.max(...filtered);
            fabricItem.metadata['avg'] = +Number(sum / filtered.length).toPrecision(5) || 0;
        }
        return fabricItems;
    }
}
exports.MetricsTransform = MetricsTransform;
MetricsTransform.mapping = {
    sanbrcd: {
        fabric: 'device',
    },
    sannav: {
        fabric: 'device',
    },
    sancisco: {
        fabric: 'sanport',
    },
};
MetricsTransform.sanbrcd = class {
    static async fabric(items, metadata, interval) {
        const devices = await (0, dataSource_1.getDataSource)()
            .getRepository(objectItem_entity_1.ObjectItem)
            .find({
            relations: {
                parents: {
                    _parent: true,
                },
            },
            where: {
                subsystem: 'device',
                parents: {
                    _parent: {
                        hw_type: 'sanbrcd',
                        subsystem: 'fabric',
                    },
                },
                item_id: (0, typeorm_1.In)(Object.keys(items)),
            },
        });
        return MetricsTransform.transformFabric(devices, items, metadata, interval);
    }
};
MetricsTransform.sancisco = class {
    static async fabric(items, metadata, interval) {
        const ports = await (0, dataSource_1.getDataSource)()
            .getRepository(objectItem_entity_1.ObjectItem)
            .find({
            relations: {
                parents: {
                    _parent: true,
                },
            },
            where: {
                subsystem: 'sanport',
                parents: {
                    _parent: {
                        hw_type: 'sancisco',
                        subsystem: 'fabric',
                    },
                },
                item_id: (0, typeorm_1.In)(Object.keys(items)),
            },
        });
        return MetricsTransform.transformFabric(ports, items, metadata, interval);
    }
};
MetricsTransform.sannav = class {
    static async fabric(items, metadata, interval) {
        const devices = await (0, dataSource_1.getDataSource)()
            .getRepository(objectItem_entity_1.ObjectItem)
            .find({
            relations: {
                parents: {
                    _parent: true,
                },
            },
            where: {
                subsystem: 'device',
                parents: {
                    _parent: {
                        hw_type: 'sannav',
                        subsystem: 'fabric',
                    },
                },
                item_id: (0, typeorm_1.In)(Object.keys(items)),
            },
        });
        return MetricsTransform.transformFabric(devices, items, metadata, interval);
    }
};
//# sourceMappingURL=MetricsTransform.js.map