import * as _Class from './Classes';
import { IConfiguration, IMetric, ITechnology } from './Classes';
import { ClassKey } from "@const/class.constants";
import { Bucket } from '../menu/menu/Top';
interface IMetricItemTimeseriesItem {
    [time: string]: number;
}
interface IMetricItemMetric {
    [metric: string]: IMetricItemTimeseriesItem;
}
interface IMetricItemSingleMetric {
    [metric: string]: number;
}
interface IMetricItemAvgMaxMetric {
    [metric: string]: {
        avg: number;
        max: number;
    };
}
export interface IMetricItem {
    [item_id: string]: IMetricItemMetric;
}
interface ISingleMetricItem {
    [item_id: string]: IMetricItemTimeseriesItem;
}
export interface ILastMetricItem {
    [item_id: string]: IMetricItemSingleMetric;
}
export interface IAvgMaxMetricItem {
    [item_id: string]: IMetricItemAvgMaxMetric;
}
export interface ISumMetricItem {
    [item_id: string]: IMetricItemSingleMetric;
}
interface ParentType {
    label: string;
    subsystem: string;
    hasPage: boolean;
}
export declare class MetricsWrapper {
    private uuids;
    private structure;
    private metrics;
    private class;
    private initialized;
    private exists;
    private entityManager;
    private excluded;
    private debug;
    constructor(metrics: string[], items?: string[]);
    setDebug(): void;
    debugLog(text: string): void;
    isStorage(): boolean;
    getClass(): string;
    getHwTypes(): string[];
    init(): Promise<boolean>;
    addItem(item_id: string): void;
    findItemsProps(): Promise<boolean>;
    getDataFromTable(start: number, end: number, separately?: boolean, heartbeat?: boolean, overrideInterval?: 1 | 5 | 60 | 360 | 1440): Promise<IMetricItem | ISingleMetricItem | {}>;
    getLastHourValueFromTable(): Promise<ILastMetricItem | {}>;
    getAvgAndMax(interval: number, bucket?: Bucket): Promise<IAvgMaxMetricItem | {}>;
    getAvgAndMaxByTime(start: number, end: number): Promise<IAvgMaxMetricItem | {}>;
    getSumByTime(start: number, end: number): Promise<IMetricItemSingleMetric | {}>;
    getLastDailyValuesFromTable(): Promise<ILastMetricItem | {}>;
    getLastValuesFromTable(): Promise<ILastMetricItem | {}>;
    getLastTimestampFromTable(chosenAggregate?: '5m' | '1h' | '6h' | '1d'): Promise<{}>;
    static getMin5MinutesIntervalFromDefinitions(): string[];
    static getMin5MinutesInterval(): Promise<string[]>;
    static getTable(hwType: string, subsystem: string, start: number, end: number): Promise<string>;
    static getMetrics(hwType: string | string[], subsystem: string, exclude?: 'alerting' | 'customGroup' | 'exporter' | 'reporter'): Promise<_Class.IMetric[]>;
    static hasMetrics(classKey: ClassKey, hwType: string, subsystem: string): boolean;
    static getHeatmapLabels(_class: ClassKey, hwType: string, subsystem: string): {
        [key: string]: string;
    };
    static getMetricsFromPages(_class: ClassKey, hwType: string, subsystem: string): IMetric[];
    static getMetric(_class: ClassKey, hwType: string, subsystem: string, metric: string): Partial<IMetric>;
    static findClassKey(hwType: string): Promise<ClassKey>;
    static getMetricFindClass(hwType: string, subsystem: string, metric: string): Promise<Partial<IMetric>>;
    static getClassDefinition(_class: ClassKey): ITechnology<_Class.IDefaultSubsystem>;
    static getHwTypeDefinition(_class: ClassKey, hwType: string): _Class.ITechnologyHwType<_Class.IDefaultSubsystem>;
    static getSubsystemDefinition(_class: ClassKey, hwType: string, subsystem: string): _Class.IDefaultSubsystem;
    static findTabLabelByMetric(_class: ClassKey, hwType: string, subsystem: string, metric: string): string;
    static getParent(_class: ClassKey, hwType: string, subsystem: string): ParentType;
    static getParentInfo(_class: ClassKey, hwType: string, parent: string): ParentType;
    static getConfiguration(_class: ClassKey, hwType: string, subsystem: string): IConfiguration;
    static getConfigurationFindClass(hwType: string, subsystem: string): Promise<IConfiguration>;
    static getTopMetrics(hwType: string, subsystem: string): Promise<string[]>;
    static getMetricsFromEntity(hwType: string, subsystem: string): string[];
    static skipOthers(_class: string, hwType: string, subsystem: string, metric: string): boolean;
    static handleLongLabel(metrics: IMetric[]): IMetric[];
    static findTopMetric(top: {
        [group: string]: string[];
    }, metric: string): boolean;
}
export {};
