import { HtmlTextBox, Metric, PredictionGraphBox, TextBox } from "@xormon/common/pageResponse.dto";
import { ObjectItem } from "@xormon/entity/objectItem.entity";
export type command = 'predict_cpu' | 'predict_capacity' | 'predict_vm';
export type frequency = '5m' | '1h' | '6h' | '1d';
export type action = 'load_prediction' | 'load_times_to_fill' | 'add_requests_and_calculate_all' | 'calculate_now';
interface PredictionItem {
    itemId: string;
    hwType: string;
    subsystem: string;
    metric: string;
    max?: number;
}
interface AnalyzerRequest {
    command: command;
    frequency: frequency;
    schema: string;
    table: string;
    column: string;
    id: string;
    max: number;
}
export interface AnalyzerResponse {
    [model: string]: {
        name: string;
        start_time: number;
        prediction: number[];
        description?: string;
        days_to_100_percent_fill?: number;
        days_to_90_percent_fill?: number;
        days_to_70_percent_fill?: number;
        slope?: number;
        std?: number;
        max_dif?: number;
        min_dif?: number;
        display?: boolean;
    };
}
export interface AnalyzerPredictCapacityResponse {
    [itemId: string]: {
        time_to_less: string;
        time_to_full: string;
    };
}
interface PredictionBoxProperties {
    item_id: string;
    graphLabel?: string;
    metric: Metric;
    other_metrics: Metric[];
    command: command;
    frequency: frequency;
    interval: 'year' | 'month';
    max: number;
}
export declare class Prediction {
    private static ANALYZER_ENDPOINT;
    static TO_SKIP: string[];
    static predict(item: PredictionItem | PredictionItem[], command: command, frequency?: frequency, action?: action): Promise<AnalyzerResponse>;
    static getCapacityTableData(items: ObjectItem[], metric: string, maxMetric: string): Promise<AnalyzerPredictCapacityResponse>;
    static reloadPoolCapacityTable(): Promise<void>;
    static reloadCapacityTableItems(items: ObjectItem[], metric: string, maxMetric: string): Promise<AnalyzerPredictCapacityResponse>;
    static getCapacityTableRequest(items: ObjectItem[], metric: string, maxMetric: string, reload?: boolean): Promise<AnalyzerPredictCapacityResponse>;
    static isAnalyzerActive(): Promise<boolean>;
    static getInterval(command: command): frequency;
    static requestData<ResponseType>(data: AnalyzerRequest | AnalyzerRequest[], action?: action): Promise<ResponseType>;
    static processRequestData(data: AnalyzerRequest | AnalyzerRequest[], action: action): Promise<string>;
    static parseJSON<Type>(json: string): Type;
    static generatePredictionBox(data: PredictionBoxProperties): Promise<PredictionGraphBox | HtmlTextBox | TextBox>;
    static filterTimeseries(timeseries: number[]): number[];
    static showPrediction(interval: 'year' | 'month', itemId: string): Promise<boolean>;
    static getCapacityPredictionTypes(): string[];
    static getCpuPredictionTypes(): string[];
    static isDefaultDisplayed(type: string, extended?: boolean): boolean;
    static analyzerTextBox: {
        'not-active': HtmlTextBox;
        'no-data': TextBox;
    };
}
export {};
