"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Container = void 0;
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const Definitions_1 = require("../Definitions");
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
class Container {
}
exports.Container = Container;
Container.order = 5;
Container.technology = {
    docker: {
        label: 'Docker',
        order: 2,
        subsystems: {
            host: {
                label: 'Host',
                excludedTools: [
                    'all'
                ],
            },
            container: {
                label: 'Container',
                parent: 'host',
                aliases: true,
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_usage',
                        percent: 'cpu_usage'
                    }],
                metrics: {
                    cpu_usage: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_usage',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                    },
                    cpu_cores_usage: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_cores_usage',
                        label: 'CPU',
                    },
                    memory_used: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 1,
                        metric: 'memory_used',
                        label: 'Mem used',
                    },
                    memory_available: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 1,
                        metric: 'memory_available',
                        label: 'Mem available',
                    },
                    memory_free: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 1,
                        metric: 'memory_free',
                        label: 'Mem free',
                    },
                    read_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'read_bytes',
                        label: 'Read',
                    },
                    write_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'write_bytes',
                        label: 'Write',
                    },
                    read_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'read_io',
                        label: 'Read IO',
                    },
                    write_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'write_io',
                        label: 'Write IO',
                    },
                    rx_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'rx_bytes',
                        label: 'In',
                    },
                    tx_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'tx_bytes',
                        label: 'Out',
                    },
                    size_rw: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'size_rw',
                        label: 'Size RW',
                    },
                    size_root_fs: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'size_root_fs',
                        label: 'Size',
                    },
                },
            },
            volume: {
                label: 'Volume',
                parent: 'host',
                metrics: {
                    size: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'size',
                        label: 'Size',
                    },
                },
            },
        },
    },
    kubernetes: {
        label: 'Kubernetes',
        order: 0,
        subsystems: {
            cluster: {
                label: "Cluster",
                excludedTools: [
                    'all'
                ],
            },
            node: {
                label: 'Node',
                configuration: {
                    osImage: {
                        label: 'OS Image'
                    },
                    architecture: {
                        label: 'Architecture'
                    },
                    kubeletVersion: {
                        label: 'Kubelet version'
                    },
                    containerRuntimeVersion: {
                        label: 'Container Runtime Version'
                    },
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_usage',
                        percent: 'cpu_usage'
                    }, {
                        label: 'Memory',
                        key: 'memory_usage',
                        percent: 'memory_usage'
                    }],
                metrics: {
                    cpu_usage: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_usage',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                    },
                    cpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu',
                        label: 'CPU',
                    },
                    cpu_allocatable: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_allocatable',
                        label: 'CPU Allocatable',
                    },
                    cpu_capacity: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_capacity',
                        label: 'CPU Capacity',
                    },
                    memory_usage: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'memory_usage',
                        label: 'Memory Usage',
                    },
                    memory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory',
                        label: 'Memory Used',
                    },
                    memory_allocatable: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_allocatable',
                        label: 'Memory Allocatable',
                    },
                    memory_capacity: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_capacity',
                        label: 'Memory Capacity',
                    },
                    pods: {
                        metric: 'pods',
                        label: 'Pods Used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    pods_allocatable: {
                        metric: 'pods_allocatable',
                        label: 'Pods Allocatable',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    pods_capacity: {
                        metric: 'pods_capacity',
                        label: 'Pods Capacity',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    container_fs_writes: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_writes',
                        label: 'Write IO',
                    },
                    container_fs_reads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_reads',
                        label: 'Read IO',
                    },
                    container_fs_writes_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_writes_bytes',
                        label: 'Data Write',
                    },
                    container_fs_reads_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_reads_bytes',
                        label: 'Data Read',
                    },
                    container_fs_write_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'container_fs_write_latency',
                        label: 'Write Latency',
                    },
                    container_fs_read_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'container_fs_read_latency',
                        label: 'Read Latency',
                    },
                    container_network_receive_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_receive_bytes',
                        label: 'In',
                    },
                    container_network_transmit_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_transmit_bytes',
                        label: 'Out',
                    },
                    container_fs_usage_bytes: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'container_fs_usage_bytes',
                        label: 'Used',
                    },
                }
            },
            namespace: {
                label: 'Namespace',
                metrics: {
                    cpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu',
                        label: 'CPU',
                    },
                    memory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory',
                        label: 'Memory Used',
                    },
                    container_fs_writes: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_writes',
                        label: 'Write IO',
                    },
                    container_fs_reads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_reads',
                        label: 'Read IO',
                    },
                    container_fs_writes_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_writes_bytes',
                        label: 'Data Write',
                    },
                    container_fs_reads_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_reads_bytes',
                        label: 'Data Read',
                    },
                    container_network_receive_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_receive_bytes',
                        label: 'In',
                    },
                    container_network_transmit_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_transmit_bytes',
                        label: 'Out',
                    },
                }
            },
            pod: {
                label: 'Pod',
                aliases: true,
                parent: 'namespace',
                top: {
                    'CPU': ['cpu'],
                    'Memory': ['memory']
                },
                configuration: {
                    podIP: {
                        label: 'Pod IP Adress'
                    },
                    status: {
                        label: 'Status'
                    }
                },
                metrics: {
                    cpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu',
                        label: 'CPU',
                    },
                    cpu_request: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_request',
                        label: 'CPU Request',
                    },
                    cpu_limit: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_limit',
                        label: 'CPU Limit',
                    },
                    memory: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mebi,
                        defaultPrecision: 0,
                        metric: 'memory',
                        label: 'Memory Used',
                    },
                    memory_request: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mebi,
                        defaultPrecision: 0,
                        metric: 'memory_request',
                        label: 'Memory Allocatable',
                    },
                    memory_limit: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mebi,
                        defaultPrecision: 0,
                        metric: 'memory_limit',
                        label: 'Memory Limit',
                    },
                    container_processes: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'container_processes',
                        label: 'Processes'
                    },
                    container_fs_writes: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_writes',
                        label: 'Write IO',
                    },
                    container_fs_reads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_reads',
                        label: 'Read IO',
                    },
                    container_fs_writes_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_writes_bytes',
                        label: 'Data Write',
                    },
                    container_fs_reads_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_reads_bytes',
                        label: 'Data Read',
                    },
                    container_network_receive_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_receive_bytes',
                        label: 'In',
                    },
                    container_network_transmit_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_transmit_bytes',
                        label: 'Out',
                    },
                }
            },
            container: {
                label: 'Container',
                aliases: true,
                metrics: {
                    cpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu',
                        label: 'CPU',
                    },
                    cpu_request: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_request',
                        label: 'CPU Request',
                    },
                    cpu_limit: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_limit',
                        label: 'CPU Limit',
                    },
                    memory: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mebi,
                        defaultPrecision: 0,
                        metric: 'memory',
                        label: 'Memory Used',
                    },
                    memory_request: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mebi,
                        defaultPrecision: 0,
                        metric: 'memory_request',
                        label: 'Memory Allocatable',
                    },
                    memory_limit: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mebi,
                        defaultPrecision: 0,
                        metric: 'memory_limit',
                        label: 'Memory Limit',
                    },
                    container_fs_writes: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_writes',
                        label: 'Write IO',
                    },
                    container_fs_reads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_reads',
                        label: 'Read IO',
                    },
                    container_fs_writes_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_writes_bytes',
                        label: 'Data Write',
                    },
                    container_fs_reads_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_reads_bytes',
                        label: 'Data Read',
                    },
                    container_fs_write_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'container_fs_write_latency',
                        label: 'Write Latency',
                    },
                    container_fs_read_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'container_fs_read_latency',
                        label: 'Read Latency',
                    },
                }
            },
        }
    },
    openshift: {
        label: 'Red Hat OpenShift',
        order: 1,
        subsystems: {
            cluster: {
                label: "Cluster",
                excludedTools: [
                    'all'
                ],
            },
            node: {
                label: 'Node',
                configuration: {
                    osImage: {
                        label: 'OS Image'
                    },
                    architecture: {
                        label: 'Architecture'
                    },
                    kubeletVersion: {
                        label: 'Kubelet version'
                    },
                    containerRuntimeVersion: {
                        label: 'Container Runtime Version'
                    },
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_usage',
                        percent: 'cpu_usage'
                    }, {
                        label: 'Memory',
                        key: 'memory_usage',
                        percent: 'memory_usage'
                    }],
                metrics: {
                    cpu_usage: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_usage',
                        label: 'CPU %',
                        longLabel: 'CPU %',
                    },
                    cpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu',
                        label: 'CPU',
                    },
                    cpu_allocatable: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_allocatable',
                        label: 'CPU Allocatable',
                    },
                    cpu_capacity: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_capacity',
                        label: 'CPU Capacity',
                    },
                    memory_usage: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'memory_usage',
                        label: 'Memory Usage',
                    },
                    memory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory',
                        label: 'Memory Used',
                    },
                    memory_allocatable: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_allocatable',
                        label: 'Memory Allocatable',
                    },
                    memory_capacity: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_capacity',
                        label: 'Memory Capacity',
                    },
                    pods: {
                        metric: 'pods',
                        label: 'Pods Used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    pods_allocatable: {
                        metric: 'pods_allocatable',
                        label: 'Pods Allocatable',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    podscapacity: {
                        metric: 'pods_capacity',
                        label: 'Pods Capacity',
                        unit: subsystemMetricsResponse_constants_1.UNIT.quantity,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.quantity,
                    },
                    container_fs_writes: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_writes',
                        label: 'Write IO',
                    },
                    container_fs_reads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_reads',
                        label: 'Read IO',
                    },
                    container_fs_writes_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_writes_bytes',
                        label: 'Data Write',
                    },
                    container_fs_reads_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_reads_bytes',
                        label: 'Data Read',
                    },
                    container_fs_write_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'container_fs_write_latency',
                        label: 'Write Latency',
                    },
                    container_fs_read_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'container_fs_read_latency',
                        label: 'Read Latency',
                    },
                    container_network_receive_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_receive_bytes',
                        label: 'In',
                    },
                    container_network_transmit_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_transmit_bytes',
                        label: 'Out',
                    },
                    container_fs_usage_bytes: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'container_fs_usage_bytes',
                        label: 'Used',
                    },
                }
            },
            project: {
                label: 'Project',
                metrics: {
                    cpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu',
                        label: 'CPU',
                    },
                    memory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory',
                        label: 'Memory Used',
                    },
                    container_fs_writes: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_writes',
                        label: 'Write IO',
                    },
                    container_fs_reads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_reads',
                        label: 'Read IO',
                    },
                    container_fs_writes_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_writes_bytes',
                        label: 'Data Write',
                    },
                    container_fs_reads_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_reads_bytes',
                        label: 'Data Read',
                    },
                    container_network_receive_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_receive_bytes',
                        label: 'In',
                    },
                    container_network_transmit_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_transmit_bytes',
                        label: 'Out',
                    },
                }
            },
            pod: {
                label: 'Pod',
                aliases: true,
                parent: 'project',
                top: {
                    'CPU': ['cpu'],
                    'Memory': ['memory']
                },
                configuration: {
                    podIP: {
                        label: 'Pod IP Adress'
                    },
                    status: {
                        label: 'Status'
                    }
                },
                metrics: {
                    cpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu',
                        label: 'CPU',
                    },
                    cpu_request: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_request',
                        label: 'CPU Request',
                    },
                    cpu_limit: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_limit',
                        label: 'CPU Limit',
                    },
                    memory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory',
                        label: 'Memory Used',
                    },
                    memory_request: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_request',
                        label: 'Memory Allocatable',
                    },
                    memory_limit: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_limit',
                        label: 'Memory Limit',
                    },
                    container_processes: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'container_processes',
                        label: 'Processes'
                    },
                    container_fs_writes: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_writes',
                        label: 'Write IO',
                    },
                    container_fs_reads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_reads',
                        label: 'Read IO',
                    },
                    container_fs_writes_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_writes_bytes',
                        label: 'Data Write',
                    },
                    container_fs_reads_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_reads_bytes',
                        label: 'Data Read',
                    },
                    container_network_receive_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_receive_bytes',
                        label: 'In',
                    },
                    container_network_transmit_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_network_transmit_bytes',
                        label: 'Out',
                    },
                }
            },
            container: {
                label: 'Container',
                aliases: true,
                metrics: {
                    cpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu',
                        label: 'CPU',
                    },
                    cpu_request: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_request',
                        label: 'CPU Request',
                    },
                    cpu_limit: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_limit',
                        label: 'CPU Limit',
                    },
                    memory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory',
                        label: 'Memory Used',
                    },
                    memory_request: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_request',
                        label: 'Memory Allocatable',
                    },
                    memory_limit: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_limit',
                        label: 'Memory Limit',
                    },
                    container_fs_writes: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_writes',
                        label: 'Write IO',
                    },
                    container_fs_reads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'container_fs_reads',
                        label: 'Read IO',
                    },
                    container_fs_writes_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_writes_bytes',
                        label: 'Data Write',
                    },
                    container_fs_reads_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'container_fs_reads_bytes',
                        label: 'Data Read',
                    },
                }
            },
        }
    },
};
//# sourceMappingURL=Container.js.map