"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiKey = void 0;
const common_1 = require("@nestjs/common");
const dataSource_1 = require("../../config/dataSource");
const user_entity_1 = require("../../entity/user.entity");
const userApiKey_entity_1 = require("../../entity/userApiKey.entity");
const typeorm_1 = require("typeorm");
const User_1 = require("../users/User");
class ApiKey {
    static async authAndGenerate(username, password) {
        const auth = await User_1.User.authByUsername(username, password);
        if (!auth) {
            throw new common_1.UnauthorizedException();
        }
        const user = await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).findOneBy({ username: username });
        const expiration = new Date();
        expiration.setDate(expiration.getDate() + 30);
        const apiKey = await (0, dataSource_1.getDataSource)().getRepository(userApiKey_entity_1.UserApiKey).save({
            user_id: user.user_id,
            api_key: this.getRanHex(),
            expiration: expiration.toISOString()
        });
        return apiKey;
    }
    static async generateApiKey(user_id, expiration, label, temporary = true) {
        const apiKey = await (0, dataSource_1.getDataSource)().getRepository(userApiKey_entity_1.UserApiKey).save({
            user_id: user_id,
            label: label,
            api_key: this.getRanHex(),
            expiration: new Date(expiration * 1000).toISOString(),
            isTemporary: temporary
        });
        return apiKey;
    }
    static async getUserByRequest(request, withGroups = false) {
        const apiKey = request.header('apiKey') ? request.header('apiKey') : request.header('apikey');
        return this.getUserByApiKey(apiKey, withGroups);
    }
    static async getUserByApiKey(apiKey, withGroups = false) {
        const key = await (0, dataSource_1.getDataSource)().getRepository(userApiKey_entity_1.UserApiKey).findOne({
            where: {
                api_key: apiKey,
                expiration: (0, typeorm_1.MoreThan)(new Date().toISOString())
            }, relations: {
                _user: withGroups
                    ? {
                        groups: true
                    } : true
            }
        });
        return key ? key._user : undefined;
    }
    static async check(apiKey) {
        return (await (0, dataSource_1.getDataSource)().getRepository(userApiKey_entity_1.UserApiKey).findOneBy({ api_key: apiKey, expiration: (0, typeorm_1.MoreThan)(new Date().toISOString()) })) !== null;
    }
    static getRanHex(size = 32) {
        let result = [];
        let hexRef = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'];
        for (let n = 0; n < size; n++) {
            result.push(hexRef[Math.floor(Math.random() * 16)]);
        }
        return result.join('');
    }
}
exports.ApiKey = ApiKey;
//# sourceMappingURL=ApiKey.js.map