"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReporterService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const reportItem_constants_1 = require("../../common/constants/reportItem.constants");
const mailGroup_entity_1 = require("../../entity/mailGroup.entity");
const report_entity_1 = require("../../entity/report.entity");
const reportFile_entity_1 = require("../../entity/reportFile.entity");
const reportItem_entity_1 = require("../../entity/reportItem.entity");
const promises_1 = require("fs/promises");
const typeorm_2 = require("typeorm");
const Reporter_1 = require("./Reporter");
const itemGroup_entity_1 = require("../../entity/itemGroup.entity");
const ItemGroupWrapper_1 = require("../custom_group/ItemGroupWrapper");
const User_1 = require("../users/User");
let ReporterService = class ReporterService {
    constructor(reportRepository, reportFileRepository, reportItemRepository, mailGroupRepository, itemGroupRepository) {
        this.reportRepository = reportRepository;
        this.reportFileRepository = reportFileRepository;
        this.reportItemRepository = reportItemRepository;
        this.mailGroupRepository = mailGroupRepository;
        this.itemGroupRepository = itemGroupRepository;
    }
    reduceMailGroupsToIds(report) {
        if (!report.mail_groups) {
            return report;
        }
        const mailGroupIds = report.mail_groups.map(mailGroup => mailGroup.mail_group_id);
        delete report.mail_groups;
        report['mail_group_ids'] = mailGroupIds;
        return report;
    }
    removeUserId(report) {
        delete report.user_id;
        return report;
    }
    async getReportsWithRelations(user_id) {
        let reports = await this.reportRepository.find({
            where: {
                user_id: user_id
            },
            relations: {
                mail_groups: true,
                report_items: {
                    _item_group: {
                        items: {
                            _item: true
                        },
                        tags: {
                            _tag: true
                        },
                        parents: {
                            _item: true
                        },
                        regex_parents: true,
                        parent_tags: {
                            _tag: true
                        }
                    }
                },
                report_files: true
            },
            order: {
                report_items: {
                    item_index: "ASC"
                },
                report_files: {
                    created_at: "DESC"
                }
            },
        });
        reports = reports
            .map(report => this.reduceMailGroupsToIds(report))
            .map(report => this.removeUserId(report));
        return reports;
    }
    async getReportWithRelations(report_id, includeFiles = true) {
        try {
            const report = await this.reportRepository.findOne({
                where: {
                    report_id: report_id
                },
                relations: {
                    mail_groups: true,
                    report_items: {
                        _item_group: {
                            items: {
                                _item: true
                            },
                            tags: {
                                _tag: true
                            },
                            parents: {
                                _item: true
                            },
                            regex_parents: true,
                            parent_tags: {
                                _tag: true
                            }
                        }
                    },
                    report_files: includeFiles ? includeFiles : undefined
                },
                order: {
                    report_items: {
                        item_index: "ASC"
                    },
                    report_files: {
                        created_at: includeFiles ? "DESC" : undefined
                    }
                },
            });
            if (!report) {
                return null;
            }
            return this.reduceMailGroupsToIds(report);
        }
        catch (e) {
            Reporter_1.Reporter.winstonLogger.warn(e);
            return null;
        }
    }
    async getReportWithItems(report_id) {
        try {
            return await this.reportRepository.findOne({
                where: {
                    report_id: report_id
                },
                relations: {
                    report_items: true
                },
                order: {
                    report_items: {
                        item_index: "ASC"
                    }
                },
            });
        }
        catch (e) {
            Reporter_1.Reporter.winstonLogger.warn(e);
            return null;
        }
    }
    async getReportWithFiles(report_id) {
        try {
            return await this.reportRepository.findOne({
                where: {
                    report_id: report_id
                },
                relations: {
                    report_files: true
                },
                order: {
                    report_files: {
                        created_at: "DESC"
                    }
                },
            });
        }
        catch (e) {
            Reporter_1.Reporter.winstonLogger.warn(e);
            return null;
        }
    }
    async getReportOnly(report_id) {
        try {
            return await this.reportRepository.findOne({
                where: {
                    report_id: report_id
                },
            });
        }
        catch (e) {
            Reporter_1.Reporter.winstonLogger.warn(e);
            return null;
        }
    }
    async getReportsOnly(user_id) {
        try {
            return await this.reportRepository.find({
                where: {
                    user_id: user_id
                },
            });
        }
        catch (e) {
            Reporter_1.Reporter.winstonLogger.warn(e);
            return null;
        }
    }
    processReportItem(reportItem, report_id) {
        const processed = reportItem;
        processed.report_id = report_id;
        if (processed.format === reportItem_constants_1.REPORT_ITEM_FORMAT.text) {
            processed.type = reportItem_constants_1.REPORT_ITEM_TYPE.text;
        }
        return processed;
    }
    handleMailGroups(report) {
        const mailGroupIds = report.mail_group_ids;
        delete report.mail_group_ids;
        if (!Array.isArray(mailGroupIds)) {
            return report;
        }
        const mailGroups = mailGroupIds.map(id => ({ mail_group_id: id }));
        report['mail_groups'] = mailGroups;
        return report;
    }
    processReportItems(reportItems, report_id) {
        return reportItems.map(i => {
            if ('parents' in i && i.parents && 'items' in i.parents && i.parents['items'] && !('item_ids' in i.parents)) {
                i.parents.item_ids = i.parents.items.map(item => item.item_id);
            }
            return this.processReportItem(i, report_id);
        });
    }
    async deleteRemovedReportItems(updateReportItems, oldReportItems) {
        if (!Array.isArray(oldReportItems) || (oldReportItems.length === 0)) {
            return;
        }
        const existingReportItemIds = updateReportItems.map(i => i.report_item_id).filter(i => i);
        const itemsToDelete = oldReportItems.filter(i => !existingReportItemIds.includes(i.report_item_id));
        const deletePromises = itemsToDelete.map(i => this.reportItemRepository.remove(i));
        return Promise.all(deletePromises);
    }
    async saveReport(report, oldReport = undefined) {
        if (oldReport) {
            report.report_id = oldReport.report_id;
            report.user_id = oldReport.user_id;
        }
        else {
            if ('report_id' in report) {
                throw new Error('ReporterService.saveReport: report with report_id but no oldReport provided.');
            }
        }
        this.handleMailGroups(report);
        const reportItemsFromInput = report.report_items;
        delete report.report_items;
        delete report['report_files'];
        delete report['created_at'];
        delete report['updated_at'];
        delete report['report_files'];
        delete report['created_by'];
        const savedReport = await this.reportRepository.save(report);
        if (Array.isArray(reportItemsFromInput)) {
            if (oldReport) {
                await this.deleteRemovedReportItems(reportItemsFromInput, oldReport.report_items);
                const oldItemGroupIds = oldReport.report_items.map(reportItem => reportItem.item_group_id);
                await this.itemGroupRepository.delete({ item_group_id: (0, typeorm_2.In)(oldItemGroupIds) });
            }
            const reportItemsSanitized = this.processReportItems(reportItemsFromInput, savedReport.report_id);
            for (const reportItemSanitized of reportItemsSanitized) {
                const itemGroupId = !reportItemSanitized.custom_group_id ? await ItemGroupWrapper_1.ItemGroupWrapper.saveItemGroup(reportItemSanitized, undefined) : null;
                reportItemSanitized['item_group_id'] = itemGroupId;
            }
            const reportItemsSavePromises = reportItemsSanitized.map(i => this.reportItemRepository.save(i));
            const savedReportItems = await Promise.all(reportItemsSavePromises);
            const transformed = this.transformReportItem(savedReportItems);
            savedReport.report_items = transformed;
        }
        return savedReport;
    }
    transformReportItem(reportItems) {
        return reportItems.map(reportItem => {
            return {
                ...reportItem,
                items: reportItem._item_group?.items && reportItem._item_group.items.length >= 1
                    ? reportItem._item_group.items.map(item => {
                        return {
                            item_id: item.item_id,
                            label: item._item.label
                        };
                    })
                    : null,
                regex: reportItem._item_group?.regex && reportItem._item_group.regex.length >= 1
                    ? reportItem._item_group.regex
                    : null,
                tags: reportItem._item_group?.tags && reportItem._item_group.tags.length >= 1
                    ? reportItem._item_group.tags.map(tag => {
                        return {
                            tag_id: tag.tag_value_id,
                            tag: tag._tag.value
                        };
                    })
                    : null,
                parents: reportItem._item_group
                    ? {
                        intersection: reportItem._item_group.intersection,
                        items: reportItem._item_group.parents && reportItem._item_group.parents.length >= 1
                            ? reportItem._item_group.parents.map(item => {
                                return {
                                    item_id: item.item_id,
                                    label: item._item.label
                                };
                            })
                            : null,
                        regex: reportItem._item_group.regex_parents && reportItem._item_group.regex_parents.length >= 1
                            ? reportItem._item_group.regex_parents.map(regex => {
                                return {
                                    subsystem: regex.subsystem,
                                    regex: regex.regex
                                };
                            })
                            : null,
                        tags: reportItem._item_group.parent_tags && reportItem._item_group.parent_tags.length >= 1
                            ? reportItem._item_group.parent_tags.map(tag => {
                                return {
                                    tag_id: tag.tag_value_id,
                                    tag: tag._tag.value
                                };
                            })
                            : null,
                    }
                    : null,
                _item_group: undefined
            };
        });
    }
    async updateReportLastRunAt(report, lastRunAt) {
        return this.reportRepository.update(report.report_id, {
            last_run_at: lastRunAt,
            updated_at: () => '"updated_at"',
        });
    }
    async removeReport(report) {
        await this.removeFilesFromReport(report);
        if (Array.isArray(report.report_items)) {
            const deletePromises = report.report_items.map(i => this.reportItemRepository.remove(i));
            await Promise.all(deletePromises);
            const itemGroupIds = report.report_items.map(reportItem => reportItem.item_group_id);
            await this.itemGroupRepository.delete({ item_group_id: (0, typeorm_2.In)(itemGroupIds) });
        }
        return this.reportRepository.remove(report);
    }
    async getReportFile(report_file_id) {
        try {
            return await this.reportFileRepository.findOne({
                where: { report_file_id: report_file_id },
                relations: { report: true }
            });
        }
        catch (e) {
            Reporter_1.Reporter.winstonLogger.warn(e);
            return null;
        }
    }
    async saveReportFile(reportFile) {
        return this.reportFileRepository.save(reportFile);
    }
    async removeReportFile(reportFile) {
        try {
            const path = Reporter_1.Reporter.getReportFilePath(reportFile);
            await (0, promises_1.rm)(path);
        }
        catch (e) {
            Reporter_1.Reporter.winstonLogger.warn(e);
        }
        return this.reportFileRepository.remove(reportFile);
    }
    async removeFilesFromReport(report) {
        if (!Array.isArray(report.report_files)) {
            return;
        }
        const removePromises = report.report_files.map(rf => this.removeReportFile(rf));
        return Promise.all(removePromises);
    }
    async getMailGroups() {
        return this.mailGroupRepository.find({ order: { label: 'ASC' } });
    }
    async getUserId(request) {
        let userId = 'session' in request && 'userId' in request.session ? request.session['userId'] : undefined;
        if (!userId) {
            const user = await User_1.User.getUserByRequest(request);
            userId = user.user_id;
        }
        return userId;
    }
};
exports.ReporterService = ReporterService;
exports.ReporterService = ReporterService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(report_entity_1.Report)),
    __param(1, (0, typeorm_1.InjectRepository)(reportFile_entity_1.ReportFile)),
    __param(2, (0, typeorm_1.InjectRepository)(reportItem_entity_1.ReportItem)),
    __param(3, (0, typeorm_1.InjectRepository)(mailGroup_entity_1.MailGroup)),
    __param(4, (0, typeorm_1.InjectRepository)(itemGroup_entity_1.ItemGroup)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], ReporterService);
//# sourceMappingURL=reporter.service.js.map