"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RecentlySearchedWrapper = void 0;
const dataSource_1 = require("../../config/dataSource");
const recentlySearched_entity_1 = require("../../entity/recentlySearched.entity");
const user_entity_1 = require("../../entity/user.entity");
class RecentlySearchedWrapper {
    constructor() { }
    async getRecentlySearched(user_id, limit) {
        let queryOptions = {};
        queryOptions.where = { user_id };
        if (limit) {
            queryOptions.take = limit;
            queryOptions.order = { updatedAt: 'DESC' };
        }
        const recentlySearched = await (0, dataSource_1.getDataSource)()
            .getRepository(recentlySearched_entity_1.RecentlySearched)
            .find(queryOptions);
        return { data: recentlySearched };
    }
    async getAllRecentlySearched() {
        const allRecentlySearched = await (0, dataSource_1.getDataSource)()
            .getRepository(recentlySearched_entity_1.RecentlySearched)
            .find();
        return { data: allRecentlySearched };
    }
    async addRecentlySearched(user_id, search_query) {
        const repository = (0, dataSource_1.getDataSource)().getRepository(recentlySearched_entity_1.RecentlySearched);
        const existing = await repository.findOne({ where: { user_id, search_query } });
        if (existing) {
            existing.updatedAt = new Date().toISOString();
            await repository.save(existing);
            return [user_id, search_query];
        }
        const recentlySearched = await repository.find({ where: { user_id }, order: { updatedAt: 'ASC' } });
        const userIdToNum = parseInt(user_id);
        const userProps = await RecentlySearchedWrapper.getUserById(userIdToNum);
        let searchHistoryLimit = userProps.configuration.searchHistoryLimit;
        if (searchHistoryLimit === undefined || searchHistoryLimit === null) {
            searchHistoryLimit = RecentlySearchedWrapper.MAX_SIZE;
        }
        if (recentlySearched.length >= searchHistoryLimit) {
            const oldest = recentlySearched[0];
            await repository.delete({ recent_search_id: oldest.recent_search_id });
        }
        const newSearch = repository.create({ user_id, search_query, updatedAt: new Date().toISOString() });
        await repository.save(newSearch);
        return [user_id, search_query];
    }
    async deleteRecentlySearched(user_id, search_query) {
        const recentlySearched = await (0, dataSource_1.getDataSource)()
            .getRepository(recentlySearched_entity_1.RecentlySearched)
            .find({ where: { user_id, search_query } });
        if (recentlySearched.length === 0) {
            throw new Error('No such query found in recently searched');
        }
        await (0, dataSource_1.getDataSource)()
            .getRepository(recentlySearched_entity_1.RecentlySearched)
            .delete({ user_id, search_query });
        return [user_id, search_query];
    }
    async findRecentlySearched(user_id, search_query) {
        const recentlySearched = await (0, dataSource_1.getDataSource)()
            .getRepository(recentlySearched_entity_1.RecentlySearched)
            .find({ where: { user_id, search_query } });
        return { data: recentlySearched };
    }
    async findRecentlySearchedById(recent_search_id) {
        const recentlySearched = await (0, dataSource_1.getDataSource)()
            .getRepository(recentlySearched_entity_1.RecentlySearched)
            .find({ where: { recent_search_id } });
        return { data: recentlySearched };
    }
    async deleteRecentlySearchedById(recent_search_id) {
        const recentlySearched = await (0, dataSource_1.getDataSource)()
            .getRepository(recentlySearched_entity_1.RecentlySearched)
            .find({ where: { recent_search_id } });
        if (recentlySearched.length === 0) {
            throw new Error('No such query found in recently searched');
        }
        await (0, dataSource_1.getDataSource)()
            .getRepository(recentlySearched_entity_1.RecentlySearched)
            .delete({ recent_search_id });
        return recentlySearched[0].search_query;
    }
    async deleteAllRecentlySearched() {
        await (0, dataSource_1.getDataSource)()
            .getRepository(recentlySearched_entity_1.RecentlySearched)
            .delete({});
    }
    static async getUserById(userId) {
        return await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).findOne({
            where: {
                user_id: userId
            }
        });
    }
}
exports.RecentlySearchedWrapper = RecentlySearchedWrapper;
RecentlySearchedWrapper.MAX_SIZE = 40;
//# sourceMappingURL=RecentlySearchedWrapper.js.map