"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SessionController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const basicResponse_dto_1 = require("../../common/basicResponse.dto");
const session_dto_1 = require("../../common/session.dto");
const auditEvent_entity_1 = require("../../entity/auditEvent.entity");
const joiValidation_pipe_1 = require("../../pipe/joiValidation.pipe");
const Audit_1 = require("../configuration/Audit");
const User_1 = require("../users/User");
const users_service_1 = require("../users/users.service");
const Joi = require("joi");
const ldap_service_1 = require("../ldap/ldap.service");
let SessionController = class SessionController {
    constructor(usersService, ldapService) {
        this.usersService = usersService;
        this.ldapService = ldapService;
    }
    logout(req) {
        req.session.destroy(function (err) {
        });
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Successfully logged out'
        };
    }
    async login(req, loginRequest) {
        let user = await User_1.User.authByUsername(loginRequest.username, loginRequest.password);
        if (!user) {
            try {
                const ldap = await this.ldapService.login(loginRequest.username, loginRequest.password);
                if (ldap) {
                    user = await this.usersService.saveLdapUser(ldap.ldap_id, loginRequest.username);
                    user.groups = await this.ldapService.getGroupsByLdapGroups(ldap.ldap_id, ldap.groups);
                }
                else {
                    throw new common_1.UnauthorizedException({
                        statusCode: common_1.HttpStatus.UNAUTHORIZED,
                        message: 'login failed',
                        error: 'bad credentials'
                    });
                }
            }
            catch (e) {
                console.error("[GenerateTokenLdapCheck] error: " + e);
                throw new common_1.UnauthorizedException({
                    statusCode: common_1.HttpStatus.UNAUTHORIZED,
                    message: 'login failed',
                    error: e?.response?.error || 'bad credentials'
                });
            }
        }
        Audit_1.Audit.log(req, auditEvent_entity_1.auditEventId.LOGIN, null, user.user_id);
        User_1.User.setLastLogged(user.user_id);
        if (user && user.active) {
            const groups = user.groups ? user.groups : (await this.usersService.getUser(user.user_id)).groups;
            const isAdmin = groups.find((group) => group.group_id === 1) ? true : false;
            req.session['userId'] = user.user_id;
            req.session['isAdmin'] = isAdmin;
            req.session['groups'] = groups.map(group => group.group_id);
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'Successfully logged in'
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.UNAUTHORIZED,
                message: 'login failed',
                error: 'user is not active',
                data: {
                    state: 'failed',
                    token: null,
                },
            }, common_1.HttpStatus.UNAUTHORIZED);
        }
    }
};
exports.SessionController = SessionController;
__decorate([
    (0, common_1.Get)('v1/logout'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", basicResponse_dto_1.BasicResponseDTO)
], SessionController.prototype, "logout", null);
__decorate([
    (0, common_1.Post)('v1/login'),
    (0, common_1.UsePipes)(new joiValidation_pipe_1.JoiValidationPipe(Joi.object({
        username: Joi.string().required(),
        password: Joi.string().required(),
    }))),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, session_dto_1.LoginRequestDTO]),
    __metadata("design:returntype", Promise)
], SessionController.prototype, "login", null);
exports.SessionController = SessionController = __decorate([
    (0, swagger_1.ApiTags)('Session'),
    (0, common_1.Controller)('api/session'),
    __metadata("design:paramtypes", [users_service_1.UsersService,
        ldap_service_1.LdapService])
], SessionController);
//# sourceMappingURL=session.controller.js.map