"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StorageGroupController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const xormon_exception_1 = require("../../exception/xormon.exception");
const storageGroups_dto_1 = require("../../common/storageGroups.dto");
const ItemRegexWrapper_1 = require("../architecture/ItemRegexWrapper");
const User_1 = require("../users/User");
const storage_group_service_1 = require("./storage_group.service");
let StorageGroupController = class StorageGroupController {
    constructor(storageGroupService) {
        this.storageGroupService = storageGroupService;
    }
    async getStorageGroups(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        const storageGroups = await this.storageGroupService.getStorageGroupsWithRelations(user.user_id, user.groups.map(g => g.group_id));
        let sgroups = [];
        let regex = false;
        for (const storageGroup of storageGroups) {
            let devices = [];
            if (storageGroup.storage_group_devices.length >= 1) {
                devices = storageGroup.storage_group_devices;
            }
            else if (storageGroup.storage_group_regex.length >= 1) {
                regex = true;
                devices = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegex('storage', 'device', storageGroup.storage_group_regex.map((regex) => regex.regex));
            }
            sgroups.push({
                storage_group_id: storageGroup.storage_group_id,
                label: storageGroup.label,
                description: storageGroup.description,
                regex: regex,
                devices: devices.map((device) => {
                    if (regex) {
                        return {
                            label: device.label,
                            item_id: device.item_id,
                            hw_type: device.hw_type,
                        };
                    }
                    else {
                        return {
                            label: device._item.label,
                            item_id: device.item_id,
                            hw_type: device._item.hw_type,
                        };
                    }
                }),
                group: storageGroup.group_id
                    ? {
                        group_id: storageGroup._group.group_id,
                        name: storageGroup._group.name
                    }
                    : null
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: sgroups,
        };
    }
    async preview(preview) {
        const devices = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegex('storage', 'device', preview.regex);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: devices.map((device) => {
                return {
                    item_id: device.item_id,
                    hw_type: device.hw_type,
                    label: device.label,
                };
            }),
        };
    }
    async createStorageGroup(request, storage_group) {
        const user = await User_1.User.getUserByRequest(request);
        const sgroup = await this.storageGroupService.saveStorageGroup(storage_group, user.user_id);
        if (sgroup.storage_group_id) {
            let devices = [];
            let regex = false;
            if (sgroup.storage_group_devices.length >= 1) {
                devices = sgroup.storage_group_devices;
            }
            else if (sgroup.storage_group_regex.length >= 1) {
                regex = true;
                devices = await this.storageGroupService.regex(sgroup.storage_group_regex.map((regex) => regex.regex));
            }
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    storage_group_id: sgroup.storage_group_id,
                    label: sgroup.label,
                    description: sgroup.description,
                    regex: regex,
                    devices: devices.map((device) => {
                        if (regex) {
                            return {
                                label: device.label,
                                item_id: device.item_id,
                                hw_type: device.hw_type,
                            };
                        }
                        else {
                            return {
                                label: device._item.label,
                                item_id: device.item_id,
                                hw_type: device._item.hw_type,
                            };
                        }
                    }),
                    group: sgroup.group_id
                        ? {
                            group_id: sgroup._group.group_id,
                            name: sgroup._group.name
                        }
                        : null
                },
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Error occurred while creating the storage group', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateStorageGroup(request, storage_group_id, storage_group) {
        const user = await User_1.User.getUserByRequest(request);
        if (!this.storageGroupService.checkStorageGroupOwner(storage_group_id, user.user_id)) {
            throw new common_1.ForbiddenException();
        }
        const sgroup = await this.storageGroupService.updateStorageGroup(storage_group_id, storage_group);
        if (sgroup.storage_group_id) {
            let devices = [];
            let regex = false;
            if (sgroup.storage_group_devices.length >= 1) {
                devices = sgroup.storage_group_devices;
            }
            else if (sgroup.storage_group_regex.length >= 1) {
                regex = true;
                devices = await this.storageGroupService.regex(sgroup.storage_group_regex.map((regex) => regex.regex));
            }
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'storage group updated',
                data: {
                    storage_group_id: sgroup.storage_group_id,
                    label: sgroup.label,
                    description: sgroup.description,
                    regex: regex,
                    devices: devices.map((device) => {
                        if (regex) {
                            return {
                                label: device.label,
                                item_id: device.item_id,
                                hw_type: device.hw_type,
                            };
                        }
                        else {
                            return {
                                label: device._item.label,
                                item_id: device.item_id,
                                hw_type: device._item.hw_type,
                            };
                        }
                    }),
                    group: sgroup.group_id
                        ? {
                            group_id: sgroup._group.group_id,
                            name: sgroup._group.name
                        }
                        : null
                },
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Error occurred while updating the storage group', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteStorageGroup(request, storage_group_id) {
        const user = await User_1.User.getUserByRequest(request);
        if (!this.storageGroupService.checkStorageGroupOwner(storage_group_id, user.user_id)) {
            throw new common_1.ForbiddenException();
        }
        const deleted = await this.storageGroupService.deleteStorageGroup(storage_group_id);
        if (deleted.affected >= 1) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'storage group deleted',
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Error occurred while deleting the storage group', common_1.HttpStatus.BAD_REQUEST);
        }
    }
};
exports.StorageGroupController = StorageGroupController;
__decorate([
    (0, common_1.Get)('v1/'),
    openapi.ApiResponse({ status: 200, type: require("../../common/storageGroups.dto").StorageGroupsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], StorageGroupController.prototype, "getStorageGroups", null);
__decorate([
    (0, common_1.Post)('v1/preview'),
    openapi.ApiResponse({ status: 201, type: require("../../common/storageGroups.dto").DevicesResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [storageGroups_dto_1.RegexPreviewDTO]),
    __metadata("design:returntype", Promise)
], StorageGroupController.prototype, "preview", null);
__decorate([
    (0, common_1.Post)('v1/'),
    openapi.ApiResponse({ status: 201, type: require("../../common/storageGroups.dto").StorageGroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, storageGroups_dto_1.StorageGroupDTO]),
    __metadata("design:returntype", Promise)
], StorageGroupController.prototype, "createStorageGroup", null);
__decorate([
    (0, common_1.Put)('v1/:storage_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/storageGroups.dto").StorageGroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('storage_group_id')),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, Object]),
    __metadata("design:returntype", Promise)
], StorageGroupController.prototype, "updateStorageGroup", null);
__decorate([
    (0, common_1.Delete)('v1/:storage_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('storage_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], StorageGroupController.prototype, "deleteStorageGroup", null);
exports.StorageGroupController = StorageGroupController = __decorate([
    (0, swagger_1.ApiTags)('Storage Group'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/storage_group'),
    __metadata("design:paramtypes", [storage_group_service_1.StorageGroupService])
], StorageGroupController);
//# sourceMappingURL=storage_group.controller.js.map