"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TagController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const tag_service_1 = require("./tag.service");
const xormon_exception_1 = require("../../exception/xormon.exception");
const swagger_1 = require("@nestjs/swagger");
const User_1 = require("../users/User");
const tagValueItem_dto_1 = require("../../common/tagValueItem.dto");
const tag_dto_1 = require("../../common/tag.dto");
const tag_interceptor_1 = require("../../interceptor/tag.interceptor");
const adminModify_guard_1 = require("../../guard/adminModify.guard");
let TagController = class TagController {
    constructor(tagService) {
        this.tagService = tagService;
    }
    async listAllTags(request) {
        const result = await this.tagService.listAllTags();
        const resultWithIndicators = await Promise.all(result.map(async (tag) => {
            const values = await this.tagService.listTagValuesByTagId(tag.tag_id);
            return {
                ...tag,
                has_values: values.length > 0,
            };
        }));
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'List of all existing tags (with has_values) property',
            data: resultWithIndicators,
        };
    }
    async listAllTagValues(request) {
        const result = await this.tagService.listAllTagValues();
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'List of all existing tag:value pairs',
            data: result,
        };
    }
    async listAllTagValueItems(request) {
        const result = await this.tagService.listAllTagValueItems();
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'List of all existing tag:value pairings linked to (object) items',
            data: result,
        };
    }
    async listTagValuesByTag(request, tag_id) {
        const result = await this.tagService.listTagValuesByTagId(tag_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'List of all existing tag:value pairings for tag_id: ' + tag_id,
            data: result,
        };
    }
    async listTagValueItemsByItem(request, item_id) {
        const result = await this.tagService.listTagValueItemsByItem(item_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'List of all existing tag:value pairings linked to item_id: ' + item_id,
            data: result,
        };
    }
    async getTag(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.label) {
            throw new xormon_exception_1.XormonException('Missing tag label.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.findTagByName(body.label);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Created tag: ' + body.label,
            data: result,
        };
    }
    async getTagById(request, tag_id) {
        const user = await User_1.User.getUserByRequest(request);
        if (!tag_id) {
            throw new xormon_exception_1.XormonException('Missing tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.findTagById(tag_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Created tag: ' + result.label,
            data: result,
        };
    }
    async getTagValuesByName(request, tag_label) {
        const user = await User_1.User.getUserByRequest(request);
        if (!tag_label) {
            throw new xormon_exception_1.XormonException('Missing tag_label.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.findTagValuesByLabel(tag_label);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Values for tag: ' + tag_label,
            data: result,
        };
    }
    async getItemsForTag(request, tag_label) {
        const user = await User_1.User.getUserByRequest(request);
        if (!tag_label) {
            throw new xormon_exception_1.XormonException('Missing tag_label.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.getItemsForTag(tag_label);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Items for tag: ' + tag_label,
            data: result,
        };
    }
    async getItemsForTagId(request, tag_id) {
        const user = await User_1.User.getUserByRequest(request);
        if (!tag_id) {
            throw new xormon_exception_1.XormonException('Missing tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.getItemsForTagId(tag_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Items for tag id: ' + tag_id,
            data: result,
        };
    }
    async getItemsForTagValueId(request, tag_value_id, hw_type, subsystem, class_name) {
        const user = await User_1.User.getUserByRequest(request);
        if (!tag_value_id) {
            throw new xormon_exception_1.XormonException('Missing tag_value_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        if (!hw_type && !subsystem && !class_name) {
            const result = await this.tagService.getItemsForTagValueId(tag_value_id);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Items for tag value id: ' + tag_value_id,
                data: result,
            };
        }
        else {
            const result = await this.tagService.getFilteredItemsForTagValueId(tag_value_id, hw_type, subsystem, class_name);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Filtered items for tag value id: ' + tag_value_id,
                data: result,
            };
        }
    }
    async saveTag(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.label) {
            throw new xormon_exception_1.XormonException('Missing tag label.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.saveTag(body.label, user.user_id, body.include_self);
        if (body.include_self) {
            await this.tagService.saveTagValue("self", result.tag_id, user.user_id);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Created tag: ' + body.label + ' by user: ' + user.username,
            data: result,
        };
    }
    async deleteTagByName(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.label) {
            throw new xormon_exception_1.XormonException('Missing tag label.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.deleteTagByName(body.label);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Deleted tag: ' + body.label + ' by user: ' + user.username,
            data: result,
        };
    }
    async updateTag(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.old_label || !body.new_label) {
            throw new xormon_exception_1.XormonException('Missing old_label or new_label.', common_1.HttpStatus.BAD_REQUEST);
        }
        const tag = await this.tagService.findTagByName(body.old_label);
        const currentIncludeSelf = tag.include_self;
        await (currentIncludeSelf && !body.include_self
            ? this.tagService.deleteTagValueByName('self', body.old_label)
            : !currentIncludeSelf && body.include_self
                ? this.tagService.saveTagValue('self', tag.tag_id, user.user_id)
                : Promise.resolve());
        await this.tagService.updateTag(body.old_label, body.new_label, body.include_self, user.user_id);
        const result = await this.tagService.findTagByName(body.new_label);
        if (!result) {
            throw new xormon_exception_1.XormonException('Saving tag by ' + body.new_label + ' failed', common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Updated tag: ' + body.old_label + ' to ' + body.new_label + ' by user: ' + user.username,
            data: result,
        };
    }
    async deleteTag(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.tag_id) {
            throw new xormon_exception_1.XormonException('Missing tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.deleteTagById(body.tag_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Deleted tag with id: ' + body.tag_id + ' by user: ' + user.username,
            data: result,
        };
    }
    async saveTagValueByName(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.value || !body.label) {
            throw new xormon_exception_1.XormonException('Missing tag value or label.', common_1.HttpStatus.BAD_REQUEST);
        }
        if (body.value === 'self') {
            throw new xormon_exception_1.XormonException('Tag value "self" is reserved.', common_1.HttpStatus.BAD_REQUEST);
        }
        const existing = await this.tagService.findTagValueByLabel(body.label, body.value);
        if (existing) {
            return {
                statusCode: common_1.HttpStatus.CONFLICT,
                message: 'pairing: ' + body.value + ' for tag_label: ' + body.label + ' already exists',
                data: existing,
            };
        }
        const result = await this.tagService.saveTagValueByName(body.value, body.label, user.user_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Created a pairing: ' + body.value + ' with tag_label: ' + body.label + ' by user: ' + user.username,
            data: result,
        };
    }
    async saveTagValueById(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.value) {
            throw new xormon_exception_1.XormonException('Missing tag value.', common_1.HttpStatus.BAD_REQUEST);
        }
        if (!body.tag_id) {
            throw new xormon_exception_1.XormonException('Missing tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        if (body.value === 'self') {
            throw new xormon_exception_1.XormonException('Tag value "self" is reserved.', common_1.HttpStatus.BAD_REQUEST);
        }
        const existing = await this.tagService.findTagValueById(body.tag_id, body.value);
        if (existing) {
            return {
                statusCode: common_1.HttpStatus.CONFLICT,
                message: 'pairing ' + 'with this tag and value: ' + body.value + ' already exists',
                data: existing,
            };
        }
        const result = await this.tagService.saveTagValue(body.value, body.tag_id, user.user_id);
        const tag = await this.tagService.findTagById(body.tag_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Created a pairing: ' + tag.label + ':' + body.value + ' by user: ' + user.username,
            data: result,
        };
    }
    async updateTagValueByName(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.old_value || !body.new_value || !body.label) {
            throw new xormon_exception_1.XormonException('Missing old value, new value, or label.', common_1.HttpStatus.BAD_REQUEST);
        }
        if (body.new_value === 'self') {
            throw new xormon_exception_1.XormonException('Tag value "self" is reserved.', common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagService.updateTagValueByName(body.old_value, body.new_value, body.label, user.user_id);
        const result = await this.tagService.findTagValueByLabel(body.label, body.new_value);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Updated pairing: ' + body.old_value + ' to ' + body.new_value + ' for tag label: ' + body.label + ' by user: ' + user.username,
            data: result,
        };
    }
    async updateTagValueById(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.old_value || !body.new_value || !body.tag_id) {
            throw new xormon_exception_1.XormonException('Missing old value, new value, or tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        if (body.new_value === 'self') {
            throw new xormon_exception_1.XormonException('Tag value "self" is reserved.', common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagService.updateTagValueById(body.old_value, body.new_value, body.tag_id, user.user_id);
        const result = await this.tagService.findTagValueById(body.tag_id, body.new_value);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Updated pairing: ' + 'from value' + body.old_value + ' to ' + body.new_value + ' by user: ' + user.username,
            data: result,
        };
    }
    async saveTagValueItem(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.item_id || !body.tag_value_id) {
            throw new xormon_exception_1.XormonException('Missing item_id or tag_value_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.saveTagValueItem(body.item_id, body.tag_value_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Assigned a pairing by user: ' + user.username,
            data: result,
        };
    }
    async deleteTagValueItem(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.item_id || !body.tag_value_id) {
            throw new xormon_exception_1.XormonException('Missing item_id or tag_value_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.deleteTagValueItem(body.item_id, body.tag_value_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Removed a pairing by user: ' + user.username,
            data: result,
        };
    }
    async updateTagValueItems(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.item_ids || !body.tag_value_id) {
            throw new xormon_exception_1.XormonException('Missing item_ids or tag_value_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagService.updateTagValueItems(body.item_ids, body.tag_value_id);
        const result = await this.tagService.listTagValueItemsByValueId(body.tag_value_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Assigned a pairing to multiple items',
            data: result,
        };
    }
    async deleteTagValueItems(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.item_ids || !body.tag_value_id) {
            throw new xormon_exception_1.XormonException('Missing item_ids or tag_value_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagService.deleteTagValueItems(body.item_ids, body.tag_value_id);
        const result = await this.tagService.listTagValueItemsByValueId(body.tag_value_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Removed a pairing from multiple items',
            data: result,
        };
    }
    async saveTagValueItems(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.item_ids || !body.tag_value_id) {
            throw new xormon_exception_1.XormonException('Missing item_ids or tag_value_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagService.saveTagValueItems(body.item_ids, body.tag_value_id);
        const result = await this.tagService.listTagValueItemsByValueId(body.tag_value_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Assigned a pairing to multiple items',
            data: result,
        };
    }
    async updateTagValueItemsByTagId(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.item_ids || !body.tag_id) {
            throw new xormon_exception_1.XormonException('Missing item_ids or tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const tagValue = await this.tagService.findTagValueById(body.tag_id, 'self');
        if (!tagValue) {
            throw new xormon_exception_1.XormonException('This tag cannot be assigned to items (missing value self)', common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagService.saveTagValueItems(body.item_ids, tagValue.tag_value_id);
        const result = await this.tagService.listTagValueItemsByValueId(tagValue.tag_value_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Assigned a pairing for a label with multiple items',
            data: result,
        };
    }
    async deleteTagValueItemsByTagId(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.item_ids || !body.tag_id) {
            throw new xormon_exception_1.XormonException('Missing item_ids or tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const tagValue = await this.tagService.findTagValueById(body.tag_id, 'self');
        if (!tagValue) {
            throw new xormon_exception_1.XormonException('This tag cannot be assigned to items (missing value self)', common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagService.deleteTagValueItems(body.item_ids, tagValue.tag_value_id);
        const result = await this.tagService.listTagValueItemsByValueId(tagValue.tag_value_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Removed a pairing from a label for multiple items',
            data: result,
        };
    }
    async saveTagValueItemsByTagId(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.item_ids || !body.tag_id) {
            throw new xormon_exception_1.XormonException('Missing item_ids or tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const tagValue = await this.tagService.findTagValueById(body.tag_id, 'self');
        if (!tagValue) {
            throw new xormon_exception_1.XormonException('This tag cannot be assigned to items (missing value self)', common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagService.saveTagValueItems(body.item_ids, tagValue.tag_value_id);
        const result = await this.tagService.listTagValueItemsByValueId(tagValue.tag_value_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Assigned a pairing for a label with multiple items',
            data: result,
        };
    }
    async deleteTagValuesById(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.tag_id) {
            throw new xormon_exception_1.XormonException('Missing tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.deleteTagValuesById(body.tag_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Deleted all tag:value pairings from the tag',
            data: result,
        };
    }
    async deleteTagValuesByLabel(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.tag_label) {
            throw new xormon_exception_1.XormonException('Missing tag_label.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.deleteTagValuesByLabel(body.tag_label);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Deleted all tag:value pairings for tag_label: ' + body.tag_label + ' by user: ' + user.username,
            data: result,
        };
    }
    async deleteTagValueByName(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.value || !body.label) {
            throw new xormon_exception_1.XormonException('Missing value or label.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.deleteTagValueByName(body.value, body.label);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Deleted pairing: ' + body.value + ' for tag_label: ' + body.label + ' by user: ' + user.username,
            data: result,
        };
    }
    async deleteTagValueById(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.value || !body.tag_id) {
            throw new xormon_exception_1.XormonException('Missing value or tag_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.deleteTagValueById(body.value, body.tag_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Deleted pairing: ' + body.value + ' with a tag',
            data: result,
        };
    }
    async deleteTagValuesByItem(request, body) {
        const user = await User_1.User.getUserByRequest(request);
        if (!body.item_id) {
            throw new xormon_exception_1.XormonException('Missing item_id.', common_1.HttpStatus.BAD_REQUEST);
        }
        const result = await this.tagService.deleteTagValuesByItem(body.item_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'Deleted all tag:value pairings for item_id: ' + body.item_id + ' by user: ' + user.username,
            data: result,
        };
    }
};
exports.TagController = TagController;
__decorate([
    (0, common_1.Get)('/v1'),
    openapi.ApiResponse({ status: 200, type: require("../../common/tag.dto").TagsWithValueIndicatorResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "listAllTags", null);
__decorate([
    (0, common_1.Get)('/v1/values'),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValue.dto").TagValuesResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "listAllTagValues", null);
__decorate([
    (0, common_1.Get)('/v1/values/items'),
    (0, common_1.UseInterceptors)(tag_interceptor_1.TagInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValueItem.dto").TagValueItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "listAllTagValueItems", null);
__decorate([
    (0, common_1.Get)('/v1/values/:tag_id'),
    (0, swagger_1.ApiParam)({ name: 'tag_id', required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValue.dto").TagValuesResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('tag_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "listTagValuesByTag", null);
__decorate([
    (0, common_1.Get)('/v1/values/items/:item_id'),
    (0, swagger_1.ApiParam)({ name: 'item_id', required: true }),
    (0, common_1.UseInterceptors)(tag_interceptor_1.TagInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValueItem.dto").TagValueItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('item_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "listTagValueItemsByItem", null);
__decorate([
    (0, common_1.Get)('/v1/label/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { label: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tag.dto").TagResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "getTag", null);
__decorate([
    (0, common_1.Get)('/v1/label/id/'),
    (0, swagger_1.ApiQuery)({ name: 'tag_id', required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tag.dto").TagResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('tag_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "getTagById", null);
__decorate([
    (0, common_1.Get)('/v1/value/name/'),
    (0, swagger_1.ApiQuery)({ name: 'tag_label', required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValue.dto").TagValuesResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('tag_label')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "getTagValuesByName", null);
__decorate([
    (0, common_1.Get)('/v1/items/tag_label/'),
    (0, swagger_1.ApiQuery)({ name: 'tag_label', required: true }),
    (0, common_1.UseInterceptors)(tag_interceptor_1.TagInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('tag_label')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "getItemsForTag", null);
__decorate([
    (0, common_1.Get)('/v1/items/tag_id/'),
    (0, swagger_1.ApiQuery)({ name: 'tag_id', required: true }),
    (0, common_1.UseInterceptors)(tag_interceptor_1.TagInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('tag_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "getItemsForTagId", null);
__decorate([
    (0, common_1.Get)('/v1/items/tag_value_id/'),
    (0, swagger_1.ApiQuery)({ name: 'tag_value_id', required: true }),
    (0, swagger_1.ApiQuery)({ name: 'hw_type', required: false }),
    (0, swagger_1.ApiQuery)({ name: 'subsystem', required: false }),
    (0, swagger_1.ApiQuery)({ name: 'class_name', required: false }),
    (0, common_1.UseInterceptors)(tag_interceptor_1.TagInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('tag_value_id')),
    __param(2, (0, common_1.Query)('hw_type')),
    __param(3, (0, common_1.Query)('subsystem')),
    __param(4, (0, common_1.Query)('class_name')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String, String, String]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "getItemsForTagValueId", null);
__decorate([
    (0, common_1.Post)('/v1/label/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { label: { type: 'string' }, include_self: { type: 'boolean' } } }, required: true }),
    openapi.ApiResponse({ status: 201, type: require("../../common/tag.dto").TagResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "saveTag", null);
__decorate([
    (0, common_1.Delete)('/v1/label/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { label: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tag.dto").TagResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTagByName", null);
__decorate([
    (0, common_1.Put)('/v1/label/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object',
            properties: {
                old_label: { type: 'string' },
                new_label: { type: 'string' },
                include_self: { type: 'boolean' }
            } },
        required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tag.dto").TagResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "updateTag", null);
__decorate([
    (0, common_1.Delete)('/v1/label/id/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { tag_id: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tag.dto").TagResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTag", null);
__decorate([
    (0, common_1.Post)('/v1/value/name/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { value: { type: 'string' }, label: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 201, type: require("../../common/tagValue.dto").TagValueResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "saveTagValueByName", null);
__decorate([
    (0, common_1.Post)('/v1/value/id'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { value: { type: 'string' }, tag_id: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 201, type: require("../../common/tagValue.dto").TagValueResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "saveTagValueById", null);
__decorate([
    (0, common_1.Put)('/v1/value/name/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { old_value: { type: 'string' }, new_value: { type: 'string' }, label: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValue.dto").TagValueResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "updateTagValueByName", null);
__decorate([
    (0, common_1.Put)('/v1/value/id/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { old_value: { type: 'string' }, new_value: { type: 'string' }, tag_id: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValue.dto").TagValueResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "updateTagValueById", null);
__decorate([
    (0, common_1.Post)('/v1/value/item/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { item_id: { type: 'string' }, tag_value_id: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 201, type: require("../../common/tagValueItem.dto").TagValueItemResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "saveTagValueItem", null);
__decorate([
    (0, common_1.Delete)('/v1/value/item/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { item_id: { type: 'string' }, tag_value_id: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValueItem.dto").TagValueItemResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTagValueItem", null);
__decorate([
    (0, common_1.Put)('/v1/value/items/'),
    (0, swagger_1.ApiBody)({ type: tagValueItem_dto_1.TagValueItemIds }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValueItem.dto").TagValueItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "updateTagValueItems", null);
__decorate([
    (0, common_1.Delete)('/v1/value/items/'),
    (0, swagger_1.ApiBody)({ type: tagValueItem_dto_1.TagValueItemIds }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValueItem.dto").TagValueItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTagValueItems", null);
__decorate([
    (0, common_1.Post)('/v1/value/items/'),
    (0, swagger_1.ApiBody)({ type: tagValueItem_dto_1.TagValueItemIds }),
    openapi.ApiResponse({ status: 201, type: require("../../common/tagValueItem.dto").TagValueItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "saveTagValueItems", null);
__decorate([
    (0, common_1.Put)('/v1/value/items/tag_id'),
    (0, swagger_1.ApiBody)({ type: tag_dto_1.TagIdItemIds }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValueItem.dto").TagValueItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "updateTagValueItemsByTagId", null);
__decorate([
    (0, common_1.Delete)('/v1/value/items/tag_id'),
    (0, swagger_1.ApiBody)({ type: tag_dto_1.TagIdItemIds }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValueItem.dto").TagValueItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTagValueItemsByTagId", null);
__decorate([
    (0, common_1.Post)('/v1/value/items/tag_id'),
    (0, swagger_1.ApiBody)({ type: tag_dto_1.TagIdItemIds }),
    openapi.ApiResponse({ status: 201, type: require("../../common/tagValueItem.dto").TagValueItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "saveTagValueItemsByTagId", null);
__decorate([
    (0, common_1.Delete)('/v1/value/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { tag_id: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValue.dto").TagValuesResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTagValuesById", null);
__decorate([
    (0, common_1.Delete)('/v1/value/label/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { tag_label: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValue.dto").TagValuesResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTagValuesByLabel", null);
__decorate([
    (0, common_1.Delete)('/v1/value/name/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { value: { type: 'string' }, label: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValue.dto").TagValueResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTagValueByName", null);
__decorate([
    (0, common_1.Delete)('/v1/value/id/'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { value: { type: 'string' }, tag_id: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValue.dto").TagValueResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTagValueById", null);
__decorate([
    (0, common_1.Delete)('/v1/value/items/all'),
    (0, swagger_1.ApiBody)({ schema: { type: 'object', properties: { item_id: { type: 'string' } } }, required: true }),
    openapi.ApiResponse({ status: 200, type: require("../../common/tagValueItem.dto").TagValueItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], TagController.prototype, "deleteTagValuesByItem", null);
exports.TagController = TagController = __decorate([
    (0, swagger_1.ApiTags)('Tags'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/tag'),
    (0, common_1.UseGuards)(adminModify_guard_1.AdminModifyGuard),
    __metadata("design:paramtypes", [tag_service_1.TagService])
], TagController);
//# sourceMappingURL=tag.controller.js.map