"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TagService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const tag_entity_1 = require("../../entity/tag.entity");
const tagValue_entity_1 = require("../../entity/tagValue.entity");
const tagValueItem_entity_1 = require("../../entity/tagValueItem.entity");
const xormon_exception_1 = require("../../exception/xormon.exception");
const typeorm_2 = require("typeorm");
const TagWrapper_1 = require("./TagWrapper");
let TagService = class TagService {
    constructor(tagRepository, tagValueRepository, tagValueItemRepository) {
        this.tagRepository = tagRepository;
        this.tagValueRepository = tagValueRepository;
        this.tagValueItemRepository = tagValueItemRepository;
    }
    async listAllTags() {
        return await this.tagRepository.find();
    }
    async listAllTagValues() {
        return await this.tagValueRepository.find();
    }
    async listAllTagValueItems() {
        return await this.tagValueItemRepository.find();
    }
    async listTagValuesByTagId(tag_id) {
        return await this.tagValueRepository.find({ where: { tag_id: tag_id } });
    }
    async listTagValueItemsByItem(item_id) {
        return await this.tagValueItemRepository.find({ where: { item_id: item_id } });
    }
    async findTagValuesByLabel(tag_label) {
        const tag = await this.tagRepository.findOne({ where: { label: tag_label } });
        if (!tag) {
            throw new xormon_exception_1.XormonException(`Tag with name ${tag_label} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        return await this.tagValueRepository.find({ where: { tag_id: tag.tag_id } });
    }
    async findTagValueByLabel(tag_label, value) {
        return await this.tagValueRepository.findOne({ where: { value: value, tag_id: (await this.findTagByName(tag_label)).tag_id } });
    }
    async findTagValueById(tag_id, value) {
        return await this.tagValueRepository.findOne({ where: { tag_id: tag_id, value: value } });
    }
    async getItemsForTag(tag_label) {
        const tag = await this.tagRepository.findOne({ where: { label: tag_label } });
        if (!tag) {
            throw new xormon_exception_1.XormonException(`Tag with name ${tag_label} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        const tagValue = await this.tagValueRepository.findOne({ where: { tag_id: tag.tag_id, value: "self" } });
        if (!tagValue) {
            return [];
        }
        return await this.getItemsForTagValueId(tagValue.tag_value_id);
    }
    async getItemsForTagValueId(tag_value_id) {
        const tagValueItems = await this.tagValueItemRepository.find({ where: { tag_value_id: tag_value_id } });
        const item_ids = tagValueItems.map(tagValueItem => tagValueItem.item_id);
        if (item_ids.length === 0) {
            return [];
        }
        return await TagWrapper_1.TagWrapper.getFullItemsForItemIds(item_ids);
    }
    async getFilteredItemsForTagValueId(tag_value_id, hw_type, subsystem, class_name) {
        const tagValueItems = await this.tagValueItemRepository.find({ where: { tag_value_id: tag_value_id } });
        const item_ids = tagValueItems.map(tagValueItem => tagValueItem.item_id);
        if (item_ids.length === 0) {
            return [];
        }
        return await TagWrapper_1.TagWrapper.getFilteredItemsForItemIds(item_ids, hw_type, subsystem, class_name);
    }
    async getItemsForTagId(tag_id) {
        const tagValue = await this.tagValueRepository.findOne({ where: { tag_id: tag_id, value: "self" } });
        if (!tagValue) {
            return [];
        }
        return await this.getItemsForTagValueId(tagValue.tag_value_id);
    }
    async listTagValueItemsByValueId(tag_value_id) {
        return await this.tagValueItemRepository.find({ where: { tag_value_id: tag_value_id } });
    }
    async deleteTagValueItems(item_ids, tag_value_id) {
        await Promise.all(item_ids.map(item_id => this.tagValueItemRepository.delete({ item_id: item_id, tag_value_id: tag_value_id })));
    }
    async saveTag(label, user_id, include_self = false) {
        const newTagLabel = this.tagRepository.create({ label, user_id, include_self });
        await this.tagRepository.save(newTagLabel);
        return this.findTagByName(label);
    }
    async updateTag(oldLabelToUse, newLabelToUse, includeSelfToUse, user_id) {
        if (includeSelfToUse) {
            await this.saveTagValue("self", (await this.findTagByName(oldLabelToUse)).tag_id, user_id);
        }
        return await this.tagRepository.update({ label: oldLabelToUse }, { label: newLabelToUse, include_self: includeSelfToUse });
    }
    async saveTagValue(value, tag_id, user_id) {
        const existingTagValue = await this.tagValueRepository.findOne({ where: { value, tag_id } });
        if (!existingTagValue) {
            const newTagValue = this.tagValueRepository.create({ value, tag_id, user_id });
            return this.tagValueRepository.save(newTagValue);
        }
    }
    async saveTagValueByName(value, tag_label, user_id) {
        const tag = await this.tagRepository.findOne({ where: { label: tag_label } });
        if (!tag) {
            throw new xormon_exception_1.XormonException(`Tag with name ${tag_label} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        const newTagValue = this.tagValueRepository.create({ value, tag_id: tag.tag_id, user_id });
        return this.tagValueRepository.save(newTagValue);
    }
    async updateTagValueByName(old_value, new_value, label, user_id) {
        const tag = await this.tagRepository.findOne({ where: { label: label } });
        if (!tag) {
            throw new xormon_exception_1.XormonException(`Tag with name ${label} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        return await this.tagValueRepository.update({ value: old_value, tag_id: tag.tag_id }, { value: new_value, tag_id: tag.tag_id });
    }
    async saveTagValueItems(item_ids, tag_value_id) {
        await this.tagValueItemRepository.delete({ tag_value_id: tag_value_id });
        this.tagValueItemRepository.save(item_ids.map(item_id => {
            return { item_id: item_id, tag_value_id: tag_value_id };
        }));
    }
    async updateTagValueItems(item_ids, tag_value_id) {
        const newTagValueItems = await Promise.all(item_ids.map(async (item_id) => {
            const existingItem = await this.tagValueItemRepository.findOne({ where: { item_id: item_id, tag_value_id: tag_value_id } });
            if (!existingItem) {
                return { item_id: item_id, tag_value_id: tag_value_id };
            }
            return null;
        }));
        const itemsToSave = newTagValueItems.filter(item => item !== null);
        if (itemsToSave.length > 0) {
            await this.tagValueItemRepository.save(itemsToSave);
        }
    }
    async updateTagValueById(old_value, new_value, tag_id, user_id) {
        return await this.tagValueRepository.update({ value: old_value, tag_id: tag_id }, { value: new_value, user_id: user_id });
    }
    async saveTagValueItem(item_id, tag_value_id) {
        const newTagValueItem = this.tagValueItemRepository.create({ item_id, tag_value_id });
        return this.tagValueItemRepository.save(newTagValueItem);
    }
    async findTagByName(label) {
        return await this.tagRepository.findOne({ where: { label: label } });
    }
    async findTagById(tag_id) {
        return await this.tagRepository.findOne({ where: { tag_id: tag_id } });
    }
    async deleteTagByName(label) {
        const tagToDelete = await this.findTagByName(label);
        if (!tagToDelete) {
            throw new xormon_exception_1.XormonException(`Tag with name ${label} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagRepository.delete({ label: label });
        return tagToDelete;
    }
    async deleteTagById(tag_id) {
        const tagToDelete = await this.findTagById(tag_id);
        if (!tagToDelete) {
            throw new xormon_exception_1.XormonException(`Tag with id ${tag_id} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagRepository.delete({ tag_id: tag_id });
        return tagToDelete;
    }
    async deleteTagValueByName(value, label) {
        const tag = await this.tagRepository.findOne({ where: { label: label } });
        if (!tag) {
            throw new xormon_exception_1.XormonException(`Tag with name ${label} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        const tagValueToDelete = await this.tagValueRepository.findOne({ where: { value: value, tag_id: tag.tag_id } });
        if (!tagValueToDelete) {
            throw new xormon_exception_1.XormonException(`TagValue with value ${value} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagValueRepository.delete({ value: value, tag_id: tag.tag_id });
        return tagValueToDelete;
    }
    async deleteTagValueById(value, tag_id) {
        const tagValueToDelete = await this.tagValueRepository.findOne({ where: { value: value, tag_id: tag_id } });
        if (!tagValueToDelete) {
            throw new xormon_exception_1.XormonException(`TagValue with value ${value} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagValueRepository.delete({ value: value, tag_id: tag_id });
        return tagValueToDelete;
    }
    async deleteTagValueItem(item_id, tag_value_id) {
        const tagValueItemToDelete = await this.tagValueItemRepository.findOne({ where: { item_id: item_id, tag_value_id: tag_value_id } });
        if (!tagValueItemToDelete) {
            throw new xormon_exception_1.XormonException(`TagValueItem with item_id ${item_id} and tag_value_id ${tag_value_id} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagValueItemRepository.delete({ item_id: item_id, tag_value_id: tag_value_id });
        return tagValueItemToDelete;
    }
    async deleteTagValuesById(tag_id) {
        const tagValuesToDelete = await this.tagValueRepository.find({ where: { tag_id: tag_id } });
        if (!tagValuesToDelete) {
            throw new xormon_exception_1.XormonException(`TagValues with tag_id ${tag_id} do not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagValueRepository.delete({ tag_id: tag_id });
        return tagValuesToDelete;
    }
    async deleteTagValuesByLabel(tag_label) {
        const tag = await this.tagRepository.findOne({ where: { label: tag_label } });
        if (!tag) {
            throw new xormon_exception_1.XormonException(`Tag with name ${tag_label} does not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        const tagValuesToDelete = await this.tagValueRepository.find({ where: { tag_id: tag.tag_id } });
        if (!tagValuesToDelete) {
            throw new xormon_exception_1.XormonException(`TagValues with tag_id ${tag.tag_id} do not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagValueRepository.delete({ tag_id: tag.tag_id });
        return tagValuesToDelete;
    }
    async deleteTagValuesByItem(item_id) {
        const tagValueItemsToDelete = await this.tagValueItemRepository.find({ where: { item_id: item_id } });
        if (!tagValueItemsToDelete) {
            throw new xormon_exception_1.XormonException(`TagValueItems with item_id ${item_id} do not exist`, common_1.HttpStatus.BAD_REQUEST);
        }
        await this.tagValueItemRepository.delete({ item_id: item_id });
        return tagValueItemsToDelete;
    }
};
exports.TagService = TagService;
exports.TagService = TagService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(tag_entity_1.Tag)),
    __param(1, (0, typeorm_1.InjectRepository)(tagValue_entity_1.TagValue)),
    __param(2, (0, typeorm_1.InjectRepository)(tagValueItem_entity_1.TagValueItem)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], TagService);
//# sourceMappingURL=tag.service.js.map