"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.User = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../../config/dataSource");
const dashboard_entity_1 = require("../../entity/dashboard.entity");
const group_entity_1 = require("../../entity/group.entity");
const user_entity_1 = require("../../entity/user.entity");
const typeorm_2 = require("typeorm");
const App_1 = require("../configuration/App");
const ApiKey_1 = require("../public/ApiKey");
const RecentlySearchedWrapper_1 = require("../search/RecentlySearchedWrapper");
let User = class User {
    static async authByEmail(email, password) {
        const user = await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User)
            .createQueryBuilder('users')
            .addSelect('users.password')
            .where("email = '" + email + "'")
            .getOne();
        if (user) {
            const match = await this.bcrypt.compareSync(password, user.password);
            return match ? user : null;
        }
        else {
            return null;
        }
    }
    static async getUserById(userId, withGroups = false) {
        return await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).findOne({
            where: {
                user_id: userId
            },
            relations: {
                groups: withGroups ? true : false
            }
        });
    }
    static async getUserByToken(token) {
        token = token.replace('Bearer ', '');
        const jwt = require('jsonwebtoken');
        const secret = App_1.App.getSecret();
        const decoded = await jwt.verify(token, secret);
        return await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).findOneBy({ username: decoded.username });
    }
    static async getUserByRequest(req, withGroups = false) {
        if (req.session['userId']) {
            const user = withGroups
                ? await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).findOne({
                    where: {
                        user_id: req.session['userId']
                    }, relations: {
                        groups: true
                    }
                })
                : await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).findOneBy({
                    user_id: req.session['userId']
                });
            if (user.ldap_id && withGroups) {
                user.groups = await (0, dataSource_1.getDataSource)().getRepository(group_entity_1.Group).findBy({
                    group_id: (0, typeorm_2.In)(req.session['groups'])
                });
            }
            if (user.configuration === null) {
                user.configuration = {};
            }
            return user;
        }
        else if (req.header('apiKey') || req.header('apikey')) {
            return ApiKey_1.ApiKey.getUserByRequest(req, withGroups);
        }
        else if (req.cookies['xormon-token'] || (req.header('authorization') && req.header('authorization').includes('Bearer'))) {
            const codedToken = req.cookies['xormon-token'] ? req.cookies['xormon-token'] : req.header('authorization').replace('Bearer ', '');
            const jwt = require('jsonwebtoken');
            const secret = App_1.App.getSecret();
            const decoded = await jwt.verify(codedToken, secret);
            const user = withGroups
                ? await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).findOne({
                    where: {
                        username: decoded.username
                    }, relations: {
                        groups: true
                    }
                })
                : await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).findOneBy({
                    username: decoded.username
                });
            if (user.ldap_id && withGroups) {
                user.groups = await (0, dataSource_1.getDataSource)().getRepository(group_entity_1.Group).findBy({
                    name: (0, typeorm_2.In)(decoded.groupLabels)
                });
            }
            return user;
        }
        else {
            return null;
        }
    }
    static async authByUsername(username, password) {
        const user = await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User)
            .createQueryBuilder('users')
            .addSelect('users.password')
            .where("username = :username", { username: username })
            .getOne();
        if (user) {
            const match = await this.bcrypt.compareSync(password, user.password);
            return match ? user : null;
        }
        else {
            return null;
        }
    }
    static async setLastLogged(user_id) {
        await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).update(user_id, { logged: new Date().toISOString() });
    }
    static async register(user) {
        const salt = await this.bcrypt.genSaltSync(10);
        let register = new user_entity_1.User();
        register.username = user.username;
        register.email = user.email;
        register.timezone = user.timezone;
        register.configuration = user.configuration ? user.configuration : {};
        register.configuration.searchHistoryLimit = register.configuration.searchHistoryLimit ? register.configuration.searchHistoryLimit : RecentlySearchedWrapper_1.RecentlySearchedWrapper.MAX_SIZE;
        register.password = await this.bcrypt.hashSync(user.password, salt);
        register.active = user.active ? user.active : true;
        let groups = [];
        const entityManager = (0, dataSource_1.getDataSource)();
        for (let i = 0; i < user.groups_id.length; i++) {
            const group = await entityManager.getRepository(group_entity_1.Group).findOneBy({ group_id: user.groups_id[i] });
            if (group) {
                groups.push(group);
            }
        }
        register.groups = groups;
        const registered = await entityManager.getRepository(user_entity_1.User).save(register);
        if (registered.user_id) {
            await (0, dataSource_1.getDataSource)().getRepository(dashboard_entity_1.Dashboard).save({
                label: 'DefaultDashboard',
                parent: null,
                type: 'personal',
                default: true,
                user_id: registered.user_id
            });
            return registered;
        }
        else {
            return null;
        }
    }
    static async update(user_id, update, logged = false) {
        const salt = await this.bcrypt.genSaltSync(10);
        const entityManager = (0, dataSource_1.getDataSource)();
        let groups = [];
        let user = await entityManager.getRepository(user_entity_1.User).findOneBy({ user_id: user_id });
        if (user === undefined) {
            return null;
        }
        for (const updateItem in update) {
            if (updateItem === 'username') {
                continue;
            }
            if (updateItem === 'configuration') {
                user.configuration = update[updateItem] ? update[updateItem] : user.configuration;
                if (user.configuration.searchHistoryLimit === undefined) {
                    user.configuration.searchHistoryLimit = RecentlySearchedWrapper_1.RecentlySearchedWrapper.MAX_SIZE;
                }
            }
            if (updateItem === 'groups_id') {
                if (user.user_id !== 1) {
                    for (let i = 0; i < update['groups_id'].length; i++) {
                        const group = await entityManager.getRepository(group_entity_1.Group).findOneBy({ group_id: update['groups_id'][i] });
                        if (group) {
                            groups.push(group);
                        }
                    }
                    user.groups = groups;
                }
            }
            else if (updateItem === 'password') {
                if (update[updateItem] === undefined || update[updateItem] === null) {
                    continue;
                }
                if (logged) {
                    if (!update['oldPassword'] || !await this.authByUsername(user.username, update['oldPassword'])) {
                        throw new common_1.HttpException({
                            statusCode: common_1.HttpStatus.BAD_REQUEST,
                            message: 'The actual password is incorrect',
                            error: 'User not updated',
                        }, common_1.HttpStatus.BAD_REQUEST);
                    }
                }
                user[updateItem] = await this.bcrypt.hashSync(update[updateItem], salt);
            }
            else if (updateItem === 'active') {
                if (user.user_id !== 1) {
                    user[updateItem] = update[updateItem];
                }
            }
            else {
                user[updateItem] = update[updateItem] ? update[updateItem] : user[updateItem];
            }
        }
        const updated = await entityManager.getRepository(user_entity_1.User).save(user);
        if (updated && updated.user_id == user_id) {
            return updated;
        }
        else {
            return null;
        }
    }
};
exports.User = User;
User.bcrypt = require('bcryptjs');
exports.User = User = __decorate([
    (0, common_1.Module)({
        imports: [typeorm_1.TypeOrmModule.forFeature([user_entity_1.User])],
    })
], User);
//# sourceMappingURL=User.js.map