"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregationTask = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../config/dataSource");
const aggregation_entity_1 = require("../entity/aggregation.entity");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const typeorm_2 = require("typeorm");
let AggregationTask = class AggregationTask {
    constructor(aggregationRepository) {
        this.aggregationRepository = aggregationRepository;
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('aggregation');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async handleInterval() {
        const offset = 1 * 60 * 60 * 1000;
        const rawDataRetention = 48 * 60 * 60 * 1000;
        const now = new Date().toISOString();
        const aggregations = await this.aggregationRepository.findBy({
            time: (0, typeorm_2.LessThan)(new Date(Date.now() - offset))
        });
        for (const aggregation of aggregations) {
            try {
                this.winstonLogger.log("info", `[${aggregation.table}] start`);
                const entity = (0, dataSource_1.getDataSource)().entityMetadatas.find(ent => ent.tableName === aggregation.table + "_1m");
                if (entity) {
                    const startTime = Date.now() - new Date(aggregation.time).getTime() > 12 * 60 * 60 * 1000 - offset
                        ? new Date(Date.now() - 12 * 60 * 60 * 1000 + offset).toISOString()
                        : new Date(aggregation.time).toISOString();
                    await this.aggregationRepository.query(`CALL refresh_continuous_aggregate('${aggregation.table}_1m', '${startTime}', '${now}');`);
                    this.winstonLogger.log("info", `[${aggregation.table}] [1m] aggregated ${startTime} - ${now}`);
                }
                let startTime = Date.now() - new Date(aggregation.time).getTime() > 36 * 60 * 60 * 1000 - offset
                    ? new Date(Date.now() - 36 * 60 * 60 * 1000 + offset).toISOString()
                    : new Date(aggregation.time).toISOString();
                await this.aggregationRepository.query(`CALL refresh_continuous_aggregate('${aggregation.table}_5m', '${startTime}', '${now}');`);
                this.winstonLogger.log("info", `[${aggregation.table}] [5m] aggregated ${startTime} - ${now}`);
                startTime = Date.now() - new Date(aggregation.time).getTime() > rawDataRetention - offset
                    ? new Date(Date.now() - rawDataRetention + offset).toISOString()
                    : new Date(aggregation.time).toISOString();
                await this.aggregationRepository.query(`CALL refresh_continuous_aggregate('${aggregation.table}_1h', '${startTime}', '${now}');`);
                this.winstonLogger.log("info", `[${aggregation.table}] [1h] aggregated ${startTime} - ${now}`);
                startTime = Date.now() - new Date(aggregation.time).getTime() > rawDataRetention - offset
                    ? new Date(Date.now() - rawDataRetention + offset).toISOString()
                    : new Date(aggregation.time).toISOString();
                await this.aggregationRepository.query(`CALL refresh_continuous_aggregate('${aggregation.table}_6h', '${startTime}', '${now}');`);
                this.winstonLogger.log("info", `[${aggregation.table}] [6h] aggregated ${startTime} - ${now}`);
                await this.aggregationRepository.delete({
                    table: aggregation.table
                });
            }
            catch (e) {
                this.winstonLogger.log("error", `Error: ${e}`);
            }
        }
    }
};
exports.AggregationTask = AggregationTask;
exports.AggregationTask = AggregationTask = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(aggregation_entity_1.Aggregation)),
    __metadata("design:paramtypes", [typeorm_2.Repository])
], AggregationTask);
//# sourceMappingURL=aggregation.task.js.map