"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmailAlert = exports.AlertHandler = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const alerting_entity_1 = require("../entity/alerting.entity");
const alertingGroup_entity_1 = require("../entity/alertingGroup.entity");
const alertingLaunch_entity_1 = require("../entity/alertingLaunch.entity");
const objectItem_entity_1 = require("../entity/objectItem.entity");
const mail_service_1 = require("../mail/mail.service");
const Settings_1 = require("../service/configuration/Settings");
const CustomGroupWrapper_1 = require("../service/custom_group/CustomGroupWrapper");
const ItemGroupWrapper_1 = require("../service/custom_group/ItemGroupWrapper");
const Color_1 = require("../service/metrics/Color");
const MetricsWrapper_1 = require("../service/metrics/MetricsWrapper");
const GraphData_1 = require("../service/reporter/GraphData");
const GraphData_factory_1 = require("../service/reporter/GraphData.factory");
const MetricTools_1 = require("../service/reporter/MetricTools");
const Vega_1 = require("../service/reporter/Vega");
const promises_1 = require("fs/promises");
const path_1 = require("path");
const typeorm_2 = require("typeorm");
const snmp_handler_1 = require("./snmp.handler");
const serviceNow_handler_1 = require("./serviceNow.handler");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const ACLCheck_1 = require("../service/acl/ACLCheck");
const User_1 = require("../service/users/User");
const URL_1 = require("../service/menu/menu/URL");
const Definitions_1 = require("../service/metrics/Definitions");
const alertingHistory_entity_1 = require("../entity/alertingHistory.entity");
const alertingHistoryItem_entity_1 = require("../entity/alertingHistoryItem.entity");
let AlertHandler = class AlertHandler {
    constructor(alertingRepository, alertingLaunchRepository, alertingGroupRepository, alertingHistoryRepository, alertingHistoryItemsRepository, objectItemsRepository, mailService, snmpHandler, serviceNowHandler) {
        this.alertingRepository = alertingRepository;
        this.alertingLaunchRepository = alertingLaunchRepository;
        this.alertingGroupRepository = alertingGroupRepository;
        this.alertingHistoryRepository = alertingHistoryRepository;
        this.alertingHistoryItemsRepository = alertingHistoryItemsRepository;
        this.objectItemsRepository = objectItemsRepository;
        this.mailService = mailService;
        this.snmpHandler = snmpHandler;
        this.serviceNowHandler = serviceNowHandler;
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('alerting');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async loadAlertingGroupItems(alertingGroup) {
        let items = [];
        let source = alertingGroup.label;
        if (alertingGroup.custom_group_id) {
            const cg = await CustomGroupWrapper_1.CustomGroupWrapper.getCustomGroup(alertingGroup.custom_group_id);
            if (!cg) {
                console.warn("[Alerting] CG " + alertingGroup.custom_group_id + " not found");
            }
            items = (await CustomGroupWrapper_1.CustomGroupWrapper.getItems(alertingGroup.custom_group_id, true));
            source = "Custom group: " + cg.label;
        }
        else {
            items = (await ItemGroupWrapper_1.ItemGroupWrapper.getItems(alertingGroup.item_group_id, "alerting"));
        }
        return {
            itemIds: items.map(item => item.item_id),
            items: items,
            source: source
        };
    }
    async getAlertingGroups() {
        return this.alertingGroupRepository.find({
            relations: {
                _group: true,
                created_by: true,
                alerts: {
                    mail_groups: true,
                    alerting_launches: true,
                }
            },
            where: {
                disabled: false
            }
        });
    }
    getDefaultRepeat() {
        return (+this.settings.ALERTING_REPEAT_TIME - 1) * 60;
    }
    getDefaultScanningLength() {
        return +this.settings.ALERTING_PEAK_TIME * 60;
    }
    getMaximumLookBackTime() {
        return 12 * 60 * 60;
    }
    findLastCompleteDataPoint(metrics, itemIds, metric) {
        const lastDataPoints = new Map();
        for (const itemId of itemIds) {
            if (!metrics[itemId] || !metrics[itemId][metric]) {
                continue;
            }
            for (const tp of Object.keys(metrics[itemId][metric]).reverse()) {
                const element = metrics[itemId][metric][tp];
                if (element !== null) {
                    lastDataPoints.set(itemId, new Date(tp));
                    break;
                }
            }
        }
        let hasNull = false;
        let leastRecent = null;
        lastDataPoints.forEach(value => {
            if (value === null) {
                hasNull = true;
            }
            else if (value && (!leastRecent || value < leastRecent)) {
                leastRecent = value;
            }
        });
        if (hasNull) {
            return null;
        }
        return leastRecent;
    }
    getItemsMetricsMap(metrics, metric) {
        const metricsMap = new Map();
        for (const itemId in metrics) {
            if (!metrics[itemId] || !metrics[itemId][metric]) {
                continue;
            }
            for (const timePoint in metrics[itemId][metric]) {
                const itemMetrics = metricsMap.get(itemId) ?? [];
                itemMetrics.push({ date: new Date(timePoint), value: metrics[itemId][metric][timePoint] });
                metricsMap.set(itemId, itemMetrics);
            }
        }
        metricsMap.forEach((itemMetrics) => { itemMetrics.sort((a, b) => (a.date.getTime() - b.date.getTime())); });
        return metricsMap;
    }
    filterOutRecentlyTriggering(itemIds, alert, checkUntil, label, debugInfo) {
        const alertingLaunchesMap = new Map();
        alert.alerting_launches.forEach(alertingLaunch => alertingLaunchesMap.set(alertingLaunch.item_id, alertingLaunch));
        const filteredItemIds = itemIds.filter(itemId => {
            const alertingLaunch = alertingLaunchesMap.get(itemId);
            if (!alertingLaunch) {
                this.winstonLogger.debug(`[AG ${label} ${alert.alerting_id} ] item ID ${itemId} is a new item triggering this alert.`);
                return true;
            }
            const repeat = alert.repeat ? ((alert.repeat - 1) * 60) : this.getDefaultRepeat();
            if (this.computeInterval(alertingLaunch.last_triggered_at, checkUntil) <= repeat) {
                this.winstonLogger.debug(`[AG ${label} ${alert.alerting_id} ] item ID ${itemId} were triggered recently (${alertingLaunch.last_triggered_at}) and repeat is ${repeat} minutes. Skipping.`);
                debugInfo.skippedItemIds.push(itemId);
                return false;
            }
            this.winstonLogger.debug(`[AG ${label} ${alert.alerting_id} ] item ID ${itemId} were triggered far ago (${alertingLaunch.last_triggered_at}) and repeat is ${repeat} minutes.`);
            return true;
        });
        return filteredItemIds;
    }
    scanForLaunch(itemMetricsArray, limit, scanningLength, start, end, debugInfo, itemId) {
        const step = 60;
        let overLimitStart = null;
        let valueArray = [];
        for (const { date, value } of itemMetricsArray) {
            if (overLimitStart) {
                if (value === null) {
                    debugInfo.nullsBridged.push({ itemId: itemId, date: date });
                    valueArray.push(limit);
                }
                else if (value >= limit) {
                    if (this.computeInterval(overLimitStart, date) >= scanningLength) {
                        this.winstonLogger.debug(`[AG] Limit: ${limit} overLimitStart: ${overLimitStart} scanning Length ${scanningLength} reached at ${date} with value ${value}`);
                        valueArray.push(value);
                        return {
                            return_value: true,
                            valueArray: valueArray,
                        };
                    }
                }
                else if (value < limit) {
                    if (this.computeInterval(overLimitStart, this.computeDate(date, -step)) >= scanningLength) {
                        this.winstonLogger.debug(`[AG] Limit: ${limit} overLimitStart: ${overLimitStart} scanning Length ${scanningLength} reached at ${date} with value ${value}`);
                        return {
                            return_value: true,
                            valueArray: valueArray,
                        };
                    }
                    valueArray = [];
                    overLimitStart = null;
                }
            }
            else {
                if (value >= limit) {
                    valueArray.push(value);
                    overLimitStart = date;
                }
            }
        }
        return {
            return_value: false,
            valueArray: valueArray,
        };
    }
    roundToWholeMinutes(date) {
        return new Date(Math.floor(date.getTime() / 60000) * 60000);
    }
    computeDate(date, ...secondsToAdd) {
        const time = secondsToAdd.reduce((acc, val) => (val * 1000 + acc), date.getTime());
        return this.roundToWholeMinutes(new Date(time));
    }
    computeTimeInSeconds(date) {
        return date.getTime() / 1000;
    }
    computeInterval(from, to) {
        return (to.getTime() - from.getTime()) / 1000;
    }
    async saveTriggeredAlert(alert, itemIds, triggeredAt) {
        alert.triggered = Math.round(triggeredAt.getTime() / 1000);
        await this.alertingRepository.save(alert);
        const alertingLaunchSavePromises = [];
        itemIds.forEach(itemId => {
            const alertingLaunch = new alertingLaunch_entity_1.AlertingLaunch();
            alertingLaunch.alerting_id = alert.alerting_id;
            alertingLaunch.item_id = itemId;
            alertingLaunch.last_triggered_at = triggeredAt;
            alertingLaunchSavePromises.push(this.alertingLaunchRepository.save(alertingLaunch));
        });
        if (!alertingLaunchSavePromises.length) {
            return;
        }
        return Promise.all(alertingLaunchSavePromises);
    }
    async gatherTriggeringItems(alert, alertingGroup, checkUntil, loadedItems) {
        let itemIds = loadedItems.itemIds;
        let valuePerId = {};
        const nowHours = new Date(checkUntil.toLocaleString("en-US", { timeZone: alertingGroup.created_by.timezone })).getHours();
        if ((nowHours >= alert.exclude_start && nowHours < alert.exclude_end) ||
            (alert.exclude_start > alert.exclude_end && nowHours >= alert.exclude_start && nowHours > alert.exclude_end) ||
            (alert.exclude_start > alert.exclude_end && nowHours <= alert.exclude_start && nowHours < alert.exclude_end)) {
            this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] exclude hours, skipping [now:${nowHours} start:${alert.exclude_start} end:${alert.exclude_end}]`);
            return [undefined, undefined];
        }
        const scanningLength = alert.peak ? (alert.peak * 60) : this.getDefaultScanningLength();
        let start;
        const end = this.roundToWholeMinutes(checkUntil);
        if (alert.checked_until && (this.computeInterval(alert.checked_until, end) < this.getMaximumLookBackTime())) {
            start = this.computeDate(alert.checked_until, -scanningLength);
            this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] checked until found (${alert.checked_until}), start: ${start} end: ${end}`);
        }
        else {
            start = this.computeDate(end, -scanningLength, -this.getMaximumLookBackTime());
            this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] checked until not found, start: ${start} end: ${end}`);
        }
        if (this.computeInterval(start, end) < scanningLength) {
            this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] Interval: ${this.computeInterval(start, end)} shorter than scanningLength: ${scanningLength}, skipping`);
            return [undefined, undefined];
        }
        let groupIds = [];
        if (alertingGroup._group && alertingGroup._group.group_id) {
            groupIds = [alertingGroup._group.group_id];
        }
        else {
            const user = await User_1.User.getUserById(alertingGroup.created_by.user_id, true);
            groupIds = user.groups.map(group => group.group_id);
        }
        itemIds = await ACLCheck_1.ACLCheck.filterItems(groupIds, itemIds);
        try {
            const metrics = await (new MetricsWrapper_1.MetricsWrapper([alert.metric], itemIds)).getDataFromTable(this.computeTimeInSeconds(start), this.computeTimeInSeconds(end), true, true);
            if (!metrics) {
                this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] No relevant metrics data found, skipping.`);
                return [undefined, undefined];
            }
            const lastCompleteDataPoint = this.findLastCompleteDataPoint(metrics, itemIds, alert.metric);
            this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] Last Complete Data Point at ${lastCompleteDataPoint}.`);
            const debugInfo = {
                alertingGroup: alertingGroup,
                start: start,
                end: end,
                scanningLength: scanningLength,
                checked_until_before: alert.checked_until,
                checked_until_after: null,
                last_complete_data_point: lastCompleteDataPoint,
                skippedItemIds: [],
                nullsBridged: [],
            };
            if (lastCompleteDataPoint && (this.computeInterval(start, lastCompleteDataPoint) >= scanningLength)) {
                alert.checked_until = lastCompleteDataPoint;
                await this.alertingRepository.save(alert);
                this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] Last Complete Data Point at ${lastCompleteDataPoint} saved.`);
            }
            debugInfo.checked_until_after = alert.checked_until;
            const itemsMetricsMap = this.getItemsMetricsMap(metrics, alert.metric);
            const triggeringItemIds = [];
            itemsMetricsMap.forEach((itemMetricsArray, itemId) => {
                let launchScan = this.scanForLaunch(itemMetricsArray, alert.limit, scanningLength, start, end, debugInfo, itemId);
                if (launchScan.return_value) {
                    const average = launchScan.valueArray.reduce((a, b) => a + b) / launchScan.valueArray.length;
                    valuePerId[itemId] = average;
                    triggeringItemIds.push(itemId);
                    this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] Alert triggered by ItemId: ${itemId}.`);
                }
            });
            if (!triggeringItemIds.length) {
                this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] No trigering items found.`);
                return [undefined, undefined];
            }
            const newTriggeringItemIds = this.filterOutRecentlyTriggering(triggeringItemIds, alert, checkUntil, alertingGroup.label, debugInfo);
            if (!newTriggeringItemIds.length) {
                this.winstonLogger.debug(`[AG ${alertingGroup.label} ${alert.alerting_id} ] All trigering items have been filtered out, because they were too recent (Repeat: ${alert.repeat} minutes).`);
                return [undefined, undefined];
            }
            await this.saveTriggeredAlert(alert, newTriggeringItemIds, checkUntil);
            const triggeringItems = await this.objectItemsRepository.find({
                where: {
                    item_id: (0, typeorm_2.In)(triggeringItemIds)
                },
                order: {
                    label: 'ASC'
                },
                relations: {
                    hw_type_entity: true,
                    parents: {
                        _parent: true
                    }
                }
            });
            return [triggeringItems, valuePerId];
        }
        catch (e) {
            this.winstonLogger.error(`[AG ${alertingGroup.label}] [A ${alert.metric}] Error: ${e}`);
        }
        return [undefined, undefined];
    }
    async sendSnmpTraps(alert, alertingGroup, triggeringItems, overlimitAveragePerId) {
        if (!this.settings.GLOB_SNMP_TRAP_HOST || !this.settings.GLOB_SNMP_COMMUNITY) {
            return;
        }
        try {
            const snmpHost = this.settings.GLOB_SNMP_TRAP_HOST;
            const communityString = this.settings.GLOB_SNMP_COMMUNITY;
            const enterpriseOid = '1.3.6.1.4.1.40540';
            const metrics = await ItemGroupWrapper_1.ItemGroupWrapper.getMetrics(alertingGroup.item_group_id, 'alerting');
            for (const item of triggeringItems) {
                const itemId = item.item_id;
                const metricSpecification = metrics.find(m => m.metric === alert.metric);
                const value = overlimitAveragePerId[itemId] && metricSpecification.metric
                    ? MetricTools_1.MetricTools.formatNumber(overlimitAveragePerId[itemId], metricSpecification, true)
                    : overlimitAveragePerId[itemId]
                        ? String(overlimitAveragePerId[itemId])
                        : 'null';
                const oidValues = {
                    [`${enterpriseOid}.1.1`]: alertingGroup.label,
                    [`${enterpriseOid}.1.2`]: item.subsystem,
                    [`${enterpriseOid}.1.3`]: item.label,
                    [`${enterpriseOid}.1.4`]: value,
                    [`${enterpriseOid}.1.5`]: alert.metric,
                    [`${enterpriseOid}.1.6`]: alert.level,
                };
                await this.snmpHandler.sendSNMPTrap(snmpHost, communityString, oidValues).catch(error => {
                    this.winstonLogger.error(`[AG ${alertingGroup.label}] Error sending SNMP trap: ${error.message}`);
                });
            }
        }
        catch (e) {
            this.winstonLogger.error(`[AG ${alertingGroup.label}] SNMP sent trap failed: ${e}`);
        }
    }
    async serviceNow(alert, alertingGroup, triggeringItems, overlimitAveragePerId) {
        if (!this.settings.SERVICENOW_INSTANCE || !this.settings.SERVICENOW_USER || !this.settings.SERVICENOW_PASS) {
            return;
        }
        const currentTime = new Date().toLocaleString();
        const serviceNowInstance = this.settings.SERVICENOW_INSTANCE;
        const serviceNowUser = this.settings.SERVICENOW_USER;
        const serviceNowPass = this.settings.SERVICENOW_PASS;
        const serviceNowType = this.settings.SERVICENOW_TYPE ?? undefined;
        const serviceNowClass = this.settings.SERVICENOW_CLASS ?? undefined;
        const severityMapping = {
            "critical": 5,
            "major": 4,
            "minor": 3,
            "warning": 2,
            "info": 1,
            "undefined": 0,
        };
        const getSeverityValue = (alertLevel) => {
            return severityMapping[alertLevel];
        };
        try {
            const metrics = await ItemGroupWrapper_1.ItemGroupWrapper.getMetrics(alertingGroup.item_group_id, 'alerting');
            for (const item of triggeringItems) {
                const itemId = item.item_id;
                const metricSpecification = metrics.find(m => m.metric === alert.metric);
                const value = overlimitAveragePerId[itemId] && metricSpecification.metric
                    ? MetricTools_1.MetricTools.formatNumber(overlimitAveragePerId[itemId], metricSpecification, true)
                    : overlimitAveragePerId[itemId]
                        ? String(overlimitAveragePerId[itemId])
                        : 'null';
                const severity = getSeverityValue(alert.level);
                const body = {
                    'records': [
                        {
                            'source': 'xormon-ng',
                            'event_class': serviceNowClass,
                            'resource': item.label,
                            'node': alertingGroup.label,
                            'metric_name': alert.metric,
                            'type': serviceNowType,
                            'severity': severity.toString(),
                            'description': `${currentTime}: ALERT: ${alertingGroup.label}: ${item.subsystem}: ${item.label}: ${alert.metric} LIMIT: ${alert.limit} VALUE: ${value}`,
                        }
                    ]
                };
                await this.serviceNowHandler.sendserviceNow(serviceNowInstance, body, {
                    user: serviceNowUser,
                    password: serviceNowPass
                }).catch(error => {
                    this.winstonLogger.error(`[AG ${alertingGroup.label}] Error sending ServiceNow: ${error.message}`);
                });
            }
        }
        catch (e) {
            this.winstonLogger.error(`[AG ${alertingGroup.label}] ServiceNow sending failed: ${e}`);
        }
    }
    async processAlerting(alert, alertingGroup, checkUntil, loadedItems) {
        const [triggeringItems, overlimitAveragePerId] = await this.gatherTriggeringItems(alert, alertingGroup, checkUntil, loadedItems);
        if (!triggeringItems) {
            return;
        }
        try {
            const history = await this.alertingHistoryRepository.save({
                alerting_group_id: alertingGroup.alerting_group_id,
                metric: alert.metric,
                limit: alert.limit,
                level: alert.level,
                targets: getEmailAddresses(alert)
            });
            await this.alertingHistoryItemsRepository.save(triggeringItems.map(i => {
                return {
                    alerting_history_id: history.alerting_history_id,
                    item_id: i.item_id
                };
            }));
        }
        catch (e) {
            this.winstonLogger.error(`[AG ${alertingGroup.label}] Failed to save alerting history. Error: ${e}`);
        }
        if (alert.integrations.snmp) {
            await this.sendSnmpTraps(alert, alertingGroup, triggeringItems, overlimitAveragePerId);
        }
        if (alert.integrations.servicenow) {
            await this.serviceNow(alert, alertingGroup, triggeringItems, overlimitAveragePerId);
        }
        const mails = getEmailAddresses(alert);
        if (mails && mails.length >= 1) {
            try {
                return new EmailAlert(this.mailService, this.settings, alert, alertingGroup).makeAndSendEmail(checkUntil, triggeringItems, loadedItems.source, overlimitAveragePerId);
            }
            catch (e) {
                this.winstonLogger.error(`[AG ${alertingGroup.label}] [A ${alert.metric}] Error: ${e}`);
            }
        }
    }
    async processAlertingGroup(alertingGroup, checkUntil) {
        if (!alertingGroup.alerts) {
            this.winstonLogger.error(`[AG ${alertingGroup.label}] no alerts found, skipped`);
            return;
        }
        try {
            const loadedItems = await this.loadAlertingGroupItems(alertingGroup);
            if (!loadedItems || loadedItems.itemIds.length < 1) {
                this.winstonLogger.error(`[AG ${alertingGroup.label}] no items found, skipped`);
                return;
            }
            const processedAlertPromises = alertingGroup.alerts.map(alert => this.processAlerting(alert, alertingGroup, checkUntil, loadedItems));
            return Promise.all(processedAlertPromises);
        }
        catch (e) {
            this.winstonLogger.error(`[AG ${alertingGroup.label}] Error: ${e}`);
        }
    }
    async checkAlerts(checkUntil) {
        const alertingGroups = await this.getAlertingGroups();
        const settings = await Settings_1.Settings.get({
            keys: [
                'ALERTING_XORMON_URL',
                'ALERTING_SEND_PER_ITEM',
                'ALERTING_REPEAT_TIME',
                'ALERTING_PEAK_TIME',
                'GLOB_SNMP_TRAP_HOST',
                'GLOB_SNMP_COMMUNITY',
                'SERVICENOW_INSTANCE',
                'SERVICENOW_USER',
                'SERVICENOW_PASS',
                'SERVICENOW_TYPE',
                'SERVICENOW_CLASS',
            ]
        });
        this.setSettings(settings);
        const processedAlertingGroupPromises = alertingGroups.map(alertingGroup => this.processAlertingGroup(alertingGroup, checkUntil));
        return Promise.all(processedAlertingGroupPromises);
    }
    setSettings(settings) {
        this.settings = settings;
        return this;
    }
};
exports.AlertHandler = AlertHandler;
exports.AlertHandler = AlertHandler = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(alerting_entity_1.Alerting)),
    __param(1, (0, typeorm_1.InjectRepository)(alertingLaunch_entity_1.AlertingLaunch)),
    __param(2, (0, typeorm_1.InjectRepository)(alertingGroup_entity_1.AlertingGroup)),
    __param(3, (0, typeorm_1.InjectRepository)(alertingHistory_entity_1.AlertingHistory)),
    __param(4, (0, typeorm_1.InjectRepository)(alertingHistoryItem_entity_1.AlertingHistoryItem)),
    __param(5, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        mail_service_1.MailService,
        snmp_handler_1.SnmpHandler,
        serviceNow_handler_1.ServiceNowHandler])
], AlertHandler);
class EmailAlert {
    constructor(mailService, settíngs, alert, alertingGroup) {
        this.mailService = mailService,
            this.alert = alert,
            this.alertingGroup = alertingGroup,
            this.settings = settíngs,
            this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('alerting');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async makeGraphFile(graphData, alertingId, singleItem = undefined) {
        if (!graphData.haveData()) {
            this.winstonLogger.error("[Graph] error: No data for graph");
            return undefined;
        }
        try {
            const vegaData = GraphData_factory_1.GraphDataFactory.makeVegaData(graphData, singleItem);
            const imageBuffer = await Vega_1.Vega.createImageBuffer(vegaData);
            const tmpPath = await asyncGetTemporaryFilesPath();
            const imagePath = `${tmpPath}/alerting-graph-${alertingId}-${graphData.end}.png`;
            await (0, promises_1.writeFile)(imagePath, imageBuffer);
            return imagePath;
        }
        catch (e) {
            this.winstonLogger.error(`[Graph] error: ${e}`);
            return undefined;
        }
    }
    makeItemsList(graphData, singleItem, debugInfo = null) {
        const ms = graphData.getMetricSpecification();
        const items = graphData.getTimeserieItems();
        const objectitems = graphData.getObjectItems();
        let unitString = ms && ms.unit ? MetricTools_1.MetricTools.makeUnitString(ms.unit, ms.defaultPrefix) : '';
        unitString = unitString ? ` [${unitString}]` : '';
        let itemsListHTML = `<table><tr><th colspan="2" style="text-align:left">Item</th>`;
        itemsListHTML += `<th style="text-align:right">&nbsp;&nbsp;&nbsp; Parent&nbsp;&nbsp;&nbsp;</th>`;
        itemsListHTML += `${debugInfo ? `<th>Item ID</th>` : ''}`;
        itemsListHTML += `<th style="text-align:right">&nbsp;&nbsp;&nbsp; AVG${unitString}&nbsp;&nbsp;&nbsp;</th>`;
        itemsListHTML += `<th style="text-align:right">MAX${unitString}&nbsp;</th>`;
        itemsListHTML += `<th></th>`;
        itemsListHTML += `</tr>`;
        let itemsListPlainText = "";
        let longestLabelLength = 0;
        let longestParentLength = 0;
        let i = 0;
        for (const timeserieItem of items) {
            if (singleItem) {
                if (timeserieItem.metadata.uuid !== singleItem.item_id) {
                    continue;
                }
            }
            const itemId = timeserieItem.metadata.uuid;
            const currentOI = objectitems.find(oi => oi.item_id === itemId);
            let itemParentLabel = graphData.getRootParentLabel(itemId);
            itemParentLabel = itemParentLabel ? itemParentLabel : "";
            const label = timeserieItem.metadata.name;
            const avg = MetricTools_1.MetricTools.formatNumber(timeserieItem.metadata.avg, ms, false);
            const max = MetricTools_1.MetricTools.formatNumber(timeserieItem.metadata.max, ms, false);
            const color = Color_1.Color.getIndexColor(i);
            const url = this.createURL(currentOI);
            const itemColumn = url ? `<a href="${url}">link</a>` : label;
            longestLabelLength = label.length > longestLabelLength ? label.length : longestLabelLength;
            longestParentLength = itemParentLabel.length > longestParentLength ? itemParentLabel.length : longestParentLength;
            itemsListHTML += `<tr><td style="background-color:${color}">&nbsp;&nbsp;</td>`;
            itemsListHTML += `<td>${label}</td>`;
            itemsListHTML += `<td style="text-align:right">&nbsp;&nbsp;&nbsp;${itemParentLabel}&nbsp;&nbsp;</td>`;
            itemsListHTML += `<td style="text-align:right">&nbsp;&nbsp;${avg}&nbsp;&nbsp;&nbsp;&nbsp;</td>`;
            itemsListHTML += `<td style="text-align:right">&nbsp;&nbsp;&nbsp;${max}&nbsp;</td>`;
            itemsListHTML += `<td>${itemColumn}</td>`;
            itemsListHTML += `</tr>`;
            itemsListPlainText += `${label.padEnd(longestLabelLength + 5)}${itemParentLabel.padEnd(longestLabelLength + 5)}${avg.padEnd(15)}${max}   ${url}\n`;
            i++;
        }
        itemsListHTML += '</table>';
        itemsListPlainText = `\n${('Item').padEnd(longestLabelLength + 5)}${('Parent').padEnd(longestParentLength + 5)}${('AVG' + unitString).padEnd(15)}MAX${unitString}\n` + itemsListPlainText;
        return {
            html: itemsListHTML,
            plainText: itemsListPlainText,
        };
    }
    createURL(objectItem) {
        let fullURL = "";
        if (this.settings.ALERTING_XORMON_URL) {
            fullURL = this.settings.ALERTING_XORMON_URL;
            if (!this.settings.ALERTING_XORMON_URL.includes("http")) {
                fullURL = `https://${this.settings.ALERTING_XORMON_URL}`;
            }
            if (!this.settings.ALERTING_XORMON_URL.includes(":")) {
                fullURL += `:${this.APP_PORT}`;
            }
            if (this.settings.ALERTING_XORMON_URL.slice(-1) !== "/") {
                fullURL += `/`;
            }
        }
        let pageType = URL_1.URL.checkTransform(objectItem.hw_type, objectItem.subsystem);
        fullURL += `${objectItem.hw_type_entity.class}/${objectItem.hw_type}/${pageType}?item_id=${objectItem.item_id}`;
        return fullURL;
    }
    makeEmailBody(graphData, date, source, metricSpecification, singleItem, debugInfo = null) {
        const debugString = debugInfo ?
            `<table>
                <tr><th>Debug info:</th><th></th></tr>
                <tr><td>Alerting Group:</td><td>${debugInfo.alertingGroup.label}</td></tr>
                <tr><td>Alerting ID:</td><td>${this.alert.alerting_id}</td></tr>
                <tr><td>Scanning started from: </td><td>${debugInfo.start}</td></tr>
                <tr><td>Scanning to: </td><td> ${debugInfo.end}</td></tr>
                <tr><td>Scanning for peak (over limit) length: </td><td>${debugInfo.scanningLength / 60} minutes</td></tr>
                <tr><td>alert.checked_until before this run: </td><td>${debugInfo.checked_until_before}</td></tr>
                <tr><td>alert.checked_until after this run: </td><td>${debugInfo.checked_until_after}</td></tr>
                <tr><td>Last Complete Data found at: </td><td>${debugInfo.last_complete_data_point}</td></tr>
                <tr><td>Skipped Item Ids: </td><td>${debugInfo.skippedItemIds}</td></tr>
                <tr><td>Repeat: </td><td>${this.alert.repeat} minutes</td></tr>
                <tr><td>Nulls bridged count: </td><td>${debugInfo.nullsBridged.length}</td></tr>
                ${debugInfo.nullsBridged.length ? `<tr><td>Nulls bridged: </td><td><table><tr><th>Item ID</th> <th>Date</th></tr>${debugInfo.nullsBridged.map(nb => `<tr><td>${nb.itemId} </td><td>${nb.date} </td></tr>`).join('')}</table></td></tr>` : ''}
            </table>
            <br><br>`
            : '';
        let formattedDate = new Intl.DateTimeFormat('sv-SE', {
            year: 'numeric',
            month: 'numeric',
            day: 'numeric',
            hour: 'numeric',
            minute: 'numeric',
            timeZone: this.alertingGroup.created_by.timezone
        }).format(date.getTime());
        let userEmail = this.alertingGroup.created_by.email ? `(${this.alertingGroup.created_by.email})` : "";
        const subsysParts = this.createSubsystemsInfo(graphData.getFirstObjectItem());
        const subsystemsInfo = `Subsystems: ${subsysParts.first} / ${subsysParts.second}<br>`;
        const alertLevel = this.alert.level && this.alert.level !== 'undefined' ? `Severity: ${this.alert.level}<br>` : "";
        const deviceLabel = source === this.alertingGroup.label ? "" : `Device: ${source}<br>`;
        const limit = MetricTools_1.MetricTools.formatNumber(graphData.alertLimit, metricSpecification, true);
        const itemLists = this.makeItemsList(graphData, singleItem, debugInfo);
        const html = `
                <b>Xormon Alert: ${this.alertingGroup.label}</b><br>
                Time: ${formattedDate} (${this.alertingGroup.created_by.timezone})<br>
                ${subsystemsInfo}
                ${deviceLabel}
                Metric: ${metricSpecification.label}<br>
                Limit: ${limit}<br>
                ${alertLevel}
                Created by: ${this.alertingGroup.created_by.username}${userEmail}<br><br>
                ${debugString}
                ${itemLists.html}
                <br><br>
              `;
        let plainText = `Xormon Alert: ${this.alertingGroup.label}\n`;
        plainText += `Metric: ${metricSpecification.label}${alertLevel ? '\n' + alertLevel.replace('<br>', '') : ""}\n`;
        plainText += `Created by: ${this.alertingGroup.created_by.username}${userEmail}\n`;
        plainText += `${itemLists.plainText}\n\n`;
        return {
            html,
            plainText
        };
    }
    async makeAndSendEmail(checkUntil, items, source, valuePerId, debugInfo) {
        const triggeringItemIds = items.map(item => item.item_id);
        const mails = getEmailAddresses(this.alert);
        const graphEnd = Math.round(checkUntil.getTime() / 1000);
        const graphStart = graphEnd - (12 * 60 * 60);
        const graphData = new GraphData_1.GraphData(this.alert.metric, triggeringItemIds, graphStart, graphEnd, this.alertingGroup.created_by.timezone);
        const itemsWithParents = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(items));
        graphData.setObjectItems(items);
        graphData.setRootParents(itemsWithParents);
        graphData.width = 640;
        graphData.alertLimit = this.alert.limit;
        graphData.includeLegend = true;
        await GraphData_factory_1.GraphDataFactory.fillGraphData(graphData);
        const group = (this.alertingGroup._group && this.alertingGroup._group.name) ? `, User group:${this.alertingGroup._group.name}` : '';
        let userEmail = this.alertingGroup.created_by.email ? `(${this.alertingGroup.created_by.email})` : "";
        let blameCreator = `${this.alertingGroup.created_by.username}${userEmail}`;
        if (this.settings.ALERTING_SEND_PER_ITEM && this.settings.ALERTING_SEND_PER_ITEM === "true") {
            for (const itemToSend of items) {
                let itemParentLabel = graphData.getRootParentLabel(itemToSend.item_id);
                itemParentLabel = itemParentLabel ? itemParentLabel : "";
                const parent = itemParentLabel ? `${itemParentLabel}:` : ``;
                const metricSpecification = graphData.getMetricSpecification();
                const alertLevel = this.alert.level && this.alert.level !== 'undefined' ? `[${this.alert.level}] ` : "";
                const tsItem = graphData.getSpecificTimeserieItem(itemToSend.item_id);
                const avg = MetricTools_1.MetricTools.formatNumber(tsItem.metadata.avg, metricSpecification, false);
                const max = MetricTools_1.MetricTools.formatNumber(tsItem.metadata.max, metricSpecification, false);
                const overLimit = MetricTools_1.MetricTools.formatNumber(valuePerId[itemToSend.item_id], metricSpecification, false);
                const limit = MetricTools_1.MetricTools.formatNumber(this.alert.limit, metricSpecification, false);
                const minLength = metricSpecification.label.length > 7 ? metricSpecification.label.length + 2 : 9;
                let metricInfo = `${(metricSpecification.label).padEnd(minLength)}${avg.padEnd(8)} ${max}`;
                graphData.graphTitle = `${parent}${itemToSend.label}:${this.makePresentable(this.alertingGroup.subsystem)}:${metricSpecification.label}`;
                let alertingOptions = {
                    legendTitle: `Metric (avg/max)`,
                    limitText: `${('Limit ').padEnd(minLength)}${limit}`,
                };
                graphData.alertingOptions = { legendTitle: alertingOptions.legendTitle, limitText: alertingOptions.limitText };
                const emailSubject = `${alertLevel}Xormon Alert: ${this.alertingGroup.label}, ${this.makePresentable(this.alertingGroup.subsystem)}:${metricSpecification.label}, ${parent}${itemToSend.label}, value: ${overLimit} (limit: ${limit})`;
                const emailBody = this.makeEmailBody(graphData, checkUntil, source, metricSpecification, itemToSend, debugInfo);
                const imagePath = await this.makeGraphFile(graphData, this.alert.alerting_id, { uuid: itemToSend.item_id, label: metricInfo });
                const emailAttachments = imagePath ? [imagePath] : undefined;
                this.winstonLogger.info(`Severity:${this.alert.level}, Alert Group:${this.alertingGroup.label}, Created by: ${blameCreator}${group}, Sent to:${mails.join(' ')}, Reason: ${metricSpecification.label}= ${limit}, Item: ${itemToSend.label}`);
                await this.mailService.send(mails, emailSubject, emailBody.html, emailAttachments, false, emailBody.plainText);
                if (imagePath) {
                    await (0, promises_1.rm)(imagePath);
                }
            }
        }
        else {
            const metricSpecification = graphData.getMetricSpecification();
            const alertLevel = this.alert.level && this.alert.level !== 'undefined' ? `[${this.alert.level}] ` : "";
            const sourceLabel = source === this.alertingGroup.label ? "" : `${source},`;
            const metricLabel = metricSpecification && 'label' in metricSpecification ? metricSpecification.label : "";
            const emailSubject = `${alertLevel}Xormon Alert: ${this.alertingGroup.label}, ${sourceLabel} ${metricLabel} : ${items.map(item => item.label).join(' ')}`;
            const emailBody = this.makeEmailBody(graphData, checkUntil, source, metricSpecification, undefined, debugInfo);
            const subsysParts = this.createSubsystemsInfo(graphData.getFirstObjectItem());
            const subsystemsInfo = `${subsysParts.first}:${subsysParts.second}`;
            graphData.graphTitle = `${this.alertingGroup.label}:${subsystemsInfo}:${metricSpecification.label}`;
            const imagePath = await this.makeGraphFile(graphData, this.alert.alerting_id);
            const emailAttachments = imagePath ? [imagePath] : undefined;
            this.winstonLogger.info(`Severity:${this.alert.level}, Alert Group:${this.alertingGroup.label}, Created by: ${blameCreator}${group}, Sent to:${mails.join(' ')}, Reason: ${metricLabel}= ${this.alert.limit}`);
            await this.mailService.send(mails, emailSubject, emailBody.html, emailAttachments, false, emailBody.plainText);
            if (imagePath) {
                await (0, promises_1.rm)(imagePath);
            }
        }
    }
    makePresentable(text) {
        return (text.charAt(0).toUpperCase() + text.slice(1)).replaceAll('_', ' ');
    }
    createSubsystemsInfo(objectItem) {
        let subsysFirstItem = ['san', 'lan', 'storage'].includes(objectItem.hw_type_entity.class) ? this.makePresentable(objectItem.hw_type_entity.class) : objectItem.hw_type_entity.label;
        let subsysSecondItem = this.makePresentable(objectItem.subsystem);
        subsysFirstItem = ['San', 'Lan'].includes(subsysFirstItem) ? subsysFirstItem.toUpperCase() : subsysFirstItem;
        subsysSecondItem = subsysSecondItem === 'Vm' && subsysSecondItem.includes('Power') ? 'LPAR' : subsysSecondItem;
        return { first: subsysFirstItem, second: subsysSecondItem };
    }
    setAppPort(newAppPort) {
        this.APP_PORT = newAppPort;
        return this;
    }
    getAppPort() {
        if (!this.APP_PORT) {
            require('dotenv').config();
            this.setAppPort(process.env.APP_PORT);
        }
        return this.APP_PORT;
    }
}
exports.EmailAlert = EmailAlert;
function getEmailAddresses(alert) {
    let result = [];
    for (const mail of alert.mail_groups) {
        result = result.concat(mail.mails);
    }
    return result;
}
const asyncGetTemporaryFilesPath = ((() => {
    let temporaryFilesPath;
    async function getExistingPath() {
        if (temporaryFilesPath) {
            return temporaryFilesPath;
        }
        const tmpDir = (0, path_1.resolve)('./files/alerting/tmp');
        await (0, promises_1.mkdir)(tmpDir, { recursive: true });
        temporaryFilesPath = tmpDir;
        return temporaryFilesPath;
    }
    return getExistingPath;
})());
//# sourceMappingURL=alert.handler.js.map