"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CleanupTask = void 0;
const common_1 = require("@nestjs/common");
const schedule_1 = require("@nestjs/schedule");
const typeorm_1 = require("@nestjs/typeorm");
const publicUrl_1 = require("../common/publicUrl");
const dataSource_1 = require("../config/dataSource");
const alertingHistory_entity_1 = require("../entity/alertingHistory.entity");
const healthStatus_entity_1 = require("../entity/healthStatus.entity");
const heatmap_entity_1 = require("../entity/heatmap.entity");
const itemProperty_entity_1 = require("../entity/itemProperty.entity");
const itemRelation_entity_1 = require("../entity/itemRelation.entity");
const microserviceJobsStatus_entity_1 = require("../entity/microserviceJobsStatus.entity");
const objectItem_entity_1 = require("../entity/objectItem.entity");
const reportFile_entity_1 = require("../entity/reportFile.entity");
const storageMapping_entity_1 = require("../entity/storageMapping.entity");
const App_1 = require("../service/configuration/App");
const Settings_1 = require("../service/configuration/Settings");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const ReportFileTools_1 = require("../service/reporter/ReportFileTools");
const path_1 = require("path");
const typeorm_2 = require("typeorm");
const DAYANDAHALF = 36;
const WEEK = 7 * 24;
const MONTH = 30 * 24;
const MONTHANDAHALF = 45 * 24;
const HALFADAY = 12;
const YEAR = MONTH * 12;
const excludeList = {
    subsystems: ['fabric', 'menugroup', 'submenugroup', 'group'],
    hw_types: [''],
};
let CleanupTask = class CleanupTask {
    constructor(objectItemsRepository, ItemRelationsRepository, ItemPropertiesRepository, MicroservicesJobsStatus, ReportFileRepository, HealthStatus, heatmapRepository, storageMappingRepository, alertingHistoryRepository) {
        this.objectItemsRepository = objectItemsRepository;
        this.ItemRelationsRepository = ItemRelationsRepository;
        this.ItemPropertiesRepository = ItemPropertiesRepository;
        this.MicroservicesJobsStatus = MicroservicesJobsStatus;
        this.ReportFileRepository = ReportFileRepository;
        this.HealthStatus = HealthStatus;
        this.heatmapRepository = heatmapRepository;
        this.storageMappingRepository = storageMappingRepository;
        this.alertingHistoryRepository = alertingHistoryRepository;
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('cleanup');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async handleFullVacuum() {
        const entityManager = (0, dataSource_1.getDataSource)();
        const result = await entityManager.query("SELECT default_version, installed_version FROM pg_available_extensions where name = 'timescaledb';");
        const tsdbVer = result.rows[0].installed_version;
        if (tsdbVer === '2.16.0' || (0, publicUrl_1.isNewerVersion)('2.16.0', tsdbVer)) {
            return;
        }
        let myQuery = "SELECT materialization_hypertable_name, view_name  FROM timescaledb_information.continuous_aggregates ca WHERE view_name like 'metrics_%_5m'";
        const tables = await entityManager.query(myQuery);
        for (const table of tables) {
            try {
                const size = await entityManager.query(`SELECT total_bytes, table_bytes FROM hypertable_detailed_size('_timescaledb_internal.${table['materialization_hypertable_name']}')`);
                const acceptableSizeRatio = 1.666;
                if (size[0] && ((size[0]['total_bytes'] - size[0]['table_bytes']) / size[0]['table_bytes'] > acceptableSizeRatio)) {
                    await entityManager.query(`VACUUM FULL  _timescaledb_internal.${table['materialization_hypertable_name']}`);
                }
            }
            catch (e) {
                this.winstonLogger.log('error', `Cleanup VACUUM error for table ${table['view_name']}: ${e} `);
            }
        }
    }
    async handleLessFrequentInterval() {
        if (!App_1.App.isProduction() && !(await App_1.App.isMaster())) {
            return;
        }
        if (!process.env['APP_DEMO']) {
            this.alertingHistory(this.calcTime(YEAR));
        }
        else {
            return;
        }
    }
    async handleInterval() {
        if (!App_1.App.isProduction() && !(await App_1.App.isMaster())) {
            return;
        }
        if (!process.env['APP_DEMO']) {
            const settings = await Settings_1.Settings.get({
                keys: [
                    'GLOB_DATA_RETENTION',
                    'GLOB_REMOVE_FROM_MENU'
                ]
            });
            this.setSettings(settings);
            this.cleanHealthStatus(this.calcTime(MONTH));
            this.cleanUpInactiveObjectItems(this.calcTime(YEAR), excludeList);
            this.cleanUpHeatmap(this.calcTime(WEEK));
            this.cleanUpItemsRelations(this.calcTime(DAYANDAHALF));
            this.cleanMicroserviceJobsStatus(this.calcTime(WEEK));
            this.cleanReports(this.calcTime(MONTHANDAHALF));
            this.cleanTmpReports(this.calcTime(DAYANDAHALF));
            this.cleanUpStorageMapping(this.calcTime(HALFADAY));
            this.cleanUpMoreFrequentItems();
        }
        else {
            return;
        }
    }
    setSettings(settings) {
        this.settings = settings;
        return this;
    }
    async tryWrapper(fnct, val) {
        try {
            fnct(val);
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `Failed with error: ${err[0]}`);
        }
    }
    getIdsPersubsys(response) {
        let dict = {};
        for (const elmnt of response) {
            if (elmnt !== undefined) {
                let hwt = elmnt['hw_type'] !== undefined ? elmnt['hw_type'] : undefined;
                let subs = elmnt['subsystem'] !== undefined ? elmnt['subsystem'] : undefined;
                let id = elmnt['item_id'];
                if (id !== undefined && hwt && subs) {
                    let tableName = "metrics_" + hwt + "_" + subs;
                    if (!dict[tableName]) {
                        dict[tableName] = "\'" + id + "\'";
                    }
                    else if (dict[tableName]) {
                        dict[tableName] = dict[tableName] + ",\'" + id + "\'";
                    }
                }
            }
        }
        return dict;
    }
    calcTime(hours) {
        let offset = hours * 60 * 60 * 1000;
        let epoch = new Date().valueOf();
        let checkTime = epoch - offset;
        let date = new Date(checkTime).toISOString();
        return date;
    }
    async cleanTmpReports(date) {
        return ReportFileTools_1.ReportFileTools.clearTmpDir(new Date(date));
    }
    async cleanReports(date) {
        let tableRes = await this.ReportFileRepository.find({
            select: ["report_file_id", 'relative_path'],
            where: { created_at: (0, typeorm_2.LessThanOrEqual)(new Date(date)) }
        });
        let reportFileIds = tableRes.map(file => file.report_file_id);
        let files = tableRes.map(file => file.relative_path);
        if (reportFileIds) {
            let ids = await this.ReportFileRepository.delete({ report_file_id: (0, typeorm_2.In)(reportFileIds) });
            this.winstonLogger.log('info', '[CleanupTask] Report Files deleted info ' + JSON.stringify(reportFileIds) + '\nList of affected IDs:\n' + ids);
        }
        if (files.length > 0) {
            let fs = require('fs');
            let reporterFilesDir = (0, path_1.resolve)('./files/reporter');
            let fileError = "";
            let fileSuccess = "";
            for (const file of files) {
                let filePath = `${reporterFilesDir}/${file}`;
                fs.stat(filePath, function (errStat, stats) {
                    if (!errStat) {
                        fs.unlink(filePath, function (errUnlk) {
                            if (errUnlk) {
                                fileError += file + " ";
                            }
                            else {
                                fileSuccess += file + " ";
                            }
                        });
                    }
                    else {
                        fileError += file + " ";
                    }
                });
            }
            if (fileError !== "") {
                this.winstonLogger.log('error', `[CleanupTask] Reporter file deletion failed for:\n ${fileError}`);
            }
            if (fileSuccess !== "") {
                this.winstonLogger.log('info', `[CleanupTask] Reporter file deletion succeeded for:\n ${fileSuccess}`);
            }
        }
    }
    async cleanHealthStatus(date) {
        try {
            let tableRes = await this.HealthStatus.find({
                select: ["item_id"],
                where: { updatedAt: (0, typeorm_2.LessThanOrEqual)(new Date(date)) }
            });
            let ids = tableRes.map(item => item.item_id);
            if (ids) {
                let hsItems = await this.HealthStatus.delete({ item_id: (0, typeorm_2.In)(ids) });
                this.winstonLogger.log('info', '[CleanupTask] Health status deleted info ' + JSON.stringify(hsItems) + '\nList of affected IDs:\n' + ids);
            }
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Failed with error: ${err[0]}`);
        }
    }
    async cleanUpInactiveObjectItems(date, excludeList) {
        if (!+this.settings.GLOB_DATA_RETENTION || +this.settings.GLOB_DATA_RETENTION < 1) {
            return;
        }
        let checkTime = new Date(+this.settings.GLOB_DATA_RETENTION * 24);
        try {
            let tableRes = [];
            tableRes = await this.objectItemsRepository.find({
                select: ["item_id", 'hw_type', 'subsystem'],
                where: {
                    updatedAt: (0, typeorm_2.LessThanOrEqual)(checkTime),
                    subsystem: (0, typeorm_2.Not)((0, typeorm_2.In)(excludeList.subsystems)),
                    hw_type: (0, typeorm_2.Not)((0, typeorm_2.In)(excludeList.hw_types)),
                    hostcfg_relations: {
                        _hostcfg: {
                            disabled: false,
                        }
                    }
                },
                relations: {
                    hostcfg_relations: {
                        _hostcfg: true
                    }
                },
                withDeleted: true,
            });
            let frequent = await this.objectItemsRepository.find({
                select: ["item_id", 'hw_type', 'subsystem'],
                where: { updatedAt: (0, typeorm_2.LessThanOrEqual)(new Date(this.calcTime(WEEK))), subsystem: (0, typeorm_2.In)(['container', 'pod']) }
            });
            tableRes.push(...frequent);
            tableRes = [...new Set(tableRes)];
            this.cleanUpMetrics(tableRes);
            let ids = tableRes.map(item => item.item_id);
            if (ids) {
                let checkTime;
                let heatmap = await this.heatmapRepository.delete({ item_id: (0, typeorm_2.In)(ids) });
                this.winstonLogger.log('info', `[CleanupTask] Heatmap items deleted info CHECK TIME ${checkTime} ` + JSON.stringify(heatmap) + '\nList of affected IDs:\n' + ids);
                let objItms = await this.objectItemsRepository.delete({ item_id: (0, typeorm_2.In)(ids) });
                this.winstonLogger.log('info', `[CleanupTask] Object items repository deleted info CHECK TIME ${checkTime} ` + JSON.stringify(objItms) + '\nList of affected IDs:\n' + ids);
                let item_prop = await this.ItemPropertiesRepository.delete({ item_id: (0, typeorm_2.In)(ids) });
                this.winstonLogger.log('info', `[CleanupTask] Item propertions repository deleted info CHECK TIME ${checkTime} ` + JSON.stringify(item_prop) + '\nList of affected IDs:\n' + ids);
                let item_rels = await this.ItemRelationsRepository.find({
                    where: [{ parent: (0, typeorm_2.In)(ids) },
                        { updatedAt: (0, typeorm_2.MoreThanOrEqual)(new Date(date)) }]
                });
                let idsToDelete = item_rels.map(item => item.parent);
                let dupes = ids.filter(item => idsToDelete.indexOf(item) >= 0);
                let item_rels_del = await this.ItemRelationsRepository.delete({ parent: (0, typeorm_2.In)(dupes) });
                this.winstonLogger.log('info', `[CleanupTask] Item relations repository deleted info CHECK TIME ${checkTime} ` + JSON.stringify(item_rels_del) + '\nList of affected IDs:\n' + ids);
            }
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Failed with error: ${err[0]}`);
        }
    }
    async cleanUpMoreFrequentItems() {
        if (!+this.settings.GLOB_REMOVE_FROM_MENU && +this.settings.GLOB_REMOVE_FROM_MENU < 1) {
            return;
        }
        let checkTime = new Date(this.calcTime(+this.settings.GLOB_REMOVE_FROM_MENU * 24));
        let hostsIds = [];
        try {
            let tableRes = [];
            tableRes = await this.objectItemsRepository.find({
                select: ["item_id", 'hw_type', 'subsystem'],
                where: {
                    updatedAt: (0, typeorm_2.LessThanOrEqual)(checkTime),
                    hostcfg_relations: {
                        _hostcfg: {
                            disabled: false
                        }
                    }
                },
                relations: {
                    hostcfg_relations: {
                        _hostcfg: true,
                    }
                }
            });
            hostsIds = tableRes.map(item => item.item_id);
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Failed with error: ${err[0]}`);
            return [];
        }
        try {
            let setToDeleted = await this.objectItemsRepository.softDelete({ item_id: (0, typeorm_2.In)(hostsIds) });
            this.winstonLogger.log('info', `[CleanupTask] Object Items set to deleted for time [${checkTime}] ` + JSON.stringify(setToDeleted) + '\nList of affected IDs:\n' + hostsIds);
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Setting items to deleted failed with an error: ${err[0]}`);
        }
    }
    async cleanUpStorageMapping(date) {
        let hours = new Date().getHours();
        if (hours !== 23 && hours !== 6) {
            return;
        }
        try {
            const manager = (0, dataSource_1.getDataSource)();
            let info = await manager.query("select sm.item_id from storage_mappings sm join object_items oi on oi.item_id=sm.item_id where oi.subsystem ='volume' GROUP BY sm.item_id HAVING COUNT(sm.item_id) > 1;");
            let ids = info.map(item => item.item_id);
            if (ids) {
                let storMap = await this.storageMappingRepository.delete({ item_id: (0, typeorm_2.In)(ids) });
                this.winstonLogger.log('info', '[CleanupTask] Storage mapping items deleted info ' + JSON.stringify(storMap) + '\nList of affected IDs:\n' + ids);
            }
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Failed with error: ${err[0]}`);
        }
    }
    async alertingHistory(date) {
        try {
            let history = await this.alertingHistoryRepository.delete({ triggeredAt: (0, typeorm_2.LessThanOrEqual)(new Date(date)) });
            this.winstonLogger.log('info', '[CleanupTask] Alerting history items deleted info ' + JSON.stringify(history));
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Alerting history Failed with error: ${err[0]}`);
        }
    }
    async cleanUpHeatmap(date) {
        try {
            let heatmap = await this.heatmapRepository.delete({ updatedAt: (0, typeorm_2.LessThanOrEqual)(new Date(date)) });
            this.winstonLogger.log('info', '[CleanupTask] Old Heatmap items deleted info ' + JSON.stringify(heatmap));
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Failed with error: ${err[0]}`);
        }
    }
    async cleanUpMetrics(tableRes) {
        try {
            const manager = (0, dataSource_1.getDataSource)();
            let tables = this.getIdsPersubsys(tableRes);
            for (const key in tables) {
                let info = await manager.query("DELETE FROM metrics." + key + " WHERE item_id in(" + tables[key] + ");");
                this.winstonLogger.log('info', '[CleanupTask] Metrics deleted info ' + JSON.stringify(info) + '\nList of affected IDs:\n' + tables[key]);
                let info1m = await manager.query("DELETE FROM metrics." + key + "_1m WHERE item_id in(" + tables[key] + ");");
                this.winstonLogger.log('info', '[CleanupTask] Metrics 1m deleted info ' + JSON.stringify(info1m) + '\nList of affected IDs:\n' + tables[key]);
                let info5m = await manager.query("DELETE FROM metrics." + key + "_5m WHERE item_id in(" + tables[key] + ");");
                this.winstonLogger.log('info', '[CleanupTask] Metrics 5m deleted info ' + JSON.stringify(info5m) + '\nList of affected IDs:\n' + tables[key]);
                let info1h = await manager.query("DELETE FROM metrics." + key + "_1h WHERE item_id in(" + tables[key] + ");");
                this.winstonLogger.log('info', '[CleanupTask] Metrics 1h deleted info ' + JSON.stringify(info1h) + '\nList of affected IDs:\n' + tables[key]);
                let info6h = await manager.query("DELETE FROM metrics." + key + "_6h WHERE item_id in(" + tables[key] + ");");
                this.winstonLogger.log('info', '[CleanupTask] Metrics 6h deleted info ' + JSON.stringify(info6h) + '\nList of affected IDs:\n' + tables[key]);
                let info1d = await manager.query("DELETE FROM metrics." + key + "_1d WHERE item_id in(" + tables[key] + ");");
                this.winstonLogger.log('info', '[CleanupTask] Metrics 1d deleted info ' + JSON.stringify(info1d) + '\nList of affected IDs:\n' + tables[key]);
            }
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Failed with error: ${err[0]}`);
        }
    }
    async cleanUpItemsRelations(date) {
        try {
            let tableRes = await this.ItemRelationsRepository.find({ where: [{ updatedAt: (0, typeorm_2.LessThanOrEqual)(new Date(date)) }] });
            let ids = tableRes.map(result => result.parent);
            if (ids) {
                let idsToDelete = await this.ItemRelationsRepository.find({
                    where: [{ parent: (0, typeorm_2.In)(ids) },
                        { updatedAt: (0, typeorm_2.MoreThanOrEqual)(new Date(date)) }]
                });
                let children = idsToDelete.map(ajdi => ajdi.child);
                let dupes = ids.filter(item => children.indexOf(item) >= 0);
                let rels_del = await this.ItemRelationsRepository.delete({ parent: (0, typeorm_2.In)(dupes) });
                this.winstonLogger.log('info', '[CleanupTask] Item relations repository deleted info ' + JSON.stringify(rels_del) + '\nList of affected IDs:\n' + dupes);
            }
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Failed with error: ${err[0]}`);
        }
    }
    async cleanMicroserviceJobsStatus(date) {
        try {
            let unixTime = Math.floor(new Date(date).valueOf() / 1000);
            let tableRes = await this.MicroservicesJobsStatus.find({
                select: ["pid"],
                where: [{ start: (0, typeorm_2.LessThanOrEqual)(unixTime) }]
            });
            let ids = tableRes.map(item => item.pid);
            let repetitions = (ids.length / 5000);
            for (let index = 0; index <= repetitions; index++) {
                let bundle = this.getBundle(ids, index, 5000);
                let rels_del = await this.MicroservicesJobsStatus.delete({ pid: (0, typeorm_2.In)(bundle) });
                this.winstonLogger.log('info', '[CleanupTask] Microservice jobs deleted info ' + JSON.stringify(rels_del) + '\nList of affected IDs:\n' + ids);
            }
        }
        catch (error) {
            let err = error.toString().split('\n');
            this.winstonLogger.log('error', `[CleanupTask] Failed with error: ${err[0]}`);
        }
    }
    async getAllBundles(original, bundleSize) {
        let bundles = [];
        let repetitions = (original.length / bundleSize);
        for (let index = 0; index <= repetitions; index++) {
            bundles.push(await this.getBundle(original, index, bundleSize));
        }
        return bundles;
    }
    getBundle(items, index, multiplier) {
        let start = index * multiplier;
        let end = (index + 1) * multiplier;
        let bundle = items.slice(start, end < items.length ? end : items.length);
        return bundle;
    }
};
exports.CleanupTask = CleanupTask;
__decorate([
    (0, schedule_1.Cron)('22 5 * * 1'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CleanupTask.prototype, "handleFullVacuum", null);
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_DAY_AT_10PM),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CleanupTask.prototype, "handleLessFrequentInterval", null);
exports.CleanupTask = CleanupTask = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(1, (0, typeorm_1.InjectRepository)(itemRelation_entity_1.ItemRelation)),
    __param(2, (0, typeorm_1.InjectRepository)(itemProperty_entity_1.ItemProperty)),
    __param(3, (0, typeorm_1.InjectRepository)(microserviceJobsStatus_entity_1.MicroserviceJobStatus)),
    __param(4, (0, typeorm_1.InjectRepository)(reportFile_entity_1.ReportFile)),
    __param(5, (0, typeorm_1.InjectRepository)(healthStatus_entity_1.HealthStatus)),
    __param(6, (0, typeorm_1.InjectRepository)(heatmap_entity_1.Heatmap)),
    __param(7, (0, typeorm_1.InjectRepository)(storageMapping_entity_1.StorageMapping)),
    __param(8, (0, typeorm_1.InjectRepository)(alertingHistory_entity_1.AlertingHistory)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], CleanupTask);
//# sourceMappingURL=cleanup.task.js.map