"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DemoTask = void 0;
const common_1 = require("@nestjs/common");
const schedule_1 = require("@nestjs/schedule");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../config/dataSource");
const hwType_entity_1 = require("../entity/hwType.entity");
const itemRelation_entity_1 = require("../entity/itemRelation.entity");
const objectItem_entity_1 = require("../entity/objectItem.entity");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const typeorm_2 = require("typeorm");
let DemoTask = class DemoTask {
    constructor(hwTypesRepository) {
        this.hwTypesRepository = hwTypesRepository;
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('demo');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async updateData() {
        if (!process.env['APP_DEMO']) {
            return;
        }
        this.winstonLogger.log("info", "*** Running demo task. ***");
        const hwTypes = await this.hwTypesRepository.find({
            relations: {
                subsystems: true
            }
        });
        for (const hwType of hwTypes) {
            for (const subsystem of hwType.subsystems) {
                const tablename = `metrics_${hwType.hw_type}_${subsystem.subsystem}`;
                try {
                    if (!await this.isTableExists(tablename)) {
                        continue;
                    }
                    this.winstonLogger.log("info", `[${tablename}] start`);
                    const entity = (0, dataSource_1.getDataSource)().manager.connection.entityMetadatas.find(ent => ent.tableName === tablename);
                    if (!entity) {
                        this.winstonLogger.log("info", `[${tablename}] skipped (no entity found)`);
                        continue;
                    }
                    const lastRow = await this.getLastRow(tablename);
                    if (!lastRow) {
                        this.winstonLogger.log("info", `[${tablename}] skipped (no data found)`);
                        continue;
                    }
                    const end = new Date(lastRow.time).toISOString();
                    const start = this.getStartTime(end);
                    const difference = Math.round((new Date().getTime() - new Date(end).getTime()) / (24 * 60 * 60 * 1000));
                    if (difference < -1) {
                        this.winstonLogger.log("info", `[${tablename}] skipped (enough data in the future)`);
                        continue;
                    }
                    const data = await (0, dataSource_1.getDataSource)().getRepository(`metrics.${tablename}`).findBy({
                        time: (0, typeorm_2.Between)(start, end)
                    });
                    for (let day = 1; day <= (difference + 2); day++) {
                        this.winstonLogger.log("info", `[${tablename}] import day ${day}`);
                        for (const timepoint of data) {
                            const time = new Date(timepoint.time);
                            time.setUTCDate(time.getUTCDate() + 1);
                            timepoint.time = time.toISOString();
                        }
                        const chunk = 1000;
                        for (let index = 0; index < data.length; index += chunk) {
                            const toSaveChunk = data.slice(index, index + chunk);
                            await (0, dataSource_1.getDataSource)().getRepository("metrics." + tablename).createQueryBuilder().insert().values(toSaveChunk).orIgnore().execute();
                        }
                        this.winstonLogger.log("info", `[${tablename}] saved`);
                    }
                    const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findBy({
                        subsystem: subsystem.subsystem,
                        hw_type: hwType.hw_type
                    });
                    for (const item of items) {
                        item.updatedAt = new Date();
                    }
                    await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).save(items);
                    const itemIds = items.map(item => item.item_id);
                    const relations = await (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation).findBy({
                        child: (0, typeorm_2.In)(itemIds),
                        parent: (0, typeorm_2.In)(itemIds)
                    });
                    for (const relation of relations) {
                        relation.updatedAt = new Date();
                    }
                    await (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation).save(relations);
                    this.winstonLogger.log("info", `[${tablename}] time updated`);
                    this.winstonLogger.log("info", `[${tablename}] finished`);
                }
                catch (e) {
                    this.winstonLogger.log("error", `[${tablename}] error: ${e}`);
                }
            }
        }
        this.winstonLogger.log("info", `*** End demo task. ***`);
    }
    async getLastRow(table) {
        return await (0, dataSource_1.getDataSource)().getRepository(`metrics.${table}`).findOne({
            where: {
                time: (0, typeorm_2.Not)((0, typeorm_2.IsNull)())
            },
            order: {
                time: 'DESC'
            }
        });
    }
    async isTableExists(table) {
        return (await (0, dataSource_1.getDataSource)().query(`
      SELECT EXISTS (
        SELECT FROM 
            pg_tables
        WHERE 
            schemaname = 'metrics' AND 
            tablename  = '${table}'
        );
    `))[0]['exists'];
    }
    ;
    getStartTime(endTime) {
        const end = new Date(endTime);
        const date = new Date();
        date.setDate(end.getDate() - 1);
        return date.toISOString();
    }
};
exports.DemoTask = DemoTask;
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_DAY_AT_MIDNIGHT),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], DemoTask.prototype, "updateData", null);
exports.DemoTask = DemoTask = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(hwType_entity_1.HwType)),
    __metadata("design:paramtypes", [typeorm_2.Repository])
], DemoTask);
//# sourceMappingURL=demo.task.js.map