"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var StuckJobTask_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.StuckJobTask = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const hwTypeKey_constants_1 = require("../common/constants/hwTypeKey.constants");
const microserviceJob_constants_1 = require("../common/constants/microserviceJob.constants");
const hostcfg_entity_1 = require("../entity/hostcfg.entity");
const microserviceJobsStatus_entity_1 = require("../entity/microserviceJobsStatus.entity");
const PM2_1 = require("../service/configuration/PM2");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const typeorm_2 = require("typeorm");
const child_process_1 = require("child_process");
const killProcessLine = (line) => {
    if (line.includes("grep"))
        return;
    const pid = line.split(/\s+/)[1];
    if (pid) {
        try {
            process.kill(parseInt(pid), "SIGTERM");
            console.log("Killed process: " + line.trim());
        }
        catch (error) {
            console.error("[DaemonKiller] Failed while killing process " + pid);
        }
    }
};
const killProcessBy = (pattern, all = false, hostcfgNames) => {
    let command = `ps aux | grep ${pattern}`;
    (0, child_process_1.exec)(command, (err, stdout, stdin) => {
        if (err)
            throw err;
        const processes = stdout.trim().split("\n");
        for (const proc of processes) {
            if (!all) {
                let killProcess = true;
                for (const name of hostcfgNames) {
                    if (proc.includes(name)) {
                        killProcess = false;
                    }
                }
                if (killProcess) {
                    killProcessLine(proc);
                }
            }
            else {
                killProcessLine(proc);
            }
        }
    });
};
let StuckJobTask = StuckJobTask_1 = class StuckJobTask {
    constructor(microserviceJobStatusRepository, hostcfgRepository) {
        this.microserviceJobStatusRepository = microserviceJobStatusRepository;
        this.hostcfgRepository = hostcfgRepository;
        this.logger = new common_1.Logger(StuckJobTask_1.name);
        this.MINUTES_5 = 5 * 60;
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('alerting');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async checkDeadTasks() {
        const runningJobs = await this.microserviceJobStatusRepository.findBy({
            end: (0, typeorm_2.IsNull)()
        });
        for (const job of runningJobs) {
            if (job.start) {
                const duration = Math.floor(Date.now() / 1000) - job.start;
                if (duration > this.MINUTES_5 && job.pid) {
                    try {
                        process.kill(job.pid, 0);
                    }
                    catch (error) {
                        job.status = microserviceJob_constants_1.JOB_STATUS.error;
                        job.end = Math.floor(Date.now() / 1000);
                    }
                }
            }
            else {
                job.start = Date.now();
            }
        }
        await this.microserviceJobStatusRepository.save(runningJobs);
    }
    async killStuckDaemonProcesses() {
        const hostcfgs = await this.hostcfgRepository.find({
            where: {
                disabled: false,
                hw_type: (0, typeorm_2.In)([hwTypeKey_constants_1.HwTypeKey.SWIZ, hwTypeKey_constants_1.HwTypeKey.SANNAV])
            }
        });
        const swizLabels = hostcfgs.filter(cfg => cfg.hw_type === "swiz").map(cfg => cfg.label);
        const sannavLabels = hostcfgs.filter(cfg => cfg.hw_type === "sannav").map(cfg => cfg.label);
        const activeLookup = ["swiz", "sannav"];
        let swizActive = false;
        let sannavActive = false;
        let list = await PM2_1.PM2.listServices();
        for (const service of list) {
            if (activeLookup[0] === service.name) {
                swizActive = service.pm2_env.status === 'online' ? true : false;
            }
            else if (activeLookup[1] === "sannav") {
                sannavActive = service.pm2_env.status === 'online' ? true : false;
                ;
            }
        }
        try {
            !swizActive ? killProcessBy("svcperf", true) : killProcessBy("svcperf", false, swizLabels);
        }
        catch (error) {
            console.error(error);
        }
        try {
            !sannavActive ? killProcessBy("sannavperf", true) : killProcessBy("sannavperf", false, sannavLabels);
        }
        catch (error) {
        }
    }
    async checkNullStartTasks() {
        await this.microserviceJobStatusRepository.delete({
            start: (0, typeorm_2.IsNull)(),
            end: (0, typeorm_2.Not)((0, typeorm_2.IsNull)())
        });
    }
};
exports.StuckJobTask = StuckJobTask;
exports.StuckJobTask = StuckJobTask = StuckJobTask_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(microserviceJobsStatus_entity_1.MicroserviceJobStatus)),
    __param(1, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository])
], StuckJobTask);
//# sourceMappingURL=stuckJob.task.js.map