"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TopTask = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const exporter_constants_1 = require("../common/constants/exporter.constants");
const objectItem_entity_1 = require("../entity/objectItem.entity");
const top_entity_1 = require("../entity/top.entity");
const Architecture_1 = require("../service/architecture/Architecture");
const Top_1 = require("../service/menu/menu/Top");
;
;
const MetricsWrapper_1 = require("../service/metrics/MetricsWrapper");
const typeorm_2 = require("typeorm");
const TaskMonitor_1 = require("./TaskMonitor");
const targetDestination_entity_1 = require("../entity/targetDestination.entity");
const targetDestinationHostcfg_entity_1 = require("../entity/targetDestinationHostcfg.entity");
const destinationSender_1 = require("./destinationSender");
let TopTask = class TopTask {
    constructor(objectItemsRepository, topRepository, targetDestinationRepository, targetDestinationHcfgRepository) {
        this.objectItemsRepository = objectItemsRepository;
        this.topRepository = topRepository;
        this.targetDestinationRepository = targetDestinationRepository;
        this.targetDestinationHcfgRepository = targetDestinationHcfgRepository;
        this.PAGE_LIMIT = 1000;
    }
    async top() {
        const taskMonitor = new TaskMonitor_1.TaskMonitor(this.top.name, 'TOP reload');
        await this.cleanupTop();
        const destination = await this.targetDestinationRepository.findOneBy({});
        const proxySender = destination ? new destinationSender_1.DestinationSender(this.targetDestinationRepository, this.targetDestinationHcfgRepository, "heatmap") : null;
        if (proxySender) {
            await proxySender.init();
        }
        const hwTypes = await Architecture_1.Architecture.getHwTypes();
        for (const hwType of hwTypes) {
            const hwTypeDefinition = MetricsWrapper_1.MetricsWrapper.getHwTypeDefinition(hwType.class, hwType.hw_type);
            if (!hwTypeDefinition || !hwTypeDefinition.subsystems) {
                continue;
            }
            for (const subsystem in hwTypeDefinition.subsystems) {
                const subsystemDefinition = hwTypeDefinition.subsystems[subsystem];
                if (subsystemDefinition.top) {
                    try {
                        const options = {
                            where: {
                                hw_type: hwType.hw_type,
                                subsystem: subsystem
                            },
                            order: {
                                item_id: "DESC"
                            }
                        };
                        const count = await this.objectItemsRepository.count(options);
                        if (count === 0) {
                            continue;
                        }
                        for (let skip = 0; skip < count; skip += this.PAGE_LIMIT) {
                            const items = (await this.objectItemsRepository.find({
                                where: {
                                    hw_type: hwType.hw_type,
                                    subsystem: subsystem
                                },
                                relations: {
                                    hostcfg_relations: proxySender ? {
                                        _hostcfg: {
                                            destinations: {
                                                _target_destination: true
                                            }
                                        }
                                    } : undefined
                                },
                                skip: skip,
                                take: this.PAGE_LIMIT
                            }));
                            const itemIds = items.map((item) => item.item_id);
                            taskMonitor.log(`Processing [${skip + this.PAGE_LIMIT < count ? skip + this.PAGE_LIMIT : count} / ${count}] ${hwTypeDefinition.label ? hwTypeDefinition.label : hwType.hw_type} ${subsystemDefinition.label ? subsystemDefinition.label : subsystem} ${itemIds.length} items`);
                            let topMetrics = [];
                            for (const group in subsystemDefinition.top) {
                                topMetrics.push(...subsystemDefinition.top[group]);
                            }
                            const metricsWrapper = new MetricsWrapper_1.MetricsWrapper(topMetrics, itemIds);
                            for (const interval of Top_1.Top.intervals) {
                                let top = [];
                                const values = await metricsWrapper.getAvgAndMax(interval.interval, interval.bucket);
                                for (const item of items) {
                                    if (values[item.item_id]) {
                                        for (const metric in values[item.item_id]) {
                                            if (values[item.item_id][metric].avg !== undefined && values[item.item_id][metric].avg !== null &&
                                                values[item.item_id][metric].max !== undefined && values[item.item_id][metric].max !== null) {
                                                top.push({
                                                    item_id: item.item_id,
                                                    metric: metric,
                                                    interval: exporter_constants_1.TOP_INTERVAL[interval.key],
                                                    avg: values[item.item_id][metric].avg,
                                                    max: values[item.item_id][metric].max,
                                                    updatedAt: new Date()
                                                });
                                                if (proxySender) {
                                                    proxySender.addItems(item, top.at(-1));
                                                }
                                            }
                                        }
                                    }
                                    if (top.length >= 500) {
                                        await this.topRepository.upsert(top, ['item_id', 'metric', 'interval']);
                                        if (proxySender) {
                                            proxySender.sendBatch();
                                        }
                                        top = [];
                                    }
                                }
                                await this.topRepository.upsert(top, ['item_id', 'metric', 'interval']);
                            }
                        }
                    }
                    catch (e) {
                        console.error(`[TopReload] ${hwType.label} ${subsystem} error: ${e}`);
                    }
                }
            }
        }
    }
    async cleanupTop() {
        await this.topRepository.delete({
            updatedAt: (0, typeorm_2.LessThan)(new Date(Date.now() - (48 * 60 * 60 * 1000)))
        });
    }
};
exports.TopTask = TopTask;
exports.TopTask = TopTask = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(1, (0, typeorm_1.InjectRepository)(top_entity_1.Top)),
    __param(2, (0, typeorm_1.InjectRepository)(targetDestination_entity_1.TargetDestination)),
    __param(3, (0, typeorm_1.InjectRepository)(targetDestinationHostcfg_entity_1.TargetDestinationHostcfg)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], TopTask);
//# sourceMappingURL=top.task.js.map