"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var BackendTask_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BackendTask = void 0;
const common_1 = require("@nestjs/common");
const schedule_1 = require("@nestjs/schedule");
const typeorm_1 = require("@nestjs/typeorm");
const backend_entity_1 = require("../entity/backend.entity");
const App_1 = require("../service/configuration/App");
const PM2_1 = require("../service/configuration/PM2");
const typeorm_2 = require("typeorm");
let BackendTask = BackendTask_1 = class BackendTask {
    constructor(backendRepository) {
        this.backendRepository = backendRepository;
        this.logger = new common_1.Logger(BackendTask_1.name);
        this.LIMIT = 20 * 60;
    }
    async handleInterval() {
        if (!PM2_1.PM2.isRunning()) {
            return;
        }
        if (!await PM2_1.PM2.shouldILive()) {
            console.error("PM2 process is not synchronized, kill the process");
            await PM2_1.PM2.commitSuicide();
        }
        const nodeId = App_1.App.getId();
        await this.backendRepository.update(nodeId, {
            echoed: Math.floor(Date.now() / 1000),
        });
        if (await App_1.App.isDebug()) {
            return;
        }
        if (!App_1.App.isProduction()) {
            const master = await this.findMasterNode();
            if (!master || (master && Math.floor(Date.now() / 1000) - master.echoed >= this.LIMIT && master.uuid !== nodeId)) {
                await App_1.App.setMaster();
            }
        }
    }
    async findMasterNode() {
        return this.backendRepository.findOneBy({ type: backend_entity_1.backendType.MASTER });
    }
    async restart() {
        let pm2 = require('pm2');
        return new Promise((resolve, reject) => {
            pm2.start('backend', (err, proc) => {
                if (err) {
                    reject(err);
                }
                resolve(proc);
            });
        });
    }
};
exports.BackendTask = BackendTask;
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_5_MINUTES),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], BackendTask.prototype, "handleInterval", null);
exports.BackendTask = BackendTask = BackendTask_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(backend_entity_1.Backend)),
    __metadata("design:paramtypes", [typeorm_2.Repository])
], BackendTask);
//# sourceMappingURL=backend.service.js.map