"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicroservicesTask = void 0;
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const event_emitter_1 = require("@nestjs/event-emitter");
const schedule_1 = require("@nestjs/schedule");
const typeorm_1 = require("@nestjs/typeorm");
const microserviceJob_constants_1 = require("../common/constants/microserviceJob.constants");
const License_1 = require("../config/License");
const hostcfg_entity_1 = require("../entity/hostcfg.entity");
const microservice_entity_1 = require("../entity/microservice.entity");
const microserviceJobs_entity_1 = require("../entity/microserviceJobs.entity");
const microserviceJobsStatus_entity_1 = require("../entity/microserviceJobsStatus.entity");
const microserviceVariables_entity_1 = require("../entity/microserviceVariables.entity");
const targetDestination_entity_1 = require("../entity/targetDestination.entity");
const targetDestinationHostcfg_entity_1 = require("../entity/targetDestinationHostcfg.entity");
const App_1 = require("../service/configuration/App");
const Variables_1 = require("../service/configuration/Variables");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const Microservice_1 = require("../service/microservices/Microservice");
const typeorm_2 = require("typeorm");
let MicroservicesTask = class MicroservicesTask {
    constructor(configService, microservicesRepository, microservicesJobsRepository, microservicesJobsStatusRepository, hostcfgRepository, targetDestinationRepository, targetDestinationHostcfgRepository, microserviceVariablesRepository, eventEmitter) {
        this.configService = configService;
        this.microservicesRepository = microservicesRepository;
        this.microservicesJobsRepository = microservicesJobsRepository;
        this.microservicesJobsStatusRepository = microservicesJobsStatusRepository;
        this.hostcfgRepository = hostcfgRepository;
        this.targetDestinationRepository = targetDestinationRepository;
        this.targetDestinationHostcfgRepository = targetDestinationHostcfgRepository;
        this.microserviceVariablesRepository = microserviceVariablesRepository;
        this.eventEmitter = eventEmitter;
        this.offset = this.configService.get('APP_JOB_OFFSET');
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('microservices');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async handleInterval() {
        if (!App_1.App.isProduction() && !(await App_1.App.isMaster())) {
            return;
        }
        const modulo = this.offset !== undefined ? +this.offset : 0;
        const hostcfgs = await this.getHostcfgByHwType();
        const globalVariables = await Variables_1.Variables.getAllKeys();
        const hasLicense = await License_1.License.hasAnyValid();
        const globalDestinations = !hasLicense ? [] : await this.targetDestinationRepository.findBy({
            target_destination_id: (0, typeorm_2.Not)((0, typeorm_2.In)((await this.targetDestinationHostcfgRepository.find())
                .map(destinationHostcfg => destinationHostcfg.target_destination_id)))
        });
        const jwt = require('jsonwebtoken');
        const date = new Date();
        const secret = App_1.App.getSecret();
        let microservices = await this.getMicroservices();
        for (const microserviceIndex in microservices) {
            let jobs = (await this.getMicroservicesJobs(microservices[microserviceIndex].microservice));
            for (const jobIndex in jobs) {
                if (date.getMinutes() % jobs[jobIndex].interval === modulo) {
                    const microservice = new Microservice_1.Microservice();
                    await microservice.init(microservices[microserviceIndex].microservice);
                    if (microservice.hw_type && microservice.isActive()) {
                        try {
                            let token = await jwt.sign({ microservice: microservice.microservice }, secret);
                            let hostCfg = hostcfgs[microservice.hw_type];
                            if (!hostCfg) {
                                continue;
                            }
                            const hwTypeVariables = await Variables_1.Variables.getAllKeys(microservice.hw_type);
                            const concurrence = hwTypeVariables.concurrence ? parseInt(hwTypeVariables.concurrence) : jobs[jobIndex].concurrence;
                            for (const hostIndex in hostCfg) {
                                let activeJobs = (await this.getMicroservicesActiveJobs(microservice.microservice, jobs[jobIndex].job, hostCfg[hostIndex].label));
                                if (activeJobs.length <= concurrence) {
                                    let data = hostCfg[hostIndex].data;
                                    const hostCfgVariables = await Variables_1.Variables.getAllKeys(null, hostCfg[hostIndex].hostcfg_id);
                                    const allHostCfgVariables = {
                                        ...globalVariables,
                                        ...hwTypeVariables,
                                        ...hostCfgVariables
                                    };
                                    if (allHostCfgVariables && Object.keys(allHostCfgVariables).length > 0) {
                                        data = {
                                            ...allHostCfgVariables,
                                            ...hostCfg[hostIndex].data
                                        };
                                    }
                                    let hostCfgData = {};
                                    if (hostCfg[hostIndex].data.device) {
                                        hostCfgData.device = hostCfg[hostIndex].data.device;
                                    }
                                    if (hostCfg[hostIndex].data.node1Host) {
                                        hostCfgData.node1Host = hostCfg[hostIndex].data.node1Host;
                                    }
                                    if (hostCfg[hostIndex].data.node2Host) {
                                        hostCfgData.node2Host = hostCfg[hostIndex].data.node2Host;
                                    }
                                    if (hostCfg[hostIndex].data.host) {
                                        hostCfgData.host = hostCfg[hostIndex].data.host;
                                    }
                                    if (hostCfg[hostIndex].data.apiHost) {
                                        hostCfgData.apiHost = hostCfg[hostIndex].data.apiHost;
                                    }
                                    if (hostCfg[hostIndex].data.snmpHost) {
                                        hostCfgData.snmpHost = hostCfg[hostIndex].data.snmpHost;
                                    }
                                    if (hostCfg[hostIndex].data.hosts && hostCfg[hostIndex].data.hosts.length > 0) {
                                        hostCfgData.hosts = hostCfg[hostIndex].data.hosts;
                                    }
                                    let message = {
                                        jwt: token,
                                        task: jobs[jobIndex].job,
                                        data: data,
                                        destinations: !hasLicense ? [] : hostCfg[hostIndex].destinations ? hostCfg[hostIndex].destinations.map(destination => {
                                            return {
                                                destination: destination._target_destination.destination,
                                                port: destination._target_destination.port,
                                                token: destination._target_destination.token,
                                                data: hostCfgData
                                            };
                                        }) : undefined
                                    };
                                    if (globalDestinations && globalDestinations.length >= 1) {
                                        message.destinations = message.destinations
                                            ? message.destinations.concat(globalDestinations.map(destination => {
                                                return {
                                                    destination: destination.destination,
                                                    port: destination.port,
                                                    token: destination.token,
                                                    data: hostCfgData
                                                };
                                            }))
                                            : globalDestinations.map(destination => {
                                                return {
                                                    destination: destination.destination,
                                                    port: destination.port,
                                                    token: destination.token,
                                                    data: hostCfgData
                                                };
                                            });
                                    }
                                    if (!message.data.hostcfg_id) {
                                        message.data.hostcfg_id = hostCfg[hostIndex].hostcfg_id;
                                    }
                                    if (!message.data.hostalias) {
                                        message.data.hostalias = hostCfg[hostIndex].label;
                                    }
                                    this.winstonLogger.log('info', '[Scheduled job] Service: ' + microservice.microservice + ' [' + jobs[jobIndex].job + '] -> ' + hostCfg[hostIndex].label + ': SEND JOB');
                                    microservice.sendData(JSON.stringify(message)).catch((error) => {
                                        this.winstonLogger.log('info', '[Scheduled job] Service: ' + microservice.microservice + ' [' + jobs[jobIndex].job + '] -> ' + hostCfg[hostIndex].label + ': ERROR ' + error);
                                    });
                                }
                                else {
                                    console.log('[Scheduled job] Service: ' + microservice.microservice + ' [' + jobs[jobIndex].job + '] -> ' + hostCfg[hostIndex].label + `": WARN There are already maximum jobs running (${activeJobs.length}) for this task. Skip this job."`);
                                    this.winstonLogger.log('warn', '[Scheduled job] Service: ' + microservice.microservice + ' [' + jobs[jobIndex].job + '] -> ' + hostCfg[hostIndex].label + `": WARN There are already maximum jobs running (${activeJobs.length}) for this task. Skip this job."`);
                                }
                            }
                        }
                        catch (e) {
                            console.error("MicroservicesTaskError: " + e);
                        }
                    }
                    else {
                        if (microservices[microserviceIndex].onStartup && !microservice.isActive()) {
                            this.winstonLogger.log('info', 'Microservice: ' + microservice.microservice + ' is not active!');
                            this.eventEmitter.emit('message.created', {
                                event: 'message',
                                message: {
                                    url: 'localhost:8443',
                                    header: 'Microservice',
                                    text: 'Microservice: ' + microservice.microservice + ' is not active!',
                                    level: 'warning',
                                    timestamp: Date.now(),
                                },
                            });
                        }
                    }
                }
            }
        }
    }
    async getMicroservices() {
        return this.microservicesRepository.find();
    }
    async getMicroservicesJobs(microservice) {
        const jobs = await this.microservicesJobsRepository.findBy({ microservice: microservice });
        return (await this.microserviceVariablesRepository.findBy({
            hw_type: microservice,
            key: (0, typeorm_2.In)(jobs.map(job => job.job))
        })).map(mJob => {
            return {
                ...jobs.find((_job) => _job.job === mJob.key),
                interval: parseInt(mJob.value)
            };
        });
    }
    async getMicroservicesActiveJobs(microservice, job, identification) {
        let seconds = 3600;
        let nowMinusSeconds = Math.floor(Date.now() / 1000 - seconds);
        const activeJobs = await this.microservicesJobsStatusRepository.find({
            where: {
                microservice: microservice,
                job: job,
                identification: identification,
                status: (0, typeorm_2.In)([microserviceJob_constants_1.JOB_STATUS.unknown, microserviceJob_constants_1.JOB_STATUS.running]),
                start: (0, typeorm_2.MoreThan)(nowMinusSeconds)
            }
        });
        return activeJobs;
    }
    async getHostcfgByHwType() {
        let hostcfgs = await this.hostcfgRepository.find({
            where: {
                disabled: false,
                target: false
            }, relations: {
                hw_type_entity: true,
                destinations: {
                    _target_destination: true
                }
            },
        });
        let hostcfgsWithDestinations = await License_1.License.checkHostcfgLimits(hostcfgs);
        let hostcfgByHwType = {};
        for (const hostcfg of hostcfgsWithDestinations) {
            if (hostcfg.limited === false) {
                if (hostcfgByHwType[hostcfg.hw_type]) {
                    hostcfgByHwType[hostcfg.hw_type].push(hostcfg);
                }
                else {
                    hostcfgByHwType[hostcfg.hw_type] = [hostcfg];
                }
            }
        }
        return hostcfgByHwType;
    }
    async hostCfg(hw_type) {
        return await this.hostcfgRepository.findBy({ hw_type: hw_type, disabled: false });
    }
};
exports.MicroservicesTask = MicroservicesTask;
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_MINUTE),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], MicroservicesTask.prototype, "handleInterval", null);
exports.MicroservicesTask = MicroservicesTask = __decorate([
    (0, common_1.Injectable)(),
    __param(1, (0, typeorm_1.InjectRepository)(microservice_entity_1.Microservice)),
    __param(2, (0, typeorm_1.InjectRepository)(microserviceJobs_entity_1.MicroserviceJob)),
    __param(3, (0, typeorm_1.InjectRepository)(microserviceJobsStatus_entity_1.MicroserviceJobStatus)),
    __param(4, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __param(5, (0, typeorm_1.InjectRepository)(targetDestination_entity_1.TargetDestination)),
    __param(6, (0, typeorm_1.InjectRepository)(targetDestinationHostcfg_entity_1.TargetDestinationHostcfg)),
    __param(7, (0, typeorm_1.InjectRepository)(microserviceVariables_entity_1.MicroserviceVariables)),
    __metadata("design:paramtypes", [config_1.ConfigService,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        event_emitter_1.EventEmitter2])
], MicroservicesTask);
//# sourceMappingURL=microservices.service.js.map