#
# $Revision: 1.7.1 $

my ($revision) = ( '$Revision: 1.7.1 $' =~ /([\d\.]+)/ );

my $xormonNG = ( $ENV{XormonNG} ) ? $ENV{XormonNG} : 0;
if ( $xormonNG ) {
  #require FindBin;
  use FindBin;
  use lib "$FindBin::Bin/"; # use lib in this folder
  use lib "../../../perl-modules";
  require Xormon;
}

# Modules
use strict;
use Storable;
use Data::Dumper;
use Date::Parse;
use HP3PAR 1.7.1;

# Options and their default value
my $storname;    # HP3PAR storage alias
my $storip;      # HP3PAR host name (IP address)
my $storuser     = "info";    # HP3PAR user name
my $storkey      = "";        # HP3PAR ssh key file name
my $interval     = 1;         # interval time for data collection
my $mininterval  = 300;       # minimal interval time for data collection (sec)
my $vvolinterval = 0;         # interval time for vvol performance data collection
my $dir          = "..";      # Data directory
my $putty        = 0;         # 1 ... run on Windows

my $debug      = 0;           # Debug mode 0:off, 1:on
my $debug_full = 0;           # Debug mode 0:off, 1:on

my $timeout;
my $webdir;

my $bindir   = $ENV{BINDIR};

if ( defined $ENV{WEBDIR} ) {
  $webdir = $ENV{WEBDIR};
}
else {
  print("hp3parperf.pl: WEBDIR environment variable is required.\n");
  exit(0);
}
if ( defined $ENV{STORAGE_NAME} ) {
  $storname = $ENV{STORAGE_NAME};
}
else {
  print("hp3parperf.pl: HP3PAR storage name alias is required.\n");
  exit(0);
}
if ( defined $ENV{'HP3PAR_IP'} ) {
  $storip = $ENV{'HP3PAR_IP'};
}
else {
  print("hp3parperf.pl: HP3PAR host name or IP address is required.\n");
  exit(0);
}

if ( defined $ENV{'HP3PAR_USER'} )           { $storuser     = $ENV{'HP3PAR_USER'}; }
if ( defined $ENV{'HP3PAR_KEYFILE'} )        { $storkey      = $ENV{'HP3PAR_KEYFILE'}; }
if ( defined $ENV{'HP3PAR_DIR'} )            { $dir          = $ENV{'HP3PAR_DIR'}; }
if ( defined $ENV{'SAMPLE_RATE'} )           { $interval     = $ENV{'SAMPLE_RATE'} }
if ( defined $ENV{'SAMPLE_RATE'} )           { $timeout      = $ENV{'SAMPLE_RATE'} }
if ( defined $ENV{'HP3PAR_VV_SAMPLE_RATE'} ) { $vvolinterval = $ENV{'HP3PAR_VV_SAMPLE_RATE'} }
if ( defined $ENV{'PUTTY'} )                 { $putty        = $ENV{'PUTTY'} }
if ( defined $ENV{DEBUG} )                   { $debug        = $ENV{DEBUG} }
if ( defined $ENV{DEBUG_FULL} )              { $debug_full   = $ENV{DEBUG_FULL} }

# Main process

# Display parameters
if ($debug) {
  print( "
hp3parperf.pl starts with the following parameters ...
\tRevision:                             $revision
\tHP3PAR alias:                         $storname
\tHP3PAR host name or IP:               $storip
\tHP3PAR admin user name:               $storuser
\tHP3PAR admin user SSH key filename:   $storkey
\tInterval time for data collection:    $interval
\tBase Directory for output files:      $dir
\tDebug:                                $debug
\n" );
} ## end if ($debug)

my $tmpdir = "$dir/tmp/";    # Directory for temporary files
if ( not -d $tmpdir ) {
  mkdir($tmpdir) or die("ERROR: Cannot create directory $tmpdir. Exitting.");
}

my $out_perf_file;           #
my $out_conf_file;           #

my $ssh;
my $ssh_tt;
my $cmd_ls       = "/bin/ls";
my $cmd_mv       = "/bin/mv";
my $cmd_rm       = "/bin/rm -f";
my $cmd_grep     = "/bin/grep";
my $cmd_zip      = "/bin/pkzip";
my $ssh_cmd      = "";
my $act_dir      = $ENV{'ACT_DIR'};
my $hp3parclidir = ""; #ENV
if ( defined $ENV{'HPE3PAR_CLIDIR'} )           { $hp3parclidir     = $ENV{'HPE3PAR_CLIDIR'}; }

if ( -f "$hp3parclidir/cli" && -f "$act_dir/etc/$storip/.3par-$storip.pwf" ) {
  #print "START CLIII PERF\n";
  # if HPE 3PAR CLI is installed then use that to avoid a problem coming in 3.2.2MU4 when IP can be blacklisted if many ssh session in certain time frame
  # password file has to be created at first
  $ENV{TPDPWFILE}  = "$act_dir/etc/$storip/.3par-$storip.pwf";                                       # use rather TPDPWFILE env var istead of cmd line -pwf
  $ENV{TPDCERTDIR} = "$act_dir/etc/$storip";                                                         # there is saved certificate file in "excp"
  $ssh             = "$hp3parclidir/cli -pwf $act_dir/etc/$storip/.3par-$storip.pwf -sys $storip";
  $ssh_tt          = $ssh;
} ## end if ( -f "$hp3parclidir/cli"...)
else {
  # using ssh way
  $ssh_cmd = $ENV{SSH};
  if ( !defined($ssh_cmd) || $ssh_cmd eq '' ) {
    $ssh_cmd = "ssh -o ConnectTimeout=80 -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey -o SendEnv=no ";
  }
  if ( defined($storkey) && !$storkey eq '' ) {
    $ssh    = "$ssh_cmd -i $storkey $storuser\@$storip";
    $ssh_tt = "$ssh_cmd -tT -i $storkey $storuser\@$storip";
  }
  else {
    # with default keyfile (.ssh/id_rsa)
    $ssh    = "$ssh_cmd $storuser\@$storip";
    $ssh_tt = $ssh;
  }
} ## end else [ if ( -f "$hp3parclidir/cli"...)]

if ($putty) {

  # just for dev purposes
  $tmpdir   = "$dir\\tmp\\";               # Directory for temporary files
  $cmd_ls   = "c:\\unix\\ls";
  $cmd_mv   = "c:\\unix\\mv";
  $cmd_rm   = "del";
  $cmd_grep = "c:\\unix\\grep";
  $cmd_zip  = "c:\\Progra~1\\7-Zip\\7z";
  my $plink = "c:\\Progra~2\\PuTTY\\plink.exe";
  $ssh = $plink . " -i $storkey $storuser\@$storip";
} ## end if ($putty)

sub message {
  my $msg = shift;
  my $tm  = localtime();
  print( $tm. " INFO    :" . " hp3parperf.pl: " . $msg . "\n" ) if ($debug);
}

sub warning {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  print STDERR ( $tm . " WARNING :" . " hp3parperf.pl: " . $msg . "\n" );
}

sub error {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  if ( defined $rc ) {
    print STDERR ( $tm . " ERROR   :" . " hp3parperf.pl: " . $msg . " - return code: " . $rc . "\n" );
  }
  else {
    print STDERR ( $tm . " ERROR   :" . " hp3parperf.pl: " . $msg . "\n" );
  }

} ## end sub error

# Get Storage release version
my $tmp_version_file = $tmpdir . $storname . ".showversion.txt";
if ( not -r $tmp_version_file ) {
  &run_showversion($tmp_version_file);
}
my $fwversion = &HP3PAR::get_version($tmp_version_file);
if ( not defined $fwversion ) {
  error( "Cannot get storage release version.", 199 );
  exit 199;
}
&message( "Storage release version: " . $fwversion ) if ($debug);
my @servers  = qw(showcfg statcpu statpd statvv statvlun statport statcmp);    #
my $time_str = &localtime_str;
##### Off-line #######
# my $time_str = "210117_000000";
# &message("Process off-line files.");
# my $tm = localtime(); printf("\n%s INFO    : hp3parperf.pl : Process off-line files [%s].\n",$tm,$time_str);
# &HP3PAR::process_files(\@servers,$dir,$storname,$time_str,$interval,$vvolinterval,$debug);
# exit;
######################

if ( $debug == 1 ) { my $tm = localtime(); printf( "%s INFO    : hp3parperf.pl : Revision: %s\n\tServers: ", $tm, $revision ); }
my @children = {};
foreach my $server (@servers) {
  my $pid;
  next if $pid = fork;    # Parent goes to next server.
  unless ( defined $pid ) { error( "Fork failed: $!", 101 ); exit 101; }

  # From here on, we're in the child. The server we want to deal
  # with is in $server.
  if ( $debug == 0 ) { printf( "%s ", $server ); }
  &childsrv($server);
} ## end foreach my $server (@servers)

# parent waits until all child processes have finished
&message("Parent ($$)");
if ( not defined($timeout) or $timeout eq '' ) {
  $timeout = 900; # previous default 360 was not enough, changed in 2.30 to 600
}
else {
  $timeout = $timeout + 600; # same as above, originally there was 60
}

# set alarm on first SSH command to make sure it does not hang
eval {
  local $SIG{ALRM} = sub { &error( "Parent died in SIG ALRM", 102 ); kill( 'TERM', "-" . $$ ); exit 102; };
  alarm($timeout);

  # Waiting for childs ends.
  while (1) {
    my $pid_found = wait();
    my $rc        = $?;
    if ( $pid_found == -1 ) { last; }
    my $msg = sprintf( "Child (%s) exited with code %s (%x) = %d", $pid_found, $rc, $rc, $rc >> 8 );
    if ( $rc == 0 ) {
      &message($msg);
    }
    else {
      &error($msg);
    }
    push( @children, $pid_found );
  } ## end while (1)

  # end of alarm
  alarm(0);
};

&message("All childern done! Process files.");
if ( $debug == 0 ) { my $tm = localtime(); printf( "\n%s INFO    : hp3parperf.pl : Process files [%s].\n", $tm, $time_str ); }
&HP3PAR::process_files( \@servers, $dir, $storname, $time_str, $interval, $vvolinterval, $debug );
&HP3PAR::set_global_health_status_summary( $storname, $time_str );
&message("Delete files.");
&deletefiles;
&message("Done.");

exit 0;

sub childsrv {
  my $name = shift;
  if ( $name eq "showcfg" )  { sleep 15; &server_showcfg; }
  if ( $name eq "statcpu" )  { &server_statcpu; }
  if ( $name eq "statld" )   { sleep 2;  &server_statld; }
  if ( $name eq "statvv" )   { sleep 4;  &server_statvv; }
  if ( $name eq "statvlun" ) { sleep 6;  &server_statvlun; }
  if ( $name eq "statport" ) { sleep 8;  &server_statport; }
  if ( $name eq "statpd" )   { sleep 10; &server_statpd; }
  if ( $name eq "statcmp" )  { sleep 12; &server_statcmp; }
} ## end sub childsrv

sub server_showcfg {
  my $scr;
  my @a = split( /\./, $fwversion );
  my $codevrmn = $a[0] * 10000 + $a[1] * 100 + $a[2];
  if ( $codevrmn >= 30000 ) {
    $scr = "
showsys
showsys -space
showversion
showhost -d
showcpg
showld
showld -d
showld -state
showvv -showcols Id,Name,Prov,Type,CopyOf,BsId,Rd,State,Adm_Used_MB,Snp_Used_MB,Usr_Used_MB,VSize_MB,Detailed_State,VV_WWN
showvvcpg
showvlun
showport
showpd
showrcopy -d
showcage -d
";
  } ## end if ( $codevrmn >= 30000)
  else {
    $scr = "
showsys
showsys -space
showversion
showhost -d
showcpg
showld
showld -d
showld -state
showvv -showcols Id,Name,Prov,Type,CopyOf,BsId,Rd,State,Adm_Used_MB,Snp_Used_MB,Usr_Used_MB,VSize_MB,Detailed_State,VV_WWN
showvlun
showport
showpd
showrcopy -d
";
  } ## end else [ if ( $codevrmn >= 30000)]
  #
  if ( not defined $timeout or $timeout eq '' ) {
    $timeout = 360;
  }
  else {
    $timeout = $timeout + 60;
  }

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub { &error( "Child server showcfg died in SIG ALRM", 103 ); kill( 'TERM', "-" . $$ ); exit 103; };

    alarm($timeout);

    # cmd
    my $filename = $tmpdir . $storname . ".showcfg." . $time_str . ".txt";
    #
    my $command;
    $command = "echo \"" . $scr . "\"|" . $ssh_tt;

    message( "\tChild server showcfg ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command showcfg failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;
    #
    # end of alarm
    alarm(0);
  };
  exit 0;    # Ends the child process.
} ## end sub server_showcfg

sub server_statcpu {
  if ( not defined $timeout or $timeout eq '' ) {
    $timeout = 360;
  }
  else {
    $timeout = $timeout + 60;
  }

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub { &error( "Child server statcpu died in SIG ALRM", 103 ); kill( 'TERM', "-" . $$ ); exit 103; };
    alarm($timeout);

    # cmd
    my $filename = $tmpdir . $storname . ".statcpu." . $time_str . ".txt";
    my $command  = $ssh . " statcpu -d " . $interval . " -iter 1";
    message( "\tChild server statcpu ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command statcpu failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;
    #
    # end of alarm
    alarm(0);
  };

  exit 0;    # Ends the child process.
} ## end sub server_statcpu

sub server_statld {
  if ( not defined $timeout or $timeout eq '' ) {
    $timeout = 360;
  }
  else {
    $timeout = $timeout + 60;
  }

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub { &error( "\tChild server statld died in SIG ALRM", 104 ); kill( 'TERM', "-" . $$ ); exit 104; };
    alarm($timeout);

    # cmd
    my $filename = $tmpdir . $storname . ".statld." . $time_str . ".txt";
    my $command  = $ssh . " statld -rw -d " . $interval . " -iter 1";
    message( "\tChild server statld ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command statld failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;

    # end of alarm
    alarm(0);
  };

  exit 0;    # Ends the child process.
} ## end sub server_statld

sub server_statport {
  if ( not defined $timeout or $timeout eq '' ) {
    $timeout = 360;
  }
  else {
    $timeout = $timeout + 60;
  }

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub { &error( "\tChild server statport died in SIG ALRM", 104 ); kill( 'TERM', "-" . $$ ); exit 104; };
    alarm($timeout);

    # cmd
    my $filename = $tmpdir . $storname . ".statport." . $time_str . ".txt";
    my $command  = $ssh . " statport -rw -d " . $interval . " -iter 1";
    message( "\tChild server statport ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command statport failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;

    # end of alarm
    alarm(0);
  };

  exit 0;    # Ends the child process.
} ## end sub server_statport

sub server_statpd {
  if ( not defined $timeout or $timeout eq '' ) {
    $timeout = 360;
  }
  else {
    $timeout = $timeout + 60;
  }

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub { &error( "\tChild server statpd died in SIG ALRM", 104 ); kill( 'TERM', "-" . $$ ); exit 104; };
    alarm($timeout);

    # cmd
    my $filename = $tmpdir . $storname . ".statpd." . $time_str . ".txt";
    my $command  = $ssh . " statpd -rw -d " . $interval . " -iter 1";
    message( "\tChild server statpd ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command statpd failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;

    # end of alarm
    alarm(0);
  };

  exit 0;    # Ends the child process.
} ## end sub server_statpd

sub server_statvv {
  if ( not defined $timeout or $timeout eq '' ) {
    $timeout = 360;
  }
  else {
    $timeout = $timeout + 60;
  }

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub { &error( "\tChild server statvv died in SIG ALRM", 104 ); kill( 'TERM', "-" . $$ ); exit 104; };
    alarm($timeout);
    my $iter = 1;
    if ( $vvolinterval > 0 ) {
      use integer;
      $iter = $interval / $vvolinterval;
      no integer;
    }

    # cmd
    my $command;
    my $filename = $tmpdir . $storname . ".statvv." . $time_str . ".txt";
    if ( $iter > 1 ) {
      $command = $ssh . " statvv -rw -d " . $vvolinterval . " -iter " . $iter;
    }
    else {
      $command = $ssh . " statvv -rw -d " . $interval . " -iter 1";
    }
    message( "\tChild server statvv ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command statvv failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;

    # end of alarm
    alarm(0);
  };

  exit 0;    # Ends the child process.
} ## end sub server_statvv

sub server_statvlun {

  # Timeout must be +300secs, it timeouted when only 60sec on big storages with many luns
  if ( not defined $timeout or $timeout eq '' ) {
    $timeout = 600;
  }
  else {
    $timeout = $timeout + 300;
  }

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub { &error( "\tChild server statvlun died in SIG ALRM", 104 ); kill( 'TERM', "-" . $$ ); exit 104; };
    alarm($timeout);

    # cmd
    my $command;
    my $filename = $tmpdir . $storname . ".statvlun." . $time_str . ".txt";
    #$command = $ssh . " statvlun -ni -rw -d " . $interval . " -iter 1";  : -ni: not null
    $command = $ssh . " statvlun -rw -d " . $interval . " -iter 1";
    message( "\tChild server statvlun ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command statvlun failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;

    # end of alarm
    alarm(0);
  };

  exit 0;    # Ends the child process.
} ## end sub server_statvlun

sub server_statcmp {
  if ( not defined $timeout or $timeout eq '' ) {
    $timeout = 360;
  }
  else {
    $timeout = $timeout + 60;
  }

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub { &error( "\tChild server statcmp died in SIG ALRM", 104 ); kill( 'TERM', "-" . $$ ); exit 104; };
    alarm($timeout);

    # cmd
    my $command;
    my $filename = $tmpdir . $storname . ".statcmp." . $time_str . ".txt";
    $command = $ssh . " statcmp -d 1 -iter 1";
    message( "\tChild server statcmp ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command statcmp failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;

    # end of alarm
    alarm(0);
  };
  eval {
    local $SIG{ALRM} = sub { &error( "\tChild server statcmp died in SIG ALRM", 104 ); kill( 'TERM', "-" . $$ ); exit 104; };
    alarm($timeout);

    # cmd
    my $command;
    my $filename = $tmpdir . $storname . ".statcmp." . $time_str . ".txt";
    $command = $ssh . " statcmp -d " . $interval . " -iter 1";
    message( "\tChild server statcmp ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command statcmp failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;

    # end of alarm
    alarm(0);
  };

  exit 0;    # Ends the child process.
} ## end sub server_statcmp

sub run_showversion {
  my $filename = shift;
  if ( not defined $timeout or $timeout eq '' ) {
    $timeout = 360;
  }
  else {
    $timeout = $timeout + 60;
  }

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub { &error( "Command showversion died in SIG ALRM", 103 ); kill( 'TERM', "-" . $$ ); exit 103; };
    alarm($timeout);

    # cmd
    #my $filename = $tmpdir.$storname.".showversion.txt";
    my $command = $ssh . " showversion";
    message( "\tCommand showversion ($$) - output: " . $filename );
    my ( $exit, $stdout ) = &execcmd($command);
    if ( $exit != 0 ) { error( "Command showversion failed with code " . $exit ); exit 150; }
    open( my $tmpfile, '>', $filename ) || error( "open file failed: $filename :  $!", 111 ) && exit 111;
    foreach my $ln ( @{$stdout} ) {
      print $tmpfile $ln;
    }
    close $tmpfile;
    #
    # end of alarm
    alarm(0);
  };
  return 0;
} ## end sub run_showversion

sub execcmd {
  my $cmd = "@_";
  my $ret_value;
  my $msg;
  my $rc = 0;
  my @out;
  #
  if ($debug) { message("Execute: @_"); }
  ( @out = `@_` );
  if ( $debug == 22 ) {    # --PH
    foreach my $l (@out) {
      print "$l\n";
    }
  }
  $ret_value = $?;
  if ( $ret_value == -1 ) {
    $msg = sprintf "Failed to execute: $!\n";
    if ($debug) { message($msg); }
    $rc = -1;
  }
  elsif ( $ret_value & 127 ) {
    $msg = sprintf "Command \'%s\' died with signal %d\n", $cmd, ( $ret_value & 127 );
    if ($debug) { message($msg); }
    $rc = ( $ret_value & 127 );
  }
  else {
    $msg = sprintf "Command \'%s\' exited with value %d\n", $cmd, $ret_value >> 8;
    if ($debug) { message($msg); }
    $rc = ( $ret_value >> 8 );
  }
  return ( $rc, \@out );
} ## end sub execcmd

sub localtime_str {

  # OUT: 150425_095000
  my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = localtime();
  my $y   = $year - 100;
  my $m   = $mon + 1;
  my $str = sprintf( "%02d%02d%02d_%02d%02d%02d", $y, $m, $mday, $hour, $min, $sec );
  return ($str);
} ## end sub localtime_str

sub deletefiles {
  my $command = $cmd_rm . " " . $tmpdir . "*" . $time_str . ".txt";
  my ( $exit, $stdout ) = &execcmd($command);
}
