"""

"""
import numpy as np
import pandas as pd

import math
import statistics

import os 

# TODO: add ACF, PACF
# TODO: remove matplotlib
#--------------------------------
# SYSTEM
#--------------------------------
def give_path():
    dir_path = os.path.dirname(os.path.realpath(__file__))
    return dir_path

def namestr(obj, namespace):
    # Returns in-code name of variable
    namespace = globals()
    return [name for name in namespace if namespace[name] is obj]

def remove_occurence(arra, x):
    return list(filter(lambda a: a != x, arra))

#--------------------------------
# TIMESERIES METHODS
#--------------------------------
def differentiate_series(arra):
    diff = []
    diff.append(arra[0])
    for i in range(1, len(arra)):
        diff.append(arra[i] - arra[i-1])
    return diff


def integrate_series(arra):
    # in TS: integration = cumulative
    integ = []
    integ.append(arra[0])
    for i in range(1, len(arra)):
        integ.append(integ[i-1] + arra[i])
    return integ


def correlogram(series):
    print("Starting correlogram")

    correlogram = np.zeros_like(series)
    mean = np.mean(series)
    for lag in range(series.size):

        cor_lag = 0
        for index_now in range(series.size - lag):
            cor_lag += ( series[ index_now ] - mean ) * ( series[ index_now + lag ] - mean )

        correlogram[lag] = cor_lag

    return correlogram


def get_upper_lower_std(model, data):
    greater_mask = np.greater(data, model) # x1>x2
    smaller_mask = np.greater_equal(model, data)

    uppers = np.std(data(greater_mask), model(greater_mask))
    lowers = np.std(data(smaller_mask), model(smaller_mask))

    return uppers, lowers


def make_trendline(series):
    n = np.array(series).size
    x = np.arange(0, n)
    polynomial = np.polyfit(x, np.array(series), 1)
    trendline = polynomial[0] * x

    shift_mean_interval = np.mean(series) - np.mean(trendline)
    trendline = np.add(trendline, shift_mean_interval)

    return trendline


def continuous_concatenation(left_series, right_series):

    series = np.concatenate((left_series, np.add(right_series, left_series[-1] - right_series[0] )))

    return series


def make_histogram(arra, container = 1):
    # TODO: use hist function
    hist = []
    index = []

    range_max = int(max(arra)/container)+3
    range_min = int(min(arra)/container)-3

    print(range_min, range_max)

    for i in range(- range_min + range_max):
        hist.append(0)
        index.append(i+range_min)
        for j, val in enumerate(arra):
            if ( container*(i+range_min) <= val < container*(i+1 + range_min)):
                hist[i] += 1

    return pd.DataFrame(hist, index = index)


def overlapping_samples(input_series, interval):

    interval = int(interval/2)

    input_series = np.array(input_series)

    split_points = np.arange(0,int(interval*(len(input_series)/interval)),interval)

    #print(split_points)

    splits_a = np.split(input_series, split_points[::2])[1:-1]
    splits_b = np.split(input_series, split_points[1::2])[1:-1]

    splits = []

    # Use concat instead
    for split in splits_a:
        splits.append(split)
    for split in splits_b:
        splits.append(split)

    return splits


def show_stats(arra, name):
    # TODO: use prepared lib functions/add as description wrapper to Series object
    print("\nARRAY STATS: \n--------------------") 
    print("NAME: ", name)
    print("MAX: ", max(arra), "MIN: ", min(arra))
    print("LENGTH: ", len(arra))
    print("SUM OF ALL VALUES: ", sum(arra))

    mean = np.mean(arra)
    variance = np.var(arra)
    sigma = np.std(arra)

    print ("Mean: ", mean)
    print ("Variance: ", variance)
    print ("Sigma: ", sigma)
    print("--------------------\n") 


"""
#--------------------------------
# VISUALISATION
#--------------------------------
# PLOT MULTIPLE ARRAYS IN ONE PICTURE
def plot_arrays(arra_all, names_all):
    amount_of_arrays = len(arra_all)
    for i, arra in enumerate(arra_all):
        plt.subplot(amount_of_arrays, 1, i+1)
        plt.plot(arra, color="blue", label=names_all[i])
        plt.legend()
    plt.show()

def figure_arrays(arra_all, names_all):
    dir_path = os.path.dirname(os.path.realpath(__file__))
    amount_of_arrays = len(arra_all)
    for i, arra in enumerate(arra_all):
        plt.subplot(amount_of_arrays, 1, i+1)
        plt.plot(arra, color="blue", label=names_all[i])
        plt.legend()
    id = str(np.random.randint(100))
    piclabel = "last_1_plot.png"
    pic_path = dir_path + '/../pictures/explorer/' + piclabel
    plt.savefig(pic_path)

def plot_group(arrays, names):
    amount_of_arrays = 0
    for i in range(len(arrays)):
        amount_of_arrays += len(arrays[i])
    print(amount_of_arrays)
    for i, arra in enumerate(arrays):
        for j, element in enumerate(arra):
            plt.subplot(amount_of_arrays, j+1, i+1)
            plt.plot(element, color="blue", label=names[i][j])
            print(j+1, i+1, names[i][j])
            plt.legend()
    plt.show()
"""

# ARRAY DOMAIN FIELD
def get_image_set(arra):
    def_set = []
    for element in arra:
        if element not in def_set:
            def_set.append(element)
    return(def_set)

def average_smoother(arra, n):
    for i, arra in enumerate(arra):
        pass
"""

def plot_band(centrals, sigmas_upper, sigmas_lower):
    
    upper = np.add(centrals, sigmas_upper)
    downer = np.subtract(centrals, sigmas_lower)

    n = np.array(centrals).size

    plt.fill_between(np.arange(0,n,1), upper, downer, color='grey', alpha=.4)

    sigmas_n_upper = np.multiply(sigmas_upper, 2)
    sigmas_n_lower = np.multiply(sigmas_lower, 2)
    
    upper = np.add(centrals, sigmas_n_upper)
    downer = np.subtract(centrals, sigmas_n_lower)

    plt.fill_between(np.arange(0,n,1), upper, downer, color='grey', alpha=.3)

    sigmas_n_upper = np.multiply(sigmas_upper, 3)
    sigmas_n_lower = np.multiply(sigmas_lower, 3)
    
    upper = np.add(centrals, sigmas_n_upper)
    downer = np.subtract(centrals, sigmas_n_lower)
    
    plt.fill_between(np.arange(0,n,1), upper, downer, color='grey', alpha=.2)
"""
