"""
TODO: uncomputable response (no data)

Low priority:
TODO: add some quality metric (take into account prediction length)
TODO: add custom percentage to max from env
TODO: rework to use times to less/full as default instead of time to 70/90/100...

"""
from utility.logger import log


multipliers = {
    "1m":  int(24*60),
    "5m":  int(24*12),
    "1h":  int(24),
    "6h":  int(4),
    "1d":  int(1)
}

# principle: if name exists, then rename to default and finish renaming
# TODO: quantitative selection criteria
model_expected_order = [
    'reactive-robust_weekly_extrapolation', # more than 14 days
    'reactive-robust_ext_daily_averages', # reasonable choice for "point" prediction
    'long_ext_daily_averages', # reasonable choice for band prediction
    # rest is kept for context
    'short_ext_daily_averages',
    'robust_ext_daily_averages',
    'robust_weekly_extrapolation',
]

def times_to_fill(prediction):
    """
    Takes computed prediction object and finds the worst case model.
    """
    models_to_use = prediction.keys()

    worse_less = 365
    worse_full  = 365

    for model_name in models_to_use:

        if ("days_to_90_percent_fill" in prediction[model_name].keys() and
            "days_to_100_percent_fill" in prediction[model_name].keys()):

            local_less = prediction[model_name]["days_to_90_percent_fill"]
            local_full = prediction[model_name]["days_to_100_percent_fill"]

            local_less = int(local_less)
            local_full = int(local_full)

            if worse_less > local_less:
                worse_less = local_less

            if worse_full > local_full:
                worse_full = local_full

        else:
            log(f"{model_name} - times to fill are not ready")

    #log(f"FINAL: {worse_less} {worse_full}")

    return worse_less, worse_full


def times_to_fill_general(prediction):
    """
    Takes computed prediction object and finds the worst case model.
    """
    models_to_use = prediction.keys()

    worse_less = 365
    worse_full  = 365

    for model_name in models_to_use:

        if ("days_to_less" in prediction[model_name].keys() and
            "days_to_full" in prediction[model_name].keys()):

            local_less = prediction[model_name]["days_to_less"]
            local_full = prediction[model_name]["days_to_full"]

            local_less = int(local_less)
            local_full = int(local_full)

            if worse_less > local_less:
                worse_less = local_less

            if worse_full > local_full:
                worse_full = local_full
        else:
            log(f"{model_name} - times to fill are not ready")

    return worse_less, worse_full


def postprocessing_for_capacity(prediction, maximal_value):
    """
        "days_to_90_percent_fill": num,
        "error_days_to_90_percent_fill": num,
        "days_to_100_percent_fill": num,
        "error_days_to_100_percent_fill": num,
    """
    log('')

    maximal_value = float(maximal_value)
    for key in prediction.keys():

        start_shift = 0

        if "start_steps" in prediction[key].keys():
            start_shift = prediction[key]["start_steps"]
        else:
            start_shift = 0

        prediction_data = prediction[key]["prediction"][start_shift:]

        uncomputed_100 = True
        uncomputed_90  = True

        computation_done = False

        # Expectation: key exists => prediction exists
        prediction[key]["days_to_100_percent_fill"] = 365
        prediction[key]["days_to_90_percent_fill"] = 365

        for step, value in enumerate(prediction_data):

            computation_done = True

            value = float(value)

            if ( uncomputed_100 and value >= maximal_value):
                prediction[key]["days_to_100_percent_fill"] = step
                uncomputed_100 = False
            if ( uncomputed_90 and value >= (maximal_value*0.9)):
                prediction[key]["days_to_90_percent_fill"] = step
                uncomputed_90 = False

        #if not computation_done:
        #    prediction[key]["days_to_100_percent_fill"] = ""
        #    prediction[key]["days_to_90_percent_fill"] = ""

    return prediction


def postprocessing_for_cpu(prediction, maximal_value, frequency):
    """
        "days_to_70_percent_fill": num,
        "error_days_to_70_percent_fill": num,
        "days_to_100_percent_fill": num,
        "error_days_to_100_percent_fill": num,
        TODO: add some quality metric (take into account prediction length)
              here? computed in-sample error -> add computation of forecasting error
    """
    log('')
    # TODO: REMOVE or use properly
    maximal_value = float(maximal_value)

    for key in prediction.keys():
        start_shift = 0

        if "start_steps" in prediction[key].keys():
            start_shift = prediction[key]["start_steps"]
        else:
            start_shift = 0

        prediction_data = prediction[key]["prediction"][start_shift:]

        uncomputed_100 = True
        uncomputed_70  = True

        # Expectation: key exists => prediction exists
        prediction[key]["days_to_100_percent_fill"] = 365
        prediction[key]["days_to_70_percent_fill"] = 365

        for step, value in enumerate(prediction_data):
            value = float(value)

            if ( uncomputed_100 and value >= maximal_value):
                prediction[key]["days_to_100_percent_fill"] = step / multipliers[frequency]
                uncomputed_100 = False
            if ( uncomputed_70 and value >= (maximal_value*0.7)):
                prediction[key]["days_to_70_percent_fill"] = step / multipliers[frequency]
                uncomputed_70 = False

    # default display
    for preference_name in model_expected_order:
        if preference_name in prediction.keys():
            prediction[preference_name]["display"] = True
            break

    return prediction


def postprocess_prediction_response(prediction, maximal_value, frequency, less_percentage=70):
    """
        General postprocess: creates 2 additional keys - time to less/full
        TODO: rework to use as default
        "days_to_less": num,
        "error_days_to_less": num,
        "days_to_full": num,
        "error_days_to_full": num,
        TODO: add some quality metric (take into account prediction length)
              here? computed in-sample error -> add computation of forecasting error
    """
    log('')
    # TODO: REMOVE or use properly
    maximal_value = float(maximal_value)

    less_coeff = less_percentage/100

    for key in prediction.keys():
        start_shift = 0

        if "start_steps" in prediction[key].keys():
            start_shift = prediction[key]["start_steps"]
        else:
            start_shift = 0

        prediction_data = prediction[key]["prediction"][start_shift:]

        full_computed = False
        less_computed = False

        # Expectation: key exists => prediction exists
        prediction[key]["days_to_full"] = 365
        prediction[key]["days_to_less"] = 365

        for step, value in enumerate(prediction_data):
            value = float(value)

            if ( not full_computed and value >= maximal_value):
                prediction[key]["days_to_full"] = step / multipliers[frequency]
                real_val = step / multipliers[frequency]
                full_computed = True

            if ( not less_computed and value >= (maximal_value*less_coeff)):
                prediction[key]["days_to_less"] = step / multipliers[frequency]
                real_val = step / multipliers[frequency]
                less_computed = True

    return prediction
