package COMPELLENT;

use strict;
use warnings;

use Data::Dumper;
use JSON qw(encode_json decode_json);
use Date::Parse;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Xormon;
use POSIX ":sys_wait_h";
use LWP::UserAgent;
use File::Copy;
use File::Temp qw(tempfile);
use Test;
use XormonLogger;
my $logger;

my %data;
my $xormon;
my $cfg;
my $stats_found = 0;
my $api_version = "";
my $urls;
my $data_directory = "$FindBin::Bin";
my $debug = 0;

my $storage_name = "";
my $instance_name = "";
my $active_minute = "";
my $debug_data = "";
my %performance_data;
my %configuration_data;
my $st_serial;
my $request_count;
my @architecture = ();
my $service_data_out;
my $st_type = "compellent";
my @rows_api_test = ();
my @rows_api_test_data = ();
my $protocol = "";
my $host_api = "";
my $wrkdir = "";
my %inventory_performance;
my %inventory_configuration;
my %header_section;
my $service_data_in;
my $connection_test = 0;
my $cookie = "";
my $instance_storage = "";
my %volume_inventory;
my @health_status = ();
my @hs_global = ();




sub collect {
  $xormon = shift;
  $cfg    = shift;

  #return 1;

#'sampleRate' => 5,
#'hostcfg_id' => '5efc41ac-67ce-44bb-9d2d-4a33af89e22d',
#'device' => 'HPE__MSA_P2K',
#'password' => 'IBM4you.',
#'username' => 'stor2rrd',
#'proto' => 'https',
#'node1Host' => '10.22.99.18',
#'created' => '1652272990955',
#'node2Host' => '10.22.99.19',
#'hostalias' => 'MSA01-PHA'
  #print Dumper \$cfg;
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!") && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!") && return "error"; }
  $logger = XormonLogger->new( 'compellent', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )       { $logger->error("Cannot continue, 'apiHost' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPort'} )       { $logger->error("Cannot continue, 'apiPort' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )   { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )   { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  unless ( exists $cfg->{'storageName'} )   { $logger->error("Cannot continue, 'storageName' is not configured!") && return "error"; }
  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  my $act_timestamp     = time();
  $service_data_in = $xormon->getData();
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }
  $storage_name = $cfg->{hostalias};
  $instance_name = $cfg->{storageName};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  $protocol = "https";
  data_structure();
  #print Dumper \$service_data_in;
  if (defined $service_data_in->{api_version} && $service_data_in->{api_version} ne "" && !(( ! exists $service_data_in->{last_conf_timestamp} ) || ( ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) || ( $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) ) ){
    $api_version = $service_data_in->{api_version};
    $cookie = login_api($api_version,1);
    if (!defined $cookie || $cookie eq ""){
      if ( exists $service_data_in->{last_perf_timestamp} ) {
        $service_data_out->{last_perf_timestamp} = $service_data_in->{last_perf_timestamp};
      }
      $logger->error("Cookie is not defined");
      $xormon->isConnected(0, "Connection failed!");
      my $save_data = $xormon->saveData(\%{ $service_data_out });
      my $status_save_data = get_status($save_data, "SaveData");
      $logger->info("Task for ".$cfg->{hostalias}." is done. Status: error");

      #return 0;
      return "error";
    }
  }
  else{
    for ( my $i = 1; $i < 11; $i++ ) {
      for ( my $j = 0; $j < 11; $j++ ) {
        $api_version = "$i.$j";
        if ($i == 10 && $j == 10){
          $cookie = login_api($api_version,1);
        }
        else{
          $cookie = login_api($api_version,0);
          if ($cookie ne ""){
            last;
          }
        }
      }
      if ($cookie ne ""){last;}
    }
    if (!defined $cookie || $cookie eq ""){

      $logger->error("Cookie is not defined");
      $xormon->isConnected(0, "Connection failed!");
      $logger->info("Task for ".$cfg->{hostalias}." is done. Status: error");
      return "error";
      #return 0;
    }
  }
  $xormon->isConnected(1);
  $instance_storage = get_instance_id();
  if ( !defined $instance_storage || $instance_storage eq "" ) {
    $logger->error("ERROR: Instance Storage \"$instance_name\" is not defined!");
    $logger->error("ERROR: Storage cannot be accessed, check if user \"$cfg->{apiUsername}\" has allowed storage \"$instance_name\"");
    logout_api();
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: error");
    return "error";
    #return 0;
  }
  #print "instance_storage is $instance_storage\n";
  set_data_port();
  set_data_volume();
  set_data_host();
  set_data_drive();
  set_data_node();
  set_data_volume_folder();
  set_data_pool();
  set_data_port_type();
  set_data_device();
  delete_empty_metrics();
  calculate_missing_metrics();
  #print Dumper \%configuration_data;
  #print Dumper \$performance_data{$st_type}{"pool"};

  #logout_api();
  #return 0;
  #print Dumper \%performance_data;
  #my $rc = 201;
  my $rc = $xormon->saveJSON2(\%performance_data);
  my $status = get_status($rc, "saveJSON2 (performance)");

  ### host_cfg_data
  if (Xormon::isdigit($service_data_out->{last_perf_timestamp})){
    if ( ( time() -  $service_data_out->{last_perf_timestamp} ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_out->{last_perf_timestamp});
    }
    else{
      $xormon->hasData(1);
    }
  }
  else{
    $xormon->hasData(0, "no available stats");
  }
  ###

  my $save_data = $xormon->saveData(\%{ $service_data_out });
  my $status_save_data = get_status($save_data, "SaveData");
  #$logger->info("Task SaveData for ".$cfg->{hostalias}." is done. Status: $status_save_data");
  # status code
  #$logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");


  #return 1;

  #### architecture

  #configuration
  if (( ! exists $service_data_in->{last_conf_timestamp} ) || ( ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) || ( $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) || ( !-f "$debug_data/config-$st_serial.out" )){
    `touch "$debug_data/config-$st_serial.out"`;
    $instance_storage = get_instance_id(1);
    if ( !defined $instance_storage || $instance_storage eq "" ) {
      $logger->error("ERROR: Instance Storage \"$instance_name\" is not defined!");
      $logger->error("ERROR: Storage cannot be accessed, check if user \"$cfg->{apiUsername}\" has allowed storage \"$instance_name\"");
      logout_api();
      #return 0;
      return "error";
    }
    $service_data_out->{instance_id} = $instance_storage;
    set_data_port_conf();
    set_data_volume_conf();
    set_data_host_conf();
    set_data_drive_conf();
    set_data_node_conf();
    set_data_volume_folder_conf();
    set_data_pool_conf();
    set_data_port_mapping();
    set_data_device_conf();
    #print Dumper \%configuration_data;


    my @types = ("pool", "volume", "fcport", "drive", "node", "host", "volume_folder", "port_mapping", "sasport", "iscsiport", "fcoeport");
    foreach my $uid (keys %configuration_data){
      my $active = 0;
      foreach my $type (@types){
        if (defined $performance_data{$st_type}{$type}{$uid}){
          $active++;
          last;
        }
        if ($type eq "port_mapping" && defined $configuration_data{$uid}{TYPE} && $configuration_data{$uid}{TYPE} eq $type){
          $active++;
          last;
        }
      }
      if ($active == 0){
        #print Dumper \$configuration_data{$uid};
        delete $configuration_data{$uid};
        #print "mazeme prvek $uid\n";
      }
    }
    #print Dumper \%configuration_data;
    #print Dumper $performance_data{$st_type}{pool};
    $service_data_out->{last_conf_timestamp} = $act_timestamp;
    ### health status
    set_data_health_status();
    ###
    ($status, my $status_configuration, my $status_health) = set_architecture();
    #$xormon->saveConfJSON(\%configuration_data);
    if ($status eq "successful" && $status_configuration eq "successful" && $status_health eq "successful" ){
      my $save_data = $xormon->saveData(\%{ $service_data_out });
      my $status_save_data = get_status($save_data, "saveData");
    }
    logout_api();
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
    return $status;
    #$logger->info("Task SaveData for ".$cfg->{hostalias}." is done. Status: $status_save_data");
  }
  logout_api();
  $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
  return $status;
}

sub set_data_health_status{
  my $cmd = "ScAlert";
  my $data = cmd_api($cmd,"GET",1,$cmd);
  my $message_global = "";
  my $status_global = "ok";

  if (!defined $data || $data eq ""){return;}
  foreach my $element ( @{$data} ) {
    my $instance = $element->{"instanceId"};
    if ( !defined $instance || $instance eq "" ) { next; }
    my $id_storage = $element->{"scSerialNumber"};
    if ( !defined $id_storage || $id_storage ne $instance_storage ) { next; }
    my $active = $element->{"active"};
    if ( Xormon::isdigit($active) && $active == 0 ) { next; }
    my $acknowledged = $element->{"acknowledged"};
    if ( Xormon::isdigit($acknowledged) && $acknowledged == 1 ) { next; }
    my $status = $element->{"alertStatus"};

    if ( !defined $status || $status eq "" ) {
      $status = $element->{"status"};
      if ( !defined $status || $status eq "" ) { next; }
    }
    my $date      = $element->{"createTime"};
    #my $test_time = str2time($date);
    if ( !defined $date || $date eq "" || ( Xormon::isdigit($date) && $date == 0 )) {
      $date      = $element->{"createdOn"};
      if ( !defined $date || $date eq "" || ( Xormon::isdigit($date) && $date == 0 )) {
        $date      = $element->{"modifiedOn"};
      }
    }
    my $utc_time = "N/A";
    if ( !defined $date || $date eq "" || ( Xormon::isdigit($date) && $date == 0)) {
      $date = "N/A";
    }
    else{
      $utc_time = get_utc_time($date);
    }
    if ( $status eq "Critical" ) {
      $status_global = "error";
      $message_global = $element->{"message"};
      if ( !defined $message_global )        { $message_global = ""; }
    }


    my $storage_center = $element->{"scName"};
    my $category       = $element->{"category"};
    my $message        = $element->{"message"};
    my $type           = $element->{"type"};
    my $object         = $element->{"scObjectName"};
    my $support        = $element->{"supportUrl"};
    if ( !defined $storage_center ) { $storage_center = ""; }
    if ( !defined $category )       { $category       = ""; }
    if ( !defined $message )        { $message        = ""; }
    if ( !defined $type )           { $type           = ""; }
    if ( !defined $object )         { $object         = ""; }
    if ( !defined $support )        { $support        = ""; }
    my %data;
    $data{severity} = $status;
    $data{date} = $utc_time;
    $data{storage_center} = $storage_center;
    $data{category} = $category;
    $data{object} = $object;
    $data{type} = $type;
    $data{message} = $message;
    $data{support_url} = $support;
    push(@health_status,\%data);
  }
  my %vm_status = (
    "item_id" => $st_serial,
    "status" => $status_global,
    "data" => $message_global
  );
  push(@hs_global, \%vm_status);
  #my $rc = $xormon->saveStatus(\@hs_global);
  #my $status = get_status($rc, "health status");
  #$logger->info("Task healh status ".$cfg->{hostalias}." is done. Status: $status");
  return;
}

sub get_utc_time{
  my $d = shift;
  my ($date, $time) = split ("T",$d);
  if (defined $date && defined $time){
    my @t = ();
    $date =~ s/-/\//g;
    $t[0] = $date;
    $t[1] = $time;
    my ($t, $timezone) = split(/\+/,$time);
    if (defined $t){
      $t[1] = $t;
    }
    if (!defined $timezone || $timezone eq ""){
      (undef, $timezone) = split(/-/,$time);
      if (defined $timezone && $timezone ne ""){
        $timezone = "-$timezone";
      }
    }
    else{
      $timezone = "+$timezone";
    }
    if (defined $timezone && $timezone ne ""){
      $t[2] = $timezone;
    }
    #print Dumper \@t;
    return str2time( join( " ", @t ) );
  }
  return $d;
}

sub set_data_device_conf{

  my $cmd = "StorageCenter";
  my $data = cmd_api($cmd,"GET",1,$cmd);
  if (!defined $data || $data eq ""){return;}
  foreach my $element ( @{$data} ) {
    my $machine_name   = $element->{"instanceName"};
    my $status         = $element->{"status"};
    my $status_message = $element->{"statusMessage"};
    my $model          = $element->{"modelSeries"};
    my $ip_adress      = $element->{"managementIp"};
    my $version        = $element->{"version"};
    my $serial_number  = $element->{"scSerialNumber"};
    my $serial_number2 = $element->{"serialNumber"};
    if ( !defined $serial_number || $serial_number ne $instance_storage ) { next; }
    if ( !defined $machine_name ) {
      $machine_name = "";
    }
    if ( !defined $status ) {
      $status = "";
    }
    if ( !defined $status_message ) {
      $status_message = "";
    }
    if ( !defined $model ) {
      $model = "";
    }
    if ( !defined $ip_adress ) {
      $ip_adress = "";
    }
    if ( !defined $version ) {
      $version = "";
    }
    if ( !defined $serial_number2 ) {
      $serial_number2 = "";
    }
    $cfg->{serial} = $serial_number2;
    $cfg->{model} = $model;
    $cfg->{version} = $version;
    $cfg->{host} = $ip_adress;
    $cfg->{"name"} = $machine_name;
    $cfg->{"status"} = $status;
    $cfg->{"status_message"} = $status_message;

  }
  #print Dumper \$cfg;
}

sub set_data_device{
  my $type = "pool";
  my @metrics = ("read_io","write_io","read","write", "resp_t_r" ,"resp_t_w","io_rate", "data_rate", "resp_t","capacity_total", "capacity_used", "capacity_free", "spare_capacity_total", "unused_capacity_total");
  my %data;
  my $time = "";
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    $time = (keys %{$performance_data{$st_type}{$type}{$uid}})[0];
    foreach my $metric (@metrics){
      if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$metric})){
        if (Xormon::isdigit($data{$metric})){
          $data{$metric} = $data{$metric} + $performance_data{$st_type}{$type}{$uid}{$time}{$metric};
        }
        else{
          $data{$metric} = $performance_data{$st_type}{$type}{$uid}{$time}{$metric};
        }
      }
    }
  }
  foreach my $metric (keys %data){
    $performance_data{$st_type}{device}{$st_serial}{$time}{$metric} = $data{$metric};
  }
  #$performance_data{$st_type}{device}{$st_serial}{$time} = \%data;
  #print Dumper \$performance_data{$st_type}{device};
  #print Dumper \%data;
}

sub get_status {
  my $rc = shift;
  my $task = shift;
  if ($rc eq "201" || $rc eq "200") {
    $logger->info("Task for $task ".$cfg->{hostalias}." is done. Status: successful");
    return "successful";
  } elsif ($rc eq "401") {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: unauthorized");
    return "unauthorized";
  } else {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
}

sub set_data_node_conf{

  my $section = "node";
  my $cmd = "ScController";
  my $data = cmd_api($cmd,"GET",1,$cmd);
  if (!defined $data || $data eq ""){return;}

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $storage_id       = $element->{"scSerialNumber"};
    my $status = $element->{"status"};
    if ( !defined $storage_id   || $storage_id ne $instance_storage ) { next; }
    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      $section_id =~ s/$instance_storage\.//;
      my $uid = Xormon::create_hash_key($st_serial.$section.$section_id);
      $configuration_data{$uid}{id} = $section_id;
      $configuration_data{$uid}{TYPE} = $section;
      if (!defined $section_name || $section_name eq ""){$section_name = $section_id;}
      $configuration_data{$uid}{label} = $section_name;
      if (defined $status && $status ne ""){
        $configuration_data{$uid}{status} = $status;
      }
    }
  }
}

sub set_data_volume_folder_conf{
  my $section     = "volume_folder";
  my $cmd = "ScVolumeFolder";
  my $data = cmd_api($cmd,"GET",1,$cmd);
  if (!defined $data || $data eq ""){return;}


  foreach my $element ( @{$data} ) {
    my $volume_instance = $element->{"instanceId"};
    my $volume_id       = $volume_instance;
    my $storage_id      = $element->{"scSerialNumber"};
    my $label           = $element->{"name"};
    my $status          = $element->{status};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( defined $volume_instance && $volume_instance ne "") {
      $volume_id =~ s/$instance_storage\.//;
      if ( defined $volume_id && $volume_id eq "0" ) { next; }    ### this is pool is per statistics for all pools
      my $uid = Xormon::create_hash_key($st_serial.$section.$volume_id);
      my $cmd = "ScVolumeFolder/$volume_instance/StorageUsage";
      my $data = cmd_api($cmd,"GET",1,"");
      if (defined $data && $data ne ""){
        my $capacity_total = $data->{configuredSpace};
        my $capacity_used = $data->{activeSpace};
        my $capacity_free = $data->{freeSpace};
        my $capacity_snapshot = $data->{replaySpace};
        if (defined $capacity_total){$capacity_total =~ s/ Bytes//g;}
        else{$capacity_total = "";}
        if (defined $capacity_used){$capacity_used =~ s/ Bytes//g;}
        else{$capacity_used = "";}
        if (defined $capacity_free){$capacity_free =~ s/ Bytes//g;}
        else{$capacity_free = "";}
        if (defined $capacity_snapshot){$capacity_snapshot =~ s/ Bytes//g;}
        else{$capacity_snapshot = "";}
        if ( Xormon::isdigit($capacity_total) && !Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
          $capacity_free = $capacity_total - $capacity_used;
        }
        if ( Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && !Xormon::isdigit($capacity_used) ) {
          $capacity_used = $capacity_total - $capacity_free;
        }
        if ( !Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
          $capacity_total = $capacity_free + $capacity_used;
        }
        $configuration_data{$uid}{capacity_total} = $capacity_total;
        $configuration_data{$uid}{capacity_used} = $capacity_used;
        $configuration_data{$uid}{capacity_free} = $capacity_free;
        $configuration_data{$uid}{capacity_snapshot} = $capacity_snapshot;
        $configuration_data{$uid}{TYPE} = $section;
        if (defined $status || $status ne ""){
          $configuration_data{$uid}{status} = $status;
        }
        if (!defined $label || $label eq ""){$label = $volume_id;}
        $configuration_data{$uid}{label} = $label;
      }
    }
  }
}

sub set_data_pool_conf{

  my $section     = "pool";
  my $uid = Xormon::create_hash_key($st_serial.$section."array_total");
  $configuration_data{$uid}{label} = "array_total";
  $configuration_data{$uid}{id} = "array_total";
  $configuration_data{$uid}{TYPE} = $section;

  my $cmd = "StorageCenter/$instance_storage/StorageUsage";
  my $data = cmd_api($cmd,"GET",1,"pool_capacity");
  $cmd = "StorageCenter";
  my $data_version = cmd_api($cmd,"GET",1,$cmd);
  $cmd = "StorageCenter/$instance_storage/DiskStorageUsage";
  my $data_disk = cmd_api($cmd,"GET",1,"DiskStorageUsage");
  if ( defined $data && $data ne "" ) {
    my $free = $data->{freeSpace};
    my $used = $data->{usedSpace};
    if ( defined $free ) {
      $free =~ s/Bytes//g;
      $free = remove_whitespace($free);
      if ( Xormon::isdigit($free) ){
        $configuration_data{$uid}{capacity_free} = $free;
      }
    }
    if ( defined $used ) {
      $used =~ s/Bytes//g;
      $used = remove_whitespace($used);
      if ( Xormon::isdigit($used) ) {
        $configuration_data{$uid}{capacity_used} = $used;
        if ( Xormon::isdigit($free) ) {
          $configuration_data{$uid}{capacity_total} = $used + $free;
        }
      }
    }
  }
  if ( defined $data_version && $data_version ne "" ) {
    foreach my $element ( @{$data_version} ) {
      my $version        = $element->{"version"};
      my $serial_number  = $element->{"scSerialNumber"};
      my $serial_number2 = $element->{"serialNumber"};
      if ( !defined $serial_number || $serial_number ne $instance_storage ) { next; }
      my $substr = substr( $version, 0, 3 );
      $substr =~ s/\.//g;
      if ( Xormon::isdigit($substr) && $substr > 72 ) {
        if ( defined $data && $data ne "" && defined $data_disk && $data_disk ne "" ) {
          my $free      = $data->{freeSpace};
          my $allocated = $data->{allocatedSpace};
          my $used      = $data->{usedSpace};
          my $unsed     = "";
          my $spare     = get_spare_capacity($data_disk);
          if ( defined $allocated ) {
            $allocated =~ s/Bytes//g;
            $allocated = remove_whitespace($allocated);
          }
          if ( defined $free ) {
            $free =~ s/Bytes//g;
            $free = remove_whitespace($free);
            if ( Xormon::isdigit($free) ) {
              $configuration_data{$uid}{capacity_free} = $free;
            }
          }
          if ( defined $used ) {
            $used =~ s/Bytes//g;
            $used = remove_whitespace($used);
            if ( Xormon::isdigit($used) ) {
              $configuration_data{$uid}{capacity_used} = $used;
              if ( Xormon::isdigit($used) && Xormon::isdigit($allocated) ) {
                $unsed = $allocated - $used;
                #$configuration_data{$uid}{allocated_capacity_total} = $allocated;
              }
            }
          }
          if ( defined $unsed ) {
            $unsed =~ s/Bytes//g;
            $unsed = remove_whitespace($unsed);
            if ( Xormon::isdigit($unsed) ) {
              $configuration_data{$uid}{unused_capacity_total} = $unsed;
            }
          }
          if ( defined $spare ) {
            $spare =~ s/Bytes//g;
            $spare = remove_whitespace($spare);
            if ( Xormon::isdigit($spare) ) {
              $configuration_data{$uid}{spare_capacity_total} = $spare;
              if ( Xormon::isdigit($unsed) && Xormon::isdigit($used) && Xormon::isdigit($free) ) {
                $configuration_data{$uid}{capacity_total} = $free + $used + $spare + $unsed;
              }
            }
          }
        }
        last;
      }
      #print "$substr\n";
    }
  }
  #print Dumper \$performance_data{$st_type}{$section}{$uid};
  #print Dumper \$data;

}

sub set_data_drive_conf{
  my $section = "drive";
  my $cmd = "ScDisk";
  my $data = cmd_api($cmd,"GET",1,$cmd);
  if (!defined $data || $data eq ""){return;}

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $serial_number    = $element->{"scSerialNumber"};
    my $disk_folder_name = $element->{"diskFolder"}{"instanceName"};
    my $disk_class_name  = $element->{"diskClass"}{"instanceName"};
    if ( !defined $serial_number || $serial_number ne $instance_storage ) { next; }
    my $status = $element->{"status"};

    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      if ( defined $serial_number && $section_id =~ m/^$serial_number\./ ) {
        $section_id =~ s/$serial_number\.//;
      }
      my $uid = Xormon::create_hash_key($st_serial.$section.$section_id);
      $configuration_data{$uid}{id} = $section_id;
      $configuration_data{$uid}{label} = $section_name;
      if ( defined $status && $status ne "" ) {
        $configuration_data{$uid}{"status"} = $status;
      }
      if ( defined $disk_folder_name && $disk_folder_name ne "" ) {
        $configuration_data{$uid}{"disk_folder_name"} = $disk_folder_name;
      }
      if ( defined $disk_class_name && $disk_class_name ne "" ) {
        $configuration_data{$uid}{"disk_class_name"} = $disk_class_name;
      }
      my $cmd = "ScDisk/$section_instance/StorageUsage";
      my $data = cmd_api($cmd,"GET",1,"");
      $configuration_data{$uid}{TYPE} = $section;
      if (defined $data && $data ne ""){
        my $capacity_total = $data->{totalSpace};
        my $capacity_used = $data->{allocatedSpace};
        my $capacity_free = $data->{freeSpace};
        if (defined $capacity_total){$capacity_total =~ s/ Bytes//g;}
        else{$capacity_total = "";}
        if (defined $capacity_used){$capacity_used =~ s/ Bytes//g;}
        else{$capacity_used = "";}
        if (defined $capacity_free){$capacity_free =~ s/ Bytes//g;}
        else{$capacity_free = "";}
        if ( Xormon::isdigit($capacity_total) && !Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
          $capacity_free = $capacity_total - $capacity_used;
        }
        if ( Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && !Xormon::isdigit($capacity_used) ) {
          $capacity_used = $capacity_total - $capacity_free;
        }
        if ( !Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
          $capacity_total = $capacity_free + $capacity_used;
        }
        $configuration_data{$uid}{capacity_total} = $capacity_total;
        $configuration_data{$uid}{capacity_used} = $capacity_used;
        $configuration_data{$uid}{capacity_free} = $capacity_free;
      }
    }
  }
}

sub set_data_host_conf{
  my $section = "host";
  my $cmd = "ScServer";
  my $data = cmd_api($cmd,"GET",1,$cmd);
  if (!defined $data || $data eq ""){return;}

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $serial_number    = $element->{"scSerialNumber"};
    if ( !defined $serial_number || $serial_number ne $instance_storage ) { next; }
    my $section_type = $element->{"type"};
    if ( !defined $section_type || $section_type ne "Physical" ) {
      next;    ### Only Physical is Host
    }
    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      if ( defined $serial_number && $section_id =~ m/^$serial_number\./ ) {
        $section_id =~ s/$serial_number\.//;
      }
      #print "$section_id\n";
      my $uid = Xormon::create_hash_key($st_serial.$section.$section_id);
      $configuration_data{$uid}{id}      = $section_id;
      $configuration_data{$uid}{host_id} = $section_id;
      set_mapping( $section_instance, "ScServer", $section );
      $configuration_data{$uid}{label} = $section_name;
      $configuration_data{$uid}{TYPE} = $section;
      if (defined $configuration_data{$uid}{"Volume IDs"}){
        my @volumes = split(" ",$configuration_data{$uid}{"Volume IDs"});
        foreach my $el (@volumes){
          if (defined $performance_data{$st_type}{volume}{$el}){
            push(@{$configuration_data{$uid}{children}}, $el);
          }
        }
      }
      #print Dumper \$configuration_data{$uid};
    }
  }
}

sub set_mapping{
  my $section_instance = shift;
  my $object = shift;
  my $section = shift;
  my $cmd = "$object/$section_instance/MappingList";
  #print "$cmd\n";
  my $data = cmd_api($cmd,"GET",1,"");
  if (!defined $data || $data eq ""){return;}

  #my $section_instance = $section_instance;
  my $uid = "";
  #print Dumper \$data;
  foreach my $element ( @{$data} ) {
    my $volume_name   = $element->{"volume"}->{"instanceName"};
    my $volume_id     = $element->{"volume"}->{"instanceId"};
    my $port_id       = $element->{"controllerPort"}->{instanceId};
    my $port_name     = $element->{"controllerPort"}->{instanceName};
    my $serial_number = $element->{"scSerialNumber"};
    my $section_id = $section_instance;
    $section_id =~ s/$serial_number\.//;
    $uid = Xormon::create_hash_key($st_serial.$section.$section_id);
    if ( !defined $serial_number || $serial_number ne $instance_storage ) { next; }
    if ( defined $configuration_data{$uid}) {
      if ( defined $volume_id && $volume_id ne "" ) {
        $configuration_data{$uid}{volume}{$volume_id}{id} = $volume_id;
        $configuration_data{$uid}{volume}{$volume_id}{name} = $volume_name;
      }
      if ( defined $port_id && $port_id ne "" ) {
        $configuration_data{$uid}{port}{$port_id}{id} = $port_id;
        if ( defined $port_name && $port_name ne "" ) {
          $configuration_data{$uid}{port}{$port_id}{name} = $port_name;
        }
      }
    }
  }
  ### volumes id to "Volume IDs"
  my $vol_inventory = "";
  if ($uid eq ""){return;}
  foreach my $vol_id ( keys %{ $configuration_data{$uid}{volume} } ) {
    my $volume_id = "";
    if ( defined $configuration_data{$uid}{volume}{$vol_id}{id} ) {
      my $volume_instance = $configuration_data{$uid}{volume}{$vol_id}{id};
      if ( defined $volume_inventory{$volume_instance} ) {
        $volume_id = $volume_inventory{$volume_instance};
      }
      else {
        next;
      }
      if ( $vol_inventory ne "" && $volume_id ne "" ) {
        $vol_inventory = $vol_inventory . " $volume_id";
      }
      else {
        $vol_inventory = $volume_id;
      }
    }
  }
  if ( $vol_inventory ne "" ) {
    $configuration_data{$uid}{"Volume IDs"} = $vol_inventory;
    #push(@{$configuration_data{$uid}{children}}, $);
  }
  ### MAPPING port
  $vol_inventory = "";
  foreach my $vol_id ( keys %{ $configuration_data{$uid}{volume} } ) {
    my $volume_name = "";
    if ( defined $configuration_data{$uid}{volume}{$vol_id}{name} ) {
      my $volume_instance = $configuration_data{$uid}{volume}{$vol_id}{id};
      if ( defined $configuration_data{$uid}{"name"} ) {
        $volume_name = $configuration_data{$uid}{"name"};
      }
      else {
        next;
      }
      if ( $vol_inventory ne "" && $volume_name ne "" ) {
        $vol_inventory = $vol_inventory . " $volume_name";
      }
      else {
        $vol_inventory = $volume_name;
      }
    }
  }
  if ( $vol_inventory ne "" ) {
    $configuration_data{$uid}{"Volumes"} = $vol_inventory;
  }
  $vol_inventory = "";
  foreach my $port_id ( keys %{ $configuration_data{$uid}{port} } ) {
    my $port_name = "";
    if ( defined $configuration_data{$uid}{port}{$port_id}{name} ) {
      $port_name = $configuration_data{$uid}{port}{$port_id}{name};
    }
    else {
      next;
    }
    if ( $vol_inventory ne "" && $port_name ne "" ) {
      $vol_inventory = $vol_inventory . " $port_name";
    }
    else {
      $vol_inventory = $port_name;
    }
  }
  if ( $vol_inventory ne "" ) {
    $configuration_data{$uid}{"Ports"} = $vol_inventory;
  }
  if (defined $configuration_data{$uid}{port}){
    delete $configuration_data{$uid}{port};
  }
  if (defined $configuration_data{$uid}{volume}){
    delete $configuration_data{$uid}{volume}
  }

}

sub set_data_volume_conf{
  my $section = "volume";
  my $cmd = "ScVolume";
  my $data = cmd_api($cmd,"GET",1,$cmd);
  if (!defined $data || $data eq ""){return;}

  #print Dumper \$data;
  foreach my $element ( @{$data} ) {
    my $volume_instance = $element->{"instanceId"};
    my $volume_id       = $element->{"deviceId"};
    my $volume_name     = $element->{"name"};
    my $pool_id         = $element->{"volumeFolder"}{"instanceId"};
    my $pool_name       = $element->{"volumeFolder"}{"instanceName"};
    my $serial_number   = $element->{"scSerialNumber"};
    my $serial          = $element->{"serialNumber"};
    if ( !defined $serial_number || $serial_number ne $instance_storage ) { next; }
    my $status = $element->{"status"};
    if ( defined $volume_instance && $volume_instance ne "" && defined $volume_id && $volume_id ne "" ) {
      if (!defined $volume_name || $volume_name eq ""){$volume_name = $volume_id;}
      #my $uid = Xormon::create_hash_key($st_serial.$section.$volume_id);
      my $uid = $volume_id;
      $configuration_data{$uid}{TYPE} = $section;
      $configuration_data{$uid}{volume_id} = $volume_id;
      $configuration_data{$uid}{"lun_id"} = $volume_id;
      $volume_inventory{$volume_instance} = $uid;
      my $cmd = "ScVolume/$volume_instance/StorageUsage";
      my $data = cmd_api($cmd,"GET",1,"");
      if (defined $data && $data ne ""){
        my $capacity_total = $data->{configuredSpace};
        my $capacity_used = $data->{activeSpace};
        my $capacity_free = $data->{freeSpace};
        my $capacity_snapshot = $data->{snapshotOverheadOnDisk};
        if (defined $capacity_total){$capacity_total =~ s/ Bytes//g;}
        else{$capacity_total = "";}
        if (defined $capacity_used){$capacity_used =~ s/ Bytes//g;}
        else{$capacity_used = "";}
        if (defined $capacity_free){$capacity_free =~ s/ Bytes//g;}
        else{$capacity_free = "";}
        if (defined $capacity_snapshot){$capacity_snapshot =~ s/ Bytes//g;}
        else{$capacity_snapshot = "";}
        if ( Xormon::isdigit($capacity_total) && !Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
          $capacity_free = $capacity_total - $capacity_used;
        }
        if ( Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && !Xormon::isdigit($capacity_used) ) {
          $capacity_used = $capacity_total - $capacity_free;
        }
        if ( !Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
          $capacity_total = $capacity_free + $capacity_used;
        }
        $configuration_data{$uid}{capacity_total} = $capacity_total;
        $configuration_data{$uid}{capacity_used} = $capacity_used;
        $configuration_data{$uid}{capacity_free} = $capacity_free;
        $configuration_data{$uid}{capacity_snapshot} = $capacity_snapshot;
        my $noname = "**NoName**";
        if ( $volume_name eq $noname ) {
          $volume_name = $volume_id;
        }
        $configuration_data{$uid}{label} = $volume_name;
        if ( defined $pool_id && $pool_id ne "" ) {
          if ( defined $serial_number && $pool_id =~ m/^$serial_number\./ ) {
            $pool_id =~ s/$serial_number\.//;
            my $parent_pool = Xormon::create_hash_key($st_serial."pool"."array_total");
            my $parent_volume_folder = Xormon::create_hash_key($st_serial."volume_folder".$pool_id);
            $configuration_data{$uid}{"pool_id"}         = "array_total";
            $configuration_data{$uid}{"volumefolder_id"} = $pool_id;
            push(@{$configuration_data{$uid}{parents}}, $parent_pool);
            if (defined $performance_data{$st_type}{volume_folder}{$parent_volume_folder}){
              push(@{$configuration_data{$uid}{parents}}, $parent_volume_folder);
            }
          }
        }
        if ( defined $pool_name && $pool_name ne "" ) {
          $configuration_data{$uid}{"pool_name"}         = "array_total";
          $configuration_data{$uid}{"volumefolder_name"} = $pool_name;
        }
        if ( defined $status && $status ne "" ) {
          $configuration_data{$uid}{"status"} = $status;
        }
        if ( defined $serial && $serial ne "" ) {
          $configuration_data{$uid}{"serial"} = $serial;
        }
        $configuration_data{$uid}{TYPE} = $section;

      }
    }
  }

}

sub set_data_port_mapping {

  my $section  = "host";
  my $section2 = "port_mapping";

  foreach my $uid (keys %configuration_data){
    if (defined $configuration_data{$uid}{TYPE} && $configuration_data{$uid}{TYPE} eq "host"){
      if (!defined $configuration_data{$uid}{id} || !defined $configuration_data{$uid}{label}){next;}
      my $id = $configuration_data{$uid}{id};
      my $new_uid = Xormon::create_hash_key($st_serial.$section2.$id);
      my $children = $configuration_data{$uid}{children};
      my $volumes = "";
      if (defined $children && ref($children) eq "ARRAY"){
        foreach my $volume (@{$children}){
          if (defined $configuration_data{$volume}{label}){
            if ($volumes eq ""){
              $volumes = $configuration_data{$volume}{label};
            }
            else{
              $volumes = $volumes . " : " . $configuration_data{$volume}{label};
            }
          }
        }
        $configuration_data{$new_uid}{Volumes} = $volumes;
        if (defined $configuration_data{$uid}{Ports}){
          $configuration_data{$new_uid}{Ports} = $configuration_data{$uid}{Ports};
        }
        $configuration_data{$new_uid}{Host} = $configuration_data{$uid}{label};
        $configuration_data{$new_uid}{label} = $configuration_data{$uid}{label};
        $configuration_data{$new_uid}{TYPE} = $section2;
      }
    }
  }
}


sub set_data_host{
  my $section     = "host";
  my $cmd = "ScServer";
  my $data = cmd_api($cmd,"GET",0,$cmd);
  if (!defined $data || $data eq ""){return;}


  my $cmd_perf = "ScServerIoUsage/GetFilter";
  ### body for request
  my %hash_filter;
  $hash_filter{"HistoricalFilter"}{"FilterTime"}    = "Other";
  $hash_filter{"HistoricalFilter"}{"UseEndOfDay"}   = "true";
  $hash_filter{"HistoricalFilter"}{"UseStartOfDay"} = "true";
  $hash_filter{"HistoricalFilter"}{"UseCurrent"}    = "true";

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $section_type     = $element->{"type"};
    my $storage_id       = $element->{"scSerialNumber"};
    if ( !defined $storage_id   || $storage_id ne $instance_storage ) { next; }
    if ( !defined $section_type || $section_type ne "Physical" ) {
      next;    ### Only Physical is Host
    }
    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      $hash_filter{"ObjectInstanceId"} = "$section_id";
      $section_id =~ s/$instance_storage\.//;
      my $data = cmd_api($cmd_perf,"POST",1,"",\%hash_filter);
      if (defined $data && $data ne ""){
        my $uid = Xormon::create_hash_key($st_serial.$section.$section_id);
        my ($new_data,$time) = convert_data($data);
        if (defined $new_data && $new_data ne "" && Xormon::isdigit($time)){
          $performance_data{$st_type}{$section}{$uid}{$time} = $new_data;
        }
      }
    }
  }
}
sub set_data_node {
  my $section = "node";
  my $cmd = "ScController";
  my $data = cmd_api($cmd,"GET",0,$cmd);
  if (!defined $data || $data eq ""){return;}


  my $cmd_perf = "ScControllerIoUsage/GetFilter";
  ### body for request
  my %hash_filter;
  $hash_filter{"HistoricalFilter"}{"FilterTime"}    = "Other";
  $hash_filter{"HistoricalFilter"}{"UseEndOfDay"}   = "true";
  $hash_filter{"HistoricalFilter"}{"UseStartOfDay"} = "true";
  $hash_filter{"HistoricalFilter"}{"UseCurrent"}    = "true";

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $storage_id       = $element->{"scSerialNumber"};
    if ( !defined $storage_id   || $storage_id ne $instance_storage ) { next; }
    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      $hash_filter{"ObjectInstanceId"} = "$section_id";
      $section_id =~ s/$instance_storage\.//;
      my $data = cmd_api($cmd_perf,"POST",1,"",\%hash_filter);
      if (defined $data && $data ne ""){
        my $uid = Xormon::create_hash_key($st_serial.$section.$section_id);
        my ($new_data,$time) = convert_data($data);
        if (defined $new_data && $new_data ne "" && Xormon::isdigit($time)){
          $performance_data{$st_type}{$section}{$uid}{$time} = $new_data;
        }
      }
    }
  }

}

sub set_data_pool {
  my $section     = "pool";
  my $uid = Xormon::create_hash_key($st_serial.$section."array_total");
  my $time_global;
  foreach my $volume (keys %{$performance_data{$st_type}{volume}}){
    my ($time) = (keys %{$performance_data{$st_type}{volume}{$volume}});
    if (!Xormon::isdigit($time_global)){
      $time_global = $time;
    }
    if (!defined $time_global || $time_global eq ""){next;}
    foreach my $metric (keys %{$performance_data{$st_type}{volume}{$volume}{$time}}){
      if ($metric eq "io_rate"){
        if (Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time_global}{"io_rate"}) && Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time}{"resp_t"})){
          my $num = $performance_data{$st_type}{volume}{$volume}{$time_global}{"io_rate"} * $performance_data{$st_type}{volume}{$volume}{$time}{"resp_t"};
          if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t"})){
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t"} = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t"} + $num;
          }
          else{
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t"} = $num;
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time_global}{"io_rate"}) && Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time}{"resp_t_xfer"})){
          my $num = $performance_data{$st_type}{volume}{$volume}{$time_global}{"io_rate"} * $performance_data{$st_type}{volume}{$volume}{$time}{"resp_t_xfer"};
          if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_xfer"})){
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_xfer"} = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_xfer"} + $num;
          }
          else{
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_xfer"} = $num;
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time_global}{"io_rate"}) && Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time}{"resp_t_qos"})){
          my $num = $performance_data{$st_type}{volume}{$volume}{$time_global}{"io_rate"} * $performance_data{$st_type}{volume}{$volume}{$time}{"resp_t_qos"};
          if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_qos"})){
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_qos"} = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_qos"} + $num;
          }
          else{
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_qos"} = $num;
          }
        }
      }
      if ($metric eq "read_io"){
        if (Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time_global}{"read_io"}) && Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time}{"resp_t_r"})){
          my $num = $performance_data{$st_type}{volume}{$volume}{$time_global}{"read_io"} * $performance_data{$st_type}{volume}{$volume}{$time}{"resp_t_r"};
          if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_r"})){
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_r"} = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_r"} + $num;
          }
          else{
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_r"} = $num;
          }
        }
      }
      if ($metric eq "write_io"){
        if (Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time_global}{"write_io"}) && Xormon::isdigit($performance_data{$st_type}{volume}{$volume}{$time}{"resp_t_w"})){
          my $num = $performance_data{$st_type}{volume}{$volume}{$time_global}{"write_io"} * $performance_data{$st_type}{volume}{$volume}{$time}{"resp_t_w"};
          if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_w"})){
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_w"} = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_w"} + $num;
          }
          else{
            $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_w"} = $num;
          }
        }
      }
      if ($metric !~ m/^resp/){
        if (Xormon::isdigit($performance_data{$st_type}{$section}{$uid}{$time_global}{$metric})){
          $performance_data{$st_type}{$section}{$uid}{$time_global}{$metric} = $performance_data{$st_type}{$section}{$uid}{$time_global}{$metric} + $performance_data{$st_type}{volume}{$volume}{$time}{$metric};
        }
        else{
          $performance_data{$st_type}{$section}{$uid}{$time_global}{$metric} = $performance_data{$st_type}{volume}{$volume}{$time}{$metric};
        }
      }
      #print $metric . " " . $performance_data{$st_type}{volume}{$volume}{$time}{$metric} . "\n";
    }
  }
  if (!defined $time_global){
    return;
  }
  my $numerator = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t"};
  my $denominator = $performance_data{$st_type}{$section}{$uid}{$time_global}{"io_rate"};
  if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator) && $denominator > 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t"} = round($numerator / $denominator,6);
  }
  if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator) && $numerator == 0 && $denominator == 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t"} = 0;
  }

  my $numerator_r = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_r"};
  my $denominator_r = $performance_data{$st_type}{$section}{$uid}{$time_global}{"write_io"};
  if (Xormon::isdigit($numerator_r) && Xormon::isdigit($denominator_r) && $denominator_r > 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t_r"} = round($numerator_r / $denominator_r,6);
  }
  if (Xormon::isdigit($numerator_r) && Xormon::isdigit($denominator_r) && $numerator_r == 0 && $denominator_r == 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t_r"} = 0;
  }

  my $numerator_w = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_w"};
  my $denominator_w = $performance_data{$st_type}{$section}{$uid}{$time_global}{"write_io"};
  if (Xormon::isdigit($numerator_w) && Xormon::isdigit($denominator_w) && $denominator_w > 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t_w"} = round($numerator_w / $denominator_w,6);
  }
  if (Xormon::isdigit($numerator_w) && Xormon::isdigit($denominator_w) && $numerator_w == 0 && $denominator_w == 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t_w"} = 0;
  }
  my $numerator_xfer = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_xfer"};
  if (Xormon::isdigit($numerator_xfer) && Xormon::isdigit($denominator) && $denominator > 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t_xfer"} = round($numerator_xfer / $denominator,6);
  }
  if (Xormon::isdigit($numerator_xfer) && Xormon::isdigit($denominator) && $numerator_xfer == 0 && $denominator == 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t_xfer"} = 0;
  }
  my $numerator_qos = $performance_data{$st_type}{$section}{$uid}{$time_global}{"numerator_resp_t_qos"};
  if (Xormon::isdigit($numerator_qos) && Xormon::isdigit($denominator) && $denominator > 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t_qos"} = round($numerator_qos / $denominator,6);
  }
  if (Xormon::isdigit($numerator_qos) && Xormon::isdigit($denominator) && $numerator_qos == 0 && $denominator == 0){
    $performance_data{$st_type}{$section}{$uid}{$time_global}{"resp_t_qos"} = 0;
  }
  my $cmd = "StorageCenter/$instance_storage/StorageUsage";
  my $data = cmd_api($cmd,"GET",1,"pool_capacity");
  $cmd = "StorageCenter";
  my $data_version = cmd_api($cmd,"GET",1,$cmd);
  $cmd = "StorageCenter/$instance_storage/DiskStorageUsage";
  my $data_disk = cmd_api($cmd,"GET",1,"DiskStorageUsage");
  if ( defined $data && $data ne "" ) {
    my $free = $data->{freeSpace};
    my $used = $data->{usedSpace};
    if ( defined $free ) {
      $free =~ s/Bytes//g;
      $free = remove_whitespace($free);
      if ( Xormon::isdigit($free) ){
        $performance_data{$st_type}{$section}{$uid}{$time_global}{capacity_free} = $free;
      }
    }
    if ( defined $used ) {
      $used =~ s/Bytes//g;
      $used = remove_whitespace($used);
      if ( Xormon::isdigit($used) ) {
        $performance_data{$st_type}{$section}{$uid}{$time_global}{capacity_used} = $used;
        if ( Xormon::isdigit($free) ) {
          $performance_data{$st_type}{$section}{$uid}{$time_global}{capacity_total} = $used + $free;
        }
      }
    }
  }
  if ( defined $data_version && $data_version ne "" ) {
    foreach my $element ( @{$data_version} ) {
      my $version        = $element->{"version"};
      my $serial_number  = $element->{"scSerialNumber"};
      my $serial_number2 = $element->{"serialNumber"};
      if ( !defined $serial_number || $serial_number ne $instance_storage ) { next; }
      my $substr = substr( $version, 0, 3 );
      $substr =~ s/\.//g;
      if ( Xormon::isdigit($substr) && $substr > 72 ) {
        if ( defined $data && $data ne "" && defined $data_disk && $data_disk ne "" ) {
          my $free      = $data->{freeSpace};
          my $allocated = $data->{allocatedSpace};
          my $used      = $data->{usedSpace};
          my $unsed     = "";
          my $spare     = get_spare_capacity($data_disk);
          if ( defined $allocated ) {
            $allocated =~ s/Bytes//g;
            $allocated = remove_whitespace($allocated);
          }
          if ( defined $free ) {
            $free =~ s/Bytes//g;
            $free = remove_whitespace($free);
            if ( Xormon::isdigit($free) ) {
              $performance_data{$st_type}{$section}{$uid}{$time_global}{capacity_free} = $free;
            }
          }
          if ( defined $used ) {
            $used =~ s/Bytes//g;
            $used = remove_whitespace($used);
            if ( Xormon::isdigit($used) ) {
              $performance_data{$st_type}{$section}{$uid}{$time_global}{capacity_used} = $used;
              if ( Xormon::isdigit($used) && Xormon::isdigit($allocated) ) {
                $unsed = $allocated - $used;
                #$performance_data{$st_type}{$section}{$uid}{$time_global}{allocated_capacity_total} = $allocated;
              }
            }
          }
          if ( defined $unsed ) {
            $unsed =~ s/Bytes//g;
            $unsed = remove_whitespace($unsed);
            if ( Xormon::isdigit($unsed) ) {
              $performance_data{$st_type}{$section}{$uid}{$time_global}{unused_capacity_total} = $unsed;
            }
          }
          if ( defined $spare ) {
            $spare =~ s/Bytes//g;
            $spare = remove_whitespace($spare);
            if ( Xormon::isdigit($spare) ) {
              $performance_data{$st_type}{$section}{$uid}{$time_global}{spare_capacity_total} = $spare;
              if ( Xormon::isdigit($unsed) && Xormon::isdigit($used) && Xormon::isdigit($free) ) {
                $performance_data{$st_type}{$section}{$uid}{$time_global}{capacity_total} = $free + $used + $spare + $unsed;
              }
            }
          }
        }
        last;
      }
      #print "$substr\n";
    }
  }
  #print Dumper \$performance_data{$st_type}{$section}{$uid};
  #print Dumper \$data;

}

sub get_spare_capacity {
  my $spare_data    = shift;
  my $spare_sum_cap = "";

  foreach my $element ( @{$spare_data} ) {
    my $spare_cap = $element->{spareSpace};
    if ( defined $spare_cap ) {
      $spare_cap =~ s/Bytes//g;
      $spare_cap = remove_whitespace($spare_cap);
    }
    if ( Xormon::isdigit($spare_cap) ) {
      if ( Xormon::isdigit($spare_sum_cap) ) {
        $spare_sum_cap = $spare_sum_cap + $spare_cap;
      }
      else {
        $spare_sum_cap = $spare_cap;
      }
    }
  }
  if ( Xormon::isdigit($spare_sum_cap) ) {
    return $spare_sum_cap;
  }
  return "";

}


sub set_data_volume {
  my $section     = "volume";
  my $cmd = "ScVolume";
  my $data = cmd_api($cmd,"GET",0,$cmd);
  if (!defined $data || $data eq ""){return;}



  my $cmd_perf = "ScVolumeIoUsage/GetFilter";
  ### body for request
  my %hash_filter;
  $hash_filter{"HistoricalFilter"}{"FilterTime"}    = "Other";
  $hash_filter{"HistoricalFilter"}{"UseEndOfDay"}   = "true";
  $hash_filter{"HistoricalFilter"}{"UseStartOfDay"} = "true";
  $hash_filter{"HistoricalFilter"}{"UseCurrent"}    = "true";


  foreach my $element ( @{$data} ) {
    my $volume_instance = $element->{"instanceId"};
    my $volume_id       = $element->{"deviceId"};
    my $volume_name     = $element->{"name"};
    my $storage_id      = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( defined $volume_instance && $volume_instance ne "" && defined $volume_id && $volume_id ne "" ) {
      $hash_filter{"ObjectInstanceId"} = "$volume_instance";
      my $data = cmd_api($cmd_perf,"POST",1,"",\%hash_filter);
      if ( defined $data && $data ne "" ) {
        #my $uid = Xormon::create_hash_key($st_serial.$section.$volume_id);
        my $uid = $volume_id;
        my ($new_data,$time) = convert_data($data);
        if (defined $new_data && $new_data ne "" && Xormon::isdigit($time)){
          $performance_data{$st_type}{$section}{$uid}{$time} = $new_data;
          my $cmd = "ScVolume/$volume_instance/StorageUsage";
          my $data = cmd_api($cmd,"GET",1,"");
          if (defined $data && $data ne ""){
            my $capacity_total = $data->{configuredSpace};
            my $capacity_used = $data->{activeSpace};
            my $capacity_free = $data->{freeSpace};
            my $capacity_snapshot = $data->{snapshotOverheadOnDisk};
            if (defined $capacity_total){$capacity_total =~ s/ Bytes//g;}
            else{$capacity_total = "";}
            if (defined $capacity_used){$capacity_used =~ s/ Bytes//g;}
            else{$capacity_used = "";}
            if (defined $capacity_free){$capacity_free =~ s/ Bytes//g;}
            else{$capacity_free = "";}
            if (defined $capacity_snapshot){$capacity_snapshot =~ s/ Bytes//g;}
            else{$capacity_snapshot = "";}
            if ( Xormon::isdigit($capacity_total) && !Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
              $capacity_free = $capacity_total - $capacity_used;
            }
            if ( Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && !Xormon::isdigit($capacity_used) ) {
              $capacity_used = $capacity_total - $capacity_free;
            }
            if ( !Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
              $capacity_total = $capacity_free + $capacity_used;
            }
            $performance_data{$st_type}{$section}{$uid}{$time}{capacity_total} = $capacity_total;
            $performance_data{$st_type}{$section}{$uid}{$time}{capacity_used} = $capacity_used;
            $performance_data{$st_type}{$section}{$uid}{$time}{capacity_free} = $capacity_free;
            $performance_data{$st_type}{$section}{$uid}{$time}{snapshot_capacity_total} = $capacity_snapshot;

          }



          #print Dumper \$data;
        }
      }
    }
  }
}
sub set_data_volume_folder {
  my $section     = "volume_folder";
  my $cmd = "ScVolumeFolder";
  my $data = cmd_api($cmd,"GET",0,$cmd);
  if (!defined $data || $data eq ""){return;}



  my $cmd_perf = "ScVolumeFolderIoUsage/GetFilter";
  ### body for request
  my %hash_filter;
  $hash_filter{"HistoricalFilter"}{"FilterTime"}    = "Other";
  $hash_filter{"HistoricalFilter"}{"UseEndOfDay"}   = "true";
  $hash_filter{"HistoricalFilter"}{"UseStartOfDay"} = "true";
  $hash_filter{"HistoricalFilter"}{"UseCurrent"}    = "true";


  foreach my $element ( @{$data} ) {
    my $volume_instance = $element->{"instanceId"};
    my $volume_id       = $volume_instance;
    my $storage_id      = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( defined $volume_instance && $volume_instance ne "") {
      $hash_filter{"ObjectInstanceId"} = "$volume_instance";
      $volume_id =~ s/$instance_storage\.//;
      if ( defined $volume_id && $volume_id eq "0" ) { next; }    ### this is pool is per statistics for all pools
      my $data = cmd_api($cmd_perf,"POST",1,"",\%hash_filter);
      if ( defined $data && $data ne "" ) {
        my $uid = Xormon::create_hash_key($st_serial.$section.$volume_id);
        my ($new_data,$time) = convert_data($data);
        if (defined $new_data && $new_data ne "" && Xormon::isdigit($time)){
          $performance_data{$st_type}{$section}{$uid}{$time} = $new_data;
          my $cmd = "ScVolumeFolder/$volume_instance/StorageUsage";
          my $data = cmd_api($cmd,"GET",1,"");
          if (defined $data && $data ne ""){
            my $capacity_total = $data->{configuredSpace};
            my $capacity_used = $data->{activeSpace};
            my $capacity_free = $data->{freeSpace};
            my $capacity_snapshot = $data->{replaySpace};
            if (defined $capacity_total){$capacity_total =~ s/ Bytes//g;}
            else{$capacity_total = "";}
            if (defined $capacity_used){$capacity_used =~ s/ Bytes//g;}
            else{$capacity_used = "";}
            if (defined $capacity_free){$capacity_free =~ s/ Bytes//g;}
            else{$capacity_free = "";}
            if (defined $capacity_snapshot){$capacity_snapshot =~ s/ Bytes//g;}
            else{$capacity_snapshot = "";}
            if ( Xormon::isdigit($capacity_total) && !Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
              $capacity_free = $capacity_total - $capacity_used;
            }
            if ( Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && !Xormon::isdigit($capacity_used) ) {
              $capacity_used = $capacity_total - $capacity_free;
            }
            if ( !Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
              $capacity_total = $capacity_free + $capacity_used;
            }
            $performance_data{$st_type}{$section}{$uid}{$time}{capacity_total} = $capacity_total;
            $performance_data{$st_type}{$section}{$uid}{$time}{capacity_used} = $capacity_used;
            $performance_data{$st_type}{$section}{$uid}{$time}{capacity_free} = $capacity_free;
            $performance_data{$st_type}{$section}{$uid}{$time}{capacity_snapshot} = $capacity_snapshot;

          }



          #print Dumper \$data;
        }
      }
    }
  }
}
sub set_data_port {
  my $section     = "port";
  my $cmd = "ScControllerPort";
  my $data = cmd_api($cmd,"GET",0,$cmd);
  if (!defined $data || $data eq ""){return;}


  my $cmd_perf = "ScControllerPortIoUsage/GetFilter";
  ### body for request
  my %hash_filter;
  $hash_filter{"HistoricalFilter"}{"FilterTime"}    = "Other";
  $hash_filter{"HistoricalFilter"}{"UseEndOfDay"}   = "true";
  $hash_filter{"HistoricalFilter"}{"UseStartOfDay"} = "true";
  $hash_filter{"HistoricalFilter"}{"UseCurrent"}    = "true";


  foreach my $element ( @{$data} ) {
    my $port_id    = $element->{"instanceId"};
    my $port_ids   = $port_id;
    my $virtual    = $element->{"virtual"};
    my $storage_id = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( Xormon::isdigit($virtual) && $virtual == 1 ) { next; }
    my $port_name = $element->{"name"};
    if ( defined $port_id && $port_id ne "" && defined $port_ids && $port_ids ne "" ) {
      $port_ids =~ s/$instance_storage\.//;
      $hash_filter{"ObjectInstanceId"} = "$port_id";
      my $data = cmd_api($cmd_perf,"POST",1,"",\%hash_filter);
      if ( defined $data && $data ne "" ) {
        my $uid = Xormon::create_hash_key($st_serial.$section.$port_ids);
        my ($new_data,$time) = convert_data($data);
        if (defined $new_data && $new_data ne "" && Xormon::isdigit($time)){
          #print Dumper \$data;
          #print Dumper \$new_data;
          $performance_data{$st_type}{$section}{$uid}{$time} = $new_data;
        }
      }
    }
  }
}

sub set_data_drive {
  my $section     = "drive";
  my $cmd = "ScDisk";
  my $data = cmd_api($cmd,"GET",0,$cmd);
  if (!defined $data || $data eq ""){return;}


  my $cmd_perf = "ScDiskIoUsage/GetFilter";
  ### body for request
  my %hash_filter;
  $hash_filter{"HistoricalFilter"}{"FilterTime"}    = "Other";
  $hash_filter{"HistoricalFilter"}{"UseEndOfDay"}   = "true";
  $hash_filter{"HistoricalFilter"}{"UseStartOfDay"} = "true";
  $hash_filter{"HistoricalFilter"}{"UseCurrent"}    = "true";

  foreach my $element ( @{$data} ) {
    my $section_instance = $element->{"instanceId"};
    my $section_id       = $section_instance;
    my $section_name     = $element->{"name"};
    my $storage_id       = $element->{"scSerialNumber"};
    if ( !defined $storage_id   || $storage_id ne $instance_storage ) { next; }
    if ( defined $section_instance && $section_instance ne "" && defined $section_id && $section_id ne "" ) {
      $hash_filter{"ObjectInstanceId"} = "$section_id";
      $section_id =~ s/$instance_storage\.//;
      my $data = cmd_api($cmd_perf,"POST",1,"",\%hash_filter);
      if (!defined $data || $data eq ""){next;}
      my $uid = Xormon::create_hash_key($st_serial.$section.$section_id);
      my ($new_data,$time) = convert_data($data);
      if (defined $new_data && $new_data ne "" && Xormon::isdigit($time)){
        $performance_data{$st_type}{$section}{$uid}{$time} = $new_data;
        my $cmd = "ScDisk/$section_instance/StorageUsage";
        my $data = cmd_api($cmd,"GET",1,"");
        if (defined $data && $data ne ""){
          my $capacity_total = $data->{totalSpace};
          my $capacity_used = $data->{allocatedSpace};
          my $capacity_free = $data->{freeSpace};
          if (defined $capacity_total){$capacity_total =~ s/ Bytes//g;}
          else{$capacity_total = "";}
          if (defined $capacity_used){$capacity_used =~ s/ Bytes//g;}
          else{$capacity_used = "";}
          if (defined $capacity_free){$capacity_free =~ s/ Bytes//g;}
          else{$capacity_free = "";}
          if ( Xormon::isdigit($capacity_total) && !Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
            $capacity_free = $capacity_total - $capacity_used;
          }
          if ( Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && !Xormon::isdigit($capacity_used) ) {
            $capacity_used = $capacity_total - $capacity_free;
          }
          if ( !Xormon::isdigit($capacity_total) && Xormon::isdigit($capacity_free) && Xormon::isdigit($capacity_used) ) {
            $capacity_total = $capacity_free + $capacity_used;
          }
          $performance_data{$st_type}{$section}{$uid}{$time}{capacity_total} = $capacity_total;
          $performance_data{$st_type}{$section}{$uid}{$time}{capacity_used} = $capacity_used;
          $performance_data{$st_type}{$section}{$uid}{$time}{capacity_free} = $capacity_free;
        }
      }
    }
  }
}

sub set_data_port_type{
  set_data_port_conf();
  my $section = "port";


  foreach my $uid (keys %configuration_data){
    my $protocol = $configuration_data{$uid}{Protocol};
    my $id = $configuration_data{$uid}{id};    
    #print "$id\n";
    if (!defined $protocol || !defined $id){next;}
    $protocol = lc($protocol);
    $protocol = $protocol . "port";
    if (defined $performance_data{$st_type}{$section}{$uid}){
      my $port_uid_new = Xormon::create_hash_key($st_serial.$protocol.$id);
      foreach my $time (keys %{$performance_data{$st_type}{$section}{$uid}}){
        foreach my $item (keys %{$performance_data{$st_type}{$section}{$uid}{$time}}){
          $performance_data{$st_type}{$protocol}{$port_uid_new}{$time}{$item} = $performance_data{$st_type}{$section}{$uid}{$time}{$item};
        }
      }
      delete $performance_data{$st_type}{$section}{$uid};
      foreach my $item (keys %{$configuration_data{$uid}}){
        $configuration_data{$port_uid_new}{$item} = $configuration_data{$uid}{$item};
        if ($item eq "label"){
          my $value = $configuration_data{$uid}{$item};
          $value =~ s/ \(.*//g;
          $configuration_data{$port_uid_new}{$item} = $value;
        }
        $configuration_data{$port_uid_new}{TYPE} = $protocol;
      }
      delete $configuration_data{$uid};
    }
  }
}

sub set_data_port_conf {
  my $section = "port";
  my $cmd = "ScControllerPort";
  my $data = cmd_api($cmd,"GET",1,$cmd);
  if (!defined $data || $data eq ""){return;}
  #print Dumper \$data;

  foreach my $element ( @{$data} ) {
    my $port_id       = $element->{"instanceId"};
    my $serial_number = $element->{"scSerialNumber"};
    my $virtual       = $element->{"virtual"};
    if ( !defined $serial_number || $serial_number ne $instance_storage ) { next; }
    if ( Xormon::isdigit($virtual) && $virtual == 1 )                             { next; }
    my $protocol  = $element->{"transportType"};
    my $port_ids  = $port_id;
    my $port_name = $element->{"name"};
    my $status    = $element->{"status"};
    my $node = "";
    if (defined $element->{controller} && defined $element->{controller}->{instanceName}){
      $node      = $element->{controller}->{"instanceName"};
    }
    my $wwn = "";
    if ( defined $port_id && $port_id ne "" && defined $port_ids && $port_ids ne "" ) {
      if ( defined $serial_number && $port_ids =~ m/^$serial_number\./ ) {
        $port_ids =~ s/$serial_number\.//;
      }
      my $uid = Xormon::create_hash_key($st_serial.$section.$port_ids);
      if (!defined $port_name || $port_name eq ""){
        $port_name = $uid;
        $wwn = undef;
      }
      else{
        $wwn = $port_name;
      }

      $configuration_data{$uid}{TYPE} = $section;
      $configuration_data{$uid}{id} = $port_ids;
      $configuration_data{$uid}{wwn} = $wwn;
      $configuration_data{$uid}{node} = $node;


      if ( defined $status && $status ne "" ) {
        $configuration_data{$uid}{status} = $status;
      }
      if ( defined $protocol && $protocol ne "" ) {
        my $protocol_short = "Unknown";
        if ( $protocol eq "FibreChannel" ) {
          $protocol_short = "FC";
        }
        if ( $protocol eq "FibreChannelOverEthernet" ) {
          $protocol_short = "FCoE";
        }
        if ( $protocol eq "Iscsi" ) {
          $protocol_short = "iSCSI";
        }
        if ( $protocol eq "Sas" ) {
          $protocol_short = "SAS";
        }
        $configuration_data{$uid}{Protocol} = $protocol_short;
        $configuration_data{$uid}{label} = "$port_id $port_name ($protocol_short)";
        if (defined $node && $node ne ""){
          $configuration_data{$uid}{label} = "$node $port_name ($protocol_short)";
        }
      }
    }
  }
  #print Dumper \%configuration_data;
}

sub convert_data{
  my $data = shift;
  my $out;
  my %metrics = convert_metric("perf");
  my $time;
  if (!defined $data->{time} || !Xormon::isdigit(str2time($data->{time}))){
    return ("", "");
  }
  else {
    $time = str2time($data->{time});
  }

  foreach my $metric (keys %{$data}){
    my $denominator = 1;
    my $rounded = 0;
    if (defined $metrics{$metric}){
      if ($metrics{$metric} =~ m/^resp_t/){
        $denominator = 1000;
        $rounded = 3;
      }
      if ($metrics{$metric} =~ m/^read$|^write$|^data_rate$/){
        $denominator = 0.001;
        $rounded = 0;
      }
      #print "$metric $denominator $rounded $data->{$metric}\n";
      $out->{$metrics{$metric}} = round($data->{$metric} / $denominator,$rounded);
    }
  }


  ### total response time
  my $read_io = $out->{read_io};
  my $write_io = $out->{write_io};
  my $resp_t_r = $out->{resp_t_r};
  my $resp_t_w = $out->{resp_t_w};
  my $data_rate = $out->{data_rate};
  my $read = $out->{read};
  my $write = $out->{write};
  my $io_rate = $out->{io_rate};
  my $numerator;
  my $denominator;
  if (Xormon::isdigit($read_io) && Xormon::isdigit($resp_t_r)){
    $numerator = $read_io * $resp_t_r;
    $denominator = $read_io;
  }
  if (Xormon::isdigit($write_io) && Xormon::isdigit($resp_t_w)){
    if (Xormon::isdigit($numerator)){
      $numerator = $numerator + ($write_io * $resp_t_w);
      $denominator = $denominator + $write_io;
    }
    else{
      $numerator = $write_io * $resp_t_w;
      $denominator = $write_io;
    }
  }
  if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator) && $denominator > 0){
    $out->{resp_t} = round($numerator / $denominator,6);
  }
  else{
    if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator) && $numerator == 0 && $denominator == 0){
      $out->{resp_t} = 0;
    }
  }

  ### block_size
  if (Xormon::isdigit($read_io) && Xormon::isdigit($read) && $read_io > 0){
    $out->{block_size_r} = round( $read / $read_io,0);
  }
  if (Xormon::isdigit($read_io) && $read_io == 0){
    $out->{block_size_r} = 0;
  }
  if (Xormon::isdigit($write_io) && Xormon::isdigit($write) && $write_io > 0){
    $out->{block_size_w} = round( $write / $write_io,0);
  }
  if (Xormon::isdigit($write_io) && $write_io == 0){
    $out->{block_size_w} = 0;
  }
  if (Xormon::isdigit($io_rate) && Xormon::isdigit($data_rate) && $io_rate > 0){
    $out->{block_size} = round( $data_rate / $io_rate,0);
  }
  if (Xormon::isdigit($io_rate) && $io_rate == 0){
    $out->{block_size} = 0;
  }


  #print Dumper \$out;

  return ($out,$time);
}

sub convert_metric{
  my $type = shift;
  my %metrics;
  if ($type eq "perf" ){
    $metrics{"readIops"} = "read_io";
    $metrics{"writeIops"} = "write_io";
    $metrics{"totalIops"} = "io_rate";
    $metrics{"readKbPerSecond"} = "read";
    $metrics{"writeKbPerSecond"} = "write";
    $metrics{"totalKbPerSecond"} = "data_rate";
    $metrics{"ReadHitIOs"} = "r_cache_hit";
    $metrics{"WriteHitIOs"} = "w_cache_hit";
    $metrics{"totalLatency"} = "resp_t";
    $metrics{"writeLatency"} = "resp_t_w";
    $metrics{"readLatency"} = "resp_t_r";
    $metrics{"capacity_total"} = "capacity_total";
    $metrics{"capacity_used"} = "capacity_used";
    $metrics{"capacity_free"} = "capacity_free";
    $metrics{ioPending} = "io_pending";
    $metrics{xferLatency} = "resp_t_xfer";
    $metrics{qosLatency} = "resp_t_qos";
    $metrics{cpuPercentUsage} = "cpu_system";
  }


  return %metrics;

}

sub get_health_status {
  my $status_code = shift;
  if ( !Xormon::isdigit($status_code) ) {
    return "$status_code";
  }

  if ( $status_code == 5 ) {
    return "Ok;The element is fully functional and is operating within normal operational parameters and without error";
  }

  if ( $status_code == 30 ) {
    return "Non-recoverable Error;The element has completely failed, and recovery is not possible. All functionality provided by this element has been lost";
  }

  if ( $status_code == 20 ) {
    return "Major Failure;The element is failing. It is possible that some or all of the functionality of this component is degraded or not working";
  }

  if ( $status_code == 10 ) {
    return "Degraded/Warning;The element is in working order and all functionality is provided. However, the element is not working to the best of its abilities. For example, the element might not be operating at optimal performance or it might be reporting recoverable errors";
  }

  if ( $status_code == 15 ) {
    return "Minor Failure;All functionality is available but some might be degraded";
  }

  if ( $status_code == 25 ) {
    return "Critical Failure;The element is non-functional and recovery might not be possible";
  }
  if ( $status_code == 0 ) {
    return "Unknown;The implementation cannot report on HealthState at this time. DMTF has reserved the unused portion of the continuum for additional HealthStates in the future";
  }

  return "$status_code";

}

sub remove_whitespace {
  my $value = shift;
  if ( !defined $value || $value eq "" ) {
    return "";
  }
  $value =~ s/^\s+|\s+$//g;
  return $value;

}

sub get_instance_id{
  my $execute = shift;
  if (!defined $execute){$execute = 0;}
  if (!$execute && defined $service_data_in->{instance_id}){
    return $service_data_in->{instance_id};

  }
  my $cmd = "StorageCenter/GetList";
  my $data = cmd_api($cmd,"POST",$execute,"GetList");
  if ( defined $data && $data ne "" ) {
    foreach my $element ( @{$data} ) {
      my $instance     = $element->{"instanceId"};
      my $name_storage = $element->{"scName"};
      if ( defined $instance && defined $name_storage && $instance ne "" && $name_storage ne "" ) {
        if ( $instance_name eq $name_storage ) {
          ### find right storage
          return $instance;
        }
      }
    }
  }
  return "";


}

sub logout_api{

  my $ua = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $url = "$protocol://$cfg->{'apiHost'}:$cfg->{apiPort}/api/rest/ApiConnection/Logout";

  #print ("Sending to $url\n");
  my $req = HTTP::Request->new( POST => $url );
  $req->header( 'Content-Type'       => 'application/json; charset=utf-8' );
  $req->header( 'Accept'             => 'application/json' );
  $req->header( 'x-dell-api-version' => $api_version );
  $req->header( 'cookie'             => $cookie );                        ### why should be for login?
  $req->authorization_basic( $cfg->{apiUsername}, $cfg->{apiPassword} );
  my $hash = {};
  my $json = encode_json($hash);
  $req->content($json);

  my $con = $req->content;
  if ( $con ne "{}" ) {
    $req->content("{}");
  }
  my $res = $ua->request($req);
  #print $res->content;
  if ( ! $res->is_success ) {
    my $report = $res->content;
    chop $report;
    if (!$connection_test){
      $logger->error("Logout from API failed: $report");
    }
    else{
      push(@rows_api_test,"Logout from API failed: $report\n");
      push(@rows_api_test_data,"Logout from API failed: $report\n");
    }
    #$logger->error("Logout from API failed");
  }
  return;
}

sub login_api{
  my $api_version_x_dell = shift;
  my $last = shift;
  #print "$api_version | $last\n";

  my $ua = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  $ua->timeout(30);
  my $url = "$protocol://$cfg->{'apiHost'}:$cfg->{apiPort}/api/rest/ApiConnection/Login";

  #print ("Sending to $url\n");
  my $req = HTTP::Request->new( POST => $url);
  $req->header( 'Content-Type'       => 'application/json; charset=utf-8' );
  $req->header( 'Accept'             => 'application/json' );
  $req->header( 'x-dell-api-version' => $api_version_x_dell );                        ### why should be for login?
  $req->authorization_basic( $cfg->{apiUsername}, $cfg->{apiPassword} );
  #print Dumper $req;
  my $hash = {};
  my $json = encode_json($hash);
  $req->content($json);

  my $con = $req->content;
  if ( $con ne "{}" ) {
    $req->content("{}");
  }

  my $res = $ua->request($req);
  #if ($connection_test){
  #  print Dumper \$res;
  #}
  if ( !$res->is_success ) {
    if (!$last){return "";}
    my $report = $res->content;
    chop $report;
    if (!$connection_test){
      $logger->error("Request error $report : command : login");
    }
    else{
      push(@rows_api_test,"Request error $report : command : login\n");
      push(@rows_api_test_data,"Request error $report : command : login\n");
    }
    return "";
  }
  else{
    my $result = decode_json( $res->content );
    if ( defined $result->{apiVersion} && $result->{apiVersion} ne "" ) {
      $api_version = $result->{apiVersion};
      if (!$connection_test){
        $service_data_out->{api_version} = $api_version;
      }
    }
    if (defined $res->headers->{"set-cookie"}){
      return $res->headers->{"set-cookie"};
    }
  }
}

sub delete_empty_metrics{
  my $first_time = "";
  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        if ($first_time eq "" && Xormon::isdigit($time)){
          $first_time = $time;
        }
        if (Xormon::isdigit($first_time) && Xormon::isdigit($time) && $time < $first_time){
          $first_time = $time;
        }
        foreach my $item (keys %{$performance_data{$st_type}{$type}{$uid}{$time}}){
          if (!Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item}) || $performance_data{$st_type}{$type}{$uid}{$time}{$item} < 0){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
          if ($item =~ m/numerator|denominator/){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
        }
      }
    }
  }
  if (Xormon::isdigit($first_time)){
    $service_data_out->{last_perf_timestamp} = $first_time;
  }
  else{
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      $service_data_out->{last_perf_timestamp} = $service_data_in->{last_perf_timestamp};
    }
  }
}

sub calculate_missing_metrics{

  my $item_total = "capacity_total";
  my $item_used = "capacity_used";
  my $item_used_percent = "capacity_used_percent";

  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        #capacity_used_percent
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_used})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_total} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_used} / $performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = round($pom,2);
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{data_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && $performance_data{$st_type}{$type}{$uid}{$time}{io_rate} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{data_rate} / $performance_data{$st_type}{$type}{$uid}{$time}{io_rate});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{read_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{read} / $performance_data{$st_type}{$type}{$uid}{$time}{read_io});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{write_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{write} / $performance_data{$st_type}{$type}{$uid}{$time}{write_io});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = 0;
        }

      }
    }
  }
}

sub cmd_api{
  my $cmd = shift;
  my $type = shift;
  my $execute = shift;
  my $debug_file = shift;
  my $hash_tmp = shift;
  my %hash_data;
  my $json_body;
  if (defined $hash_tmp){
    %hash_data = %{$hash_tmp};
    $json_body = encode_json( \%hash_data );
  }
  if (!defined $type || $type eq ""){$type = "GET";}
  if (!defined $execute || $execute eq ""){
    $execute = 1;
  }
  my $url = "$protocol://$cfg->{'apiHost'}:$cfg->{apiPort}/api/rest/StorageCenter/$cmd";
  my $ua = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( $type => $url );
  $req->header( 'Content-Type'       => 'application/json; charset=utf-8' );
  $req->header( 'Accept'             => 'application/json' );
  $req->header( 'x-dell-api-version' => $api_version );
  $req->header( 'cookie'             => $cookie );
  if (defined $json_body){
    $req->content($json_body);
  }
  if (!$execute && defined $debug_file && $debug_file ne "" && -f "$debug_data/$debug_file.json" ){
    #print "bereme data z json file $debug_data/$debug_file.json | $cmd | $type | $execute\n";
    my $data;
    my $file = "$debug_data/$debug_file.json";
    eval { $data = Xormon::read_json($file); };
    if ($@){
      $logger->error("Read file json $file is failed");
      return "";
    }
    else{
      return $data;

    }
  }
  my $res = $ua->request($req);
  #print $res->content;
  if ( ! $res->is_success ) {
    if (!$connection_test){
      $logger->error("API command $cmd failed");
    }
    else{
      push(@rows_api_test,"API command $cmd failed\n");
      push(@rows_api_test_data,"API command $cmd failed\n");

    }
    my $out = $res->content;
    $logger->error($out);
    return "";
  }
  #my $out = $res->content;
  my $data = decode_json( $res->content );
  if (defined $debug_file && $debug_file ne ""){
    eval {Xormon::save_as_json("$debug_data/$debug_file.json",$data);};
    if ($@){
      $logger->error("Save data to file $debug_data/$debug_file.json failed");
    }
  }
  if (defined $json_body){
    my $first_record = @{$data}[-1];
    return $first_record;
  }
  #print Dumper \$data;
  return $data;
}

sub set_architecture{

  my %device_arch = (
    "item_id" => $cfg->{'uuid'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => $st_type,
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'uuid'}
  );
  #if (defined $cfg->{serial_number}){
  #  $device_arch{serial_number} = $cfg->{serial_number};
  #}
  #if (defined $cfg->{version}){
  #  $device_arch{version} = $cfg->{version};
  #}
  #if (defined $cfg->{model}){
  #  $device_arch{model} = $cfg->{model};
  #}
  push(@architecture, \%device_arch);

  foreach my $uid (keys %configuration_data){
    foreach my $metric (keys %{$configuration_data{$uid}}){
      if ($metric =~ m/disk_folder_name|disk_class_name/){
        delete $configuration_data{$uid}{$metric};
      }
    }
  }

  #print Dumper \%configuration_data;
  #return;

  foreach my $uid (keys %configuration_data){
    my $name = $configuration_data{$uid}{label};
    my $type = $configuration_data{$uid}{TYPE};
    if (!defined $name || !defined $type){next;}
    my @children = ();
    my @parents = ();
    if (defined $configuration_data{$uid}{children}){
      @children = @{$configuration_data{$uid}{children}};
      ## delete children and parent from item_properties
      delete $configuration_data{$uid}{children};

    }
    if (defined $configuration_data{$uid}{parents}){
      @parents = @{$configuration_data{$uid}{parents}};
      delete $configuration_data{$uid}{parents};

    }
    #print Dumper \@children;
    #print Dumper \@parents;
    my %arch = (
      "item_id" => $uid,
      "label" => $name,
      "hw_type" => $st_type,
      "subsystem" => $type,
      "parents" => [$cfg->{'uuid'}],
      "hostcfg_id" => $cfg->{'uuid'}
      );
    if (@parents){
      push(@{$arch{parents}},@parents);
    }
    if (@children){
      push(@{$arch{children}},@children);
    }
    if ($type eq "volume"){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($uid);
    }
    if ($type =~ m/port/ && defined $configuration_data{$uid}{wwn}){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
      $configuration_data{$uid}{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
    }
    #push(@{$arch{parents}},$cfg->{'uuid'});
    push(@architecture, \%arch);

  }
  #print Dumper \@architecture;
  #foreach my $item (keys %{$data{conf}{device}}){
  #  $configuration_data{$st_serial}{$item} = $data{conf}{device}{$item};
  #}
  #print Dumper \$configuration_data{device}{$st_serial};

  #save configuration
  my @keys = ("serial", "version", "model", "host", "name", "status", "status_message");
  foreach my $key (@keys){
    if (defined $cfg->{$key}){
      if ($key eq "name"){
        $configuration_data{$cfg->{'uuid'}}{"node_name"} = $cfg->{$key};
      }
      else{
        $configuration_data{$cfg->{'uuid'}}{$key} = $cfg->{$key};
      }
    }
  }
  #event log
  $configuration_data{$cfg->{'uuid'}}{alerts} = encode_json(\@health_status);
  #print Dumper \$configuration_data{$cfg->{'uuid'}}{alerts};
  #print Dumper \@architecture;
  ### device information save


  my $rc = $xormon->saveConfJSON(\%configuration_data);
  my $status_configuration = get_status($rc, "saveConfJSON (configuration)");
  #$logger->info("Task for configuration ".$cfg->{hostalias}." is done. Status: $status");

  #save architecture
  #print Dumper \@architecture;

  $rc = $xormon->saveArchitecture(\@architecture);
  my $status_architecture = get_status($rc, "saveArchitecture");
  #$logger->info("Task for saveArchitecture ".$cfg->{hostalias}." is done. Status: $status");

  $rc = $xormon->saveStatus(\@hs_global);
  my $status_health = get_status($rc, "health status");

  return ($status_architecture, $status_configuration, $status_health);
}


sub connection_test {
  my $client = shift;
  $cfg = shift;
  $connection_test = 1;


  #print Dumper \$cfg;
  #print "aa\n";
  # test main variables
  my $test = Test->new($client);
  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiPort", "apiUsername", "apiPassword", "storageName");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  #unless ( exists $cfg->{'hostcfg_id'} )    { push(@rows_api_test, "Cannot continue, 'hostcfg_id' is not configured!");    return Xormon::conn_test(0, \@rows_api_test); }
  #unless ( exists $cfg->{'hostalias'} )   { push(@rows_api_test, "Cannot continue, 'hostalias' is not configured!"); return Xormon::conn_test(0, \@rows_api_test);  }
  #unless ( exists $cfg->{'apiHost'} )          { push(@rows_api_test, "Cannot continue, 'apiHost' is not configured!"); return Xormon::conn_test(0, \@rows_api_test); }
  #unless ( exists $cfg->{'apiPort'} )      { push(@rows_api_test, "Cannot continue, 'apiPort' is not configured!"); return Xormon::conn_test(0, \@rows_api_test);}
  #unless ( exists $cfg->{'apiUsername'} )      { push(@rows_api_test, "Cannot continue, 'apiUsername' is not configured!"); return Xormon::conn_test(0, \@rows_api_test);}
  #unless ( exists $cfg->{'apiPassword'} )      { push(@rows_api_test, "Cannot continue, 'apiPassword' is not configured!"); return Xormon::conn_test(0, \@rows_api_test); }
  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  $logger = XormonLogger->new( 'compellent', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  $storage_name = $cfg->{hostalias};
  $instance_name = $cfg->{storageName};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  $protocol = "https";

  data_structure();

  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($cfg->{'apiHost'},$cfg->{'apiPort'});
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }
  #push(@rows_api_test, "Code is " . $code . " Text is " . $text);
  #return Xormon::conn_test(0, \@rows_api_test);


  for ( my $i = 1; $i < 11; $i++ ) {
    for ( my $j = 0; $j < 11; $j++ ) {
      $api_version = "$i.$j";
      if ($i == 10 && $j == 10){
        $cookie = login_api($api_version,1);
      }
      else{
        $cookie = login_api($api_version,0);
        if ($cookie ne ""){
          last;
        }
      }
    }
    if ($cookie ne ""){last;}
  }
  if (!defined $cookie || $cookie eq ""){
    push(@rows_api_test, "Cookie is not defined\n");
    #my $scal = join( "\n", @rows_api_test );
    $test->addSection("API authorization", 0, \@rows_api_test);
    return $test->stop();
    #return Xormon::conn_test(0, \@rows_api_test);
  }
  #return $test->genResult();
  $instance_storage = get_instance_id(1);
  if ( !defined $instance_storage || $instance_storage eq "" ) {
    push(@rows_api_test,"ERROR: Instance Storage \"$instance_name\" is not defined!\n");
    push(@rows_api_test,"ERROR: Storage cannot be accessed, check if user \"$cfg->{apiUsername}\" has allowed storage \"$instance_name\"\n");
    logout_api();
    #my $scal = join( "\n", @rows_api_test );
    $test->addSection("API authorization", 0, \@rows_api_test);
    return $test->stop();
    #return Xormon::conn_test(0, \@rows_api_test);
  }
  ### version
  my $cmd = "StorageCenter";
  my $data = cmd_api($cmd,"GET",1,$cmd);
  if (!defined $data || $data eq "" || ref($data) ne "ARRAY"){
    push(@rows_api_test, "ERROR: API request $cmd failed\n");
    logout_api();
    #return Xormon::conn_test(0, \@rows_api_test);
    #my $scal = join( "\n", @rows_api_test );
    $test->addSection("API authorization", 0, \@rows_api_test);
    return $test->stop();
  }
  foreach my $element ( @{$data} ) {
    my $machine_name   = $element->{"instanceName"};
    my $status         = $element->{"status"};
    my $model          = $element->{"modelSeries"};
    my $version        = $element->{"version"};
    my $serial_number  = $element->{"scSerialNumber"};
    if ( !defined $serial_number || $serial_number ne $instance_storage ) { next; }
    if ( !defined $status ) {
      $status = "";
    }
    if ( !defined $model ) {
      $model = "";
    }
    if ( !defined $version ) {
      $version = "";
    }
    push(@rows_api_test,"Version of the Storage Center : $version\n");
    push(@rows_api_test,"Model : $model\n");
    push(@rows_api_test,"Status : $status\n");
    push(@rows_api_test,"API version : $api_version\n");
  }
  $test->addSection("API authorization", 1, \@rows_api_test);
  ###

  $cmd = "ScVolume";
  my $cmd_perf = "ScVolumeIoUsage/GetFilter";
  $data = cmd_api($cmd,"GET",1,$cmd);
  if (!defined $data || $data eq ""){
    push(@rows_api_test_data, "ERROR: Volume configuration failed\n");
    logout_api();
    #return Xormon::conn_test(0, \@rows_api_test);
    $test->addSection("API data test", 0, \@rows_api_test_data);
    return $test->stop();
  }
  my %hash_filter;
  $hash_filter{"HistoricalFilter"}{"FilterTime"}    = "Other";
  $hash_filter{"HistoricalFilter"}{"UseEndOfDay"}   = "true";
  $hash_filter{"HistoricalFilter"}{"UseStartOfDay"} = "true";
  $hash_filter{"HistoricalFilter"}{"UseCurrent"}    = "true";
  foreach my $element ( @{$data} ) {
    my $volume_instance = $element->{"instanceId"};
    my $volume_id       = $element->{"deviceId"};
    my $volume_name     = $element->{"name"};
    my $storage_id      = $element->{"scSerialNumber"};
    if ( !defined $storage_id || $storage_id ne $instance_storage ) { next; }
    if ( defined $volume_instance && $volume_instance ne "" && defined $volume_id && $volume_id ne "" ) {
      $hash_filter{"ObjectInstanceId"} = "$volume_instance";
      my $data = cmd_api($cmd_perf,"POST",1,"",\%hash_filter);
      if ( defined $data && $data ne "" ) {
        #push(@rows_api_test,"API VERSION: $api_version");
        #push(@rows_api_test,"Configuration data: ok");
        push(@rows_api_test_data,"Performance data ok\n");
        last;
      }
      else{
        push(@rows_api_test_data, "ERROR: Volume performance failed\n");
        $connection_test = 0;
        last;
      }
    }
  }
  logout_api();
  $test->addSection("API data test", $connection_test, \@rows_api_test_data);
  return $test->stop();
  #return Xormon::conn_test($connection_test, \@rows_api_test);
}

sub data_structure {
  if (!-d "$data_directory/data"){
    mkdir "$data_directory/data" || $logger->error( "Cannot mkdir $data_directory/data: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial"){
    mkdir "$data_directory/data/$st_serial" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial/IOSTATS"){
    mkdir "$data_directory/data/$st_serial/IOSTATS" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial/IOSTATS: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }


  my $directory = "$data_directory/data/$st_serial/IOSTATS";
  if ( !-d $directory ) {
    mkdir $directory || $logger->error( "Cannot mkdir $directory: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || $logger->error( "Cannot mkdir $directory/configuration: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || $logger->error( "Cannot mkdir $directory/0$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || $logger->error( "Cannot mkdir $directory/$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
  my @entry_time = split( " ", localtime() );
  ( undef, $active_minute, undef ) = split( ":", $entry_time[3] );
  if ( !-d "$directory/$active_minute" ) {
    mkdir "$directory/$active_minute" || $logger->error( "Cannot mkdir $directory/$active_minute: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  $debug_data = "$directory/configuration";
  $wrkdir = "$data_directory/data/$st_serial";

} ## end sub data_structure

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

1;
