package Db2;

use strict;
use warnings;

use JSON;
use Data::Dumper;
use Digest::MD5 qw(md5 md5_hex md5_base64);
use File::Temp qw(tempfile);
use File::Copy;



my @health_status;
my %arc;
my @perf_types = ( "size","member","member","main","bufferpool");
my $alias;
my $mvc_dir = $FindBin::Bin;
my $iostats_dir = "$mvc_dir/iostats";
my @architecture;

my %query = (
  "member" => "SELECT * FROM TABLE(MON_GET_DATABASE(-2))",
  "bufferpool" => qq(SELECT "BP_NAME", "POOL_TEMP_XDA_P_READS", "POOL_TEMP_DATA_L_READS", "POOL_TEMP_XDA_L_READS", "POOL_INDEX_L_READS", "POOL_XDA_L_READS", "POOL_DATA_L_READS", "POOL_DATA_WRITES", "POOL_INDEX_WRITES", "POOL_XDA_WRITES", "DIRECT_WRITE_TIME", "POOL_INDEX_P_READS", "POOL_WRITE_TIME", "DIRECT_WRITES", "POOL_TEMP_INDEX_P_READS", "DIRECT_READS", "POOL_DATA_P_READS", "POOL_TEMP_DATA_P_READS", "DIRECT_READ_TIME", "POOL_XDA_P_READS", "POOL_TEMP_INDEX_L_READS", "POOL_READ_TIME" FROM TABLE(MON_GET_BUFFERPOOL('',-2));)
);

my %c_query = (
  "size" => "
      SELECT varchar(tbsp_name, 30) as tbsp_name,
        TBSP_TOTAL_PAGES * TBSP_PAGE_SIZE AS SIZE,
        TBSP_FREE_PAGES * TBSP_PAGE_SIZE AS FREE,
        (TBSP_PAGE_TOP * 1.0 / TBSP_TOTAL_PAGES) * 100 AS USED_PCT,
        TBSP_PAGE_TOP * TBSP_PAGE_SIZE AS MAXUSED
      FROM TABLE(MON_GET_TABLESPACE('',-2)) AS t 
      ORDER BY free ASC;",
	"members" => "
       SELECT ID,
         HOME_HOST AS HOME_HOST, 
         CURRENT_HOST AS CUR_HOST, 
         STATE AS STATE, 
         ALERT 
			 FROM SYSIBMADM.DB2_MEMBER;",
  "main" => "SELECT MEMBER, DB2_STATUS,TIMEZONEID,PRODUCT_NAME,SERVER_PLATFORM,SERVICE_LEVEL,DB2START_TIME,AGENTS_REGISTERED,IDLE_AGENTS,AGENTS_STOLEN FROM TABLE(MON_GET_INSTANCE(-2));",
);

sub get_stats {
  my $db_name     = shift;
  my $port        = shift;
  my $username    = shift;
  my $password    = shift;
  my $ip          = shift;
  $alias          = shift;
  my $uuid        = shift;
  my %whole_perf;

  my $module = "DBD::DB2";
  my $module_def = 1;
  eval "use $module; 1" or $module_def = 0;
  if ( !$module_def ) {
    $whole_perf{error} = "no module";
    return \%whole_perf;
  }
  require DBD::DB2::Constants;
  require DBD::DB2;
  require DBI;

  $arc{hostnames}{ $uuid }{alias} = $alias;
  my %cluster_arch = (
    "item_id" => $uuid,
    "label" => $alias,
    "hw_type" => "DB2",
    "subsystem" => "DBFOLDER",
    "parents" => [],
    "hostcfg_id" => $uuid
  );
  push(@architecture, \%cluster_arch);


  my $pre_perf = get_data("PERF", $db_name, $db_name, $ip, $port, $username, $password, $uuid);
  #print Dumper $pre_perf;
  my $act_perf = make_perf($pre_perf, $uuid);
  my $perf = work_counters($act_perf,$uuid);

  if ( $perf and $perf ne 0 ) {
    write_json( "$iostats_dir/db_perf.json-nperf", $perf );
    for my $subsystem (keys %{$perf}){
      next if ($subsystem eq "_info");
      for my $name (keys %{$perf->{$subsystem}}){
        my $db_id = Xormon::create_hash_key_long("$uuid-$alias-$name");

        if ($subsystem eq "_cluster"){


          $whole_perf{perf}{db2}{$subsystem}{$db_id}{$perf->{_info}{timestamp}} = $perf->{$subsystem}{$name};
        }else{
          my %arch = (
            "item_id" => $db_id,
            "label" => $name,
            "hw_type" => "DB2",
            "subsystem" => "$subsystem",
            "parents" => [$uuid],
            "hostcfg_id" => $uuid
          );
  
          push(@architecture, \%arch);
          #print Dumper $act_conf;
          #print "$subsystem\n";
          #if($act_conf->{_cluster}->{flgrps}->{$subsystem}){
          #  $cnf{$db_id}{flgrps} = encode_json($act_conf->{_cluster}->{flgrps}->{$subsystem});
          #}
          #if($act_conf->{_cluster}->{datafiles2}->{$subsystem}){
          #  $cnf{$db_id}{datafiles2} = encode_json($act_conf->{_cluster}->{datafiles2}->{$subsystem});
          #}
          #print Dumper $act_conf->{_cluster}->{flgrps}->{$subsystem};
  
          $whole_perf{perf}{db2}{$subsystem}{$db_id}{$perf->{_info}{timestamp}} = $perf->{$subsystem}{$name};
        }
      }
    }
    $whole_perf{hasData} = 1;
  } else {
    $whole_perf{hasData} = 0;
  }
  my $act_conf = make_conf($pre_perf, $uuid);
  #warn Dumper $act_conf;
    
  if($act_conf->{_cluster}{size}){
    $whole_perf{conf}{$uuid}{db2size} = encode_json($act_conf->{_cluster}{size});
  }
  if($act_conf->{_cluster}{main}){
    $whole_perf{conf}{$uuid}{db2main} = encode_json($act_conf->{_cluster}{main});
  }
  if($act_conf->{_cluster}{member}){
    $whole_perf{conf}{$uuid}{db2member} = encode_json($act_conf->{_cluster}{member});
  }

  $whole_perf{arch} = \@architecture;
  $whole_perf{health_status} = \@health_status;
  #print Dumper $act_conf;
  #warn Dumper $whole_perf{conf};
  #warn Dumper \%whole_perf;
  return \%whole_perf;

}

sub make_perf {
  my $_data = shift;
  my $uuid  = shift;
  my %_data = %{$_data};

  my %perf;
  my $mem_total;
  foreach my $type ( @perf_types ) {

    if ( $type eq "member" ) {

      foreach my $row ( @{ $_data{$type}{data} } ) {
        my %clean_row = clean_data($row);
        #$mem_total = add_to_hash($mem_total, \%clean_row);
        my $n_id =  Xormon::create_hash_key_long("$row->{MEMBER}-$alias"); 
        $arc{hostnames}{ $uuid }{_dbs}{$n_id}{label} = "$row->{MEMBER}";

        $perf{$type}{ $row->{MEMBER} } = \%clean_row;
       }
    }elsif ( $type eq "bufferpool" ) {

      foreach my $row ( @{ $_data{$type}{data} } ) {
        my %clean_row = clean_data($row);
        
        my $n_id =  Xormon::create_hash_key_long("$row->{BP_NAME}-$alias"); 
        $arc{hostnames}{ $uuid }{_bps}{$n_id}{label} = "$row->{BP_NAME}";

        $perf{$type}{ $row->{BP_NAME} } = \%clean_row;
       }
    }elsif ( $type eq "main" ) {
      foreach my $row ( @{ $_data{$type}{data} } ) {
        my %row_dup = %{$row};
        delete $row_dup{'MEMBER'};
        delete $row_dup{'DB2_STATUS'};
        delete $row_dup{'TIMEZONEID'};
        delete $row_dup{'PRODUCT_NAME'};
        delete $row_dup{'SERVER_PLATFORM'};
        delete $row_dup{'SERVICE_LEVEL'};
        delete $row_dup{'DB2START_TIME'};
        #$mem_total = add_to_hash($mem_total, \%row_dup);
      
        $perf{member}{ $row->{MEMBER} }{ 'AGENTSSTOLEN'}     = $row->{ 'AGENTS_STOLEN'}; 
        $perf{member}{ $row->{MEMBER} }{ 'IDLEAGENTS'}       = $row->{ 'IDLE_AGENTS'};   
        $perf{member}{ $row->{MEMBER} }{ 'AGENTSREGISTERED'} = $row->{ 'AGENTS_REGISTERED'};
      }
    }
  }
  $perf{_info}{timestamp}     = time;
  $perf{_info}{readable_time} = localtime( $perf{_info}{timestamp} );

  return \%perf;
}

sub make_conf {
  my $_data = shift;
  my $uuid  = shift;
  my %_data = %{$_data};
  my %conf_types = (
    "size"   => 1,
    "members" => 1,
    "main"   => 1,
  );

  my %perf;
  for my $type ( keys %conf_types ) {
    #print Dumper \%perf;
    if ( $type eq "size" and $_data{$type}) {
      foreach my $row ( @{ $_data{$type}{data} } ) {
        my %hash;
        my %nr;
        my %row_dup = %{$row};
        $hash{'Free'}    = $row_dup{'FREE'};
        $hash{'Used'}    = $row_dup{'MAXUSED'};
        $hash{'Used %'}  = sprintf( "%.0f", $row_dup{'USED_PCT'});
        $hash{'Total'}   = $row_dup{'SIZE'};
        $hash{'Name'}    = $row_dup{'TBSP_NAME'};
        push(@{$perf{_cluster}{size}}, \%hash);
#        $hash{'Free'}    += $row_dup{'FREE'}; 
#        $hash{'Used'}    += $row_dup{'MAXUSED'}; 
#        $hash{'Total'}   += $row_dup{'SIZE'}; 
      }
 #     $hash{'Name'}    = "Total"; 
 #     $hash{'Total'}   = $hash{'Total'} ? $hash{'Total'} : 1;
 #     $hash{'Used'}    = $hash{'Used'} ? $hash{'Used'} : 1;
 #     $hash{'Used %'}  = sprintf( "%.0f", ($hash{'Used'} / $hash{'Total'}) * 100); 
    }elsif ( $type eq "members" ) {
      my $row_i = 0;
      $perf{_cluster}{member} = [];
      foreach my $row ( @{ $_data{$type}{data} } ) {
        my %row_dup = %{$row};
        my %nr;
        my %hash;
        $hash{'ID'}        = $row_dup{'ID'};
        $hash{'ALERT'}     = $row_dup{'ALERT'};
        $hash{'STATE'}     = $row_dup{'STATE'};
        $hash{'CUR_HOST'}  = $row_dup{'CUR_HOST'};
        $hash{'HOME_HOST'} = $row_dup{'HOME_HOST'};
        push(@{$perf{_cluster}{member}}, \%hash);
      }
    }elsif ( $type eq "main" ) {
      my $row_i = 0;
      foreach my $row ( @{ $_data{$type}{data} } ) {
        my %row_dup = %{$row};
        my %nr;
        my %hash;
        $hash{'Member'}          = $row_dup{'MEMBER'};
        $hash{'Status'}          = $row_dup{'DB2_STATUS'};
        $hash{'Timezone'}        = $row_dup{'TIMEZONEID'};
        $hash{'Product name'}    = $row_dup{'PRODUCT_NAME'};
        $hash{'Server platform'} = $row_dup{'SERVER_PLATFORM'};
        $hash{'Service level'}   = $row_dup{'SERVICE_LEVEL'};
        $hash{'Start time'}      = $row_dup{'DB2START_TIME'};
        push(@{$perf{_cluster}{main}}, \%hash);
      }
    }
  }

  $perf{_info}{timestamp}     = time;
  $perf{_info}{readable_time} = localtime( $perf{_info}{timestamp} );

  return \%perf;
}

sub clean_data {
  my $data = shift;

   my @perf_metrics = (
    "AGENTS_TOP",
    "NUM_POOLED_AGENTS",
    "NUM_ASSOC_AGENTS",
    "NUM_COORD_AGENTS",
    "NUM_LOCKS_HELD",
    "NUM_LOCKS_WAITING",
    "LOCK_ESCALS",
    "LOCK_TIMEOUTS",
    "DEADLOCKS",
    "POOL_DATA_L_READS",
    "POOL_TEMP_DATA_L_READS",
    "POOL_INDEX_L_READS",
    "POOL_TEMP_INDEX_L_READS",
    "POOL_XDA_L_READS",
    "POOL_TEMP_XDA_L_READS",
    "POOL_DATA_P_READS",
    "POOL_TEMP_DATA_P_READS",
    "POOL_INDEX_P_READS",
    "POOL_TEMP_INDEX_P_READS",
    "POOL_XDA_P_READS",
    "POOL_TEMP_XDA_P_READS",
    "POOL_DATA_WRITES",
    "POOL_INDEX_WRITES",
    "POOL_XDA_WRITES",
    "DIRECT_READS",
    "DIRECT_WRITES",
    "DIRECT_READ_TIME",
    "POOL_READ_TIME",
    "POOL_WRITE_TIME",
    "ROWS_MODIFIED",
    "ROWS_RETURNED",
    "ROWS_READ",
    "ROWS_UPDATED",
    "ROWS_DELETED",
    "ROWS_INSERTED",
    "INT_ROWS_DELETED",
    "INT_ROWS_INSERTED",
    "INT_ROWS_UPDATED",
    "FED_ROWS_DELETED",
    "FED_ROWS_INSERTED",
    "FED_ROWS_UPDATED",
    "FED_ROWS_READ",
    "TCPIP_SEND_VOLUME",
    "TCPIP_RECV_VOLUME",
    "IPC_SEND_VOLUME",
    "IPC_RECV_VOLUME",
    "FCM_SEND_VOLUME",
    "FCM_RECV_VOLUME",
    "PKG_CACHE_INSERTS",
    "PKG_CACHE_LOOKUPS",
    "TOTAL_APP_COMMITS",
    "TOTAL_APP_ROLLBACKS",
    "DIRECT_WRITE_TIME",
    "LOG_DISK_WAIT_TIME",
    "TCPIP_SEND_WAIT_TIME",
    "TCPIP_RECV_WAIT_TIME",
    "IPC_SEND_WAIT_TIME",
    "IPC_RECV_WAIT_TIME",
    "FCM_SEND_WAIT_TIME",
    "FCM_RECV_WAIT_TIME",
    "CF_WAIT_TIME",
    "CLIENT_IDLE_WAIT_TIME",
    "LOCK_WAIT_TIME",
    "AGENT_WAIT_TIME",
    "WLM_QUEUE_TIME_TOTAL",
    "CONNECTIONS_TOP",
    "TOTAL_CONS",
    "TOTAL_SEC_CONS",
    "APPLS_CUR_CONS",
  );

  my %new_perf;
  for my $metric (@perf_metrics){
    if (defined $data->{$metric}){
      my $clean_metric = $metric;
      $clean_metric =~ s/_//g;
      $new_perf{$clean_metric} = $data->{$metric};
    }#else{
     # #$new_perf{$metric} = 0;
     #}
  }
  $new_perf{LOGICALREADS}  = $new_perf{POOLDATALREADS} + $new_perf{POOLTEMPDATALREADS} + $new_perf{POOLINDEXLREADS} + $new_perf{POOLTEMPINDEXLREADS} + $new_perf{POOLXDALREADS} + $new_perf{POOLTEMPXDALREADS};
  $new_perf{PHYSICALREADS} = $new_perf{POOLDATAPREADS} + $new_perf{POOLTEMPDATAPREADS} + $new_perf{POOLINDEXPREADS} + $new_perf{POOLTEMPINDEXPREADS} + $new_perf{POOLXDAPREADS} + $new_perf{POOLTEMPXDAPREADS};
  $new_perf{WRITES}        = $new_perf{POOLDATAWRITES} + $new_perf{POOLINDEXWRITES} + $new_perf{POOLXDAWRITES};
  $new_perf{POOLREADS}     = $new_perf{LOGICALREADS} + $new_perf{PHYSICALREADS};
  $new_perf{POOLWRITES}    = $new_perf{WRITES};

  return %new_perf;
}

sub add_to_hash {
  my $prev_hash = shift;
  my $cur_hash  = shift;

  if(keys %{$prev_hash} == 0){
    return $cur_hash;
  }else{
    for my $key (keys %{$cur_hash}){
      if(defined $prev_hash->{$key}){
        $prev_hash->{$key} = $prev_hash->{$key} + $cur_hash->{$key};
      }else{
        $prev_hash->{$key} = $cur_hash->{$key};
      }
    }
    return $prev_hash;
  }
}

sub get_data {
  my $_type     = shift;
  my $_db_name  = shift;
  my $_db       = shift;
  my $_hostname = shift;
  my $_port     = shift;
  my $_user     = shift;
  my $_pass     = shift;
  my $_uuid     = shift;



  my $string = "dbi:DB2:DATABASE=$_db; HOSTNAME=$_hostname; PORT=$_port; PROTOCOL=TCPIP; UID=$_user; PWD=$_pass;";
  my $dbh = DBI->connect($string, { PrintError => 0 } );

  my $result;

  my $time_is_now = time;
  my $err         = 0;
  $err = DBI->errstr;

  if($err and !(defined $dbh)){ 
    if(defined $_uuid){
      push(@health_status, { "item_id" => $_uuid, "status" => "error", "data" => $err });
      push(@health_status, { "item_id" => $_uuid, "type" => "connection", "status" => "error", "data" => $err });
    }
  }else{
    if(defined $_uuid){
      push(@health_status, { "item_id" => $_uuid, "status" => "ok", "data" => "" });
      push(@health_status, { "item_id" => $_uuid, "type" => "connection", "status" => "ok", "data" => "" });
    }
  }

  if ( !( defined $dbh ) ) {
    warn $err;
    return "err";
  }

  my %_data;

  #trade
  for my $query_name ( keys %query ) {
    print "$query_name\n\n";
    my $array_ref = get_query_result( $dbh, $query{$query_name} );
    $_data{$query_name}{data} = $array_ref;
    $_data{$query_name}{info}{db_name} = $_db_name;
  }

  if (1 or can_update("$iostats_dir/size_hourly", 3600, 1)){
    for my $query_name ( keys %c_query ) {
      print "$query_name\n\n";
      my $array_ref = get_query_result( $dbh, $c_query{$query_name} );
      $_data{$query_name}{data} = $array_ref;
      $_data{$query_name}{info}{db_name} = $_db_name;
    }
  }

  # clean up
  $dbh->disconnect();

  return \%_data;
}

sub get_query_result {
  my $_dbh  = shift;
  my $query = shift;
  my @array;

  #warn $query;
  my $sth = $_dbh->prepare($query);
  $sth->execute();

  while ( my $ref = $sth->fetchrow_hashref() ) {
    push( @array, $ref );
  }
  return \@array;
}

sub convert_to_gb {
  my $_value  = shift;
  my $type    = shift;
  my $new_val = $_value;
  if ( defined $new_val and $new_val =~ / KB/ ) {
    $new_val  =~ s/ KB//g;
    $new_val /= 1000;
  }
  elsif ( defined $new_val and $new_val =~ / MB/ ) {
    $new_val  =~ s/ MB//g;
    $new_val /= 1;
  }
  elsif ( defined $new_val and $new_val =~ / GB/ ) {
    $new_val  =~ s/ GB//g;
  }
  elsif ( defined $new_val and $new_val =~ / TB/ ) {
    $new_val  =~ s/ TB//g;
    $new_val *= 1;
  }
  else {
    warn "$_value, $type";
    return 0;
  }

  return $new_val;
}

sub work_counters {
  my $_data    = shift;
  my $_uuid    = shift;
  my %act_perf = %{$_data};
  my %new_perf;
  my %old_perf;
  my $iostats_dir = "$mvc_dir/iostats";


  unless ( -d $iostats_dir ) {
    mkdir( "$iostats_dir", 0755 ) || warn( "Cannot mkdir $iostats_dir: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }


  my $perf_l = "$iostats_dir/".$_uuid."_db_perf_last.json";
  my ( $p_can_read, $p_ref );
  my $timediff;
  if ( -e $perf_l ) {
    my $modtime  = ( stat($perf_l) )[9];
    $timediff = time - $modtime;
    if ( $timediff >= 500 ) {
      unlink($perf_l);
      write_json( "$perf_l", $_data );
      return 0;
    }
    ( $p_can_read, $p_ref ) = read_json($perf_l);
  }
  else {
    write_json( "$perf_l", $_data );
    return 0;
  }
  if ($p_can_read) {
    %old_perf = %{$p_ref};
    #print Dumper $p_ref;
    #print Dumper \%act_perf;
    my @dbs;
    my %raw_metrics = %{get_raw_metrics()};
    my %lp_metrics  = %{get_lastperiod_metrics()};
    for my $type ( keys %act_perf ) {
      if ( $type ne "_info" ) {
        for my $db ( keys %{ $act_perf{$type} } ) {
          for my $metric ( keys %{ $act_perf{$type}{$db} } ) {

            #            if(!$non_counters{$metric}){
            next if ( !defined $old_perf{$type}{$db}{$metric}
            #  or $old_perf{$type}{$db}{$metric} eq "0"
            #  or $old_perf{$type}{$db}{$metric} == 0 #return after testing
              or $act_perf{$type}{$db}{$metric} < $old_perf{$type}{$db}{$metric} );
            if ($raw_metrics{$metric}) {
              $new_perf{$type}{$db}{$metric} = $act_perf{$type}{$db}{$metric};
            }elsif($lp_metrics{$metric}){
              $new_perf{$type}{$db}{$metric} = ( $act_perf{$type}{$db}{$metric} - $old_perf{$type}{$db}{$metric});
            }
            else {
              $new_perf{$type}{$db}{$metric} = ( $act_perf{$type}{$db}{$metric} - $old_perf{$type}{$db}{$metric}) / 300;#$timediff
            }

            if ($alias eq "DB2test"){
              $new_perf{$type}{$db}{$metric} = $act_perf{$type}{$db}{$metric};
            }
          }
          my %ratios = (
            'DIRECTREADTIME'     => ['DIRECTREADTIME','DIRECTREADS'],
            'DIRECTWRITETIME'    => ['DIRECTWRITETIME','DIRECTWRITES'],
            'POOLREADTIME'       => ['POOLREADTIME','POOLREADS'],
            'POOLWRITETIME'      => ['POOLWRITETIME','POOLWRITES'],
            'LOGDISKWAITTIME'    => ['LOGDISKWAITTIME','TOTALIO'],
            'RATINLOG'           => ['PHYSICALREADS','LOGICALREADS'],
          );
 
          for my $r_type ( keys %ratios ) {
            if ( $new_perf{$type}{$db}{$ratios{$r_type}[0]} and $new_perf{$type}{$db}{ $ratios{$r_type}[1] } ) {
              if ($r_type eq "RATINLOG"){
                $new_perf{$type}{$db}{$r_type}  = 1 - $new_perf{$type}{$db}{$ratios{$r_type}[0]} / $new_perf{$type}{$db}{ $ratios{$r_type}[1] };
                $new_perf{$type}{$db}{$r_type} *= 100;
              }else{
                $new_perf{$type}{$db}{$r_type}  = $new_perf{$type}{$db}{$ratios{$r_type}[0]} / $new_perf{$type}{$db}{ $ratios{$r_type}[1] };
              }
            }
          }
          push( @dbs, $db );
        }
      }
    }

    $new_perf{_info} = $act_perf{_info};

    #$new_perf{_vacuum} = $act_perf{_vacuum};
    write_json( $perf_l, \%act_perf );
    return \%new_perf;
  }
  else {
    write_json( $perf_l, \%act_perf );
    return 0;
  }
}

sub basename {
  my $full      = shift;
  my $separator = shift;
  my $out = "";
  #my $length = length($full);
  if(defined $separator and defined $full and  index($full, $separator) != -1){
    $out = substr($full, length($full) - index(reverse($full), $separator), length($full));
    return $out;
  }
  return $full;
}

sub can_update {
  my $file = shift;
  my $time = shift;
  my $reset = shift;

  my $run_conf     = 0;
  my $checkup_file = $file;
  if ( -e $checkup_file ) {
    my $timediff = get_file_timediff($checkup_file);
    if ( $timediff >= $time - 100 ) {
      $run_conf = 1;
      if (defined $reset and $reset eq "1"){
        open my $fh, '>', $checkup_file;
        print $fh "1\n";
        close $fh;
      }
    }
  }
  elsif ( !-e $checkup_file ) {
    $run_conf = 1;
    if (defined $reset and $reset eq "1"){
      open my $fh, '>', $checkup_file;
      print $fh "1\n";
      close $fh;
    }
  }

  return $run_conf;
}

sub get_lastperiod_metrics {
    my %raw_metrics = (
      'NUMLOCKSHELD'     => "1",
      'NUMLOCKSWAITING'  => "1",
      'DEADLOCKS'        => "1",
      'LOCKESCALS'       => "1",
      'LOCKTIMEOUTS'     => "1",
    );

  return \%raw_metrics;
}


sub get_raw_metrics {
    my %raw_metrics = (
      'AGENTSTOP'         => "1",
      'TOTALCONNECTIONS'  => "1",
      'NUMPOOLEDAGENTS'   => "1",
      'NUMASSOCAGENTS'    => "1",
      'NUMCOORDAGENTS'    => "1",
      'AGENTSREGISTERED'  => "1",
      'IDLEAGENTS'        => "1",
      'AGENTSSTOLEN'      => "1",
      'APPLSCURCONS'      => "1",
      'CONNECTIONSTOP'    => "1",
      'TOTALSECCONS'      => "1",
    );

  return \%raw_metrics;
}


sub get_file_timediff {
  my $file = shift;

  my $modtime  = ( stat($file) )[9];
  my $timediff = time - $modtime;

  return $timediff;
}

sub write_json {
  my $path        = shift;
  my $hash_ref    = shift;
  my $path_suffix = "$path-tmp" . $$;
  my $json        = JSON->new->utf8;

  my ($fh, $file_name) = tempfile( UNLINK => 1 ); #use tempfile instead of the $path_suffix (HD)

  if ( ref($hash_ref) ne "HASH" && ref($hash_ref) ne "ARRAY"){
    warn ("Hash ref or array ref expected (path:$path), got: \"$hash_ref\" (Ref:" . ref($hash_ref) . ") in \n");
    return 0;
  }
#  if ($ENV{JSON_PRETTY}){
    $json->pretty([1]);
#  }

  #open( my $FILE, '>', $path_suffix ) || error( "Couldn't open file $path_suffix $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  #print $FILE $json->encode( $hash_ref );
  #close $FILE;
  #rename($path_suffix, $path);

  print $fh $json->encode( $hash_ref );
  close($fh);

  copy($file_name, $path);

  return 1;
}

sub read_json {
  my $file = shift;
  my $json = JSON->new->utf8;
  my $string;
  my $output;

  {
    local $/=undef;
    open ( my $FILE, '<', $file ) or warn "Couldn't open file $file $!" && return ( 0, undef );
    $string = <$FILE>;
    close $FILE;
  }

  if ($string) {
    eval {
      $output = $json->decode($string);
      1;
    } or do {
      warn "Couldn't parse json file $file $@ " && return ( 0, undef );
    };
  }

  return ( 1, $output );
}

1;