package DOTHILL;

use strict;
use warnings;

use Data::Dumper;
use JSON qw(encode_json decode_json);
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Xormon;
use Date::Parse;
use POSIX ":sys_wait_h";
use LWP::UserAgent;
use XML::LibXML;
use File::Copy;
use File::Temp qw(tempfile);
use Test;
use XormonLogger;
my $logger;



#my $md5module = 1;
#eval "use Digest::MD5 qw(md5_hex); 1" or $md5module = 0;

#if ( !$md5module ) {    # binary MD5 module not found, use bundled pure Perl one
#  use Digest::Perl::MD5 qw(md5_hex);
#}

### check MD5 module
my $md5module = 1;
eval "use Digest::MD5 qw(md5_hex); 1" or $md5module = 0;

if ( !$md5module ) {    # binary MD5 module not found, use bundled pure Perl one
  eval "use Digest::Perl::MD5 qw(md5_hex); 1" or $md5module = 0;
}

require "xml.pl";



my %data;
my $xormon;
my $cfg;
my $stats_found = 0;
my $api_version = "";
my $urls;
my $data_directory = "$FindBin::Bin";
my $debug = 0;

my $storage_name = "";
my $active_minute = "";
my $debug_data = "";
my %performance_data;
my %configuration_data;
my $st_serial;
my $request_count;
my @architecture = ();
my $service_data_out;
my $st_type = "dothill";
my @rows_api_test = ();
my @rows_api_test_data = ();
my $port = "";
my $path_wbencli = "";
my $protocol = "";
my $storage_type = "";
my $host_smis = "";
my $host_api = "";
my $session_key = "";
my $wrkdir = "";
my $port_file2_xml = "";
my %performance_latest;
my %performance_old;
my %inventory_port;
my %inventory_performance;
my %inventory_configuration;
my %header_section;
my $service_data_in;
my $connection_test = 0;
my @rows_api_test_tmp = ();
my @health_status_global = ();
my $hs_error = 0;
my $hs_description = "";
my @health_status = ();
my $wbi = 0;
my $time_event         = get_time_events();
my @cmds_performance   = ( "volume-statistics", "disk-statistics", "pool-statistics", "controller-statistics", "host-port-statistics" );
my @cmds_configuration = ( "disks", "controllers", "volumes", "pools", "versions detail", "ports detail", "host-maps", "maps", "system", "events from $time_event" );



sub collect {
  $xormon = shift;
  $cfg    = shift;

  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!") && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!") && return "error"; }
  $logger = XormonLogger->new( 'dothill', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'node1Host'} )     { $logger->error("Cannot continue, 'node1Host' is not configured!") && return "error"; }
  #unless ( exists $cfg->{'node2Host'} )     { $logger->error("Cannot continue, 'node2Host' is not configured!") && return 0; }
  unless ( exists $cfg->{'proto'} )         { $logger->error("Cannot continue, 'proto' is not configured!") && return "error"; }
  unless ( exists $cfg->{'username'} )      { $logger->error("Cannot continue, 'username' is not configured!") && return "error"; }
  unless ( exists $cfg->{'password'} )      { $logger->error("Cannot continue, 'password' is not configured!") && return "error"; }
  #$cfg->{'api-pass'}    = Xormon::unobscure_password($cfg->{'apiPassword'});
  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  my $act_timestamp     = time();
  $service_data_in = $xormon->getData();
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }

  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  $protocol = $cfg->{'proto'};

  if ( $protocol eq "http" ) {
      $port = 5988;
  }
  if ( $protocol eq "https" ) {
    $port = 5989;
  }
  $path_wbencli = "/usr/bin/wbemcli";
  if ( !-f $path_wbencli ) {
    $path_wbencli = "/opt/freeware/bin/wbemcli";
  }
  if ( !-f $path_wbencli ) {
    $path_wbencli = "/usr/local/bin/wbemcli";
  }
  $host_smis = $cfg->{'node1Host'};
  $host_api = $cfg->{'node1Host'};
  data_structure();
  if (defined $cfg->{'interface'} && $cfg->{'interface'} eq "WBI"){
    $wbi = 1;
  }
  $storage_type = get_storage_type();
  if ($session_key eq ""){
    $logger->error("XML API authorization error");
    return "error";
  }
  if ($session_key eq "Authentication Unsuccessful"){
    $logger->error("XML API authorization error: $session_key");
    return "error";
  }
  if ($storage_type eq "wbi"){
    #WBI section
    $service_data_out->{host_api} = $host_api;
    $service_data_out->{type} = $storage_type;
    foreach my $cmd (@cmds_performance) {
      if ( -f "$debug_data/$cmd-old.xml" ) {
        rename( "$debug_data/$cmd-old.xml", "$debug_data/$cmd-oldest.xml" );
      }
      if ( -f "$debug_data/$cmd.xml" ) {
        rename( "$debug_data/$cmd.xml", "$debug_data/$cmd-old.xml" );
      }
      cmd_api($cmd,"name",1);
    }
    %performance_latest = set_inventory_xml_data( "latest", "perf" );
    %performance_old    = set_inventory_xml_data( "old",    "perf" );
    #print Dumper \%performance_latest;
    #print Dumper \%performance_old;
    set_global_data2( \%performance_latest, \%performance_old );
    my $section = "Volume Level Statistics";
    %inventory_performance = set_capacity_volume( "volumes", \%inventory_performance, "perf", $section );
    calculate_hit_io_pool();
    $section                = "Drive Level Statistics";
    set_data_port_response_time($section, "disks");
    set_data_raid_group( "disks", "perf");
    set_interval_pool_raid_group();
    #%header_section         = set_header("perf");
    %inventory_performance = set_data_pool("", \%inventory_performance);
    set_data_pool_capacity();
    set_data_host();
    convert_data();
    set_data_device();
    set_data_port_type();
    delete_pool_response_metrics();
    #return "successful";
    #%inventory_performance2 = set_global_pool( \%inventory_performance2 );
    #write_data_performance( \%inventory_performance2, $storage_name );
  }
  elsif ($storage_type =~ m/smis|hpq|dhs/){
    ### performance data
    $service_data_out->{type} = $storage_type;
    $service_data_out->{host_smis} = $host_smis;
    $service_data_out->{host_api} = $host_api;
    my $cmd = "CIM_BlockStorageStatisticalData";
    cmd_smis($cmd,$storage_type,600);
    if (-f "$debug_data/$cmd.txt" && -f "$debug_data/$cmd.txt_old"){
      my $check_perf_file = ( stat("$debug_data/$cmd.txt") )[9] - ( stat("$debug_data/$cmd.txt_old") )[9];
      if ($check_perf_file < 3600){
        %inventory_port     = set_configuration_element( "Port Level Statistics", \%inventory_port, "$debug_data/CIM_FCport.txt" );
        %performance_latest = set_inventory_data( "$debug_data/$cmd.txt", "perf" );
        %performance_old    = set_inventory_data( "$debug_data/$cmd.txt_old", "perf" );
        set_global_data( \%performance_latest, \%performance_old, "$debug_data/$cmd.txt", "$debug_data/$cmd.txt_old" );
        %inventory_performance = set_configuration_element( "Drive Level Statistics", \%inventory_performance, "$debug_data/CIM_DiskDrive.txt" );
        my $section = "Volume Level Statistics";
        %inventory_performance = set_capacity_volume( "volumes", \%inventory_performance, "perf", $section );
        %inventory_performance = set_data_pool( "$debug_data/CIM_StoragePool.txt", \%inventory_performance );
        #print Dumper \%inventory_performance;
         ### response time port
        $section = "Port Level Statistics";
        set_data_port_response_time($section, "host-port-statistics" );
        ### response time drive
        $section = "Drive Level Statistics";
        set_data_port_response_time($section, "disks" );
        set_data_raid_group("disks");
        set_interval_pool_raid_group();
        set_data_host();
        ##print Dumper \%inventory_performance > "$debug_data/performance_hash.txt";
        ##print Dumper set_configuration() > "$debug_data/configuration_hash.txt";
        #write_data(%performance_data,"$debug_data/performnace_hash.txt");
        #my %header = set_header("perf");
        #write_data_performance(\%header);
        #%inventory_configuration = set_configuration();
        convert_data();
        set_data_device();
        set_data_port_type();
        #print Dumper \%performance_data;
        #convert_data_config();

      }
      else{
        unlink("$debug_data/$cmd.txt");
        unlink("$debug_data/$cmd.txt_old");
      }
    }
  }
  else{
    logout_api();
    $xormon->isConnected(0, "Connection failed!");
    return "error";
  }
  ### hostcfg_status
  $xormon->isConnected(1);
  ### hostcfg_status
  delete_empty_metrics();
  calculate_missing_metrics();
  my $rc = $xormon->saveJSON2(\%performance_data);
  my $status = get_status($rc, "saveJSON2 (performance)");
  if ($status eq "error"){
    #print Dumper \%performance_data;
  }

  ### host_cfg_data
  if (Xormon::isdigit($service_data_out->{last_perf_timestamp})){
    if ( ( time() -  $service_data_out->{last_perf_timestamp} ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_out->{last_perf_timestamp});
    }
    else{
      $xormon->hasData(1);
    }
  }
  else{
    $xormon->hasData(0, "no available stats");
  }
  ###

  if (( ! exists $service_data_in->{last_conf_timestamp} ) || ( ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) || ( $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) || ( !-f "$debug_data/config-$st_serial.out" )) {
    `touch "$debug_data/config-$st_serial.out"`;
    ### logout and login
    logout_api();
    ### configuration command from smis
    my @cmds = ("CIM_FCport", "CIM_StorageVolume", "CIM_StoragePool", "CIM_DiskDrive", "CIM_ComputerSystem", "CIM_LogRecord");
    foreach my $cmd (@cmds){
      if ($storage_type eq "wbi"){last;}
      cmd_smis($cmd,$storage_type,60);
    }
    $session_key = login_api($host_api);
    @cmds = ("host-maps","disks", "volumes", "host-port-statistics", "ports detail", "system", "versions detail", "controllers", "maps", "pools");
    if ($storage_type eq "wbi"){
      @cmds = @cmds_configuration;
    }
    foreach my $cmd(@cmds){
      cmd_api($cmd,"name",1);
    }
    if ($storage_type ne "wbi"){
      %inventory_configuration = set_configuration();
    }
    else{
      %inventory_configuration = set_inventory_xml_conf_data();
      %inventory_configuration = set_capacity_volume( "volumes", \%inventory_configuration, "conf", "Volume Level Configuration" );
    }
    #print Dumper \%inventory_configuration;
    convert_data_config();
    my @types = ("pool", "volume", "fcport", "sasport", "iscsiport", "drive", "node", "raid_group", "port_mapping", "device", "host");
    foreach my $uid (keys %configuration_data){
      my $active = 0;
      foreach my $type (@types){
        if (defined $performance_data{$st_type}{$type}{$uid}){
          $active++;
          last;
        }
        if ($type eq "port_mapping" && defined $configuration_data{$uid}{TYPE} && $configuration_data{$uid}{TYPE} eq $type){
          $active++;
          last;
        }
      }
      if ($active == 0){
        #print Dumper \$configuration_data{$uid};
        #print "mazame $uid\n";
        delete $configuration_data{$uid};
        #print "mazeme prvek $uid\n";
      }
    }
    #print Dumper \%configuration_data;
    #print Dumper $performance_data{$st_type}{pool};
    foreach my $uid (keys %configuration_data ){
      if ($uid eq $st_serial){next;}
      if (defined $configuration_data{$uid}{status}){
        my %hs;
        $hs{item_id} = $uid;
        if (defined $configuration_data{$uid}{description}){
          $hs{data} = $configuration_data{$uid}{description};
          $hs_description = $hs{data};
        }
        else{
          $hs{data} = "";
        }
        if ( $configuration_data{$uid}{status} =~ m/ok/i){
          $hs{status} = "ok";
        }
        elsif( $configuration_data{$uid}{status} =~ m/warning/i){
          $hs{status} = "warning";
          $hs_error = 2;
        }
        elsif( $configuration_data{$uid}{status} =~ m/unknown/i || $configuration_data{$uid}{status} eq "N/A"){
          $hs{status} = "unknown";
        }
        else{
          $hs{status} = "error";
          $hs_error = 1;
        }
        #print Dumper \%hs;
        push(@health_status_global,\%hs);
      }
    }
    $service_data_out->{last_conf_timestamp} = $act_timestamp;
    set_data_health_status();
    ($status, my $status_configuration, my $status_health) = set_architecture();
    if ($status eq "successful" && $status_configuration eq "successful" && $status_health eq "successful" ){
      my $save_data = $xormon->saveData(\%{ $service_data_out });
      my $status_save_data = get_status($save_data, "SaveData");
    }
    logout_api();
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
    return $status;
  }
  logout_api();
  my $save_data = $xormon->saveData(\%{ $service_data_out });
  my $status_save_data = get_status($save_data, "SaveData");
  $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
  return $status;
}

sub get_status {
  my $rc = shift;
  my $task = shift;
  if ($rc eq "201" || $rc eq "200") {
    $logger->info("Task for $task ".$cfg->{hostalias}." is done. Status: successful");
    return "successful";
  } elsif ($rc eq "401") {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: unauthorized");
    return "unauthorized";
  } else {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
}

sub set_data_port_type{

  my %data;
  my $section = "port";
  #print Dumper \$performance_data{$st_type}{$section};
  #print Dumper %performance_data;
  %data = set_configuration_port_type( \%data, "ports detail", $section );
  #my $type = "port_type";
  my %port_data;
  foreach my $item (keys %{$data{$section}}){
    my $id = $data{$section}{$item}{id};
    my $name = $data{$section}{$item}{name};
    my $protocol = $data{$section}{$item}{Protocol};
    if (!defined $id || !defined $protocol || !defined $name){next;}
    $protocol = lc($protocol);
    $protocol = $protocol . "port";
    #my $uid = Xormon::create_hash_key($st_serial.$type.$protocol);
    my $uid_port = Xormon::create_hash_key($st_serial.$section.$id);
    my $port_uid_new = Xormon::create_hash_key($st_serial.$protocol.$id);
    $configuration_data{$port_uid_new}{label} = $name;
    $configuration_data{$port_uid_new}{TYPE} = $protocol;
    $configuration_data{$port_uid_new}{id} = $id;
    if (defined $performance_data{$st_type}{$section}{$uid_port}){
      foreach my $time (keys %{$performance_data{$st_type}{$section}{$uid_port}}){
        foreach my $metric (keys %{$performance_data{$st_type}{$section}{$uid_port}{$time}}){
          $performance_data{$st_type}{$protocol}{$port_uid_new}{$time}{$metric} = $performance_data{$st_type}{$section}{$uid_port}{$time}{$metric};
        }
      }
      delete $performance_data{$st_type}{$section}{$uid_port};
    }
  }
  delete $performance_data{$st_type}{$section};
  #print Dumper \%configuration_data;
  #print Dumper \%performance_data;
}

sub set_data_response_time{
  my $metric1 = shift;
  my $metric2 = shift;
  my $metric3 = shift;
  my $type = shift;
  my $data_tmp = shift;
  my $metric4 = shift;
  my $metric5 = shift;
  my %port_data = %{$data_tmp};

  delete $port_data{$type}{data}{$metric1};
  if (Xormon::isdigit($port_data{$type}{data}{$metric2}) && Xormon::isdigit($port_data{$type}{data}{$metric3})){
    if ($port_data{$type}{data}{$metric2} == 0){
      $port_data{$type}{data}{$metric1} = 0;
    }
    else{
      $port_data{$type}{data}{$metric1} = $port_data{$type}{data}{$metric3} / $port_data{$type}{data}{$metric2};
    }
  }
  delete $port_data{$type}{data}{$metric3};

  #block size
  if (Xormon::isdigit($port_data{$type}{data}{$metric2}) && Xormon::isdigit($port_data{$type}{data}{$metric4})){
    if ($port_data{$type}{data}{$metric2} == 0){
      $port_data{$type}{data}{$metric5} = 0;
    }
    else{
      $port_data{$type}{data}{$metric5} = round($port_data{$type}{data}{$metric4} / $port_data{$type}{data}{$metric2},0);
    }
  }
  ###

  return %port_data;
}


sub set_numerator{
  my $metric1 = shift;
  my $metric2 = shift;
  my $metric3 = shift;
  my $data_tmp = shift;
  my $protocol = shift;
  my $port_uid = shift;
  my $section = shift;
  my $time = shift;
  my %data = %{$data_tmp};

  if (Xormon::isdigit($performance_data{$st_type}{$section}{$port_uid}{$time}{$metric1}) && Xormon::isdigit($performance_data{$st_type}{$section}{$port_uid}{$time}{$metric2})){
    if (Xormon::isdigit($data{$protocol}{data}{$metric3})){
      $data{$protocol}{data}{$metric3} = $data{$protocol}{data}{$metric3} + ($performance_data{$st_type}{$section}{$port_uid}{$time}{$metric1} * $performance_data{$st_type}{$section}{$port_uid}{$time}{$metric2});
    }
    else{
      $data{$protocol}{data}{$metric3} = $performance_data{$st_type}{$section}{$port_uid}{$time}{$metric1} * $performance_data{$st_type}{$section}{$port_uid}{$time}{$metric2};
    }
  }
  return %data;
}

sub delete_pool_response_metrics{
  my $type = "pool";
  my @metrics = ("resp_t", "resp_t_r", "resp_t_w");
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    my $time = (keys %{$performance_data{$st_type}{$type}{$uid}})[0];
    foreach my $metric (@metrics){
      if (defined $performance_data{$st_type}{$type}{$uid}{$time}{$metric}){
        delete $performance_data{$st_type}{$type}{$uid}{$time}{$metric};
      }
    }
  }
}

sub set_data_device{
  my $type = "pool";
  my @metrics = ("io_rate", "read_io", "write_io", "data_rate", "read", "write", "capacity_total", "capacity_used", "capacity_free", "r_cache_hit_io", "w_cache_hit_io");
  my %data;
  my $time = "";
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    $time = (keys %{$performance_data{$st_type}{$type}{$uid}})[0];
    foreach my $metric (@metrics){
      if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$metric})){
        #print  localtime() . " $st_type | $type | $uid | $time | $metric | " . $performance_data{$st_type}{$type}{$uid}{$time}{$metric} . "\n";
        if (Xormon::isdigit($data{$metric})){
          $data{$metric} = $data{$metric} + $performance_data{$st_type}{$type}{$uid}{$time}{$metric};
        }
        else{
          $data{$metric} = $performance_data{$st_type}{$type}{$uid}{$time}{$metric};
        }
      }
    }
  }
  foreach my $metric (keys %data){
    $performance_data{$st_type}{device}{$st_serial}{$time}{$metric} = $data{$metric};
  }
  #array total response time from ports data

  $type = "port";
  calculate_weighted_average($type,"resp_t_r", "read_io",$time);
  calculate_weighted_average($type,"resp_t_w", "write_io",$time);
  calculate_weighted_average($type,"resp_t", "io_rate",$time);

  #
  #$performance_data{$st_type}{device}{$st_serial}{$time} = \%data;
  #print Dumper \$performance_data{$st_type}{DEVICE};
  #print Dumper \$performance_data{$st_type}{device};
  #print Dumper \$performance_data{$st_type}{pool};
}

sub calculate_weighted_average{
  my $section = shift;
  my $item1 = shift;
  my $item2 = shift;
  my $timestamp = shift;
  foreach my $uid (keys %{$performance_data{$st_type}{$section}}){
    my $time = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
    if (!Xormon::isdigit($time)){next;}
    my $value1 = $performance_data{$st_type}{$section}{$uid}{$time}{$item1};
    my $value2 = $performance_data{$st_type}{$section}{$uid}{$time}{$item2};
    if (Xormon::isdigit($value1) && Xormon::isdigit($value2)){
      if (Xormon::isdigit($performance_data{$st_type}{device}{$st_serial}{$timestamp}{"numerator".$item1})){
        $performance_data{$st_type}{device}{$st_serial}{$timestamp}{"numerator".$item1} +=  $value1 * $value2;
        $performance_data{$st_type}{device}{$st_serial}{$timestamp}{"denominator".$item1} +=  $value2;
      }
      else{
        $performance_data{$st_type}{device}{$st_serial}{$timestamp}{"numerator".$item1} =  $value1 * $value2;
        $performance_data{$st_type}{device}{$st_serial}{$timestamp}{"denominator".$item1} =  $value2;
      }
    }
  }
  foreach my $time (keys %{$performance_data{$st_type}{device}{$st_serial}}){
    my $numerator = $performance_data{$st_type}{device}{$st_serial}{$time}{"numerator".$item1};
    my $denominator = $performance_data{$st_type}{device}{$st_serial}{$time}{"denominator".$item1};
    if (Xormon::isdigit($numerator) && Xormon::isdigit($denominator)){
      if ($denominator == 0){
        $performance_data{$st_type}{device}{$st_serial}{$time}{$item1} = 0;
      }
      else{
        $performance_data{$st_type}{device}{$st_serial}{$time}{$item1} = round($numerator / $denominator,3);
      }
    }
    delete $performance_data{$st_type}{device}{$st_serial}{$time}{"numerator".$item1};
    delete $performance_data{$st_type}{device}{$st_serial}{$time}{"denominator".$item1};
  }
}

sub set_data_host{
  my %data;
  my $section = "Host Level Configuration";
  my $section_volume = "Volume Level Statistics";
  my $section_host = "Host Level Statistics";
  %data = get_host_section( $section, \%data, "host-maps", "conf" );
  #my $inventory_metrics = "TotalIOs,KBytesTransferred,KBytesWritten,ReadIOs,ReadHitIOs,KBytesRead,WriteIOs,WriteHitIOs";
  my $inventory_metrics = "TotalIOs,KBytesTransferred,KBytesWritten,ReadIOs,KBytesRead,WriteIOs";
  #remove read write hit io
  my @metrics           = split( ",", $inventory_metrics );

  #print Dumper \%data;
  foreach my $uid (keys %{$data{$section}}){
    if (!defined $data{$section}{$uid}{"Volume IDs"}){next;}
    my $volumes_raw = $data{$section}{$uid}{"Volume IDs"};
    my @volumes = split(" ", $volumes_raw);
    foreach my $volume (@volumes){
      if (defined $inventory_performance{$section_volume}{$volume}){
        foreach my $metric (@metrics){
          if (Xormon::isdigit($inventory_performance{$section_volume}{$volume}{$metric})){
            if (Xormon::isdigit($inventory_performance{$section_host}{$uid}{$metric})){
              $inventory_performance{$section_host}{$uid}{$metric} = $inventory_performance{$section_host}{$uid}{$metric} + $inventory_performance{$section_volume}{$volume}{$metric};
            }
            else{
              $inventory_performance{$section_host}{$uid}{$metric} = $inventory_performance{$section_volume}{$volume}{$metric};
            }
          }
        }
      }
    }
    $inventory_performance{$section_host}{$uid}{ID} = $uid;
  }
  $inventory_performance{$section_host}{TIME} = $inventory_performance{$section_volume}{TIME};
  #print Dumper \$inventory_performance{$section_host};
}


sub set_configuration {
  #my ( $pool_file, $volume_file, $disk_file, $port_file, $node_file, $event_file, $host_file, $raid_group_file, $volume_capacity_file, $config_file_2 ) = @_;
  my %inventory_configuration;
  my $section = "";

  ### pool
  $section                 = "Pool Level Configuration";
  %inventory_configuration = set_configuration_element( $section, \%inventory_configuration, "$debug_data/CIM_StoragePool.txt", "GB" );

  #print Dumper \%inventory_configuration;
  ###

  ### volume
  $section                 = "Volume Level Configuration";
  %inventory_configuration = set_configuration_element( $section, \%inventory_configuration, "$debug_data/CIM_StorageVolume.txt" );
  ### volume capacity
  %inventory_configuration = set_capacity_volume( "volumes", \%inventory_configuration, "conf", $section );
  ###
  #print Dumper \%inventory_configuration;

  ### drive
  $section                 = "Drive Level Configuration";
  %inventory_configuration = set_configuration_element( $section, \%inventory_configuration, "$debug_data/CIM_DiskDrive.txt" );
  ###

  ### port
  $section = "Port Level Configuration";

  #%inventory_configuration = set_configuration_element($section,\%inventory_configuration,$port_file);
  %inventory_configuration = set_configuration_port( \%inventory_configuration, "host-port-statistics", $section );
  %inventory_configuration = set_configuration_element( $section, \%inventory_configuration, "$debug_data/CIM_FCport.txt" );
  %inventory_configuration = set_configuration_port_type( \%inventory_configuration, "ports detail", $section );
  ### add type port

  ### controller
  $section                 = "Node Level Configuration";
  %inventory_configuration = set_configuration_element( $section, \%inventory_configuration, "$debug_data/CIM_ComputerSystem.txt" );

  ### host
  $section                 = "Host Level Configuration";
  %inventory_configuration = get_host_section( $section, \%inventory_configuration, "host-maps", "conf" );

  #print Dumper \%inventory_configuration;

  ### raid group
  $section = "Raid Group Level Configuration";
  my %raid_group = get_raid_group_section( "disks");

  ### add raid group to configuration
  $inventory_configuration{$section} = \%raid_group;

  ### configuration per node
  $section                 = "Node Level Configuration";
  %inventory_configuration = get_ctl_section( $section, \%inventory_configuration, "versions detail", "controllers", "conf" );
  ###

  ### health status

  #my $table = get_event_log($event_file);
  #write_health_status_html("$webdir/$storage_name/health_status.html", $table);
  #summary_health_status($error_health);

  return %inventory_configuration;
}


sub get_ctl_section {
  my ( $section, $inventory_configuration_tmp, $cmd1, $cmd2, $type ) = @_;
  my %inventory_configuration = %{$inventory_configuration_tmp};
  my $xml_inventory = cmd_api($cmd1, "name", 0);
  if ( defined $xml_inventory && ref($xml_inventory) eq 'HASH' ) {
    my $node_inventory = $xml_inventory->{OBJECT};
    if ( ref($node_inventory) eq 'ARRAY' ) {
      foreach my $node ( @{$node_inventory} ) {
        my $name = $node->{name};
        if ( !defined $name || $name eq "" || $name eq "status" ) { next; }
        $name =~ s/controller-//g;
        $name =~ s/-versions//g;
        if ( !defined $name ) { next; }
        my $up_name = uc($name);
        if ( !defined $inventory_configuration{$section}{$up_name} ) { next; }
        foreach my $item ( keys %{ $node->{PROPERTY} } ) {
          my $content = $node->{PROPERTY}->{$item}->{content};
          my $name    = $node->{PROPERTY}->{$item}->{"display-name"};
          if ( !defined $content || !defined $name ) { next; }
          $inventory_configuration{$section}{$up_name}{$name} = $content;
        }
      }
    }
  }

  ### node xml
  my $xml_inventory2 = cmd_api($cmd2,"name",0);
  if ( defined $xml_inventory2->{OBJECT} && ref( $xml_inventory2->{OBJECT} ) eq "ARRAY" ) {
    foreach my $node ( @{ $xml_inventory2->{OBJECT} } ) {
      my $name = $node->{PROPERTY}->{"controller-id"}->{content};
      if ( !defined $name || $name eq "" || !defined $inventory_configuration{$section}{$name} ) { next; }
      my $serial_number = $node->{PROPERTY}->{"serial-number"}->{content};
      my $model         = $node->{PROPERTY}->{"model"}->{content};
      my $ip            = $node->{PROPERTY}->{"ip-address"}->{content};
      my $version       = $node->{PROPERTY}->{"sc-fw"}->{content};
      if ( defined $serial_number ) {
        $inventory_configuration{$section}{$name}{"serial_number"} = $serial_number;
      }
      if ( defined $model ) {
        $inventory_configuration{$section}{$name}{"model"} = $model;
      }
      if ( defined $ip ) {
        $inventory_configuration{$section}{$name}{"host"} = $ip;
      }
      if ( defined $version ) {
        $inventory_configuration{$section}{$name}{"version"} = $version;
      }
    }
  }

  #print Dumper \$inventory_configuration{$section};
  return %inventory_configuration;
}



sub get_host_section {
  my ( $section, $inventory_configuration_tmp, $cmd, $type ) = @_;
  my %inventory_configuration = %{$inventory_configuration_tmp};
  my $xml_inventory = cmd_api($cmd,"name",0);
  if (!defined $xml_inventory){
    $xml_inventory = cmd_api("maps","name",0);
  }


  if ( defined $xml_inventory && ref($xml_inventory) eq 'HASH' ) {
    my $host_inventory = $xml_inventory->{OBJECT};
    if ( ref($host_inventory) eq 'ARRAY' ) {
      foreach my $host ( @{$host_inventory} ) {
        my $host_name = $host->{PROPERTY}->{"hba-nickname"}->{content};
        my $host_id   = $host->{PROPERTY}->{"id"}->{content};
        if ( defined $host_name && $host_name ne "" && defined $host_id && $host_id ne "" ) {
          $inventory_configuration{$section}{$host_id}{id}   = $host_id;
          $inventory_configuration{$section}{$host_id}{name} = $host_name;
        }
        if ((!defined $host_name || $host_name eq "" ) && defined $host_id && $host_id ne ""){
          $inventory_configuration{$section}{$host_id}{id}   = $host_id;
          $inventory_configuration{$section}{$host_id}{name} = $host_id;
          $host_name = $host_id;
        }
        my $vol_inventory = $host->{OBJECT};
        if ( ref($vol_inventory) eq 'ARRAY' ) {
          foreach my $volume ( @{$vol_inventory} ) {
            my $volume_id   = $volume->{PROPERTY}->{"volume-serial"}->{content};
            my $volume_name = $volume->{PROPERTY}->{"volume-name"}->{content};
            my $ports       = $volume->{PROPERTY}->{"ports"}->{content};
            if ( defined $volume_id && $volume_id ne "" ) {
              my $volumes = $inventory_configuration{$section}{$host_id}{"Volume IDs"};
              if ( defined $volumes ) {
                $volumes = $volumes . " " . $volume_id;
                $inventory_configuration{$section}{$host_id}{"Volume IDs"} = $volumes;
              }
              else {
                $inventory_configuration{$section}{$host_id}{"Volume IDs"} = $volume_id;
              }
            }
            if ( defined $volume_name && $volume_name ne "" && defined $ports && $ports ne "" ) {
              my $section = "Port-Mapping Level Configuration";
              my $volumes = $inventory_configuration{$section}{$host_id}{"Volumes"};
              if ( defined $volumes ) {
                $volumes = $volumes . " " . $volume_name;
                $inventory_configuration{$section}{$host_id}{"Volumes"} = $volumes;
              }
              else {
                $inventory_configuration{$section}{$host_id}{"Volumes"} = $volume_name;
              }
              $inventory_configuration{$section}{$host_id}{"Host"} = $host_name;
              $ports =~ s/,/ /g;
              $inventory_configuration{$section}{$host_id}{"Ports"} = $ports;
            }
          }
        }
        if ( ref($vol_inventory) eq 'HASH' ) {    ### only one volume
          my $volume_id   = $vol_inventory->{PROPERTY}->{"volume-serial"}->{content};
          my $volume_name = $vol_inventory->{PROPERTY}->{"volume-name"}->{content};
          my $ports       = $vol_inventory->{PROPERTY}->{"ports"}->{content};
          if ( defined $volume_id && $volume_id ne "" ) {
            my $volumes = $inventory_configuration{$section}{$host_id}{"Volume IDs"};
            if ( defined $volumes ) {
              $volumes = $volumes . " " . $volume_id;
              $inventory_configuration{$section}{$host_id}{"Volume IDs"} = $volumes;
            }
            else {
              $inventory_configuration{$section}{$host_id}{"Volume IDs"} = $volume_id;
            }
          }
          if ( defined $volume_name && $volume_name ne "" && defined $ports && $ports ne "" ) {
            my $section = "Port-Mapping Level Configuration";
            my $volumes = $inventory_configuration{$section}{$host_id}{"Volumes"};
            if ( defined $volumes ) {
              $volumes = $volumes . " " . $volume_name;
              $inventory_configuration{$section}{$host_id}{"Volumes"} = $volumes;
            }
            else {
              $inventory_configuration{$section}{$host_id}{"Volumes"} = $volume_name;
            }
            $inventory_configuration{$section}{$host_id}{"Host"} = $host_name;
            $ports =~ s/,/ /g;
            $inventory_configuration{$section}{$host_id}{"Ports"} = $ports;
          }
        }
      }
    }
  }

  #print Dumper \%hosts;
  #print Dumper \%inventory_configuration;
  return %inventory_configuration;
}


sub set_configuration_port_type {
  my ( $tmp_hash, $cmd, $section ) = @_;
  my %inventory_configuration = %{$tmp_hash};
  my $hash = cmd_api($cmd,"name",0);
  if ( defined $hash->{OBJECT} && ref( $hash->{OBJECT} ) eq "ARRAY" ) {
    foreach my $port ( @{ $hash->{OBJECT} } ) {
      my $port_name = $port->{'PROPERTY'}->{port}->{content};
      my $port_type = $port->{'PROPERTY'}->{"port-type"}->{content};
      if ( !defined $port_name || $port_name eq "" || !defined $port_type || $port_type eq "") { next; }
      if ( !defined $inventory_configuration{$section}{$port_name} ) {
        $inventory_configuration{$section}{$port_name}{name} = $port_name;
        $inventory_configuration{$section}{$port_name}{id}   = $port_name;
      }
      my $speed = $port->{'PROPERTY'}->{"actual-speed"}->{content};
      if ( defined $speed && defined $inventory_configuration{$section}{$port_name} && !defined $inventory_configuration{$section}{$port_name}{"port_speed"} ) {
        if ( $speed =~ m/Gb/ ) {
          $speed =~ s/Gb//g;
          if ( Xormon::isdigit($speed) ) {
            $inventory_configuration{$section}{$port_name}{port_speed} = $speed * 1000000000;
          }
          elsif ( $speed =~ m/Mb/ ) {
            $speed =~ s/Mb//g;
            if ( Xormon::isdigit($speed) ) {
              $inventory_configuration{$section}{$port_name}{port_speed} = $speed * 1000000;
            }
          }
          else {
            $inventory_configuration{$section}{$port_name}{port_speed} = $speed;
          }
        }
      }
      my $health = $port->{'PROPERTY'}->{"health"}->{content};
      if ( defined $health && defined $inventory_configuration{$section}{$port_name} && !defined $inventory_configuration{$section}{$port_name}{"status"} ) {
        $inventory_configuration{$section}{$port_name}{"status"} = $health;
      }

      if ( defined $port_type && defined $inventory_configuration{$section}{$port_name} && !defined $inventory_configuration{$section}{$port_name}{"Protocol"} ) {
        $inventory_configuration{$section}{$port_name}{"Protocol"} = $port_type;
      }
      my $port_wwn = $port->{'PROPERTY'}->{"target-id"}->{content};
      if ( defined $port_wwn && defined $inventory_configuration{$section}{$port_name} && !defined $inventory_configuration{$section}{$port_name}{"wwn"} ) {
        $inventory_configuration{$section}{$port_name}{"wwn"} = $port_wwn;
      }
    }
  }
  return %inventory_configuration;
}



sub set_configuration_port {
  my ( $tmp_hash, $cmd, $section ) = @_;
  my %inventory_configuration = %{$tmp_hash};
  my $xml_inventory = cmd_api($cmd,"name",0);
  if ( defined $xml_inventory && ref($xml_inventory) eq 'HASH' ) {
    my $port_inventory = $xml_inventory->{OBJECT};
    if ( ref($port_inventory) eq 'ARRAY' ) {
      foreach my $port ( @{$port_inventory} ) {
        my $port_name = $port->{PROPERTY}->{"durable-id"}->{content};
        if ( defined $port_name && $port_name ne "" ) {
          $port_name =~ s/hostport_//g;
          $inventory_configuration{$section}{$port_name}{id}   = $port_name;
          $inventory_configuration{$section}{$port_name}{name} = $port_name;
          next;
        }
      }
    }
  }
  return %inventory_configuration;
}






sub set_interval_pool_raid_group {
  my $section     = "Pool Level Statistics";
  my $section_pom = "Volume Level Statistics";

  if ( defined $inventory_performance{$section_pom}{TIME}{FROM} && defined $inventory_performance{$section_pom}{TIME}{TO} && defined $inventory_performance{$section_pom}{TIME}{LENGTH} ) {
    $inventory_performance{$section}{TIME}{FROM}   = $inventory_performance{$section_pom}{TIME}{FROM};
    $inventory_performance{$section}{TIME}{TO}     = $inventory_performance{$section_pom}{TIME}{TO};
    $inventory_performance{$section}{TIME}{LENGTH} = $inventory_performance{$section_pom}{TIME}{LENGTH};

    $section                                       = "Raid Group Level Statistics";
    $inventory_performance{$section}{TIME}{FROM}   = $inventory_performance{$section_pom}{TIME}{FROM};
    $inventory_performance{$section}{TIME}{TO}     = $inventory_performance{$section_pom}{TIME}{TO};
    $inventory_performance{$section}{TIME}{LENGTH} = $inventory_performance{$section_pom}{TIME}{LENGTH};
  }
}


sub get_raid_group_section {
  my $cmd = shift;
  my %raid_group;


  my $xml_inventory = cmd_api($cmd,"name",0);
  if ( defined $xml_inventory && ref($xml_inventory) eq 'HASH' ) {
    my $raid_group_inventory = $xml_inventory->{OBJECT};
    if ( ref($raid_group_inventory) eq 'ARRAY' ) {
      foreach my $group ( @{$raid_group_inventory} ) {
        my $disk_name = $group->{PROPERTY}->{"durable-id"}->{content};
        my $raid_name = $group->{PROPERTY}->{"disk-group"}->{content};
        my $disk_capacity = $group->{PROPERTY}->{blocks}->{content};
        if ( defined $disk_name && $disk_name ne "" && defined $raid_name && $raid_name ne "" ) {
          $raid_group{$raid_name}{name}                     = $raid_name;
          $raid_group{$raid_name}{id}                       = $raid_name;
          $raid_group{$raid_name}{drives}{$disk_name}{name} = $disk_name;
          if ( Xormon::isdigit($disk_capacity) ) {
            $raid_group{$raid_name}{drives}{$disk_name}{"capacity_total"} = round( ($disk_capacity * 512),3 );
          }
        }
      }
    }
  }
  #print "$storage_name\n";
  #print Dumper \%raid_group;

  return %raid_group;

}


sub set_data_raid_group {
  my $cmd         = shift;
  my $section      = "Raid Group Level Statistics";
  my $section_disk = "Drive Level Statistics";

  my %raid_group        = get_raid_group_section( $cmd);
  my $inventory_metrics = "TotalIOs,KBytesTransferred,KBytesWritten,ReadIOs,KBytesRead,WriteIOs,resp_t";
  my @metrics           = split( ",", $inventory_metrics );
  foreach my $raid_groups ( keys %raid_group ) {
    foreach my $drive ( keys %{ $raid_group{$raid_groups}{drives} } ) {
      foreach my $metric_act (@metrics) {
        foreach my $drive_id ( keys %{ $inventory_performance{$section_disk} } ) {
          if ( !defined $inventory_performance{$section_disk}{$drive_id}{name} || $inventory_performance{$section_disk}{$drive_id}{name} eq "" || $inventory_performance{$section_disk}{$drive_id}{name} ne $drive ) {
            next;
          }
          else {
            if ( defined $raid_group{$raid_groups}{drives}{$drive}{"capacity_total"} && !defined $inventory_performance{$section_disk}{$drive_id}{"capacity_total"} ) {
              $inventory_performance{$section_disk}{$drive_id}{"capacity_total"} = $raid_group{$raid_groups}{drives}{$drive}{"capacity_total"};
            }
          }
          if ( defined $inventory_performance{$section_disk}{$drive_id}{$metric_act} && Xormon::isdigit( $inventory_performance{$section_disk}{$drive_id}{$metric_act} ) ) {
            if ( defined $inventory_performance{$section}{$raid_groups}{$metric_act} ) {
              my $sum   = get_value( $inventory_performance{$section}{$raid_groups}{$metric_act} );
              my $value = $inventory_performance{$section_disk}{$drive_id}{$metric_act};
              $inventory_performance{$section}{$raid_groups}{$metric_act} = $sum + $value;
              if ( $metric_act eq "WriteIOs" ) {
                if ( defined $inventory_performance{$section}{$raid_groups}{WriteIOs} && defined $inventory_performance{$section}{$raid_groups}{ReadIOs} ) {
                  my $io_read  = get_value( $inventory_performance{$section}{$raid_groups}{WriteIOs} );
                  my $io_write = get_value( $inventory_performance{$section}{$raid_groups}{ReadIOs} );
                  my $io_rate  = $io_read + $io_write;
                  $inventory_performance{$section}{$raid_groups}{TotalIOs} = $io_rate;
                }
              }
            }
            else {
              my $value = $inventory_performance{$section_disk}{$drive_id}{$metric_act};
              $inventory_performance{$section}{$raid_groups}{$metric_act} = $value;
              $inventory_performance{$section}{$raid_groups}{ID}          = $raid_groups;
              $inventory_performance{$section}{$raid_groups}{name}        = $raid_groups;
            }
            ### response time ###
            if ( $metric_act eq "resp_t" ) {
              my $numerator   = get_value( $inventory_performance{$section}{$raid_groups}{"numerator"} );
              my $denominator = get_value( $inventory_performance{$section}{$raid_groups}{"denominator"} );
              my $value       = $inventory_performance{$section_disk}{$drive_id}{$metric_act};
              my $iops        = 0;
              if ( defined $inventory_performance{$section_disk}{$drive_id}{WriteIOs} && defined $inventory_performance{$section_disk}{$drive_id}{ReadIOs} && Xormon::isdigit( $inventory_performance{$section_disk}{$drive_id}{WriteIOs} ) && Xormon::isdigit( $inventory_performance{$section_disk}{$drive_id}{ReadIOs} ) ) {
                $iops = $inventory_performance{$section_disk}{$drive_id}{WriteIOs} + $inventory_performance{$section_disk}{$drive_id}{ReadIOs};
              }
              $inventory_performance{$section}{$raid_groups}{"numerator"}   = $numerator + ( $value * $iops );
              $inventory_performance{$section}{$raid_groups}{"denominator"} = $iops + $denominator;
            }
            last;
            ###
          }
        }

      }
    }
  }
  ### response time overall calculation
  foreach my $id ( keys %{ $inventory_performance{$section} } ) {
    my $metric    = "numerator";
    my $numerator = $inventory_performance{$section}{$id}{$metric};
    $metric = "denominator";
    my $denominator = $inventory_performance{$section}{$id}{$metric};
    if (defined $denominator && Xormon::isdigit($denominator) && $denominator == 0){
      $denominator = 1;
    }
    if ( defined $numerator && Xormon::isdigit($numerator) && defined $denominator && Xormon::isdigit($denominator) && $denominator != 0 ) {
      $metric = "resp_t";
      $inventory_performance{$section}{$id}{$metric} = round( $numerator / $denominator, 3 );
      next;
    }
  }

  #print Dumper \%raid_group;
  #print "$storage_name\n";
  #print Dumper \$inventory_performance{$section};
  #print Dumper \$inventory_performance{$section_disk};
  #print Dumper \$inventory_performance{$section_disk};

}


sub set_data_port_response_time {
  my ( $section, $cmd ) = @_;
  my $xml_inventory = cmd_api($cmd,"name", 1);

  if ( defined $xml_inventory && ref($xml_inventory) eq 'HASH' ) {
    my $port_inventory = $xml_inventory->{OBJECT};
    if ( ref($port_inventory) eq 'ARRAY' ) {
      foreach my $port ( @{$port_inventory} ) {
        my $port_name  = $port->{PROPERTY}->{"durable-id"}->{content};
        my $resp_read  = $port->{PROPERTY}->{"avg-read-rsp-time"}->{content};
        my $resp_write = $port->{PROPERTY}->{"avg-write-rsp-time"}->{content};
        my $resp_io    = $port->{PROPERTY}->{"avg-rsp-time"}->{content};
        ### find key hash for drive statistics by name
        if ( $section =~ m/Drive Level/ ) {
          foreach my $id ( keys %{ $inventory_performance{$section} } ) {
            if ( defined $inventory_performance{$section}{$id}{name} && defined $port_name && $inventory_performance{$section}{$id}{name} eq $port_name ) {
              $port_name = $id;
              last;
            }
          }
        }
        if ( defined $port_name && $port_name ne "" ) {
          if ( $port_name =~ m/^hostport_/ && $section !~ m/Drive Level/ ) {
            $port_name =~ s/hostport_//;
          }
          if ( Xormon::isdigit($resp_read) && defined $inventory_performance{$section}{$port_name} ) {
            #my $metric = "Response read (ms)";
            my $metric = "resp_t_r";
            $inventory_performance{$section}{$port_name}{$metric} = round( $resp_read / 1000, 3 );
          }
          if ( Xormon::isdigit($resp_write) && defined $inventory_performance{$section}{$port_name} ) {
            my $metric = "resp_t_w";
            $inventory_performance{$section}{$port_name}{$metric} = round( $resp_write / 1000, 3 );
          }
          if ( Xormon::isdigit($resp_io) && defined $inventory_performance{$section}{$port_name} ) {
            my $metric = "resp_t";
            $inventory_performance{$section}{$port_name}{$metric} = round( $resp_io / 1000, 3 );
          }
        }
      }
    }
  }

  #return %inventory_performance;
}


sub set_global_data {
  my ( $tmp_hash, $tmp_hash_inventory, $file_new, $file_old ) = @_;
  my %new = %{$tmp_hash};
  my %old = %{$tmp_hash_inventory};

  my $start_t     = ( stat("$file_old") )[9];
  my $end_t       = ( stat("$file_new") )[9];
  my $sample_rate = $end_t - $start_t;
  my $start       = get_time($start_t);
  my $end         = get_time($end_t);
  #my $sample_rate = ( stat("$file_new") )[9] - ( stat("$file_old") )[9];
  if ( $sample_rate == 0 ) { $sample_rate = 1; }

  foreach my $section ( keys %new ) {
    foreach my $id ( keys %{ $new{$section} } ) {
      my $active      = 0;
      my $name        = "";
      my $id_act      = "";
      my $section_act = "";
      foreach my $metric ( keys %{ $new{$section}{$id} } ) {
        if ( $metric eq "name" ) {
          $name = "$new{$section}{$id}{$metric}";
          next;
        }
        if ( $metric eq "id" ) {
          $id_act = "$new{$section}{$id}{$metric}";
          next;
        }
        if ( $metric eq "SECTION" ) {
          $section_act = "$new{$section}{$id}{$metric}";
          next;
        }
        if ( $metric =~ m/^Total|^Read|^Write|^KBytes|^Response/ ) {
          if ( defined $old{$section}{$id}{$metric} && Xormon::isdigit( $old{$section}{$id}{$metric} ) && Xormon::isdigit( $new{$section}{$id}{$metric} ) ) {
            my $number = $new{$section}{$id}{$metric} - $old{$section}{$id}{$metric};
            if ( Xormon::isdigit($number) && $number !~ m/-/ ) {
              $inventory_performance{$section}{$id}{$metric} = round( $number / $sample_rate, 0 );
              $active = 1;
            }
          }
        }
      }
      if ( $name ne "" && $id_act ne "" && $active == 1 && $section_act ne "" ) {
        $inventory_performance{$section}{$id}{name}    = $name;
        $inventory_performance{$section}{$id}{ID}      = $id_act;
        $inventory_performance{$section}{$id}{SECTION} = $section_act;
      }
      else {
        delete $inventory_performance{$section}{$id};
      }
    }
    $inventory_performance{$section}{TIME}{FROM}   = $start;
    $inventory_performance{$section}{TIME}{TO}     = $end;
    $inventory_performance{$section}{TIME}{LENGTH} = $sample_rate;
  }
}

sub get_time {
  my $timestamp_end = shift;
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $time_end = "$year:$month:$day" . "T" . "$hour:$minute:$sec";
  return $time_end;

}



sub set_inventory_data {
  my $file = shift;
  my $type = shift;
  my @data = ();
  if ( $type eq "event" && -f $file && -z $file ) {    ### event file may be empty.
    @data = ();
  }
  else {
    @data = get_array_data($file);
  }
  my %pom_hash;
  my %inventory;

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) {
      ### call function for save pom hash to global_hash
      %inventory = set_data( \%pom_hash, \%inventory, $type );
      %pom_hash  = ();
      next;
    }
    $line = remove_whitespace($line);
    if ( $line !~ m/^-/ && $line !~ m/=/ ) { next; }
    ( my $name, my $value ) = split( /=/, $line );
    $name  =~ s/-//g;
    $value =~ s/"//g;
    $pom_hash{$name} = $value;
    next;

  }
  return %inventory;

}

sub set_data_pool {
  my $file_pool   = shift;
  my $data = shift;
  my %inventory_performance = %{$data};
  my $section     = "Volume Level Statistics";

  my %inventory_volume = %{ $inventory_performance{$section} };
  my %inventory_pool;
  my $inventory_metrics = "TotalIOs,KBytesTransferred,KBytesWritten,ReadIOs,ReadHitIOs,KBytesRead,WriteIOs,WriteHitIOs";
  my @metrics           = split( ",", $inventory_metrics );
  foreach my $id ( keys %inventory_volume ) {
    my $metric         = "";
    my $pool_id        = "";
    my $block          = "";
    my $total_capacity = "";
    my $used_capacity  = 0;
    my $free_capacity  = "";

    #if (!defined $inventory_volume{$id}{$metric}){next;}
    #$name = $inventory_volume{$id}{$metric};
    $metric = "pool_id";
    if ( !defined $inventory_volume{$id}{$metric} ) { next; }
    $pool_id = $inventory_volume{$id}{$metric};
    ### sum volumes belonging pool
    foreach my $metric_act (@metrics) {

      #print Dumper \$inventory_performance{$section}{$id};
      if ( defined $inventory_performance{$section}{$id}{$metric_act} && Xormon::isdigit( $inventory_performance{$section}{$id}{$metric_act} ) ) {
        if ( defined $inventory_pool{$pool_id}{$metric_act} ) {
          my $sum = $inventory_performance{$section}{$id}{$metric_act} + get_value( $inventory_pool{$pool_id}{$metric_act} );
          $inventory_pool{$pool_id}{$metric_act} = $sum;
        }
        else {
          my $sum = $inventory_performance{$section}{$id}{$metric_act};
          $inventory_pool{$pool_id}{$metric_act} = $sum;
        }
      }
    }
    #print Dumper \%inventory_pool;
    $inventory_pool{$pool_id}{ID} = $pool_id;
    my $section_pool = "Pool Level Statistics";
    $inventory_performance{$section_pool}{$pool_id} = $inventory_pool{$pool_id};
  }
  if ($storage_type eq "wbi"){
    return %inventory_performance;
  }
  ### add pool capacity
  $section               = "Pool Level Statistics";
  %inventory_performance = set_configuration_element( $section, \%inventory_performance, $file_pool );
  #print `cat $file_pool`;

  #### check capacities (sometimes smi-s provides nonsensical data)
  my $xml_inventory = cmd_api("pools","name",0);
  #print Dumper \$xml_inventory;
  if ( defined $xml_inventory && ref($xml_inventory) eq 'HASH' ) {
    my $pool_inventory = $xml_inventory->{OBJECT};
    if ( ref($pool_inventory) eq 'ARRAY' ) {
      foreach my $pool ( @{$pool_inventory} ) {
        #print Dumper \$pool;
        my $pool_id = $pool->{PROPERTY}->{"serial-number"}->{content};
        my $cap_total = $pool->{PROPERTY}->{"total-size-numeric"}->{content};
        my $cap_free = $pool->{PROPERTY}->{"total-avail-numeric"}->{content};
        if (!Xormon::isdigit($cap_total) || !Xormon::isdigit($cap_free) || !defined $pool_id){next;}
        my $block_size = $pool->{PROPERTY}->{"blocksize"}->{content};
        if (!Xormon::isdigit($block_size)){
          $block_size = 512;
        }
        $cap_total = $cap_total * $block_size;
        $cap_free = $cap_free * $block_size;
        my $cap_used = $cap_total - $cap_free;
        if (defined $inventory_performance{$section}{$pool_id}){
          if (Xormon::isdigit($inventory_performance{$section}{$pool_id}{"capacity_total"})){
            my $cap_total_prev = $inventory_performance{$section}{$pool_id}{"capacity_total"};
            if ($cap_total_prev - (2 * $cap_total) > 0){
              $inventory_performance{$section}{$pool_id}{"capacity_total"} = $cap_total;
              $inventory_performance{$section}{$pool_id}{"capacity_used"} = $cap_used;
              $inventory_performance{$section}{$pool_id}{"capacity_free"} = $cap_free;
            }
          }
        }
      }
    }
  }
  return %inventory_performance;
}

sub get_value {
  my $value = shift;

  if ( Xormon::isdigit($value) ) {
    return $value;
  }
  else {
    return 0;
  }

}

sub convert_unit {
  my $unit        = shift;
  my $denominator = 1;
  ### find out what unit eq count

  if ( $unit eq "Bytes" || $unit eq "count" ) {
    $denominator = 1000 * 1000 * 1000 * 1000;
    return $denominator;
  }
  if ( $unit =~ m/KB/i ) {
    $denominator = 1000 * 1000 * 1000;
    return $denominator;
  }
  if ( $unit =~ m/MB/i ) {
    $denominator = 1000 * 1000;
    return $denominator;
  }

  if ( $unit =~ m/GB/i ) {
    $denominator = 1000;
    return $denominator;
  }
  if ( $unit =~ m/TB/i ) {
    $denominator = 1;
    return $denominator;
  }

  return $denominator;
}

sub set_capacity_volume {
  my ( $cmd, $hash_tmp, $type, $section ) = @_;
  my %inventory_performance = %{$hash_tmp};

  my $xml_inventory = cmd_api($cmd,"name",0);
  if ( defined $xml_inventory && ref($xml_inventory) eq 'HASH' ) {
    my $volume_inventory = $xml_inventory->{OBJECT};
    if ( ref($volume_inventory) eq 'ARRAY' ) {
      foreach my $volume ( @{$volume_inventory} ) {
        #print Dumper \$volume;
        my $volume_id = $volume->{PROPERTY}->{"serial-number"}->{content};

        #my $volume_name = $volume->{PROPERTY}->{"volume-name"}->{content};
        my $cap = $volume->{PROPERTY}->{"total-size-numeric"}->{content};
        my $block_size_cap = $volume->{PROPERTY}->{"total-size-numeric"}->{units};
        if (defined $block_size_cap && defined $volume->{PROPERTY}->{"size-numeric"}->{units}){
          $block_size_cap = $volume->{PROPERTY}->{"size-numeric"}->{units}
        }
        if (!defined $block_size_cap){
          $block_size_cap = 512;
        }
        else{
          $block_size_cap =~ s/blocks//g;
          if (!Xormon::isdigit($block_size_cap)){
            $block_size_cap = 512;
          }
        }

        if ( !defined $cap || !Xormon::isdigit($cap) ) {
          $cap = $volume->{PROPERTY}->{"size-numeric"}->{content};
        }
        my $used_cap  = $volume->{PROPERTY}->{"allocated-size-numeric"}->{content};
        my $block_size_cap_used = $volume->{PROPERTY}->{"allocated-size-numeric"}->{units};
        if (!defined $block_size_cap_used){
          $block_size_cap_used = 512;
        }
        else{
          $block_size_cap_used =~ s/blocks//g;
          if (!Xormon::isdigit($block_size_cap_used)){
            $block_size_cap_used = 512;
          }
        }

        my $free_cap  = "";
        my $pool_id   = $volume->{PROPERTY}->{"container-serial"}->{content};
        my $pool_name = $volume->{PROPERTY}->{"storage-pool-name"}->{content};
        if ( !defined $pool_id || $pool_id eq "" ) {
          $pool_id = $volume->{PROPERTY}->{"virtual-disk-serial"}->{content};
        }
        if ( defined $pool_id && $pool_id ne "" && defined $volume_id && $volume_id ne "" && defined $inventory_performance{$section}{$volume_id} ) {
          $inventory_performance{$section}{$volume_id}{pool_id} = $pool_id;
          if ( defined $pool_name ) {
            $inventory_performance{$section}{$volume_id}{pool_name} = $pool_name;
          }
        }
        if ( defined $volume_id && $volume_id ne "" ) {
          if ( defined $cap && $cap ne "" ) {
            $inventory_performance{$section}{$volume_id}{"capacity_total"} = $cap * $block_size_cap;
          }
          if ( defined $used_cap && $used_cap ne "" ){
            $inventory_performance{$section}{$volume_id}{"capacity_used"} = $used_cap * $block_size_cap_used;
          }
          if ( defined $inventory_performance{$section}{$volume_id}{"capacity_used"} && defined $inventory_performance{$section}{$volume_id}{"capacity_total"} ) {
            my $capacity = $inventory_performance{$section}{$volume_id}{"capacity_total"};
            my $used     = $inventory_performance{$section}{$volume_id}{"capacity_used"};
            if ( Xormon::isdigit($capacity) && Xormon::isdigit($used) ) {
              $inventory_performance{$section}{$volume_id}{"capacity_free"} = round( $capacity - $used, 0 );
            }
          }
        }
      }
    }
  }
  return %inventory_performance;
}




    #print Dumper \$xml_inventory;

sub set_data {
  my ( $tmp_hash, $tmp_hash_inventory, $type_data ) = @_;
  my %data                  = %{$tmp_hash};
  my %inventory_performance = %{$tmp_hash_inventory};
  my $name                  = "";
  my $type                  = "";
  my $id                    = "";

  if ( defined $data{"DeviceID#"} && ( $type_data eq "volume" || $type_data eq "drive" || $type_data eq "port" ) ) {
    $id = $data{"DeviceID#"};
  }
  if ( $type_data eq "port" && defined $data{Name} ) {
    $id = $data{Name};
  }

  #if (defined $data{"PoolID"} && $type_data eq "pool"){
  #  $id = $data{"PoolID"};
  #}
  if ( defined $data{"InstanceID#"} && $type_data eq "pool" ) {
    ( undef, undef, $id ) = split( ":", $data{"InstanceID#"} );
  }
  if ( defined $data{"Name#"} && $type_data eq "node" ) {
    ( undef, $id ) = split( ":", $data{"Name#"} );
    if ( defined $id && $id ne "" ) {
      $name = $id;
    }
    elsif ( defined $data{"ElementName"} ) {
      $id   = $data{"ElementName"};
      $name = $id;
    }
    else { $id = ""; }
  }
  if ( defined $data{"RecordID#"} && $type_data eq "event" ) {
    $id = $data{"RecordID#"};
  }
  if ( defined $data{"ElementName"} && $type_data eq "event" ) {
    $name = $data{"ElementName"};
  }

  foreach my $element ( sort keys %data ) {
    if ( $element eq "InstanceID#" && $type_data eq "perf" ) {
      if ( $data{$element} =~ m/ConstituentVolumeStatistics/ ) {
        ### exeption for ConstituentVolumeStatistics in volumes statistics
        last;
      }
    }
    if ( $element eq "ElementName" && $type_data ne "node" ) {    ### node has got id and name same
      $name = $data{$element};
      if ( $type_data eq "perf" && $name =~ m/controller_/i ) {
        $name =~ s/controller_//g;
        my $upper_name = uc $name;
        $name = $upper_name;
      }
      next;
    }
    if ( $element eq "ElementType" ) {
      $type = $data{$element};
      next;
    }
    if ( $element eq "Name" && $type_data eq "port" ) {
      $name = $data{$element};
      next;
    }
    if ( $element =~ m/^Total|^Read|^Write|^KBytes/ && $type ne "" && $name ne "" && $type_data eq "perf" ) {
      my $section = get_name_section($type);
      if ( $section eq "" ) { next; }
      if ( $section eq "Port Level Statistics" && $name =~ m/^hostport_/ ) {
        $name =~ s/hostport_//;
        $id = $name;

        #my $id_port = get_id_port($name);
        #if (defined $id_port && $id_port ne ""){
        #  $id = $id_port;
        #}
      }
      if ( $section eq "Drive Level Statistics" && $name =~ m/_/ && $name =~ m/\./ ) {
        my $new_name = get_id_from_name_drive($name);
        if ( $new_name ne "" ) {
          $name = $new_name;
        }

      }
      $inventory_performance{$section}{$name}{$element} = $data{$element};
      $inventory_performance{$section}{$name}{name} = $name;
      if ( $id ne "" ) {
        $inventory_performance{$section}{$name}{id} = $id;
      }
      else {
        $inventory_performance{$section}{$name}{id} = $name;
      }
      $inventory_performance{$section}{$name}{SECTION} = $section;
      next;
    }
    if ( $id ne "" && $type_data eq "volume" && $element ne "Name" ) {
      $inventory_performance{$id}{$element} = $data{$element};
      $inventory_performance{$id}{name}     = $name;
      $inventory_performance{$id}{id}       = $id;
      next;
    }
    if ( $id ne "" && ( $type_data eq "pool" || $type_data eq "drive" || $type_data eq "port" || $type_data eq "node" || $type_data eq "event" ) ) {
      $inventory_performance{$id}{$element} = $data{$element};
      $inventory_performance{$id}{name}     = $name;
      $inventory_performance{$id}{id}       = $id;
      next;
    }
  }
  return %inventory_performance;

}

sub get_id_from_name_drive {
  my $name = shift;
  ### disk_00.02 to 0-2
  ( undef, my $number ) = split( /_/, $name );
  ( my $first_n, my $second_n ) = split( /\./, $number );
  my $first_val  = "";
  my $second_val = "";
  if ( $first_n =~ m/^0/ ) {
    $first_val = chop($first_n);
  }
  else {
    $first_val = $first_n;
  }
  if ( $second_n =~ m/^0/ ) {
    $second_val = chop($second_n);
  }
  else {
    $second_val = $second_n;
  }

  if ( defined $first_val && $first_val ne "" && defined $second_val && $second_val ne "" ) {
    return "$first_val-$second_val";
  }
  else {
    return "";
  }

}

sub convert_data_config{
  #my %metrics = convert_metric("conf");
  #print Dumper \%inventory_configuration;
  foreach my $section (keys %inventory_configuration){
    foreach my $id (keys %{$inventory_configuration{$section}}){
      my $uid = $id;
      my ($type) = split(" ", $section);
      $type = lc($type);
      if ($section =~ m/Raid Group/){
        $type = "raid_group";
      }
      if ($section =~ m/Port-Mapping/){
        $type = "port_mapping";
      }
      if ($type eq "port"){
        delete $inventory_configuration{$section}{$id};
        next;
      }
      if ($section !~ m/Pool|Volume/){
        #print "$st_serial | $st_type | $type | $id\n";
        $uid = Xormon::create_hash_key($st_serial.$type.$id);
      }
      #else{
      #  #if ($storage_type eq "wbi"){
      #  #  $uid = $uid . "wbi";
      #  #}
      #}
      foreach my $metric (keys %{$inventory_configuration{$section}{$id}}){
        if (defined $inventory_configuration{$section}{$id}{$metric}){
          $configuration_data{$uid}{$metric} = $inventory_configuration{$section}{$id}{$metric};
        }
      }
      if (defined $inventory_configuration{$section}{$id}{name}){
        $configuration_data{$uid}{label} = $inventory_configuration{$section}{$id}{name};
      }
      if ($type eq "port_mapping" && defined $inventory_configuration{$section}{$id}{Host}){
        $configuration_data{$uid}{label} = $inventory_configuration{$section}{$id}{Host};
      }
      if (defined $inventory_configuration{$section}{$id}{pool_id} && $type eq "volume"){
        if (defined $performance_data{$st_type}{$type}{$id}){
          push(@{$configuration_data{$uid}{parents}}, $inventory_configuration{$section}{$id}{pool_id});
        }
      }
      if (defined $inventory_configuration{$section}{$id}{"Volume IDs"} && $type eq "host"){
        my @volumes = split(" ",$inventory_configuration{$section}{$id}{"Volume IDs"});
        foreach my $el (@volumes){
          if (defined $performance_data{$st_type}{volume}{$el}){
            push(@{$configuration_data{$uid}{children}}, $el);
          }
        }
      }
      $configuration_data{$uid}{TYPE} = $type;
      #print Dumper \$configuration_data{$uid};
    }
  }
  ### add device parameters (storage info)
  my $section = "node";
  if ($storage_type ne "wbi"){
    undef %inventory_configuration;
  }
  if ($storage_type ne "wbi"){
    %inventory_configuration = set_configuration_element( $section, \%inventory_configuration, "$debug_data/CIM_ComputerSystem.txt" );
    #my @metrics = ("status", "PrimaryOwnerName", "PrimaryOwnerContact", "OtherIdentifyingInfo", "NumberOfDrives", "NumberOfStoragePools");
    my @metrics = ("PrimaryOwnerName", "PrimaryOwnerContact", "OtherIdentifyingInfo", "NumberOfDrives", "NumberOfStoragePools");
    foreach my $id ( keys %{ $inventory_configuration{$section} } ) {
      $configuration_data{$st_serial}{"machine_name"} = $id;
      foreach my $metric (@metrics){
        if (defined $inventory_configuration{$section}{$id}{$metric}){
          $configuration_data{$st_serial}{$metric} = $inventory_configuration{$section}{$id}{$metric};
        #if ($metric eq "OtherIdentifyingInfo"){
        #  my ($value, undef) = split(",", $inventory_configuration{$section}{$id}{$metric});
        #  $configuration_data{$st_serial}{host} = $value;
        #  $configuration_data{$id}{host} = $value;
        #}
        }
      }
    }
    #if (defined $inventory_configuration{$section}{$id}{"Bundle Version"}){
    #  $configuration_data{$id}{version} = $inventory_configuration{$section}{$id}{"Bundle Version"};
    #}
    #else{
    #  print Dumper \$inventory_configuration{$section}{$id};
    #}
  }
  my $system = cmd_api("system","name",0);
  if ( defined $system->{OBJECT} && ref( $system->{OBJECT} ) eq "ARRAY" ) {
    my $machine_name   = "";
    my $system_contact = "";
    my $health_status  = "";
    foreach my $node ( @{ $system->{OBJECT} } ) {
      my $serial = $node->{PROPERTY}->{"midplane-serial-number"}->{content};
      if ( !defined $serial || $serial eq "" ) { next; }
      $configuration_data{$st_serial}{"serial"} = $serial;
      my $system_name = $node->{PROPERTY}->{"system-name"}->{content};
      my $product = $node->{PROPERTY}->{"product-id"}->{content};
      my $vendor  = $node->{PROPERTY}->{"scsi-vendor-id"}->{content};
      if ( defined $system_name ) {
        $machine_name = $system_name;
      }
      if ( defined $vendor ) {
        $machine_name = $machine_name . " ($vendor ";
      }
      if ( defined $product ) {
        $machine_name = $machine_name . "$product)";
      }
      if ( defined $node->{PROPERTY}->{"system-contact"}->{content} ) {
        $system_contact = $node->{PROPERTY}->{"system-contact"}->{content};
      }
      if ( defined $node->{PROPERTY}->{"health"}->{content} ) {
        $health_status = $node->{PROPERTY}->{"health"}->{content};
      }
      if ($storage_type eq "wbi"){
        $configuration_data{$st_serial}{"machine_name"} = $machine_name;
        $configuration_data{$st_serial}{"status"} = $health_status;
        $configuration_data{$st_serial}{"PrimaryOwnerName"} = $system_contact;
        $configuration_data{$st_serial}{"PrimaryOwnerContact"} = $system_contact;
        my $size_pools = keys %{$inventory_configuration{"Pool Level Configuration"}};
        my $size_drives = keys %{$inventory_configuration{"Drive Level Configuration"}};
        $configuration_data{$st_serial}{"NumberOfDrives"} = $size_drives;
        $configuration_data{$st_serial}{"NumberOfStoragePools"} = $size_pools;
      }
    }
  }
  if ( defined $system->{OBJECT} && ref( $system->{OBJECT} ) eq "HASH" ) {
    my $serial = $system->{OBJECT}->{PROPERTY}->{"midplane-serial-number"}->{content};
    if ( !defined $serial || $serial eq "" ) { return; }
    $configuration_data{$st_serial}{"serial"} = $serial;
    return;
  }

  #print Dumper \%configuration_data;
}

sub convert_data{
  my %metrics = convert_metric("perf");
  #print Dumper \%inventory_performance;
  foreach my $section (keys %inventory_performance){
    foreach my $id (keys %{$inventory_performance{$section}}){
      #print Dumper \$inventory_performance{$section}{$id};
      #print "$section $id\n";
      #print Dumper \$inventory_performance{$section}{$id};
      if ($id eq "TIME" || !defined $inventory_performance{$section}{TIME}{TO} || !Xormon::isdigit(str2time($inventory_performance{$section}{TIME}{TO})) || !defined $inventory_performance{$section}{$id}{ID}){next;}
      my ($type) = split(" ", $section);
      $type = lc($type);
      #print "$section $id 2222\n";
      #my $test = split(" ",$section);
      #my ($test2) = split(" ", $section);
      #print "$test $test2\n";
      my $time = str2time($inventory_performance{$section}{TIME}{TO});
      if ($section =~ m/Raid Group/){
        $type = "raid_group";
      }
      #print "$section\n";
      #if ($type ne "drive" && $type ne "volume" && $type ne "pool" && $type ne "raid_group && $type ne "port"){next;}
      #print "$type\n";
      my $uid = $id;
      if ($section !~ m/Pool|Volume/){
        #print "$st_serial | $type | $id\n";
        $uid = Xormon::create_hash_key($st_serial.$type.$id);
      }
      #else{
        #if ($storage_type eq "wbi"){
        #  $uid = $uid . "wbi";
        #}
      #}
      foreach my $metric (keys %{$inventory_performance{$section}{$id}}){
        if (defined $metrics{$metric} && Xormon::isdigit($inventory_performance{$section}{$id}{$metric})){
          #print "jsem tu $type $id\n";
          if ($metric =~ m/KBytes/ ){
            $inventory_performance{$section}{$id}{$metric} = round($inventory_performance{$section}{$id}{$metric} * 1000,0);
          }
          #print "$st_type | $type $uid $time " . $metrics{$metric} . " " . $inventory_performance{$section}{$id}{$metric} . "\n";
          $performance_data{$st_type}{$type}{$uid}{$time}{$metrics{$metric}} = $inventory_performance{$section}{$id}{$metric};
        }
      }
    }
  }
  #print Dumper \%performance_data;

}

sub convert_metric{
  my $type = shift;
  my %metrics;
  if ($type eq "perf" ){
    $metrics{"ReadIOs"} = "read_io";
    $metrics{"WriteIOs"} = "write_io";
    $metrics{"TotalIOs"} = "io_rate";
    $metrics{"KBytesRead"} = "read";
    $metrics{"KBytesWritten"} = "write";
    $metrics{"KBytesTransferred"} = "data_rate";
    $metrics{"ReadHitIOs"} = "r_cache_hit_io";
    $metrics{"WriteHitIOs"} = "w_cache_hit_io";
    $metrics{"resp_t"} = "resp_t";
    $metrics{"resp_t_w"} = "resp_t_w";
    $metrics{"resp_t_r"} = "resp_t_r";
    $metrics{"Response time (ms)"} = "resp_t";
    $metrics{"Response write (ms)"} = "resp_t_w";
    $metrics{"Response read (ms)"} = "resp_t_r";
    $metrics{"capacity_total"} = "capacity_total";
    $metrics{"capacity_used"} = "capacity_used";
    $metrics{"capacity_free"} = "capacity_free";
  }


  return %metrics;

}


sub set_configuration_element {

  ###Maximum size, in KBytes, of media supported by this Device. KBytes is interpreted as the number of bytes multiplied by 1000 (NOT the number of bytes multiplied by 1000) = disk level statistics

  my ( $section, $tmp_hash, $file, $unit ) = @_;
  my %inventory_performance = %{$tmp_hash};
  ( my $type ) = split( / /, $section );
  my $lc_type = lc $type;
  $type = $lc_type;
  if ( !defined $unit ) {
    $unit = "TB";    ### default
  }

  my %inventory_disk = set_inventory_data( $file, "$type" );
  if ( $type eq "event" ) {
    return %inventory_disk;
  }

  foreach my $id ( keys %inventory_disk ) {
    if ( $type eq "drive" ) {
      if ( $section =~ m/Configuration/ ) {
        $inventory_performance{$section}{$id} = $inventory_disk{$id};
      }
      ### status
      my $metric = "HealthState";
      if ( defined $inventory_disk{$id}{$metric} ) {

        #$inventory_performance{$section}{$id}{"status"} = get_health_status( $inventory_disk{$id}{$metric} );
        ( my $status, my $description ) = split( ";", get_health_status( $inventory_disk{$id}{$metric} ) );
        if ( defined $status ) {
          $inventory_performance{$section}{$id}{"status"} = $status;
        }
        if ( defined $description ) {
          $inventory_performance{$section}{$id}{"description"} = $description;
          }
      }
      $metric = "MaxMediaSize";
      #MaxMediaSize	uint64
      #Description	string	Maximum size, in KBytes, of media supported by this Device. KBytes is interpreted as the number of bytes multiplied by 1000 (NOT the number of bytes multiplied by 1024).
      if ( defined $inventory_disk{$id}{$metric} && Xormon::isdigit( $inventory_disk{$id}{$metric} ) ) {
        ### KB to B
        my $transfer_unit = $inventory_disk{$id}{$metric} * 1000;
        if ( defined $inventory_performance{$section}{$id} ) {
          #print "$section $id capacity_total $transfer_unit\n";
          $inventory_performance{$section}{$id}{"capacity_total"} = round( $transfer_unit, 0 );
        }
        else {
          delete $inventory_performance{$section}{$id};
        }
      }
      $metric = "Name";
      if ( defined $inventory_disk{$id}{$metric} && defined $inventory_performance{$section}{$id} ) {
        $inventory_performance{$section}{$id}{"name"} = $inventory_disk{$id}{$metric};
      }
      next;
    }
    if ( $type eq "node" && $section =~ m/Configuration/ ) {
      #print Dumper \$inventory_disk{$id};
      $inventory_performance{$section}{$id} = $inventory_disk{$id};
      if ( defined $inventory_performance{$section}{$id}{"Name#"} && $inventory_performance{$section}{$id}{"Name#"} !~ m/:/ ) {
        ### delete not controller here global system it is display under configuration information
        delete $inventory_performance{$section}{$id};
        next;
      }
      my $metric = "HealthState";
      if ( defined $inventory_disk{$id}{$metric} ) {

        #$inventory_performance{$section}{$id}{"status"} = get_health_status( $inventory_disk{$id}{$metric} );
        ( my $status, my $description ) = split( ";", get_health_status( $inventory_disk{$id}{$metric} ) );
        if ( defined $status ) {
          $inventory_performance{$section}{$id}{"status"} = $status;
        }
        if ( defined $description ) {
          $inventory_performance{$section}{$id}{"description"} = $description;
        }
      }
      $metric = "OtherIdentifyingInfo[]";
      if ( defined $inventory_disk{$id}{$metric} ) {
        $inventory_performance{$section}{$id}{"OtherIdentifyingInfo"} = $inventory_disk{$id}{$metric};
      }
      next;

    }
    if ( $type eq "node" && $section eq "node" ) {
      ### global information
      $inventory_performance{$section}{$id} = $inventory_disk{$id};
      if ( defined $inventory_performance{$section}{$id}{"Name#"} && $inventory_performance{$section}{$id}{"Name#"} =~ m/:/ ) {
        ### delete not controller here global system it is display under configuration information
        delete $inventory_performance{$section}{$id};
        next;
      }
      my $metric = "HealthState";
      if ( defined $inventory_disk{$id}{$metric} ) {

        #$inventory_performance{$section}{$id}{"status"} = get_health_status( $inventory_disk{$id}{$metric} );
        ( my $status, my $description ) = split( ";", get_health_status( $inventory_disk{$id}{$metric} ) );
        if ( defined $status ) {
          $inventory_performance{$section}{$id}{"status"} = $status;
        }
        if ( defined $description ) {
          $inventory_performance{$section}{$id}{"description"} = $description;
        }
      }
      $metric = "OtherIdentifyingInfo[]";
      if ( defined $inventory_disk{$id}{$metric} ) {
        $inventory_performance{$section}{$id}{"OtherIdentifyingInfo"} = $inventory_disk{$id}{$metric};
      }
      next;

    }
    if ( $type eq "port" ) {
      if ( $section !~ m/Configuration/ ) {
        $inventory_performance{$section}{$id} = $inventory_disk{$id};
      }
      if ( $section =~ m/Configuration/ && defined $inventory_performance{$section}{$id} ) {
        my $metric = "Speed";
        if ( defined $inventory_disk{$id}{$metric} ) {
          $inventory_performance{$section}{$id}{"port_speed"} = $inventory_disk{$id}{$metric};
        }
        $metric = "HealthState";
        if ( defined $inventory_disk{$id}{$metric} ) {

          #$inventory_performance{$section}{$id}{"status"} = get_health_status( $inventory_disk{$id}{$metric} );
          ( my $status, my $description ) = split( ";", get_health_status( $inventory_disk{$id}{$metric} ) );
          if ( defined $status ) {
            $inventory_performance{$section}{$id}{"status"} = $status;
          }
          if ( defined $description ) {
            $inventory_performance{$section}{$id}{"description"} = $description;
          }
        }
      }
      next;
    }
    if ( $type eq "volume" ) {
      my $metric         = "CreationClassName#";
      my $total_capacity = "";
      my $free_capacity  = "";
      my $used_capacity  = "";
      my $block          = "";

      ### for configuration
      if ( $section =~ m/Configuration/ ) {
        if ( defined $inventory_disk{$id}{$metric} ) {
          if ( $inventory_disk{$id}{$metric} =~ m/ConstituentVolume/ ) {
            next;
          }
          else {
            $metric = "id";
          }
        }
        $inventory_performance{$section}{$id} = $inventory_disk{$id};
        if ( defined $inventory_disk{$id}{$metric} ) {
          $inventory_performance{$section}{$id}{lun_id} = $inventory_disk{$id}{$metric};
        }
      }
      ###
      ### pool id
      $metric = "CurrentOwner";
      if ( defined $inventory_disk{$id}{$metric} ) {
        $inventory_performance{$section}{$id}{pool_id} = $inventory_disk{$id}{$metric};
        ### if exist pool set up value true for active pool
        my $pool_id = $inventory_disk{$id}{$metric};
        if ( defined $inventory_performance{"Pool Level Configuration"}{$pool_id} ) {
          $inventory_performance{"Pool Level Configuration"}{$pool_id}{ACTIVE} = "TRUE";
        }
      }
      $metric = "BlockSize";
      if ( defined $inventory_disk{$id}{$metric} ) {
        $block = $inventory_disk{$id}{$metric};
      }
      $metric = "NumberOfBlocks";
      if ( defined $inventory_disk{$id}{$metric} ) {
        $total_capacity = $inventory_disk{$id}{$metric};
      }
      $metric = "ConsumableBlocks";
      if ( defined $inventory_disk{$id}{$metric} ) {
        $used_capacity = $inventory_disk{$id}{$metric};
      }
      if ( $block ne "" && $total_capacity ne "" && $used_capacity ne "" && Xormon::isdigit($block) && Xormon::isdigit($total_capacity) && Xormon::isdigit($used_capacity) ) {
        ### convert from block to MB
        my $sum = ( $used_capacity * 512 );
        $used_capacity  = round( $sum, 0 );
        $sum            = ( $total_capacity * 512 );
        $total_capacity = round( $sum, 0 );
        $free_capacity  = round( $total_capacity - $used_capacity, 0 );
        $metric         = "capacity_total";
        if ( defined $inventory_performance{$section}{$id} ) {
          $inventory_performance{$section}{$id}{$metric} = $total_capacity;
        }
        else {
          delete $inventory_performance{$section}{$id};
        }

        #$metric = "Used(MB)";
        #if (defined $inventory_performance{$section}{$id}){
        #  $inventory_performance{$section}{$id}{$metric} = $used_capacity;
        #}
        #else{
        #  delete $inventory_performance{$section}{$id};
        #}
        #$metric = "Free(MB)";
        #if (defined $inventory_performance{$section}{$id}){
        #  $inventory_performance{$section}{$id}{$metric} = $free_capacity;
        #}
        #else{
        #  delete $inventory_performance{$section}{$id};
        #}
      }
      next;
    }
    if ( $type eq "pool" ) {
      my $total_capacity = "";
      my $used_capacity  = "";
      my $free_capacity  = "";
      my $convert        = "";
      ### for configuration add element because hash is empty
      if ( $section =~ m/Configuration/ ) {
        $inventory_performance{$section}{$id} = $inventory_disk{$id};
      }
      ### status
      my $metric = "HealthState";
      if ( defined $inventory_disk{$id}{$metric} ) {

        #$inventory_performance{$section}{$id}{"status"} = get_health_status( $inventory_disk{$id}{$metric} );
        ( my $status, my $description ) = split( ";", get_health_status( $inventory_disk{$id}{$metric} ) );
        if ( defined $status ) {
          $inventory_performance{$section}{$id}{"status"} = $status;
        }
        if ( defined $description ) {
          $inventory_performance{$section}{$id}{"description"} = $description;
        }
      }
      ###
      $metric = "TotalManagedSpace";
      if ( defined $inventory_disk{$id}{$metric} ) {
        $total_capacity = $inventory_disk{$id}{$metric};
      }
      $metric = "RemainingManagedSpace";
      if ( defined $inventory_disk{$id}{$metric} ) {
        $free_capacity = $inventory_disk{$id}{$metric};

      }
      $metric = "ConsumedResourceUnits";
      if ( defined $inventory_disk{$id}{$metric} ) {
        $inventory_disk{$id}{$metric} =~ s/"//g;
        $convert = $inventory_disk{$id}{$metric};
      }
      if ( $convert ne "" && $total_capacity ne "" && $free_capacity ne "" && Xormon::isdigit($total_capacity) && Xormon::isdigit($free_capacity) ) {
        my $denominator = convert_unit($convert);
        if ( $unit eq "GB" ) { $denominator = $denominator / 1000; }
        $denominator = 1;

        my $sum = $total_capacity / $denominator;
        #$total_capacity = round( $sum, 3 );
        #if ( $unit eq "GB" ) {
        #  my $check = $total_capacity;
        #  $check = $check / ( 1000 * 1000 );
        #  if ( $check > 100 ) {
        #    $total_capacity = "";
        #  }
        #}
        #if ( $unit ne "GB" ) {
        #  my $check = $total_capacity;
        #  $check = $check / (1000);
        #  if ( $check > 100 ) {
        #    $total_capacity = "";
        #    }
        #}
        if ( !Xormon::isdigit($total_capacity) ) {
          $free_capacity = "";
          $used_capacity = "";
        }
        else {
          $sum           = $free_capacity / $denominator;
          $free_capacity = round( $sum, 0);
          $used_capacity = round( $total_capacity - $free_capacity, 0);
        }
        #$metric = "capacity ($unit)";
        if ( defined $inventory_performance{$section}{$id} ) {
          $inventory_performance{$section}{$id}{"capacity_total"} = $total_capacity;
        }
        else {
          delete $inventory_performance{$section}{$id};
        }
        #$metric = "used_capacity ($unit)";
        if ( defined $inventory_performance{$section}{$id} ) {
          $inventory_performance{$section}{$id}{"capacity_used"} = $used_capacity;
        }
        else {
          delete $inventory_performance{$section}{$id};
        }
        #$metric = "free_capacity ($unit)";
        if ( defined $inventory_performance{$section}{$id} ) {
          $inventory_performance{$section}{$id}{"capacity_free"} = $free_capacity;
        }
        else {
          delete $inventory_performance{$section}{$id};
        }
      }
    }
    next;
  }
  return %inventory_performance;
}

sub set_data_health_status{
  my $status = "ok";

  if ($hs_error == 1){
    $status = "error";
  }
  if ($hs_error == 2){
    $status = "warninig";
  }
  my $reason = $hs_description;

  ### event log
  my %event_log;
  if ($storage_type ne "wbi"){
    %event_log = set_configuration_element( "Event log", \%event_log, "$debug_data/CIM_LogRecord.txt" );
  }
  else{
    my $section = "Event Log";
    if ( defined $inventory_configuration{$section} ) {
      %event_log = %{ $inventory_configuration{$section} };
      #print Dumper \%event_log;
    }
  }
  #print Dumper \%event_log;
  foreach my $id ( sort keys %event_log ) {
    my $name        = "";
    my $event_code  = "";
    my $timestamp   = "";
    my $record_data = "";
    my $error_code  = "";
    my $metric      = "name";
    if ( defined $event_log{$id}{$metric} && $event_log{$id}{$metric} ne "" ) {
      $name = $event_log{$id}{$metric};
    }
    $metric = "eventCode";
    if ( defined $event_log{$id}{$metric} && $event_log{$id}{$metric} ne "" ) {
      $event_code = $event_log{$id}{$metric};
    }
    $metric = "MessageTimestamp#";
    if ( defined $event_log{$id}{$metric} && $event_log{$id}{$metric} ne "" ) {
      if ($event_log{$id}{$metric} =~ m/ / && $storage_type eq "wbi"){
        $event_log{$id}{$metric} =~ s/ /T/;
        $timestamp = str2time($event_log{$id}{$metric});
      }
      else{
        $timestamp = get_utc_time($event_log{$id}{$metric});
      }
    }
    $metric = "RecordData";
    if ( defined $event_log{$id}{$metric} && $event_log{$id}{$metric} ne "" ) {
      $record_data = $event_log{$id}{$metric};
    }
    $metric = "errorCode";
    if ( defined $event_log{$id}{$metric} && $event_log{$id}{$metric} ne "" ) {
      $error_code = $event_log{$id}{$metric};
    }
    if (Xormon::isdigit($error_code) && $error_code > 0){
      $status = "error";
      $reason = $record_data;
    }
    #print "$timestamp\n";
    my %data;
    $data{id} = $id;
    $data{date} = $timestamp;
    $data{name} = $name;
    $data{description} = $record_data;
    $data{event_code} = $event_code;
    $data{error_code} = $error_code;
    push(@health_status,\%data);
  }
  my %vm_status = (
    "item_id" => $st_serial,
    "status" => $status,
    "data" => $reason
  );
  #my @health_status_global = ();
  push(@health_status_global, \%vm_status);

  #push(@health_status_global, \%vm);
  #print Dumper \@health_status_global;
  #my $rc = $xormon->saveStatus(\@health_status_global);
  #my $status_job = get_status($rc, "health status");
  #$logger->info("Task healh status ".$cfg->{hostalias}." is done. Status: $status_job");
  return;

}

sub get_utc_time{
  #20201110122248
  #20201110122248.000000+120
  my $time = shift;
  my @date = split("",$time);
  my @t = ();
  if (defined $date[0] && defined $date[1] && defined $date[2] && defined $date[3] && defined $date[4] && defined $date[5] && defined $date[6] && defined $date[7] && defined $date[8] && defined $date[9] && defined $date[10] && defined $date[11] && defined $date[12] && defined $date[13]){
    $t[0] = "$date[0]$date[1]$date[2]$date[3]/$date[4]$date[5]/$date[6]$date[7]";
    $t[1] = "$date[8]$date[9]:$date[10]$date[11]:$date[12]$date[13]";
    my (undef, $timezone) = split(/\+/,$time);
    if (Xormon::isdigit($timezone)){
      $timezone = $timezone / 60;
      if (length($timezone) == 1){
        $t[2] = "+0$timezone:00";
      }
      else{
        $t[2] = "+$timezone:00";
      }
    }
    else{
      my (undef, $timezone) = split(/-/,$time);
      if (Xormon::isdigit($timezone)){
        $timezone = $timezone / 60;
        if (length($timezone) == 1){
          $t[2] = "-0$timezone:00";
        }
        else{
          $t[2] = "-$timezone:00";
        }
      }
    }
  }
  if ((scalar @t) > 0){
    return str2time( join( " ", @t ) );
  }
  return $time;
}

sub get_health_status {
  my $status_code = shift;
  if ( !Xormon::isdigit($status_code) ) {
    return "$status_code";
  }

  if ( $status_code == 5 ) {
    return "Ok;The element is fully functional and is operating within normal operational parameters and without error";
  }

  if ( $status_code == 30 ) {
    return "Non-recoverable Error;The element has completely failed, and recovery is not possible. All functionality provided by this element has been lost";
  }

  if ( $status_code == 20 ) {
    return "Major Failure;The element is failing. It is possible that some or all of the functionality of this component is degraded or not working";
  }

  if ( $status_code == 10 ) {
    return "Degraded/Warning;The element is in working order and all functionality is provided. However, the element is not working to the best of its abilities. For example, the element might not be operating at optimal performance or it might be reporting recoverable errors";
  }

  if ( $status_code == 15 ) {
    return "Minor Failure;All functionality is available but some might be degraded";
  }

  if ( $status_code == 25 ) {
    return "Critical Failure;The element is non-functional and recovery might not be possible";
  }
  if ( $status_code == 0 ) {
    return "Unknown;The implementation cannot report on HealthState at this time. DMTF has reserved the unused portion of the continuum for additional HealthStates in the future";
  }

  return "$status_code";

}


sub get_array_data {
  my $file = shift;
  if ( test_file_exist($file) ) {
    open( FH, "< $file" ) || $logger->error( "Cannot read $file: $!" . __FILE__ . ":" . __LINE__ ) && return ();
    my @file_all = <FH>;
    close(FH);
    return @file_all;
  }
  else {
    $logger->error( "File not exist $file: $!" . __FILE__ . ":" . __LINE__ );
    my @file_empty = "";
    return @file_empty;
  }
}


sub cmd_smis{
  my $endpoint = shift;
  my $type = shift;
  my $timeout = shift;
  my $save = shift;
  my $cmd = "$path_wbencli -nl -t -noverify ei" . " \'$protocol://$cfg->{'username'}:'$cfg->{'password'}'" . "@" . "$host_smis:$port/root/$type:$endpoint\'";
  my $cmd_to_log = "$path_wbencli -nl -t -noverify ei" . " \'$protocol://$cfg->{'username'}:'XXXXXX'" . "@" . "$host_smis:$port/root/$type:$endpoint\'";
  my ( $fh, $filename ) = tempfile( UNLINK => 1 );
  my $return = 1;
  $ENV{ERROR_DOTHILL_LOG} = $filename;
  my $file = "$endpoint.txt";
  if ($endpoint eq "CIM_BlockStorageStatisticalData" && -f "$debug_data/$file"){
    rename( "$debug_data/$file", "$debug_data/$file" . "_old" );
  }

  my $pid = fork();
  if ( !defined $pid ) {
    if (!$connection_test){
      $logger->error( "no fork $!" . __FILE__ . ":" . __LINE__ );
    }
    else{
      push(@rows_api_test,"no fork $!" . __FILE__ . ":" . __LINE__ . "\n");
      push(@rows_api_test_data,"no fork $!" . __FILE__ . ":" . __LINE__ . "\n");
    }
    return 0;
  }
  elsif ( $pid == 0 ) {
    setpgrp( 0, 0 );
    if (defined $save){
      my $out = `$cmd >> $debug_data/$file 2>>\$ERROR_DOTHILL_LOG`;
    }
    else{
      my $out = `$cmd > $debug_data/$file 2>>\$ERROR_DOTHILL_LOG`;
    }
    exit(0);
  }
  eval {
    local $SIG{ALRM} = sub { die "die in SIGALRM"; };
    alarm($timeout);
    waitpid( $pid, 0 );
    alarm(0);
  };
  if ( $@ =~ /die in SIGALRM/ ) {
    my $act_time = localtime();
    if (!$connection_test){
      $logger->error( "$act_time command: $cmd_to_log timed out after: $timeout seconds " . __FILE__ . ":" . __LINE__ );
    }
    else{
      push(@rows_api_test,"$act_time command: $cmd_to_log timed out after: $timeout seconds " . __FILE__ . ":" . __LINE__ . "\n");
      push(@rows_api_test_data,"$act_time command: $cmd_to_log timed out after: $timeout seconds " . __FILE__ . ":" . __LINE__ . "\n");

    }
    kill -15, $pid;
    waitpid( $pid, 0 );
  }
  if (test_file_exist($filename) && !-z $filename){
    my $error = `cat $filename`;
    $cmd =~ s/([^:]*:[^:]*):.*\@/$1:XXXXX\@/;
    if (!$connection_test){
      $logger->error( "Command failed: $cmd " . __FILE__ . ":" . __LINE__ );
      $logger->error($error);
    }
    else{
      push(@rows_api_test_tmp,"Command failed: $cmd " . __FILE__ . ":" . __LINE__ . "\n");
      push(@rows_api_test_tmp,"$error\n");
      push(@rows_api_test_data,"Command failed: $cmd " . __FILE__ . ":" . __LINE__ . "\n");
      push(@rows_api_test_data,"$error\n");

    }
    return 0;
  }
  return $return;
}

sub get_name_section {
  my $type = shift;

  if ( $type == 4 ) {
    return "Node Level Statistics";
  }
  if ( $type == 6 ) {
    return "Port Level Statistics";
  }
  if ( $type == 8 ) {
    return "Volume Level Statistics";
  }
  if ( $type == 10 ) {
    return "Drive Level Statistics";
  }

  return "";

}

sub get_data {
  my $section          = shift;
  my $section_contents = shift;
  my @values           = "";
  my @headers          = split( ",", $section_contents );
  foreach my $property ( keys %{ $inventory_performance{$section} } ) {
    if ( !defined $inventory_performance{$section}{$property}{ $headers[0] } || $inventory_performance{$section}{$property}{ $headers[0] } eq "" ) { next; }
    if ( !@values eq "" ) {
      my $last = pop(@values);
      $last =~ s/,//g;
      push( @values, $last );
    }
    push( @values, "\n" );
    my $index = 1;
    foreach my $element (@headers) {
      chomp $element;
      $element =~ s/^\s+|\s+$//g;
      if ( defined $inventory_performance{$section}{$property}{$element} && $inventory_performance{$section}{$property}{$element} ne "" ) {
        if ( $index != 1 && Xormon::isdigit( $inventory_performance{$section}{$property}{$element} ) && $inventory_performance{$section}{$property}{$element} !~ m/-/ ) {
          push( @values, "$inventory_performance{$section}{$property}{$element}," );
          $index++;
          next;
        }
        if ( $index == 1 ) {
          push( @values, "$inventory_performance{$section}{$property}{$element}," );
          $index++;
          next;
        }
        else {
          push( @values, "," );
          $index++;
          next;
        }
      }
      else {
        push( @values, "," );
        $index++;
      }
      }
  }
  my $last = pop(@values);
  $last =~ s/,//g;
  push( @values, $last );
  return \@values;
}


sub write_data_performance {
  my $header_section_tmp = shift;
  my %header_section = %{$header_section_tmp};
  ( my $file, my $file_tmp ) = split( ",", get_name_file("perf") );
  #print "$file $file_tmp |||\n";
  #print Dumper \%header_section;

  foreach my $title ( keys %header_section ) {
    open( DATA, ">>$file_tmp" ) or error_die("Cannot open file: $file_tmp : $!");
    my ($values_tmp) = get_data( "$title", "$header_section{$title}" );
    my @values = @{$values_tmp};
    if ( !defined $inventory_performance{$title}{TIME}{FROM} || !defined $inventory_performance{$title}{TIME}{TO} || !defined $inventory_performance{$title}{TIME}{LENGTH} ) {
      close DATA;
      next;
    }
    print DATA "$title\n";
    my $interval_start  = " Interval Start:  " . $inventory_performance{$title}{TIME}{FROM};
    my $interval_end    = " Interval End:    " . $inventory_performance{$title}{TIME}{TO};
    my $interval_length = " Interval Length: " . $inventory_performance{$title}{TIME}{LENGTH};
    print DATA" $interval_start";
    print DATA"\n";
    print DATA" $interval_end";
    print DATA"\n";
    print DATA" $interval_length";
    print DATA"\n";
    print DATA "-----------------------------------\n";
    print DATA $header_section{$title};
    print DATA @values;
    print DATA "\n\n";
    close DATA;
  }
  rename( "$file_tmp", "$file" );

}

sub remove_whitespace {
  my $value = shift;
  if ( !defined $value || $value eq "" ) {
    return "";
  }
  $value =~ s/^\s+|\s+$//g;
  return $value;

}


sub logout_api{
  if (!defined $session_key || $session_key eq ""){return;}
  my $url = "$protocol" . "://$host_api/api/exit";
  my $ua = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  $ua->timeout(60);
  my $req = HTTP::Request->new( GET => $url );
  $req->header( 'sessionKey' => $session_key );
  $req->header( 'dataType'   => 'api' );
  my $res = $ua->request($req);

  #print $res->content;
  if ( ! $res->is_success ) {
    $logger->error("Logout from API failed");
  }
  return;
}

sub login_api{
  my $ip = shift;
  my $md5_data = "$cfg->{'username'}" . "_" . "$cfg->{'password'}";
  my $md5_hash = md5_hex($md5_data);

  #print "$md5_hash\n";
  # create the URL and send an http GET request
  my $ua = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  $ua->timeout(60);

  #my $ua = LWP::UserAgent->new;
  my $url = "$protocol" . "://$ip/api/login/" . $md5_hash;

  #print ("Sending to $url\n");
  my $req = HTTP::Request->new( GET => $url );
  my $res = $ua->request($req);
  if ( !$res->is_success ) {
    my $report = $res->content;
    chop $report;
    if (!$connection_test){
      $logger->error("Request error $report : command : login");
    }
    else{
      push(@rows_api_test,"Request error $report : command : login\n");
    }
    return "";
  }
  my $parser  = XML::LibXML->new();
  my $doc     = $parser->parse_string( $res->content );
  my $root    = $doc->getDocumentElement;
  my @objects = $root->getElementsByTagName('OBJECT');
  my @props   = $objects[0]->getElementsByTagName('PROPERTY');
  my $sessionKey = "";
  foreach my $prop (@props) {
    my $name = $prop->getAttribute('name');
    #print "Property = " . $name . "\n";
    if ( $name eq 'response' ) {
      if ( $prop->textContent eq "Authentication Unsuccessful" ) {
        $sessionKey = get_sessionkey($ip);
      }
      else {
        $sessionKey = $prop->textContent;
      }
    }
  }
  return $sessionKey;
}

sub get_sessionkey {

  use MIME::Base64 qw( decode_base64 encode_base64 );
  my $ip = shift;
  my $key        = "$cfg->{username}:$cfg->{password}";
  my $base64_key = encode_base64($key);

  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  $ua->timeout(60);
  my $url = "$protocol" . "://$ip/api/login";
  my $req = HTTP::Request->new( GET => $url );
  $req->header( 'Authorization' => "Basic $base64_key" );
  my $res = $ua->request($req);

  if ( !$res->is_success ) {
    my $report = $res->content;
    if ( $protocol =~ m/https/ || $protocol =~ m/HTTPS/ ) {
      if (!$connection_test){
        $logger->error("Request error \"$report\" : $protocol://$ip/api/login/");
      }
      else{
        push(@rows_api_test,"Request error \"$report\" : $protocol://$ip/api/login/n");
      }
    }
    else {
      if (!$connection_test){
        $logger->error("Request error \"$report\" : $protocol://$ip/api/login/");
      }
      else{
        push(@rows_api_test,"Request error \"$report\" : $protocol://$ip/api/login/\n");
      }
    }
    return "";
  }

  my $parser  = XML::LibXML->new();
  my $doc     = $parser->parse_string( $res->content );
  my $root    = $doc->getDocumentElement;
  my @objects = $root->getElementsByTagName('OBJECT');
  my @props   = $objects[0]->getElementsByTagName('PROPERTY');
  my $sessionKey = "";
  foreach my $prop (@props) {
    my $name = $prop->getAttribute('name');

    #print "Property = " . $name . "\n";
    if ( $name eq 'response' ) {
      $sessionKey = $prop->textContent;
    }
  }
  return $sessionKey;
}

sub calculate_missing_metrics{

  my $item_total = "capacity_total";
  my $item_used = "capacity_used";
  my $item_used_percent = "capacity_used_percent";
  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        #capacity_used_percent
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_used})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_total} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_used} / $performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = round($pom,2);
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{data_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && $performance_data{$st_type}{$type}{$uid}{$time}{io_rate} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{data_rate} / $performance_data{$st_type}{$type}{$uid}{$time}{io_rate});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{read_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{read} / $performance_data{$st_type}{$type}{$uid}{$time}{read_io});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{write_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{write} / $performance_data{$st_type}{$type}{$uid}{$time}{write_io});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = 0;
        }
        my $item_io = "read_io";
        my $item_cache_io = "r_cache_hit_io";
        my $item_cache_hit = "r_cache_hit";
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_cache_io})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_io} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_cache_hit} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_cache_io} /  $performance_data{$st_type}{$type}{$uid}{$time}{$item_io}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_cache_hit} = round($pom,2);
          }
        }
        $item_io = "write_io";
        $item_cache_io = "w_cache_hit_io";
        $item_cache_hit = "w_cache_hit";
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_cache_io})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_io} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_cache_hit} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_cache_io} /  $performance_data{$st_type}{$type}{$uid}{$time}{$item_io}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_cache_hit} = round($pom,2);
          }
        }
      }
    }
  }
}

sub delete_empty_metrics{
  my $first_time = "";
  foreach my $type (keys %{$performance_data{$st_type}}){
    if (!keys %{$performance_data{$st_type}{$type}}){
      delete $performance_data{$st_type}{$type};
      next;
    }
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      if ($type eq "device" && $uid ne $st_serial){ ### I do not know how can be in perf file another uid for device!!!!
        delete $performance_data{$st_type}{$type}{$uid};
        next;
      }
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        if (!Xormon::isdigit($time)){
          delete $performance_data{$st_type}{$type}{$uid};
          last;
        }
        if ($first_time eq "" && Xormon::isdigit($time)){
          $first_time = $time;
        }
        if (Xormon::isdigit($first_time) && Xormon::isdigit($time) && $time < $first_time){
          $first_time = $time;
        }
        foreach my $item (keys %{$performance_data{$st_type}{$type}{$uid}{$time}}){
          if (!Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item}) || $performance_data{$st_type}{$type}{$uid}{$time}{$item} < 0){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
        }
      }
    }
  }
  if (Xormon::isdigit($first_time)){
    $service_data_out->{last_perf_timestamp} = $first_time;
  }
  else{
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      $service_data_out->{last_perf_timestamp} = $service_data_in->{last_perf_timestamp};
    }
  }
}

sub get_storage_type{
  my @storage_types = ( "dhs", "hpq", "smis" );
  my @storage_ip = ();
  push( @storage_ip, $cfg->{'node1Host'} );
  if ( defined $cfg->{'node2Host'} && $cfg->{'node2Host'} ne "" ) {
    push( @storage_ip, $cfg->{'node2Host'});
  }
  my $class_element = "CIM_ComputerSystem";
  my $file          = "$wrkdir/DOTHILL";
  my $timeout       = 60;
  if (defined $service_data_in->{host_api} && $service_data_in->{host_api} ne ""){
    unshift(@storage_ip,$service_data_in->{host_api});
  }
  foreach my $ip (@storage_ip) {
    $session_key = login_api($ip);
    if ($session_key ne ""){
      $host_api = $ip;
      last;
    }
  }
  if ($wbi){
    return "wbi";
  }
  ### test exist DOTHILL file and not empty
  if (defined $service_data_in->{host_smis} && $service_data_in->{host_smis} ne ""){
    unshift(@storage_ip,$service_data_in->{host_smis});
  }
  if (defined $service_data_in->{type} && $service_data_in->{type} ne ""){
    my $type = $service_data_in->{type};
    foreach my $ip (@storage_ip){
      $host_smis = $ip;
      my $out = cmd_smis($class_element,$type,60);
      if ( $out ne "0" && $out ne "" ) {
        $host_smis = $ip;
        return $type;
      }
    }
  }
  foreach my $type (@storage_types) {
    foreach my $ip (@storage_ip) {
      $host_smis = $ip;
      my $out = cmd_smis($class_element,$type,60);
      if ( $out ne "0" && $out ne "" ) {
        write_data( "$type", "$file" );
        $host_smis = $ip;
        return $type;
      }
    }
  }
  if (!$connection_test){
    $logger->error("DOTHILL ERROR: not defined type storage dhs, hpq or smis");
  }
  return "";
}

sub cmd_api{
  my $cmd = shift;
  my $key_word = shift;
  my $execute = shift;
  if (!defined $execute){
    $execute = 1;
  }
  my $cmd_file = $cmd;
  $cmd_file =~ s/ /-/g;
  if ( $cmd =~ m/events/ ) {
    $cmd_file = "events";
  }
  #if ($cmd eq "pools"){
  #  $cmd_file = "pools_" . time();
  #}
  if (!$execute && test_file_exist("$debug_data/$cmd_file.xml")){
    my $data;
    my $file = "$debug_data/$cmd_file.xml";
    eval { $data = XMLin( $file, KeyAttr => { 'PROPERTY' => "$key_word" } ); };
    if ($@) {
      if (!$connection_test){
        $logger->error("XML parsing error: $@");
      }
      else{
        push(@rows_api_test,"XML parsing error: $@\n");
        return;
      }
      eval {
        my $linted = `xmllint --recover $file`;
        $data = XMLin( $linted, KeyAttr => { 'PROPERTY' => "$key_word" } );
      };
      if ($@) {
        if (!$connection_test){
          $logger->error("XML parsing error: $@");
        }
        else{
          push(@rows_api_test,"XML parsing error: $@\n");
          return;
        }
      }
    }
    return $data;
  }
  my $url = "$protocol" . "://$host_api/api/show/$cmd";
  my $ua = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  $ua->timeout(60);
  my $req = HTTP::Request->new( GET => $url );
  $req->header( 'sessionKey' => $session_key );
  $req->header( 'dataType'   => 'ipa' );
  my $res = $ua->request($req);
  #print $res->content;
  if ( ! $res->is_success ) {
    my $out = $res->content;
    if (!$connection_test){
      $logger->error("API command $cmd failed");
      $logger->error($out);
    }
    else{
      push(@rows_api_test,"API command $cmd failed\n");
      push(@rows_api_test,"$out\n");
    }
    return;
  }
  if (!$connection_test){
    $logger->info("CMD API $cmd successful");
  }
  my $out = $res->content;
  if (!$connection_test){
    write_data( "$out", "$debug_data/$cmd_file.xml" );
  }
  my $data;
  eval { $data = XMLin("$out", KeyAttr => { 'PROPERTY' => "$key_word" } ); };
  if ($@) {
    if (!$connection_test){
      $logger->error("XML parsing error: $@");
    }
    else{
      push(@rows_api_test,"XML parsing error: $@\n");
      return;
    }
    eval {
      my $linted = `xmllint --recover "$out"`;
      $data = XMLin( $linted, KeyAttr => { 'PROPERTY' => "$key_word" } );
    };
    if ($@) {
      if (!$connection_test){
        $logger->error("XML parsing error: $@");
      }
      else{
        push(@rows_api_test,"XML parsing error: $@\n");
        return;
      }
    }
  }
  #print Dumper \$data;
  return $data;
}

sub write_data {
  my $out      = shift;
  my $file     = shift;
  my $file_tmp = "$file-tmp";
  if ( $out ne "" ) {
    open( DATA, ">$file_tmp" ) or $logger->error("Cannot open file: $file_tmp : $!") && return 0;
    print DATA $out;
    close(DATA);
    rename( "$file_tmp", "$file" );

  }
}

sub get_name_file {
  my $type       = shift;
  my @time       = localtime();
  my $time_human = localtime();
  my $sec        = $time[0];
  my $minute     = $time[1];
  my $hour       = $time[2];
  my $day        = $time[3];
  my $month      = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;

  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $name_file     = "$wrkdir/$storage_name" . "_dothill$type" . "_" . "$year$month$day" . "_" . "$hour$minute" . ".out";
  my $name_file_tmp = "$wrkdir/$storage_name" . "_dothill$type" . "_" . "$year$month$day" . "_" . "$hour$minute" . ".out-tmp";
  return "$name_file,$name_file_tmp";

}



sub test_file_exist {
  my $file = shift;
  my $run  = shift;
  if ( !defined $file || $file eq "" ) {
    $logger->error("File does not defined !");
    return 0;
  }
  if ( !-e $file || -z $file ) {
    if ( defined $run && -z $file ) { return 1; }
    else {
      return 0;
    }
  }
  else {
    return 1;
  }
}


sub set_architecture{

  my %device_arch = (
    "item_id" => $cfg->{'uuid'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => $st_type,
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'uuid'}
  );
  push(@architecture, \%device_arch);
  #return;

  ### deleting unnecessary metrics
  foreach my $uid (keys %configuration_data){
    foreach my $metric (keys %{$configuration_data{$uid}}){
      if ($metric =~ m/ConsumedResourceUnits|TotalManagedSpace|Usage|SpaceLimit|LowSpaceWarningThreshold|RemainingManagedSpace|StatusDescriptions|Primordial|OperationalStatus|InstallDate|SpaceLimitDetermination|PagedStoragePool|Name|InstanceID|Description|Caption|ThinProvisionMetaDataSpace|description|ResourceType/){
        if ($metric eq "PrimaryOwnerName"){next;};
        delete $configuration_data{$uid}{$metric};
      }
      if ($metric =~ m/DeviceID|TransitioningToState|RequestedState|MaxBlockSize|MinBlockSize|Encryption|MaxMediaSize|NumberOfMediaSupported|EnabledDefault|DiskType|EnabledState|FormFactor|PowerManagementSupported/){
        delete $configuration_data{$uid}{$metric};
      }
      if ($metric =~ m/EnabledState|FU Version|RequestedState| |OtherIdentifyingInfo|Dedicated|TransitioningToState|OtherIdentifyingInfo/){
        delete $configuration_data{$uid}{$metric};
      }
      if ($metric =~ m/ExtentStatus|ExtentDiscriminator|CurrentOwner|CanDelete|BlockSize|NumberOfBlocks|Access|ConsumableBlocks|Type|ThinlyProvisioned|SequentialAccess|DeltaReservation|PreferredOwner|IsComposite|DataOrganization/){
        delete $configuration_data{$uid}{$metric};
      }

    }
  }
  #print Dumper \%configuration_data;
  foreach my $uid (keys %configuration_data){
    my $name = $configuration_data{$uid}{label};
    my $type = $configuration_data{$uid}{TYPE};
    if (!defined $name || !defined $type){
      if ($uid eq $st_serial){next;}
      delete $configuration_data{$uid};
      $logger->error("Element $uid has not label or type! ");
      next;
    }
    my @children = ();
    my @parents = ();
    if (defined $configuration_data{$uid}{children}){
      @children = @{$configuration_data{$uid}{children}};
      delete $configuration_data{$uid}{children};

    }
    if (defined $configuration_data{$uid}{parents}){
      @parents = @{$configuration_data{$uid}{parents}};
      delete $configuration_data{$uid}{parents};
    }
    #print Dumper \@children;
    #print Dumper \@parents;
    my %arch = (
      "item_id" => $uid,
      "label" => $name,
      "hw_type" => $st_type,
      "subsystem" => $type,
      "hostcfg_id" => $cfg->{'uuid'},
      "parents" => [$cfg->{'uuid'}]
      );
    if (@parents){
      push(@{$arch{parents}},@parents);
    }
    if (@children){
      push(@{$arch{children}},@children);
    }
    #if ($type eq "port_type"){
    #  $arch{port_type} = $name;
    #}
    if ($type eq "volume"){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($uid);
    }
    if ($type eq "port" && defined $configuration_data{$uid}{wwn}){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
      $configuration_data{$uid}{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
    }
    #push(@{$arch{parents}},$cfg->{'uuid'});
    push(@architecture, \%arch);
  }
  #print Dumper \@architecture;
  #foreach my $item (keys %{$data{conf}{DEVICE}}){
  #  $configuration_data{$st_serial}{$item} = $data{conf}{DEVICE}{$item};
  #}
  #print Dumper \$configuration_data{DEVICE}{$st_serial};

  #save configuration
  #print Dumper \@architecture;
  $configuration_data{$cfg->{'uuid'}}{events} = encode_json(\@health_status);

  my $rc = $xormon->saveConfJSON(\%configuration_data);
  my $status_configuration = get_status($rc, "saveConfJSON (configuration)");
  #if ($status_configuration eq "error"){
  #  print Dumper \%configuration_data;
  #}
  #$logger->info("Task for configuration ".$cfg->{hostalias}." is done. Status: $status");

  #save architecture
  $rc = $xormon->saveArchitecture(\@architecture);
  my $status_architecture = get_status($rc, "saveArchitecture");
  #$logger->info("Task for saveArchitecture ".$cfg->{hostalias}." is done. Status: $status");

  ### save health status
  #print Dumper \@health_status_global;

  $rc = $xormon->saveStatus(\@health_status_global);
  my $status_health = get_status($rc, "health status");
  return ($status_architecture, $status_configuration, $status_health);

}

sub connection_test {
  my $client = shift;
  $cfg = shift;
  $connection_test = 1;
  my @ports = ();
  my @hosts = ();


  #print Dumper \$cfg;
  # test main variables
  #unless ( exists $cfg->{'hostcfg_id'} )    { push(@rows_api_test, "Cannot continue, 'hostcfg_id' is not configured!");    return Xormon::conn_test(0, \@rows_api_test); }
  #unless ( exists $cfg->{'hostalias'} )   { push(@rows_api_test, "Cannot continue, 'hostalias' is not configured!");   return Xormon::conn_test(0, \@rows_api_test); }
  #unless ( exists $cfg->{'node1Host'} )          { push(@rows_api_test, "Cannot continue, 'node1Host' is not configured!");          return Xormon::conn_test(0, \@rows_api_test); }
  #unless ( exists $cfg->{'proto'} )      { push(@rows_api_test, "Cannot continue, 'proto' is not configured!");      return Xormon::conn_test(0, \@rows_api_test); }
  #unless ( exists $cfg->{'username'} )      { push(@rows_api_test, "Cannot continue, 'username' is not configured!");      return Xormon::conn_test(0, \@rows_api_test); }
  #unless ( exists $cfg->{'password'} )      { push(@rows_api_test, "Cannot continue, 'password' is not configured!"); return Xormon::conn_test(0, \@rows_api_test); }

  ### Validation of variables

  #my $test = Test->new();
  my $test = Test->new($client);
  my @variables = ("hostcfg_id", "hostalias", "node1Host", "proto", "username", "password");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      #return $test->genResult();
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");
  push(@hosts,$cfg->{node1Host});
  if (defined $cfg->{node2Host} && $cfg->{node2Host} ne ""){
    push(@hosts,$cfg->{node2Host});
  }
  #return $test->genResult();

  ## wbemcli check
  if (defined $cfg->{'interface'} && $cfg->{'interface'} eq "WBI"){
    $wbi = 1;
  }
  @variables = ("/usr/bin/wbemcli", "/opt/freeware/bin/wbemcli", "/usr/local/bin/wbemcli");
  foreach my $path (@variables){
    if (-f $path){
      $path_wbencli = $path;
      last;
    }
  }
  if (($path_wbencli eq "" || !-f $path_wbencli) && !$wbi ){
    $test->addSection("Wbemcli install",0, "Error: could not found wbemcli binnary, searched /usr/bin/wbemcli, /opt/freeware/bin/wbemcli, /usr/local/bin/wbemcli\nCheck install sblim-wbemcli https://stor2rrd.com/DotHill-install.php\n");
    #return $test->genResult();
    return $test->stop();
  }
  if (!$wbi){
    $test->addSection("Wbemcli install",1,"");
  }
  $logger = XormonLogger->new( 'dothill', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  $protocol = $cfg->{'proto'};

  if ( $protocol eq "http" ) {
      $port = 5988;
      if (!$wbi){
        push(@ports,$port);
      }
      push(@ports,80);
  }

  if ( $protocol eq "https" ) {
    $port = 5989;
    if (!$wbi){
      push(@ports,$port);
    }
    push(@ports,443);
  }
  $host_smis = $cfg->{'node1Host'};
  $host_api = $cfg->{'node1Host'};
  data_structure();

  ### connection network test ####
  my $code_global = 1;
  my %network_test_check;
  foreach my $ip (@hosts){
    foreach my $port (@ports){
      my ($code, $text) = Xormon::conntest_tcp($ip,$port);
      if ($code_global <= 1 ){
        $code_global = $code;
      }
      if ($code == 1 ){
        if (defined $network_test_check{$ip}{PORT}){
          $network_test_check{$ip}{PORT}++;
          $code_global = 2;
        }
        else{
          $network_test_check{$ip}{PORT} = 1;
        }
      }
      push(@{$network_test_check{DATA}},"$text\n");
    }
  }
  if ($code_global == 2){$code_global = 1;}
  $test->addSection("API host network connection", $code_global, \@{$network_test_check{DATA}});
  if ($code_global == 0){
    #return $test->genResult();
    return $test->stop();
  }
  ### SMI-S API ###
  $storage_type = get_storage_type();
  if ($storage_type eq ""){
    push(@rows_api_test,"SMI-S API failed!\n");
    push(@rows_api_test,"STORAGE TYPE (dhs,hpq,smis) is not defined\n");
    push(@rows_api_test,@rows_api_test_tmp);
    logout_api();
    $test->addSection("SMI-S API authorization", 0 , \@rows_api_test);
    #return $test->genResult();
    #return Xormon::conn_test(0, \@rows_api_test);
    return $test->stop();
  }
  my $return_code;
  if (!$wbi){
    $return_code = cmd_smis("CIM_ComputerSystem",$storage_type,60);
    if (!defined $return_code || $return_code eq "" || $return_code eq ""){
      push(@rows_api_test,"SMI-S API authorization failed!\n");
      logout_api();
      $test->addSection("SMI-S API authorization", 0,\@rows_api_test);
      #return $test->genResult();
      return $test->stop();
    }
  }
  if (-f "$debug_data/CIM_ComputerSystem.txt" && !$wbi){
    my @data = get_array_data("$debug_data/CIM_ComputerSystem.txt");
    my @api_information = ();
    my $id = "";
    foreach my $line (@data) {
      chomp $line;
      $line =~ s/"//g;
      if ( $line eq "" ) { next; }
      if ( $line =~ m/^-Name#=|^-Name=/ ) {
        ( my $element, my $value ) = split( "=", $line );
        if ( defined $value && $value !~ m/:/ ) {
          $id = $value;
          push( @api_information, "Machine name: $id\n" );
          next;
        }
      }
      if ( defined $id && $id ne "" && $line =~ m/PrimaryOwnerName|PrimaryOwnerContact|OtherIdentifyingInfo|NumberOfDrives|NumberOfStoragePools/ ) {
        ( my $element, my $value ) = split( "=", $line );
        if ( defined $element && defined $value ) {
          push( @api_information, "$element: $value\n" );
        }
      }
    }
    push(@rows_api_test,@api_information);
  }
  else{
    push(@rows_api_test,"");
  }
  if (!$wbi){
    $test->addSection("SMI-S API authorization", 1,\@rows_api_test);
  }
  #return $test->genResult();
  if ($session_key eq "Authentication Unsuccessful"){
    push(@rows_api_test,"XML API authorization failed!\n");
    push(@rows_api_test,"Request error: login $session_key\n");
    $test->addSection("XML API authorization",0,\@rows_api_test);
    return $test->stop();
  }
  ### rest api test
  my $data = cmd_api("versions detail", "name", 1);
  if (!defined $data){
    push(@rows_api_test,"XML API authorization failed!\n");
    $test->addSection("XML API authorization",0,\@rows_api_test);
    logout_api();
    #return $test->genResult();
    return $test->stop();
  }
  if ( defined $data && ref($data) eq 'HASH' ) {
    my $version_warning = 0;
    my $node_count      = 0;
    my $node_inventory = $data->{OBJECT};
    if ( ref($node_inventory) eq 'ARRAY' ) {
      $node_count = scalar @{$node_inventory};
      $node_count--;
      foreach my $node ( @{$node_inventory} ) {
        foreach my $item ( keys %{ $node->{PROPERTY} } ) {
          my $content = $node->{PROPERTY}->{$item}->{content};
          my $name    = $node->{PROPERTY}->{$item}->{"display-name"};
          if ( !defined $content || !defined $name || $name ne "Bundle Version" ) { next; }
          if ( $content eq "GT280R010-01" ) {
            $version_warning++;
          }
        }
      }
    }
    if ( $node_count == $version_warning ) {
      $test->addSection("XML API authorization",0,"*** WARNING *** You have firmware GT280R010-01, it has a bug which prevents storage SMI-S provide data, Dell expecting a fix at the end of Q2 2023, no workaround possible\n");
      logout_api();
      return $test->stop();
      #return $test->genResult();
    }
  }

  #print STDERR Dumper \$data;
  if (!$wbi){
    logout_api();
  }
  $test->addSection("XML API authorization", 1,"");
  #return $test->genResult();

  ###########################
  ### API DATA TEST ####
  if (!$wbi){
    $return_code = cmd_smis("CIM_BlockStorageStatisticalData",$storage_type,60);
    if (!defined $return_code || $return_code eq "" || $return_code eq ""){
      push(@rows_api_test_data,"SMI-S data failed!\n");
      $test->addSection("SMI-S data test", 0,\@rows_api_test_data);
      #return $test->genResult();
      return $test->stop();
    }
    $test->addSection("SMI-S data test", 1,"Performance data ok\n");
  }
  else{
    $return_code = cmd_api($cmds_performance[0],"name",1);
    if (!defined $return_code){
      push(@rows_api_test_data,"XML API data failed!\n");
      $test->addSection("XML API data test", 0,\@rows_api_test);
      logout_api();
      #return $test->genResult();
      return $test->stop();
    }
    logout_api();
    $test->addSection("XML API data test", 1, "Performance data ok\n");
  }
  return $test->stop();

}

sub data_structure {
  if (!-d "$data_directory/data"){
    mkdir "$data_directory/data" || $logger->error( "Cannot mkdir $data_directory/data: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial"){
    mkdir "$data_directory/data/$st_serial" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial/IOSTATS"){
    mkdir "$data_directory/data/$st_serial/IOSTATS" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial/IOSTATS: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }


  my $directory = "$data_directory/data/$st_serial/IOSTATS";
  if ( !-d $directory ) {
    mkdir $directory || $logger->error( "Cannot mkdir $directory: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || $logger->error( "Cannot mkdir $directory/configuration: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || $logger->error( "Cannot mkdir $directory/0$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || $logger->error( "Cannot mkdir $directory/$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
  my @entry_time = split( " ", localtime() );
  ( undef, $active_minute, undef ) = split( ":", $entry_time[3] );
  if ( !-d "$directory/$active_minute" ) {
    mkdir "$directory/$active_minute" || $logger->error( "Cannot mkdir $directory/$active_minute: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  $debug_data = "$directory/configuration";
  $wrkdir = "$data_directory/data/$st_serial";

} ## end sub data_structure

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

1;

sub get_time_events {
  my $timestamp_end = time() - ( 3600 * 24 );
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  my $year_new = substr( $year, -2 );
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }

  #get MMDDYYhhmmss
  return "$month$day$year_new$hour$minute$sec";

}

sub set_inventory_xml_data {
  my $suffix = shift;
  my $type   = shift;
  my %data;
  foreach my $cmd (@cmds_performance) {
    my $section = get_section( $cmd, $type );
    if ( $section eq "" ) { next; }
    my %items = get_items( $cmd, $type );
    if ( !%items ) { next; }
    my $file = "$debug_data/$cmd.xml";
    if ( $suffix eq "old" ) {
      $file = "$debug_data/$cmd-old.xml";
    }
    if ( !-f $file ) { next; }
    my $end_timestamp       = ( stat("$file") )[9];
    my $end_date         = get_time($end_timestamp);
    my $data = get_data_from_xml_file($file);
    if ( defined $data->{OBJECT} && ref( $data->{OBJECT} ) eq "ARRAY" ) {
      foreach my $element ( @{ $data->{OBJECT} } ) {
        my $sample_time         = "";
        my $sample_time_numeric = "";
        if ( !defined $element->{PROPERTY}->{"serial-number"}->{content} && $section eq "Volume Level Statistics" )                                                                                  { next; }
        if ( !defined $element->{PROPERTY}->{"durable-id"}->{content}    && ( $section eq "Node Level Statistics" || $section eq "Drive Level Statistics" || $section eq "Port Level Statistics" ) ) { next; }
        my $id = "";
        if ( $section eq "Volume Level Statistics" ) {
          $id = $element->{PROPERTY}->{"serial-number"}->{content};
        }
        if ( $section eq "Node Level Statistics" ) {
          $id = $element->{PROPERTY}->{"durable-id"}->{content};
          $id =~ s/controller_//g;
        }
        if ( $section eq "Port Level Statistics" ) {
          $id = $element->{PROPERTY}->{"durable-id"}->{content};
          $id =~ s/hostport_//g;
        }
        if ( $section eq "Drive Level Statistics" ) {
          $id = $element->{PROPERTY}->{"durable-id"}->{content};
          $id = get_id_from_name_drive($id);
        }
        #if ($id eq ""){next;}
        if ( $id ne "" ) {
          foreach my $item ( keys %items ) {
            if ( defined $element->{PROPERTY}->{$item}->{content} ) {
              $data{$section}{$id}{ $items{$item} } = $element->{PROPERTY}->{$item}->{content};
            }
          }
        }
        else {
          if ( defined $element->{PROPERTY}->{"sample-time"}->{content} ) {
            $sample_time = $element->{PROPERTY}->{"sample-time"}->{content};
          }
          if ( defined $element->{PROPERTY}->{"sample-time-numeric"}->{content} ) {
            $sample_time_numeric = $element->{PROPERTY}->{"sample-time-numeric"}->{content};
          }
        }
        if ( defined $element->{OBJECT} && ref( $element->{OBJECT} ) eq "ARRAY" ) {
          foreach my $element2 ( @{ $element->{OBJECT} } ) {
            if ( !defined $element2->{PROPERTY}->{"serial-number"}->{content} ) { next; }
            my $id = $element2->{PROPERTY}->{"serial-number"}->{content};
            foreach my $item ( keys %items ) {
              if ( defined $element2->{PROPERTY}->{$item}->{content} ) {
                $data{$section}{$id}{ $items{$item} } = $element2->{PROPERTY}->{$item}->{content};
                if ( $sample_time ne "" && !defined $data{$section}{$id}{ $items{"sample-time"} } ) {
                  $data{$section}{$id}{ $items{"sample-time"} } = $sample_time;
                }
                if ( $sample_time_numeric ne "" && !defined $data{$section}{$id}{ $items{"sample-time-numeric"} } ) {
                  $data{$section}{$id}{ $items{"sample-time-numeric"} } = $sample_time_numeric;
                }
              }
            }
          }
        }
      }
    }
    foreach my $element (keys %{$data{$section}}){
      if (Xormon::isdigit($end_timestamp)){
        $data{$section}{$element}{time} = $end_timestamp;
        $data{$section}{$element}{TO} = $end_date;
      } 
    }
  }
  foreach my $section ( keys %data ) {
    foreach my $element ( keys %{ $data{$section} } ) {
      $data{$section}{$element}{TotalIOs}          = sum_metrics( $data{$section}{$element}, "WriteIOs",      "ReadIOs" );
      $data{$section}{$element}{KBytesTransferred} = sum_metrics( $data{$section}{$element}, "KBytesWritten", "KBytesRead" );
      if ( ( !defined $data{$section}{$element}{TotalIOs} || $data{$section}{$element}{TotalIOs} eq "" ) && ( !defined $data{$section}{$element}{KBytesTransferred} || $data{$section}{$element}{KBytesTransferred} eq "" ) ) {
        delete $data{$section}{$element};
      }
    }
  }
  return %data;
}

sub sum_metrics {
  my $data_tmp = shift;
  my $metric1  = shift;
  my $metric2  = shift;
  my %data     = %{$data_tmp};
  my $sum      = "";
  if ( Xormon::isdigit( $data{$metric1} ) ) {
    $sum = $data{$metric1};
  }
  if ( Xormon::isdigit( $data{$metric2} ) ) {
    if ( Xormon::isdigit($sum) ) {
      $sum = $sum + $data{$metric2};
    }
    else {
      $sum = $data{$metric2};
    }
  }
  return $sum;
}

sub set_global_data2 {
  my $new_data_tmp = shift;
  my $old_data_tmp = shift;
  my %data_new     = %{$new_data_tmp};
  my %data_old     = %{$old_data_tmp};
  my @items        = ( "ReadIOs", "WriteIOs", "TotalIOs", "KBytesTransferred", "ReadHitIOs", "WriteHitIOs", "KBytesRead", "KBytesWritten", "Response time (ms)", "Response read (ms)", "Response write (ms)" );
  foreach my $section ( keys %data_new ) {
    foreach my $element ( keys %{ $data_new{$section} } ) {
      if ( !defined $data_new{$section}{$element}{id} || !defined $data_new{$section}{$element}{time} || !defined $data_old{$section} || !defined $data_old{$section}{$element} || !defined $data_old{$section}{$element}{id} || !defined $data_old{$section}{$element}{time} || !defined $data_new{$section}{$element}{TO} || !defined $data_old{$section}{$element}{TO} ) { next; }
      my $sample_rate = $data_new{$section}{$element}{time} - $data_old{$section}{$element}{time};
      if ( !Xormon::isdigit($sample_rate) || $sample_rate eq "0" || $sample_rate =~ m/^-/ ) { next; }
      $inventory_performance{$section}{$element}{ID}   = $element;
      $inventory_performance{$section}{$element}{name} = $element;
      if ( $section eq "Drive Level Statistics" && defined $data_new{$section}{$element}{id} ) {
        $inventory_performance{$section}{$element}{name} = $data_new{$section}{$element}{id};
      }
      $inventory_performance{$section}{$element}{SECTION} = $section;
      $data_old{$section}{$element}{TO} =~ s/-/:/g;
      $data_old{$section}{$element}{TO} =~ s/ /T/g;
      $data_new{$section}{$element}{TO} =~ s/-/:/g;
      $data_new{$section}{$element}{TO} =~ s/ /T/g;
      $inventory_performance{$section}{TIME}{FROM}   = $data_old{$section}{$element}{TO};
      $inventory_performance{$section}{TIME}{LENGTH} = $sample_rate;
      $inventory_performance{$section}{TIME}{TO}     = $data_new{$section}{$element}{TO};

      foreach my $item (@items) {
        if ( Xormon::isdigit( $data_new{$section}{$element}{$item} ) && Xormon::isdigit( $data_old{$section}{$element}{$item} ) ) {
          my $value = round( ( $data_new{$section}{$element}{$item} - $data_old{$section}{$element}{$item} ) / $sample_rate, 0 );
          if ( $item =~ m/^KBytesTransferred$|^KBytesRead$|^KBytesWritten$/ ) {
            $value = round( ( $data_new{$section}{$element}{$item} - $data_old{$section}{$element}{$item} ) / ( $sample_rate * 1024 ), 0 );
          }
          if ( $item =~ m/Response/ ) {
            $value = round( $data_new{$section}{$element}{$item} / 1000, 3 );
          }
          if ( $value !~ m/^-/ ) {
            $inventory_performance{$section}{$element}{$item} = $value;
            }
        }
      }
    }
  }
  #print Dumper \%inventory_performance;
}
sub calculate_hit_io_pool {
  my $section     = "Pool Level Statistics";
  my $section_vol = "Volume Level Statistics";
  foreach my $volume ( keys %{ $inventory_performance{$section_vol} } ) {
    my $pool = $inventory_performance{$section_vol}{$volume}{pool_id};
    if ( !defined $pool || !defined $inventory_performance{$section}{$pool} ) { next; }
    my $item  = "ReadHitIOs";
    my $value = $inventory_performance{$section_vol}{$volume}{$item};
    if ( Xormon::isdigit($value) ) {
      if ( Xormon::isdigit( $inventory_performance{$section}{$pool}{$item} ) ) {
        $inventory_performance{$section}{$pool}{$item} = $inventory_performance{$section}{$pool}{$item} + $value;
      }
      else {
        $inventory_performance{$section}{$pool}{$item} = $value;
      }
    }
    $item = "WriteHitIOs";
    undef $value;
    $value = $inventory_performance{$section_vol}{$volume}{$item};
    if ( Xormon::isdigit($value) ) {
      if ( Xormon::isdigit( $inventory_performance{$section}{$pool}{$item} ) ) {
        $inventory_performance{$section}{$pool}{$item} = $inventory_performance{$section}{$pool}{$item} + $value;
      }
      else {
        $inventory_performance{$section}{$pool}{$item} = $value;
      }
    }
  }
}
sub set_data_pool_capacity {
  my $section               = "Pool Level Statistics";
  my $xml_inventory = cmd_api("pools","name",0);
  if ( defined $xml_inventory && ref($xml_inventory) eq 'HASH' ) {
    my $pool_inventory = $xml_inventory->{OBJECT};
    if ( ref($pool_inventory) eq 'ARRAY' ) {
      foreach my $pool ( @{$pool_inventory} ) {
        #print Dumper \$pool;
        my $pool_id = $pool->{PROPERTY}->{"serial-number"}->{content};
        my $cap_total = $pool->{PROPERTY}->{"total-size-numeric"}->{content};
        my $cap_free = $pool->{PROPERTY}->{"total-avail-numeric"}->{content};
        if (!Xormon::isdigit($cap_total) || !Xormon::isdigit($cap_free) || !defined $pool_id){next;}
        my $block_size = $pool->{PROPERTY}->{"blocksize"}->{content};
        if (!Xormon::isdigit($block_size)){
          $block_size = 512;
        }
        $cap_total = $cap_total * $block_size;
        $cap_free = $cap_free * $block_size;
        my $cap_used = $cap_total - $cap_free;
        if (defined $inventory_performance{$section}{$pool_id}){
          $inventory_performance{$section}{$pool_id}{"capacity_total"} = $cap_total;
          $inventory_performance{$section}{$pool_id}{"capacity_used"} = $cap_used;
          $inventory_performance{$section}{$pool_id}{"capacity_free"} = $cap_free;
        }
      }
    }
  }
}

sub get_section {
  my $cmd  = shift;
  my $type = shift;
  if ( $cmd =~ m/events/ && $type eq "conf" ) {
    return "Event Log";
  }
  if ( $cmd =~ m/disks/ && $type eq "conf" ) {
    return "Drive Level Configuration";
  }
  if ( $cmd =~ m/host-maps|maps/ && $type eq "conf" ) {
    return "Host Level Configuration";
  }
  if ( $cmd =~ m/ports detail/ && $type eq "conf" ) {
    return "Port Level Configuration";
  }
  if ( $cmd =~ m/controllers|versions detail/ && $type eq "conf" ) {
    return "Node Level Configuration";
  }
  if ( $cmd =~ m/volumes/ && $type eq "conf" ) {
    return "Volume Level Configuration";
  }
  if ( $cmd =~ m/pools/ && $type eq "conf" ) {
    return "Pool Level Configuration";
  }
  if ( $cmd =~ m/^volume/ ) {
    return "Volume Level Statistics";
  }
  if ( $cmd =~ m/^pool/ ) {
    return "";
    return "Pool Level Statistics";
  }
  if ( $cmd =~ m/^controller/ ) {
    return "Node Level Statistics";
  }
  if ( $cmd =~ m/^disk/ ) {
    return "Drive Level Statistics";
  }
  if ( $cmd =~ m/^host/ ) {
    return "Port Level Statistics";
  }
  return "";
}

sub get_items {
  my $cmd  = shift;
  my $type = shift;
  my %data;
  if ( $cmd =~ m/events/ && $type eq "conf" ) {
    push( @{ $data{"event-id"} }, "id" );
    push( @{ $data{"event-id"} }, "name" );
    $data{"time-stamp"}             = "MessageTimestamp#";
    $data{"additional-information"} = "additional information";
    $data{"event-code"}             = "eventCode";
    $data{"severity"}               = "severity";
    $data{"severity-numeric"}       = "errorCode";
    $data{"message"}                = "RecordData";
  }
  if ( $cmd =~ m/disks/ && $type eq "conf" ) {
    $data{"serial-number"} = "serial_number";
    $data{"durable-id"}    = "name";
    $data{health}          = "status";
    $data{blocks}          = "capacity_total";
    $data{"disk-group"}    = "raid_group";
    $data{"health-reason"} = "description";
    return %data;
  }
  if ( $cmd =~ m/ports detail/ && $type eq "conf" ) {
    $data{"durable-id"}   = "name";
    $data{"target-id"}    = "wwn";
    $data{"actual-speed"} = "port_speed";
    $data{"health"}       = "status";
    $data{"port-type"}    = "Protocol";
    $data{"health-reason"} = "description";
    return %data;
  }
  if ( $cmd =~ m/host-maps|maps/ && $type eq "conf" ) {
    $data{"hba-nickname"}  = "name";
    $data{"volume-serial"} = "Volume IDs";
    $data{ports}           = "ports";
    return %data;
  }
  if ( $cmd =~ m/controllers|versions detail/ && $type eq "conf" ) {
    $data{"serial-number"} = "serial_number";
    $data{"controller-id"} = "name";
    $data{health}          = "status";
    $data{"health-reason"} = "description";
    $data{"ip-address"} = "host";
    #push( @{ $data{"ip-address"} }, "OtherIdentifyingInfo" );
    #push( @{ $data{"ip-address"} }, "ip_address" );

    #push(@{$data{"sc-fw"}},"Bundle Version");
    #push( @{ $data{"sc-fw"} }, "version" );

    #push(@{$data{"sc-fw"}},"Base Bundle Version");
    #$data{"bundle-version"}        = "version";
    $data{"bundle-base-version"}   = "Base Bundle Version";
    $data{"build-date"}            = "Build Date";
    $data{"sc-cpu-type"}           = "Storage Controller CPU Type";
    $data{"sc-fw"}                 = "version";
    $data{"sc-baselevel"}          = "Storage Controller Code Baselevel";
    $data{"sc-fu-version"}         = "FU Version";
    $data{"capi-version"}          = "CAPI Version";
    $data{"sc-loader"}             = "Storage Controller Loader Code Version";
    $data{"mc-fw"}                 = "Management Controller Code Version";
    $data{"mc-loader"}             = "Management Controller Loader Code Version";
    $data{"mc-base-fw"}            = "Management Controller Base Version";
    $data{"model"}                 = "model";
    $data{"ec-fw"}                 = "Expander Controller Code Version";
    $data{"pld-rev"}               = "CPLD Code Version";
    $data{"hw-rev"}                = "Hardware Version";
    $data{"him-rev"}               = "Host Interface Module Version";
    $data{"him-model"}             = "Host Interface Module Model";
    $data{"backplane-type"}        = "Backplane Type";
    $data{"host-channel_revision"} = "Host Interface Hardware (Chip) Version";
    $data{"disk-channel_revision"} = "Disk Interface Hardware (Chip) Version";
    $data{"mrc-version"}           = "SC Boot Memory Reference Code Version";
    $data{"ctk-version"}           = "CTK Version";
    return %data;
  }
  if ( $cmd =~ m/volumes/ && $type eq "conf" ) {
    $data{"serial-number"}       = "lun_id";
    $data{"volume-name"}         = "name";
    $data{"container-serial"}    = "pool_id";
    $data{"virtual-disk-serial"} = "pool_id";
    $data{"storage-pool-name"}   = "pool_name";
    $data{"health-reason"} = "description";
    $data{health}                = "status";
    #$data{"total-size"}          = "capacity_total";
    #$data{"allocated-size"}      = "capacity_used";
    return %data;
  }
  if ( $cmd =~ m/pools/ && $type eq "conf" ) {
    $data{"serial-number"}       = "id";
    $data{"name"}                = "name";
    $data{"total-size-numeric"}  = "capacity_total";
    $data{"total-avail-numeric"} = "capacity_free";
    $data{health}                = "status";
    $data{"health-reason"} = "description";
    return %data;
  }
  if ( $cmd =~ m/^volume|^pool|^controller|^disk|^host/ ) {
    $data{"serial-number"}            = "id";
    $data{"durable-id"}               = "id";
    if ($cmd =~ m/^disk/){
      delete $data{"serial-number"};
    }
    $data{"number-of-reads"}          = "ReadIOs";
    $data{"number-of-writes"}         = "WriteIOs";
    $data{"data-read-numeric"}        = "KBytesRead";
    $data{"data-written-numeric"}     = "KBytesWritten";
    $data{"write-cache-hits"}         = "WriteHitIOs";
    $data{"read-cache-hits"}          = "ReadHitIOs";
    $data{"stop-sample-time-numeric"} = "time";
    $data{"avg-rsp-time"}             = "Response time (ms)";
    $data{"avg-read-rsp-time"}        = "Response read (ms)";
    $data{"avg-write-rsp-time"}       = "Response write (ms)";
    $data{"sample-time-numeric"}      = "time";
    $data{"stop-sample-time"}         = "TO";
    $data{"sample-time"}              = "TO";
  }
  return %data;
}

sub get_data_from_xml_file {
  my $file = shift;
  my $data;
  eval { $data = XMLin( "$file", KeyAttr => { 'PROPERTY' => 'name' } ); };
  if ($@) {
    error("XML parsing error: $@");
    eval {
      my $linted = `xmllint --recover $file`;
      $data = XMLin( $linted, KeyAttr => { 'PROPERTY' => 'name' } );
    };
  }
  return $data;
}

sub set_inventory_xml_conf_data {
  my %configuration_data;
  foreach my $cmd (@cmds_configuration) {
    my $section = get_section( $cmd, "conf" );
    if ( $section eq "" ) { next; }
    my %items = get_items( $cmd, "conf" );
    if ( !%items ) { next; }
    my $file = "$debug_data/$cmd.xml";
    if ( $cmd =~ m/ / ) {
      my $cmd_new = $cmd;
      $cmd_new =~ s/ /-/g;
      $file = "$debug_data/$cmd_new.xml";
      if ($cmd =~ m/events/){
        $file = "$debug_data/events.xml";
      }
    }
    #print "$file $section\n";
    if ( !-f $file ) { next; }
    my $data = get_data_from_xml_file($file);
    #if ($section eq "Event Log"){
    #  print Dumper \$data;
    #}
    if ( defined $data->{OBJECT} && ref( $data->{OBJECT} ) eq "ARRAY" ) {
      foreach my $element ( @{ $data->{OBJECT} } ) {
        my $id = "";
        if ( $section eq "Drive Level Configuration" ) {
          $id = $element->{PROPERTY}->{"durable-id"}->{content};
          if ( !defined $id || $id eq "" ) { next; }
          $id = get_id_from_name_drive($id);
        }
        if ( $section eq "Node Level Configuration" ) {
          $id = $element->{PROPERTY}->{"controller-id"}->{content};

          #if (!defined $id || $id eq ""){next;}
          if ( $cmd =~ m/versions/ ) {
            $id = $element->{name};
            if ( !defined $id || $id eq "" || $id eq "status" ) { next; }
            $id =~ s/controller-//g;
            $id =~ s/-versions//g;
            if ( !defined $id ) { next; }
            my $uc_id = uc($id);
            $id = $uc_id;
          }
        }
        if ( $section eq "Volume Level Configuration" ) {
          $id = $element->{PROPERTY}->{"serial-number"}->{content};
        }
        if ( $section eq "Event Log" ) {
          $id = $element->{PROPERTY}->{"event-id"}->{content};
        }
        if ( $section eq "Host Level Configuration" ) {
          $id = $element->{PROPERTY}->{"id"}->{content};
          if ( $cmd eq "maps" ) {
            $id = $element->{PROPERTY}->{"volume-serial"}->{content};

            #print STDERR Dumper \$element;
          }
        }
        if ( $section eq "Pool Level Configuration" ) {
          $id = $element->{PROPERTY}->{"serial-number"}->{content};
        }
        if ( $section eq "Port Level Configuration" ) {
          $id = $element->{PROPERTY}->{"durable-id"}->{content};
          if ( !defined $id || $id eq "" ) { next; }
          $id =~ s/hostport_//g;
        }
        if ( !defined $id || $id eq "" ) { next; }
        foreach my $item ( keys %items ) {
          if ( $cmd eq "maps" ) { next; }
          if ( defined $element->{PROPERTY}->{$item}->{content} ) {
            if ( ref( $items{$item} ) eq "ARRAY" ) {
              foreach my $i ( @{ $items{$item} } ) {
                $configuration_data{$section}{$id}{$i} = $element->{PROPERTY}->{$item}->{content};
              }
              next;
            }

            #print "$section | $id | $item\n";
            $configuration_data{$section}{$id}{ $items{$item} } = $element->{PROPERTY}->{$item}->{content};
            if ( !defined $configuration_data{$section}{$id}{id} ) {
              $configuration_data{$section}{$id}{id} = $id;
            }
          }
        }
        if ( defined $element->{OBJECT} && ref( $element->{OBJECT} ) eq "ARRAY" && $cmd =~ m/host-maps|maps/ ) {
          foreach my $element2 ( @{ $element->{OBJECT} } ) {
            if ( $cmd eq "maps" ) {
              my $host_id   = $element2->{PROPERTY}->{identifier}->{content};
              my $host_name = $element2->{PROPERTY}->{nickname}->{content};
              my $ports     = $element2->{PROPERTY}->{ports}->{content};
              if ( !defined $host_id || !defined $host_name ) { next; }
              $host_name =~ s/\*//g;
              $host_name =~ s/\.//g;
              if ( defined $configuration_data{$section}{$host_id} ) {
                if ( defined $configuration_data{$section}{$host_id}{"Volume IDs"} ) {
                  if ( $configuration_data{$section}{$host_id}{"Volume IDs"} !~ m/$id/ ) {
                    $configuration_data{$section}{$host_id}{"Volume IDs"} = $configuration_data{$section}{$host_id}{"Volume IDs"} . " " . $id;
                  }
                }
                else {
                  $configuration_data{$section}{$host_id}{"Volume IDs"} = $id;
                }
              }
              else {
                $configuration_data{$section}{$host_id}{name}         = $host_name;
                $configuration_data{$section}{$host_id}{"Volume IDs"} = $id;
              }
              if ( defined $ports ) {
                $configuration_data{$section}{$host_id}{ports} = $ports;
              }
              next;
            }
            foreach my $item ( keys %items ) {
              if ( defined $element2->{PROPERTY}->{$item}->{content} ) {
                if ( ref( $items{$item} ) eq "ARRAY" ) {
                  foreach my $i ( @{ $items{$item} } ) {
                    $configuration_data{$section}{$id}{$i} = $element2->{PROPERTY}->{$item}->{content};
                  }
                  next;
                }
                if ( defined $configuration_data{$section}{$id}{ $items{$item} } && $item ne "ports" && $item ne "hba-nickname" ) {
                  $configuration_data{$section}{$id}{ $items{$item} } = $configuration_data{$section}{$id}{ $items{$item} } . " " . $element2->{PROPERTY}->{$item}->{content};
                  next;
                }
                $configuration_data{$section}{$id}{ $items{$item} } = $element2->{PROPERTY}->{$item}->{content};
              }
            }
          }
        }
        if ( defined $element->{OBJECT} && ref( $element->{OBJECT} ) eq "HASH" && $cmd =~ m/host-maps|maps/ ) {
          if ( $cmd eq "maps" ) {
            my $host_id   = $element->{OBJECT}->{PROPERTY}->{identifier}->{content};
            my $host_name = $element->{OBJECT}->{PROPERTY}->{nickname}->{content};
            my $ports     = $element->{OBJECT}->{PROPERTY}->{ports}->{content};
            if ( !defined $host_id || !defined $host_name ) { next; }
            $host_name =~ s/\*//g;
            $host_name =~ s/\.//g;
            if ( defined $configuration_data{$section}{$host_id} ) {
              if ( defined $configuration_data{$section}{$host_id}{"Volume IDs"} ) {
                if ( $configuration_data{$section}{$host_id}{"Volume IDs"} !~ m/$id/ ) {
                  $configuration_data{$section}{$host_id}{"Volume IDs"} = $configuration_data{$section}{$host_id}{"Volume IDs"} . " " . $id;
                }
              }
              else {
                $configuration_data{$section}{$host_id}{"Volume IDs"} = $id;
              }
            }
            else {
              $configuration_data{$section}{$host_id}{name}         = $host_name;
              $configuration_data{$section}{$host_id}{"Volume IDs"} = $id;
            }
            if ( defined $ports ) {
              $configuration_data{$section}{$host_id}{ports} = $ports;
            }
            next;
          }
          foreach my $item ( keys %items ) {
            if ( defined $element->{OBJECT}->{PROPERTY}->{$item}->{content} ) {
              if ( ref( $items{$item} ) eq "ARRAY" ) {
                foreach my $i ( @{ $items{$item} } ) {
                  $configuration_data{$section}{$id}{$i} = $element->{OBJECT}->{PROPERTY}->{$item}->{content};
                }
                next;
              }
              if ( defined $configuration_data{$section}{$id}{ $items{$item} } && $item ne "ports" && $item ne "hba-nickname" ) {
                $configuration_data{$section}{$id}{ $items{$item} } = $configuration_data{$section}{$id}{ $items{$item} } . " " . $element->{PROPERTY}->{$item}->{content};
                next;
              }
              $configuration_data{$section}{$id}{ $items{$item} } = $element->{OBJECT}->{PROPERTY}->{$item}->{content};
            }
          }
        }
      }
    }
  }

  #print STDERR Dumper \%configuration_data;
  foreach my $section ( keys %configuration_data ) {
    foreach my $element ( keys %{ $configuration_data{$section} } ) {
      if ( $section =~ m/Drive/ ) {
        if ( Xormon::isdigit( $configuration_data{$section}{$element}{"capacity_total"} ) ) {
          $configuration_data{$section}{$element}{"capacity_total"} = $configuration_data{$section}{$element}{"capacity_total"} * 512;
        }
        if ( defined $configuration_data{$section}{$element}{raid_group} ) {
          $configuration_data{"Raid Group Level Configuration"}{ $configuration_data{$section}{$element}{raid_group} }{id} = $configuration_data{$section}{$element}{raid_group};
          $configuration_data{"Raid Group Level Configuration"}{ $configuration_data{$section}{$element}{raid_group} }{name} = $configuration_data{$section}{$element}{raid_group};
        }
      }
      if ( $section =~ m/Event/ ) {
        my $item = "additional information";
        if ( defined $configuration_data{$section}{$element}{$item} && defined $configuration_data{$section}{$element}{"RecordData"} ) {
          $configuration_data{$section}{$element}{"RecordData"} = $configuration_data{$section}{$element}{"RecordData"} . " ($configuration_data{$section}{$element}{$item})";
        }
      }
      if ( $section =~ m/Volume/ ) {
        if (Xormon::isdigit($configuration_data{$section}{$element}{capacity_total}) && Xormon::isdigit($configuration_data{$section}{$element}{capacity_used})){
          $configuration_data{$section}{$element}{capacity_free} = $configuration_data{$section}{$element}{capacity_total} - $configuration_data{$section}{$element}{capacity_used};
        }
      }
      if ( $section =~ m/Pool/ ) {
        my $item        = "capacity_total";
        my $cap_total   = $configuration_data{$section}{$element}{$item};
        if ( Xormon::isdigit($cap_total) ) {
          $cap_total                                     = $cap_total * 512;
          $configuration_data{$section}{$element}{$item} = $cap_total;
        }
        $item = "capacity_free";
        my $cap_free = $configuration_data{$section}{$element}{$item};
        if ( Xormon::isdigit($cap_free) ) {
          $cap_free                                      = $cap_free * 512;
          $configuration_data{$section}{$element}{$item} = $cap_free;
          if ( Xormon::isdigit( $configuration_data{$section}{$element}{"capacity_total"} ) ) {
            $configuration_data{$section}{$element}{"capacity_used"} = $configuration_data{$section}{$element}{"capacity_total"} - $configuration_data{$section}{$element}{$item};
          }
        }
      }
      if ( $section =~ m/Port Level Configuration/ ) {
        my $item = "port_speed";
        if ( !defined $configuration_data{$section}{$element}{$item} ) { $configuration_data{$section}{$element}{$item} = ""; }
        my $speed = $configuration_data{$section}{$element}{$item};
        if ( $speed =~ m/Gb/ ) {
          $speed =~ s/Gb//g;
          if ( Xormon::isdigit($speed) ) {
            $configuration_data{$section}{$element}{$item} = $speed * 1000000000;
          }
          elsif ( $speed =~ m/Mb/ ) {
            $speed =~ s/Mb//g;
            if ( Xormon::isdigit($speed) ) {
              $configuration_data{$section}{$element}{$item} = $speed * 1000000;
            }
          }
          else {
            $configuration_data{$section}{$element}{$item} = $speed;
          }
        }
        my $name = $element;
        $name =~ s/hostport_//g;
        $configuration_data{$section}{$element}{name} = $name;
      }
      if ( $section =~ m/Host Level Configuration/ ) {
        my $item           = "name";
        my $section_volume = "Volume Level Configuration";
        my $name           = $configuration_data{$section}{$element}{$item};
        if ( !defined $name ) { next; }
        $item = "ports";
        my $ports     = "";
        my $vols_name = "";
        if ( defined $configuration_data{$section}{$element}{$item} ) {
          $ports = $configuration_data{$section}{$element}{$item};
          $ports =~ s/,/ /g;
        }
        $item = "Volume IDs";
        if ( $configuration_data{$section}{$element}{$item} ) {
          my @vols = split( " ", $configuration_data{$section}{$element}{$item} );
          foreach my $vol (@vols) {
            if ( defined $configuration_data{$section_volume}{$vol} && $configuration_data{$section_volume}{$vol}{name} ) {
              if ( $vols_name eq "" ) {
                $vols_name = $configuration_data{$section_volume}{$vol}{name};
              }
              else {
                $vols_name = $vols_name . " " . $configuration_data{$section_volume}{$vol}{name};
              }
            }
          }
        }
        $configuration_data{"Port-Mapping Level Configuration"}{$element}{Host}    = $name;
        $configuration_data{"Port-Mapping Level Configuration"}{$element}{Ports}   = $ports;
        $configuration_data{"Port-Mapping Level Configuration"}{$element}{Volumes} = $vols_name;
      }
    }
  }
  #print Dumper \%configuration_data;
  return %configuration_data;
}





