package INFINIBOX;

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use JSON qw(encode_json decode_json);
use Time::Local;
use Math::BigInt;
use Test;
use XormonLogger;

my $data_directory = "$FindBin::Bin";

#use Date::Parse;

my $xormon;
my $cfg;
my $logger;

my $st_serial;
my $ms_status;
my $sample_rate = 300000;

my $service_data_in  = {};
my $service_data_out = {};

my @architecture;
my %inventory_data;
my @san_filters;
my @nas_filters;
my %collectors;
my %device;
my %conf;


sub collect {
  $xormon = shift;
  $cfg    = shift;
  $ms_status      = "successful";
  @san_filters    = ();
  @nas_filters    = ();

  undef %collectors;
  undef %inventory_data;
  undef %device;
  undef %conf;

  # print Dumper $cfg;

  # test main variables
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!")     && return "error"; }
  $logger = XormonLogger->new( 'infinibox', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )       { $logger->error("Cannot continue, 'apiHost' is not configured!")       && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )   { $logger->error("Cannot continue, 'apiUsername' is not configured!")   && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )   { $logger->error("Cannot continue, 'apiPassword' is not configured!")   && return "error"; }

  $st_serial = $cfg->{'hostcfg_id'};

  ## GET MICROSERVICE DATA
  $service_data_in  = $xormon->getData();
  $service_data_out = {};

  my $active = 0;
  if ( !defined $service_data_in->{san_filters} || !defined $service_data_in->{nas_filters} || !defined $service_data_in->{collectors} ) {
    $active = 1;
    $service_data_in->{collectors} = undef;
  }

  if ( !defined $service_data_in->{cookie} ) {
    $logger->info('No last cookie found, new login..');
    $cfg->{'cookie'} = login();
    $service_data_out->{cookie} = $cfg->{'cookie'};
  }
  else {
    # $logger->info('Last cookie found, using it..');
    $cfg->{'cookie'} = $service_data_in->{cookie};
    # print Dumper $cfg->{'cookie'};
    my $out = cmd("gui/overview");
    # print Dumper $out;
    if ( $out eq "" ) {
      $logger->info('Last cookie expired, new login..');
      logout($cfg->{'cookie'});
      my $cookie = login();
      # print "$cookie\n";
      if ( defined $cookie && $cookie ne "" ) {
        $cfg->{'cookie'} = $cookie;
      }
    }
    $service_data_out->{cookie} = $cfg->{'cookie'};
  }

  if ( !defined $cfg->{'cookie'} || $cfg->{'cookie'} eq "" ) {
    $xormon->isConnected(0, "cookie not defined");
    $logger->error('Cookie is not defined! Exit..') && return "error";
  }
  else {
    $xormon->isConnected(1);
  }

  get_configuration();

  # if ($infinibox_debug) { print "DEBUG: $$ start set filters " . localtime() . "\n"; }
  set_filters();
  # if ($infinibox_debug) { print "DEBUG: $$ end set filters " . localtime() . "\n"; }
  my $san_filters_size = scalar @san_filters;
  my $nas_filters_size = scalar @nas_filters;

  if ( $san_filters_size != 2 || $nas_filters_size !=2 ) {
    $logger->info("New filters");
    @san_filters = ();
    @nas_filters = ();
    undef $service_data_in->{san_filters};
    undef $service_data_in->{nas_filters};

    # if ($infinibox_debug) { print "DEBUG: $$ start set filters " . localtime() . "\n"; }
    set_filters();
    # if ($infinibox_debug) { print "DEBUG: $$ end set filters " . localtime() . "\n"; }
    $san_filters_size = scalar @san_filters;
    $nas_filters_size = scalar @nas_filters;
  }
  # $logger->info("Filter size $filters_size");

  if ( $san_filters_size == 2 && $nas_filters_size == 2) {
    # if ($infinibox_debug) { print "DEBUG: $$ start set collectors " . localtime() . "\n"; }
    set_collectors();
    # if ($infinibox_debug) { print "DEBUG: $$ end set collectors " . localtime() . "\n"; }
    if ( $active == 0 ) {
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_pool() " . localtime() . "\n"; }
      set_data_pool();
      # if ($infinibox_debug) { print "DEBUG: $$ end set_data_pool() " . localtime() . "\n"; }
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_volume() " . localtime() . "\n"; }
      set_data_volume();
      # if ($infinibox_debug) { print "DEBUG: $$ end set_data_volume() " . localtime() . "\n"; }
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_host() " . localtime() . "\n"; }
      set_data_host();
      # if ($infinibox_debug) { print "DEBUG: $$ end set_data_host() " . localtime() . "\n"; }
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_port() " . localtime() . "\n"; }
      set_data_port();
      # if ($infinibox_debug) { print "DEBUG: $$ end set_data_port() " . localtime() . "\n"; }
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_iscsiport() " . localtime() . "\n"; }
      set_data_iscsiport();
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_iscsiport() " . localtime() . "\n"; }
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_fs() " . localtime() . "\n"; }
      set_data_fs();
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_fs() " . localtime() . "\n"; }
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_naspool() " . localtime() . "\n"; }
      set_data_naspool();
      # if ($infinibox_debug) { print "DEBUG: $$ start set_data_naspool() " . localtime() . "\n"; }
      # if ($infinibox_debug) { print "DEBUG: $$ start set_total_metric() " . localtime() . "\n"; }
      set_total_metric();
      # if ($infinibox_debug) { print "DEBUG: $$ end set_total_metric() " . localtime() . "\n"; }
      # if ($infinibox_debug) { print "DEBUG: $$ start write_data() " . localtime() . "\n"; }
      write_data();
      # if ($infinibox_debug) { print "DEBUG: $$ end write_data() " . localtime() . "\n"; }
    }
    # if ( !defined $service_data_in->{data}{value}{filters} || !defined $service_data_in->{data}{value}{collectors} ) {
    if ( !defined $service_data_in->{collectors}{pool_id}{read} || !defined $service_data_in->{collectors}{nas_fs}{read}) {
      @san_filters = ();
      @nas_filters = ();
      $service_data_in->{san_filters} = undef;
      $service_data_in->{nas_filters} = undef;
      $service_data_out->{san_filters} = undef;
      $service_data_out->{nas_filters} = undef;
      # if ($infinibox_debug) { print "DEBUG: $$ start set filters " . localtime() . "\n"; }
      set_filters();
      # if ($infinibox_debug) { print "DEBUG: $$ end set filters " . localtime() . "\n"; }
      # if ($infinibox_debug) { print "DEBUG: $$ start set collectors " . localtime() . "\n"; }
      set_collectors();
      # if ( !keys %collectors ) {
      #   if ( defined $service_data_in->{data}{value}{san_filters} ) {
      #     $service_data_out->{san_filters} = undef;
      #   }
      #   if ( defined $service_data_in->{data}{value}{nas_filters} ) {
      #     $service_data_out->{nas_filters} = undef;
      #   }
      #   if ( defined $service_data_in->{data}{value}{collectors} ) {
      #     $service_data_out->{collectors} = undef;
      #   }
      # }
    }
  }
  else {
    $logger->error("Count filters is not 2");
    # $logger->error("\@san_filters");
  }

  health_status();

  # print Dumper \%inventory_data;

  ## SAVE TO DB
  # print Dumper $service_data_out;
  $xormon->saveData(\%{ $service_data_out });

  return $ms_status;
}

sub login {
  my $url = "https://$cfg->{'apiHost'}/api/rest/users/login";
  my %hash_filter;
  $hash_filter{"username"} = "$cfg->{'apiUsername'}";
  $hash_filter{"password"} = "$cfg->{'apiPassword'}";
  $hash_filter{"clientid"} = "optional client identifier";
  my $json_body = encode_json( \%hash_filter );

  ### agent
  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( POST => $url );
  $req->header( 'Content-Type' => 'application/json' );
  $req->header( 'Accept'       => 'application/json' );
  $req->header( 'Connection'   => 'keep-alive' );
  $req->content($json_body);
  my $res = $ua->request($req);

  # print Dumper \$res;
  if ( !$res->is_success ) {
    $logger->error("ERROR: Login to API failed\n");
    $logger->error( $res->content );
    return "";
  }
  else {
    #my $data  = decode_json( $res->content );
    my $cookie = $res->headers->{"set-cookie"};
    return $cookie;

    #print Dumper \$data;
  }
}

sub logout {
  my $cookie = shift;
  my $url = "https://$cfg->{'apiHost'}/api/rest/users/logout";
  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( POST => $url );
  $req->header( 'Content-Type' => 'application/json' );
  $req->header( 'Accept'       => 'application/json' );
  $req->header( 'Connection'   => 'keep-alive' );

  $req->header( 'cookie' => $cookie );
  my $res = $ua->request($req);
  if ( !$res->is_success ) {
    $logger->error("ERROR: Logout to API failed\n");
    $logger->error( $res->content );
    return "";
  }
}

sub create_filter {
  my $json = shift;

  #print Dumper \%json_body;
  my $out = cmd( "metrics/filters", "POST", $json );
  if ( $out eq "" ) {
    return;
  }
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }
  my $filter_id = $out->{result}->{id};
  if ( !defined $filter_id ) {
    $logger->error("ERROR: filter is not defined!");
    return;
  }
  return $filter_id;
}

sub modify_filter {
  my $filter_id = shift;
  my $json      = shift;
  my $out       = cmd( "metrics/filters/$filter_id", "PUT", $json );
  if ( $out eq "" ) {
    return;
  }
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }
  return 1;
}

sub set_filters {
  ## SAN FILTERS
  # ref($data->{nas_filters}) eq 'ARRAY' && @{$data->{nas_filters}} == 2)
  if ( !defined $service_data_in->{san_filters} || @{$service_data_in->{san_filters}} !=2) {
    # $logger->info("New filters");
    my $filter = create_filter('{ "protocol_type": "SAN" }');
    # commented because data not received
    # modify_filter( $filter, '{ "protocol": "FC" }' );
    modify_filter( $filter, '{ "operation_category": "read" }' );
    if ( defined $filter ) {
      push( @san_filters, $filter );
      # write_line( $filter_file, $filter );
    }
    my $filter2 = create_filter('{ "protocol_type": "SAN" }');
    # commented because data not received
    # modify_filter( $filter2, '{ "protocol": "FC" }' );
    modify_filter( $filter2, '{ "operation_category": "write" }' );
    if ( defined $filter2 ) {
      push( @san_filters, $filter2 );
      # write_line( $filter_file, $filter2 );
    }
  }
  else {
    # $logger->info("Loading filters from db");
    @san_filters = @{$service_data_in->{san_filters}};
  }
  $service_data_out->{san_filters} = \@san_filters;

  ## NAS FILTERS
  if ( !defined $service_data_in->{nas_filters} || @{$service_data_in->{nas_filters}} !=2) {
    # $logger->info("New filters");
    my $filter = create_filter('{ "protocol_type": "NAS" }');
    # commented because data not received
    # modify_filter( $filter, '{ "protocol": "FC" }' );
    modify_filter( $filter, '{ "operation_category": "read" }' );
    if ( defined $filter ) {
      push( @nas_filters, $filter );
      # write_line( $filter_file, $filter );
    }
    my $filter2 = create_filter('{ "protocol_type": "NAS" }');
    # commented because data not received
    # modify_filter( $filter2, '{ "protocol": "FC" }' );
    modify_filter( $filter2, '{ "operation_category": "write" }' );
    if ( defined $filter2 ) {
      push( @nas_filters, $filter2 );
      # write_line( $filter_file, $filter2 );
    }
  }
  else {
    # $logger->info("Loading filters from db");
    @nas_filters = @{$service_data_in->{nas_filters}};
  }
  $service_data_out->{nas_filters} = \@nas_filters;
}

sub create_collector {
  my $json = shift;

  #print Dumper \%json_body;
  my $out = cmd( "metrics/collectors", "POST", $json );
  if ( $out eq "" ) {
    return;
  }
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }
  my $filter_id = $out->{result}->{id};
  if ( !defined $filter_id ) {
    $logger->error("ERROR: collector is not defined!");
    return;
  }
  return $filter_id;
}

sub cmd {
  my $element      = shift;
  my $type         = shift;
  my $json         = shift;
  my $element_name = shift;
  if ( !defined $type || $type eq "" ) {
    $type = "GET";
  }

  my $url = "https://$cfg->{'apiHost'}/api/rest/$element";
  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( $type => $url );
  $req->header( 'Content-Type' => 'application/json' );
  $req->header( 'Accept'       => 'application/json' );
  $req->header( 'Connection'   => 'keep-alive' );

  #$req->header( 'Token' =>  "$token");
  $req->header( 'cookie' => $cfg->{'cookie'} );
  if ( defined $json && $json ne "" ) {
    $req->content($json);
  }
  # if ($infinibox_debug) { print "DEBUG: $$ start url $url " . localtime() . "\n"; }
  my $res = $ua->request($req);
  # if ($infinibox_debug) { print "DEBUG: $$ end url $url " . localtime() . "\n"; }
  if ( !$res->is_success ) {
    $logger->error("ERROR: Command $url : json type $json failed!");
    $logger->error( $res->content );
    return "";
  }
  else {
    # ### for debug
    # if ( $element !~ m/counter/ && $type eq "GET" ) {
    #   $element =~ s/page_size=.*//g;
    #   $element =~ s/\?//g;
    #   $element =~ s/\///g;
    #   my $file = "$debug_data/$element.json";
    #   if ( $element =~ m/collector_id/ && defined $element_name ) {
    #     $file = "$debug_data/$element_name.json";
    #   }
    #   my $decode_json = decode_json( $res->content );
    #   eval { Xorux_lib::save_as_json( $file, $decode_json ); };
    #   if ($@) {
    #     error("Save data to file $file failed");
    #   }

    #   #print "nejsme tu\n";
    # }
    # ###
    # if ( defined $element_name && $element_name eq "1" ) {
    #   my $active_minute = substr( $debug_data, -2 );
    #   if ( isdigit($active_minute) ) {
    #     $active_minute = $active_minute - 5;
    #     if ( $active_minute =~ m/^-/ ) {
    #       $active_minute = "00";
    #     }
    #     if ( $active_minute < 10 ) {
    #       $active_minute = "0$active_minute";
    #     }
    #     if ( -f "$data_directory/$active_minute/systemstats.json" ) {
    #       my $file_mtime = ( stat("$data_directory/$active_minute/systemstats.json") )[9];
    #       my $time_now   = time();
    #       my $diff       = $time_now - $file_mtime;
    #       if ( $diff > 1200 ) { return ""; }

    #       my $js;
    #       eval { $js = Xorux_lib::read_json("$data_directory/$active_minute/systemstats.json"); };
    #       if ($@) {
    #         error("Retrieve data from file $data_directory/$active_minute/systemstats.json failed");
    #         return "";
    #       }
    #       if ( !defined $js ) {
    #         error("Retrieve data from file $data_directory/$active_minute/systemstats.json failed");
    #         return "";
    #       }
    #       $start_time_node = ( stat("$data_directory/$active_minute/systemstats.json") )[9];
    #       return $js;
    #     }
    #     else { return ""; }
    #   }
    #   else { return ""; }
    # }
    my $data = decode_json( $res->content );
    return $data;
  }
}


sub set_collectors {
  if ( !defined $service_data_in->{collectors} || !defined $service_data_in->{collectors}{pool_id}{read}|| !defined $service_data_in->{collectors}{nas_pool}{read}) {
    # $logger->info("New collectors");
    my $collector  = create_collector("{\"filter_id\":$san_filters[0],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\",\"sections_read_from_disk\",\"sections_read\"],\"grouping_field\":\"pool_id\",\"interval_milliseconds\":$sample_rate}");
    my $collector2 = create_collector("{\"filter_id\":$san_filters[1],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"pool_id\",\"interval_milliseconds\":$sample_rate}");
    if ( defined $collector ) {
      $collectors{pool_id}{read} = $collector;
    }
    if ( defined $collector2 ) {
      $collectors{pool_id}{write} = $collector2;
    }
    $collector  = create_collector("{\"filter_id\":$san_filters[0],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\",\"sections_read_from_disk\",\"sections_read\"],\"grouping_field\":\"vol_id\",\"interval_milliseconds\":$sample_rate}");
    $collector2 = create_collector("{\"filter_id\":$san_filters[1],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"vol_id\",\"interval_milliseconds\":$sample_rate}");
    if ( defined $collector ) {
      $collectors{vol_id}{read} = $collector;
    }
    if ( defined $collector2 ) {
      $collectors{vol_id}{write} = $collector2;
    }
    $collector  = create_collector("{\"filter_id\":$san_filters[0],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"host_id\",\"interval_milliseconds\":$sample_rate}");
    $collector2 = create_collector("{\"filter_id\":$san_filters[1],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"host_id\",\"interval_milliseconds\":$sample_rate}");
    if ( defined $collector ) {
      $collectors{host_id}{read} = $collector;
    }
    if ( defined $collector2 ) {
      $collectors{host_id}{write} = $collector2;
    }
    ## SCSI PORT
    $collector  = create_collector("{\"filter_id\":$san_filters[0],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"initiator_IQN\",\"interval_milliseconds\":$sample_rate}");
    $collector2 = create_collector("{\"filter_id\":$san_filters[1],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"initiator_IQN\",\"interval_milliseconds\":$sample_rate}");
    if ( defined $collector ) {
      $collectors{scsi}{read} = $collector;
    }
    if ( defined $collector2 ) {
      $collectors{scsi}{write} = $collector2;
    }
    ## FC PORT
    $collector  = create_collector("{\"filter_id\":$san_filters[0],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"target_WWPN\",\"interval_milliseconds\":$sample_rate}");
    $collector2 = create_collector("{\"filter_id\":$san_filters[1],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"target_WWPN\",\"interval_milliseconds\":$sample_rate}");
    if ( defined $collector ) {
      $collectors{target_WWPN}{read} = $collector;
    }
    if ( defined $collector2 ) {
      $collectors{target_WWPN}{write} = $collector2;
    }
    #### NAS
    ## NAS POOL
    $collector  = create_collector("{\"filter_id\":$nas_filters[0],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"latency\",\"sections_read_from_disk\",\"sections_read\"],\"grouping_field\":\"pool_id\",\"interval_milliseconds\":$sample_rate}");
    $collector2 = create_collector("{\"filter_id\":$nas_filters[1],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"latency\"],\"grouping_field\":\"pool_id\",\"interval_milliseconds\":$sample_rate}");
    if ( defined $collector ) {
      $collectors{nas_pool}{read} = $collector;
    }
    if ( defined $collector2 ) {
      $collectors{nas_pool}{write} = $collector2;
    }
    ## NAS FILESYSTEM
    $collector  = create_collector("{\"filter_id\":$nas_filters[0],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"latency\",\"sections_read_from_disk\",\"sections_read\"],\"grouping_field\":\"fs_entity_id\",\"interval_milliseconds\":$sample_rate}");
    $collector2 = create_collector("{\"filter_id\":$nas_filters[1],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"latency\"],\"grouping_field\":\"fs_entity_id\",\"interval_milliseconds\":$sample_rate}");
    if ( defined $collector ) {
      $collectors{nas_fs}{read} = $collector;
    }
    if ( defined $collector2 ) {
      $collectors{nas_fs}{write} = $collector2;
    }
    ##
    $service_data_out->{collectors} = \%collectors;
  }
  else {
    # $logger->info("Collectors from DB");
    %collectors = %{ $service_data_in->{collectors} }
  }
  # print Dumper \%collectors;
  $service_data_out->{collectors} = \%collectors;
}

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  }
  else {
    return $num;
  }
}

sub get_time {
  my $timestamp_end = shift;
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $time_end = "$year:$month:$day" . "T" . "$hour:$minute:$sec";
  return $time_end;
}


sub set_data_pool {

  my $section = "pool";

  my $out  = "";
  my $out2 = "";

  if ( defined $collectors{pool_id}{read} ) {
    my $collector = $collectors{pool_id}{read};
    $out = cmd( "metrics/collectors/data?collector_id=$collector", "", "", "pool_read_data" );
  }
  if ( defined $collectors{pool_id}{write} ) {
    my $collector2 = $collectors{pool_id}{write};
    $out2 = cmd( "metrics/collectors/data?collector_id=$collector2", "", "", "pool_write_data" );
  }
  if ( $out ne "" ) {
    my $error = $out->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }

    # print Dumper \$out;
  }
  if ( $out2 ne "" ) {
    my $error = $out2->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }

    # print Dumper \$out2;
  }
  # if ( ( $out eq "" || $out2 eq "" ) && ( defined $service_data_in->{data}{value}{san_filters} && defined $service_data_in->{data}{value}{collectors} ) ) {
  if ( $out eq "" || $out2 eq "" && ( defined $service_data_in->{san_filters} && defined $service_data_in->{collectors} )) {
    $service_data_out->{san_filters} = undef;
    $service_data_out->{collectors} = undef;
    return;
  }
  set_data( $out,  $section, "read" );
  set_data( $out2, $section, "write" );

  $out = cmd("pools?page_size=1000&page=1&page=2&page=3&page=4&page=5");
  if ( $out eq "" ) {
    return;
  }
  # print Dumper $out;
  my $data  = $out->{result};
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }

  foreach my $pool ( @{$data} ) {
    my $id = $pool->{id};
    my $name = $pool->{name};
    if ( !defined $id || $id eq "" ) { next; }
    if ( !defined $name || $name eq "" ) {
      $name = $id;
    }
    if ( !defined $inventory_data{$section}{$id}{id} ) {
      $inventory_data{$section}{$id}{id} = $id;
    }
    if ( !defined $inventory_data{$section}{$id}{name} ) {
      $inventory_data{$section}{$id}{name} = $name;
    }
    my $free_physical_space = $pool->{"free_physical_space"};
    my $size_physical_space = $pool->{"physical_capacity"};
    my $free_virtual_space  = $pool->{"free_virtual_space"};
    my $size_virtual_space  = $pool->{"virtual_capacity"};

    my $allocated_physical_space = $pool->{allocated_physical_space};
    my $capacity_savings         = $pool->{capacity_savings};
    my $reserved_capacity        = $pool->{reserved_capacity};
    my $allocated_virtual_space  = $pool->{allocated_virtual_space};

    if ( Xormon::isdigit($allocated_physical_space) ) {
      $inventory_data{$section}{$id}{"allocated_physical_space"} = $allocated_physical_space;
    }
    if ( Xormon::isdigit($allocated_virtual_space) ) {
      $inventory_data{$section}{$id}{"allocated_virtual_space"} = $allocated_virtual_space;
    }
    if ( !Xormon::isdigit($allocated_virtual_space) && Xormon::isdigit($size_virtual_space) && Xormon::isdigit($free_virtual_space) ) {
      $inventory_data{$section}{$id}{"allocated_virtual_space"} = $size_virtual_space - $free_virtual_space;
    }
    if ( Xormon::isdigit($capacity_savings) ) {
      $inventory_data{$section}{$id}{"capacity_savings"} = $capacity_savings;
    }
    if ( Xormon::isdigit($reserved_capacity) ) {
      $inventory_data{$section}{$id}{"reserved_capacity"} = $reserved_capacity;
    }

    # CAPACITY
    if ( Xormon::isdigit($size_physical_space) ) {
      $inventory_data{$section}{$id}{"Capacity"} = $size_physical_space;
    }
    if ( Xormon::isdigit($free_physical_space) && Xormon::isdigit($size_physical_space) ) {
      $inventory_data{$section}{$id}{"Used"} = $size_physical_space - $free_physical_space;
    }
    if ( Xormon::isdigit($free_physical_space) ) {
      $inventory_data{$section}{$id}{"Free"} = $free_physical_space;
    }

    # VIRTUAL CAPACITY
    if ( Xormon::isdigit($size_virtual_space) ) {
      $inventory_data{$section}{$id}{"Capacity_virtual"} = $size_virtual_space;
    }
    if ( Xormon::isdigit($free_virtual_space) && Xormon::isdigit($size_virtual_space) ) {
      $inventory_data{$section}{$id}{"Used_virtual"} = $size_virtual_space - $free_virtual_space;
    }
    if ( Xormon::isdigit($free_virtual_space) ) {
      $inventory_data{$section}{$id}{"Free_virtual"} = $free_virtual_space;
    }
    set_zero_values( $section, $id );
  }
}

sub set_data_naspool {

  my $section = "naspool";

  my $out  = "";
  my $out2 = "";

  if ( defined $collectors{nas_pool}{read} ) {
    my $collector = $collectors{nas_pool}{read};
    $out = cmd( "metrics/collectors/data?collector_id=$collector", "", "", "naspool_read_data" );
  }
  if ( defined $collectors{nas_pool}{write} ) {
    my $collector2 = $collectors{nas_pool}{write};
    $out2 = cmd( "metrics/collectors/data?collector_id=$collector2", "", "", "naspool_write_data" );
  }

  if ( $out ne "" ) {
    my $error = $out->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out;
  }
  if ( $out2 ne "" ) {
    my $error = $out2->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out2;
  }
  if ( $out eq "" || $out2 eq "" ) {
    return;
  }
  set_data( $out,  $section, "read" );
  set_data( $out2, $section, "write" );

  foreach my $fs (keys %{$inventory_data{fs}}) {
    my $pool_id = $inventory_data{fs}{$fs}{pool_id};
    if (defined $pool_id && $pool_id ne "") {
      if(defined $inventory_data{pool}{$pool_id}{name} && $inventory_data{pool}{$pool_id}{name} ne "") {
        $inventory_data{$section}{$pool_id}{name} = $inventory_data{pool}{$pool_id}{name};
      }
      set_zero_values( $section, $pool_id );
    }
  }
}

sub set_data_volume {
  my $section = "volume";

  my $out  = "";
  my $out2 = "";

  if ( defined $collectors{vol_id}{read} ) {
    my $collector = $collectors{vol_id}{read};
    $out = cmd( "metrics/collectors/data?collector_id=$collector", "", "", "volume_read_data" );
  }
  if ( defined $collectors{vol_id}{write} ) {
    my $collector2 = $collectors{vol_id}{write};
    $out2 = cmd( "metrics/collectors/data?collector_id=$collector2", "", "", "volume_write_data" );
  }
  if ( $out ne "" ) {
    my $error = $out->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }

    # print Dumper \$out;
  }
  if ( $out2 ne "" ) {
    my $error = $out2->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }

    # print Dumper \$out2;
  }

  if ( $out eq "" || $out2 eq "" ) {
    return;
  }

  set_data( $out,  $section, "read" );
  set_data( $out2, $section, "write" );

  $out = cmd("volumes?page_size=1000&page=1");
  if ( $out eq "" ) {
    return;
  }
  # print Dumper $out;
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }
  my $total_page = $out->{metadata}->{pages_total};
  if ( Xormon::isdigit($total_page) && $total_page > 1 ) {
    for ( my $i = 2; $i <= $total_page; $i++ ) {
      my $sub_cmd = cmd("volumes?page_size=1000&page=$i");
      if ( $sub_cmd eq "" ) {
        next;
      }
      my $error = $sub_cmd->{error};

      if ( defined $error && $error ne "" ) {
        $logger->error("ERROR: $error");
        next;
      }
      if ( ref( $sub_cmd->{result} ) eq "ARRAY" ) {
        push( @{ $out->{result} }, @{ $sub_cmd->{result} } );
      }
    }
  }

  my $data = $out->{result};

  # print Dumper $data;

  foreach my $volume ( @{$data} ) {
    my $id      = $volume->{id};
    my $name    = $volume->{name};
    my $size    = $volume->{size};
    my $free    = "";
    my $used    = $volume->{used};
    my $pool_id = $volume->{pool_id};
    my $serial  = $volume->{serial};
    if ( defined $id && $id ne "" ) {
      if ( !defined $inventory_data{$section}{$id}{id} ) {
        $inventory_data{$section}{$id}{id} = $id;
      }
      if ( !defined $name || $name eq "" ) {
        $name = $id;
      }
      if ( !defined $inventory_data{$section}{$id}{name} ) {
        $inventory_data{$section}{$id}{name} = $name;
      }
      if ( Xormon::isdigit($size) ) {
        $inventory_data{$section}{$id}{"Capacity"} = $size;
      }
      if ( Xormon::isdigit($used) ) {
        $inventory_data{$section}{$id}{"Used"} = $used;
      }

      if ( Xormon::isdigit($size) && Xormon::isdigit($used) ) {
        $inventory_data{$section}{$id}{"Free"} = $size - $used;
      }
      if ( defined $pool_id && $pool_id ne "" ) {
        $inventory_data{$section}{$id}{"pool_id"} = $pool_id;
      }
      if ( defined $serial && $serial ne "" ) {
        $inventory_data{$section}{$id}{"serial"} = $serial;
      }
    }
    set_zero_values( $section, $id );
  }
}

sub set_data_host {
  my $section = "host";

  my $out  = "";
  my $out2 = "";

  if ( defined $collectors{host_id}{read} ) {
    my $collector = $collectors{host_id}{read};
    $out = cmd( "metrics/collectors/data?collector_id=$collector", "", "", "host_read_data" );
  }
  if ( defined $collectors{host_id}{write} ) {
    my $collector2 = $collectors{host_id}{write};
    $out2 = cmd( "metrics/collectors/data?collector_id=$collector2", "", "", "host_write_data" );
  }
  if ( $out ne "" ) {
    my $error = $out->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out;
  }
  if ( $out2 ne "" ) {
    my $error = $out2->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out2;
  }

  if ( $out eq "" || $out2 eq "" ) {
    return;
  }

  set_data( $out,  $section, "read" );
  set_data( $out2, $section, "write" );

  $out = cmd("hosts?page_size=1000&page=1&page=2&page=3&page=4&page=5&page=6&page=7&page=8&page=9&page=10");
  if ( $out eq "" ) {
    return;
  }
  my $data  = $out->{result};
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }
  # print Dumper $data;

  foreach my $host ( @{$data} ) {
    my @volumes_inventory = ();
    my $id   = $host->{id};
    my $name = $host->{name};
    if ( defined $id && $id ne "" && defined $name && $name ne "" ) {
      if ( !defined $inventory_data{$section}{$id}{id} ) {
        $inventory_data{$section}{$id}{id} = $id;
      }
      if ( !defined $inventory_data{$section}{$id}{name} ) {
        $inventory_data{$section}{$id}{name} = $name;
      }

      my $volumes = $host->{luns};
      my $ports   = $host->{ports};

      foreach my $volume ( @{$volumes} ) {
        my $volume_id = $volume->{"volume_id"};
        if ( defined $volume_id && $volume_id ne "" ) {
          push (@volumes_inventory, $volume_id);
        }
      }

      foreach my $port ( @{$ports} ) {
        my $port_address = $port->{address};
        my $port_type = $port->{type};
        if ( defined $port_address && $port_address ne "" && defined $port_type) {
          $inventory_data{$section}{$id}{PORT}{$port_type}{$port_address} = $port_address;
        }
      }
      $inventory_data{$section}{$id}{"volume_ids"} = \@volumes_inventory;

      set_zero_values( $section, $id );
    }
    else {
      next;
    }
  }
}


sub set_data_iscsiport {

  my $section = "iscsiport";

  my $out  = "";
  my $out2 = "";

  if ( defined $collectors{scsi}{read} ) {
    my $collector = $collectors{scsi}{read};
    $out = cmd( "metrics/collectors/data?collector_id=$collector", "", "", "iscsiport_read_data" );
  }
  if ( defined $collectors{scsi}{write} ) {
    my $collector2 = $collectors{scsi}{write};
    $out2 = cmd( "metrics/collectors/data?collector_id=$collector2", "", "", "iscsiport_write_data" );
  }
  if ( $out ne "" ) {
    my $error = $out->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out;
  }
  if ( $out2 ne "" ) {
    my $error = $out2->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out2;
  }

  if ( $out eq "" || $out2 eq "" ) {
    return;
  }

  set_data( $out,  $section, "read" );
  set_data( $out2, $section, "write" );

  $out = cmd("initiators?page_size=1000&page=1&page=2&page=3&page=4&page=5");
  if ( $out eq "" ) {
    return;
  }
  # print Dumper $out;
  my $data  = $out->{result};
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }

  foreach my $initiator ( @{$data} ) {
    my $id        = $initiator->{address};
    my $name      = $initiator->{address};
    my $size      = $initiator->{size};
    my $allocated = $initiator->{allocated};
    my $used      = $initiator->{used};
    my $pool_id   = $initiator->{pool_id};
    my $serial    = $initiator->{serial};
    if ( defined $id && $id ne "" ) {
      if ( !defined $inventory_data{$section}{$id}{id} ) {
        $inventory_data{$section}{$id}{id} = $id;
      }
      if ( !defined $name || $name eq "" ) {
        $name = $id;
      }
      if ( !defined $inventory_data{$section}{$id}{name} ) {
        $inventory_data{$section}{$id}{name} = $name;
      }
    }
    set_zero_values( $section, $id );
  }
}

sub set_data_port {

  my $section = "port";

  my $out  = "";
  my $out2 = "";

  if ( defined $collectors{target_WWPN}{read} ) {
    my $collector = $collectors{target_WWPN}{read};
    $out = cmd( "metrics/collectors/data?collector_id=$collector", "", "", "port_read_data" );
  }
  if ( defined $collectors{target_WWPN}{write} ) {
    my $collector2 = $collectors{target_WWPN}{write};
    $out2 = cmd( "metrics/collectors/data?collector_id=$collector2", "", "", "port_write_data" );
  }
  if ( $out ne "" ) {
    my $error = $out->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out;
  }
  if ( $out2 ne "" ) {
    my $error = $out2->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out2;
  }

  if ( $out eq "" || $out2 eq "" ) {
    return;
  }

  set_data( $out,  $section, "read" );
  set_data( $out2, $section, "write" );

  $out = cmd("components?page_size=1000&page=1&page=2&page=3&page=4&page=5");
  if ( $out eq "" ) {
    return;
  }
  my $data  = $out->{result}->{"patch_panels"};
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }

  # print Dumper \$data;

  foreach my $volume ( @{$data} ) {
    my $hosts = $volume->{frames};
    foreach my $host ( @{$hosts} ) {
      my $ports = $host->{ports};

      #print Dumper \$ports;
      foreach my $port ( @{$ports} ) {
        my $id       = $port->{"conn_port_uuid"};
        my $address  = $port->{hw_addr};
        my $protocol = $port->{type};
        my $port_id  = $port->{"conn_port_comp_id"};
        if ( defined $id && $id ne "" && defined $address && $address ne "" ) {
          my $mac     = $address;
          my $decimal = hex2dec($mac);
          $mac = $decimal;
          my $label = $id;
          $inventory_data{$section}{$mac}{id} = $id;
          if ( defined $inventory_data{$section}{$mac} ) {
            $inventory_data{$section}{$mac}{id}      = $id;
            $inventory_data{$section}{$mac}{address} = $address;
            if ( defined $protocol && $protocol ne "" ) {
              $inventory_data{$section}{$mac}{protocol} = $protocol;
              if ( defined $port_id && $port_id ne "" ) {
                $label = "$port_id" . " ($protocol)";
                $inventory_data{$section}{$mac}{name} = $label;
              }
            }
          }
          set_zero_values( $section, $mac );
        }
      }
    }
  }
}

sub set_data_fs {

  my $section = "fs";

  my $out  = "";
  my $out2 = "";

  if ( defined $collectors{nas_fs}{read} ) {
    my $collector = $collectors{nas_fs}{read};
    $out = cmd( "metrics/collectors/data?collector_id=$collector", "", "", "fs_read_data" );
  }
  if ( defined $collectors{nas_fs}{write} ) {
    my $collector2 = $collectors{nas_fs}{write};
    $out2 = cmd( "metrics/collectors/data?collector_id=$collector2", "", "", "fs_write_data" );
  }

  if ( $out ne "" ) {
    my $error = $out->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out;
  }
  if ( $out2 ne "" ) {
    my $error = $out2->{error};
    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
    }
    # print Dumper \$out2;
  }
  if ( $out eq "" || $out2 eq "" ) {
    return;
  }

  set_data( $out,  $section, "read" );
  set_data( $out2, $section, "write" );

  $out = cmd("filesystems?page_size=1000&page=1");
  if ( $out eq "" ) {
    return;
  }
  # print Dumper $out;
  my $data  = $out->{result};
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }

  foreach my $fs ( @{$data} ) {
    my $id        = $fs->{id};
    my $name      = $fs->{name};
    my $size      = $fs->{size};
    my $allocated = $fs->{allocated};
    my $used      = $fs->{used};
    my $pool_id   = $fs->{pool_id};
    my $serial    = $fs->{serial};
    if ( defined $id && $id ne "" ) {
      if ( !defined $inventory_data{$section}{$id}{id} ) {
        $inventory_data{$section}{$id}{id} = $id;
      }
      if ( !defined $name || $name eq "" ) {
        $name = $id;
      }
      if ( !defined $inventory_data{$section}{$id}{name} ) {
        $inventory_data{$section}{$id}{name} = $name;
      }
      if ( Xormon::isdigit($allocated) ) {
        $inventory_data{$section}{$id}{"Allocated"} = $allocated;
      }
      if ( Xormon::isdigit($size) ) {
        $inventory_data{$section}{$id}{"Capacity"} = $size;
      }
      if ( Xormon::isdigit($used) ) {
        $inventory_data{$section}{$id}{"Used"} = $used;
      }
      if ( Xormon::isdigit($size) && Xormon::isdigit($used) ) {
        $inventory_data{$section}{$id}{"Free"} = $size - $used;
      }
      if ( defined $pool_id && $pool_id ne "" ) {
        $inventory_data{$section}{$id}{"pool_id"} = $pool_id;
        if ( !defined $inventory_data{naspool}{$pool_id}{id} ) {
          $inventory_data{naspool}{$pool_id}{id} = $pool_id;
        }
      }
      if ( defined $serial && $serial ne "" ) {
        $inventory_data{$section}{$id}{"serial"} = $serial;
      }
    }
    set_zero_values( $section, $id );
  }
}

sub hex2dec {
  my $num = shift;

  $num =~ s/\s+//g;
  $num =~ s/://g;
  my $hex = "0x$num";

  my $dec = Math::BigInt->new($hex);

  return ($dec);
}

sub set_zero_values {
  my $section = shift;
  my $id      = shift;
  my $active  = 0;

  my @metrics = ( "read", "write", "data_rate", "io_rate", "write_io", "read_io", "resp_t_r", "resp_t_w", "resp_t" );
  foreach my $metric (@metrics) {
    if ( defined $inventory_data{$section}{$id}{id} && !Xormon::isdigit( $inventory_data{$section}{$id}{$metric} ) ) {
      $active++;
    }
  }
  if ( $active == 9 ) {
    foreach my $metric (@metrics) {
      $inventory_data{$section}{$id}{$metric} = 0;
    }
  }
}

sub set_data {
  my ( $data, $section, $type ) = @_;
  my %fields;
  if ( $type eq "read" ) {
    $fields{ops}              = "read_io";
    $fields{throughput}       = "read";
    $fields{external_latency} = "resp_t_r";
    $fields{latency}          = "resp_t_r";

    $fields{sections_read_from_disk} = "sections_read_from_disk";
    $fields{sections_read}           = "sections_read";

  }
  else {
    $fields{ops}              = "write_io";
    $fields{throughput}       = "write";
    $fields{external_latency} = "resp_t_w";
    $fields{latency}          = "resp_t_w";
  }

  #print Dumper \$data;
  # print "$section\n";
  if ( defined $data->{result}->{collectors}->[0]->{fields} && defined $data->{result}->{collectors}->[0]->{data} && defined $data->{result}->{collectors}->[0]->{data}->[0] && Xormon::isdigit( $data->{result}->{collectors}->[0]->{end_timestamp_milliseconds} ) ) {
    my @field  = @{ $data->{result}->{collectors}->[0]->{fields} };
    my @datass = @{ $data->{result}->{collectors}->[0]->{data}->[0] };
    my $time   = $data->{result}->{collectors}->[0]->{end_timestamp_milliseconds};
    $time = round( $time / 1000, 0 );
    my $start_time = $time - 300;
    $inventory_data{$section}{TIME}{START} = get_time($start_time);
    $inventory_data{$section}{TIME}{END}   = get_time($time);
    $inventory_data{$section}{TIME}{STEP}  = 300;
    $inventory_data{$section}{TIME}{END_TIMESTAMP} = $time;

    foreach my $elements (@datass) {
      my @datas = @{$elements};
      my $index = -1;
      foreach my $property (@field) {
        $index++;
        if ( $index == 0 && defined $datas[$index] ) {
          $property =~ s/pool_//g;
          $property =~ s/vol_//g;
          $property =~ s/host_//g;
          $property =~ s/target_WWPN/id/g;
          $property =~ s/initiator_IQN/id/g;

          #if ($section eq "Port Level Statistics"){
          #  my $hex = $datas[$index];
          #  print "decimal $hex\n";
          #  my $convert = sprintf("0x%X", $hex);
          #  print "convert ---- $convert\n";
          #  $convert =~ s/x//;
          #  $datas[$index] = $convert;
          #  print $datas[$index] . " \n";
          #}
          $inventory_data{$section}{ $datas[$index] }{$property} = $datas[$index];
          next;
        }
        if ( Xormon::isdigit( $datas[$index] ) && defined $fields{$property} ) {
          if ( $property eq "throughput" ) {
            $inventory_data{$section}{ $datas[0] }{ $fields{$property} } = $datas[$index];
          }
          elsif ( $property eq "external_latency" || $property eq "latency") {
            $inventory_data{$section}{ $datas[0] }{ $fields{$property} } = round( $datas[$index] / 1000, 3 )
          }
          else {
            $inventory_data{$section}{ $datas[0] }{ $fields{$property} } = $datas[$index];
          }
        }
      }
    }
  }
}

sub set_total_metric {

  foreach my $section ( keys %inventory_data ) {
    foreach my $element ( keys %{ $inventory_data{$section} } ) {
      if ( $element eq "TIME" ) { next; }
      my $io_rate   = "";
      my $data_rate = "";
      my $resp_t    = "";
      my $read      = $inventory_data{$section}{$element}{read};
      my $write     = $inventory_data{$section}{$element}{write};
      my $read_io   = $inventory_data{$section}{$element}{read_io};
      my $write_io  = $inventory_data{$section}{$element}{write_io};
      my $resp_t_r  = $inventory_data{$section}{$element}{resp_t_r};
      my $resp_t_w  = $inventory_data{$section}{$element}{resp_t_w};
      my $num       = "";

      if ( Xormon::isdigit($read) ) {
        $data_rate = $read;
      }
      if ( Xormon::isdigit($write) ) {
        if ( Xormon::isdigit($data_rate) ) {
          $data_rate = $data_rate + $write;
        }
        else {
          $data_rate = $write;
        }
      }
      if ( Xormon::isdigit($read_io) ) {
        $io_rate = $read_io;
        if ( Xormon::isdigit($resp_t_r) ) {
          $num = $resp_t_r * $read_io;
        }
      }
      if ( Xormon::isdigit($write_io) ) {
        if ( Xormon::isdigit($io_rate) ) {
          $io_rate = $io_rate + $write_io;
          if ( Xormon::isdigit($num) && Xormon::isdigit($resp_t_w) ) {
            $num = $num + ( $resp_t_w * $write_io );
          }
        }
        else {
          $io_rate = $write_io;
          if ( Xormon::isdigit($resp_t_w) ) {
            $num = $resp_t_w * $write_io;
          }
        }
      }
      if ( Xormon::isdigit($num) && Xormon::isdigit($io_rate) && $io_rate != 0 ) {
        $inventory_data{$section}{$element}{resp_t} = round( $num / $io_rate, 3 );
      }
      $inventory_data{$section}{$element}{io_rate}   = $io_rate;
      $inventory_data{$section}{$element}{data_rate} = $data_rate;
      # Calculate cache hit percentage
      if (Xormon::isdigit($inventory_data{$section}{$element}{sections_read_from_disk})) {
        my $cache_hit = 0;
        my $read_from_disk = $inventory_data{$section}{$element}{sections_read_from_disk};
        if (Xormon::isdigit($inventory_data{$section}{$element}{sections_read})) {
          my $sections_read = $inventory_data{$section}{$element}{sections_read};
          if ($sections_read !=0) {
            $cache_hit = ($sections_read - $read_from_disk) / $sections_read * 100;
          }
        }
        $inventory_data{$section}{$element}{r_cache_hit} = $cache_hit;
      }
    }
  }
}

sub write_data {

  @architecture = ();
  my $timestamp;
  my $rc;
  my $status;
  my $csv = "";
  my $hash_key;

  foreach my $subsystem ( keys %inventory_data ) {
    if ( Xormon::isdigit($inventory_data{$subsystem}{TIME}{END_TIMESTAMP})) {
      $timestamp = $inventory_data{$subsystem}{TIME}{END_TIMESTAMP};
    }
    process_subsystem($subsystem, $timestamp);
  }

  #
  # perf data status
  #
  if ( Xormon::isdigit($timestamp) ) {
    # check if there is current data
    # if ( ( time() - $timestamp ) > 10800 ) {
    #   $xormon->hasData(0, "last available stats are older than 3 hours: $timestamp");
    # }
    # else {
      $xormon->hasData(1);
    # }
  }
  else {
    $xormon->hasData(0, "no available stats");
  }

  # print Dumper \%device;

  ######
  ## SAN

  # SAN data saved also to DEVICE totals

  $hash_key = Xormon::create_hash_key($st_serial . "SAN");
  $hash_key = lc($hash_key);

  my %san_arch = (
    "item_id" => $hash_key,
    "label" => "SAN",
    "hw_type" => "infinibox",
    "subsystem" => "system",
    "parents" => [$st_serial],
    "hostcfg_id" => $cfg->{'hostcfg_id'}
  );
  # print Dumper \%arch;
  push(@architecture, \%san_arch);

  my $io_r      = defined $device{san}{read_io}   ? $device{san}{read_io}   : "";
  my $io_w      = defined $device{san}{write_io}  ? $device{san}{write_io}  : "";
  my $io_rate   = defined $device{san}{io_rate}   ? $device{san}{io_rate}   : "";
  my $data_r    = defined $device{san}{read}      ? $device{san}{read}      : "";
  my $data_w    = defined $device{san}{write}     ? $device{san}{write}     : "";
  my $data_rate = defined $device{san}{data_rate} ? $device{san}{data_rate} : "";
  my $resp_t_r  = defined $device{san}{resp_t_r}  ? $device{san}{resp_t_r}  : "";
  my $resp_t_w  = defined $device{san}{resp_t_w}  ? $device{san}{resp_t_w}  : "";
  my $resp_t    = defined $device{san}{resp_t}    ? $device{san}{resp_t}    : "";
  my $cache_hit = 0;

  if (Xormon::isdigit($device{san}{sections_from_disk})) {
    my $read_from_disk = $device{san}{sections_from_disk};
    if (Xormon::isdigit($device{san}{sections})) {
      my $sections_read = $device{san}{sections};
      if ($sections_read !=0) {
        $cache_hit = ($sections_read - $read_from_disk) / $sections_read * 100;
      }
    }
  }
  if ( Xormon::isdigit($resp_t) && Xormon::isdigit($io_rate) && $io_rate != 0 ) {
    $resp_t = $resp_t / $io_rate;
  }
    if ( Xormon::isdigit($resp_t_r) && Xormon::isdigit($io_r) && $io_r != 0 ) {
    $resp_t_r = $resp_t_r / $io_r;
  }
  if ( Xormon::isdigit($resp_t_w) && Xormon::isdigit($io_w) && $io_w != 0 ) {
    $resp_t_w = $resp_t_w / $io_w;
  }


  $csv = "id;timestamp;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t;r_cache_hit\n";

  if (defined $device{san}{timestamp} && Xormon::isdigit($device{san}{timestamp})) {
    my $time = $device{san}{timestamp};
    $csv   .= "$hash_key;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t_r;$resp_t_w;$resp_t;$cache_hit\n";
  }
  $rc = $xormon->saveCSV($csv,"infinibox","system");
  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("SAN status : $status");

  $csv = "";

  ######
  ## NAS
  $hash_key = Xormon::create_hash_key($st_serial . "NAS");
  $hash_key = lc($hash_key);

  my %nas_arch = (
    "item_id" => $hash_key,
    "label" => "NAS",
    "hw_type" => "infinibox",
    "subsystem" => "system",
    "parents" => [$st_serial],
    "hostcfg_id" => $cfg->{'hostcfg_id'}
  );
  push(@architecture, \%nas_arch);

  my $nas_io_r      = defined $device{nas}{read_io}   ? $device{nas}{read_io}   : "";
  my $nas_io_w      = defined $device{nas}{write_io}  ? $device{nas}{write_io}  : "";
  my $nas_io_rate   = defined $device{nas}{io_rate}   ? $device{nas}{io_rate}   : "";
  my $nas_data_r    = defined $device{nas}{read}      ? $device{nas}{read}      : "";
  my $nas_data_w    = defined $device{nas}{write}     ? $device{nas}{write}     : "";
  my $nas_data_rate = defined $device{nas}{data_rate} ? $device{nas}{data_rate} : "";
  my $nas_resp_t_r  = defined $device{nas}{resp_t_r}  ? $device{nas}{resp_t_r}  : "";
  my $nas_resp_t_w  = defined $device{nas}{resp_t_w}  ? $device{nas}{resp_t_w}  : "";
  my $nas_resp_t    = defined $device{nas}{resp_t}    ? $device{nas}{resp_t}    : "";
  my $nas_cache_hit = 0;

  if (Xormon::isdigit($device{nas}{sections_from_disk})) {
    my $read_from_disk = $device{nas}{sections_from_disk};
    if (Xormon::isdigit($device{nas}{sections})) {
      my $sections_read = $device{nas}{sections};
      if ($sections_read !=0) {
        $nas_cache_hit = ($sections_read - $read_from_disk) / $sections_read * 100;
      }
    }
  }

  if ( Xormon::isdigit($nas_resp_t) && Xormon::isdigit($nas_io_rate) && $nas_io_rate != 0 ) {
    $nas_resp_t = $nas_resp_t / $nas_io_rate;
  }
    if ( Xormon::isdigit($nas_resp_t_r) && Xormon::isdigit($nas_io_r) && $nas_io_r != 0 ) {
    $nas_resp_t_r = $nas_resp_t_r / $nas_io_r;
  }
  if ( Xormon::isdigit($nas_resp_t_w) && Xormon::isdigit($nas_io_w) && $nas_io_w != 0 ) {
    $nas_resp_t_w = $nas_resp_t_w / $nas_io_w;
  }


  $csv = "id;timestamp;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t;r_cache_hit\n";

  if (defined $device{nas}{timestamp} && Xormon::isdigit($device{nas}{timestamp})) {
    my $time = $device{nas}{timestamp};
    $csv   .= "$hash_key;$time;$nas_io_r;$nas_io_w;$nas_io_rate;$nas_data_r;$nas_data_w;$nas_data_rate;$nas_resp_t_r;$nas_resp_t_w;$nas_resp_t;$nas_cache_hit\n";

    $rc = $xormon->saveCSV($csv,"infinibox","system");
    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("NAS status : $status");
  }

  # print Dumper \%device;

  #########
  ## DEVICE
  my %dev_arch = (
    "item_id" => $st_serial,
    "label" => $cfg->{'hostalias'},
    "hw_type" => "infinibox",
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'hostcfg_id'}
  );
  push(@architecture, \%dev_arch);
  my $dev_csv = "id;timestamp;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t;capacity_total;capacity_used;capacity_free;capacity_used_percent;virtual_capacity_total;virtual_capacity_used;virtual_capacity_free;r_cache_hit\n";

  my $total    = defined $device{san}{total_physical_capacity} ? $device{san}{total_physical_capacity}  : "";
  my $used     = defined $device{san}{used_physical_space}     ? $device{san}{used_physical_space}      : "";
  my $free     = defined $device{san}{free_physical_space}     ? $device{san}{free_physical_space}      : "";
  my $used_pct = defined $device{san}{used_percent}            ? $device{san}{used_percent}             : "";

  my $total_v  = defined $device{san}{total_virtual_capacity}  ? $device{san}{total_virtual_capacity}   : "";
  my $used_v   = defined $device{san}{used_virtual_space}      ? $device{san}{used_virtual_space}       : "";
  my $free_v   = defined $device{san}{free_virtual_space}      ? $device{san}{free_virtual_space}       : "";


  if (Xormon::isdigit($resp_t) && Xormon::isdigit($io_rate) && $io_rate != 0 ){
    $resp_t = Xormon::formatNumberByType("float4", ( $resp_t / $io_rate ) );
  }
  if (Xormon::isdigit($resp_t_r) && Xormon::isdigit($io_r) && $io_r != 0 ){
    $resp_t_r = Xormon::formatNumberByType("float4", ( $resp_t_r / $io_r ) );
  }
  if (Xormon::isdigit($resp_t_w) && Xormon::isdigit($io_w) && $io_w != 0 ){
    $resp_t_w = Xormon::formatNumberByType("float4", ( $resp_t_w / $io_w ) );
  }

  if (Xormon::isdigit($timestamp)) {
    $dev_csv .= "$st_serial;$timestamp;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t_r;$resp_t_w;$resp_t;$total;$used;$free;$used_pct;$total_v;$used_v;$free_v;$cache_hit\n";
  }
  my $dev_rc = $xormon->saveCSV($dev_csv,"infinibox","device");
  # status code
  if ($dev_rc eq "201") {
    $status = "successful";
  } elsif ($dev_rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("DEVICE status : $status");


  # #save configuration
  $xormon->saveConfJSON(\%conf);

  # save architecture
  # print Dumper \@architecture;
  $xormon->saveArchitecture(\@architecture);

}

sub process_subsystem {
  my ($subsystem, $timestamp) = @_;

  my $rc;
  my $status;
  my $i = 0;
  my $csv = "";

  if ($subsystem eq "pool") {
    $csv .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t;block_size_r;block_size_w;block_size;capacity_total;capacity_used;capacity_free;capacity_used_percent;virtual_capacity_total;virtual_capacity_used;virtual_capacity_free;capacity_allocated;virtual_capacity_allocated;r_cache_hit;capacity_oversubscr;capacity_oversubscr_percent\n";
  }
  elsif ($subsystem eq "volume") {
    $csv .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t;block_size_r;block_size_w;block_size;capacity_total;capacity_used;capacity_free;capacity_used_percent;r_cache_hit\n";
  }
  elsif ($subsystem eq "host" || $subsystem eq "iscsiport" || $subsystem eq "port") {
    $csv .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t\n";
  }
  elsif ($subsystem eq "naspool") {
    $csv .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t;r_cache_hit\n";
  }
  elsif ($subsystem eq "fs") {
    $csv .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t;capacity_total;capacity_used;capacity_free;capacity_used_percent;capacity_allocated;r_cache_hit\n";
  }

  foreach my $item (keys %{$inventory_data{$subsystem}}) {
    next if $item eq "TIME";

    $i++;

    my $ref = $inventory_data{$subsystem}{$item};
    my $name = $ref->{'name'};
    my $hash_key = Xormon::create_hash_key($st_serial . uc($subsystem) . $item);
    $hash_key = lc($hash_key);

    my @parents = ();
    my @children = ();

    if ($subsystem eq "volume") {

      if (defined $ref->{'pool_id'}) {
        my $pool_hash = Xormon::create_hash_key($st_serial . "POOL" . $ref->{'pool_id'});
        push(@parents, $pool_hash);
      }
      if (defined $ref->{'serial'}) {
        $conf{$hash_key}{serial} = $ref->{'serial'};
      }
    }
    if ($subsystem eq "host") {
      if (defined $ref->{'volume_ids'}) {
        foreach my $vol_id ( @{ $ref->{'volume_ids'}} ) {
          my $vol_hash = Xormon::create_hash_key($st_serial . "VOLUME" . $vol_id);
          # print "$vol_id\n";
          push(@children, $vol_hash);
        }
      }
    }

    my %arch = (
        "item_id"   => $hash_key,
        "label"     => $name,
        "hw_type"   => "infinibox",
        "subsystem" => $subsystem,
        "parents"   => [$st_serial],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
    );

    if (@parents){
      push(@{$arch{parents}},@parents);
    }
    if (@children){
      push(@{$arch{children}},@children);
    }
    if ($subsystem eq "volume"){
      # WWN for mapping
      if (defined $ref->{'serial'}) {
        $arch{wwn} = Xormon::formatWWN($ref->{'serial'});
      }
    }

    push(@architecture, \%arch);
    # print Dumper \%arch;

    ## PERF
    if (Xormon::isdigit($timestamp) && $csv ne "") {
      if ($subsystem eq "pool" ) {

        my $io_r      = defined $ref->{read_io}   ? $ref->{read_io}   : "";
        my $io_w      = defined $ref->{write_io}  ? $ref->{write_io}  : "";
        my $io_rate   = defined $ref->{io_rate}   ? $ref->{io_rate}   : "";
        my $data_r    = defined $ref->{read}      ? $ref->{read}      : "";
        my $data_w    = defined $ref->{write}     ? $ref->{write}     : "";
        my $data_rate = defined $ref->{data_rate} ? $ref->{data_rate} : "";
        my $resp_t_r  = defined $ref->{resp_t_r}  ? $ref->{resp_t_r}  : "";
        my $resp_t_w  = defined $ref->{resp_t_w}  ? $ref->{resp_t_w}  : "";
        my $resp_t    = defined $ref->{resp_t}    ? $ref->{resp_t}    : "";

        my $sections  = defined $ref->{sections_read} ? $ref->{sections_read}    : "";
        my $sections_from_disk  = defined $ref->{sections_read_from_disk} ? $ref->{sections_read_from_disk}    : "";
        my $cache_hit = defined $ref->{r_cache_hit}   ? $ref->{r_cache_hit}      : 0;

        my $block_size    = "";
        my $block_size_r  = "";
        my $block_size_w  = "";
        # calculate block size
        if ( Xormon::isdigit($io_rate) && Xormon::isdigit($data_rate) ) {
          if ( $io_rate == 0 ) {
            $block_size = 0;
          }
          else{
            $block_size = Xormon::formatNumberByType("float4", ($data_rate / $io_rate));
          }
        }
        if ( Xormon::isdigit($io_r) && Xormon::isdigit($data_r) ) {
          if ( $io_r == 0 ) {
            $block_size_r = 0;
          }
          else{
            $block_size_r = Xormon::formatNumberByType("float4", ($data_r / $io_r));
          }
        }
        if ( Xormon::isdigit($io_w) && Xormon::isdigit($data_w) ) {
          if ( $io_w == 0 ) {
            $block_size_w = 0;
          }
          else{
            $block_size_w = Xormon::formatNumberByType("float4", ($data_w / $io_w));
          }
        }

        my $total   = defined $ref->{Capacity}         ? $ref->{Capacity}         : "";
        my $used    = defined $ref->{Used}             ? $ref->{Used}             : "";
        my $free    = defined $ref->{Free}             ? $ref->{Free}             : "";
        my $total_v = defined $ref->{Capacity_virtual} ? $ref->{Capacity_virtual} : "";
        my $used_v  = defined $ref->{Used_virtual}     ? $ref->{Used_virtual}     : "";
        my $free_v  = defined $ref->{Free_virtual}     ? $ref->{Free_virtual}     : "";

        ## oversubscr calculation
        my $oversubscr         = "";
        my $oversubscr_percent = "";

        if (Xormon::isdigit($total) && Xormon::isdigit($total_v)) {
          $oversubscr = $total_v - $total;
          if ($oversubscr < 0){$oversubscr = 0;}
          if ($total != 0) {
            $oversubscr_percent = round(($total_v / $total) * 100,0);
          }
        }

        my $used_pct = "";
        if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
          $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
        }

        my $allocated_phys  = defined $ref->{allocated_physical_space}    ? $ref->{allocated_physical_space}    : "";
        my $allocated_virt  = defined $ref->{allocated_virtual_space}     ? $ref->{allocated_virtual_space}     : "";

        ## DEVICE STATS
        if (!defined $device{san}{timestamp}) {
          $device{san}{timestamp} = $timestamp;
        }
        if (Xormon::isdigit($io_r)) {
          $device{san}{read_io} = Xormon::isdigit($device{san}{read_io}) ? $device{san}{read_io} + $io_r : $io_r;
        }
        if (Xormon::isdigit($io_w)) {
          $device{san}{write_io} = Xormon::isdigit($device{san}{write_io}) ? $device{san}{write_io} + $io_w : $io_w;
        }
        if (Xormon::isdigit($io_rate)) {
          $device{san}{io_rate} = Xormon::isdigit($device{san}{io_rate}) ? $device{san}{io_rate} + $io_rate : $io_rate;
        }
        if (Xormon::isdigit($data_r)) {
          $device{san}{read} = Xormon::isdigit($device{san}{read}) ? $device{san}{read} + $data_r : $data_r;
        }
        if (Xormon::isdigit($data_w)) {
          $device{san}{write} = Xormon::isdigit($device{san}{write}) ? $device{san}{write} + $data_w : $data_w;
        }
        if (Xormon::isdigit($data_rate)) {
          $device{san}{data_rate} = Xormon::isdigit($device{san}{data_rate}) ? $device{san}{data_rate} + $data_rate : $data_rate;
        }
        if (Xormon::isdigit($sections)) {
          $device{san}{sections} = Xormon::isdigit($device{san}{sections}) ? $device{san}{sections} + $sections : $sections;
        }
        if (Xormon::isdigit($sections_from_disk)) {
          $device{san}{sections_from_disk} = Xormon::isdigit($device{san}{sections_from_disk}) ? $device{san}{sections_from_disk} + $sections_from_disk : $sections_from_disk;
        }
        # DEVICE resp_t
        if (Xormon::isdigit($resp_t) && Xormon::isdigit($io_rate)) {
          $device{san}{resp_t} = Xormon::isdigit($device{'san'}{'resp_t'})
            ? $device{san}{resp_t} + ($io_rate * $resp_t)
            : $io_rate * $resp_t;
        }
        if (Xormon::isdigit($resp_t_r) && Xormon::isdigit($io_r)) {
          $device{san}{resp_t_r} = Xormon::isdigit($device{'san'}{'resp_t_r'})
            ? $device{san}{resp_t_r} + ($io_r * $resp_t_r)
            : $io_r * $resp_t_r;
        }
        if (Xormon::isdigit($resp_t_w) && Xormon::isdigit($io_w)) {
          $device{san}{resp_t_w} = Xormon::isdigit($device{'san'}{'resp_t_w'})
            ? $device{san}{resp_t_w} + ($io_w * $resp_t_w)
            : $io_w * $resp_t_w;
        }

        $csv .= "$hash_key;$timestamp;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t_r;$resp_t_w;$resp_t;$block_size_r;$block_size_w;$block_size;$total;$used;$free;$used_pct;$total_v;$used_v;$free_v;$allocated_phys;$allocated_virt;$cache_hit;$oversubscr;$oversubscr_percent\n";
      }
      elsif ($subsystem eq "volume") {
        my $io_r      = defined $ref->{read_io}   ? $ref->{read_io}   : "";
        my $io_w      = defined $ref->{write_io}  ? $ref->{write_io}  : "";
        my $io_rate   = defined $ref->{io_rate}   ? $ref->{io_rate}   : "";
        my $data_r    = defined $ref->{read}      ? $ref->{read}      : "";
        my $data_w    = defined $ref->{write}     ? $ref->{write}     : "";
        my $data_rate = defined $ref->{data_rate} ? $ref->{data_rate} : "";
        my $resp_t_r  = defined $ref->{resp_t_r}  ? $ref->{resp_t_r}  : "";
        my $resp_t_w  = defined $ref->{resp_t_w}  ? $ref->{resp_t_w}  : "";
        my $resp_t    = defined $ref->{resp_t}    ? $ref->{resp_t}    : "";

        my $cache_hit = defined $ref->{r_cache_hit} ? $ref->{r_cache_hit} : 0;

        my $block_size    = "";
        my $block_size_r  = "";
        my $block_size_w  = "";
        # calculate block size
        if ( Xormon::isdigit($io_rate) && Xormon::isdigit($data_rate) ) {
          if ( $io_rate == 0 ) {
            $block_size = 0;
          }
          else{
            $block_size = Xormon::formatNumberByType("float4", ($data_rate / $io_rate));
          }
        }
        if ( Xormon::isdigit($io_r) && Xormon::isdigit($data_r) ) {
          if ( $io_r == 0 ) {
            $block_size_r = 0;
          }
          else{
            $block_size_r = Xormon::formatNumberByType("float4", ($data_r / $io_r));
          }
        }
        if ( Xormon::isdigit($io_w) && Xormon::isdigit($data_w) ) {
          if ( $io_w == 0 ) {
            $block_size_w = 0;
          }
          else{
            $block_size_w = Xormon::formatNumberByType("float4", ($data_w / $io_w));
          }
        }

        my $total   = defined $ref->{Capacity}         ? $ref->{Capacity}         : "";
        my $used    = defined $ref->{Used}             ? $ref->{Used}             : "";
        my $free    = defined $ref->{Free}             ? $ref->{Free}             : "";

        my $used_pct = "";
        if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
          $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
        }

        $csv .= "$hash_key;$timestamp;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t_r;$resp_t_w;$resp_t;$block_size_r;$block_size_w;$block_size;$total;$used;$free;$used_pct;$cache_hit\n";
      }
      elsif ($subsystem eq "host" || $subsystem eq "iscsiport" || $subsystem eq "port") {
        my $io_r      = defined $ref->{read_io}   ? $ref->{read_io}   : "";
        my $io_w      = defined $ref->{write_io}  ? $ref->{write_io}  : "";
        my $io_rate   = defined $ref->{io_rate}   ? $ref->{io_rate}   : "";
        my $data_r    = defined $ref->{read}      ? $ref->{read}      : "";
        my $data_w    = defined $ref->{write}     ? $ref->{write}     : "";
        my $data_rate = defined $ref->{data_rate} ? $ref->{data_rate} : "";
        my $resp_t_r  = defined $ref->{resp_t_r}  ? $ref->{resp_t_r}  : "";
        my $resp_t_w  = defined $ref->{resp_t_w}  ? $ref->{resp_t_w}  : "";
        my $resp_t    = defined $ref->{resp_t}    ? $ref->{resp_t}    : "";

        $csv .= "$hash_key;$timestamp;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t_r;$resp_t_w;$resp_t\n";
      }
      elsif ($subsystem eq "naspool") {
        my $io_r      = defined $ref->{read_io}   ? $ref->{read_io}   : "";
        my $io_w      = defined $ref->{write_io}  ? $ref->{write_io}  : "";
        my $io_rate   = defined $ref->{io_rate}   ? $ref->{io_rate}   : "";
        my $data_r    = defined $ref->{read}      ? $ref->{read}      : "";
        my $data_w    = defined $ref->{write}     ? $ref->{write}     : "";
        my $data_rate = defined $ref->{data_rate} ? $ref->{data_rate} : "";
        my $resp_t_r  = defined $ref->{resp_t_r}  ? $ref->{resp_t_r}  : "";
        my $resp_t_w  = defined $ref->{resp_t_w}  ? $ref->{resp_t_w}  : "";
        my $resp_t    = defined $ref->{resp_t}    ? $ref->{resp_t}    : "";

        my $sections  = defined $ref->{sections_read} ? $ref->{sections_read}    : "";
        my $sections_from_disk  = defined $ref->{sections_read_from_disk} ? $ref->{sections_read_from_disk}    : "";
        my $cache_hit = defined $ref->{r_cache_hit}   ? $ref->{r_cache_hit}      : 0;

        $csv .= "$hash_key;$timestamp;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t_r;$resp_t_w;$resp_t;$cache_hit\n";

        ## DEVICE NAS STATS
        if (!defined $device{nas}{timestamp}) {
          $device{nas}{timestamp} = $timestamp;
        }
        if (Xormon::isdigit($io_r)) {
          $device{nas}{read_io} = Xormon::isdigit($device{nas}{read_io}) ? $device{nas}{read_io} + $io_r : $io_r;
        }
        if (Xormon::isdigit($io_w)) {
          $device{nas}{write_io} = Xormon::isdigit($device{nas}{write_io}) ? $device{nas}{write_io} + $io_w : $io_w;
        }
        if (Xormon::isdigit($io_rate)) {
          $device{nas}{io_rate} = Xormon::isdigit($device{nas}{io_rate}) ? $device{nas}{io_rate} + $io_rate : $io_rate;
        }
        if (Xormon::isdigit($data_r)) {
          $device{nas}{read} = Xormon::isdigit($device{nas}{read}) ? $device{nas}{read} + $data_r : $data_r;
        }
        if (Xormon::isdigit($data_w)) {
          $device{nas}{write} = Xormon::isdigit($device{nas}{write}) ? $device{nas}{write} + $data_w : $data_w;
        }
        if (Xormon::isdigit($data_rate)) {
          $device{nas}{data_rate} = Xormon::isdigit($device{nas}{data_rate}) ? $device{nas}{data_rate} + $data_rate : $data_rate;
        }
        if (Xormon::isdigit($sections)) {
          $device{nas}{sections} = Xormon::isdigit($device{nas}{sections}) ? $device{nas}{sections} + $sections : $sections;
        }
        if (Xormon::isdigit($sections_from_disk)) {
          $device{nas}{sections_from_disk} = Xormon::isdigit($device{nas}{sections_from_disk}) ? $device{nas}{sections_from_disk} + $sections_from_disk : $sections_from_disk;
        }
        # DEVICE resp_t
        if (Xormon::isdigit($resp_t) && Xormon::isdigit($io_rate)) {
          $device{nas}{resp_t} = Xormon::isdigit($device{'nas'}{'resp_t'})
            ? $device{nas}{resp_t} + ($io_rate * $resp_t)
            : $io_rate * $resp_t;
        }
        if (Xormon::isdigit($resp_t_r) && Xormon::isdigit($io_r)) {
          $device{nas}{resp_t_r} = Xormon::isdigit($device{'nas'}{'resp_t_r'})
            ? $device{nas}{resp_t_r} + ($io_r * $resp_t_r)
            : $io_r * $resp_t_r;
        }
        if (Xormon::isdigit($resp_t_w) && Xormon::isdigit($io_w)) {
          $device{nas}{resp_t_w} = Xormon::isdigit($device{'nas'}{'resp_t_w'})
            ? $device{nas}{resp_t_w} + ($io_w * $resp_t_w)
            : $io_w * $resp_t_w;
        }
      }
      elsif ($subsystem eq "fs") {
        my $io_r      = defined $ref->{read_io}   ? $ref->{read_io}   : "";
        my $io_w      = defined $ref->{write_io}  ? $ref->{write_io}  : "";
        my $io_rate   = defined $ref->{io_rate}   ? $ref->{io_rate}   : "";
        my $data_r    = defined $ref->{read}      ? $ref->{read}      : "";
        my $data_w    = defined $ref->{write}     ? $ref->{write}     : "";
        my $data_rate = defined $ref->{data_rate} ? $ref->{data_rate} : "";
        my $resp_t_r  = defined $ref->{resp_t_r}  ? $ref->{resp_t_r}  : "";
        my $resp_t_w  = defined $ref->{resp_t_w}  ? $ref->{resp_t_w}  : "";
        my $resp_t    = defined $ref->{resp_t}    ? $ref->{resp_t}    : "";

        my $cache_hit = defined $ref->{r_cache_hit} ? $ref->{r_cache_hit} : 0;

        my $total     = defined $ref->{Capacity}         ? $ref->{Capacity}         : "";
        my $used      = defined $ref->{Used}             ? $ref->{Used}             : "";
        my $free      = defined $ref->{Free}             ? $ref->{Free}             : "";

        my $allocated = defined $ref->{Allocated}        ? $ref->{Allocated}        : "";

        my $used_pct = "";
        if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
          $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
        }

        $csv .= "$hash_key;$timestamp;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t_r;$resp_t_w;$resp_t;$total;$used;$free;$used_pct;$allocated;$cache_hit\n";
      }
    }
  }
  if ($i > 0) {
    ## SAVE SUBSYSTEM
    # print $csv;
    $rc = $xormon->saveCSV($csv,"infinibox","$subsystem");
      # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("$subsystem status : $status");
  }
}



sub get_configuration {

  my $out = cmd("gui/overview");
  if ( $out eq "" ) {
    return;
  }
  my $data  = $out->{result}->{system};
  my $error = $out->{error};

  # print Dumper \$data;

  if ( defined $error && $error ne "" ) {
    $logger->error("$error");
    return;
  }

  $conf{$st_serial}{op_state} = defined $data->{"operational_state"}->{"state"} ? $data->{"operational_state"}->{"state"} : "undef";
  $conf{$st_serial}{model} = defined $data->{"model"} ? $data->{"model"} : "";
  $conf{$st_serial}{node_name} = defined $data->{"name"} ? $data->{"name"} : "";
  $conf{$st_serial}{wwnn} = defined $data->{"wwnn"} ? $data->{"wwnn"} : "";
  $conf{$st_serial}{serial} = defined $data->{"serial_number"} ? $data->{"serial_number"} : "";
  $conf{$st_serial}{version} = defined $data->{"version"} ? $data->{"version"} : "";
  $conf{$st_serial}{uptime} = defined $data->{"uptime"} ? $data->{"uptime"} : "";
  $conf{$st_serial}{ip} = $cfg->{"apiHost"};

  #### capacity section
  $conf{$st_serial}{capacity_total}                 = defined $data->{capacity}->{"total_physical_capacity"} ? $data->{capacity}->{"total_physical_capacity"} : "";
  $conf{$st_serial}{capacity_free}                  = defined $data->{capacity}->{"free_physical_space"} ? $data->{capacity}->{"free_physical_space"} : "";
  if (Xormon::isdigit($conf{$st_serial}{capacity_total}) && Xormon::isdigit($conf{$st_serial}{capacity_free})) {
    $conf{$st_serial}{capacity_used} = $conf{$st_serial}{capacity_total} - $conf{$st_serial}{capacity_free};
  }
  # $conf{$st_serial}{used_spare_partitions}          = defined $data->{capacity}->{"used_spare_partitions"} ? $data->{capacity}->{"used_spare_partitions"} : "";
  # $conf{$st_serial}{free_virtual_space}             = defined $data->{capacity}->{"free_virtual_space"} ? $data->{capacity}->{"free_virtual_space"} : "";
  # $conf{$st_serial}{total_virtual_capacity}         = defined $data->{capacity}->{"total_virtual_capacity"} ? $data->{capacity}->{"total_virtual_capacity"} : "";
  # $conf{$st_serial}{dynamic_spare_drive_cost}       = defined $data->{capacity}->{"dynamic_spare_drive_cost"} ? $data->{capacity}->{"dynamic_spare_drive_cost"} : "";
  # $conf{$st_serial}{total_allocated_physical_space} = defined $data->{capacity}->{"total_allocated_physical_space"} ? $data->{capacity}->{"total_allocated_physical_space"} : "";
  # $conf{$st_serial}{total_spare_bytes}              = defined $data->{capacity}->{"total_spare_bytes"} ? $data->{capacity}->{"total_spare_bytes"} : "";
  # $conf{$st_serial}{used_spare_bytes}               = defined $data->{capacity}->{"used_spare_bytes"} ? $data->{capacity}->{"used_spare_bytes"} : "";
  # $conf{$st_serial}{used_dynamic_spare_partitions}  = defined $data->{capacity}->{"used_dynamic_spare_partitions"} ? $data->{capacity}->{"used_dynamic_spare_partitions"} : "";
  # $conf{$st_serial}{used_dynamic_spare_bytes}       = defined $data->{capacity}->{"used_dynamic_spare_bytes"} ? $data->{capacity}->{"used_dynamic_spare_bytes"} : "";
  # $conf{$st_serial}{total_spare_partitions}         = defined $data->{capacity}->{"total_spare_partitions"} ? $data->{capacity}->{"total_spare_partitions"} : "";


  if (Xormon::isdigit($data->{capacity}->{"total_physical_capacity"})) {
    $device{san}{total_physical_capacity} = $data->{capacity}->{"total_physical_capacity"};
  }
  if (Xormon::isdigit($data->{capacity}->{"free_physical_space"})) {
    $device{san}{free_physical_space} = $data->{capacity}->{"free_physical_space"};
  }
  if (Xormon::isdigit($device{san}{total_physical_capacity}) && Xormon::isdigit($device{san}{free_physical_space})) {
    $device{san}{used_physical_space} = $device{san}{total_physical_capacity} - $device{san}{free_physical_space};
    if ($device{san}{total_physical_capacity} != 0) {
      $device{san}{used_percent} = Xormon::formatNumberByType("float4", $device{san}{used_physical_space} / ($device{san}{total_physical_capacity} / 100) );
    }
  }
  if (Xormon::isdigit($data->{capacity}->{"total_virtual_capacity"})) {
    $device{san}{total_virtual_capacity} = $data->{capacity}->{"total_virtual_capacity"};
  }
  if (Xormon::isdigit($data->{capacity}->{"free_virtual_space"})) {
    $device{san}{free_virtual_space} = $data->{capacity}->{"free_virtual_space"};
  }
  if (Xormon::isdigit($device{san}{total_virtual_capacity}) && Xormon::isdigit($device{san}{free_virtual_space})) {
    $device{san}{used_virtual_space} = $device{san}{total_virtual_capacity} - $device{san}{free_virtual_space};
  }

  #### convert uptime to d h m
  if ( Xormon::isdigit($conf{$st_serial}{uptime}) ) {
    my $secs = $conf{$st_serial}{uptime} / 1000;

    my $days = int( $secs / ( 24 * 60 * 60 ) );
    $secs -= ( $days * 24 * 60 * 60 );

    my $hours = int( $secs / 3600 );
    $secs -= ( $hours * 3600 );

    my $mins = int( $secs / 60 );
    $secs -= ( $mins * 60 );

    $conf{$st_serial}{uptime} = "$days" . "d" . " $hours" . "h" . " $mins" . "m";
  }


}

sub health_status {
  my %hs_data;
  my $out;
  my @error_message  = ();
  my $main_state     = "ok";

  if (defined $conf{$st_serial}{op_state} && $conf{$st_serial}{op_state} ne "undef") {
    my $op_state = $conf{$st_serial}{op_state};
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{device}{op_state} = $op_state;
    if ( $op_state eq "FAILED" || $op_state =~ m/^ERROR/ ) {
      $main_state = "error";
      push( @error_message, "Operational state: $op_state\n" );
    }
  }
  if (defined $conf{$st_serial}{uptime} && $conf{$st_serial}{uptime} ne "") {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{device}{uptime} = $conf{$st_serial}{uptime};
  }

  $out = cmd("services");
  if ( $out eq "" ) {
    return;
  }
  my $data  = $out->{result};
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }

  # print Dumper $out;

  foreach my $service ( @{$data} ) {

    my $name   = $service->{"name"};
    my $status = $service->{"cluster_state"};
    if ( !defined $name || !defined $status ) {
      next;
    }
    my $uppercase = uc($name);
    $name = $uppercase;
    if ( $status eq "FAILED" || $status =~ m/^ERROR|^DEGRADED/ ) {
      $main_state = "error";
      push( @error_message, "Service $name: $status\n" );
    }
    my %service;
    $service{name}   = $name;
    $service{status} = $status;
    push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{services} }, \%service );
  }
  ### health status
  $out = cmd( "system/health_state", "", "", "health_state" );
  if ( $out eq "" ) {
    undef $out;
    $out->{result}->{node_bbu_protection} = "";
  }
  $data  = $out->{result};
  $error = $out->{error};
  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }
  my $node_bbu_protection = $data->{node_bbu_protection};
  if ( defined $node_bbu_protection && ref($node_bbu_protection) eq "HASH" ) {
    foreach my $node ( keys %{$node_bbu_protection} ) {
      if ( $node_bbu_protection->{$node} ne "protected" ) {
        ### error
        push( @error_message, "Node $node is not protected by BBU\n" );
        $main_state = "error";        
        last;
      }
    }
  }
  ###

  ## EVENTS
  my $last_half_year = ( time() - 15778463 ) * 1000;
  my %events_data;
  $events_data{"critical"} = cmd( "events?page_size=1000&level=eq:critical&timestamp=gt:$last_half_year", "", "","events_critical" );
  $events_data{"error"}    = cmd( "events?page_size=1000&level=eq:error&timestamp=gt:$last_half_year", "", "","events_error" );
  $events_data{"warning"}  = cmd( "events?page_size=1000&level=eq:warning&timestamp=gt:$last_half_year", "", "","events_warning" );
  foreach my $event_type ( keys %events_data ) {
    if ( $events_data{$event_type} eq "" ) { next; }
    $data  = $events_data{$event_type}{result};
    $error = $events_data{$event_type}{error};

    if ( defined $error && $error ne "" ) {
      $logger->error("ERROR: $error");
      next;
    }
    foreach my $event ( @{$data} ) {

      my $timestamp   = $event->{"timestamp"};
      my $reporter    = $event->{"reporter"};
      my $user        = $event->{"username"};
      my $seq         = $event->{"seq_num"};
      my $description = $event->{"description"};
      my $code        = $event->{"code"};
      my $level       = $event->{level};

      if ( !defined $level || $level eq "" ) { next; }

      if ( !defined $timestamp || $timestamp eq "" ) {
        $timestamp = "";
      }
      # $timestamp =~ s/T/ /g;
      if ( !defined $reporter || $reporter eq "" ) {
        $reporter = "";
      }
      if ( !defined $user || $user eq "" ) {
        $user = "";
      }
      if ( !defined $seq || $seq eq "" ) {
        $seq = "";
      }
      if ( !defined $description || $description eq "" ) {
        $description = "";
      }
      if ( !defined $code || $code eq "" ) {
        $code = "";
      }
      my %event;
      $event{time}        = int($timestamp);
      $event{seq}         = $seq;
      $event{level}       = $level;
      $event{reporter}    = $reporter;
      $event{user}        = $user;
      $event{code}        = $code;
      $event{description} = $description;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{events} }, \%event );
    }
  }

  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }

  my @hs = (
    {
      "item_id" => $cfg->{'hostcfg_id'},
      "status" => $main_state,
      "data" => join("\n", @error_message),
    }
  );

  # print Dumper \%hs_data;
  # print \@hs;


  my $rc = $xormon->saveStatus(\@hs);
  my $rc_hs = $xormon->saveConfJSON(\%hs_data, 1);

}

sub connection_test {
  my $client = shift;
  $cfg = shift;

  my @rows;
  my @filters;

  my $test = Test->new($client);

  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $logger = XormonLogger->new( 'infinibox', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  $test->addSection("Validation of variables", 1 , "");

  my ($code, $text) = Xormon::conntest_tcp($cfg->{apiHost}, "443");
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  my $cookie = login();
  $cfg->{'cookie'} = $cookie;

  if ( defined $cookie && $cookie ne "" ) {
    $test->addSection("API authorization", 1 , "Cookie granted, login OK");
    my $out = cmd("gui/overview");
    if ( $out eq "" ) {
      logout($cookie);
      $test->addSection("CMD to API", 0, "ERROR: CMD not returned!");
      return $test->stop();
    }

    my $data  = $out->{result}->{system};
    my $error = $out->{error};

    if ( defined $error && $error ne "" ) {
      $test->addSection("CMD to API", 0, "ERROR: $error");
      logout($cookie);
      return $test->stop();
    }
    else {
      push (@rows, "Data returned OK from API\n");
      my $model         = defined $data->{"model"} ? $data->{"model"} : "Model is not defined!";
      my $name          = defined $data->{"name"} ? $data->{"name"} : "Name is not defined!";
      my $serial        = defined $data->{"serial_number"} ? $data->{"serial_number"} : "Serial number is not defined!";
      my $version       = defined $data->{"version"} ? $data->{"version"} : "Version is not defined!";

      push (@rows, "Model: $model\n");
      push (@rows, "Name: $name\n");
      push (@rows, "Serial: $serial\n");
      push (@rows, "Version: $version\n");
      $test->addSection("CMD to API", 1, "@rows");
    }

    my $filter = create_filter('{ "protocol_type": "SAN" }');
    if ( defined $filter ) {
      modify_filter( $filter, '{ "operation_category": "read" }' );
      push( @filters, $filter );
      my $filter2 = create_filter('{ "protocol_type": "SAN" }');
      if ( defined $filter2 ) {
        modify_filter( $filter2, '{ "operation_category": "write" }' );
        push( @filters, $filter2 );
        my $collector  = create_collector("{\"filter_id\":$filters[0],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"pool_id\",\"interval_milliseconds\":10000}");
        my $collector2 = create_collector("{\"filter_id\":$filters[1],\"type\":\"AGGREGATOR\",\"collected_fields\":[\"ops\",\"throughput\",\"external_latency\"],\"grouping_field\":\"pool_id\",\"interval_milliseconds\":10000}");
        if ( defined $collector && defined $collector2 ) {
          my $out  = cmd("metrics/collectors/data?collector_id=$collector");
          my $out2 = cmd("metrics/collectors/data?collector_id=$collector2");
          if ( $out ne "" ) {
            my $error = $out->{error};
            if ( defined $error && $error ne "" ) {
              $test->addSection("Performance data", 0, "$error");
            }
            else {
              if ( $out2 ne "" ) {
                my $error = $out2->{error};
                if ( defined $error && $error ne "" ) {
                  $test->addSection("Performance data", 0, "$error");
                }
                else {
                  $test->addSection("Performance data", 1, "Performance data OK");
                }
              }
            }
          }
          else {
            $test->addSection("Performance data", 0, "No output from collectors!");
          }
          delete_collector($collector);
          delete_collector($collector2);
        }
        else {
          $test->addSection("Performance data", 0, "Collector write or read is not defined!");
        }
        delete_filter($filter2);
      }
      else {
        $test->addSection("Performance data", 0, "Filter write is not defined!");
      }
      delete_filter($filter);
    }
    else {
      $test->addSection("Performance data", 0, "Filter read is not defined!");
    }
  }
  else {
    $test->addSection("API authorization", 0, "Cookie not granted, login failed!");
  }

  return $test->stop();
}

sub delete_filter {
  my $filter_id = shift;
  my $out       = cmd( "metrics/filters/$filter_id", "DELETE" );
  if ( $out eq "" ) {
    return;
  }
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }
  return 1;
}

sub delete_collector {
  my $filter_id = shift;
  my $out       = cmd( "metrics/collectors/$filter_id", "DELETE" );
  if ( $out eq "" ) {
    return;
  }
  my $error = $out->{error};

  if ( defined $error && $error ne "" ) {
    $logger->error("ERROR: $error");
    return;
  }
  return 1;
}
1;
