use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;                
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Xormon;
use INFINIBOX;

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

# microservice port
my $port = 3224;

my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

 # waiting for a new client connection
while ($client = $server->accept()) {

    my $pid;
    unless ( defined( $pid = fork() ) ) {
        Xormon::error("Wait for fork");
        sleep 5;
    }

    if ($pid == 0) {
   
        # get information about a newly connected client
        my $client_address = $client->peerhost();
        my $client_port = $client->peerport();
        Xormon::log("Connection from $client_address:$client_port");

        # read up to 2048 characters from the connected client
        my $data = "";
        $client->recv($data, 2048);

        my $decoded = from_json($data);
        my $cfg = $decoded->{data};
        
        Xormon::log("Received task: ".$decoded->{task});
        my $response = '';
        if ($decoded->{task} eq "fetch") {

            $client->send("Task successfuly started");
            $client->shutdown(SHUT_WR);

            my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'infinibox', $cfg->{hostcfg_id}, $cfg->{hostalias});

            $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

            # target destinations check
            if (defined $decoded->{'destinations'}) {
                $xormon->setDestinations($decoded->{'destinations'});
            }

            my $status = INFINIBOX::collect($xormon,$cfg);

            # end task
            $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);
            Xormon::log("Task [fetch] for ".$cfg->{hostalias}." is done");

        } elsif ($decoded->{task} eq "test") {
            INFINIBOX::connection_test($client,$cfg);
            $client->shutdown(SHUT_WR);
            Xormon::log("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done.");
        } else {
            $response = "undefined task";

            $client->send($response);
            $client->shutdown(SHUT_WR);

        }

        exit();

    } else {
        $client->autoflush(1);    
    }
}

$server->close();