use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/";    # use lib in this folder
use lib "../../../perl-modules";
use Xormon;
use Digest::MD5 qw(md5 md5_hex);
use File::Copy;

use Storable;

# store \%table, 'file';
# $hashref = retrieve('file');

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);
use List::Util qw(min);

my $pwd = `pwd`;
chomp $pwd;

# print "PWD = $pwd\n";
my $TMPDIR = "$pwd/tmp";
if ( !-d $TMPDIR ) { mkdir $TMPDIR || die "Cant create dir $TMPDIR: $@"; }

# microservice port
my $port     = 8162;
my $xormon   = Xormon->new( "127.0.0.1", "", 'linux', "", "" );
my $vars     = $xormon->getMicroserviceVariables("linux");
my $varsHash = ();
if ( defined $vars->{"microserviceVariables"} ) {
  for my $value ( @{ $vars->{"microserviceVariables"} } ) {
    $varsHash->{ $value->{"key"} } = $value->{"value"};
  }
}

if ( defined $varsHash->{"PORT"} ) {
  $port = $varsHash->{"PORT"} + 0;
}

my $forkLimit = 300;
if ( defined $varsHash->{"FORK_LIMIT"} ) {
  $forkLimit = $varsHash->{"FORK_LIMIT"} + 0;
}

if ( Xormon::isPortOpen($port) == 0 ) {
  Xormon::log("The port [$port] is in use by another process. Is the lpar2rrd daemon running? Docs: https://xormon.com/port_conflict.php");
  exit(1);
}

my $server = IO::Socket->new(
  Domain    => AF_INET,
  Type      => SOCK_STREAM,
  Proto     => 'tcp',
  LocalHost => '0.0.0.0',
  LocalPort => $port,
  ReusePort => 1,
  Listen    => 10000,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");
Xormon::log("Fork limit: $forkLimit");

my $client;

my $xormon_address;
my $xormon_token;
my $address;

### definitions

my $minutes = 5; # every XX minutes send architecture to DB
my $windows_data_dir_name = ""; # if ! empty call windows routine
my $max_data_delay = 302; # max delay for agent perf data, a little more than 5 mins
my $timeout = 300;
my $transfer_rows_lim_per_alert = 1440;
my $mapping                     = "";
my $first_mapping               = 1;      #call mapping only once in session
my $nmon_lines                  = 0;

### global variables
my %perf = ();
my %conf;
my %linux_arch       = ();                # this is only one
my %job_arch_list    = ();
my @architecture_job = ();
my %lan_arch_list    = ();
my @architecture_lan = ();
my %san_arch_list    = ();
my @architecture_sea = ();
my %sea_arch_list    = ();
my @architecture_san = ();

#my %san_power_arch_list = ();
my @architecture_san_power = ();
my %wlm_arch_list          = ();
my @architecture_wlm       = ();
my $linux_arch_conf        = "$TMPDIR/LINUX_ARCH_CONF";    # is global

my $read_rec          = 0;                                 # for debug
my $last_but_one_line = "";

my $uuid = "";

# waiting for a new client connection
while ( $client = $server->accept() ) {

  my $agent = 0;
  my $data  = "";

  my $client_address = $client->peerhost();
  my $client_port    = $client->peerport();
  my @ps             = "";
  my $pscmd          = `which ps`;
  chomp $pscmd;
  my $target = readlink($pscmd);
  if ( $target && $target =~ "busybox" ) {
    @ps = `ps w`;

    #@ps = `ps w 2>>$error_log`;
  }
  else {
    @ps = `ps -ef`;

    #@ps = `ps -ef 2>>$error_log`;
  }

  # flush after every write
  $| = $client;

  # use alert here to be sure it does not hang due to 1 hanging connection
  eval {
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM: after : $timeout seconds - client: $client_address ($client_port)"; };
    alarm($timeout);

    # Xormon::log("before client->recv $client_address:$client_port ".localtime());
    # $client->recv($data, 2048);
    $client->recv( $data, 8388608 );    # 8 MiB
                                        # Xormon::log("after client->recv $client_address:$client_port ".localtime());

    alarm(0);
  };
  alarm(0);

  chomp($@);
  if ($@) {
    if ( $@ =~ m/died in SIG ALRM/ ) {
      Xormon::log( "Client connection timed out after : $timeout seconds - client: $client_address ($client_port) " . __FILE__ . ":" . __LINE__ );
      Xormon::error( "Client connection timed out after : $timeout seconds - client: $client_address ($client_port) " . __FILE__ . ":" . __LINE__ );
    }
    else {
      Xormon::log("timeout result - client: $client_address ($client_port) $@");
      Xormon::error("timeout result - client: $client_address ($client_port) $@");
    }
    next;
  }
  my $decoded = eval { decode_json($data) };
  if ( $@ || ref($decoded) ne "HASH" ) {
    $agent = 1;
  }
  else {
    # token from xormon server
    $xormon_address = $client->peerhost();
    $xormon_token   = $decoded->{jwt};
    Xormon::log("Server data updated");

    #if (defined $decoded->{task} && $decoded->{task} eq "metrics") {
    #    my $xormon = Xormon->new("$xormon_address:3000", $xormon_token, 'linux');
    #    &metrics_def($xormon);
    #}

    $client->send("Server data updated");
    $client->shutdown(SHUT_WR);
  }
    if ($agent == 1) {

        my $pid;
        unless ( defined( $pid = fork() ) ) {
            Xormon::error("Wait for fork");
            sleep 5;
        }

        if ($pid == 0) {
            if ( process_limits( $0, $forkLimit, \@ps ) == 1 ) {
              Xormon::log("Exiting as more than $forkLimit forks is running");
              exit();
            }
            # get information about a newly connected client
            my $client_address = $client->peerhost();
            my $client_port = $client->peerport();
            $address = $client_address;
            Xormon::log("Connection from $client_address:$client_port protocol:$data,");

            if (!defined $xormon_token) {
                Xormon::error("No token is specified");
                $client->shutdown(SHUT_WR);
                exit();
            }
            if ($data eq "") {
              Xormon::log("Connection from $client_address:$client_port empty PROTOCOL->exit");
                    Xormon::error("Connection from $client_address:$client_port empty PROTOCOL->exit");
                    $client->shutdown(SHUT_WR);
                    exit();
                  }
            # flush after every write
            $| = 1;

            chomp $data;
            # sometimes there are two lines like:  Argument "4.1\nLinux:vm-jindra:FF884D5..." isn't numeric in numeric ge (>=)
            ($data, my $perf_line) = split "\n",$data;
            chomp $data;
            $data = substr( $data, 0, 3 );

            if ( isdigit($data) == 0 ) {
              $data =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg;    # to make it readable in logs
              Xormon::log("Connection from $client_address:$client_port exit for bad data ,$data,");
              Xormon::error("Connection from $client_address:$client_port exit for bad data ,$data,");
              $client->shutdown(SHUT_WR);
              exit();
            }
            if ( $data < 2 ) {
              Xormon::log("Connection from $client_address:$client_port exit for unknown protocol ,$data,");
              Xormon::error("Connection from $client_address:$client_port exit for unknown protocol ,$data,");
              $client->shutdown(SHUT_WR);
              exit();
            }
            if ( $data > 5 && $data < 6 ) { # AS400 (5.0)
              # Xormon::log("Connection from $client_address:$client_port exit for not supported protocol (AS400) ,$data,");
              # Xormon::error("Connection from $client_address:$client_port exit for not supported protocol (AS400) ,$data,");
              Xormon::log("Connection from $client_address:$client_port  protocol (AS400) ,$data,");
              # Xormon::error("Connection from $client_address:$client_port protocol (AS400) ,$data,");
              my $prot_version = $data;
              chomp($prot_version);
              $prot_version = substr( $prot_version, 0, 3 );
              print $client "$prot_version\n"; # return the protocol
              Xormon::log("Return protocol: $client_address:$client_port protocol:$prot_version,");
              #if ($client_address eq "10.22.33.105") {
              # $client->shutdown(SHUT_WR);
              # exit();
              #}
            }
            # if you want to stop one client
            #if ($client_address eq "10.22.33.105") {
            #  $client->shutdown(SHUT_WR);
            #  exit();
            #}

            my $xormon = Xormon->new("$xormon_address", $xormon_token, 'linux');

            if ($data >= 6) { # WIN agent (6.3)
              print $client "$data\n"; # return the protocol
               Xormon::log("Return protocol: $client_address:$client_port protocol:$data,");
            }
            if (defined $perf_line and $perf_line ne "") {
              Xormon::log("ATT immediate line : $client_address:$client_port protocol:$data,"); 
              my $saved = &process_line($xormon, $perf_line, $client_address);
              if (defined $saved) {
                    $client->send("$saved\n");
                    Xormon::log("\$saved ,$saved,");
              }
              else {
                 Xormon::log("\$saved is not defined"); 
              }
            }

            # use alert here to be sure it does not hang
            eval {
              my $act_time = localtime();
              my $timeout_glob = $timeout;
              local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM: after : $timeout_glob seconds - client: $client_address ($client_port)"; };
              alarm($timeout);
              # my $read_rec = 0;
              $read_rec = 0;

              while(<$client>)
              {
                if ( $read_rec > $transfer_rows_lim_per_alert ) {
                  Xormon::log("while->last: $read_rec > $transfer_rows_lim_per_alert client: $client_address ($client_port)");
                  Xormon::error("while->last: $read_rec > $transfer_rows_lim_per_alert client: $client_address ($client_port)");
                  last;
                  # set new alarm for comming data to let proccess even big amount of stored data on the client
                  $read_rec = 0;
                  alarm($timeout);
                  $timeout_glob = $timeout_glob + $timeout;
                }
                my $saved = &process_line($xormon, $_, $client_address,$data); #data is a protocol
                if (defined $saved) {
                    $client->send("$saved\n");
                }
                $read_rec++;
                $last_but_one_line = $_;
              }

              alarm(0);
            };
            alarm(0);

            chomp($@);
            if ($@) {
              if ( $@ =~ m/died in SIG ALRM/ ) {
                Xormon::log( "Client connection timed out after : $timeout seconds - client: $client_address ($client_port) " . __FILE__ . ":" . __LINE__ );
                Xormon::error( "Client connection timed out after : $timeout seconds - client: $client_address ($client_port) " . __FILE__ . ":" . __LINE__ );
              }
               else {
                Xormon::log("timeout result - client: $client_address ($client_port) $@");
                 Xormon::error("timeout result - client: $client_address ($client_port) $@");
              }
            }

            $client->shutdown(SHUT_WR);

            if ($nmon_lines > 0) {
              Xormon::log( "NMON connection finished & $nmon_lines NMON lines left out.");
              exit();
            }

            # exit() if ($type eq "???" ); # $type is not known

            # possible windows data
            if ($windows_data_dir_name) {
              Xormon::log("WIN new data in $windows_data_dir_name");
              $ENV{INPUTDIR} = $pwd;
              $ENV{WIN_DATA} = $windows_data_dir_name;
              $ENV{WIN_CLIENT_ADDRESS} = $client_address;
              require "$pwd/bin/hyp2rrd.pl";
              #eval "hyp2rrd.pl";
              hyp2rrd_main ($xormon);
                if ($@) {
                my $ret = $@;
                chomp($ret);
                Xormon::log("hyp2rrd failed when called: $ret ");
              }
              exit();
            }
      
            # save architecture
            my @architecture = ();
        
            # my $linux_arch_conf = "$TMPDIR/LINUX_ARCH_CONF"; # is global
            if (! -d "$linux_arch_conf") { mkdir "$linux_arch_conf" || die "Cant create dir $linux_arch_conf: $@";};

            my $file_to_save_snd = "$linux_arch_conf/arch_$uuid.storable.snd";
            my $file_to_save_tmp = "$linux_arch_conf/arch_$uuid.storable.tmp";
            my $file_time_to_send= "$linux_arch_conf/time_to_send";
            my $hashref = "";

            # touch that file to start period after that arch & conf is sent
            if (!-f $file_time_to_send) {open my $fh, '>', $file_time_to_send || die "Cant create file $file_time_to_send: $@";};
             
            # get previous architecture if any
            if (-f $file_to_save_snd) {
              move $file_to_save_snd, $file_to_save_tmp;
              $hashref = Storable::retrieve("$file_to_save_tmp"); 
            }
            @architecture = @$hashref if $hashref;

            # add actual architecture
            push(@architecture, \%linux_arch) if %linux_arch;
            @architecture = (@architecture,@architecture_lan) if @architecture_lan;
            @architecture = (@architecture,@architecture_job) if @architecture_job;
            @architecture = (@architecture,@architecture_sea) if @architecture_sea;
            @architecture = (@architecture,@architecture_san) if @architecture_san;
            @architecture = (@architecture,@architecture_san_power) if @architecture_san_power;
            @architecture = (@architecture,@architecture_wlm) if @architecture_wlm;

            if ( Storable::store \@architecture, "$file_to_save_tmp" ) {
                # Xormon::log("343 data stored to $file_to_save_tmp\n");
				move $file_to_save_tmp, $file_to_save_snd || Xormon::error("Cant rename file $file_to_save_tmp to $file_to_save_snd");
            }
            else {
                Xormon::log("369 data NOT stored to $file_to_save_tmp\n");
            }

            # save conf if exists
            my $conf_file_to_save_snd = "$linux_arch_conf/conf_$uuid.snd";
            my $conf_file_to_save_tmp = "$linux_arch_conf/conf_$uuid.tmp";
            if (%conf && Xormon::save_as_json( "$conf_file_to_save_tmp", \%conf )) {
			    # Xormon::log("359 conf stored as json to $conf_file_to_save_tmp\n");
				move $conf_file_to_save_tmp, $conf_file_to_save_snd || Xormon::error("Cant rename file $conf_file_to_save_tmp to $conf_file_to_save_snd");
			}
			
            # it is necessary every XX minutes to collect architecture (& conf) from all linuxes and send it to DB, perf is send in every call
            my $mtime = "";
            $mtime = (stat($file_time_to_send))[9] if -f $file_time_to_send;
            if ($mtime && (time - $mtime > ($minutes * 60 ))) { 
              unlink $file_time_to_send; # signal for other agents to not send arch
              opendir( DIR, "$linux_arch_conf" ) || Xormon::log( "Cant read directory $linux_arch_conf " . __FILE__ . ":" . __LINE__ ) && exit();
              my @send_files = grep( /\.snd$/, readdir(DIR));
              closedir(DIR);
			  my $number_of_files = scalar @send_files; 
              Xormon::log( "388:files to send $number_of_files : \$linux_arch_conf is $linux_arch_conf\n");

              @architecture = ();
              my @arr_of_conf  = ();
               
              foreach (@send_files) {
                if ($_ =~ /arch_/) {
                  my $file = "$linux_arch_conf/".$_;
                  # Xormon::log("361 \$file is $file");
                  my $hashref = Storable::retrieve("$file") || Xormon::log( "Cant read file $file\n") && next;
                  unlink $file;
                  @architecture = (@architecture,@$hashref);
                }
              }
              #save architecture
			  my %dupl = ();
              if (@architecture) {
				# print Dumper ("339",@architecture);
				# remove duplicate arch items
				my $total = scalar @architecture;
				my @d_architecture = ();
                %dupl = ();
				my $dupl_count = 0;
                foreach (@architecture) {
                  if (!exists $dupl{$_->{item_id}}) {
                    push (@d_architecture,$_);
                    $dupl{$_->{item_id}} = 1;
                  }
				  else {
                    $dupl_count++;
				  }
                }
				Xormon::log("421:Removed $dupl_count arch duplicated items from total $total") if $dupl_count > 0;

                my $rc = $xormon->saveArchitectureAgent(\@d_architecture);
                # status code
                if ($rc eq "401") {
                  Xormon::error("line 426: Unauthorized (401)");
                  # return ();
                } elsif ($rc ne "201" ) {
                  Xormon::error("line 429 Error from $client_address \$rc $rc");
                  print Dumper ("429",\@architecture);
                  # return ();
                }
                else {
                  # print "434 seems to be ok \$rc $rc\n";
                }
              }
              # save configuration immediately in cycle
              foreach (@send_files) {
				if ($_ =~ /conf_/) {
				  my $file = "$linux_arch_conf/".$_;
				  # Xormon::log("441 \$file is $file");
                  my $hashref = Xormon::read_json("$file") || Xormon::log( "Cant read file $file\n") && next;
                  unlink $file;
                  my $rc = $xormon->saveConfJSON($hashref);
                  # status code
                  if ($rc eq "401") {
                    Xormon::error("line 447: Unauthorized");
                    # return ();
                  } elsif ($rc ne "201" ) {
                    Xormon::error("456 Error from $client_address \$rc $rc");
                    print Dumper ("456",$hashref);
                    # return ();
                  }
                }
              }
            }
            # send perf data to backend
        	if (%perf) {
	          # print Dumper ("459",\%perf);
			  # test if exists file with job data and is older than 10 minutes, then send it with other perf data
			  # get uuid to create perf file name
			  my  @key = ();
              @key = (keys %{$perf{"linux"}{"server"}});
              # print "@key\n";
              print "471,$key[0],\n";
              my $job_perf_file = "$TMPDIR/linux_perf_"."$key[0]".".storable";
			  # print "473,$job_perf_file\n";
			  if (-f $job_perf_file) {
				print "475 file exists $job_perf_file\n";
			    my $mtime = "";
				my $minutes = 5;
                $mtime = (stat($job_perf_file))[9] if -f $job_perf_file;
                if ($mtime && (time - $mtime > ($minutes * 60 ))) {
				  my $hashref = Storable::retrieve("$job_perf_file");
				  # print Dumper ("481",\$perf{"linux"}{"job"});
				  $perf{"linux"}{"job"} = $$hashref;
				  # print Dumper ("483",\$perf{"linux"}{"job"});
				  unlink $job_perf_file;
				  print "483 sent & unlinked $job_perf_file\n";
				}
			  }
			  else {
				print "489 file DOES not exist $job_perf_file\n";
              }

              my $rc = $xormon->saveJSON2(\%perf);

              # status code
              if ($rc eq "401") {
                Xormon::error("Unauthorized");
                # return ();
              } elsif ($rc ne "201" ) {
                Xormon::error("line 499: from $client_address \$rc $rc");
		        print Dumper ("499",\%perf);
                # return ();
              }
			  else {
                print "504 ------ perf data saved to DB for $client_address $client_port\n";
			  }
			}  

            exit();

        } else {
            $client->autoflush(1);    
        }
    } 
    $agent = 0;
}

$server->close();

sub process_line() {
  my $xormon     = shift;
  my $line       = shift;
  my $ip_address = shift;
  my $protocol   = shift;

  my @parts = split( '\|', $line );
  if ( !defined $parts[1] ) {
  }
  my @confs = split(':', $parts[0]);
  # Xormon::log("151 process_line \$ip_address $ip_address:$line,");
  # type
  my $type = $confs[0];

  # test NMON & refuse it
  # LINUX-RedHat:vm-jindra:0:1718920803:Fri Jun 21 00:00:03 2024 version 6.11-0::300::N:mem:::3865497.6
  if ( defined $confs[10] && $confs[10] eq "N") {
    $nmon_lines++;
    return $confs[3];
  }

    if ($type eq "Linux") {
        return &linux($xormon, $line, $ip_address);
    } elsif (substr($type, 4, 1) eq "-" && substr($type, 8, 1) eq "*") {
        return &power($xormon, $line, $ip_address, $protocol);
    } elsif ($type eq "Docker" || $type eq "Docker-container") {
        return &docker($xormon, $line, 'container');
    } elsif ($type eq "Docker-volume") {
        return &docker($xormon, $line, 'volume');
    } elsif ($type eq "HYPERV") {
        return &store_data_63($xormon, \$line);
    } elsif ($type =~ /Solaris|SOLARIS/) {
      return &solaris($xormon, $line, $ip_address);
    } else {
        Xormon::log( "520 unknown type my \$read_rec $read_rec $type\n"); # \$line $line\n");
		Xormon::error("520 unknown type $type\nline $line\n\$last_but_one_line $last_but_one_line\n");
        return ();
    }
}

sub process_limits {
  my ( $cmd, $limit, $ps_all_tmp ) = @_;
  my @ps_all   = @{$ps_all_tmp};
  my $ps_count = 0;
  foreach my $ps_line (@ps_all) {
    if ( $ps_line =~ m/$cmd/ && $ps_line !~ m/grep/ ) {
      $ps_count++;
    }
  }

  if ( $ps_count > $limit ) {
    return 1;
  }
  return 0;
}

sub solaris {
  my $xormon = shift;
  my $line = shift;
  my $ip_address = shift;
  my %perf;
  my %conf;
  my @architecture;
  my %counter_data;
  my %last_agent_counter_data = ();
  my $last_agent_counter_data_file;
  my $hashref = \%counter_data;

  my @parts = split('\|', $line);
  my @confs = split(':', $parts[0]);
  
  # Data from CDOM example
  # Solaris11:<ldom_name>:cdom/<uuid_cdom>/<host_id>:1718094721:Tue Jun 11 10:32:01 2024 version 7.00-0:
  
  # Data from LDOM under CDOM example 
  # <name>_ldom:<uuid_ldom>/<host_id>:active:256:412316860416:2.0:41966532:2.0:: mam

  # mem:::32898904:28391560:4507344:0:0:16366352.8125: mam
  # pgs:::0:0:32767:0.0::: mam
  # cpu:::0:1:0:0::: mam
  # san_l:b7a7d20e-dc35-e588-7984-4420fa45a8:c0:0:117000:0.0:17.5:::
  # san_tresp:b7a7d20e-dc35-e588-7984-4420fa45a8:c0:2.8::::::
  # san_l:b7a7d20e-dc35-e588-7984-4420fa45a8:c10:9083100:1395300:233.4:78.5:::
  # san_tresp:b7a7d20e-dc35-e588-7984-4420fa45a8:c10:1.0::::::
  # total/b7a7d20e-dc35-e588-7984-4420fa45a8:U:33554432:0.46:0.72:12652853:37.70:19.78:0.38:
  # system/b7a7d20e-dc35-e588-7984-4420fa45a8:U:33554432:0.32:0.50:11366541:33.87:17.70:U:
  # global/b7a7d20e-dc35-e588-7984-4420fa45a8:U:33554432:0.13:0.21:1286312:3.83:2.07:0.38:
  # netstat:b7a7d20e-dc35-e588-7984-4420fa45a8/865d12c8:net8:0:0:0:0:::
  # netstat:b7a7d20e-dc35-e588-7984-4420fa45a8/865d12c8:net7:0:0:0:0:::
	# path_sol:5000c500179bc65b/c0t5000C500179BC65Bd0s2/SEAGATE/ST914603SSUN146G/0868/unknowntype/no/shortest-path:0/:no/:OK/:::::
  #print "=======PARTS=============\n";
  #print "@parts\n";
  #print "=======CONFS=============\n";
  #print "@confs\n";

  my $type_of_solaris = $confs[0];
  my $ldom_name       = $confs[1];
  # time
  my $time            = $confs[3];
  my $type_of_ldom    = "";
  my $id              = "";
  my $host_id         = "";
  my $uuid            = "";
  if ($confs[2]=~ /\//){
    ($type_of_ldom,$id,$host_id) = split ("/",$confs[2]);
  }

  if ($type_of_ldom eq "no_ldom"){
    $uuid = "Solaris_" . "$id" . "_osagent";
  }
  elsif ($id eq "-") { # when Solaris does not provide uuid info
    my $hash_key = Xormon::create_hash_key("Solaris".$ldom_name); $hash_key = lc($hash_key);
    $id          = $hash_key;
    $uuid = "Solaris_" . "$id" . "_osagent";
  }
  else{
    $uuid = "Solaris_" . $id;
  }

  my @metrics = split(':', $parts[1]);
	shift @metrics;
	shift @metrics;
	shift @metrics;
	shift @metrics; # so start with the first atom info most probably with 'mem'
  
  # CPU and MEM from CDOM (this data is only provided by the CDOM)
  # CDOM provided data about LDOMs too
  my %cpu = ();
  my %mem = ();
  my %perf_test = ();

  # Sometime LDOM does not have data about CPU/MEM, so we have to modify the uuid
  my (@index_arr) = grep { $metrics[$_] =~ /_ldom/ } (0 .. @metrics-1);

  if (!@index_arr) {
    $uuid = "Solaris_" . "$id" . "_osagent";
  }

	Xormon::log( "201 \$uuid $uuid from $ip_address ($type_of_solaris,$ldom_name,$type_of_ldom)\n");

  # conf data
  my @split1 = split(/ version /, $line);
  my @split2 = split(/:/, $split1[1]); # not /:|:/ just /:/
  # version of lpar2rrd-agent.pl 
  if (defined $split2[0]) {
    $conf{$uuid}{version} = $split2[0];
  }

  $last_agent_counter_data_file = "$TMPDIR/"."$uuid".".storable";
  $hashref = retrieve($last_agent_counter_data_file) if -f $last_agent_counter_data_file;
  %counter_data = %{$hashref};
  
  # LDOM data
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
		  next;
	  }
    # <name>_ldom:a7600e-dc25-e478-0284-85661152dfb7/865504c8:active:64:34359738368:1.3:41973190:1.3::
	  # LDOM arch
    my $label_ldom_name = $metrics[$index+0];
    $label_ldom_name    =~ s/_ldom//g;
    my ($uuid_ldom,$host_id) = split ("/", $metrics[$index+1]);
    $uuid_ldom       = "Solaris_" . "$uuid_ldom";
    my $type_ldom = "";
    if ($label_ldom_name ne "$ldom_name") { # only LDOMs to arch
      $type_ldom = "ldom";
      my %ldom_arch = (
        "item_id" => $uuid_ldom,
        "label" => $label_ldom_name,
        "hw_type" => "solaris",
        "subsystem" => "ldom",
        "parents" => [$uuid] # CDOM uuid
      );
      push(@architecture, \%ldom_arch);
    }
    else { # Architecture for CDOM
      $type_ldom = "cdom";
      my %cdom_arch = (
        "item_id" => $uuid,
        "label" => $ldom_name,
        "hw_type" => "solaris",
        "subsystem" => "cdom",
        "parents" => []
      );
      push(@architecture, \%cdom_arch);
    }
    if (Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+5]) and Xormon::isdigit($metrics[$index+4]) ) {
      $perf{'solaris'}{$type_ldom}{$uuid_ldom}{$time}{'virtual_cpu'}       = $metrics[$index+3];
      $perf{'solaris'}{$type_ldom}{$uuid_ldom}{$time}{'cpu_utilization'}   = $metrics[$index+5];
      $perf{'solaris'}{$type_ldom}{$uuid_ldom}{$time}{'memory_allocated'}  = $metrics[$index+4];
	  }
	  else {
       Xormon::error("Solaris $uuid($ldom_name) $time agent cpu/mem($type_ldom data) not number data $metrics[$index+3] $metrics[$index+5] $metrics[$index+4]");
	  }
	}

  # CPU OS
  my %cpu_os = ();
  (@index_arr) = grep { $metrics[$_] eq 'cpu' } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
		  next;
	  }
    my $uuid_os_agent = "Solaris_" . "$id" . "_osagent";
    my %os_agent_arch = (
      "item_id" => $uuid_os_agent,
      "label" => $ldom_name,
      "hw_type" => "solaris",
      "subsystem" => "osagent",
      "parents" => []
    );
    push(@architecture, \%os_agent_arch);
	  if (Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+5])) {
      %cpu_os = (
        "cpu_idle" => (100 - ($metrics[$index+3] + $metrics[$index+4] + $metrics[$index+5])),
        "cpu_entitled" => $metrics[$index+3],
        "cpu_system" => $metrics[$index+4],
        "cpu_user" => $metrics[$index+5],
        "cpu_wait" => $metrics[$index+6]
      );
	  }
	  else {
        Xormon::error("Solaris $uuid($ldom_name) $time agent cpu not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5]");
	  }
	}
  #print Dumper \%cpu_os;

  # CPU QUEUE
  my %cpu_queue = ();
  (@index_arr) = grep { $metrics[$_] eq 'queue_cpu' } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
		  next;
	  }
	  if (Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+5])) {
      %cpu_queue = (
        "load" => $metrics[$index+3],
        "virtual_p" => $metrics[$index+4],
        "blocked_p" => $metrics[$index+5]
      );
	  }
	  else {
        Xormon::error("Solaris $uuid($ldom_name) $time agent cpu queue not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5]");
	  }
	}
  #print Dumper \%cpu_queue;

  # MEM
  my %mem_os = ();
  (@index_arr) = grep { $metrics[$_] eq 'mem' } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
		  next;
	  }
    #print "$index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4],$metrics[$index+5],$metrics[$index+8]\n";
    if (Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+5]) and Xormon::isdigit($metrics[$index+8])) {
      %mem_os = (
        "mem_size" => Xormon::returnBytes($metrics[$index+3], "KB"),
        "mem_used" => Xormon::returnBytes($metrics[$index+4], "KB") - Xormon::returnBytes($metrics[$index+8], "KB"),
        "mem_free" => Xormon::returnBytes($metrics[$index+5], "KB"),
        "mem_in_use_client" => Xormon::returnBytes($metrics[$index+8], "KB"),
      );
	  }
	  else {
        Xormon::error("Solaris $uuid($ldom_name) $time agent mem not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+8]");
	  }
  }
  #print Dumper \%mem_os;

  # Solaris pool
  (@index_arr) = grep { $metrics[$_] eq 'pool-sol' } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
		  next;
	  }
    # architecture;
    my $pool_uuid = "Solaris_" . "$id" . "_$metrics[$index+1]";
    my %sol_pool_arch = (
      "item_id" => $pool_uuid,
      "label" => $metrics[$index+1],
      "hw_type" => "solaris",
      "subsystem" => "pool",
      "parents" => [$uuid]
    );
    push(@architecture, \%sol_pool_arch);
	  if (Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+5])) {
      $perf{'solaris'}{'pool'}{$pool_uuid}{$time}{'size'}  = $metrics[$index+3]; 
      $perf{'solaris'}{'pool'}{$pool_uuid}{$time}{'used'}  = $metrics[$index+4]; 
      $perf{'solaris'}{'pool'}{$pool_uuid}{$time}{'load'}  = $metrics[$index+5]; 
	  }
	  else {
        Xormon::error("Solaris $uuid($ldom_name) $time agent pool not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5]");
	  }
	}
  #print Dumper \%cpu_queue;

  # SAN from LDOM
  (@index_arr) = grep { $metrics[$_] eq 'san_l' } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
		  next;
	  }
    # san_l:b7a7d20e-dc35-e588-7984-4420fa45a8:c0:0:117000:0.0:17.5:::
    my $san_ldom_uuid = "";
    if ($type_of_ldom eq "no_ldom"){
      $san_ldom_uuid = "Solaris_" . "$id" . "_$metrics[$index+2]";
    }
    else{
      $san_ldom_uuid = "Solaris_" . "$metrics[$index+1]" . "_$metrics[$index+2]";
    }
    # architecture;
    my %san_ldom_arch = (
      "item_id" => $san_ldom_uuid,
      "label" => $metrics[$index+2],
      "hw_type" => "solaris",
      "subsystem" => "sanldom",
      "parents" => [$uuid]
    );
    push(@architecture, \%san_ldom_arch);
	  if (Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+5]) and Xormon::isdigit($metrics[$index+6])) {
      $perf{'solaris'}{'sanldom'}{$san_ldom_uuid}{$time}{'bytes_read'}     = $metrics[$index+3]; 
      $perf{'solaris'}{'sanldom'}{$san_ldom_uuid}{$time}{'bytes_write'}    = $metrics[$index+4]; 
      $perf{'solaris'}{'sanldom'}{$san_ldom_uuid}{$time}{'io_read'}        = $metrics[$index+5]; 
      $perf{'solaris'}{'sanldom'}{$san_ldom_uuid}{$time}{'io_write'}       = $metrics[$index+6]; 
	  }
	  else {
      Xormon::error("Solaris $uuid($ldom_name) $time agent san from ldom not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5]");
	  }
	}
  
  # SAN response time from LDOM
  my %san_resp_ldom = ();
  (@index_arr) = grep { $metrics[$_] eq 'san_tresp' } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
		  next;
	  }
    my $san_ldom_uuid = "";
    if ($type_of_ldom eq "no_ldom"){
      $san_ldom_uuid = "Solaris_" . "$id" . "_$metrics[$index+2]";
    }
    else{
      $san_ldom_uuid = "Solaris_" . "$metrics[$index+1]" . "_$metrics[$index+2]";
    }
	  if (Xormon::isdigit($metrics[$index+3])) {
      $perf{'solaris'}{'sanldom'}{$san_ldom_uuid}{$time}{'resp_time'} = $metrics[$index+3]; 
	  }
	  else {
      Xormon::error("Solaris $uuid($ldom_name) $time agent san resp from ldom not number data $metrics[$index+3]");
	  }
	}

  # NETSTAT from LDOM
  my %net_ldom = ();
  (@index_arr) = grep { $metrics[$_] eq 'netstat' } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
		  next;
	  }
    # netstat:c7b5a20e-dc25-e478-8404-8631a852dfb7/865d12c8:net8:0:0:0:0:::
    my ($net_uuid,$host_id) = split ("/",$metrics[$index+1]);
    my $net_ldom_uuid   = "Solaris_" . "$net_uuid" . "_$metrics[$index+2]";
    my $net_parent_uuid = "Solaris_" . "$net_uuid";
    # architecture;
    my %net_ldom_arch = (
      "item_id" => $net_ldom_uuid,
      "label" => $metrics[$index+2],
      "hw_type" => "solaris",
      "subsystem" => "netldom",
      "parents" => [$net_parent_uuid]
    );
    push(@architecture, \%net_ldom_arch);
    my $net_uuid_perf = "Solaris_" . "$net_uuid" . "_$metrics[$index+2]";
    if (Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+5]) and Xormon::isdigit($metrics[$index+6])) {
      $perf{'solaris'}{'netldom'}{$net_uuid_perf}{$time}{'ipacket'}      = $metrics[$index+3];
      $perf{'solaris'}{'netldom'}{$net_uuid_perf}{$time}{'opacket'}      = $metrics[$index+5]; 
      $perf{'solaris'}{'netldom'}{$net_uuid_perf}{$time}{'read_bytes'}   = $metrics[$index+4]; 
      $perf{'solaris'}{'netldom'}{$net_uuid_perf}{$time}{'write_bytes'}  = $metrics[$index+6]; 
	  }
	  else {
      Xormon::error("Solaris $uuid($ldom_name) $time agent netstat from ldom not number data $metrics[$index+3],$metrics[$index+5],$metrics[$index+4],$metrics[$index+6]");
	  }
	}

  # VNETSTAT from LDOM
  my %vnet_ldom = ();
  (@index_arr) = grep { $metrics[$_] eq 'vnetstat' } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
		  next;
	  }
    # vnetstat:519a073a-5292-47f8-ff23453ac212:ldoms-vsw4.vport1:23:1563:47:342147:::
    my ($vnet_uuid,$host_id) = split ("/",$metrics[$index+1]);
    my $vnet_ldom_uuid   = "Solaris_" . "$vnet_uuid" . "_$metrics[$index+2]";
    my $vnet_parent_uuid = "Solaris_" . "$vnet_uuid";
    # architecture;
    my %vnet_ldom_arch = (
      "item_id" => $vnet_ldom_uuid,
      "label" => $metrics[$index+2],
      "hw_type" => "solaris",
      "subsystem" => "vnetldom",
      "parents" => [$vnet_parent_uuid]
    );
    push(@architecture, \%vnet_ldom_arch);
    my $vnet_uuid_perf = "Solaris_" . "$vnet_uuid" . "_$metrics[$index+2]";
    if (Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+5]) and Xormon::isdigit($metrics[$index+6])) {
      $perf{'solaris'}{'vnetldom'}{$vnet_uuid_perf}{$time}{'ipacket'}      = $metrics[$index+3];
      $perf{'solaris'}{'vnetldom'}{$vnet_uuid_perf}{$time}{'opacket'}      = $metrics[$index+5]; 
      $perf{'solaris'}{'vnetldom'}{$vnet_uuid_perf}{$time}{'read_bytes'}   = $metrics[$index+4]; 
      $perf{'solaris'}{'vnetldom'}{$vnet_uuid_perf}{$time}{'write_bytes'}  = $metrics[$index+6]; 
	  }
	  else {
      Xormon::error("Solaris $uuid($ldom_name) $time agent vnetstat from ldom not number data $metrics[$index+3],$metrics[$index+5],$metrics[$index+4],$metrics[$index+6]");
	  }
	}

  # PGS
  my %pgs = ();
  my $INPUT_LIMIT_PGS     = 500000;         # 500MB/sec
  (@index_arr) = grep { $metrics[$_] eq 'pgs' } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
		  next;
	  }
	  if (Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+5]) and Xormon::isdigit($metrics[$index+6])) {
      if (exists $counter_data{$uuid}{time}) {
        if (($time - $counter_data{$uuid}{time}) < $max_data_delay) { # last data not older x minutes ?
          # pg1 & pg2 are in 4096 B pages
          %pgs = (
            "pgs1" => min(($metrics[$index+3] - $counter_data{$uuid}{pgs1})/60,$INPUT_LIMIT_PGS ) * 4096, # page in
            "pgs2" => min(($metrics[$index+4] - $counter_data{$uuid}{pgs2})/60,$INPUT_LIMIT_PGS ) * 4096, # page out
			      "pgs3" => $metrics[$index+5], # paging space
            "pgs4" => $metrics[$index+6]  # paging in %
          );
			    $pgs{pgs1} = 0 if $pgs{pgs1} < 0;
			    $pgs{pgs2} = 0 if $pgs{pgs2} < 0;
        }
        else {
          print "probably longer interval actual: $time last ". $counter_data{$uuid}{time}." for Linux pgs $uuid\n";
		    }
	    }
      else {
        print "probably 1st write to Solaris pgs $uuid($ldom_name)\n";
	    }
      # save for next run
      $counter_data{$uuid}{pgs1} = $metrics[$index+3];
	    $counter_data{$uuid}{pgs2} = $metrics[$index+4];
      $counter_data{$uuid}{time} = $time;
      #print Dumper %counter_data;
    }
    else {
      Xormon::error("Solaris $uuid($ldom_name) $time agent pgs not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6]");
    }
  }

  # Solaris zones
  # total/9303b6f8-fcfe-438a-8ab3-98cfff840ff3:U:8388608:0.54:6.83:3210549:38.27:27.88:0.00:
  (@index_arr) = grep { $metrics[$_] =~ /(.*)\// } (0 .. @metrics-1);
  foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  next;
	  }
    my ($zone_name) = split ("/",$metrics[$index+0]);
    # architecture;
    my $zone_uuid = "Solaris_" . "$id" . "_$zone_name";
    my %zone_arch = (
      "item_id" => $zone_uuid,
      "label" => $zone_name,
      "hw_type" => "solaris",
      "subsystem" => "zone",
      "parents" => [$uuid]
    );
    push(@architecture, \%zone_arch);
    my $total_memory      = "";
    my $used_memory       = "";
    my $cpu_used_in_cores = "";
    my $cpu_used_in_perc  = "";
    my $net_used_in_perc  = "";
	  if (Xormon::isdigit($metrics[$index+2]) and Xormon::isdigit($metrics[$index+3]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+4]) and Xormon::isdigit($metrics[$index+5]) and Xormon::isdigit($metrics[$index+6]) and Xormon::isdigit($metrics[$index+7]) ) {
      if ($metrics[$index+8] eq "U"){
        $metrics[$index+8] = "";
      }
      $perf{'solaris'}{'zone'}{$zone_uuid}{$time}{'total_memory'}              =  Xormon::returnBytes($metrics[$index+2], "KB"); 
      $perf{'solaris'}{'zone'}{$zone_uuid}{$time}{'used_memory'}               =  Xormon::returnBytes($metrics[$index+5], "KB");
      $perf{'solaris'}{'zone'}{$zone_uuid}{$time}{'cpu_used_in_cores'}         = $metrics[$index+3]; 
      $perf{'solaris'}{'zone'}{$zone_uuid}{$time}{'cpu_used_in_perc'}          = $metrics[$index+4]; 
      $perf{'solaris'}{'zone'}{$zone_uuid}{$time}{'net_used_in_perc'}          = $metrics[$index+8];    
      $perf{'solaris'}{'zone'}{$zone_uuid}{$time}{'memory_used_in_perc'}       = $metrics[$index+6];
      $perf{'solaris'}{'zone'}{$zone_uuid}{$time}{'virt_memory_used_in_perc'}  = $metrics[$index+7];
	  }
	  else {
        Xormon::error("Solaris $uuid($ldom_name) $time agent zone($zone_name) not number data  $metrics[$index+2] $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6] $metrics[$index+7] $metrics[$index+8]");
	  }
	}
  
  # Solaris multipath
	# path_sol:5000c500179bc65b/c0t5000C500179BC65Bd0s2/SEAGATE/ST914603SSUN146G/0868/unknowntype/no/shortest-path:0/:no/:OK/:::::
  my %multi_sol = ();
  (@index_arr) = grep { $metrics[$_] eq 'path_sol' } (0 .. @metrics-1); # get all indexes
	foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
		  next;
	  }
	  #print "437 multipath_solaris \$index $index\n";
    my ($disk_id,$disk_alias,$vendor,$product,$revision,$name_type,$asymmetric,$curr_load_balance) = split ('/',$metrics[$index+1]);
    my $string1 = "$metrics[$index+2]";
    my $string2 = "$metrics[$index+3]";
    my $string3 = "$metrics[$index+4]";
    $string1 =~ s|/|,ID:|g;
    $string2 =~ s|/|,Disabled:|g;
    $string3 =~ s|/|,Path:|g;
    $string1 =~ s|,ID:$||g;
    $string2 =~ s|,Disabled:$||g;
    $string3 =~ s|,Path:$||g;
    $multi_sol{$index} = {
      'disk_name'        => $disk_id,
      'disk_alias'       => $disk_alias,
      'path_properties'  => "$vendor," . "$product," . "$revision," . "$name_type," . "$asymmetric," . "$curr_load_balance",
      'path_info'        => "ID:$string1" . " Disabled:$string2" . " Path:$string3\n",
    };
  }
	if (%multi_sol) {
    $conf{"$uuid"}{PATHSOL} = encode_json(\%multi_sol);
	  #print Dumper \%conf;
  }

  # FS:/var:rpool/ROOT/be-name/var:133.88:2.95:87.67:4:::
  # FS:/system/volatile:swap:4.02:0.01:4.01:1:::
  my %fs = ();
  (@index_arr) = grep { $metrics[$_] eq 'FS' } (0 .. @metrics-1); # get all indexes
	foreach my $index (@index_arr) {
	  if ($index % 9 > 0) { # skip possible coincidence
		  Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
		  next;
	  }
    $fs{$index} = {
      'mounted_on'  => $metrics[$index+1],
      'label'       => $metrics[$index+2],
      'total'       => $metrics[$index+3],
      'used'        => $metrics[$index+4],
      'available'   => $metrics[$index+5],
      'usage'       => $metrics[$index+6]
     };
  }
	if (%fs) {
    $conf{"$uuid"}{FS_SOL} = encode_json(\%fs);
  }
  
  my $os_agent_uuid = "Solaris_" . "$id" . "_osagent";
  my %solaris_ldom_data = (%cpu_os, %mem_os, %pgs, %cpu_queue);
  $perf{'solaris'}{'osagent'}{$os_agent_uuid}{$time} = (\%solaris_ldom_data);

  #print "====ARCH=====\n";
  #print Dumper \@architecture;
  #print "====PERF=====\n";
  #print Dumper %perf;
  #print "====CONF=====\n";
  #print Dumper %conf;
  #save architecture
  if (@architecture) {
    # print Dumper ("536",@architecture);
    my $rc = $xormon->saveArchitectureAgent(\@architecture);
    # status code
    if ($rc eq "401") {
      Xormon::error("Solaris architecture: Unauthorized");
      # return ();
    } elsif ($rc ne "201" ) {
      Xormon::error("Solaris error from $ip_address \$rc $rc");
      print Dumper ("Solaris-architecture-error",@architecture);
      # return ();
    }
    else {
      print "Solaris architecture seems to be ok \$rc $rc\n";
    }
  }

  #save configuration
  if (%conf) {
    # print Dumper ("554",\%conf);
    my $rc = $xormon->saveConfJSON(\%conf);
    # status code
    if ($rc eq "401") {
      Xormon::error("Solaris conf: Unauthorized");
      # return ();
    } elsif ($rc ne "201" ) {
      Xormon::error("Solaris conf from $ip_address \$rc $rc");
      print Dumper ("Solaris-conf-error",\%conf);
      # return ();
    }
    else {
       print "Solaris conf seems to be ok \$rc $rc\n";
    }
  }

  # send data to backend
  if (%perf) {
    # print Dumper ("571",\%perf);
    my $rc = $xormon->saveJSON2(\%perf);
    # status code
    if ($rc eq "401") {
      Xormon::error("Solaris perf: Unauthorized");
      return ();
    } elsif ($rc ne "201" ) {
      Xormon::error("Solaris perf error from $ip_address \$rc $rc");
      print Dumper ("SOLARIS-PERF-ERROR",\%perf);
      return ();
    }
    else {
      # print "584 seems to be ok \$rc $rc\n";
      if (%counter_data) {
        if (store \%counter_data, "$last_agent_counter_data_file") {
            print "Solaris perf data stored to $last_agent_counter_data_file\n";
         }
         else {
          print "Solaris perf data NOT stored to $last_agent_counter_data_file\n";
         }
      }
      return $time;
    }
  }

}

sub linux() {
  my $xormon     = shift;
  my $line       = shift;
  my $ip_address = shift;

  # my %perf; # is global
  # my %conf; # is global
  # my @architecture;
  my %last_agent_counter_data = ();
  my $last_agent_counter_data_file;
  my %counter_data;
  my $hashref = \%counter_data;

  my $INPUT_LIMIT_PGS     = 500000;         # 500MB/sec
  my $INPUT_LIMIT_LAN     = 12500000000;    # 125Gbits/sec
  my $INPUT_LIMIT_PCK_LAN = 125000000;      # Packets
                                            # Xormon::log( "165 \$line $line\n");

  # Linux:vm-jindra:FF884D56-C6AD-8B59-80DA-337B507DA56B:1699601401:Fri Nov 10 08:30:01 2023 version 6.11-0:|::::
  # mem:::3865536:3064884:800652:-1:462388:2602496:pgs:::9832:36531:3968.99609375:2.4:::lan:ens192:10.22.11.101:5283961007:73939659438:14076119:26233974:::
  # cpu:::0:0:2:0:::st-cpu:::0::::::disk-total:0.00:0.00:0.00:6.50:223095.47:5.24:::queue_cpu:::0.0:4:0::::
  # CPUTOP:412:root:[btrfs-transacti]:2178:2:0:0::linux_cpu:4:2296.878:1:1::::

  my @parts = split( '\|', $line );
  my @confs = split( ':',  $parts[0] );

  # uuid
  my $id = $confs[2];
  if ( !$id ) {
    $id = $confs[1];
  }

  #uuid and ng_uuid will always be the same
  #unless it is selfmonitoring in docker
  $uuid = "Linux_" . $id;    # is global
  my $ng_uuid = "Linux_" . $id;

  if ( $ip_address eq '127.0.0.1' or $ip_address eq 'localhost' ) {   #for the sake of Brian Frank triplecheck that this is not changed 
    $ip_address = '127.0.0.1';
    if ( defined $ENV{DOCKER} and $ENV{DOCKER} eq "1" ) {
      $ng_uuid = 'Linux_selfmonitoring';
    }
  }

  # Xormon::log( "201 \$uuid $uuid from $ip_address\n");

  # get previous conf if exists
  my $conf_file_to_save_snd = "$linux_arch_conf/conf_$uuid.snd";
  my $conf_file_to_save_tmp = "$linux_arch_conf/conf_$uuid.tmp";
  my $hashref_conf          = "";

  if ( -f $conf_file_to_save_snd ) {
    move $conf_file_to_save_snd, $conf_file_to_save_tmp;
    $hashref_conf = Xormon::read_json("$conf_file_to_save_tmp") || Xormon::log("Cant read file $conf_file_to_save_tmp\n");
  }
  %conf = %$hashref_conf if $hashref_conf;

  my $agent_uuid;
  if ($first_mapping) {
    $first_mapping = 0;
    $mapping       = $xormon->linuxAgentMapping($id);

    # sometimes it is neccessary to try big endian uuid format
    if ( !defined $mapping->{item_id} ) {
      my $id_be = "";
      if ( length($id) == 36 ) {
        $id_be = uuid_big_endian_format( $id, '-' );

        # print "332 get big endian uuid for $id is $id_be\n";
        $mapping = $xormon->linuxAgentMapping("$id_be");
      }
    }
  }
  $agent_uuid = $mapping->{item_id} if defined $mapping->{item_id};

  $last_agent_counter_data_file = "$TMPDIR/" . "$uuid" . ".storable";
  $hashref                      = Storable::retrieve($last_agent_counter_data_file) if -f $last_agent_counter_data_file;
  %counter_data                 = %{$hashref};

  #print "last \%counter_data\n";
  #print Dumper \%counter_data;

  # label
  my $label = $confs[1];

  # time
  my $time = $confs[3];

  # conf data
  my @split1 = split( / version /, $line );
  my @split2 = split( /:/,         $split1[1] );    # not /:|:/ just /:/
  if ( defined $split2[0] ) {
    $conf{$ng_uuid}{version} = $split2[0];
  }

  $conf{$ng_uuid}{ip_address} = $ip_address;

  my @metrics = split( ':', $parts[1] );
  shift @metrics;
  shift @metrics;
  shift @metrics;
  shift @metrics;                                   # so start with the first atom info most probably with 'mem'

  # linux_cpu:4:2296.878:1:1::::
  # cpu_core util [alias stog2] = (cpusy + cpuus + cpuwa) * cpu_count / 100
  # see below CPU part
  my $logical_processors = 0;
  my (@index_arr) = grep { $metrics[$_] eq 'linux_cpu' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                         # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 1 ] )) {
	  $logical_processors = $metrics[ $index + 1 ];
    }
    else {
      Xormon::error("Linux $uuid $time agent cpu not number data $metrics[$index+0] $metrics[$index+1]");
    }
  }

  # CPU
  my %cpu = ();
  (@index_arr) = grep { $metrics[$_] eq 'cpu' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                         # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) ) {
      %cpu = (
        "cpu_idle" => ( 100 - ( $metrics[ $index + 4 ] + $metrics[ $index + 5 ] + $metrics[ $index + 6 ] ) ),
        "cpu_sys"  => $metrics[ $index + 4 ],
        "cpu_usr"  => $metrics[ $index + 5 ],
        "cpu_wait" => $metrics[ $index + 6 ],
        "cpu_util" => ( $metrics[ $index + 4 ] + $metrics[ $index + 5 ] ),
		"cpu_core" => (( $metrics[ $index + 4 ] + $metrics[ $index + 5 ] + $metrics[ $index + 6 ] ) * $logical_processors /100)
      );
    }
    else {
      Xormon::error("Linux $uuid $time agent cpu not number data $metrics[$index+4] $metrics[$index+5] $metrics[$index+6]");
    }
  }

  # CPU Stolen st-cpu:::0::::::
  (@index_arr) = grep { $metrics[$_] eq 'st-cpu' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) ) {

      # $metrics[$index+3] = 15; # for debug
      $cpu{"st_cpu"} = $metrics[ $index + 3 ];
    }
    else {
      Xormon::error("Linux $uuid $time agent st-cpu not number data $metrics[$index+3]");
    }
  }

  # MEM
  my %mem = ();
  (@index_arr) = grep { $metrics[$_] eq 'mem' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # rank: size, nuse, free, pin, in_use_work, in_use_clnt
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) and Xormon::isdigit( $metrics[ $index + 7 ] ) and Xormon::isdigit( $metrics[ $index + 8 ] ) ) {
      %mem = (
        "mem_total"    => Xormon::returnBytes( $metrics[ $index + 3 ], "kiB" ),
        "mem2"         => Xormon::returnBytes( $metrics[ $index + 4 ], "kiB" ),
        "mem_free"     => Xormon::returnBytes( $metrics[ $index + 5 ], "kiB" ),
        "mem4"         => Xormon::returnBytes( $metrics[ $index + 6 ], "kiB" ),
        "mem_used"     => Xormon::returnBytes( $metrics[ $index + 7 ], "kiB" ),
        "mem_fs_cache" => Xormon::returnBytes( $metrics[ $index + 8 ], "kiB" ),
        "mem_util"     => ( $metrics[ $index + 3 ] - $metrics[ $index + 7 ] ) / $metrics[ $index + 3 ] * 100
      );
    }
    else {
      Xormon::error("Linux $uuid $time agent mem not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6] $metrics[$index+7]");
    }
  }

  # PGS
  my %pgs = ();
  (@index_arr) = grep { $metrics[$_] eq 'pgs' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) ) {
      if ( exists $counter_data{$uuid}{time} ) {
        if ( ( $time - $counter_data{$uuid}{time} ) < $max_data_delay ) {    # last data not older x minutes ?
                                                                             # pg1 & pg2 are in 4096 B pages
          %pgs = (
            "pgs1" => min( ( $metrics[ $index + 3 ] - $counter_data{$uuid}{pgs1} ) / 60, $INPUT_LIMIT_PGS ) * 4096,
            "pgs2" => min( ( $metrics[ $index + 4 ] - $counter_data{$uuid}{pgs2} ) / 60, $INPUT_LIMIT_PGS ) * 4096,
            "pgs3" => $metrics[ $index + 5 ],
            "pgs4" => $metrics[ $index + 6 ]
          );
          $pgs{pgs1} = 0 if $pgs{pgs1} < 0;
          $pgs{pgs2} = 0 if $pgs{pgs2} < 0;
        }
        else {
          print "267 probably longer interval actual: $time last " . $counter_data{$uuid}{time} . " for Linux pgs $uuid\n";
        }
      }
      else {
        print "519 probably 1st write to Linux pgs $uuid\n";
      }

      # save for next run
      $counter_data{$uuid}{pgs1} = $metrics[ $index + 3 ];
      $counter_data{$uuid}{pgs2} = $metrics[ $index + 4 ];
      $counter_data{$uuid}{time} = $time;
      $conf{$ng_uuid}{pgs}       = $metrics[ $index + 6 ];
    }
    else {
      Xormon::error("Linux $uuid $time agent pgs not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6]");
    }
  }

  # queue_cpu
  #  "DS:load:GAUGE:$no_time:0:U"
  #  "DS:virtual_p:GAUGE:$no_time:0:U"
  #  "DS:blocked_p:GAUGE:$no_time:0:U"

  my %qca = ();
  (@index_arr) = grep { $metrics[$_] eq 'queue_cpu' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) ) {
      %qca = (
        "load_avrg"          => $metrics[ $index + 3 ],
        "logical_processors" => $metrics[ $index + 4 ],
        "blocked_processes"  => $metrics[ $index + 5 ],
      );
    }
    else {
      Xormon::error("Linux $uuid $time agent queue_cpu not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5]");
    }
  }

  # JOBS rss is in kBytes
  # data format ":CPUTOP:$pid:$user:$command_out:$time_1_sec:$time_difference:$rss:$vzs\:";
  #              :CPUTOP:109944:lpar2rrd:yes:3635:1792:608:107900::

  (@index_arr) = grep { $metrics[$_] eq 'CPUTOP' } ( 0 .. @metrics - 1 );    # get all indexes
  if (scalar @index_arr) {
    my $linux_perf_file = "$TMPDIR/"."linux_perf_$ng_uuid".".storable";
	# print "1347 test if file $linux_perf_file\n";
    if (-f $linux_perf_file) {
      my $hashref = Storable::retrieve("$linux_perf_file");
      # print Dumper ("1349",$hashref);
      $perf{"linux"}{"job"} = $$hashref;
	}
  }
  # (@index_arr) = grep { $metrics[$_] eq 'CPUTOP' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                  # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    # print "655 JOBS \$index $index \$metrics[$index] $metrics[$index] $metrics[$index+1]\n";

    if ( ! (Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) and Xormon::isdigit( $metrics[ $index + 7 ] ) ) ) {
      #Xormon::error("Linux $uuid $time agent job $metrics[$index+1] not number data $metrics[$index+4] $metrics[$index+5] $metrics[$index+6] $metrics[$index+7]");
      Xormon::error("Linux $uuid $time agent job $metrics[$index+1] not number data");
	  next;
    }

    my $uuid_job    = "$uuid" . "_job." . "$metrics[$index+1]";
    my $ng_uuid_job = "$ng_uuid" . "_job." . "$metrics[$index+1]";

    if ( !exists $job_arch_list{$ng_uuid_job} ) {
      $job_arch_list{$ng_uuid_job} = "1";

      # architecture;
      my %job_arch = (
        "item_id"   => $ng_uuid_job,
        "label"     => $metrics[ $index + 1 ],
        "hw_type"   => "linux",
        "subsystem" => "job",
        "parents"   => [$ng_uuid]
      );
      push( @architecture_job, \%job_arch );
    }

    # Configuration
    $conf{$ng_uuid_job}{pid}     = $metrics[ $index + 1 ];
    $conf{$ng_uuid_job}{user}    = $metrics[ $index + 2 ];
    $conf{$ng_uuid_job}{command} = $metrics[ $index + 3 ];

    # time_difference is divided 1/2 an hour because data is scanned every 30 minutes
    # rss from kB changed to B
    my %job_perf = (
        "time_1_sec"      => $metrics[ $index + 4 ],
        "time_difference" => ( $metrics[ $index + 5 ] / 1800 ),
        "rss"             => ( $metrics[ $index + 6 ] * 1024 ),
        "vzs"             => $metrics[ $index + 7 ]
    );
    $perf{'linux'}{'job'}{$ng_uuid_job}{$time} = ( \%job_perf ) if %job_perf;
    # print Dumper ("1389",\%perf);
  }
  # store jobs perf for later sending
  my $linux_perf_file = "$TMPDIR/"."linux_perf_$ng_uuid".".storable";
  #$last_agent_counter_data_file = "$TMPDIR/"."$uuid".".storable";
  if ($perf{'linux'}{'job'}) {
    if ( Storable::store \$perf{'linux'}{'job'}, "$linux_perf_file" ) {
	  # print Dumper ("1416 before delete from perf",\%perf);
      delete $perf{'linux'}{'job'};
      # print "1417 perf data stored to $linux_perf_file\n";
    }
    else {
      print "1420 perf data NOT stored to $linux_perf_file\n";
    }
  }

  # LAN
  # lan:ens192:10.22.11.101:6001783686:84639757273:16334884:30214499:::
  # lan:ens192:10.22.11.101:6001806237:84639789293:16335048:30214834:::
  # data are COUNTER type so it is necessary to make from them GAUGE type, it is difference 'this_value - last_value'
  # older agents do not send packets - last 2 data are empty (not numbers)

  my %total_lan_perf = ();
  (@index_arr) = grep { $metrics[$_] eq 'lan' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                               # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # print "369 \$index $index \$metrics[$index] $metrics[$index]\n";
    my $uuid_lan    = "$uuid" . "." . "$metrics[$index+1]";
    my $ng_uuid_lan = "$ng_uuid" . "." . "$metrics[$index+1]";

    # architecture;
    if ( !exists $lan_arch_list{$ng_uuid_lan} ) {
      $lan_arch_list{$ng_uuid_lan} = "1";

      my %lan_arch = (
        "item_id"   => $ng_uuid_lan,
        "label"     => $metrics[ $index + 1 ],
        "hw_type"   => "linux",
        "subsystem" => "lan",
        "parents"   => [$ng_uuid]
      );
      push( @architecture_lan, \%lan_arch );
    }

    # Configuration
    # $conf{$uuid_lan}{ip} = $metrics[$index+2]; # it is not here

    # "DS:recv_bytes:$ds_mode:$no_time:0:$INPUT_LIMIT_LAN"
    # "DS:trans_bytes:$ds_mode:$no_time:0:$INPUT_LIMIT_LAN"
    # "DS:recv_packets:$ds_mode:$no_time:0:$INPUT_LIMIT_PCK_LAN"
    # "DS:trans_packets:$ds_mode:$no_time:0:$INPUT_LIMIT_PCK_LAN"

    my %lan_perf;
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) ) {
      if ( exists $counter_data{$uuid_lan}{time} ) {
        if ( ( $time - $counter_data{$uuid_lan}{time} ) < $max_data_delay ) {    # last data not older x minutes ?
          %lan_perf = (
            "trans_bytes" => min( ( $metrics[ $index + 3 ] - $counter_data{$uuid_lan}{trans_bytes} ) / 60, $INPUT_LIMIT_LAN ),
            "recv_bytes"  => min( ( $metrics[ $index + 4 ] - $counter_data{$uuid_lan}{recv_bytes} ) / 60,  $INPUT_LIMIT_LAN )
          );
          $lan_perf{"trans_bytes"} = 0 if $lan_perf{"trans_bytes"} < 0;
          $lan_perf{"recv_bytes"}  = 0 if $lan_perf{"recv_bytes"} < 0;
          if (exists $total_lan_perf{"received_bytes"}) {
            $total_lan_perf{"received_bytes"} += $lan_perf{"recv_bytes"};
            $total_lan_perf{"send_bytes"} += $lan_perf{"trans_bytes"};
          }
          else {
            $total_lan_perf{"received_bytes"} = $lan_perf{"recv_bytes"};
            $total_lan_perf{"send_bytes"} = $lan_perf{"trans_bytes"};
          }
          if ( Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) ) {
            $lan_perf{"trans_packets"} = min( ( $metrics[ $index + 5 ] - $counter_data{$uuid_lan}{trans_packets} ) / 60, $INPUT_LIMIT_PCK_LAN );
            $lan_perf{"recv_packets"}  = min( ( $metrics[ $index + 6 ] - $counter_data{$uuid_lan}{recv_packets} ) / 60,  $INPUT_LIMIT_PCK_LAN );
            $lan_perf{trans_packets}   = 0 if $lan_perf{trans_packets} < 0;
            $lan_perf{recv_packets}    = 0 if $lan_perf{recv_packets} < 0;
          }
        }
        else {
          print "672 probably longer interval actual: $time last " . $counter_data{$uuid_lan}{time} . " for LAN $uuid_lan\n";
        }
      }
      else {
        print "676 probably 1st write to $uuid_lan\n";
      }
      $perf{'linux'}{'lan'}{$ng_uuid_lan}{$time} = ( \%lan_perf ) if %lan_perf;

      # save for next run
      $counter_data{$uuid_lan}{trans_bytes}   = $metrics[ $index + 3 ];
      $counter_data{$uuid_lan}{recv_bytes}    = $metrics[ $index + 4 ];
      $counter_data{$uuid_lan}{trans_packets} = $metrics[ $index + 5 ];
      $counter_data{$uuid_lan}{recv_packets}  = $metrics[ $index + 6 ];
      $counter_data{$uuid_lan}{time}          = $time;
    }
    else {
      Xormon::error("$uuid $time agent lan not number data $metrics[$index+3] $metrics[$index+4]");
    }
  }

  # IOPS, DATA, LATENCY
  # "DS:read_iops:GAUGE:$no_time:0:U"
  # "DS:read_data:GAUGE:$no_time:0:U"
  # "DS:read_latency:GAUGE:$no_time:0:U"
  # "DS:write_iops:GAUGE:$no_time:0:U"
  # "DS:write_data:GAUGE:$no_time:0:U"
  # "DS:write_latency:GAUGE:$no_time:0:U"
  # disk-total:0.00:0.00:0.00:1.70:66355.20:1.95:::
  # disk-total:0.00:0.00:0.00:6.32:272725.33:4.68:::
  my %disk_total_perf = ();
  (@index_arr) = grep { $metrics[$_] eq 'disk-total' } ( 0 .. @metrics - 1 );    # get all indexes, here should be only one
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                      # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # print "438 \$index $index \$metrics[$index] $metrics[$index],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4],$metrics[$index+5]\n";
    %disk_total_perf = (
      'read_iops'     => $metrics[ $index + 1 ],
      'read_data'     => $metrics[ $index + 2 ],
      'read_latency'  => $metrics[ $index + 3 ],
      'write_iops'    => $metrics[ $index + 4 ],
      'write_data'    => $metrics[ $index + 5 ],
      'write_latency' => $metrics[ $index + 6 ]
    );
	$conf{"$ng_uuid"}{"disk-total"} = $time;
  }

  # FS:/:/dev/hd4:1.00:0.17:0.83:18:::FS:/usr:/dev/hd2:4.00:1.91:2.09:48:::FS:/var:/dev/hd9var:3.00:0.93:2.07:31::: and so on
  # FS:/:/dev/sda3:95.12:76.42:17.91:82:::FS:/home:/dev/sda3:95.12:76.42:17.91:82:::FS:/boot:/dev/sda1:0.99:0.13:0.86:14::
  my %fs = ();
  (@index_arr) = grep { $metrics[$_] eq 'FS' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                              # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    $fs{$index} = {
      'mounted_on' => $metrics[ $index + 1 ],
      'label'      => $metrics[ $index + 2 ],
      'total'      => $metrics[ $index + 3 ],
      'used'       => $metrics[ $index + 4 ],
      'available'  => $metrics[ $index + 5 ],
      'usage'      => $metrics[ $index + 6 ]
    };
  }
  if (%fs) {
    $conf{"$ng_uuid"}{FS} = encode_json( \%fs );
    if (%conf) {

      # my $file_to_save = "$tmpdir/conf_$host_ip"."_$datacenter_uuid"."_$index_from"."_mounted.json";
      # if ( !Xormon::save_as_json( $file_to_save, \%ds_conf ) ) {
      #   error( "Cannot save $file_to_save: " . __FILE__ . ":" . __LINE__ );
      # }
    }
  }

  my %linux_data = (%cpu, %mem, %pgs, %qca, %disk_total_perf, %total_lan_perf);

  $perf{'linux'}{'server'}{$ng_uuid}{$time} = ( \%linux_data );

  # Architecture
  %linux_arch = (
    "item_id"   => $ng_uuid,
    "label"     => $label,
    "hw_type"   => "linux",
    "subsystem" => "server",
    "parents"   => []
  );
  $linux_arch{'agent_mapping'} = $agent_uuid if $agent_uuid;

  # push(@architecture, \%linux_arch);

  #save architecture
  #if (@architecture) {
  #  # print Dumper ("536",@architecture);
  #  my $rc = $xormon->saveArchitectureAgent(\@architecture);
  #  # status code
  #  if ($rc eq "401") {
  #    Xormon::error("540 Unauthorized");
  #    # return ();
  #  } elsif ($rc ne "201" ) {
  #    Xormon::error("Error 543 from $ip_address \$rc $rc");
  #    print Dumper ("543",@architecture);
  #    # return ();
  #  }
  #  else {
  #    # print "548 seems to be ok \$rc $rc\n";
  #  }
  #}

  #save configuration
  #if (%conf) {
  #  # print Dumper ("554",\%conf);
  #  my $rc = $xormon->saveConfJSON(\%conf);
  #  # status code
  #  if ($rc eq "401") {
  #    Xormon::error("558 Unauthorized");
  #    # return ();
  #  } elsif ($rc ne "201" ) {
  #    Xormon::error("Error 561 from $ip_address \$rc $rc");
  #    print Dumper ("561",\%conf);
  #    # return ();
  #  }
  #  else {
  #    # print "566 seems to be ok \$rc $rc\n";
  #  }
  #}

  # send data to backend
  if (%perf) {

    # print Dumper ("1633 \$ip_address $ip_address",\%perf) if $ip_address eq "10.22.33.105";
	# print Dumper ("1633 \$ip_address $ip_address",\%perf);
    #my $rc = $xormon->saveJSON2(\%perf);
    my $rc = "201";

    # status code
    if ( $rc eq "401" ) {
      Xormon::error("Unauthorized");
      return ();
    }
    elsif ( $rc ne "201" ) {
      Xormon::error("Error 579 from $ip_address \$rc $rc");
      print Dumper ( "579", \%perf );
      return ();
    }
    else {
      # print "584 seems to be ok \$rc $rc\n";
      if (%counter_data) {
        if ( Storable::store \%counter_data, "$last_agent_counter_data_file" ) {

          # print "587 data stored to $last_agent_counter_data_file\n";
        }
        else {
          print "590 data NOT stored to $last_agent_counter_data_file\n";
        }
      }
      return $time;
    }
  }
}

sub power() {
  my $xormon     = shift;
  my $line       = shift;
  my $ip_address = shift;
  my $protocol   = shift;

  #
  # AS400 test
  #
  if ( defined $protocol and $protocol >= 5 and $protocol < 6 ) {
    return as400( $xormon, \$line, $ip_address );
  }

  # my %perf; #global
  # my %conf; #global
  # my @architecture;
  my %last_agent_counter_data = ();
  my $last_agent_counter_data_file;
  my %counter_data;
  my $hashref = \%counter_data;

  # It must be there to avoid peaks caused by reseting counters
  my $INPUT_LIMIT_PGS     = 500000;         # 500MB/sec
  my $INPUT_LIMIT_LAN     = 12500000000;    # 125Gbits/sec
  my $INPUT_LIMIT_PCK_LAN = 125000000;      # Packets
  my $INPUT_LIMIT_SEA     = 12500000000;    # 125Gbits/sec
  my $INPUT_LIMIT_PCK_SEA = 12500000;       # Packets
  my $INPUT_LIMIT_SAN1    = 12800000000;    # 128Gbits/sec
  my $INPUT_LIMIT_SAN2    = 1000000;        # IOPS

  # 9117-MMC*44K8102:p770-aix2:6:1695016501:Mon Sep 18 07:55:01 2023 version 6.11-0:3724000000|4:p770-aix2:0::mem:::10485760:4415440:6070320:2191056:0:2002468:pgs:::0:0:512:3:::lan:en0:10.22.11.82:6859254716:59387509045:27115214:145955225:::cpu:::0:2:3:0:::san:vscsi0::0:9500:0:2.1:::san_resp:vscsi0::0.0:0.6:::::queue_cpu_aix:::1.6:8:0:0:0:1:lsdisk:hdisk0:0:810000000000:vscsi0:Available:Enabled:::FS:/:/dev/hd4:1.00:0.17:0.83:18:::FS:/usr:/dev/hd2:4.00:1.91:2.09:48:::FS:/var:/dev/hd9var:3.00:0.93:2.07:31:::FS:/tmp:/dev/hd3:3.00:0.11:2.89:4:::FS:/home:/dev/hd1:20.00:1.63:18.37:9:::FS:/admin:/dev/hd11admin:0.12:0.00:0.12:1:::FS:/opt:/dev/hd10opt:2.00:1.18:0.82:59:::FS:/var/adm/ras/livedump:/dev/livedump:0.25:0.00:0.25:1::
  # 9117-MMC*44K8102:Accept:10:1708511161:Wed Feb 21 11:26:01 2024 version 6.11-0:|:Accept:::mem:::51307200:15829376:35477824:-1:2936192:12893184:pgs:::0:0:3071.9375:0.0:::lan:eth0:10.22.11.31:2704726101:10944762449:12951710:145735035:::cpu:::0:0:0:0:546 days::st-cpu:::0::::::disk-total:0.00:0.00:0.00:0.43:7475.20:75.77:::queue_cpu:::0.0:16:0:::1:capacity_disk_lin:/dev/loop0:3.3G:::::::capacity_disk_lin:/dev/sda:30G:::::::capacity_disk_lin:/dev/sdb:50G:::::::FS:/:/dev/mapper/rhel-root:25.98:9.11:16.87:36:::FS:/boot:/dev/sda2:0.99:0.18:0.81:19:::FS:/home:/dev/mapper/compellent--vg-home--lv:49.94:16.39:33.55:33::
  # 9117-MMC*44K8102:Accept:10:1708511401:Wed Feb 21 11:30:01 2024 version 6.11-0:|:Accept:::mem:::51307200:15841152:35466048:-1:2948352:12892800:pgs:::0:0:3071.9375:0.0:::lan:eth0:10.22.11.31:2704730245:10944797595:12951750:145735525:::cpu:::0:0:0:2:546 days::st-cpu:::0::::::disk-total:0.00:0.00:0.00:0.43:6809.60:18056.92:::queue_cpu:::0.3:16:0::::CPUTOP:1:root:/usr/lib/systemd/systemd --switched-root --system --deserialize 21:7784:2:10816:159616::CPUTOP:1167:dbus:/bin/dbus-daemon --system --address=systemd --nofork --nopidfile --systemd-activation:5972:2:5760:7104:
  # 9117-MMC*44K8102:p770-aix3:11:1708512421:Wed Feb 21 11:47:01 2024 version 6.11-0:3724000000|4:p770-aix3:0::mem:::10485760:9968064:517696:2905868:0:6777208:pgs:::0:0:512:4:::lan:en0:10.22.11.84:17123122844:106508390803:142060701:298744338:::cpu:::0:1:2:0:::san:vscsi0::0:18600:0:3.6:::san_resp:vscsi0::0.0:0.6:::::queue_cpu_aix:::1.8:40:0:0:0:1:lsdisk:hdisk0:0:810000000000:vscsi0:Available:Enabled:::FS:/:/dev/hd4:1.00:0.19:0.81:20:::FS:/usr:/dev/hd2:4.00:2.31:1.69:58:::FS:/var:/dev/hd9var:3.00:2.40:0.60:80:::FS:/tmp:/dev/hd3:3.00:1.18:1.82:40:::FS:/home:/dev/hd1:20.00:14.81:5.19:75:::FS:/admin:/dev/hd11admin:0.12:0.00:0.12:1:::FS:/opt:/dev/hd10opt:2.00:1.56:0.44:79:::FS:/var/adm/ras/livedump:/dev/livedump:0.25:0.00:0.25:1::
  # 9105-41B*7889C11:p10-vios:3:1708533060:Wed Feb 21 10:31:00 2024 version 7.60-12:3000000000|8:p10-vios:0::mem:::6291456:6221948:69508:2334132:0:3617944:pgs:::0:0:2048:1:::sea:ent3:ent5:7130605237:103608363317:34955370:112412013:::lan:en5:10.22.11.40:27540973114:109048390904:82202694:132978885:::lan_error:en5:10.22.11.40:0::::::cpu:::0:1:2:0:45 days::san:fcs0:0x20000090FAA05B46:1886879424:215730729472:195521:7392951:::san_error:fcs0:22:::::::san:fcs1:0x20000090FAA05B47:3068027572:239287425024:215865:10851417:::san_error:fcs1:19:::::::san_resp:fcs0::0.0:0.9:::::san_resp:fcs1::0.0:0.9:::::queue_cpu_aix:::0.4:16:0:0:0:1:lsdisk:hdisk2:0:pdisk0:5000d3100377ce22,1000000000000:Available:Enabled:102400:OK:lsdisk:hdisk2:1:pdisk1:5000d3100377ce22,1000000000000:Available:Enabled:102400:OK:lsdisk:hdisk1:0:nvme1:1:Available:Enabled:763097:OK:lsdisk:hdisk0:0:nvme0:1:Available:Enabled:763097:OK:FS:/:/dev/hd4:1.00:0.06:0.94:7:::FS:/usr:/dev/hd2:4.00:2.82:1.18:71:::FS:/var:/dev/hd9var:3.00:0.79:2.21:27:::FS:/tmp:/dev/hd3:5.00:0.00:5.00:1:::FS:/home:/dev/hd1:10.00:0.34:9.66:4:::FS:/admin:/dev/hd11admin:1.00:0.00:1.00:1:::FS:/opt:/dev/hd10opt:1.00:0.04:0.96:5:::FS:/var/adm/ras/livedump:/dev/livedump:1.00:0.00:1.00:1:::FS:/iso:/dev/fslv00:50.00:32.82:17.18:66:::FS:/var/vio/VMLibrary:/dev/VMLibrary:55.00:25.93:29.07:48::a
  #
  # also is possible
  # queue_cpu:::0.1:16:0:::1: eg for p770-demo it is linux on power
  # san:host1::3469348779008:2245584051200:1029485299:1752626694:::san:host2::3384845092864:871251380224:365009291:1709537979:::
  # san:host1::3469348909056:2245595314176:1029492548:1752629785:::san:host2::3384845220864:871262986240:365016682:1709541023:::
  # disk-total:0.00:0.00:0.00:1.35:27340.80:135.56::: # only for demo? it is linux on power read_io,read_data,read_latency,write_io,write_data,write_latency
  # path_lin:compellent-demo02 (36000d3100377ce000000000000000011) dm-3 COMPELNT:size=100G features='1 queue_if_no_path' hwhandler='0' wp=rw:policy='service-time 0' prio=1 status=active/:- 1=====double-colon=====0=====double-colon=====0=====double-colon=====2 sdb 8=====double-colon=====16 active ready running/2=====double-colon=====0=====double-colon=====1=====double-colon=====2 sde 8=====double-colon=====64 active ready running/:::::
  # disk_id:hdisk0:60050768138101F0580000000000080F:::::::
  # disk_id:hdisk1:60050768138101F05800000000000822:::::::
  # disk_id:hdisk2:60050768138101F05800000000000828::::::1
  # lsdisk:hdisk0:0:5005076813153e0c,0:fscsi0:Available:Enabled:::lsdisk:hdisk0:1:5005076813153e0b,0:fscsi0:Available:Enabled:::
  # lsdisk:hdisk0:2:5005076813163e0c,0:fscsi1:Available:Enabled:::lsdisk:hdisk0:3:5005076813163e0b,0:fscsi1:Available:Enabled:::
  # lsdisk:hdisk1:0:5005076813153e0c,1000000000000:fscsi0:Available:Enabled:::
  # lsdisk:hdisk1:1:5005076813153e0b,1000000000000:fscsi0:Available:Enabled:::lsdisk:hdisk1:2:5005076813163e0c,
  # ame:::$comem:$coratio:$codefic:::
  # wlm:Unclassified:0.00:3:0.00:sup::::wlm:TOTAL:94.71:30514:3.77:sup::::wlm:bip1dev1:3.82:126:0.19:sup::::wlm:bip1int1:5.55:142:0.12:sup::::

  # Xormon::log( "241 \$line $line\n");

  my @parts = split( '\|', $line );
  my @confs = split( ':',  $parts[0] );

  # label
  my $label = $confs[1];

  # time
  my $time = $confs[3];

  my $serial = substr( $confs[0], 9, 7 );
  if ($first_mapping) {
    $first_mapping = 0;
    $mapping       = $xormon->powerAgentMapping( $serial, $label );
    if ( $ip_address !~ /^127/ ) {    # do not test localhost as it is internal monitoring
      if ( !defined $mapping->{item_id} ) {
        Xormon::log("1091 No mapping for $serial, $label\n");
        Xormon::error("1091 No mapping for $serial, $label\n");
        return ($time);               # as symbol of success, to not store many data on agent side
      }
    }
  }
  if ( $ip_address !~ /^127/ ) {    # do not test localhost as it is internal monitoring
    if ( !defined $mapping->{item_id} ) {
      return ($time);               # as symbol of success, to not store many data on agent side
    }
  }

  # uuid
  my $agent_uuid = $mapping->{item_id};
  $uuid = $mapping->{item_id};    # is global
  $uuid = "power_$uuid";          # diff from power LPAR uuid
                                  #Xormon::log( "662 \$uuid $uuid from $ip_address\n");

  $last_agent_counter_data_file = "$TMPDIR/agent_$serial" . "_" . "$uuid" . ".storable";
  $hashref                      = Storable::retrieve($last_agent_counter_data_file) if -f $last_agent_counter_data_file;
  %counter_data                 = %{$hashref};

  #print "last \%counter_data\n";
  #print Dumper \%counter_data;

  # conf data
  my @split1 = split( / version /, $line );
  my @split2 = split( /:/,         $split1[1] );    # not /:|:/ just /:/
  if ( defined $split2[0] ) {
    $conf{$uuid}{version} = $split2[0];
  }
  $conf{$uuid}{ip_address} = $ip_address;

  # Xormon::log( "901 \$line $line\n");
  my @metrics = split( ':', $parts[1] );
  shift @metrics;
  shift @metrics;
  shift @metrics;
  shift @metrics;                                   # so start with the first atom info most probably with 'mem'

  # CPU
  my %cpu = ();
  my (@index_arr) = grep { $metrics[$_] eq 'cpu' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                         # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) ) {
      %cpu = (
        "cpu_idle" => ( 100 - ( $metrics[ $index + 4 ] + $metrics[ $index + 5 ] + $metrics[ $index + 6 ] ) ),
        "cpu_sys"  => $metrics[ $index + 4 ],
        "cpu_usr"  => $metrics[ $index + 5 ],
        "cpu_wait" => $metrics[ $index + 6 ],
        "cpu_util" => ( $metrics[ $index + 4 ] + $metrics[ $index + 5 ] )
      );
    }
    else {
      Xormon::error("$serial $uuid $time agent cpu not number data $metrics[$index+4] $metrics[$index+5] $metrics[$index+6]");
    }
  }

  # AME
  my %ame = ();
  (@index_arr) = grep { $metrics[$_] eq 'ame' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) ) {
      %ame = (
        "ame_compres" => ( $metrics[ $index + 3 ] * 1024 * 1024 ),
        "ame_deficit" => ( $metrics[ $index + 5 ] * 1024 * 1024 ),
        "ame_ratio"   => ( $metrics[ $index + 4 ] / 100 )
      );
      $ame{ame_compres}         = 0 if $ame{ame_compres} < 0;
      $ame{ame_deficit}         = 0 if $ame{ame_deficit} < 0;
      $ame{ame_ratio}           = 0 if $ame{ame_ratio} < 0;
      $conf{"$uuid"}{ame_ratio} = $ame{ame_ratio};
    }
    else {
      Xormon::error("$serial $uuid $time agent AME not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5]");
    }
  }

  # CPU Stolen st-cpu:::0::::::
  (@index_arr) = grep { $metrics[$_] eq 'st-cpu' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) ) {
      $cpu{"st_cpu"} = $metrics[ $index + 3 ];
    }
    else {
      Xormon::error("$serial $uuid $time agent st-cpu not number data $metrics[$index+3]");
    }
  }

  # MEM
  my %mem = ();
  (@index_arr) = grep { $metrics[$_] eq 'mem' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # rank: size, nuse, free, pin, in_use_work, in_use_clnt
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) and Xormon::isdigit( $metrics[ $index + 7 ] ) and Xormon::isdigit( $metrics[ $index + 8 ] ) ) {
      %mem = (
        "mem_total"    => Xormon::returnBytes( $metrics[ $index + 3 ], "kiB" ),
        "mem_used"     => ( Xormon::returnBytes( $metrics[ $index + 4 ], "kiB" ) - Xormon::returnBytes( $metrics[ $index + 8 ], "kiB" ) ),
        "mem_fs_cache" => Xormon::returnBytes( $metrics[ $index + 8 ], "kiB" ),
        "mem_free"     => Xormon::returnBytes( $metrics[ $index + 5 ], "kiB" ),
        "mem2"         => Xormon::returnBytes( $metrics[ $index + 6 ], "kiB" )
      );
    }
    else {
      Xormon::error("$serial $uuid $time agent mem not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6] $metrics[$index+7]");
    }

    # print "288 \$index $index\n";
  }

  # PGS
  my %pgs = ();
  (@index_arr) = grep { $metrics[$_] eq 'pgs' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) ) {
      if ( exists $counter_data{$uuid}{time} ) {
        if ( ( $time - $counter_data{$uuid}{time} ) < $max_data_delay ) {    # last data not older x minutes ?
          %pgs = (
            "pgs1" => min( ( $metrics[ $index + 3 ] - $counter_data{$uuid}{pgs1} ) / 60, $INPUT_LIMIT_PGS ) * 4096,
            "pgs2" => min( ( $metrics[ $index + 4 ] - $counter_data{$uuid}{pgs2} ) / 60, $INPUT_LIMIT_PGS ) * 4096,
            "pgs3" => $metrics[ $index + 5 ],
            "pgs4" => $metrics[ $index + 6 ]
          );
          $pgs{pgs1} = 0 if $pgs{pgs1} < 0;
          $pgs{pgs2} = 0 if $pgs{pgs2} < 0;
        }
        else {
          print "515 probably longer interval actual: $time last " . $counter_data{$uuid}{time} . " for Linux pgs $uuid\n";
        }
      }
      else {
        print "519 probably 1st write to Linux pgs $uuid\n";
      }

      # save for next run
      $counter_data{$uuid}{pgs1} = $metrics[ $index + 3 ];
      $counter_data{$uuid}{pgs2} = $metrics[ $index + 4 ];
      $counter_data{$uuid}{time} = $time;
    }
    else {
      Xormon::error("Linux $uuid $time agent pgs not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6]");
    }
  }

  # queue_cpu_aix
  #  "DS:load:GAUGE:$no_time:0:U"
  #  "DS:virtual_p:GAUGE:$no_time:0:U"
  #  "DS:blocked_p:GAUGE:$no_time:0:U"
  #  "DS:blocked_raw:GAUGE:$no_time:0:U"
  #  "DS:blocked_IO:GAUGE:$no_time:0:U"

  my %qca = ();
  (@index_arr) = grep { $metrics[$_] eq 'queue_cpu_aix' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) and Xormon::isdigit( $metrics[ $index + 7 ] ) ) {
      %qca = (
        "load_avrg"          => $metrics[ $index + 3 ],
        "logical_processors" => $metrics[ $index + 4 ],
        "blocked_processes"  => $metrics[ $index + 5 ],
        "blocked_raw"        => $metrics[ $index + 6 ],
        "blocked_direct"     => $metrics[ $index + 7 ],
      );
	  if ($metrics[ $index + 6 ] > 0 or $metrics[ $index + 7 ] > 0) {
		print "1899 blocked_raw => $metrics[ $index + 6 ] blocked_direct => $metrics[ $index + 7 ] for $time\n";
	  }
	    #"blocked_raw"        => $metrics[ $index + 6 ],
        #"blocked_direct"     => $metrics[ $index + 7 ],
    }
    else {
      Xormon::error("$serial $uuid $time agent queue_cpu_aix not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6] $metrics[$index+7]");
    }
  }

  if ( !@index_arr ) {
    (@index_arr) = grep { $metrics[$_] eq 'queue_cpu' } ( 0 .. @metrics - 1 );
 
    foreach my $index (@index_arr) {
      if ( $index % 9 > 0 ) {    # skip possible coincidence
        Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
        next;
      }
      if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) ) {
        %qca = (
          "load_avrg"          => $metrics[ $index + 3 ],
          "logical_processors" => $metrics[ $index + 4 ],
          "blocked_processes"  => $metrics[ $index + 5 ],
          "blocked_raw"        => 0,
          "blocked_direct"     => 0,
        );
      }
      else {
        Xormon::error("$serial $uuid $time agent queue_cpu not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5]");
      }
	}
  }

  my %disk_total_perf = ();
  (@index_arr) = grep { $metrics[$_] eq 'disk-total' } ( 0 .. @metrics - 1 );    # get all indexes, here should be only one
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                      # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # print "787 \$index $index \$metrics[$index] $metrics[$index],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4],$metrics[$index+5]\n";
    %disk_total_perf = (
      'read_iops'     => $metrics[ $index + 1 ],
      'read_data'     => $metrics[ $index + 2 ],
      'read_latency'  => $metrics[ $index + 3 ],
      'write_iops'    => $metrics[ $index + 4 ],
      'write_data'    => $metrics[ $index + 5 ],
      'write_latency' => $metrics[ $index + 6 ]
    );
    $conf{$uuid}{"disk-total"} = $time;
  }

  # JOBS rss is in kBytes
  # data format ":CPUTOP:$pid:$user:$command_out:$time_1_sec:$time_difference:$rss:$vzs\:";
  #              :CPUTOP:109944:lpar2rrd:yes:3635:1792:608:107900::

  (@index_arr) = grep { $metrics[$_] eq 'CPUTOP' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                  # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    # print "1317 JOBS \$index $index \$metrics[$index] $metrics[$index] $metrics[$index+1]\n";

    if ( ! (Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) and Xormon::isdigit( $metrics[ $index + 7 ] ) ) ) {
      Xormon::error("$serial $uuid $time agent job $metrics[$index+1] not number data $metrics[$index+4] $metrics[$index+5] $metrics[$index+6] $metrics[$index+7]");
	  next;
	}

    my $uuid_job = "$uuid" . "_job." . "$metrics[$index+1]";
    if ( !exists $job_arch_list{$uuid_job} ) {
      $job_arch_list{$uuid_job} = "1";

      # architecture;
      my %job_arch = (
        "item_id"   => $uuid_job,
        "label"     => $metrics[ $index + 1 ],
        "hw_type"   => "linux",
        "subsystem" => "job",
        "parents"   => [$uuid]
      );
      push( @architecture_job, \%job_arch );
    }

    # Configuration
    $conf{$uuid_job}{pid}     = $metrics[ $index + 1 ];
    $conf{$uuid_job}{user}    = $metrics[ $index + 2 ];
    $conf{$uuid_job}{command} = $metrics[ $index + 3 ];

    # time_difference is devided 1/2 an hour because data is scanned every 30 minutes
    # rss from kB changed to B
    my %job_perf = (
        "time_1_sec"      => $metrics[ $index + 4 ],
        "time_difference" => ( $metrics[ $index + 5 ] / 1800 ),
        "rss"             => ( $metrics[ $index + 6 ] * 1024 ),
        "vzs"             => $metrics[ $index + 7 ]
    );
    $perf{'linux'}{'job'}{$uuid_job}{$time} = ( \%job_perf ) if %job_perf;
    # print Dumper \%perf;
  }

  # wlm:bip1dev1:3.82:126:0.19:sup::::
  # "DS:wlm_cpu:GAUGE:$no_time:0:100"
  # "DS:wlm_mem:GAUGE:$no_time:0:U"
  #  "DS:wlm_dkio:GAUGE:$no_time:0:100"

  (@index_arr) = grep { $metrics[$_] eq 'wlm' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                               # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # print "1366 wlm \$index $index \$metrics[$index] $metrics[$index] $metrics[$index+1]\n";
    my $uuid_wlm = "$uuid" . "_wlm_" . "$metrics[$index+1]";
    if ( !exists $wlm_arch_list{$uuid_wlm} ) {
      $wlm_arch_list{$uuid_wlm} = "1";

      # architecture;
      my %wlm_arch = (
        "item_id"   => $uuid_wlm,
        "label"     => $metrics[ $index + 1 ],
        "hw_type"   => "linux",
        "subsystem" => "wlm",
        "parents"   => [$uuid]
      );
      push( @architecture_wlm, \%wlm_arch );
    }

    my %wlm_perf = ();
	$wlm_perf{"wlm_cpu"} = $metrics[ $index + 2 ] if isdigit( $metrics[ $index + 2 ] );
	print "2052 wlm_cpu is U(ndefined)\n" if $metrics[ $index + 2 ] eq "U";
	$wlm_perf{"wlm_mem"} = ($metrics[ $index + 3 ] * 1024 * 1024) if isdigit( $metrics[ $index + 3 ] );
	print "2054 wlm_mem is U(ndefined)\n" if $metrics[ $index + 3 ] eq "U";
	$wlm_perf{"wlm_dkio"} = $metrics[ $index + 4 ] if isdigit( $metrics[ $index + 4 ] );
	print "2056 wlm_dkio is U(ndefined)\n" if $metrics[ $index + 4 ] eq "U";
	
	$perf{'linux'}{'wlm'}{$uuid_wlm}{$time} = ( \%wlm_perf ) if %wlm_perf;
  }

  # FS:/:/dev/hd4:1.00:0.17:0.83:18:::FS:/usr:/dev/hd2:4.00:1.91:2.09:48:::FS:/var:/dev/hd9var:3.00:0.93:2.07:31::: and so on
  my %fs = ();
  (@index_arr) = grep { $metrics[$_] eq 'FS' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                              # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    $fs{$index} = {
      'mounted_on' => $metrics[ $index + 1 ],
      'label'      => $metrics[ $index + 2 ],
      'total'      => $metrics[ $index + 3 ],
      'used'       => $metrics[ $index + 4 ],
      'available'  => $metrics[ $index + 5 ],
      'usage'      => $metrics[ $index + 6 ]
    };
  }
  if (%fs) {
    $conf{"$uuid"}{FS} = encode_json( \%fs );
    if (%conf) {

      # my $file_to_save = "$tmpdir/conf_$host_ip"."_$datacenter_uuid"."_$index_from"."_mounted.json";
      # if ( !Xormon::save_as_json( $file_to_save, \%ds_conf ) ) {
      #   error( "Cannot save $file_to_save: " . __FILE__ . ":" . __LINE__ );
      # }
    }
  }

  # MULTIPATH
  # AIX option
  # lsdisk:hdisk0:0:810000000000:vscsi0:Available:Enabled:::
  my %lsdisk = ();
  (@index_arr) = grep { $metrics[$_] eq 'lsdisk' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                  # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # print "1211 lsdisk \$index $index\n";
    $lsdisk{$index} = {
      'disk_size' => $metrics[ $index + 7 ],
      'label'     => $metrics[ $index + 1 ],
      'path_prop' => "$metrics[$index+4] , $metrics[$index+3]",
      'path_info' => $metrics[ $index + 5 ],
      'path_stat' => $metrics[ $index + 6 ],
      'status'    => $metrics[ $index + 8 ]
    };
  }
  if (%lsdisk) {
    $conf{"$uuid"}{LSDISK} = encode_json( \%lsdisk );

    # print Dumper \%conf;
    if (%conf) {

      # my $file_to_save = "$tmpdir/conf_$host_ip"."_$datacenter_uuid"."_$index_from"."_mounted.json";
      # if ( !Xormon::save_as_json( $file_to_save, \%ds_conf ) ) {
      #   error( "Cannot save $file_to_save: " . __FILE__ . ":" . __LINE__ );
      # }
    }
  }

  # Linux option
  my %multi_lin = ();
  (@index_arr) = grep { $metrics[$_] eq 'path_lin' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    my ( $alias, $wwid ) = split( /\(/, $metrics[ $index + 1 ] );
    $metrics[ $index + 4 ] =~ s/=====double-colon=====/\:/g;
    $wwid                  =~ s/\(//g;
    $wwid                  =~ s/\)//g;

    # print "1211 path_lin \$index $index\n";
    $multi_lin{$index} = {
      'wwid'       => $wwid,
      'alias'      => $alias,
      'attributes' => $metrics[ $index + 2 ],
      'paths'      => "$metrics[$index+3] , $metrics[$index+4]"
    };
  }
  if (%multi_lin) {
    $conf{"$uuid"}{MULTILIN} = encode_json( \%multi_lin );

    #print Dumper \%multi_lin;

  }

  # LAN
  # lan:en2:172.31.241.171:1418275448:444418173::::: # probably older version
  # lan:en0:10.22.11.82:6859254716:59387509045:27115214:145955225:::
  # data are COUNTER type so it is necessary to make from them GAUGE type, it is difference 'this_value - last_value'

  my %total_lan_perf = ();
  (@index_arr) = grep { $metrics[$_] eq 'lan' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                               # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # print "379 \$index $index \$metrics[$index] $metrics[$index]\n";
    my $uuid_lan = "$uuid" . "." . "$metrics[$index+1]";

    # architecture;
    if ( !exists $lan_arch_list{$uuid_lan} ) {
      $lan_arch_list{$uuid_lan} = "1";

      my %lan_arch = (
        "item_id"   => $uuid_lan,
        "label"     => $metrics[ $index + 1 ],
        "hw_type"   => "linux",
        "subsystem" => "lan",
        "parents"   => [$uuid]
      );
      push( @architecture_lan, \%lan_arch );
    }

    # Configuration
    $conf{$uuid_lan}{ip} = $metrics[ $index + 2 ];

    # "DS:recv_bytes:$ds_mode:$no_time:0:$INPUT_LIMIT_LAN"
    # "DS:trans_bytes:$ds_mode:$no_time:0:$INPUT_LIMIT_LAN"
    # "DS:recv_packets:$ds_mode:$no_time:0:$INPUT_LIMIT_PCK_LAN"
    # "DS:trans_packets:$ds_mode:$no_time:0:$INPUT_LIMIT_PCK_LAN"

    my %lan_perf;
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) ) {
      if ( exists $counter_data{$uuid_lan}{time} ) {
        if ( ( $time - $counter_data{$uuid_lan}{time} ) < $max_data_delay ) {    # last data not older x minutes ?
          %lan_perf = (
            "trans_bytes" => min( ( $metrics[ $index + 3 ] - $counter_data{$uuid_lan}{trans_bytes} ) / 60, $INPUT_LIMIT_LAN ),
            "recv_bytes"  => min( ( $metrics[ $index + 4 ] - $counter_data{$uuid_lan}{recv_bytes} ) / 60,  $INPUT_LIMIT_LAN )
          );
          $lan_perf{"trans_bytes"} = 0 if $lan_perf{"trans_bytes"} < 0;
          $lan_perf{"recv_bytes"}  = 0 if $lan_perf{"recv_bytes"} < 0;
          if (exists $total_lan_perf{"received_bytes"}) {
            $total_lan_perf{"received_bytes"} += $lan_perf{"recv_bytes"};
            $total_lan_perf{"send_bytes"} += $lan_perf{"trans_bytes"};
          }
          else {
            $total_lan_perf{"received_bytes"} = $lan_perf{"recv_bytes"};
            $total_lan_perf{"send_bytes"} = $lan_perf{"trans_bytes"};
          }

          if ( Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) ) {
            $lan_perf{"trans_packets"} = min( ( $metrics[ $index + 5 ] - $counter_data{$uuid_lan}{trans_packets} ) / 60, $INPUT_LIMIT_PCK_LAN );
            $lan_perf{"recv_packets"}  = min( ( $metrics[ $index + 6 ] - $counter_data{$uuid_lan}{recv_packets} ) / 60,  $INPUT_LIMIT_PCK_LAN );
            $lan_perf{trans_packets}   = 0 if $lan_perf{trans_packets} < 0;
            $lan_perf{recv_packets}    = 0 if $lan_perf{recv_packets} < 0;
          }
        }
        else {
          print "1166 probably longer interval actual: $time last " . $counter_data{$uuid_lan}{time} . " for $uuid_lan\n";
        }
      }
      else {
        print "1169 probably 1st write to $uuid_lan\n";
      }
      $perf{'linux'}{'lan'}{$uuid_lan}{$time} = ( \%lan_perf ) if %lan_perf;

      # save for next run
      $counter_data{$uuid_lan}{trans_bytes}   = $metrics[ $index + 3 ];
      $counter_data{$uuid_lan}{recv_bytes}    = $metrics[ $index + 4 ];
      $counter_data{$uuid_lan}{trans_packets} = $metrics[ $index + 5 ];
      $counter_data{$uuid_lan}{recv_packets}  = $metrics[ $index + 6 ];
      $counter_data{$uuid_lan}{time}          = $time;
    }
    else {
      Xormon::error("$serial $uuid $time agent lan not number data $metrics[$index+3] $metrics[$index+4]");
    }
  }

  # lan_error
  # lan_error:en0:10.22.11.41:0::::::
  # data are COUNTER type so it is necessary to make from them GAUGE type, it is difference 'this_value - last_value'
  (@index_arr) = grep { $metrics[$_] eq 'lan_error' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                     # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    #print "1033 \$index $index \$metrics[$index] $metrics[$index]\n";
    my $uuid_lan_error = "$uuid" . "_lan_error." . "$metrics[$index+1]";
    my $uuid_lan       = "$uuid" . "." . "$metrics[$index+1]";

    my $lan_error_perf = 0;
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) ) {
      if ( exists $counter_data{$uuid_lan_error}{time} ) {
        if ( ( $time - $counter_data{$uuid_lan_error}{time} ) < $max_data_delay ) {    # last data not older x minutes ?
          $lan_error_perf = ( $metrics[ $index + 3 ] - $counter_data{$uuid_lan_error}{lan_error} ) / 60;
        }
        else {
          print "1058 probably longer interval actual: $time last " . $counter_data{$uuid_lan_error}{time} . " for $uuid_lan_error\n";
        }
      }
      else {
        print "1062 probably 1st write to $uuid_lan_error\n";
      }
      if ( exists $perf{'linux'}{'lan'}{$uuid_lan}{$time} and $lan_error_perf >= 0 ) {
        $perf{'linux'}{'lan'}{$uuid_lan}{$time}{lan_error} = $lan_error_perf;

        #print "1097 add lan_error $lan_error_perf\n";
      }

      # save for next run
      $counter_data{$uuid_lan_error}{lan_error} = $metrics[ $index + 3 ];
      $counter_data{$uuid_lan_error}{time}      = $time;
    }
    else {
      Xormon::error("$serial $uuid $time agent lan_error not number data $metrics[$index+3]");
    }
  }

  # SEA
  # sea:ent0:ent8:325816622569:654717158475:689140597:1060578023:::sea:ent1:ent9:0:8863354964:0:129032545:::
  # data are COUNTER type so it is necessary to make from them GAUGE type, it is difference 'this_value - last_value'

  (@index_arr) = grep { $metrics[$_] eq 'sea' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                               # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # print "588 \$index $index \$metrics[$index] $metrics[$index]\n";
    my $uuid_sea = "$uuid" . "_sea_" . "$metrics[$index+1]";

    # architecture;
    if ( !exists $sea_arch_list{$uuid_sea} ) {
      $sea_arch_list{$uuid_sea} = "1";

      my %sea_arch = (
        "item_id"   => $uuid_sea,
        "label"     => "$metrics[$index+1]" . "($metrics[$index+2])",
        "hw_type"   => "linux",
        "subsystem" => "sea",
        "parents"   => [$uuid]
      );
      push( @architecture_sea, \%sea_arch );

      # print Dumper ("599",\%sea_arch);
    }

    # Configuration
    $conf{$uuid_sea}{ip} = $metrics[ $index + 2 ];

    # "DS:recv_bytes:$ds_mode:$no_time:0:$INPUT_LIMIT_SEA"
    # "DS:trans_bytes:$ds_mode:$no_time:0:$INPUT_LIMIT_SEA"
    # "DS:recv_packets:$ds_mode:$no_time:0:$INPUT_LIMIT_PCK_SEA"
    # "DS:trans_packets:$ds_mode:$no_time:0:$INPUT_LIMIT_PCK_SEA"

    my %sea_perf;
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) ) {
      if ( exists $counter_data{$uuid_sea}{time} ) {
        if ( ( $time - $counter_data{$uuid_sea}{time} ) < $max_data_delay ) {    # last data not older x minutes ?
          %sea_perf = (
            "recv_bytes"    => min( ( $metrics[ $index + 3 ] - $counter_data{$uuid_sea}{recv_bytes} ) / 60,    $INPUT_LIMIT_SEA ),
            "trans_bytes"   => min( ( $metrics[ $index + 4 ] - $counter_data{$uuid_sea}{trans_bytes} ) / 60,   $INPUT_LIMIT_SEA ),
            "recv_packets"  => min( ( $metrics[ $index + 5 ] - $counter_data{$uuid_sea}{recv_packets} ) / 60,  $INPUT_LIMIT_PCK_SEA ),
            "trans_packets" => min( ( $metrics[ $index + 6 ] - $counter_data{$uuid_sea}{trans_packets} ) / 60, $INPUT_LIMIT_PCK_SEA )
          );
          $sea_perf{trans_bytes}   = 0 if $sea_perf{trans_bytes} < 0;
          $sea_perf{recv_bytes}    = 0 if $sea_perf{recv_bytes} < 0;
          $sea_perf{trans_packets} = 0 if $sea_perf{trans_packets} < 0;
          $sea_perf{recv_packets}  = 0 if $sea_perf{recv_packets} < 0;
        }
        else {
          print "626 probably longer interval actual: $time last " . $counter_data{$uuid_sea}{time} . " for $uuid_sea\n";
        }
      }
      else {
        print "630 probably 1st write to $uuid_sea\n";
      }
      $perf{'linux'}{'sea'}{$uuid_sea}{$time} = ( \%sea_perf ) if %sea_perf;

      # save for next run
      $counter_data{$uuid_sea}{recv_bytes}    = $metrics[ $index + 3 ];
      $counter_data{$uuid_sea}{trans_bytes}   = $metrics[ $index + 4 ];
      $counter_data{$uuid_sea}{recv_packets}  = $metrics[ $index + 5 ];
      $counter_data{$uuid_sea}{trans_packets} = $metrics[ $index + 6 ];
      $counter_data{$uuid_sea}{time}          = $time;
    }
    else {
      Xormon::error("$serial $uuid $time agent sea not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6]");
    }
  }

  #    if (%fs) {
  #      $conf{"$uuid"}{FS} = encode_json(\%fs);
  #      if (%conf) {
  #        # my $file_to_save = "$tmpdir/conf_$host_ip"."_$datacenter_uuid"."_$index_from"."_mounted.json";
  #        # if ( !Xormon::save_as_json( $file_to_save, \%ds_conf ) ) {
  #        #   error( "Cannot save $file_to_save: " . __FILE__ . ":" . __LINE__ );
  #        # }
  #      }
  #    }

  # SAN
  # san:vscsi0::0:4000:0:1:::
  # san:vscsi1::0:5000:0:1:::
  # vscsi is absolute value (not COUNTER)
  # san:sissas1::0.0:94300:0:21.6:::
  # san:sissas2::0.0:76500:0:9.4:::
  # sissas is absolute value (not COUNTER)
  # san:fcs0:0x20000120FA109998:122204787990:2316970070528:10017993:211681877:::
  # san:fcs1:0x20000120FA109999:126146287845:2397674542592:10039004:220953662:::
  # fcs is COUNTER type so it is necessary to make from them GAUGE type, it is difference 'this_value - last_value'
  # all others than vscsi0/sissas1 are counter ?

  my %total_san_perf = ();
  (@index_arr) = grep { $metrics[$_] eq 'san' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                               # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }
    if ( $metrics[ $index + 1 ] =~ "vscsi" or $metrics[ $index + 1 ] =~ "sissas" ) {

      # print "729 \$index $index \$metrics[$index] $metrics[$index] $metrics[$index+1]\n";
      my $uuid_san = "$uuid" . "." . "$metrics[$index+1]";

      # architecture;
      if ( !exists $san_arch_list{$uuid_san} ) {
        $san_arch_list{$uuid_san} = "1";

        my %san_arch = (
          "item_id"   => $uuid_san,
          "label"     => $metrics[ $index + 1 ],
          "hw_type"   => "linux",
          "subsystem" => "san",
          "parents"   => [$uuid]
        );
        push( @architecture_san, \%san_arch );

        # print Dumper ("740", \%san_arch);
      }

      # Configuration
      $conf{$uuid_san}{ip} = $metrics[ $index + 2 ];

      #"DS:recv_bytes:GAUGE:$no_time:0:$INPUT_LIMIT_SAN1"
      #"DS:trans_bytes:GAUGE:$no_time:0:$INPUT_LIMIT_SAN1"
      #"DS:iops_in:GAUGE:$no_time:0:$INPUT_LIMIT_SAN2"
      #"DS:iops_out:GAUGE:$no_time:0:$INPUT_LIMIT_SAN2"
      # the limit is not necessary JK

      my %san_perf;
      if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) ) {
        %san_perf = (
          "recv_bytes"  => $metrics[ $index + 3 ],
          "trans_bytes" => $metrics[ $index + 4 ],
          "iops_read"   => $metrics[ $index + 5 ],
          "iops_write"  => $metrics[ $index + 6 ]
        );
        $perf{'linux'}{'san'}{$uuid_san}{$time} = ( \%san_perf ) if %san_perf;

        if (exists $total_san_perf{"san_received_bytes"}) {
          $total_san_perf{"san_received_bytes"} += $san_perf{"recv_bytes"};
          $total_san_perf{"san_send_bytes"} += $san_perf{"trans_bytes"};
        }
        else {
          $total_san_perf{"san_received_bytes"} = $san_perf{"recv_bytes"};
          $total_san_perf{"san_send_bytes"} = $san_perf{"trans_bytes"};
        }

        # print Dumper \%perf;
      }
      else {
        Xormon::error("$serial $uuid $time agent san $metrics[$index+1] not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6]");
      }
    }

    # elsif ($metrics[$index+1] =~ "fcs") left_curly # & host are counter, but only fcs has mixed IO in/IO out
    else {
      # print "759 \$index $index \$metrics[$index] $metrics[$index] $metrics[$index+1]\n";
      my $uuid_san = "$uuid" . "." . "$metrics[$index+1]";

      # architecture;
      if ( !exists $san_arch_list{$uuid_san} ) {
        $san_arch_list{$uuid_san} = "1";

        my %san_arch = (
          "item_id"   => $uuid_san,
          "label"     => $metrics[ $index + 1 ],
          "hw_type"   => "linux",
          "subsystem" => "san",
          "parents"   => [$uuid]
        );
        push( @architecture_san, \%san_arch );

        # print Dumper ("770", \%san_arch);
      }

      # Configuration
      $conf{$uuid_san}{ip} = $metrics[ $index + 2 ];

      #"DS:recv_bytes:$ds_mode:$no_time:0:$INPUT_LIMIT_SAN1"
      #"DS:trans_bytes:$ds_mode:$no_time:0:$INPUT_LIMIT_SAN1"
      #"DS:iops_in:$ds_mode:$no_time:0:$INPUT_LIMIT_SAN2"
      #"DS:iops_out:$ds_mode:$no_time:0:$INPUT_LIMIT_SAN2"

      ###  !!! Attention !!! data from lpar2rrd-agent.pl iops_in and iops_out are swapped only for fcs, see notice in lpar2rrd-agent.pl
      my %san_perf;
      if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) ) {
        if ( exists $counter_data{$uuid_san}{time} ) {
          if ( ( $time - $counter_data{$uuid_san}{time} ) < $max_data_delay ) {    # last data not older x minutes ?
            if ( $metrics[ $index + 1 ] =~ "fcs" ) {
              %san_perf = (
                "recv_bytes"  => min( ( $metrics[ $index + 3 ] - $counter_data{$uuid_san}{recv_bytes} ) / 60,    $INPUT_LIMIT_SAN1 ),
                "trans_bytes" => min( ( $metrics[ $index + 4 ] - $counter_data{$uuid_san}{trans_bytes} ) / 60,   $INPUT_LIMIT_SAN1 ),
                "iops_read"   => min( ( $metrics[ $index + 5 ] - $counter_data{$uuid_san}{recv_packets} ) / 60,  $INPUT_LIMIT_SAN2 ),
                "iops_write"  => min( ( $metrics[ $index + 6 ] - $counter_data{$uuid_san}{trans_packets} ) / 60, $INPUT_LIMIT_SAN2 )
              );
            }
            else {    # for host & other
              %san_perf = (
                "recv_bytes"  => min( ( $metrics[ $index + 3 ] - $counter_data{$uuid_san}{recv_bytes} ) / 60,    $INPUT_LIMIT_SAN1 ),
                "trans_bytes" => min( ( $metrics[ $index + 4 ] - $counter_data{$uuid_san}{trans_bytes} ) / 60,   $INPUT_LIMIT_SAN1 ),
                "iops_write"  => min( ( $metrics[ $index + 5 ] - $counter_data{$uuid_san}{recv_packets} ) / 60,  $INPUT_LIMIT_SAN2 ),
                "iops_read"   => min( ( $metrics[ $index + 6 ] - $counter_data{$uuid_san}{trans_packets} ) / 60, $INPUT_LIMIT_SAN2 )
              );
            }
            $san_perf{trans_bytes} = 0 if $san_perf{trans_bytes} < 0;
            $san_perf{recv_bytes}  = 0 if $san_perf{recv_bytes} < 0;
            $san_perf{iops_write}  = 0 if $san_perf{iops_write} < 0;
            $san_perf{iops_read}   = 0 if $san_perf{iops_read} < 0;

            if (exists $total_san_perf{"san_received_bytes"}) {
              $total_san_perf{"san_received_bytes"} += $san_perf{"recv_bytes"};
              $total_san_perf{"san_send_bytes"} += $san_perf{"trans_bytes"};
            }
            else {
              $total_san_perf{"san_received_bytes"} = $san_perf{"recv_bytes"};
              $total_san_perf{"san_send_bytes"} = $san_perf{"trans_bytes"};
            }
          }
          else {
            print "792 probably longer interval actual: $time last " . $counter_data{$uuid_san}{time} . " for $uuid_san\n";
          }
        }
        else {
          print "796 probably 1st write to $uuid_san\n";
        }
        $perf{'linux'}{'san'}{$uuid_san}{$time} = ( \%san_perf ) if %san_perf;

        # save for next run
        $counter_data{$uuid_san}{recv_bytes}    = $metrics[ $index + 3 ];
        $counter_data{$uuid_san}{trans_bytes}   = $metrics[ $index + 4 ];
        $counter_data{$uuid_san}{recv_packets}  = $metrics[ $index + 5 ];
        $counter_data{$uuid_san}{trans_packets} = $metrics[ $index + 6 ];
        $counter_data{$uuid_san}{time}          = $time;
      }
      else {
        Xormon::error("$serial $uuid $time agent san not number data $metrics[$index+3] $metrics[$index+4] $metrics[$index+5] $metrics[$index+6]");
      }
    }

    #      else {
    #        Xormon::error("unknow SAN type adapter $serial $metrics[$index+1]");
    #        next;
    #      }
  }

  # san_error
  # san_error:fcs0:104:::::::san_error:fcs1:52:::::::
  # data are COUNTER type so it is necessary to make from them GAUGE type, it is difference 'this_value - last_value'

  (@index_arr) = grep { $metrics[$_] eq 'san_error' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                     # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # print "658 \$index $index \$metrics[$index] $metrics[$index]\n";
    my $uuid_san_error = "$uuid" . "_san_error." . "$metrics[$index+1]";
    my $uuid_san       = "$uuid" . "." . "$metrics[$index+1]";

    # "DS:error_fcs:COUNTER:$no_time:0:U"

    my $san_error_perf = 0;
    if ( Xormon::isdigit( $metrics[ $index + 2 ] ) ) {
      if ( exists $counter_data{$uuid_san_error}{time} ) {
        if ( ( $time - $counter_data{$uuid_san_error}{time} ) < $max_data_delay ) {    # last data not older x minutes ?
          $san_error_perf = ( $metrics[ $index + 2 ] - $counter_data{$uuid_san_error}{san_error} ) / 60;

          # print "1208 new san_error ,$san_error_perf,\n";
        }
        else {
          print "683 probably longer interval actual: $time last " . $counter_data{$uuid_san_error}{time} . " for $uuid_san_error\n";
        }
      }
      else {
        print "687 probably 1st write to $uuid_san_error\n";
      }
      if ( exists $perf{'linux'}{'san'}{$uuid_san}{$time} and ( $san_error_perf >= 0 ) ) {
        $perf{'linux'}{'san'}{$uuid_san}{$time}{san_error} = $san_error_perf;

        #print "1219 add san_error $san_error_perf\n";
      }

      # save for next run
      $counter_data{$uuid_san_error}{san_error} = $metrics[ $index + 2 ];
      $counter_data{$uuid_san_error}{time}      = $time;
    }
    else {
      Xormon::error("$serial $uuid $time agent san_error not number data $metrics[$index+2]");
    }
  }

  # SAN RESP
  # san_resp:vscsi0::0.0:0.6:::::
  # san_resp:fcs0::8.6:0.0:::::san_resp:fcs1::8.6:0.0:::::san_resp:sissas1::0.0:4.3:::::san_resp:sissas2::0.0:13.3:::::
  # absolute value (not COUNTER)

  (@index_arr) = grep { $metrics[$_] eq 'san_resp' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                    # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    # print "769 \$index $index \$metrics[$index] $metrics[$index] $metrics[$index+1]\n";
    # my $uuid_san_resp = "$uuid"."_resp."."$metrics[$index+1]";
    my $uuid_san = "$uuid" . "." . "$metrics[$index+1]";

    # "DS:resp_t_r:GAUGE:$no_time:0:$INPUT_LIMIT_SAN1"
    # "DS:resp_t_w:GAUGE:$no_time:0:$INPUT_LIMIT_SAN1"
    # the limit is not necessary JK

    my %san_resp_perf;
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) ) {
      if ( exists $perf{'linux'}{'san'}{$uuid_san} ) {
        $perf{'linux'}{'san'}{$uuid_san}{$time}{resp_t_r} = $metrics[ $index + 3 ];
        $perf{'linux'}{'san'}{$uuid_san}{$time}{resp_t_w} = $metrics[ $index + 4 ];

        # print "1399 add san_resp $metrics[$index+3] $metrics[$index+4]\n";
      }

      # print Dumper \%perf;
    }
    else {
      Xormon::error("$serial $uuid $time agent san_resp $metrics[$index+1] not number data $metrics[$index+3] $metrics[$index+4]");
    }
  }

  # san_power
  # san_power:fcs0:-2.3225:-5.1698::::::
  # san_power:fcs0:UNKNOWN:UNKNOWN::::::
  # absolute value (not COUNTER)
  my %power_of_san = ();
  (@index_arr) = grep { $metrics[$_] eq 'san_power' } ( 0 .. @metrics - 1 );    # get all indexes
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {                                                     # skip possible coincidence
      Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3],$metrics[$index+4]");
      next;
    }

    my $uuid_san = "$uuid" . "." . "$metrics[$index+1]";

    if ( Xormon::isdigit( $metrics[ $index + 2 ] ) and Xormon::isdigit( $metrics[ $index + 3 ] ) ) {
      if ( exists $perf{'linux'}{'san'}{$uuid_san} ) {
        $perf{'linux'}{'san'}{$uuid_san}{$time}{rx_power} = $metrics[ $index + 2 ];
        $perf{'linux'}{'san'}{$uuid_san}{$time}{tx_power} = $metrics[ $index + 3 ];

        # print "1399 add san_resp $metrics[$index+3] $metrics[$index+4]\n";
      }
    }
    else {
      if ( !( ( $metrics[ $index + 2 ] eq "UNKNOWN" ) && ( $metrics[ $index + 3 ] eq "UNKNOWN" ) ) ) {
        Xormon::error("$serial $uuid $time agent san_power not number data $metrics[$index+2] & $metrics[$index+3]");
      }
    }
  }

  # all server data
  my %linux_data;
  if (%ame) {
	%linux_data = (%cpu, %mem, %pgs, %qca, %disk_total_perf, %ame, %total_lan_perf, %total_san_perf);
  }
  else {
	%linux_data = (%cpu, %mem, %pgs, %qca, %disk_total_perf, %total_lan_perf, %total_san_perf);
  }

  $perf{'linux'}{'server'}{$uuid}{$time} = ( \%linux_data );

  #Volumes Disk WWNs
  my @wwns;
  my @disk_array = split( ":", $line );
  for ( my $i = 0; $i < scalar(@disk_array); $i++ ) {
    push @wwns, $disk_array[ $i + 2 ] if ( $disk_array[$i] eq "disk_id" );
  }

  # Architecture
  %linux_arch = (
    "item_id"   => $uuid,
    "label"     => $label,
    "hw_type"   => "linux",
    "subsystem" => "server",
    "wwns"      => \@wwns,
    "parents"   => []
  );

  $linux_arch{'agent_mapping'} = $agent_uuid if $agent_uuid;

  #push(@architecture, \%linux_arch);

  #save architecture
  #if (@architecture) {
  #  # print Dumper ("1335",@architecture);
  #  my $rc = $xormon->saveArchitectureAgent(\@architecture);
  #  # status code
  #  if ($rc eq "401") {
  #    Xormon::error("1339 Unauthorized");
  #    # return ();
  #  } elsif ($rc ne "201" ) {
  #    Xormon::error("Error 1342 from $ip_address \$rc $rc");
  #    print Dumper ("1342",@architecture);
  #    # return ();
  #  }
  #  else {
  #    # print "1347 seems to be ok \$rc $rc\n";
  #  }
  #}

  #save configuration
  #if (%conf) {
  #  # print Dumper ("1353",\%conf);
  #  my $rc = $xormon->saveConfJSON(\%conf);
  #  # status code
  #  if ($rc eq "401") {
  #    Xormon::error("1357 Unauthorized");
  #    # return ();
  #  } elsif ($rc ne "201" ) {
  #    Xormon::error("Error 1360 from $ip_address \$rc $rc");
  #    print Dumper ("1360",\%conf);
  #    # return ();
  #  }
  #  else {
  #    # print "1365 seems to be ok \$rc $rc\n";
  #  }
  #}

  # send data to backend
  if (%perf) {

    #print Dumper ("1371",\%perf);
    # my $rc = $xormon->saveJSON2(\%perf);
    my $rc = "201";

    # status code
    if ( $rc eq "401" ) {
      Xormon::error("Unauthorized");
      return ();
    }
    elsif ( $rc ne "201" ) {
      Xormon::error("Error 1379 from $ip_address \$rc $rc");
      print Dumper ( "1379", \%perf );
      return ();
    }
    else {
      # print "1384 seems to be ok \$rc $rc\n";
      if (%counter_data) {
        if ( Storable::store \%counter_data, "$last_agent_counter_data_file" ) {

          # print "1387 data stored to $last_agent_counter_data_file\n";
        }
        else {
          print "1390 data NOT stored to $last_agent_counter_data_file\n";
        }
      }
      return $time;
    }
  }
}

#
# this protocol accepts perf data file (from hyperv), stores it to tmpdir/< 1st atom contain >/< 2nd atom contain >/<filename is 8th atom contain>
# for future: you can use 1st & 2nd atom as a config and on this base you can choose dir for saving files
#
sub store_data_63 {
  my $xormon = shift;
  my $data   = shift;    # this is pointer !!

  # print $$data;
  # print "data_end\n";

  #example of $data here incl item names for docum purpose ! data is one line !
  # HYPERV:Notebook:::Wed Feb 12 11:58:34 2014:file_name:file_length:::file_content i.e. data to transfer
  # data to transfer is test file but char pipe"|" is used as eol because of different eol in Windows and Linux
  # char pipe"|" in orig data is hardcoded as "=====svislitko====="

  ( my $folder_name, my $subfolder_name, undef, undef, my $date1, my $date2, my $date3, my $filename, my $file_length ) = split( ":", $$data );

  my $dir_name = "$TMPDIR/$folder_name";
  if ( !-d $dir_name ) {
    mkdir( $dir_name, 0777 ) || error( "Can't mkdir $dir_name: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  $subfolder_name =~ s/=====colon=====/:/g;
  $dir_name = "$TMPDIR/$folder_name/$subfolder_name";
  if ( !-d $dir_name ) {
    mkdir( $dir_name, 0777 ) || error( "Can't mkdir $dir_name: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  my $perf_file   = "$dir_name/$filename";
  my $perf_string = ( split( ":", $$data, 12 ) )[11];
  my $size        = length $perf_string;

  # create linux eol
  $perf_string =~ tr/\|/\n/;
  $perf_string =~ s/=====svislitko=====/\|/g;

  open( PERF_FILE, ">", $perf_file ) || error( "Can't open $perf_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print PERF_FILE $perf_string;    # "$file_content\n";
  close(PERF_FILE);

  #$windows_data_dir_name = $dir_name; # signal to call win routine
  $windows_data_dir_name = $subfolder_name;
  return $size;
}

sub docker() {
  my $xormon = shift;
  my $line   = shift;
  my $type   = shift;
  my %perf;

  my @parts = split( '\|', $line );
  my @confs = split( ':',  $parts[0] );

  # uuid
  my $uuid = $confs[2];

  # label
  my $label = $confs[1];

  # time
  my $time = $confs[3];

  my $data = from_json( $parts[1] );
  $perf{'docker'}{$type}{$uuid}{$time} = $data->{metrics} ? $data->{metrics} : $data;

  # Arch
  my @architecture;
  my %docker_arch = (
    "item_id"   => $uuid,
    "label"     => $label,
    "hw_type"   => "docker",
    "subsystem" => lc($type),
    "parents"   => $data->{parents}  ? $data->{parents}  : [],
    "children"  => $data->{children} ? $data->{children} : []
  );
  push( @architecture, \%docker_arch );

  if ( $data->{hostname} ) {
    my $toCheck = md5_hex( 'docker-' . $data->{hostname}{label} );
    my $check   = md5_hex('docker-Others');
    if ( $check eq $toCheck ) {
      my $address_uuid  = md5_hex( 'docker-' . $address );
      my %hostname_arch = (
        "item_id"   => $address_uuid,
        "label"     => $address,
        "hw_type"   => "docker",
        "subsystem" => "host",
        "children"  => [$uuid]
      );
      push( @architecture, \%hostname_arch );
    }
    else {
      my %hostname_arch = (
        "item_id"   => $data->{hostname}{uuid},
        "label"     => $data->{hostname}{label},
        "hw_type"   => "docker",
        "subsystem" => "host",
        "children"  => [$uuid]
      );
      push( @architecture, \%hostname_arch );
    }
  }

  #save architecture
  $xormon->saveArchitecture( \@architecture );

  # send data to backend
  my $rc = $xormon->saveJSON2( \%perf );

  # status code
  if ( $rc eq "401" ) {
    Xormon::error("Unauthorized");
    return ();
  }
  elsif ( $rc ne "201" ) {
    Xormon::error("Error");
    return ();
  }
  else {
    return $time;
  }
}

sub isdigit {
  my $digit = shift;
  my $text  = shift;

  if ( !defined $digit || $digit eq '' ) {
    return 0;
  }

  #if ( $digit eq 'U' ) {
  #  return 1;
  #}

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
}

sub uuid_big_endian_format {
  my $uuid       = uc shift;
  my $delimiter  = shift;
  my @uuid_parts = ();

  if ($delimiter) {
    if ( $uuid =~ /(.{8})$delimiter(.{4})$delimiter(.{4})$delimiter(.{4})$delimiter(.{12})/ ) {
      @uuid_parts = ( $1, $2, $3, $4, $5 );
    }
  }
  else {
    @uuid_parts = unpack( "a8 a4 a4 a4 a12", $uuid );
  }

  for ( my $i = 0; $i < 3; $i++ ) {
    $uuid_parts[$i] = join( '', reverse( split /(..)/, $uuid_parts[$i] ) );
  }
  return join( '-', @uuid_parts );
}

#
# AS400 part
# training data
#my $message ="9117-MMC*44K8102:as401:8:1716235534:May 20 2024 22:05:34.440 version 1.1.8.2024-07-27:2024-07-27::::S0200INFO:::AS401:000100:0:0.0:13:8:S0200PROCS:::0.3:1.00:2:1:0.0:672:S0200ASPJOB:::246:93177:59.0274:102125:193:163520:S0200ADDR:::0.013:0.073:0.000:0.000:0.000:0.000:S0200STORAGE:::6351:6425:5201408:0:0.3:5201408:S0400STORAGE:::AS401:000100:5201408:300308:260070::S040001Parm1:::*FIXED:*MACHINE:575988:291968:::S040001Parm2:::32767:0.0:0.0:0.0:0.0:14.9:S040001Parm3:::0.0:0.0:575988:97:0::S040001Parm4:::1:8.08:100.00:10.00:0.00:10.00:S040001Parm5:::0:0:575988:575988:::S040002Parm1:::*CALC:*BASE:4053288:4600:::S040002Parm2:::89:0.0:0.0:0.0:0.0:3912.0:S040002Parm3:::0.0:0.0:0:573:0::S040002Parm4:::1:4.99:100.00:12.00:1.00:200.00:S040002Parm5:::5:32767:4053288:0:::S040003Parm1:::*CALC:*INTERACT:520120:0:::S040003Parm2:::127:0.0:0.0:0.0:0.0:0.0:S040003Parm3:::0.0:0.0:520120:2:0::S040003Parm4:::1:10.00:100.00:12.00:1.00:200.00:S040003Parm5:::5:32767:520120:520120:::S040004Parm1:::*CALC:*SPOOL:52012:0:::S040004Parm2:::5:0.0:0.0:0.0:0.0:0.0:S040004Parm3:::0.0:0.0:52012:0:0::S040004Parm4:::2:1.00:100.00:5.00:1.00:100.00:S040004Parm5:::5:32767:52012:52012:::ASP001Parm1:::::not known:no status:::ASP002Parm1:::::not known:no status:::ASP033Parm1:::IASP001:IASP001:primary iASP:vary off:IASP001::ASP001Parm2:::1:44:0:90:system ASP::ASP001Parm3:::93177:38177:0:0:93177:38177:ASP002Parm2:::1:2:0:90:user ASP not contain library::ASP002Parm3:::8948:8945:0:0:8948:8945:ASP033Parm2:::1:2:0:90:user ASP contain library::ASP033Parm3:::8948:0:0:0:8948:0:ASP001Parm4:::0:32:0:3:24:2:ASP001Parm5:::0::::::ASP002Parm4:::0:0:0:0:0:0:ASP002Parm5:::0::::::ASP033Parm4:::0:0:0:0:0:0:ASP033Parm5:::0::::::ASPParm7:::1:SV-YA9GTJG:0:0:3:2:ASPParm7:::2:KT-S4HDLYS:0:0:0:0:ASPParm7:::33:SY-GEFHTM4:0:0:0:0:IFCD0100PAR1:::ETHLINE:Active:ELAN:B6xC5xD5x19x43x02:4:2:IFCD0100PAR2:::0:0:0:0:0:0:IFCD0100PAR1:::*LOOPBACK:Active:LOOPBACK:00x00x00x00x00x00:0:0:IFCD0100PAR2:::0:0:0:0:0:0:LTCParm1:::1:0:0:5.733:0.000::LTCParm1:::2:0:0:0.000:0.000::S040005Parm1:::*FIXED:1:256:0:MILAN:QGPL:S040005Parm2:::1:0.0:0.0:0.0:0.0:0.0:S040005Parm3:::0.0:0.0:256:0:0::S040005Parm4:::0:0.00:0.00:0.00:0.00:0.00:S040005Parm5:::0:0:256:256:::LST300JOB1:::Q1ACPDST   QBRMS      226828:Q1ABRMNET:QBRMS:PJ :0.1:DLYW:LST300JOB2:::04.59.492:1:DLY-60:39:*BASE:30:LST300JOB3:::193:0006.16.32.006:0:0:0000.00.00.337:0:LST300JOB4:::1:106977:1:0:::LST300JOB1:::SCPF       QSYS       000000::QSYS:SYS:0.0:EVTW:LST300JOB2:::04.59.492:1::25:*BASE:40:LST300JOB3:::193:0000.00.04.133:0:0:0000.00.00.000:0:LST300JOB4:::0:113725:0:0:::LST300JOB1:::QSYSARB3   QSYS       226678::QSYS:SYS:0.0:EVTW:LST300JOB2:::04.59.492:1::0:*BASE:00:LST300JOB3:::193:0000.00.00.039:0:0:0000.00.00.000:0:LST300JOB4:::0:2013:0:0:::LST300JOB1:::QDBSRV01   QSYS       226684::QSYS:SYS:0.0:EVTW:LST300JOB2:::04.59.492:1::5:*BASE:09:LST300JOB3:::193:0000.00.56.329:0:0:0000.00.00.001:0:LST300JOB4:::0:21044:0:0:::LST300JOB1:::QDBSRV02   QSYS       226685::QSYS:SYS:0.0:DEQW:LST300JOB2:::04.59.492:1::5:*BASE:16:LST300JOB3:::193:0000.00.00.036:0:0:0000.00.00.000:0:LST300JOB4:::0:927:0:0:::LST300JOB1:::QDBSRV03   QSYS       226686::QSYS:SYS:0.0:DEQW:LST300JOB2:::04.59.492:1::5:*BASE:16:LST300JOB3:::193:0000.00.00.040:0:0:0000.00.00.000:0:LST300JOB4:::0:1039:0:0:::LST300JOB1:::QDBSRV04   QSYS       226687::QSYS:SYS:0.0:DEQW:LST300JOB2:::04.59.492:1::9:*BASE:52:LST300JOB3:::193:0000.08.50.045:0:0:0000.00.00.008:0:LST300JOB4:::0:14454:0:0:::LST300JOB1:::QDBSRV05   QSYS       226688::QSYS:SYS:0.0:DEQW:LST300JOB2:::04.59.492:5::10:*BASE:52:LST300JOB3:::193:0000.08.57.117:0:0:0000.00.00.010:0:LST300JOB4:::0:147589:0:0:::LST300JOB1:::QDCPOBJ1   QSYS       226689::QSYS:SYS:0.0:EVTW:LST300JOB2:::04.59.492:1::0:*BASE:60:LST300JOB3:::193:0000.00.00.001:0:0:0000.00.00.000:0:LST300JOB4:::0:7:0:0:::LST300JOB1:::QDCPOBJ2   QSYS       226690::QSYS:SYS:0.0:EVTW:LST300JOB2:::04.59.492:1::0:*BASE:60:LST300JOB3:::193:0000.00.00.001:0:0:0000.00.00.000:0:LST300JOB4:::0:7:0:0:::";

#line2 example - pools counted from 01 to 64
# 8203-E4A*0659DC4:prg2i:4:1450041258:Dec 13 2015 21:14:18.360 version 1.0.0:::::
# S0220PARAM1:::PRG2I:000001:0:2:1.6:305:
# S0220PARAM2:::3:0.80:1:261:821:163520:
# S0400STORAGE:::PRG2I:000001:7396352:270016:422656::
# S0400STORAGEL:::7396352:270016:422656:::
# S040001Parm1:::*FIXED:*MACHINE:716800:355164:::
# S040001Parm2:::32767:0.0:0.0:0.0:0.0:28.0:
# S040001Parm3:::0.0:0.0:716800:95:0::
# S040001Parm4:::1:6.02:100.00:10.00:0.00:10.00:
# S040001Parm5:::0:0:716800:716800:::
# S040002Parm1:::*CALC:*BASE:6609664:4528:::S040002Parm2:::100:0.1:0.6:0.2:2.5:6752.9:S040002Parm3:::0.0:0.0:0:599:0::
# S040002Parm4:::1:6.16:100.00:12.00:1.00:200.00:S040002Parm5:::5:32767:6609664:0:::
# S040003Parm1:::*FIXED:*INTERACT:1024000:0:::S040003Parm2:::7:0.0:0.0:0.1:1.3:143.4:S040003Parm3:::0.0:0.0:1024000:12:0::
# S040003Parm4:::1:10.00:100.00:12.00:1.00:200.00:S040003Parm5:::5:32767:1024000:1024000:::
# S040004Parm1:::*FIXED:*SPOOL:102400:0:::S040004Parm2:::2:0.0:0.0:0.0:0.0:0.0:S040004Parm3:::0.0:0.0:102400:1:0::
# S040004Parm4:::2:1.00:100.00:5.00:1.00:100.00:S040004Parm5:::5:32767:102400:102400:::
# S040005Parm1:::*FIXED:1:256:0:MILAN:QGPL:
# S040005Parm2:::1:0.0:0.0:0.0:0.0:0.0:
# S040005Parm3:::0.0:0.0:256:0:0::
# S040005Parm4:::0:0.00:0.00:0.00:0.00:0.00:
# S040005Parm5:::0:0:256:256:::
#line4 example
# 8203-E4A*0659DC4:prg2i:4:1454495512:Feb 03 2016 10:31:52.720 version 1.0.0:::::
# LST300JOB1:::RTV_SYSSTS CZ50257PH  177793:PH_SBS:CZ50257PH:BCH:0.5:RUN:
# LST300JOB2:::01.056:1:PGM-C_RTVSTS:10:*BASE:50:
# LST300JOB3:::258:0000.00.00.429:0:0:0000.00.00.005:0:
# LST300JOB4:::5:340:2:3:::
#line5 example
# ASP123Parm1:::resource:device:1:1:database:primary:ASP123Parm1:::
# ASP123Parm2:::1:2:3:4:5::
# ASP123Parm3:::1:2:3:4:5:6:
# ASP123Parm4:::1:2:3:4:5:6:
#line6 example
# IFCD0100PAR1:::ETHLINE:Active:ELAN:00x21x5Ex19x76xA1:134458:6512106:IFCD0100PAR1:::
# IFCD0100PAR2:::2586:0:0:2385:0:0:IFCD0100PAR2:::
#line7 example
# ASPParm7:::1:68-0ECD5C0:1:1:2:2:
#line8 example - latency
# LTCParm1:::ASP number:Buffer overruns:Buffer underruns:Disk service time:Disk wait time::
# LTCParm1:::1:0:0:2.971:0.013::
# LTCParm1:::2:0:0:1.696:0.000::
# LTCParm1:::33:0:0:1.340:0.000::

sub as400 {
  my $xormon     = shift;
  my $line_ref   = shift;
  my $ip_address = shift;

  my $line = $$line_ref;
  my %perf;
  my %conf;
  my @architecture;

  # Xormon::log( "1995 \$line $line\n");

  my @atoms   = split( ':', $line );
  my @confs   = @atoms[ 0 .. 10 ];
  my @metrics = @atoms[ 11 .. $#atoms ];

  # uuid
  #    my $id = $confs[0]."_".$confs[2];
  #    if (!$id) {
  #        $id = $confs[1];
  #    }
  #    my $uuid = "as400_" . $id;
  #    Xormon::log( "201 \$uuid $uuid from $ip_address\n");

  # label
  my $label = $confs[1];

  # time
  my $time = $confs[3];

  my $serial = substr( $confs[0], 9, 7 );
  if ($first_mapping) {
    $first_mapping = 0;
    $mapping       = $xormon->powerAgentMapping( $serial, $label );
  }
  if ( $ip_address !~ /^127/ ) {    # do not test localhost as it is internal monitoring
    if ( !defined $mapping->{item_id} ) {
      Xormon::log("2026 No mapping for $serial, $label\n");
      Xormon::error("2026 No mapping for $serial, $label\n");
      return ($time);               # as symbol of success, to not store many data on agent side
    }
  }

  # uuid
  my $agent_uuid = $mapping->{item_id};
  my $uuid       = $mapping->{item_id};
  $uuid = "as400_$uuid";    # diff from power LPAR uuid
                            # Xormon::log( "2036 \$uuid $uuid from $ip_address\n");

  my $last_agent_counter_data_file = "$TMPDIR/" . "$uuid" . ".storable";
  my $hashref                      = Storable::retrieve($last_agent_counter_data_file) if -f $last_agent_counter_data_file;
  my %counter_data                 = %{$hashref}                                       if defined $hashref;

  #print "last \%counter_data\n";
  #print Dumper ("2039",\%counter_data) if %counter_data;

  # conf data
  # 6th and 7th atom is :34.440 version 1.1.8.2024-07-27:2024-07-27:
  #
  my @split1  = split( / version /, $confs[6] );
  my $version = substr( $split1[1], 0, 6 );
  if ( defined $version ) {
    $conf{$uuid}{version} = $version;
    if ( defined $confs[7] ) {
      $conf{$uuid}{version_date} = $confs[7];
    }
    else {
      $conf{$uuid}{version_date} = substr( $split1[1], 6, 10 );
    }
  }
  $conf{$uuid}{ip_address} = $ip_address;

  # server
  my %server = ();
  my %cpu    = ();

  # Architecture
  my %as400_arch = (
    "item_id"   => $uuid,
    "label"     => $label,
    "hw_type"   => "as400",
    "subsystem" => "server",
    "parents"   => []
  );

  $as400_arch{'agent_mapping'} = $agent_uuid if $agent_uuid;

  push( @architecture, \%as400_arch );

  my (@index_arr) = grep { $metrics[$_] eq 'S0200PROCS' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence

      #            Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 8 ] ) ) {
      $cpu{"active_threads_in_system"} = $metrics[ $index + 8 ];
    }
    else {
      Xormon::error("AS400 $uuid $time active_threads_in_system not number data $metrics[$index+8]");
    }
  }

  (@index_arr) = grep { $metrics[$_] eq 'S0200ASPJOB' } ( 0 .. @metrics - 1 );
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence

      #            Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 7 ] ) ) {
      $cpu{"jobs_in_system"}        = $metrics[ $index + 3 ];
      $cpu{"active_jobs_in_system"} = $metrics[ $index + 7 ];
    }
    else {
      Xormon::error("AS400 $uuid $time (active_)jobs_in_system not number data $metrics[$index+3] $metrics[$index+7]");
    }
  }
  $perf{'as400'}{'server'}{$uuid}{$time} = ( \%cpu ) if %cpu;

  # ASP
  # ASP latency (srv & wait), prepare it in at once, save cus it is coming not in same line as other ASP data

  my (@index_asplat) = grep { $metrics[$_] eq "LTCParm1" } ( 0 .. @metrics - 1 );

  # print "178 LTCParm1 \@index_asplat @index_asplat\n";
  my %asplat = ();

  foreach my $index (@index_asplat) {

    # print "182 \$index $index\n";
    if ( $index % 9 > 0 ) {    # skip possible coincidence

      #       Xormon::log("skip coincidence \$index $index $metrics[$index,$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) and Xormon::isdigit( $metrics[ $index + 7 ] ) ) {    #pool number, SVC time, wait time
      $asplat{ $metrics[ $index + 3 ] }{svc_time}  = $metrics[ $index + 6 ];
      $asplat{ $metrics[ $index + 3 ] }{wait_time} = $metrics[ $index + 7 ];
    }
  }

  # print Dumper ("2115",\%asplat);

  my $last_agent_asplat_data_file = "$TMPDIR/" . "$uuid" . "_ASP.storable";
  if (%asplat) {
    if ( Storable::store \%asplat, "$last_agent_asplat_data_file" ) {
      print "2119 data stored to $last_agent_asplat_data_file\n";
    }
    else {
      print "2122 data NOT stored to $last_agent_asplat_data_file\n";
    }
  }

  if ( !%asplat ) {
    my $hashref = Storable::retrieve($last_agent_asplat_data_file) if -f $last_agent_asplat_data_file;
    %asplat = %{$hashref} if defined $hashref;

    #print "last \%asplat_data\n";
    #print Dumper \%asplat_data;
  }

  #
  # ASP disk_busy & disk IO aggregated), prepare it in at once
  # ASPParm7:::1:SV-YA9GTJG:0:0:3:2:ASPParm7:::2:KT-S4HDLYS:0:0:0:0:ASPParm7:::33:SY-GEFHTM4:0:0:0:0:

  (@index_asplat) = grep { $metrics[$_] eq "ASPParm7" } ( 0 .. @metrics - 1 );

  # print "193 \@index_asplat @index_asplat\n";

  foreach my $index (@index_asplat) {
    print "196 \$index $index\n";
    if ( $index % 9 > 0 ) {    # skip possible coincidence

      #       Xormon::log("skip coincidence \$index $index $metrics[$index,$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 5 ] ) and Xormon::isdigit( $metrics[ $index + 6 ] ) and Xormon::isdigit( $metrics[ $index + 7 ] ) ) {    # busy, IO in, IO out
      $asplat{ $metrics[ $index + 3 ] }{disk_busy}      = $metrics[ $index + 5 ];
      $asplat{ $metrics[ $index + 3 ] }{disk_io_aggreg} = $metrics[ $index + 6 ] + $metrics[ $index + 7 ];
      $asplat{ $metrics[ $index + 3 ] }{disk_serial}    = $metrics[ $index + 4 ];
    }
  }

  # print Dumper ("207",\%asplat);

  #
  # ASP cycle
  #
  (@index_arr) = grep { $metrics[$_] =~ "ASP...Parm3" } ( 0 .. @metrics - 1 );

  # print "195: @index_arr\n";
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence

      #            Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 3 ] ) and Xormon::isdigit( $metrics[ $index + 4 ] ) ) {    #ASP capacity total, ASP capacity available total
                                                                                                        # ASP001Parm3:::93177:38177:0:0:93177:38177:ASP002Parm3:::8948:8945:0:0:8948:8945:ASP033Parm3:::8948:0:0:0:8948:0
      my $pool_number      = substr( $metrics[$index], 3, 3 );
      my $orig_pool_number = $pool_number;
      my $uuid_asp         = "$uuid" . "_ASP" . "$pool_number";

      print "2177: \$pool_number $pool_number\n";
      my $asp_label = substr( $metrics[$index], 0, 6 );
      $asp_label = "*SYSTEM" if $pool_number eq "001";
      if ( $pool_number > 32 ) {
        $pool_number -= 32;
        if ( $pool_number < 10 ) {
          $asp_label = "iASP00$pool_number";
        }
        else {
          $asp_label = "iASP0$pool_number";
        }
      }
      print "2189 \$asp_label $asp_label\n";
      my %asp_perf = ();
      $asp_perf{mem_alloc}      = ( $metrics[ $index + 3 ] - $metrics[ $index + 4 ] ) * 1024 * 1024;                    #Bytes
      $asp_perf{mem_free}       = $metrics[ $index + 4 ] * 1024 * 1024;
	  $asp_perf{mem_alloc_perc} =0;
      $asp_perf{mem_alloc_perc} = ( $metrics[ $index + 3 ] - $metrics[ $index + 4 ] ) / $metrics[ $index + 3 ] * 100 if $metrics[ $index + 3 ] > 0;

      # data flow and IOPS

      my (@index_asppar4) = grep { $metrics[$_] eq "ASP$orig_pool_number" . "Parm4" } ( 0 .. @metrics - 1 );
      print "2198 \@index_asppar4 @index_asppar4\n";

      next if ! defined $index_asppar4[0];

	  if ( $index_asppar4[0] % 9 > 0 ) {                                                                                # skip possible coincidence

        #           Xormon::log("skip coincidence \$index_asppar4[0] $index_asppar4[0] $metrics[$index_asppar4[0]],$metrics[+$index_asppar4[0]1],$metrics[+$index_asppar4[0]2],$metrics[$index_asppar4[0]+3]");
        next;
      }
      if ( Xormon::isdigit( $metrics[ $index_asppar4[0] + 3 ] ) and Xormon::isdigit( $metrics[ $index_asppar4[0] + 4 ] ) and Xormon::isdigit( $metrics[ $index_asppar4[0] + 5 ] ) and Xormon::isdigit( $metrics[ $index_asppar4[0] + 6 ] ) ) {    #data tranfered to, data tranfered from, IO to, TO from
        $asp_perf{data_to_memory}   = $metrics[ $index_asppar4[0] + 3 ] * 512;
        $asp_perf{data_from_memory} = $metrics[ $index_asppar4[0] + 4 ] * 512;
        $asp_perf{io_to_memory}     = $metrics[ $index_asppar4[0] + 5 ];
        $asp_perf{io_from_memory}   = $metrics[ $index_asppar4[0] + 6 ];
      }

      # ASP latency (srv & wait) & disk_busy & disk IO aggreg + conf
      my $orig_pool_num = $orig_pool_number * 1;
      $asp_perf{svc_time}       = $asplat{$orig_pool_num}{svc_time}                                          if exists $asplat{$orig_pool_num}{svc_time};
      $asp_perf{wait_time}      = $asplat{$orig_pool_num}{wait_time}                                         if exists $asplat{$orig_pool_num}{wait_time};
      $asp_perf{latency_time}   = ( $asplat{$orig_pool_num}{svc_time} + $asplat{$orig_pool_num}{wait_time} ) if exists $asplat{$orig_pool_num}{svc_time};
      $asp_perf{disk_busy}      = $asplat{$orig_pool_num}{disk_busy}                                         if exists $asplat{$orig_pool_num}{disk_busy};
      $asp_perf{disk_io_aggreg} = $asplat{$orig_pool_num}{disk_io_aggreg}                                    if exists $asplat{$orig_pool_num}{disk_io_aggreg};

      $perf{'as400'}{'asp'}{$uuid_asp}{$time} = ( \%asp_perf ) if %asp_perf;

      # Architecture
      my %as400_asp_arch = (
        "item_id"   => $uuid_asp,
        "label"     => $asp_label,
        "hw_type"   => "as400",
        "subsystem" => "asp",
        "parents"   => [$uuid]
      );
      push( @architecture, \%as400_asp_arch );

      # ASP conf is in Parm1
      # ASP001Parm1:::::not known:no status:::
      # ASP033Parm1:::IASP001:IASP001:primary iASP:vary off:IASP001::ASP001Parm2:::1:44:0:90:system ASP::
      my (@index_asppar1) = grep { $metrics[$_] eq "ASP$orig_pool_number" . "Parm1" } ( 0 .. @metrics - 1 );
      print "2235 \@index_asppar1 @index_asppar1 ASP$orig_pool_number\n";

      if ( $index_asppar1[0] % 9 > 0 ) {    # skip possible coincidence

        #           Xormon::log("skip coincidence \$index_asppar1[0] $index_asppar1[0] $metrics[$index_asppar1[0]],$metrics[$index_asppar1[0]+1],$metrics[$index_asppar1[0]+2],$metrics[$index_asppar1[0]+3]");
        next;
      }
      my $asp_config = "";
      if ( ( !$metrics[ $index_asppar1[0] + 3 ] eq '' ) && $metrics[ $index_asppar1[0] + 3 ] !~ m/not known/ && $metrics[ $index_asppar1[0] + 3 ] !~ m/no status/ ) {
        $asp_config .= "Res name: $metrics[$index_asppar1[0]+3],";
      }
      if ( ( !$metrics[ $index_asppar1[0] + 4 ] eq '' ) && $metrics[ $index_asppar1[0] + 4 ] !~ m/not known/ && $metrics[ $index_asppar1[0] + 4 ] !~ m/no status/ ) {
        $asp_config .= "Dev name: $metrics[$index_asppar1[0]+4],";
      }
      if ( ( !$metrics[ $index_asppar1[0] + 5 ] eq '' ) && $metrics[ $index_asppar1[0] + 5 ] !~ m/not known/ && $metrics[ $index_asppar1[0] + 5 ] !~ m/no status/ ) {
        $asp_config .= "Usage: $metrics[$index_asppar1[0]+5],";
      }
      if ( ( !$metrics[ $index_asppar1[0] + 6 ] eq '' ) && $metrics[ $index_asppar1[0] + 6 ] !~ m/not known/ && $metrics[ $index_asppar1[0] + 6 ] !~ m/no status/ ) {
        $asp_config .= "Status: $metrics[$index_asppar1[0]+6],";
      }
      if ( ( !$metrics[ $index_asppar1[0] + 7 ] eq '' ) && $metrics[ $index_asppar1[0] + 7 ] !~ m/not known/ && $metrics[ $index_asppar1[0] + 7 ] !~ m/no status/ ) {
        $asp_config .= "Db name: $metrics[$index_asppar1[0]+7],";
      }

      #if ( (!$metrics[$index_asppar1[0]+8] eq '') && $metrics[$index_asppar1[0]+8] !~ m/not known/ && $metrics[$index_asppar1[0]+8] !~ m/no status/ ) {
      #  $asp_config .= "Asp number: $metrics[$index_asppar1[0]+8],";
      #}
      if ( length($asp_config) > 0 ) {
        $asp_config .= "Asp number:$orig_pool_number";
      }

      $conf{$uuid_asp}{bubble}      = $asp_config;
      $conf{$uuid_asp}{time}        = $time;
      $conf{$uuid_asp}{disk_serial} = $asplat{$orig_pool_num}{disk_serial} if exists $asplat{$orig_pool_num}{disk_serial};
    }
    else {
      Xormon::error("AS400 $uuid $time $metrics[$index] not number data $metrics[$index+3] $metrics[$index+4]");
    }
  }

  #
  # S0400 share pools
  # get all cfg files -> cycle on them
  # S040001Parm1:::*FIXED:*MACHINE:575988:291968:::
  # S040005Parm1:::*FIXED:1:256:0:MILAN:QGPL:

  (@index_arr) = grep { $metrics[$_] =~ "S0400..Parm1" } ( 0 .. @metrics - 1 );
  print "2280 S0400..Parm1: @index_arr\n";
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence

      #            Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }
    if ( !Xormon::isdigit( $metrics[ $index + 5 ] ) ) {    # pool size
      Xormon::error(" $metrics[$index] ($index) not number data $metrics[$index+5]");
      next;
    }
    my $pool_number = substr( $metrics[$index], 5, 2 );
    print "2291 \$pool_number $pool_number\n";

    my $uuid_shpool = "$uuid" . "_$pool_number";

    #        # Architecture
    #        my %as400_shpool_arch = (
    #          "item_id" => $uuid_shpool,
    #          "label" => $metrics[$index+4],
    #          "hw_type" => "as400",
    #          "subsystem" => "shrpool",
    #          "parents" => [$uuid]
    #        );
    #        push(@architecture, \%as400_shpool_arch);

    my %pool_perf = ();

    # data POOL SIZE
    $pool_perf{size} = $metrics[ $index + 5 ] * 1024;    #Bytes

    # data THREADS
    my (@index_threads) = grep { $metrics[$_] eq "S0400$pool_number" . "Parm3" } ( 0 .. @metrics - 1 );

    #print "305 \@index_threads @index_threads\n";

    if ( $index_threads[0] % 9 > 0 ) {                   # skip possible coincidence

      #           Xormon::log("skip coincidence \$index_threads[0] $index_threads[0] $metrics[$index_threads[0]],$metrics[$index_threads[0]+1],$metrics[$index_threads[0]+2],$metrics[$index_threads[0]+3]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index_threads[0] + 6 ] ) ) {    #data threads
      $pool_perf{threads} = $metrics[ $index_threads[0] + 6 ];
    }

    # data FAULTS & PAGES
    my (@index_f_p) = grep { $metrics[$_] eq "S0400$pool_number" . "Parm2" } ( 0 .. @metrics - 1 );

    #print "318 \@index_f_p @index_f_p\n";

    if ( $index_f_p[0] % 9 > 0 ) {                                   # skip possible coincidence

      #           Xormon::log("skip coincidence \$index_f_p[0] $index_f_p[0] $metrics[$index_f_p[0]],$metrics[$index_f_p[0]+1],$metrics[$index_f_p[0]+2],$metrics[$index_f_p[0]+3]");
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index_f_p[0] + 4 ] ) && Xormon::isdigit( $metrics[ $index_f_p[0] + 5 ] ) && Xormon::isdigit( $metrics[ $index_f_p[0] + 6 ] ) && Xormon::isdigit( $metrics[ $index_f_p[0] + 7 ] ) ) {    #data fault and pages
      $pool_perf{database_faults}     = $metrics[ $index_f_p[0] + 4 ];
      $pool_perf{non_database_faults} = $metrics[ $index_f_p[0] + 6 ];
      $pool_perf{database_pages}      = $metrics[ $index_f_p[0] + 5 ];
      $pool_perf{non_database_pages}  = $metrics[ $index_f_p[0] + 7 ];
    }

    $perf{'as400'}{'shrpool'}{$uuid_shpool}{$time} = ( \%pool_perf ) if %pool_perf;

    # label
    # my $shrpool_config = $datar[ $cycle_var + 4 ] . " " . $datar[ $cycle_var + 7 ] . " " . $datar[ $cycle_var + 8 ]; # it is label
    my (@index_asppar1) = grep { $metrics[$_] eq "S0400$pool_number" . "Parm1" } ( 0 .. @metrics - 1 );
    print "2340 \@index_asppar1 @index_asppar1 S0400$pool_number\n";

    if ( $index_asppar1[0] % 9 > 0 ) {    # skip possible coincidence

      #         Xormon::log("skip coincidence \$index_asppar1[0] $index_asppar1[0] $metrics[$index_asppar1[0]],$metrics[$index_asppar1[0]+1],$metrics[$index_asppar1[0]+2],$metrics[$index_asppar1[0]+3]");
      next;
    }
    my $shrpool_config = "$pool_number" . " " . $metrics[ $index_asppar1[0] + 4 ] . " " . $metrics[ $index_asppar1[0] + 7 ] . " " . $metrics[ $index_asppar1[0] + 8 ];

    # Architecture
    my %as400_shpool_arch = (
      "item_id"   => $uuid_shpool,
      "label"     => $shrpool_config,
      "hw_type"   => "as400",
      "subsystem" => "shrpool",
      "parents"   => [$uuid]
    );
    push( @architecture, \%as400_shpool_arch );

    # config
    $conf{$uuid_shpool}{paging_option} = $metrics[ $index_asppar1[0] + 3 ];
    $conf{$uuid_shpool}{time}          = $time;
  }

  #
  # JOB WRKACTJOB CPUTOP IOTOP
  # LST300JOB1:::RTV_SYSSTS CZ50257PH  177793:PH_SBS:CZ50257PH:BCH:0.5:RUN:

  (@index_arr) = grep { $metrics[$_] =~ "LST300JOB1" } ( 0 .. @metrics - 1 );
  print "2366 LST300JOB1: @index_arr\n";
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence

      #            Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }

    # we use actually only 3 values
    # my $job_update_pattern = ":percent_used:diskIO_total:proctime_total_ms";
    # in JOB1
    # s/percent_used/$datar[$cycle_var + 7]/;
    # in JOB3
    #  my $proctime_total_ms = 0;
    #  ( my $hours, my $mins, my $secs ) = split( /\./, $datar[ $cycle_var + 4 ], 3 );
    #  my $milis = ( $secs + ( $mins * 60 ) + ( $hours * 3600 ) ) * 1000;
    #  print_as400_debug("$datar[$cycle_var + 4] $hours $mins $secs $milis\n") if $DEBUG == 2;
    #  $job_update =~ s/proctime_total_ms/$milis/;
    # in JOB4
    # s/diskIO_total/$datar[$cycle_var + 4]/;

    my $trio_atom = $metrics[ $index + 3 ];
    $trio_atom =~ s/\s+/_/g;
    my ( $jobname, $job_user, $jobnumber ) = split( "_", $trio_atom );
    print "2388 ,$trio_atom,$jobname,$job_user,$jobnumber,\n";
    my $uuid_job = "$uuid" . "$trio_atom";    # same as in lpar2rrd, repl spaces for underscore

    if ( !Xormon::isdigit( $metrics[ $index + 7 ] ) ) {    # percent used
      Xormon::error(" $metrics[$index] ($index) not number data $metrics[$index+7]");
      print "2393 $metrics[$index] ($index) not number data $metrics[$index+7]\n";
      next;
    }
    my $proctime = $metrics[ $index + 22 ];
    ( my $hours, my $mins, my $secs ) = split( /\./, $proctime, 3 );
    my $proctime_s = ( $secs + ( $mins * 60 ) + ( $hours * 3600 ) );

    # Architecture
    my %as400_job_arch = (
      "item_id"   => $uuid_job,
      "label"     => $jobnumber,
      "hw_type"   => "as400",
      "subsystem" => "job",
      "parents"   => [$uuid]
    );
    push( @architecture, \%as400_job_arch );

    # config
    $conf{$uuid_job}{jobname}     = $jobname;
    $conf{$uuid_job}{job_user}    = $job_user;
    $conf{$uuid_job}{jobnumber}   = $jobnumber;
    $conf{$uuid_job}{jobtype}     = $metrics[ $index + 6 ];
    $conf{$uuid_job}{jobstatus}   = $metrics[ $index + 8 ];
    $conf{$uuid_job}{jobfunction} = $metrics[ $index + 14 ];
    $conf{$uuid_job}{time}        = $time;

    my %job_perf = ();
    $job_perf{percent_used} = $metrics[ $index + 7 ];

    #$job_perf{proctime_total} = $proctime_s;
    #$job_perf{diskIO_total} = $metrics[$index+31];

    if ( exists $counter_data{$uuid_job}{time} ) {
      if ( ( $time - $counter_data{$uuid_job}{time} ) < $max_data_delay ) {    # last data not older x minutes ?
        $job_perf{proctime_total} = ( $proctime_s - $counter_data{$uuid_job}{proctime_total} ) / 300;        # 5 minutes
        $job_perf{diskIO_total}   = ( $metrics[ $index + 31 ] - $counter_data{$uuid_job}{diskIO_total} );    # /60 ?
        $job_perf{proctime_total} = 0 if $job_perf{proctime_total} < 0;
        $job_perf{diskIO_total}   = 0 if $job_perf{diskIO_total} < 0;
      }
      else {
        print "2431 probably longer interval actual: $time last " . $counter_data{$uuid_job}{time} . " for AS400 job $uuid_job\n";
      }
    }
    else {
      print "2435 probably 1st write to AS400 job $uuid_job\n";
    }

    # save for next run
    $counter_data{$uuid_job}{proctime_total} = $proctime_s;
    $counter_data{$uuid_job}{diskIO_total}   = $metrics[ $index + 31 ];
    $counter_data{$uuid_job}{time}           = $time;

    $perf{'as400'}{'job'}{$uuid_job}{$time} = ( \%job_perf ) if %job_perf;
  }

  #
  # LAN
  # IFCD0100PAR1:::ETHLINE:Active:ELAN:B6xC5xD5x19x43x02:4:2:IFCD0100PAR2:::0:0:0:0:0:0:
  # IFCD0100PAR1:::*LOOPBACK:Active:LOOPBACK:00x00x00x00x00x00:0:0:IFCD0100PAR2:::0:0:0:0:0:0:

  (@index_arr) = grep { $metrics[$_] =~ "IFCD0100PAR1" } ( 0 .. @metrics - 1 );
  print "2451 IFCD0100PAR1: @index_arr\n";
  foreach my $index (@index_arr) {
    if ( $index % 9 > 0 ) {    # skip possible coincidence

      #            Xormon::log("skip coincidence \$index $index $metrics[$index+0],$metrics[$index+1],$metrics[$index+2],$metrics[$index+3]");
      next;
    }

    # leave out *LOOPBACK
    print "2458 ,$metrics[$index+3],\n";
    if ( $metrics[ $index + 3 ] eq "*LOOPBACK" ) {
      next;
    }
    if ( Xormon::isdigit( $metrics[ $index + 7 ] ) && Xormon::isdigit( $metrics[ $index + 8 ] ) && Xormon::isdigit( $metrics[ $index + 12 ] ) && Xormon::isdigit( $metrics[ $index + 13 ] ) && Xormon::isdigit( $metrics[ $index + 14 ] ) && Xormon::isdigit( $metrics[ $index + 15 ] ) && Xormon::isdigit( $metrics[ $index + 16 ] ) && Xormon::isdigit( $metrics[ $index + 17 ] ) ) {
      my $uuid_lan = "$uuid" . "_ifc_$metrics[$index+3]";
      my %lan_perf = ();
      $lan_perf{total_bytes_recv}                 = $metrics[ $index + 7 ];
      $lan_perf{total_bytes_sent}                 = $metrics[ $index + 8 ];
      $lan_perf{total_inbound_packets_discarded}  = $metrics[ $index + 14 ];
      $lan_perf{total_outbound_packets_discarded} = $metrics[ $index + 17 ];
      $lan_perf{total_packets_recv}               = $metrics[ $index + 12 ] + $metrics[ $index + 13 ];
      $lan_perf{total_packets_sent}               = $metrics[ $index + 15 ] + $metrics[ $index + 16 ];

      $perf{'as400'}{'lan'}{$uuid_lan}{$time} = ( \%lan_perf ) if %lan_perf;

      # Architecture
      my %as400_lan_arch = (
        "item_id"   => $uuid_lan,
        "label"     => $metrics[ $index + 3 ],
        "hw_type"   => "as400",
        "subsystem" => "lan",
        "parents"   => [$uuid]
      );
      push( @architecture, \%as400_lan_arch );

      # config
      $conf{$uuid_lan}{status}    = $metrics[ $index + 4 ];
      $conf{$uuid_lan}{line_type} = $metrics[ $index + 5 ];
      $conf{$uuid_lan}{mac_addr}  = $metrics[ $index + 6 ];
      $conf{$uuid_lan}{time}      = $time;
    }
    else {
      Xormon::error("AS400 $uuid $time $metrics[$index] not number data IFCD LAN $metrics[$index+7] $metrics[$index+8] $metrics[$index+12] $metrics[$index+13] $metrics[$index+14] $metrics[$index+15] $metrics[$index+16] $metrics[$index+17]");
    }
  }

  print "-----------\n";

  #    print Dumper ("architecture",@architecture);
  #    print Dumper ("conf",\%conf);
  #    print Dumper ("perf",\%perf);

  #save architecture
  if (@architecture) {

    # print Dumper ("2457",@architecture);
    my $rc = $xormon->saveArchitectureAgent( \@architecture );

    # status code
    if ( $rc eq "401" ) {
      Xormon::error("2461 Unauthorized");

      # return ();
    }
    elsif ( $rc ne "201" ) {
      Xormon::error("Error 2508 from $ip_address \$rc $rc");
      print Dumper ( "2509", @architecture );

      # return ();
    }
    else {
      # print "2469 seems to be ok \$rc $rc\n";
    }
  }

  #save configuration
  if (%conf) {

    # print Dumper ("2475",\%conf);
    my $rc = $xormon->saveConfJSON( \%conf );

    # status code
    if ( $rc eq "401" ) {
      Xormon::error("2523 Unauthorized");

      # return ();
    }
    elsif ( $rc ne "201" ) {
      Xormon::error("Error 2526 from $ip_address \$rc $rc");
      print Dumper ( "2527", \%conf );

      # return ();
    }
    else {
      # print "2487 seems to be ok \$rc $rc\n";
    }
  }

  # send data to backend
  if (%perf) {

    # print Dumper ("2493",\%perf);
    my $rc = $xormon->saveJSON2( \%perf );

    # status code
    if ( $rc eq "401" ) {
      Xormon::error("2542 Unauthorized");

      # return ();
    }
    elsif ( $rc ne "201" ) {
      Xormon::error("Error 2545 from $ip_address \$rc $rc");
      print Dumper ( "2546", \%perf );

      # return ();
    }
    else {
      # print "2550 seems to be ok \$rc $rc\n";
      if (%counter_data) {
        if ( Storable::store \%counter_data, "$last_agent_counter_data_file" ) {

          # print "2553 data stored to $last_agent_counter_data_file\n";
        }
        else {
          print "2556 data NOT stored to $last_agent_counter_data_file\n";
        }
      }
    }
  }
  return $time;
}

# my $ret_value = as400("xormon", \$message, "IP");
# print "x530 \$ret_value $ret_value\n";
