package OCEANSTORNAS;

use strict;
use warnings;
use Data::Dumper;
use JSON qw(encode_json decode_json);
use Date::Parse;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
#use lib "/home/xormon/nextgen/perl-modules";
use POSIX ":sys_wait_h";
use LWP::UserAgent;
use File::Copy;
use File::Temp qw(tempfile);
use Test;
use XormonLogger;
use Xormon;
no warnings 'utf8';
my $logger;

my %data;
my $xormon;
my $cfg;
my $stats_found = 0;
my $data_directory = "$FindBin::Bin";
my $debug = 0;

my $storage_name = "";
my $active_minute = "";
my $debug_data = "";
my $debug_data_configuration = "";
my %performance_data;
my %configuration_data;
my $st_serial;
my @architecture = ();
my $service_data_out;
my $st_type = "oceanstornas";
my @rows_api_test = ();
my @rows_api_test_data = ();
my $protocol = "";
my $wrkdir = "";
my $service_data_in;
my $connection_test = 0;
my $cookie = "";
my @health_status = ();
my %hosts;
my @hardware = ();
my $replication_capacity_used = "";
my @health_status_global = ();
my $token = "";
my $device_id = "";
my $api_host = "";
my $port = 8088;
my %performance_statistics;
my %metric_inventory;
my $timestamp_global;
my $enviroment = 1;






sub collect {
  $xormon = shift;
  $cfg    = shift;

  $timestamp_global = get_global_timestamp();
  unless ( exists $cfg->{'hostcfg_id'} )        { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!") && return "error"; }
  $logger = XormonLogger->new( 'oceanstornas', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'node1Host'} )         { $logger->error("Cannot continue, 'node1Host' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )      { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )      { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  my $act_timestamp     = time();
  $service_data_in = $xormon->getData();
  #if (ref(\$service_data_in) eq "SCALAR"){
  #  $service_data_in = {};
  #}
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }

  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  $api_host = $cfg->{'node1Host'};
  if (exists $service_data_in->{api_host} && $service_data_in->{api_host} ne $api_host){
    $api_host = $service_data_in->{api_host};
  }
  if (defined $cfg->{OCEANSTORNAS_ENVIROMENT} && $cfg->{OCEANSTORNAS_ENVIROMENT} eq "development"){
    $enviroment = 0;
  }
  data_structure();
  set_data_perf_list();
  ### login
  login();
  if (!defined $token || $token eq "" || !defined $device_id || $device_id eq "" || !defined $cookie || $cookie eq ""){
    if (defined $cfg->{node2Host}){
      if ($cfg->{node2Host} eq $api_host){
        $api_host = $cfg->{'node1Host'};
      }
      $api_host = $cfg->{'node2Host'};
      login();
      if (!defined $token || $token eq "" || !defined $device_id || $device_id eq "" || !defined $cookie || $cookie eq ""){
        $xormon->isConnected(0, "Connection failed!");
        $logger->error("Cookie, token or device_id is not defined") && return "error";
      }
    }
    else{
      $xormon->isConnected(0, "Connection failed!");
      $logger->error("Cookie, token or device_id is not defined") && return "error";
    }
  }
  ###
  ### check connection
  if (!check_connection()){
    logout();
    $xormon->isConnected(0, "Connection failed!");
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
  $service_data_out->{api_host} = $api_host;
  ###
  #print "$token $device_id $cookie\n";
  $xormon->isConnected(1);
  set_data_pool(); ### capacity only
  set_data_drive();
  set_data_node();
  set_data_port_eth();
  set_data_cluster_nas();
  set_data_device();
  logout();
  delete_empty_metrics();
  calculate_missing_metrics();
  #print Dumper \%performance_data;
  my $rc = $xormon->saveJSON2(\%performance_data);
  my $status = get_status($rc, "saveJSON2 (performance)");

  ### host_cfg_data
  if (Xormon::isdigit($service_data_out->{last_perf_timestamp})){
    if ( ( time() -  $service_data_out->{last_perf_timestamp} ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_out->{last_perf_timestamp});
    }
    else{
      $xormon->hasData(1);
    }
  }
  else{
    $xormon->hasData(0, "no available stats");
  }

  my $save_data = $xormon->saveData(\%{ $service_data_out });
  my $status_save_data = get_status($save_data, "SaveData");
  if (( ! exists $service_data_in->{last_conf_timestamp} ) || ( ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) || ( $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) || ( !-f "$debug_data_configuration/config-$st_serial.out" )){
    `touch "$debug_data_configuration/config-$st_serial.out"`;
    ### login
    login();
    if (!defined $token || $token eq "" || !defined $device_id || $device_id eq "" || !defined $cookie || $cookie eq ""){
      if (defined $cfg->{node2Host}){
        if ($cfg->{node2Host} eq $api_host){
          $api_host = $cfg->{'node1Host'};
        }
        $api_host = $cfg->{'node2Host'};
        login();
        if (!defined $token || $token eq "" || !defined $device_id || $device_id eq "" || !defined $cookie || $cookie eq ""){
          $logger->error("Cookie, token or device_id is not defined") && return "error";
        }
      }
      else{
        $logger->error("Cookie, token or device_id is not defined") && return "error";
      }
    }
    ###
    ### check connection
    if (!check_connection()){
      logout();
      $logger->info("Task for ".$cfg->{hostalias}." is done. Status: error");
      return "error";
    }
    set_data_pool_conf();
    set_data_drive_conf();
    set_data_node_conf();
    set_data_port_eth_conf();
    set_data_cluster_nas_conf();
    set_data_device_conf();
    set_data_health_status();
    logout();
    my @types = ("node", "drive", "cluster", "ethport", "device", "pool");
    foreach my $uid (keys %configuration_data){
      my $active = 0;
      foreach my $type (@types){
        if (defined $performance_data{$st_type}{$type}{$uid}){
          $active++;
          last;
        }
      }
      if ($active == 0){
        delete $configuration_data{$uid};
      }
    }
    $service_data_out->{last_conf_timestamp} = $act_timestamp;
    ($status, my $status_configuration, my $status_health) = set_architecture();
    if ($status eq "successful" && $status_configuration eq "successful" && $status_health eq "successful" ){
      my $save_data = $xormon->saveData(\%{ $service_data_out });
      my $status_save_data = get_status($save_data, "saveData");
    }
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
    return $status;
  }
  $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
  return $status;
}



sub set_data_health_status{
  my $data = cmd("alarm/currentalarm?filter=level::3 5 6", "GET", 1, "alarm");
  #print Dumper \$data;
  if ( !defined $data || $data eq "" || ref($data->{data}) ne "ARRAY" ) {
    $data = cmd("alarm/currentalarm?filter=level::3&level::5&level::6", "GET", 1, "alarm");
  }
  if ( !defined $data || $data eq "" || ref($data->{data}) ne "ARRAY" ) {
    return;

  }

  my $message_global = "";
  my $status_global = "ok";
  my %items;
  $items{eventID} = "event";
  $items{startTime} = "time";
  $items{name} = "name";
  $items{type} = "type";
  $items{level} = "level";
  $items{description} = "description";

  foreach my $element ( @{$data->{data}} ) {
    if ( !defined $element->{eventID} || $element->{eventID} eq "") { next; }
    my %data;
    foreach my $item (sort keys %items){
      if (defined $element->{$item}){
        if ($item eq "type"){
          $data{$items{$item}} = get_type_text($element->{$item});
        }
        elsif($item eq "level"){
          $data{$items{$item}} = get_level_text($element->{$item});
          if ($data{$items{$item}} =~ m/critical|major/){
            $status_global = "error";
            $message_global = $data{description};
          }
        }
        else{
          $data{$items{$item}} = $element->{$item};
        }
      }
      else{
        if ($item eq "level"){
          $data{$items{$item}} = "unknown";
        }
        else{
          $data{$items{$item}} = "";
        }
      }
    }
    #print Dumper \%data;
    push(@health_status,\%data);
  }

  my %vm_status = (
    "item_id" => $st_serial,
    "status" => $status_global,
    "data" => $message_global
  );
  #print "$st_serial | $status_global | $message_global\n";
  push(@health_status_global, \%vm_status);
  #my $rc = $xormon->saveStatus(\@hs);
  #my $status = get_status($rc, "health status");
  #$logger->info("Task healh status ".$cfg->{hostalias}." is done. Status: $status");
  #print Dumper \@health_status_global;
  #print Dumper \@health_status;
  return;
}

sub set_data_device{
  my $type = "cluster";
  my $section = "device";
  my @items = ("read_io","write_io","io_rate","read","write","data_rate","capacity_total", "capacity_used", "capacity_free");
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$type}{$uid}}){
      foreach my $item (@items){
        my $value = $performance_data{$st_type}{$type}{$uid}{$timestamp}{$item};
        my $io_rate = $performance_data{$st_type}{$type}{$uid}{$timestamp}{io_rate};
        if (Xormon::isdigit($value)){
          if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$st_serial} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item}){
            $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item} = $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item} + $value;
          }
          else{
            $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item} = $value;
          }
        }
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub get_status {
  my $rc = shift;
  my $task = shift;
  if ($rc eq "201" || $rc eq "200") {
    $logger->info("Task for $task ".$cfg->{hostalias}." is done. Status: successful");
    return "successful";
  } elsif ($rc eq "401") {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: unauthorized");
    return "unauthorized";
  } else {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
}

sub set_data_drive_conf{
  my $section = "drive";
  my $config_data          = cmd("Disk", "GET", 1, "Disk");
  if ( $config_data eq "" ) {
    $config_data = cmd("disk", "GET", 1, "disk");
  }
  if ($config_data eq ""){return;}
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id  = $element_id->{ID};
    my $type = $element_id->{TYPE};
    my $health_status    = $element_id->{"HEALTHSTATUS"};
    my $running_status   = $element_id->{"RUNNINGSTATUS"};
    my $model            = $element_id->{"MODEL"};
    my $serial_number    = $element_id->{"SERIALNUMBER"};
    if ( !defined $id || !defined $type ) { next; }
    my $name = $element_id->{NAME};
    if (!defined $name){$name = $id;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    $configuration_data{$uid}{id} = $id;
    $configuration_data{$uid}{label} = $name;
    $configuration_data{$uid}{TYPE} = $section;
    my %capacity = get_capacity_section($element_id,$section);
    foreach my $item (keys %capacity){
      $configuration_data{$uid}{$item} = $capacity{$item};
    }
    if ( defined $health_status && $health_status ne "" ) {
      $configuration_data{$uid}{"health_status"} = get_health_status_text($health_status);
    }
    if ( defined $running_status && $running_status ne "" ) {
      $configuration_data{$uid}{"running_status"} = get_running_status_text($running_status);
    }
    if (defined $model){
      $configuration_data{$uid}{"model"} = $model;
    }
    if (defined $serial_number){
      $configuration_data{$uid}{"serial_number"} = $serial_number;
    }
  }
  #print Dumper \%configuration_data;
}


sub set_data_node_conf{
  my $section = "node";
  my $config_data          = cmd("sys_node", "GET", 1, "sys_node");
  if ( $config_data eq "" ) { return; }
  #print Dumper \$config_data;
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id  = $element_id->{id};
    my $health_status    = $element_id->{"healthStatus"};
    my $running_status   = $element_id->{"runningStatus"};
    my $serial_number    = $element_id->{"devSn"};
    if ( !defined $id ) { next; }
    my $name = $element_id->{name};
    if (!defined $name){$name = $id;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    $configuration_data{$uid}{id} = $id;
    $configuration_data{$uid}{label} = $name;
    $configuration_data{$uid}{TYPE} = $section;
    if ( defined $health_status && $health_status ne "" ) {
      $configuration_data{$uid}{"health_status"} = get_health_status_text($health_status);
    }
    if ( defined $running_status && $running_status ne "" ) {
      $configuration_data{$uid}{"running_status"} = get_running_status_text($running_status);
    }
    if (defined $serial_number && $serial_number ne ""){
      $configuration_data{$uid}{"serial_number"} = $serial_number;
    }
  }
  #print Dumper \%configuration_data;
}

sub sum_metric{
  my $metric = shift;
  my $data_tmp = shift;
  my @data = @{$data_tmp};
  my $response_metric = shift;
  my $numerator = shift;
  my $sum = "";
  my $numerator_sum = "";
  my $index = 0;
  foreach my $element (@data){
    if (Xormon::isdigit($element->{$metric})){
      #if ($element->{$metric} == 0){
      #  $element->{$metric} = int(rand(300));
      #}
      if (Xormon::isdigit($sum)){
        $sum = $sum + $element->{$metric};
        $index++;
      }
      else{
        $sum = $element->{$metric};
        $index++;
      }
      if (defined $response_metric && Xormon::isdigit($element->{$response_metric})){
        #if ($element->{$response_metric} == 0){
        #  my $x = 1000 + int(rand(3000 - 1000));
        #  $element->{$response_metric} = $x;
        #}
        if (Xormon::isdigit($numerator_sum)){
          $numerator_sum = $numerator_sum + ($element->{$response_metric} * $element->{$metric});
        }
        else{
          $numerator_sum = $element->{$response_metric} * $element->{$metric};
        }
      }
    }
  }
  if ($index == 0){return ("","","");}
  return ($sum / $index, $numerator_sum, $sum);
}

sub sum_metrics{
  my $data_tmp = shift;
  my $metric1 = shift;
  my $metric2 = shift;
  my %data = %{$data_tmp};
  my $sum = "";
  if (Xormon::isdigit($data{$metric1})){
    $sum = $data{$metric1};
  }
  if (Xormon::isdigit($data{$metric2})){
    if (Xormon::isdigit($sum)){
      $sum = $sum + $data{$metric2};
    }
    else{
      $sum = $data{$metric2};
    }
  }
  return $sum;
}

sub calculate_weighted_average{
  my $data_tmp = shift;
  my $numerator_metric1 = shift;
  my $denominator_metric1 = shift;
  my $numerator_metric2 = shift;
  my $denominator_metric2 = shift;


  my %data = %{$data_tmp};

  my $sum = "";
  my $denominator = "";
  if (Xormon::isdigit($data{$denominator_metric1}) && Xormon::isdigit($data{$numerator_metric1})){
    $sum = $data{$denominator_metric1} * $data{$numerator_metric1};
    $denominator = $data{$denominator_metric1};
  }
  if (Xormon::isdigit($data{$denominator_metric2}) && Xormon::isdigit($data{$numerator_metric2})){
    if (Xormon::isdigit($sum)){
      $sum = $sum + ($data{$denominator_metric2} * $data{$numerator_metric2});
      $denominator = $denominator + $data{$denominator_metric2};
    }
    else{
      $sum = $data{$denominator_metric2} * $data{$numerator_metric2};
      $denominator = $data{$denominator_metric2};
    }
  }
  if (Xormon::isdigit($sum) && Xormon::isdigit($denominator)){
    if ($denominator == 0){
      return 0;
    }
    return $sum / $denominator;
  }
  return "";
}

sub set_data_port_eth_conf{

  my $section = "ethport";
  my $config_data          = cmd("eth_port", "GET", 1, "eth_port");
  if ($config_data eq ""){return;}
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id             = $element_id->{"ID"};
    my $name           = $element_id->{"LOCATION"};
    my $wwn            = $element_id->{"WWN"};
    my $health_status  = $element_id->{"HEALTHSTATUS"};
    my $speed          = $element_id->{"RUNSPEED"};
    my $running_status = $element_id->{"RUNNINGSTATUS"};
    if ( !defined $id || !defined $name) { next; }
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    $configuration_data{$uid}{id} = $id;
    $configuration_data{$uid}{label} = "$id $name";
    $configuration_data{$uid}{TYPE} = $section;
    if (defined $wwn){
      $configuration_data{$uid}{wwn} = $wwn;
    }
    if ( defined $speed && $speed ne "" ) {
      if ( Xormon::isdigit($speed) && $speed !~ m/^-/ ) {
        my $pom = round( $speed / 1000, 0 );
        $speed = $pom;
      }
      if ( $speed eq "-1" ) {
        $speed = "port works incorrectly";
      }
      $configuration_data{$uid}{"port_speed(Gbit/s)"} = $speed;
    }
    if ( defined $health_status && $health_status ne "" ) {
      $configuration_data{$uid}{"health_status"} = get_health_status_text($health_status);
    }
    if ( defined $running_status && $running_status ne "" ) {
      $configuration_data{$uid}{"running_status"} = get_running_status_text($running_status);
    }
  }
  #print Dumper \%configuration_data;

}


sub sum_all_metrics {

  my $data_tmp = shift;
  my %data     = %{$data_tmp};
  my $sum      = 0;
  foreach my $item ( keys %data ) {
    if ( Xormon::isdigit( $data{$item} ) ) {
      $sum = $sum + $data{$item};
    }
  }
  return $sum;
}

sub set_data_cluster_nas_conf{

  my $section = "cluster";
  my $config_data          = cmd("cluster_nas_service", "GET", 1, "cluster_nas_service");
  if ( $config_data eq "" || !defined $config_data->{data} || ref( $config_data->{data} ) ne "ARRAY" ) { return; }  
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id_key   = "ID";
    if ( !defined $element_id->{$id_key} ) {
      $id_key = "id";
    }
    my $id     = $element_id->{$id_key};
    my $name           = $element_id->{name};
    my $health_status  = $element_id->{healthStatus};
    my $running_status = $element_id->{"runningStatus"};
    if ( !defined $id) { next; }
    if (!defined $name){$name = $id};
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    $configuration_data{$uid}{id} = $id;
    $configuration_data{$uid}{label} = $name;
    $configuration_data{$uid}{TYPE} = $section;
    if ( defined $health_status && $health_status ne "" ) {
      $configuration_data{$uid}{"health_status"} = get_health_status_text($health_status);
    }
    if ( defined $running_status && $running_status ne "" ) {
      $configuration_data{$uid}{"running_status"} = get_running_status_text($running_status);
    }
    my %capacity = get_capacity_section($element_id,$section);
    foreach my $item (keys %capacity){
      $configuration_data{$uid}{$item} = $capacity{$item};
      $configuration_data{$st_serial}{$item} = $capacity{$item};
    }
    #$configuration_data{$st_serial}{id} = $id;
    #$configuration_data{$st_serial}{cluster_name} = $name;
  }
  #print Dumper \%configuration_data;

}

sub set_data_device_conf{
  my $conn_test = shift;
  if (!defined $conn_test){$conn_test = 0;}

  my %items;
  $items{HEALTHSTATUS} = "health_status";
  $items{RUNNINGSTATUS} = "running_status";
  $items{PRODUCTVERSION} = "version";
  $items{PRODUCTMODE} = "model";
  $items{ID} = "id";
  $items{NAME} = "node_name";

  my $section = "device";
  my $data = cmd("system/", "GET", $conn_test, "system");
  #print Dumper \$data;
  if ( $data eq "" ) {
    return;
  }

  if (ref($data->{data}) eq "HASH"){
    foreach my $item (keys %items){
      if (defined $data->{data}->{$item}){
        if ($items{$item} eq "model"){
          $configuration_data{$st_serial}{$items{$item}} = get_model($data->{data}->{$item});
        }
        elsif($items{$item} =~ m/health_status/){
          $configuration_data{$st_serial}{$items{$item}} = get_health_status_text($data->{data}->{$item});
        }
        elsif($items{$item} eq "running_status"){
          $configuration_data{$st_serial}{$items{$item}} = get_running_status_text($data->{data}->{$item});
        }
        else{
          $configuration_data{$st_serial}{$items{$item}} = $data->{data}->{$item};
        }
      }
      else{
        $configuration_data{$st_serial}{$items{$item}} = "";
      }
    }
  }
}

sub set_data_cluster_nas{
  my $section = "cluster";
  my $config_data          = cmd("cluster_nas_service", "GET", 0, "cluster_nas_service");
  if ($config_data eq "" || !defined $config_data->{data} || ref( $config_data->{data} ) ne "ARRAY"){return;}
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id_key   = "ID";
    my $type_key = "TYPE";
    if ( !defined $element_id->{$id_key} ) {
      $id_key = "id";
    }
    if ( !defined $element_id->{$type_key} ) {
      $type_key = "type";
    }
    my $id_element     = $element_id->{$id_key};
    my $type_element   = $element_id->{$type_key};
    if ( !defined $id_element || !defined $type_element ) { next; }
    my $cmd_perf = "performace_statistic/cur_statistic_data?CMO_STATISTIC_UUID=$type_element:$id_element&CMO_STATISTIC_DATA_ID_LIST=$performance_statistics{$section}";
    my $data = cmd($cmd_perf,"GET", 1, "$type_element" . "_$id_element");
    if ( $data eq "" ) { next; }
    my $uid = Xormon::create_hash_key($st_serial.$section.$id_element);
    $performance_data{$st_type}{$section}{$uid} = get_performance_data($data);
    my $timestamp = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
    if (Xormon::isdigit($timestamp)){
      my %capacity = get_capacity_section($element_id,$section);
      foreach my $item (keys %capacity){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{$item} = $capacity{$item};
      }
    }
  }
  #print Dumper \%performance_data;
}

sub set_data_port_eth{

  my $section     = "ethport";
  my $config_data          = cmd("eth_port", "GET", 0, "eth_port");
  if ($config_data eq ""){return;}
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id  = $element_id->{ID};
    my $type = $element_id->{TYPE};
    if ( !defined $id || !defined $type ) { next; }
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    my $cmd_perf = "performace_statistic/cur_statistic_data?CMO_STATISTIC_UUID=$type:$id&CMO_STATISTIC_DATA_ID_LIST=$performance_statistics{$section}";
    my $data = cmd($cmd_perf,"GET", 1, "$type" . "_$id");
    if ( $data eq "" ) { next; }
    $performance_data{$st_type}{$section}{$uid} = get_performance_data($data);
  }
  #print Dumper \$performance_data{$st_type}{$section};
}


sub set_data_drive {
  my $section     = "drive";
  my $config_data          = cmd("Disk", "GET", 0, "Disk");
  if ( $config_data eq "" ) {
    $config_data = cmd("disk", "GET", 0, "disk");
  }
  if ( $config_data eq "" ) { return; }
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id  = $element_id->{ID};
    my $type = $element_id->{TYPE};
    if ( !defined $id || !defined $type ) { next; }
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);

    my $cmd_perf = "performace_statistic/cur_statistic_data?CMO_STATISTIC_UUID=$type:$id&CMO_STATISTIC_DATA_ID_LIST=$performance_statistics{$section}";
    my $data = cmd($cmd_perf,"GET", 1, "$type" . "_$id");  ### for develop only id but change "$type" . "_$id"
    if ( $data eq "" ) { next; }
    $performance_data{$st_type}{$section}{$uid} = get_performance_data($data);
    my $timestamp = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
    if (Xormon::isdigit($timestamp)){
      my %capacity = get_capacity_section($element_id,$section);
      foreach my $item (keys %capacity){
        $performance_data{$st_type}{$section}{$uid}{$timestamp}{$item} = $capacity{$item};
      }
    }
  }
}
sub set_data_pool {
  my $section     = "pool";
  my $config_data          = cmd("nodePool", "GET", 0, "nodePool");
  if ( $config_data eq "" ) { return; }
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id  = $element_id->{id};
    if ( !defined $id ) { next; }
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    my $timestamp = time();
    my %capacity = get_capacity_section($element_id,$section);
    foreach my $item (keys %capacity){
      $performance_data{$st_type}{$section}{$uid}{$timestamp}{$item} = $capacity{$item};
    }
  }
}
sub set_data_pool_conf {
  my $section     = "pool";
  my $config_data          = cmd("nodePool", "GET", 1, "nodePool");
  if ( $config_data eq "" ) { return; }
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id  = $element_id->{id};
    my $name = $element_id->{name};
    if ( !defined $id ) { next; }
    if (!defined $name || $name eq ""){$name = $id;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    my %capacity = get_capacity_section($element_id,$section);
    $configuration_data{$uid}{id} = $id;
    $configuration_data{$uid}{label} = $name;
    $configuration_data{$uid}{TYPE} = $section;
    foreach my $item (keys %capacity){
      $configuration_data{$uid}{$item} = $capacity{$item};
    }
  }
}

sub set_data_node {
  my $section     = "node";
  my $config_data          = cmd("sys_node", "GET", 0, "sys_node");
  if ( $config_data eq "" ) { return; }
  #print Dumper $config_data;
  foreach my $element_id ( @{ $config_data->{data} } ) {
    my $id  = $element_id->{id};
    my $type = $element_id->{type};
    if ( !defined $id || !defined $type ) { next; }
    my $uid = Xormon::create_hash_key($st_serial.$section.$id);
    #print "$st_serial | $section | $id | $uid\n";

    my $cmd_perf = "performace_statistic/cur_statistic_data?CMO_STATISTIC_UUID=$type:$id&CMO_STATISTIC_DATA_ID_LIST=$performance_statistics{$section}";
    my $data = cmd($cmd_perf,"GET", 1, "$type" . "_$id");
    if ( $data eq "" ) { next; }
    $performance_data{$st_type}{$section}{$uid} = get_performance_data($data);
  }
  #print Dumper \$enclosure_data;
  # NAS section
  #print Dumper \$performance_data{$st_type}{$section};
}

sub get_performance_data{
  my $data = shift;
  my $section = shift;
  my %data_out;
  if (defined $section && $section =~ m/Controller/){
    $section =~ s/Controller//g;
    $section         = lc($section);
  }
  if (!defined $data || $data eq "" || ref($data->{data}) ne "ARRAY"){return \%data_out;}
  foreach my $element ( @{ $data->{data} } ) {
    my @statistics = ();
    my @statistics_id = ();
    my $timestamp = "";
    if ( !defined $element->{CMO_STATISTIC_DATA_LIST} || !defined $element->{CMO_STATISTIC_DATA_ID_LIST} || !defined $element->{CMO_STATISTIC_TIMESTAMP} ){
      if ( defined $element->{CMO_STATISTIC_DATA_ID} && defined $element->{CMO_STATISTIC_TIMESTAMP} && defined $element->{CMO_STATISTIC_DATA} ) {
        @statistics    = split( ",", $element->{CMO_STATISTIC_DATA} );
        @statistics_id = split( ",", $element->{CMO_STATISTIC_DATA_ID} );
        $timestamp     = $element->{CMO_STATISTIC_TIMESTAMP};
      }
    }
    else{
      @statistics    = split( ",", $element->{CMO_STATISTIC_DATA_LIST} );
      @statistics_id = split( ",", $element->{CMO_STATISTIC_DATA_ID_LIST} );
      $timestamp     = $element->{CMO_STATISTIC_TIMESTAMP};
    }
    
    if (!$enviroment){
      $timestamp = $timestamp_global;
    }
    my $index = -1;
    foreach my $statistic (@statistics_id) {
      $index++;
      if (defined $section && $section =~ m/nfs|smb/){
        $statistic = $section . "_" . $statistic;
      }
      if (!defined $metric_inventory{$statistic} ){
        next;
      }
      if ( $metric_inventory{$statistic} =~ m/^read$|^write$|^fs_read$|^fs_write$|^fs_data_rate$/ ) {
        $data_out{$timestamp}{ $metric_inventory{$statistic} } = $statistics[$index] * 1000;
        next;
      }
      if ( $metric_inventory{$statistic} =~ m/^resp_t/ ) {
        $data_out{$timestamp}{ $metric_inventory{$statistic} } = round( $statistics[$index] / 1000 );    ### micro to mili
      }
      else {
        $data_out{$timestamp}{ $metric_inventory{$statistic} } = $statistics[$index];
      }
    }
  }
  foreach my $t (keys %data_out){
    if ( Xormon::isdigit($data_out{$t}{block_size_r}) && Xormon::isdigit( $data_out{$t}{read_io} ) && !Xormon::isdigit( $data_out{$t}{read} ) ) {
      $data_out{$t}{block_size_r} = $data_out{$t}{block_size_r} * 1000;
      $data_out{$t}{read} = $data_out{$t}{block_size_r} * $data_out{$t}{read_io};
    }
    if ( Xormon::isdigit($data_out{$t}{block_size_w}) && Xormon::isdigit( $data_out{$t}{write_io} ) && !Xormon::isdigit( $data_out{$t}{write} ) ) {
      $data_out{$t}{block_size_w} = $data_out{$t}{block_size_w} * 1000;
      $data_out{$t}{write} = $data_out{$t}{block_size_w} * $data_out{$t}{write_io};
    }
    my $data_rate = "";
    if (Xormon::isdigit($data_out{$t}{read})){
      $data_rate = $data_out{$t}{read};
    }
    if (Xormon::isdigit($data_out{$t}{write})){
      if (Xormon::isdigit($data_rate)){
        $data_rate += $data_out{$t}{write};
      }
      else{
        $data_rate = $data_out{$t}{write};
      }
    }
    if (Xormon::isdigit($data_rate)){
      $data_out{$t}{data_rate} = $data_rate;
    }
  }

  return \%data_out;

}

sub get_capacity_section{
  my $data = shift;
  my $section = shift;
  my %data_out;

  if ($section eq "drive"){
    my $count_sectors    = $data->{"SECTORS"};
    my $size_sector      = $data->{"SECTORSIZE"};
    my $used_cap_percent = $data->{"CAPACITYUSAGE"};
    if ( Xormon::isdigit($size_sector) && Xormon::isdigit($count_sectors) && Xormon::isdigit($used_cap_percent)) {
      my $pom = ( $size_sector * $count_sectors );    ## in B
      my $cap = round( $pom, 0 );
      $pom      = ( $used_cap_percent / 100 ) * $cap;
      my $used_cap = round( $pom,             0 );
      my $free_cap = round( $cap - $used_cap, 0 );
      $data_out{capacity_total} = $cap;
      $data_out{capacity_used} = $used_cap;
      $data_out{capacity_free}= $free_cap;
    }
    elsif (Xormon::isdigit($data->{diskSize}) && Xormon::isdigit($data->{diskAvailableSize}) && Xormon::isdigit($data->{diskUsedSize})){
      $data_out{capacity_total} = $data->{diskSize} * 1024 * 1024;
      $data_out{capacity_used} = $data->{diskUsedSize} * 1024 * 1024;
      $data_out{capacity_free} = $data->{diskAvailableSize} * 1024 * 1024;

    }
  }
  if ($section eq "cluster"){
    my $cap          = $data->{"SystemCapacity"};
    my $used_cap     = $data->{"SystemUsedCapacity"};
    my $free_cap  = "";
    if ( Xormon::isdigit($cap) && Xormon::isdigit($used_cap) && $cap >= $used_cap ) {
      $free_cap = $cap - $used_cap;
    }
    if (Xormon::isdigit($cap)){
      $data_out{capacity_total} = $cap * 1024 * 1024;
    }
    if (Xormon::isdigit($used_cap)){
      $data_out{capacity_used} = $used_cap * 1024 * 1024;
    }
    if (Xormon::isdigit($free_cap)){
      $data_out{capacity_free} = $free_cap * 1024 * 1024;
    }
  }
  if ($section eq "pool"){
    my $total = $data->{totalHDDCap};
    my $used  = $data->{usedHDDCap};
    if (Xormon::isdigit($total) && Xormon::isdigit($used)){
      $data_out{capacity_total} = $total * 1024 * 1024;
      $data_out{capacity_used} = $used * 1024 * 1024;
      if ($total >= $used){
        $data_out{capacity_free} = ($total - $used) * 1024 * 1024;
      }
    }
  }
  return %data_out;
}


sub delete_empty_metrics{
  my $first_time = "";
  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        if ($first_time eq "" && Xormon::isdigit($time)){
          $first_time = $time;
        }
        if (Xormon::isdigit($first_time) && Xormon::isdigit($time) && $time < $first_time){
          $first_time = $time;
        }
        foreach my $item (keys %{$performance_data{$st_type}{$type}{$uid}{$time}}){
          if (!Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item}) || $performance_data{$st_type}{$type}{$uid}{$time}{$item} < 0){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
          if ($item =~ m/numerator|denominator/){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
        }
      }
      if (!keys %{$performance_data{$st_type}{$type}{$uid}}){
        delete $performance_data{$st_type}{$type}{$uid};
        ### poznamka
      }
    }
  }
  if (Xormon::isdigit($first_time)){
    $service_data_out->{last_perf_timestamp} = $first_time;
  }
  else{
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      $service_data_out->{last_perf_timestamp} = $service_data_in->{last_perf_timestamp};
    }
  }
}

sub calculate_missing_metrics{

  my $item_total = "capacity_total";
  my $item_used = "capacity_used";
  my $item_used_percent = "capacity_used_percent";

  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        #capacity_used_percent
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_used})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_total} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_used} / $performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = round($pom,2);
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{data_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && $performance_data{$st_type}{$type}{$uid}{$time}{io_rate} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{data_rate} / $performance_data{$st_type}{$type}{$uid}{$time}{io_rate},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{read_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{read} / $performance_data{$st_type}{$type}{$uid}{$time}{read_io},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{write_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{write} / $performance_data{$st_type}{$type}{$uid}{$time}{write_io},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = 0;
        }
      }
    }
  }
}

sub logout{
  my $cmd = "sessions";
  cmd($cmd,"DELETE",1,"close_session");
}

sub login{
  my $cmd =  "xxxxx/sessions";
  my %hash_filter;
  $hash_filter{"username"} = $cfg->{apiUsername};
  $hash_filter{"password"} = $cfg->{apiPassword};
  $hash_filter{"scope"}    = "0";
  my $json_body = encode_json( \%hash_filter );
  my $data = cmd($cmd, "POST", 1, "sessions", $json_body);
  if ($data eq ""){return;}
  if (defined $data->{data}->{deviceid}){
    $device_id = $data->{data}->{deviceid};
  }
  if (defined $data->{data}->{iBaseToken}){
    $token     = $data->{data}->{iBaseToken};
  }
}

sub cmd{
  my $cmd = shift;
  my $type = shift;
  my $execute = shift;
  my $debug_file = shift;
  my $json_body = shift;
  if (!defined $type || $type eq ""){$type = "GET";}
  if (!defined $execute || $execute eq ""){
    $execute = 1;
  }
  ### debug
  if (!$enviroment){
    $execute = 0;
  }
  if (!defined $debug_file){$debug_file = "";}
  if (!$enviroment && $debug_file =~ m/_/ && $execute == 0 && $debug_file !~ m/close_session|sys_node|eth_port|^cluster_nas/){
    my (undef, $postfix) = split("_",$debug_file);
    $debug_file = $postfix;
  }
  if (!$enviroment && $cmd =~ m/213:/){
    my (undef, $postfix, $postfix2) = split("-",$debug_file);
    if (defined $postfix2){
      $debug_file = "$postfix-$postfix2";
      $debug_file =~ s/\.//g;
    }
    else{
      $debug_file = $postfix;
    }
  }
  if ($execute == 0 && -f "$debug_data_configuration/$debug_file.json"){
    if ($cmd =~ m/sessions/ && !$enviroment){
      $cookie = "123456789";
    }
    return Xormon::read_json("$debug_data_configuration/$debug_file.json");
  }
  elsif(-f "$debug_data/$debug_file.json" && $execute == 0){
    return Xormon::read_json("$debug_data/$debug_file.json");
  }
  elsif ($debug_file eq "close_session" && !$enviroment){return;}
  #print "file $debug_data_configuration/$debug_file.json or $debug_data/$debug_file.json do not exist!\n";
  return "";
  my $url = "https://$api_host:$port/deviceManager/rest/$device_id/$cmd";
  if ($cmd =~ m/xxxxx/){
    $url = "https://$api_host:$port/deviceManager/rest/$cmd";
  }
  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( $type => $url );
  $req->header( 'Content-Type' => 'application/json' );
  $req->header( 'Accept'       => 'application/json' );
  $req->header( 'Connection'   => 'keep-alive' );
  $req->header( 'Accept-Language' => 'en' );
  if (defined $json_body && $json_body ne ""){
    $req->content($json_body);
  }
  if (defined $cookie && $cookie ne ""){
    $req->header( 'cookie' => $cookie );
  }
  if (defined $token && $token ne ""){
    $req->header( 'iBaseToken'   => $token );
  }
  my $res = $ua->request($req);
  #print $res->content;
  if ( ! $res->is_success ) {
    if (!$connection_test){
      $logger->error("API command $cmd failed");
    }
    else{
      push(@rows_api_test,"API command $cmd failed\n");
      push(@rows_api_test,$res->content . "\n");
      push(@rows_api_test_data,"API command $cmd failed\n");
      push(@rows_api_test_data,$res->content . "\n");
    }
    my $out = $res->content;
    $logger->error($out);
    return "";
  }
  if ($cmd =~ m/xxxxx/){
    $cookie    = $res->headers->{"set-cookie"};
  }
  my $data = decode_json( $res->content );
  if (defined $debug_file && $debug_file ne ""){
    if ($debug_file =~ m/_/){
      eval {Xormon::save_as_json("$debug_data/$debug_file.json",$data);};
    }
    else{
      eval {Xormon::save_as_json("$debug_data_configuration/$debug_file.json",$data);};
    }
    if ($@){
      $logger->error("Save data to file $debug_data/$debug_file.json failed");
    }
  }
  if (!defined $data->{data}){
    print Dumper \$data;
  }
  if ((defined $data->{data} && ref($data->{data}) eq "HASH" && defined $data->{error}->{code} && $data->{error}->{code} ne "0") || ( defined $data->{data} && ref( $data->{data} ) ne "ARRAY" && defined $data->{error}->{code} && $data->{error}->{code} ne "0")){
    my $error_description = $data->{error}->{description};
    my $error_code        = $data->{error}->{code};
    if ( defined $error_code && $error_code eq "83890432" ) {
      return $error_code;
    }
    if (!$connection_test){
      $logger->error("No data found for cmd $cmd  description : $error_description error_code : $error_code");
    }
    else{
      push(@rows_api_test,"No data found for cmd $cmd  description : $error_description error_code : $error_code\n");
      push(@rows_api_test_data,"No data found cmd $cmd  description : $error_description error_code : $error_code\n");
    }
    return "";
  }
  return $data;
}

sub set_architecture{

  my %device_arch = (
    "item_id" => $cfg->{'uuid'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => $st_type,
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'uuid'}
  );
  #if (defined $cfg->{serial_number}){
  #  $device_arch{serial_number} = $cfg->{serial_number};
  #}
  #if (defined $cfg->{version}){
  #  $device_arch{version} = $cfg->{version};
  #}
  #if (defined $cfg->{model}){
  #  $device_arch{model} = $cfg->{model};
  #}
  push(@architecture, \%device_arch);
  #print Dumper \$configuration_data{$st_serial};
  #return;

  foreach my $uid (keys %configuration_data){
    my $name = $configuration_data{$uid}{label};
    my $type = $configuration_data{$uid}{TYPE};
    if (!defined $name || !defined $type){next;}
    my @children = ();
    my @parents = ();
    if (defined $configuration_data{$uid}{children}){
      @children = @{$configuration_data{$uid}{children}};
      ## delete children and parent from item_properties
      delete $configuration_data{$uid}{children};

    }
    if (defined $configuration_data{$uid}{parents}){
      @parents = @{$configuration_data{$uid}{parents}};
      delete $configuration_data{$uid}{parents};

    }
    #print Dumper \@children;
    #print Dumper \@parents;
    my %arch = (
      "item_id" => $uid,
      "label" => $name,
      "hw_type" => $st_type,
      "subsystem" => $type,
      "hostcfg_id" => $cfg->{'uuid'},
      "parents" => [$cfg->{'uuid'}]
      );
    if (@parents){
      push(@{$arch{parents}},@parents);
    }
    if (@children){
      push(@{$arch{children}},@children);
    }
    if ($type eq "port_type"){
      $arch{port_type} = $name;
    }
    if (($type eq "port" || $type eq "volume" ) && defined $configuration_data{$uid}{wwn}){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
      $configuration_data{$uid}{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
    }
    #push(@{$arch{parents}},$cfg->{'uuid'});
    push(@architecture, \%arch);

  }
  #event log
  $configuration_data{$cfg->{'uuid'}}{events} = encode_json(\@health_status);

  my $rc = $xormon->saveConfJSON(\%configuration_data);
  my $status_configuration = get_status($rc, "saveConfJSON (configuration)");
  

  $rc = $xormon->saveArchitecture(\@architecture);
  my $status_architecture = get_status($rc, "saveArchitecture");
  #$logger->info("Task for saveArchitecture ".$cfg->{hostalias}." is done. Status: $status");

  $rc = $xormon->saveStatus(\@health_status_global);
  my $status_health = get_status($rc, "health status");
  return ($status_architecture, $status_configuration, $status_health);
}


sub connection_test {
  my $client = shift;
  $cfg = shift;
  $connection_test = 1;
  if (defined $cfg->{OCEANSTORNAS_ENVIROMENT} && $cfg->{OCEANSTORNAS_ENVIROMENT} eq "development"){
    $enviroment = 0;
  }

  my $test = Test->new($client);
  my @variables = ("hostcfg_id", "hostalias", "node1Host", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  $logger = XormonLogger->new( 'oceanstornas', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  $storage_name = $cfg->{hostalias};
  $st_serial = $cfg->{hostcfg_id};
  data_structure();

  #### connection network test ####
  my $tcp_text = "";
  my ($code1, $text) = Xormon::conntest_tcp($cfg->{node1Host},$port);
  if (!$enviroment){
    $code1 = 1;
  }
  $tcp_text = $tcp_text . "$text\n";
  if (defined $cfg->{node2Host}){
    my ($code2, $text) = Xormon::conntest_tcp($cfg->{node2Host},$port);
    $tcp_text = $tcp_text . "$text\n";
    if ($code1 == 0 && $code2 == 0){
      $test->addSection("API host network connection", 0, $tcp_text);
      return $test->stop();
    }
    $test->addSection("API host network connection", 1, $tcp_text);
  }
  else{
    $test->addSection("API host network connection", $code1, $tcp_text);
    if ($code1 == 0){
      return $test->stop();
    }
  }
  ### API authorization
  $api_host = $cfg->{'node1Host'};
  login();
  if (!defined $token || $token eq "" || !defined $device_id || $device_id eq "" || !defined $cookie || $cookie eq ""){
    if (defined $cfg->{node2Host}){
      if ($cfg->{node2Host} eq $api_host){
        $api_host = $cfg->{'node1Host'};
      }
      $api_host = $cfg->{'node2Host'};
      login();
      if (!defined $token || $token eq "" || !defined $device_id || $device_id eq "" || !defined $cookie || $cookie eq ""){
        push(@rows_api_test, "Cookie, token or device_id is not defined\n");
        $test->addSection("API authorization", 0, \@rows_api_test);
        return $test->stop();
      }
    }
    else{
      push(@rows_api_test, "Cookie, token or device_id is not defined\n");
      $test->addSection("API authorization", 0, \@rows_api_test);
      return $test->stop();
    }
  }
  set_data_device_conf();
  if (%configuration_data){
    my %items;
    $items{4}{version} = "Version";
    $items{3}{model} = "Model";
    $items{1}{id} = "ID";
    $items{2}{node_name} = "Name";
    $items{5}{health_status} = "Health status";
    $items{6}{running_status} = "Running status";
    foreach my $number (sort keys %items){
      foreach my $item (keys %{$items{$number}}){
        my $line;
        if (defined $configuration_data{$st_serial}{$item}){
          $line = $items{$number}{$item} . " : ". $configuration_data{$st_serial}{$item};
        }
        else{
          $line = $items{$number}{$item} . " : ";
        }
        push(@rows_api_test,"$line\n");
      }
    }
    $test->addSection("API authorization", 1, \@rows_api_test);
  }
  else{
    logout();
    $test->addSection("API authorization", 0, \@rows_api_test);
    return $test->stop();
  }
  ###
  ### API test data
  set_data_perf_list();
  if (!check_connection()){
    $test->addSection("API test data", 0, \@rows_api_test_data);
    return $test->stop();
  }
  else{
    $test->addSection("API test data", 1, "Performance data ok");
  }
  ###

  return $test->stop();
  #return Xormon::conn_test($connection_test, \@rows_api_test);
}

sub data_structure {
  if (!-d "$data_directory/data"){
    mkdir "$data_directory/data" || $logger->error( "Cannot mkdir $data_directory/data: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial"){
    mkdir "$data_directory/data/$st_serial" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial/IOSTATS"){
    mkdir "$data_directory/data/$st_serial/IOSTATS" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial/IOSTATS: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }


  my $directory = "$data_directory/data/$st_serial/IOSTATS";
  if ( !-d $directory ) {
    mkdir $directory || $logger->error( "Cannot mkdir $directory: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || $logger->error( "Cannot mkdir $directory/configuration: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || $logger->error( "Cannot mkdir $directory/0$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || $logger->error( "Cannot mkdir $directory/$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
  my @entry_time = split( " ", localtime() );
  ( undef, $active_minute, undef ) = split( ":", $entry_time[3] );
  if ( !-d "$directory/$active_minute" ) {
    mkdir "$directory/$active_minute" || $logger->error( "Cannot mkdir $directory/$active_minute: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  #$debug_data_performance = "$directory/$active_minute";
  #$debug_data = "$directory/configuration";
  $debug_data = "$directory/$active_minute";
  $debug_data_configuration = "$directory/configuration";
  #print "$debug_data\n";
  $wrkdir = "$data_directory/data/$st_serial";

} ## end sub data_structure

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

sub check_connection{
  my $data = cmd("system/", "GET", 1, "system");
  if ( $data eq "" ) {
    if ($connection_test){
      push(@rows_api_test_data,"connection to $api_host on port $port has failed!\n");
    }
    else{
      $logger->error("connection to $api_host on port $port has failed!");
    }
    return 0;
  }
  $data = cmd("sys_node", "GET", 1,"sys_node");
  if ($data ne ""){
    foreach my $node (@{$data->{data}}){
      my $id = $node->{ID};
      my $type = $node->{TYPE};
      if ( !defined $id || !defined $type ) { next; }
      my $cmd_perf = "performace_statistic/cur_statistic_data?CMO_STATISTIC_UUID=$type:$id&CMO_STATISTIC_DATA_ID_LIST=$performance_statistics{node}";
      my $perf_data = cmd($cmd_perf,"GET", 1, "$type" . "_$id");
      if ($perf_data eq ""){
        return 0;
      }
      if (!defined $perf_data->{data} && $perf_data eq "83890432"){
        return relogin();
      }
    }
  }
  else{
    if ($connection_test){
      push(@rows_api_test_data,"connection to $api_host on port $port has failed!\n");
    }
    else{
      $logger->error("connection to $api_host on port $port has failed!");
    }
    return 0;
  }
  return 1;
}

sub relogin{
  logout();
  if ($api_host ne $cfg->{'node2Host'}){
    $api_host = $cfg->{'node2Host'};
  }
  else{
    $api_host = $cfg->{'node1Host'};
  }
  $token = "";
  $device_id = "";
  $cookie = "";
  login();
  if (!defined $token || $token eq "" || !defined $device_id || $device_id eq "" || !defined $cookie || $cookie eq ""){
    return 0;
  }
  return 1;
}

sub set_data_perf_list{
  $performance_statistics{node} = "22,25,28,68,69,123,124,410,412,414,416,418,420,428,430,432,90001,90002,90003,534,535,536";
  $performance_statistics{drive} = "22,25,28,78,192,197,198,24,27,311,312,313";
  $performance_statistics{ethport} = "229,230,231,67,236,124,123";
  $performance_statistics{cluster} = "68,69,22,25,28,123,124,90001,90002,90003,534,535,536";

  
  $metric_inventory{68}  = "cpu_system";
  $metric_inventory{69}  = "cache_usage_percent";
  $metric_inventory{110} = "r_cache_usage_percent";
  $metric_inventory{120} = "w_cache_usage_percent";
 

  $metric_inventory{22}  = "io_rate";
  $metric_inventory{25}  = "read_io";
  $metric_inventory{28}  = "write_io";
  $metric_inventory{23}  = "read";
  $metric_inventory{26}  = "write";
  $metric_inventory{370} = "resp_t";
  $metric_inventory{384} = "resp_t_r";
  $metric_inventory{385} = "resp_t_w";
  $metric_inventory{93}  = "r_cache_hit";
  $metric_inventory{95}  = "w_cache_hit";
  $metric_inventory{19}  = "queue_length";
  $metric_inventory{182} = "io_rate";
  $metric_inventory{232} = "read_io";
  $metric_inventory{123} = "read";
  $metric_inventory{524} = "resp_t_r";
  $metric_inventory{233} = "write_io";
  $metric_inventory{124} = "write";
  $metric_inventory{525} = "resp_t_w";
  $metric_inventory{523} = "resp_t";
  $metric_inventory{721} = "nfs_ops";
  $metric_inventory{724} = "cifs_ops";
  $metric_inventory{727} = "nfs_read_ops";
  $metric_inventory{730} = "cifs_read_ops";
  $metric_inventory{733} = "nfs_write_ops";
  $metric_inventory{736} = "cifs_write_ops";
  $metric_inventory{739} = "nfs_other_ops";
  $metric_inventory{742} = "cifs_other_ops";
  $metric_inventory{745} = "nfs_resp_t";
  $metric_inventory{748} = "cifs_resp_t";
  $metric_inventory{751} = "nfs_resp_t_r";
  $metric_inventory{754} = "cifs_resp_t_r";
  $metric_inventory{757} = "nfs_resp_t_w";
  $metric_inventory{760} = "cifs_resp_t_w";
  $metric_inventory{763} = "nfs_other_resp_t";
  $metric_inventory{90001} = "capacity_total";
  $metric_inventory{90002} = "capacity_used";
  $metric_inventory{90003} = "capacity_free";
  $metric_inventory{534}   = "fs_data_rate";
  $metric_inventory{535}   = "fs_read";
  $metric_inventory{536}   = "fs_write";
  $metric_inventory{410}   = "nfs_resp_t";
  $metric_inventory{412}   = "nfs_resp_t_w";
  $metric_inventory{414}   = "nfs_resp_t_r";
  $metric_inventory{416}   = "cifs_resp_t";
  $metric_inventory{418}   = "cifs_resp_t_w";
  $metric_inventory{420}   = "cifs_resp_t_r";
  $metric_inventory{428}   = "resp_t";
  $metric_inventory{432}   = "resp_t_w";
  $metric_inventory{430}   = "resp_t_r";
  $metric_inventory{627}   = "nfs_ops";
  $metric_inventory{24}    = "block_size_r";
  $metric_inventory{27}    = "block_size_w";
  $metric_inventory{78}    = "resp_t";
  $metric_inventory{197}   = "resp_t_r";
  $metric_inventory{198}   = "resp_t_w";


}

sub get_health_status_text {
  my $number = shift;

  if ( $number == 1 ) {
    return "normal";
  }
  if ( $number == 0 ) {
    return "unknown";
  }
  if ( $number == 2 ) {
    return "faulty";
  }
  if ( $number == 3 ) {
    return "Pre-Fail";
  }
  if ( $number == 9 ) {
    return "inconsistent";
  }
  if ( $number == 4 ) {
    return "partially damaged";
  }
  if ( $number == 12 ) {
    return "low battery";
  }
  if ( $number == 5 ) {
    return "degraded";
  }

  return $number;

}

sub get_running_status_text {
  my $number = shift;

  if ( $number == 1 ) {
    return "normal";
  }
  if ( $number == 0 ) {
    return "unknown";
  }
  if ( $number == 2 ) {
    return "running";
  }
  if ( $number == 27 ) {
    return "online";
  }
  if ( $number == 28 ) {
    return "offline";
  }
  if ( $number == 48 ) {
    return "charging completed";
  }
  if ( $number == 50 ) {
    return "discharging";
  }
  if ( $number == 10 ) {
    return "link up";
  }
  if ( $number == 11 ) {
    return "link down";
  }
  if ( $number == 14 ) {
    return "pre-copy";
  }
  if ( $number == 16 ) {
    return "reconstruction";
  }
  if ( $number == 32 ) {
    return "balancing";
  }
    if ( $number == 53 ) {
    return "Initializing";
  }
  return $number;
}

sub get_global_timestamp{
  my $act_timestamp = time();
  (my $sec,my $min,my $hour,my $mday,my $mon, my $year,my $wday,my $yday,my $isdst) = localtime();
  return $act_timestamp - $sec;

}

sub get_level_text {
  my $type = shift;
  if ( !defined $type ) { return "unknown"; }
  if ( $type == 2 )     { return "info"; }
  if ( $type == 3 )     { return "warning"; }
  if ( $type == 5 )     { return "major"; }
  if ( $type == 6 )     { return "critical"; }
  return "unknown";

}

sub get_type_text {
  my $type = shift;
  if ( !Xormon::isdigit($type) ) { return "unknown"; }
  if ( $type == 0 )      { return "fault"; }
  if ( $type == 1 )      { return "event"; }
  if ( $type == 2 )      { return "recovery"; }
  if ( $type == 3 )      { return "operation log"; }
  if ( $type == 4 )      { return "running"; }  
  return "unknown";

}


sub get_model{
  my $model = shift;
  if ( $model eq "48"){return "OceanStor 9000 Series";}
  if ( $model eq "80"){return "FusionStorage Object";}
  if ( $model eq "83"){return "FusionStorage File/Object";}
  if ( $model eq "91"){return "FusionStorage File";}
  if ( $model eq "61" )  { return "6800 V3"; }
  if ( $model eq "62" )  { return "6900 V3"; }
  if ( $model eq "63" )  { return "5600 V3"; }
  if ( $model eq "64" )  { return "5800 V3"; }
  if ( $model eq "68" )  { return "5500 V3"; }
  if ( $model eq "69" )  { return "2600 V3"; }
  if ( $model eq "70" )  { return "5300 V3"; }
  if ( $model eq "71" )  { return "2800 V3"; }
  if ( $model eq "72" )  { return "18500 V3"; }
  if ( $model eq "73" )  { return "18800 V3"; }
  if ( $model eq "74" )  { return "2200 V3"; }
  if ( $model eq "82" )  { return "2600 V3 for Video"; }
  if ( $model eq "84" )  { return "2600F V3"; }
  if ( $model eq "85" )  { return "5500F V3"; }
  if ( $model eq "86" )  { return "5600F V3"; }
  if ( $model eq "87" )  { return "5800F V3"; }
  if ( $model eq "88" )  { return "6800F V3"; }
  if ( $model eq "89" )  { return "18500F V3"; }
  if ( $model eq "90" )  { return "18800F V3"; }
  if ( $model eq "92" )  { return "2800 V5"; }
  if ( $model eq "93" )  { return "5300 V5"; }
  if ( $model eq "94" )  { return "5300F V5"; }
  if ( $model eq "95" )  { return "5500 V5"; }
  if ( $model eq "96" )  { return "5500F V5"; }
  if ( $model eq "97" )  { return "5600 V5"; }
  if ( $model eq "98" )  { return "5600F V5"; }
  if ( $model eq "99" )  { return "5800 V5"; }
  if ( $model eq "100" ) { return "5800F V5"; }
  if ( $model eq "101" ) { return "6800 V5"; }
  if ( $model eq "102" ) { return "6800F V5"; }
  if ( $model eq "103" ) { return "18500 V5"; }
  if ( $model eq "104" ) { return "18500F V5"; }
  if ( $model eq "105" ) { return "18800 V5"; }
  if ( $model eq "106" ) { return "18800F V5"; }
  if ( $model eq "107" ) { return "5500 V5 Elite"; }
  if ( $model eq "108" ) { return "2100 V3"; }
  if ( $model eq "116" ) { return "5110 V5"; }
  if ( $model eq "810" ) { return "Dorado3000 V3"; }
  if ( $model =~ m/^811$|^812$|^821$/){return "OceanStor Dorado 5000 V6";}
  if ( $model eq "834"){return "OceanStor 5310";}
  if ( $model =~ m/^813$|^814$|^822$/){ return "OceanStor Dorado 6000 V6";}
  if ( $model =~ m/^815$|^816$|^823$/){ return "OceanStor Dorado 8000 V6";}
  if ( $model =~ m/^817$|^818$|^824$/){ return "OceanStor Dorado 18000 V6";}
  if ( $model =~ m/^819$/){ return "OceanStor Dorado 3000 V6";}
  if ( $model =~ m/^825$/){ return "OceanStor Dorado 5300 V6";}
  if ( $model =~ m/^826$/){ return "OceanStor Dorado 5500 V6";}
  if ( $model =~ m/^827$/){ return "OceanStor Dorado 5600 V6";}
  if ( $model =~ m/^828$/){ return "OceanStor Dorado 5800 V6";}
  if ( $model =~ m/^829$/){ return "OceanStor Dorado 6800 V6";}
  if ( $model =~ m/^830$/){ return "OceanStor Dorado 18500 V6";}
  if ( $model =~ m/^831$/){ return "OceanStor Dorado 18800 V6";}




  return $model;

}



1;
