package OracleDB;

use strict;
use warnings;

use JSON;
use Data::Dumper;
use Digest::MD5 qw(md5 md5_hex md5_base64);

sub get_metrics_dictionary {
my %metrics = (
  'CPU Usage Per Sec'  => 'CPUUsagePerSec',
  'CPU Usage Per Txn'  => 'CPUUsagePerTxn',
  'Host CPU Utilization (%)'  => 'HostCPUUtilization',
  'Network Traffic Volume Per Sec'  => 'NetworkTrafficVolumePerSec',
  'Database Wait Time Ratio'  => 'DatabaseWaitTimeRatio',
  'Memory Sorts Ratio'  => 'MemorySortsRatio',
  'PGA Cache Hit %'  => 'PGACacheHit',
  'Database CPU Time Ratio'  => 'DatabaseCPUTimeRatio',
  'Soft Parse Ratio'  => 'SoftParseRatio',
  'Buffer Cache Hit Ratio'  => 'BufferCacheHitRatio',
  'Buffer Cache Ratio'  => 'BufferCacheRatio',
  'Executions Per Sec'  => 'ExecutionsPerSec',
  'Hard Parse Count Per Sec'  => 'HardParseCountPerSec',
  'User Transaction Per Sec'  => 'UserTransactionPerSec',
  'User Commits Per Sec'  => 'UserCommitsPerSec',
  'Current Open Cursors Count'  => 'CurrentOpenCursorsCount',
  'Open Cursors Per Sec'  => 'OpenCursorsPerSec',
  'IO Requests per Second'  => 'IORequestsperSecond',
  'IO Megabytes per Second'  => 'IOMegabytesperSecond',
  'I/O Megabytes per Second'  => 'IOMegabytesperSecond',
  'Physical Read Bytes Per Sec'  => 'PhysicalReadBytesPerSec',
  'Physical Writes Per Sec'  => 'PhysicalWritesPerSec',
  'I/O Requests per Second'  => 'IORequestsperSecond',
  'Redo Generated Per Sec'  => 'RedoGeneratedPerSec',
  'Logical Reads Per Sec'  => 'LogicalReadsPerSec',
  'Physical Reads Per Sec'  => 'PhysicalReadsPerSec',
  'Physical Write Bytes Per Sec'  => 'PhysicalWriteBytesPerSec',
  'DB Block Changes Per Sec'  => 'DBBlockChangesPerSec',
  'Current Logons Count'  => 'CurrentLogonsCount',
  'Logons Per Sec'  => 'LogonsPerSec',
  'Active Serial Sessions'  => 'ActiveSerialSessions',
  'Average Active Sessions'  => 'AverageActiveSessions',
  'Active Parallel Sessions'  => 'ActiveParallelSessions',
  'Global Cache Blocks Lost'  => 'GlobalCacheBlocksLost',
  'Global Cache Average CR Get Time'  => 'GlobalCacheAverageCRGetTime',
  'Global Cache Average Current Get Time'  => 'GlobalCacheAverageCurrentGetTime',
  'GC CR Block Received Per Second'  => 'GCCRBlockReceivedPerSecond',
  'GC Current Block Received Per Second'  => 'GCCurrentBlockReceivedPerSecond',
  'Cell Physical IO Interconnect Bytes'  => 'CellPhysicalIOInterconnectBytes',
  'Global Cache Blocks Corrupted'  => 'GlobalCacheBlocksCorrupted',
  'GC Avg CR Block receive ms'  => 'GCAvgCRBlockreceivems',
  'GC Avg CUR Block receive ms'  => 'GCAvgCURBlockreceivems',
  'DB files read latency'  => 'DBfilesreadlatency',
  'DB files write latency'  => 'DBfileswritelatency',
  'LOG files write latency'  => 'LOGfileswritelatency',
  'gc cr block 2-way'  => 'gccrblock2way',
  'gc cr block 3-way'  => 'gccrblock3way',
  'gc current block 2-way'  => 'gccurrentblock2way',
  'gc current block 3-way'  => 'gccurrentblock3way',
  'gc cr block busy'  => 'gccrblockbusy',
  'gc cr block congested'  => 'gccrblockcongested',
  'gc cr grant 2-way'  => 'gccrgrant2way',
  'gc cr grant congested'  => 'gccrgrantcongested',
  'gc current block busy'  => 'gccurrentblockbusy',
  'gc current block congested'  => 'gccurrentblockcongested',
  'gc cr block lost'  => 'gccrblocklost',
  'gc current block lost'  => 'gccurrentblocklost',
  'gc cr failure'  => 'gccrfailure',
  'gc current retry'  => 'gccurrentretry',
  'gc current split'  => 'gccurrentsplit',
  'gc current multi block request'  => 'gccurrentmultiblockrequest',
  'gc current grant busy'  => 'gccurrentgrantbusy',
  'gc cr disk read'  => 'gccrdiskread',
  'gc cr multi block request'  => 'gccrmultiblockrequest',
  'gc buffer busy acquire'  => 'gcbufferbusyacquire',
  'gc buffer busy release'  => 'gcbufferbusyrelease',
  'gc current grant 2-way'  => 'gccurrentgrant2way',
  'gc current grant congested'  => 'gccurrentgrantcongested',
  'gc remaster'  => 'gcremaster',
  'Global Cache Blocks Lost'  => 'GlobalCacheBlocksLost',
  'Global Cache Average CR Get Time'  => 'GlobalCacheAverageCRGetTime',
  'Global Cache Average Current Get Time'  => 'GlobalCacheAverageCurrentGetTime',
  'GC CR Block Received Per Second'  => 'GCCRBlockReceivedPerSecond',
  'GC Current Block Received Per Second'  => 'GCCurrentBlockReceivedPerSecond',
  'Cell Physical IO Interconnect Bytes'  => 'CellPhysicalIOInterconnectBytes',
  'Global Cache Blocks Corrupted'  => 'GlobalCacheBlocksCorrupted',
  'GC Avg CR Block receive ms'  => 'GCAvgCRBlockreceivems',
  'GC Avg CUR Block receive ms'  => 'GCAvgCURBlockreceivems',
  'db file sequential read'  => 'dbfilesequentialread',
  'db file single write'  => 'dbfilesinglewrite',
  'db file parallel write'  => 'dbfileparallelwrite',
  'log file sync'  => 'logfilesync',
  'log file single write'  => 'logfilesinglewrite',
  'log file parallel write'  => 'logfileparallelwrite',
  'flashback log file sync'  => 'flashbacklogfilesync',
  'db file scattered read'  => 'dbfilescatteredread',
  'Average wait FG ms'  => 'AveragewaitFGms',
  'Average wait ms'  => 'Averagewaitms',
  'Physical Writes'  => 'PhysicalWrites',
  'Physical Reads'  => 'PhysicalReads',
  'Avg write read ms'  => 'Avgwritereadms',
  'Avg read wait ms'  => 'Avgreadwaitms',
  'Read/Write %'  => 'ReadWrite',
  'physical reads'  => 'physicalreads',
  'physical writes'  => 'physicalwrites',
  'used'  => 'used',
  'free'  => 'free',
  'log_capacity'  => 'log_capacity',
  'controlfiles'  => 'control_files',
  'tempfiles  '  => 'temp_files',
  'recoverysize'  => 'recoverysize',
);
  return \%metrics;
}


sub basename {
  my $full      = shift;
  my $separator = shift;
  my $out = "";
  #my $length = length($full);
  if(defined $separator and defined $full and  index($full, $separator) != -1){
    $out = substr($full, length($full) - index(reverse($full), $separator), length($full));
    return $out;
  }
  return $full;
}


my %metrics = (
    'CPU_info' => [
      'CPU Usage Per Sec',
      'CPU Usage Per Txn',
      'Host CPU Utilization (%)'
    ],
    'Network'=> [
      'Network Traffic Volume Per Sec'
    ],
    'Ratio'=> [
      'Database Wait Time Ratio',
      'Memory Sorts Ratio',
      'PGA Cache Hit %',
      'Database CPU Time Ratio',
      'Soft Parse Ratio',
      'Buffer Cache Hit Ratio'
    ],
    'SQL_query'=> [
      'Executions Per Sec',
      'Hard Parse Count Per Sec',
      'User Transaction Per Sec',
      'User Commits Per Sec',
      'Current Open Cursors Count',
      'Open Cursors Per Sec'
    ],
    'Data_rate'=> [
      'I/O Megabytes per Second',
      'Physical Read Bytes Per Sec',
      'Physical Writes Per Sec',
      'I/O Requests per Second',
      'Redo Generated Per Sec',
      'Logical Reads Per Sec',
      'Physical Reads Per Sec',
      'Physical Write Bytes Per Sec',
      'DB Block Changes Per Sec'
    ],
    'Session_info' => [
      'Current Logons Count',
      'Logons Per Sec',
      'Active Serial Sessions',
      'Average Active Sessions',
      'Active Parallel Sessions'
    ],
    'RAC'=> [
      'Global Cache Blocks Lost',
      'Global Cache Average CR Get Time',
      'Global Cache Average Current Get Time',
      'GC CR Block Received Per Second',
      'GC Current Block Received Per Second',
      'Cell Physical IO Interconnect Bytes',
      'Global Cache Blocks Corrupted',
      'GC Avg CR Block receive ms',
      'GC Avg CUR Block receive ms',
      'DB files read latency',
      'DB files write latency',
      'LOG files write latency',
      'gc cr block 2-way',
      'gc cr block 3-way',
      'gc current block 2-way',
      'gc current block 3-way',
      'gc cr block busy',
      'gc cr block congested',
      'gc cr grant 2-way',
      'gc cr grant congested',
      'gc current block busy',
      'gc current block congested',
      'gc cr block lost',
      'gc current block lost',
      'gc cr failure',
      'gc current retry',
      'gc current split',
      'gc current multi block request',
      'gc current grant busy',
      'gc cr disk read',
      'gc cr multi block request',
      'gc buffer busy acquire',
      'gc buffer busy release',
      'gc current grant 2-way',
      'gc current grant congested',
      'gc remaster'
    ],
    'Cache'=> [
      'Global Cache Blocks Lost',
      'Global Cache Average CR Get Time',
      'Global Cache Average Current Get Time',
      'GC CR Block Received Per Second',
      'GC Current Block Received Per Second',
      'Cell Physical IO Interconnect Bytes',
      'Global Cache Blocks Corrupted',
      'GC Avg CR Block receive ms',
      'GC Avg CUR Block receive ms'
    ],
    'Disk_latency' => [
      'db file sequential read',
      'db file single write',
      'db file parallel write',
      'log file sync',
      'log file single write',
      'log file parallel write',
      'flashback log file sync',
      'db file scattered read'
    ],
    'Wait_class_Main' => [
      'Average wait FG ms',
      'Average wait ms'
    ],
    'IO_Read_Write_per_datafile' => [
      'Physical Writes',
      'Physical Reads',
      'Avg write read ms',
      'Avg read wait ms',
      'Read/Write %'
    ],
    'Services' => [
      'physical reads',
      'physical writes'
    ],
    'Capacity' => [
      'used',
      'free',
      'log_capacity'
    ]

);


my %conf_types = (
  "configuration" => 1,
  "configuration_S" => 1,
  "configuration_Multitenant" => 1,
  "configuration_PDB" => 1,
  "configuration_Total" => 1,
  "configuration_dg" => 1,
  "viewone" => 1,
  "viewtwo" => 1,
  "viewthree" => 1,
  "viewfour" => 1,
  "viewfive" => 1,
  "waitclass" => 1,
  "datafiles" => 1,
  "Interconnect" => 1,
  "healthstatus" => 1,
);

my %host_metrics = (
  "CPU info"  => 1,
  "Data rate" => 1,            
  "Capacity"  => 1,
);
################################################################################

sub is_host_metric {
  my $metric = shift;
  if($host_metrics{$metric}){
    return 1;
  }else{
    return 0;
  }
}

sub get_host_metrics {
  return \%host_metrics;
}

sub md5_string {
  my $data = shift;
  my $out = md5_hex($data);

  return $out;
}

1;