use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib '../../../perl-modules';
use PostgreSQL;
use Xormon;
use File::Temp qw(tempfile);
use File::Copy;
use XormonLogger;
use Test;

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3301;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);
my $logger;

my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

 # waiting for a new client connection
while ($client = $server->accept()) {

  my $pid;
  unless ( defined( $pid = fork() ) ) {
      Xormon::error("Wait for fork");
      sleep 5;
  }

  if ($pid == 0) {

      # get information about a newly connected client
      my $client_address = $client->peerhost();
      my $client_port = $client->peerport();

        # received data template
        #
        # {
        #   "jwt"  : "<backendToken>",
        #   "task" : "<taskName>"
        #   "data" : {
        #     <configuration_from_hostcfg>
        #   }
        # }

        # read up to 2048 characters from the connected client
      my $data = "";
      $client->recv($data, 2048);

      my $decoded = from_json($data);
      my $cfg = $decoded->{data};
      $logger = XormonLogger->new('postgres', $cfg->{hostcfg_id}, $cfg->{hostalias} );
      $logger->info("Connection from $client_address:$client_port");
      $logger->info("Received task: ".$decoded->{task});

      my $response = '';
      if ($decoded->{task} eq "fetch") {

        $client->send("Task successfuly started");
        $client->shutdown(SHUT_WR);

        my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'postgres', $cfg->{hostcfg_id}, $cfg->{hostalias});


        #print $cfg;
        $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

        ############################### AGENT SECTION ###############################

        # target destinations check
        if (defined $decoded->{'destinations'}) {
          $xormon->setDestinations($decoded->{'destinations'});
        }

        # get data from technology

        my $dataToBackend = api2json($xormon, $cfg);
          #print Dumper $dataToBackend;
          # testing
          #sleep(30);

          # send data to backend
        my $rc = $xormon->saveJSON2($dataToBackend);

        ############################### END AGENT SECTION ###############################

        # status code
        my $status;
        if ($rc eq "201") {
            $status = "successful";
        } elsif ($rc eq "401") {
            $status = "unauthorized";
        } else {
            $status = "error";
        }

        $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);

        $logger->info("Task [fetch] for ".$cfg->{hostalias}." is done");

      }
      elsif ($decoded->{task} eq "test") {
        apitest($client, $cfg);
        $client->shutdown(SHUT_WR);
        $logger->info("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done.");
      } else {
        $response = "undefined task";

        $client->send($response);
        $client->shutdown(SHUT_WR);
      }

      exit();

  } else {
    $client->autoflush(1);
  }
}




my %whitelisted;
my $alias;
my @pre_conf;
my $current_full_db;
my $use_whitelist;
my @health_status;
my %db_ids;
my @architecture;
my %full_db_names;


sub api2json {
  my $xormon = shift;
  my $data   = shift;
  #print Dumper $data;
  my $db_name    = $data->{instance};
  my $port       = $data->{port};
  my $username   = $data->{username};
  my $password   = $data->{password};
  my $ip         = $data->{host};
  $alias         = $data->{hostalias};
  $use_whitelist = $data->{use_whitelist};

  if ($use_whitelist){
    foreach my $db (@{$data->{dbs}}) {
      $whitelisted{$db} = 1;
    }
  }

  my $connected = Test::connTest($data->{host}, $data->{port});
  if ($connected == 1) {
    $xormon->isConnected(1);
  } else {
    $xormon->isConnected(0, "TCP connection to " . $data->{host} . ":" . $data->{port} . " has failed!");
    return ();
  }

  my $pre_perf = get_data("PERF", $db_name, $ip, $port, $username, $password, $alias, "skip");
  if (defined $pre_perf and ref($pre_perf) eq "ARRAY") {
    $xormon->hasData(1);
  } else {
    $xormon->hasData(0, "Unable to get data")
  }

  my $act_perf = make_perf($pre_perf);
  my $ret = compare_perf($alias, $act_perf);

  #print Dumper $ret;
  #print Dumper $act_perf;
  my $module = "DBI";
  my $module_def = 1;
  eval "use $module; 1" or $module_def = 0;
  if ( !$module_def ) {
    $logger->error("Missing module: $module\n");
    return "";
  }

  my $data_csv = prepare_perf($data->{hostcfg_id},$alias, $ret);

  #warn Dumper \@architecture;
  #save architecture
  my $arc_resp = $xormon->saveArchitecture(\@architecture);
  $logger->info("Save Architecture for host  to DB. RC : $arc_resp\n");

  my $db_longnames = rrd_check();
  my $current_full_db;
  #my @pre_conf;
  my $g_main = 0;
  for my $db (keys %{$db_longnames}){
    $current_full_db = $db;
    my @db_parts = split("-", $db);
    pop(@db_parts);
    my $alm_full = join("-", @db_parts);
    next if (!(defined $db_parts[0]) or $db_parts[0] eq "template0" or $db_parts[0] eq "shrdrltn");
    if ($g_main == 0){
      get_data("CONF+", $alm_full, $ip, $port, $username, $password, $alias, undef, $current_full_db);
      $g_main = 1;
    }else{
      get_data("CONF", $alm_full, $ip, $port, $username, $password, $alias, undef, $current_full_db);
    }
  }


#generate_hsfiles();
  my $cluster_id = PostgreSQL::md5_string("$data->{hostcfg_id}-$alias-DBFOLDERS");
  my $act_conf = make_conf(\@pre_conf, $cluster_id);
  #warn Dumper $act_conf;

  #save configuration
  my $configuration_rc = $xormon->saveConfJSON( $act_conf, 1 );
  $logger->info("Save Configuration for host  to DB. RC : $configuration_rc\n");

  #warn Dumper \@health_status;
  my $rc = $xormon->saveStatus(\@health_status);
  $logger->info("Save Health Status for host  to DB. RC : $rc\n");

  return $data_csv;
}

sub apitest {
  my $client  = shift;
  my $data    = shift;

  my $db_namea    = $data->{instance};
  my $porta       = $data->{port};
  my $usernamea   = $data->{username};
  my $passworda   = $data->{password};
  my $ipa         = $data->{host};
  $alias          = $data->{hostalias};
  my $test = Test->new($client);
  my @rows;

  my ($code, $text) = Xormon::conntest_tcp($ipa, $porta);
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  my $module = "DBI";
  my $module_def = 1;
  eval "use $module; 1" or $module_def = 0;
  if ( !$module_def ) {
    push(@rows, "ERROR: $module is not installed. https://lpar2rrd.com/PostgreSQL-performance-monitoring-install.php#PostgreSQL");
    $test->addSection("Module test", 0 , \@rows);
    return $test->stop();
  }
  require DBI;

  my $pre_perf = get_data("PERF", $db_namea, $ipa, $porta, $usernamea, $passworda, $alias, "apitest");
  if ( $pre_perf->[0] ){
    push(@rows, "Data ok.");
    $test->addSection("Data test", 1 , \@rows);
    return $test->stop();
  }elsif($pre_perf and $pre_perf =~ /SCRAM/){
    push(@rows, "ERROR:</br> $pre_perf</br> </br>Either upgrade libpq to version 10 and above or change users password authentication to md5 and generate new password" );
    $test->addSection("Data test", 0 , \@rows);
    return $test->stop();
  }else{
    push(@rows, "Invalid data");

    $test->addSection("Data test", 0 , \@rows);
    return $test->stop();
  }
}


sub get_data {
  my $type      = shift;
  my $_db_name  = shift;
  my $_ip       = shift;
  my $_port     = shift;
  my $_username = shift;
  my $_password = shift;
  my $_alias    = shift;
  my $skip      = shift;
  my $current_full_db = shift;
  my $result;
  require DBI;

  my $dbh = DBI->connect("DBI:Pg:dbname=$_db_name;host=$_ip;port=$_port", "$_username", "$_password",{ PrintError => 0 });
  my %queries = get_queries();
  my @perf_types = ("_dbs", "_cluster", "_locks", "_sessions", "_wait_event");

  my $time_is_now = time;
  my $err = 0;
  $err = DBI->errstr;
  if(defined $skip and $skip eq "apitest"){
    if ( $err =~ /SCRAM/ ) {
      return $err;
    }
  }

  my $db_id = PostgreSQL::md5_string($_db_name."-".$_alias);
  if ($type eq "CONF" or $type eq "CONF+"){
    if( defined $current_full_db){
      $db_id = PostgreSQL::md5_string("$current_full_db-$_alias");
    }
    if($err and !(defined $dbh)){
      if(defined $db_id){
        push(@health_status, { "item_id" => $db_id, "status" => "error", "data" => $err });
        push(@health_status, { "item_id" => $db_id, "type" => "connection", "status" => "error", "data" => $err });
      }
    }else{
      if(defined $db_id){
        push(@health_status, { "item_id" => $db_id, "status" => "ok", "data" => "" });
        push(@health_status, { "item_id" => $db_id, "type" => "connection", "status" => "ok", "data" => "" });
      }
    }
  }

  if (!(defined $dbh)){
    if (defined $skip){
      warn $err;
    }
    return "empty";
  }

  if ($type eq "PERF"){
    my @pre_perf;
    foreach my $_type (@perf_types){
      if($_type and $queries{$_type}){
        my $array = get_query_result($dbh, $_type, $queries{$_type});
        my %temp_hash;
        $temp_hash{type} = $_type;
        $temp_hash{db_name} = $_db_name;
        $temp_hash{data} = $array;
        push(@pre_perf, \%temp_hash);
      }
    }
    return \@pre_perf;
  }elsif($type =~ /CONF/){
    if ($type eq "CONF+"){
      my $arr = get_query_result($dbh, "_main", $queries{_main});
      my %temp_hsh;
      $temp_hsh{type} = "_main";
      $temp_hsh{data} = $arr;
      $temp_hsh{db_name} = $_db_name;
      push(@pre_conf, \%temp_hsh);
    }
    my $array = get_query_result($dbh, "_relations", $queries{_relations});
    my %temp_hash;
    $temp_hash{type} = "_relations";
    $temp_hash{data} = $array;
    $temp_hash{db_name} = $_db_name;
    push(@pre_conf, \%temp_hash);
  }
  # clean up
  $dbh->disconnect();

  return "empty";
}

sub get_query_result {
  my $_dbh   = shift;
  my $_type  = shift;
  my $_query = shift;
  my @array;

  my %conf_sa = (
    "shared_buffers"       => "1",
    "work_mem"             => "1",
    "max_connections"      => "1",
    "temp_buffers"         => "1",
    "maintenance_work_mem" => "1",
    "autovacuum_work_mem"  => "1",
    "wal_buffers"          => "1",
    "effective_cache_size" => "1",
    "random_page_cost"     => "1",
    "min_wal_size"         => "1",
    "max_wal_size"         => "1",
    "wal_sync_method"      => "1",
    "wal_buffers"          => "1",
  );

  if (defined $_query){
  my $sth = $_dbh->prepare($_query);

  $sth->execute();
  if($_type eq "_main"){
    while(my $ref = $sth->fetchrow_hashref()) {
      if($conf_sa{$ref->{name}}){
        push(@array, $ref);
      }
    }
  }else{
    while(my $ref = $sth->fetchrow_hashref()) {
      push(@array, $ref);
    }
  }

  return \@array;

  }else{
    #print Dumper "$_dbh $_type $_query\n";
    @array = [];
    return \@array;
  }
}

sub make_perf {
  my $rows = shift;
  my @r_array = @{$rows};

  my %perf;
  foreach my $row_type (@r_array){
    if ($row_type->{type} eq "_dbs"){
      foreach my $row (@{$row_type->{data}}){
        my %hash_row = %{$row};
        my %hr_dupl = %hash_row;
        if (!(defined $hash_row{datname})){
          $hash_row{datname} = "shrdrltn";
        }
        delete $hr_dupl{datid};
        delete $hr_dupl{datname};
        my $accepted = 0;
        if ($use_whitelist){
          if ($whitelisted{$hash_row{datname}}){
            $accepted = 1;
          }
        }else{
          $accepted = 1;
        }

        if($accepted){
          $hr_dupl{cursors} = 0;
          $hr_dupl{size} = to_gib($hr_dupl{size});
          $perf{"$hash_row{datname}-$hash_row{datid}"} = \%hr_dupl;
          #my $uuid = PostgresSQL::md5_string("$alias-$hash_row{datname}-$hash_row{datid}");
          my $uuid = "alias-$hash_row{datname}-$hash_row{datid}";
          $db_ids{$hash_row{datid}} = "$hash_row{datname}-$hash_row{datid}";
        }
      }
    }elsif($row_type->{type} eq "_locks"){
      foreach my $row (@{$row_type->{data}}){
        my %hash_row = %{$row};
        next if(!(defined $hash_row{database}) or $hash_row{database} eq "undef");
        if($hash_row{granted}){
          next if (! (defined $db_ids{$hash_row{database}}));
          if(exists $perf{$db_ids{$hash_row{database}}}{"h".$hash_row{locktype}}){
            $perf{$db_ids{$hash_row{database}}}{"h".$hash_row{locktype}} += 1;
          }else{
            $perf{$db_ids{$hash_row{database}}}{"h".$hash_row{locktype}} = 1;
          }
        }else{
          next if (! (defined $hash_row{database}));
          if(exists $perf{$db_ids{$hash_row{database}}}{"a".$hash_row{locktype}}){
            $perf{$db_ids{$hash_row{database}}}{"a".$hash_row{locktype}} += 1;
          }else{
            $perf{$db_ids{$hash_row{database}}}{"a".$hash_row{locktype}} = 1;
          }
        }
      }
    }elsif($row_type->{type} eq "_sessions"){
      foreach my $row (@{$row_type->{data}}){
        my %hash_row = %{$row};
        my $state = $hash_row{state};
        next if(!(defined $state));
        $state =~ s/\(//g;
        $state =~ s/\)//g;
        $state =~ s/ /_/g;
        if(exists $perf{"$hash_row{datname}-$hash_row{datid}"}{$state}){
          $perf{"$hash_row{datname}-$hash_row{datid}"}{$state} += 1;
        }else{
          $perf{"$hash_row{datname}-$hash_row{datid}"}{$state} = 1;
        }
      }
    }elsif($row_type->{type} eq "_wait_event"){
      foreach my $row (@{$row_type->{data}}){
        my %hash_row = %{$row};
        next if (!defined $hash_row{datname} or !defined $hash_row{datid});
        if(defined $hash_row{wait_event_type}){
          my $state = $hash_row{wait_event_type};
          if(exists $perf{"$hash_row{datname}-$hash_row{datid}"}{$state}){
            $perf{"$hash_row{datname}-$hash_row{datid}"}{$state} += 1;
          }else{
            $perf{"$hash_row{datname}-$hash_row{datid}"}{$state} = 1;
          }
        }elsif(defined $hash_row{wait_event}){
          if(exists $perf{"$hash_row{datname}-$hash_row{datid}"}{_event}{$hash_row{wait_event}}){
            $perf{"$hash_row{datname}-$hash_row{datid}"}{$hash_row{wait_event}} += 1;
          }else{
            $perf{"$hash_row{datname}-$hash_row{datid}"}{$hash_row{wait_event}} = 1;
          }
        }
      }
    }
    elsif($row_type->{type} eq "_cluster"){
      $perf{_cluster} = $row_type->{data}->[0];
    }
  }

  $perf{_info}{timestamp} = time;
  $perf{_info}{readable_time} = localtime($perf{_info}{timestamp});

  return \%perf;
}

sub get_queries {
  my %queries = (
    "_dbs"        => "SELECT datid, datname, blks_read, blks_hit, tup_returned, tup_fetched, tup_inserted, tup_updated, tup_deleted, xact_commit, xact_rollback, deadlocks, temp_files,temp_bytes, pg_database_size(datname) as \"size\" FROM pg_stat_database;",
    "_cluster"    => "SELECT buffers_backend, buffers_clean, buffers_checkpoint, checkpoints_req, checkpoints_timed FROM pg_stat_bgwriter;",
    "_main"       => "show all;",
    "_locks"      => "SELECT database,locktype,granted FROM pg_locks;",
    "_sessions"   => "SELECT datid, datname, usename, state FROM pg_stat_activity WHERE state != 'undefined';",
    "_wait_event" => "SELECT datid, datname, wait_event_type, wait_event FROM pg_stat_activity WHERE wait_event_type != 'undefined' AND wait_event != 'undefined';",
    #  "_vacuum"     => "SELECT datid, datname, phase FROM pg_stat_progress_vacuum WHERE phase != 'undefined';",
    "_relations"  => "SELECT nspname AS \"Namespace\", relname AS \"Name\", relkind AS \"Type\",
                             pg_relation_size(C.oid) AS \"Size\", relpages AS \"Pages\",reltuples AS \"Rows\"
                      FROM pg_class C
                      LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace)
                      WHERE nspname NOT IN ('pg_catalog', 'information_schema')
                      ORDER BY pg_relation_size(C.oid) DESC
                      LIMIT 50;"
  );

  return %queries;
}

sub rrd_check {
  my %_empty_dbs;
  for my $id (keys %db_ids){
    $_empty_dbs{$db_ids{$id}} = {};
  }

  return \%_empty_dbs;
}

sub to_gib {
  my $value = shift;
  #my $GiB   = 1073741824;
  my $GiB   = 1;
  if(!(defined $value) or $value eq 0){
    return 0;
  }
  return int($value);
}

sub to_tib {
  my $value = shift;
  my $GiB   = 1073741824;
  if(!(defined $value) or $value eq 0){
    return 0;
  }
  return sprintf("%.3f", (($value / $GiB) / 1024));
}

sub make_conf {
  my $rows = shift;
  my @r_array = @{$rows};
  my $cluster_id = shift;

  my %rel_types = (
    "r" => "ordinary table",
    "i" => "index",
    "S" => "sequence",
    "v" => "view",
    "m" => "materialized view",
    "c" => "composite type",
    "t" => "TOAST table",
    "f" => "foreign table",
  );

  #my $cluster_id = PostgreSQL::md5_string("$_parent-$_alias-DBFOLDERS");
  #my $uuid = PostgreSQL::md5_string($db);

  my %conf;
  foreach my $row_type (@r_array){
    if($row_type->{type} eq "_main"){
      #my @array;
      foreach my $row (@{$row_type->{data}}){
        my %hash_row = %{$row};
        $conf{$cluster_id}{main}{$row->{name}} = $row->{setting};
        #push(@array, \%hash_row);
      }
      my @array = $conf{$cluster_id}{main};
      $conf{$cluster_id}{main} = encode_json(\@array);
    }elsif($row_type->{type} eq "_relations"){
      my @array;
      foreach my $row (@{$row_type->{data}}){
        my %hash_row = %{$row};
        $hash_row{Type} = $rel_types{$hash_row{Type}};
        #$hash_row{Size} = to_gib($hash_row{Size});
        push(@array, \%hash_row);
      }
      my $uuid = PostgreSQL::md5_string($full_db_names{$row_type->{db_name}}."-$alias");
      $conf{$uuid}{relations} = encode_json(\@array);
    }
  }

  return \%conf;
}

sub prepare_perf {
  my $_parent = shift;
  my $_alias = shift;
  my $_act_perf = shift;
  my %json;
  my $cluster_id = PostgreSQL::md5_string("$_parent-$_alias-DBFOLDERS");
  for my $db (keys %{$_act_perf}){
    next if ($db eq "_info");
    my @db_parts = split("-", $db);
    pop(@db_parts);
    my $name = join("-", @db_parts);
    $full_db_names{$name} = $db;
    my $uuid = PostgreSQL::md5_string("$db-$alias");
    my $subsystem = "";
    my $label = "";
    if($db eq "_cluster"){
      $label = $_alias;
      $uuid = $cluster_id;
      $subsystem = "DBFOLDERS";
    }else{
      if (defined $use_whitelist and $use_whitelist and !defined $whitelisted{$name}) {
        next;
      }
      $label = $name;
      $subsystem = "DB";
    }


    #architecture
    my %cluster_arch = (
        "item_id" => $uuid,
        "label" => $label,
        "hw_type" => "POSTGRES",
        "subsystem" => $subsystem,
        "parents" => [$cluster_id],
        "hostcfg_id" => $_parent
    );
    push(@architecture, \%cluster_arch);

    my $sb = lc($subsystem);
    if ($_act_perf->{$db}) {
      push(@health_status, { "item_id" => $uuid, "type" => "data", "status" => "ok", "data" => "" });
    } else {
      push(@health_status, { "item_id" => $uuid, "type" => "data", "status" => "error", "data" => "Couldn't collect data" });
    }
    $json{postgres}{$sb}{$uuid}{$_act_perf->{_info}->{timestamp}} = $_act_perf->{$db};
  }

  return \%json;
}

sub compare_perf {
  my $alias = shift;
  my $_act_perf = shift;
  my ( $can_read, $ref );
  my $mvc_dir = $FindBin::Bin;
  my $iostats_dir = "$mvc_dir/iostats";


  unless ( -d $iostats_dir ) {
    mkdir( "$iostats_dir", 0755 ) || warn( "Cannot mkdir $iostats_dir: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }

  my $data = work_counters($alias, $_act_perf, $iostats_dir);
  if($data and $data ne 0){
    write_json( "$iostats_dir/".$alias."_pstgrs_perf.json-nperf", $data);
    return $data;
  }else{
    return 0;
  }
}

sub work_counters {
  my $alias = shift;
  my $_data = shift;
  my $iostats_dir = shift;
  my %act_perf = %{$_data};
  my %new_perf;
  my %old_perf;

  my %counters = %{PostgreSQL::get_counters()};
  my $perf_l = "$iostats_dir/".$alias."_pstgrs_perf_last.json";
  my $timediff = 1;
  my ($p_can_read, $p_ref);
  if(-e $perf_l){
    my $modtime = (stat($perf_l))[9];
    $timediff = time - $modtime;
    if($timediff == 1) {
      unlink($perf_l);
      write_json( "$perf_l", $_data);
      return 0;
    }
    ( $p_can_read, $p_ref ) = read_json($perf_l);
  }else{
    write_json( "$perf_l", $_data);
    return 0;
  }
  if($p_can_read){
    %old_perf = %{$p_ref};
    for my $db (keys %act_perf){
      next if($db eq "_info");
      if($db eq "_cluster"){
        for my $metric (keys %{$act_perf{$db}}){
          my $ntrd_metric = $metric;
          $ntrd_metric =~ s/_//g;
          $ntrd_metric =~ s/ //g;
          if($counters{$metric}){
            $new_perf{$db}{$ntrd_metric} = ($act_perf{$db}{$metric} - $old_perf{$db}{$metric}) / $timediff;
          }else{
            $new_perf{$db}{$ntrd_metric} = $act_perf{$db}{$metric};
          }
        }
      }else{
        for my $metric (keys %{$act_perf{$db}}){
          my $ntrd_metric = $metric;
          $ntrd_metric =~ s/_//g;
          $ntrd_metric =~ s/ //g;
          if($counters{$metric}){
            next if(!defined $old_perf{$db}{$metric} or $old_perf{$db}{$metric} eq "0" or
                      $old_perf{$db}{$metric} == 0 or $act_perf{$db}{$metric} < $old_perf{$db}{$metric});
            #if(!defined $old_perf{$db}{$metric} or $old_perf{$db}{$metric} eq "0" or
            #     $old_perf{$db}{$metric} == 0 or $act_perf{$db}{$metric} < $old_perf{$db}{$metric}){ print "$db $metric $act_perf{$db}{$metric} and $old_perf{$db}{$metric}\n";}

            $new_perf{$db}{$ntrd_metric} = int(($act_perf{$db}{$metric} - $old_perf{$db}{$metric}) / $timediff);
          }else{
            $new_perf{$db}{$ntrd_metric} = $act_perf{$db}{$metric};
          }
        }
      }
    }
    my @dbs = keys %act_perf;
    foreach my $db (@dbs){
      my $delimiter = 1;
      if(defined $new_perf{$db}{blkshit} and defined $new_perf{$db}{blksread}){
        $delimiter = ($new_perf{$db}{blkshit} + $new_perf{$db}{blksread});
        if ($delimiter == 0){
          $delimiter = 1;
        }
        $new_perf{$db}{cachehitratio} = 100*($new_perf{$db}{blkshit} / $delimiter);
      }

      if(defined $new_perf{$db}{xactcommit} and defined $new_perf{$db}{xactrollback}){
        $new_perf{$db}{currenttransactions} = $new_perf{$db}{xactcommit} + $new_perf{$db}{xactrollback};
        $delimiter = 1;
        $delimiter = $new_perf{$db}{xactcommit} + $new_perf{$db}{xactrollback};
        if ($delimiter == 0){
          $delimiter = 1;
        }
        $new_perf{$db}{commitratio} = 100*($new_perf{$db}{xactcommit} / $delimiter);
      }
    }

    $new_perf{_info} = $act_perf{_info};

    write_json($perf_l, \%act_perf);
    return \%new_perf;
  }else{
    write_json($perf_l, \%act_perf);
    return 0;
  }
}

sub write_json {
  my $path        = shift;
  my $hash_ref    = shift;
  my $path_suffix = "$path-tmp" . $$;
  my $json        = JSON->new->utf8;

  my ($fh, $file_name) = tempfile( UNLINK => 1 ); #use tempfile instead of the $path_suffix (HD)

  if ( ref($hash_ref) ne "HASH" && ref($hash_ref) ne "ARRAY"){
    warn ("Hash ref or array ref expected (path:$path), got: \"$hash_ref\" (Ref:" . ref($hash_ref) . ") in \n");
    return 0;
  }
#  if ($ENV{JSON_PRETTY}){
    $json->pretty([1]);
#  }

  #open( my $FILE, '>', $path_suffix ) || error( "Couldn't open file $path_suffix $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  #print $FILE $json->encode( $hash_ref );
  #close $FILE;
  #rename($path_suffix, $path);

  print $fh $json->encode( $hash_ref );
  close($fh);

  copy($file_name, $path);

  return 1;
}

sub read_json {
  my $file = shift;
  my $json = JSON->new->utf8;
  my $string;
  my $output;

  {
    local $/=undef;
    open ( my $FILE, '<', $file ) or warn "Couldn't open file $file $!" && return ( 0, undef );
    $string = <$FILE>;
    close $FILE;
  }

  if ($string) {
    eval {
      $output = $json->decode($string);
      1;
    } or do {
      warn "Couldn't parse json file $file $@ " && return ( 0, undef );
    };
  }

  return ( 1, $output );
}



$server->close();
