package CMC;

use strict;
use warnings;

use CMC_COMMON;
use CMC_COLLECT;
use CMC_CSV;
use Xormon;

use Data::Dumper;

my $HW_TYPE = "power";

# TODO: change units (?)

# This should be equivalent in purpose to power_cmc_caller in lpar2rrd
sub collect {

  my $xormon  = shift;
  my $cfg     = shift;

  my ($host, $username, $password, $proxy_url) = CMC_COMMON::load_config($cfg);

  my $hostcfg_id    = $cfg->{hostcfg_id};
  my $console_name  = $cfg->{hostalias};
  my $console_id    = $host; # NOTE


  print "==============================================================================\n";
  print "Collect CMC \n";
  print "==============================================================================\n";

  #my @refs = CMC_COLLECT::collect($host, $username, $password, $proxy_url);
  my ($data_ref,
      $data_timed_ref,
      $credit_info) = CMC_COLLECT::collect( $host, $username, $password, $proxy_url);



  print "==============================================================================\n";
  print "Data collected \n";
  print "==============================================================================\n";

  my %data                = %{$data_ref};
  # *section - *id - Configuration - *name - value
  my %data_timed          = %{$data_timed_ref};
  # *section - *id - Metrics - *timestamp - *name - value

  print "==============================================================================\n";
  print "Architecture \n";
  print "==============================================================================\n";

  my ($architectures_ref, $known_ids_ref) = prepare_architecture( "hostcfg_id"    => $hostcfg_id,
                                                                  "console_id"    => $console_id,
                                                                  "console_name"  => $console_name,
                                                                  "data_ref"      => $data_ref);

  my %architectures = %{$architectures_ref};
  my %known_ids     = %{$known_ids_ref};

  my @architecture_save_order = (
    'console',
    'pool',
    'server',
  );

  #print Dumper @{$architectures{'console'}};

  for my $section (@architecture_save_order) {

    my @section_arch = @{$architectures{$section}};
    #print Dumper @section_arch;
    my $arch_count = scalar @section_arch;

    print "Save architecture: $section [${arch_count}]\n";
    $xormon->saveArchitecture(\@section_arch);

  }

  print "==============================================================================\n";
  print "Configuration \n";
  print "==============================================================================\n";

  my %section_rename = (
    "Pools"   => "pool",
    "Systems" => "server",
  );

  my @allowed_sections = keys %section_rename;

  for my $section (keys %section_rename) {
    my $renamed_section = $section_rename{$section};

    print("Make configuration - section:$section \n");

    for my $UUID (keys %{$data{$section}}) {
      my %configuration_hashed = ();

      my $save_uid = CMC_COMMON::unique_id( ${console_id}, $renamed_section, $UUID );

      #print("[CONF] PROCESS: $section - id: $save_uid \n");

      if ( defined $known_ids{$save_uid} ) {
        $configuration_hashed{$save_uid} = $data{$section}{$UUID}{'Configuration'};
      }
      else {
        print("[CONF] WARNING: $section - unknown id: $save_uid \n");
      }

      $configuration_hashed{$save_uid}{'Name'} = $data{$section}{$UUID}{'Name'};

      #print Dumper %configuration_hashed;
      $xormon->saveConfJSON(\%configuration_hashed);

    }

    print "Saving $section Configuration \n";

  }

  print "Saving Credit Info \n";
  my %credit_per_interval = ();

  for my $pool_id_loc ( keys %{$credit_info} ) {
    my $id_to_save = CMC_COMMON::unique_id($console_id, "pool", $pool_id_loc);

    if ( defined $known_ids{$id_to_save} ) {
      $credit_per_interval{$id_to_save} = $credit_info->{$pool_id_loc};
    }
    else {
      print("[CONF] WARNING: CREDITS - unknown id: $id_to_save \n");
    }

  }

  $xormon->saveConfJSON(\%credit_per_interval);

  print "==============================================================================\n";
  print "Performance \n";
  print "==============================================================================\n";

  my %govern_perf_hash = %{CMC_CSV::get_govern_hash_timed()};

  print "==============================================================================\n";

  # Structure: section - csv
  my %section_csv_perf = %{CMC_CSV::make_csv(\%data_timed, \%govern_perf_hash, $console_id, $host)};

  print "==============================================================================\n";

  # SAVE performance
  for my $section (keys %section_csv_perf){
    my $csv_string = $section_csv_perf{$section};
    print "Saving CSV - Section: $section\n";

    my $rc = $xormon->saveCSV($csv_string, $HW_TYPE, "cmc$section");

    print "Response code: $rc \n";

  }

  return 1;
}

#
# CMC
#  |
#  |- Pool
#  |   |
#  `---`--- Server
#
sub prepare_architecture {
  my %Params = @_;

  my $_hostcfg_id   = $Params{"hostcfg_id"};
  my $console_id    = $Params{"console_id"};
  my $console_name  = $Params{"console_name"};
  my $data_ref      = $Params{"data_ref"};

  my %data = %{$data_ref};

  my %architectures = ();
  my %known_ids = ();

  print "[ARCH]- console: $console_name [$console_id]\n";

  my @architecture_console = ();
  $known_ids{$_hostcfg_id} = 1;
  $known_ids{$console_id} = 1;

  my $console_arch = {
    "hw_type"     => $HW_TYPE,
    "hostcfg_id"  => $_hostcfg_id,
    "parents"     => [],
    "subsystem"   => "device",
    "item_id"     => "top-$console_id",
    "label"       => $console_name,
  };

  my $cmc_arch = {
    "hw_type"     => $HW_TYPE,
    "hostcfg_id"  => $_hostcfg_id,
    "parents"     => ["top-$console_id"],
    "subsystem"   => "cmc",
    "item_id"     => $console_id,
    "label"       => $console_name,
  };

  push(@architecture_console, $console_arch);
  push(@architecture_console, $cmc_arch);
  $architectures{'console'} = \@architecture_console;


  print "[ARCH]- pool\n";
  my @pool_architecture = ();

  for my $pool_id (keys %{$data{'Pools'}}) {
    my $pool_name = $data{'Pools'}{$pool_id}{"Name"};

    my $item_id = CMC_COMMON::unique_id($console_id, "pool", $pool_id);
    $known_ids{$item_id} = 1;

    my $console_arch = {
      "hw_type"     => $HW_TYPE,
      "hostcfg_id"  => $_hostcfg_id,
      "parents"     => ["$console_id"],
      "subsystem"   => "cmcpool",
      "item_id"     => $item_id,
      "label"       => $pool_name,
    };
    push(@pool_architecture, $console_arch);
  }

  $architectures{'pool'} = \@pool_architecture;


  print "[ARCH]- server\n";
  my @server_architecture = ();

  for my $server_id (keys %{$data{'Systems'}}) {
    my $server_name = $data{'Systems'}{$server_id}{"Name"};

    my @pool_parents = keys %{$data{'Systems'}{$server_id}{"Pools"}};

    my @server_parents = ();

    push(@server_parents, $console_id);


    for my $parent_pool (@pool_parents) {

      my $pool_uuid = CMC_COMMON::unique_id($console_id, "pool", $parent_pool);
      push(@server_parents, $pool_uuid);

    }


    my $item_id = CMC_COMMON::unique_id($console_id, "server", $server_id);
    $known_ids{$item_id} = 1;

    my $server_arch = {
      "hw_type"     => $HW_TYPE,
      "hostcfg_id"  => $_hostcfg_id,
      "parents"     => \@server_parents,
      "subsystem"   => "cmcserver",
      "item_id"     => $item_id,
      "label"       => $server_name,
    };

    push(@server_architecture, $server_arch);
  }
  $architectures{'server'} = \@server_architecture;

  return (\%architectures, \%known_ids);
}


1;
