package PURE;

use strict;
use warnings;

use Data::Dumper;
use JSON qw(encode_json decode_json);
use Date::Parse;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Xormon;
use POSIX ":sys_wait_h";
use LWP::UserAgent;
use File::Copy;
use File::Temp qw(tempfile);
use Test;
use XormonLogger;
my $logger;

my %data;
my $xormon;
my $cfg;
my $stats_found = 0;
my $api_version = "";
my $urls;
my $data_directory = "$FindBin::Bin";
my $debug = 0;

my $storage_name = "";
my $active_minute = "";
my $debug_data = "";
my %performance_data;
my %configuration_data;
my $st_serial;
my @architecture = ();
my $service_data_out;
my $st_type = "pure";
my @rows_api_test = ();
my @rows_api_test_data = ();
my $protocol = "";
my $wrkdir = "";
my $service_data_in;
my $connection_test = 0;
my $cookie = "";
my @health_status = ();
my %hosts;
my @hardware = ();
my $replication_capacity_used = "";
my @health_status_global = ();
my $enviroment = 1;
my $debug_data_configuration = "";





sub collect {
  $xormon = shift;
  $cfg    = shift;

 #print Dumper \$cfg;
 #print "ahoj\n";
  #return "successful";

  #$VAR1 = \{
  #9|pure  |             'updated' => undef,
  #9|pure  |             'apiHost' => '10.5.5.1',
  #9|pure  |             'created' => '1683031339796',
  #9|pure  |             'device' => 'PURE_STORAGE__FLASH_ARRAY',
  #9|pure  |             'hostalias' => 'pure011',
  #9|pure  |             'hostcfg_id' => 'b95ab658-e459-4510-a5ab-dfb1a3493ad5',
  #9|pure  |             'apiUsername' => 'stor2rrd'
  #9|pure  |           };
  unless ( exists $cfg->{'hostcfg_id'} )        { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!") && return "error"; }
  unless ( exists $cfg->{'hostalias'} )         { Xormon::error("Cannot continue, 'hostalias' is not configured!") && return "error"; }
  $logger = XormonLogger->new( 'pure', $cfg->{hostcfg_id}, $cfg->{hostalias});
  unless ( exists $cfg->{'apiHost'} )          { $logger->error("Cannot continue, 'apiHost' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )      { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )      { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  my $act_timestamp     = time();
  $service_data_in = $xormon->getData();
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }
  if (defined $cfg->{PURE_ENVIROMENT} && $cfg->{PURE_ENVIROMENT} eq "development"){
    $enviroment = 0;
  }

  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  data_structure();
  $api_version = get_version_api();
  if ($api_version eq "" ){
    $xormon->isConnected(0, "Connection failed!");
    $logger->error("REST API version is not defined") && return "error";
  }
  $cookie = get_cookie();
  if ($cookie eq "" ){
    $xormon->isConnected(0, "Connection failed!");
    $logger->error("Cookie is not defined") && return "error";
  }
  $xormon->isConnected(1);
  for (my $i = 1; $i < 11; $i++){
    set_data_host($i);
    set_data_host_group($i);
    sleep 30;
  }
  set_data_volume();
  set_data_pool();
  set_data_pod();
  set_data_device();
  #print Dumper \%hosts;
  logout();
  delete_empty_metrics();
  calculate_missing_metrics();
  #print Dumper \%performance_data;
  my $rc = $xormon->saveJSON2(\%performance_data);
  #print Dumper \%performance_data;
  my $status = get_status($rc, "saveJSON2 (performance)");

  ### host_cfg_data
  if (Xormon::isdigit($service_data_out->{last_perf_timestamp})){
    if ( ( time() -  $service_data_out->{last_perf_timestamp} ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_out->{last_perf_timestamp});
    }
    else{
      $xormon->hasData(1);
    }
  }
  else{
    $xormon->hasData(0, "no available stats");
  }

  my $save_data = $xormon->saveData(\%{ $service_data_out });
  my $status_save_data = get_status($save_data, "SaveData");
  #print Dumper \%performance_data;
  if (( ! exists $service_data_in->{last_conf_timestamp} ) || ( ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) || ( $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) || ( !-f "$debug_data_configuration/config-$st_serial.out" )){
    `touch "$debug_data_configuration/config-$st_serial.out"`;
    $debug_data = "$data_directory/data/$st_serial/IOSTATS";
    $api_version = get_version_api();
    if ($api_version eq "" ){
      $logger->error("REST API version is not defined") && return "error";
    }
    $cookie = get_cookie();
    if ($cookie eq "" ){
      $logger->error("Cookie is not defined") && return "error";
    }
    set_data_pool_conf();
    set_data_volume_conf();
    set_data_host_conf();
    set_data_host_group_conf();
    set_data_pod_conf();
    #print Dumper \%configuration_data;
    set_data_health_status();
    logout();
    my @types = ("pool", "volume", "host", "host_group", "pod");
    foreach my $uid (keys %configuration_data){
      my $active = 0;
      foreach my $type (@types){
        if (defined $performance_data{$st_type}{$type}{$uid}){
          $active++;
          last;
        }
      }
      if ($active == 0){
        #print Dumper \$configuration_data{$uid};
        delete $configuration_data{$uid};
        #print "mazeme prvek $uid\n";
      }
    }
    $service_data_out->{last_conf_timestamp} = $act_timestamp;
    ($status, my $status_configuration, my $status_health) = set_architecture();
    if ($status eq "successful" && $status_configuration eq "successful" && $status_health eq "successful" ){
      my $save_data = $xormon->saveData(\%{ $service_data_out });
      my $status_save_data = get_status($save_data, "saveData");
    }
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
    return $status;
    #print Dumper \%configuration_data;
  }
  $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
  return $status;
}

sub get_cookie{
  if (!$enviroment){
    return "123456";
  }
  my %hash_filter;
  $hash_filter{"username"} = $cfg->{'apiUsername'};
  $hash_filter{"password"} = $cfg->{'apiPassword'};
  my $json_body = encode_json( \%hash_filter );
  my $data = cmd("$api_version/auth/apitoken", "POST", 1, "apitoken", $json_body);
  if ($data ne "" && defined $data->{api_token}){
    my %body;
    $body{"api_token"} = $data->{api_token};
    my $json_body = encode_json( \%body );
    my $cookie_data = cmd("$api_version/auth/session", "POST", 1, "cookie_data", $json_body, "header");
    return $cookie_data->headers->{"set-cookie"};

  }
  return "";
}

sub get_version_api{
  my $data = cmd("api_version", "GET", 1, "api_version");
  if ($data eq "" || !defined $data->{version}){
    return "";
  }
  my $api_version = $data->{version};
  if ( ref($api_version) eq "ARRAY" ) {
    my $version = pop @{$api_version};
    if ( defined $version && $version =~ m/^2/ ) {
      my $count = scalar @{$api_version};
      for ( my $i = 0; $i < $count; $i++ ) {
        my $last_one = pop @{$api_version};
        if ( defined $last_one && $last_one =~ m/^1/ ) {
          return $last_one;
        }
      }
      return $version;
    }
    return $version;
  }
  return $api_version;
}

sub set_data_health_status{
  my $data = cmd("$api_version/message?open=true", "GET", 1, "alerts");
  cmd("$api_version/message?flagged=true", "GET", 1, "alerts_flagged");
  #my $data2 = cmd("$api_version/message?open=true", "GET", 1, "alerts2");
  #print Dumper \$data2;

  if ( !defined $data || $data eq "" || ref($data) ne "ARRAY" ) {
    $data = ();
  }

  #my @hs = ();
  my $message_global = "";
  my $status_global = "ok";

  foreach my $element ( @{$data} ) {
    if ( !defined $element->{id} || !defined $element->{current_severity} ) { next; }
    my $id             = $element->{id};
    my $severity       = $element->{current_severity};
    my $created        = $element->{opened};
    my $code           = $element->{code};
    my $component_type = $element->{component_type};
    my $component_name = $element->{component_name};
    my $event          = $element->{event};
    my $detail         = $element->{details};

    if ( !defined $created ) {
      $created = "";
    }
    else{
      $created = str2time($created);
    }
    if ( !defined $code ) {
      $code = "";
    }
    if ( !defined $component_type ) {
      $component_type = "";
    }
    if ( !defined $component_name ) {
      $component_name = "";
    }
    if ( !defined $event ) {
      $event = "";
    }
    if ( !defined $detail ) {
      $detail = "";
    }
    my $summary = "($component_type:$component_name): $event";
    if ( $severity =~ m/critical/i ) {
      $status_global = "error";
      $message_global = $detail;
      if ($message_global eq ""){
        $message_global = $event;
      }
    }
    if ( $severity =~ m/warning/i && $status_global ne "error"){
      $status_global = "warning";
      $message_global = $detail;
      if ($message_global eq ""){
        $message_global = $event;
      }
    }
    my %data;
    $data{severity} = $severity;
    $data{created} = $created;
    $data{code} = $code;
    $data{summary} = $summary;
    $data{issue} = $event;
    $data{detail} = $detail;
    $data{id} = $id;
    push(@health_status,\%data);
  }

  ## hardware
  $data = cmd("$api_version/hardware", "GET", 1, "hardware");
  if ( !defined $data || $data eq "" || ref($data) ne "ARRAY" ) {
    $data = ();
  }
  foreach my $element ( @{$data} ) {
    if ( !defined $element->{name} ) { next; }
    my $name        = $element->{name};
    my $details     = $element->{"details"};
    my $identify    = $element->{"identify"};
    my $slot        = $element->{"slot"};
    my $speed       = $element->{"speed"};
    my $status      = $element->{"status"};
    my $temperature = $element->{"temperature"};
    my $voltage     = $element->{"voltage"};
    if ( !defined $details )     { $details     = ""; }
    if ( !defined $identify )    { $identify    = ""; }
    if ( !defined $slot )        { $slot        = ""; }
    if ( !defined $speed )       { $speed       = ""; }
    if ( !defined $status )      { $status      = ""; }
    if ( !defined $temperature ) { $temperature = ""; }
    if ( !defined $voltage )     { $voltage     = ""; }
    if ( $status !~ m/ok/i && $status !~ m/warn/i && $status !~ m/not_installed/i && $status_global ne "error") {
      $status_global = "error";
      $message_global = $details;
    }
    if ($status =~ m/warn/ && $status_global eq "ok"){
      $status_global = "warning";
      $message_global = $details;
    }
    my %data;
    $data{name} = $name;
    $data{status} = $status;
    $data{identify} = $identify;
    $data{details} = $details;
    $data{slot} = $slot;
    $data{speed} = $speed;
    $data{temperature} = $temperature;
    $data{voltage} = $voltage;
    push(@hardware,\%data);
  }

  my %vm_status = (
    "item_id" => $st_serial,
    "status" => $status_global,
    "data" => $message_global
  );
  #print "$st_serial | $status_global | $message_global\n";
  push(@health_status_global, \%vm_status);
  #my $rc = $xormon->saveStatus(\@hs);
  #my $status = get_status($rc, "health status");
  #$logger->info("Task healh status ".$cfg->{hostalias}." is done. Status: $status");
  return;
}

sub get_utc_time{
  my $d = shift;
  my ($date, $time) = split ("T",$d);
  if (defined $date && defined $time){
    my @t = ();
    $date =~ s/-/\//g;
    $t[0] = $date;
    $t[1] = $time;
    my ($t, $timezone) = split(/\+/,$time);
    if (defined $t){
      $t[1] = $t;
    }
    if (!defined $timezone || $timezone eq ""){
      (undef, $timezone) = split(/-/,$time);
      if (defined $timezone && $timezone ne ""){
        $timezone = "-$timezone";
      }
    }
    else{
      $timezone = "+$timezone";
    }
    if (defined $timezone && $timezone ne ""){
      $t[2] = $timezone;
    }
    #print Dumper \@t;
    return str2time( join( " ", @t ) );
  }
  return $d;
}

sub set_data_device{
  my $type = "pool";
  my @metrics = ("read_io","write_io","read","write", "resp_t_r" ,"resp_t_w","io_rate", "data_rate", "resp_t","capacity_total", "capacity_used", "capacity_free", "shared_space_capacity_used", "volumes_capacity_used", "system_capacity_used", "virtual_capacity_total", "snapshot_capacity_used", "replication_capacity_used", "local_io_write", "local_write", "local_resp_t_w", "mirrored_io_write", "mirrored_write", "mirrored_resp_t_w" );
  my %data;
  my $time = "";
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    $time = (keys %{$performance_data{$st_type}{$type}{$uid}})[0];
    foreach my $metric (@metrics){
      if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$metric})){
        if (Xormon::isdigit($data{$metric})){
          $data{$metric} = $data{$metric} + $performance_data{$st_type}{$type}{$uid}{$time}{$metric};
        }
        else{
          $data{$metric} = $performance_data{$st_type}{$type}{$uid}{$time}{$metric};
        }
      }
    }
  }
  foreach my $metric (keys %data){
    $performance_data{$st_type}{device}{$st_serial}{$time}{$metric} = $data{$metric};
  }
  #$performance_data{$st_type}{device}{$st_serial}{$time} = \%data;
  #print Dumper \$performance_data{$st_type}{device};
  #print Dumper \%data;
}

sub get_status {
  my $rc = shift;
  my $task = shift;
  if ($rc eq "201" || $rc eq "200") {
    $logger->info("Task for $task ".$cfg->{hostalias}." is done. Status: successful");
    return "successful";
  } elsif ($rc eq "401") {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: unauthorized");
    return "unauthorized";
  } else {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
}


sub set_data_pool_conf{

  my $section     = "pool";
  my $data_config = cmd("$api_version/array", "GET", 1, $section);
  my $data_capacity = cmd("$api_version/array?space=true", "GET", 1, "pool_capacity");
  my $data_ctls = cmd("$api_version/array?controllers=true", "GET", 1, "controllers");
  my $data_performance = cmd("$api_version/array?action=monitor&historical=1h", "GET", 1, "array_historical");
  #print Dumper \$data_capacity;
  #return;
  #my $data_performance = cmd("$api_version/volume?action=monitor&names=rco-vg01/vol01&historical=1h", "GET", 1);
  my %test;
  if (!defined $data_config || $data_config eq "" || !defined $data_config->{id} || !defined $data_config->{array_name}){return;}
  my $id = $data_config->{id};
  my $name = $data_config->{array_name};

  $cfg->{name} = $name;
  $cfg->{serial} = $id;
  $cfg->{apiVersion} = $api_version;
  if (defined $data_config->{version}){
    $cfg->{purity_version} = $data_config->{version};
    $cfg->{version} = $data_config->{version};
  }

  $test{$id}{id} = $id;
  $test{$id}{name} = $name;
  if ($data_capacity ne "" && ref($data_capacity) eq "HASH"){
    my %capacity = %{$data_capacity};
    $data_capacity = undef;
    push(@{$data_capacity},\%capacity);
  }
  if ($data_capacity ne "" && ref($data_capacity) eq "ARRAY"){
    foreach my $element (@{$data_capacity}){
      if (!defined $element->{hostname} || $element->{hostname} ne $name){next;}
      if (defined $test{$id} && Xormon::isdigit($element->{total})){
        $test{$id}{capacity_used} = $element->{total};
      }
      if (defined $test{$id} && Xormon::isdigit($element->{capacity})){
        $test{$id}{capacity_total} = $element->{capacity};
      }
      if (defined $test{$id} && Xormon::isdigit($element->{system})){
        $test{$id}{system_capacity_used} = $element->{system};
      }
      if (defined $test{$id} && Xormon::isdigit($element->{shared_space})){
        $test{$id}{shared_space_capacity_used} = $element->{shared_space};
      }
      if (defined $test{$id} && Xormon::isdigit($element->{volumes})){
        $test{$id}{volumes_capacity_used} = $element->{volumes};
      }
      #if (defined $test{$id} && Xormon::isdigit($element->{data_reduction})){
      #  $test{$id}{data_reduction_ratio} = $element->{data_reduction};
      #}
      #if (defined $test{$id} && Xormon::isdigit($element->{total_reduction})){
      #  $test{$id}{total_reduction} = $element->{total_reduction};
      #}
      if (defined $test{$id} && Xormon::isdigit($element->{provisioned})){
        $test{$id}{virtual_capacity_total} = $element->{provisioned};
      }
      if (defined $test{$id} && Xormon::isdigit($element->{"snapshots"})){
        $test{$id}{snapshot_capacity_used} = $element->{"snapshots"};
      }
    }
  }
  if (Xormon::isdigit($test{$id}{capacity_total}) && Xormon::isdigit($test{$id}{capacity_used})){
    $test{$id}{capacity_free} = $test{$id}{capacity_total} - $test{$id}{capacity_used};
  }
  if (Xormon::isdigit($replication_capacity_used)){
    $test{$id}{replication_capacity_used} = $replication_capacity_used;
  }
  #print Dumper \%test;
  convert_data($section,\%test, "conf");
  #print Dumper \%performance_data;
  if ( $data_ctls eq "" ) { return; }
  if ( ref($data_ctls) eq "ARRAY" ) {
    foreach my $element ( @{$data_ctls} ) {
      if ( defined $element->{"model"} ) {
        $cfg->{model} = $element->{"model"};
      }
    }
  }
  else {
    if ( defined $data_ctls->{model} ) {
      $cfg->{model} = $data_ctls->{model};
    }
  }
  #print Dumper \$cfg;
}


sub set_data_host_conf{
  my $section = "host";
  my $data_config = cmd("$api_version/host", "GET", 1, $section);
  my $data_capacity = cmd("$api_version/host?space=true", "GET", 1, "host_capacity");
  #print Dumper \$data_config;
  my %test;
  if ($data_config eq ""){return;}
  foreach my $element (@{$data_config}){
    if (!defined $element->{name} || $element->{name} eq ""){next;}
    $test{$element->{name}}{id} = $element->{name};
    my $wwn = $element->{wwn};
    if (defined $wwn && @{$wwn}){
      $wwn = join( " ", @{$wwn} );
      $test{$element->{name}}{wwn} = $wwn;
    }
    my $iqn = $element->{iqn};
    if (defined $iqn && @{$iqn}){
      $iqn = join( " ", @{$iqn} );
      $test{$element->{name}}{iqn} = $iqn;
    }
    my $hgroup    = $element->{"hgroup"};
    if (defined $hgroup && $hgroup ne ""){
      my $uid = Xormon::create_hash_key($st_serial."host_group".$hgroup);
      if (defined $performance_data{$st_type}{"host_group"}{$uid}){
        push(@{$test{$element->{name}}{parents}},$uid);
        $test{$element->{name}}{host_group} = $hgroup;
      }
    }
    ### mapping host volumes
    my $data = cmd("$api_version/host/" . $element->{name} . "/volume", "GET", 1, $element->{name} . "-volumes");
    foreach my $volume (@{$data}){
      my $vol = $volume->{vol};
      foreach my $instance (keys %configuration_data){
        if (defined $configuration_data{$instance}{label} && defined $configuration_data{$instance}{pool_id} && $vol eq $configuration_data{$instance}{label}){
          push(@{$test{$element->{name}}{children}}, $instance);
          last;
        }
      }
    }
    #print Dumper \$data;
  }
  if ($data_capacity ne ""){
    foreach my $element (@{$data_capacity}){
      if ( !defined $element->{name} || $element->{name} eq "" ){next;}
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{size})){
        $test{$element->{name}}{capacity_total} = $element->{size};
      }
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{total})){
        $test{$element->{name}}{capacity_used} = $element->{total};
        if (Xormon::isdigit($test{$element->{name}}{capacity_total})){
          $test{$element->{name}}{capacity_free} = $test{$element->{name}}{capacity_total} - $test{$element->{name}}{capacity_used};
        }
      }
    }
  }
  #print Dumper \%test;
  convert_data($section,\%test, "conf");
}


sub set_data_volume_conf{
  my $section = "volume";
  my %test;
  my $data_config = cmd("$api_version/volume", "GET", 1, $section);
  my $data_capacity = cmd("$api_version/volume?space=true", "GET", 1, "volume_capacity");
  if ($data_config eq ""){return;}
  foreach my $element (@{$data_config}){
    if (!defined $element->{serial} || $element->{serial} eq "" || !defined $element->{name} || $element->{name} eq ""){next;}
    $test{$element->{name}}{id} = $element->{serial};
    $test{$element->{name}}{"lun_id"} = $element->{serial};
    if (Xormon::isdigit($element->{size})){
      $test{$element->{name}}{capacity_total} = $element->{size};
    }
  }
  if ($data_capacity ne ""){
    foreach my $element (@{$data_capacity}){
      if ( !defined $element->{name} || $element->{name} eq "" ){next;}
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{total})){
        $test{$element->{name}}{capacity_used} = $element->{total};
        if (Xormon::isdigit($test{$element->{name}}{capacity_total})){
          $test{$element->{name}}{capacity_free} = $test{$element->{name}}{capacity_total} - $test{$element->{name}}{capacity_used};
        }
      }
      if (Xormon::isdigit($element->{system})){
        $test{$element->{name}}{system_capacity_used} = $element->{system};
      }
      if (Xormon::isdigit($element->{shared_space})){
        $test{$element->{name}}{shared_space_capacity_used} = $element->{shared_space};
      }
      if (Xormon::isdigit($element->{volumes})){
        $test{$element->{name}}{volumes_capacity_used} = $element->{volumes};
      }
      #if (Xormon::isdigit($element->{data_reduction})){
      #  $test{$element->{name}}{data_reduction_ratio} = $element->{data_reduction};
      #}
      #if (Xormon::isdigit($element->{total_reduction})){
      #  $test{$element->{name}}{total_reduction} = $element->{total_reduction};
      #}
      if (Xormon::isdigit($element->{thin_provisioning}) && Xormon::isdigit($test{$element->{name}}{capacity_total})){
        $test{$element->{name}}{virtual_capacity_total} = $element->{thin_provisioning} * $test{$element->{name}}{capacity_total};
      }
      if (Xormon::isdigit($element->{"snapshots"})){
        $test{$element->{name}}{snapshot_capacity_used} = $element->{"snapshots"};
      }
    }
  }
  #print Dumper \%test;
  foreach my $uid (keys %configuration_data){
    foreach my $volume (keys %test){
      push(@{$test{$volume}{parents}}, $uid);
      $test{$volume}{pool_name} = $configuration_data{$uid}{name};
      $test{$volume}{pool_id} = $uid;
    }
  }
  convert_data($section,\%test,"conf");
  #print Dumper \%configuration_data;
}


sub sum_metric{
  my $metric = shift;
  my $data_tmp = shift;
  my @data = @{$data_tmp};
  my $response_metric = shift;
  my $numerator = shift;
  my $sum = "";
  my $numerator_sum = "";
  my $index = 0;
  foreach my $element (@data){
    if (Xormon::isdigit($element->{$metric})){
      #if ($element->{$metric} == 0){
      #  $element->{$metric} = int(rand(300));
      #}
      if (Xormon::isdigit($sum)){
        $sum = $sum + $element->{$metric};
        $index++;
      }
      else{
        $sum = $element->{$metric};
        $index++;
      }
      if (defined $response_metric && Xormon::isdigit($element->{$response_metric})){
        #if ($element->{$response_metric} == 0){
        #  my $x = 1000 + int(rand(3000 - 1000));
        #  $element->{$response_metric} = $x;
        #}
        if (Xormon::isdigit($numerator_sum)){
          $numerator_sum = $numerator_sum + ($element->{$response_metric} * $element->{$metric});
        }
        else{
          $numerator_sum = $element->{$response_metric} * $element->{$metric};
        }
      }
    }
  }
  if ($index == 0){return ("","","");}
  return ($sum / $index, $numerator_sum, $sum);
}

sub sum_metrics{
  my $data_tmp = shift;
  my $metric1 = shift;
  my $metric2 = shift;
  my %data = %{$data_tmp};
  my $sum = "";
  if (Xormon::isdigit($data{$metric1})){
    $sum = $data{$metric1};
  }
  if (Xormon::isdigit($data{$metric2})){
    if (Xormon::isdigit($sum)){
      $sum = $sum + $data{$metric2};
    }
    else{
      $sum = $data{$metric2};
    }
  }
  return $sum;
}

sub calculate_weighted_average{
  my $data_tmp = shift;
  my $numerator_metric1 = shift;
  my $denominator_metric1 = shift;
  my $numerator_metric2 = shift;
  my $denominator_metric2 = shift;


  my %data = %{$data_tmp};

  my $sum = "";
  my $denominator = "";
  if (Xormon::isdigit($data{$denominator_metric1}) && Xormon::isdigit($data{$numerator_metric1})){
    $sum = $data{$denominator_metric1} * $data{$numerator_metric1};
    $denominator = $data{$denominator_metric1};
  }
  if (Xormon::isdigit($data{$denominator_metric2}) && Xormon::isdigit($data{$numerator_metric2})){
    if (Xormon::isdigit($sum)){
      $sum = $sum + ($data{$denominator_metric2} * $data{$numerator_metric2});
      $denominator = $denominator + $data{$denominator_metric2};
    }
    else{
      $sum = $data{$denominator_metric2} * $data{$numerator_metric2};
      $denominator = $data{$denominator_metric2};
    }
  }
  if (Xormon::isdigit($sum) && Xormon::isdigit($denominator)){
    if ($denominator == 0){
      return 0;
    }
    return $sum / $denominator;
  }
  return "";
}

sub set_data_host {
  my $i = shift;
  my $section     = "host";
  #my $data_capacity = "";
  my $data_performance = cmd("$api_version/host?action=monitor", "GET", 1, "host_performance$i");
  if (ref($data_performance) eq "ARRAY"){
    foreach my $element (@{$data_performance}){
      if (!defined $element->{name}){next;}
      push(@{$hosts{$element->{name}}{performance}},$element);
    }
  }
  undef $data_performance;
  $data_performance = cmd("$api_version/host?action=monitor&mirrored=true", "GET", 1, "host_performance_mirror$i");
  if (ref($data_performance) eq "ARRAY"){
    foreach my $element (@{$data_performance}){
      if (!defined $element->{name}){next;}
      push(@{$hosts{$element->{name}}{performance2}},$element);
    }
  }
  #print Dumper \%hosts;
  if ($i < 10){return;}

  my $data_config = cmd("$api_version/host", "GET", 0, $section);

  #print Dumper \$data_config;
  my $data_capacity = cmd("$api_version/host?space=true", "GET", 0, "host_capacity");
  #my $data_performance = cmd("$api_version/volume?action=monitor&names=rco-vg01/vol01&historical=1h", "GET", 1);
  my %test;
  if ($data_config eq ""){return;}
  foreach my $element (@{$data_config}){
    if (!defined $element->{name} || $element->{name} eq ""){next;}
    $test{$element->{name}}{id} = $element->{name};
    #if (Xormon::isdigit($element->{size})){
    #  $test{$element->{name}}{capacity_total} = $element->{size};
    #}
    #my $data_performance = cmd("$api_version/host?action=monitor&names=" . $element->{name} . "", "GET", 1);
    #my $data_performance = cmd("$api_version/host?action=monitor&historical=1h", "GET", 1);
    #print Dumper \$data_performance;
    if (!defined $hosts{$element->{name}}{performance}){next;}
    my $data_performance = $hosts{$element->{name}}{performance};
    if (ref($data_performance) eq "ARRAY"){
      $test{$element->{name}}{performance} = $data_performance;
      $test{$element->{name}}{time} = $test{$element->{name}}{performance}[-1]{time};
      #for (my $i = 1; $i < 13;$i++){
      #  push(@{$test{$element->{name}}{performance}}, pop(@{$data_performance}));
      #  if ($i == 1){
      #    $test{$element->{name}}{time} = $test{$element->{name}}{performance}[0]{time};
      #  }
      #}
    }
    if (ref($test{$element->{name}}{performance}) eq "ARRAY"){
      my ($read_io, $numerator_read, $denominator) = sum_metric("reads_per_sec", $test{$element->{name}}{performance}, "usec_per_read_op");
      $test{$element->{name}}{read_io} = $read_io;
      if (Xormon::isdigit($numerator_read) && Xormon::isdigit($read_io)){
        if ($read_io == 0){
          $test{$element->{name}}{resp_t_r} = 0;
        }
        else{
          $test{$element->{name}}{resp_t_r} = $numerator_read / ($denominator * 1000);
        }
      }
      my ($write_io, $numerator_write, $denominator2) = sum_metric("writes_per_sec", $test{$element->{name}}{performance}, "usec_per_write_op");
      $test{$element->{name}}{local_write_io} = $write_io;
      if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
        if ($write_io == 0){
          $test{$element->{name}}{local_resp_t_w} = 0;
        }
        else{
          $test{$element->{name}}{local_resp_t_w} = $numerator_write / ($denominator2 * 1000);
        }
      }
      ($test{$element->{name}}{read}, undef) = sum_metric("output_per_sec",$test{$element->{name}}{performance});
      ($test{$element->{name}}{local_write}, undef) = sum_metric("input_per_sec",$test{$element->{name}}{performance});
      delete $test{$element->{name}}{performance};
      #$test{$element->{name}}{io_rate} = sum_metrics($test{$element->{name}},"read_io", "write_io");
      #$test{$element->{name}}{data_rate} = sum_metrics($test{$element->{name}},"read", "write");
      #$test{$element->{name}}{data_rate} = sum_metrics($test{$element->{name}},"read", "write");
    }

    $data_performance = undef;
    #if (Xormon::isdigit($test{$element->{name}}{write_io}) && $test{$element->{name}}{write_io} == 0 && Xormon::isdigit($test{$element->{name}}{write}) && $test{$element->{name}}{write} == 0 && Xormon::isdigit($test{$element->{name}}{resp_t_w}) && $test{$element->{name}}{resp_t_w} == 0){
      #$data_performance = cmd("$api_version/volume?action=monitor&names=" . $element->{name} . "&historical=1h&mirrored=true", "GET", 1);
    $data_performance = $hosts{$element->{name}}{performance2};
    #}
    if (ref($data_performance) eq "ARRAY"){
      #for (my $i = 1; $i < 13;$i++){
      #  push(@{$test{$element->{name}}{performance2}}, pop(@{$data_performance}));
      #}
      $test{$element->{name}}{performance2} = $data_performance;
    }
    if (ref($test{$element->{name}}{performance2}) eq "ARRAY"){
      my ($write_io, $numerator_write, $denominator) = sum_metric("mirrored_writes_per_sec", $test{$element->{name}}{performance2}, "usec_per_mirrored_write_op");
      $test{$element->{name}}{mirrored_write_io} = $write_io;
      if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
        if ($write_io == 0){
          $test{$element->{name}}{mirrored_resp_t_w} = 0;
        }
        else{
          $test{$element->{name}}{mirrored_resp_t_w} = $numerator_write / ($denominator * 1000);
        }
      }
      ($test{$element->{name}}{mirrored_write}, undef) = sum_metric("mirrored_input_per_sec",$test{$element->{name}}{performance2});
      delete $test{$element->{name}}{performance2};
    }
    #$test{$element->{name}}{resp_t} = calculate_weighted_average($test{$element->{name}}, "resp_t_r", "read_io","resp_t_w", "write_io");
  }
  if ($data_capacity ne ""){
    foreach my $element (@{$data_capacity}){
      if ( !defined $element->{name} || $element->{name} eq "" ){next;}
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{size})){
        $test{$element->{name}}{capacity_total} = $element->{size};
      }
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{total})){
        $test{$element->{name}}{capacity_used} = $element->{total};
        if (Xormon::isdigit($test{$element->{name}}{capacity_total})){
          $test{$element->{name}}{capacity_free} = $test{$element->{name}}{capacity_total} - $test{$element->{name}}{capacity_used};
        }
      }
    }
  }
  foreach my $element (keys %test){
    $test{$element}{write_io} = sum_metrics($test{$element},"local_write_io", "mirrored_write_io");
    $test{$element}{io_rate} =  sum_metrics($test{$element},"write_io", "read_io");
    $test{$element}{write} = sum_metrics($test{$element},"local_write", "mirrored_write");
    $test{$element}{data_rate} =  sum_metrics($test{$element},"write", "read");
    $test{$element}{resp_t_w} = calculate_weighted_average($test{$element}, "local_resp_t_w", "local_write_io","mirrored_resp_t_w", "mirrored_write_io");
    $test{$element}{resp_t} = calculate_weighted_average($test{$element}, "resp_t_r", "read_io","resp_t_w", "write_io");
  }
  convert_data($section,\%test, "perf");
}

sub set_data_host_group {
  my $i = shift;
  my $section     = "host_group";
  #my $data_capacity = "";
  my $data_performance = cmd("$api_version/hgroup?action=monitor", "GET", 1, "hgroup_performance$i");
  if (ref($data_performance) eq "ARRAY"){
    foreach my $element (@{$data_performance}){
      if (!defined $element->{name}){next;}
      push(@{$hosts{$element->{name}}{performance}},$element);
    }
  }
  undef $data_performance;
  $data_performance = cmd("$api_version/hgroup?action=monitor&mirrored=true", "GET", 1, "hgroup_performance_mirror$i");
  if (ref($data_performance) eq "ARRAY"){
    foreach my $element (@{$data_performance}){
      if (!defined $element->{name}){next;}
      push(@{$hosts{$element->{name}}{performance2}},$element);
    }
  }
  #print Dumper \%hosts;
  if ($i < 10){return;}

  my $data_config = cmd("$api_version/hgroup", "GET", 0, $section);

  #print Dumper \$data_config;
  my $data_capacity = cmd("$api_version/hgroup?space=true", "GET", 0, "host_group_capacity");
  #my $data_performance = cmd("$api_version/volume?action=monitor&names=rco-vg01/vol01&historical=1h", "GET", 1);
  my %test;
  if ($data_config eq ""){return;}
  foreach my $element (@{$data_config}){
    if (!defined $element->{name} || $element->{name} eq ""){next;}
    $test{$element->{name}}{id} = $element->{name};
    #if (Xormon::isdigit($element->{size})){
    #  $test{$element->{name}}{capacity_total} = $element->{size};
    #}
    #my $data_performance = cmd("$api_version/host?action=monitor&names=" . $element->{name} . "", "GET", 1);
    #my $data_performance = cmd("$api_version/host?action=monitor&historical=1h", "GET", 1);
    #print Dumper \$data_performance;
    if (!defined $hosts{$element->{name}}{performance}){next;}
    my $data_performance = $hosts{$element->{name}}{performance};
    if (ref($data_performance) eq "ARRAY"){
      $test{$element->{name}}{performance} = $data_performance;
      $test{$element->{name}}{time} = $test{$element->{name}}{performance}[-1]{time};
      #for (my $i = 1; $i < 13;$i++){
      #  push(@{$test{$element->{name}}{performance}}, pop(@{$data_performance}));
      #  if ($i == 1){
      #    $test{$element->{name}}{time} = $test{$element->{name}}{performance}[0]{time};
      #  }
      #}
    }
    if (ref($test{$element->{name}}{performance}) eq "ARRAY"){
      my ($read_io, $numerator_read, $denominator) = sum_metric("reads_per_sec", $test{$element->{name}}{performance}, "usec_per_read_op");
      $test{$element->{name}}{read_io} = $read_io;
      if (Xormon::isdigit($numerator_read) && Xormon::isdigit($read_io)){
        if ($read_io == 0){
          $test{$element->{name}}{resp_t_r} = 0;
        }
        else{
          $test{$element->{name}}{resp_t_r} = $numerator_read / ($denominator * 1000);
        }
      }
      my ($write_io, $numerator_write, $denominator2) = sum_metric("writes_per_sec", $test{$element->{name}}{performance}, "usec_per_write_op");
      $test{$element->{name}}{local_write_io} = $write_io;
      if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
        if ($write_io == 0){
          $test{$element->{name}}{local_resp_t_w} = 0;
        }
        else{
          $test{$element->{name}}{local_resp_t_w} = $numerator_write / ($denominator2 * 1000);
        }
      }
      ($test{$element->{name}}{read}, undef) = sum_metric("output_per_sec",$test{$element->{name}}{performance});
      ($test{$element->{name}}{local_write}, undef) = sum_metric("input_per_sec",$test{$element->{name}}{performance});
      delete $test{$element->{name}}{performance};
      #$test{$element->{name}}{io_rate} = sum_metrics($test{$element->{name}},"read_io", "write_io");
      #$test{$element->{name}}{data_rate} = sum_metrics($test{$element->{name}},"read", "write");
      #$test{$element->{name}}{data_rate} = sum_metrics($test{$element->{name}},"read", "write");
    }

    $data_performance = undef;
    #if (Xormon::isdigit($test{$element->{name}}{write_io}) && $test{$element->{name}}{write_io} == 0 && Xormon::isdigit($test{$element->{name}}{write}) && $test{$element->{name}}{write} == 0 && Xormon::isdigit($test{$element->{name}}{resp_t_w}) && $test{$element->{name}}{resp_t_w} == 0){
      #$data_performance = cmd("$api_version/volume?action=monitor&names=" . $element->{name} . "&historical=1h&mirrored=true", "GET", 1);
    $data_performance = $hosts{$element->{name}}{performance2};

    #}
    if (ref($data_performance) eq "ARRAY"){
      #for (my $i = 1; $i < 13;$i++){
      #  push(@{$test{$element->{name}}{performance2}}, pop(@{$data_performance}));
      #}
      $test{$element->{name}}{performance2} = $data_performance;
    }
    if (ref($test{$element->{name}}{performance2}) eq "ARRAY"){
      my ($write_io, $numerator_write, $denominator) = sum_metric("mirrored_writes_per_sec", $test{$element->{name}}{performance2}, "usec_per_mirrored_write_op");
      $test{$element->{name}}{mirrored_write_io} = $write_io;
      if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
        if ($write_io == 0){
          $test{$element->{name}}{mirrored_resp_t_w} = 0;
        }
        else{
          $test{$element->{name}}{mirrored_resp_t_w} = $numerator_write / ($denominator * 1000);
        }
      }
      ($test{$element->{name}}{mirrored_write}, undef) = sum_metric("mirrored_input_per_sec",$test{$element->{name}}{performance2});
      delete $test{$element->{name}}{performance2};
    }
    #$test{$element->{name}}{resp_t} = calculate_weighted_average($test{$element->{name}}, "resp_t_r", "read_io","resp_t_w", "write_io");
  }
  if ($data_capacity ne ""){
    foreach my $element (@{$data_capacity}){
      if ( !defined $element->{name} || $element->{name} eq "" ){next;}
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{size})){
        $test{$element->{name}}{capacity_total} = $element->{size};
      }
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{total})){
        $test{$element->{name}}{capacity_used} = $element->{total};
        if (Xormon::isdigit($test{$element->{name}}{capacity_total})){
          $test{$element->{name}}{capacity_free} = $test{$element->{name}}{capacity_total} - $test{$element->{name}}{capacity_used};
        }
      }
    }
  }
  foreach my $element (keys %test){
    $test{$element}{write_io} = sum_metrics($test{$element},"local_write_io", "mirrored_write_io");
    $test{$element}{io_rate} =  sum_metrics($test{$element},"write_io", "read_io");
    $test{$element}{write} = sum_metrics($test{$element},"local_write", "mirrored_write");
    $test{$element}{data_rate} =  sum_metrics($test{$element},"write", "read");
    $test{$element}{resp_t_w} = calculate_weighted_average($test{$element}, "local_resp_t_w", "local_write_io","mirrored_resp_t_w", "mirrored_write_io");
    $test{$element}{resp_t} = calculate_weighted_average($test{$element}, "resp_t_r", "read_io","resp_t_w", "write_io");
  }
  convert_data($section,\%test, "perf");
}

sub set_data_host_group_conf {
  my $section     = "host_group";
  #my $data_capacity = "";
  my $data_config = cmd("$api_version/hgroup", "GET", 0, $section);
  #print Dumper \$data_config;
  my $data_capacity = cmd("$api_version/hgroup?space=true", "GET", 0, "host_group_capacity");
  my %test;
  if ($data_config eq ""){return;}
  foreach my $element (@{$data_config}){
    if (!defined $element->{name} || $element->{name} eq ""){next;}
    $test{$element->{name}}{id} = $element->{name};
    my $hosts = $element->{hosts};
    foreach my $host (@{$hosts}){
      #my $uid = Xormon::create_hash_key($st_serial."host".$host);
      #push(@{$test{$element->{name}}{children}},$uid); host-group -> host mapping in set_data_host_conf
      if (defined $test{$element->{name}}{hosts}){
        $test{$element->{name}}{hosts} = $test{$element->{name}}{hosts} . " " . $host;
      }
      else{
        $test{$element->{name}}{hosts} = $host;
      }
    }
  }
  if ($data_capacity ne ""){
    foreach my $element (@{$data_capacity}){
      if ( !defined $element->{name} || $element->{name} eq "" ){next;}
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{size})){
        $test{$element->{name}}{capacity_total} = $element->{size};
      }
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{total})){
        $test{$element->{name}}{capacity_used} = $element->{total};
        if (Xormon::isdigit($test{$element->{name}}{capacity_total})){
          $test{$element->{name}}{capacity_free} = $test{$element->{name}}{capacity_total} - $test{$element->{name}}{capacity_used};
        }
      }
    }
  }
  convert_data($section,\%test, "conf");
}

sub set_data_volume {
  my $section     = "volume";
  my $data_config = cmd("$api_version/volume", "GET", 0, $section);
  my $data_capacity = cmd("$api_version/volume?space=true", "GET", 0, "volume_capacity");
  #print Dumper \$data_config;
  #print Dumper $data_capacity;
  #my $data_performance = cmd("$api_version/volume?action=monitor&names=rco-vg01/vol01&historical=1h", "GET", 1);
  my %test;
  if ($data_config eq ""){return;}
  foreach my $element (@{$data_config}){
    if (!defined $element->{serial} || $element->{serial} eq "" || !defined $element->{name} || $element->{name} eq ""){next;}
    $test{$element->{name}}{id} = $element->{serial};
    if (Xormon::isdigit($element->{size})){
      $test{$element->{name}}{capacity_total} = $element->{size};
    }
    my $data_performance = cmd("$api_version/volume?action=monitor&names=" . $element->{name} . "&historical=1h", "GET", 1, $element->{name});
    #print Dumper \$data_performance;
    if (ref($data_performance) eq "ARRAY"){
      for (my $i = 1; $i < 11;$i++){
        push(@{$test{$element->{name}}{performance}}, pop(@{$data_performance}));
        if ($i == 1){
          $test{$element->{name}}{time} = $test{$element->{name}}{performance}[0]{time};
        }
      }
    }
    if (ref($test{$element->{name}}{performance}) eq "ARRAY"){
      my ($read_io, $numerator_read, $denominator) = sum_metric("reads_per_sec", $test{$element->{name}}{performance}, "usec_per_read_op");
      $test{$element->{name}}{read_io} = $read_io;
      if (Xormon::isdigit($numerator_read) && Xormon::isdigit($read_io)){
        if ($read_io == 0){
          $test{$element->{name}}{resp_t_r} = 0;
        }
        else{
          $test{$element->{name}}{resp_t_r} = $numerator_read / ($denominator * 1000);
        }
      }
      my ($write_io, $numerator_write, $denominator2) = sum_metric("writes_per_sec", $test{$element->{name}}{performance}, "usec_per_write_op");
      $test{$element->{name}}{local_write_io} = $write_io;
      if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
        if ($write_io == 0){
          $test{$element->{name}}{local_resp_t_w} = 0;
        }
        else{
          $test{$element->{name}}{local_resp_t_w} = $numerator_write / ($denominator2 * 1000);
        }
      }
      ($test{$element->{name}}{read}, undef) = sum_metric("output_per_sec",$test{$element->{name}}{performance});
      ($test{$element->{name}}{local_write}, undef) = sum_metric("input_per_sec",$test{$element->{name}}{performance});
      delete $test{$element->{name}}{performance};
      #$test{$element->{name}}{io_rate} = sum_metrics($test{$element->{name}},"read_io", "write_io");
      #$test{$element->{name}}{data_rate} = sum_metrics($test{$element->{name}},"read", "write");
      #$test{$element->{name}}{data_rate} = sum_metrics($test{$element->{name}},"read", "write");
    }

    $data_performance = undef;
    #if (Xormon::isdigit($test{$element->{name}}{write_io}) && $test{$element->{name}}{write_io} == 0 && Xormon::isdigit($test{$element->{name}}{write}) && $test{$element->{name}}{write} == 0 && Xormon::isdigit($test{$element->{name}}{resp_t_w}) && $test{$element->{name}}{resp_t_w} == 0){
    my $mirror_file = $element->{name} . "_mirror";
    $data_performance = cmd("$api_version/volume?action=monitor&names=" . $element->{name} . "&historical=1h&mirrored=true", "GET", 1, $mirror_file);
    #}
    if (ref($data_performance) eq "ARRAY"){
      for (my $i = 1; $i < 11;$i++){
        push(@{$test{$element->{name}}{performance2}}, pop(@{$data_performance}));
      }
    }
    if (ref($test{$element->{name}}{performance2}) eq "ARRAY"){
      my ($write_io, $numerator_write, $denominator) = sum_metric("mirrored_writes_per_sec", $test{$element->{name}}{performance2}, "usec_per_mirrored_write_op");
      $test{$element->{name}}{mirrored_write_io} = $write_io;
      if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
        if ($write_io == 0){
          $test{$element->{name}}{mirrored_resp_t_w} = 0;
        }
        else{
          $test{$element->{name}}{mirrored_resp_t_w} = $numerator_write / ($denominator * 1000);
        }
      }
      ($test{$element->{name}}{mirrored_write}, undef) = sum_metric("mirrored_input_per_sec",$test{$element->{name}}{performance2});
      delete $test{$element->{name}}{performance2};
    }
    #$test{$element->{name}}{resp_t} = calculate_weighted_average($test{$element->{name}}, "resp_t_r", "read_io","resp_t_w", "write_io");
  }
  if ($data_capacity ne ""){
    foreach my $element (@{$data_capacity}){
      if ( !defined $element->{name} || $element->{name} eq "" ){next;}
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{total})){
        $test{$element->{name}}{capacity_used} = $element->{total};
        if (Xormon::isdigit($test{$element->{name}}{capacity_total})){
          $test{$element->{name}}{capacity_free} = $test{$element->{name}}{capacity_total} - $test{$element->{name}}{capacity_used};
        }
      }
      if (Xormon::isdigit($element->{system})){
        $test{$element->{name}}{system_capacity_used} = $element->{system};
      }
      if (Xormon::isdigit($element->{shared_space})){
        $test{$element->{name}}{shared_space_capacity_used} = $element->{shared_space};
      }
      if (Xormon::isdigit($element->{volumes})){
        $test{$element->{name}}{volumes_capacity_used} = $element->{volumes};
      }
      #if (Xormon::isdigit($element->{data_reduction})){
      #  $test{$element->{name}}{data_reduction_ratio} = $element->{data_reduction};
      #}
      #if (Xormon::isdigit($element->{total_reduction})){
      #  $test{$element->{name}}{total_reduction} = $element->{total_reduction};
      #}
      if (Xormon::isdigit($element->{thin_provisioning}) && Xormon::isdigit($test{$element->{name}}{capacity_total})){
        $test{$element->{name}}{virtual_capacity_total} = $element->{thin_provisioning} * $test{$element->{name}}{capacity_total};
      }
      if (Xormon::isdigit($element->{"snapshots"})){
        $test{$element->{name}}{snapshot_capacity_used} = $element->{"snapshots"};
      }
    }
  }
  foreach my $element (keys %test){
    $test{$element}{write_io} = sum_metrics($test{$element},"local_write_io", "mirrored_write_io");
    $test{$element}{io_rate} =  sum_metrics($test{$element},"write_io", "read_io");
    $test{$element}{write} = sum_metrics($test{$element},"local_write", "mirrored_write");
    $test{$element}{data_rate} =  sum_metrics($test{$element},"write", "read");
    $test{$element}{resp_t_w} = calculate_weighted_average($test{$element}, "local_resp_t_w", "local_write_io","mirrored_resp_t_w", "mirrored_write_io");
    $test{$element}{resp_t} = calculate_weighted_average($test{$element}, "resp_t_r", "read_io","resp_t_w", "write_io");
  }
  #print Dumper \%test;
  convert_data($section,\%test, "perf");
}
sub set_data_pod {
  my $section     = "pod";
  my $data_config = cmd("$api_version/pod", "GET", 0, $section);
  my $data_capacity = cmd("$api_version/pod?space=true", "GET", 0, "pod_capacity");
  #my $data_performance = cmd("$api_version/volume?action=monitor&names=rco-vg01/vol01&historical=1h", "GET", 1);
  my %test;
  if ($data_config eq ""){return;}
  #print Dumper \$data_config;
  #print Dumper \$data_capacity;
  #return;
  foreach my $element (@{$data_config}){
    if (!defined $element->{name} || $element->{name} eq ""){next;}
    $test{$element->{name}}{id} = $element->{name};
    my $data_performance = cmd("$api_version/pod?action=monitor&names=" . $element->{name} . "&historical=1h", "GET", 1, $element->{name});
    #print Dumper \$data_performance;
    #return;
    if (ref($data_performance) eq "ARRAY"){
      for (my $i = 1; $i < 11;$i++){
        push(@{$test{$element->{name}}{performance}}, pop(@{$data_performance}));
        if ($i == 1){
          $test{$element->{name}}{time} = $test{$element->{name}}{performance}[0]{time};
        }
      }
    }
    if (ref($test{$element->{name}}{performance}) eq "ARRAY"){
      my ($read_io, $numerator_read, $denominator) = sum_metric("reads_per_sec", $test{$element->{name}}{performance}, "usec_per_read_op");
      $test{$element->{name}}{read_io} = $read_io;
      if (Xormon::isdigit($numerator_read) && Xormon::isdigit($read_io)){
        if ($read_io == 0){
          $test{$element->{name}}{resp_t_r} = 0;
        }
        else{
          $test{$element->{name}}{resp_t_r} = $numerator_read / ($denominator * 1000);
        }
      }
      my ($write_io, $numerator_write, $denominator2) = sum_metric("writes_per_sec", $test{$element->{name}}{performance}, "usec_per_write_op");
      $test{$element->{name}}{local_write_io} = $write_io;
      if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
        if ($write_io == 0){
          $test{$element->{name}}{local_resp_t_w} = 0;
        }
        else{
          $test{$element->{name}}{local_resp_t_w} = $numerator_write / ($denominator2 * 1000);
        }
      }
      ($test{$element->{name}}{read}, undef) = sum_metric("output_per_sec",$test{$element->{name}}{performance});
      ($test{$element->{name}}{local_write}, undef) = sum_metric("input_per_sec",$test{$element->{name}}{performance});
      delete $test{$element->{name}}{performance};
      #$test{$element->{name}}{io_rate} = sum_metrics($test{$element->{name}},"read_io", "write_io");
      #$test{$element->{name}}{data_rate} = sum_metrics($test{$element->{name}},"read", "write");
      #$test{$element->{name}}{data_rate} = sum_metrics($test{$element->{name}},"read", "write");
    }

    $data_performance = undef;
    #if (Xormon::isdigit($test{$element->{name}}{write_io}) && $test{$element->{name}}{write_io} == 0 && Xormon::isdigit($test{$element->{name}}{write}) && $test{$element->{name}}{write} == 0 && Xormon::isdigit($test{$element->{name}}{resp_t_w}) && $test{$element->{name}}{resp_t_w} == 0){
    my $mirror_file = $element->{name} . "_mirror";
    $data_performance = cmd("$api_version/pod?action=monitor&names=" . $element->{name} . "&historical=1h&mirrored=true", "GET", 1, $element->{name});
    #}
    if (ref($data_performance) eq "ARRAY"){
      for (my $i = 1; $i < 11;$i++){
        push(@{$test{$element->{name}}{performance2}}, pop(@{$data_performance}));
      }
    }
    if (ref($test{$element->{name}}{performance2}) eq "ARRAY"){
      my ($write_io, $numerator_write, $denominator) = sum_metric("mirrored_writes_per_sec", $test{$element->{name}}{performance2}, "usec_per_mirrored_write_op");
      $test{$element->{name}}{mirrored_write_io} = $write_io;
      if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
        if ($write_io == 0){
          $test{$element->{name}}{mirrored_resp_t_w} = 0;
        }
        else{
          $test{$element->{name}}{mirrored_resp_t_w} = $numerator_write / ($denominator * 1000);
        }
      }
      ($test{$element->{name}}{mirrored_write}, undef) = sum_metric("mirrored_input_per_sec",$test{$element->{name}}{performance2});
      delete $test{$element->{name}}{performance2};
    }
    #$test{$element->{name}}{resp_t} = calculate_weighted_average($test{$element->{name}}, "resp_t_r", "read_io","resp_t_w", "write_io");
  }
  if ($data_capacity ne ""){
    foreach my $element (@{$data_capacity}){
      if ( !defined $element->{name} || $element->{name} eq "" ){next;}
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{size})){
        $test{$element->{name}}{capacity_total} = $element->{size};
      }
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{total})){
        $test{$element->{name}}{capacity_used} = $element->{total};
        if (Xormon::isdigit($test{$element->{name}}{capacity_total})){
          $test{$element->{name}}{capacity_free} = $test{$element->{name}}{capacity_total} - $test{$element->{name}}{capacity_used};
        }
      }
      if (Xormon::isdigit($element->{replication})){
        $test{$element->{name}}{replication_capacity_used} = $element->{replication};
        if (Xormon::isdigit($replication_capacity_used)){
          $replication_capacity_used = $replication_capacity_used + $element->{replication};
        }
        else{
          $replication_capacity_used = $element->{replication};
        }
      }
      if (Xormon::isdigit($element->{shared})){
        $test{$element->{name}}{shared_space_capacity_used} = $element->{shared};
      }
      if (Xormon::isdigit($element->{volumes})){
        $test{$element->{name}}{volumes_capacity_used} = $element->{volumes};
      }
      #if (Xormon::isdigit($element->{data_reduction})){
      #  $test{$element->{name}}{data_reduction_ratio} = $element->{data_reduction};
      #}
      #if (Xormon::isdigit($element->{total_reduction})){
      #  $test{$element->{name}}{total_reduction} = $element->{total_reduction};
      #}
      if (Xormon::isdigit($element->{thin_provisioning}) && Xormon::isdigit($test{$element->{name}}{capacity_total})){
        $test{$element->{name}}{virtual_capacity_total} = $element->{thin_provisioning} * $test{$element->{name}}{capacity_total};
      }
      if (Xormon::isdigit($element->{"snapshots"})){
        $test{$element->{name}}{snapshot_capacity_used} = $element->{"snapshots"};
      }
    }
  }
  foreach my $element (keys %test){
    $test{$element}{write_io} = sum_metrics($test{$element},"local_write_io", "mirrored_write_io");
    $test{$element}{io_rate} =  sum_metrics($test{$element},"write_io", "read_io");
    $test{$element}{write} = sum_metrics($test{$element},"local_write", "mirrored_write");
    $test{$element}{data_rate} =  sum_metrics($test{$element},"write", "read");
    $test{$element}{resp_t_w} = calculate_weighted_average($test{$element}, "local_resp_t_w", "local_write_io","mirrored_resp_t_w", "mirrored_write_io");
    $test{$element}{resp_t} = calculate_weighted_average($test{$element}, "resp_t_r", "read_io","resp_t_w", "write_io");
  }
  #print Dumper \%test;
  convert_data($section,\%test, "perf");
}
sub set_data_pod_conf {
  $replication_capacity_used = "";
  my %test;
  my $section     = "pod";
  my $data_config = cmd("$api_version/pod", "GET", 1, $section);
  my $data_capacity = cmd("$api_version/pod?space=true", "GET", 1, "pod_capacity");
  if ($data_config eq ""){return;}
  #print Dumper \$data_config;
  #print Dumper \$data_capacity;
  #return;
  foreach my $element (@{$data_config}){
    if (!defined $element->{name} || $element->{name} eq ""){next;}
    $test{$element->{name}}{id} = $element->{name};
    $test{$element->{name}}{label} = $element->{name};

  }
  if ($data_capacity ne ""){
    foreach my $element (@{$data_capacity}){
      if ( !defined $element->{name} || $element->{name} eq "" ){next;}
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{size})){
        $test{$element->{name}}{capacity_total} = $element->{size};
      }
      if (defined $test{$element->{name}} && Xormon::isdigit($element->{total})){
        $test{$element->{name}}{capacity_used} = $element->{total};
        if (Xormon::isdigit($test{$element->{name}}{capacity_total})){
          $test{$element->{name}}{capacity_free} = $test{$element->{name}}{capacity_total} - $test{$element->{name}}{capacity_used};
        }
      }
      if (Xormon::isdigit($element->{replication})){
        $test{$element->{name}}{replication_capacity_used} = $element->{replication};
        if (Xormon::isdigit($replication_capacity_used)){
          $replication_capacity_used = $replication_capacity_used + $element->{replication};
        }
        else{
          $replication_capacity_used = $element->{replication};
        }
      }
      if (Xormon::isdigit($element->{shared})){
        $test{$element->{name}}{shared_space_capacity_used} = $element->{shared};
      }
      if (Xormon::isdigit($element->{volumes})){
        $test{$element->{name}}{volumes_capacity_used} = $element->{volumes};
      }
      #if (Xormon::isdigit($element->{data_reduction})){
      #  $test{$element->{name}}{data_reduction_ratio} = $element->{data_reduction};
      #}
      #if (Xormon::isdigit($element->{total_reduction})){
      #  $test{$element->{name}}{total_reduction} = $element->{total_reduction};
      #}
      if (Xormon::isdigit($element->{thin_provisioning}) && Xormon::isdigit($test{$element->{name}}{capacity_total})){
        $test{$element->{name}}{virtual_capacity_total} = $element->{thin_provisioning} * $test{$element->{name}}{capacity_total};
      }
      if (Xormon::isdigit($element->{"snapshots"})){
        $test{$element->{name}}{snapshot_capacity_used} = $element->{"snapshots"};
      }
    }
  }
  #print Dumper \%test;
  convert_data($section,\%test, "conf");
}

sub set_data_pool {
  my $section     = "pool";
  my $data_config = cmd("$api_version/array", "GET", 0, $section);
  my $data_capacity = cmd("$api_version/array?space=true", "GET", 0, "pool_capacity");
  my $data_performance = cmd("$api_version/array?action=monitor&historical=1h", "GET", 1, "performance_array");
  #print Dumper \$data_capacity;
  #return;
  #my $data_performance = cmd("$api_version/volume?action=monitor&names=rco-vg01/vol01&historical=1h", "GET", 1);
  my %test;
  if (!defined $data_config || $data_config eq "" || !defined $data_config->{id} || !defined $data_config->{array_name}){return;}
  my $id = $data_config->{id};
  my $name = $data_config->{array_name};

  $test{$id}{id} = $id;
  $test{$id}{name} = $name;
  if (ref($data_performance) eq "ARRAY"){
    for (my $i = 1; $i < 11;$i++){
      push(@{$test{$id}{performance}}, pop(@{$data_performance}));
      if ($i == 1){
        $test{$id}{time} = $test{$id}{performance}[0]{time};
      }
    }
  }
  if (ref($test{$id}{performance}) eq "ARRAY"){
    my ($read_io, $numerator_read, $denominator) = sum_metric("reads_per_sec", $test{$id}{performance}, "usec_per_read_op");
    $test{$id}{read_io} = $read_io;
    if (Xormon::isdigit($numerator_read) && Xormon::isdigit($read_io)){
      if ($read_io == 0){
        $test{$id}{resp_t_r} = 0;
      }
      else{
        $test{$id}{resp_t_r} = $numerator_read / ($denominator * 1000);
      }
    }
    my ($write_io, $numerator_write, $denominator2) = sum_metric("writes_per_sec", $test{$id}{performance}, "usec_per_write_op");
    $test{$id}{local_write_io} = $write_io;
    if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
      if ($write_io == 0){
        $test{$id}{local_resp_t_w} = 0;
      }
      else{
        $test{$id}{local_resp_t_w} = $numerator_write / ($denominator2 * 1000);
      }
    }
    ($test{$id}{read}, undef) = sum_metric("output_per_sec",$test{$id}{performance});
    ($test{$id}{local_write}, undef) = sum_metric("input_per_sec",$test{$id}{performance});
    delete $test{$id}{performance};
    #$test{$id}{io_rate} = sum_metrics($test{$id},"read_io", "write_io");
    #$test{$id}{data_rate} = sum_metrics($test{$id},"read", "write");
    #$test{$id}{data_rate} = sum_metrics($test{$id},"read", "write");
  }
  $data_performance = undef;
  #if (Xormon::isdigit($test{$id}{write_io}) && $test{$id}{write_io} == 0 && Xormon::isdigit($test{$id}{write}) && $test{$id}{write} == 0 && Xormon::isdigit($test{$id}{resp_t_w}) && $test{$id}{resp_t_w} == 0){
    $data_performance = cmd("$api_version/array?action=monitor&historical=1h&mirrored=true", "GET", 1, "array_performance_mirror");
  #}
  if (ref($data_performance) eq "ARRAY"){
    for (my $i = 1; $i < 11;$i++){
      push(@{$test{$id}{performance2}}, pop(@{$data_performance}));
    }
  }
  if (ref($test{$id}{performance2}) eq "ARRAY"){
    my ($write_io, $numerator_write, $denominator) = sum_metric("mirrored_writes_per_sec", $test{$id}{performance2}, "usec_per_mirrored_write_op");
    $test{$id}{mirrored_write_io} = $write_io;
    if (Xormon::isdigit($numerator_write) && Xormon::isdigit($write_io)){
      if ($write_io == 0){
        $test{$id}{mirrored_resp_t_w} = 0;
      }
      else{
        $test{$id}{mirrored_resp_t_w} = $numerator_write / ($denominator * 1000);
      }
    }
    ($test{$id}{mirrored_write}, undef) = sum_metric("mirrored_input_per_sec",$test{$id}{performance2});
    delete $test{$id}{performance2};
  }
  #$test{$id}{resp_t} = calculate_weighted_average($test{$id}, "resp_t_r", "read_io","resp_t_w", "write_io");
  #print Dumper \$data_capacity;
  if ($data_capacity ne "" && ref($data_capacity) eq "HASH"){
    my %capacity = %{$data_capacity};
    $data_capacity = undef;
    push(@{$data_capacity},\%capacity);
  }
  if ($data_capacity ne "" && ref($data_capacity) eq "ARRAY"){
    foreach my $element (@{$data_capacity}){
      if (!defined $element->{hostname} || $element->{hostname} ne $name){next;}
      if (defined $test{$id} && Xormon::isdigit($element->{total})){
        $test{$id}{capacity_used} = $element->{total};
      }
      if (defined $test{$id} && Xormon::isdigit($element->{capacity})){
        $test{$id}{capacity_total} = $element->{capacity};
      }
      if (defined $test{$id} && Xormon::isdigit($element->{system})){
        $test{$id}{system_capacity_used} = $element->{system};
      }
      if (defined $test{$id} && Xormon::isdigit($element->{shared_space})){
        $test{$id}{shared_space_capacity_used} = $element->{shared_space};
      }
      if (defined $test{$id} && Xormon::isdigit($element->{volumes})){
        $test{$id}{volumes_capacity_used} = $element->{volumes};
      }
      #if (defined $test{$id} && Xormon::isdigit($element->{data_reduction})){
      #  $test{$id}{data_reduction_ratio} = $element->{data_reduction};
      #}
      #if (defined $test{$id} && Xormon::isdigit($element->{total_reduction})){
      #  $test{$id}{total_reduction} = $element->{total_reduction};
      #}
      if (defined $test{$id} && Xormon::isdigit($element->{provisioned})){
        $test{$id}{virtual_capacity_total} = $element->{provisioned};
        if (Xormon::isdigit($test{$id}{capacity_total})){
          $test{$id}{"capacity_oversubscr"} = $test{$id}{"virtual_capacity_total"} - $test{$id}{"capacity_total"};
          if ($test{$id}{"capacity_oversubscr"} < 0){$test{$id}{"capacity_oversubscr"} = 0; $test{$id}{"capacity_oversubscr_percent"} = 0;}
          elsif ($test{$id}{"capacity_total"} != 0){
            $test{$id}{"capacity_oversubscr_percent"} = round(($test{$id}{"virtual_capacity_total"} / $test{$id}{"capacity_total"}) * 100,0);
          }
        }
      }
      if (defined $test{$id} && Xormon::isdigit($element->{"snapshots"})){
        $test{$id}{snapshot_capacity_used} = $element->{"snapshots"};
      }
      if (defined $test{$id} && Xormon::isdigit($replication_capacity_used)){
        $test{$id}{replication_capacity_used} = $replication_capacity_used;
      }
    }
  }
  if (Xormon::isdigit($test{$id}{capacity_total}) && Xormon::isdigit($test{$id}{capacity_used})){
    $test{$id}{capacity_free} = $test{$id}{capacity_total} - $test{$id}{capacity_used};
  }
  foreach my $element (keys %test){
    $test{$element}{write_io} = sum_metrics($test{$element},"local_write_io", "mirrored_write_io");
    $test{$element}{io_rate} =  sum_metrics($test{$element},"write_io", "read_io");
    $test{$element}{write} = sum_metrics($test{$element},"local_write", "mirrored_write");
    $test{$element}{data_rate} =  sum_metrics($test{$element},"write", "read");
    $test{$element}{resp_t_w} = calculate_weighted_average($test{$element}, "local_resp_t_w", "local_write_io","mirrored_resp_t_w", "mirrored_write_io");
    $test{$element}{resp_t} = calculate_weighted_average($test{$element}, "resp_t_r", "read_io","resp_t_w", "write_io");
  }
  ##print Dumper \%test;
  convert_data($section,\%test, "perf");
}

sub convert_data{
  #$performance_data{$st_type}{$section}{$uid}{$time}{capacity_used} = $capacity_used;
  my $section = shift;
  my $data_tmp = shift;
  my $type = shift;
  my %data = %{$data_tmp};
  foreach my $name (keys %data){
    if (!defined $data{$name}{id} || $data{$name}{id} eq "" || ($type eq "perf" && (!defined $data{$name}{time} || $data{$name}{time} eq "" ))){next;}
    my $uid = $data{$name}{id};
    my $time = "";
    if ($section eq "host" || $section eq "host_group" || $section eq "pool" || $section eq "pod"){
      $uid = Xormon::create_hash_key($st_serial.$section.$uid);
    }
    #if ($section eq "pool"){
    #  if (!defined $data{$name}{name} || $data{$name}{name} eq ""){next;}
      #print "$uid " . Xormon::create_hash_key($st_serial.$section.$uid) . "\n";
    #  $uid = Xormon::create_hash_key($st_serial.$section.$uid);
      #print "$uid\n";

    #}
    if ($type eq "perf"){
      $time = str2time($data{$name}{time});
      if (!$enviroment){
        $time = time();
      }
    }
    foreach my $metric (keys %{$data{$name}}){
      my $rounded = 0;
      if ($metric =~ m/id|time|name/ && $type eq "perf"){next;}
      if ($metric =~ m/resp/){
        $rounded = 3;
      }
      if ($metric =~ m/reduction/){
        $rounded = 1;
      }
      if ($type eq "conf"){
        if (!defined $configuration_data{$uid}{label}){
          $configuration_data{$uid}{label} = $name;
        }
        if ($section eq "pool" && defined $data{$name}{name}){
          $configuration_data{$uid}{label} = $data{$name}{name};
        }
        if (Xormon::isdigit($data{$name}{$metric})){
          $configuration_data{$uid}{$metric} = round($data{$name}{$metric},$rounded);
        }
        else{
          $configuration_data{$uid}{$metric} = $data{$name}{$metric};
        }
        $configuration_data{$uid}{TYPE} = $section;
      }
      else{
        if (!Xormon::isdigit($data{$name}{$metric})){next;}
        $performance_data{$st_type}{$section}{$uid}{$time}{$metric} = round($data{$name}{$metric},$rounded);
      }
    }
  }
}

sub delete_empty_metrics{
  my $first_time = "";
  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        if ($first_time eq "" && Xormon::isdigit($time)){
          $first_time = $time;
        }
        if (Xormon::isdigit($first_time) && Xormon::isdigit($time) && $time < $first_time){
          $first_time = $time;
        }
        foreach my $item (keys %{$performance_data{$st_type}{$type}{$uid}{$time}}){
          if (!Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item}) || $performance_data{$st_type}{$type}{$uid}{$time}{$item} < 0){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
          if ($item =~ m/numerator|denominator/){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
        }
      }
    }
  }
  if (Xormon::isdigit($first_time)){
    $service_data_out->{last_perf_timestamp} = $first_time;
  }
  else{
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      $service_data_out->{last_perf_timestamp} = $service_data_in->{last_perf_timestamp};
    }
  }
}

sub calculate_missing_metrics{

  my $item_total = "capacity_total";
  my $item_used = "capacity_used";
  my $item_used_percent = "capacity_used_percent";

  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        #capacity_used_percent
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_used})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_total} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_used} / $performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = round($pom,2);
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{data_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && $performance_data{$st_type}{$type}{$uid}{$time}{io_rate} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{data_rate} / $performance_data{$st_type}{$type}{$uid}{$time}{io_rate},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{read_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{read} / $performance_data{$st_type}{$type}{$uid}{$time}{read_io},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{write_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{write} / $performance_data{$st_type}{$type}{$uid}{$time}{write_io},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = 0;
        }
      }
    }
  }
}

sub logout{
  if (!$enviroment){return;}
  my $cmd = "$api_version/auth/session";
  cmd($cmd,"DELETE",1);
}

sub cmd{
  my $cmd = shift;
  my $type = shift;
  my $execute = shift;
  my $debug_file = shift;
  my $json_body = shift;
  my $header = shift;
  if (!defined $type || $type eq ""){$type = "GET";}
  if (!defined $execute || $execute eq ""){
    $execute = 1;
  }
  if (!defined $debug_file){$debug_file = "";}
  if (!$enviroment){
    $debug_file =~ s/\///g;
    $execute = 0;
  }
  ### config file must be configuration
  my $file = "$debug_data/$debug_file.json";
  if ($cmd !~ /action/){
    $file = "$debug_data_configuration/$debug_file.json";
  }
  if ($execute == 0 && -f $file){
    my $data;
    eval {$data = Xormon::read_json($file);};
    if ($@){
      $logger->error("Read json data failed");
      unlink($file);
    }
    else{
      if (defined $data && ref($data) eq "HASH" && !keys %{$data} ){
        unlink($file); ### read jso data failed {}
      }
      else{
        return $data;
      }
    }
  }
  my $url = "https://$cfg->{'apiHost'}/api/$cmd";
  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( $type => $url );
  $req->header( 'Content-Type' => 'application/json' );
  $req->header( 'Accept'       => 'application/json' );
  $req->header( 'Connection'   => 'keep-alive' );
  if (defined $json_body && $json_body ne ""){
    $req->content($json_body);
  }
  if (defined $cookie && $cookie ne ""){
    $req->header( 'cookie' => $cookie );
  }
  #print Dumper \$req;
  my $res = $ua->request($req);
  #print $res->content;
  if ( ! $res->is_success ) {
    if (!$connection_test){
      if ($cmd =~ m/mirror/ ){return "";}
      $logger->error("API command $cmd failed");
    }
    else{
      push(@rows_api_test,"API command $cmd failed\n");
      push(@rows_api_test,$res->content . "\n");
      push(@rows_api_test_data,"API command $cmd failed\n");
      push(@rows_api_test_data,$res->content . "\n");
    }
    my $out = $res->content;
    $logger->error($out);
    return "";
  }
  if (defined $header){
    return $res;
  }
  my $data = decode_json( $res->content );
  if (defined $debug_file && $debug_file ne ""){
    $debug_file =~ s/\///g;
    my $file = "$debug_data/$debug_file.json";
    if ($cmd !~ /action/){
      $file = "$debug_data_configuration/$debug_file.json";
    }
    eval {Xormon::save_as_json($file,$data);};
    if ($@){
      $logger->error("Save data to file $file failed");
    }
  }
  return $data;
}

sub set_architecture{

  my %device_arch = (
    "item_id" => $cfg->{'uuid'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => $st_type,
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'uuid'}
  );
  #if (defined $cfg->{serial_number}){
  #  $device_arch{serial_number} = $cfg->{serial_number};
  #}
  #if (defined $cfg->{version}){
  #  $device_arch{version} = $cfg->{version};
  #}
  #if (defined $cfg->{model}){
  #  $device_arch{model} = $cfg->{model};
  #}
  push(@architecture, \%device_arch);
  #print Dumper \%configuration_data;
  #return;

  foreach my $uid (keys %configuration_data){
    my $name = $configuration_data{$uid}{label};
    my $type = $configuration_data{$uid}{TYPE};
    if (!defined $name || !defined $type){next;}
    my @children = ();
    my @parents = ();
    if (defined $configuration_data{$uid}{children}){
      @children = @{$configuration_data{$uid}{children}};
      ## delete children and parent from item_properties
      delete $configuration_data{$uid}{children};

    }
    if (defined $configuration_data{$uid}{parents}){
      @parents = @{$configuration_data{$uid}{parents}};
      delete $configuration_data{$uid}{parents};

    }
    #print Dumper \@children;
    #print Dumper \@parents;
    my %arch = (
      "item_id" => $uid,
      "label" => $name,
      "hw_type" => $st_type,
      "subsystem" => $type,
      "parents" => [$cfg->{'uuid'}],
      "hostcfg_id" => $cfg->{'uuid'}
      );
    if (@parents){
      push(@{$arch{parents}},@parents);
    }
    if (@children){
      push(@{$arch{children}},@children);
    }
    if ($type eq "port_type"){
      $arch{port_type} = $name;
    }
    if ($type eq "volume"){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($uid);
    }
    if ($type eq "port" && defined $configuration_data{$uid}{wwn}){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
    }
    #push(@{$arch{parents}},$cfg->{'uuid'});
    push(@architecture, \%arch);

  }
  #print Dumper \@architecture;
  #foreach my $item (keys %{$data{conf}{device}}){
  #  $configuration_data{$st_serial}{$item} = $data{conf}{device}{$item};
  #}
  #print Dumper \$configuration_data{device}{$st_serial};

  #save configuration
  my @keys = ("serial", "version", "model", "name", "apiVersion" );
  foreach my $key (@keys){
    if (defined $cfg->{$key}){
      if ($key eq "name"){
        $configuration_data{$cfg->{'uuid'}}{"node_name"} = $cfg->{$key};
      }
      else{
        $configuration_data{$cfg->{'uuid'}}{$key} = $cfg->{$key};
      }
    }
  }
  #$configuration_data{$cfg->{'uuid'}}{"label"} = $cfg->{'hostalias'};
  #event log
  $configuration_data{$cfg->{'uuid'}}{alerts} = encode_json(\@health_status);
  $configuration_data{$cfg->{'uuid'}}{hardware} = encode_json(\@hardware);
  #print Dumper \$configuration_data{$cfg->{'uuid'}}{alerts};
  #print Dumper \@architecture;
  #print Dumper \%configuration_data;
  ### device information save


  my $rc = $xormon->saveConfJSON(\%configuration_data);
  my $status_configuration = get_status($rc, "saveConfJSON (configuration)");
  #$logger->info("Task for configuration ".$cfg->{hostalias}." is done. Status: $status");

  #save architecture
  #print Dumper \@architecture;

  $rc = $xormon->saveArchitecture(\@architecture);
  my $status_architecture = get_status($rc, "saveArchitecture");
  #$logger->info("Task for saveArchitecture ".$cfg->{hostalias}." is done. Status: $status");

  $rc = $xormon->saveStatus(\@health_status_global);
  my $status_health = get_status($rc, "health status");

  return ($status_architecture, $status_configuration, $status_health);
}


sub connection_test {
  my $client = shift;
  $cfg = shift;
  $connection_test = 1;


  #print Dumper \$cfg;
  #print "aa\n";
  # test main variables
  my $test = Test->new($client);
  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");
  if (defined $cfg->{PURE_ENVIROMENT} && $cfg->{PURE_ENVIROMENT} eq "development"){
    $enviroment = 0;
  }

  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  $logger = XormonLogger->new( 'pure', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  $storage_name = $cfg->{hostalias};
  $st_serial = $cfg->{hostcfg_id};
  data_structure(1);

  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($cfg->{'apiHost'},443);
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }
  $api_version = get_version_api();
  if ($api_version eq "" ){
    push(@rows_api_test,"REST API version is not defined\n");
    $test->addSection("API authorization", 0,\@rows_api_test);
    return $test->stop();
  }
  $cookie = get_cookie();
  if ($cookie eq "" ){
    push(@rows_api_test,"Cookie is not defined");
    $test->addSection("API authorization", 0,\@rows_api_test);
    return $test->stop();
  }

  my $data_config = cmd("$api_version/array", "GET", 1, "array");
  if ($data_config eq "" || !defined $data_config->{id} || !defined $data_config->{array_name}){
    logout();
    $test->addSection("API authorization", 0, \@rows_api_test);
    return $test->stop();
  }
  my $data_ctls = cmd("$api_version/array?controllers=true", "GET", 1, "controllers");
  if ($data_ctls eq ""){
    logout();
    $test->addSection("API authorization", 0, \@rows_api_test);
    return $test->stop();
  }
  my $serial = $data_config->{id};
  my $name   = $data_config->{array_name};
  my $version = $data_config->{version};
  my $model = "";
  if ( ref($data_ctls) eq "ARRAY" ) {
    foreach my $element ( @{$data_ctls} ) {
      if ( defined $element->{"model"} ) {
        $model = $element->{"model"};
      }
    }
  }
  else {
    if ( defined $data_ctls->{model} ) {
      $model = $data_ctls->{model};
    }
  }
  if (!defined $version){$version = "";}
  push(@rows_api_test,"Name : $name\n");
  push(@rows_api_test,"Model : $model\n");
  push(@rows_api_test,"Version : $version\n");
  push(@rows_api_test,"API version : $api_version\n");
  push(@rows_api_test,"Serial : $serial\n");
  $test->addSection("API authorization", 1, \@rows_api_test);

  my $data_performance = cmd("$api_version/array?action=monitor&historical=1h", "GET", 1);
  if ($data_performance eq ""){
    logout();
    push(@rows_api_test_data, "ERROR: Array performance failed\n");
    $test->addSection("API authorization", 0, \@rows_api_test_data);
    return $test->stop();
  }
  logout();
  $test->addSection("API data test",1, \@rows_api_test_data);
  return $test->stop();
  #return Xormon::conn_test($connection_test, \@rows_api_test);
}

sub data_structure {
  my $test = shift;
  if (!-d "$data_directory/data"){
    mkdir "$data_directory/data" || $logger->error( "Cannot mkdir $data_directory/data: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial"){
    mkdir "$data_directory/data/$st_serial" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial/IOSTATS"){
    mkdir "$data_directory/data/$st_serial/IOSTATS" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial/IOSTATS: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }


  my $directory = "$data_directory/data/$st_serial/IOSTATS";
  if ( !-d $directory ) {
    mkdir $directory || $logger->error( "Cannot mkdir $directory: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || $logger->error( "Cannot mkdir $directory/configuration: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || $logger->error( "Cannot mkdir $directory/0$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || $logger->error( "Cannot mkdir $directory/$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
  my @entry_time = split( " ", localtime() );
  ( undef, $active_minute, undef ) = split( ":", $entry_time[3] );
  if (defined $test ){$active_minute = "00";}
  if ( !-d "$directory/$active_minute" ) {
    mkdir "$directory/$active_minute" || $logger->error( "Cannot mkdir $directory/$active_minute: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  #$debug_data_performance = "$directory/$active_minute";
  $debug_data_configuration = "$directory/configuration";
  $debug_data = "$directory/$active_minute";
  $wrkdir = "$data_directory/data/$st_serial";

} ## end sub data_structure

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

1;
