use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use Xormon;
use Cwd qw(cwd);
use SANnav;
use Test;
use XormonLogger;
use POSIX ":sys_wait_h";

my $logger;

# microservice's directory
my $act_dir  = cwd;
my $wrkdir   = "";
my $kafkadir = "";

#$SIG{CHLD} = 'IGNORE';
# cannot be used in this MS, bcs it produces warning "Can't ignore signal CHLD, forcing to default."
# it's probably caused by that there are running another new child process for each device

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3406;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);

my $server = IO::Socket->new(
  Domain => AF_INET,
  Type => SOCK_STREAM,
  Proto => 'tcp',
  LocalHost => '0.0.0.0',
  LocalPort => $port,
  ReusePort => 1,
  Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

#
# kill all running sannavperf.py processes
#
SANnav::terminate_old_processes();

my $client;

# waiting for a new client connection
while ($client = $server->accept()) {

  my $pid;
  unless ( defined( $pid = fork() ) ) {
    Xormon::error("Wait for fork");
    sleep 5;
  }

  if ($pid == 0) {

    # get information about a newly connected client
    my $client_address = $client->peerhost();
    my $client_port = $client->peerport();
    Xormon::log("Connection from $client_address:$client_port");

    # read up to 2048 characters from the connected client
    my $data = "";
    $client->recv($data, 2048);

    my $decoded = from_json($data);
    my $cfg     = $decoded->{data};

    # set and create directories
    set_directories($cfg);

    Xormon::log("Received task: $decoded->{task} for Device: $cfg->{hostalias}");
    my $response = '';
    if ($decoded->{task} eq "fetch") {
      #
      # data collection
      #
      $client->send("Task $decoded->{task} successfuly started");
      $client->shutdown(SHUT_WR);

      my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'sannav', $cfg->{hostcfg_id}, $cfg->{hostalias});

      # start task
      $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

      # target destinations check
      if (defined $decoded->{'destinations'}) {
        $xormon->setDestinations($decoded->{'destinations'});
      }

      #
      # start Kafka consumer
      #
      SANnav::run_kafka_consumer($cfg,$wrkdir,$kafkadir,$act_dir);

      #
      # collect conf/perf, save arch, conf, stats
      #
      my $status = SANnav::collect($xormon,$cfg,$wrkdir,$kafkadir);

      # end task
      $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);
      Xormon::log("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done. Status: $status");
    }
    elsif ( $decoded->{task} eq "test" ) {
      #
      # connection test
      #
      SANnav::connection_test($client,$cfg,$wrkdir);
      $client->shutdown(SHUT_WR);
      Xormon::log("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done.");
    }
    else {
      $response = "undefined task";

      $client->send($response);
      $client->shutdown(SHUT_WR);

    }

    exit();

  } else {
    $client->autoflush(1);
  }
waitpid( -1, WNOHANG );
}

$server->close();

exit();

sub set_directories {
  my $cfg = shift;

  #
  # mandatory cfg
  #
  unless ( exists $cfg->{'hostcfg_id'} && $cfg->{'hostcfg_id'} ne '' ) { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!") && exit 0; }
  $logger = XormonLogger->new( 'sannav', $cfg->{hostcfg_id}, $cfg->{hostalias} );

  #
  # create directories
  #
  $wrkdir   = "$act_dir/data/$cfg->{'hostcfg_id'}";
  $kafkadir = "$wrkdir/SANnav-kafka-perf";
  #my $logsdir  = "$wrkdir/logs";

  unless ( -d "$act_dir/data" ) { mkdir( "$act_dir/data", 0755 ) || $logger->error( "Cannot mkdir '$act_dir/data': $!" . __FILE__ . ":" . __LINE__ ) && exit 0; }
  unless ( -d "$wrkdir" )       { mkdir( "$wrkdir", 0755 )       || $logger->error( "Cannot mkdir '$wrkdir': $!" . __FILE__ . ":" . __LINE__ )       && exit 0; }
  unless ( -d "$kafkadir" )     { mkdir( "$kafkadir", 0755 )     || $logger->error( "Cannot mkdir '$kafkadir': $!" . __FILE__ . ":" . __LINE__ )     && exit 0; }

  return 1;
}
