use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib '../../../perl-modules';
use XormonLogger;
use SQLServer;
use Xormon;
use DBI;
use File::Temp qw(tempfile);
use File::Copy;
use Test;


$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3302;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);
my $logger;
my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

 # waiting for a new client connection
while ($client = $server->accept()) {

  my $pid;
  unless ( defined( $pid = fork() ) ) {
    Xormon::error("Wait for fork");
    sleep 5;
  }
  if ($pid == 0) {
    # get information about a newly connected client
    my $client_address = $client->peerhost();
    my $client_port = $client->peerport();

    # received data template
    #
    # {
    #   "jwt"  : "<backendToken>",
    #   "task" : "<taskName>"
    #   "data" : {
    #     <configuration_from_hostcfg>
    #   }
    # }
    # read up to 2048 characters from the connected client
    my $data = "";
    $client->recv($data, 2048);
    my $decoded = from_json($data);
    my $cfg = $decoded->{data};
    $logger = XormonLogger->new( 'sqlserver', $cfg->{hostcfg_id}, $cfg->{hostalias} );
    $logger->info("Connection from $client_address:$client_port");
    $logger->info("Received task: ".$decoded->{task});
    my $response = '';
    if ($decoded->{task} eq "fetch") {
      $client->send("Task successfuly started");
      $client->shutdown(SHUT_WR);
      my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'sqlserver', $cfg->{hostcfg_id}, $cfg->{hostalias});

      $xormon->startJob('fetch', $cfg->{hostalias}, $$);
      ############################### AGENT SECTION ###############################

      # target destinations check
      if (defined $decoded->{'destinations'}) {
        $xormon->setDestinations($decoded->{'destinations'});
      }

      # get data from technology
      my $dataToBackend = api2json($xormon, $cfg);
      #print Dumper $dataToBackend;
      # send data to backend
      my $rc = $xormon->saveJSON2($dataToBackend);
      ############################### END AGENT SECTION ###############################
      # status code
      my $status;
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }
      $xormon->endJob('fetch', $cfg->{hostalias}, $status, $$);
      $logger->info("Task [fetch] for ".$cfg->{hostalias}." is done");
    }
    elsif ($decoded->{task} eq "test") {
      apitest($client, $cfg);
      $client->shutdown(SHUT_WR);
      $logger->info("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done.");
    } else {
      $response = "undefined task";
      $client->send($response);
      $client->shutdown(SHUT_WR);
    }
    exit();
  } else {
    $client->autoflush(1);
  }
}


my %whitelisted;
my $alias;
my @pre_conf;
my $use_whitelist;
my @health_status;
my @architecture;
my %instances;


sub api2json {
  my $xormon = shift;
  my $cdata  = shift;

  my $db_name     = $cdata->{instance};
  my $port        = $cdata->{port};
  my $username    = $cdata->{username};
  my $password    = $cdata->{password};
  my $ip          = $cdata->{host};
  $alias          = $cdata->{hostalias};
  $use_whitelist  = $cdata->{use_whitelist};
  my $odbc_inst   = "/etc/odbcinst.ini";
  my $mvc_dir     = $FindBin::Bin;
  my $driver_file = "$mvc_dir/sqlserver_driver.txt";
  my %perf;
  my $additional_options = ";MultiSubnetFailover=Yes";


  if (defined $cdata->{$alias}){
    if ($cdata->{$alias} =~ /MultiSubnetFailover/){
      $additional_options  = ";$cdata->{$alias}";
    }else{
      $additional_options .= ";$cdata->{$alias}";
    }
  }



  if ($use_whitelist){
    foreach my $db (@{$cdata->{dbs}}) {
      $whitelisted{$db} = 1;
    }
  }
  my @sp = split(/\\/, $cdata->{host});
  my $connected = Test::connTest($sp[0], $cdata->{port});
  if ($connected == 1) {
    $xormon->isConnected(1);
  } else {
    $xormon->isConnected(0, "TCP connection to " . $sp[0] . ":" . $cdata->{port} . " has failed!");
    return ();
  }

  my $driver = SQLServer::get_driver();
  chomp($driver);

  if ($driver eq "err-incodriver"){
    #my $cat = `cat $driver_file`;
    warn("Incorrect driver in the file \"$driver_file\" ");#\nFILE contents: $cat");
    exit 1;
  }elsif ($driver eq "err-driverne"){
  #my $cat = `cat $odbc_inst`;
    warn("Could't find suitable driver in the file \"$odbc_inst\" \n ");#FILE contents:\n $cat");
    exit 1;
  }elsif ($driver eq "err-instne"){
    warn( "File \"$odbc_inst\" doesn't exist.");
    exit 1;
  }
  my $db_id = SQLServer::md5_string("$cdata->{hostcfg_id}-$alias-$db_name");
  my $data = get_data($driver, $ip, $db_name, $port, $username, $password,"",$db_id, $additional_options);


  if ($data ne "err"){
    my %arc;

  #  warn Dumper \%arc;
    ##Xorux_lib::write_json("$conf_dir/arc.json", \%arc);

    my $act_perf = make_perf($data,$db_name);
    #warn Dumper \%instances;
    for my $instance (keys %instances){
      next if ($instance eq $db_name);
      my $db_id = SQLServer::md5_string("$cdata->{hostcfg_id}-$alias-$instance");
      my $n_data = get_data($driver, $ip, $instance, $port, $username, $password,"MULTIDATA",$db_id,$additional_options);
      next if ($n_data eq "err");
  #    warn Dumper $n_data;
      for my $data_type (keys %{$n_data}){
        for my $db (keys %{$n_data->{$data_type}}){
          $data->{$data_type}->{$db} = $n_data->{$data_type}->{$db};
        }
      }
      #warn Dumper $data;
    }

    my $act_conf = make_conf($data,$db_name);

    $act_perf->{capacity} = $act_conf->{capacity};

    my $ret = compare_perf($alias, $act_perf);
    #print Dumper \@architecture;
    #save architecture
    #$xormon->saveArchitecture(\@architecture);


    my %cnf;
    for my $current_db (keys %{$ret}){
      next if ($current_db eq "_info");

      my $db_id = SQLServer::md5_string("$cdata->{hostcfg_id}-$alias-$current_db");
      my $cluster_id = SQLServer::md5_string("$cdata->{hostcfg_id}-$alias-SQLSFOLDER");
      if ($current_db eq "Cluster"){
        $db_id = $cluster_id;
        my %cluster_arch = (
          "item_id" => $db_id,
          "label" => $alias,
          "hw_type" => "SQLSERVER",
          "subsystem" => "SQLSFOLDER",
          "parents" => [$cluster_id],
          "hostcfg_id" => $cdata->{hostcfg_id}
        );
        push(@architecture, \%cluster_arch);

        if($act_conf->{_cluster}->{main}->{cluster}){
          $cnf{$db_id}{main} = encode_json($act_conf->{_cluster}->{main}->{cluster});
        }
        $perf{sqlserver}{sqlsfolder}{$db_id}{$ret->{_info}{timestamp}} = $ret->{$current_db};
      }else{
        if (defined $use_whitelist and $use_whitelist and !defined $whitelisted{$current_db}) {
          next;
        }
        my %arch = (
          "item_id" => $db_id,
          "label" => $current_db,
          "hw_type" => "SQLSERVER",
          "subsystem" => "DB",
          "parents" => [$cluster_id],
          "hostcfg_id" => $cdata->{hostcfg_id}
        );

        push(@architecture, \%arch);
        #warn Dumper $act_conf;
        #print "$current_db\n";
        if($act_conf->{_cluster}->{flgrps}->{$current_db}){
          $cnf{$db_id}{flgrps} = encode_json($act_conf->{_cluster}->{flgrps}->{$current_db});
        }
        if($act_conf->{_cluster}->{datafiles2}->{$current_db}){
          $cnf{$db_id}{datafiles2} = encode_json($act_conf->{_cluster}->{datafiles2}->{$current_db});
        }
        #print Dumper $act_conf->{_cluster}->{flgrps}->{$current_db};
        if ($ret->{$current_db}) {
          push(@health_status, { "item_id" => $db_id, "type" => "data", "status" => "ok", "data" => "" });
        } else {
          push(@health_status, { "item_id" => $db_id, "type" => "data", "status" => "ok", "data" => "Couldn't collect data" });
        }
        $perf{sqlserver}{db}{$db_id}{$ret->{_info}{timestamp}} = $ret->{$current_db};
      }

    }

    #warn Dumper \@architecture;
    if (defined $perf{sqlserver}{db}) {
      $xormon->hasData(1);
    } else {
      $xormon->hasData(0, "Unable to get data")
    }
    #warn Dumper \%cnf;
    my $rc = $xormon->saveStatus(\@health_status);
    $logger->info("Save Health Status for host  to DB. RC : $rc\n");
    my $arc_resp = $xormon->saveArchitecture(\@architecture);
    $logger->info("Save Architecture for host  to DB. RC : $arc_resp\n");
    my $configuration_rc = $xormon->saveConfJSON( \%cnf, 1  );
    $logger->info("Save Configuration for host  to DB. RC : $configuration_rc\n");
    #warn Dumper \%perf;
    return \%perf;
  }
}

sub apitest {
  my $client  = shift;
  my $cdata   = shift;
  my $db_name     = $cdata->{instance};
  my $port        = $cdata->{port};
  my $username    = $cdata->{username};
  my $password    = $cdata->{password};
  my $ip          = $cdata->{host};
  $alias          = $cdata->{hostalias};
  $use_whitelist  = $cdata->{use_whitelist};
  my $odbc_inst   = "/etc/odbcinst.ini";
  my $mvc_dir     = $FindBin::Bin;
  my $driver_file = "$mvc_dir/sqlserver_driver.txt";
  my $additional_options = ";MultiSubnetFailover=Yes";


  if (defined $cdata->{$alias}){
    if ($cdata->{$alias} =~ /MultiSubnetFailover/){
      $additional_options  = ";$cdata->{$alias}";
    }else{
      $additional_options .= ";$cdata->{$alias}";
    }
  }


  #my $test        = Test->new($client);
  my $test = Test->new($client);
  my @rows;
  my @sp = split(/\\/, $cdata->{host});
  my ($code, $text) = Xormon::conntest_tcp($sp[0], $port);
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  my $driver = SQLServer::get_driver();
  chomp($driver);

  if ($driver eq "err-incodriver"){
    push(@rows, "Incorrect driver in the file \"$driver_file\" ");#\nFILE contents: $cat");
    $test->addSection("Data test", 0 , \@rows);
    return $test->stop();
  }elsif ($driver eq "err-driverne"){
    push(@rows, "Could't find suitable driver in the file \"$odbc_inst\" \n Follow the installation documentation");
    $test->addSection("Data test", 0 , \@rows);
    return $test->stop();
  }elsif ($driver eq "err-instne"){
    push(@rows, "File \"$odbc_inst\" doesn't exist.");
    $test->addSection("Data test", 0 , \@rows);
    return $test->stop();
  }else{
    push(@rows, "Used driver: $driver");
    $test->addSection("Driver test", 1 , \@rows);
  }

  my $data = get_data($driver, $ip, $db_name, $port, $username, $password,undef,undef,$additional_options);
  undef @rows;
  if (defined $data and $data ne "err"){
    push(@rows, "Data ok.");
    $test->addSection("Data test", 1 , \@rows);
    return $test->stop();
  }else{
    push(@rows, "Invalid data.");
    $test->addSection("Data test", 0 , \@rows);
    return $test->stop();
  }
}


sub get_data {
  my $_driver   = shift;
  my $_ip       = shift;
  my $_database = shift;
  my $_port     = shift;
  my $_user     = shift;
  my $_password = shift;
  my $multidata = shift;
  my $db_id     = shift;
  my $additional_options = shift;

  my $dbh = DBI->connect("DBI:ODBC:driver=$_driver;Server=$_ip;database=$_database;Port=$_port;
                          ApplicationIntent=ReadOnly;Encrypt=no$additional_options",
                          $_user, $_password,{PrintError => 0, RaiseError => 0, ReadOnly => 1});

  my $time_is_now = time;
  my $err = DBI->errstr;
  #warn "ERROR $err";
  if($err and !(defined $dbh)){
    if(defined $db_id){
      push(@health_status, { "item_id" => $db_id, "status" => "error", "data" => $err });
      push(@health_status, { "item_id" => $db_id, "type" => "connection", "status" => "error", "data" => "$err" });
    }
  }else{
    if(defined $db_id){
      push(@health_status, { "item_id" => $db_id, "status" => "ok", "data" => "" });
      push(@health_status, { "item_id" => $db_id, "type" => "connection", "status" => "ok", "data" => "" });
    }
  }

  if (!(defined $dbh)){
    warn $err;
    return "err";
  }


  my %_data;
  my %queries = %{ get_queries($multidata) };

  #trade
  for my $query_name (keys %queries){
    my $array_ref = get_query_result($dbh, $queries{$query_name});
    $_data{$query_name}{$_database}{data} = $array_ref;
    $_data{$query_name}{$_database}{info}{db_name} = $_database;
  }
  # clean up
  $dbh->disconnect();

  return \%_data;
}

sub get_query_result {
  my $_dbh  = shift;
  my $query = shift;
  my @array;
  #warn $query;
  my $sth = $_dbh->prepare($query);
  $sth->execute();
  while(my $ref = $sth->fetchrow_hashref()) {
    push(@array, $ref);
  }

  return \@array;
}

sub make_perf {
  my $_data   = shift;
  my $db_name = shift;
  my %_data   = %{$_data};
  my %perf_types = (
    virtual     => 1,
    c_counters  => 1,
    counters    => 1,
    wait_events => 1,
    capacity    => 1,
  );

  my %perf;
  for my $type (keys %perf_types){
#    print Dumper $_data{$type}{$db_name}{data};
    if($type eq "virtual"){
      foreach my $row (@{$_data{$type}{$db_name}{data}}){
        my %row_dup = %{$row};
        delete $row_dup{name};
        my $accepted = 0;
        if ($use_whitelist){
          if ($whitelisted{$row->{name}}){
            $accepted = 1;
          }
        }else{
          $accepted = 1;
        }

        if($accepted){
          $instances{$row->{name}} = 1;
          $perf{$type}{$row->{name}} = \%row_dup;
        }
      }
    }
    elsif($type eq "counters"){
       foreach my $row (@{$_data{$type}{$db_name}{data}}){
        my %row_dup = %{$row};
        $row_dup{instance_name} =~ s/\s+$//g;
        $row_dup{counter_name}  =~ s/\s+$//g;
        my $accepted = 0;
        if ($use_whitelist){
          if ($whitelisted{$row_dup{instance_name}}){
            $accepted = 1;
          }
        }else{
          $accepted = 1;
        }

        if($accepted){
          $instances{$row_dup{instance_name}} = 1;
          $perf{$type}{$row_dup{instance_name}}{$row_dup{counter_name}} = $row_dup{cntr_value};
        }
      }
    }
    elsif($type eq "c_counters"){
       foreach my $row (@{$_data{$type}{$db_name}{data}}){
        my %row_dup = %{$row};
        $row_dup{instance_name} =~ s/\s+$//g;
        $row_dup{counter_name}  =~ s/\s+$//g;
        my $accepted = 0;
        $perf{counters}{Cluster}{$row_dup{counter_name}} = $row_dup{cntr_value};
      }
    }
    elsif($type eq "wait_events"){
      foreach my $row (@{$_data{$type}{$db_name}{data}}){
        my %row_dup = %{$row};
        $perf{$type}{Cluster}{$row_dup{wait_type}} = $row_dup{wait_time_ms};
      }
    }
  }

  $perf{_info}{timestamp} = time;
  $perf{_info}{readable_time} = localtime($perf{_info}{timestamp});

  return \%perf;
}

sub work_counters {
  my $alias = shift;
  my $_data = shift;
  my $iostats_dir = shift;
  my %act_perf = %{$_data};
  my %new_perf;
  my %old_perf;

  my $perf_l = "$iostats_dir/$alias"."_sqlsrvr_perf_last.json";
  my ($p_can_read, $p_ref);

  if(-e $perf_l){
    my $modtime = (stat($perf_l))[9];
    my $timediff = time - $modtime;
    if($timediff >= 420){
      unlink($perf_l);
      write_json( "$perf_l", $_data);
      return 0;
    }
    ( $p_can_read, $p_ref ) = read_json($perf_l);
  }else{
    write_json( "$perf_l", $_data);
    return 0;
  }
  if($p_can_read){
    %old_perf = %{$p_ref};
    my @dbs;
    my %raw_metrics = (
        "Database pages" => "1",
        "Extension allocated pages" => "1",
        "Extension free pages" => "1",
        "Page life expectancy" => "1",
        "Target pages" => "1",
        "Transactions" => "1",
        "User Connections" => "1",
        "Free Memory (KB)" => "1",
        "Reserved Server Memory (KB)" => "1",
        "Target Server Memory (KB)" => "1",
        "Total Server Memory (KB)" => "1",
        "Active Transactions" => "1",
        "Cache Hit Ratio" => "1",
        "Cache Hit Ratio Base" => "1",
        "Log Cache Hit Ratio" => "1",
        "Log Cache Hit Ratio Base" => "1",
        'Buffer cache hit ratio' => "1",
        'Buffer cache hit ratio base' => "1"
    );
    for my $type (keys %act_perf){
      if($type ne "_info"){
        for my $db (keys %{$act_perf{$type}}){
          for my $metric (keys %{$act_perf{$type}{$db}}){
            my $neutered_metric = $metric;
            $neutered_metric =~ s/ //g;
            $neutered_metric =~ s/\///g;
            $neutered_metric =~ s/\(KB//g;
            $neutered_metric =~ s/\)//g;
            $neutered_metric =~ s/\%//g;
            $neutered_metric =~ s/-//g;
#            if(!$non_counters{$metric}){
            if(!defined $old_perf{$type}{$db}{$metric} or $old_perf{$type}{$db}{$metric} eq "0" or
               $old_perf{$type}{$db}{$metric} == 0 or $act_perf{$type}{$db}{$metric} < $old_perf{$type}{$db}{$metric}){
              if(!$raw_metrics{$metric} && !$type eq "capacity" ){
                $new_perf{$db}{$neutered_metric} = 0;
                next;
              }
            }
            if ($type eq "wait_events"){
                $new_perf{$db}{$neutered_metric} = ($act_perf{$type}{$db}{$metric} - $old_perf{$type}{$db}{$metric});
            }else{
                if( $type eq "virtual"){
                  $new_perf{$db}{$neutered_metric} = int(($act_perf{$type}{$db}{$metric} - $old_perf{$type}{$db}{$metric}) / 300);
                }else{
                  $new_perf{$db}{$neutered_metric} = ($act_perf{$type}{$db}{$metric} - $old_perf{$type}{$db}{$metric}) / 300;
                }
            }

            if($raw_metrics{$metric} or $type eq "capacity" ){#or $type eq "wait_events"){
              if ($metric eq 'Free Memory (KB)' or
                $metric eq 'Reserved Server Memory (KB)' or
                $metric eq 'Target Server Memory (KB)' or
                $metric eq 'Total Server Memory (KB)')
              {
                $new_perf{$db}{$neutered_metric} = $act_perf{$type}{$db}{$metric} * 1000;
              }else{
                $new_perf{$db}{$neutered_metric} = $act_perf{$type}{$db}{$metric};
              }
            }
          }
          my %ratios = (
            "CacheHitRatio" => "CacheHitRatioBase",
            "LogCacheHitRatio" => "LogCacheHitRatioBase",
            'Buffercachehitratio' => 'Buffercachehitratiobase',
          );
          for my $r_type (keys %ratios){
            if ($new_perf{$db}{$r_type} and $new_perf{$db}{$ratios{$r_type}}){
                $new_perf{$db}{$r_type}  =  $new_perf{$db}{$r_type} / $new_perf{$db}{$ratios{$r_type}};
                $new_perf{$db}{$r_type} *= 100;
            }
          }
          push(@dbs, $db);
        }
      }
    }

    $new_perf{_info} = $act_perf{_info};

    write_json($perf_l, \%act_perf);
    return \%new_perf;
  }else{
    write_json($perf_l, \%act_perf);
    return 0;
  }
}

sub get_queries {
  my $multidata = shift;

  my $dbs = q(
  SELECT [name] AS database_name,
      database_id,
      create_date
  FROM sys.databases
  ORDER BY name;
  );

  my $dbs2 = q(
  SELECT name FROM master.dbo.sysdatabases;
  );

  my $dbs3 = q(
  EXEC sp_databases;
  );

  my $cpu = q(
  USE [master]

  SELECT
    cpu_idle = record.value('(./Record/SchedulerMonitorEvent/SystemHealth/SystemIdle)[1]', 'int'),
    cpu_sql = record.value('(./Record/SchedulerMonitorEvent/SystemHealth/ProcessUtilization)[1]', 'int')
  FROM (
      SELECT TOP 1 CONVERT(XML, record) AS record
      FROM sys.dm_os_ring_buffers
      WHERE ring_buffer_type = N'RING_BUFFER_SCHEDULER_MONITOR'
        AND record LIKE '% %'
    ORDER BY TIMESTAMP DESC
  ) as cpu_usage;
  );

  #CPU time
  my $cpu_per_db = q(
  WITH CPU_Per_DB
  AS
  (
   SELECT
    dmpa.DatabaseID,
    DB_Name(dmpa.DatabaseID) AS [Database],
    SUM(dmqs.total_worker_time) AS CPUTimeAsMS
    FROM sys.dm_exec_query_stats dmqs
    CROSS APPLY
   (
    SELECT
     CONVERT(INT, value) AS [DatabaseID]
     FROM sys.dm_exec_plan_attributes(dmqs.plan_handle)
    WHERE attribute = N'dbid'
   ) dmpa GROUP BY dmpa.DatabaseID
  )

   SELECT
   [DatabaseID],
   [Database],
   [CPUTimeAsMS],
   CAST([CPUTimeAsMS] * 1.0 / SUM([CPUTimeAsMS]) OVER() * 100.0 AS DECIMAL(5, 2)) AS [CPUTimeAs_percent]
   FROM CPU_Per_DB
   ORDER BY [CPUTimeAsMS] DESC;
  );

  my $cpu_lgcl = q(
  SELECT (cpu_count / hyperthread_ratio) AS PhysicalCPUs,
  cpu_count AS logicalCPUs
  FROM sys.dm_os_sys_info;
  );

  # great showcase of using CTE as a way to save myself the trouble of calculating it afterwards
  my $datafiles = q(
  ;WITH f AS
  (
    SELECT name, size = size/128.0 FROM sys.database_files
  ),
  s AS
  (
    SELECT name, size, free = size-CONVERT(INT,FILEPROPERTY(name,'SpaceUsed'))/128.0
    FROM f
  )
  SELECT name, size, free, percent_free = free * 100.0 / size
  FROM s;
  );

  my $datafiles2 = q(
  SELECT
    TOP (50)
    s.Name AS SchemaName,
    t.NAME AS TableName,
    o.type_desc AS Type,
    p.rows,
    SUM(a.total_pages) * 8000 AS TotalSpace,
    SUM(a.used_pages) * 8000  AS UsedSpace,
    CAST((CAST(SUM(a.used_pages) AS float) / NULLIF(CAST(SUM(a.total_pages) AS float), 0.0) *100) AS int) AS UsedPercent
  FROM
    sys.tables t
  INNER JOIN
    sys.indexes i ON t.OBJECT_ID = i.object_id
  INNER JOIN
    sys.partitions p ON i.object_id = p.OBJECT_ID AND i.index_id = p.index_id
  INNER JOIN
    sys.objects o ON t.OBJECT_ID = o.OBJECT_ID OR i.object_id = o.OBJECT_ID OR i.index_id = o.object_id
  INNER JOIN
    sys.allocation_units a ON p.partition_id = a.container_id
  LEFT OUTER JOIN
    sys.schemas s ON t.schema_id = s.schema_id
  WHERE
    t.NAME NOT LIKE 'dt%'
    AND t.is_ms_shipped = 0
    AND i.OBJECT_ID > 255
  GROUP BY
    t.Name, s.Name, p.Rows,o.type_desc
  ORDER BY
    TotalSpace DESC, t.Name;
  );

  my $logspace = q(
  SELECT *
  FROM sys.dm_db_log_space_usage;
  );

  #GRANT EXECUTE ON xp_readerrorlog TO _USERNAME_or_GROUPNAME
  #sp_readerrorlog
  my $err_log = q(
  EXEC xp_ReadErrorLog 0;
  );

  my $hosts = q(
  SELECT * FROM sys.dm_os_hosts;
  );

  my $blocks = q(
  USE [master]

  SELECT
  session_id,
  blocking_session_id,
  wait_time,
  wait_type,
  last_wait_type,
  wait_resource,
  transaction_isolation_level,
  lock_timeout
  FROM sys.dm_exec_requests
  WHERE blocking_session_id >= 0;
  );

  my $event_log = q(
  SELECT [Begin Time], [Transaction Name] from fn_dblog(null, null);
  );

  my $transactions = q(
  SELECT I.instance_name
        ,I.counter_name
        ,I.cntr_type
        ,I.cntr_value
        ,I.object_name
  FROM sys.dm_os_performance_counters as I
    INNER JOIN sys.databases AS D
      ON I.instance_name = d.name;
  );

  my $cluster_node = q(
  SELECT * FROM sys.dm_os_cluster_nodes;
  );

  my $filegroups = q(
  SELECT
  df.name AS [DB File Name],
  df.size AS [File Size],
  fg.name AS [File Group Name],
  df.physical_name AS [File Path]
  FROM sys.database_files AS df
  INNER JOIN sys.filegroups AS fg
  ON df.data_space_id = fg.data_space_id;
  );
# Make sure to monitor MAXDOP, really usefull for parallelism control
  my $p_counters = q(
  SELECT I.instance_name
        ,I.counter_name
        ,I.cntr_type
        ,I.cntr_value
  FROM sys.dm_os_performance_counters as I
    INNER JOIN sys.databases AS D
      ON I.instance_name = d.name
  WHERE I.counter_name IN
      ('Background writer pages/sec','Checkpoint pages/sec','Page life expectancy',
      'Lazy writes/sec','Page reads/sec','Page writes/sec','Readahead pages/sec',
      'Readahead time/sec','Database pages','Extension allocated pages',
      'Extension free pages','Target pages','User Connections','Connection Reset/sec',
      'Logins/sec','Logouts/sec','Active cursors','Cursor Requests/sec',
      'Cached Cursors Counts','Transactions','Transactions/sec','Tracked transactions/sec',
      'Write Transactions/sec','Active Transactions',
      'Cache Hit Ratio','Cache Hit Ratio base',
      'Buffer cache hit ratio','Buffer cache hit ratio base',
      'Log Cache Hit Ratio','Log Cache Hit Ratio base');
  );
#WHERE object_name = 'Resource';

# Make sure to monitor MAXDOP, really usefull for parallelism control
  my $c_counters = q(
  SELECT instance_name
        ,counter_name
        ,cntr_type
        ,cntr_value
  FROM sys.dm_os_performance_counters as I
  WHERE instance_name in('', '_Total') AND I.counter_name IN
      ('Background writer pages/sec','Buffer cache hit ratio','Buffer cache hit ratio base',
       'Checkpoint pages/sec','Database pages','Extension allocated pages',
       'Extension free pages','Lazy writes/sec','Logins/sec',
       'Logouts/sec','Page life expectancy','Page reads/sec',
       'Page writes/sec','Readahead pages/sec','Readahead time/sec',
       'Target pages','Transactions','User Connections','Connection Reset/sec',
       'Free Memory (KB)','Reserved Server Memory (KB)',
       'Target Server Memory (KB)','Total Server Memory (KB)','Locks Requests/sec',
       'Locks Waits/sec','Number of Deadlocks/sec','Lock Timeouts/sec'
      );
  );

  my $waitevents_l = q(
    SELECT * FROM sys.dm_os_wait_stats  WHERE ( wait_type LIKE '%LATCH_%' OR wait_type LIKE 'LCK_M_%') OR wait_type in (
      'RESOURCE_SEMAPHORE_QUERY_COMPILE','SOS_SCHEDULER_YIELD','ASYNC_NETWORK_IO',
      'MSQL_XP','EXECSYNC,WRITE_COMPLETION',
      'IO_COMPLETION','CXPACKET','WRITELOG','THREADPOOL');
  );

  my $waitevents = q(
  WITH waits AS
  (SELECT
    wait_type,
    wait_time_ms / 1000.0 AS waits,
    (wait_time_ms - signal_wait_time_ms) / 1000.0 AS resources,
    signal_wait_time_ms / 1000.0 AS signals,
    waiting_tasks_count AS waitcount,
    100.0 * wait_time_ms / sum (wait_time_ms) over() AS percentage,
    row_number() over(order by wait_time_ms desc) AS rownum
  FROM sys.dm_os_wait_stats
  WHERE wait_type NOT IN (
    N'CLR_SEMAPHORE', N'LAZYWRITER_SLEEP',
    N'RESOURCE_QUEUE', N'SQLTRACE_BUFFER_FLUSH',
    N'SLEEP_TASK', N'SLEEP_SYSTEMTASK',
    N'WAITFOR', N'HADR_FILESTREAM_IOMGR_IOCOMPLETION',
    N'CHECKPOINT_QUEUE', N'REQUEST_FOR_DEADLOCK_SEARCH',
    N'XE_TIMER_EVENT', N'XE_DISPATCHER_JOIN',
    N'LOGMGR_QUEUE', N'FT_IFTS_SCHEDULER_IDLE_WAIT',
    N'BROKER_TASK_STOP', N'CLR_MANUAL_EVENT',
    N'CLR_AUTO_EVENT', N'DISPATCHER_QUEUE_SEMAPHORE',
    N'TRACEWRITE', N'XE_DISPATCHER_WAIT',
    N'BROKER_TO_FLUSH', N'BROKER_EVENTHANDLER',
    N'FT_IFTSHC_MUTEX', N'SQLTRACE_INCREMENTAL_FLUSH_SLEEP',
    N'DIRTY_PAGE_POLL', N'SP_SERVER_DIAGNOSTICS_SLEEP')
  )
  SELECT
    w1.wait_type AS waittype,
    CAST (w1.waits AS decimal(14, 2)) wait_s,
    CAST (w1.resources AS decimal(14, 2)) resource_s,
    CAST (w1.signals AS decimal(14, 2)) signal_s,
    w1.waitcount wait_count,
    CAST (w1.percentage AS decimal(4, 2)) percentage,
    CAST ((w1.waits / w1.waitcount) AS decimal (14, 4)) avgWait_s,
    CAST ((w1.resources / w1.waitcount) AS decimal (14, 4)) avgResource_s,
    CAST ((w1.signals / w1.waitcount) AS decimal (14, 4)) avgSignal_s
  FROM waits AS w1
  INNER JOIN waits AS w2 ON w2.rownum <= w1.rownum
  GROUP BY w1.rownum, w1.wait_type, w1.waits, w1.resources, w1.signals, w1.waitcount, w1.percentage
  HAVING sum (w2.percentage) - w1.percentage < 95; -- percentage threshold;
  );

  my $virtual = q(
  SELECT name
      ,SUM(num_of_reads) AS 'io_rd'
      ,SUM(num_of_writes) AS 'io_wr'
      ,SUM(num_of_reads + num_of_writes) AS 'io_total'
      ,SUM(num_of_bytes_read / 1024) AS 'data_rd'
      ,SUM(num_of_bytes_written / 1024) AS 'data_wr'
      ,SUM((num_of_bytes_read + num_of_bytes_written) / 1024) AS 'data_total'
      ,CAST(SUM(io_stall_read_ms) / (1.0 + SUM(num_of_reads)) AS NUMERIC(10,1)) AS 'latency_rd'
      ,CAST(SUM(io_stall_write_ms) / (1.0 + SUM(num_of_writes) ) AS NUMERIC(10,1)) AS 'latency_wr'
      ,CAST(SUM(io_stall) / (1.0 + SUM(num_of_reads + num_of_writes)) AS NUMERIC(10,1)) AS 'latency_total'
  FROM sys.dm_io_virtual_file_stats(NULL, NULL) AS I
    INNER JOIN sys.databases AS D
      ON I.database_id = d.database_id
  GROUP BY name ORDER BY 'name' DESC;
  );

  my $vrtl = q(
  SELECT name AS 'Database Name'
      ,SUM(num_of_reads) AS 'Number of Read'
      ,SUM(num_of_writes) AS 'Number of Writes'
  FROM sys.dm_io_virtual_file_stats(NULL, NULL) I
  INNER JOIN sys.databases D
      ON I.database_id = d.database_id
  GROUP BY name ORDER BY 'Number of Read' DESC;
  );

  my $main01 = q(
  SELECT name,value FROM sys.configurations WHERE name IN('remote access','remote data archive','max degree of parallelism');
  );

  my $main02 = q(
  SELECT physical_memory_kb*1000 as physical_memory,virtual_memory_kb*1000 as virtual_memory,max_workers_count,socket_count,
         cores_per_socket,cpu_count,hyperthread_ratio,numa_node_count,softnuma_configuration FROM sys.dm_os_sys_info;
  );

  my $main03 = q(
  SELECT host_platform, host_distribution, host_release FROM sys.dm_os_host_info;

  );

  my $size = q(
   exec sp_spaceused @oneresultset = 1
  );


  my %queries = (
    capacity    => $size,
    db_list     => $dbs,
##    datafiles   => $datafiles,
    datafiles2  => $datafiles2,
    counters    => $p_counters,
    c_counters  => $c_counters,
    virtual     => $virtual,
##    txn         => $logspace,
    m1          => $main01,
    m2          => $main02,
    m3          => $main03,
    flgrps      => $filegroups,
#    test        => $,
    wait_events => $waitevents_l,
  );
  my %multidata = (
    capacity    => $size,
    datafiles2  => $datafiles2,
    flgrps      => $filegroups,
  );
  if($multidata){
    return \%multidata;
  }else{
    return \%queries;
  }
}

sub make_conf {
  my $_data = shift;
  my $db_name = shift;
  my %_data = %{$_data};
  my %conf_types = (
          flgrps => 1,
          datafiles2 => 1,
          m1 => 1,
          m2 => 1,
          m3 => 1,
          capacity => 1,
  );

  my %perf;
  for my $type (keys %conf_types){
#    if($conf_types{$type}){
#      warn "Couldn't find the data for type: $type in conf";
#    }

    if($type =~ m/m[1,2,3]/){
      if ($type eq "m1"){
        foreach my $row (@{$_data{$type}{$db_name}{data}}){
          my %row_dup = %{$row};
          my %nr;
          my $value = $row_dup{value};
          $value =~ s/[\r\n]//g;
          $perf{_cluster}{main}{cluster}[0]{$row_dup{name}} = $value;
        }
      }else{
        foreach my $row (@{$_data{$type}{$db_name}{data}}){
          my %row_dup = %{$row};
          for my $key (keys %row_dup){
            my $value = $row_dup{$key};
            $value =~ s/[\r\n]//g;
            $perf{_cluster}{main}{cluster}[0]{$key} = $value;
          }
        }
      }
    }elsif($type eq "capacity"){
      for my $_db_name (keys %{$_data{$type}}){
        foreach my $row (@{$_data{$type}{$_db_name}{data}}){
          my %row_dup = %{$row};

          my $_data = convert_to_gb($row->{'data'},'data');
          my $_index_size = convert_to_gb($row->{'index_size'},'index_size');
          my $_reserved = convert_to_gb($row->{'reserved'},'reserved');
          my $_unallocated_space = convert_to_gb($row->{'unallocated space'},'unallocated space');
          my $_database_size = convert_to_gb($row->{'database_size'}, 'database_size');
          my $_unused = convert_to_gb($row->{'unused'},'unused');
          $perf{$type}{$_db_name}{used} = $_data + $_index_size;
          $perf{$type}{$_db_name}{available} = $_unallocated_space;
          $perf{$type}{$_db_name}{log_space} = $_database_size - ($_unallocated_space + $_reserved);
          $perf{$type}{$_db_name}{unused} = $_unused;
        }
      }
    }else{
      for my $d_name (keys %{$_data{$type}}){
        my $row_counter = 0;
        foreach my $row (@{$_data{$type}{$d_name}{data}}){
          my %row_dup = %{$row};
          next if(!defined $row or !defined $type or !defined $d_name or !defined $_data{$type}{$d_name}{data});
          $perf{_cluster}{$type}{$d_name}[$row_counter] = \%row_dup;
          $row_counter++;
        }
      }
    }
  }

  $perf{_info}{timestamp} = time;
  $perf{_info}{readable_time} = localtime($perf{_info}{timestamp});

  return \%perf;
}

sub make_csv {
  my $_parent = shift;
  my $_alias = shift;
  my $_act_perf = shift;

  my $metrics_r = get_metrics();
  my %metrics = %{$metrics_r};
  my $csv = "";
  my $header = "uuid;timestamp;";
  my @metric_keys = keys %metrics;
  $header .= join(";", @metric_keys);
  $header .=";\n";
  my $cluster_id = SQLServer::md5_string("$_parent-$_alias-DB_FOLDERS");
  for my $db (keys %{$_act_perf}){
    next if ($db eq "_info");
    my $uuid = SQLServer::md5_string($db);
    if($db eq "_cluster"){
      $uuid = $cluster_id;
    }
    my @name = split("-", $db);
    #architecture
    my %cluster_arch = (
        "item_id" => $uuid,
        "label" => $name[0],
        "hw_type" => "SQLSERVER",
        "subsystem" => "DB",
        "parents" => [$cluster_id,$_parent],
        "hostcfg_id" => $_parent
    );
    push(@architecture, \%cluster_arch);


    my $line = "$uuid;$_act_perf->{_info}->{timestamp};";
    for my $metric (keys %metrics){
      if ($_act_perf->{$db}->{$metric}){
        $line .= "$_act_perf->{$db}->{$metric};";
      }else{
        $line  .= "0;";
      }
    }
    $line .= "\n";
    $csv .= $line;
  }

  $csv = "$header".$csv;
  return $csv;
}

sub convert_to_gb {
  my $_value = shift;
  my $type = shift;
  my $new_val = $_value;
  if(defined $new_val and $new_val =~ / KB/){
    $new_val =~ s/ KB//g;
    $new_val *= 1000;
  }elsif(defined $new_val and $new_val =~ / MB/){
    $new_val =~ s/ MB//g;
    $new_val *= 1000000;
  }elsif(defined $new_val and $new_val =~ / GB/){
    $new_val =~ s/ GB//g;
    $new_val *= 1000000000;
  }elsif(defined $new_val and $new_val =~ / TB/){
    $new_val =~ s/ TB//g;
    $new_val *= 1000000000000;
  }else{
    warn "$_value, $type";
    return 0;
  }

  return $new_val;
}

sub compare_perf {
  my $alias = shift;
  my $_act_perf = shift;
  my ( $can_read, $ref );
  my $mvc_dir = $FindBin::Bin;
  my $iostats_dir = "$mvc_dir/iostats";

  unless ( -d $iostats_dir ) {
    mkdir( "$iostats_dir", 0755 ) || warn( "Cannot mkdir $iostats_dir: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }

  my $data = work_counters($alias, $_act_perf, $iostats_dir);
  if($data and $data ne 0){
    write_json( "$iostats_dir/".$alias."_sqlsrvr_perf.json-nperf", $data);
    return $data;
  }else{
    return 0;
  }
}

sub write_json {
  my $path        = shift;
  my $hash_ref    = shift;
  my $path_suffix = "$path-tmp" . $$;
  my $json        = JSON->new->utf8;

  my ($fh, $file_name) = tempfile( UNLINK => 1 ); #use tempfile instead of the $path_suffix (HD)

  if ( ref($hash_ref) ne "HASH" && ref($hash_ref) ne "ARRAY"){
    warn ("Hash ref or array ref expected (path:$path), got: \"$hash_ref\" (Ref:" . ref($hash_ref) . ") in \n");
    return 0;
  }
#  if ($ENV{JSON_PRETTY}){
    $json->pretty([1]);
#  }

  #open( my $FILE, '>', $path_suffix ) || error( "Couldn't open file $path_suffix $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  #print $FILE $json->encode( $hash_ref );
  #close $FILE;
  #rename($path_suffix, $path);

  print $fh $json->encode( $hash_ref );
  close($fh);

  copy($file_name, $path);

  return 1;
}

sub read_json {
  my $file = shift;
  my $json = JSON->new->utf8;
  my $string;
  my $output;

  {
    local $/=undef;
    open ( my $FILE, '<', $file ) or warn "Couldn't open file $file $!" && return ( 0, undef );
    $string = <$FILE>;
    close $FILE;
  }

  if ($string) {
    eval {
      $output = $json->decode($string);
      1;
    } or do {
      warn "Couldn't parse json file $file $@ " && return ( 0, undef );
    };
  }

  return ( 1, $output );
}



$server->close();
