package STOREONCE;

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib '../../../perl-modules';
use Xormon;
use JSON qw(encode_json decode_json);
use HTTP::Date;
use Date::Parse;
use Test;
use XormonLogger;
#use POSIX qw( strftime );

my %data;
my $xormon;
my $cfg;
my $logger;

sub collect_api {
  $xormon = shift;
  $cfg    = shift;

  #################
  # SET VARIABLES #
  #################
  unless ( exists $cfg->{'hostcfg_id'} )        { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")   && return 0; }
  unless ( exists $cfg->{'hostalias'} )         { Xormon::error("Cannot continue, 'hostalias' is not configured!")    && return 0; }
  $logger = XormonLogger->new( 'storeonce', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )           { $logger->error("Cannot continue, 'apiHost' is not configured!")     && return 0; }
  unless ( exists $cfg->{'apiUsername'} )       { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return 0; }
  unless ( exists $cfg->{'apiPassword'} )       { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return 0; }

  my $url = "https://$cfg->{'apiHost'}/pml/login/authenticatewithobject";
  my $return_collect;

  #### TEST VERSIONS
  my $agent = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( POST => $url );

  $request->header('Content-Type' => 'application/json', 'Accept' => 'application/json');

  my $json_con ="{\"username\":\"$cfg->{'apiUsername'}\",\"password\":\"$cfg->{'apiPassword'}\",\"grant_type\":\"password\"}";

  $request->content('' . $json_con . '');
  my $results = $agent->request($request);
  #print Dumper $results;
  if (!$results->is_success){
      #CALL STOREONCE FUNCTION
      #Xormon::log("VERSION STOREONCE");
      # $logger->error("Login failed!") && return 0;
  }
  else {
    ###STOREONCE VERSION 4###
    if (defined $results->content && $results->content ne ""){
      my $result = decode_json($results->content);
      if (defined $result->{'access_token'} && $result->{'access_token'} ne ""){
        $xormon->isConnected(1);
        my $cookie = $result->{'access_token'};
        my $api_time = "";
        chomp($cookie);
        if ( defined $results->headers->{'date'} && $results->headers->{'date'} ne ""){
          $api_time = $results->headers->{'date'};
        }
        else{
          $logger->error("Storage time (date) not found!") && return 0;
        }
        $return_collect = storeonce_v4($cookie,$api_time);
      }
      else{
        $xormon->isConnected(0, "access_token not found");
        $logger->error("Cookie not found! Login failed! access_token not found!") && return 0;
      }
    }
  }
  return $return_collect;
}

sub storeonce_v4 {
    $logger->info("STOREONCE V4 START\n");
    $cfg->{'token'} = shift;
    my $mytime = shift;
    $cfg->{'debug'} = 0;

    if ( $cfg->{'token'} eq "0" ){
      $logger->error("Login failed : Cant' login to API") && return 0;
    }

    #my $timestamp = time();
    #print "MY TIME: $mytime";
    my $timestamp = str2time("$mytime");
    $cfg->{'gtime'} = gmtime($timestamp);
    $cfg->{'gtime'} =~ s/ {2,}/ /g;

    ($cfg->{'start_time'},$cfg->{'end_time'}) = set_time();

    #Debug
    if ($cfg->{'debug'} == 1){print "\nSTART\n";}

    ###Alerts
    get_alerts("https://$cfg->{'apiHost'}/rest/alerts?sort=created:desc");

    ###Nas
    create_conf("https://$cfg->{'apiHost'}/api/v1/data-services/nas/shares?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","NAS");
    ###CAT
    create_conf("https://$cfg->{'apiHost'}/api/v1/data-services/cat/stores?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","CAT");
    ###VTL
    create_conf("https://$cfg->{'apiHost'}/api/v1/data-services/vtl/libraries?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","VTL");
    #### STORAGE Appliance
    create_conf("https://$cfg->{'apiHost'}/api/v1/management-services/federation/members?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","storagedata");
    ### Storage CAP
    create_conf("https://$cfg->{'apiHost'}/api/v1/data-services/dashboard/appliance/localhost?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","storagestats");

    if (defined $data{'conf'}{storagestats}{'uuid'} && $data{'conf'}{storagestats}{'uuid'} ne "") {
      $xormon->hasData(1);
    }
    else {
      $xormon->hasData(0, "no uuid from api query");
    }

    ###NAS - THROUGHPUT
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/nas/parametrics-throughput/share/","NAS","throughput");
    ###NAS - STORAGE
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/nas/parametrics-storage/share/","NAS","storage");
    ###NAS - REPLICATION
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/rep/parametrics-share-throughput/share/","NAS","replication");

    ###CAT - THROUGHPUT
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/cat/parametrics-throughput/store/","CAT","throughput");
    ###CAT - STORAGE
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/cat/parametrics-storage/store/","CAT","storage");
    ###CAT - Cloud Bank
    #create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/cat/parametrics-storage/parametrics-cloud/","CAT","cloud");
    ###CAT - JOBS
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/cat/datajobs/store/","CAT","datajobs");
    ###CAT - HOUSEKEEPING
    #create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/smm/parametrics-housekeeping/store/","CAT","housekeeping");
    ###CAT - DEDUPE
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/smm/stores/store/","CAT","dedupe");

    ###VTL - THROUGHPUT
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/vtl/parametrics-throughput/library/","VTL","throughput");
    ###VTL - STORAGE
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/vtl/parametrics-storage/library/","VTL","storage");
    ###VTL - REPLICATION
    create_perf("https://$cfg->{'apiHost'}/api/v1/data-services/rep/parametrics-library-throughput/library/","VTL","replication");

    #CPU
    get_storage_data_cpu("https://$cfg->{'apiHost'}/api/v1/management-services/hardware/parametrics-cpu?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","CPU");
    #MEMORY
    #get_storage_data("https://$cfg->{'apiHost'}/api/v1/management-services/hardware/parametrics-memory?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","MEMORY");
    #DISK - DATA/OS
    get_storage_data_disk("https://$cfg->{'apiHost'}/api/v1/management-services/hardware/parametrics-disk?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","DISK");
    #PORTS
    get_storage_data_ports("https://$cfg->{'apiHost'}/api/v1/management-services/hardware/parametrics-ethernet?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","ETHPORT");
    get_storage_data_ports("https://$cfg->{'apiHost'}/api/v1/management-services/hardware/parametrics-fibre-channel?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","FCPORT");

    ###HOUSEKEEPING
    get_storage_data_house("https://$cfg->{'apiHost'}/api/v1/data-services/smm/parametrics-housekeeping?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","HOUSE");

    print_perf();
    health_status();
    print_conf();
    if ($cfg->{'debug'} == 1){
      print "\nEND\n";
    }

    logout();
    return 1;
}

################
# STOREONCE V4 #
#    START     #
################
sub set_time{
  my @t = split( / /, $cfg->{'gtime'});
  my (undef, undef, $sec) = split (":",$t[3]);
  if ($sec =~ m/^0/){
    $sec =~ s/0//;
  }
  my $end_time = str2time( join( " ", @t ) );
  #my $diff = $ltime - $end_time;
  $end_time = $end_time - $sec - 1;
  my $start_time = $end_time - 300;
  my $start_utc = get_time_utc($start_time);
  my $end_utc = get_time_utc($end_time);
  return ( "$start_utc","$end_utc" );
}

sub get_alerts{
  my $url = shift;
  my $return = new_get_request("$url");
  if ($return){
    my $i = 0;
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY"){
      foreach my $array ( @{ $return->{'members'} } ) {
        if (defined $array->{'level'} && $array->{'level'} ne ""){
          $data{'alerts'}{$i}{'level'} = $array->{'level'};
        }
        if (defined $array->{'created'} && $array->{'created'} ne ""){
          $data{'alerts'}{$i}{'time'} = $array->{'created'};
        }
        if (defined $array->{'uuid'} && $array->{'uuid'} ne ""){
          $data{'alerts'}{$i}{'event_id'} = $array->{'uuid'};
        }
        if (defined $array->{'eventCode'} && $array->{'eventCode'} ne ""){
          $data{'alerts'}{$i}{'event_type'} = $array->{'eventCode'};
        }
        if (defined $array->{'clearedByUser'} && $array->{'clearedByUser'} ne ""){
          $data{'alerts'}{$i}{'event_code'} = $array->{'clearedByUser'};
        }
        if (defined $array->{'description'} && $array->{'description'} ne ""){
          $data{'alerts'}{$i}{'message'} = $array->{'description'};
        }
        if (defined $array->{'correctiveAction'} && $array->{'correctiveAction'} ne ""){
          if (defined $data{'alerts'}{$i}{'message'} && $data{'alerts'}{$i}{'message'} ne ""){
            $data{'alerts'}{$i}{'message'} .= "\n $array->{'correctiveAction'}";
          }
          else{
            $data{'alerts'}{$i}{'message'} = "\n $array->{'correctiveAction'}";
          }
        }
        $i++
      }
    }
  }
}

sub get_storage_data_disk{
  my $url = shift;
  my $stat = shift;
  my $type = shift;
  my $divisor = 0;
  my $disk_type = "";
  my $return = new_get_request("$url");
  if ($cfg->{'debug'} == 1){
    print "\n\n";
    print "###############\n";
    print "#     Disk    #\n";
    print "###############\n";
    print "URL       : $url\n";
    print "Start     : $cfg->{'start_time'}\n";
    print "End       : $cfg->{'end_time'}\n";
  }
  if ($return){
    if (defined $return->{'numMembers'} && $return->{'numMembers'} ne ""){
      $divisor = $return->{'numMembers'};
    }
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" && $divisor != 0){
      foreach my $array ( @{ $return->{'members'} } ) {
        if(defined $array->{'disks'} && $array->{'disks'} ne "" &&  ref($array->{'disks'}) eq "ARRAY" ){
          foreach my $disk ( @{ $array->{'disks'} } ) {
            if (defined $disk->{'diskType'} && $disk->{'diskType'} ne ""){
              $disk_type = $disk->{'diskType'};
            }
            if (defined $disk->{'readThroughput'} && $disk->{'readThroughput'} ne ""){
              if (defined $data{'perf'}{'DISK'}{$disk_type}{'readThroughput'} && Xormon::isdigit($data{'perf'}{'DISK'}{$disk_type}{'readThroughput'}) ){
                $data{'perf'}{'DISK'}{$disk_type}{'readThroughput'} += $disk->{'readThroughput'};
              }
              else{
                $data{'perf'}{'DISK'}{$disk_type}{'readThroughput'} = $disk->{'readThroughput'};
              }
            }
            if (defined $disk->{'writeThroughput'} && $disk->{'writeThroughput'} ne ""){
              if (defined $data{'perf'}{'DISK'}{$disk_type}{'writeThroughput'} && Xormon::isdigit($data{'perf'}{'DISK'}{$disk_type}{'writeThroughput'}) ){
                $data{'perf'}{'DISK'}{$disk_type}{'writeThroughput'} += $disk->{'writeThroughput'};
              }
              else{
                $data{'perf'}{'DISK'}{$disk_type}{'writeThroughput'} = $disk->{'writeThroughput'};
              }
            }
          }
        }
      }
      foreach my $disk (keys ( %{ $data{'perf'}{'DISK'} } ) ) {
        if (defined $data{'perf'}{'DISK'}{$disk} && defined $data{'perf'}{'DISK'}{$disk} ne ""){
          foreach my $metrics (keys ( %{ $data{'perf'}{'DISK'}{$disk} } ) ) {
            if ($cfg->{'debug'} == 1){
              print "Metrics    : $metrics\n";
              print "Total      : $data{'perf'}{'DISK'}{$disk}{$metrics}\n";
              print "Divisor    : $divisor\n";
            }
            $data{'perf'}{'DISK'}{$disk}{$metrics} = $data{'perf'}{'DISK'}{$disk}{$metrics} / $divisor;
            if ($cfg->{'debug'} == 1){
              print "Result     : $data{'perf'}{'DISK'}{$disk}{$metrics}\n";
            }
          }
        }
      }
    }
    return 1;
  }
  else{
    return 0;
  }
}

sub get_storage_data_cpu{
  my $url = shift;
  my $stat = shift;
  my $type = shift;
  my $divisor = 0;
  my $return = new_get_request("$url");
  if ($return){
    if ($cfg->{'debug'} == 1){
      print "\n\n";
      print "###############\n";
      print "#     CPU     #\n";
      print "###############\n";
      print "URL        : $url\n";
      print "START      : $cfg->{'start_time'}\n";
      print "END        : $cfg->{'end_time'}\n";
    }
    if (defined $return->{'numMembers'} && $return->{'numMembers'} ne ""){
      $divisor = $return->{'numMembers'};
    }
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" && $divisor != 0){
      foreach my $array ( @{ $return->{'members'} } ){
        if(defined $array->{'percentageIoWait'} && $array->{'percentageIoWait'} ne "" ){
          if (defined $data{'perf'}{'CPU'}{'percentageIoWait'} && Xormon::isdigit($data{'perf'}{'CPU'}{'percentageIoWait'}) ){
            $data{'perf'}{'CPU'}{'percentageIoWait'} += $array->{'percentageIoWait'};
          }
          else{
            $data{'perf'}{'CPU'}{'percentageIoWait'} = $array->{'percentageIoWait'};
          }
        }
        if(defined $array->{'percentageCpuUsage'} && $array->{'percentageCpuUsage'} ne ""){
          if (defined $data{'perf'}{'CPU'}{'percentageCpuUsage'} && Xormon::isdigit($data{'perf'}{'CPU'}{'percentageCpuUsage'}) ){
            $data{'perf'}{'CPU'}{'percentageCpuUsage'} += $array->{'percentageCpuUsage'};
          }
          else{
            $data{'perf'}{'CPU'}{'percentageCpuUsage'} = $array->{'percentageCpuUsage'};
          }
        }
      }
      if (defined $data{'perf'}{'CPU'}{'percentageIoWait'} && Xormon::isdigit($data{'perf'}{'CPU'}{'percentageIoWait'}) ){
        if ($cfg->{'debug'} == 1){
          print "Total      : $data{'perf'}{'CPU'}{'percentageIoWait'}\n";
          print "Divisor    : $divisor\n";
        }
        $data{'perf'}{'CPU'}{'percentageIoWait'} = ($data{'perf'}{'CPU'}{'percentageIoWait'} / $divisor);
        if ($cfg->{'debug'} == 1){
          print "Result     : $data{'perf'}{'CPU'}{'percentageIoWait'}\n";
        }
      }
      if (defined $data{'perf'}{'CPU'}{'percentageCpuUsage'} && Xormon::isdigit($data{'perf'}{'CPU'}{'percentageCpuUsage'}) ){
        $data{'perf'}{'CPU'}{'percentageCpuUsage'} = ( $data{'perf'}{'CPU'}{'percentageCpuUsage'} / $divisor );
      }
    }
    return 1
  }
  else{
    return 0
  }
}

sub get_storage_data_house{
  my $url = shift;
  my $stat = shift;
  my $type = shift;
  my $divisor = 0;
  my $disk_type = "";
  my $return = new_get_request("$url");
  if ($cfg->{'debug'} == 1){
    print "\n\n";
    print "##################\n";
    print "#  HOUSEKEEPING  #\n";
    print "##################\n";
    print "URL        : $url\n";
    print "Start      : $cfg->{'start_time'}\n";
    print "END        : $cfg->{'end_time'}\n";
  }
  if ($return){
    # print Dumper $return;
    if (defined $return->{'numMembers'} && $return->{'numMembers'} ne ""){
      $divisor = $return->{'numMembers'};
    }
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" && $divisor != 0){
      foreach my $members ( @{ $return->{'members'} } ){
        if (defined $members->{'pendingBytes'} && $members->{'pendingBytes'} ne ""){
          if ( defined $data{'pooldata'}{'housekeepingPending'} &&  Xormon::isdigit($data{'pooldata'}{'housekeepingPending'} ) ){
            $data{'pooldata'}{'housekeepingPending'} += $members->{'pendingBytes'}
          }
          else{
            $data{'pooldata'}{'housekeepingPending'} = $members->{'pendingBytes'}
          }
        }
      }
      if ($cfg->{'debug'} == 1){
        print "Total       : $data{'pooldata'}{'housekeepingPending'}\n";
        print "Divisor     : $divisor\n";
      }
      $data{'pooldata'}{'housekeepingPending'} = $data{'pooldata'}{'housekeepingPending'} / $divisor;
      if ($cfg->{'debug'} == 1){
        print "Result      : $data{'pooldata'}{'housekeepingPending'}\n";
      }
    }
  }
}

sub get_storage_data_ports{
  my $url = shift;
  my $stat = shift;
  my $type = shift;
  my $port_name = "";
  my $divisor = 0;
  my $return = new_get_request("$url");
  if ($cfg->{'debug'} == 1){
    print "\n\n";
    print "#################\n";
    print "#     PORT      #\n";
    print "#################\n";
    print "URL        : $url\n";
    print "Start      : $cfg->{'start_time'}\n";
    print "End        : $cfg->{'end_time'}\n";
  }
  if ($return){
    if (defined $return->{'numMembers'} && $return->{'numMembers'} ne ""){
      $divisor = $return->{'numMembers'};
    }
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" && $divisor != 0){
      foreach my $array ( @{ $return->{'members'} } ) {
        if (defined $array && $array ne ""){
          foreach my $keys (keys ( %{ $array } ) ){
            if (defined $array->{$keys} && $array->{$keys} ne "" && ref($array->{$keys}) eq "ARRAY"){
              foreach my $ports ( @{ $array->{$keys} } ) {
                if (defined $ports->{'portName'} && $ports->{'portName'} ne ""){
                  $port_name = $ports->{'portName'};
                }
                else{
                  next;
                }
                if (defined $port_name && $port_name ne "" && defined $ports->{'receiveRate'} && $ports->{'receiveRate'} ne ""){
                  if (defined $data{'perf'}{$type}{$port_name}{'receiveRate'} && Xormon::isdigit($data{'perf'}{$type}{$port_name}{'receiveRate'}) ){
                    $data{'perf'}{$type}{$port_name}{'receiveRate'} += $ports->{'receiveRate'};
                  }
                  else{
                    $data{'perf'}{$type}{$port_name}{'receiveRate'} = $ports->{'receiveRate'};
                  }
                }
                if (defined $port_name && $port_name ne "" && defined $ports->{'transmitRate'} && $ports->{'transmitRate'} ne ""){
                  if (defined $data{'perf'}{$type}{$port_name}{'transmitRate'} && Xormon::isdigit($data{'perf'}{$type}{$port_name}{'transmitRate'}) ){
                    $data{'perf'}{$type}{$port_name}{'transmitRate'} += $ports->{'transmitRate'};
                  }
                  else{
                    $data{'perf'}{$type}{$port_name}{'transmitRate'} = $ports->{'transmitRate'};
                  }
                }
                if (defined $port_name && $port_name ne ""){
                   $data{'perf'}{$type}{$port_name}{'type'} = $keys;
                }
              }
            }
          }
        }
      }
      if (defined $data{'perf'}{$type}{$port_name}{'transmitRate'} && $data{'perf'}{$type}{$port_name}{'transmitRate'} ne ""){
        if ($cfg->{'debug'} == 1){
          print "Port name  : $port_name\n";
          print "Metrics    : transmitRate\n";
          print "Total      : $data{'perf'}{$type}{$port_name}{'transmitRate'}\n";
          print "Divisor    : $divisor\n";
        }
        $data{'perf'}{$type}{$port_name}{'transmitRate'} = $data{'perf'}{$type}{$port_name}{'transmitRate'} / $divisor;
        if ($cfg->{'debug'} == 1){
          print "Result     : $data{'perf'}{$type}{$port_name}{'transmitRate'}\n";
        }
      }
      if (defined $data{'perf'}{$type}{$port_name}{'receiveRate'} && $data{'perf'}{$type}{$port_name}{'receiveRate'} ne ""){
        if ($cfg->{'debug'} == 1){
          print "Metrics    : receiveRate\n";
          print "Total      : $data{'perf'}{$type}{$port_name}{'transmitRate'}\n";
          print "Divisor    : $divisor\n";
        }
        $data{'perf'}{$type}{$port_name}{'receiveRate'} = $data{'perf'}{$type}{$port_name}{'receiveRate'} / $divisor;
        if ($cfg->{'debug'} == 1){
          print "Result     : $data{'perf'}{$type}{$port_name}{'transmitRate'}\n";
        }
      }
    }
    return 1;
  }
  else{
    return 0;
  }
  #print Dumper $return;
}

sub create_conf {
  my $url = shift;
  my $type = shift;
  my $return;

  $return = new_get_request("$url");
  if ($return){
    if ($type eq "storagestats"){
      if (defined $return->{'localCapacityBytes'} && $return->{'localCapacityBytes'} ne ""){
        $data{'conf'}{$type}{'localCapacityBytes'} = $return->{'localCapacityBytes'};
      }
      if (defined $return->{'localDiskBytes'} && $return->{'localDiskBytes'} ne ""){
        $data{'conf'}{$type}{'localDiskBytes'} = $return->{'localDiskBytes'};
      }
      if (defined $return->{'localFreeBytes'} && $return->{'localFreeBytes'} ne ""){
        $data{'conf'}{$type}{'localFreeBytes'} = $return->{'localFreeBytes'};
      }
      if (defined $return->{'uuid'} && $return->{'uuid'} ne ""){
        $data{'conf'}{$type}{'uuid'} = $return->{'uuid'};
      }
      if (defined $return->{'softwareVersion'} && $return->{'softwareVersion'} ne ""){
        $data{'conf'}{$type}{'softwareVersion'} = $return->{'softwareVersion'};
      }
      if (defined $return->{'catStoresSummary'}->{statusSummary} && $return->{'catStoresSummary'}->{statusSummary} ne ""){
        if ($return->{'catStoresSummary'}->{statusSummary}->{'numCritical'} > 0){
          $data{'conf'}{'storagedata'}{'statusCat'} = "CRITICAL";
        }
        elsif ($return->{'catStoresSummary'}->{statusSummary}->{'numWarning'} > 0){
          $data{'conf'}{'storagedata'}{'statusCat'} = "WARNING";
        }
        elsif ($return->{'catStoresSummary'}->{statusSummary}->{'numOk'} > 0){
          $data{'conf'}{'storagedata'}{'statusCat'} = "OK";
        }
        else{
          $data{'conf'}{'storagedata'}{'statusCat'} = "UNKNOWN";
        }
      }
      if (defined $return->{'vtlLibrariesSummary'}->{statusSummary} && $return->{'vtlLibrariesSummary'}->{statusSummary} ne ""){
        if ($return->{'vtlLibrariesSummary'}->{statusSummary}->{'numCritical'} > 0){
          $data{'conf'}{'storagedata'}{'statusVtl'} = "CRITICAL";
        }
        elsif ($return->{'vtlLibrariesSummary'}->{statusSummary}->{'numWarning'} > 0){
          $data{'conf'}{'storagedata'}{'statusVtl'} = "WARNING";
        }
        elsif ($return->{'vtlLibrariesSummary'}->{statusSummary}->{'numOk'} > 0){
          $data{'conf'}{'storagedata'}{'statusVtl'} = "OK";
        }
        else{
          $data{'conf'}{'storagedata'}{'statusVtl'} = "UNKNOWN";
        }
      }
      if (defined $return->{'nasSharesSummary'}->{statusSummary} && $return->{'nasSharesSummary'}->{statusSummary} ne ""){
        if ($return->{'nasSharesSummary'}->{statusSummary}->{'numCritical'} > 0){
          $data{'conf'}{'storagedata'}{'statusNas'} = "CRITICAL";
        }
        elsif ($return->{'nasSharesSummary'}->{statusSummary}->{'numWarning'} > 0){
          $data{'conf'}{'storagedata'}{'statusNas'} = "WARNING";
        }
        elsif ($return->{'nasSharesSummary'}->{statusSummary}->{'numOk'} > 0){
          $data{'conf'}{'storagedata'}{'statusNas'} = "OK";
        }
        else{
          $data{'conf'}{'storagedata'}{'statusNas'} = "UNKNOWN";
        }
      }
      ## new HS values
      # my @ordered = ( 'Data Services', 'License', 'Storage', 'Hardware', 'Remote Support' );
      if ( defined $return->{'applianceStatus'} && $return->{'applianceStatus'} ne "" ) {
        $data{'conf'}{'storagedata'}{'applianceStatus'} = $return->{'applianceStatus'};
      }
      if ( defined $return->{'dataServicesStatus'} && $return->{'dataServicesStatus'} ne "" ) {
        $data{'conf'}{'storagedata'}{'Data Services'} = $return->{'dataServicesStatus'};
      }
      if ( defined $return->{'licenseStatus'} && $return->{'licenseStatus'} ne "" ) {
        $data{'conf'}{'storagedata'}{'License'} = $return->{'licenseStatus'};
      }
      if ( defined $return->{'userStorageStatus'} && $return->{'userStorageStatus'} ne "" ) {
        $data{'conf'}{'storagedata'}{'Storage'} = $return->{'userStorageStatus'};
      }
      if ( defined $return->{'hardwareStatus'} && $return->{'hardwareStatus'} ne "" ) {
        $data{'conf'}{'storagedata'}{'Hardware'} = $return->{'hardwareStatus'};
      }
      if ( defined $return->{'remoteSupportStatus'} && $return->{'remoteSupportStatus'} ne "" ) {
        $data{'conf'}{'storagedata'}{'Remote Support'} = $return->{'remoteSupportStatus'};
      }
    }
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY"){
      foreach my $array ( @{ $return->{'members'} } ){
        my $id       = "";
        if (defined $array->{'id'} && $array->{'id'} ne ""){
          $id = $array->{'id'};
        }
        elsif(defined $array->{'' . $type . 'UUID'} && $array->{'' . $type . 'UUID'} ne ""){
          $id = $array->{'' . $type . 'UUID'};
        }
        elsif(defined $array->{'uuid'} && $array->{'uuid'} ne ""){
          $id = $array->{'uuid'};
        }

        foreach my $keys ( keys( %{ $array } ) ){
          if (defined $id && $id ne ""){
            $data{'conf'}{$type}{$id}{$keys} = $array->{$keys};
          }
        }
      }
    }
  }
  else{
    $logger->error("Get $type failed : $url");
  }
}

sub create_perf {
  my $url   = shift;
  my $type  = shift;
  my $stats = shift;
  my $error = 1;
  my $return;

  #####DEBUG
  if ($cfg->{'debug'} == 1){
    print "\n\n";
    print "################\n";
    print "##### $type ####\n";
    print "################\n";
  }

  if (defined $data{'conf'}{$type} && $data{'conf'}{$type} ne ""){
    foreach my $ids (keys ( %{$data{'conf'}{$type}} ) ){
      $url .= "$ids?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}";
      if ($cfg->{'debug'} == 1){
        print "################\n";
        print "#    $stats    #\n";
        print "################\n";
        print "Start Time : $cfg->{'start_time'}\n";
        print "End Time   : $cfg->{'end_time'}\n";
        print "URL        : $url\n";
        print "ID         : $ids\n";
      }
      $return = new_get_request("$url");
      $url =~ s/$ids\?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}//g;
      if ($return){
        if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY"){
          my $divisor = "";
          if (defined $return->{'numMembers'}){
            $divisor = $return->{'numMembers'};
          }
          else{
            $logger->error("data for $type - $stats not found") && next;
          }
          foreach my $array ( @{ $return->{'members'} } ){
            foreach my $keys (keys ( %{ $array } ) ){
              if (defined $array->{$keys} && $array->{$keys} ne ""){
                if ( defined $data{'preperf'}{$type}{$ids}{$stats}{$keys} && Xormon::isdigit($data{'preperf'}{$type}{$ids}{$stats}{$keys}) && Xormon::isdigit($array->{$keys}) ){
                  ###Debug
                  if ($cfg->{'debug'} == 1){
                    $data{'debug'}{'csv'}{$keys} .= ",$array->{$keys}";
                  }
                  $data{'preperf'}{$type}{$ids}{$stats}{$keys} += $array->{$keys};
                }
                elsif(Xormon::isdigit( $array->{$keys})){
                  if ($cfg->{'debug'} == 1){
                    $data{'debug'}{'csv'}{$keys} = "$array->{$keys}";
                  }
                  $data{'preperf'}{$type}{$ids}{$stats}{$keys} = $array->{$keys};
                }
              }
            }
          }

          if (defined $data{'preperf'}{$type}{$ids}{$stats} ){
            foreach my $keys (keys ( %{$data{'preperf'}{$type}{$ids}{$stats}})){
              if (Xormon::isdigit($data{'preperf'}{$type}{$ids}{$stats}{$keys}) && $divisor != 0){
                ####Debug
                if ($cfg->{'debug'} == 1){
                  print "ID         : $ids\n";
                  print "Metrics    : $keys\n";
                  print "CSV Metric : $data{'debug'}{'csv'}{$keys}\n";
                  print "Total      : $data{'preperf'}{$type}{$ids}{$stats}{$keys}\n";
                  print "Divisor    : $divisor\n";
                }
                $data{'perf'}{$type}{$ids}{$keys} = $data{'preperf'}{$type}{$ids}{$stats}{$keys} / $divisor;
                if ($cfg->{'debug'} == 1){
                  print "Result     : $data{'perf'}{$type}{$ids}{$keys}\n";
                }
              }
            }
          }
        }
      }
      else{
        $error = 0;
      }
    }
  }
  return $error;
}

sub print_perf {
  my @architecture;
  my $data_out = "";
  my %conf;
  my $st_serial = $cfg->{'hostcfg_id'};
  my $rc;
  my $status;
  my $perf_timestamp = str2time( $cfg->{'end_time'} );

  #### DRIVE ####
  $data_out = "id;timestamp;read;write;data_rate\n";
  foreach my $ids (keys ( %{$data{'perf'}{'DISK'}})){
    if ($ids ne "DATA"){ next; }
    my $id = 1; ## DATA DISK
    my $read = "";
    my $write = "";
    my $data_rate = "";
    if (defined $data{'perf'}{'DISK'}{$ids}{'readThroughput'} && $data{'perf'}{'DISK'}{$ids}{'readThroughput'} ne ""){
      $read = $data{'perf'}{'DISK'}{$ids}{'readThroughput'}; #bytes/s
    }
    if (defined $data{'perf'}{'DISK'}{$ids}{'writeThroughput'} && $data{'perf'}{'DISK'}{$ids}{'writeThroughput'} ne ""){
      $write = $data{'perf'}{'DISK'}{$ids}{'writeThroughput'}; #bytes/s
    }
    if (defined $read && $read ne "" && defined $write && $write ne "" ){
      $data_rate = $read + $write;
    }

    my $hash_key = Xormon::create_hash_key($st_serial."Drive".$id); $hash_key = lc($hash_key);

    #print DATA "$id,$cfg->{'end_time'},$read,$write,$data_rate\n";
    $data_out .= "$hash_key;$perf_timestamp;$read;$write;$data_rate\n";
  }

  #Save DRIVE
  $rc = $xormon->saveCSV($data_out,"storeonce","drive");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }

  $logger->info("Drive status : $status");

  #### CATALYST ####
  # print Dumper $data{'perf'}{'CAT'};
  $data_out = "id;timestamp;data_job_read;data_job_write;copy_job_read;copy_job_write;user_capacity_used;disk_size;dedupe_ratio;stream_rate;stream_write_job;stream_read_job;logical_data_read;logical_data_write;logical_data_rate;logical_copy_job_read;logical_copy_job_write;logical_data_job_read;logical_data_job_write;user_capacity_used;disk_size;disk_quota;disk_prct;usr_prct;user_quota;ratio;read;write;data_rate;stream_job_rate\n";
  my @CAT = ("dataJobTxThroughput","dataJobRxThroughput","copyJobTxThroughput","copyJobRxThroughput","User_data_store(MB)","Disk_size(MB)","dedupeRatio","dataJobSessionCount","copyJobInSessionCount","copyJobOutSessionCount");
  foreach my $ids (keys ( %{$data{'perf'}{'CAT'}} ) ){
    my $hash_key = Xormon::create_hash_key($st_serial."CAT".$ids); $hash_key = lc($hash_key);
    my $line = "$hash_key;$perf_timestamp";

    # if (defined $data{'perf'}{'CAT'}{$ids}{'compactedBytes'} && $data{'perf'}{'CAT'}{$ids}{'compactedBytes'} ne "" && defined $data{'perf'}{'CAT'}{$ids}{'compactedDurationMicroseconds'} && $data{'perf'}{'CAT'}{$ids}{'compactedDurationMicroseconds'}   ne ""){
    #   if ( $data{'perf'}{'CAT'}{$ids}{'compactedDurationMicroseconds'} != 0 ){
    #     $data{'perf'}{'CAT'}{$ids}{'BytesCompactedThroughput'} = ( ( $data{'perf'}{'CAT'}{$ids}{'compactedBytes'} ) / ( $data{'perf'}{'CAT'}{$ids}{'compactedDurationMicroseconds'} / 1000  ) );
    #   }
    #   else{
    #     $data{'perf'}{'CAT'}{$ids}{'BytesCompactedThroughput'} = 0;
    #   }
    # }
    # else{
    #   $data{'perf'}{'CAT'}{$ids}{'BytesCompactedThroughput'} = "";
    # }
    # if (defined $data{'perf'}{'CAT'}{$ids}{'secureRemovedBytes'} && $data{'perf'}{'CAT'}{$ids}{'secureRemovedBytes'} ne "" && defined $data{'perf'}{'CAT'}{$ids}{'secureRemovedDurationMicroseconds'} && $data{'perf'}{'CAT'}{$ids}{'secureRemovedDurationMicroseconds'}   ne ""){
    #   if ( $data{'perf'}{'CAT'}{$ids}{'secureRemovedDurationMicroseconds'} != 0 ) {
    #     $data{'perf'}{'CAT'}{$ids}{'secureBytesRemovedThroughput'} = ( ( $data{'perf'}{'CAT'}{$ids}{'secureRemovedBytes'} ) / ( $data{'perf'}{'CAT'}{$ids}{'secureRemovedDurationMicroseconds'} / 1000  ) );
    #   }
    #   else{
    #     $data{'perf'}{'CAT'}{$ids}{'secureBytesRemovedThroughput'} = 0;
    #   }
    # }
    # else{
    #   $data{'perf'}{'CAT'}{$ids}{'secureBytesRemovedThroughput'} = "";
    # }

    # if (defined $data{'perf'}{'CAT'}{$ids}{'secureCompactedBytes'} && $data{'perf'}{'CAT'}{$ids}{'secureCompactedBytes'} ne "" && defined $data{'perf'}{'CAT'}{$ids}{'secureCompactedDurationMicroseconds'} && $data{'perf'}{'CAT'}{$ids}{'secureCompactedDurationMicroseconds'}   ne ""){
    #   if ( $data{'perf'}{'CAT'}{$ids}{'secureCompactedDurationMicroseconds'} != 0 ){
    #     $data{'perf'}{'CAT'}{$ids}{'secureBytesCompactedThroughput'} = ( ( $data{'perf'}{'CAT'}{$ids}{'secureCompactedBytes'} ) / ( $data{'perf'}{'CAT'}{$ids}{'secureCompactedDurationMicroseconds'} / 1000  ) );
    #   }
    #   else{
    #     $data{'perf'}{'CAT'}{$ids}{'secureBytesCompactedThroughput'} = 0;
    #   }
    # }
    # else{
    #   $data{'perf'}{'CAT'}{$ids}{'secureBytesCompactedThroughput'} = "";
    # }

    # if (defined $data{'perf'}{'CAT'}{$ids}{'secureDereferencedBytes'} && $data{'perf'}{'CAT'}{$ids}{'secureDereferencedBytes'} ne "" && defined $data{'perf'}{'CAT'}{$ids}{'secureDereferencedDurationMicroseconds'} && $data{'perf'}{'CAT'}{$ids}{'secureDereferencedDurationMicroseconds'}   ne ""){
    #   if ( $data{'perf'}{'CAT'}{$ids}{'secureDereferencedDurationMicroseconds'} != 0 ){
    #     $data{'perf'}{'CAT'}{$ids}{'secureBytesDereferencedThroughput'} = ( ( $data{'perf'}{'CAT'}{$ids}{'secureDereferencedBytes'} ) / ( $data{'perf'}{'CAT'}{$ids}{'secureDereferencedDurationMicroseconds'} / 1000  ) );
    #   }
    #   else{
    #     $data{'perf'}{'CAT'}{$ids}{'secureBytesDereferencedThroughput'} = 0;
    #   }
    # }else{
    #   $data{'perf'}{'CAT'}{$ids}{'secureBytesDereferencedThroughput'} = "";
    # }

    # if (defined $data{'perf'}{'CAT'}{$ids}{'dereferencedBytes'} && $data{'perf'}{'CAT'}{$ids}{'dereferencedBytes'} ne "" && defined $data{'perf'}{'CAT'}{$ids}{'dereferencedDurationMicroseconds'} && $data{'perf'}{'CAT'}{$ids}{'dereferencedDurationMicroseconds'}   ne ""){
    #   if ( $data{'perf'}{'CAT'}{$ids}{'dereferencedDurationMicroseconds'} != 0 ){
    #     $data{'perf'}{'CAT'}{$ids}{'BytesDereferencedThroughput'} = ( ( $data{'perf'}{'CAT'}{$ids}{'dereferencedBytes'} ) / ( $data{'perf'}{'CAT'}{$ids}{'dereferencedDurationMicroseconds'} / 1000  ) );
    #   }
    #   else{
    #     $data{'perf'}{'CAT'}{$ids}{'BytesDereferencedThroughput'} = 0;
    #   }
    # }
    # else{
    #   $data{'perf'}{'CAT'}{$ids}{'BytesDereferencedThroughput'} = "";
    # }

    #NETWORK

    # ###Copy job
    # if (defined $data{'perf'}{'CAT'}{$ids}{'outboundCopyjobNetworkThroughput'} && $data{'perf'}{'CAT'}{$ids}{'outboundCopyjobNetworkThroughput'} ne ""){
    #   $data{'perf'}{'CAT'}{$ids}{'outboundCopyjobNetworkThroughput'} = $data{'perf'}{'CAT'}{$ids}{'outboundCopyjobNetworkThroughput'};
    # }

    # if (defined $data{'perf'}{'CAT'}{$ids}{'inboundCopyjobNetworkThroughput'} && $data{'perf'}{'CAT'}{$ids}{'inboundCopyjobNetworkThroughput'} ne ""){
    #   $data{'perf'}{'CAT'}{$ids}{'inboundCopyjobNetworkThroughput'} = $data{'perf'}{'CAT'}{$ids}{'inboundCopyjobNetworkThroughput'};
    # }


    #Data job
    if (defined $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'} && $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'} ne ""){
      $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'} = $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'};
      if( defined $data{'pooldata'}{'read'} && Xormon::isdigit($data{'pooldata'}{'read'}) ){
        $data{'pooldata'}{'read'} += $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'};
      }
      else{
        $data{'pooldata'}{'read'} = $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'};
      }
    }

    if (defined $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'} && $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'} ne ""){
      $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'} = $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'};
      if( defined $data{'pooldata'}{'write'} && Xormon::isdigit($data{'pooldata'}{'write'}) ){
        $data{'pooldata'}{'write'} += $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'};
      }
      else{
        $data{'pooldata'}{'write'} = $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'};
      }
    }
    if (defined $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'} && $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'} ne "" && defined $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'} && $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'} ne ""){
      $data{'perf'}{'CAT'}{$ids}{'datarate'} = ( $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'} + $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'} );
      if( defined $data{'pooldata'}{'data_rate'} && Xormon::isdigit($data{'pooldata'}{'data_rate'}) ){
        $data{'pooldata'}{'data_rate'} += $data{'perf'}{'CAT'}{$ids}{'datarate'};
      }
      else{
        $data{'pooldata'}{'data_rate'} = $data{'perf'}{'CAT'}{$ids}{'datarate'};
      }
    }

    ### LOGICAL READ, WRITE, TOTAL
    my ($logical_copy_read, $logical_copy_write, $logical_data_read, $logical_data_write) = ("", "", "", "");

    ### Data job logical
    $logical_data_read  = $data{'perf'}{'CAT'}{$ids}{'outboundDatajobLogicalThroughput'} // "";
    $logical_data_write = $data{'perf'}{'CAT'}{$ids}{'inboundDatajobLogicalThroughput'} // "";
    ### Copy job logical
    $logical_copy_read  = $data{'perf'}{'CAT'}{$ids}{'outboundCopyjobLogicalThroughput'} // "";
    $logical_copy_write = $data{'perf'}{'CAT'}{$ids}{'inboundCopyjobLogicalThroughput'} // "";

    my ($logical_read, $logical_write, $logical) = ("", "", "");

    $logical_read  = $logical_copy_read if Xormon::isdigit($logical_copy_read);
    $logical_read += $logical_data_read if Xormon::isdigit($logical_data_read);

    $logical_write = $logical_copy_write if Xormon::isdigit($logical_copy_write);
    $logical_write += $logical_data_write if Xormon::isdigit($logical_data_write);

    if (Xormon::isdigit($logical_read)) {
      $logical = $logical_read;
      $logical += $logical_write if Xormon::isdigit($logical_write);
    } elsif (Xormon::isdigit($logical_write)) {
      $logical = $logical_write;
    }

    ### READ, WRITE, DATA_RATE
    my ($read, $write, $data_rate) = ("", "", "");

    ### DATA
    ### Data job
    my $data_read  = $data{'perf'}{'CAT'}{$ids}{'outboundDatajobNetworkThroughput'};
    my $data_write = $data{'perf'}{'CAT'}{$ids}{'inboundDatajobNetworkThroughput'};

    ### Copy job
    my $copy_read  = $data{'perf'}{'CAT'}{$ids}{'outboundCopyjobNetworkThroughput'};
    my $copy_write = $data{'perf'}{'CAT'}{$ids}{'inboundCopyjobNetworkThroughput'};

    #read
    $read = $copy_read if Xormon::isdigit($copy_read);
    $read += $data_read if Xormon::isdigit($data_read);

    #write
    $write = $copy_write if Xormon::isdigit($copy_write);
    $write += $data_write if Xormon::isdigit($data_write);

    #total
    if (Xormon::isdigit($read)) {
      $data_rate = $read;
      $data_rate += $write if Xormon::isdigit($write);
    } elsif (Xormon::isdigit($write)) {
      $data_rate = $write;
    }

    ### STREAM JOB RATE = stream_read_job + stream_write_job
    my $stream_job_rate = "";
    if (defined $data{'perf'}{'CAT'}{$ids}{'averageInboundCopyjobSessions'} && $data{'perf'}{'CAT'}{$ids}{'averageInboundCopyjobSessions'} ne "") {
      $stream_job_rate = $data{'perf'}{'CAT'}{$ids}{'averageInboundCopyjobSessions'};
    }
    if (defined $data{'perf'}{'CAT'}{$ids}{'averageOutboundCopyjobSessions'} && $data{'perf'}{'CAT'}{$ids}{'averageOutboundCopyjobSessions'} ne "") {
      $stream_job_rate += $data{'perf'}{'CAT'}{$ids}{'averageOutboundCopyjobSessions'};
    }

    #Capacity
    my $user_B         = "";
    my $disk_B         = "";
    my $disk_quota_B   = "";
    my $disk_prct        = "";
    my $usr_prct         = "";
    my $user_quota_B   = "";
    my $ratio            = "";
    my $service          = "CAT";

    if ( defined $data{'conf'}{$service}{$ids}{'userBytes'} && $data{'conf'}{$service}{$ids}{'userBytes'} ) {
      $user_B = sprintf( '%.2f',$data{'conf'}{$service}{$ids}{'userBytes'} );
    }
    if ( defined $data{'conf'}{$service}{$ids}{'diskBytes'} && $data{'conf'}{$service}{$ids}{'diskBytes'} ) {
      $disk_B = sprintf( '%.2f',$data{'conf'}{$service}{$ids}{'diskBytes'} );
    }
    if ( defined $data{'conf'}{$service}{$ids}{'sizeOnDiskQuotaBytes'} && $data{'conf'}{$service}{$ids}{'sizeOnDiskQuotaBytes'} ) {
      $disk_quota_B = sprintf( '%.2f', $data{'conf'}{$service}{$ids}{'sizeOnDiskQuotaBytes'} );
    }
    if ( defined $data{'conf'}{$service}{$ids}{'userDataStoredQuotaBytes'} && $data{'conf'}{$service}{$ids}{'userDataStoredQuotaBytes'} ) {
      $user_quota_B = sprintf( '%.2f', $data{'conf'}{$service}{$ids}{'userDataStoredQuotaBytes'} );
    }
    if ( defined $data{'conf'}{$service}{$ids}{'dedupeRatio'} && $data{'conf'}{$service}{$ids}{'dedupeRatio'} ) {
      $ratio = sprintf( '%.2f', $data{'conf'}{$service}{$ids}{'dedupeRatio'} );
    }
    if ( Xormon::isdigit( $disk_quota_B ) ){
      $disk_prct = sprintf( '%.0f',$disk_B / ( $disk_quota_B / 100 ) );
    }
    if ( Xormon::isdigit( $user_quota_B ) ){
      $usr_prct  = sprintf( '%.0f',$user_B / ( $user_quota_B / 100 ) );
    }
    my @metrics = ("outboundDatajobNetworkThroughput","inboundDatajobNetworkThroughput","outboundCopyjobNetworkThroughput","inboundCopyjobNetworkThroughput","userBytes","diskBytes","dedupeRatio","averageDataJobSessions","averageInboundCopyjobSessions","averageOutboundCopyjobSessions");

    my $i = 0;
    # my $test = $line;
    foreach ( @metrics ){
      if ( defined $data{'perf'}{'CAT'}{$ids}{$_} && $data{'perf'}{'CAT'}{$ids}{$_} ne "" ) {
        $line .= ";$data{'perf'}{'CAT'}{$ids}{$_}";
        # $test .= ";$_";
        if( defined $data{'pooldata'}{$CAT[$i]} && Xormon::isdigit($data{'pooldata'}{$CAT[$i]}) ){
          $data{'pooldata'}{$CAT[$i]} += $data{'perf'}{'CAT'}{$ids}{$_};
        }
        else{
          $data{'pooldata'}{$CAT[$i]} = $data{'perf'}{'CAT'}{$ids}{$_};
        }
      }
      else{
        $line .= ";";
        # $test .= ";";
        $data{'pooldata'}{$CAT[$i]} = "";
      }
      $i++;
    }
    # print "$test\n";
    $data_out .= "$line;$logical_read;$logical_write;$logical;$logical_copy_read;$logical_copy_write;$logical_data_read;$logical_data_write;$user_B;$disk_B;$disk_quota_B;$disk_prct;$usr_prct;$user_quota_B;$ratio;$read;$write;$data_rate;$stream_job_rate\n";
    # print $data_out;
  }

  #SAVE Catalyst";
  $rc = $xormon->saveCSV($data_out,"storeonce","cat");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }

  $logger->info("CATALYST status : $status");

  #### ETHPORT ####
  $data_out = "uuid;time;read;write;data_rate\n";
  foreach my $ids (keys ( %{ $data{'perf'}{'ETHPORT'} } ) ){
    if ($ids eq "UNSET"){
      next;
    }
    my $read = "";
    my $write = "";
    my $data_rate = "";
    my $type = "";
    if (defined $data{'perf'}{'ETHPORT'}{$ids}{'receiveRate'} && $data{'perf'}{'ETHPORT'}{$ids}{'receiveRate'} ne ""){
      $write = $data{'perf'}{'ETHPORT'}{$ids}{'receiveRate'};
    }
    if (defined $data{'perf'}{'ETHPORT'}{$ids}{'transmitRate'} && $data{'perf'}{'ETHPORT'}{$ids}{'transmitRate'} ne ""){
      $read =  $data{'perf'}{'ETHPORT'}{$ids}{'transmitRate'};
    }
    if (defined $data{'perf'}{'ETHPORT'}{$ids}{'transmitRate'} && $data{'perf'}{'ETHPORT'}{$ids}{'transmitRate'} ne "" && defined $data{'perf'}{'ETHPORT'}{$ids}{'receiveRate'} && $data{'perf'}{'ETHPORT'}{$ids}{'receiveRate'} ne ""){
      $data_rate = $read  + $write;
    }
    my $hash_key = Xormon::create_hash_key($st_serial."ETHPORT".$ids); $hash_key = lc($hash_key);

    $data_out .= "$hash_key;$perf_timestamp;$read;$write;$data_rate\n";
  }
  #SAVE ETHPORT";
  $rc = $xormon->saveCSV($data_out,"storeonce","ethport");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }

  $logger->info("ETHPORT status : $status");

  #### FCPORT ####
  $data_out = "uuid;time;read;write;data_rate\n";
  foreach my $ids (keys ( %{ $data{'perf'}{'FCPORT'} } ) ){
    if ($ids eq "UNSET"){
      next;
    }
    my $read = "";
    my $write = "";
    my $data_rate = "";
    my $type = "";
    if (defined $data{'perf'}{'FCPORT'}{$ids}{'receiveRate'} && $data{'perf'}{'FCPORT'}{$ids}{'receiveRate'} ne ""){
      $write = Xormon::formatNumberByType( "float4", $data{'perf'}{'FCPORT'}{$ids}{'receiveRate'} );
    }
    if (defined $data{'perf'}{'FCPORT'}{$ids}{'transmitRate'} && $data{'perf'}{'FCPORT'}{$ids}{'transmitRate'} ne ""){
      $read = Xormon::formatNumberByType( "float4", $data{'perf'}{'FCPORT'}{$ids}{'transmitRate'} );
    }
    if (defined $data{'perf'}{'FCPORT'}{$ids}{'transmitRate'} && $data{'perf'}{'FCPORT'}{$ids}{'transmitRate'} ne "" && defined $data{'perf'}{'FCPORT'}{$ids}{'receiveRate'} && $data{'perf'}{'FCPORT'}{$ids}{'receiveRate'} ne ""){
      $data_rate = Xormon::formatNumberByType("float4", ($read  + $write) );
    }
    my $hash_key = Xormon::create_hash_key($st_serial."FCPORT".$ids); $hash_key = lc($hash_key);

    $data_out .= "$hash_key;$perf_timestamp;$read;$write;$data_rate\n";
  }
  #SAVE FCPORT";
  $rc = $xormon->saveCSV($data_out,"storeonce","fcport");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }

  $logger->info("FCPORT status : $status");

  #### VTL ####
  # print Dumper \%data;
  # $data_out = "id;timestamp;read;write;data_rate;user_capacity_used;disks_size;dedupe_ratio;repTxBandwidth;write_replication;read_replication;repRxBandwidth;stream_read_replication;stream_write_replication;stream_rate;user_capacity_used;disk_size;disk_quota;disk_prct;usr_prct;user_quota;ratio\n";
  $data_out = "id;timestamp;read;write;data_rate;user_capacity_used;disk_size;dedupe_ratio;repTxBandwidth;write_replication;read_replication;repRxBandwidth;stream_read_replication;stream_write_replication;stream_rate\n";
  my @VTL = ("read","write","data_rate","User_data_store(MB)","Disk_size(MB)","deduperatio","repTxBandwidth","repTxThroughput","repRxThroughput","repRxBandwidth","repInSessionCount","repOutSessionCount","openCartridgesCount");
  foreach my $ids (keys ( %{$data{'perf'}{'VTL'}} ) ){
    my $hash_key = Xormon::create_hash_key($st_serial."VTL".$ids); $hash_key = lc($hash_key);
    my $line = "$hash_key;$perf_timestamp";

    # if (defined $data{'perf'}{'VTL'}{$ids}{'readThroughput'} && $data{'perf'}{'VTL'}{$ids}{'readThroughput'} ne ""){
    #   $data{'perf'}{'VTL'}{$ids}{'readThroughput'} = ( ($data{'perf'}{'VTL'}{$ids}{'readThroughput'}) );
    # }
    # if (defined $data{'perf'}{'VTL'}{$ids}{'writeThroughput'} && $data{'perf'}{'VTL'}{$ids}{'writeThroughput'} ne ""){
    #   $data{'perf'}{'VTL'}{$ids}{'writeThroughput'} = ( ($data{'perf'}{'VTL'}{$ids}{'writeThroughput'}) );
    # }
    if (defined $data{'perf'}{'VTL'}{$ids}{'readThroughput'} && $data{'perf'}{'VTL'}{$ids}{'readThroughput'} ne "" && defined $data{'perf'}{'VTL'}{$ids}{'writeThroughput'} && $data{'perf'}{'VTL'}{$ids}{'writeThroughput'} ne ""){
      $data{'perf'}{'VTL'}{$ids}{'datarate'} = ( $data{'perf'}{'VTL'}{$ids}{'readThroughput'} + $data{'perf'}{'VTL'}{$ids}{'writeThroughput'} );
    }
    # if (defined $data{'perf'}{'VTL'}{$ids}{'userBytes'} && $data{'perf'}{'VTL'}{$ids}{'userBytes'} ne ""){
    #   $data{'perf'}{'VTL'}{$ids}{'userBytes'} = ( $data{'perf'}{'VTL'}{$ids}{'userBytes'} );
    # }
    # if (defined $data{'perf'}{'VTL'}{$ids}{'diskBytes'} && $data{'perf'}{'VTL'}{$ids}{'diskBytes'} ne ""){
    #   $data{'perf'}{'VTL'}{$ids}{'diskBytes'} = ( $data{'perf'}{'VTL'}{$ids}{'diskBytes'} );
    # }
    # if (defined $data{'perf'}{'VTL'}{$ids}{'averageTransmittedBandwidth'} && $data{'perf'}{'VTL'}{$ids}{'averageTransmittedBandwidth'} ne ""){
    #   $data{'perf'}{'VTL'}{$ids}{'averageTransmittedBandwidth'} = ( $data{'perf'}{'VTL'}{$ids}{'averageTransmittedBandwidth'} );
    # }
    # if (defined $data{'perf'}{'VTL'}{$ids}{'averageTransmittedThroughput'} && $data{'perf'}{'VTL'}{$ids}{'averageTransmittedThroughput'} ne ""){
    #   $data{'perf'}{'VTL'}{$ids}{'averageTransmittedThroughput'} = ( $data{'perf'}{'VTL'}{$ids}{'averageTransmittedThroughput'} );
    # }
    # if (defined $data{'perf'}{'VTL'}{$ids}{'averageReceivedThroughput'} && $data{'perf'}{'VTL'}{$ids}{'averageReceivedThroughput'} ne ""){
    #   $data{'perf'}{'VTL'}{$ids}{'averageReceivedThroughput'} = ( $data{'perf'}{'VTL'}{$ids}{'averageReceivedThroughput'} );
    # }
    # if (defined $data{'perf'}{'VTL'}{$ids}{'averageReceivedBandwidth'} && $data{'perf'}{'VTL'}{$ids}{'averageReceivedBandwidth'} ne ""){
    #   $data{'perf'}{'VTL'}{$ids}{'averageReceivedBandwidth'} = ( $data{'perf'}{'VTL'}{$ids}{'averageReceivedBandwidth'} );
    # }

    #Capacity
    # my $user_B         = "";
    # my $disk_B         = "";
    # my $disk_quota_B   = "";
    # my $disk_prct        = "";
    # my $usr_prct         = "";
    # my $user_quota_B   = "";
    # my $ratio            = "";
    # my $service          = "VTL";

    # print Dumper $data{'conf'}{'VTL'};

    # if ( defined $data{'conf'}{$service}{$ids}{'userBytes'} && $data{'conf'}{$service}{$ids}{'userBytes'} ) {
    #   $user_B = sprintf( '%.2f',$data{'conf'}{$service}{$ids}{'userBytes'} );
    # }
    # if ( defined $data{'conf'}{$service}{$ids}{'diskBytes'} && $data{'conf'}{$service}{$ids}{'diskBytes'} ) {
    #   $disk_B = sprintf( '%.2f',$data{'conf'}{$service}{$ids}{'diskBytes'} );
    # }
    # if ( defined $data{'conf'}{$service}{$ids}{'sizeOnDiskQuotaBytes'} && $data{'conf'}{$service}{$ids}{'sizeOnDiskQuotaBytes'} ) {
    #   $disk_quota_B = sprintf( '%.2f', $data{'conf'}{$service}{$ids}{'sizeOnDiskQuotaBytes'} );
    # }
    # if ( defined $data{'conf'}{$service}{$ids}{'userDataStoredQuotaBytes'} && $data{'conf'}{$service}{$ids}{'userDataStoredQuotaBytes'} ) {
    #   $user_quota_B = sprintf( '%.2f', $data{'conf'}{$service}{$ids}{'userDataStoredQuotaBytes'} );
    # }
    # if ( defined $data{'conf'}{$service}{$ids}{'dedupeRatio'} && $data{'conf'}{$service}{$ids}{'dedupeRatio'} ) {
    #   $ratio = sprintf( '%.2f', $data{'conf'}{$service}{$ids}{'dedupeRatio'} );
    # }

    # if ( Xormon::isdigit( $disk_quota_B ) ){
    #   $disk_prct = sprintf( '%.0f',$disk_B / ( $disk_quota_B / 100 ) );
    # }

    # if ( Xormon::isdigit( $user_quota_B ) ){
    #   $usr_prct  = sprintf( '%.0f',$user_B / ( $user_quota_B / 100 ) );
    # }


    my @metrics = ("readThroughput","writeThroughput","datarate","userBytes","diskBytes","dedupeRatio","averageTransmittedBandwidth","averageTransmittedThroughput","averageReceivedThroughput","averageReceivedBandwidth","averageSourceSessions","averageTargetSessions","averageOpenCartridges");

    my $i = 0;
    foreach ( @metrics ){
      if ( defined $data{'perf'}{'VTL'}{$ids}{$_} && $data{'perf'}{'VTL'}{$ids}{$_} ne "" ) {
        my $var = sprintf('%.0f' , $data{'perf'}{'VTL'}{$ids}{$_});
        $line .= ";$var";
        if( defined $data{'pooldata'}{$VTL[$i]} && Xormon::isdigit($data{'pooldata'}{$VTL[$i]}) ){
          $data{'pooldata'}{$VTL[$i]} += $data{'perf'}{'VTL'}{$ids}{$_};
        }
        else{
          $data{'pooldata'}{$VTL[$i]} = $data{'perf'}{'VTL'}{$ids}{$_};
        }
      }
      else{
        $line .= ";";
        $data{'pooldata'}{$VTL[$i]} = "";
      }
      $i++;
    }
    $data_out .= "$line\n";
  }
  # print $data_out;

  #Save VTL
  $rc = $xormon->saveCSV($data_out,"storeonce","vtl");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }

  $logger->info("VTL status : $status");


  #### CPU ####
  $data_out = "id;timestamp;cpu_system\n";
  my $cpuUsage = "";
  if( defined $data{'perf'}{'CPU'}{'percentageIoWait'} && Xormon::isdigit($data{'perf'}{'CPU'}{'percentageIoWait'}) && defined $data{'perf'}{'CPU'}{'percentageCpuUsage'} && Xormon::isdigit($data{'perf'}{'CPU'}{'percentageCpuUsage'} ) ){
    $cpuUsage = (sprintf ('%.0f', $data{'perf'}{'CPU'}{'percentageIoWait'} + $data{'perf'}{'CPU'}{'percentageCpuUsage'} ) );
  }
  #print DATA "1,$cfg->{'end_time'},$cpuUsage\n";
  my $hash_key = Xormon::create_hash_key($st_serial."CPU"); $hash_key = lc($hash_key);
  $data_out .= "$hash_key;$perf_timestamp;$cpuUsage\n";

  #Save CPU
  $rc = $xormon->saveCSV($data_out,"storeonce","cpu");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }

  $logger->info("CPU status : $status");

  #### NAS ####
  $data_out = "id;timestamp;read;write;data_rate;user_capacity_used;disk_size;dedupe_ratio;repTxBandwidth;write_replication;read_replication;repRxBandwidth;stream_read_replication;stream_write_replication;stream_rate\n";
  my @NAS = ("read","write","data_rate","User_data_store(MB)","Disk_size(MB)","deduperatio","repTxBandwidth","repTxThroughput","repRxThroughput","repRxBandwidth","repInSessionCount","repOutSessionCount","openCartridgesCount");
  foreach my $ids (keys ( %{$data{'perf'}{'NAS'}} ) ){
    my $hash_key = Xormon::create_hash_key($st_serial."NAS".$ids); $hash_key = lc($hash_key);
    my $line = "$hash_key;$perf_timestamp";
    if ( defined $data{'perf'}{'NAS'}{$ids}{'receivedThroughput'} && $data{'perf'}{'NAS'}{$ids}{'receivedThroughput'} ne "" && defined $data{'perf'}{'NAS'}{$ids}{'transmittedThroughput'} && $data{'perf'}{'NAS'}{$ids}{'transmittedThroughput'} ne "" ) {
      $data{'perf'}{'NAS'}{$ids}{'datarate'} = ( $data{'perf'}{'NAS'}{$ids}{'receivedThroughput'} + $data{'perf'}{'NAS'}{$ids}{'transmittedThroughput'} );
    }
    # if ( defined $data{'perf'}{'NAS'}{$ids}{'userBytes'} && $data{'perf'}{'NAS'}{$ids}{'userBytes'} ne "" ) {
    #   $data{'perf'}{'NAS'}{$ids}{'userBytes'} = ( $data{'perf'}{'NAS'}{$ids}{'userBytes'} );
    # }
    # if ( defined $data{'perf'}{'NAS'}{$ids}{'diskBytes'} && $data{'perf'}{'NAS'}{$ids}{'diskBytes'} ne "" ) {
    #   $data{'perf'}{'NAS'}{$ids}{'diskBytes'} = ( $data{'perf'}{'NAS'}{$ids}{'diskBytes'} )
    # }
    # if ( defined $data{'perf'}{'NAS'}{$ids}{'averageTransmittedBandwidth'} && $data{'perf'}{'NAS'}{$ids}{'averageTransmittedBandwidth'} ne "" ){
    #   $data{'perf'}{'NAS'}{$ids}{'averageTransmittedBandwidth'} = ( $data{'perf'}{'NAS'}{$ids}{'averageTransmittedBandwidth'} );
    # }
    # if ( defined $data{'perf'}{'NAS'}{$ids}{'averageTransmittedThroughput'} && $data{'perf'}{'NAS'}{$ids}{'averageTransmittedThroughput'} ne "" ){
    #   $data{'perf'}{'NAS'}{$ids}{'averageTransmittedThroughput'} = ( $data{'perf'}{'NAS'}{$ids}{'averageTransmittedThroughput'} );
    # }
    # if ( defined $data{'perf'}{'NAS'}{$ids}{'averageReceivedThroughput'} && $data{'perf'}{'NAS'}{$ids}{'averageReceivedThroughput'} ne "" ){
    #   $data{'perf'}{'NAS'}{$ids}{'averageReceivedThroughput'} = ( $data{'perf'}{'NAS'}{$ids}{'averageReceivedThroughput'} );
    # }
    # if ( defined $data{'perf'}{'NAS'}{$ids}{'averageReceivedBandwidth'} && $data{'perf'}{'NAS'}{$ids}{'averageReceivedBandwidth'} ne "" ){
    #   $data{'perf'}{'NAS'}{$ids}{'averageReceivedBandwidth'} = ( $data{'perf'}{'NAS'}{$ids}{'averageReceivedBandwidth'} );
    # }

    # #Capacity
    # my $user_B         = "";
    # my $disk_B         = "";
    # my $disk_quota_B   = "";
    # my $disk_prct        = "";
    # my $usr_prct         = "";
    # my $user_quota_B   = "";
    # my $ratio            = "";
    # my $service          = "NAS";

    # if ( defined $data{'conf'}{$service}{$ids}{'userBytes'} && $data{'conf'}{$service}{$ids}{'userBytes'} ) {
    #   $user_B = sprintf( '%.2f',$data{'conf'}{$service}{$ids}{'userBytes'} );
    # }
    # if ( defined $data{'conf'}{$service}{$ids}{'diskBytes'} && $data{'conf'}{$service}{$ids}{'diskBytes'} ) {
    #   $disk_B = sprintf( '%.2f',$data{'conf'}{$service}{$ids}{'diskBytes'} );
    # }
    # if ( defined $data{'conf'}{$service}{$ids}{'sizeOnDiskQuotaBytes'} && $data{'conf'}{$service}{$ids}{'sizeOnDiskQuotaBytes'} ) {
    #   $disk_quota_B = sprintf( '%.2f', $data{'conf'}{$service}{$ids}{'sizeOnDiskQuotaBytes'} );
    # }
    # if ( defined $data{'conf'}{$service}{$ids}{'userDataStoredQuotaBytes'} && $data{'conf'}{$service}{$ids}{'userDataStoredQuotaBytes'} ) {
    #   $user_quota_B = sprintf( '%.2f', $data{'conf'}{$service}{$ids}{'userDataStoredQuotaBytes'} );
    # }
    # if ( defined $data{'conf'}{$service}{$ids}{'dedupeRatio'} && $data{'conf'}{$service}{$ids}{'dedupeRatio'} ) {
    #   $ratio = sprintf( '%.2f', $data{'conf'}{$service}{$ids}{'dedupeRatio'} );
    # }

    # if ( Xormon::isdigit( $disk_quota_B ) ){
    #   $disk_prct = sprintf( '%.0f',$disk_B / ( $disk_quota_B / 100 ) );
    # }

    # if ( Xormon::isdigit( $user_quota_B ) ){
    #   $usr_prct  = sprintf( '%.0f',$user_B / ( $user_quota_B / 100 ) );
    # }


    my @metrics = ("transmittedThroughput","receivedThroughput","datarate","userBytes","diskBytes","dedupeRatio","averageTransmittedBandwidth","averageTransmittedThroughput","averageReceivedThroughput","averageReceivedBandwidth","numDedupeSessions","averageSourceSessions","averageTargetSessions");
    # my @NAS = ("read","write","data_rate","User_data_store(MB)","Disk_size(MB)","deduperatio","repTxBandwidth","repTxThroughput","repRxThroughput","repRxBandwidth","cluster_id","repInSessionCount","repOutSessionCount","openCartridgesCount");
    my $i = 0;
    foreach ( @metrics ){
      if ( defined $data{'perf'}{'NAS'}{$ids}{$_} && $data{'perf'}{'NAS'}{$ids}{$_} ne "" ) {
        my $var = sprintf('%.0f',$data{'perf'}{'NAS'}{$ids}{$_});
        $line .= ";$var";
        if( defined $data{'pooldata'}{$NAS[$i]} && Xormon::isdigit($data{'pooldata'}{$NAS[$i]}) ){
          $data{'pooldata'}{$NAS[$i]} += $data{'perf'}{'NAS'}{$ids}{$_};
        }
        else{
          $data{'pooldata'}{$NAS[$i]} = $data{'perf'}{'NAS'}{$ids}{$_};
        }
      }
      else{
        $line .= ";";
        $data{'pooldata'}{$NAS[$i]} = "";
      }
      $i++;
    }

    # $data_out .= "$line;$user_B;$disk_B;$disk_quota_B;$disk_prct;$usr_prct;$user_quota_B;$ratio\n";
    $data_out .= "$line\n";
  }

  #SAVE NAS
  $rc = $xormon->saveCSV($data_out,"storeonce","nas");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("NAS status : $status");

  # print Dumper $data{'pooldata'};

  #### Pool ####
  $data_out = "id;timestamp;read;write;data_rate;user_data_store;disk_size;write_replication;read_replication;copy_job_read;copy_job_write;data_job_read;data_job_write;capacity_total;capacity_used;capacity_used_percent;capacity_free;dedupe_ratio;housekeep_status;housekeep_pend_data\n";
  my @POOL = ("read","write","data_rate","User_data_store(B)","Disk_size(B)","repTxThroughput","repRxThroughput","copyJobTxThroughput","copyJobRxThroughput","dataJobTxThroughput","dataJobRxThroughput","Capacity(B)","Used_capacity(B)","capacity_used_percent","Free_capacity(B)","dedupeRatio","housekeepingStatus","housekeepingPending");
  $hash_key = Xormon::create_hash_key($st_serial."POOL"."1"); $hash_key = lc($hash_key);
  my $line = "$hash_key;$perf_timestamp";
  if (defined $data{'pooldata'}{'User_data_store(MB)'} && $data{'pooldata'}{'User_data_store(MB)'} ne ""){
    #$data{'pooldata'}{'User_data_store(B)'} = sprintf('%.3f', $data{'pooldata'}{'User_data_store(MB)'} * 1024 * 1024);
    $data{'pooldata'}{'User_data_store(B)'}  = Xormon::returnBytes($data{'pooldata'}{'User_data_store(MB)'},"MB");
  }
  if (defined $data{'pooldata'}{'Disk_size(MB)'} && $data{'pooldata'}{'Disk_size(MB)'} ne ""){
    #$data{'pooldata'}{'Disk_size(B)'} =  sprintf('%.3f',$data{'pooldata'}{'Disk_size(MB)'} / 1024 / 1024);
    $data{'pooldata'}{'Disk_size(B)'} = Xormon::returnBytes($data{'pooldata'}{'Disk_size(MB)'},"MB");
  }
  if (defined $data{'conf'}{'storagestats'}{'localCapacityBytes'} && $data{'conf'}{'storagestats'}{'localCapacityBytes'} ne "" ){
    $data{'pooldata'}{'Capacity(B)'} = $data{'conf'}{'storagestats'}{'localCapacityBytes'};
  }
  if (defined $data{'conf'}{'storagestats'}{'localDiskBytes'} && $data{'conf'}{'storagestats'}{'localDiskBytes'} ne "" ){
    $data{'pooldata'}{'Used_capacity(B)'} =$data{'conf'}{'storagestats'}{'localDiskBytes'};
  }
  if (defined $data{'conf'}{'storagestats'}{'localFreeBytes'} && $data{'conf'}{'storagestats'}{'localFreeBytes'} ne "" ){
    $data{'pooldata'}{'Free_capacity(B)'} = $data{'conf'}{'storagestats'}{'localFreeBytes'};
  }
  if ( Xormon::isdigit($data{'conf'}{'storagestats'}{'localCapacityBytes'}) && Xormon::isdigit($data{'conf'}{'storagestats'}{'localDiskBytes'}) && ($data{'conf'}{'storagestats'}{'localCapacityBytes'} != 0) ){
    $data{'pooldata'}{'capacity_used_percent'} = Xormon::formatNumberByType("float4", $data{'conf'}{'storagestats'}{'localDiskBytes'} / ($data{'conf'}{'storagestats'}{'localCapacityBytes'} / 100) );
  }
  if ( Xormon::isdigit($data{'pooldata'}{'housekeepingPending'}) ) {
    $data{'pooldata'}{'housekeepingPending'} = $data{'pooldata'}{'housekeepingPending'};
  }
  foreach (@POOL){
    if (defined $data{'pooldata'}{$_} && defined $data{'pooldata'}{$_}){
      $line .= ";$data{'pooldata'}{$_}";
    }
    else{
      $line .= ";";
    }
  }

  $data_out .= "$line\n";

  # print $data_out;
  # print "read,write,data_rate,User_data_store(B),Disk_size(B),repTxThroughput,repRxThroughput,copyJobTxThroughput,copyJobRxThroughput,dataJobTxThroughput,dataJobRxThroughput,Capacity(B),Used_capacity(B),capacity_used_percent,Free_capacity(B),dedupeRatio,housekeepingStatus,housekeepingPending\n";
  # print Dumper $data{'pooldata'};

  #SAVE POOL
  $rc = $xormon->saveCSV($data_out,"storeonce","pool");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("POOL status : $status");

  #### DEVICE ####
  $data_out = "id;timestamp;capacity_total;capacity_free;capacity_used;read;write;data_rate;capacity_used_percent\n";
  my $total     = "";
  my $free      = "";
  my $used      = "";
  my $used_pct  = "";
  my $read      = "";
  my $write     = "";
  my $data_rate = "";
  if (exists $data{'pooldata'}{'Free_capacity(B)'}){
    $free = $data{'pooldata'}{'Free_capacity(B)'};
  }
  if (exists $data{'pooldata'}{'Capacity(B)'} ){
    $total = $data{'pooldata'}{'Capacity(B)'};
  }
  if (exists $data{'pooldata'}{'Used_capacity(B)'} ){
    $used = $data{'pooldata'}{'Used_capacity(B)'};
  }
  if (exists $data{'pooldata'}{'read'} ){
    $read = $data{'pooldata'}{'read'};
  }
  if (exists $data{'pooldata'}{'write'} ){
    $write = $data{'pooldata'}{'write'};
  }
  if (exists $data{'pooldata'}{'data_rate'} ){
    $data_rate = $data{'pooldata'}{'data_rate'};
  }
  if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
    $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
  }

  $data_out .= "$cfg->{'hostcfg_id'};$perf_timestamp;$total;$free;$used;$read;$write;$data_rate;$used_pct\n";

  #### SAVE DEVICE ####
  $rc = $xormon->saveCSV($data_out,"storeonce","device");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("DEVICE status : $status");

  # print Dumper \%data;
}

sub print_conf {
  my @architecture;
  my $st_serial = $cfg->{'hostcfg_id'};
  my %conf;

  my $uuid     = "";
  my $name     = "";
  my $hostname = "";
  my $serial   = "";
  my $api      = "";
  my $version  = "";
  if (defined $data{'conf'}{'storagestats'}{'uuid'} && $data{'conf'}{'storagestats'}{'uuid'} ne ""){
    $uuid = $data{'conf'}{'storagestats'}{'uuid'};
  }
  if (defined $data{'conf'}{'storagedata'}{$uuid}{'productName'} && $data{'conf'}{'storagedata'}{$uuid}{'productName'} ne ""){
    $conf{ $cfg->{'hostcfg_id'} }{'model'} = $data{'conf'}{'storagedata'}{$uuid}{'productName'};
  }
  if (defined $data{'conf'}{'storagedata'}{$uuid}{'serialNumber'} && $data{'conf'}{'storagedata'}{$uuid}{'serialNumber'} ne ""){
    $conf{ $cfg->{'hostcfg_id'} }{'serial'} = $data{'conf'}{'storagedata'}{$uuid}{'serialNumber'};
  }
  if (defined $data{'conf'}{'storagedata'}{$uuid}{'federationApiVersion'} && $data{'conf'}{'storagedata'}{$uuid}{'federationApiVersion'} ne ""){
    $api = $data{'conf'}{'storagedata'}{$uuid}{'federationApiVersion'};
  }
  if (defined $data{'conf'}{'storagestats'}{'softwareVersion'} && $data{'conf'}{'storagestats'}{'softwareVersion'} ne ""){
    $conf{ $cfg->{'hostcfg_id'} }{'version'} = $data{'conf'}{'storagestats'}{'softwareVersion'};
  }
  if (defined $data{'conf'}{'storagedata'}{$uuid}{'hostname'} && $data{'conf'}{'storagedata'}{$uuid}{'hostname'} ne ""){
    $hostname = $data{'conf'}{'storagedata'}{$uuid}{'hostname'};
  }

  #SYSTEM STORAGE CONFIGURATION";
  my %device_arch = (
    "item_id" => $cfg->{'hostcfg_id'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => "storeonce",
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'hostcfg_id'}
  );

  push(@architecture, \%device_arch);

  if (defined $data{'conf'}{'storagestats'}{'localCapacityBytes'} && $data{'conf'}{'storagestats'}{'localCapacityBytes'} ne "" ){
    $conf{ $cfg->{'hostcfg_id'} }{'capacity_total'} = Xormon::formatNumberByType( "float4", $data{'conf'}{'storagestats'}{'localCapacityBytes'} );
  }
  if (defined $data{'conf'}{'storagestats'}{'localDiskBytes'} && $data{'conf'}{'storagestats'}{'localDiskBytes'} ne "" ){
    $conf{ $cfg->{'hostcfg_id'} }{'capacity_used'} = Xormon::formatNumberByType( "float4", $data{'conf'}{'storagestats'}{'localDiskBytes'} );
  }
  if (defined $data{'conf'}{'storagestats'}{'localFreeBytes'} && $data{'conf'}{'storagestats'}{'localFreeBytes'} ne "" ){
    $conf{ $cfg->{'hostcfg_id'} }{'capacity_free'} = Xormon::formatNumberByType( "float4", $data{'conf'}{'storagestats'}{'localFreeBytes'} );
  }
  $conf{ $cfg->{'hostcfg_id'} }{'ip'} = $cfg->{'apiHost'};


  #VTL CONFIGURATION";
  foreach my $ids (keys(%{$data{'conf'}{'VTL'}})){
    my $name = "";
    my $status = "";
    my $cluster_id = $uuid;

    if (defined $data{'conf'}{'VTL'}{$ids}{'name'} && $data{'conf'}{'VTL'}{$ids}{'name'} ne ""){
      $name = $data{'conf'}{'VTL'}{$ids}{'name'};
    }
    if (defined $data{'conf'}{'VTL'}{$ids}{'healthLevelString'} && $data{'conf'}{'VTL'}{$ids}{'healthLevelString'} ne ""){
      $status = $data{'conf'}{'VTL'}{$ids}{'healthLevelString'};
    }

    my $hash_key = Xormon::create_hash_key($st_serial."VTL".$ids); $hash_key = lc($hash_key);
    my %arch = (
        "item_id" => $hash_key,
        "label" => $name,
        "hw_type" => "storeonce",
        "subsystem" => "vtl",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );

    push(@architecture, \%arch);
  }

  # ETHPORT CONFIGURATION
  foreach my $ids (keys ( %{ $data{'perf'}{'ETHPORT'} } ) ){
    if ($ids eq "UNSET"){
      next;
    }
    #print CONF "1-$ids,$ids,,,,,,,,,,$type,,\n";
    my $hash_key = Xormon::create_hash_key($st_serial."ETHPORT".$ids); $hash_key = lc($hash_key);
    my $name = $ids;
    my %arch = (
        "item_id" => $hash_key,
        "label" => $name,
        "hw_type" => "storeonce",
        "subsystem" => "ethport",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );

    push(@architecture, \%arch);
  }

  # FCPORT CONFIGURATION
  foreach my $ids (keys ( %{ $data{'perf'}{'FCPORT'} } ) ){
    if ($ids eq "UNSET"){
      next;
    }
    #print CONF "1-$ids,$ids,,,,,,,,,,$type,,\n";
    my $hash_key = Xormon::create_hash_key($st_serial."FCPORT".$ids); $hash_key = lc($hash_key);
    my $name = $ids;
    my %arch = (
        "item_id" => $hash_key,
        "label" => $name,
        "hw_type" => "storeonce",
        "subsystem" => "fcport",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );

    push(@architecture, \%arch);
  }

  # foreach my $type (keys ( %{ $data{'perf'}{'PORT_TYPE'} } ) ){
  #   my $hash_key = Xormon::create_hash_key($st_serial."PORT_TYPE".$type); $hash_key = lc($hash_key);
  #   my $name = "$type";
  #   my %arch = (
  #       "item_id" => $hash_key,
  #       "label" => $name,
  #       "hw_type" => "storeonce",
  #       "subsystem" => "port_type",
  #       "hostcfg_id" => "$cfg->{'hostcfg_id'}",
  #       "parents" => [$cfg->{'hostcfg_id'}]
  #   );

  #   push(@architecture, \%arch);
  # }

  #POOL CONFIGURATION";
  my $capacity = "";
  my $free     = "";
  my $used     = "";
  my $user     = "";
  my $disk     = "";
  if (defined $data{'pooldata'}{'User_data_store(TB)'} && $data{'pooldata'}{'User_data_store(TB)'} ne ""){
    $user = sprintf('%.3f' , $data{'pooldata'}{'User_data_store(TB)'} * 1024);
  }
  if (defined $data{'pooldata'}{'Disk_size(TB)'} && $data{'pooldata'}{'Disk_size(TB)'} ne ""){
    $disk = sprintf('%.3f' , $data{'pooldata'}{'Disk_size(TB)'} * 1024);
  }
  if (defined $data{'pooldata'}{'Capacity(TB)'} && $data{'pooldata'}{'Capacity(TB)'} ne ""){
    $capacity = sprintf('%.3f' , $data{'pooldata'}{'Capacity(TB)'} * 1024);
  }
  if (defined $data{'pooldata'}{'Used_capacity(TB)'} && $data{'pooldata'}{'Used_capacity(TB)'} ne ""){
    $used = sprintf('%.3f' , $data{'pooldata'}{'Used_capacity(TB)'} * 1024);
  }
  if (defined $data{'pooldata'}{'Free_capacity(TB)'} && $data{'pooldata'}{'Free_capacity(TB)'} ne ""){
    $free = sprintf('%.3f' , $data{'pooldata'}{'Free_capacity(TB)'} * 1024);
  }

  my $hash_key = Xormon::create_hash_key($st_serial."POOL"."1"); $hash_key = lc($hash_key);
  my %pool_arch = (
        "item_id" => $hash_key,
        "label" => "Service Set 1",
        "hw_type" => "storeonce",
        "subsystem" => "pool",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents" => [$cfg->{'hostcfg_id'}]
      );

  push(@architecture, \%pool_arch);

  #NAS CONFIGURATION";
  foreach my $ids (keys(%{$data{'conf'}{'NAS'}})){
    my $name = "";
    my $status = "";
    my $cluster_id = $uuid;
    if (defined $data{'conf'}{'NAS'}{$ids}{'name'} && $data{'conf'}{'NAS'}{$ids}{'name'} ne ""){
      $name = $data{'conf'}{'NAS'}{$ids}{'name'};
    }
    if (defined $data{'conf'}{'NAS'}{$ids}{'healthLevelString'} && $data{'conf'}{'NAS'}{$ids}{'healthLevelString'} ne ""){
      $status = $data{'conf'}{'NAS'}{$ids}{'healthLevelString'};
    }

    my $hash_key = Xormon::create_hash_key($st_serial."NAS".$ids); $hash_key = lc($hash_key);
    my %arch = (
        "item_id" => $hash_key,
        "label" => "$name",
        "hw_type" => "storeonce",
        "subsystem" => "nas",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );
    push(@architecture, \%arch);
  }

  #DRIVE CONFIGURATION";
  my $id = 1;
  $name = "node1";
  $hash_key = Xormon::create_hash_key($st_serial."Drive".$id); $hash_key = lc($hash_key);
  my %drive_arch = (
        "item_id" => $hash_key,
        "label" => "$name",
        "hw_type" => "storeonce",
        "subsystem" => "drive",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents" => [$cfg->{'hostcfg_id'}]
  );
  push(@architecture, \%drive_arch);

  #CATALYST CONFIGURATION";
  foreach my $ids (keys(%{$data{'conf'}{'CAT'}})){
    my $name = "";
    my $status = "";
    my $cluster_id = $uuid;
    if (defined $data{'conf'}{'CAT'}{$ids}{'name'} && $data{'conf'}{'CAT'}{$ids}{'name'} ne ""){
      $name = $data{'conf'}{'CAT'}{$ids}{'name'};
    }
    if (defined $data{'conf'}{'CAT'}{$ids}{'healthLevelString'} && $data{'conf'}{'CAT'}{$ids}{'healthLevelString'} ne ""){
      $status = $data{'conf'}{'CAT'}{$ids}{'healthLevelString'};
    }

    my $hash_key = Xormon::create_hash_key($st_serial."CAT".$ids); $hash_key = lc($hash_key);
    my %arch = (
        "item_id" => $hash_key,
        "label" => "$name",
        "hw_type" => "storeonce",
        "subsystem" => "cat",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );
    push(@architecture, \%arch);
  }

  #CPU NODE CONFIGURATION";
  $hash_key = Xormon::create_hash_key($st_serial."CPU"); $hash_key = lc($hash_key);
  my %cpu_arch = (
        "item_id" => $hash_key,
        "label" => "node1",
        "hw_type" => "storeonce",
        "subsystem" => "cpu",
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "parents" => [$cfg->{'hostcfg_id'}]
      );
  push(@architecture, \%cpu_arch);

  $xormon->saveArchitecture(\@architecture);
  $xormon->saveConfJSON(\%conf);
}

sub new_get_request {
  my $url = shift;
  my $result = "";

  my $agent = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( GET  => $url );
  $request->header(
    'Accept' => 'application/json',
    'Authorization' => 'Bearer '. $cfg->{'token'} . ''
  );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    $logger->error("GET url           : $url");
    $logger->error("Request error     : $st_line");

    return 0;
  }

  if (defined $results->content && $results->content ne ""){
    my $result = decode_json($results->content);
    return $result;
  }

  return 0;
}

sub get_time_utc {
  my $timestamp_end = shift;
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $time_end = "$year-$month-$day" . "T" . "$hour:$minute:$sec" . "Z";
  return $time_end;
}

sub health_status {

  my %hs_data;

  my $error_health  = 0;
  my $warn_health   = 0;
  my $report        = "";
  my $app_st        = "";
  my $nas_status    = "";
  my $cat_status    = "";
  my $vtl_status    = "";
  my @error_message = ();

  if ( defined $data{'conf'}{'storagedata'}{'statusNas'} && $data{'conf'}{'storagedata'}{'statusNas'} ne "" ) {
    $nas_status = $data{'conf'}{'storagedata'}{'statusNas'};
    my %nas_hash = ( service => 'NAS', status => $nas_status );
    push(@{$hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{services}}, \%nas_hash);
  }
  if ( defined $data{'conf'}{'storagedata'}{'statusCat'} && $data{'conf'}{'storagedata'}{'statusCat'} ne "" ) {
    $cat_status = $data{'conf'}{'storagedata'}{'statusCat'};
    my %cat_hash = ( service => 'Catalyst', status => $cat_status );
    push(@{$hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{services}}, \%cat_hash);
  }
  if ( defined $data{'conf'}{'storagedata'}{'statusVtl'} && $data{'conf'}{'storagedata'}{'statusVtl'} ne "" ) {
    $vtl_status = $data{'conf'}{'storagedata'}{'statusVtl'};
    my %vtl_hash = ( service => 'Virtual Tape Library', status => $vtl_status );
    push(@{$hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{services}}, \%vtl_hash);
  }

  if ( $nas_status eq "CRITICAL" || $cat_status eq "CRITICAL" || $vtl_status eq "CRITICAL" ) {
    $error_health++;
  }
  elsif ( $nas_status eq "WARNING" || $cat_status eq "WARNING" || $vtl_status eq "WARNING" ) {
    $warn_health++;
  }

  if ( exists $data{'conf'}{'storagedata'}{applianceStatus} && $data{'conf'}{'storagedata'}{applianceStatus} ne "" ) {
    $app_st = $data{'conf'}{'storagedata'}{applianceStatus};
    if ( $app_st eq "CRITICAL" ) {
      $error_health++;
      push( @error_message, "Appliance status : $app_st\n" );
    }
    elsif ( $app_st eq "WARNING" ) {
      $warn_health++;
      push( @error_message, "Appliance status : $app_st\n" );
    }
  }

  my @ordered = ( 'Data Services', 'License', 'Storage', 'Hardware', 'Remote Support' );
  foreach my $key (@ordered) {
    if ( exists $data{'conf'}{'storagedata'}{$key} ) {
      my $new_status = $data{'conf'}{'storagedata'}{$key};
      my %new_hash = ( service => $key, status => $new_status );
      if ( $new_status eq "CRITICAL" ) {
        $error_health++;
        push( @error_message, "$key status : $new_status\n" );
      }
      elsif ( $new_status eq "WARNING" ) {
        $warn_health++;
        push( @error_message, "$key status : $new_status\n" );
      }
      push(@{$hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{dash}}, \%new_hash);
    }
  }

  if ( $error_health == 0 ) {
    if ( $warn_health == 0 ) {
      $report = "ok";
    }
    else {
      $report = "warning";
    }
  }
  else {
    $report = "error";
  }

  if ( defined $data{'alerts'} && $data{'alerts'} ne "" ) {
    foreach my $alert_id (keys %{$data{alerts}}) {
      my $alert = $data{alerts}{$alert_id};

      my $al_level = $alert->{'level'};
      
      # Skip alerts with level "INFO"
      if ($al_level && $al_level eq "INFO") {
        next;
      }

      my $al_time = $alert->{'time'};
      my $timestamp = defined $al_time ? str2time($al_time) : "";

      my %alert = (
        level   => $al_level // "",
        time    => $timestamp // "",
        id      => $alert->{'event_id'} // "",
        type    => $alert->{'event_type'} // "",
        code    => $alert->{'event_code'} // "",
        message => $alert->{'message'} // "",
      );
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{alerts} }, \%alert );
    }
  }

  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }

  my @device_status = (
    {
    "item_id" => $cfg->{'hostcfg_id'},
    "status" => "$report",
    "data" => join("\n", @error_message),
    }
  );

  my $rc = $xormon->saveStatus(\@device_status);
  my $rc_hs = $xormon->saveConfJSON(\%hs_data, 1);
}

sub logout{

  my $url = "https://$cfg->{'apiHost'}/pml/login/delete";
  my $ua        = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 });
  my $req       = HTTP::Request->new( DELETE => $url );
  $req->header(
    'Accept' => 'application/json',
    'Authorization' => 'Bearer '. $cfg->{'token'} . ''
  );
  my $res = $ua->request($req);
  if ( !$res->is_success ) {
    $logger->error("ERROR: Logout to API failed\n");
    $logger->error( $res->content );
    #print Dumper \$res;
    return "";
  }
}
################
# STOREONCE V4 #
#      END     #
################

sub connection_test {
  my $client = shift;
  $cfg = shift;
  $logger = XormonLogger->new('storeonce', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  $cfg->{'debug'} = 0;

  my $port;
  my $http;
  my @rows;

  my $test = Test->new($client);

  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiUsername", "apiPassword", "proto");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  if ($cfg->{proto} eq "http") {
    $port = "80";
    $http = "http";
  }
  elsif ($cfg->{proto} eq "https") {
    $port = "443";
    $http = "https";
  }

  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($cfg->{'apiHost'}, $port);
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  my $url = "$http://$cfg->{'apiHost'}/pml/login/authenticatewithobject";

  ### TEST VERSIONS
  my $agent = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( POST => $url );

  $request->header('Content-Type' => 'application/json', 'Accept' => 'application/json');

  my $json_con ="{\"username\":\"$cfg->{'apiUsername'}\",\"password\":\"$cfg->{'apiPassword'}\",\"grant_type\":\"password\"}";

  $request->content('' . $json_con . '');
  my $results = $agent->request($request);
  if (!$results->is_success){
    #### CALL STOREONCE FUNCTION OLD version ####
    if (defined $results->content && $results->content ne ""){
      my $result = decode_json($results->content);
      # print Dumper $result;
      if (defined $result->{'extendedError'}->{'message'} && $result->{'extendedError'}->{'message'} ne ""){
        push(@rows, "$result->{'extendedError'}->{'message'}\n");
      }
      $test->addSection("API login", 0 , \@rows);
    }
  }
  else {
    ###STOREONCE VERSION 4###
    if (defined $results->content && $results->content ne ""){
      my $result = decode_json($results->content);
      if (defined $result->{'access_token'} && $result->{'access_token'} ne ""){
        $cfg->{'token'} = $result->{'access_token'};
        chomp($cfg->{'token'});
        if ( defined $results->headers->{'date'} && $results->headers->{'date'} ne ""){
          my $timestamp = str2time($results->headers->{'date'});
          $cfg->{'gtime'} = gmtime($timestamp);
          $cfg->{'gtime'} =~ s/ {2,}/ /g;

          ($cfg->{'start_time'},$cfg->{'end_time'}) = set_time();
        }
        else{
          $test->addSection("Storage time", 0, "Storage time (date) not found!");
          return $test->stop();
        }

        ######## TEST CODE ######

        ####### CONFIGURATION
        ###Nas
        create_conf("https://$cfg->{'apiHost'}/api/v1/data-services/nas/shares?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","NAS");
        ###CAT
        create_conf("https://$cfg->{'apiHost'}/api/v1/data-services/cat/stores?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","CAT");
        ###VTL
        create_conf("https://$cfg->{'apiHost'}/api/v1/data-services/vtl/libraries?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","VTL");
        #### STORAGE Appliance
        create_conf("https://$cfg->{'apiHost'}/api/v1/management-services/federation/members?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","storagedata");
        ### Storage CAP
        create_conf("https://$cfg->{'apiHost'}/api/v1/data-services/dashboard/appliance/localhost?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","storagestats");

        if (defined $data{'conf'}{'storagestats'}{'uuid'} && $data{'conf'}{'storagestats'}{'uuid'} ne ""){
          my $uuid = $data{'conf'}{'storagestats'}{'uuid'};
          push(@rows, "UUID : $data{'conf'}{'storagestats'}{'uuid'}\n");

          if (defined $data{'conf'}{'storagedata'}{$uuid}{'productName'} && $data{'conf'}{'storagedata'}{$uuid}{'productName'} ne ""){
            push(@rows, "Model : $data{'conf'}{'storagedata'}{$uuid}{'productName'}\n");
          }
          if (defined $data{'conf'}{'storagedata'}{$uuid}{'serialNumber'} && $data{'conf'}{'storagedata'}{$uuid}{'serialNumber'} ne ""){
            push(@rows, "Serial : $data{'conf'}{'storagedata'}{$uuid}{'serialNumber'}\n");
          }
          if (defined $data{'conf'}{'storagedata'}{$uuid}{'federationApiVersion'} && $data{'conf'}{'storagedata'}{$uuid}{'federationApiVersion'} ne ""){
            push(@rows, "API Version : $data{'conf'}{'storagedata'}{$uuid}{'federationApiVersion'}\n");
          }
          if (defined $data{'conf'}{'storagestats'}{'softwareVersion'} && $data{'conf'}{'storagestats'}{'softwareVersion'} ne ""){
            push(@rows, "SW Version : $data{'conf'}{'storagestats'}{'softwareVersion'}\n");
          }
          if (defined $data{'conf'}{'storagedata'}{$uuid}{'hostname'} && $data{'conf'}{'storagedata'}{$uuid}{'hostname'} ne ""){
            push(@rows, "Host Name : $data{'conf'}{'storagedata'}{$uuid}{'hostname'}\n");
          }

          my $error = 1;
          my $nas_status = "";
          my $cat_status = "";
          my $vtl_status = "";
          my $ove_status = "";

          push(@rows, "\n");

          if (defined $data{'conf'}{'storagedata'}{'statusNas'} && $data{'conf'}{'storagedata'}{'statusNas'} ne ""){
            $nas_status = $data{'conf'}{'storagedata'}{'statusNas'};
            push(@rows, "NAS: $nas_status\n");
          }
          if (defined $data{'conf'}{'storagedata'}{'statusCat'} && $data{'conf'}{'storagedata'}{'statusCat'} ne ""){
            $cat_status = $data{'conf'}{'storagedata'}{'statusCat'};
            push(@rows, "CAT: $cat_status\n");
          }
          if (defined $data{'conf'}{'storagedata'}{'statusVtl'} && $data{'conf'}{'storagedata'}{'statusVtl'} ne ""){
            $vtl_status = $data{'conf'}{'storagedata'}{'statusVtl'};
            push(@rows, "VTL: $cat_status\n");
          }

          if ($nas_status eq "CRITICAL" || $cat_status eq "CRITICAL" || $vtl_status eq "CRITICAL"){
            $ove_status = "CRITICAL";
          }
          elsif ( $nas_status eq "WARNING" || $cat_status eq "WARNING" || $vtl_status eq "WARNING" ){
            $ove_status = "WARNING";
          }
          elsif ( $nas_status eq "OK" && $cat_status eq "OK" && $vtl_status eq "OK" ){
            $ove_status = "OK";
          }
          else{
            $ove_status = "UNKNOWN";
          }

          push(@rows, "\nOverall status: $ove_status\n");

          $test->addSection("Configuration status", 1 , \@rows);

          @rows = ();

          ##### PERFORMANCE
          #my $cpu_status = get_storage_data_cpu("https://$cfg->{'apiHost'}/api/v1/management-services/hardware/parametrics-cpu?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","CPU");
          if (get_storage_data_cpu("https://$cfg->{'apiHost'}/api/v1/management-services/hardware/parametrics-cpu?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","CPU")){
            push(@rows, "CPU Status: OK\n");
          }
          else {
            push(@rows, "CPU Status: Error\n");
            $error = 0
          }

          if (get_storage_data_disk("https://$cfg->{'apiHost'}/api/v1/management-services/hardware/parametrics-disk?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","DISK")){
            push(@rows, "Drive Status: OK\n");
          }
          else {
            push(@rows, "Drive Status: Error\n");
            $error = 0
          }

          if (get_storage_data_ports("https://$cfg->{'apiHost'}/api/v1/management-services/hardware/parametrics-ethernet?startDate=$cfg->{'start_time'}&endDate=$cfg->{'end_time'}","perf","PORT")){
            push(@rows, "Port Status: OK\n");
          }
          else {
            push(@rows, "Port Status: Error\n");
            $error = 0
          }
          $test->addSection("Peformance data", $error , \@rows);
          return $test->stop();
        }
        else {
          $test->addSection("Peformance data", 0 , "Storage UUID not found");
          return $test->stop();
        }
      }
      else {
        $test->addSection("Peformance data", 0 , "Token not found! Login failed! access_token not found!");
        return $test->stop();
      }
    }
  }
  return $test->stop();
}

1;
