
# XormonNG uses different libs
my $xormonNG = ( $ENV{XormonNG} ) ? $ENV{XormonNG} : 0;
if ($xormonNG) {

  #require FindBin;
  use FindBin;
  use lib "$FindBin::Bin/";    # use lib in this folder
  use lib "../../../perl-modules";
}

use strict;
use warnings;
use SVC 2.2;
use Data::Dumper;
use File::Temp qw/tempdir/;

# $Xorux::Net::OpenSSH::debug = 4;

my $net_openssh = 1;
my $dir_remove  = 0;

if ( defined $ENV{NET_OPENSSH} ) { $net_openssh = $ENV{NET_OPENSSH}; }
if ($net_openssh) {

  # use require here to do not affect all storage whe it does not exist and "use" is on the top
  require Xorux::Net::OpenSSH;
  if ( $Xorux::Net::OpenSSH::VERSION < 0.78 ) {
    &error( "Current version of Xorux::Net::OpenSSH is " . $Xorux::Net::OpenSSH::VERSION . ". Use version 0.78 and higher." );
  }
  Xorux::Net::OpenSSH->import(qw(new disconnect capture scp_get pipe_out));
}

my ( $svc, $user, $key, $ssh_timeout, $gui, $password, $pass_enable ) = @ARGV;
if ( !defined $key ) { $key = ""; }
if ( !defined $gui ) { $gui = ""; }

my $ctl_dir = tempdir( CLEANUP => 1 );

#### test module IO::Pty
my $module     = "IO::Tty";
my $module_def = 1;

if ( $pass_enable == 1 ) {
  eval "use $module; 1" or $module_def = 0;
  if ( !$module_def ) {
    if ($xormonNG) {
      error("perl-IO-Tty is not installed, install it by: yum install perl-IO-Tty\nRead more here: http://www.stor2rrd.com/IO-Tty.htm");
    }
    else {
      print "ERROR: perl-IO-Tty is not installed, install it by: yum install perl-IO-Tty\nRead more here: http://www.stor2rrd.com/IO-Tty.htm\n";
    }
    exit 1;

  }
}

my $net_ssh = SVC::openssh_new( $svc, "22", $user, $key, $ssh_timeout, $password, $pass_enable, $ctl_dir );
if ( not defined $net_ssh ) {
  error( "Unable to connect to remote machine " . $svc, 1 );
  exit(1);
}

if ( $gui eq "sshdatatest" ) {
  my $cmd     = "svcinfo lsiostatsdumps -nohdr -delim :";
  my $cmd_out = SVC::openssh_runcmd( $net_ssh, $cmd, $ssh_timeout );
  if ( !defined $cmd_out ) {
    error( "Remote Command: " . $cmd . " failed" );
    disconnect();
    exit(1);
  }
  if ( $cmd_out =~ m/command not found/ ) {
    error("ERROR: $cmd_out");
    disconnect();
    exit(1);
  }

  # test SCP file transfer
  my $file_list = ["/dumps/svc.config.cron.xml_*"];
  $cmd_out = SVC::openssh_getfile( $net_ssh, $file_list, $ctl_dir, $ssh_timeout );
  if ( !defined $cmd_out ) {
    print STDERR ( "ERROR: SCP transfer failed: " . join( " ", @{$file_list} ), 1 );
    disconnect();
    exit(1);
  }
}

my $cmd = "lscurrentuser";
my $out = SVC::openssh_runcmd( $net_ssh, $cmd, $ssh_timeout );
if ( !defined $out ) {
  error( "Remote Command: " . $cmd . " failed" );
  disconnect();
  exit(1);
}

my @data = split( "\n", $out );
my $role = grep {/^role/} @data;
if ($role) {
  my $user_role = grep {/Administrator|RestrictedAdmin|SecurityAdmin/} @data;
  if ($user_role) {

  }
  elsif ( grep {/Monitor/} @data ) {
    my ( $version, $version_original ) = get_version();
    if ( defined $version && $version >= 8030102 ) {

    }
    else {
      $version_original =~ s/ .*//g;
      print Dumper \@data;
      error("Actual version is $version_original. Current account Monitor works only with version 8.3.1.2+");
      disconnect();
      exit(1);
    }
  }
  else {
    print Dumper \@data;
    error("  current account does not have Administration,RestrictedAdmin or SecurityAdmin access!!");
    error("$user user on all IBM SVC/Storwize must belong to \"RestrictedAdmin\", \"Administrator\" or \"SecurityAdmin\" UserRole");
    disconnect();
    exit(1);
  }
}
else {
  $role = grep {/UserRole /} @data;
  if ($role) {
    print " Looks like it is a Unified box and you use wrong IP, use System IP\n";
    print " Refer to http://www.stor2rrd.com/storwize-unified.htm\n";
    print Dumper \@data;
    disconnect();
    exit(1);
  }
}

my $authorization_test = grep {/CMMVC7016E Authorization has failed because the private key is not valid for the user name that you have specified/} @data;
if ($authorization_test) {
  error("  User \"$user\" is wrong, specify it in: UI --> settings --> Storages --> storage name");
  disconnect();
  exit(1);
}
if ( $gui eq "" ) {
  my $out = SVC::openssh_disconnect( $net_ssh, $ssh_timeout );
  if ( not defined $out ) {
    error("Unable to disconnect from remote machine");
    exit(1);
  }
  exit;
}

if ( $gui eq "sshtest" ) {
  my $cmd     = "svcinfo lssystem -delim :";
  my $cmd_out = SVC::openssh_runcmd( $net_ssh, $cmd, $ssh_timeout );
  if ( !defined $cmd_out ) {
    error( "Remote Command: " . $cmd . " failed" );
    disconnect();
    exit(1);
  }
  my @data = split( "\n", $cmd_out );
  ( my $name )         = grep {/^name:/} @data;
  ( my $code )         = grep {/^code_level:/} @data;
  ( my $product_name ) = grep {/^product_name:/} @data;
  my @output = ();
  if ( defined $name ) {
    $name =~ s/^name://g;

    #push(@output,"Machine Name:       $name\n");
    $output[0] = "Machine Name:           $name\n";
  }
  else {
    $output[0] = "\n";
  }
  if ( defined $product_name ) {
    $product_name =~ s/^product_name://g;

    #push(@output,"Product Name:       $product_name\n");
    $output[4] = "Product Name:           $product_name\n";
  }
  else {
    $output[4] = "\n";
  }
  if ( defined $code ) {
    $code =~ s/^code_level://g;

    #push(@output,"Code Level:         $code\n");
    $output[3] = "Code Level:             $code\n";
  }
  else {
    $output[3] = "\n";
  }
  $cmd     = "svcinfo lsenclosure -delim :";
  $cmd_out = SVC::openssh_runcmd( $net_ssh, $cmd, $ssh_timeout );
  if ( !defined $cmd_out ) {
    error( "Remote Command: " . $cmd . " failed" );
    disconnect();
    exit(1);
  }
  @data = split( "\n", $cmd_out );
  my %hash_key;
  my $serial_number = "";
  my $machine_type  = "";
  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^id:/ ) {
      my @data_line = split( ":", $line );
      my $index     = -1;
      foreach my $element (@data_line) {
        $index++;
        $hash_key{$element} = $index;
      }
    }
    else {
      my @data_line = split( ":", $line );
      if ( defined $hash_key{"product_MTM"} && defined $data_line[ $hash_key{"product_MTM"} ] ) {
        $machine_type = $data_line[ $hash_key{"product_MTM"} ];
      }
      if ( defined $hash_key{"serial_number"} && defined $data_line[ $hash_key{"serial_number"} ] ) {
        $serial_number = $data_line[ $hash_key{"serial_number"} ];
      }
    }
  }
  $output[1] = "Machine Type-Model:     $machine_type\n";
  $output[2] = "Machine Serial:         $serial_number\n";
  my $scal = join( "", @output );
  print "$scal\n";

  my $out = SVC::openssh_disconnect( $net_ssh, $ssh_timeout );
  if ( not defined $out ) {
    error("Unable to disconnect from remote machine");
    exit(1);
  }
  exit;
}
else {
  my $out = SVC::openssh_disconnect( $net_ssh, $ssh_timeout );
  if ( not defined $out ) {
    error("Unable to disconnect from remote machine");
    exit(1);
  }
  exit;
}

sub remove_whitespace {
  my $value = shift;
  if ( !defined $value ) { return ""; }
  $value =~ s/^\s+|\s+$//g;
  return $value;

}

sub disconnect {
  my $out = SVC::openssh_disconnect( $net_ssh, $ssh_timeout );
  if ( not defined $out ) {
    error("Unable to disconnect from remote machine");
  }
  return;
}

sub error {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  if ( defined $rc ) {
    print STDERR ( $tm . " - ERROR   - ssh_test.pl: " . $msg . " - Return code: " . $rc . "\n" );
  }
  else {
    print STDERR ( $tm . " - ERROR   - ssh_test.pl: " . $msg . "\n" );
  }
}

sub get_version {
  my $cmd     = "svcinfo lssystem -delim :";
  my $cmd_out = SVC::openssh_runcmd( $net_ssh, $cmd, $ssh_timeout );
  if ( !defined $cmd_out ) {
    error( "Remote Command: " . $cmd . " failed" );
    return;
  }
  my @data = split( "\n", $cmd_out );
  ( my $code ) = grep {/^code_level:/} @data;
  if ( defined $code ) {
    $code =~ s/code_level://g;
  }
  if ( defined $code && $code =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)\s+\(build\s+(.*)\).*$/ ) {
    return ( $1 * 1000000 + $2 * 10000 + $3 * 100 + $4, $code );
  }
  return;
}
