use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use Xormon;
use Cwd qw(cwd);
use SWIZ;
use Test;
use XormonLogger;
use POSIX ":sys_wait_h";

my $logger;

# microservice's directory
my $act_dir = cwd;
my $wrkdir  = "";

#$SIG{CHLD} = 'IGNORE';
# cannot be used in this MS, bcs it produces warning "Can't ignore signal CHLD, forcing to default."
# it's probably caused by that there are running 2 another new child processes for each device (svcconfig.pl and svcperf.pl)

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3202;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);

my $server = IO::Socket->new(
  Domain => AF_INET,
  Type => SOCK_STREAM,
  Proto => 'tcp',
  LocalHost => '0.0.0.0',
  LocalPort => $port,
  ReusePort => 1,
  Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

#
# kill all running svcperf.pl processes
#
terminate_old_processes();

my $client;

# waiting for a new client connection
while ($client = $server->accept()) {

  my $pid;
  unless ( defined( $pid = fork() ) ) {
    Xormon::error("Wait for fork");
    sleep 5;
  }

  if ($pid == 0) {

    # get information about a newly connected client
    my $client_address = $client->peerhost();
    my $client_port = $client->peerport();
    Xormon::log("Connection from $client_address:$client_port");

    # read up to 2048 characters from the connected client
    my $data = "";
    $client->recv($data, 2048);

    my $decoded = from_json($data);
    my $cfg     = $decoded->{data};

    #
    # Device configuration
    #
    export_device_cfg($cfg);

    Xormon::log("Received task: $decoded->{task} for Device: $cfg->{hostalias}");
    my $response = '';
    if ($decoded->{task} eq "fetch") {
      #
      # data collection
      #
      $client->send("Task $decoded->{task} successfuly started");
      $client->shutdown(SHUT_WR);

      my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'swiz', $cfg->{hostcfg_id}, $cfg->{hostalias});

      # start task
      $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

      # target destinations check
      if (defined $decoded->{'destinations'}) {
        $xormon->setDestinations($decoded->{'destinations'});
      }

      #
      # create conf/perf
      #
      collect_data();

      #
      # parse conf/perf files and send data
      #
      my $status = SWIZ::process_collected_data($xormon,$cfg,$wrkdir);

      # end task
      $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);
      Xormon::log("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done. Status: $status");
    }
    elsif ( $decoded->{task} eq "test" ) {
      #
      # connection test
      #
      connection_test();
      $client->shutdown(SHUT_WR);
      Xormon::log("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done.");
    }
    else {
      $response = "undefined task";

      $client->send($response);
      $client->shutdown(SHUT_WR);

    }

    exit();

  } else {
    $client->autoflush(1);
  }
waitpid( -1, WNOHANG );
}

$server->close();

exit();

sub collect_data {
  # system()    you want to execute a command and don't want to capture its output
  # exec        you don't want to return to the calling perl script
  # backticks   you want to capture the output of the command
  # open        you want to pipe the command (as input or output) to your script

  #
  # create conf
  #
  if ( is_process_running("svcconfig.pl $ENV{'STORAGE_NAME'} XormonNG") ) {
    $logger->error( "WARNING: Process 'svcconfig.pl $ENV{'STORAGE_NAME'} XormonNG' is still running, cannot run the new one... $!" . __FILE__ . ":" . __LINE__ );
  }
  else {
    $logger->info( "starting: $act_dir/svcconfig.pl $ENV{'STORAGE_NAME'} XormonNG" );
    #my $deb_file = "/tmp/strace-svcconfig-$$-".time().".log";
    #system ("strace", "-fo", "$deb_file", "perl", "-w", "$act_dir/svcconfig.pl", "$ENV{'STORAGE_NAME'}", "XormonNG");
    system ("perl", "-w", "$act_dir/svcconfig.pl", "$ENV{'STORAGE_NAME'}", "XormonNG");
  }

  #
  # create perf
  #
  if ( is_process_running("svcperf.pl $ENV{'STORAGE_NAME'} XormonNG") ) {
    # restart svcperf process only in the first run after midnight
    restart_svcperf("$wrkdir/svcperf-run", "svcperf.pl $ENV{'STORAGE_NAME'} XormonNG");
  }
  unless ( is_process_running("svcperf.pl $ENV{'STORAGE_NAME'} XormonNG") ) {
    #system ("$PERL", "-w", "$act_dir/svcperf.pl", "&"); # cannot be used like this, bcs this will not run it on the backgroud
    $logger->info( "starting a background task: $act_dir/svcperf.pl $ENV{'STORAGE_NAME'} XormonNG" );
    #my $deb_file = "/tmp/strace-svcperf-$$-".time().".log";
    #system "strace -fo $deb_file perl -w $act_dir/svcperf.pl $ENV{'STORAGE_NAME'} XormonNG &";
    system "perl -w $act_dir/svcperf.pl $ENV{'STORAGE_NAME'} XormonNG &";
    system ("touch", "$wrkdir/svcperf-run");
  }

  return 1;
}

sub restart_svcperf {
  my $file           = shift;
  my $process_string = shift;

  if ( -f $file ) {
    if ( ( localtime( ( stat($file) )[9] ) )[3] != ( localtime() )[3] ) {
      # day of the last modification --> ( localtime( ( stat($file) )[9] ) )[3]
      # today                        --> ( localtime() )[3] )
      #print "############# ".( localtime( ( stat($file) )[9] ) )[3]." : ".( localtime() )[3]."\n";

      $logger->info( "First run after midnight, restart old '$process_string' processes:" );
      my @pids = ();
      foreach ( grep {/$process_string$/} @{ get_ps_list() } ) {
        chomp $_;
        my ( undef, $pid, undef ) = split(/\s+/, $_);
        push(@pids, $pid);
        print "$_\n";
      }
      if ( scalar(@pids) > 0 ) {
        #kill(15,@pids);
        #$logger->info( "kill 15 " . join(" ", @pids) );
        kill('TERM',@pids);
        $logger->info( "kill 'TERM', " . join(" ", @pids) );
      }
      else {
        $logger->info( "Old svcperf.pl processes have not been found..." );
      }
    }
  }

  return 1;
}

sub terminate_old_processes {
  Xormon::log( "Terminate old svcperf.pl processes:" );
  my @pids = ();
  foreach ( @{ get_ps_list() } ) {
    if ( $_ =~ m/svcperf\.pl.*XormonNG$/ ) {
      my ( undef, $pid, undef ) = split(/\s+/, $_);
      push(@pids, $pid);
      print "$_\n";
    }
  }
  if ( scalar(@pids) > 0 ) {
    #kill(15,@pids);
    #$logger->info( "kill 15 " . join(" ", @pids) );
    kill('TERM',@pids);
    Xormon::log( "kill 'TERM', " . join(" ", @pids) );
  }
  else {
    Xormon::log( "Old svcperf.pl processes have not been found..." );
  }

  return 1;
}

sub is_process_running {
  my $process_string = shift;

  if ( ( grep {/$process_string$/} @{ get_ps_list() } ) > 0 ) {
    return 1;
  }
  else {
    return 0;
  }
}

sub get_ps_list {
  open(my $ps,"ps -ef |") || Xormon::error("'ps -ef' command failed! $!" . __FILE__ . ":" . __LINE__ ) && exit 0;
  my @ps_list = <$ps>;
  chomp @ps_list;
  close($ps);

  return \@ps_list;
}

sub export_device_cfg {
  my $cfg = shift;

  #
  # mandatory cfg
  #
  unless ( exists $cfg->{'hostcfg_id'}  && $cfg->{'hostcfg_id'} ne '' )  { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")           && exit 0; }
  unless ( exists $cfg->{'hostalias'}   && $cfg->{'hostalias'} ne '' )   { Xormon::error("Cannot continue, 'hostalias' is not configured!")           && exit 0; }
  $logger = XormonLogger->new( 'swiz', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'host'}        && $cfg->{'host'} ne '' )        { $logger->error("Cannot continue, 'host' is not configured!")                && exit 0; }
  unless ( exists $cfg->{'sshUsername'} && $cfg->{'sshUsername'} ne '' ) { $logger->error("Cannot continue, 'sshUsername' is not configured!")         && exit 0; }

  $ENV{'XormonNG'}            = 1;
  $ENV{'STORAGE_NAME'}        = $cfg->{'hostalias'};
  $ENV{'SVC_IP'}              = $cfg->{'host'};
  $ENV{'SVC_USER'}            = $cfg->{'sshUsername'};
  $ENV{'SVC_KEY'} = "";
  $ENV{'SVC_PASSWORD_ENABLE'} = 0;


  #
  # SSH key or password
  #
  if ( $cfg->{'isKeyFile'} && exists $cfg->{'sshKeyFile'} ) {
    $ENV{'SVC_KEY'} = $cfg->{'sshKeyFile'};

    unless ( -f $cfg->{'sshKeyFile'} ) {
      $logger->error("Cannot set SSH session, sshKeyFile doesn't exist!" . __FILE__ . ":" . __LINE__ );
    }
  }
  elsif ( ! $cfg->{'isKeyFile'} && exists $cfg->{'sshPassword'} ) {
    $ENV{'SVC_PASSWORD'}        = $cfg->{'sshPassword'};
    $ENV{'SVC_PASSWORD_ENABLE'} = 1;
  }
  else {
    #$logger->error("Cannot set SSH session, sshPassword or sshKeyFile was not found!" . __FILE__ . ":" . __LINE__ ) && exit 0;
    $logger->error("Cannot set SSH session, sshPassword or sshKeyFile was not found!" . __FILE__ . ":" . __LINE__ );
  }

  #
  # configurable cfg
  #
  if ( exists $cfg->{'sampleRate'} && Xormon::isdigit($cfg->{'sampleRate'}) ) {
    $ENV{'STOR2RRD_STEP'} = $cfg->{'sampleRate'} * 60; # in seconds
    $ENV{'SVC_INTERVAL'}  = $cfg->{'sampleRate'};      # in minutes
  }
  else {
    # default
    $ENV{'STOR2RRD_STEP'} = 300; # in seconds
    $ENV{'SVC_INTERVAL'}  = 5;   # in minutes
  }

  #
  # create directories
  #
  $wrkdir = "$act_dir/data/$cfg->{'hostcfg_id'}";
  $ENV{'SVC_DIR'}  = $wrkdir;

  unless ( -d "$act_dir/data" )   { mkdir( "$act_dir/data", 0755 )   || $logger->error( "Cannot mkdir '$act_dir/data': $!" . __FILE__ . ":" . __LINE__ )   && exit; }
  unless ( -d "$wrkdir" )         { mkdir( "$wrkdir", 0755 )         || $logger->error( "Cannot mkdir '$wrkdir': $!" . __FILE__ . ":" . __LINE__ )         && exit; }
  unless ( -d "$wrkdir/iostats" ) { mkdir( "$wrkdir/iostats", 0755 ) || $logger->error( "Cannot mkdir '$wrkdir/iostats': $!" . __FILE__ . ":" . __LINE__ ) && exit; }

  #
  # test agent scripts
  #
  unless ( -f "$act_dir/svcconfig.pl" )   { $logger->error( "Script '$act_dir/svcconfig.pl doesn't exist!': $!" . __FILE__ . ":" . __LINE__ )   && exit; }
  unless ( -f "$act_dir/svcperf.pl" )     { $logger->error( "Script '$act_dir/svcperf.pl doesn't exist!': $!" . __FILE__ . ":" . __LINE__ )     && exit; }
  unless ( -f "$act_dir/SVC.pm" )         { $logger->error( "Module '$act_dir/SVC.pm doesn't exist!': $!" . __FILE__ . ":" . __LINE__ )         && exit; }
  unless ( -f "$act_dir/FlashSystem.pm" ) { $logger->error( "Module '$act_dir/FlashSystem.pm doesn't exist!': $!" . __FILE__ . ":" . __LINE__ ) && exit; }

  #
  # remove old files
  #
  #find $SVC_DIR -name '*svcconf*data' -mmin +30 -exec rm {} \;
  #find $SVC_DIR -name '*svcconf*out' -mtime +1 -exec rm {} \;
  #find $SVC_DIR -name '*svcperf*out' -mtime +7 -exec rm {} \;
  #find $SVC_DIR -name 'N*_stats_*' -mmin +120 -exec rm {} \;
  opendir( my $dir, "$wrkdir" ) || $logger->error( "Cannot open directory '$wrkdir' " . __FILE__ . ":" . __LINE__ ) && exit 1;
  my @files = grep !/^\.\.?$/, readdir($dir);
  closedir($dir);
  chomp @files;

  foreach my $filename (@files) {
    my $filepath = "$wrkdir/$filename";
    if ( -f $filepath ) {
      if    ( $filename =~ m/svcconf.*data/ && Xormon::file_time_diff($filepath) > 1800 )   { unlink($filepath); } # 30 minutes
      elsif ( $filename =~ m/svcconf.*out/  && Xormon::file_time_diff($filepath) > 86400 )  { unlink($filepath); } #  1 day
      elsif ( $filename =~ m/svcperf.*out/  && Xormon::file_time_diff($filepath) > 604800 ) { unlink($filepath); } #  7 days
    }
  }
  # iostats
  opendir( $dir, "$wrkdir/iostats" ) || $logger->error( "Cannot open directory '$wrkdir/iostats' " . __FILE__ . ":" . __LINE__ ) && exit 1;
  @files = grep !/^\.\.?$/, readdir($dir);
  closedir($dir);
  chomp @files;

  foreach my $filename (@files) {
    my $filepath = "$wrkdir/iostats/$filename";
    if ( -f $filepath ) {
      if ( $filename =~ m/N.*_stats_/ && Xormon::file_time_diff($filepath) > 7200 ) { unlink($filepath); } #  2 hours
    }
  }

  return 1;
}

sub connection_test {
  my $test = Test->new($client);

  #
  # test IP and port
  #
  my ( $code, $text ) = Xormon::conntest_tcp($ENV{'SVC_IP'},22); # always port 22, there is no possibility to change it
  $test->addSection("SSH host network connection", $code, $text);
  if ( $code == 0 ) { return $test->stop(); }

  #
  # SSH connection
  #
  my ( $status, $reason ) = ssh_connection_test();
  if ( $status ) {
    $test->addSection("SSH connection", 1 , \@$reason);
  }
  else {
    $test->addSection("SSH connection", 0 , \@$reason);
  }

  return $test->stop();
}

sub ssh_connection_test {
  # system()    you want to execute a command and don't want to capture its output
  # exec        you don't want to return to the calling perl script
  # backticks   you want to capture the output of the command
  # open        you want to pipe the command (as input or output) to your script

  my $error_log = "$ENV{'SVC_DIR'}/connection_test_error.log";
  my @output    = ();
  my @errors    = ();
  my $connected = 1;
  my $ssh_cmd   = "";
  my $ssh_note  = "";

  if ( -f $error_log ) {
    unlink($error_log);
  }

  if ( exists $ENV{'SVC_KEY'} && $ENV{'SVC_KEY'} ne "" ) {
    # $PERL $BINDIR/ssh_test_svc.pl $SVC_IP $SVC_USER $SVC_KEY "30" "sshtest" "$encode_pw" "$SVC_PASS_ENABLE" ""
    open(my $cmd,"perl $act_dir/ssh_test_svc.pl $ENV{'SVC_IP'} $ENV{'SVC_USER'} '$ENV{'SVC_KEY'}' '30' 'sshtest' '' '0' 2>$error_log |") || $logger->error("'ps -ef' command failed! $!" . __FILE__ . ":" . __LINE__ ) && exit 0;
    @output = <$cmd>;
    #chomp @output;
    close($cmd);

    # $SSH -i $SVC_KEY  $SVC_USER@$SVC_IP  \"lscurrentuser\"
    $ssh_note = "This SSH command must work without providing password:";
    $ssh_cmd  = "ssh $ENV{'SVC_USER'}\@$ENV{'SVC_IP'} \"lscurrentuser\"";
    if (-f $ENV{'SVC_KEY'}){
      $ssh_cmd  = "ssh -i $ENV{'SVC_KEY'} $ENV{'SVC_USER'}\@$ENV{'SVC_IP'} \"lscurrentuser\"";
    }
  }
  else {
    # $PERL $BINDIR/ssh_test_svc.pl $SVC_IP $SVC_USER "" "30" "sshtest" "$encode_pw" "$SVC_PASS_ENABLE" ""
    open(my $cmd,"perl $act_dir/ssh_test_svc.pl $ENV{'SVC_IP'} $ENV{'SVC_USER'} '' '30' 'sshtest' '$ENV{'SVC_PASSWORD'}' $ENV{'SVC_PASSWORD_ENABLE'} 2>$error_log |") || $logger->error("'ps -ef' command failed! $!" . __FILE__ . ":" . __LINE__ ) && exit 0;
    @output = <$cmd>;
    #chomp @output;
    close($cmd);

    # $SSH $SVC_USER@$SVC_IP  \"lscurrentuser\"
    $ssh_note = "This SSH command must work:";
    $ssh_cmd  = "ssh $ENV{'SVC_USER'}\@$ENV{'SVC_IP'} \"lscurrentuser\"";
  }

  if ( -f $error_log ) {
    open( my $file, "<$error_log" ) || $logger->error( "Couldn't open file $error_log $!" . __FILE__ . ":" . __LINE__ ) && return (0, ["Couldn't open file $error_log $!"]);
    @errors = <$file>;
    close($file);

    if ( scalar(@errors) ) {
      $connected = 0;

      if ( exists $ENV{'SVC_KEY'} && ! -f $ENV{'SVC_KEY'} ) {
        push(@errors, "\n");
        push(@errors, "Cannot set SSH session, sshKeyFile '$ENV{'SVC_KEY'}' doesn't exist!\n");
      }

      push(@errors, "\n");
      push(@errors, "$ssh_note\n");
      push(@errors, "$ssh_cmd\n");
    }

    unlink($error_log);
  }

  if ( $connected ) {
    return (1, \@output);
  }
  else {
    return (0, \@errors);
  }
}
