#
# v0.5.3

my ($revision) = ( '$Revision: 0.5.3 $' =~ /([\d\.]+)/ );

# Modules
use strict;
use Storable;
use Data::Dumper;
use Date::Parse;

# Options and their default value
my $wbemcli_cmd = "/usr/bin/wbemcli";
my $storname;    # XIV storage alias
my $xivip;       # XIV host name (IP address)
my $xivuser     = "info";    # XIV user name
my $xivpwd      = "";        # XIV user password
my $mininterval = 300;       # minimal interval time for data collection (sec)
my $cfginterval = 60;        # interval time for configuration data collection (min)
my $dir         = "..";      # Data directory
my $timeout     = 600;       # Command timeout

my $debug      = 0;          # Debug mode 0:off, 1:on
my $debug_full = 0;          # Debug mode 0:off, 1:on
my $debug_scr  = 0;          # Debug script 0:off, 1:on

if ( defined $ENV{WBEMCLI_CMD} && -f $ENV{WBEMCLI_CMD} ) {
  $wbemcli_cmd = $ENV{WBEMCLI_CMD};
}
if ( !-f $wbemcli_cmd ) {
  $wbemcli_cmd = "/opt/freeware/bin/wbemcli";
}
if ( !-f $wbemcli_cmd ) {
  $wbemcli_cmd = "/usr/local/bin/wbemcli";
}

my $bindir = $ENV{BINDIR};
if ( defined $ENV{STORAGE_NAME} ) {
  $storname = $ENV{STORAGE_NAME};
}
else {
  print("xivperf.pl: XIV storage name alias is required.\n");
  exit(1);
}
if ( defined $ENV{XIV_IP} ) {
  $xivip = $ENV{XIV_IP};
}
else {
  print("xivperf.pl: XIV host name or IP address is required.\n");
  exit(1);
}
if ( defined $ENV{XIV_USER} )         { $xivuser     = $ENV{XIV_USER} }
if ( defined $ENV{XIV_PASSWD} )       { $xivpwd      = $ENV{XIV_PASSWD} }
if ( defined $ENV{XIV_DIR} )          { $dir         = $ENV{XIV_DIR} }
if ( defined $ENV{XIV_CFG_INTERVAL} ) { $cfginterval = $ENV{XIV_CFG_INTERVAL} }
if ( defined $ENV{DEBUG} )            { $debug       = $ENV{DEBUG} }
if ( defined $ENV{SAMPLE_RATE} )      { $timeout     = $ENV{SAMPLE_RATE} * 60 }
if ( defined $ENV{DEBUG_FULL} ) {
  if ( $ENV{DEBUG_FULL} eq "1" ) { $debug_full = 1; $debug = 1; }
  else {
    if ( $ENV{DEBUG_FULL} eq $storname ) { $debug_full = 1; $debug = 1; }
    else                                 { $debug_full = 0; }
  }
}
if ( defined $ENV{DEBUG_SCRIPT} ) { $debug_scr = $ENV{DEBUG_SCRIPT}; }

my $tmpdir = "$dir/";    # Directory for temporary files

my $out_perf_file;       #
my $out_conf_file;       #
my $cacheFile       = $tmpdir . "cache.file";
my $configFile      = $tmpdir . "config.file";
my $debugScriptFile = $tmpdir . $storname . "_script.sh";
my $scr_fh;

my @RAW_COUNTERS    = ( 'StatisticTime', 'KBytesRead',  'KBytesWritten', 'KBytesTransferred', 'ReadIOs',     'WriteIOs',    'TotalIOs',   'IOTimeCounter', 'ReadIOTimeCounter', 'WriteIOTimeCounter' );
my @VOLUME_COUNTERS = ( 'ReadRateKB',    'WriteRateKB', 'TotalRateKB',   'ReadIORate',        'WriteIORate', 'TotalIORate', 'ReadIOTime', 'WriteIOTime',   'ReadIOPct' );

# RC_PROPERTIES Definitions (CIM_Synchronized,CIM_StorageSynchronized)
# CopyMethodology specifies what copy methodology the service uses to create and/or maintain the target element. Values are: Not Specified: The method of maintaining the copy is not specified. Full Copy: This indicates that a full copy of the source object is (or will be) generated . Incremental-Copy: Only changed data from source element is copied to target element. Differential-Copy: Only the new writes to source element are copied to the target element. Copy-On-Write: Affected data is copied on the first write to the source or to the target elements. Copy-On-Access: Affected data is copied on the first access to the source element. Delta-Update: Difference based replication where after the initial copy, only updates to source are copied to target. Snap-And-Clone: The service creates a snapshot of the source element first, then uses the the snapshot as the source of the copy operation to the target element.
# CopyPriority allows the priority of background copy engine I/O to be managed relative to host I/O operations during a sequential background copy operation. Values are: Low: copy engine I/O lower priority than host I/O. Same: copy engine I/O has the same priority as host I/O. High: copy engine I/O has higher priority than host I/O. Urgent: copy operation to be performed as soon as possible, regardless of the host I/O requests.
# CopyRecoveryMode describes whether the copy operation continues after a broken link is restored. Automatic: copy operation resumes automatically. Manual: CopyState is set to Suspended after the link is restored. It is required to issue the Resume operation to continue.
# CopyState describes the state of the association with respect to Replication activity. Values are: Initialized: The link to enable replication is established and source/replica elements are associated, but the data flow has not started. Unsynchronized: Not all the source element data has been copied to the target element. Synchronized: For the Mirror, Snapshot, or Clone replication, the target represents a copy of the source. Broken: The relationship is non-functional due to errors in the source, the target, the path between the two or space constraints. Fractured: Target is split from the source. Split: The target element was gracefully (or systematically) split from its source element -- consistency is guaranteed. Inactive: Data flow has stopped, writes to source element will not be sent to target element. Suspended: Data flow between the source and target elements has stopped. Writes to source element are held until the association is Resumed. Failedover: Reads and writes to/from the target element. Source element is not reachable. Prepared: Initialization is completed, the data flow has started, however, the data flow has not started. Aborted: The copy operation is aborted with the Abort operation. Use the Resync Replica operation to restart the copy operation. Skewed: The target has been modified and is no longer synchronized with the source element or the point-in-time view. Mixed: Applies to the CopyState of GroupSynchronized. It indicates the StorageSynchronized associations of the elements in the groups have different CopyState values. Partitioned: State of replication relationship can not be determined, for example, due to a connection problem. Invalid: The array is unable to determine the state of the replication relationship, for example, after the connection is restored; however, either source or target elements have an unknown status.
# CopyType describes the Replication Policy. Values are: Async: create and maintain an asynchronous copy of the source. Sync: create and maintain a synchronized copy of the source. UnSyncAssoc: create an unsynchronized copy and maintain an association to the source. UnSyncUnAssoc: create an unsynchronized copy with a temporary association that is deleted upon completion of the copy operation.
# Mode describes whether the target elements will be updated synchronously or asynchronously. If NULL, implementaton decides the mode.
# ProgressStatus describes the status of the association with respect to Replication activity. Values are: Completed: The request is completed. Data flow is idle. Dormant: Indicates that the data flow is inactive suspended or quiesced. Initializing: In the process of establishing source/replica association and the data flow has not started. Preparing: preparation-in-progress. Synchronizing: sync-in-progress. Resyncing: resync-in-progess. Restoring: restore-in-progress. Fracturing: fracture-in-progress. Splitting: split-in-progress. Failing over: in the process of switching source and target. Failing back: Undoing the result of failover. Detaching: detach-in-progress. Aborting: abort-in-progress. Mixed: Applies to groups with element pairs with different statuses. Generally, the individual statuses need to be examined.Suspending: The copy operation is in the process of being suspended. Requires fracture: The requested operation has completed, however, the synchronization relationship needs to be fractured before further copy operations can be issued. Requires resync: The requested operation has completed, however, the synchronization relationship needs to be resynced before further copy operations can be issued. Requires activate: The requested operation has completed, however, the synchronization relationship needs to be activated before further copy operations can be issued. Pending: The flow of data has stopped momentarily due to limited bandwidth or busy system.
# ReadOnly - This property specifies whether the source, the target, or both elements are "read only" to the host.SystemElement: The source element. SyncedElement: The target element. Both: Both the source and the target elements.are read only to the host.
# RequestedCopyState is an integer enumeration that indicates the last requested or desired state for the association. The actual state of the association is represented by CopyState. Note that when CopyState reaches the requested state, this property will be set to 'Not Applicable.
# ReplicaType provides information on how the Replica is being maintained. Values are: Full Copy: This indicates that a full copy of the source object is (or will be) generated . Before Delta: This indicates that the source object will be maintained as a delta data from the replica. After Delta: This indicates that the replica will be maintained as delta data from the source object. Log: This indicates that the replica object is being maintained as a log of changes to the source. Not Specified: The method of maintaining the copy is not specified.
# SyncState describes the state of the association with respect to Replication activity. Values are: Initialized: The link to enable replication is established. and source/replica elements are associated, but the Copy engine has not started. PrepareInProgress: Preparation for Replication is in progress and the Copy engine has started. Prepared: All necessary preparation has completed. ResyncInProgress: Synchronization or Resynchronization is in progress. This may be the initial 'copy' or subsequent changes being copied. Synchronized: An Async or Sync replication is currently synchronized. When this value is set, SyncMaintained will be true. FractureInProgress: An operation to fracture an Async or Sync replication is in progress. Fractured: An Async or Sync replication is fractured. QuiesceInProgress: A quiesce operation is in progress. Quiesced: The replication has been quiesced and is ready for a change. RestoreInProgress: An operation is in progress to copy the Synced object to the System object. Idle: The 'normal' state for an UnSyncAssoc replica. Frozen: All blocks copied from source to an UnSyncAssoc replica and the copy engine is stopped. CopyInProgress: A deferred background copy operation is in progress to copy the source to the replica target for an UnSyncAssoc association. Broken: The relationship is non-functional due to errors in the source, the target, the path between the two or space constraints.
# SyncType describes the intended outcome of the replication.Values are: Mirror: create and maintain a copy of the source. Snapshot: create a PIT, virtual copy of the source. Clone: create a PIT, full copy the source.
# UndiscoveredElement - This property specifies whether the source, the target, or both elements involved in a copy operation are undiscovered. An element is considered undiscovered if its object model is not known to the service performing the copy operation. The values are: SystemElement: The source element. SyncedElement: The target element. Both: Both the source and the target elements. If both the source and the target elements are discovered, the value of this property shall be NULL.
#
my $CP_PROPERTIES = {
  'CopyMethodology' => { '0' => 'Not Specified', '1' => 'Other', '2' => 'Implementation decides', '3' => 'Full Copy', '4' => 'Incremental-Copy', '5' => 'Differential-Copy', '6' => 'Copy-On-Write', '7' => 'Copy-On-Access', '8' => 'Delta-Update', '9' => 'Snap-And-Clone' },
  'CopyPriority'    => { '0' => 'Not Managed',   '1' => 'Low',   '2' => 'Same',                   '3' => 'High',      '4' => 'Urgent', },
  'CopyRecoveryMode'    => { '0' => 'Unknown',       '2' => 'Automatic',         '3' => 'Manual' },
  'CopyState'           => { '2' => 'Initialized',   '3' => 'Unsynchronized',    '4' => 'Synchronized', '5' => 'Broken', '6' => 'Fractured', '7' => 'Split', '8' => 'Inactive', '9' => 'Suspended', '10' => 'Failedover', '11' => 'Prepared', '12' => 'Aborted', '13' => 'Skewed', '14' => 'Mixed', '15' => 'Not Applicable', '16' => 'Partitioned', '17' => 'Invalid' },
  'CopyType'            => { '2' => 'Async',         '3' => 'Sync',              '4' => 'UnSyncAssoc', '5' => 'UnSyncUnAssoc' },
  'Mode'                => { '0' => 'Unknown',       '2' => 'Synchronous',       '3' => 'Asynchronous' },
  'ProgressStatus'      => { '0' => 'Unknown',       '2' => 'Completed',         '3' => 'Dormant', '4' => 'Initializing', '5' => 'Preparing', '6' => 'Synchronizing', '7' => 'Resyncing', '8' => 'Restoring', '9' => 'Fracturing', '10' => 'Splitting', '11' => 'Failing over', '12' => 'Failing back', '13' => 'Aborting', '14' => 'Mixed', '15' => 'Not Applicable', '16' => 'Suspending', '17' => 'Requires fracture', '18' => 'Requires resync', '19' => 'Requires activate', '20' => 'Pending', '21' => 'Detaching' },
  'ReadOnly'            => { '2' => 'SystemElement', '3' => 'SyncedElement',     '4' => 'Both' },
  'RequestedCopyState'  => { '2' => 'Initialized',   '3' => 'Unsynchronized',    '4' => 'Synchronized', '5' => 'Broken', '6' => 'Fractured', '7' => 'Split', '8' => 'Inactive', '9' => 'Suspended', '10' => 'Failedover', '11' => 'Prepared', '12' => 'Aborted', '13' => 'Skewed', '14' => 'Mixed', '15' => 'Not Applicable', '16' => 'Partitioned', '17' => 'Invalid' },
  'ReplicaType'         => { '0' => 'Not Specified', '2' => 'Full Copy',         '3' => 'Before Delta', '4' => 'After Delta', '5' => 'Log' },
  'SyncState'           => { '2' => 'Initialized',   '3' => 'PrepareInProgress', '4' => 'Prepared', '5' => 'ResyncInProgress', '6' => 'Synchronized', '7' => 'Fracture In Progress', '8' => 'QuiesceInProgress', '9' => 'Quiesced', '10' => 'Restore In Progresss', '11' => 'Idle', '12' => 'Broken', '13' => 'Fractured', '14' => 'Frozen', '15' => 'Copy In Progress' },
  'SyncType'            => { '6' => 'Mirror',        '7' => 'Snapshot',          '8' => 'Clone' },
  'UndiscoveredElement' => { '2' => 'SystemElement', '3' => 'SyncedElement',     '4' => 'Both' },
};

# CG_PROPERTIES Definitions (CIM_Synchronized,CIM_GroupSynchronized)
# ConsistencyState - Indicates the current state of consistency.
# ConsistencyStatus - Indicates the current status of consistency. Consistency may have been disabled or is experiencing an error condition.
# ConsistencyType - Indicates the consistency type used by the source and its associated target group.
my $CG_PROPERTIES = {
  'ConsistencyState'  => { '0' => 'Unknown', '1' => 'Other', '2' => 'Not Applicable', '3' => 'Consistent',              '4' => 'Inconsistent' },
  'ConsistencyStatus' => { '0' => 'Unknown', '1' => 'Other', '2' => 'Completed',      '3' => 'Consistency-in-progress', '4' => 'Consistency disabled', '5' => 'Consistency-error' },
  'ConsistencyType' => { '2' => 'Sequentially Consistent' }
};

# Storage Hardware Types
my %storage_info = ( "is_xiv" => 0, "is_a9000" => 0 );

my $storage;
my $cache;
my $data;
my $config;
my $tzoffset;

sub message {
  my $msg = shift;
  my $tm  = localtime();
  print( "INFO " . $tm . " xivperf.pl: " . $msg . "\n" );
}

sub warning {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  print STDERR ( "WARNING " . $tm . " xivperf.pl: " . $msg . "\n" );
}

sub error {
  my ( $msg, $rc ) = @_;
  my $tm = localtime();
  print STDERR ( "ERROR " . $tm . " xivperf.pl: " . $msg . "\n" );
}

sub error_die {
  my $msg = shift;
  my $tm  = localtime();
  print STDERR ( "ERROR " . $tm . " xivperf.pl: " . $msg . "\n" );
  exit(1);
}

sub wbemcli {
  my ( $op, $objpath ) = @_;
  `$wbemcli_cmd -noverify $op \'$objpath\'`;
}

sub wbemclinl {
  my ( $op, $objpath ) = @_;
  `$wbemcli_cmd -noverify -nl $op \'$objpath\'`;
}

sub enumStorage {
  my $elementClass = shift;
  my %storage;
  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die("$act_time: died in SIG ALRM"); };
    alarm($timeout);

    # Get Element Storage
    my $objectPath   = "https://" . $xivuser . ":" . $xivpwd . "@" . $xivip . ":5989/root/ibm:" . $elementClass;
    my $objectPathAn = "https://" . $xivuser . ":" . "******" . "@" . $xivip . ":5989/root/ibm:" . $elementClass;
    if ($debug_full) { message( "+$wbemcli_cmd -noverify -nl ei " . $objectPathAn . "." ); }
    if ($debug_scr)  { print $scr_fh ( $wbemcli_cmd . " -noverify -nl ei " . $objectPath . " > " . $elementClass . ".txt\n" ); }
    for my $line ( &wbemclinl( "ei", $objectPath ) ) {
      chomp($line);
      if ( $line     =~ /.*5989\/root\/ibm:IBMTSDS_StorageSystem.CreationClassName.*/ ) { $storage{'InstanceName'} = $line; }
      if ( not $line =~ /=/ )                                                           { next; }
      my @a = split( "=", $line );
      if ( $a[0] ) {
        if ( $a[0] =~ /^-(.*)$/ ) { $a[0] = $1; }
        if ( $a[1] ) { $a[1] =~ s/"//g; }
        $storage{ $a[0] } = $a[1];
      }
    }

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      error("command timed out after : $timeout seconds");
      exit(1);
    }
  }

  return \%storage;
}

sub enumNames {
  my $elementClass = shift;
  my %names;
  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    # Get Element Names
    my $objectPath   = "https://" . $xivuser . ":" . $xivpwd . "@" . $xivip . ":5989/root/ibm:" . $elementClass;
    my $objectPathAn = "https://" . $xivuser . ":" . "******" . "@" . $xivip . ":5989/root/ibm:" . $elementClass;
    if ($debug_full) { message( "+$wbemcli_cmd -noverify ei " . $objectPathAn . "." ); }
    for my $line ( &wbemcli( "ei", $objectPath ) ) {
      chomp($line);
      my @a = split( ",", $line );
      my $id;
      my $name;
      foreach my $elem (@a) {
        if    ( $elem =~ /^InstanceID="([-:.\w]*)"$/ ) { $id   = $1; }
        elsif ( $elem =~ /^ElementName="([-.\w]+)"$/ ) { $name = $1; }
      }
      if ($id) {
        $names{$id} = $name;
      }
    }

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      error("command timed out after : $timeout seconds");
      exit(1);
    }
  }
  return \%names;
}

sub enumStats {
  my $elementClass = shift;
  my @stats;
  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    # Get Element Statistics
    my $objectPath   = "https://" . $xivuser . ":" . $xivpwd . "@" . $xivip . ":5989/root/ibm:" . $elementClass;
    my $objectPathAn = "https://" . $xivuser . ":" . "******" . "@" . $xivip . ":5989/root/ibm:" . $elementClass;
    if ($debug_full) { message( "+$wbemcli_cmd -noverify ei " . $objectPathAn . "." ); }
    if ($debug_scr)  { print $scr_fh ( $wbemcli_cmd . " -noverify ei " . $objectPath . " > " . $elementClass . ".txt\n" ); }
    for my $line ( &wbemcli( "ei", $objectPath ) ) {
      chomp($line);
      my @a = split( ",", $line );
      my $id;
      my $name;
      my %st;
      foreach my $elem (@a) {
        if    ( $elem =~ /^(\w+)="(.*)"$/ ) { $st{$1} = $2; }
        elsif ( $elem =~ /^(\w+)=(.*)$/ )   { $st{$1} = $2; }
      }
      if ( $st{'InstanceID'} ) {
        push( @stats, \%st );
      }
    }

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      error("command timed out after : $timeout seconds");
      exit(1);
    }
  }
  return \@stats;
}

sub calculateStats {
  my ( $old_counters, $new_counters ) = @_;

  # Calculate perf statistic values from raw counters
  my %stats    = ();
  my $interval = 0;

  # check that we have timestamp in cached sample
  if ( $old_counters->{'storage_timestamp'} ) {
    $interval = $new_counters->{'storage_timestamp'} - $old_counters->{'storage_timestamp'};

    if ($interval) {
      my $deltaReadKB             = $new_counters->{'KBytesRead'} - $old_counters->{'KBytesRead'};
      my $deltaWriteKB            = $new_counters->{'KBytesWritten'} - $old_counters->{'KBytesWritten'};
      my $deltaTotalKB            = $new_counters->{'KBytesTransferred'} - $old_counters->{'KBytesTransferred'};
      my $deltaReadIO             = $new_counters->{'ReadIOs'} - $old_counters->{'ReadIOs'};
      my $deltaWriteIO            = $new_counters->{'WriteIOs'} - $old_counters->{'WriteIOs'};
      my $deltaTotalIO            = $new_counters->{'TotalIOs'} - $old_counters->{'TotalIOs'};
      my $deltaReadIOTimeCounter  = $new_counters->{'ReadIOTimeCounter'} - $old_counters->{'ReadIOTimeCounter'};
      my $deltaWriteIOTimeCounter = $new_counters->{'WriteIOTimeCounter'} - $old_counters->{'WriteIOTimeCounter'};

      $stats{'ReadRateKB'}  = $deltaReadKB / $interval;
      $stats{'WriteRateKB'} = $deltaWriteKB / $interval;
      $stats{'TotalRateKB'} = $deltaTotalKB / $interval;
      $stats{'ReadIORate'}  = $deltaReadIO / $interval;
      $stats{'WriteIORate'} = $deltaWriteIO / $interval;
      $stats{'TotalIORate'} = $deltaTotalIO / $interval;

      if ( $deltaReadIO > 0 && $deltaReadIOTimeCounter > 0 ) {
        $stats{'ReadIOTime'} = $deltaReadIOTimeCounter / $deltaReadIO;
      }
      else {
        $stats{'ReadIOTime'} = 0;
      }
      if ( $deltaWriteIO > 0 && $deltaWriteIOTimeCounter > 0 ) {
        $stats{'WriteIOTime'} = $deltaWriteIOTimeCounter / $deltaWriteIO;
      }
      else {
        $stats{'WriteIOTime'} = 0;
      }
      if ( $deltaTotalIO > 0 && $deltaReadIO > 0 ) {
        $stats{'ReadIOPct'} = $deltaReadIO / $deltaTotalIO * 100;
      }
      else {
        $stats{'ReadIOPct'} = 0;
      }
      $stats{'Interval'} = $interval;
    }
    else {
      &error("Interval between samples is 0, skipping.");
    }
  }
  else {
    &error("No storage timestamp in previous sample, skipping.");
  }
  return \%stats;
}

sub collectStats {
  my ( $elementType, $elementClass, $statisticsClass, $elementCounters ) = @_;
  my $server_epoch;
  my $retry_cnt = 5;
  my $element   = &enumElement($elementClass);
  if ( not defined $element ) {
    &error("Read volume configuration data failed. Exiting.");
    exit(1);
  }
  if ( scalar( keys %{$element} ) == 0 ) {
    &error("Zero number of volumes. Read volume configuration data failed. Exiting.");
    exit(1);
  }
  if ($debug) { &message( "Number of volumes = " . scalar( keys %{$element} ) . "." ); }
  $config->{$elementType} = $element;
  #
  while ($retry_cnt) {
    my $stats = &enumStats($statisticsClass);
    if ($debug) { print( "R#" . $retry_cnt . ":" ); }

    # Save time of statistic data creation
    $server_epoch = time();
    foreach my $stat ( @{$stats} ) {
      my $elementID;
      if ( $element->{ $stat->{'InstanceID'} }->{'DeviceID'} =~ /IBM.\d{4}-\w{7}-(.*)/ ) {

        #my $elementID = $element->{$stat->{'InstanceID'}}->{'DeviceID'};
        $elementID = $1;
      }
      my $elementName = $element->{ $stat->{'InstanceID'} }->{'ElementName'};
      if ( !defined $stat->{'StatisticTime'} ) {
        if ($debug) { print("e\n"); }
        if ($debug) { &warning("Statistic Time is not found in input data."); }
        last;
      }
      my $storage_epoch = &str2time( &isotime( $stat->{'StatisticTime'} ) );
      if ( !defined $storage_epoch ) {
        if ($debug) { print("E\n"); }
        if ($debug) { &warning( "Storage epoch time is not parsed. Statistic Time = " . $stat->{'StatisticTime'} . " ISO Time: " . &isotime( $stat->{'StatisticTime'} ) . "." ); }
        last;
      }

      #if($debug) {&message("Statistic Time = ".$stat->{'StatisticTime'}.". Timestamp = ".$storage_epoch."."); }
      #if($debug) {&message("Local Time = ".localtime($storage_epoch)."."); }
      # Prepare current samples
      my %newcacheElem;
      my $new_raw_counters = \%newcacheElem;

      #my $cacheKey = sprintf("%s.%s.%s",$storage->{'Name'},$elementType,$elementName);
      my $cacheKey = sprintf( "%s.%s.%s", $storage->{'Name'}, $elementType, $elementID );
      foreach my $countType (@RAW_COUNTERS) {
        if ($countType) { $new_raw_counters->{$countType} = $stat->{$countType}; }
      }
      $new_raw_counters->{'storage_timestamp'} = sprintf( "%d", $storage_epoch );
      $new_raw_counters->{'server_timestamp'}  = $server_epoch;
      $new_raw_counters->{'name'}              = $elementName;

      # Get previous samples from cache
      my $calc_flag = 0;
      my $cached_raw_counters;
      if ( exists( $cache->{$cacheKey} ) ) {
        $cached_raw_counters = $cache->{$cacheKey};
        if ( !defined $cached_raw_counters->{'storage_timestamp'} ) {
          if ($debug) { print("e"); }

          #&warning("No timestamp in cached data, skipping.");
        }
        else {
          if ( $storage_epoch == $cached_raw_counters->{'storage_timestamp'} ) {
            &warning( "Same sample: " . $cacheKey . " = " . $stat->{'StatisticTime'} . ", skipping." );
            $retry_cnt = 0;
            if ($debug) { print("="); }

            #last;
          }
          elsif ( $new_raw_counters->{'storage_timestamp'} - $cached_raw_counters->{'storage_timestamp'} < $mininterval ) {
            my $int = $new_raw_counters->{'storage_timestamp'} - $cached_raw_counters->{'storage_timestamp'};
            &message( "Short interval " . $int . " second, waiting\n", );
            $retry_cnt--;
            if ($debug) { print("<"); }

            #sleep 30;
            #last;
          }
          else {
            # Both samples ok -> calculate
            $calc_flag = 1;
            if ($debug_full) { print("*"); }
          }
        }
      }
      else {
        # No data for element in cache
        if ($debug) { print("."); }
      }

      # Done - Get previous samples from cache
      $retry_cnt = 0;

      # Save current samples to cache
      $cache->{$cacheKey} = $new_raw_counters;

      # calculate statistics
      if ($calc_flag) {
        my $stat_values = &calculateStats( $cached_raw_counters, $new_raw_counters );
        #
        $stat_values->{'ID'}                             = $new_raw_counters->{'InstanceID'};
        $stat_values->{'Name'}                           = $elementName;
        $stat_values->{'Type'}                           = $elementType;
        $stat_values->{'Time'}                           = &epoch2isotime( $cached_raw_counters->{'server_timestamp'}, $tzoffset );
        $stat_values->{'Timestamp'}                      = $cached_raw_counters->{'server_timestamp'};
        $data->{$elementType}->{ $stat->{'InstanceID'} } = $stat_values;
        $data->{'IntervalStartTime'}                     = $cached_raw_counters->{'server_timestamp'};
        $data->{'IntervalEndTime'}                       = $new_raw_counters->{'server_timestamp'};

        if ( defined $cached_raw_counters->{'storage_timestamp'} ) {
          $data->{'Interval'} = $new_raw_counters->{'storage_timestamp'} - $cached_raw_counters->{'storage_timestamp'};
        }
      }
    }

    # Done all elements in statistic data
    if ($debug) { print("\n"); }
  }
}

sub collectStorageCopy {
  my $server_epoch;
  my $retry_cnt = 5;
  my $element;

  # Local Copy - Primary Volumes and status
  $element = &enumElement("IBMTSDS_StorageSynchronized");
  if ( defined $element ) {
    if ( scalar( keys %{$element} ) != 0 ) {
      if ($debug) { &message( "Local Copy - Number of source volumes = " . scalar( keys %{$element} ) . "." ); }
      $config->{'lc_volume'} = $element;
    }
  }
  #
  # Remote Copy - Local Primary Volumes and status
  $element = &enumElement("IBMTSDS_SourceConsistencyStorageSynchronized");
  if ( defined $element ) {
    if ( scalar( keys %{$element} ) != 0 ) {
      if ($debug) { &message( "Remote Copy - Number of primary volumes = " . scalar( keys %{$element} ) . "." ); }
      $config->{'rc_localvolume_pri'} = $element;
    }
  }

  # Remote Copy - Local Aux Volumes and status
  $element = undef;
  $element = &enumElement("IBMTSDS_TargetConsistencyStorageSynchronized");
  if ( defined $element ) {
    if ( scalar( keys %{$element} ) != 0 ) {
      if ($debug) { &message( "Remote Copy - Number of auxiliary volumes = " . scalar( keys %{$element} ) . "." ); }
      $config->{'rc_localvolume_aux'} = $element;
    }
  }

  # Remote Volumes
  $element = undef;
  $element = &enumElement("IBMTSDS_ReplicationEntity");
  if ( defined $element ) {
    if ( scalar( keys %{$element} ) != 0 ) {
      if ($debug) { &message( "Number of target volumes = " . scalar( keys %{$element} ) . "." ); }
      $config->{'rc_tgtvolume'} = $element;
    }
  }
  #
  # Local Copy - Mapping Snapshot Group -> Primary Volumes
  $element = &enumElement("IBMTSDS_SnapshotGroupToOrderedMembers");
  if ( defined $element ) {
    if ( scalar( keys %{$element} ) != 0 ) {
      if ($debug) { &message( "Local Copy - Number of group -> primary volumes = " . scalar( keys %{$element} ) . "." ); }
      $config->{'lc_group'} = $element;
    }
  }
  #
  # Remote Copy - Mapping Consistency Group -> Local Primary Volumes
  $element = &enumElement("IBMTSDS_ConsistencyGroupToOrderedMembers");
  if ( defined $element ) {
    if ( scalar( keys %{$element} ) != 0 ) {
      if ($debug) { &message( "Remote Copy - Number of primary volumes = " . scalar( keys %{$element} ) . "." ); }
      $config->{'rc_group'} = $element;
    }
  }
  return;
}    ## sub collectStorageCopy

sub writeStats {
  if ( !defined $data->{'IntervalEndTime'} ) {
    return;
  }

  # Open output file
  $out_perf_file = $tmpdir . $storname . "_xivperf_" . &fileextTime( $data->{'IntervalEndTime'} ) . ".out";
  open( PERFOUT, ">>", ${out_perf_file} . ".tmp" ) || die "Couldn't open file ${out_perf_file}.tmp.";
  #
  my $elementType = shift;
  if ( $elementType eq "volume" ) {
    &writeVolumeStats;
  }
  close(PERFOUT);
}

sub writeVolumeStats {
  #
  print PERFOUT "\nVolume Level Statistics\n";
  print PERFOUT "\tInterval Start:   ", &epoch2isotime( $data->{'IntervalStartTime'}, $tzoffset ), "\n" if ( defined $data->{'IntervalStartTime'} );
  print PERFOUT "\tInterval End:     ", &epoch2isotime( $data->{'IntervalEndTime'},   $tzoffset ), "\n" if ( defined $data->{'IntervalEndTime'} );
  print PERFOUT "\tInterval Length:  ", $data->{'Interval'}, " seconds\n" if ( defined $data->{'Interval'} );
  print PERFOUT "---------------------\n";
  #
  # Volume ID,Time,Interval (s),
  # Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),
  # Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),
  # Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),
  # Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),
  # Peak Read Response Time (ms),Peak Write Response Time (ms),
  # Host Delay (assuming that all host delay is writes) (ms),Host Delay (assuming that host delay is evenly spread between read and writes) (ms),
  # Read Hits,Write Hits,Data Read (KB),Data Written (KB),
  # Volume (Vdisk) Name,Managed Disk Group ID,Managed Disk Group Name,IO Group ID,IO Group Name,
  # Remote Copy relationship ID,Remote Copy relationship name,Remote Copy Change Volume relationship,
  # FlashCopy map ID,FlashCopy map name,FlashCopy map count,
  # Copy Count,Space Efficient Copy Count,Cache state,Easy Tier On/Off,Easy Tier Status,
  # Preferred Node ID,Capacity (TB),Real Capacity (TB),Used Capacity (TB),
  # Auto Expand,Grain Size,Throttle Unit,Throttle Rate,UDID (for HPUX),
  #
  print PERFOUT "Volume ID,Time,Interval (s),Read IO Rate (IO/s),Write IO Rate (IO/s),Total IO Rate (IO/s),Read Data Rate (KB/s),Write Data Rate (KB/s),Total Data Rate (KB/s),Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),Read Response Time (ms),Write Response Time (ms),Overall Response Time (ms),Peak Read Response Time (ms),Peak Write Response Time (ms),,,Read Hits,Write Hits,Data Read (KB),Data Written (KB),Volume Name,Pool ID,Pool Name,,,,,,,,,,,,,,,,,,,,,,,Source Volume Name\n";
  my @disks = keys( %{ $data->{'volume'} } );
  foreach my $dsk ( sort (@disks) ) {
    my $p = $data->{'volume'}->{$dsk};
    my $id;
    my $poolid;
    my $rts = 0;
    my $wts = 0;
    my $tts = 0;
    if ( !defined $p->{'ReadRateKB'} && !defined $p->{'ReadIORate'} && !defined $p->{'ReadIOTime'} ) { last; }
    $rts = $p->{'ReadRateKB'} / $p->{'ReadIORate'}   if ( $p->{'ReadIORate'} > 0 );
    $wts = $p->{'WriteRateKB'} / $p->{'WriteIORate'} if ( $p->{'WriteIORate'} > 0 );
    $tts = $p->{'TotalRateKB'} / $p->{'TotalIORate'} if ( $p->{'TotalIORate'} > 0 );
    if ( $dsk =~ /^IBM\.\d{4}\-\d+\-(\w+)$/ ) { $id = $1; }

    printf PERFOUT (
      "%s,%s,%d,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,%.3f,,,,,,,,,%s,%s,%s,,,,,,,,,,,,,,,,,,,,,,,%s\n",
      $id,                $p->{'Time'},        $p->{'Interval'},
      $p->{'ReadIORate'}, $p->{'WriteIORate'}, $p->{'TotalIORate'},
      $p->{'ReadRateKB'}, $p->{'WriteRateKB'}, $p->{'TotalRateKB'},
      $rts, $wts, $tts,    # Read Transfer Size (kB),Write Transfer Size (kB),Overall Transfer Size (kB),
      $p->{'ReadIOTime'} / 1000, $p->{'WriteIOTime'} / 1000, 0,
      $p->{'Name'},              $p->{'PoolID'},             $p->{'PoolName'}, $p->{'SourceVolumeName'}
    );
  }

}

sub writeConf {

  # Open output file
  if ( !defined $data->{'IntervalEndTime'} ) {
    return;
  }
  $out_conf_file = $tmpdir . $storname . "_xivconf_" . &fileextTime( $data->{'IntervalEndTime'} ) . ".out";
  open( PERFOUT, ">>", ${out_conf_file} . ".tmp" ) || die "Couldn't open file ${out_perf_file}.tmp.";
  if ($debug_full) { &message( "Config file name: " . $out_conf_file ); }
  #
  my $elementType = shift;
  &writeSystemConf     if ( $elementType eq "system" );
  &writePoolConf       if ( $elementType eq "pool" );
  &writeVolumeConf     if ( $elementType eq "volume" );
  &writeVolumeMap      if ( $elementType eq "volumemap" );
  &writeLocalCopyConf  if ( $elementType eq "localcopy" );
  &writeRemoteCopyConf if ( $elementType eq "remotecopy" );
  close(PERFOUT);
}

sub writeSystemConf {
  my @mtms   = split( /\./, &getMachineType );
  my $capref = &getMachineCapacityRef;

  print PERFOUT "\nConfiguration Data\n------------------\n";
  print PERFOUT "\tMachine Name:\t" . $config->{'system'}->{'Name'} . "\n";
  print PERFOUT "\tMachine Type-Model:\t" . $mtms[1] . "-" . $mtms[2] . "\n";
  print PERFOUT "\tMachine Serial:\t" . $mtms[3] . "\n";
  print PERFOUT "\tCode Level:\t" . $config->{'system'}->{'CodeLevel'} . "\n";
  print PERFOUT "\n";
  print PERFOUT "\tDate:\t" .. "\n";
  print PERFOUT "\n";

  if ( $storage_info{'is_a9000'} ) {
    printf PERFOUT ( "\tTotal Capacity     :\t%.3f TiB \n", $capref->{'TotalManagedSpace'} / 1099511627776 );
    printf PERFOUT ( "\tTotal Free Capacity:\t%.3f TiB \n", $capref->{'RemainingManagedSpace'} / 1099511627776 );
  }
  else {
    printf PERFOUT ( "\tTotal Hard Capacity     :\t%.3f TiB \n", $capref->{'TotalManagedSpace'} / 1099511627776 );
    printf PERFOUT ( "\tTotal Hard Free Capacity:\t%.3f TiB \n", $capref->{'RemainingManagedSpace'} / 1099511627776 );
    printf PERFOUT ( "\tTotal Soft Capacity     :\t%.3f TiB \n", $capref->{'TotalVirtualSpace'} / 1099511627776 );
    printf PERFOUT ( "\tTotal Soft Free Capacity:\t%.3f TiB \n", $capref->{'RemainingVirtualSpace'} / 1099511627776 );
  }
  print PERFOUT "\n";
}

sub setStorageInfo {
  my $mtms = &getMachineType;
  my @a    = split( /\./, $mtms );
  if    ( $a[1] =~ m/983\d{1}/ ) { $storage_info{'is_a9000'} = 1; }
  elsif ( $a[1] =~ m/281\d{1}/ ) { $storage_info{'is_xiv'}   = 1; }
}

sub getMachineType {
  my @hdr  = split( /,/, $config->{'system'}->{'IdentifyingDescriptions'} );
  my @data = split( /,/, $config->{'system'}->{'OtherIdentifyingInfo'} );
  my %hash = ();
  @hash{@hdr} = @data;
  return ( $hash{'MTMS'} );
}

sub getMachineCapacityRef {
  my @ekys = keys( %{ $config->{'storcap'} } );
  return ( $config->{'storcap'}->{ $ekys[0] } );
}

sub writePoolConf {
  print PERFOUT "\nPool Level Configuration\n------------------------\n";

  #print PERFOUT "name,id,status,mdisk_count,vdisk_count,capacity,extent_size,free_capacity,virtual_capacity,used_capacity,real_capacity,overallocation,warning,easy_tier,easy_tier_status,compression_active,compression_virtual_capacity,compression_compressed_capacity,compression_uncompressed_capacity\n";
  print PERFOUT "name,id,status,mdisk_count,vdisk_count,capacity,extent_size,free_capacity,snapshot_size,\n";
  foreach my $elem ( sort ( keys( %{ $config->{'pool'} } ) ) ) {
    my $d = $config->{'pool'}->{$elem};
    my $id;
    if ( $d->{'PoolID'} =~ /.*VP(\w+)$/ ) { $id = $1; }
    my $capacity;
    if ( $storage_info{'is_a9000'} ) {
      $capacity = $d->{'TotalManagedSpace'};
    }
    else {
      $capacity = $d->{'HardSize'};
    }

    # name,id,status,mdisk_count,vdisk_count,capacity,extent_size,free_capacity
    printf PERFOUT (
      "%s,%s,%s,,,%.3f,,%.3f,%.3f,\n", $d->{'ElementName'},
      $id, $d->{'Status'}, &ConvSizeUnits( "GB", $capacity ),
      &ConvSizeUnits( "GB", $d->{'RemainingManagedSpace'} ), &ConvSizeUnits( "GB", $d->{'SnapshotSize'} )
    );
  }

}

sub writeVolumeConf {
  print PERFOUT "\nVolume Level Configuration\n--------------------------\n";
  print PERFOUT "volume_id,id,name,,,,pool_id,pool_name,capacity,,,,,,vdisk_UID,,,,,,\n";
  foreach my $elem ( sort ( keys( %{ $config->{'volume'} } ) ) ) {
    my $d     = $config->{'volume'}->{$elem};
    my $pelm  = "IBMTSDS:" . $d->{'SystemName'} . "-VP" . $d->{'Pool'};
    my $pname = $config->{"pool"}->{$pelm}->{'ElementName'};
    my $id;
    if ( $d->{'DeviceID'} =~ /.*\-(\w+)\s*$/ ) { $id = $1; }
    printf PERFOUT (
      "%s,%s,%s,,,,%s,%s,%.3f,,,,,,%s,,,,,,\n", $id,    $d->{'DeviceID'},                     $d->{'ElementName'},
      $d->{'Pool'},                             $pname, &ConvSizeUnits( "GB", $d->{'Size'} ), $d->{'Name'},
    );
  }

}

sub writeLocalCopyConf {
  print PERFOUT "\nLocalCopy Level Configuration\n--------------------------\n";

  # id,name,
  # source_vdisk_id,source_vdisk_name,target_vdisk_id,target_vdisk_name,
  # group_id,group_name,
  # status,progress,copy_rate,clean_progress,incremental,
  # partner_FC_id,partner_FC_name,
  # restoring,start_time,rc_controlled
  #root/ibm:IBMTSDS_SEVolume.SystemCreationClassName="IBMTSDS_StorageSystem"
  print PERFOUT "id,name,source_vdisk_id,source_vdisk_name,target_vdisk_id,target_vdisk_name,group_id,group_name,status,progress,copy_rate,clean_progress,incremental,partner_FC_id,partner_FC_name,restoring,start_time,rc_controlled\n";
  foreach my $elem ( sort ( keys( %{ $config->{'lc_volume'} } ) ) ) {
    my $src        = $config->{'lc_volume'}->{$elem};
    my $group_id   = "";
    my $group_name = "";
    if ( defined $config->{'lc_group'}->{ $src->{'DeviceID'} }->{'DeviceID'} ) {
      if ( $config->{'lc_group'}->{ $src->{'DeviceID'} }->{'Collection'} =~ /.*root\/ibm:IBMTSDS_SnapshotGroup.InstanceID=\"IBMTSDS:IBM\.\d{4}-\w{7}-SG(.*)\"/ ) {
        $group_id   = $config->{'lc_group'}->{ $src->{'DeviceID'} }->{'DeviceID'};
        $group_name = $1;
        print( "_DBG_SnapGroup: ", $group_id, " - ", $group_name, "\n" );
      }
    }
    #
    printf PERFOUT (
      "%s,,%s,%s,%s,%s,%s,%s,%s,%s,,,,,,,%s,,\n",

      # id,,source_vdisk_id,source_vdisk_name,
      $src->{'DeviceID'}, $src->{'DeviceID_2'}, $config->{'volume'}->{ $src->{'DeviceID_2'} }->{'ElementName'},

      # target_vdisk_id,target_vdisk_name,
      $src->{'DeviceID'}, $config->{'volume'}->{ $src->{'DeviceID'} }->{'ElementName'},

      # group_id,group_name,
      $group_id, $group_name,

      # status,progress,
      $CP_PROPERTIES->{'CopyState'}->{ $src->{'CopyState'} }, $src->{'PercentSynced'},

      # copy_rate,clean_progress,incremental,
      # partner_FC_id,partner_FC_name,
      # restoring,start_time,rc_controlled
      $src->{'WhenSynced'}
    );
  }

}

sub writeRemoteCopyConf {
  print PERFOUT "\nRemoteCopy Level Configuration\n--------------------------\n";

  # id,name,
  # master_cluster_id,master_cluster_name,master_vdisk_id,master_vdisk_name,
  # aux_cluster_id,aux_cluster_name,aux_vdisk_id,aux_vdisk_name,
  # primary,consistency_group_id,consistency_group_name,
  # state,bg_copy_priority,progress,copy_type,cycling_mode,freeze_time
  print PERFOUT "id,name,local_system_id,local_system_name,local_volume_id,local_volume_name,remote_system_id,remote_system_name,remote_volume_id,remote_volume_name,primary,consistency_group_id,consistency_group_name,state,bg_copy_priority,progress,copy_type,cycling_mode,freeze_time\n";
  foreach my $elem ( sort ( keys( %{ $config->{'rc_localvolume_pri'} } ) ) ) {
    my $src      = $config->{'rc_localvolume_pri'}->{$elem};    # ->{'IBM.2810-6030000-43dee6e12402803'}
                                                                # 'SyncedElement' => 'root/ibm:IBMTSDS_ReplicationEntity.InstanceID="IBMTSDS:IBM.2810-6030000-Volume124e14500271"',
    my $syncelem = $src->{'SyncedElement'};
    if ( $syncelem =~ /root\/ibm:IBMTSDS_ReplicationEntity.InstanceID="(.*)"/ ) {
      $syncelem = $1;
    }
    else {
      $syncelem = "";
    }
    my $tgt = $config->{'rc_tgtvolume'}->{$syncelem};           # ->{'IBMTSDS:IBM.2810-6030000-Volume124e14500271'}
    my $id;
    #
    my $tgtSystemName = $tgt->{'SystemName'};
    $tgtSystemName =~ s/[\\\"]//g;
    my $tgtDeviceID = $tgt->{'DeviceID'};
    $tgtDeviceID =~ s/[\\\"]//g;
    printf PERFOUT (
      "%s,,%s,,%s,%s,%s,,%s,,%s,%s,%s,%s,%s,%s,%s,,%s\n",

      # id,,master_cluster_id,,master_vdisk_id,master_vdisk_name,
      $src->{'DeviceID'}, $src->{'SystemName'}, $src->{'DeviceID'}, $config->{'volume'}->{ $src->{'DeviceID'} }->{'ElementName'},

      # aux_cluster_id,,aux_vdisk_id,,
      $tgtSystemName, $tgtDeviceID,

      # primary(local/remote),consistency_group_id,consistency_group_name,
      "local", "", "",

      # state,bg_copy_priority,
      $CP_PROPERTIES->{'CopyState'}->{ $src->{'CopyState'} }, $CP_PROPERTIES->{'CopyPriority'}->{ $src->{'CopyPriority'} },

      # progress,copy_type,cycling_mode,freeze_time
      $src->{'PercentSynced'}, $CP_PROPERTIES->{'CopyType'}->{ $src->{'Mode'} }, $src->{'WhenSynced'}
    );
  }
  foreach my $elem ( sort ( keys( %{ $config->{'rc_localvolume_aux'} } ) ) ) {
    my $src      = $config->{'rc_localvolume_aux'}->{$elem};    # ->{'IBM.2810-6030000-43dee6e12402803'}
                                                                # 'SyncedElement' => 'root/ibm:IBMTSDS_ReplicationEntity.InstanceID="IBMTSDS:IBM.2810-6030000-Volume124e14500271"',
    my $syncelem = $src->{'SyncedElement'};
    if ( $syncelem =~ /root\/ibm:IBMTSDS_ReplicationEntity.InstanceID="(.*)"/ ) {
      $syncelem = $1;
    }
    else {
      $syncelem = "";
    }
    my $tgt = $config->{'rc_tgtvolume'}->{$syncelem};           # ->{'IBMTSDS:IBM.2810-6030000-Volume124e14500271'}
    my $id;
    #
    printf PERFOUT (
      "%s,,%s,,%s,%s,%s,,%s,,%s,%s,%s,%s,%s,%s,%s,,%s\n",

      # id,,master_cluster_id,,master_vdisk_id,master_vdisk_name,
      $src->{'DeviceID'}, $src->{'SystemName'}, $src->{'DeviceID'}, $config->{'volume'}->{ $src->{'DeviceID'} }->{'ElementName'},

      # aux_cluster_id,,aux_vdisk_id,,
      "", "",

      # primary(local/remote),consistency_group_id,consistency_group_name,
      "remote", "", "",

      # state,bg_copy_priority,
      $CP_PROPERTIES->{'CopyState'}->{ $src->{'CopyState'} }, $CP_PROPERTIES->{'CopyPriority'}->{ $src->{'CopyPriority'} },

      # progress,copy_type,cycling_mode,freeze_time
      $src->{'PercentSynced'}, $CP_PROPERTIES->{'CopyType'}->{ $src->{'Mode'} }, $src->{'WhenSynced'}
    );
  }

}

sub collectConf {
  my ( $elementType, $elementClass ) = @_;
  my $element = &enumElement($elementClass);
  if ( not defined $element ) {
    &error( "Read " . $elementType . " data failed. Exiting." );
    exit(1);
  }
  if ( scalar( keys %{$element} ) == 0 ) {
    &error( "Zero number of " . $elementType . ". Read " . $elementType . " data failed. Exiting." );
  }
  if ($debug) { &message( "Number of " . $elementType . " = " . scalar( keys %{$element} ) . "." ); }
  $config->{$elementType} = $element;
}

sub enumElement {
  my $elementClass = shift;
  my %data;
  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    # Get Element Statistics
    my $objectPath   = "https://" . $xivuser . ":" . $xivpwd . "@" . $xivip . ":5989/root/ibm:" . $elementClass;
    my $objectPathAn = "https://" . $xivuser . ":" . "******" . "@" . $xivip . ":5989/root/ibm:" . $elementClass;
    if ($debug_full) { message( "+$wbemcli_cmd -noverify ei " . $objectPathAn . "." ); }
    if ($debug_scr)  { print $scr_fh ( $wbemcli_cmd . " -noverify ei " . $objectPath . " > " . $elementClass . ".txt\n" ); }
    for my $line ( &wbemcli( "ei", $objectPath ) ) {
      chomp($line);
      my %st;
      if ( $line =~ /^([^\s]*)\s+(.*)$/ ) {
        $st{'InstanceName'} = $1;
        $line = $2;
      }
      my @a = split( ",", $line );
      my $id;
      foreach my $elem (@a) {
        if ( $elem =~ /^InstanceID="([-:.\w]*)"$/ ) { $id = $1; $st{'InstanceID'} = $1; }
        elsif ( $elem =~ /^DeviceID="([-:.\w]*)"$/ ) {
          if ( not defined $st{'DeviceID'} ) {
            $id = $1;
            $st{'DeviceID'} = $1;
          }
          else {
            $st{'DeviceID_2'} = $1;
          }
        }
        elsif ( $elem =~ /^(\w+)="(.*)"$/ ) { $st{$1} = $2; }
        elsif ( $elem =~ /^(\w+)=(.*)$/ )   { $st{$1} = $2; }
      }
      if ($id) {
        $data{$id} = \%st;
      }
    }

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      error("command timed out after : $timeout seconds");
      exit(1);
    }
  }
  return \%data;
}

sub insertPoolName {
  foreach my $velm ( keys %{ $data->{'volume'} } ) {
    my $poolid = $config->{'volume'}->{$velm}->{'Pool'};

    # Insert 'SourceVolumeName'
    $data->{"volume"}->{$velm}->{'SourceVolumeName'} = $config->{'volume'}->{$velm}->{'SourceVolumeName'};

    # Insert 'PoolID','PoolName'
    foreach my $pelm ( keys %{ $config->{"pool"} } ) {
      if ( $pelm =~ /.*VP${poolid}$/ ) {
        $data->{"volume"}->{$velm}->{'PoolID'}   = $poolid;
        $data->{"volume"}->{$velm}->{'PoolName'} = $config->{"pool"}->{$pelm}->{'ElementName'};
      }
    }
  }
}

sub collectVolumeMap {
  foreach my $hostid ( keys( %{ $config->{'host'} } ) ) {
    &assocInstance( $hostid, $config->{'host'}->{$hostid}->{'InstanceName'} );
  }
}

sub assocInstance {
  my $id          = shift;
  my $elementName = shift;
  my %data;
  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);

    # Get Element Statistics
    my $objectPath   = "https://" . $xivuser . ":" . $xivpwd . "@" . $elementName;
    my $objectPathAn = "https://" . $xivuser . ":" . "******" . "@" . $elementName;
    if ($debug_full) { message( "+$wbemcli_cmd -noverify ain \'" . $objectPathAn . "\'." ); }
    if ($debug_scr)  { print $scr_fh ( $wbemcli_cmd . " -noverify ain \'" . $objectPath . "\' > " . $id . ".txt\n" ); }
    my @volid;
    for my $line ( &wbemcli( "ain", $objectPath ) ) {
      chomp($line);
      if ( $storage_info{'is_a9000'} && $line =~ /^.*CreationClassName="IBMTSDS_SEVolume",DeviceID="([-:.\w]*)".*$/ ) {
        push( @volid, $1 );
      }
      elsif ( $line =~ /^.*IBMTSDS_SEVolume.CreationClassName="IBMTSDS_SEVolume",DeviceID="([-:.\w]*)",.*$/ ) {
        push( @volid, $1 );
      }
      elsif ( $line =~ /^.*CreationClassName="IBMTSDS_SEVolume",DeviceID="([-:\.\w]*)".*$/ ) {
        push( @volid, $1 );
      }
    }
    if ($id) {
      $config->{'host'}->{$id}->{'VolumeMap'} = \@volid;
    }

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      error("command timed out after : $timeout seconds");
      exit(1);
    }
  }
  return \%data;
}

sub writeVolumeMap {
  print PERFOUT "\nHost Level Configuration\n------------------------\n";
  print PERFOUT "id,host_id,name,port_count,IQN WWPN,Volume IDs,Volume Names\n";
  foreach my $elem ( sort ( keys( %{ $config->{'host'} } ) ) ) {
    my $d = $config->{'host'}->{$elem};
    my @volids;
    my @volnames;
    foreach my $volid ( @{ $d->{'VolumeMap'} } ) {
      my $vid;
      if ( $volid =~ /IBM\.\d+\-\w+\-(\w+)$/ ) { $vid = $1; }
      push( @volids,   $vid );
      push( @volnames, $config->{'volume'}->{$volid}->{'ElementName'} );
    }
    if ( not defined $d->{'VolumeMap'} ) { next; }
    my $id;
    if ( $d->{'DeviceID'} =~ /IBM\.\d+\-\w+\-(\w+)$/ ) { $id = $1; }
    if ( $d->{'OtherNameFormat'} eq 'Host name' ) {
      printf PERFOUT (
        "%s,%s,%s,,,%s,%s,\n", $id, $d->{'DeviceID'}, $d->{'ElementName'},
        join( " ", @volids ),  join( " ", @volnames )
      );
    }
  }

}

sub fileextTime {
  my $t = shift;
  if ( !defined $t ) { return "NA"; }
  my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = localtime($t);
  my $y = $year + 1900;
  my $m = $mon + 1;
  return sprintf( "%d%02d%02d_%02d%02d", $y, $m, $mday, $hour, $min );
}

sub isotime {
  my $t = shift;
  if ( $t =~ /^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2}).(\d{6})([\+-])(\d+)$/ ) {
    my $tmod = $9 % 60;
    use integer;
    my $tdiv = $9 / 60;
    no integer;
    my $tz = sprintf( "%04d", ( $tdiv * 100 ) + $tmod );
    return sprintf( "%d:%02d:%02dT%02d:%02d:%02d.%06d%s%s", $1, $2, $3, $4, $5, $6, $7, $8, $tz );
  }
}

sub tzoffset {
  my $sign       = "";
  my $t          = time();
  my $local_time = localtime($t);
  my $gm_time    = gmtime($t);
  my $local      = str2time($local_time);
  my $utc        = str2time($gm_time);
  my $tzo        = ( ( $local - $utc ) / 3600 );
  if ( $tzo > 0 ) { $sign = "+"; }
  return sprintf( "%s%02d00", $sign, $tzo );
}

sub epoch2isotime {

  # Output: 2015:02:05T19:54:07.000000+0100
  my ( $tm, $tz ) = @_;    # epoch, TZ offset (+0100)
  my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = localtime($tm);
  my $y   = $year + 1900;
  my $m   = $mon + 1;
  my $mcs = 0;
  my $str = sprintf( "%4d:%02d:%02dT%02d:%02d:%02d.%06d%s", $y, $m, $mday, $hour, $min, $sec, $mcs, $tz );
  return ($str);
}

sub once_a_interval {
  my $interval = shift;    # minutes
  my $file     = shift;

  if ( !-f $file ) {
    return 0;
  }
  else {
    my $diff_time = ( time() - ( stat("$file") )[9] ) / 60;
    if ( $diff_time < $interval ) {

      # If time difference < then interval return 0
      return 0;
    }
    else {
      #
      return 1;
    }
  }
}

sub ConvSizeUnits {
  my ( $unit, $size ) = @_;
  if ( $unit eq "KB" ) { return ( $size / 1024 ) }
  if ( $unit eq "MB" ) { return ( $size / 1048576 ) }
  if ( $unit eq "GB" ) { return ( $size / 1073741824 ) }
  if ( $unit eq "TB" ) { return ( $size / 1099511627776 ) }
}

# MAIN

&message( "Start. [v" . $revision . "]" );
if ($debug_scr) {
  &message( "Create XIV debug script file: " . $debugScriptFile );
  open( $scr_fh, ">>", $debugScriptFile ) or die( "Couldn't open file: " . $debugScriptFile . "\n" );
  print $scr_fh ("echo \"XIV Debug script - Start\"\n");
}
$storage = &enumStorage("IBMTSDS_StorageSystem");
if ( !defined $storage->{'Name'} ) {
  &error("Cannot get storage name. Connection error. Exiting.");
  exit 1;
}
$config->{"system"} = $storage;
&setStorageInfo;
&message( "Get interval performance data for storage " . $storage->{'Name'} . "." );
if ($debug) { &message( "Storage Name: " . $storage->{'Name'} . " CodeLevel: " . $storage->{'CodeLevel'} . "." ); }

# Load cache from file
if ( -r $cacheFile ) {
  if ($debug_full) { message("Read cache file $cacheFile."); }
  eval { $cache = retrieve($cacheFile); };
  if ($@) {
    &message( $@ . __FILE__ . ":" . __LINE__ . " file:" . $cacheFile );
    &message( "Retrieve data from file " . $cacheFile . " failed." );
    my %cache_array;
    $cache = \%cache_array;
  }
}
else {
  my %cache_array;
  $cache = \%cache_array;
}

$tzoffset = &tzoffset();

# Collect data
&collectConf( "storcap", "IBMTSDS_PrimordialStoragePool" );
&collectConf( "pool",    "IBMTSDS_VirtualPool" );
&collectConf( "host",    "IBMTSDS_SCSIProtocolController" );
&collectStats( "volume", "IBMTSDS_SEVolume", "IBMTSDS_SEVolumeStatistics", \@VOLUME_COUNTERS );
&collectStorageCopy();

# Compute data
&insertPoolName();

# Write data
&writeConf("system");
&writeConf("pool");
&writeConf("volume");
&writeConf("localcopy");
&writeConf("remotecopy");
&writeStats("volume");

# Retreive config data with volume mappimgs from file if exist.
if ( -r $configFile ) {
  if ($debug_full) { message("Read config file $configFile."); }
  eval { $config = retrieve($configFile); };
  if ($@) {
    &message( $@ . __FILE__ . ":" . __LINE__ . " file:" . $configFile );
    &message( "Retrieve data from file " . $configFile . " failed." );
    my %config_array;
    $config = \%config_array;
  }
  else {
    &writeConf("volumemap");
  }
}
else {
  my %config_array;
  $config = \%config_array;
}

if ($debug_full) { &message("Store cache file $cacheFile."); }
store( $cache, $cacheFile );

# Create config data with volume mappimgs and store it to file. Long interval.
if ( once_a_interval( $cfginterval, $configFile ) || !-f $configFile || $debug_scr ) {
  my %config_array;
  $config = \%config_array;
  if ( -f $configFile ) {
    unlink("$configFile") or error_die( "Unable to unlink " . $configFile . ": $!" );
  }
  if ($debug) { &message("Create config data with volume mappimgs."); }
  $config->{"system"} = &enumStorage("IBMTSDS_StorageSystem");
  &collectConf( "storcap", "IBMTSDS_PrimordialStoragePool" );
  &collectConf( "pool",    "IBMTSDS_VirtualPool" );
  &collectConf( "host",    "IBMTSDS_SCSIProtocolController" );
  &collectConf( "volume",  "IBMTSDS_SEVolume" );
  &collectVolumeMap;
  if ($debug_full) { &message("Store config file $configFile."); }
  store( $config, $configFile );
}
if ($debug_scr) {
  print $scr_fh ("echo \"XIV Debug script - Done\"\n");
  close($scr_fh);
}
&message("Done.");

exit 0;
