package XTREMIO;

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use Xormon;
use JSON qw(encode_json decode_json);
use Test;
use XormonLogger;
use Cwd qw(cwd);

my %data;
my $xormon;
my $cfg;
my $logger;
my @rows;
my %sample_rate_intervals;

my $debug                    = 0;
my $exclude_unmapped_volumes = 1;
my $url                      = "";
my $cluster_name             = "";
my $act_dir                  = cwd;
my $wrkdir                   = "";
my $sample_rate              = 300;
my $connection_test          = 0;
my $perf_stats_error         = 0;
my $last_stat_timestamp      = "";
my $last_perf_timestamp      = "";
my $prev_perf_timestamp      = "";

#NOTE
#XTREMIO_ONE_HOUR_MODE --> fetch and conf job are automatically run only once per hour

my @cluster_metrics = ( "iops", "rd_iops", "wr_iops", "avg_latency", "rd_latency", "wr_latency", "bw", "rd_bw", "wr_bw", "compression_factor", "data_reduction_ratio", "dedup_ratio" );
my @cluster_items   = ( "hardware-platform", "license-id", "logical-space-in-use", "node-list", "size-and-capacity", "ssd-space-in-use", "sys-health-state", "sys-id", "sys-psnt-serial-number", "sys-state", "sys-sw-version", "total-memory-in-use", "total-memory-in-use-in-percent", "ud-ssd-space", "ud-ssd-space-in-use", "vol-size", "xms-id" );

my @volumes_metrics = ( "iops", "rd_iops", "wr_iops", "avg_latency", "rd_latency", "wr_latency", "bw", "rd_bw", "wr_bw" );
my @volumes_items   = ( "vol-size", "vol-id", "lun-mapping-list", "naa-name", "vol-access", "num-of-lun-mappings" );

my @ssds_metrics = ( "iops", "rd_iops", "wr_iops", "bw", "rd_bw", "wr_bw" );
my @ssds_items   = ( "brick-id", "ssd-space-in-use", "model-name", "health-state", "ssd-uid", "name", "ssd-size-in-kb", "brick-name", "slot-num", "rg-index", "ssd-name", "guid", "ssd-index", "fw-version", "serial-number", "ssd-rg-state", "useful-ssd-space", "rg-name", "brick-index", "ssd-size" );

my @xenvs_metrics = ("cpu_usage");
my @xenvs_items   = ( "node-name", "xenv-state", "cpu-usage", "brick-name" );

my @initiators_metrics = ( "iops", "rd_iops", "wr_iops", "avg_latency", "rd_latency", "wr_latency", "bw", "rd_bw", "wr_bw" );
my @initiators_items   = ( "initiator-conn-state", "ig-id", "acc-size-of-rd", "port-type", "guid", "ig-name", "port-address", "num-of-conn-tars", "operating-system" );

my @initiator_groups_metrics = ( "iops", "rd_iops", "wr_iops", "bw", "rd_bw", "wr_bw" );
my @initiator_groups_items   = ( "num-of-initiators", "ig-id", "num-of-vols", "guid", "num-of-conn-tars" );

sub conf {
  $xormon = shift;
  $cfg    = shift;

  #
  # test main variables
  #
  unless ( exists $cfg->{'hostcfg_id'} )  { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")   && return "error"; }
  unless ( exists $cfg->{'hostalias'} )   { Xormon::error("Cannot continue, 'hostalias' is not configured!")    && return "error"; }
  $logger = XormonLogger->new('xtremio', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )     { $logger->error("Cannot continue, 'apiHost' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'apiPort'} )     { $logger->error("Cannot continue, 'apiPort' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'apiUsername'} ) { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} ) { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  unless ( exists $cfg->{'cluster'} )     { $logger->error("Cannot continue, 'cluster' is not configured!")     && return "error"; }
  $url          = "https://$cfg->{'apiHost'}:$cfg->{'apiPort'}";
  $cluster_name = $cfg->{'cluster'};
  $wrkdir       = "$act_dir/data/$cfg->{'hostcfg_id'}";
  $debug        = ( exists $cfg->{'debug'} ) ? $cfg->{'debug'} : 0;

  #
  # check available clusters
  #
  $logger->info("check_clusters      : START") if $debug == 9;
  unless ( check_clusters() ) {
    return "error";
  }
  $logger->info("check_clusters      : END") if $debug == 9;

  #
  # collect volumes cfg
  #
  $logger->info("volumes_conf        : START") if $debug == 9;
  collect_volumes_conf();
  $logger->info("volumes_conf        : END") if $debug == 9;

  return "successful";
}

sub perf {
  $xormon = shift;
  $cfg    = shift;

  #
  # test main variables
  #
  unless ( exists $cfg->{'hostcfg_id'} )  { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")   && return "error"; }
  unless ( exists $cfg->{'hostalias'} )   { $logger->error("Cannot continue, 'hostalias' is not configured!")   && return "error"; }
  $logger = XormonLogger->new('xtremio', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )     { $logger->error("Cannot continue, 'apiHost' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'apiPort'} )     { $logger->error("Cannot continue, 'apiPort' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'apiUsername'} ) { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} ) { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  unless ( exists $cfg->{'cluster'} )     { $logger->error("Cannot continue, 'cluster' is not configured!")     && return "error"; }
  $url          = "https://$cfg->{'apiHost'}:$cfg->{'apiPort'}";
  $cluster_name = $cfg->{'cluster'};
  $wrkdir       = "$act_dir/data/$cfg->{'hostcfg_id'}";
  $debug        = ( exists $cfg->{'debug'} ) ? $cfg->{'debug'} : 0;
  $sample_rate  = ( exists $cfg->{'sample_rate'} && Xormon::isdigit($cfg->{'sample_rate'}) ) ? $cfg->{'sample_rate'} : 300;
  my $status    = "error"; # status code

  if ( exists $cfg->{'XTREMIO_EXCLUDE_UNMAPPED_VOLUMES'} && $cfg->{'XTREMIO_EXCLUDE_UNMAPPED_VOLUMES'} eq "0" ) { $exclude_unmapped_volumes = 0; }

  #
  # get microservice data
  #
  my $service_data_in  = $xormon->getData();
  my $service_data_out = {};
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }
  if ( exists $service_data_in->{last_perf_timestamp} ) {
    $prev_perf_timestamp = $service_data_in->{last_perf_timestamp};
  }

  #
  # check available clusters + connection status
  #
  $logger->info("check_clusters      : START") if $debug == 9;
  if ( check_clusters() ) {
    my $conn_ret = $xormon->isConnected(1);
    $logger->info("isConnected: $conn_ret");
  }
  else {
    my $conn_ret = $xormon->isConnected(0, "Configured Cluster has not been found!");
    $logger->info("isConnected: $conn_ret");

    my $data_ret = $xormon->hasData(0, "Connection issue, cannot collect stats!");
    $logger->info("hasData: $data_ret");

    $logger->error( "Cannot continue, Configured Cluster has not been found!" . __FILE__ . ":" . __LINE__ ) && return $status;
  }
  $logger->info("check_clusters      : END") if $debug == 9;

  #
  # collect all the necessary data
  #
  collect_perf();

  #
  # set stat time for this agent run + perf data status
  #
  if ( defined $last_stat_timestamp && Xormon::isdigit($last_stat_timestamp) ) {
    $logger->info("set_time_range      :") if $debug == 9;
    set_time_range($last_stat_timestamp);

    my $data_ret = $xormon->hasData(1);
    $logger->info("hasData: $data_ret");
  }
  else {
    my $data_ret = $xormon->hasData(0, "Last stat timestamp does not exists!");
    $logger->info("hasData: $data_ret");

    $logger->error( "Cannot continue! Last stat timestamp does not exists! $!" . __FILE__ . ":" . __LINE__ ) && return $status;
  }

  #
  # conf/arch data
  #
  my $act_timestamp = time();
  if ( ! exists $service_data_in->{last_conf_timestamp} || ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) || $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) {
    $service_data_out->{last_conf_timestamp} = $act_timestamp;

    host_mapping();
    send_conf();
  }

  #
  # perf data
  #
  $logger->info("print_perf          : START") if $debug == 9;
  if (%sample_rate_intervals) {
    foreach ( sort { $a <=> $b } keys(%sample_rate_intervals) ) {
      if ( exists $sample_rate_intervals{$_}{'start_timestamp'} && exists $sample_rate_intervals{$_}{'end_timestamp'} ) {
        if ( defined $last_stat_timestamp && Xormon::isdigit($last_stat_timestamp) && $sample_rate_intervals{$_}{'end_timestamp'} <= $last_stat_timestamp ) {
          if ( defined $prev_perf_timestamp && Xormon::isdigit($prev_perf_timestamp) && $prev_perf_timestamp >= $sample_rate_intervals{$_}{'end_timestamp'} ) { next; } # skip perf file from previous MS run

          my $perf_out = prepare_perf( "$sample_rate_intervals{$_}{'start_timestamp'}", "$sample_rate_intervals{$_}{'end_timestamp'}" );
          my $rc       = $xormon->saveJSON2($perf_out);
          $logger->info("save perf: $sample_rate_intervals{$_}{'start_timestamp'} - $sample_rate_intervals{$_}{'end_timestamp'} : ".localtime($sample_rate_intervals{$_}{'start_timestamp'})." - ".localtime($sample_rate_intervals{$_}{'end_timestamp'}).": $rc");

          if ( $rc eq "201" ) {
            $status = "successful";
          }
          elsif ( $rc eq "401" ) {
            $status = "unauthorized";
            last;
          }
          else {
            $status = "error";
            last;
          }

          $last_perf_timestamp = $sample_rate_intervals{$_}{'end_timestamp'};
        }
      }
    }
  }
  $logger->info("print_perf          : END") if $debug == 9;

  #
  # Health status and Events
  #
  $logger->info("health_status       : START") if $debug == 9;
  health_status();
  $logger->info("health_status       : END") if $debug == 9;

  #
  # save microservice data
  #
  if ( defined $last_stat_timestamp && $last_stat_timestamp ne '' ) { $service_data_out->{last_stat_timestamp} = $last_stat_timestamp; }
  if ( defined $last_perf_timestamp && $last_perf_timestamp ne '' ) { $service_data_out->{last_perf_timestamp} = $last_perf_timestamp; }
  $xormon->saveData(\%{ $service_data_out });


  return $status;
}

sub connection_test {
  my $client = shift;
  $cfg       = shift;

  unless ( exists $cfg->{'cluster'} )     { $logger->error("Cannot continue, 'cluster' is not configured!")     && return "error"; }
  $url          = "https://$cfg->{'apiHost'}:$cfg->{'apiPort'}";
  $cluster_name = $cfg->{'cluster'};

  $connection_test = 1;
  my $test         = Test->new($client);

  #
  # test main variables
  #
  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiPort", "apiUsername", "apiPassword", "cluster");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Device cfg", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Device cfg", 1 , "");

  $logger = XormonLogger->new('xtremio', $cfg->{hostcfg_id}, $cfg->{hostalias} );

  #
  # test IP and port
  #
  my ( $code, $text ) = Xormon::conntest_tcp($cfg->{'apiHost'},$cfg->{'apiPort'});
  $test->addSection("API host network connection", $code, $text);
  if ( $code == 0 ) { return $test->stop(); }

  #
  # check available clusters + connection status
  #
  $logger->info("check_clusters      : START") if $debug == 9;
  if ( check_clusters() ) {
    $test->addSection("API authorization", 1 , \@rows);
    @rows = ();
  }
  else {
    push(@rows, "Configured Cluster has not been found!\n");
    push(@rows, "Connection failed!\n");
    $test->addSection("API authorization", 0 , \@rows);
    return $test->stop();
  }

  #
  # collect stats
  #
  cluster_perf();
  if ( defined $last_stat_timestamp && Xormon::isdigit($last_stat_timestamp) ) {
    push(@rows, "Stats found!\n");
    $test->addSection("API data test", 1 , \@rows);
  }
  else {
    push(@rows, "Array stats have not been collected!\n");
    $test->addSection("API data test", 0 , \@rows);
    return $test->stop();
  }

  return $test->stop();
}

sub health_status {
  my $main_status   = "ok";
  my @error_message = ();
  my @hs            = ();
  my %hs_data;

  my $sys_health_state = "";
  if ( defined $data{'cluster'}{$cluster_name}{'conf'}{'sys-health-state'} && $data{'cluster'}{$cluster_name}{'conf'}{'sys-health-state'} ne '' ) {
    if ( $data{'cluster'}{$cluster_name}{'conf'}{'sys-health-state'} ne "healthy" ) {
      $main_status = "error";
    }
    $sys_health_state = $data{'cluster'}{$cluster_name}{'conf'}{'sys-health-state'};
    push( @error_message, "sys-health-state : $data{'cluster'}{$cluster_name}{'conf'}{'sys-health-state'}" );
  }
  my $sys_state = "";
  if ( defined $data{'cluster'}{$cluster_name}{'conf'}{'sys-state'} && $data{'cluster'}{$cluster_name}{'conf'}{'sys-state'} ne '' ) {
    $sys_state = $data{'cluster'}{$cluster_name}{'conf'}{'sys-state'};
    push( @error_message, "sys-state        : $data{'cluster'}{$cluster_name}{'conf'}{'sys-state'}" );
  }
  push( @{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} }, { "sys_health_state" => $sys_health_state, "sys_state" => $sys_state } );

  #
  # Events/Alerts (last 90 days, critical events only)
  #
  my $act_time = time();
  my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime( $act_time - 7776000 );
  my $from_time = sprintf( "%4d-%02d-%02d %02d:%02d:%02d", $year + 1900, $month + 1, $day, $hour, $min, $sec );

  my $result = new_get_request("$url/api/json/v2/types/events?cluster-name=$cluster_name&from-date-time=$from_time&severity=critical");
  if ( defined $result ) {
    if ( defined $result->{'events'} && ref( $result->{'events'} ) eq "ARRAY" ) {
      foreach ( @{ $result->{'events'} } ) {
        my $event_code     = $_->{'event_code'}     ||= "";
        my $description    = $_->{'description'}    ||= "";
        my $entity_details = $_->{'entity_details'} ||= "";
        my $entity         = $_->{'entity'}         ||= "";
        my $timestamp      = $_->{'timestamp'}      ||= "";
        my $severity       = $_->{'severity'}       ||= "";

        #my $description_short = $description;
        #if ( length($description) > 100 ) {
        #  $description_short = substr( $description_short, 0, 100 );
        #  $description_short = "$description_short...";
        #}

        # add HS data to DB to be able to show them in the HS view
        push( @{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_events} }, { "timestamp" => $timestamp, "severity" => $severity, "entity" => $entity, "entity_details" => $entity_details, "description" => $description, "event_code" => $event_code } );
      }
    }
  }

  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_events} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_events} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_events});
  }
  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }

  push(@hs, { "item_id" => $cfg->{'hostcfg_id'}, "status" => $main_status, "data" => join("\n", @error_message) });

  # send statuses
  my $rc = $xormon->saveStatus(\@hs);
  $logger->info("save health status: $rc");

  # save HS data
  my $rc_hs = $xormon->saveConfJSON(\%hs_data,1);
  $logger->info("save health status table: $rc_hs");

  return 1;
}

sub collect_perf {
  #
  # conf
  #
  # cluster cfg
  $logger->info("get_cluster_conf    : START") if $debug == 9;
  get_cluster_conf();
  $logger->info("get_cluster_conf    : END") if $debug == 9;

  # volumes cfg
  $logger->info("volumes_conf        : START") if $debug == 9;
  get_volumes_conf();
  $logger->info("volumes_conf        : END") if $debug == 9;

  # Disks (SSDs)
  $logger->info("ssds_conf           : START") if $debug == 9;
  ssds_conf();
  $logger->info("ssds_conf           : END") if $debug == 9;

  # Node (XEnvs)
  $logger->info("xenvs_conf          : START") if $debug == 9;
  xenvs_conf();
  $logger->info("xenvs_conf          : END") if $debug == 9;

  # Hosts (Initiators)
  $logger->info("initiators_conf     : START") if $debug == 9;
  initiators_conf();
  $logger->info("initiators_conf     : END") if $debug == 9;

  # Host Group (Initiator Groups)
  $logger->info("initiators_gr_p     : START") if $debug == 9;
  initiator_groups_conf();
  $logger->info("initiators_gr_p     : END") if $debug == 9;

  #
  # perf
  #
  # cluster (Array Total - pool stats)
  $logger->info("cluster_perf        : START") if $debug == 9;
  cluster_perf();
  $logger->info("cluster_perf        : END") if $debug == 9;

  # volumes
  $logger->info("volumes_perf        : START") if $debug == 9;
  volumes_perf();
  $logger->info("volumes_perf        : END") if $debug == 9;

  # Disks (SSDs)
  $logger->info("ssds_perf           : START") if $debug == 9;
  ssds_perf();
  $logger->info("ssds_perf           : END") if $debug == 9;

  # Node (XEnvs)
  $logger->info("xenvs_perf          : START") if $debug == 9;
  xenvs_perf();
  $logger->info("xenvs_perf          : END") if $debug == 9;

  # Hosts (Initiators)
  $logger->info("initiators_perf     : START") if $debug == 9;
  initiators_perf();
  $logger->info("initiators_perf     : END") if $debug == 9;

  # Host Group (Initiator Groups)
  $logger->info("initiators_gr_c     : START") if $debug == 9;
  initiator_groups_perf();
  $logger->info("initiators_gr_c     : END") if $debug == 9;

  return 1;
}

sub send_conf {
  my @architecture;
  my %conf;

  #
  # DEVICE
  #
  # Architecture
  my %device_arch = (
    "item_id"    => $cfg->{'hostcfg_id'},
    "hostcfg_id" => $cfg->{'hostcfg_id'},
    "label"      => $cfg->{'hostalias'},
    "hw_type"    => "xtremio",
    "subsystem"  => "device",
    "parents"    => []
  );
  push(@architecture, \%device_arch);
  # conf
  $conf{ $cfg->{'hostcfg_id'} }{ip} = $cfg->{'apiHost'};
  if ( exists $data{'cluster'}{$cluster_name}{'conf'}{'sys-psnt-serial-number'} ) { $conf{ $cfg->{'hostcfg_id'} }{serial}             = $data{'cluster'}{$cluster_name}{'conf'}{'sys-psnt-serial-number'}; }
  #if ( exists $data{'cluster'}{$cluster_name}{'conf'}{'sys-sw-version'} )         { $conf{ $cfg->{'hostcfg_id'} }{'sys_sw_version'}   = $data{'cluster'}{$cluster_name}{'conf'}{'sys-sw-version'}; }
  if ( exists $data{'cluster'}{$cluster_name}{'conf'}{'sys-sw-version'} )         { $conf{ $cfg->{'hostcfg_id'} }{'version'}          = $data{'cluster'}{$cluster_name}{'conf'}{'sys-sw-version'}; }
  if ( exists $data{'cluster'}{$cluster_name}{'conf'}{'hardware-platform'} )      { $conf{ $cfg->{'hostcfg_id'} }{'model'}            = $data{'cluster'}{$cluster_name}{'conf'}{'hardware-platform'}; }
  if ( exists $data{'cluster'}{$cluster_name}{'conf'}{'sys-health-state'} )       { $conf{ $cfg->{'hostcfg_id'} }{'sys_health_state'} = $data{'cluster'}{$cluster_name}{'conf'}{'sys-health-state'}; }
  if ( exists $data{'cluster'}{$cluster_name}{'conf'}{'sys-state'} )              { $conf{ $cfg->{'hostcfg_id'} }{'sys_state'}        = $data{'cluster'}{$cluster_name}{'conf'}{'sys-state'}; }
  if ( exists $data{'cluster'}{$cluster_name}{'conf'}{'license-id'} )             { $conf{ $cfg->{'hostcfg_id'} }{'license_id'}       = $data{'cluster'}{$cluster_name}{'conf'}{'license-id'}; }

  #  my $id    = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."pool".$label) );

  #
  # Cluster / Pool (Array Total)
  #
  my $pool_id = "";
  if ( exists $data{'cluster'}{$cluster_name}{perf} ) {
    my $label = "Array Total";
    $pool_id  = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."pool".$label) );
    # Architecture
    my %arch = (
      "item_id"   => $pool_id,
      "label"     => $label,
      "hw_type"   => "xtremio",
      "subsystem" => "pool",
      "hostcfg_id" => $cfg->{'hostcfg_id'},
      "parents"   => [$cfg->{'hostcfg_id'}]
    );
    push(@architecture, \%arch);
    # conf
  }

  #
  # volumes
  #
  if ( exists $data{'volumes'} ) {
    foreach my $volume_name ( keys( %{ $data{'volumes'} } ) ) {
      if ( exists $data{'volumes'}{$volume_name}{perf} ) {
        if ( defined $data{'volumes'}{$volume_name}{conf}{'guid'} && $data{'volumes'}{$volume_name}{conf}{'guid'} ne '' ) {
          my $id = $data{'volumes'}{$volume_name}{conf}{'guid'};
          # Architecture
          my %arch = (
            "item_id"    => $id,
            "label"      => $volume_name,
            "hw_type"    => "xtremio",
            "subsystem"  => "volume",
            "hostcfg_id" => $cfg->{'hostcfg_id'},
            "parents"    => [$cfg->{'hostcfg_id'}]
          );
          if ( defined $pool_id && $pool_id ne '' ) { push( @{ $arch{parents} }, $pool_id );}
          push(@architecture, \%arch);
          # conf
          $conf{$id}{lun_id} = $id;
        }
      }
    }
  }

  #
  # drive
  #
  if ( exists $data{'ssds'} ) {
    foreach my $ssd_name ( keys( %{ $data{'ssds'} } ) ) {
      if ( exists $data{'ssds'}{$ssd_name}{perf} ) {
        if ( defined $data{'ssds'}{$ssd_name}{conf}{'guid'} && $data{'ssds'}{$ssd_name}{conf}{'guid'} ne '' ) {
          my $id = $data{'ssds'}{$ssd_name}{conf}{'guid'};
          # Architecture
          my %arch = (
            "item_id"    => $id,
            "label"      => $ssd_name,
            "hw_type"    => "xtremio",
            "subsystem"  => "drive",
            "hostcfg_id" => $cfg->{'hostcfg_id'},
            "parents"    => [$cfg->{'hostcfg_id'}]
          );
          push(@architecture, \%arch);
          # conf
        }
      }
    }
  }

  #
  # host
  #
  if ( exists $data{'initiators'} ) {
    foreach my $initiator_name ( keys( %{ $data{'initiators'} } ) ) {
      if ( exists $data{'initiators'}{$initiator_name}{perf} ) {
        if ( defined $data{'initiators'}{$initiator_name}{conf}{'guid'} && $data{'initiators'}{$initiator_name}{conf}{'guid'} ne '' ) {
          my $id = $data{'initiators'}{$initiator_name}{conf}{'guid'};
          # Architecture
          my %arch = (
            "item_id"    => $id,
            "label"      => $initiator_name,
            "hw_type"    => "xtremio",
            "subsystem"  => "host",
            "hostcfg_id" => $cfg->{'hostcfg_id'},
            "parents"    => [$cfg->{'hostcfg_id'}]
          );
          # add volumes
          if ( defined $data{'initiators'}{$initiator_name}{'conf'}{'ig-name'} && $data{'initiators'}{$initiator_name}{'conf'}{'ig-name'} ne '' ) {
            my $initiator_group_name = $data{'initiators'}{$initiator_name}{'conf'}{'ig-name'};
            if ( exists $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'} ) {
              if ( exists $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'} ) {
                foreach my $volume_name ( keys( %{ $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'} } ) ) {
                  if ( defined $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'}{$volume_name}{'id'} && $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'}{$volume_name}{'id'} ne '' ) {
                    my $volume_id = $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'}{$volume_name}{'id'};
                    push( @{ $arch{children} }, $volume_id );
                  }
                }
              }
            }
          }
          push(@architecture, \%arch);
          # conf
        }
      }
    }
  }

  #
  # host_group
  #
  if ( exists $data{'initiator_groups'} ) {
    foreach my $initiator_group_name ( keys( %{ $data{'initiator_groups'} } ) ) {
      if ( exists $data{'initiator_groups'}{$initiator_group_name}{perf} ) {
        if ( defined $data{'initiator_groups'}{$initiator_group_name}{conf}{'guid'} && $data{'initiator_groups'}{$initiator_group_name}{conf}{'guid'} ne '' ) {
          my $id = $data{'initiator_groups'}{$initiator_group_name}{conf}{'guid'};
          # Architecture
          my %arch = (
            "item_id"    => $id,
            "label"      => $initiator_group_name,
            "hw_type"    => "xtremio",
            "subsystem"  => "host_group",
            "hostcfg_id" => $cfg->{'hostcfg_id'},
            "parents"    => [$cfg->{'hostcfg_id'}]
          );
          # add volumes
          if ( exists $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'} ) {
            foreach my $volume_name ( keys( %{ $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'} } ) ) {
              if ( defined $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'}{$volume_name}{'id'} && $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'}{$volume_name}{'id'} ne '' ) {
                my $volume_id = $data{'initiator_groups'}{$initiator_group_name}{'conf'}{'volumes'}{$volume_name}{'id'};
                push( @{ $arch{children} }, $volume_id );
              }
            }
          }
          push(@architecture, \%arch);
          # conf
        }
      }
    }
  }

  #
  # node
  #
  my @nodes = ();
  if ( exists $data{'xenvs'} ) {
    foreach my $xenv ( keys( %{ $data{'xenvs'} } ) ) {
      if ( exists $data{'xenvs'}{$xenv}{perf} ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."node".$xenv) );
        # Architecture
        my %arch = (
          "item_id"    => $id,
          "label"      => $xenv,
          "hw_type"    => "xtremio",
          "subsystem"  => "node",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents"    => [$cfg->{'hostcfg_id'}]
        );
        push(@architecture, \%arch);
        # conf
        push(@nodes, $xenv);
      }
    }
  }

  # add nodes to the device cfg
  if ( scalar(@nodes) > 0 ) { $conf{ $cfg->{'hostcfg_id'} }{node_name} = join(", ", @nodes); }

  #save architecture
  my $rc_arch = $xormon->saveArchitecture(\@architecture);
  $logger->info("save architecture: $rc_arch");

  #save configuration
  my $rc_conf = $xormon->saveConfJSON(\%conf);
  $logger->info("save conf: $rc_conf");

  return 1;
}

sub host_mapping {

  # set volumes to host groups (from lun-mapping-list)
  if ( defined $data{'volumes'} && $data{'volumes'} ne '' ) {
    foreach my $volume_name ( keys( %{ $data{'volumes'} } ) ) {

      #print "$volume_name :\n";
      my $volume_id = "";
      if ( defined $data{'volumes'}{$volume_name}{conf}{'guid'} && $data{'volumes'}{$volume_name}{conf}{'guid'} ne '' ) {
        $volume_id = $data{'volumes'}{$volume_name}{conf}{'guid'};
      }

      if ( defined $data{'volumes'}{$volume_name}{conf}{'lun-mapping-list'} && ref( $data{'volumes'}{$volume_name}{conf}{'lun-mapping-list'} ) eq "ARRAY" ) {
        foreach ( @{ $data{'volumes'}{$volume_name}{conf}{'lun-mapping-list'} } ) {
          if ( ref($_) eq "ARRAY" ) {
            my @arr = @{$_};

            #print "  host: $arr[1]\n";
            foreach my $line (@arr) {
              if ( ref($line) eq "ARRAY" ) {
                my @arr_n = @{$line};
                if ( defined $arr_n[1] && $arr_n[1] ne '' ) {
                  my $host_group = $arr_n[1];
                  if ( exists $data{'initiator_groups'}{$host_group} ) {
                    $data{'initiator_groups'}{$host_group}{'conf'}{'volumes'}{$volume_name}{'name'} = $volume_name;
                    $data{'initiator_groups'}{$host_group}{'conf'}{'volumes'}{$volume_name}{'id'}   = $volume_id;
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub check_clusters {
  #
  # check available clusters in this XMS
  #
  my $result = new_get_request("$url/api/json/v2/types/clusters");

  if ( defined $result ) {
    if ( defined $result->{'clusters'} && ref( $result->{'clusters'} ) eq "ARRAY" ) {
      foreach ( @{ $result->{'clusters'} } ) {
        if ( defined $_->{'name'} && $_->{'name'} eq "$cluster_name" ) {
          $data{'cluster'}{$cluster_name}{'conf'}{'name'} = $cluster_name;
          if ( defined $_->{'href'} && $_->{'href'} ne '' ) {
            my $cluster_id = $_->{'href'};
            $cluster_id =~ s/$url\/api\/json\/v2\/types\/clusters\///;

            $data{'cluster'}{$cluster_name}{'conf'}{'href'}       = $_->{'href'};
            $data{'cluster'}{$cluster_name}{'conf'}{'cluster_id'} = $cluster_id;

            $logger->info("cluster_conf    : cluster \"$cluster_name\" was found! $_->{'href'}") if $debug == 9;
            last;
          }
        }
      }
    }
    if ( !exists $data{'cluster'}{$cluster_name} ) {
      $logger->error("Cluster \"$cluster_name\" was not found in the XMS!");
      $logger->error("Try this URL in your web browser: \"$url/api/json/v2/types/clusters\"");
      $logger->error("You should see there your configured cluster name: \"$cluster_name\"! Exiting...") && return 0;
    }
  }
  else {
    $logger->error("No clusters were found in the XMS!");
    $logger->error("Try this URL in your web browser: \"$url/api/json/v2/types/clusters\"");
    $logger->error("You should see there your configured cluster name: \"$cluster_name\"! Exiting...") && return 0;
  }

  return 1;
}

sub get_cluster_conf {
  #
  # Get cluster information
  #
  my $cluster_metrics = "&prop=".join("&prop=", @cluster_items);

  if ( defined $data{'cluster'}{$cluster_name}{'conf'}{'href'} && $data{'cluster'}{$cluster_name}{'conf'}{'href'} ne '' ) {
    my $result = new_get_request("$data{'cluster'}{$cluster_name}{'conf'}{'href'}?cluster-name=$cluster_name$cluster_metrics");
    unless ( defined $result ) { return 1; }

    #print Dumper $result;

    if ( defined $result->{'content'} && ref( $result->{'content'} ) eq "HASH" ) {
      foreach ( keys( %{ $result->{'content'} } ) ) {
        if ( defined $result->{'content'}{$_} ) {
          $data{'cluster'}{$cluster_name}{'conf'}{$_} = $result->{'content'}{$_};

          #print "$_    :     $result->{'content'}{$_}\n";
          #if ( ref($result->{'content'}{$_}) eq "ARRAY" ) {
          #  foreach my $arr_string ( @{ $result->{'content'}{$_}}) {
          #    print "$_ : $arr_string\n";
          #  }
          #}
        }
      }
    }
  }

  return 1;
}

sub get_volumes_conf {
  my $vol_cfg_file = "$wrkdir/volumes_cfg.json";

  #
  # load volumes_cfg.json which is created once per 6 hours in 'conf' job
  #
  if ( -f $vol_cfg_file && Xormon::file_time_diff($vol_cfg_file) < 43200 ) { # use max 12 hours old cfg only
    $logger->info("volumes_conf        : read_json $vol_cfg_file") if $debug == 9;
    $data{'volumes'} = Xormon::read_json($vol_cfg_file);
  }

  return 1;
}

sub collect_volumes_conf {
  my $vol_cfg_file = "$wrkdir/volumes_cfg.json";
  my $vol_metrics  = "&prop=".join("&prop=", @volumes_items);

  my $result = new_get_request("$url/api/json/v2/types/volumes?cluster-name=$cluster_name");

  unless ( defined $result ) { return 0; }

  my $vol_idx = 0;
  if ( defined $result->{'volumes'} && ref( $result->{'volumes'} ) eq "ARRAY" ) {
    foreach ( @{ $result->{'volumes'} } ) {
      if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
        my $volume_name = $_->{'name'};
        $data{'volumes'}{$volume_name}{'conf'}{'name'} = $volume_name;

        if ( defined $_->{'href'} && $_->{'href'} ne '' ) {
          $vol_idx++;
          $data{'volumes'}{$volume_name}{'conf'}{'href'} = $_->{'href'};

          my $vol_info = new_get_request("$_->{'href'}?cluster-name=$cluster_name$vol_metrics");

          if ( defined $vol_info ) {

            #print Dumper $vol_info;
            if ( defined $vol_info->{'content'} && ref( $vol_info->{'content'} ) eq "HASH" ) {
              foreach my $metric ( keys( %{ $vol_info->{'content'} } ) ) {
                if ( defined $vol_info->{'content'}{$metric} ) {
                  $data{'volumes'}{$volume_name}{'conf'}{$metric} = $vol_info->{'content'}{$metric};
                }
              }
            }
          }
        }
      }
    }
  }

  if ( $vol_idx == 0 && -f $vol_cfg_file ) {
    # remove vols cfg if there are no volumes
    $logger->info("volumes_conf    : volumes=$vol_idx : remove $vol_cfg_file") if $debug == 9;
    unlink($vol_cfg_file);
  }

  if ( $vol_idx > 0 ) {
    # collection of the volumes cfg can take long time (approx. 80 minutes for approx. 6k volumes (1800 maped volumes) )
    # save current cfg and collect new cfg only once per 6 hours
    $logger->info("volumes_conf    : volumes=$vol_idx : save_as_json $vol_cfg_file") if $debug == 9;
    Xormon::save_as_json($vol_cfg_file,$data{'volumes'});
  }

  return 1;
}

sub ssds_conf {
  my $ssd_metrics = "&prop=".join("&prop=", @ssds_items);
  my $result      = new_get_request("$url/api/json/v2/types/ssds?cluster-name=$cluster_name");

  unless ( defined $result ) { return 0; }

  #print Dumper $result;
  if ( defined $result->{'ssds'} && ref( $result->{'ssds'} ) eq "ARRAY" ) {
    foreach ( @{ $result->{'ssds'} } ) {
      if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
        my $ssd_name = $_->{'name'};
        $data{'ssds'}{$ssd_name}{'conf'}{'name'} = $ssd_name;

        if ( defined $_->{'href'} && $_->{'href'} ne '' ) {
          $data{'ssds'}{$ssd_name}{'conf'}{'href'} = $_->{'href'};

          my $ssd_info = new_get_request("$_->{'href'}?cluster-name=$cluster_name$ssd_metrics");

          if ( defined $ssd_info ) {
            #print Dumper $ssd_info;
            if ( defined $ssd_info->{'content'} && ref( $ssd_info->{'content'} ) eq "HASH" ) {
              foreach my $metric ( keys( %{ $ssd_info->{'content'} } ) ) {
                if ( defined $ssd_info->{'content'}{$metric} ) {
                  $data{'ssds'}{$ssd_name}{'conf'}{$metric} = $ssd_info->{'content'}{$metric};
                }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub xenvs_conf {
  my $xenv_metrics = "&prop=".join("&prop=", @xenvs_items);
  my $result       = new_get_request("$url/api/json/v2/types/xenvs?cluster-name=$cluster_name");

  unless ( defined $result ) { return 0; }

  #print Dumper $result;
  if ( defined $result->{'xenvs'} && ref( $result->{'xenvs'} ) eq "ARRAY" ) {
    foreach ( @{ $result->{'xenvs'} } ) {
      if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
        my $xenv_name = $_->{'name'};
        $data{'xenvs'}{$xenv_name}{'conf'}{'name'} = $xenv_name;

        if ( defined $_->{'href'} && $_->{'href'} ne '' ) {
          $data{'xenvs'}{$xenv_name}{'conf'}{'href'} = $_->{'href'};

          my $xenv_info = new_get_request("$_->{'href'}?cluster-name=$cluster_name$xenv_metrics");

          if ( defined $xenv_info ) {
            #print Dumper $xenv_info;
            if ( defined $xenv_info->{'content'} && ref( $xenv_info->{'content'} ) eq "HASH" ) {
              foreach my $metric ( keys( %{ $xenv_info->{'content'} } ) ) {
                if ( defined $xenv_info->{'content'}{$metric} ) {
                  $data{'xenvs'}{$xenv_name}{'conf'}{$metric} = $xenv_info->{'content'}{$metric};
                }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub initiators_conf {
  my $initiator_metrics = "&prop=".join("&prop=", @initiators_items);
  my $result            = new_get_request("$url/api/json/v2/types/initiators?cluster-name=$cluster_name");

  unless ( defined $result ) { return 0; }

  #print Dumper $result;
  if ( defined $result->{'initiators'} && ref( $result->{'initiators'} ) eq "ARRAY" ) {
    foreach ( @{ $result->{'initiators'} } ) {
      if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
        my $initiator_name = $_->{'name'};
        $data{'initiators'}{$initiator_name}{'conf'}{'name'} = $initiator_name;

        if ( defined $_->{'href'} && $_->{'href'} ne '' ) {
          $data{'initiators'}{$initiator_name}{'conf'}{'href'} = $_->{'href'};

          my $initiator_info = new_get_request("$_->{'href'}?cluster-name=$cluster_name$initiator_metrics");

          if ( defined $initiator_info ) {
            #print Dumper $initiator_info;
            if ( defined $initiator_info->{'content'} && ref( $initiator_info->{'content'} ) eq "HASH" ) {
              foreach my $metric ( keys( %{ $initiator_info->{'content'} } ) ) {
                if ( defined $initiator_info->{'content'}{$metric} ) {
                  $data{'initiators'}{$initiator_name}{'conf'}{$metric} = $initiator_info->{'content'}{$metric};
                }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub initiator_groups_conf {
  my $initiator_metrics = "&prop=".join("&prop=", @initiator_groups_items);
  my $result            = new_get_request("$url/api/json/v2/types/initiator-groups?cluster-name=$cluster_name");

  unless ( defined $result ) { return 0; }

  #print Dumper $result;
  if ( defined $result->{'initiator-groups'} && ref( $result->{'initiator-groups'} ) eq "ARRAY" ) {
    foreach ( @{ $result->{'initiator-groups'} } ) {
      if ( defined $_->{'name'} && $_->{'name'} ne '' ) {
        my $initiator_name = $_->{'name'};
        $data{'initiator_groups'}{$initiator_name}{'conf'}{'name'} = $initiator_name;

        if ( defined $_->{'href'} && $_->{'href'} ne '' ) {
          $data{'initiator_groups'}{$initiator_name}{'conf'}{'href'} = $_->{'href'};

          my $initiator_info = new_get_request("$_->{'href'}?cluster-name=$cluster_name$initiator_metrics");

          if ( defined $initiator_info ) {
            #print Dumper $initiator_info;
            if ( defined $initiator_info->{'content'} && ref( $initiator_info->{'content'} ) eq "HASH" ) {
              foreach my $metric ( keys( %{ $initiator_info->{'content'} } ) ) {
                if ( defined $initiator_info->{'content'}{$metric} ) {
                  $data{'initiator_groups'}{$initiator_name}{'conf'}{$metric} = $initiator_info->{'content'}{$metric};
                }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub cluster_perf {
  my $cluster_metrics = "&prop=".join("&prop=", @cluster_metrics);

  if ( defined $data{'cluster'}{$cluster_name} && $data{'cluster'}{$cluster_name} ne '' ) {
    my $result = new_get_request("$url/api/json/v2/types/performance?cluster-name=$cluster_name&entity=Cluster&entity-name=$cluster_name&time-frame=last_hour&granularity=one_minute$cluster_metrics");

    if ( defined $result ) {
      #print Dumper $result;
      parse_perf_output( "cluster", "$cluster_name", $result );
    }
  }

  return 1;
}

sub volumes_perf {
  my $vol_metrics = "&prop=".join("&prop=", @volumes_metrics);

  if ( defined $data{'volumes'} && $data{'volumes'} ne '' ) {
    my $vol_idx = 0;
    foreach my $volume_name ( keys( %{ $data{'volumes'} } ) ) {

      # approx since version 2.81-22 are unmapped volumes excluded, you can disable it by the following variable
      if ( $exclude_unmapped_volumes == 1 && exists $data{'volumes'}{$volume_name}{conf}{'num-of-lun-mappings'} && Xormon::isdigit( $data{'volumes'}{$volume_name}{conf}{'num-of-lun-mappings'} ) && $data{'volumes'}{$volume_name}{conf}{'num-of-lun-mappings'} == 0 ) { next; }    # do not collect perf data for unmapped volumes

      if ( defined $data{'volumes'}{$volume_name}{'conf'}{'index'} && $data{'volumes'}{$volume_name}{'conf'}{'index'} ne '' ) {
        $vol_idx++;
        my $index  = $data{'volumes'}{$volume_name}{'conf'}{'index'};
        my $result = new_get_request("$url/api/json/v2/types/performance?cluster-name=$cluster_name&entity=Volume&entity-index=$index&time-frame=last_hour&granularity=one_minute$vol_metrics");
        if ( defined $result ) {
          parse_perf_output( "volumes", "$volume_name", $result );
        }
      }
    }
  }

  return 1;
}

sub ssds_perf {
  my $ssd_metrics = "&prop=".join("&prop=", @ssds_metrics);

  if ( defined $data{'ssds'} && $data{'ssds'} ne '' ) {
    my $ssd_idx = 0;
    foreach my $ssd_name ( keys( %{ $data{'ssds'} } ) ) {
      $ssd_idx++;
      my $result = new_get_request("$url/api/json/v2/types/performance?cluster-name=$cluster_name&entity=SSD&entity-name=$ssd_name&time-frame=last_hour&granularity=one_minute$ssd_metrics");
      if ( defined $result ) {
        parse_perf_output( "ssds", "$ssd_name", $result );
      }
    }
  }

  return 1;
}

sub xenvs_perf {
  my $xenv_metrics = "&prop=".join("&prop=", @xenvs_metrics);

  if ( defined $data{'xenvs'} && $data{'xenvs'} ne '' ) {
    my $xenv_idx = 0;
    foreach my $xenv_name ( keys( %{ $data{'xenvs'} } ) ) {
      $xenv_idx++;
      my $result = new_get_request("$url/api/json/v2/types/performance?cluster-name=$cluster_name&entity=XEnv&entity-name=$xenv_name&time-frame=last_hour&granularity=one_minute$xenv_metrics");
      if ( defined $result ) {
        parse_perf_output( "xenvs", "$xenv_name", $result );
      }
    }
  }

  return 1;
}

sub initiators_perf {
  my $initiator_metrics = "&prop=".join("&prop=", @initiators_metrics);

  if ( defined $data{'initiators'} && $data{'initiators'} ne '' ) {
    my $initiator_idx = 0;
    foreach my $initiator_name ( keys( %{ $data{'initiators'} } ) ) {
      $initiator_idx++;
      my $result = new_get_request("$url/api/json/v2/types/performance?cluster-name=$cluster_name&entity=Initiator&entity-name=$initiator_name&time-frame=last_hour&granularity=one_minute$initiator_metrics");
      if ( defined $result ) {
        parse_perf_output( "initiators", "$initiator_name", $result );
      }
    }
  }

  return 1;
}

sub initiator_groups_perf {
  my $initiator_metrics = "&prop=".join("&prop=", @initiator_groups_metrics);

  if ( defined $data{'initiator_groups'} && $data{'initiator_groups'} ne '' ) {
    my $initiator_idx = 0;
    foreach my $initiator_name ( keys( %{ $data{'initiator_groups'} } ) ) {
      $initiator_idx++;
      my $result = new_get_request("$url/api/json/v2/types/performance?cluster-name=$cluster_name&entity=InitiatorGroup&entity-name=$initiator_name&time-frame=last_hour&granularity=one_minute$initiator_metrics");
      if ( defined $result ) {
        parse_perf_output( "initiator_groups", "$initiator_name", $result );
      }
    }
  }

  return 1;
}

sub parse_perf_output {
  my $section = shift;
  my $name    = shift;
  my $result  = shift;

  my $last_stat_timestamp_tmp = "";

  if ( defined $result->{'members'} && ref( $result->{'members'} ) eq "ARRAY" ) {
    my @header = @{ $result->{'members'} };

    if ( defined $result->{'counters'} && ref( $result->{'counters'} ) eq "ARRAY" ) {
      my $data_idx = 0;
      foreach my $arr ( @{ $result->{'counters'} } ) {
        $data_idx++;
        if ( ref($arr) eq "ARRAY" ) {
          my $metric_idx     = 0;
          my $stat_timestamp = "";
          my $stat_date      = "";
          foreach my $line ( @{$arr} ) {
            if ( !defined $line || $line eq "" ) { next; }
            chomp $line;

            #print "$header[$metric_idx]  :  $line\n";
            if ( defined $header[$metric_idx] && $header[$metric_idx] eq "timestamp" && Xormon::isdigit($line) ) {
              $stat_timestamp = $line / 1000;    # from ms to sec.
            }

            #if ( defined $stat_timestamp && $stat_timestamp >= $start_timestamp && $stat_timestamp < $end_timestamp ) {
            #  #print "$stat_date : $header[$metric_idx] : $line\n";
            #  $data{$section}{$name}{'perf'}{$stat_timestamp}{$header[$metric_idx]} = $line;
            #  if ( defined $header[$metric_idx] && $header[$metric_idx] eq "timestamp" && Xormon::isdigit($line) ) {
            #    $stat_date = localtime($stat_timestamp);
            #    $data{$section}{$name}{'perf'}{$stat_timestamp}{'stat_date'} = $stat_date;
            #  }
            #}

            if ( defined $stat_timestamp && Xormon::isdigit($stat_timestamp) ) {
              $header[$metric_idx] =~ s/^avg__//;
              if ( $header[$metric_idx] =~ m/^max__/ && ( $header[$metric_idx] =~ m/dedup_ratio/ || $header[$metric_idx] =~ m/data_reduction_ratio/ || $header[$metric_idx] =~ m/compression_factor/ ) ) {
                # allow max metrics/values only for these metrics
                $header[$metric_idx] =~ s/^max__//;
              }

              #print "$stat_date : $header[$metric_idx] : $line\n";
              $data{$section}{$name}{'perf'}{$stat_timestamp}{ $header[$metric_idx] } = $line;
              if ( defined $header[$metric_idx] && $header[$metric_idx] eq "timestamp" && Xormon::isdigit($line) ) {
                $stat_date = localtime($stat_timestamp);
                $data{$section}{$name}{'perf'}{$stat_timestamp}{'stat_date'} = $stat_date;

                if ( $section eq "cluster" ) {
                  $last_stat_timestamp_tmp = $stat_timestamp;
                }
              }
            }

            $metric_idx++;
          }
        }
      }
    }
  }

  if ( $section eq "cluster" ) {
    if ( defined $last_stat_timestamp_tmp && Xormon::isdigit($last_stat_timestamp_tmp) ) {
      $last_stat_timestamp_tmp = int( $last_stat_timestamp_tmp / $sample_rate );
      $last_stat_timestamp_tmp = $last_stat_timestamp_tmp * $sample_rate;

      if ( defined $last_stat_timestamp && Xormon::isdigit($last_stat_timestamp) ) {
        if ( $last_stat_timestamp > $last_stat_timestamp_tmp ) {
          $last_stat_timestamp = $last_stat_timestamp_tmp;
        }
      }
      else {
        $last_stat_timestamp = $last_stat_timestamp_tmp;
      }
    }
  }

  return 1;
}

sub prepare_perf {
  my $interval_start_timestamp = shift;
  my $interval_end_timestamp   = shift;

  my $timestamp = $interval_end_timestamp;
  my $date      = localtime($timestamp);

  $logger->info("perf sample          : $timestamp : $date") if $debug == 9;

  my %data_out;

  #
  # Cluster / Pool (Array Total) + device
  #
  if ( exists $data{'cluster'}{$cluster_name}{perf} ) {
    my $label = "Array Total";
    my $id    = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."pool".$label) );

    my $cap_tot          = "";
    my $cap_used         = "";
    my $cap_free         = "";
    my $oversubscription = "";
    my $virtual_cap      = "";

    my $io_rate              = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "iops" );
    my $read_io              = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "rd_iops" );
    my $write_io             = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "wr_iops" );
    my $data_rate            = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "bw" ); # Cluster’s total realtime read and write bandwidth, in MB per second
    my $read                 = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "rd_bw" );
    my $write                = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "wr_bw" );
    my $resp_t               = sum_value_latency( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "avg_latency", "iops" );
    my $resp_t_r             = sum_value_latency( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "rd_latency",  "rd_iops" );
    my $resp_t_w             = sum_value_latency( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "wr_latency",  "wr_iops" );
    my $compression_factor   = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "compression_factor" );
    my $data_reduction_ratio = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "data_reduction_ratio" );
    my $dedup_ratio          = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "cluster", "$cluster_name", "dedup_ratio" );

    if ( Xormon::isdigit($io_rate) )   { $data_out{xtremio}{pool}{$id}{$timestamp}{io_rate}   = Xormon::formatNumberByType("int",$io_rate); }
    if ( Xormon::isdigit($read_io) )   { $data_out{xtremio}{pool}{$id}{$timestamp}{read_io}   = Xormon::formatNumberByType("int",$read_io); }
    if ( Xormon::isdigit($write_io) )  { $data_out{xtremio}{pool}{$id}{$timestamp}{write_io}  = Xormon::formatNumberByType("int",$write_io); }
    if ( Xormon::isdigit($data_rate) ) { $data_out{xtremio}{pool}{$id}{$timestamp}{data_rate} = Xormon::formatNumberByType("int",Xormon::returnBytes($data_rate,"MB")); } # Cluster’s total realtime read and write bandwidth, in MB per second
    if ( Xormon::isdigit($read) )      { $data_out{xtremio}{pool}{$id}{$timestamp}{read}      = Xormon::formatNumberByType("int",Xormon::returnBytes($read,"MB")); }
    if ( Xormon::isdigit($write) )     { $data_out{xtremio}{pool}{$id}{$timestamp}{write}     = Xormon::formatNumberByType("int",Xormon::returnBytes($write,"MB")); }
    if ( Xormon::isdigit($resp_t) )    { $data_out{xtremio}{pool}{$id}{$timestamp}{resp_t}    = Xormon::formatNumberByType("float4",$resp_t / 1000); }      # from usec to msec
    if ( Xormon::isdigit($resp_t_r) )  { $data_out{xtremio}{pool}{$id}{$timestamp}{resp_t_r}  = Xormon::formatNumberByType("float4",$resp_t_r / 1000); }    # from usec to msec
    if ( Xormon::isdigit($resp_t_w) )  { $data_out{xtremio}{pool}{$id}{$timestamp}{resp_t_w}  = Xormon::formatNumberByType("float4",$resp_t_w / 1000); }    # from usec to msec

    if ( Xormon::isdigit($compression_factor) )   { $data_out{xtremio}{pool}{$id}{$timestamp}{total_comp_factor}    = Xormon::formatNumberByType("float4",$compression_factor); }   # 'compression-factor' => '2.1071556587'
    if ( Xormon::isdigit($data_reduction_ratio) ) { $data_out{xtremio}{pool}{$id}{$timestamp}{data_reduction_ratio} = Xormon::formatNumberByType("float4",$data_reduction_ratio); } # 'data-reduction-ratio' => '7.2549386038',
    if ( Xormon::isdigit($dedup_ratio) )          { $data_out{xtremio}{pool}{$id}{$timestamp}{dedupe_ratio}         = Xormon::formatNumberByType("float4",$dedup_ratio); }          # 'dedup-ratio' => '3.443000793',

    if ( defined $data{'cluster'}{$cluster_name}{'conf'}{'ud-ssd-space'} && Xormon::isdigit( $data{'cluster'}{$cluster_name}{'conf'}{'ud-ssd-space'} ) ) {
      $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total} = Xormon::returnBytes($data{'cluster'}{$cluster_name}{'conf'}{'ud-ssd-space'},"KB");
      #$data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total} = $data{'cluster'}{$cluster_name}{'conf'}{'ud-ssd-space'}; # already in B ??? confirmation needed
    }
    if ( defined $data{'cluster'}{$cluster_name}{'conf'}{'ud-ssd-space-in-use'} && Xormon::isdigit( $data{'cluster'}{$cluster_name}{'conf'}{'ud-ssd-space-in-use'} ) ) {
      $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_used} = Xormon::returnBytes($data{'cluster'}{$cluster_name}{'conf'}{'ud-ssd-space-in-use'},"KB");
      #$data_out{xtremio}{pool}{$id}{$timestamp}{capacity_used} = $data{'cluster'}{$cluster_name}{'conf'}{'ud-ssd-space-in-use'}; # already in B ??? confirmation needed
    }
    # capacity_free
    if ( exists $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total} && Xormon::isdigit($data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total}) && exists $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_used} && Xormon::isdigit($data_out{xtremio}{pool}{$id}{$timestamp}{capacity_used}) ) {
      $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_free}         = $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total} - $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_used};
      $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_used_percent} = $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_used} / ( $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total} / 100 );
    }
    if ( defined $data{'cluster'}{$cluster_name}{'conf'}{'vol-size'} && Xormon::isdigit( $data{'cluster'}{$cluster_name}{'conf'}{'vol-size'} ) ) {
      $data_out{xtremio}{pool}{$id}{$timestamp}{virtual_capacity_total} = Xormon::returnBytes($data{'cluster'}{$cluster_name}{'conf'}{'vol-size'},"KB"); # Volume size (in KB) as exposed to initiators
    }
    # oversubscription = vol-size - ud-ssd-space (volume total cap - cluster total cap)
    if ( exists $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total} && Xormon::isdigit($data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total}) && exists $data_out{xtremio}{pool}{$id}{$timestamp}{virtual_capacity_total} && Xormon::isdigit($data_out{xtremio}{pool}{$id}{$timestamp}{virtual_capacity_total}) ) {
      if ( $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total} < $data_out{xtremio}{pool}{$id}{$timestamp}{virtual_capacity_total} ) {
        $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_oversubscr}         = $data_out{xtremio}{pool}{$id}{$timestamp}{virtual_capacity_total} - $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total};
        $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_oversubscr_percent} = int( $data_out{xtremio}{pool}{$id}{$timestamp}{virtual_capacity_total} / $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_total} ) * 100;
      }
      else {
        $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_oversubscr}         = 0;
        $data_out{xtremio}{pool}{$id}{$timestamp}{capacity_oversubscr_percent} = 0;
      }
    }

    # calculate block size
    my $data_act = $data_out{xtremio}{pool}{$id}{$timestamp};
    if ( exists $data_act->{io_rate} && Xormon::isdigit($data_act->{io_rate}) && exists $data_act->{data_rate} ) {
      if ( $data_act->{io_rate} == 0 ) {
        $data_out{xtremio}{pool}{$id}{$timestamp}{block_size} = 0;
      }
      else {
        $data_out{xtremio}{pool}{$id}{$timestamp}{block_size} = Xormon::formatNumberByType("float4",$data_act->{data_rate} / $data_act->{io_rate});
      }
    }
    if ( exists $data_act->{read_io} && Xormon::isdigit($data_act->{read_io}) && exists $data_act->{read} ) {
      if ( $data_act->{read_io} == 0 ) {
        $data_out{xtremio}{pool}{$id}{$timestamp}{block_size_r} = 0;
      }
      else {
        $data_out{xtremio}{pool}{$id}{$timestamp}{block_size_r} = Xormon::formatNumberByType("float4",$data_act->{read} / $data_act->{read_io});
      }
    }
    if ( exists $data_act->{write_io} && Xormon::isdigit($data_act->{write_io}) && exists $data_act->{write} ) {
      if ( $data_act->{write_io} == 0 ) {
        $data_out{xtremio}{pool}{$id}{$timestamp}{block_size_w} = 0;
      }
      else {
        $data_out{xtremio}{pool}{$id}{$timestamp}{block_size_w} = Xormon::formatNumberByType("float4",$data_act->{write} / $data_act->{write_io});
      }
    }

    #
    # device
    #
    #$data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp} = $data_out{xtremio}{pool}{$id}{$timestamp}; # cannot be used, e.g. block size must be excluded here
    my $pool_data = $data_out{xtremio}{pool}{$id}{$timestamp};
    if ( exists $pool_data->{io_rate} )                     { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{io_rate}                     = $pool_data->{io_rate}; }
    if ( exists $pool_data->{read_io} )                     { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{read_io}                     = $pool_data->{read_io}; }
    if ( exists $pool_data->{write_io} )                    { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{write_io}                    = $pool_data->{write_io}; }
    if ( exists $pool_data->{data_rate} )                   { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{data_rate}                   = $pool_data->{data_rate}; }
    if ( exists $pool_data->{read} )                        { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{read}                        = $pool_data->{read}; }
    if ( exists $pool_data->{write} )                       { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{write}                       = $pool_data->{write}; }
    if ( exists $pool_data->{resp_t} )                      { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{resp_t}                      = $pool_data->{resp_t}; }
    if ( exists $pool_data->{resp_t_r} )                    { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{resp_t_r}                    = $pool_data->{resp_t_r}; }
    if ( exists $pool_data->{resp_t_w} )                    { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{resp_t_w}                    = $pool_data->{resp_t_w}; }
    if ( exists $pool_data->{capacity_total} )              { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{capacity_total}              = $pool_data->{capacity_total}; }
    if ( exists $pool_data->{capacity_used} )               { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{capacity_used}               = $pool_data->{capacity_used}; }
    if ( exists $pool_data->{capacity_free} )               { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{capacity_free}               = $pool_data->{capacity_free}; }
    if ( exists $pool_data->{capacity_used_percent} )       { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{capacity_used_percent}       = $pool_data->{capacity_used_percent}; }
    if ( exists $pool_data->{virtual_capacity_total} )      { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{virtual_capacity_total}      = $pool_data->{virtual_capacity_total}; }
    if ( exists $pool_data->{capacity_oversubscr} )         { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{capacity_oversubscr}         = $pool_data->{capacity_oversubscr}; }
    if ( exists $pool_data->{capacity_oversubscr_percent} ) { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{capacity_oversubscr_percent} = $pool_data->{capacity_oversubscr_percent}; }
    #if ( exists $pool_data->{block_size} )             { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{block_size}             = $pool_data->{block_size}; }
    #if ( exists $pool_data->{block_size_r} )           { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{block_size_r}           = $pool_data->{block_size_r}; }
    #if ( exists $pool_data->{block_size_w} )           { $data_out{xtremio}{device}{ $cfg->{'hostcfg_id'} }{$timestamp}{block_size_w}           = $pool_data->{block_size_w}; }
  }

  #
  # volumes
  #
  if ( exists $data{'volumes'} ) {
    foreach my $volume_name ( keys( %{ $data{'volumes'} } ) ) {
      if ( exists $data{'volumes'}{$volume_name}{perf} ) {
        my $id = "";

        if ( defined $data{'volumes'}{$volume_name}{conf}{'guid'} && $data{'volumes'}{$volume_name}{conf}{'guid'} ne '' ) {
          $id = $data{'volumes'}{$volume_name}{conf}{'guid'};
        }
        else {
          $logger->error("print_perf : Volume ID \(guid\) was not found for volume \"$volume_name\"! Volume skipped...") && next;
        }
        if ( defined $data{'volumes'}{$volume_name}{conf}{'vol-size'} && Xormon::isdigit( $data{'volumes'}{$volume_name}{conf}{'vol-size'} ) ) {
          $data_out{xtremio}{volume}{$id}{$timestamp}{capacity_total} = Xormon::returnBytes($data{'volumes'}{$volume_name}{conf}{'vol-size'},"KB");
        }

        my $io_rate   = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "volumes", "$volume_name", "iops" );
        my $read_io   = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "volumes", "$volume_name", "rd_iops" );
        my $write_io  = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "volumes", "$volume_name", "wr_iops" );
        my $data_rate = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "volumes", "$volume_name", "bw" );
        my $read      = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "volumes", "$volume_name", "rd_bw" );
        my $write     = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "volumes", "$volume_name", "wr_bw" );
        my $resp_t   = sum_value_latency( "$interval_start_timestamp", "$interval_end_timestamp", "volumes", "$volume_name", "avg_latency", "iops" );
        my $resp_t_r = sum_value_latency( "$interval_start_timestamp", "$interval_end_timestamp", "volumes", "$volume_name", "rd_latency",  "rd_iops" );
        my $resp_t_w = sum_value_latency( "$interval_start_timestamp", "$interval_end_timestamp", "volumes", "$volume_name", "wr_latency",  "wr_iops" );

        if ( Xormon::isdigit($io_rate) )   { $data_out{xtremio}{volume}{$id}{$timestamp}{io_rate}   = Xormon::formatNumberByType("int",$io_rate); }
        if ( Xormon::isdigit($read_io) )   { $data_out{xtremio}{volume}{$id}{$timestamp}{read_io}   = Xormon::formatNumberByType("int",$read_io); }
        if ( Xormon::isdigit($write_io) )  { $data_out{xtremio}{volume}{$id}{$timestamp}{write_io}  = Xormon::formatNumberByType("int",$write_io); }
        if ( Xormon::isdigit($data_rate) ) { $data_out{xtremio}{volume}{$id}{$timestamp}{data_rate} = Xormon::formatNumberByType("int",Xormon::returnBytes($data_rate,"MB")); }
        if ( Xormon::isdigit($read) )      { $data_out{xtremio}{volume}{$id}{$timestamp}{read}      = Xormon::formatNumberByType("int",Xormon::returnBytes($read,"MB")); }
        if ( Xormon::isdigit($write) )     { $data_out{xtremio}{volume}{$id}{$timestamp}{write}     = Xormon::formatNumberByType("int",Xormon::returnBytes($write,"MB")); }
        if ( Xormon::isdigit($resp_t) )    { $data_out{xtremio}{volume}{$id}{$timestamp}{resp_t}    = Xormon::formatNumberByType("float4",$resp_t / 1000); }      # from usec to msec
        if ( Xormon::isdigit($resp_t_r) )  { $data_out{xtremio}{volume}{$id}{$timestamp}{resp_t_r}  = Xormon::formatNumberByType("float4",$resp_t_r / 1000); }    # from usec to msec
        if ( Xormon::isdigit($resp_t_w) )  { $data_out{xtremio}{volume}{$id}{$timestamp}{resp_t_w}  = Xormon::formatNumberByType("float4",$resp_t_w / 1000); }    # from usec to msec

        # calculate block size
        my $data_act = $data_out{xtremio}{volume}{$id}{$timestamp};
        if ( exists $data_act->{io_rate} && Xormon::isdigit($data_act->{io_rate}) && exists $data_act->{data_rate} ) {
          if ( $data_act->{io_rate} == 0 ) {
            $data_out{xtremio}{volume}{$id}{$timestamp}{block_size} = 0;
          }
          else {
            $data_out{xtremio}{volume}{$id}{$timestamp}{block_size} = Xormon::formatNumberByType("float4",$data_act->{data_rate} / $data_act->{io_rate});
          }
        }
        if ( exists $data_act->{read_io} && Xormon::isdigit($data_act->{read_io}) && exists $data_act->{read} ) {
          if ( $data_act->{read_io} == 0 ) {
            $data_out{xtremio}{volume}{$id}{$timestamp}{block_size_r} = 0;
          }
          else {
            $data_out{xtremio}{volume}{$id}{$timestamp}{block_size_r} = Xormon::formatNumberByType("float4",$data_act->{read} / $data_act->{read_io});
          }
        }
        if ( exists $data_act->{write_io} && Xormon::isdigit($data_act->{write_io}) && exists $data_act->{write} ) {
          if ( $data_act->{write_io} == 0 ) {
            $data_out{xtremio}{volume}{$id}{$timestamp}{block_size_w} = 0;
          }
          else {
            $data_out{xtremio}{volume}{$id}{$timestamp}{block_size_w} = Xormon::formatNumberByType("float4",$data_act->{write} / $data_act->{write_io});
          }
        }
      }
    }
  }

  #
  # drive
  #
  if ( exists $data{'ssds'} ) {
    foreach my $ssd_name ( keys( %{ $data{'ssds'} } ) ) {
      if ( exists $data{'ssds'}{$ssd_name}{perf} ) {
        my $id = "";

        if ( defined $data{'ssds'}{$ssd_name}{conf}{'guid'} && $data{'ssds'}{$ssd_name}{conf}{'guid'} ne '' ) {
          $id = $data{'ssds'}{$ssd_name}{conf}{'guid'};
        }
        else {
          $logger->error("print_perf : SSD ID \(guid\) was not found for \"$ssd_name\"! SSD skipped...") && next;
        }

        my $io_rate   = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "ssds", "$ssd_name", "iops" );
        my $read_io   = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "ssds", "$ssd_name", "rd_iops" );
        my $write_io  = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "ssds", "$ssd_name", "wr_iops" );
        my $data_rate = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "ssds", "$ssd_name", "bw" );
        my $read      = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "ssds", "$ssd_name", "rd_bw" );
        my $write     = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "ssds", "$ssd_name", "wr_bw" );

        if ( Xormon::isdigit($io_rate) )   { $data_out{xtremio}{drive}{$id}{$timestamp}{io_rate}   = Xormon::formatNumberByType("int",$io_rate); }
        if ( Xormon::isdigit($read_io) )   { $data_out{xtremio}{drive}{$id}{$timestamp}{read_io}   = Xormon::formatNumberByType("int",$read_io); }
        if ( Xormon::isdigit($write_io) )  { $data_out{xtremio}{drive}{$id}{$timestamp}{write_io}  = Xormon::formatNumberByType("int",$write_io); }
        if ( Xormon::isdigit($data_rate) ) { $data_out{xtremio}{drive}{$id}{$timestamp}{data_rate} = Xormon::formatNumberByType("int",Xormon::returnBytes($data_rate,"MB")); }
        if ( Xormon::isdigit($read) )      { $data_out{xtremio}{drive}{$id}{$timestamp}{read}      = Xormon::formatNumberByType("int",Xormon::returnBytes($read,"MB")); }
        if ( Xormon::isdigit($write) )     { $data_out{xtremio}{drive}{$id}{$timestamp}{write}     = Xormon::formatNumberByType("int",Xormon::returnBytes($write,"MB")); }
      }
    }
  }

  #
  # host
  #
  if ( exists $data{'initiators'} ) {
    foreach my $initiator_name ( keys( %{ $data{'initiators'} } ) ) {
      if ( exists $data{'initiators'}{$initiator_name}{perf} ) {
        my $id = "";

        if ( defined $data{'initiators'}{$initiator_name}{conf}{'guid'} && $data{'initiators'}{$initiator_name}{conf}{'guid'} ne '' ) {
          $id = $data{'initiators'}{$initiator_name}{conf}{'guid'};
        }
        else {
          $logger->error("print_perf : Initiator ID \(guid\) was not found for \"$initiator_name\"! host skipped...") && next;
        }

        my $io_rate   = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiators", "$initiator_name", "iops" );
        my $read_io   = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiators", "$initiator_name", "rd_iops" );
        my $write_io  = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiators", "$initiator_name", "wr_iops" );
        my $data_rate = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiators", "$initiator_name", "bw" );
        my $read      = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiators", "$initiator_name", "rd_bw" );
        my $write     = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiators", "$initiator_name", "wr_bw" );
        my $resp_t   = sum_value_latency( "$interval_start_timestamp", "$interval_end_timestamp", "initiators", "$initiator_name", "avg_latency", "iops" );
        my $resp_t_r = sum_value_latency( "$interval_start_timestamp", "$interval_end_timestamp", "initiators", "$initiator_name", "rd_latency",  "rd_iops" );
        my $resp_t_w = sum_value_latency( "$interval_start_timestamp", "$interval_end_timestamp", "initiators", "$initiator_name", "wr_latency",  "wr_iops" );

        if ( Xormon::isdigit($io_rate) )   { $data_out{xtremio}{host}{$id}{$timestamp}{io_rate}   = Xormon::formatNumberByType("int",$io_rate); }
        if ( Xormon::isdigit($read_io) )   { $data_out{xtremio}{host}{$id}{$timestamp}{read_io}   = Xormon::formatNumberByType("int",$read_io); }
        if ( Xormon::isdigit($write_io) )  { $data_out{xtremio}{host}{$id}{$timestamp}{write_io}  = Xormon::formatNumberByType("int",$write_io); }
        if ( Xormon::isdigit($data_rate) ) { $data_out{xtremio}{host}{$id}{$timestamp}{data_rate} = Xormon::formatNumberByType("int",Xormon::returnBytes($data_rate,"MB")); }
        if ( Xormon::isdigit($read) )      { $data_out{xtremio}{host}{$id}{$timestamp}{read}      = Xormon::formatNumberByType("int",Xormon::returnBytes($read,"MB")); }
        if ( Xormon::isdigit($write) )     { $data_out{xtremio}{host}{$id}{$timestamp}{write}     = Xormon::formatNumberByType("int",Xormon::returnBytes($write,"MB")); }
        if ( Xormon::isdigit($resp_t) )    { $data_out{xtremio}{host}{$id}{$timestamp}{resp_t}    = Xormon::formatNumberByType("float4",$resp_t / 1000); }      # from usec to msec
        if ( Xormon::isdigit($resp_t_r) )  { $data_out{xtremio}{host}{$id}{$timestamp}{resp_t_r}  = Xormon::formatNumberByType("float4",$resp_t_r / 1000); }    # from usec to msec
        if ( Xormon::isdigit($resp_t_w) )  { $data_out{xtremio}{host}{$id}{$timestamp}{resp_t_w}  = Xormon::formatNumberByType("float4",$resp_t_w / 1000); }    # from usec to msec
      }
    }
  }

  #
  # host_group
  #
  if ( exists $data{'initiator_groups'} ) {
    foreach my $initiator_group_name ( keys( %{ $data{'initiator_groups'} } ) ) {
      if ( exists $data{'initiator_groups'}{$initiator_group_name}{perf} ) {
        my $id = "";

        if ( defined $data{'initiator_groups'}{$initiator_group_name}{conf}{'guid'} && $data{'initiator_groups'}{$initiator_group_name}{conf}{'guid'} ne '' ) {
          $id = $data{'initiator_groups'}{$initiator_group_name}{conf}{'guid'};
        }
        else {
          $logger->error("print_perf : Initiator_group ID \(guid\) was not found for \"$initiator_group_name\"! host_group skipped...") && next;
        }

        my $io_rate   = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiator_groups", "$initiator_group_name", "iops" );
        my $read_io   = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiator_groups", "$initiator_group_name", "rd_iops" );
        my $write_io  = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiator_groups", "$initiator_group_name", "wr_iops" );
        my $data_rate = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiator_groups", "$initiator_group_name", "bw" );
        my $read      = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiator_groups", "$initiator_group_name", "rd_bw" );
        my $write     = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "initiator_groups", "$initiator_group_name", "wr_bw" );

        if ( Xormon::isdigit($io_rate) )   { $data_out{xtremio}{host_group}{$id}{$timestamp}{io_rate}   = Xormon::formatNumberByType("int",$io_rate); }
        if ( Xormon::isdigit($read_io) )   { $data_out{xtremio}{host_group}{$id}{$timestamp}{read_io}   = Xormon::formatNumberByType("int",$read_io); }
        if ( Xormon::isdigit($write_io) )  { $data_out{xtremio}{host_group}{$id}{$timestamp}{write_io}  = Xormon::formatNumberByType("int",$write_io); }
        if ( Xormon::isdigit($data_rate) ) { $data_out{xtremio}{host_group}{$id}{$timestamp}{data_rate} = Xormon::formatNumberByType("int",Xormon::returnBytes($data_rate,"MB")); }
        if ( Xormon::isdigit($read) )      { $data_out{xtremio}{host_group}{$id}{$timestamp}{read}      = Xormon::formatNumberByType("int",Xormon::returnBytes($read,"MB")); }
        if ( Xormon::isdigit($write) )     { $data_out{xtremio}{host_group}{$id}{$timestamp}{write}     = Xormon::formatNumberByType("int",Xormon::returnBytes($write,"MB")); }
      }
    }
  }

  #
  # node
  #
  if ( exists $data{'xenvs'} ) {
    foreach my $xenv ( keys( %{ $data{'xenvs'} } ) ) {
      if ( exists $data{'xenvs'}{$xenv}{perf} ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."node".$xenv) );

        my $cpu_usage = sum_value( "$interval_start_timestamp", "$interval_end_timestamp", "xenvs", "$xenv", "cpu_usage" );

        if ( Xormon::isdigit($cpu_usage) ) { $data_out{xtremio}{node}{$id}{$timestamp}{cpu_system}   = Xormon::formatNumberByType("float4",$cpu_usage); }
      }
    }
  }


  return \%data_out;
}

sub sum_value {
  my $start_timestamp = shift;
  my $end_timestamp   = shift;
  my $section         = shift;
  my $item_name       = shift;
  my $metric          = shift;
  my $total           = "";

  if ( defined $data{$section}{$item_name}{perf} && $data{$section}{$item_name}{perf} ne '' ) {
    my $data_idx = 0;
    my $stat_idx = 0;
    my $value    = "";
    foreach my $stat_timestamp ( sort { $a <=> $b } keys( %{ $data{$section}{$item_name}{perf} } ) ) {
      if ( Xormon::isdigit($stat_timestamp) && $stat_timestamp >= $start_timestamp && $stat_timestamp < $end_timestamp ) {
        $data_idx++;
        if ( defined $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric} && Xormon::isdigit( $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric} ) ) {
          $stat_idx++;

          #print "$metric : $data_idx : $stat_idx : $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric}\n";
          if ( defined $value && Xormon::isdigit($value) ) {
            $value += $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric};
          }
          else {
            $value = $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric};
          }
        }
      }
    }
    if ( defined $value && Xormon::isdigit($value) && defined $stat_idx && Xormon::isdigit($stat_idx) && $stat_idx != 0 ) {
      if ( $value == 0 ) {
        $total = 0;
      }
      else {
        $total = $value / $stat_idx;
      }
    }
    #
    # check if there were all required count of data
    #
    if ( $perf_stats_error == 0 ) {

      #my $req_count = $sample_rate / 5;
      my $req_count = $sample_rate / 60;
      if ( $data_idx > 0 && $data_idx != $req_count ) {
        $logger->error("sum_value : $section : $item_name : $metric : Some missing stats, there should be required \"$req_count\" count of data, but there are only \"data_idx=$data_idx\"! This error is reported only once per agent run...");
      }
      if ( $stat_idx > 0 && $stat_idx != $req_count ) {
        $logger->error("sum_value : $section : $item_name : $metric : Some missing stats, there should be required \"$req_count\" count of data, but there are only \"stat_idx=$stat_idx\"! This error is reported only once per agent run...");
      }
      $perf_stats_error = 1;
    }
  }

  return $total;
}

sub sum_value_latency {
  my $start_timestamp = shift;
  my $end_timestamp   = shift;
  my $section         = shift;
  my $item_name       = shift;
  my $metric_lat      = shift;
  my $metric_io       = shift;
  my $total_lat       = "";

  if ( defined $data{$section}{$item_name}{perf} && $data{$section}{$item_name}{perf} ne '' ) {
    my $data_idx  = 0;
    my $stat_idx  = 0;
    my $value_lat = "";
    my $value_io  = "";
    foreach my $stat_timestamp ( sort { $a <=> $b } keys( %{ $data{$section}{$item_name}{perf} } ) ) {
      if ( Xormon::isdigit($stat_timestamp) && $stat_timestamp >= $start_timestamp && $stat_timestamp < $end_timestamp ) {
        $data_idx++;
        if ( defined $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric_lat} && Xormon::isdigit( $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric_lat} ) && defined $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric_io} && Xormon::isdigit( $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric_io} ) ) {
          $stat_idx++;

          #print "$metric : $data_idx : $stat_idx : $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric}\n";

          # sum io
          if ( defined $value_io && Xormon::isdigit($value_io) ) {
            $value_io += $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric_io};
          }
          else {
            $value_io = $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric_io};
          }

          # sum latency
          my $act_latency = $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric_lat} * $data{$section}{$item_name}{perf}{$stat_timestamp}{$metric_io};
          if ( defined $value_lat && Xormon::isdigit($value_lat) ) {
            $value_lat += $act_latency;
          }
          else {
            $value_lat = $act_latency;
          }

        }
      }
    }
    if ( defined $value_lat && Xormon::isdigit($value_lat) && defined $value_io && Xormon::isdigit($value_io) && $value_io != 0 ) {
      if ( $value_lat == 0 ) {
        $total_lat = 0;
      }
      else {
        $total_lat = $value_lat / $value_io;
      }
    }
    #
    # check if there were all required count of data
    #
    if ( $perf_stats_error == 0 ) {

      #my $req_count = $sample_rate / 5;
      my $req_count = $sample_rate / 60;
      if ( $data_idx > 0 && $data_idx != $req_count ) {
        $logger->error("sum_value : $section : $item_name : $metric_lat : $metric_io : Some missing stats, there should be required \"$req_count\" count of data, but there are only \"data_idx=$data_idx\"! This error is reported only once per agent run...");
      }
      if ( $stat_idx > 0 && $stat_idx != $req_count ) {
        $logger->error("sum_value : $section : $item_name : $metric_lat : $metric_io : Some missing stats, there should be required \"$req_count\" count of data, but there are only \"stat_idx=$stat_idx\"! This error is reported only once per agent run...");
      }
      $perf_stats_error = 1;
    }
  }

  return $total_lat;
}

sub set_time_range {
  my $last_stat_timestamp = shift;
  my $one_hour_secs       = 3600;
  my $act_time            = time();
  my $num                 = int( $last_stat_timestamp / $sample_rate );
  my $end_timestamp       = $num * $sample_rate;
  my $start_timestamp     = "";

  if ( defined $last_perf_timestamp && Xormon::isdigit($last_perf_timestamp) && ( $last_stat_timestamp - $last_perf_timestamp ) < 3600 ) {
    $start_timestamp = $last_perf_timestamp;
  }
  else {
    # last hour
    $start_timestamp = $end_timestamp - $one_hour_secs;
  }

  # set interval start/end time
  my $idx = 0;
  for ( my $i = $start_timestamp; $i <= $end_timestamp; $i += $sample_rate ) {
    $idx++;
    my $interval_start_timestamp = $i;
    my $interval_end_timestamp   = $interval_start_timestamp + $sample_rate;

    $sample_rate_intervals{$idx}{'start_timestamp'} = $interval_start_timestamp;
    $sample_rate_intervals{$idx}{'end_timestamp'}   = $interval_end_timestamp;
  }

  if ( defined $debug && $debug eq "9" ) {
    my $act_date   = localtime($act_time);
    my $end_date   = localtime($end_timestamp);
    my $start_date = localtime($start_timestamp);
    $logger->info("act_time            : $act_time : $act_date")          if $debug == 9;
    $logger->info("start_timestamp     : $start_timestamp : $start_date") if $debug == 9;
    $logger->info("end_timestamp       : $end_timestamp : $end_date")     if $debug == 9;
  }

  return 1;
}

sub new_get_request {
  my $url_act = shift;

  my $agent   = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( GET => $url_act );

  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $cfg->{'apiUsername'}, $cfg->{'apiPassword'} );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    my $res_con = $results->content;
    $logger->error("GET url      : $url_act");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");

    if ( $connection_test ) {
      push(@rows, "GET url      : $url\n");
      push(@rows, "Request error: $st_line\n");
      push(@rows, "Request error: $res_con\n");
    }

    return undef;
  }

  if ( defined $results->content && $results->content ne '' ) {
    my $result = {};
    eval { $result = decode_json( $results->content ); 1 };

    return $result;
  }

  return undef;
}


1;
