package HP3PARCOLLECT;

use strict;
use warnings;

#Modules for rest api
use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
#use Date::parse;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use HP3PAR;
use Xormon;
use JSON qw(encode_json decode_json);
use HTTP::Date;
use Cwd qw(cwd);
use Test;
use XormonLogger;

my $act_dir = cwd;

my %data;
my $xormon;
my $cfg;
my $logger;
my $stats_found = 0;

sub collect {
    $xormon = shift;
    $cfg    = shift;


    if ( (defined $cfg->{'cliApiSshRadio'} && $cfg->{'cliApiSshRadio'} eq "cli" ) || ( defined $cfg->{'cliApiSshRadio'} && $cfg->{'cliApiSshRadio'} eq "ssh" ) ){
      Xormon::log("START 3PAR - CLI/SSH");
      #Xormon::log("CFG UUID : $cfg->{'hostcfg_id'}");
      collect_ssh($cfg);
      return 1;
    }
    else
    {
      #Xormon::log("###########");
      Xormon::log("START 3PAR - REST API VERSION");
      #################
      # 3PAR REST API #
      #################

      #
      # test main variables
      #
      unless ( exists $cfg->{'hostcfg_id'} )   { Xormon::error("Cannot continue, 'uuid' is not configured!")           && return "error"; }
      unless ( exists $cfg->{'hostalias'} )    { Xormon::error("Cannot continue, 'devicealias' is not configured!")   && return "error"; }
      $logger = XormonLogger->new( '3par', $cfg->{hostcfg_id}, $cfg->{hostalias} );
      unless ( exists $cfg->{'host'} )         { $logger->error("Cannot continue, 'host' is not configured!")          && return "error"; }
      unless ( exists $cfg->{'apiPort'} )      { $logger->error("Cannot continue, 'api-port' is not configured!")      && return "error"; }
      unless ( exists $cfg->{'apiUsername'} )  { $logger->error("Cannot continue, 'api-user' is not configured!")      && return "error"; }
      unless ( exists $cfg->{'apiPassword'} )  { $logger->error("Cannot continue, 'api-password' is not configured!")  && return "error"; }
      #unless ( exists $cfg->{'cliApiRadio'} )         { Xormon::error("Cannot continue, 'radio' is not configured!")         && exit 0; }

      #$cfg->{'apiPassword'} = Xormon::unobscure_password($cfg->{'apiPassword'});
      #print "PASSWORD : $cfg->{'apiPassword'}\n";
      my $token =  get_api_token("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/credentials");
      if ( $token eq "0" ){
        $xormon->isConnected(0, "Unsuccessful login! Token not granted");
        $logger->error("Login failed : Cant' login to API") && return "error";
      }
      $xormon->isConnected(1);
      $cfg->{'token'} = $token;

      my $timestamp = time();
      $timestamp = int( $timestamp / 300 ) * 300;
      $cfg->{'gtime'}  = gmtime($timestamp);
      $cfg->{'gtime'}  =~ s/ {2,}/ /g;
      #Xormon::log("GTIME : $cfg->{'gtime'}");

      $cfg->{'timestamp'} = $timestamp;

      my ($start_time,$end_time) = set_time();
      $cfg->{'start-time'} = $start_time;
      $cfg->{'end-time'} = $end_time;

      ##Get Data
      get_api_data();

      ##Volume Capacity
      vol_cap();

      # print Dumper \%data;

      ##Print Data
      print_data();
      # Xormon::log("###### END API $cfg->{'hostalias'} #####");
      return 1;
    }
}

sub connection_test {
  my $client = shift;
  $cfg = shift;
  $logger = XormonLogger->new('3par', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  my @rows;

  my $test = Test->new($client);

  my $timestamp = time();
  $timestamp = int( $timestamp / 300 ) * 300;
  $cfg->{'gtime'}  = gmtime($timestamp);
  $cfg->{'gtime'}  =~ s/ {2,}/ /g;
  #Xormon::log("GTIME : $cfg->{'gtime'}");

  ## VARIABLES CHECK ##
  my @variables = ("hostcfg_id", "hostalias", "host", "apiPort", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  ## NETWORK CONNECTION CHECK##
  my ($code, $text) = Xormon::conntest_tcp($cfg->{host},$cfg->{apiPort});
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  ## API AUTHORIZATION CHECK ##
  my $token =  get_api_token("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/credentials");
  if ( $token eq "0" ){
    $test->addSection("API Authorization", 0 , "Login failed : Cant' login to API");
    return $test->stop();
  }
  else {
    $test->addSection("API Authorization", 1 , "Login OK, token granted");
    $cfg->{'token'} = $token;
  }

  my ($start_time,$end_time) = set_time();
  $cfg->{'start-time'} = $start_time;
  $cfg->{'end-time'} = $end_time;

  my $data;

  ## WSAPI VERSION ##
  my $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/wsapiconfiguration");
  if ( defined $return && $return ne "" ) {
    # print Dumper $return;
    if ( defined $return->{'version'} && $return->{'version'} ne "" ) {
      $data->{'system'}{'wsapiVersion'} = $return->{'version'};
    }
  }
  if (defined $data->{'system'} && $data->{'system'} ne ""){
    my $wsapi_version = "";
    if ( defined $data->{'system'}{'wsapiVersion'} && $data->{'system'}{'wsapiVersion'} ne ""){
      $wsapi_version = $data->{'system'}{'wsapiVersion'};
      my $wsapi = $wsapi_version;
      $wsapi =~ s/\.//g;
      if ( $wsapi < 163 ){
        push(@rows, "WSAPI version: $wsapi_version\nMinimum supported WSAPI version: 1.6.3\n");
        $test->addSection("API Data", 0 , \@rows);
        return $test->stop();
      }
      else {
        push(@rows, "WSAPI version: $wsapi_version\n");
      }
    }
    else {
      push(@rows, "WSAPI Version not found!\n");
      $test->addSection("API Data", 0 , \@rows);
      return $test->stop();
    }
  }
  else {
    push(@rows, "System Information : is not defined!\n");
    $test->addSection("API Data", 0 , \@rows);
    return $test->stop();
  }

  ## SYSTEM INFO ##
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/system");
  if (defined $return && $return ne ""){
    if (defined $return->{'name'} && defined $return->{'name'} ne ""){
      push(@rows, "Name: $return->{'name'}\n");
    }
    if (defined $return->{'systemVersion'} && defined $return->{'systemVersion'} ne ""){
      push(@rows, "Release version: $return->{'systemVersion'}\n");
    }
    if (defined $return->{'id'} && defined $return->{'id'} ne ""){
      push(@rows, "System id: $return->{'id'}\n");
    }
    if (defined $return->{'model'} && defined $return->{'model'} ne ""){
      push(@rows, "Model: $return->{'model'}\n");
    }
    if (defined $return->{'serialNumber'} && defined $return->{'serialNumber'} ne ""){
      push(@rows, "Serial: $return->{'serialNumber'}\n");
    }
  }
  else {
    push(@rows, "No data from /api/v1/system query\n");
    $test->addSection("API Data", 0 , \@rows);
    return $test->stop();
  }

  ## PERF DATA ##
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/cpustatistics/hires?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
    push(@rows, "\nPerformance data OK\n");
  }
  else {
    push(@rows, "Performance data ERROR\n");
    $test->addSection("API Data", 0 , \@rows);
    return $test->stop();
  }

  $test->addSection("API Data", 1 , \@rows);
  return $test->stop();
}

sub connection_test_cli {
  #my $xormon = shift;
  my $cfg    = shift;
  #print "#############\n";
  #print Dumper $cfg;
  my $host   = $cfg->{'host'};
  my $password = "testpw"; #CFG
  my $sshkey = "";
  my $device = $cfg->{'device'};
  my $hp3par_clidir = $cfg->{'cliDir'};
  my @output = ();
  my $cmd    = "";
  my $basedir = cwd;
  my $ssh = "ssh"; # ??

  if (defined $cfg->{'sshKeyFile'} && $cfg->{'sshKeyFile'} ne ""){
    $sshkey = $cfg->{'sshKeyFile'};
  }

  if ( !-f "$hp3par_clidir/cli" ) {
    if ($cfg->{'cliApiSshRadio'} eq "cli"){
      push(@output,'CLIDIR Not found. Trying to start SSH');
    }
    if (!defined $cfg->{'sshUsername'} && $cfg->{'sshUsername'} eq "" ){
      push(@output,'SSH user is not defined');
      return Xormon::conn_test(0, \@output);
    }
    if ( $sshkey eq "" ) {
      $cmd = "$ssh $cfg->{'sshUsername'}" . "@" . "$host statvv -rw -d 1 -iter 1";
    }
    else {
      $cmd = "$ssh -i $sshkey $cfg->{'sshUsername'}" . "@" . "$host statvv -rw -d 1 -iter 1";
    }
    my $cmd_out = cmd($cmd);
    if ( $cmd_out =~ m/ssh command failed/ ) {
      $cmd_out =~ s/-q //g;
      push(@output,$cmd_out);
      push(@output,$cmd);
      return Xormon::conn_test(0, \@output);
    }
    else {
      push(@output,$cmd_out);
      push(@output,$cmd);
      return Xormon::conn_test(1, \@output);
    }
  }
  else {
    my $pwf = "$basedir/etc/$host/.3par-$host.pwf";
    if ( !defined $cfg->{'cliUsername'} && $cfg->{'cliUsername'} ne "" ){
      push(@output,'CLI user is not defined');
      return Xormon::conn_test(0, \@output);
    }
    if ( !defined $cfg->{'cliPassword'} && $cfg->{'cliPassword'} ne "" ){
      push(@output,'CLI password is not defined');
      return Xormon::conn_test(0, \@output);
    }
    if ( !-f $pwf ) {
      if ( not -d "$basedir/etc" ) {
        mkdir("$basedir/etc") or push(@output,"ERROR: Cannot create directory '$basedir/etc'. Exitting.") && return Xormon::conn_test(0, \@output);
      }
      if ( not -d "$basedir/etc/$host" ) {
        mkdir("$basedir/etc/$host") or push(@output,"ERROR: Cannot create directory '$basedir/etc/$host'. Exitting.") && return Xormon::conn_test(0, \@output);
      }

      my $result = create_password_file($cfg->{'cliUsername'}, $cfg->{'cliPassword'}, $host, $basedir);
      #print "result $result\n";
      if ($result =~ m/failed/ ){
        push(@output,"$result");
        return Xormon::conn_test(0, \@output);
      }
      else{
        $cmd = "export TPDCERTDIR=$basedir/etc/$host; export TPDPWFILE=$pwf; yes yes | $hp3par_clidir/cli -sys $host -pwf $basedir/etc/$host/.3par-$host.pwf statvv -rw -d 1 -iter 1";
        my $cmd_out = cmd($cmd);
        if ( $cmd_out !~ m/VVname/ ) {
          push(@output, "Data test Failed\n");
          push(@output,$cmd_out);
          push(@output,$cmd);
          return Xormon::conn_test(0, \@output);
        }
        else {
          my $out = get_hp3par_info($cmd_out);
          #push(@output,$cmd_out);
          push(@output,$out);
          #push(@output,$cmd);
          return Xormon::conn_test(1, \@output);
        }
      }
    }
    else {
      $cmd = "export TPDCERTDIR=$basedir/etc/$host; export TPDPWFILE=$pwf; yes yes | $hp3par_clidir/cli -sys $host -pwf $basedir/etc/$host/.3par-$host.pwf statvv -rw -d 1 -iter 1";
      my $cmd_out = cmd($cmd);
      if ( $cmd_out !~ m/VVname/ ) {
        push(@output, "Data test Failed\n");
        #push(@output,$cmd_out);
        push(@output,$cmd);
        return Xormon::conn_test(0, \@output);
      }
      else {
        push(@output,$cmd_out);
        push(@output,$cmd);
        return Xormon::conn_test(1, \@output);
      }
    }
  }

}

sub cmd {

  my $cmd  = shift;
  my @out  = ();
  my @data = ();
  my $timeout = 30;

  eval {
    # Set alarm
    my $act_time = localtime();
    local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
    alarm($timeout);
    @out = `$cmd 2>&1`;

    # end of alarm
    alarm(0);
  };
  if ($@) {
    if ( $@ =~ /died in SIG ALRM/ ) {
      my $act_time = localtime();

      #error( "command timed out in eval after : $timeout seconds : " . __FILE__ . ":" . __LINE__ );
      push( @data, "command timed out in eval after : $timeout seconds\n" );
      push( @data, "ssh command failed: \"$cmd\"\n" );

      #error( "uemcli command failed: \"$cmd\" : " . __FILE__ . ":" . __LINE__ );
      #@out = ();
      my $scal = join( "", @data );
      return $scal;
    }
  }
  my $scal = join( "", @out );
  return $scal;
}

sub get_hp3par_info {
  my $out     = shift;
  my $output  = "";
  my $skip    = 1;
  #print "OUT START : $out\n";
  my @data    = split( "\n", $out );
  foreach my $line (@data) {
    if ( $line =~ m/r\/w I\/O per second/ ){
      $skip = 0;
    }
    if ($skip){
      next;
    }
    else{
      if ($line =~ m/Continue connecting \(yes\/no\)\? Permanently add this certificate as an exception \(yes\/no\)\?/){
        $line =~ s/Continue connecting \(yes\/no\)\? Permanently add this certificate as an exception \(yes\/no\)\?//;
      }
      $output .= "$line\n";
    }
  }
  #print "FINAL OUTPUT : $output\n";
  return $output;
}


sub remove_whitespace {
  my $value = shift;
  if ( !defined $value || $value eq "" ) { return ""; }
  $value =~ s/^\s+|\s+$//g;
  return $value;

}

sub print_data {
  my @architecture;
  my $data_out = "";
  my %conf;
  my $st_serial = $cfg->{'hostcfg_id'};
  my $rc;
  my $status;
  #print Dumper \%data;
  my $time = $cfg->{'timestamp'};
  #my $time = $cfg->{'end-time'};

  #########
  # SYSTEM/DEVICE
  #########
  my %device_arch = (
    "item_id" => $cfg->{'hostcfg_id'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => "3par",
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'hostcfg_id'}
  );
  push(@architecture, \%device_arch);

  if ( exists $data{'system'}{'patches'} )               { $conf{ $cfg->{'hostcfg_id'} }{patches}              = $data{'system'}{'patches'}; }
  if ( exists $data{'system'}{'serial'} )                { $conf{ $cfg->{'hostcfg_id'} }{serial}               = $data{'system'}{'serial'}; }
  if ( exists $data{'system'}{'release_version'} )       { $conf{ $cfg->{'hostcfg_id'} }{version}              = $data{'system'}{'release_version'}; }
  if ( exists $data{'system'}{'model'} )                 { $conf{ $cfg->{'hostcfg_id'} }{model}                = $data{'system'}{'model'}; }
  if ( exists $data{'system'}{'id'} )                    { $conf{ $cfg->{'hostcfg_id'} }{id}                   = $data{'system'}{'id'}; }
  if ( exists $data{'system'}{'allocated'} )             { $conf{ $cfg->{'hostcfg_id'} }{capacity_allocated}   = $data{'system'}{'allocated'}; }
  if ( exists $data{'system'}{'failed'} )                { $conf{ $cfg->{'hostcfg_id'} }{capacity_failed}      = $data{'system'}{'failed'}; }
  if ( exists $data{'system'}{'free'} )                  { $conf{ $cfg->{'hostcfg_id'} }{capacity_free}        = $data{'system'}{'free'}; }
  if ( exists $data{'system'}{'total'} )                 { $conf{ $cfg->{'hostcfg_id'} }{capacity_total}       = $data{'system'}{'total'}; }
  if ( exists $data{'system'}{'name'} )                  { $conf{ $cfg->{'hostcfg_id'} }{node_name}            = $data{'system'}{'name'}; }

  $conf{ $cfg->{'hostcfg_id'} }{'ip'} = $cfg->{'apiHost'};

  ########
  # CPU
  ########

  if ( defined $data{'perf'}{'cpu'} && $data{'perf'}{'cpu'} ne "" ){
    $data_out .= "uuid;time;cpu_user;cpu_system;cpu_idle;cpu_compress\n";
    while ((my $cpu, my $value) = each (%{ $data{'perf'}{'cpu'} })){
      #Conf
      my $name     = "$cpu";
      my $hash_key = Xormon::create_hash_key($st_serial."CPU-CORE".$cpu); $hash_key = lc($hash_key);
      my %arch = (
          "item_id" => $hash_key,
          "label" => $cpu,
          "hw_type" => "3par",
          "subsystem" => "cpu",
          "hostcfg_id" => $cfg->{'hostcfg_id'},
          "parents" => [$cfg->{'hostcfg_id'}],
        );

      push(@architecture, \%arch);


      #Perf
      my $usr      = "";
      my $sys      = "";
      my $idle     = "";
      my $compress = "";
      if (defined $data{'perf'}{'cpu'}{$cpu}{'usr'} && $data{'perf'}{'cpu'}{$cpu}{'usr'} ne ""){
        $usr = $data{'perf'}{'cpu'}{$cpu}{'usr'};
      }
      if (defined $data{'perf'}{'cpu'}{$cpu}{'idle'} && $data{'perf'}{'cpu'}{$cpu}{'idle'} ne ""){
        $idle = $data{'perf'}{'cpu'}{$cpu}{'idle'};
      }
      if (defined $data{'perf'}{'cpu'}{$cpu}{'sys'} && $data{'perf'}{'cpu'}{$cpu}{'sys'} ne ""){
        $sys = $data{'perf'}{'cpu'}{$cpu}{'sys'};
      }
      if (defined $data{'perf'}{'cpu'}{$cpu}{'compress'} && $data{'perf'}{'cpu'}{$cpu}{'compress'} ne ""){
        $compress = $data{'perf'}{'cpu'}{$cpu}{'compress'};
      }

      #print PERF "$time,$name,$usr,$sys,$idle,$compress\n";
      $data_out .= "$hash_key;$time;$usr;$sys;$idle;$compress\n";

    }
  }

  #Save CPU
  #print "CPU\n";
  #print "$data_out\n";
  $rc = $xormon->saveCSV($data_out,"3par","cpu");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }

  $logger->info("CPU : $status");

  $data_out = "";

  ########
  # DISK
  ########
  if ( defined $data{'perf'}{'disk'} && $data{'perf'}{'disk'} ne "" ){
    #Table Capacity
    $data_out .= "uuid;time;io_rate;read_io;write_io;data_rate;read;write;resp_t_r;resp_t_w;resp_t;user_capacity_total;user_capacity_free;user_capacity_used;user_capacity_used_percent;spare_capacity_total;spare_capacity_free;spare_capacity_used;spare_capacity_used_percent;capacity_total;capacity_free;capacity_used;capacity_used_percent\n";
    #my $disk_cap = "$webdir/$cfg->{'hostalias'}/disk_capacity.html";
    #Table Capacity
    # my $disk_table     = "<table class = \"tablesorter\">\n";
    # my $disk_thead     = "<thead><tr><th rowspan=\"2\" class='sortable tablesorter-header tablesorter-headerUnSorted'>Disk</th><th rowspan=\"2\" class='sortable tablesorter-header tablesorter-headerUnSorted'>Size [TiB]</th><th colspan=\"4\" style=\"text-align:center;\">Data [TiB]</th><th colspan=\"4\" style=\"text-align:center;\">Spare [TiB]</th></tr><tr><th class='sortable tablesorter-header tablesorter-headerUnSorted'>Total</th><th class='sortable tablesorter-header tablesorter-headerUnSorted'>Used</th><th>Used %</th><th class='sortable tablesorter-header tablesorter-headerUnSorted'>Avail</th><th class='sortable tablesorter-header tablesorter-headerUnSorted'>Total</th><th class='sortable tablesorter-header tablesorter-headerUnSorted'>Used</th><th>Used %</th><th class='sortable tablesorter-header tablesorter-headerUnSorted'>Avail</th></tr></thead>\n";
    # my $disk_tbody     = "<tbody>\n";
    # my $disk_data_tr   = "";

    while ((my $disk, my $value) = each (%{ $data{'perf'}{'disk'} })){
      my $id        = $disk;
      my $name      = "";
      my $io_rate   = "";
      my $read_io   = "";
      my $write_io  = "";
      my $data_rate = "";
      my $read      = "";
      my $write     = "";
      my $resp_t_r  = "";
      my $resp_t_w  = "";
      my $resp_t    = "";
      my $type      = "";
      my $normal_t  = "";
      my $normal_f  = "";
      my $normal_u  = "";
      my $normal_pct = "";
      my $spare_t   = "";
      my $spare_f   = "";
      my $spare_u   = "";
      my $spare_pct = "";
      my $used      = "";
      my $used_pct  = "";

      #Conf
      my $type_word = "";
      my $rpm       = "";
      my $total     = "";
      my $free      = "";

      if ( defined $data{'perf'}{'disk'}{$disk}{'name'} && $data{'perf'}{'disk'}{$disk}{'name'} ne ""){
        $name = $data{'perf'}{'disk'}{$disk}{'name'};
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'io_rate'} && $data{'perf'}{'disk'}{$disk}{'io_rate'} ne ""){
        $io_rate = Xormon::formatNumberByType("float4",$data{'perf'}{'disk'}{$disk}{'io_rate'});
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'read_io'} && $data{'perf'}{'disk'}{$disk}{'read_io'} ne ""){
        $read_io = Xormon::formatNumberByType("float4",$data{'perf'}{'disk'}{$disk}{'read_io'});
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'write_io'} && $data{'perf'}{'disk'}{$disk}{'write_io'} ne ""){
        $write_io = Xormon::formatNumberByType("float4",$data{'perf'}{'disk'}{$disk}{'write_io'});
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'data_rate'} && $data{'perf'}{'disk'}{$disk}{'data_rate'} ne ""){
        $data_rate = Xormon::returnBytes( $data{'perf'}{'disk'}{$disk}{'data_rate'}, "kB" );
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'read'} && $data{'perf'}{'disk'}{$disk}{'read'} ne ""){
        $read = Xormon::returnBytes( $data{'perf'}{'disk'}{$disk}{'read'}, "kB" );
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'write'} && $data{'perf'}{'disk'}{$disk}{'write'} ne ""){
        $write = Xormon::returnBytes( $data{'perf'}{'disk'}{$disk}{'write'}, "kB" );
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'resp_t_r'} && $data{'perf'}{'disk'}{$disk}{'resp_t_r'} ne ""){
        $resp_t_r = Xormon::formatNumberByType("float4",$data{'perf'}{'disk'}{$disk}{'resp_t_r'});
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'resp_t_w'} && $data{'perf'}{'disk'}{$disk}{'resp_t_w'} ne ""){
        $resp_t_w = Xormon::formatNumberByType("float4",$data{'perf'}{'disk'}{$disk}{'resp_t_w'});
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'resp_t'} && $data{'perf'}{'disk'}{$disk}{'resp_t'} ne ""){
        $resp_t = Xormon::formatNumberByType("float4",$data{'perf'}{'disk'}{$disk}{'resp_t'});
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'type'} && $data{'perf'}{'disk'}{$disk}{'type'} ne ""){
        $type = $data{'perf'}{'disk'}{$disk}{'type'};
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'type_word'} && $data{'perf'}{'disk'}{$disk}{'type_word'} ne ""){
        $type_word = $data{'perf'}{'disk'}{$disk}{'type_word'};
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'rpm'} && $data{'perf'}{'disk'}{$disk}{'rpm'} ne ""){
        $rpm = Xormon::formatNumberByType("float4",$data{'perf'}{'disk'}{$disk}{'rpm'});
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'total'} && $data{'perf'}{'disk'}{$disk}{'total'} ne ""){
        #$total = Xormon::formatNumberByType("float4",$data{'perf'}{'disk'}{$disk}{'total'});
        $total = Xormon::returnBytes($data{'perf'}{'disk'}{$disk}{'total'},"MiB");
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'free'} && $data{'perf'}{'disk'}{$disk}{'free'} ne ""){
        $free = Xormon::returnBytes($data{'perf'}{'disk'}{$disk}{'free'},"MiB");
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'used'} && $data{'perf'}{'disk'}{$disk}{'used'} ne ""){
        $used = Xormon::returnBytes($data{'perf'}{'disk'}{$disk}{'used'},"MiB");
      }
      if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
        $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
      }
      # normal
      if ( defined $data{'perf'}{'disk'}{$disk}{'normal'}{'total'} && $data{'perf'}{'disk'}{$disk}{'normal'}{'total'} ne ""){
        $normal_t = Xormon::returnBytes($data{'perf'}{'disk'}{$disk}{'normal'}{'total'},"MiB");
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'normal'}{'free'} && $data{'perf'}{'disk'}{$disk}{'normal'}{'free'} ne ""){
        $normal_f = Xormon::returnBytes($data{'perf'}{'disk'}{$disk}{'normal'}{'free'},"MiB");
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'normal'}{'used'} && $data{'perf'}{'disk'}{$disk}{'normal'}{'used'} ne ""){
        $normal_u = Xormon::returnBytes($data{'perf'}{'disk'}{$disk}{'normal'}{'used'},"MiB");
      }
      if ( Xormon::isdigit($normal_t) && Xormon::isdigit($normal_u) && ($normal_t != 0) ){
        $normal_pct = Xormon::formatNumberByType("float4", $normal_u / ($normal_t / 100) );
      }
      # spare
      if ( defined $data{'perf'}{'disk'}{$disk}{'spare'}{'total'} && $data{'perf'}{'disk'}{$disk}{'spare'}{'total'} ne ""){
        $spare_t = Xormon::returnBytes($data{'perf'}{'disk'}{$disk}{'spare'}{'total'},"MiB");
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'spare'}{'free'} && $data{'perf'}{'disk'}{$disk}{'spare'}{'free'} ne ""){
        $spare_f = Xormon::returnBytes($data{'perf'}{'disk'}{$disk}{'spare'}{'free'},"MiB");
      }
      if ( defined $data{'perf'}{'disk'}{$disk}{'spare'}{'used'} && $data{'perf'}{'disk'}{$disk}{'spare'}{'used'} ne ""){
        $spare_u = Xormon::returnBytes($data{'perf'}{'disk'}{$disk}{'spare'}{'used'},"MiB");
      }
      if ( Xormon::isdigit($spare_t) && Xormon::isdigit($spare_u) && ($spare_t != 0) ){
        $spare_pct = Xormon::formatNumberByType("float4", $spare_u / ($spare_t / 100) );
      }

      #######
      #Configuration
      #######
      #print CONF "ID,Cage Position,Type,RPM,Total Size [MB],Free Size [MB]\n";
      #print CONF "$id,$name,$type_word,$rpm,$total,$free\n";
      my $hash_key = Xormon::create_hash_key($st_serial."DRIVE".$name); $hash_key = lc($hash_key);
      my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "drive",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );

      # $conf{$hash_key}{cage_position}   = $name;
      # $conf{$hash_key}{rpm}             = $rpm;
      # $conf{$hash_key}{total_size}      = $total;
      # $conf{$hash_key}{free_size}       = $free;
      # $conf{$hash_key}{type}            = $type_word;

      push(@architecture, \%arch);

      ########
      #Performance
      ########
      #print PERF "time,id,name,io_rate,read_io,write_io,data_rate,read,write,resp_t_r,resp_t_w,resp_t,type,normal_t,normal_f,normal_u,spare_t,spare_f,spare_u,total,free,used\n";
      #$data_out .= "uuid;time;io_rate;read_io;write_io;data_rate;read;write;resp_t_r;resp_t_w;resp_t;type;normal_t;normal_f;normal_u;spare_t;spare_f;spare_u;total;free;used\n";
      $data_out .= "$hash_key;$time;$io_rate;$read_io;$write_io;$data_rate;$read;$write;$resp_t_r;$resp_t_w;$resp_t;$normal_t;$normal_f;$normal_u;$normal_pct;$spare_t;$spare_f;$spare_u;$spare_pct;$total;$free;$used;$used_pct\n";
    }
  }

  #print "#################### Data Out DISK\n";
  #print "$data_out";
  $rc = $xormon->saveCSV($data_out,"3par","drive");
  #Xormon::log("Drive data :");
  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("Drive status : $status");
  #print "### $status\n";
  $data_out = "";


  #######
  # Host
  #######

  if ( defined $data{'perf'}{'host'} && $data{'perf'}{'host'} ne "" ){
    $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t\n";
    while ((my $host, my $value) = each (%{ $data{'perf'}{'host'} })){
      my $name      = $host;
      my $io_rate   = "";
      my $read_io   = "";
      my $write_io  = "";
      my $data_rate = "";
      my $read      = "";
      my $write     = "";
      my $resp_t_r  = "";
      my $resp_t_w  = "";
      my $resp_t    = "";
      my $type      = "";
      my $wwn       = "";

      if ( defined $data{'perf'}{'host'}{$host}{'io_rate'} && $data{'perf'}{'host'}{$host}{'io_rate'} ne ""){
        $io_rate = $data{'perf'}{'host'}{$host}{'io_rate'};
      }
      if ( defined $data{'perf'}{'host'}{$host}{'read_io'} && $data{'perf'}{'host'}{$host}{'read_io'} ne ""){
        $read_io = $data{'perf'}{'host'}{$host}{'read_io'};
      }
      if ( defined $data{'perf'}{'host'}{$host}{'write_io'} && $data{'perf'}{'host'}{$host}{'write_io'} ne ""){
        $write_io = $data{'perf'}{'host'}{$host}{'write_io'};
      }
      if ( defined $data{'perf'}{'host'}{$host}{'data_rate'} && $data{'perf'}{'host'}{$host}{'data_rate'} ne ""){
        $data_rate = Xormon::returnBytes( $data{'perf'}{'host'}{$host}{'data_rate'}, "kB" );
      }
      if ( defined $data{'perf'}{'host'}{$host}{'read'} && $data{'perf'}{'host'}{$host}{'read'} ne ""){
        $read = Xormon::returnBytes( $data{'perf'}{'host'}{$host}{'read'}, "kB" );
      }
      if ( defined $data{'perf'}{'host'}{$host}{'write'} && $data{'perf'}{'host'}{$host}{'write'} ne ""){
        $write = Xormon::returnBytes( $data{'perf'}{'host'}{$host}{'write'}, "kB" );
      }
      if ( defined $data{'perf'}{'host'}{$host}{'resp_t_r'} && $data{'perf'}{'host'}{$host}{'resp_t_r'} ne ""){
        $resp_t_r = $data{'perf'}{'host'}{$host}{'resp_t_r'};
      }
      if ( defined $data{'perf'}{'host'}{$host}{'resp_t_w'} && $data{'perf'}{'host'}{$host}{'resp_t_w'} ne ""){
        $resp_t_w = $data{'perf'}{'host'}{$host}{'resp_t_w'};
      }
      if ( defined $data{'perf'}{'host'}{$host}{'resp_t'} && $data{'perf'}{'host'}{$host}{'resp_t'} ne ""){
        $resp_t = $data{'perf'}{'host'}{$host}{'resp_t'};
      }
      if ( defined $data{'perf'}{'host'}{$host}{'wwn'} && $data{'perf'}{'host'}{$host}{'wwn'} ne ""){
        $wwn = $data{'perf'}{'host'}{$host}{'wwn'};
      }

      my $hash_key = Xormon::create_hash_key($st_serial."HOST".$host); $hash_key = lc($hash_key);
      ########
      # Config
      ########
      #print CONF "$name,$volume_id\n";
      my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "host",
          "wwn" => Xormon::formatWWN($wwn),
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );

      if ( defined $data{'perf'}{'host'}{$host}{'volume_id'} && $data{'perf'}{'host'}{$host}{'volume_id'} ne ""){
        ( @{ $arch{'childrens'} }) = split( " ", $data{'perf'}{'host'}{$host}{'volume_id'});
      }

      $conf{$hash_key}{wwn}            = Xormon::formatWWN($wwn);

      #Xormon::log("volume id : $volume_id");
      #my $vol_id = Xormon::create_hash_key($st_serial."VOLUME".$volume_id ); $hash_key = lc($hash_key);

      push(@architecture, \%arch);

      #$data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t\n";

      $data_out .= "$hash_key;$time;$read_io;$write_io;$io_rate;$read;$write;$data_rate;$resp_t_r;$resp_t_w;$resp_t\n";


    }
  }

  #Save Host
  $rc = $xormon->saveCSV($data_out,"3par","host");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("Host status : $status");
  #print "### $status\n";
  $data_out = "";

  #######
  # Node
  #######
  if ( defined $data{'perf'}{'node'} && $data{'perf'}{'node'} ne "" ){
    $data_out .= "uuid;time;r_io_acc;w_io_acc;r_cache_hit_io;w_cache_hit_io;r_cache_hit;w_cache_hit;r_lock_block;w_lock_block\n";
    while ((my $node, my $value) = each (%{ $data{'perf'}{'node'} })){
      my $name           = $node;
      my $r_io_acc       = "";
      my $w_io_acc       = "";
      my $r_cache_hit_io = "";
      my $w_cache_hit_io = "";
      my $r_cache_hit    = "";
      my $w_cache_hit    = "";
      my $r_lock_block   = "";
      my $w_lock_block   = "";

      if ( defined $data{'perf'}{'node'}{$node}{'r_io_acc'} && $data{'perf'}{'node'}{$node}{'r_io_acc'} ne ""){
        $r_io_acc = $data{'perf'}{'node'}{$node}{'r_io_acc'};
      }
      if ( defined $data{'perf'}{'node'}{$node}{'w_io_acc'} && $data{'perf'}{'node'}{$node}{'w_io_acc'} ne ""){
        $w_io_acc = $data{'perf'}{'node'}{$node}{'w_io_acc'};
      }
      if ( defined $data{'perf'}{'node'}{$node}{'r_cache_hit_io'} && $data{'perf'}{'node'}{$node}{'r_cache_hit_io'} ne ""){
        $r_cache_hit_io = $data{'perf'}{'node'}{$node}{'r_cache_hit_io'};
      }
      if ( defined $data{'perf'}{'node'}{$node}{'w_cache_hit_io'} && $data{'perf'}{'node'}{$node}{'w_cache_hit_io'} ne ""){
        $w_cache_hit_io = $data{'perf'}{'node'}{$node}{'w_cache_hit_io'};
      }
      if ( defined $data{'perf'}{'node'}{$node}{'w_cache_hit'} && $data{'perf'}{'node'}{$node}{'w_cache_hit'} ne ""){
        $w_cache_hit = $data{'perf'}{'node'}{$node}{'w_cache_hit'};
      }
      if ( defined $data{'perf'}{'node'}{$node}{'r_cache_hit'} && $data{'perf'}{'node'}{$node}{'r_cache_hit'} ne ""){
        $r_cache_hit = $data{'perf'}{'node'}{$node}{'r_cache_hit'};
      }
      if ( defined $data{'perf'}{'node'}{$node}{'r_lock_block'} && $data{'perf'}{'node'}{$node}{'r_lock_block'} ne ""){
        $r_lock_block = $data{'perf'}{'node'}{$node}{'r_lock_block'};
      }
      if ( defined $data{'perf'}{'node'}{$node}{'w_lock_block'} && $data{'perf'}{'node'}{$node}{'w_lock_block'} ne ""){
        $w_lock_block = $data{'perf'}{'node'}{$node}{'w_lock_block'};
      }

      my $hash_key = Xormon::create_hash_key($st_serial."CACHE".$name ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "cache",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );
      push(@architecture, \%arch);

      #print PERF "$time,$name,$r_io_acc,$w_io_acc,$r_cache_hit_io,$w_cache_hit_io,$r_cache_hit,$w_cache_hit,$r_lock_block,$w_lock_block\n";
      $data_out .= "$hash_key;$time;$r_io_acc;$w_io_acc;$r_cache_hit_io;$w_cache_hit_io;$r_cache_hit;$w_cache_hit;$r_lock_block;$w_lock_block\n";

    }
  }

  #Save  Node
  $rc = $xormon->saveCSV($data_out,"3par","cache");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("Node status : $status");
  $data_out = "";

  if ( defined $data{'perf'}{'nodeFC'} && $data{'perf'}{'nodeFC'} ne "" ){
    $data_out .= "uuid;time;cfc_dirty;cfc_max;del_ack\n";
    while ((my $node, my $value) = each (%{ $data{'perf'}{'nodeFC'} })){
      my $name           = "FC-$node";
      my $cfc_dirty = "";
      my $cfc_max   = "";
      my $del_ack   = "";

      if ( defined $data{'perf'}{'nodeFC'}{$node}{'cfc_dirty'} && $data{'perf'}{'nodeFC'}{$node}{'cfc_dirty'} ne ""){
        $cfc_dirty = $data{'perf'}{'nodeFC'}{$node}{'cfc_dirty'};
      }
      if ( defined $data{'perf'}{'nodeFC'}{$node}{'cfc_max'} && $data{'perf'}{'nodeFC'}{$node}{'cfc_max'} ne ""){
        $cfc_max = $data{'perf'}{'nodeFC'}{$node}{'cfc_max'};
      }
      if ( defined $data{'perf'}{'nodeFC'}{$node}{'del_ack'} && $data{'perf'}{'nodeFC'}{$node}{'del_ack'} ne ""){
        $del_ack = $data{'perf'}{'nodeFC'}{$node}{'del_ack'};
      }

      my $hash_key = Xormon::create_hash_key($st_serial."CACHE".$name ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "cache",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );
      push(@architecture, \%arch);

      #print PERF "$time,$name,$cfc_dirty,$cfc_max,$del_ack\n";
      $data_out .= "$hash_key;$time;$cfc_dirty;$cfc_max;$del_ack\n";

    }
  }

  #save FC-NODE CACHE
  $rc = $xormon->saveCSV($data_out,"3par","cache");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("FC-NODE CACHE status : $status");
  $data_out = "";

  if ( defined $data{'perf'}{'nodeFC_10'} && $data{'perf'}{'nodeFC_10'} ne "" ){
    $data_out .= "uuid;time;cfc_dirty;cfc_max;del_ack\n";
    while ((my $node, my $value) = each (%{ $data{'perf'}{'nodeFC_10'} })){
      my $name           = "FC_10KRPM-$node";
      my $cfc_dirty = "";
      my $cfc_max   = "";
      my $del_ack   = "";

      if ( defined $data{'perf'}{'nodeFC_10'}{$node}{'cfc_dirty'} && $data{'perf'}{'nodeFC_10'}{$node}{'cfc_dirty'} ne ""){
        $cfc_dirty = $data{'perf'}{'nodeFC_10'}{$node}{'cfc_dirty'};
      }
      if ( defined $data{'perf'}{'nodeFC_10'}{$node}{'cfc_max'} && $data{'perf'}{'nodeFC_10'}{$node}{'cfc_max'} ne ""){
        $cfc_max = $data{'perf'}{'nodeFC_10'}{$node}{'cfc_max'};
      }
      if ( defined $data{'perf'}{'nodeFC_10'}{$node}{'del_ack'} && $data{'perf'}{'nodeFC_10'}{$node}{'del_ack'} ne ""){
        $del_ack = $data{'perf'}{'nodeFC_10'}{$node}{'del_ack'};
      }


      my $hash_key = Xormon::create_hash_key($st_serial."CACHE".$name ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "cache",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );
      push(@architecture, \%arch);

      #print PERF "$time,$name,$cfc_dirty,$cfc_max,$del_ack\n";
      $data_out .= "$hash_key;$time;$cfc_dirty;$cfc_max;$del_ack\n";

    }
  }

  #SAVE FC-10KRPM\n";
  $rc = $xormon->saveCSV($data_out,"3par","cache");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("FC-10KRPM-NODE CACHE status : $status");
  $data_out = "";

  if ( defined $data{'perf'}{'nodeFC_15'} && $data{'perf'}{'nodeFC_15'} ne "" ){
    $data_out .= "uuid;time;cfc_dirty;cfc_max;del_ack\n";
    while ((my $node, my $value) = each (%{ $data{'perf'}{'nodeFC_15'} })){
      my $name           = "FC_15KRPM-$node";
      my $cfc_dirty = "";
      my $cfc_max   = "";
      my $del_ack   = "";

      if ( defined $data{'perf'}{'nodeFC_15'}{$node}{'cfc_dirty'} && $data{'perf'}{'nodeFC_15'}{$node}{'cfc_dirty'} ne ""){
        $cfc_dirty = $data{'perf'}{'nodeFC_15'}{$node}{'cfc_dirty'};
      }
      if ( defined $data{'perf'}{'nodeFC_15'}{$node}{'cfc_max'} && $data{'perf'}{'nodeFC_15'}{$node}{'cfc_max'} ne ""){
        $cfc_max = $data{'perf'}{'nodeFC_15'}{$node}{'cfc_max'};
      }
      if ( defined $data{'perf'}{'nodeFC_15'}{$node}{'del_ack'} && $data{'perf'}{'nodeFC_15'}{$node}{'del_ack'} ne ""){
        $del_ack = $data{'perf'}{'nodeFC_15'}{$node}{'del_ack'};
      }

      my $hash_key = Xormon::create_hash_key($st_serial."CACHE".$name ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "cache",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );
      push(@architecture, \%arch);

      #print PERF "$time,$name,$cfc_dirty,$cfc_max,$del_ack\n";
      $data_out .= "$hash_key;$time;$cfc_dirty;$cfc_max;$del_ack\n";

    }
  }

  #SAVE FC-15KRPM\n";
  $rc = $xormon->saveCSV($data_out,"3par","cache");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("FC-15KRPM-NODE CACHE status : $status");
  $data_out = "";

  if ( defined $data{'perf'}{'nodeNL'} && $data{'perf'}{'nodeNL'} ne "" ){
    $data_out .= "uuid;time;cfc_dirty;cfc_max;del_ack\n";
    while ((my $node, my $value) = each (%{ $data{'perf'}{'nodeNL'} })){
      my $name           = "NL-$node";
      my $cfc_dirty = "";
      my $cfc_max   = "";
      my $del_ack   = "";

      if ( defined $data{'perf'}{'nodeNL'}{$node}{'cfc_dirty'} && $data{'perf'}{'nodeNL'}{$node}{'cfc_dirty'} ne ""){
        $cfc_dirty = $data{'perf'}{'nodeNL'}{$node}{'cfc_dirty'};
      }
      if ( defined $data{'perf'}{'nodeNL'}{$node}{'cfc_max'} && $data{'perf'}{'nodeNL'}{$node}{'cfc_max'} ne ""){
        $cfc_max = $data{'perf'}{'nodeNL'}{$node}{'cfc_max'};
      }
      if ( defined $data{'perf'}{'nodeNL'}{$node}{'del_ack'} && $data{'perf'}{'nodeNL'}{$node}{'del_ack'} ne ""){
        $del_ack = $data{'perf'}{'nodeNL'}{$node}{'del_ack'};
      }


      my $hash_key = Xormon::create_hash_key($st_serial."CACHE".$name ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "cache",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );
      push(@architecture, \%arch);

      #print PERF "$time,$name,$cfc_dirty,$cfc_max,$del_ack\n";
      $data_out .= "$hash_key;$time;$cfc_dirty;$cfc_max;$del_ack\n";

    }
  }

  #SAVE NL NODE\n";
  $rc = $xormon->saveCSV($data_out,"3par","cache");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("NL-NODE CACHE status : $status");
  $data_out = "";

  if ( defined $data{'perf'}{'nodeSSD'} && $data{'perf'}{'nodeSSD'} ne "" ){
    $data_out .= "uuid;time;cfc_dirty;cfc_max;del_ack\n";
    while ((my $node, my $value) = each (%{ $data{'perf'}{'nodeSSD'} })){
      my $name           = "SSD-$node";
      my $cfc_dirty = "";
      my $cfc_max   = "";
      my $del_ack   = "";

      if ( defined $data{'perf'}{'nodeSSD'}{$node}{'cfc_dirty'} && $data{'perf'}{'nodeSSD'}{$node}{'cfc_dirty'} ne ""){
        $cfc_dirty = $data{'perf'}{'nodeSSD'}{$node}{'cfc_dirty'};
      }
      if ( defined $data{'perf'}{'nodeSSD'}{$node}{'cfc_max'} && $data{'perf'}{'nodeSSD'}{$node}{'cfc_max'} ne ""){
        $cfc_max = $data{'perf'}{'nodeSSD'}{$node}{'cfc_max'};
      }
      if ( defined $data{'perf'}{'nodeSSD'}{$node}{'del_ack'} && $data{'perf'}{'nodeSSD'}{$node}{'del_ack'} ne ""){
        $del_ack = $data{'perf'}{'nodeSSD'}{$node}{'del_ack'};
      }


      my $hash_key = Xormon::create_hash_key($st_serial."CACHE".$name ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "cache",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );
      push(@architecture, \%arch);

      #print PERF "$time,$name,$cfc_dirty,$cfc_max,$del_ack\n";
      $data_out .= "$hash_key;$time;$cfc_dirty;$cfc_max;$del_ack\n";

    }
  }

  #SAVE SSD NODE\n";
  $rc = $xormon->saveCSV($data_out,"3par","cache");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("SSD-NODE  CACHE status : $status");
  $data_out = "";

  if ( defined $data{'perf'}{'nodeSSD_100'} && $data{'perf'}{'nodeSSD_100'} ne "" ){
    $data_out .= "uuid;time;cfc_dirty;cfc_max;del_ack\n";
    while ((my $node, my $value) = each (%{ $data{'perf'}{'nodeSSD_100'} })){
      my $name           = "SSD_100KRPM-$node";
      my $cfc_dirty = "";
      my $cfc_max   = "";
      my $del_ack   = "";

      if ( defined $data{'perf'}{'nodeSSD_100'}{$node}{'cfc_dirty'} && $data{'perf'}{'nodeSSD_100'}{$node}{'cfc_dirty'} ne ""){
        $cfc_dirty = $data{'perf'}{'nodeSSD_100'}{$node}{'cfc_dirty'};
      }
      if ( defined $data{'perf'}{'nodeSSD_100'}{$node}{'cfc_max'} && $data{'perf'}{'nodeSSD_100'}{$node}{'cfc_max'} ne ""){
        $cfc_max = $data{'perf'}{'nodeSSD_100'}{$node}{'cfc_max'};
      }
      if ( defined $data{'perf'}{'nodeSSD_100'}{$node}{'del_ack'} && $data{'perf'}{'nodeSSD_100'}{$node}{'del_ack'} ne ""){
        $del_ack = $data{'perf'}{'nodeSSD_100'}{$node}{'del_ack'};
      }


      my $hash_key = Xormon::create_hash_key($st_serial."CACHE".$name ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "cache",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );
      push(@architecture, \%arch);

      #print PERF "$time,$name,$cfc_dirty,$cfc_max,$del_ack\n";
      $data_out .= "$hash_key;$time;$cfc_dirty;$cfc_max;$del_ack\n";

    }
  }

  #print "SAVE SSD100-NODE\n";
  $rc = $xormon->saveCSV($data_out,"3par","cache");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("SSD100-NODE CACHE status : $status");
  $data_out = "";

  if ( defined $data{'perf'}{'nodeSSD_150'} && $data{'perf'}{'nodeSSD_150'} ne "" ){
    $data_out .= "uuid;time;cfc_dirty;cfc_max;del_ack\n";
    while ((my $node, my $value) = each (%{ $data{'perf'}{'nodeSSD_150'} })){
      my $name           = "SSD_150KRPM-$node";
      my $cfc_dirty = "";
      my $cfc_max   = "";
      my $del_ack   = "";

      if ( defined $data{'perf'}{'nodeSSD_150'}{$node}{'cfc_dirty'} && $data{'perf'}{'nodeSSD_150'}{$node}{'cfc_dirty'} ne ""){
        $cfc_dirty = $data{'perf'}{'nodeSSD_150'}{$node}{'cfc_dirty'};
      }
      if ( defined $data{'perf'}{'nodeSSD_150'}{$node}{'cfc_max'} && $data{'perf'}{'nodeSSD_150'}{$node}{'cfc_max'} ne ""){
        $cfc_max = $data{'perf'}{'nodeSSD_150'}{$node}{'cfc_max'};
      }
      if ( defined $data{'perf'}{'nodeSSD_150'}{$node}{'del_ack'} && $data{'perf'}{'nodeSSD_150'}{$node}{'del_ack'} ne ""){
        $del_ack = $data{'perf'}{'nodeSSD_150'}{$node}{'del_ack'};
      }


      my $hash_key = Xormon::create_hash_key($st_serial."CACHE".$name ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "cache",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );
      push(@architecture, \%arch);

      #print PERF "$time,$name,$cfc_dirty,$cfc_max,$del_ack\n";
      $data_out .= "$hash_key;$time;$cfc_dirty;$cfc_max;$del_ack\n";


    }
  }

  #SAVE SSD150 - CACHE\n";
  $rc = $xormon->saveCSV($data_out,"3par","cache");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("SSD150-NODE CACHE status : $status");
  $data_out = "";

  #########
  # VOLUME
  #########
  if ( defined $data{'perf'}{'volume'} && $data{'perf'}{'volume'} ne "" ){
    $data_out .= "uuid;time;io_rate;read_io;write_io;resp_t;resp_t_r;resp_t_w;read;write;data_rate;r_cache_hit;w_cache_hit;capacity_total;capacity_used;capacity_free;capacity_used_percent;sum_capacity_used;snapshot_capacity_used;sum_capacity_used_percent;sum_capacity_reserved;capacity_reserved;snapshot_capacity_reserved\n";
    while ((my $volume, my $value) = each (%{ $data{'perf'}{'volume'} })){
      my $name           = "$volume";
      my $id             = "";
      my $io_rate        = "";
      my $read_io        = "";
      my $write_io       = "";
      my $resp_t         = "";
      my $resp_t_r       = "";
      my $resp_t_w       = "";
      my $read           = "";
      my $write          = "";
      my $data_rate      = "";
      my $r_cache_hit    = "";
      my $w_cache_hit    = "";
      my $total          = "";
      my $used           = "";
      my $free           = "";
      my $used_pct       = "";
      my $snap_used      = "";
      my $sum_capacity_used = "";
      my $sum_capacity_used_percent  = "";

      my $sum_capacity_reserved      = "";
      my $capacity_reserved          = "";
      my $snapshot_capacity_reserved = "";
      # my $cap            = "";
      # my $cap_total      = "";
      # my $cap_used       = "";
      # my $cap_real       = "";
      my $vol_pool_name  = "";
      my $wwn            = "";
      my @hosts          = ();

      if (defined $data{'perf'}{'volume'}{$volume}{'id'} && $data{'perf'}{'volume'}{$volume}{'id'} ne ""){

        $id = $data{'perf'}{'volume'}{$volume}{'id'};

        # inside if because sometimes volumes dont have "id" and generated hash_key from id "" is duplicated
        if (defined $data{'perf'}{'volume'}{$volume}{'io_rate'} && $data{'perf'}{'volume'}{$volume}{'io_rate'} ne ""){
          $io_rate =  Xormon::formatNumberByType("float4",$data{'perf'}{'volume'}{$volume}{'io_rate'});
        }
        if (defined $data{'perf'}{'volume'}{$volume}{'read_io'} && $data{'perf'}{'volume'}{$volume}{'read_io'} ne ""){
          $read_io =  Xormon::formatNumberByType("float4",$data{'perf'}{'volume'}{$volume}{'read_io'});
        }
        if (defined $data{'perf'}{'volume'}{$volume}{'write_io'} && $data{'perf'}{'volume'}{$volume}{'write_io'} ne ""){
          $write_io =  Xormon::formatNumberByType("float4",$data{'perf'}{'volume'}{$volume}{'write_io'});
        }
        if (defined $data{'perf'}{'volume'}{$volume}{'resp_t'} && $data{'perf'}{'volume'}{$volume}{'resp_t'} ne ""){
          $resp_t =  Xormon::formatNumberByType("float4",$data{'perf'}{'volume'}{$volume}{'resp_t'});
        }
        if (defined $data{'perf'}{'volume'}{$volume}{'resp_t_r'} && $data{'perf'}{'volume'}{$volume}{'resp_t_r'} ne ""){
          $resp_t_r =  Xormon::formatNumberByType("float4",$data{'perf'}{'volume'}{$volume}{'resp_t_r'});
        }
        if (defined $data{'perf'}{'volume'}{$volume}{'resp_t_w'} && $data{'perf'}{'volume'}{$volume}{'resp_t_w'} ne ""){
          $resp_t_w = Xormon::formatNumberByType("float4",$data{'perf'}{'volume'}{$volume}{'resp_t_w'});
        }
        if (defined $data{'perf'}{'volume'}{$volume}{'read'} && $data{'perf'}{'volume'}{$volume}{'read'} ne ""){
          $read = Xormon::returnBytes( $data{'perf'}{'volume'}{$volume}{'read'}, "kB" );
        }
        if (defined $data{'perf'}{'volume'}{$volume}{'write'} && $data{'perf'}{'volume'}{$volume}{'write'} ne ""){
          $write = Xormon::returnBytes( $data{'perf'}{'volume'}{$volume}{'write'}, "kB" );
        }
        if (defined $data{'perf'}{'volume'}{$volume}{'data_rate'} && $data{'perf'}{'volume'}{$volume}{'data_rate'} ne ""){
          $data_rate = Xormon::returnBytes( $data{'perf'}{'volume'}{$volume}{'data_rate'}, "kB" );
        }
        # does not exist in API
        $r_cache_hit = "";
        $w_cache_hit = "";
        if (defined $data{'perf'}{'volume'}{$volume}{'wwn'} && $data{'perf'}{'volume'}{$volume}{'wwn'} ne ""){
          $wwn = $data{'perf'}{'volume'}{$volume}{'wwn'};
        }
        if (defined $data{'perf'}{'volume'}{$volume}{'hosts'} && $data{'perf'}{'volume'}{$volume}{'hosts'} ne ""){
          (@hosts) = split( " ", $data{'perf'}{'volume'}{$volume}{'hosts'});
        }

        # capacity magic

        # if (defined $data{'perf'}{'volume'}{$volume}{'cap'} && $data{'perf'}{'volume'}{$volume}{'cap'} ne ""){
        #   $cap = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'cap'},"MiB"); # {'rawReserved'}->{'totalMiB'}
        # }
        # if (defined $data{'perf'}{'volume'}{$volume}{'virt'} && $data{'perf'}{'volume'}{$volume}{'virt'} ne ""){
        #   $cap_total = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'virt'},"MiB"); # virtualSizeMib
        # }
        # if (defined $data{'perf'}{'volume'}{$volume}{'real'} && $data{'perf'}{'volume'}{$volume}{'real'} ne ""){
        #   $cap_real = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'real'},"MiB"); # {'totalSpace'}->{'hostWriteMiB'}
        # }
        # if (defined $data{'perf'}{'volume'}{$volume}{'cap_used'} && $data{'perf'}{'volume'}{$volume}{'cap_used'} ne ""){
        #   $used = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'cap_used'},"MiB"); # {'totalSpace'}->{'usedMiB'}
        # }
        # if ( Xormon::isdigit($total) && Xormon::isdigit($user_used) && ($total != 0) ){
        #   $user_used_pct = Xormon::formatNumberByType("float4", $user_used / ($total / 100) );
        # }

        ## SIZE OF VOLUME - CAPACITY_TOTAL
        if (defined $data{'perf'}{'volume'}{$volume}{'size'} && $data{'perf'}{'volume'}{$volume}{'size'} ne ""){
          $total = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'size'},"MiB");
        }
        ## SPECIFIC FOR 3PAR, USER CAPACITY USED SHOULD BE TREATED AS CAPACITY_USED IN NG
        if (defined $data{'perf'}{'volume'}{$volume}{'used_user'} && $data{'perf'}{'volume'}{$volume}{'used_user'} ne ""){
          $used = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'used_user'},"MiB"); # $member->{'userSpace'}->{'usedMiB'}
        }
        ## SIMPLE DIFF SIZE VS USED_USER WILL GET FREE
        if ( Xormon::isdigit($total) && Xormon::isdigit($used) ){
          $free = $total - $used;
        }
        if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
          $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
        }
        ## SNAPSHOT USED
        if (defined $data{'perf'}{'volume'}{$volume}{'used_snap'} && $data{'perf'}{'volume'}{$volume}{'used_snap'} ne ""){
          $snap_used = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'used_snap'},"MiB"); # {'snapshotSpace'}->{'usedMiB'}
        }
        ## 3PAR TOTAL_USED METRIC IS USER_USED+SNAPSHOT_USED, not as capacity_used but new metric
        if (defined $data{'perf'}{'volume'}{$volume}{'total_used'} && $data{'perf'}{'volume'}{$volume}{'total_used'} ne ""){
          $sum_capacity_used = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'total_used'},"MiB"); # $total_u = $user_u + $snap_u;
        }
        ## PERCENT OF SIZE(TOTAL) and SUM USED (USER_USED+SNAPSHOT)
        if ( Xormon::isdigit($total) && Xormon::isdigit($sum_capacity_used) && ($total != 0) ){
          $sum_capacity_used_percent = Xormon::formatNumberByType("float4", $sum_capacity_used / ($total / 100) );
        }
        ## RESERVED CAPS FOR FUTURE USE
        if (defined $data{'perf'}{'volume'}{$volume}{'sum_capacity_reserved'} && $data{'perf'}{'volume'}{$volume}{'sum_capacity_reserved'} ne ""){
          $sum_capacity_reserved = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'sum_capacity_reserved'},"MiB");
        }        
        if (defined $data{'perf'}{'volume'}{$volume}{'capacity_reserved'} && $data{'perf'}{'volume'}{$volume}{'capacity_reserved'} ne ""){
          $capacity_reserved = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'capacity_reserved'},"MiB");
        }        
        if (defined $data{'perf'}{'volume'}{$volume}{'snapshot_capacity_reserved'} && $data{'perf'}{'volume'}{$volume}{'snapshot_capacity_reserved'} ne ""){
          $snapshot_capacity_reserved = Xormon::returnBytes($data{'perf'}{'volume'}{$volume}{'snapshot_capacity_reserved'},"MiB");
        }
        ###############################
        ###############################
        if ( defined $data{'perf'}{'volume'}{$volume}{'poolName'} && $data{'perf'}{'volume'}{$volume}{'poolName'} ne "" ){
          $vol_pool_name = $data{'perf'}{'volume'}{$volume}{'poolName'};
        }

        # data for POOL(CPG)
        while ((my $pool, my $value) = each (%{ $data{'perf'}{'pool'} })){
          if ( $vol_pool_name eq $pool ){
            #Pool to volume
            my $hash_vol_id = Xormon::create_hash_key($st_serial."VOLUME".$data{'perf'}{'volume'}{$volume}{'id'} ); $hash_vol_id = lc($hash_vol_id);
            if (defined $data{'perf'}{'pool'}{$pool}{'volumes'} && $data{'perf'}{'pool'}{$pool}{'volumes'} ne ""){
              $data{'perf'}{'pool'}{$pool}{'volumes'} .=" $hash_vol_id";
            }
            else{
              $data{'perf'}{'pool'}{$pool}{'volumes'} = $hash_vol_id;
            }
            #Virt
            if (defined $data{'perf'}{'volume'}{$volume}{'virt'} && $data{'perf'}{'volume'}{$volume}{'virt'} ne ""){
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'virt'} ) ){
                $data{'perf'}{'pool'}{$pool}{'virt'} += $data{'perf'}{'volume'}{$volume}{'virt'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'virt'} = $data{'perf'}{'volume'}{$volume}{'virt'};
              }
            }
            #Admin
            if (defined $data{'perf'}{'volume'}{$volume}{'admin'} && $data{'perf'}{'volume'}{$volume}{'admin'} ne ""){
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'admin'} ) ){
                $data{'perf'}{'pool'}{$pool}{'vol_admin'} += $data{'perf'}{'volume'}{$volume}{'admin'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'vol_admin'} = $data{'perf'}{'volume'}{$volume}{'admin'};
              }
            }
            #Snap
            if (defined $data{'perf'}{'volume'}{$volume}{'snap'} && $data{'perf'}{'volume'}{$volume}{'snap'} ne ""){
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'snap'} ) ){
                $data{'perf'}{'pool'}{$pool}{'vol_snap'} += $data{'perf'}{'volume'}{$volume}{'snap'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'vol_snap'} = $data{'perf'}{'volume'}{$volume}{'snap'};
              }
            }
            #User
            if (defined $data{'perf'}{'volume'}{$volume}{'user'} && $data{'perf'}{'volume'}{$volume}{'user'} ne ""){
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'user'} ) ){
                $data{'perf'}{'pool'}{$pool}{'vol_user'} += $data{'perf'}{'volume'}{$volume}{'user'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'vol_user'} = $data{'perf'}{'volume'}{$volume}{'user'};
              }
            }

            #Real
            if (defined $data{'perf'}{'volume'}{$volume}{'real'} && $data{'perf'}{'volume'}{$volume}{'real'} ne ""){
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'real'} ) ){
                $data{'perf'}{'pool'}{$pool}{'real'} += $data{'perf'}{'volume'}{$volume}{'real'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'real'} = $data{'perf'}{'volume'}{$volume}{'real'};
              }
            }
            #IO RATE && RESP T
            if (defined $data{'perf'}{'volume'}{$volume}{'resp_t'} && $data{'perf'}{'volume'}{$volume}{'resp_t'} ne "" && $data{'perf'}{'volume'}{$volume}{'io_rate'} && $data{'perf'}{'volume'}{$volume}{'io_rate'} ne "" ){
              #IO RATE
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'io_rate'} ) ){
                $data{'perf'}{'pool'}{$pool}{'io_rate'} += $data{'perf'}{'volume'}{$volume}{'io_rate'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'io_rate'} = $data{'perf'}{'volume'}{$volume}{'io_rate'};
              }

              #RESP T
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'resp_t'} ) ){
                $data{'perf'}{'pool'}{$pool}{'resp_t'} += ( $data{'perf'}{'volume'}{$volume}{'io_rate'} * $data{'perf'}{'volume'}{$volume}{'resp_t'} );
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'resp_t'} = ( $data{'perf'}{'volume'}{$volume}{'io_rate'} * $data{'perf'}{'volume'}{$volume}{'resp_t'} );
              }
            }

            #IO READ && RESP T R
            if (defined $data{'perf'}{'volume'}{$volume}{'resp_t_r'} && $data{'perf'}{'volume'}{$volume}{'resp_t_r'} ne "" && $data{'perf'}{'volume'}{$volume}{'read_io'} && $data{'perf'}{'volume'}{$volume}{'read_io'} ne "" ){
              #IO READ
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'read_io'} ) ){
                $data{'perf'}{'pool'}{$pool}{'read_io'} += $data{'perf'}{'volume'}{$volume}{'read_io'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'read_io'} = $data{'perf'}{'volume'}{$volume}{'read_io'};
              }

              #RESP T R
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'resp_t_r'} ) ){
                $data{'perf'}{'pool'}{$pool}{'resp_t_r'} += ( $data{'perf'}{'volume'}{$volume}{'read_io'} * $data{'perf'}{'volume'}{$volume}{'resp_t_r'} );
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'resp_t_r'} = ( $data{'perf'}{'volume'}{$volume}{'read_io'} * $data{'perf'}{'volume'}{$volume}{'resp_t_r'} );
              }
            }

            #IO WRITE && RESP T W
            if (defined $data{'perf'}{'volume'}{$volume}{'resp_t_w'} && $data{'perf'}{'volume'}{$volume}{'resp_t_w'} ne "" && $data{'perf'}{'volume'}{$volume}{'write_io'} && $data{'perf'}{'volume'}{$volume}{'write_io'} ne "" ){
              #IO WRITE
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'write_io'} ) ){
                $data{'perf'}{'pool'}{$pool}{'write_io'} += $data{'perf'}{'volume'}{$volume}{'write_io'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'write_io'} = $data{'perf'}{'volume'}{$volume}{'write_io'};
              }

              #RESP T W
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'resp_t_w'} ) ){
                $data{'perf'}{'pool'}{$pool}{'resp_t_w'} += ( $data{'perf'}{'volume'}{$volume}{'write_io'} * $data{'perf'}{'volume'}{$volume}{'resp_t_w'} );
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'resp_t_w'} = ( $data{'perf'}{'volume'}{$volume}{'write_io'} * $data{'perf'}{'volume'}{$volume}{'resp_t_w'} );
              }
            }

            #DATA RATE
            if (defined $data{'perf'}{'volume'}{$volume}{'data_rate'} && $data{'perf'}{'volume'}{$volume}{'data_rate'} ne "" ){
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'data_rate'} ) ){
                $data{'perf'}{'pool'}{$pool}{'data_rate'} += $data{'perf'}{'volume'}{$volume}{'data_rate'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'data_rate'} = $data{'perf'}{'volume'}{$volume}{'data_rate'};
              }
            }

            #DATA READ
            if (defined $data{'perf'}{'volume'}{$volume}{'read'} && $data{'perf'}{'volume'}{$volume}{'read'} ne "" ){
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'read'} ) ){
                $data{'perf'}{'pool'}{$pool}{'read'} += $data{'perf'}{'volume'}{$volume}{'read'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'read'} = $data{'perf'}{'volume'}{$volume}{'read'};
              }
            }

            #DATA WRITE
            if (defined $data{'perf'}{'volume'}{$volume}{'write'} && $data{'perf'}{'volume'}{$volume}{'write'} ne "" ){
              if ( Xormon::isdigit( $data{'perf'}{'pool'}{$pool}{'read'} ) ){
                $data{'perf'}{'pool'}{$pool}{'write'} += $data{'perf'}{'volume'}{$volume}{'write'};
              }
              else
              {
                $data{'perf'}{'pool'}{$pool}{'write'} = $data{'perf'}{'volume'}{$volume}{'write'};
              }
            }
          }
        }

        #Hash Volume
        my $hash_key = Xormon::create_hash_key($st_serial."VOLUME".$id ); $hash_key = lc($hash_key);
        #print "VOLUME HASH KEY : $hash_key\n";
        #Hash Pool
        my $pool_hash_key = Xormon::create_hash_key($st_serial."POOL".$vol_pool_name ); $pool_hash_key = lc($pool_hash_key);
        my %arch = (
            "item_id" => $hash_key,
            "label" => $name,
            "hw_type" => "3par",
            "subsystem" => "volume",
            "wwn" => Xormon::formatWWN($wwn),
            "hostcfg_id" => "$cfg->{'hostcfg_id'}",
            "parents" => [$cfg->{'hostcfg_id'},$pool_hash_key],
          );
          push(@{$arch{'parents'}} , @hosts);

          # conf part
          if (defined $data{'perf'}{'volume'}{$volume}{'size'} && $data{'perf'}{'volume'}{$volume}{'size'} ne "" ){
            $conf{$hash_key}{size} = $data{'perf'}{'volume'}{$volume}{'size'};
          }

          $conf{$hash_key}{name} = $name;
          $conf{$hash_key}{lun_id} = $wwn;
          $conf{$hash_key}{wwn} = Xormon::formatWWN($wwn);

        push(@architecture, \%arch);
        $data_out .= "$hash_key;$time;$io_rate;$read_io;$write_io;$resp_t;$resp_t_r;$resp_t_w;$read;$write;$data_rate;$r_cache_hit;$w_cache_hit;$total;$used;$free;$used_pct;$sum_capacity_used;$snap_used;$sum_capacity_used_percent;$sum_capacity_reserved;$capacity_reserved;$snapshot_capacity_reserved\n";
        #$data_out .= "uuid;time;io_rate;read_io;write_io;resp_t;resp_t_r;resp_t_w;read;write;data_rate;r_cache_hit;w_cache_hit;cap;capacity_used;cap_real;total_u;capacity_used_snap;capacity_used_user\n";

      } ## end if (defined $data{'perf'}{'volume'}{$volume}{'id'}
    } ## end while each
    # print $data_out;
  } # if defined volume

  #SAVE VOLUME\n";
  #print "VOLUME DATA OUT\n";
  #print "$data_out";
  $rc = $xormon->saveCSV($data_out,"3par","volume");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("VOLUME status : $status");
  $data_out = "";

  ########
  # POOL
  ########
  if ( defined $data{'perf'}{'pool'} && $data{'perf'}{'pool'} ne "" ){
    #Table Capacity
    #my $pool_cap = "$webdir/$cfg->{'hostalias'}/pool_capacity.html";
    # my $pool_table = "<table class = \"tablesorter\">";
    # my $pool_thead = "<thead><tr><th rowspan=\"2\" class='sortable tablesorter-header tablesorter-headerUnSorted'>CPGS</th>";
    # $pool_thead .= "<th colspan=\"4\" style=\"text-align:center;\">Allocated [TiB]</th>";
    # $pool_thead .= "<th colspan=\"4\" style=\"text-align:center;\">Used [TiB]</th>";
    # $pool_thead .= "<th colspan=\"4\" style=\"text-align:center;\">Free [TiB]</th>";
    # $pool_thead .= "<th rowspan=\"2\" class='sortable tablesorter-header tablesorter-headerUnSorted'>Used %</th>";
    # $pool_thead .= "</tr><tr><th class='sortable tablesorter-header tablesorter-headerUnSorted'>Total</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>User</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>Snap</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>Admin</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>Total</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>User</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>Snap</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>Admin</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>Total</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>User</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>Snap</th>";
    # $pool_thead .= "<th class='sortable tablesorter-header tablesorter-headerUnSorted'>Admin</th></th></tr></thead>";
    # my $pool_tbody = "<tbody>\n";
    # my $pool_tr    = "";

    my $device_out = "id;timestamp;io_rate;read_io;write_io;data_rate;read;write;resp_t;resp_t_r;resp_t_w;capacity_total;capacity_used;capacity_free;capacity_used_percent\n";

    my $device_io       = "";
    my $device_io_r     = "";
    my $device_io_w     = "";

    my $device_data     = "";
    my $device_data_r   = "";
    my $device_data_w   = "";

    my $device_resp_t   = "";
    my $device_resp_t_r = "";
    my $device_resp_t_w = "";

    my $device_total    = "";
    my $device_free     = "";
    my $device_used     = "";
    my $device_used_pct = "";

    $data_out .= "uuid;time;read;write;read_io;write_io;resp_t_r;resp_t_w;data_rate;io_rate;resp_t;admin_capacity_allocated;admin_capacity_free;admin_capacity_used;user_capacity_allocated;user_capacity_free;user_capacity_used;snapshot_capacity_allocated;snapshot_capacity_free;snapshot_capacity_used;capacity_allocated;sum_capacity_used;sum_capacity_used_percent;capacity_free\n";
    while ((my $pool, my $value) = each (%{ $data{'perf'}{'pool'} })){
      my $name      = $pool;
      my $id        = "";
      # my $admin     = "";
      # my $virt      = "";
      # my $snap      = "";
      # my $user      = "";
      # my $real      = "";
      # my $total     = "";
      # my $free      = "";
      # my $used      = "";

      my $used_pct  = "";

      my $read      = "";
      my $write     = "";
      my $read_io   = "";
      my $write_io  = "";
      my $resp_t_r  = "";
      my $resp_t_w  = "";
      my $data_rate = "";
      my $io_rate   = "";
      my $resp_t    = "";

      my $admin_t   = "";
      my $admin_f   = "";
      my $admin_u   = "";
      my $user_t    = "";
      my $user_f    = "";
      my $user_u    = "";
      my $snap_t    = "";
      my $snap_f    = "";
      my $snap_u    = "";
      # my $raw_admin_t   = "";
      # my $raw_admin_f   = "";
      # my $raw_admin_u   = "";
      # my $raw_user_t    = "";
      # my $raw_user_f    = "";
      # my $raw_user_u    = "";
      # my $raw_snap_t    = "";
      # my $raw_snap_f    = "";
      # my $raw_snap_u    = "";
      # my $comp          = "";
      # my $provisioning  = "";
      # my $compress      = "";
      # my $deduplic      = "";
      if (defined $data{'perf'}{'pool'}{$pool}{'id'} && $data{'perf'}{'pool'}{$pool}{'id'} ne ""){
        $id = $data{'perf'}{'pool'}{$pool}{'id'};
      }
      #cap rrc
      # if (defined $data{'perf'}{'pool'}{$pool}{'vol_admin'} && $data{'perf'}{'pool'}{$pool}{'vol_admin'} ne ""){
      #   $admin = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'vol_admin'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'virt'} && $data{'perf'}{'pool'}{$pool}{'virt'} ne ""){
      #   $virt = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'virt'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'vol_snap'} && $data{'perf'}{'pool'}{$pool}{'vol_snap'} ne ""){
      #   $snap = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'vol_snap'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'vol_user'} && $data{'perf'}{'pool'}{$pool}{'vol_user'} ne ""){
      #   $user = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'vol_user'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'real'} && $data{'perf'}{'pool'}{$pool}{'real'} ne ""){
      #   $real = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'real'});
      # }
      #cap rrd
      # if (defined $data{'perf'}{'pool'}{$pool}{'total'} && $data{'perf'}{'pool'}{$pool}{'total'} ne ""){
      #   $total = $data{'perf'}{'pool'}{$pool}{'total'};
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'free'} && $data{'perf'}{'pool'}{$pool}{'free'} ne ""){
      #   $free = $data{'perf'}{'pool'}{$pool}{'free'};
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'used'} && $data{'perf'}{'pool'}{$pool}{'used'} ne ""){
      #   $used = $data{'perf'}{'pool'}{$pool}{'used'};
      # }

      #rrd
      if (defined $data{'perf'}{'pool'}{$pool}{'read'} && $data{'perf'}{'pool'}{$pool}{'read'} ne ""){
        $read = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'read'}, "kB" );
        if (Xormon::isdigit($device_data_r)){
          $device_data_r += $read;
        }
        else{
          $device_data_r = $read;
        }
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'write'} && $data{'perf'}{'pool'}{$pool}{'write'} ne ""){
        $write = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'write'}, "kB" );
        if (Xormon::isdigit($device_data_w)){
          $device_data_w += $write;
        }
        else{
          $device_data_w = $write;
        }
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'read_io'} && $data{'perf'}{'pool'}{$pool}{'read_io'} ne ""){
        $read_io = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'read_io'});
        if (Xormon::isdigit($device_io_r)){
          $device_io_r += $read_io;
        }
        else{
          $device_io_r = $read_io;
        }
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'write_io'} && $data{'perf'}{'pool'}{$pool}{'write_io'} ne ""){
        $write_io = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'write_io'});
        if (Xormon::isdigit($device_io_w)){
          $device_io_w += $write_io;
        }
        else{
          $device_io_w = $write_io;
        }
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'resp_t_r'} && $data{'perf'}{'pool'}{$pool}{'resp_t_r'} ne "" && defined $data{'perf'}{'pool'}{$pool}{'read_io'} && $data{'perf'}{'pool'}{$pool}{'read_io'} ne "" && $data{'perf'}{'pool'}{$pool}{'read_io'} != 0){
        $resp_t_r = Xormon::formatNumberByType("float4", ( $data{'perf'}{'pool'}{$pool}{'resp_t_r'} / $data{'perf'}{'pool'}{$pool}{'read_io'}  ) );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'resp_t_w'} && $data{'perf'}{'pool'}{$pool}{'resp_t_w'} ne "" && defined $data{'perf'}{'pool'}{$pool}{'write_io'} && $data{'perf'}{'pool'}{$pool}{'write_io'} ne "" && $data{'perf'}{'pool'}{$pool}{'write_io'} != 0){
        $resp_t_w = Xormon::formatNumberByType("float4", ( $data{'perf'}{'pool'}{$pool}{'resp_t_w'} / $data{'perf'}{'pool'}{$pool}{'write_io'} ) );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'data_rate'} && $data{'perf'}{'pool'}{$pool}{'data_rate'} ne ""){
        $data_rate = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'data_rate'}, "kB" );
        if (Xormon::isdigit($device_data)){
          $device_data += $data_rate;
        }
        else{
          $device_data = $data_rate;
        }
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'io_rate'} && $data{'perf'}{'pool'}{$pool}{'io_rate'} ne ""){
        $io_rate = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'io_rate'});
        if (Xormon::isdigit($device_io)){
          $device_io += $io_rate;
        }
        else{
          $device_io = $io_rate;
        }
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'resp_t'} && $data{'perf'}{'pool'}{$pool}{'resp_t'} ne "" && defined $data{'perf'}{'pool'}{$pool}{'io_rate'} && $data{'perf'}{'pool'}{$pool}{'io_rate'} ne "" && $data{'perf'}{'pool'}{$pool}{'io_rate'} != 0 ){
        $resp_t = Xormon::formatNumberByType("float4", ( $data{'perf'}{'pool'}{$pool}{'resp_t'} / $data{'perf'}{'pool'}{$pool}{'io_rate'} ) );
      }
      #rre new metrics
      if (defined $data{'perf'}{'pool'}{$pool}{'admin'}{'free'} && $data{'perf'}{'pool'}{$pool}{'admin'}{'free'} ne ""){
        $admin_f = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'admin'}{'free'}, "MiB" );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'admin'}{'used'} && $data{'perf'}{'pool'}{$pool}{'admin'}{'used'} ne ""){
        $admin_u = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'admin'}{'used'}, "MiB" );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'admin'}{'total'} && $data{'perf'}{'pool'}{$pool}{'admin'}{'total'} ne ""){
        $admin_t = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'admin'}{'total'}, "MiB" );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'user'}{'total'} && $data{'perf'}{'pool'}{$pool}{'user'}{'total'} ne ""){
        $user_t = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'user'}{'total'}, "MiB" );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'user'}{'free'} && $data{'perf'}{'pool'}{$pool}{'user'}{'free'} ne ""){
        $user_f = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'user'}{'free'}, "MiB" );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'user'}{'used'} && $data{'perf'}{'pool'}{$pool}{'user'}{'used'} ne ""){
        $user_u = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'user'}{'used'}, "MiB" );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'snap'}{'total'} && $data{'perf'}{'pool'}{$pool}{'snap'}{'total'} ne ""){
        $snap_t = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'snap'}{'total'}, "MiB" );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'snap'}{'free'} && $data{'perf'}{'pool'}{$pool}{'snap'}{'free'} ne ""){
        $snap_f = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'snap'}{'free'}, "MiB" );
      }
      if (defined $data{'perf'}{'pool'}{$pool}{'snap'}{'used'} && $data{'perf'}{'pool'}{$pool}{'snap'}{'used'} ne ""){
        $snap_u = Xormon::returnBytes( $data{'perf'}{'pool'}{$pool}{'snap'}{'used'}, "MiB");
      }
      # if (defined $data{'perf'}{'pool'}{$pool}{'admin'}{'rawTotal'} && $data{'perf'}{'pool'}{$pool}{'admin'}{'rawTotal'} ne ""){
      #   $raw_admin_t = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'admin'}{'rawTotal'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'admin'}{'rawFree'} && $data{'perf'}{'pool'}{$pool}{'admin'}{'rawFree'} ne ""){
      #   $raw_admin_f = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'admin'}{'rawFree'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'admin'}{'rawUsed'} && $data{'perf'}{'pool'}{$pool}{'admin'}{'rawUsed'} ne ""){
      #   $raw_admin_u = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'admin'}{'rawUsed'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'user'}{'rawTotal'} && $data{'perf'}{'pool'}{$pool}{'user'}{'rawTotal'} ne ""){
      #   $raw_user_t = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'user'}{'rawTotal'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'user'}{'rawFree'} && $data{'perf'}{'pool'}{$pool}{'user'}{'rawFree'} ne ""){
      #   $raw_user_f = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'user'}{'rawFree'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'user'}{'rawUsed'} && $data{'perf'}{'pool'}{$pool}{'user'}{'rawUsed'} ne ""){
      #   $raw_user_u = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'user'}{'rawUsed'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'snap'}{'rawTotal'} && $data{'perf'}{'pool'}{$pool}{'snap'}{'rawTotal'} ne ""){
      #   $raw_snap_t = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'snap'}{'rawTotal'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'snap'}{'rawFree'} && $data{'perf'}{'pool'}{$pool}{'snap'}{'rawFree'} ne ""){
      #   $raw_snap_f = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'snap'}{'rawFree'});
      # }
      # if (defined $data{'perf'}{'pool'}{$pool}{'snap'}{'rawUsed'} && $data{'perf'}{'pool'}{$pool}{'snap'}{'rawUsed'} ne ""){
      #   $raw_snap_u = Xormon::formatNumberByType("float4",$data{'perf'}{'pool'}{$pool}{'snap'}{'rawUsed'});
      # }
      #Efficiency
      # if ( defined $data{'perf'}{'pool'}{$name}{'overProvisioning'} && $data{'perf'}{'pool'}{$name}{'overProvisioning'} ne ""){
      #   $provisioning = $data{'perf'}{'pool'}{$name}{'overProvisioning'};
      #  }
      # if ( defined $data{'perf'}{'pool'}{$name}{'compaction'} && $data{'perf'}{'pool'}{$name}{'compaction'} ne "" ){
      #   $comp = $data{'perf'}{'pool'}{$name}{'compaction'};
      # }
      # if ( defined $data{'perf'}{'pool'}{$name}{'compression'} && $data{'perf'}{'pool'}{$name}{'compression'} ne "" ){
      #   $compress = $data{'perf'}{'pool'}{$name}{'compression'};
      # }
      # if ( defined $data{'perf'}{'pool'}{$name}{'deduplicationGcKBPS'} && $data{'perf'}{'pool'}{$name}{'deduplicationGcKBPS'} ne "" ) {
      #   $deduplic = $data{'perf'}{'pool'}{$name}{'deduplicationGcKBPS'};
      # }

      # DEVICE resp_t
      if (defined $resp_t && $resp_t ne "" && $io_rate && $io_rate ne "" ){
        if ( Xormon::isdigit( $data{'perf'}{'device'}{'resp_t'} ) ){
          $data{'perf'}{'device'}{'resp_t'} += $io_rate * $resp_t;
        }
        else
        {
          $data{'perf'}{'device'}{'resp_t'} = $io_rate * $resp_t;
        }
      }
      # DEVICE resp_t_r
      if (defined $resp_t_r && $resp_t_r ne "" && $read_io && $read_io ne "" ){
        if ( Xormon::isdigit( $data{'perf'}{'device'}{'resp_t_r'} ) ){
          $data{'perf'}{'device'}{'resp_t_r'} += $read_io * $resp_t_r;
        }
        else
        {
          $data{'perf'}{'device'}{'resp_t_r'} = $read_io * $resp_t_r;
        }
      }
      # DEVICE resp_t_w
      if (defined $resp_t_w && $resp_t_w ne "" && $write_io && $write_io ne "" ){
        if ( Xormon::isdigit( $data{'perf'}{'device'}{'resp_t_w'} ) ){
          $data{'perf'}{'device'}{'resp_t_w'} += $write_io * $resp_t_w;
        }
        else
        {
          $data{'perf'}{'device'}{'resp_t_w'} = $write_io * $resp_t_w;
        }
      }

      my $used_total = "";
      if ($admin_u ne "" && $user_u ne "" && $snap_u ne ""){
        $used_total = $admin_u + $user_u + $snap_u;
      }
      my $allocated_total = "";
      if ($admin_t ne "" && $user_t ne "" && $snap_t ne ""){
        $allocated_total = $admin_t + $user_t + $snap_t;
      }
      my $free_total = "";
      if ($admin_f ne "" && $user_f ne "" && $snap_f ne ""){
        $free_total = $admin_f + $user_f + $snap_f;
      }

      if ( Xormon::isdigit($allocated_total) && Xormon::isdigit($used_total) && ($allocated_total != 0) ){
        $used_pct = Xormon::formatNumberByType("float4", $used_total / ($allocated_total / 100) );
      }

      my $hash_key = Xormon::create_hash_key($st_serial."POOL".$pool ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "pool",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );

        if(defined $data{'perf'}{'pool'}{$pool}{'volumes'} && $data{'perf'}{'pool'}{$pool}{'volumes'} ne "" ){
          (@{$arch{'childrens'}}) = split( " ", $data{'perf'}{'pool'}{$pool}{'volumes'});
        }

        # $conf{$hash_key}{capacity_total}            = $total;
        # $conf{$hash_key}{capacity_free}             = $free;
        # $conf{$hash_key}{capacity_used}             = $used;
        # $conf{$hash_key}{capacity_user}             = $user;
        # $conf{$hash_key}{capacity_snap}             = $snap;
        # $conf{$hash_key}{capacity_admin}            = $admin;

      push(@architecture, \%arch);

      #print PERF "$time,$name,$id,$admin,$virt,$snap,$user,$real,$total,$free,$used,$read,$write,$read_io,$write_io,$resp_t_r,$resp_t_w,$data_rate,$io_rate,$resp_t,$admin_t,$admin_f,$admin_u,$user_t,$user_f,$user_u,$snap_t,$snap_f,$snap_u,$raw_admin_t,$raw_admin_f,$raw_admin_u,$raw_user_t,$raw_user_f,$raw_user_u,$raw_snap_t,$raw_snap_f,$raw_snap_u,$allocated_total,$user_t,$snap_t,$used_total,$compress\n";
      # $data_out .= "$hash_key;$time;$total;$free;$used;$used_pct;$read;$write;$read_io;$write_io;$resp_t_r;$resp_t_w;$data_rate;$io_rate;$resp_t;$admin_t;$admin_f;$admin_u;$user_t;$user_f;$user_u;$snap_t;$snap_f;$snap_u;$raw_admin_t;$raw_admin_f;$raw_admin_u;$raw_user_t;$raw_user_f;$raw_user_u;$raw_snap_t;$raw_snap_f;$raw_snap_u;$allocated_total;$user_t;$snap_t;$used_total\n";
      $data_out .= "$hash_key;$time;$read;$write;$read_io;$write_io;$resp_t_r;$resp_t_w;$data_rate;$io_rate;$resp_t;$admin_t;$admin_f;$admin_u;$user_t;$user_f;$user_u;$snap_t;$snap_f;$snap_u;$allocated_total;$used_total;$used_pct;$free_total\n";

      #Xormon::set_link_item( { device_label => $cfg->{'hostalias'}, subsys => 'POOL', item_label => $name, device_hwtype => '3PAR', label => $name, metric => 'used' } )
      # $pool_tr .= "<tr><td><b>" . "Set link item" . "</b></td>\n";
      # my $allocated_tbl = "";
      # $user_t  = sprintf( '%.2f', $user_t / 1024);
      # $snap_t  = sprintf( '%.2f', $snap_t / 1024);
      # $admin_t = sprintf( '%.2f', $admin_t / 1024);

      # $allocated_tbl = $admin_t + $user_t + $snap_t;
      # $pool_tr .= "<td>$allocated_tbl</td>";
      # $pool_tr .= "<td>$user_t</td>";
      # $pool_tr .= "<td>$snap_t</td>";
      # $pool_tr .= "<td>$admin_t</td>";

      # my $used_tbl = "";
      # $user_u  = sprintf( '%.2f', $user_u / 1024);
      # $snap_u  = sprintf( '%.2f', $snap_u / 1024);
      # $admin_u = sprintf( '%.2f', $admin_u / 1024);

      # $used_tbl = $admin_u + $user_u + $snap_u;
      # $pool_tr .= "<td>$used_tbl</td>";
      # $pool_tr .= "<td>$user_u</td>";
      # $pool_tr .= "<td>$snap_u</td>";
      # $pool_tr .= "<td>$admin_u</td>";

      # my $my_free = "";
      # $user_f  = sprintf( '%.2f', $user_f / 1024);
      # $snap_f  = sprintf( '%.2f', $snap_f / 1024);
      # $admin_f = sprintf( '%.2f', $admin_f / 1024);

      # $my_free = $admin_f + $user_f + $snap_f;
      # $pool_tr .= "<td>$my_free</td>";
      # $pool_tr .= "<td>$user_f</td>";
      # $pool_tr .= "<td>$snap_f</td>";
      # $pool_tr .= "<td>$admin_f</td>";

      # my $used_pct = "";
      # if ( Xormon::isdigit($allocated_tbl) && Xormon::isdigit($used_tbl) && ($allocated_tbl != 0) ){
      #   $used_pct = sprintf( '%.2f', $used_tbl / ($allocated_tbl / 100) );
      # }

      # $pool_tr .= "<td><meter value=\"$used_pct\" min=\"0\" max=\"100\" optimum=\"0\" low=\"80\" high=\"95\" title=\"$used_pct % of user capacity used\">$used_pct</meter></td></tr>";

    } # while pool
    ## RESP_T DEVICE WEIGHTED from POOLS
    if (defined $data{'perf'}{'device'}{'resp_t'} && $data{'perf'}{'device'}{'resp_t'} ne "" && defined $device_io && $device_io ne "" && $device_io != 0 ){
      $device_resp_t = Xormon::formatNumberByType("float4", ( $data{'perf'}{'device'}{'resp_t'} / $device_io ) );
    }
    if (defined $data{'perf'}{'device'}{'resp_t_r'} && $data{'perf'}{'device'}{'resp_t_r'} ne "" && defined $device_io_r && $device_io_r ne "" && $device_io_r != 0 ){
      $device_resp_t_r = Xormon::formatNumberByType("float4", ( $data{'perf'}{'device'}{'resp_t_r'} / $device_io_r ) );
    }
    if (defined $data{'perf'}{'device'}{'resp_t_w'} && $data{'perf'}{'device'}{'resp_t_w'} ne "" && defined $device_io_w && $device_io_w ne "" && $device_io_w != 0 ){
      $device_resp_t_w = Xormon::formatNumberByType("float4", ( $data{'perf'}{'device'}{'resp_t_w'} / $device_io_w ) );
    }
    ## DEVICE ENTITY METRICS from system hash key
    if ( defined $data{'system'}{'total'} && $data{'system'}{'total'} ne "" ){
      $device_total = $data{'system'}{'total'};
    }
    if ( defined $data{'system'}{'allocated'} && $data{'system'}{'allocated'} ne "" ){
      $device_used = $data{'system'}{'allocated'};
    }
    if ( defined $data{'system'}{'free'} && $data{'system'}{'free'} ne "" ){
      $device_free = $data{'system'}{'free'};
    }
    if ( Xormon::isdigit($device_total) && Xormon::isdigit($device_used) && ($device_total != 0) ){
      $device_used_pct = Xormon::formatNumberByType("float4", $device_used / ($device_total / 100) );
    }

    $device_out .= "$cfg->{'hostcfg_id'};$time;$device_io;$device_io_r;$device_io_w;$device_data;$device_data_r;$device_data_w;$device_resp_t;$device_resp_t_r;$device_resp_t_w;$device_total;$device_used;$device_free;$device_used_pct\n";

    # #Save Device
    $rc = $xormon->saveCSV($device_out,"3par","device");

    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }

    # $pool_tbody = $pool_tbody . $pool_tr . "</tbody>\n";
    # my $table       = $pool_table . $pool_thead . $pool_tbody . "</table>\n";

    #open( PC, ">$pool_cap" ) || Xormon::error( "Couldn't open file $pool_cap $!") && exit;
    #print PC "<br>Last update:" . localtime(time) . "\n";
    #print PC "<h3>Pool Capacity</h3>\n";
    #print PC "$table\n";
    #print PC "<br><br>\n";
    #close(PC);
  } # if defined pool

  #SAVE POOL\n";
  # print $data_out;
  $rc = $xormon->saveCSV($data_out,"3par","pool");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("POOL status : $status");
  $data_out = "";

  #######
  # PORT
  #######
  if ( defined $data{'perf'}{'port'} && $data{'perf'}{'port'} ne "" ){
    #print Dumper $data{'perf'}{'port'};
    $data_out .= "uuid;time;read_io;write_io;read;write;resp_t_r;resp_t_w;io_rate;data_rate;resp_t;type\n";
    while ((my $port, my $value) = each (%{ $data{'perf'}{'port'} })){
      my $id        = $port;
      my $read_io   = "";
      my $write_io  = "";
      my $read      = "";
      my $write     = "";
      my $resp_t_r  = "";
      my $resp_t_w  = "";
      my $resp_t    = "";
      my $io_rate   = "";
      my $data_rate = "";
      my $type      = "";
      my $name      = "";
      my $port_wwn  = "";
      my $label     = "";

      if (defined $data{'perf'}{'port'}{$port}{'read_io'} && $data{'perf'}{'port'}{$port}{'read_io'} ne ""){
        $read_io = $data{'perf'}{'port'}{$port}{'read_io'};
      }
      if (defined $data{'perf'}{'port'}{$port}{'write_io'} && $data{'perf'}{'port'}{$port}{'write_io'} ne ""){
        $write_io = $data{'perf'}{'port'}{$port}{'write_io'};
      }
      if (defined $data{'perf'}{'port'}{$port}{'read'} && $data{'perf'}{'port'}{$port}{'read'} ne ""){
        $read = Xormon::returnBytes( $data{'perf'}{'port'}{$port}{'read'}, "kB" );
      }
      if (defined $data{'perf'}{'port'}{$port}{'write'} && $data{'perf'}{'port'}{$port}{'write'} ne ""){
        $write = Xormon::returnBytes( $data{'perf'}{'port'}{$port}{'write'}, "kB" );
      }
      if (defined $data{'perf'}{'port'}{$port}{'resp_t_r'} && $data{'perf'}{'port'}{$port}{'resp_t_r'} ne ""){
        $resp_t_r = $data{'perf'}{'port'}{$port}{'resp_t_r'};
      }
      if ( defined $data{'perf'}{'port'}{$port}{'resp_t_w'} && $data{'perf'}{'port'}{$port}{'resp_t_w'} ne "" ){
        $resp_t_w = $data{'perf'}{'port'}{$port}{'resp_t_w'};
      }
      if ( defined $data{'perf'}{'port'}{$port}{'resp_t'} && $data{'perf'}{'port'}{$port}{'resp_t'} ne "" ){
        $resp_t = $data{'perf'}{'port'}{$port}{'resp_t'};
      }
      if ( defined $data{'perf'}{'port'}{$port}{'io_rate'} && $data{'perf'}{'port'}{$port}{'io_rate'} ne "" ){
        $io_rate = $data{'perf'}{'port'}{$port}{'io_rate'};
      }
      if ( defined $data{'perf'}{'port'}{$port}{'data_rate'} && $data{'perf'}{'port'}{$port}{'data_rate'} ne "" ){
        $data_rate = Xormon::returnBytes( $data{'perf'}{'port'}{$port}{'data_rate'}, "kB" );
      }
      if ( defined $data{'perf'}{'port'}{$port}{'type'} && $data{'perf'}{'port'}{$port}{'type'} ne "" ){
        $type = $data{'perf'}{'port'}{$port}{'type'};
      }
      if ( defined $data{'perf'}{'port'}{$port}{'name'} && $data{'perf'}{'port'}{$port}{'name'} ne "" ){
        $name = $data{'perf'}{'port'}{$port}{'name'};
        if (index($name, $id) != -1) {
          $label = $name;
        }
        else {
          $label = $id . " " . $name;
        }
      }
      if (defined $data{'perf'}{'port'}{$id}{'portWWN'} && $data{'perf'}{'port'}{$id}{'portWWN'} ne "" ){
        $port_wwn = $data{'perf'}{'port'}{$id}{'portWWN'};
      }

      my $hash_key = Xormon::create_hash_key($st_serial."PORT".$port ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $label,
          "hw_type" => "3par",
          "subsystem" => "port",
          "wwn" => Xormon::formatWWN($port_wwn),
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );

        $conf{$hash_key}{wwn} = Xormon::formatWWN($port_wwn);

      push(@architecture, \%arch);
      #print PERF "$time,$id,$read_io,$write_io,$read,$write,$resp_t_r,$resp_t_w,$io_rate,$data_rate,$type\n";
      $data_out .= "$hash_key;$time;$read_io;$write_io;$read;$write;$resp_t_r;$resp_t_w;$io_rate;$data_rate;$resp_t;$type\n";
    }
  }

  #print "SAVE PORT\n";
  #print "$data_out";
  $rc = $xormon->saveCSV($data_out,"3par","port");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("PORT status : $status");
  $data_out = "";

  #######
  # TIER
  #######
  if ( defined $data{'perf'}{'tier'} && $data{'perf'}{'tier'} ne "" ){
    $data_out .= "uuid;time;read;write;read_io;write_io;resp_t_r;resp_t_w;data_rate;resp_t;io_rate;capacity_total;capacity_free;capacity_used;capacity_used_percent;system_capacity_used;capacity_failed;volumes_capacity_used\n";
    while ((my $tier, my $value) = each (%{ $data{'perf'}{'tier'} })){
      #print "TIER $tier\n";
      my $name        = $tier;
      my $read        = "";
      my $write       = "";
      my $read_io     = "";
      my $write_io    = "";
      my $resp_t_r    = "";
      my $resp_t_w    = "";
      my $data_rate   = "";
      my $resp_t      = "";
      my $io_rate     = "";

      my $total       = "";
      my $free        = "";
      my $used        = "";
      my $used_pct    = "0";

      my $system      = "";
      my $failed      = "";
      my $unmaped     = "";

      my $cpg_user    = "";
      my $cpg_snap    = "";
      my $cpg_admin   = "";
      my $non_cpg     = "";
      my $cpg         = "";

      my $volumes_used   = "";

      ###################
      # Total/Used/Free #
      ###################

      if ( defined $data{'perf'}{'tier'}{$tier}{'totalMiB'} && defined $data{'perf'}{'tier'}{$tier}{'totalMiB'} ne "" ){
        $total = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'totalMiB'}, "MiB" );
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'freeMiB'} && defined $data{'perf'}{'tier'}{$tier}{'freeMiB'} ne "" ){
        $free = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'freeMiB'}, "MiB" );
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'allocated'} && $data{'perf'}{'tier'}{$tier}{'allocated'} ne "" ){
        $used = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'allocated'}, "MiB" );
      }
      if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
        $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
      }

      #SYSTEM
      if ( defined $data{'perf'}{'tier'}{$tier}{'system'}{'totalSystemMiB'} && $data{'perf'}{'tier'}{$tier}{'system'}{'totalSystemMiB'} ne "" ){
        $system = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'system'}{'totalSystemMiB'}, "MiB" );
      }
      #FAILED
      if ( defined $data{'perf'}{'tier'}{$tier}{'failedCapacityMiB'} && $data{'perf'}{'tier'}{$tier}{'failedCapacityMiB'} ne "" ){
        $failed = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'failedCapacityMiB'}, "MiB" );
      }
      #unmaped
      if ( defined $data{'perf'}{'tier'}{$tier}{'volumes'}{'unmappedMiB'} && $data{'perf'}{'tier'}{$tier}{'volumes'}{'unmappedMiB'} ne "" ){
        $unmaped = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'volumes'}{'unmappedMiB'}, "MiB" );
      }
      #user
      if ( defined $data{'perf'}{'tier'}{$tier}{'volumes'}{'CPGUserMiB'} && $data{'perf'}{'tier'}{$tier}{'volumes'}{'CPGUserMiB'} ne "" ){
        $cpg_user = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'volumes'}{'CPGUserMiB'}, "MiB" );
      }
      #snapshot
      if ( defined $data{'perf'}{'tier'}{$tier}{'volumes'}{'CPGSnapshotMiB'} && $data{'perf'}{'tier'}{$tier}{'volumes'}{'CPGSnapshotMiB'} ne "" ){
        $cpg_snap = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'volumes'}{'CPGSnapshotMiB'}, "MiB" );
      }
      #admin
      if ( defined $data{'perf'}{'tier'}{$tier}{'volumes'}{'CPGAdminMiB'} && $data{'perf'}{'tier'}{$tier}{'volumes'}{'CPGAdminMiB'} ne "" ){
        $cpg_admin = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'volumes'}{'CPGAdminMiB'}, "MiB" );
      }
      #Non Cpg
      if ( defined $data{'perf'}{'tier'}{$tier}{'volumes'}{'nonCPGsMiB'} && $data{'perf'}{'tier'}{$tier}{'volumes'}{'nonCPGsMiB'} ne "" ){
        $non_cpg = Xormon::returnBytes ($data{'perf'}{'tier'}{$tier}{'volumes'}{'nonCPGsMiB'}, "MiB");
      }

      if ( Xormon::isdigit($cpg_user) && Xormon::isdigit($cpg_snap) && Xormon::isdigit($cpg_admin) ) {
        $cpg = $cpg_user + $cpg_snap + $cpg_admin;
      }
      if ( Xormon::isdigit($non_cpg) && Xormon::isdigit($cpg) && Xormon::isdigit($unmaped) ) {
        $volumes_used = $non_cpg + $cpg + $unmaped;
      }

      if ( defined $data{'perf'}{'tier'}{$tier}{'read'} && $data{'perf'}{'tier'}{$tier}{'read'} ne "" ){
        $read = Xormon::returnBytes( $data{'perf'}{'tier'}{$tier}{'read'}, "kB" );
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'write'} && $data{'perf'}{'tier'}{$tier}{'write'} ne "" ){
        $write = Xormon::returnBytes( $data{'perf'}{'tier'}{$tier}{'write'}, "kB" );
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'read_io'} && $data{'perf'}{'tier'}{$tier}{'read_io'} ne "" ){
        $read_io = $data{'perf'}{'tier'}{$tier}{'read_io'};
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'write_io'} && $data{'perf'}{'tier'}{$tier}{'write_io'} ne "" ){
        $write_io = $data{'perf'}{'tier'}{$tier}{'write_io'};
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'resp_t_r'} && $data{'perf'}{'tier'}{$tier}{'resp_t_r'} ne "" ){
        $resp_t_r = $data{'perf'}{'tier'}{$tier}{'resp_t_r'};
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'resp_t_w'} && $data{'perf'}{'tier'}{$tier}{'resp_t_w'} ne "" ){
        $resp_t_w = $data{'perf'}{'tier'}{$tier}{'resp_t_w'};
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'data_rate'} && $data{'perf'}{'tier'}{$tier}{'data_rate'} ne "" ){
        $data_rate = Xormon::returnBytes( $data{'perf'}{'tier'}{$tier}{'data_rate'}, "kB" );
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'resp_t'} && $data{'perf'}{'tier'}{$tier}{'resp_t'} ne "" ){
        $resp_t = $data{'perf'}{'tier'}{$tier}{'resp_t'};
      }
      if ( defined $data{'perf'}{'tier'}{$tier}{'io_rate'} && $data{'perf'}{'tier'}{$tier}{'io_rate'} ne "" ){
        $io_rate = $data{'perf'}{'tier'}{$tier}{'io_rate'};
      }

      my $hash_key = Xormon::create_hash_key($st_serial."TIER".$tier ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $name,
          "hw_type" => "3par",
          "subsystem" => "tier",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );

      push(@architecture, \%arch);


      #print PERF "$time,$name,$total,$free,$used,$read,$write,$read_io,$write_io,$resp_t_r,$resp_t_w,$data_rate,$resp_t,$io_rate,$read_size,$write_size,$data_size,$normal_t,$normal_f,$normal_u,$spare_t,$spare_f,$spare_u\n";
      $data_out .= "$hash_key;$time;$read;$write;$read_io;$write_io;$resp_t_r;$resp_t_w;$data_rate;$resp_t;$io_rate;$total;$free;$used;$used_pct;$system;$failed;$volumes_used\n";
    } # while tier
  } # if defined tier
  # print Dumper \%data;

  #SAVE TIER\n";
  #print "$data_out";
  $rc = $xormon->saveCSV($data_out,"3par","tier");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("TIER status : $status");
  $data_out = "";


  #########
  # MIRROR
  #########
  if ( defined $data{'perf'}{'mirror'} && $data{'perf'}{'mirror'} ne "" ){
    #print CONF "\nMirror Level Configuration\n";
    #print CONF "--------------------------\n";
    #print CONF "Local VV ID,Local VV Name,Remote system name,Remote VV ID,Remote VV Name,Status,Syncing Level,Remote Copy Group ID,Remote Copy Group Name,Remote Copy Group Mode,Remote Copy Group Status,Remote Copy Group Syncing Level,Remote Copy Group Role,\n";
    while ((my $mirror, my $value) = each (%{ $data{'perf'}{'mirror'} })){
      my $vv_id          = "";
      my $vv_name        = "$mirror";
      my $system_name    = "";
      my $rm_vv_id       = "";
      my $rm_vv_name     = "";
      my $stat           = "";
      my $sync_lvl       = "";
      my $group_id       = "";
      my $group_name     = "";
      my $group_mode     = "";
      my $group_status   = "";
      my $group_sync_lvl = "";
      my $group_role     = "";

      if ( defined $data{'perf'}{'mirror'}{$mirror}{'id'} && $data{'perf'}{'mirror'}{$mirror}{'id'} ne "" ){
        $vv_id = $data{'perf'}{'mirror'}{$mirror}{'id'};
      }
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'systemName'} && $data{'perf'}{'mirror'}{$mirror}{'systemName'} ne "" ){
        $system_name = $data{'perf'}{'mirror'}{$mirror}{'systemName'};
      }
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeID'} && $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeID'} ne "" ){
        $rm_vv_id = $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeID'};
      }
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeName'} && $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeName'} ne "" ){
        $rm_vv_name = $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeName'};
      }
      #if ( defined $data{'perf'}{'mirror'}{$mirror}{'status'} && $data{'perf'}{'mirror'}{$mirror}{'status'} ne "" ){
      #  $status = $data{'perf'}{'mirror'}{$mirror}{'status'};
      #}
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeStatus'} && $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeStatus'} ne "" ){
        $stat = $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeStatus'};
      }
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeStatus'} && $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeStatus'} ne "" ){
        $sync_lvl = $data{'perf'}{'mirror'}{$mirror}{'remoteVolumeStatus'};
      }
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'groupID'} && $data{'perf'}{'mirror'}{$mirror}{'groupID'} ne "" ){
        $group_id = $data{'perf'}{'mirror'}{$mirror}{'groupID'};
      }
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'groupName'} && $data{'perf'}{'mirror'}{$mirror}{'groupName'} ne "" ){
        $group_name = $data{'perf'}{'mirror'}{$mirror}{'groupName'};
      }
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'groupMode'} && $data{'perf'}{'mirror'}{$mirror}{'groupMode'} ne "" ){
        $group_mode = $data{'perf'}{'mirror'}{$mirror}{'groupMode'};
      }
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'groupStatus'} && $data{'perf'}{'mirror'}{$mirror}{'groupStatus'} ne "" ){
        $group_status = $data{'perf'}{'mirror'}{$mirror}{'groupStatus'};
      }
      if ( defined $data{'perf'}{'mirror'}{$mirror}{'groupRole'} && $data{'perf'}{'mirror'}{$mirror}{'groupRole'} ne "" ){
        $group_role = $data{'perf'}{'mirror'}{$mirror}{'groupRole'};
      }

       my $hash_key = Xormon::create_hash_key($st_serial."MIRROR".$vv_id ); $hash_key = lc($hash_key);
       my %arch = (
          "item_id" => $hash_key,
          "label" => $vv_name ,
          "hw_type" => "3par",
          "subsystem" => "mirror",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents" => [$cfg->{'hostcfg_id'}],
        );

        $conf{$hash_key}{local_vv_id}                    = $vv_id; # same as volume_id
        $conf{$hash_key}{local_vv_name}                  = $vv_name;
        $conf{$hash_key}{remote_system_name}             = $system_name;
        $conf{$hash_key}{remote_vv_id}                   = $rm_vv_id;
        $conf{$hash_key}{remote_vv_name}                 = $rm_vv_name;
        $conf{$hash_key}{status}                         = $stat;
        $conf{$hash_key}{syncing_lvl}                    = $sync_lvl;
        $conf{$hash_key}{remote_copy_group_id}           = $group_id;
        $conf{$hash_key}{remote_copy_group_name}         = $group_name;
        $conf{$hash_key}{remote_copy_group_mode}         = $group_mode;
        $conf{$hash_key}{remote_copy_group_status}       = $group_status;
        $conf{$hash_key}{remote_copy_group_syncing_lvl}  = $group_sync_lvl;
        $conf{$hash_key}{remote_copy_group_role}         = $group_role;

      push(@architecture, \%arch);

      #print CONF "$vv_id,$vv_name,$system_name,$rm_vv_id,$rm_vv_name,$status,$sync_lvl,$group_id,$group_name,$group_mode,$group_status,$group_sync_lvl,$group_role\n";
    }
  }

  ##Health Status (GET HS TABLES)
  health_status();

  #save configuration
  $xormon->saveConfJSON(\%conf);

  #save architecture
  #print "ARCHITECTURE\n";
  #print Dumper @architecture;
  $xormon->saveArchitecture(\@architecture);

  #print "#################### Architecture\n";
  #print Dumper @architecture;
  #print "#################### CONF\n";
  #print Dumper \%conf;

  $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");

  #close (PERF);
  #close (CONF);
}

sub get_api_token {
  my $url = shift;
  my $agent = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0} );
  my $request = HTTP::Request->new( POST => $url );

  $request->header('Content-Type' => 'application/json', 'Accept' => 'application/json');

  my $json_con ="{\"user\":\"$cfg->{'apiUsername'}\",\"password\":\"$cfg->{'apiPassword'}\"}";
  $request->content('' . $json_con . '');
  #print Dumper $request;
  my $results = $agent->request($request);


  if (!$results->is_success){
    my $st_line = $results->status_line;
    $logger->error("Request error: $st_line");
    $logger->error("Post url     : $url");
    return 0;
  }
  else {
    if (defined $results->content && $results->content ne ""){
      my $result = decode_json($results->content);
      #print Dumper $result;
      if (defined $result->{'key'} && $result->{'key'} ne ""){
        my $cookie = $result->{'key'};
        chomp($cookie);
        return $cookie;
      }
      else{
        $logger->error("Cookie not found! Login failed! access_token not found!") && return "error";
      }
    }
    else {
      $logger->error("Cookie not found! Login failed!") && return "error";
    }
  }

  return 0;
}

sub new_get_request {
  my $url = shift;
  #print "$url\n";
  my $result = "";

  my $agent = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0} );
  my $request = HTTP::Request->new( GET  => $url );
  $request->header(
    'Accept' => 'application/json',
    'X-HP3PAR-WSAPI-SessionKey' => '' . $cfg->{'token'} . ''
  );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    if ( $st_line !~ m/^501/ && $st_line !~ m/^403/){
      $logger->error("GET url           : $url");
      $logger->error("Request error     : $st_line");
    }

    return "";
  }

  if (defined $results->content && $results->content ne ""){
    my $result = decode_json($results->content);
    #print Dumper $result;
    return $result;
  }

  return "";
}

sub set_time{
  my @t = split( / /, $cfg->{'gtime'});
  my (undef, undef, $sec) = split (":",$t[3]);
  if ($sec =~ m/^0/){
    $sec =~ s/0//;
  }
  my $end_time = str2time( join( " ", @t ) );
  #my $diff = $ltime - $end_time;
  $end_time = $end_time - $sec - 1;
  my $start_time = $end_time - 300;
  my $start_utc = get_time_utc($start_time);
  my $end_utc = get_time_utc($end_time);
  return ( "$start_utc","$end_utc" );
}

sub get_time_utc {
  my $timestamp_end = shift;
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $time_end = "$year-$month-$day" . "T" . "$hour:$minute:$sec" . "Z";
  return $time_end;
}

sub get_api_data {
  my $return = "";
  my $time = "";
  ##############
  #  CAPACITY  #
  ##############
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/capacity");
  # print Dumper $return;
  if (defined $return && $return ne ""){
    $xormon->hasData(1);

    foreach my $cap_name (keys ( %{ $return } ) ){
      my $cap = "";
      if ($cap_name eq "FCCapacity" || $cap_name eq "SSDCapacity" || $cap_name eq "NLCapacity" ) {

        if ($cap_name eq "FCCapacity") {
          $cap = "FC";
        }
        if ($cap_name eq "SSDCapacity") {
          $cap = "SSD";
        }
        if ($cap_name eq "NLCapacity") {
          $cap = "NL";
        }

        # Total
        if ( defined $return->{$cap_name}->{'totalMiB'} && $return->{$cap_name}->{'totalMiB'} ne "" ){
          $data{'perf'}{'tier'}{$cap}{'totalMiB'} = $return->{$cap_name}->{'totalMiB'};
        }
        # Allocated
        if (defined $return->{$cap_name}->{'allocated'}->{'totalAllocatedMiB'} && $return->{$cap_name}->{'allocated'}->{'totalAllocatedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'allocated'} = $return->{$cap_name}->{'allocated'}->{'totalAllocatedMiB'}; # as used for tier
        }
        #VOLUMES
        #non cpg
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGsMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGsMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'nonCPGsMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGsMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGSnapshotMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGSnapshotMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'nonCPGSnapshotMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGSnapshotMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGUserMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGUserMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'nonCPGUserMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGUserMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGAdminMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGAdminMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'nonCPGAdminMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'nonCPGAdminMiB'};
        }
        #cpgs
        #user
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGUserMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGUserMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'CPGUserMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGUserMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGUserUsedMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGUserUsedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'CPGUserUsedMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGUserUsedMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGUserUnusedMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGUserUnusedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'CPGUserUnusedMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGUserUnusedMiB'};
        }
        #snap
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGSnapshotMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGSnapshotMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'CPGSnapshotMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGSnapshotMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGSnapshotUsedMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGSnapshotUsedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'CPGUSnapshotUsedMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGSnapshotUsedMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGSnapshotUnusedMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGSnapshotUnusedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'CPGSnapshotUnusedMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGSnapshotUnusedMiB'};
        }
        #admin
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGAdminMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGAdminMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'CPGAdminMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGAdminMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGAdminUsedMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGAdminUsedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'CPGUAdminUsedMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGAdminUsedMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGAdminUnusedMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGAdminUnusedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'CPGAdminUnusedMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'CPGAdminUnusedMiB'};
        }
        #unmaped
        if (defined $return->{$cap_name}->{'allocated'}->{'volumes'}->{'unmappedMiB'} && $return->{$cap_name}->{'allocated'}->{'volumes'}->{'unmappedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'volumes'}{'unmappedMiB'} = $return->{$cap_name}->{'allocated'}->{'volumes'}->{'unmappedMiB'};
        }
        ##########
        # System #
        ##########
        if (defined $return->{$cap_name}->{'allocated'}->{'system'}->{'totalSystemMiB'} && $return->{$cap_name}->{'allocated'}->{'system'}->{'totalSystemMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'system'}{'totalSystemMiB'} = $return->{$cap_name}->{'allocated'}->{'system'}->{'totalSystemMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'system'}->{'internalMiB'} && $return->{$cap_name}->{'allocated'}->{'system'}->{'internalMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'system'}{'internalMiB'} = $return->{$cap_name}->{'allocated'}->{'system'}->{'internalMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'system'}->{'spareMiB'} && $return->{$cap_name}->{'allocated'}->{'system'}->{'spareMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'system'}{'spareMiB'} = $return->{$cap_name}->{'allocated'}->{'system'}->{'spareMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'system'}->{'spareUsedMiB'} && $return->{$cap_name}->{'allocated'}->{'system'}->{'spareUsedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'system'}{'spareUsedMiB'} = $return->{$cap_name}->{'allocated'}->{'system'}->{'spareUsedMiB'};
        }
        if (defined $return->{$cap_name}->{'allocated'}->{'system'}->{'spareUnusedMiB'} && $return->{$cap_name}->{'allocated'}->{'system'}->{'spareUnusedMiB'} ne ""){
          $data{'perf'}{'tier'}{$cap}{'system'}{'spareUnusedMiB'} = $return->{$cap_name}->{'allocated'}->{'system'}->{'spareUnusedMiB'};
        }
        ########
        # Free #
        ########
        if ( defined $return->{$cap_name}->{'freeMiB'} && $return->{$cap_name}->{'freeMiB'} ne "" ){
          $data{'perf'}{'tier'}{$cap}{'freeMiB'} = $return->{$cap_name}->{'freeMiB'};
        }
        if ( defined $return->{$cap_name}->{'freeInitializedMiB'} && $return->{$cap_name}->{'freeInitializedMiB'} ne "" ){
          $data{'perf'}{'tier'}{$cap}{'freeInitializedMiB'} = $return->{$cap_name}->{'freeInitializedMiB'};
        }
        if ( defined $return->{$cap_name}->{'freeUninitializedMiB'} && $return->{$cap_name}->{'freeUninitializedMiB'} ne "" ){
          $data{'perf'}{'tier'}{$cap}{'freeUninitializedMiB'} = $return->{$cap_name}->{'freeUninitializedMiB'};
        }
        ##########
        # Failed #
        ##########
        if ( defined $return->{$cap_name}->{'failedCapacityMiB'} && $return->{$cap_name}->{'failedCapacityMiB'} ne "" ){
          $data{'perf'}{'tier'}{$cap}{'failedCapacityMiB'} = $return->{$cap_name}->{'failedCapacityMiB'};
        }
      }
    }
  }
  else {
    $xormon->hasData(0, "no return from capacity query");
  }
  #print Dumper $return;
  ############
  #  SYSTEM  #
  ############
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/system");
  if (defined $return && $return ne ""){
    if (defined $return->{'name'} && defined $return->{'name'} ne ""){
      $data{'system'}{'name'} = $return->{'name'};
    }
    if ( defined $return->{'patches'} && $return->{'patches'} ne "" ){
      $data{'system'}{'patches'} = $return->{'patches'};
    }
    if ( defined $return->{'systemVersion'} && $return->{'systemVersion'} ne "" ){
      $data{'system'}{'release_version'} = $return->{'systemVersion'};
    }
    if ( defined $return->{'id'} && $return->{'id'} ne "" ){
      $data{'system'}{'id'} = $return->{'id'};
    }
    if ( defined $return->{'model'} && $return->{'model'} ne "" ){
      $data{'system'}{'model'} = $return->{'model'};
    }
    if ( defined $return->{'serialNumber'} && $return->{'serialNumber'} ne "" ){
      $data{'system'}{'serial'} = $return->{'serialNumber'};
    }
    if ( defined $return->{'allocatedCapacityMiB'} && $return->{'allocatedCapacityMiB'} ne "" ){
      $data{'system'}{'allocated'} = Xormon::returnBytes( $return->{'allocatedCapacityMiB'}, "MiB" );
    }
    if ( defined $return->{'failedCapacityMiB'} && $return->{'failedCapacityMiB'} ne "" ){
      $data{'system'}{'failed'} = Xormon::returnBytes( $return->{'failedCapacityMiB'}, "MiB" );
    }
    if ( defined $return->{'freeCapacityMiB'} && $return->{'freeCapacityMiB'} ne "" ){
      $data{'system'}{'free'} = Xormon::returnBytes( $return->{'freeCapacityMiB'}, "MiB" );
    }
    if ( defined $return->{'totalCapacityMiB'} && $return->{'totalCapacityMiB'} ne "" ){
      $data{'system'}{'total'} = Xormon::returnBytes( $return->{'totalCapacityMiB'}, "MiB" );
    }
    if ( defined $return->{'chunkletSizeMiB'} && $return->{'chunkletSizeMiB'} ne "" ){
      $data{'system'}{'chunkletSizeMiB'} = $return->{'chunkletSizeMiB'};
    }
  }

  ###########
  #  CACHE  #
  ###########
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/cachememorystatistics/hires?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        ###########
        # node0.rrd
        ###########
        my $id = "";
        my $name = "node";
        if ( defined $member->{'node'} && $member->{'node'} ne ""){
          $id = $member->{'node'};
          $name .= "$id";
          if ($name ne "" && $id ne ""){
            $data{'perf'}{'node'}{$name}{'id'} = $id;
          }
        }
        if ( defined $member->{'accessIO'}->{'read'} && $member->{'accessIO'}->{'read'} ne "" && $id ne ""){
          $data{'perf'}{'node'}{$name}{'r_io_acc'} = $member->{'accessIO'}->{'read'};
        }
        if ( defined $member->{'accessIO'}->{'write'} && $member->{'accessIO'}->{'write'} ne "" && $id ne ""){
          $data{'perf'}{'node'}{$name}{'w_io_acc'} = $member->{'accessIO'}->{'write'};
        }
        if ( defined $member->{'hitIO'}->{'read'} && $member->{'hitIO'}->{'read'} ne "" && $id ne ""){
          $data{'perf'}{'node'}{$name}{'r_cache_hit_io'} = $member->{'hitIO'}->{'read'};
        }
        if ( defined $member->{'hitIO'}->{'write'} && $member->{'hitIO'}->{'write'} ne "" && $id ne ""){
          $data{'perf'}{'node'}{$name}{'w_cache_hit_io'} = $member->{'hitIO'}->{'write'};
        }
        if ( defined $member->{'hitPct'}->{'read'} && $member->{'hitPct'}->{'read'} ne "" && $id ne ""){
          $data{'perf'}{'node'}{$name}{'r_cache_hit'} = $member->{'hitPct'}->{'read'};
        }
        if ( defined $member->{'hitPct'}->{'write'} && $member->{'hitPct'}->{'write'} ne "" && $id ne ""){
          $data{'perf'}{'node'}{$name}{'w_cache_hit'} = $member->{'hitPct'}->{'write'};
        }
        ###########
        # FC_10KRPM-node0.rrt
        ########### cfc_dirty             cfc_max             del_ack
        #if ( defined $member->{'pageStatistic'}->{'dirtyPages'}->{'FC_10'} && $member->{'pageStatistic'}->{'dirtyPages'}->{'FC_10'} ne "" ){
        #  $data{'perf'}{'nodeFC_10'}{$name}{'cfc_dirty'} = $member->{'pageStatistic'}->{'dirtyPages'}->{'FC_10'};
        #}
        #if ( defined $member->{'pageStatistic'}->{'maxDirtyPages'}->{'FC_10'} && $member->{'pageStatistic'}->{'maxDirtyPages'}->{'FC_10'} ne "" ){
        #  $data{'perf'}{'nodeFC_10'}{$name}{'cfc_max'} = $member->{'pageStatistic'}->{'maxDirtyPages'}->{'FC_10'};
        #}
        #if ( defined $member->{'pageStatistic'}->{'delayAckPages'}->{'FC_10'} && $member->{'pageStatistic'}->{'delayAckPages'}->{'FC_10'} ne "" ){
        #  $data{'perf'}{'nodeFC_10'}{$name}{'del_ack'} = $member->{'pageStatistic'}->{'delayAckPages'}->{'FC_10'};
        #}
        ###########
        # FC_15KRPM-node0.rrt
        ###########
        # if ( defined $member->{'pageStatistic'}->{'dirtyPages'}->{'FC_15'} && $member->{'pageStatistic'}->{'dirtyPages'}->{'FC_15'} ne "" ){
        #  $data{'perf'}{'nodeFC_15'}{$name}{'cfc_dirty'} = $member->{'pageStatistic'}->{'dirtyPages'}->{'FC_15'};
        #}
        #if ( defined $member->{'pageStatistic'}->{'maxDirtyPages'}->{'FC_15'} && $member->{'pageStatistic'}->{'maxDirtyPages'}->{'FC_15'} ne "" ){
        #  $data{'perf'}{'nodeFC_15'}{$name}{'cfc_max'} = $member->{'pageStatistic'}->{'maxDirtyPages'}->{'FC_15'};
        #}
        #if ( defined $member->{'pageStatistic'}->{'delayAckPages'}->{'FC_15'} && $member->{'pageStatistic'}->{'delayAckPages'}->{'FC_15'} ne "" ){
        #  $data{'perf'}{'nodeFC_15'}{$name}{'del_ack'} = $member->{'pageStatistic'}->{'delayAckPages'}->{'FC_15'};
        #}
        ###########
        # FC-node0.rrt
        ###########
        if ( defined $member->{'pageStatistic'}->{'dirtyPages'}->{'FC'} && $member->{'pageStatistic'}->{'dirtyPages'}->{'FC'} ne "" ){
          $data{'perf'}{'nodeFC'}{$name}{'cfc_dirty'} = $member->{'pageStatistic'}->{'dirtyPages'}->{'FC'};
        }
        if ( defined $member->{'pageStatistic'}->{'maxDirtyPages'}->{'FC'} && $member->{'pageStatistic'}->{'maxDirtyPages'}->{'FC'} ne "" ){
          $data{'perf'}{'nodeFC'}{$name}{'cfc_max'} = $member->{'pageStatistic'}->{'maxDirtyPages'}->{'FC'};
        }
        if ( defined $member->{'pageStatistic'}->{'delayAckPages'}->{'FC'} && $member->{'pageStatistic'}->{'delayAckPages'}->{'FC'} ne "" ){
          $data{'perf'}{'nodeFC'}{$name}{'del_ack'} = $member->{'pageStatistic'}->{'delayAckPages'}->{'FC'};
        }
        ##########
        # NL-node0.rrt
        ##########
        if ( defined $member->{'pageStatistic'}->{'dirtyPages'}->{'NL_7'} && $member->{'pageStatistic'}->{'dirtyPages'}->{'NL_7'} ne "" ){
          $data{'perf'}{'nodeNL'}{$name}{'cfc_dirty'} = $member->{'pageStatistic'}->{'dirtyPages'}->{'NL_7'};
        }
        if ( defined $member->{'pageStatistic'}->{'maxDirtyPages'}->{'NL_7'} && $member->{'pageStatistic'}->{'maxDirtyPages'}->{'NL_7'} ne "" ){
          $data{'perf'}{'nodeNL'}{$name}{'cfc_max'} = $member->{'pageStatistic'}->{'maxDirtyPages'}->{'NL_7'};
        }
        if ( defined $member->{'pageStatistic'}->{'delayAckPages'}->{'NL_7'} && $member->{'pageStatistic'}->{'delayAckPages'}->{'NL_7'} ne "" ){
          $data{'perf'}{'nodeNL'}{$name}{'del_ack'} = $member->{'pageStatistic'}->{'delayAckPages'}->{'NL_7'};
        }
        ##########
        # SSD_100KRPM-node0.rrt
        ##########
        #if ( defined $member->{'pageStatistic'}->{'dirtyPages'}->{'SSD_100'} && $member->{'pageStatistic'}->{'dirtyPages'}->{'SSD_100'} ne "" ){
        #  $data{'perf'}{'nodeSSD_100'}{$name}{'cfc_dirty'} = $member->{'pageStatistic'}->{'dirtyPages'}->{'SSD_100'};
        #}
        #if ( defined $member->{'pageStatistic'}->{'maxDirtyPages'}->{'SSD_100'} && $member->{'pageStatistic'}->{'maxDirtyPages'}->{'SSD_100'} ne "" ){
        #  $data{'perf'}{'nodeSSD_100'}{$name}{'cfc_max'} = $member->{'pageStatistic'}->{'maxDirtyPages'}->{'SSD_100'};
        #}
        #if ( defined $member->{'pageStatistic'}->{'delayAckPages'}->{'SSD_100'} && $member->{'pageStatistic'}->{'delayAckPages'}->{'SSD_100'} ne "" ){
        #  $data{'perf'}{'nodeSSD_100'}{$name}{'del_ack'} = $member->{'pageStatistic'}->{'delayAckPages'}->{'SSD_100'};
        #}
        ##########
        # SSD_150KRPM-node0.rrt
        ##########
        #if ( defined $member->{'pageStatistic'}->{'dirtyPages'}->{'SSD_150'} && $member->{'pageStatistic'}->{'dirtyPages'}->{'SSD_150'} ne "" ){
        #  $data{'perf'}{'nodeSSD_150'}{$name}{'cfc_dirty'} = $member->{'pageStatistic'}->{'dirtyPages'}->{'SSD_150'};
        #}
        #if ( defined $member->{'pageStatistic'}->{'maxDirtyPages'}->{'SSD_150'} && $member->{'pageStatistic'}->{'maxDirtyPages'}->{'SSD_150'} ne "" ){
        #  $data{'perf'}{'nodeSSD_150'}{$name}{'cfc_max'} = $member->{'pageStatistic'}->{'maxDirtyPages'}->{'SSD_150'};
        #}
        #if ( defined $member->{'pageStatistic'}->{'delayAckPages'}->{'SSD_150'} && $member->{'pageStatistic'}->{'delayAckPages'}->{'SSD_150'} ne "" ){
        #  $data{'perf'}{'nodeSSD_150'}{$name}{'del_ack'} = $member->{'pageStatistic'}->{'delayAckPages'}->{'SSD_150'};
        #}
        ##########
        # SSD-node0.rrt
        ##########
        if ( defined $member->{'pageStatistic'}->{'dirtyPages'}->{'SSD'} && $member->{'pageStatistic'}->{'dirtyPages'}->{'SSD'} ne "" ){
          $data{'perf'}{'nodeSSD'}{$name}{'cfc_dirty'} = $member->{'pageStatistic'}->{'dirtyPages'}->{'SSD'};
        }
        if ( defined $member->{'pageStatistic'}->{'maxDirtyPages'}->{'SSD'} && $member->{'pageStatistic'}->{'maxDirtyPages'}->{'SSD'} ne "" ){
          $data{'perf'}{'nodeSSD'}{$name}{'cfc_max'} = $member->{'pageStatistic'}->{'maxDirtyPages'}->{'SSD'};
        }
        if ( defined $member->{'pageStatistic'}->{'delayAckPages'}->{'SSD'} && $member->{'pageStatistic'}->{'delayAckPages'}->{'SSD'} ne "" ){
          $data{'perf'}{'nodeSSD'}{$name}{'del_ack'} = $member->{'pageStatistic'}->{'delayAckPages'}->{'SSD'};
        }

      }
    }
  }

  #############
  # Pool(CPG)
  #############
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/cpgspacedata/hires?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        if ( defined $member->{'name'} && $member->{'name'} ne "" ) {
          my $name = $member->{'name'};
          if ( defined $member->{'id'} && $member->{'id'} ne "" ) {
            $data{'perf'}{'pool'}{$name}{'id'} = $member->{'id'};
          }
          #NET
          if ( defined $member->{'totalSpaceMiB'} && $member->{'totalSpaceMiB'} ne "" ) {
            $data{'perf'}{'pool'}{$name}{'total'} = Xormon::returnBytes( $member->{'totalSpaceMiB'}, "MiB" );
          }
          if ( defined $member->{'freeSpaceMiB'} && $member->{'freeSpaceMiB'} ne "" ) {
            $data{'perf'}{'pool'}{$name}{'free'} = Xormon::returnBytes( $member->{'freeSpaceMiB'}, "MiB" );
             if ( defined $member->{'totalSpaceMiB'} && $member->{'totalSpaceMiB'} ne "" ) {
              $data{'perf'}{'pool'}{$name}{'used'} = Xormon::returnBytes( $member->{'totalSpaceMiB'}, "MiB" ) - Xormon::returnBytes($member->{'freeSpaceMiB'}, "MiB" );
             }
          }
          #Efficiency
          if ( defined $member->{'capacityEfficiency'}->{'overProvisioning'} && $member->{'capacityEfficiency'}->{'overProvisioning'} ne "" ){
            $data{'perf'}{'pool'}{$name}{'overProvisioning'} = $member->{'capacityEfficiency'}->{'overProvisioning'};
          }
          if ( defined $member->{'capacityEfficiency'}->{'compaction'} && $member->{'capacityEfficiency'}->{'compaction'} ne "" ){
            $data{'perf'}{'pool'}{$name}{'compaction'} = $member->{'capacityEfficiency'}->{'compaction'};
          }
          if ( defined $member->{'capacityEfficiency'}->{'compression'} && $member->{'capacityEfficiency'}->{'compression'} ne "" ){
            $data{'perf'}{'pool'}{$name}{'compression'} = $member->{'capacityEfficiency'}->{'compression'};
          }
          if ( defined $member->{'deduplicationGcKBPS'} && $member->{'deduplicationGcKBPS'} ne "" ){
             $data{'perf'}{'pool'}{$name}{'deduplicationGcKBPS'} = $member->{'deduplicationGcKBPS'};
          }
        }
      }
    }
  }

  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/cpgs");
  if (defined $return && $return ne ""){
    # print Dumper $return;
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        if ( defined $member->{'name'} && $member->{'name'} ne "" ) {
          my $name = $member->{'name'};
          #Status
          if ( defined $member->{'state'} && $member->{'state'} ne "" ){
            if (  $member->{'state'} == 1 ){
              $data{'perf'}{'pool'}{$name}{'status'} = "NORMAL";
            }
            elsif( $member->{'state'} == 2 ){
              $data{'perf'}{'pool'}{$name}{'status'} = "DEGRADED";
            }
            elsif( $member->{'state'} == 3 ){
              $data{'perf'}{'pool'}{$name}{'status'} = "FAILED";
            }
            elsif( $member->{'state'} == 99 ){
              $data{'perf'}{'pool'}{$name}{'status'} = "UNKNOWN";
            }
          }
          ####### CAPACITY
          #######
          # user
          #######
          #Net
          if (defined $member->{'UsrUsage'}->{'usedMiB'} && $member->{'UsrUsage'}->{'usedMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'user'}{'used'} = $member->{'UsrUsage'}->{'usedMiB'};
          }
          if (defined $member->{'UsrUsage'}->{'totalMiB'} && $member->{'UsrUsage'}->{'totalMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'user'}{'total'} = $member->{'UsrUsage'}->{'totalMiB'};
          }
          if ( defined $member->{'UsrUsage'}->{'usedMiB'} && $member->{'UsrUsage'}->{'usedMiB'} ne "" && defined $member->{'UsrUsage'}->{'totalMiB'} && $member->{'UsrUsage'}->{'totalMiB'} ne "" ){
            $data{'perf'}{'pool'}{$name}{'user'}{'free'} = ( $member->{'UsrUsage'}->{'totalMiB'} - $member->{'UsrUsage'}->{'usedMiB'} );
          }
          #Raw
          if (defined $member->{'UsrUsage'}->{'rawUsedMiB'} && $member->{'UsrUsage'}->{'rawUsedMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'user'}{'rawUsed'} = $member->{'UsrUsage'}->{'rawUsedMiB'};
          }
          if (defined $member->{'UsrUsage'}->{'rawTotalMiB'} && $member->{'UsrUsage'}->{'rawTotalMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'user'}{'rawTotal'} = $member->{'UsrUsage'}->{'rawTotalMiB'};
          }
          if ( defined $member->{'UsrUsage'}->{'rawUsedMiB'} && $member->{'UsrUsage'}->{'rawUsedMiB'} ne "" && defined $member->{'UsrUsage'}->{'rawTotalMiB'} && $member->{'UsrUsage'}->{'rawTotalMiB'} ne "" ){
            $data{'perf'}{'pool'}{$name}{'user'}{'rawFree'} = ( $member->{'UsrUsage'}->{'rawTotalMiB'} - $member->{'UsrUsage'}->{'rawUsedMiB'} );
          }
          #######
          # admin
          #######
          #Net
          if (defined $member->{'SAUsage'}->{'usedMiB'} && $member->{'SAUsage'}->{'usedMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'admin'}{'used'} = $member->{'SAUsage'}->{'usedMiB'};
          }
          if (defined $member->{'SAUsage'}->{'totalMiB'} && $member->{'SAUsage'}->{'totalMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'admin'}{'total'} = $member->{'SAUsage'}->{'totalMiB'};
          }
          if ( defined $member->{'SAUsage'}->{'usedMiB'} && $member->{'SAUsage'}->{'usedMiB'} ne "" && defined $member->{'SAUsage'}->{'totalMiB'} && $member->{'SAUsage'}->{'totalMiB'} ne "" ){
            $data{'perf'}{'pool'}{$name}{'admin'}{'free'} = ( $member->{'SAUsage'}->{'totalMiB'} - $member->{'SAUsage'}->{'usedMiB'} );
          }
          #Raw
          if (defined $member->{'SAUsage'}->{'rawUsedMiB'} && $member->{'SAUsage'}->{'rawUsedMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'admin'}{'rawUsed'} = $member->{'SAUsage'}->{'rawUsedMiB'};
          }
          if (defined $member->{'SAUsage'}->{'rawTotalMiB'} && $member->{'SAUsage'}->{'rawTotalMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'admin'}{'rawTotal'} = $member->{'SAUsage'}->{'rawTotalMiB'};
          }
          if ( defined $member->{'SAUsage'}->{'rawUsedMiB'} && $member->{'SAUsage'}->{'rawUsedMiB'} ne "" && defined $member->{'SAUsage'}->{'rawTotalMiB'} && $member->{'SAUsage'}->{'rawTotalMiB'} ne "" ){
            $data{'perf'}{'pool'}{$name}{'admin'}{'rawFree'} = ( $member->{'SAUsage'}->{'rawTotalMiB'} - $member->{'SAUsage'}->{'rawUsedMiB'} );
          }
          #######
          # snapshots
          #######
          #Net
          if (defined $member->{'SDUsage'}->{'usedMiB'} && $member->{'SDUsage'}->{'usedMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'snap'}{'used'} = $member->{'SDUsage'}->{'usedMiB'};
          }
          if (defined $member->{'SDUsage'}->{'totalMiB'} && $member->{'SDUsage'}->{'totalMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'snap'}{'total'} = $member->{'SDUsage'}->{'totalMiB'};
          }
          if ( defined $member->{'SDUsage'}->{'usedMiB'} && $member->{'SDUsage'}->{'usedMiB'} ne "" && defined $member->{'SDUsage'}->{'totalMiB'} && $member->{'SDUsage'}->{'totalMiB'} ne "" ){
            $data{'perf'}{'pool'}{$name}{'snap'}{'free'} = ( $member->{'SDUsage'}->{'totalMiB'} - $member->{'SDUsage'}->{'usedMiB'} );
          }
          #Raw
          if (defined $member->{'SDUsage'}->{'rawUsedMiB'} && $member->{'SDUsage'}->{'rawUsedMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'snap'}{'rawUsed'} = $member->{'SDUsage'}->{'rawUsedMiB'};
          }
          if (defined $member->{'SDUsage'}->{'rawTotalMiB'} && $member->{'SDUsage'}->{'rawTotalMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'snap'}{'rawTotal'} = $member->{'SDUsage'}->{'rawTotalMiB'};
          }
          if ( defined $member->{'SDUsage'}->{'rawUsedMiB'} && $member->{'SDUsage'}->{'rawUsedMiB'} ne "" && defined $member->{'SDUsage'}->{'rawTotalMiB'} && $member->{'SDUsage'}->{'rawTotalMiB'} ne "" ){
            $data{'perf'}{'pool'}{$name}{'snap'}{'rawFree'} = ( $member->{'SDUsage'}->{'rawTotalMiB'} - $member->{'SDUsage'}->{'rawUsedMiB'} );
          }
          #######
          # Raw capacity
          #######
          if (defined $member->{'rawTotalSpaceMiB'} && $member->{'rawTotalSpaceMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'rawTotal'} = $member->{'rawTotalSpaceMiB'};
          }
          if (defined $member->{'rawFreeSpaceMiB'} && $member->{'rawFreeSpaceMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'rawFree'} = $member->{'rawFreeSpaceMiB'};
          }
          if (defined $member->{'rawFreeSpaceMiB'} && $member->{'rawFreeSpaceMiB'} ne "" && defined $member->{'rawTotalSpaceMiB'} && $member->{'rawTotalSpaceMiB'} ne ""){
            $data{'perf'}{'pool'}{$name}{'rawUsed'} = ( $member->{'rawTotalSpaceMiB'} - $member->{'rawFreeSpaceMiB'} );
          }
        }
      }
    }
  }


  ##########
  # CPU
  ##########
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/cpustatistics/hires?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        my $node_id = "";
        my $cpu_id  = "";
        if ( defined $member->{'node'} && $member->{'node'} ne "" ){
          $node_id = $member->{'node'};
        }
        if ( defined $member->{'cpu'} && $member->{'cpu'} ne "" ){
          $cpu_id = $member->{'cpu'};
        }
        if ($node_id ne "" && $cpu_id ne ""){
          my $name = "node$node_id\_core$cpu_id";

          if ( defined $member->{'userPct'} && $member->{'userPct'} ne "" ){
            $data{'perf'}{'cpu'}{$name}{'usr'} = $member->{'userPct'};
          }
          if ( defined $member->{'systemPct'} && $member->{'systemPct'} ne "" ){
            $data{'perf'}{'cpu'}{$name}{'sys'} = $member->{'systemPct'};
          }
          if ( defined $member->{'idlePct'} && $member->{'idlePct'} ne "" ){
            $data{'perf'}{'cpu'}{$name}{'idle'} = $member->{'idlePct'};
          }
        }
      }
    }
  }

  #########
  # Disk
  #########

  ###Capacity
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/physicaldiskcapacity/hires?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        my $id = "";
        my $name = "";
        if ( defined $member->{'id'} && $member->{'id'} ne "" && defined $member->{'cageID'} && $member->{'cageID'} ne "" && defined $member->{'mag'} && $member->{'mag'} ne "" && defined $member->{'diskPos'} && $member->{'diskPos'} ne "" ) {
          $id = $member->{'id'};
          $name = "$member->{'cageID'}\-$member->{'mag'}\-$member->{'diskPos'}";
          $data{'perf'}{'disk'}{$id}{'name'} = $name;

          $data{'perf'}{'disk'}{$id}{'cageID'}   = $member->{'cageID'};
          $data{'perf'}{'disk'}{$id}{'mag'}      = $member->{'mag'};
          $data{'perf'}{'disk'}{$id}{'diskPos'}  = $member->{'diskPos'};

          if (defined $member->{'RPM'} && $member->{'RPM'} ne ""){
            $data{'perf'}{'disk'}{$id}{'rpm'} = $member->{'RPM'};
          }
          if (defined $member->{'type'} && $member->{'type'} ne "") {
            $data{'perf'}{'disk'}{$id}{'type'} = $member->{'type'};
          }
          if (defined $member->{'totalMiB'} && $member->{'totalMiB'} ne ""){
            $data{'perf'}{'disk'}{$id}{'total'} = $member->{'totalMiB'};
          }
          if (defined $member->{'allocatedMiB'} && $member->{'allocatedMiB'} ne ""){
            $data{'perf'}{'disk'}{$id}{'used'} = $member->{'allocatedMiB'};
          }
          if (defined $member->{'freeMiB'} && $member->{'freeMiB'} ne ""){
            $data{'perf'}{'disk'}{$id}{'free'} = $member->{'freeMiB'};
          }
        }
      }
    }
  }

  ###Statistics
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/physicaldiskspacedata/hires;groupby:id?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return && $return ne "" && defined $data{'system'}{'chunkletSizeMiB'} && $data{'system'}{'chunkletSizeMiB'} ne "" ){
    while ((my $index, my $members) = each (@{$return->{'members'}}))
    {
      ###Spare chunklets
      if ( defined $members->{'id'} && $members->{'id'} ne "" ){
        my $id = $members->{'id'};

        if ( Xormon::isdigit($members->{'spareChunklets'}->{'availFailed'}) && Xormon::isdigit($members->{'spareChunklets'}->{'availClean'}) && Xormon::isdigit($members->{'spareChunklets'}->{'availDirty'}) ){
          $data{'perf'}{'disk'}{$id}{'spare'}{'free'} = ($members->{'spareChunklets'}->{'availFailed'} + $members->{'spareChunklets'}->{'availClean'} + $members->{'spareChunklets'}->{'availDirty'}) * $data{'system'}{'chunkletSizeMiB'};
        }

        if ( Xormon::isdigit($members->{'spareChunklets'}->{'usedFailed'}) && Xormon::isdigit($members->{'spareChunklets'}->{'usedOK'}) ){
          $data{'perf'}{'disk'}{$id}{'spare'}{'used'} = ($members->{'spareChunklets'}->{'usedFailed'} + $members->{'spareChunklets'}->{'usedOK'}) * $data{'system'}{'chunkletSizeMiB'};
        }

        if ( Xormon::isdigit($data{'perf'}{'disk'}{$id}{'spare'}{'free'}) && Xormon::isdigit($data{'perf'}{'disk'}{$id}{'spare'}{'used'}) ){
          $data{'perf'}{'disk'}{$id}{'spare'}{'total'} = $data{'perf'}{'disk'}{$id}{'spare'}{'free'} + $data{'perf'}{'disk'}{$id}{'spare'}{'used'};
        }

      }
      ###Normal chunklets
      if ( defined $members->{'id'} && $members->{'id'} ne "" ){
        my $id = $members->{'id'};

        if ( Xormon::isdigit($members->{'normalChunklets'}->{'availFailed'}) && Xormon::isdigit($members->{'normalChunklets'}->{'availClean'}) && Xormon::isdigit($members->{'normalChunklets'}->{'availDirty'}) ){
          $data{'perf'}{'disk'}{$id}{'normal'}{'free'} = ($members->{'normalChunklets'}->{'availFailed'} + $members->{'normalChunklets'}->{'availClean'} + $members->{'normalChunklets'}->{'availDirty'}) * $data{'system'}{'chunkletSizeMiB'};
        }

        if ( Xormon::isdigit($members->{'normalChunklets'}->{'usedFailed'}) && Xormon::isdigit($members->{'normalChunklets'}->{'usedOK'}) ){
          $data{'perf'}{'disk'}{$id}{'normal'}{'used'} = ($members->{'normalChunklets'}->{'usedFailed'} + $members->{'normalChunklets'}->{'usedOK'}) * $data{'system'}{'chunkletSizeMiB'};
        }

        if ( Xormon::isdigit($data{'perf'}{'disk'}{$id}{'normal'}{'free'}) && Xormon::isdigit($data{'perf'}{'disk'}{$id}{'normal'}{'used'}) ){
          $data{'perf'}{'disk'}{$id}{'normal'}{'total'} = $data{'perf'}{'disk'}{$id}{'normal'}{'free'} + $data{'perf'}{'disk'}{$id}{'normal'}{'used'};
        }
      }
    }
  }


  ###Statistics
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/physicaldiskstatistics/hires;groupby:id?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      my @tier_types = ('FC','NL','SSD','SCM');
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        my $id        = "";
        my $name      = "";
        my $tier_type = "";
        if ( defined $member->{'id'} && $member->{'id'} ne "" ) {
          $id = $member->{'id'};

          if ( $data{'perf'}{'disk'}{$id}{'type'} eq "1" ){
            $tier_type = "FC";
            $data{'perf'}{'disk'}{$id}{'type_word'} = "FC";
          }
          elsif( $data{'perf'}{'disk'}{$id}{'type'} eq "2"  ){
            $tier_type = "NL";
            $data{'perf'}{'disk'}{$id}{'type_word'} = "NL";
          }
          elsif( $data{'perf'}{'disk'}{$id}{'type'} eq "3"  ){
            $tier_type = "SSD";
            $data{'perf'}{'disk'}{$id}{'type_word'} = "SSD";
          }
          elsif( $data{'perf'}{'disk'}{$id}{'type'} eq "4"  ){
            $tier_type = "SCM";
            $data{'perf'}{'disk'}{$id}{'type_word'} = "SCM";
          }


          #IO
          ##Total IO
          if (defined $member->{'IO'}->{'total'} && $member->{'IO'}->{'total'} ne ""){
            $data{'perf'}{'disk'}{$id}{'io_rate'} = $member->{'IO'}->{'total'};
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'io_rate'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'io_rate'} += $member->{'IO'}->{'total'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'io_rate'} = $member->{'IO'}->{'total'};
            }
          }
          ##Write IO
          if (defined $member->{'IO'}->{'write'} && $member->{'IO'}->{'write'} ne ""){
            $data{'perf'}{'disk'}{$id}{'write_io'} = $member->{'IO'}->{'write'};
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'write_io'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'write_io'} += $member->{'IO'}->{'write'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'write_io'} = $member->{'IO'}->{'write'};
            }
          }
          ##Read IO
          if (defined $member->{'IO'}->{'read'} && $member->{'IO'}->{'read'} ne ""){
            $data{'perf'}{'disk'}{$id}{'read_io'} = $member->{'IO'}->{'read'};
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'read_io'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'read_io'} += $member->{'IO'}->{'read'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'read_io'} = $member->{'IO'}->{'read'};
            }
          }

          #Resp time
          if (defined $member->{'serviceTimeMS'}->{'total'} && $member->{'serviceTimeMS'}->{'total'} ne "" ){
            $data{'perf'}{'disk'}{$id}{'resp_t'} = $member->{'serviceTimeMS'}->{'total'};
            #For tier statistics
            #### weighted average resp_t * io_total
            if ( defined $member->{'IO'}->{'total'} && $member->{'IO'}->{'total'} ne "" ){
              if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'resp_t'} ) ){
                $data{'perf'}{'tier'}{$tier_type}{'resp_t'} += $member->{'serviceTimeMS'}->{'total'} * $member->{'IO'}->{'total'};
              }
              else{
                $data{'perf'}{'tier'}{$tier_type}{'resp_t'} = $member->{'serviceTimeMS'}->{'total'} * $member->{'IO'}->{'total'} ;
              }
            }
          }
          if (defined $member->{'serviceTimeMS'}->{'write'} && $member->{'serviceTimeMS'}->{'write'} ne ""){
            $data{'perf'}{'disk'}{$id}{'resp_t_w'} = $member->{'serviceTimeMS'}->{'write'};
            #For tier statistics
            if ( defined $member->{'IO'}->{'write'} && $member->{'IO'}->{'write'} ne "" ){
              if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'resp_t_w'} ) ){
                $data{'perf'}{'tier'}{$tier_type}{'resp_t_w'} += $member->{'serviceTimeMS'}->{'write'} * $member->{'IO'}->{'write'};
              }
              else{
                $data{'perf'}{'tier'}{$tier_type}{'resp_t_w'} = $member->{'serviceTimeMS'}->{'write'} * $member->{'IO'}->{'write'} ;
              }
            }
          }
          if (defined $member->{'serviceTimeMS'}->{'read'} && $member->{'serviceTimeMS'}->{'read'} ne ""){
            $data{'perf'}{'disk'}{$id}{'resp_t_r'} = $member->{'serviceTimeMS'}->{'read'};
            #For tier statistics
            if ( defined $member->{'IO'}->{'read'} && $member->{'IO'}->{'read'} ne "" ){
              if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'resp_t_r'} ) ){
                $data{'perf'}{'tier'}{$tier_type}{'resp_t_r'} += $member->{'serviceTimeMS'}->{'read'} * $member->{'IO'}->{'read'};
              }
              else{
                $data{'perf'}{'tier'}{$tier_type}{'resp_t_r'} = $member->{'serviceTimeMS'}->{'read'} * $member->{'IO'}->{'read'};
              }
            }
          }


          #Data
          if (defined $member->{'KBytes'}->{'total'} && $member->{'KBytes'}->{'total'} ne ""){
            $data{'perf'}{'disk'}{$id}{'data_rate'} = $member->{'KBytes'}->{'total'};
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'data_rate'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'data_rate'} += $member->{'KBytes'}->{'total'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'data_rate'} = $member->{'KBytes'}->{'total'};
            }
          }
          if (defined $member->{'KBytes'}->{'write'} && $member->{'KBytes'}->{'write'} ne ""){
            $data{'perf'}{'disk'}{$id}{'write'} = $member->{'KBytes'}->{'write'};
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'write'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'write'} += $member->{'KBytes'}->{'write'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'write'} = $member->{'KBytes'}->{'write'};
            }
          }
          if (defined $member->{'KBytes'}->{'read'} && $member->{'KBytes'}->{'read'} ne ""){
            $data{'perf'}{'disk'}{$id}{'read'} = $member->{'KBytes'}->{'read'};
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'read'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'read'} += $member->{'KBytes'}->{'read'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'read'} = $member->{'KBytes'}->{'read'};
            }
          }

          #Tier size
          if (defined $member->{'IOSizeKB'}->{'total'} && $member->{'IOSizeKB'}->{'total'} ne ""){
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'data_size'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'data_size'} += $member->{'IOSizeKB'}->{'total'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'data_size'} = $member->{'IOSizeKB'}->{'total'};
            }
          }
          if (defined $member->{'IOSizeKB'}->{'read'} && $member->{'IOSizeKB'}->{'read'} ne ""){
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'read_size'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'read_size'} += $member->{'IOSizeKB'}->{'read'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'read_size'} = $member->{'IOSizeKB'}->{'read'};
            }
          }
          if (defined $member->{'IOSizeKB'}->{'write'} && $member->{'IOSizeKB'}->{'write'} ne ""){
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'write_size'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'write_size'} += $member->{'IOSizeKB'}->{'write'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'write_size'} = $member->{'IOSizeKB'}->{'write'};
            }
          }

          ##Tier capacity
          if ( defined $data{'perf'}{'disk'}{$id}{'total'} && $data{'perf'}{'disk'}{$id}{'total'} ne "" ){
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'total'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'total'} += $data{'perf'}{'disk'}{$id}{'total'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'total'} = $data{'perf'}{'disk'}{$id}{'total'};
            }
          }
          if ( defined $data{'perf'}{'disk'}{$id}{'used'} && $data{'perf'}{'disk'}{$id}{'used'} ne "" ){
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'used'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'used'} += $data{'perf'}{'disk'}{$id}{'used'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'used'} = $data{'perf'}{'disk'}{$id}{'used'};
            }
          }
          if ( defined $data{'perf'}{'disk'}{$id}{'free'} && $data{'perf'}{'disk'}{$id}{'free'} ne ""){
            #For tier statistics
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'free'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'free'} += $data{'perf'}{'disk'}{$id}{'free'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'free'} = $data{'perf'}{'disk'}{$id}{'free'};
            }
          }

          ##Tier Spare Chunklet
          if ( defined $data{'perf'}{'disk'}{$id}{'spare'}{'free'} && $data{'perf'}{'disk'}{$id}{'spare'}{'free'} ne "" ){
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'spare'}{'free'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'spare'}{'free'} += $data{'perf'}{'disk'}{$id}{'spare'}{'free'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'spare'}{'free'} = $data{'perf'}{'disk'}{$id}{'spare'}{'free'};
            }
          }

          if ( defined $data{'perf'}{'disk'}{$id}{'spare'}{'used'} && $data{'perf'}{'disk'}{$id}{'spare'}{'used'} ne "" ){
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'spare'}{'used'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'spare'}{'used'} += $data{'perf'}{'disk'}{$id}{'spare'}{'used'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'spare'}{'used'} = $data{'perf'}{'disk'}{$id}{'spare'}{'used'};
            }
          }

          if ( defined $data{'perf'}{'disk'}{$id}{'spare'}{'total'} && $data{'perf'}{'disk'}{$id}{'spare'}{'total'} ne "" ){
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'spare'}{'total'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'spare'}{'total'} += $data{'perf'}{'disk'}{$id}{'spare'}{'total'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'spare'}{'total'} = $data{'perf'}{'disk'}{$id}{'spare'}{'total'};
            }
          }


          ##Tier normal Chunklet
          if ( defined $data{'perf'}{'disk'}{$id}{'normal'}{'free'} && $data{'perf'}{'disk'}{$id}{'normal'}{'free'} ne "" ){
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'normal'}{'free'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'normal'}{'free'} += $data{'perf'}{'disk'}{$id}{'normal'}{'free'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'normal'}{'free'} = $data{'perf'}{'disk'}{$id}{'normal'}{'free'};
            }
          }

          if ( defined $data{'perf'}{'disk'}{$id}{'normal'}{'used'} && $data{'perf'}{'disk'}{$id}{'normal'}{'used'} ne "" ){
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'normal'}{'used'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'normal'}{'used'} += $data{'perf'}{'disk'}{$id}{'normal'}{'used'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'normal'}{'used'} = $data{'perf'}{'disk'}{$id}{'normal'}{'used'};
            }
          }

          if ( defined $data{'perf'}{'disk'}{$id}{'normal'}{'total'} && $data{'perf'}{'disk'}{$id}{'normal'}{'total'} ne "" ){
            if ( Xormon::isdigit( $data{'perf'}{'tier'}{$tier_type}{'normal'}{'total'} ) ){
              $data{'perf'}{'tier'}{$tier_type}{'normal'}{'total'} += $data{'perf'}{'disk'}{$id}{'normal'}{'total'};
            }
            else{
              $data{'perf'}{'tier'}{$tier_type}{'normal'}{'total'} = $data{'perf'}{'disk'}{$id}{'normal'}{'total'};
            }
          }

        }
      }
      foreach my $tier_type (@tier_types){
        #
        #weighted average average from disk
        #
        if (defined $data{'perf'}{'tier'}{$tier_type}{'resp_t_w'} && $data{'perf'}{'tier'}{$tier_type}{'resp_t_w'}  ne "" && defined $data{'perf'}{'tier'}{$tier_type}{'write_io'} && $data{'perf'}{'tier'}{$tier_type}{'write_io'} ne "" && $data{'perf'}{'tier'}{$tier_type}{'write_io'} != 0){
          $data{'perf'}{'tier'}{$tier_type}{'resp_t_w'} = $data{'perf'}{'tier'}{$tier_type}{'resp_t_w'} / $data{'perf'}{'tier'}{$tier_type}{'write_io'};
        }
        if (defined $data{'perf'}{'tier'}{$tier_type}{'resp_t_r'} && $data{'perf'}{'tier'}{$tier_type}{'resp_t_r'}  ne "" && defined $data{'perf'}{'tier'}{$tier_type}{'read_io'} && $data{'perf'}{'tier'}{$tier_type}{'read_io'} ne "" && $data{'perf'}{'tier'}{$tier_type}{'read_io'} != 0){
          $data{'perf'}{'tier'}{$tier_type}{'resp_t_r'} = $data{'perf'}{'tier'}{$tier_type}{'resp_t_r'} / $data{'perf'}{'tier'}{$tier_type}{'read_io'};
        }
        if (defined $data{'perf'}{'tier'}{$tier_type}{'resp_t'} && $data{'perf'}{'tier'}{$tier_type}{'resp_t'}  ne "" && defined $data{'perf'}{'tier'}{$tier_type}{'io_rate'} && $data{'perf'}{'tier'}{$tier_type}{'io_rate'} ne "" && $data{'perf'}{'tier'}{$tier_type}{'io_rate'} != 0){
          $data{'perf'}{'tier'}{$tier_type}{'resp_t'} = $data{'perf'}{'tier'}{$tier_type}{'resp_t'} / $data{'perf'}{'tier'}{$tier_type}{'io_rate'};
        }

        #
        #from MB to GB
        #
        if (defined $data{'perf'}{'tier'}{$tier_type}{'free'} && $data{'perf'}{'tier'}{$tier_type}{'free'}  ne "" ){
          $data{'perf'}{'tier'}{$tier_type}{'free'} = $data{'perf'}{'tier'}{$tier_type}{'free'} / 1024 / 1024;
        }
        if (defined $data{'perf'}{'tier'}{$tier_type}{'used'} && $data{'perf'}{'tier'}{$tier_type}{'used'}  ne "" ){
          $data{'perf'}{'tier'}{$tier_type}{'used'} = $data{'perf'}{'tier'}{$tier_type}{'used'} / 1024 / 1024;
        }
        if (defined $data{'perf'}{'tier'}{$tier_type}{'total'} && $data{'perf'}{'tier'}{$tier_type}{'total'}  ne "" ){
          $data{'perf'}{'tier'}{$tier_type}{'total'} = $data{'perf'}{'tier'}{$tier_type}{'total'} / 1024 / 1024;
        }
      }
    }
  }

  ##########
  # Port
  #########
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/ports");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $members) = each (@{$return->{'members'}}))
      {
        my $id = "";
        if (defined $members->{'portPos'}->{'cardPort'} && $members->{'portPos'}->{'cardPort'} ne "" && defined $members->{'portPos'}->{'slot'} && $members->{'portPos'}->{'slot'} ne "" && defined $members->{'portPos'}->{'node'} && $members->{'portPos'}->{'node'} ne "" ){
          $id = "$members->{'portPos'}->{'node'}\-$members->{'portPos'}->{'slot'}\-$members->{'portPos'}->{'cardPort'}";
          my $protocol = "";

          if (defined $members->{'protocol'} && $members->{'protocol'} ne ""){
            if ( $members->{'protocol'} eq "1" ) {
              $protocol = "FC";
              $data{'perf'}{'port'}{$id}{'type'} = 0;
            }
            elsif ( $members->{'protocol'} eq "2" ) {
              $protocol = "iSCSI";
              $data{'perf'}{'port'}{$id}{'type'} = 1;
            }
            elsif ( $members->{'protocol'} eq "3" ) {
              $protocol = "FCOE";
              $data{'perf'}{'port'}{$id}{'type'} = 1;
            }
            elsif ( $members->{'protocol'} eq "4" ) {
              $protocol = "IP";
              $data{'perf'}{'port'}{$id}{'type'} = 1;
            }
            elsif ( $members->{'protocol'} eq "5" ) {
              $protocol = "SAS";
              $data{'perf'}{'port'}{$id}{'type'} = 1;
            }
            elsif ( $members->{'protocol'} eq "6" ) {
              $protocol = "NVMe";
              $data{'perf'}{'port'}{$id}{'type'} = 1;
            }
          }

          if (defined $members->{'label'} && $members->{'label'} ne ""){
            $data{'perf'}{'port'}{$id}{'name'} = "$members->{'label'} ($protocol)";
          }
          else{
            $data{'perf'}{'port'}{$id}{'name'} = "$id ($protocol)";
          }

          if ( defined $members->{'portWWN'} && $members->{'portWWN'} ne "" ){
            $data{'perf'}{'port'}{$id}{'portWWN'} = $members->{'portWWN'};
          }

        }
      }
    }
  }


  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/portstatistics/hires?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        my $id = "";
        if (defined $member->{'node'} && $member->{'node'} ne "" && defined $member->{'slot'} && $member->{'slot'} ne "" && defined $member->{'cardPort'} && $member->{'cardPort'} ne ""){
          $id = "$member->{'node'}\-$member->{'slot'}\-$member->{'cardPort'}";

          if (defined $member->{'IO'}->{'total'} && $member->{'IO'}->{'total'} ne ""){
            $data{'perf'}{'port'}{$id}{'io_rate'} = $member->{'IO'}->{'total'};
          }
          if (defined $member->{'IO'}->{'write'} && $member->{'IO'}->{'write'} ne ""){
            $data{'perf'}{'port'}{$id}{'write_io'} = $member->{'IO'}->{'write'};
          }
          if (defined $member->{'IO'}->{'read'} && $member->{'IO'}->{'read'} ne ""){
            $data{'perf'}{'port'}{$id}{'read_io'} = $member->{'IO'}->{'read'};
          }

          if (defined $member->{'KBytes'}->{'total'} && $member->{'KBytes'}->{'total'} ne ""){
            $data{'perf'}{'port'}{$id}{'data_rate'} = $member->{'KBytes'}->{'total'};
          }
          if (defined $member->{'KBytes'}->{'write'} && $member->{'KBytes'}->{'write'} ne ""){
            $data{'perf'}{'port'}{$id}{'write'} = $member->{'KBytes'}->{'write'};
          }
          if (defined $member->{'KBytes'}->{'read'} && $member->{'KBytes'}->{'read'} ne ""){
            $data{'perf'}{'port'}{$id}{'read'} = $member->{'KBytes'}->{'read'};
          }

          if (defined $member->{'serviceTimeMS'}->{'total'} && $member->{'serviceTimeMS'}->{'total'} ne ""){
            $data{'perf'}{'port'}{$id}{'resp_t'} = $member->{'serviceTimeMS'}->{'total'};
          }
          if (defined $member->{'serviceTimeMS'}->{'write'} && $member->{'serviceTimeMS'}->{'write'} ne ""){
            $data{'perf'}{'port'}{$id}{'resp_t_w'} = $member->{'serviceTimeMS'}->{'write'};
          }
          if (defined $member->{'serviceTimeMS'}->{'read'} && $member->{'serviceTimeMS'}->{'read'} ne ""){
            $data{'perf'}{'port'}{$id}{'resp_t_r'} = $member->{'serviceTimeMS'}->{'read'};
          }
        }
      }
    }
  }


  #########
  # HOST
  #########
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/vlunstatistics/hires;groupby:hostname?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        if ( defined $member->{'hostname'} && $member->{'hostname'} ne ""){
          my $name = $member->{'hostname'};
          #Iops
          if (defined $member->{IO}->{'total'} && $member->{IO}->{'total'} ne ""){
            $data{'perf'}{'host'}{$name}{'io_rate'} = $member->{IO}->{'total'};
          }
          if (defined $member->{IO}->{'read'} && $member->{IO}->{'read'} ne ""){
            $data{'perf'}{'host'}{$name}{'read_io'} = $member->{IO}->{'read'};
          }
          if (defined $member->{IO}->{'write'} && $member->{IO}->{'write'} ne ""){
            $data{'perf'}{'host'}{$name}{'write_io'} = $member->{IO}->{'write'};
          }
          #Response time
          if (defined $member->{serviceTimeMS}->{'total'} && $member->{serviceTimeMS}->{'total'} ne ""){
            $data{'perf'}{'host'}{$name}{'resp_t'} = $member->{serviceTimeMS}->{'total'};
          }
          if (defined $member->{serviceTimeMS}->{'read'} && $member->{serviceTimeMS}->{'read'} ne ""){
            $data{'perf'}{'host'}{$name}{'resp_t_r'} = $member->{serviceTimeMS}->{'read'};
          }
          if (defined $member->{serviceTimeMS}->{'write'} && $member->{serviceTimeMS}->{'write'} ne ""){
            $data{'perf'}{'host'}{$name}{'resp_t_w'} = $member->{serviceTimeMS}->{'write'};
          }
          #Data
          if (defined $member->{'KBytes'}->{'total'} && $member->{'KBytes'}->{'total'} ne ""){
            $data{'perf'}{'host'}{$name}{'data_rate'} = $member->{'KBytes'}->{'total'};
          }
          if (defined $member->{'KBytes'}->{'read'} && $member->{'KBytes'}->{'read'} ne ""){
            $data{'perf'}{'host'}{$name}{'read'} = $member->{'KBytes'}->{'read'};
          }
          if (defined $member->{'KBytes'}->{'write'} && $member->{'KBytes'}->{'write'} ne ""){
            $data{'perf'}{'host'}{$name}{'write'} = $member->{'KBytes'}->{'write'};
          }
        }
      }
    }
  }

  #########
  # Volume
  #########
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/vlunstatistics/hires;groupby:volumeName?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  if (defined $return && $return ne ""){
    #print Dumper $return;
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        if ( defined $member->{'volumeName'} && $member->{'volumeName'} ne ""){
          my $name = $member->{'volumeName'};
          #Iops
          if (defined $member->{IO}->{'total'} && $member->{IO}->{'total'} ne ""){
            $data{'perf'}{'volume'}{$name}{'io_rate'} = $member->{IO}->{'total'};
          }
          if (defined $member->{IO}->{'read'} && $member->{IO}->{'read'} ne ""){
            $data{'perf'}{'volume'}{$name}{'read_io'} = $member->{IO}->{'read'};
          }
          if (defined $member->{IO}->{'write'} && $member->{IO}->{'write'} ne ""){
            $data{'perf'}{'volume'}{$name}{'write_io'} = $member->{IO}->{'write'};
          }
          #Response time
          if (defined $member->{serviceTimeMS}->{'total'} && $member->{serviceTimeMS}->{'total'} ne ""){
            $data{'perf'}{'volume'}{$name}{'resp_t'} = $member->{serviceTimeMS}->{'total'};
          }
          if (defined $member->{serviceTimeMS}->{'read'} && $member->{serviceTimeMS}->{'read'} ne ""){
            $data{'perf'}{'volume'}{$name}{'resp_t_r'} = $member->{serviceTimeMS}->{'read'};
          }
          if (defined $member->{serviceTimeMS}->{'write'} && $member->{serviceTimeMS}->{'write'} ne ""){
            $data{'perf'}{'volume'}{$name}{'resp_t_w'} = $member->{serviceTimeMS}->{'write'};
          }
          #Data
          if (defined $member->{'KBytes'}->{'total'} && $member->{'KBytes'}->{'total'} ne ""){
            $data{'perf'}{'volume'}{$name}{'data_rate'} = $member->{'KBytes'}->{'total'};
          }
          if (defined $member->{'KBytes'}->{'read'} && $member->{'KBytes'}->{'read'} ne ""){
            $data{'perf'}{'volume'}{$name}{'read'} = $member->{'KBytes'}->{'read'};
          }
          if (defined $member->{'KBytes'}->{'write'} && $member->{'KBytes'}->{'write'} ne ""){
            $data{'perf'}{'volume'}{$name}{'write'} = $member->{'KBytes'}->{'write'};
          }
        }
      }
    }
  }

  #VOL
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/volumespacedata/hires?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  # print Dumper $return;
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        if ( defined $member->{'name'} && $member->{'name'} ne "" ){
          my $name = $member->{'name'};
          if (defined $data{'perf'}{'volume'}{$name} && $data{'perf'}{'volume'}{$name} ne ""){
            if (defined $member->{'id'} && $member->{'id'} ne ""){
              $data{'perf'}{'volume'}{$name}{'id'} = $member->{'id'};
            }
            if (defined $member->{'wwn'} && $member->{'wwn'} ne ""){
              $data{'perf'}{'volume'}{$name}{'wwn'} = $member->{'wwn'};
            }
            if (defined $member->{'totalSpace'}->{'reservedMiB'} && $member->{'totalSpace'}->{'reservedMiB'} ne ""){
              $data{'perf'}{'volume'}{$name}{'user'} = $member->{'totalSpace'}->{'reservedMiB'};
              $data{'perf'}{'volume'}{$name}{'sum_capacity_reserved'} = $member->{'totalSpace'}->{'reservedMiB'}; # for future use
            }
            ## reserved capacities for possible future use
            if (defined $member->{'userSpace'}->{'reservedMiB'} && $member->{'userSpace'}->{'reservedMiB'} ne ""){
              $data{'perf'}{'volume'}{$name}{'capacity_reserved'} = $member->{'userSpace'}->{'reservedMiB'};
            }
            if (defined $member->{'snapSpace'}->{'reservedMiB'} && $member->{'snapSpace'}->{'reservedMiB'} ne ""){
              $data{'perf'}{'volume'}{$name}{'snapshot_capacity_reserved'} = $member->{'snapSpace'}->{'reservedMiB'};
            }
            ######################
            if (defined $member->{'totalSpace'}->{'virtualSizeMiB'} && $member->{'totalSpace'}->{'virtualSizeMiB'} ne ""){
              $data{'perf'}{'volume'}{$name}{'virt'} = $member->{'totalSpace'}->{'virtualSizeMiB'};
            }
            if (defined $member->{'totalSpace'}->{'usedMiB'} && $member->{'totalSpace'}->{'usedMiB'} ne ""){
              $data{'perf'}{'volume'}{$name}{'cap_used'} = $member->{'totalSpace'}->{'usedMiB'};
            }
            if (defined $member->{'totalSpace'}->{'vcopyMiB'} && $member->{'totalSpace'}->{'vcopyMiB'} ne ""){
              $data{'perf'}{'volume'}{$name}{'snap'} = $member->{'totalSpace'}->{'vcopyMiB'};
            }
            if (defined $member->{'totalSpace'}->{'hostWriteMiB'} && $member->{'totalSpace'}->{'hostWriteMiB'} ne ""){
              $data{'perf'}{'volume'}{$name}{'real'} = $member->{'totalSpace'}->{'hostWriteMiB'};
            }
            if (defined $member->{'rawReserved'}->{'adminMiB'} && $member->{'rawReserved'}->{'adminMiB'} ne ""){
              $data{'perf'}{'volume'}{$name}{'admin'} = $member->{'rawReserved'}->{'adminMiB'};
            }
            if (defined $member->{'rawReserved'}->{'totalMiB'} && $member->{'rawReserved'}->{'totalMiB'} ne ""){
              $data{'perf'}{'volume'}{$name}{'cap'} = $member->{'rawReserved'}->{'totalMiB'};
            }
            if (defined $member->{'userCPG'} && $member->{'userCPG'} ne ""){
              $data{'perf'}{'volume'}{$name}{'poolName'} = $member->{'userCPG'};
            }
          }
        }
      }
    }
  }

  #VVOL
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/systemreporter/attime/vlunstatistics/hires?query=\"sampleTime GE $cfg->{'start-time'} AND sampleTime LE $cfg->{'end-time'}\"");
  #Mapping host2volume
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        my $vol_name = "";
        my $host_name = "";
        if ( defined $member->{'volumeName'} && $member->{'volumeName'} ne "" ){
          $vol_name = $member->{'volumeName'};
        }
        if ( defined $member->{'hostname'} && $member->{'hostname'} ne "" ){
          $host_name = $member->{'hostname'};
        }
        if ( defined $member->{'hostWWN'} && $member->{'hostWWN'} ne "" ){
          $data{'perf'}{'host'}{$host_name}{'wwn'} = $member->{'hostWWN'};
        }

        ###### HOST IDS TO VOLUMES
        my $host_hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'}."HOST".$host_name); $host_hash_key = lc($host_hash_key);

        if (defined $data{'perf'}{'volume'}{$vol_name}{'hosts'} && $data{'perf'}{'volume'}{$vol_name}{'hosts'} ne ""){
          if ($data{'perf'}{'volume'}{$vol_name}{'hosts'} !~ m/$host_hash_key/ ){
            $data{'perf'}{'volume'}{$vol_name}{'hosts'} .= " $host_hash_key";
          }
        }else{
          $data{'perf'}{'volume'}{$vol_name}{'hosts'} = $host_hash_key;
        }

        ###### VOLUME IDS TO HOST
        if (defined $data{'perf'}{'volume'}{$vol_name}{'id'} && $data{'perf'}{'volume'}{$vol_name}{'id'} ne ""){

          if (defined $data{'perf'}{'host'}{$host_name}{'volume_id'} && $data{'perf'}{'host'}{$host_name}{'volume_id'} ne ""){
            my $vol_id = Xormon::create_hash_key($cfg->{'hostcfg_id'}."VOLUME".$data{'perf'}{'volume'}{$vol_name}{'id'}); $vol_id = lc($vol_id);
            if ( $data{'perf'}{'host'}{$host_name}{'volume_id'} !~ m/$vol_id/ ){
              my $vol_hash_id = Xormon::create_hash_key($cfg->{'hostcfg_id'}."VOLUME".$data{'perf'}{'volume'}{$vol_name}{'id'}); $vol_hash_id = lc($vol_hash_id);
              $data{'perf'}{'host'}{$host_name}{'volume_id'} .= " $vol_hash_id";
            }
          }
          else{
            my $vol_hash_id = Xormon::create_hash_key($cfg->{'hostcfg_id'}."VOLUME".$data{'perf'}{'volume'}{$vol_name}{'id'}); $vol_hash_id = lc($vol_hash_id);
            $data{'perf'}{'host'}{$host_name}{'volume_id'} = $vol_hash_id;
          }
        }
      }
    }
  }
  #Remote Copy Configuration
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/remotecopy");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      if ( defined $return->{'mode'} && $return->{'mode'} ne ""){
        if ($return->{'mode'} == 1){
          $data{'perf'}{'remotecopy'}{'mode'} = "not configured";
        }
        elsif ($return->{'mode'} == 2){
          $data{'perf'}{'remotecopy'}{'mode'} = "Started";
        }
        elsif ($return->{'mode'} == 3){
          $data{'perf'}{'remotecopy'}{'mode'} = "Stopped";
        }
      }
      if ( defined $return->{'status'} && $return->{'status'} ne ""){
        if ($return->{'status'} == 1){
          $data{'perf'}{'remotecopy'}{'status'} = "NORMAL";
        }
        elsif ($return->{'status'} == 2){
          $data{'perf'}{'remotecopy'}{'status'} = "STARTUP";
        }
        elsif ($return->{'status'} == 3){
          $data{'perf'}{'remotecopy'}{'status'} = "SHUTDOWN";
        }
        elsif ($return->{'status'} == 4){
          $data{'perf'}{'remotecopy'}{'status'} = "ENABLE";
        }
        elsif ($return->{'status'} == 5){
          $data{'perf'}{'remotecopy'}{'status'} = "DISABLE";
        }
        elsif ($return->{'status'} == 6){
          $data{'perf'}{'remotecopy'}{'status'} = "INVALID";
        }
        elsif ($return->{'status'} == 7){
          $data{'perf'}{'remotecopy'}{'status'} = "NODEUP";
        }
        elsif ($return->{'status'} == 8){
          $data{'perf'}{'remotecopy'}{'status'} = "UPGRADE";
        }
      }
    }
  }

  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/remotecopytargets");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        if ( defined $member->{'name'} && $member->{'name'} ne "" ){
          my $name = $member->{'name'};
          if ( defined $member->{'id'} && $member->{'id'} ne ""){
            $data{'perf'}{'target'}{$name}{'id'} = $member->{'id'};
          }
          if ( defined $member->{'status'} && $member->{'status'} ne ""){
            if ($member->{'status'} == 1){
              $data{'perf'}{'target'}{$name}{'status'} = "Disabled";
            }
            elsif ($member->{'status'} == 2){
              $data{'perf'}{'target'}{$name}{'status'} = "New";
            }
            elsif ($member->{'status'} == 3){
              $data{'perf'}{'target'}{$name}{'status'} = "Ready";
            }
            elsif ($member->{'status'} == 4){
              $data{'perf'}{'target'}{$name}{'status'} = "Unsupported";
            }
            elsif ($member->{'status'} == 5){
              $data{'perf'}{'target'}{$name}{'status'} = "Failing";
            }
            elsif ($member->{'status'} == 6){
              $data{'perf'}{'target'}{$name}{'status'} = "Failed";
            }
            elsif ($member->{'status'} == 99){
              $data{'perf'}{'target'}{$name}{'status'} = "Unknown";
            }
          }
          if ( defined $member->{'state'} && $member->{'state'} ne ""){
            if ($member->{'state'} == 1){
              $data{'perf'}{'target'}{$name}{'state'} = "Normal";
            }
            elsif ($member->{'state'} == 2){
              $data{'perf'}{'target'}{$name}{'state'} = "Degraded";
            }
            elsif ($member->{'state'} == 3){
              $data{'perf'}{'target'}{$name}{'state'} = "Failed";
            }
            elsif ($member->{'state'} == 99){
              $data{'perf'}{'target'}{$name}{'state'} = "Unknown";
            }
          }
          if ( defined $member->{'systemName'} && $member->{'systemName'} ne ""){
            $data{'perf'}{'target'}{$name}{'system'} = $member->{'systemName'};
          }
        }
      }
    }
  }
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/remotecopygroups");
  if (defined $return && $return ne ""){
    if (defined $return->{'members'} && $return->{'members'} ne "" && ref($return->{'members'}) eq "ARRAY" ){
      while ((my $index, my $member) = each (@{$return->{'members'}}))
      {
        if ( defined $member->{'targets'} && ref( $member->{'targets'} ) eq "ARRAY"){
          while ( ( my $index, my $target ) = each ( @{ $member->{'targets'} } ) )
          {
            if ( defined $target->{'targetName'} && $target->{'targetName'} ne "" ){
              my $t_name = $target->{'targetName'};
              if ( defined $target->{'mode'} && $target->{'mode'} ne "" ){
                if ( $target->{'mode'} == 1 ){
                  $data{'perf'}{'target'}{$t_name}{'mode'} = "Sync";
                }
                elsif ($target->{'mode'} == 2 ){
                  $data{'perf'}{'target'}{$t_name}{'mode'} = "Periodic";
                }
                elsif ($target->{'mode'} == 3 ){
                  $data{'perf'}{'target'}{$t_name}{'mode'} = "Periodic";
                }
                elsif ($target->{'mode'} == 4 ){
                  $data{'perf'}{'target'}{$t_name}{'mode'} = "Async";
                }
              }
            }
          }
        }
        if ( defined $member->{'volumes'} && ref( $member->{'volumes'} ) eq "ARRAY"){
          while ( ( my $index, my $lc_vol ) = each ( @{ $member->{'volumes'} } ) )
          {
            my $lc_vol_name = ""; #Local VV Name
            my $lc_vol_id   = ""; #Local VV ID
            if (defined $lc_vol->{'localVolumeName'} && $lc_vol->{'localVolumeName'} ne ""){
              $lc_vol_name = $lc_vol->{'localVolumeName'};
            }
            if (defined $lc_vol->{'localVolumeID'} && $lc_vol->{'localVolumeID'} ne ""){
              $lc_vol_id = $lc_vol->{'localVolumeID'};
            }
            if (defined $lc_vol->{'remoteVolumes'} && ref( $lc_vol->{'remoteVolumes'} ) eq "ARRAY"){
              while ( ( my $index, my $rm_vol ) = each ( @{ $lc_vol->{'remoteVolumes'} } ) )
              {
                if ( defined $lc_vol_name && $lc_vol_name ne "" && defined $lc_vol_id && $lc_vol_id ne "" ){
                  $data{'perf'}{'mirror'}{$lc_vol_name}{'id'} = $lc_vol_id;

                  if ( defined $rm_vol->{'remoteVolumeName'} && $rm_vol->{'remoteVolumeName'} ne "" ){
                    $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeName'} = $rm_vol->{'remoteVolumeName'};
                  }
                  if ( defined $rm_vol->{'remoteVolumeID'} && $rm_vol->{'remoteVolumeID'} ne "" ){
                    $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeID'} = $rm_vol->{'remoteVolumeID'};
                  }
                  if ( defined $rm_vol->{'syncStatus'} && $rm_vol->{'syncStatus'} ne "" ){
                    if ( $rm_vol->{'syncStatus'} == 1 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "New";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 2 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "Syncing";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 3 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "Synced";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 4 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "Unsync";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 5 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "Stale";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 6 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "Newpresynced";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 7 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "NewSyncedFromSnap";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 8 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "Stopped";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 9 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "Failsafe";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 10 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "Unknown";
                    }
                    elsif ( $rm_vol->{'syncStatus'} == 11 ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'remoteVolumeStatus'} = "Logging";
                    }
                  }
                  if ( defined $member->{'name'} && $member->{'name'} ne "" ){
                    $data{'perf'}{'mirror'}{$lc_vol_name}{'groupName'} = $member->{'name'};
                  }
                  if ( defined $member->{'id'} && $member->{'id'} ne "" ){
                    $data{'perf'}{'mirror'}{$lc_vol_name}{'groupID'} = $member->{'id'};
                  }
                  if ( defined $member->{'role'} && $member->{'role'} ne "" ){
                    if ( $member->{'role'} == 1 ){
                    $data{'perf'}{'mirror'}{$lc_vol_name}{'groupRole'} = "Primary";
                    }
                    elsif ( $member->{'role'} == 2 ){
                    $data{'perf'}{'mirror'}{$lc_vol_name}{'groupRole'} = "Secondary";
                    }
                  }
                  if (defined $rm_vol->{'targetName'} && $rm_vol->{'targetName'} ne "" ){
                    my $target_name = $rm_vol->{'targetName'};
                    if ( defined $data{'perf'}{'target'}{$target_name}{'system'} && $data{'perf'}{'target'}{$target_name}{'system'} ne "" ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'systemName'} = $data{'perf'}{'target'}{$target_name}{'system'};
                    }
                    if ( defined $data{'perf'}{'target'}{$target_name}{'status'} && $data{'perf'}{'target'}{$target_name}{'status'} ne "" ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'groupStatus'} = $data{'perf'}{'target'}{$target_name}{'status'};
                    }
                    if ( defined $data{'perf'}{'target'}{$target_name}{'status'} && $data{'perf'}{'target'}{$target_name}{'status'} ne "" ){
                      $data{'perf'}{'mirror'}{$lc_vol_name}{'groupMode'} = $data{'perf'}{'target'}{$target_name}{'mode'};
                    }
                    #if ( defined $data{'perf'}{'target'}{$target_name}{'status'} && $data{'perf'}{'target'}{$target_name}{'status'} ne "" ){
                    #  $data{'perf'}{'mirror'}{$lc_vol_name}{'status'} = $data{'perf'}{'target'}{$target_name}{'status'};
                    #}
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  return \%data;
}

sub vol_cap {

  my $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/volumes");

  if ( defined $return && $return ne "" ) {
    if ( defined $return->{'members'} && $return->{'members'} ne "" && ref( $return->{'members'} ) eq "ARRAY" ) {
      while ( ( my $index, my $member ) = each( @{ $return->{'members'} } ) ) {
        my $name         = "";
        my $size         = "";
        my $user_u       = "";
        my $snap_u       = "";
        my $total_u      = "";
        my $provisioning = "";
        my $copy_type    = "";
        my $copy_of      = "";
        my $user_cpg     = "";
        my $snap_cpg     = "";

        if ( defined $member->{'name'} && $member->{'name'} ne "" ) {
          $name = $member->{'name'};

          if ( $name eq "admin" || $name eq ".srdata" ) {
            next;
          }

          #size
          if ( defined $member->{'sizeMiB'} && $member->{'sizeMiB'} ne "" ) {
            $size = $member->{'sizeMiB'};
          }
          #user used
          if ( defined $member->{'userSpace'}->{'usedMiB'} && $member->{'userSpace'}->{'usedMiB'} ne "" ) {
            $user_u = $member->{'userSpace'}->{'usedMiB'};
          }
          #snap used
          if ( defined $member->{'snapshotSpace'}->{'usedMiB'} && $member->{'snapshotSpace'}->{'usedMiB'} ne "" ) {
            $snap_u = $member->{'snapshotSpace'}->{'usedMiB'};
          }
          #total used sum
          if ( Xormon::isdigit($user_u) && Xormon::isdigit($snap_u) ) {
            $total_u = $user_u + $snap_u;
          }
          #provisioning
          if ( defined $member->{'provisioningType'} && $member->{'provisioningType'} ne "" ) {
            if ( $member->{'provisioningType'} == 1 ) {
              $provisioning = "FULL";
            }
            if ( $member->{'provisioningType'} == 2 ) {
              $provisioning = "TPVV";
            }
            if ( $member->{'provisioningType'} == 3 ) {
              $provisioning = "SNAP";
            }
            if ( $member->{'provisioningType'} == 4 ) {
              $provisioning = "PEER";
            }
            if ( $member->{'provisioningType'} == 5 ) {
              $provisioning = "UNKN";
            }
            if ( $member->{'provisioningType'} == 6 ) {
              $provisioning = "TDVV";
            }
            if ( $member->{'provisioningType'} == 7 ) {
              $provisioning = "DDS";
            }
          }
          #copyType
          if ( defined $member->{'copyType'} && $member->{'copyType'} ne "" ) {
            if ( $member->{'copyType'} == 1 ) {
              $copy_type = "BASE";
            }
            if ( $member->{'copyType'} == 2 ) {
              $copy_type = "PHYSICAL";
            }
            if ( $member->{'copyType'} == 3 ) {
              $copy_type = "VIRTUAL";
            }
          }
          #copyOf
          if ( defined $member->{'copyOf'} && $member->{'copyOf'} ne "" ) {
            $copy_of = $member->{'copyOf'};
          }
          #userCPG
          if ( defined $member->{'userCPG'} && $member->{'userCPG'} ne "" ) {
            $user_cpg = $member->{'userCPG'};
          }
          #snapCPG
          if ( defined $member->{'snapCPG'} && $member->{'snapCPG'} ne "" ) {
            $snap_cpg = $member->{'snapCPG'};
          }

          $data{'perf'}{'volume'}{$name}{'size'}         = $size;
          $data{'perf'}{'volume'}{$name}{'used_user'}    = $user_u;
          $data{'perf'}{'volume'}{$name}{'used_snap'}    = $snap_u;
          $data{'perf'}{'volume'}{$name}{'total_used'}   = $total_u;
          $data{'perf'}{'volume'}{$name}{'provisioning'} = $provisioning;
          $data{'perf'}{'volume'}{$name}{'copy_type'}    = $copy_type;
          $data{'perf'}{'volume'}{$name}{'copy_of'}      = $copy_of;
          $data{'perf'}{'volume'}{$name}{'user_cpg'}     = $user_cpg;
          $data{'perf'}{'volume'}{$name}{'snap_cpg'}     = $snap_cpg;
        }
      }
    }
  }
}

sub health_status {
  my $return        = "";
  my $status        = "ok";
  my @error_message = ();
  my %conf = ();
  my @hs = ();

  #Remote Copy
  my @remote_copy = ();
  if ( defined $data{'perf'}{'remotecopy'} && $data{'perf'}{'remotecopy'} ne "" ) {
    my $mode         = "";
    my $stat         = "";
    if ( defined $data{'perf'}{'remotecopy'}{'mode'} && $data{'perf'}{'remotecopy'}{'mode'} ne "" ) {
      $mode = $data{'perf'}{'remotecopy'}{'mode'};
      if ( $mode eq "STOPPED" ) {
        $status = "error";
        push( @error_message, "Remote copy mode : $mode\n" );
      }
    }
    if ( defined $data{'perf'}{'remotecopy'}{'status'} && $data{'perf'}{'remotecopy'}{'status'} ne "" ) {
      $stat = $data{'perf'}{'remotecopy'}{'status'};
      if ( $stat ne "NORMAL" && $stat ne "STARTUP" ) {
        $status = "error";
        push( @error_message, "Remote copy status : $status\n" );
      }
    }
    my %data;
    $data{mode}         = $mode;
    $data{status}       = $stat;
    push(@remote_copy,\%data);
    $conf{$cfg->{'hostcfg_id'}}{'epar_remotecopy'} = encode_json(\@remote_copy);
  }


  #CPGS POOL
  my @pool = ();
  while ( ( my $pool, my $value ) = each( %{ $data{'perf'}{'pool'} } ) ) {
    my $name  = $pool;
    my $state = "";
    my $id    = "";
    if ( defined $data{'perf'}{'pool'}{$name}{'id'} && $data{'perf'}{'pool'}{$name}{'id'} ne "" ) {
      $id = $data{'perf'}{'pool'}{$name}{'id'};
    }
    if ( defined $data{'perf'}{'pool'}{$name}{'status'} && $data{'perf'}{'pool'}{$name}{'status'} ne "" ) {
      $state = $data{'perf'}{'pool'}{$name}{'status'};
      if ( $state ne "NORMAL" ) {
        $status = "error";
        push( @error_message, "CPG $name status : $status\n" );
      }
    }
    my %data;
    $data{id}         = $id;
    $data{name}       = $name;
    $data{state}       = $state;
    push(@pool,\%data);
  }

  $conf{$cfg->{'hostcfg_id'}}{'epar_pool'} = encode_json(\@pool);

  #Volume
  my @volumes = ();

  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/volumes");
  if ( defined $return && $return ne "" ) {
    if ( defined $return->{'members'} && $return->{'members'} ne "" && ref( $return->{'members'} ) eq "ARRAY" ) {
      while ( ( my $index, my $member ) = each( @{ $return->{'members'} } ) ) {
        if ( defined $member->{'id'} && $member->{'id'} ne "" ) {
          my $id     = $member->{'id'};
          my $name   = "";
          my $state  = "";
          my $dstate = "";

          if ( defined $member->{'name'} && $member->{'name'} ne "" ) {
            $name = $member->{'name'};
          }

          if ( defined $member->{'state'} && $member->{'state'} ne "" ) {
            if ( $member->{'state'} == 1 ) {
              $state = "NORMAL";
            }
            elsif ( $member->{'state'} == 2 ) {
              $state  = "DEGRADED";
              $status = "error";
              push( @error_message, "Volume $name state : $state\n" );
            }
            elsif ( $member->{'state'} == 3 ) {
              $state  = "FAILED";
              $status = "error";
              push( @error_message, "Volume $name state : $state\n" );
            }
            elsif ( $member->{'state'} == 99 ) {
              $state  = "UNKNOWN";
              $status = "error";
              push( @error_message, "Volume $name state : $state\n" );
            }
          }

          if ( defined $member->{'deduplicationState'} && $member->{'deduplicationState'} ne "" ) {
            if ( $member->{'state'} == 1 ) {
              $dstate = "YES";
            }
            elsif ( $member->{'state'} == 2 ) {
              $dstate = "NO";
            }
            elsif ( $member->{'state'} == 3 ) {
              $dstate = "NA";
            }
          }

          my %data;
          $data{id}         = $id;
          $data{name}       = $name;
          $data{state}      = $state;
          $data{dstate}     = $dstate;
          push(@volumes,\%data);
        }
      }
    }
  }

  $conf{$cfg->{'hostcfg_id'}}{'epar_volumes'} = encode_json(\@volumes);

  #Port
  my @ports = ();

  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/ports");
  if ( defined $return && $return ne "" ) {
    if ( defined $return->{'members'} && $return->{'members'} ne "" && ref( $return->{'members'} ) eq "ARRAY" ) {
      while ( ( my $index, my $member ) = each( @{ $return->{'members'} } ) ) {
        my $id       = "";
        my $node     = "";
        my $slot     = "";
        my $port     = "";
        my $type     = "";
        my $state    = "";
        my $failover = "";
        my @types    = ( "", "host", "disk", "free", "iport", "rcfc", "peer", "rcip", "iscsi", "cna", "fs" );
        my @links    = ( "", "Configuration wait", "ALPA wait", "Login wait", "ready", "loss_sync,error_state", "xxx", "nonparticipate", "coredump", "offline", "fwdead", "idle_for_reset", "dhcp_in_progress", "pending_reset" );
        my @fails    = ( "", "none", "failover_pending", "failed_over", "active", "active_down", "active_failed", "failback_pending" );

        if ( defined $member->{'portPos'}->{'cardPort'} && $member->{'portPos'}->{'cardPort'} ne "" && defined $member->{'portPos'}->{'slot'} && $member->{'portPos'}->{'slot'} ne "" && defined $member->{'portPos'}->{'node'} && $member->{'portPos'}->{'node'} ne "" ) {
          $id   = "$member->{'portPos'}->{'node'}\-$member->{'portPos'}->{'slot'}\-$member->{'portPos'}->{'cardPort'}";
          $node = $member->{'portPos'}->{'node'};
          $slot = $member->{'portPos'}->{'slot'};
          $port = $member->{'portPos'}->{'cardPort'};

          if ( defined $member->{'type'} && $member->{'type'} ne "" ) {
            $type = $types[ $member->{'type'} ];
          }

          if ( defined $member->{'linkState'} && $member->{'linkState'} ne "" ) {
            $state = $links[ $member->{'linkState'} ];
            if ( $type ne "free" && $type ne "rcip" ) {
              if ( $member->{'linkState'} == 5 || $member->{'linkState'} == 6 || $member->{'linkState'} == 7 || $member->{'linkState'} == 8 || $member->{'linkState'} == 9 || $member->{'linkState'} == 10 || $member->{'linkState'} == 11 ) {
                $status = "error";
                push( @error_message, "Port $id linkState : $member->{'linkState'}\n" );
              }
            }
          }

          if ( defined $member->{'failoverState'} && $member->{'failoverState'} ne "" ) {
            $failover = $fails[ $member->{'failoverState'} ];
          }

          my %data;
          $data{nsp}        = "$node:$slot:$port";
          $data{type}       = $type;
          $data{state}      = $state;
          $data{failover}    = $failover;
          push(@ports,\%data);
        }
      }
    }
  }

  $conf{$cfg->{'hostcfg_id'}}{'epar_ports'} = encode_json(\@ports);

  #Disk
  my @disks = ();
  $return = new_get_request("https://$cfg->{'host'}:$cfg->{'apiPort'}/api/v1/disks");
  if ( defined $return && $return ne "" ) {
    if ( defined $return->{'members'} && $return->{'members'} ne "" && ref( $return->{'members'} ) eq "ARRAY" ) {
      while ( ( my $index, my $member ) = each( @{ $return->{'members'} } ) ) {
        my $id       = "";
        my $position = "";
        my $state    = "";
        if ( defined $member->{'id'} && $member->{'id'} ne "" ) {
          $id = $member->{'id'};
        }
        if ( defined $member->{'position'} && $member->{'position'} ne "" ) {
          $position = $member->{'position'};
        }
        if ( defined $member->{'state'} && $member->{'state'} ne "" ) {
          if ( $member->{'state'} == 1 ) {
            $state = "Normal";
          }
          elsif ( $member->{'state'} == 2 ) {
            $state  = "Degraded";
            $status = "error";
            push( @error_message, "Disk $id state : $state\n" );
          }
          elsif ( $member->{'state'} == 3 ) {
            $state  = "New";
            $status = "error";
            push( @error_message, "Disk $id state : $state\n" );
          }
          elsif ( $member->{'state'} == 4 ) {
            $state  = "Failed";
            $status = "error";
            push( @error_message, "Disk $id state : $state\n" );
          }
          elsif ( $member->{'state'} == 99 ) {
            $state  = "Unknown";
            $status = "error";
            push( @error_message, "Disk $id state : $state\n" );
          }
        }
        my %data;
          $data{id}        = "$id";
          $data{position}  = $position;
          $data{state}     = $state;
          push(@disks,\%data);
      }
    }
    $conf{$cfg->{'hostcfg_id'}}{'epar_disks'} = encode_json(\@disks);
  }

   my %device_status = (
    "item_id" => $cfg->{'hostcfg_id'},
    "status" => "$status",
    "data" => "@error_message",
    #"data" => "testujeme status",
  );

  push(@hs, \%device_status);

  $xormon->saveStatus(\@hs);
  $xormon->saveConfJSON(\%conf);
}

######################
#  3PAR SSH/CLI VERSION  #
######################

sub collect_ssh{
  my $cfg = shift;
  #Xormon::log("CLIIIIIIIIIII");
  #print Dumper $cfg;

  #
  # mandatory cfg
  #

  unless ( exists $cfg->{'hostcfg_id'}  && $cfg->{'hostcfg_id'} ne '' )  { $logger->error("Cannot continue, 'hostcfg_id' is not configured!") && return "error"; }
  unless ( exists $cfg->{'hostalias'}   && $cfg->{'hostalias'} ne '' )   { $logger->error("Cannot continue, 'hostalias' is not configured!")  && return "error"; }
  unless ( exists $cfg->{'device'}      && $cfg->{'device'} ne '' )      { $logger->error("Cannot continue, 'device' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'host'}        && $cfg->{'host'} ne '' )        { $logger->error("Cannot continue, 'host' is not configured!")       && return "error"; }
  #unless ( exists $cfg->{'ssh-port'} )                                   { Xormon::error("Cannot continue, 'ssh-port' is not configured!")            && exit 0; }



  my $wrkdir = "$act_dir/data/$cfg->{'hostcfg_id'}";

  $ENV{'XormonNG'}                = 1;
  $ENV{'HP3PAR_USER'}             = $cfg->{'sshUsername'};
  $ENV{'HP3PAR_KEYFILE'}          = $cfg->{'sshKeyFile'} || "";
  $ENV{'HP3PAR_DIR'}              = $wrkdir;
  $ENV{'HP3PAR_HOSTCFG_ID'}       = $cfg->{'hostcfg_id'};
  $ENV{'HPE3PAR_CLIDIR'}          = $cfg->{'cliDir'} || "";
  $ENV{'ACT_DIR'}                 = $act_dir;
  #$ENV{'SAMPLE_RATE'}             = Global set....
  #$ENV{'HP3PAR_VV_SAMPLE_RATE'}   = Global set....

  #$ENV{'PUTTY'}                   = ????
  $ENV{'DEBUG'}                   = 0; ## Set global? 0 off 1 on
  $ENV{'DEBUG_FULL'}              = 0; ## Set global? 0 off 1 on
  $ENV{'HP3PAR_IP'}               = $cfg->{'host'};
  $ENV{'STORAGE_NAME'}            = $cfg->{'hostalias'};

  $ENV{'CLI_USER'}                = $cfg->{'cliUsername'} || '';
  $ENV{'CLI_PASS'}                = $cfg->{'cliPassword'} || '';

  if ( not -d "$act_dir/etc" ) {
    mkdir("$act_dir/etc") or die("ERROR: Cannot create directory '$act_dir/etc'. Exitting.");
  }

  if ( not -d "$act_dir/data" ) {
    mkdir("$act_dir/data") or die("ERROR: Cannot create directory '$act_dir/data'. Exitting.");
  }

  if ( not -d "$wrkdir" ) {
    mkdir("$wrkdir") or die("ERROR: Cannot create directory '$wrkdir'. Exitting.");
  }

  if ( not -d "$wrkdir/bin" ) {
    mkdir("$wrkdir/bin") or die("ERROR: Cannot create directory '$wrkdir/bin'. Exitting.");
  }

  $ENV{BINDIR}                    = "$wrkdir/bin";
  if ( not -d "$wrkdir/www" ) {
    mkdir("$wrkdir/www") or die("ERROR: Cannot create directory '$wrkdir/www'. Exitting.");
  }

  $ENV{WEBDIR}                    = "$wrkdir/www";



  #
  # create password file only for CLI
  #
  if (defined $cfg->{'cliDir'} && $cfg->{'cliDir'} ne ""){
    if (!-f "$act_dir/etc/$cfg->{'host'}/.3par-$cfg->{'host'}.pwf"){ # If not exist password file
      if ( -f "$cfg->{'cliDir'}/cli" ) { # if exist cli dir
        if ( defined $cfg->{'cliUsername'} && $cfg->{'cliUsername'} ne "" && defined $cfg->{'cliPassword'} && $cfg->{'cliPassword'} ne "" ){
          my $result = create_password_file($cfg->{'cliUsername'}, $cfg->{'cliPassword'}, $cfg->{'host'}, $act_dir);
          if ($result !~ m/failed/ ){
            my $pwf = "$act_dir/etc/$cfg->{'host'}/.3par-$cfg->{'host'}.pwf";
            my $cmd = "export TPDCERTDIR=$act_dir/etc/$cfg->{'host'}; export TPDPWFILE=$pwf; yes yes | $cfg->{'cliDir'}/cli -sys $cfg->{'host'} -pwf $act_dir/etc/$cfg->{'host'}/.3par-$cfg->{'host'}.pwf statvv -rw -d 1 -iter 1";
            my $cmd_out = cmd($cmd);
            if ( $cmd_out !~ m/VVname/ ) {
              die("ERROR: Testing password command failed: export TPDCERTDIR=$act_dir/etc/$cfg->{'host'}; export TPDPWFILE=$pwf; yes yes | $cfg->{'cliDir'}/cli -sys $cfg->{'host'} -pwf $act_dir/etc/$cfg->{'host'}/.3par-$cfg->{'host'}.pwf statvv -rw -d 1 -iter 1");
            }
          }
          else{
            die("ERROR: Create password command failed: $result");
          }
        }
      }
    }
  }


  #
  # configurable cfg
  #
  if ( exists $cfg->{'sample-rate'} && Xormon::isdigit($cfg->{'sample-rate'}) ) {
    $ENV{'SAMPLE_RATE'} = $cfg->{'sample-rate'} / 60;
    $ENV{'HP3PAR_VV_SAMPLE_RATE'}  = $cfg->{'sample-rate'} / 60;
  }
  else {
    # default
    $ENV{'SAMPLE_RATE'} = 5;
    $ENV{'HP3PAR_VV_SAMPLE_RATE'}  = 5;
  }

  #
  # create directories
  #
  #$wrkdir = "$act_dir/data/$cfg->{'hostalias'}";
  #$wrkdir = "$act_dir/data/$cfg->{'hostcfg_id'}";
  $wrkdir = "$act_dir/data/$cfg->{'hostcfg_id'}";
  $ENV{'HP3PAR_DIR'}  = $wrkdir;

  unless ( -d "$act_dir/data" )   { mkdir( "$act_dir/data", 0755 )   || $logger->error( "Cannot mkdir '$act_dir/data': $!" . __FILE__ . ":" . __LINE__ )   && return "error"; }
  unless ( -d "$wrkdir" )         { mkdir( "$wrkdir", 0755 )         || $logger->error( "Cannot mkdir '$wrkdir': $!" . __FILE__ . ":" . __LINE__ )         && return "error"; }
  unless ( -d "$wrkdir/iostats" ) { mkdir( "$wrkdir/iostats", 0755 ) || $logger->error( "Cannot mkdir '$wrkdir/iostats': $!" . __FILE__ . ":" . __LINE__ ) && return "error"; }

  #NS HS
  unless ( -d "$wrkdir/health_status" ) { mkdir( "$wrkdir/health_status", 0755 ) || $logger->error( "Cannot mkdir '$wrkdir/health_status': $!" . __FILE__ . ":" . __LINE__ ) && return "error"; }

  #
  # test agent scripts
  #
  unless ( -f "$act_dir/hp3parperf.pl" )   { $logger->error( "Script '$act_dir/hp3parperf.pl doesn't exist!': $!" . __FILE__ . ":" . __LINE__ ) && return "error"; }
  unless ( -f "$act_dir/HP3PAR.pm" )       { $logger->error( "Script '$act_dir/HP3PAR.pm doesn't exist!': $!" . __FILE__ . ":" . __LINE__ )     && return "error"; }


  #
  # create perf and conf
  #
  if ( is_process_running("hp3parperf.pl $ENV{'STORAGE_NAME'}") ) {
    # restart hp3parperf process only in the first run after midnight
    restart_hp3parperf("$wrkdir/hp3parperf-run", "hp3parperf.pl $ENV{'STORAGE_NAME'}");
  }
  unless ( is_process_running("hp3parperf.pl $ENV{'STORAGE_NAME'}") ) {
    #system ("$PERL", "-w", "$act_dir/hp3parperf.pl", "&"); # cannot be used like this, bcs this will not run it on the backgroud
    $logger->info( "starting a background task: $act_dir/hp3parperf.pl $ENV{'STORAGE_NAME'}" );
    system "perl -w $act_dir/hp3parperf.pl $ENV{'STORAGE_NAME'} &";
    system ("touch", "$wrkdir/hp3parperf-run");
  }

  process_collected_data($xormon,$cfg);
  return 1;
}

sub restart_hp3parperf {
  my $file           = shift;
  my $process_string = shift;

  if ( -f $file ) {
    if ( ( localtime( ( stat($file) )[9] ) )[3] != ( localtime() )[3] ) {
      # day of the last modification --> ( localtime( ( stat($file) )[9] ) )[3]
      # today                        --> ( localtime() )[3] )
      #print "############# ".( localtime( ( stat($file) )[9] ) )[3]." : ".( localtime() )[3]."\n";

      $logger->info( "First run after midnight, restart old '$process_string' processes:" );
      my @pids = ();
      foreach ( grep {/$process_string/} @{ get_ps_list() } ) {
        chomp $_;
        my ( undef, $pid, undef ) = split(/\s+/, $_);
        push(@pids, $pid);
        #print "$_\n";
      }
      if ( scalar(@pids) > 0 ) {
        #kill(15,@pids);
        #Xormon::log( "kill 15 " . join(" ", @pids) );
        kill('SIGTERM',@pids);
        $logger->info( "kill 'SIGTERM', " . join(" ", @pids) );
      }
      else {
        $logger->info( "Old hp3parperf.pl processes have not been found..." );
      }
    }
  }

  return 1;
}

sub terminate_old_processes {
  #Xormon::log( "Terminate old hp3parperf.pl processes:" );
  my @pids = ();
  foreach ( @{ get_ps_list() } ) {
    if ( $_ =~ m/hp3parperf\.pl/ ) {
      my ( undef, $pid, undef ) = split(/\s+/, $_);
      push(@pids, $pid);
      #print "$_\n";
    }
  }
  if ( scalar(@pids) > 0 ) {
    #kill(15,@pids);
    #Xormon::log( "kill 15 " . join(" ", @pids) );
    kill('SIGTERM',@pids);
    $logger->info( "kill 'SIGTERM', " . join(" ", @pids) );
  }
  else {
    $logger->info( "Old hp3parperf.pl processes have not been found..." );
  }

  return 1;
}

sub is_process_running {
  my $process_string = shift;

  if ( ( grep {/$process_string$/} @{ get_ps_list() } ) > 0 ) {
    return 1;
  }
  else {
    return 0;
  }
}

sub get_ps_list {
  open(my $ps,"ps -ef |") || $logger->error("'ps -ef' command failed! $!" . __FILE__ . ":" . __LINE__ ) && return "error";
  my @ps_list = <$ps>;
  chomp @ps_list;
  close($ps);

  return \@ps_list;
}

sub process_collected_data {
  $xormon = shift;
  $cfg    = shift;

  #Xormon::log("process_collected_data");

  #
  # Health Status !!! only NG !!
  #

  my $table = "$ENV{'HP3PAR_DIR'}/health_status/tables.json";
  my $hs    = "$ENV{'HP3PAR_DIR'}/health_status/hs.json";

  open( JSONTABLE, "< $table" ) || $logger->error(" Cannot open file $table") && return "error";
    my $table_data = <JSONTABLE>;
  close(JSONTABLE);

  if ( defined $table_data && $table_data ne "" ) {
    my %hs_tables = %{ decode_json($table_data) };
    $xormon->saveConfJSON(\%hs_tables);
  }

  open( JSONHS, "< $hs" ) || $logger->error(" Cannot open file $hs") && return "error";
    my $hs_data = <JSONHS>;
  close(JSONHS);

  if ( defined $hs_data && $hs_data ne "" ) {
    my @hs = ();
    my %device_status = %{ decode_json($hs_data) };
    push(@hs, \%device_status);
    $xormon->saveStatus(\@hs);
  }

  #
  # test main variables
  #

  unless ( exists $cfg->{'hostcfg_id'} )        { $logger->error("Cannot continue, 'uuid' is not configured!")        && return "error"; }
  unless ( exists $cfg->{'hostalias'} )         { $logger->error("Cannot continue, 'devicealias' is not configured!") && return "error"; }

  my $wrkdir = "$act_dir/data/$cfg->{'hostcfg_id'}";

  opendir( my $dir, "$wrkdir" ) || $logger->error( "Cannot open directory '$wrkdir' " . __FILE__ . ":" . __LINE__ ) && return "error";
  my @files = grep !/^\.\.?$/, readdir($dir);
  closedir($dir);
  chomp @files;

  my @conf_files        = grep( /$cfg->{'hostalias'}\_hp3parconf\_.*out$/, @files );
  my @perf_files        = grep( /$cfg->{'hostalias'}\_hp3parperf\_.*out$/, @files );
  my @state_files       = grep( /$cfg->{'hostalias'}\_hp3parstate\_.*out$/, @files );

  #
  # conf files
  #
  if ( scalar(@conf_files) > 0 ) {
    chomp @conf_files;

    foreach my $file ( sort { lc $a cmp lc $b } @conf_files ) {
      my $data = parse_conf("$wrkdir/$file");
      send_conf($data);
      print "$wrkdir/$file\n";
      if (-f "$wrkdir/$file" ){
        print "$wrkdir/$file\n";
        unlink ("$wrkdir/$file");
      }
      last;
      #if ( $data ) {
      #  # send conf data
      #}

      # always save last processed file for debug purposes
      #LoadDataModule::delete_storage_data( $ret, "$wrkdir/$st_name/$file", $st_name, $DEBUG, $wrkdir );
    }
  }

  #
  # perf files
  #
  my $status = ""; # status code
  if ( scalar(@perf_files) > 0 ) {
    chomp @perf_files;

    foreach my $file ( sort { lc $a cmp lc $b } @perf_files ) {
      my $data     = parse_perf("$wrkdir/$file");
      my $perf_out = prepare_perf($data);
      #print Dumper $perf_out;
      my $rc       = $xormon->saveJSON2($perf_out);

      #print "RC: $rc\n";

      if ( $rc eq "201" ) {
        $status = "successful";
        unlink ("$wrkdir/$file");
      }
      else {
        if ( $rc eq "401" ) {
          $status = "unauthorized";
        }
        else {
          $status = "error";
        }
        last;
      }
      #last;
      #if ( $data ) {
      #  # send conf data
      #}

      # always save last processed file for debug purposes
      #LoadDataModule::delete_storage_data( $ret, "$wrkdir/$st_name/$file", $st_name, $DEBUG, $wrkdir );
    }
  }

  #
  # State files
  #
  if ( scalar(@state_files ) > 0 ) {
    chomp @state_files ;

    foreach my $file ( sort { lc $a cmp lc $b } @state_files  ) {
      if (-f "$wrkdir/$file" ){
        #print "STATE FILE EXISTUJE\n";
        #print "$wrkdir/$file\n";
        unlink ("$wrkdir/$file");
      }
      last;
      #if ( $data ) {
      #  # send conf data
      #}

      # always save last processed file for debug purposes
      #LoadDataModule::delete_storage_data( $ret, "$wrkdir/$st_name/$file", $st_name, $DEBUG, $wrkdir );
    }
  }

}

sub parse_conf {
  my $conf_file = shift;

  open( IN, "<$conf_file" ) || $logger->error( "Couldn't open file $conf_file $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <IN>;
  close(IN);

  my %data_in;
  my $section    = "";
  my $act_header = "";

  my $line_idx = 0;
  foreach my $line (@lines) {
    chomp $line;
    $line_idx++;

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^----------/ )              { next; }
    if ( $line =~ m/Interval Start/ )           { next; }
    if ( $line =~ m/Interval Length/ )          { next; }

    if ( $line =~ m/^$/ && $section ne "DEVICE" )        { $section = "NA";           next; }
    if ( $line =~ m/Pool Level Configuration/ )          { $section = "POOL";         next; }
    if ( $line =~ m/Host Level Configuration/ )          { $section = "HOST";         next; }
    if ( $line =~ m/Physical Disk Configuration/ )       { $section = "DRIVE";        next; }
    if ( $line =~ m/Port Level Configuration/ )          { $section = "PORT";         next; }
    if ( $line =~ m/System Level Configuration/ )        { $section = "SYSTEM";       next; }
    if ( $line =~ m/Volume Level Configuration/ )        { $section = "VOLUME";       next; }

    # main cfg, Configuration Data
    if ( $section eq "SYSTEM" ) {
      if ( $line =~ m/^Total Capacity,/ ) {
        $line =~ s/^Total Capacity,//g;
        $data_in{DEVICE}{'Total Capacity'} = $line;
      }
      if ( $line =~ m/^Allocated,/ ) {
        $line =~ s/^Allocated,//g;
        $data_in{DEVICE}{'Allocated'} = $line;
      }
      if ( $line =~ m/^Allocated_Volumes,/ ) {
        $line =~ s/^Allocated_Volumes,//g;
        $data_in{DEVICE}{'Allocated_Volumes'} = $line;
      }
      if ( $line =~ m/^Allocated_System,/ ) {
        $line =~ s/^Allocated_System,//g;
        $data_in{DEVICE}{'Allocated_System'} = $line;
      }
      if ( $line =~ m/^Free,/ ) {
        $line =~ s/^Free,//g;
        $data_in{DEVICE}{'Free'} = $line;
      }
      if ( $line =~ m/^Failed,/ ) {
        $line =~ s/^Failed,//g;
        $data_in{DEVICE}{'Failed'} = $line;
      }
      if ( $line =~ m/^Release version,/ ) {
        $line =~ s/^Release version,\s+//;
        $data_in{DEVICE}{'Release version'} = $line;
      }
      if ( $line =~ m/^Patches,/ ) {
        $line =~ s/^Patches,//g;
        $data_in{DEVICE}{'Patches'} = $line;
      }
      if ( $line =~ m/^Storage System ID,/ ) {
        $line =~ s/^Storage System ID,//g;
        $data_in{DEVICE}{'Storage System ID'} = $line;
      }
      if ( $line =~ m/^Storage System Name,/ ) {
        $line =~ s/^Storage System Name,//g;
        $data_in{DEVICE}{'Storage System Name'} = $line;
      }
      if ( $line =~ m/^Storage System Model,\s+/ ) {
        $line =~ s/^Storage System Model,//g;
        $data_in{DEVICE}{'Storage System Model'} = $line;
      }
      if ( $line =~ m/^Storage System Serial,\s+/ ) {
        $line =~ s/^Storage System Serial,//g;
        $data_in{DEVICE}{'Storage System Serial'} = $line;
      }


    }

    # Pool Level Configuration
    if ( $section eq "POOL" ) {
      #Xormon::log("Jsem tady POOL");
      # parse header
      if ( $line =~ m/^CPG ID,CPG name,/ ) {
        $line =~ s/ /_/g;
        $act_header = $line;
        next;
      }
      # parse data lines
      $data_in{$section}{$line_idx} = Xormon::parse_CSV_line_by_header(",",$act_header,$line);
    }

    # Host Level Configuration
    if ( $section eq "HOST" ) {
      # parse header
      if ( $line =~ m/^host_id,id,name,port_count,,status,IQN WWPN,Volume IDs,Volume Names/ ) {
        $line =~ s/ /_/g;
        $act_header = $line;
        next;
      }
      # parse data lines
      $data_in{$section}{$line_idx} = Xormon::parse_CSV_line_by_header(",",$act_header,$line);
    }

    # Physical Disk Configuration
    if ( $section eq "DRIVE" ) {
      # parse header
      if ( $line =~ m/^Physical Disk ID,Cage Position/ ) {
        $line =~ s/ /_/g;
        $act_header = $line;
        next;
      }
      # parse data lines
      $data_in{$section}{$line_idx} = Xormon::parse_CSV_line_by_header(",",$act_header,$line);
    }

    # Port Level Configuration
    if ( $section eq "PORT" ) {
      # parse header
      if ( $line =~ m/^N\:S\:P,Mode,State,Node WWN,Port WWN,Type,Protocol/ ) {
        $line =~ s/ /_/g;
        $act_header = $line;
        next;
      }
      # parse data lines
      $data_in{$section}{$line_idx} = Xormon::parse_CSV_line_by_header(",",$act_header,$line);
    }

    # Volume Level Configuration
    if ( $section eq "VOLUME" ) {
      # parse header
      if ( $line =~ m/^Volume ID,Volume name,CPG ID,CPG name,State,Provisioning,Type,Copy Of/ ) {
        $line =~ s/ /_/g;
        $act_header = $line;

        next;
      }
      # parse data lines
      $data_in{$section}{$line_idx} = Xormon::parse_CSV_line_by_header(",",$act_header,$line);
    }
  }

  #print Dumper \%data_in;

  return \%data_in;
}

sub send_conf {
  my $data = shift;

  #print Dumper $data;

  my @architecture;
  my %conf;
  my $translator;
  my $io_group;
  my $cluster;
  my $host;

  #
  # DEVICE
  #
  # Architecture
  my %device_arch = (
    "item_id"   => $cfg->{'hostcfg_id'},
    "label"     => $cfg->{'hostalias'},
    "hw_type"   => "3par",
    "subsystem" => "device",
    "parents"   => [],
    "hostcfg_id" => $cfg->{'hostcfg_id'}
  );
  push(@architecture, \%device_arch);

  # conf
  if ( exists $data->{DEVICE}{'Allocated_Volumes'} )                  { $conf{ $cfg->{'hostcfg_id'} }{'allocated_volumes'}            = $data->{DEVICE}{'Allocated_Volumes'}; }
  if ( exists $data->{DEVICE}{'Patches'} )                            { $conf{ $cfg->{'hostcfg_id'} }{'patches'}                      = $data->{DEVICE}{'Patches'}; }
  if ( exists $data->{DEVICE}{'Allocated_System'} )                   { $conf{ $cfg->{'hostcfg_id'} }{'allocated_system'}             = $data->{DEVICE}{'Allocated_System'}; }
  if ( exists $data->{DEVICE}{'Failed'} )                             { $conf{ $cfg->{'hostcfg_id'} }{'failed'}                       = $data->{DEVICE}{'Failed'}; }
  if ( exists $data->{DEVICE}{'Release_version'} )                    { $conf{ $cfg->{'hostcfg_id'} }{'release_version'}              = $data->{DEVICE}{'Release_version'}; }
  if ( exists $data->{DEVICE}{'Allocated'} )                          { $conf{ $cfg->{'hostcfg_id'} }{'allocated'}                    = $data->{DEVICE}{'Allocated'}; }
  if ( exists $data->{DEVICE}{'Free'} )                               { $conf{ $cfg->{'hostcfg_id'} }{'free'}                         = $data->{DEVICE}{'Free'}; }
  if ( exists $data->{DEVICE}{'Total_Capacity'} )                     { $conf{ $cfg->{'hostcfg_id'} }{'total_capacity'}               = $data->{DEVICE}{'Total_Capacity'}; }
  if ( exists $data->{DEVICE}{'Storage_System_ID'} )                  { $conf{ $cfg->{'hostcfg_id'} }{'storage_system_id'}            = $data->{DEVICE}{'Storage_System_ID'}; }
  if ( exists $data->{DEVICE}{'Storage_System_Name'} )                { $conf{ $cfg->{'hostcfg_id'} }{'storage_system_name'}          = $data->{DEVICE}{'Storage_System_Name'}; }

  #
  # POOL
  #
  if ( exists $data->{POOL} ) {
    foreach ( sort keys %{ $data->{POOL} } ) {
      my $var = $data->{POOL}{$_};
      if ( exists $var->{CPG_ID} && exists $var->{CPG_name} ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."POOL".$var->{CPG_ID}) );
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => $var->{CPG_name},
          "hw_type"   => "3par",
          "subsystem" => "pool",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents"   => [$cfg->{'hostcfg_id'}],
        );

        # conf
        #
        if ( exists $var->{'CPG_name'} )                        { $conf{$id}{'cpg_name'}                       = $var->{'CPG_name'}; }
        if ( exists $var->{'CPG_ID'} )                          { $conf{$id}{'cpg_id'}                         = $var->{'CPG_ID'}; }
        if ( exists $var->{'Snapshot_Total_Capacity_[MB]'} )    { $conf{$id}{'snapshot_total_capacity'}        = $var->{'Snapshot_Total_Capacity_[MB]'}; }
        if ( exists $var->{'Admin_Used_Capacity_[MB]'} )        { $conf{$id}{'admin_used_capacity'}            = $var->{'Admin_Used_Capacity_[MB]'}; }
        if ( exists $var->{'Snapshot_Used_Capacity_[MB]'} )     { $conf{$id}{'snapshot_used_capacity'}         = $var->{'Snapshot_Used_Capacity_[MB]'}; }
        if ( exists $var->{'User_Used_Capacity_[MB]'} )         { $conf{$id}{'user_used_capacity'}             = $var->{'User_Used_Capacity_[MB]'}; }
        if ( exists $var->{'User_Total_Capacity_[MB]'} )        { $conf{$id}{'user_total_capacity'}            = $var->{'User_Total_Capacity_[MB]'}; }
        if ( exists $var->{'Admin_Total_Capacity_[MB]'} )       { $conf{$id}{'admin_total_capacity'}           = $var->{'Admin_Total_Capacity_[MB]'}; }

        push(@architecture, \%arch);
      }
    }
  }

  #
  # HOST
  #
  if ( exists $data->{HOST} ) {
    foreach ( sort keys %{ $data->{HOST} } ) {
      my $var = $data->{HOST}{$_};
      if ( exists $var->{name} && exists $var->{id} ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."HOST".$var->{name}) );
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => $var->{name},
          "hw_type"   => "3par",
          "subsystem" => "host",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents"   => [$cfg->{'hostcfg_id'}],
        );
        #print "HOST CFG";
        #print Dumper \%arch;
        # conf
        #
        if ( exists $var->{'name'} )                            { $conf{$id}{'name'}                       = $var->{'name'}; }
        if ( exists $var->{'id'} )                              { $conf{$id}{'id'}                         = $var->{'id'}; }
        if ( exists $var->{'IQN_WWPN'} )                        { $conf{$id}{'IQN_WWPN'}                   = $var->{'IQN_WWPN'}; }
        if ( exists $var->{'host_id'} )                         { $conf{$id}{'host_id'}                    = $var->{'host_id'}; }
        if ( exists $var->{'port_count'} )                      { $conf{$id}{'port_count'}                 = $var->{'port_count'}; }

        push(@architecture, \%arch);
      }
    }
  }

  #
  # PORT
  #
  if ( exists $data->{PORT} ) {
    foreach ( sort keys %{ $data->{PORT} } ) {
      my $var = $data->{PORT}{$_};
      if ( exists $var->{'N:S:P'} && exists $var->{Port_WWN} ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."PORT".$var->{'N:S:P'}) );
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => "$var->{'N:S:P'} $var->{'Protocol'} $var->{'Label'}",
          "hw_type"   => "3par",
          "subsystem" => "port",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents"   => [$cfg->{'hostcfg_id'}],
        );

        # conf
        #
        if ( exists $var->{'Type'} )                            { $conf{$id}{'type'}                    = $var->{'Type'}; }
        if ( exists $var->{'Label'} )                           { $conf{$id}{'label'}                   = $var->{'Label'}; }
        if ( exists $var->{'N\:S\:P'} )                         { $conf{$id}{'n:s:p'}                   = $var->{'N\:S\:P'}; }
        if ( exists $var->{'Failover_State'} )                  { $conf{$id}{'failover_state'}          = $var->{'Failover_State'}; }
        if ( exists $var->{'Partner'} )                         { $conf{$id}{'partner'}                 = $var->{'Partner'}; }
        if ( exists $var->{'Mode'} )                            { $conf{$id}{'mode'}                    = $var->{'Mode'}; }
        if ( exists $var->{'Protocol'} )                        { $conf{$id}{'protocol'}                = $var->{'Protocol'}; }
        if ( exists $var->{'State'} )                           { $conf{$id}{'state'}                   = $var->{'State'}; }
        if ( exists $var->{'Port_WWN'} )                        { $conf{$id}{'port_wwn'}                = $var->{'Port_WWN'}; }
        if ( exists $var->{'Node_WWN'} )                        { $conf{$id}{'node_wwn'}                = $var->{'Node_WWN'}; }

        push(@architecture, \%arch);
      }
    }
  }

  #
  # DRIVE
  #
  if ( exists $data->{DRIVE} ) {
    foreach ( sort keys %{ $data->{DRIVE} } ) {
      my $var = $data->{DRIVE}{$_};
      if ( exists $var->{Physical_Disk_ID} ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."DRIVE".$var->{Physical_Disk_ID}) );
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => $var->{Cage_Position},
          "hw_type"   => "3par",
          "subsystem" => "drive",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents"   => [$cfg->{'hostcfg_id'}],
        );

        # conf
        #
        if ( exists $var->{'Port_B'} )                          { $conf{$id}{'port_b'}                = $var->{'Port_B'}; }
        if ( exists $var->{'Port_A'} )                          { $conf{$id}{'port_a'}                = $var->{'Port_A'}; }
        if ( exists $var->{'Physical_Disk_ID'} )                { $conf{$id}{'physical_disk_id'}      = $var->{'Physical_Disk_ID'}; }
        if ( exists $var->{'Type'} )                            { $conf{$id}{'type'}                  = $var->{'Type'}; }
        if ( exists $var->{'Total_Size_[MB]'} )                 { $conf{$id}{'total_size_[MB]'}       = $var->{'Total_Size_[MB]'}; }
        if ( exists $var->{'Cage_Position'} )                   { $conf{$id}{'cage_position'}         = $var->{'Cage_Position'}; }
        if ( exists $var->{'Capacity_[GB]'} )                   { $conf{$id}{'capacity_[GB]'}         = $var->{'Capacity_[GB]'}; }
        if ( exists $var->{'RPM'} )                             { $conf{$id}{'rpm'}                   = $var->{'RPM'}; }
        if ( exists $var->{'State'} )                           { $conf{$id}{'state'}                 = $var->{'State'}; }
        if ( exists $var->{'Free_Size_[MB]'} )                  { $conf{$id}{'free_size_[MB]'}        = $var->{'Free_size_[MB]'}; }

        push(@architecture, \%arch);
      }
    }
  }

  #
  # VOLUME
  #
  if ( exists $data->{VOLUME} ) {
    foreach ( sort keys %{ $data->{VOLUME} } ) {
      my $var = $data->{VOLUME}{$_};
      if ( exists $var->{'Volume_ID'} ) {
        my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."VOLUME".$var->{'Volume_ID'}) );
        # Architecture
        my %arch = (
          "item_id"   => $id,
          "label"     => $var->{'Volume_name'},
          "hw_type"   => "3par",
          "subsystem" => "volume",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents"   => [$cfg->{'hostcfg_id'}],
        );

        # conf
        #
        if ( exists $var->{'Type'} )                            { $conf{$id}{'type'}                  = $var->{'Type'}; }
        if ( exists $var->{'Partner'} )                         { $conf{$id}{'partner'}               = $var->{'Partner'}; }
        if ( exists $var->{'Label'} )                           { $conf{$id}{'label'}                 = $var->{'Label'}; }
        if ( exists $var->{'N:S:P'} )                           { $conf{$id}{'n:s:p'}                 = $var->{'N:S:P'}; }
        if ( exists $var->{'Volume_IDs'} )                      { $conf{$id}{'volume_ids'}            = $var->{'Volume_IDs'}; }
        if ( exists $var->{'IQN_WWPN'} )                        { $conf{$id}{'iqn_wwpn'}              = $var->{'IQN_WWPN'}; }
        if ( exists $var->{'name'} )                            { $conf{$id}{'name'}                  = $var->{'name'}; }
        if ( exists $var->{'status'} )                          { $conf{$id}{'status'}                = $var->{'status'}; }

        push(@architecture, \%arch);
      }
    }
  }

  #save configuration
  $xormon->saveConfJSON(\%conf);

  #save architecture
  #print "SAVE ARCHITECTURE\n";
  #print Dumper \@architecture;
  $xormon->saveArchitecture(\@architecture);

  # save translator
  #Xormon::save_as_json("$wrkdir/translator.txt",$translator);

  #print "#################### Architecture\n";
  #print Dumper @architecture;
  #print "#################### CONF\n";
  #print Dumper \%conf;

}
sub prepare_perf{
  my $data_in = shift;
  #print Dumper $data_in;
  my %data_out;

  #Create Architecture from non-confile data (perf)
  my @architecture;
  my %conf;

  foreach my $timestamp ( sort { $a <=> $b } keys %{ $data_in } ) {
    #
    # VOLUME && POOL
    #
    if ( exists $data_in->{$timestamp}{VOLUME} ) {
      my @pool_ids = ();
      foreach my $line_idx ( sort { $a <=> $b } keys %{ $data_in->{$timestamp}{VOLUME} } ) {
        my $var = $data_in->{$timestamp}{VOLUME}{$line_idx};
        if ( exists $var->{'Volume ID'} ) {
          #print "VOLUME : $var->{'Volume ID'}\n";
          my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."VOLUME".$var->{'Volume ID'}) );
          #print "VOLUME ID: $id\n";
          if ( exists $var->{'Read IO Rate (IO/s)'} )        { $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io}        = Xormon::formatNumberByType("float4",$var->{'Read IO Rate (IO/s)'}); }
          if ( exists $var->{'Write IO Rate (IO/s)'} )       { $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io}       = Xormon::formatNumberByType("float4",$var->{'Write IO Rate (IO/s)'}); }
          if ( exists $var->{'Total IO Rate (IO/s)'} )       { $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate}        = Xormon::formatNumberByType("float4",$var->{'Total IO Rate (IO/s)'}); }
          if ( exists $var->{'Read Data Rate (KB/s)'} )      { $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read}           = Xormon::formatNumberByType("float4",$var->{'Read Data Rate (KB/s)'}); }
          if ( exists $var->{'Write Data Rate (KB/s)'} )     { $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write}          = Xormon::formatNumberByType("float4",$var->{'Write Data Rate (KB/s)'}); }
          if ( exists $var->{'Total Data Rate (KB/s)'} )     { $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{data_rate}      = Xormon::formatNumberByType("float4",$var->{'Total Data Rate (KB/s)'}); }
          if ( exists $var->{'Read Response Time (ms)'} )    { $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_r}       = Xormon::formatNumberByType("float4",$var->{'Read Response Time (ms)'}); }
          if ( exists $var->{'Write Response Time (ms)'} )   { $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_w}       = Xormon::formatNumberByType("float4",$var->{'Write Response Time (ms)'}); }
          if ( exists $var->{'Overall Response Time (ms)'} ) { $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t}         = Xormon::formatNumberByType("float4",$var->{'Overall Response Time (ms)'}); }

          # calculate resp_t
          if ( exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_r} && exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io} ) {
            if ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io} != 0 ) {
              my $val = ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_r} / $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io});
              $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_r} = $val;
            }
          } ## end if ( isdigit($resp_t_r...))

          if ( exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_w} && exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io} ) {
            if ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io} != 0 ) {
              my $val = ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_w} / $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io});
              $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_w} = $val;
            }
          } ## end if ( isdigit($resp_t_w...))

          if ( exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t} && exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate} ) {
            if ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate} != 0 ) {
              my $val = ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t} / $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate});
              $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t} = $val;
            }
          } ## end if ( isdigit($resp_t) ...)

          # calculate block size
          if ( exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate} && $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate} != 0 && exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{data_rate} ) {
            $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{block_size} = Xormon::formatNumberByType("float4", ($data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{data_rate} / $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate}));
          }
          if ( exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io} && $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io} != 0 && exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read} ) {
            $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{block_size_r} = Xormon::formatNumberByType("float4", ($data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read} / $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io}));
          }
          if ( exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io} && $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io} != 0 && exists $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write} ) {
            $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{block_size_w} = Xormon::formatNumberByType("float4", ($data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write} / $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io}));
          }

          #
          # POOL
          #
          if ( exists $var->{'CPG ID'} && $var->{'CPG ID'} ne "-" ) {
            my $pool_id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."POOL".$var->{'CPG ID'}) );
           push(@pool_ids, $pool_id);
            #Read IO
            if (defined $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read_io} && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read_io} ne ""){
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read_io}        += $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io};
            }
            else{
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read_io}        = $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io};
            }
            #Write IO
            if (defined $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write_io} && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write_io} ne ""){
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write_io}       += $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io};
            }
            else{
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write_io}       = $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io};
            }
            #Total IO
            if (defined $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{io_rate} && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{io_rate} ne ""){
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{io_rate}        += $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate};
            }
            else{
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{io_rate}        = $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate};
            }
            #Read Data
            if (defined $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read} && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read} ne ""){
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read}           += $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read};
            }
            else{
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read}           = $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read};
            }
            #Write Data
            if (defined $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write} && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write} ne ""){
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write}          += $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write};
            }
            else{
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write}          = $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write};
            }
            #Total Data
            if (defined $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{data_rate}  && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{data_rate}  ne ""){
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{data_rate}      += $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{data_rate};
            }
            else{
              $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{data_rate}      = $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{data_rate};
            }
            #Resp read
            if (Xormon::isdigit($data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_r}) && Xormon::isdigit($data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io}) ){
              if (defined $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_r} && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_r}  ne ""){
                $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_r}       += ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_r} * $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io} );
              }
              else{
                $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_r}       =  ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_r} * $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{read_io} );
              }
            }
            #Resp write
            if (Xormon::isdigit($data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_w}) && Xormon::isdigit($data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io}) ){
              if (defined $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_w} && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_w} ne "" ){
                $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_w}       += ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_w} * $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io} );
              }
              else{
                $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_w}       = ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t_w} * $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{write_io} );
              }
            }
            #Resp total
            if (Xormon::isdigit($data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t}) && Xormon::isdigit($data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate}) ){
              if (defined $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t} && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t} ne "" ){
                $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t}       += ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t} * $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate} );
              }
              else{
                $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t}       = ( $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{resp_t} * $data_out{'3PAR'}{'VOLUME'}{$id}{$timestamp}{io_rate} );
              }
            }
          }

        }
      }
      foreach my $pool_id (@pool_ids){
        if (Xormon::isdigit($data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_r}) && Xormon::isdigit($data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read_io}) && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read_io} != 0){
          $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_r} = Xormon::formatNumberByType("float4",( $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_r} / $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{read_io}));
        }
        if (Xormon::isdigit($data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_w}) && Xormon::isdigit($data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write_io}) && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write_io} != 0 ){
          $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_w} = Xormon::formatNumberByType("float4",( $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t_w} / $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{write_io}));
        }
        if (Xormon::isdigit($data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t}) && Xormon::isdigit($data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{io_rate}) && $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{io_rate} !=0 ){
          $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t} = Xormon::formatNumberByType("float4",( $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{resp_t} / $data_out{'3PAR'}{'POOL'}{$pool_id}{$timestamp}{io_rate} ));
        }
      }
    }


    #
    # DRIVE && TIER
    #
    if ( exists $data_in->{$timestamp}{DRIVE} ) {
      my @tier_ids = ();
      foreach my $line_idx ( sort { $a <=> $b } keys %{ $data_in->{$timestamp}{DRIVE} } ) {
        my $var = $data_in->{$timestamp}{DRIVE}{$line_idx};
        if ( exists $var->{'Physical Disk ID'} ) {
          my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."DRIVE".$var->{'Physical Disk ID'}) );

          if ( exists $var->{'Read IO Rate (IO/s)'})        { $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{read_io}        = Xormon::formatNumberByType("float4",$var->{'Read IO Rate (IO/s)'} ); }
          if ( exists $var->{'Write IO Rate (IO/s)'})       { $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{write_io}       = Xormon::formatNumberByType("float4",$var->{'Write IO Rate (IO/s)'} ); }
          if ( exists $var->{'Total IO Rate (IO/s)'})       { $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{io_rate}        = Xormon::formatNumberByType("float4",$var->{'Total IO Rate (IO/s)'} ); }
          if ( exists $var->{'Read Data Rate (KB/s)'})      { $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{read}           = Xormon::formatNumberByType("float4",$var->{'Read Data Rate (KB/s)'} * 1000 ); }
          if ( exists $var->{'Write Data Rate (KB/s)'})     { $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{write}          = Xormon::formatNumberByType("float4",$var->{'Write Data Rate (KB/s)'} * 1000 ); }
          if ( exists $var->{'Total Data Rate (KB/s)'})     { $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{data_rate}      = Xormon::formatNumberByType("float4",$var->{'Total Data Rate (KB/s)'} * 1000 ); }
          if ( exists $var->{'Read Response Time (ms)'})    { $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t_r}       = Xormon::formatNumberByType("float4",$var->{'Read Response Time (ms)'} ); }
          if ( exists $var->{'Write Response Time (ms)'})   { $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t_w}       = Xormon::formatNumberByType("float4",$var->{'Write Response Time (ms)'} ); }
          if ( exists $var->{'Overall Response Time (ms)'}) { $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t}         = Xormon::formatNumberByType("float4",$var->{'Overall Response Time (ms)'} ); }

          #
          # TIER
          #
          if ( exists $var->{'Type'} && $var->{'Type'} ne "-" && $var->{'Type'} ne "" ) {
            my $tier_id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."TIER".$var->{'Type'}) );
            #my $tier_id = $var->{'Type'};
            if ( !grep( /^$tier_id$/, @tier_ids ) ) {
              push(@tier_ids, $tier_id);
            }

            #Type
            $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{type} = $var->{'Type'};
            #Read IO
            if (defined $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read_io} && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read_io} ne ""){
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read_io}        += $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{read_io};
            }
            else{
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read_io}        = $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{read_io};
            }
            #Write IO
            if (defined $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write_io} && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write_io} ne ""){
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write_io}       += $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{write_io};
            }
            else{
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write_io}       = $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{write_io};
            }
            #Total IO
            if (defined $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{io_rate} && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{io_rate} ne ""){
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{io_rate}        += $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{io_rate};
            }
            else{
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{io_rate}        = $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{io_rate};
            }
            #Read Data
            if (defined $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read} && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read} ne ""){
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read}           += $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{read};
            }
            else{
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read}           = $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{read};
            }
            #Write Data
            if (defined $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write} && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write} ne ""){
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write}          += $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{write};
            }
            else{
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write}          = $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{write};
            }
            #Total Data
            if (defined $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{data_rate}  && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{data_rate}  ne ""){
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{data_rate}      += $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{data_rate};
            }
            else{
              $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{data_rate}      = $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{data_rate};
            }
            #Resp read
            if (Xormon::isdigit($data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t_r}) && Xormon::isdigit($data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{read_io}) ){
              if (defined $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_r} && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_r}  ne ""){
                $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_r}       += ( $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t_r} * $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{read_io});
              }
              else{
                $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_r}       =  ( $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t_r} * $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{read_io});
              }
            }
            #Resp write
            if (Xormon::isdigit($data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t_w}) && Xormon::isdigit($data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{write_io}) ){
              if (defined $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_w} && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_w} ne "" ){
                $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_w}       += ( $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t_w} * $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{write_io});
              }
              else{
                $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_w}       = ( $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t_w} * $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{write_io});
              }
            }
            #Resp total
            if (Xormon::isdigit($data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t}) && Xormon::isdigit($data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{io_rate}) ){
              if (defined $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t} && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t} ne "" ){
                $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t}       += ( $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t} * $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{io_rate});
              }
              else{
                $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t}       = ( $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{resp_t} * $data_out{'3PAR'}{'DRIVE'}{$id}{$timestamp}{io_rate});
              }
            }
          }
        }
      }
      foreach my $tier_id (@tier_ids){
        ###CONF ARCHITECTURE
        my %arch = (
           "item_id"   => $tier_id,
           "label"     => $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{type},
           "hw_type"   => "3par",
           "subsystem" => "tier",
           "hostcfg_id" => "$cfg->{'hostcfg_id'}",
           "parents"   => [$cfg->{'hostcfg_id'}],
        );
        push(@architecture, \%arch);
        ####END CONF ARCHITECTURE
        if (Xormon::isdigit($data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_r}) && Xormon::isdigit($data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read_io}) && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read_io} != 0){
          $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_r} = Xormon::formatNumberByType("float4", ($data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_r} / $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{read_io}));
        }
        if (Xormon::isdigit($data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_w}) && Xormon::isdigit($data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write_io}) && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write_io} != 0 ){
          $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_w} = Xormon::formatNumberByType("float4", ($data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t_w} / $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{write_io}));
        }
        if (Xormon::isdigit($data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t}) && Xormon::isdigit($data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{io_rate}) && $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{io_rate} !=0 ){
          $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t} = Xormon::formatNumberByType("float4", ($data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{resp_t} / $data_out{'3PAR'}{'TIER'}{$tier_id}{$timestamp}{io_rate}));
        }
      }
    }

    #
    #HOST
    #
    if ( exists $data_in->{$timestamp}{HOST} ) {
      my @vvol_ids = ();
      foreach my $line_idx ( sort { $a <=> $b } keys %{ $data_in->{$timestamp}{HOST} } ) {
        my $var = $data_in->{$timestamp}{HOST}{$line_idx};
        if ( exists $var->{'Host Name'} ) {
          my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."HOST".$var->{'Host Name'}) );
          #print "HOST ID : $id\n";
          if (exists $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read_io} && $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t} && $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{data_rate} ){
            if ( exists $var->{'Read IO Rate (IO/s)'})        { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read_io}        += Xormon::formatNumberByType("float4",$var->{'Read IO Rate (IO/s)'}); }
            if ( exists $var->{'Write IO Rate (IO/s)'})       { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write_io}       += Xormon::formatNumberByType("float4",$var->{'Write IO Rate (IO/s)'}); }
            if ( exists $var->{'Total IO Rate (IO/s)'})       { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{io_rate}        += Xormon::formatNumberByType("float4",$var->{'Total IO Rate (IO/s)'}); }
            if ( exists $var->{'Read Data Rate (KB/s)'})      { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read}           += Xormon::formatNumberByType("float4",($var->{'Read Data Rate (KB/s)'} * 1000)); }
            if ( exists $var->{'Write Data Rate (KB/s)'})     { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write}          += Xormon::formatNumberByType("float4",($var->{'Write Data Rate (KB/s)'} * 1000)); }
            if ( exists $var->{'Total Data Rate (KB/s)'})     { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{data_rate}      += Xormon::formatNumberByType("float4",($var->{'Total Data Rate (KB/s)'} * 1000)); }
            if ( exists $var->{'Read Response Time (ms)'})    { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_r}       += Xormon::formatNumberByType("float4",$var->{'Read Response Time (ms)'}); }
            if ( exists $var->{'Write Response Time (ms)'})   { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_w}       += Xormon::formatNumberByType("float4",$var->{'Write Response Time (ms)'}); }
            if ( exists $var->{'Overall Response Time (ms)'}) { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t}         += Xormon::formatNumberByType("float4",$var->{'Overall Response Time (ms)'}); }
          }
          else{
            if ( exists $var->{'Read IO Rate (IO/s)'})        { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read_io}        = Xormon::formatNumberByType("float4",$var->{'Read IO Rate (IO/s)'}); }
            if ( exists $var->{'Write IO Rate (IO/s)'})       { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write_io}       = Xormon::formatNumberByType("float4",$var->{'Write IO Rate (IO/s)'}); }
            if ( exists $var->{'Total IO Rate (IO/s)'})       { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{io_rate}        = Xormon::formatNumberByType("float4",$var->{'Total IO Rate (IO/s)'}); }
            if ( exists $var->{'Read Data Rate (KB/s)'})      { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read}           = Xormon::formatNumberByType("float4",($var->{'Read Data Rate (KB/s)'} * 1000)); }
            if ( exists $var->{'Write Data Rate (KB/s)'})     { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write}          = Xormon::formatNumberByType("float4",($var->{'Write Data Rate (KB/s)'} * 1000)); }
            if ( exists $var->{'Total Data Rate (KB/s)'})     { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{data_rate}      = Xormon::formatNumberByType("float4",($var->{'Total Data Rate (KB/s)'} * 1000)); }
            if ( exists $var->{'Read Response Time (ms)'})    { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_r}       = Xormon::formatNumberByType("float4",$var->{'Read Response Time (ms)'}); }
            if ( exists $var->{'Write Response Time (ms)'})   { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_w}       = Xormon::formatNumberByType("float4",$var->{'Write Response Time (ms)'}); }
            if ( exists $var->{'Overall Response Time (ms)'}) { $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t}         = Xormon::formatNumberByType("float4",$var->{'Overall Response Time (ms)'}); }
          }

          #
          # VVOL
          #
          if ( exists $var->{'Volume (Vdisk) Name'} && $var->{'Volume (Vdisk) Name'} ne "-" ) {
            my $vvol_id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."VVOL".$var->{'Volume (Vdisk) Name'}) );
            if ( !grep( /^$vvol_id$/, @vvol_ids ) ) {
              push(@vvol_ids, $vvol_id);
            }

            $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{label} = $var->{'Volume (Vdisk) Name'};
            #Read IO
            if (defined $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read_io} && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read_io} ne ""){
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read_io}        += $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read_io};
            }
            else{
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read_io}        = $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read_io};
            }
            #Write IO
            if (defined $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write_io} && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write_io} ne ""){
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write_io}       += $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write_io};
            }
            else{
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write_io}       = $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write_io};
            }
            #Total IO
            if (defined $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{io_rate} && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{io_rate} ne ""){
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{io_rate}        += $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{io_rate};
            }
            else{
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{io_rate}        = $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{io_rate};
            }
            #Read Data
            if (defined $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read} && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read} ne ""){
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read}           += $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read};
            }
            else{
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read}           = $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read};
            }
            #Write Data
            if (defined $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write} && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write} ne ""){
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write}          += $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write};
            }
            else{
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write}          = $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write};
            }
            #Total Data
            if (defined $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{data_rate}  && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{data_rate}  ne ""){
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{data_rate}      += $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{data_rate};
            }
            else{
              $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{data_rate}      = $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{data_rate};
            }
            #Resp read
            if (Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_r}) && Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read_io}) ){
              if (defined $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_r} && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_r}  ne ""){
                $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_r}       += ( $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_r} * $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read_io} );
              }
              else{
                $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_r}       =  ( $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_r} * $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{read_io} );
              }
            }
            #Resp write
            if (Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_w}) && Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write_io}) ){
              if (defined $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_w} && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_w} ne "" ){
                $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_w}       += ( $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_w} * $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write_io} );
              }
              else{
                $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_w}       = ( $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t_w} * $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{write_io} );
              }
            }
            #Resp total
            if (Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t}) && Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{io_rate}) ){
              if (defined $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t} && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t} ne "" ){
                $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t}       += ( $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t} * $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{io_rate} );
              }
              else{
                $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t}       = ( $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{resp_t} * $data_out{'3PAR'}{'HOST'}{$id}{$timestamp}{io_rate} );
              }
            }
          }
        }
      }
      ###Host RESP_T
      foreach my $host_id (keys( %{$data_out{'3PAR'}{'HOST'}})){
        if (Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{resp_t_r}) && Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{read_io}) && $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{read_io} != 0){
          $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{resp_t_r} = Xormon::formatNumberByType("float4", $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{resp_t_r} / $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{read_io});
        }
        if (Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{resp_t_w}) && Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{write_io}) && $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{write_io} != 0 ){
          $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{resp_t_w} = Xormon::formatNumberByType("float4", $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{resp_t_w} / $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{write_io});
        }
        if (Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{resp_t}) && Xormon::isdigit($data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{io_rate}) && $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{io_rate} !=0 ){
          $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{resp_t} = Xormon::formatNumberByType("float4", $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{resp_t} / $data_out{'3PAR'}{'HOST'}{$host_id}{$timestamp}{io_rate})
        }
      }
      ###VVOL RESP_T
      foreach my $vvol_id (@vvol_ids){
        ###CONF ARCHITECTURE
        my %arch = (
          "item_id"   => $vvol_id,
          "label"     => $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{label},
          "hw_type"   => "3par",
          "subsystem" => "vvol",
          "hostcfg_id" => "$cfg->{'hostcfg_id'}",
          "parents"   => [$cfg->{'hostcfg_id'}],
        );
        push(@architecture, \%arch);
        ####END CONF ARCHITECTURE
        if (Xormon::isdigit($data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_r}) && Xormon::isdigit($data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read_io}) && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read_io} != 0){
          $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_r} = Xormon::formatNumberByType("float4", $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_r} / $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{read_io});
        }
        if (Xormon::isdigit($data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_w}) && Xormon::isdigit($data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write_io}) && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write_io} != 0 ){
          $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_w} = Xormon::formatNumberByType("float4", $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t_w} / $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{write_io});
        }
        if (Xormon::isdigit($data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t}) && Xormon::isdigit($data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{io_rate}) && $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{io_rate} !=0 ){
          $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t} = Xormon::formatNumberByType("float4", $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{resp_t} / $data_out{'3PAR'}{'VVOL'}{$vvol_id}{$timestamp}{io_rate})
        }
      }
    }
    #print "HOST DATA OUT\n";
    #print Dumper $data_out{'3PAR'}{'HOST'};
    #print "VVOLS DATA OUT\n";
    #print Dumper $data_out{'3PAR'}{'VVOL'};

    #
    #PORT
    #
    if ( exists $data_in->{$timestamp}{PORT} ) {
      foreach my $line_idx ( sort { $a <=> $b } keys %{ $data_in->{$timestamp}{PORT} } ) {
        my $var = $data_in->{$timestamp}{PORT}{$line_idx};
        if ( exists $var->{'N:S:P'} ) {
          my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."PORT".$var->{'N:S:P'}) );
          if ( exists $var->{'Read IO Rate (IO/s)'})        { $data_out{'3PAR'}{'PORT'}{$id}{$timestamp}{read_io}        = Xormon::formatNumberByType("float4",$var->{'Read IO Rate (IO/s)'}); }
          if ( exists $var->{'Write IO Rate (IO/s)'})       { $data_out{'3PAR'}{'PORT'}{$id}{$timestamp}{write_io}       = Xormon::formatNumberByType("float4",$var->{'Write IO Rate (IO/s)'}); }
          if ( exists $var->{'Total IO Rate (IO/s)'})       { $data_out{'3PAR'}{'PORT'}{$id}{$timestamp}{io_rate}        = Xormon::formatNumberByType("float4",$var->{'Total IO Rate (IO/s)'}); }
          if ( exists $var->{'Read Data Rate (KB/s)'})      { $data_out{'3PAR'}{'PORT'}{$id}{$timestamp}{read}           = Xormon::formatNumberByType("float4",$var->{'Read Data Rate (KB/s)'}); }
          if ( exists $var->{'Write Data Rate (KB/s)'})     { $data_out{'3PAR'}{'PORT'}{$id}{$timestamp}{write}          = Xormon::formatNumberByType("float4",$var->{'Write Data Rate (KB/s)'}); }
          if ( exists $var->{'Total Data Rate (KB/s)'})     { $data_out{'3PAR'}{'PORT'}{$id}{$timestamp}{data_rate}      = Xormon::formatNumberByType("float4",$var->{'Total Data Rate (KB/s)'}); }
          if ( exists $var->{'Read Response Time (ms)'})    { $data_out{'3PAR'}{'PORT'}{$id}{$timestamp}{resp_t_r}       = Xormon::formatNumberByType("float4",$var->{'Read Response Time (ms)'}); }
          if ( exists $var->{'Write Response Time (ms)'})   { $data_out{'3PAR'}{'PORT'}{$id}{$timestamp}{resp_t_w}       = Xormon::formatNumberByType("float4",$var->{'Write Response Time (ms)'}); }
          if ( exists $var->{'Overall Response Time (ms)'}) { $data_out{'3PAR'}{'PORT'}{$id}{$timestamp}{resp_t}         = Xormon::formatNumberByType("float4",$var->{'Overall Response Time (ms)'}); }
        }
      }
    }

    #
    #NODE (cache)
    #
    if ( exists $data_in->{$timestamp}{NODE} ) {
      foreach my $line_idx ( sort { $a <=> $b } keys %{ $data_in->{$timestamp}{NODE} } ) {
        my $var = $data_in->{$timestamp}{NODE}{$line_idx};
        if ( exists $var->{'Node'} ) {
          my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."CACHE".$var->{'Node'}) );
          my %arch = (
              "item_id"   => $id,
              "label"     => "node" . "$var->{'Node'}",
              "hw_type"   => "3par",
              "subsystem" => "cache",
              "hostcfg_id" => "$cfg->{'hostcfg_id'}",
              "parents"   => [$cfg->{'hostcfg_id'}],
            );
            #print Dumper \$var;
            #print Dumper \%arch;
            push(@architecture, \%arch);

          if ( exists $var->{'Total Write IO Hits Percent'}) { $data_out{'3PAR'}{'CACHE'}{$id}{$timestamp}{w_cache_hit}    = Xormon::formatNumberByType("float4",$var->{'Total Write IO Hits Percent'}); }
          if ( exists $var->{'Total Read IO Hits Percent'})  { $data_out{'3PAR'}{'CACHE'}{$id}{$timestamp}{r_cache_hit}    = Xormon::formatNumberByType("float4",$var->{'Total Read IO Hits Percent'}); }
          if ( exists $var->{'CfcDirty'}) { $data_out{'3PAR'}{'CACHE'}{$id}{$timestamp}{cfc_dirty}    = Xormon::formatNumberByType("float4",$var->{'CfcDirty'}); }
          if ( exists $var->{'DelAck'})    { $data_out{'3PAR'}{'CACHE'}{$id}{$timestamp}{del_ack}      = Xormon::formatNumberByType("float4",$var->{'DelAck'}); }

        }
      }
    }


    #
    #CPU CORE
    #
    if ( exists $data_in->{$timestamp}{'CPU-CORE'} ) {
      foreach my $line_idx ( sort { $a <=> $b } keys %{ $data_in->{$timestamp}{'CPU-CORE'} } ) {
        my $var = $data_in->{$timestamp}{'CPU-CORE'}{$line_idx};
        if ( exists $var->{'Node-CPU Core'} ) {
          my $id = lc( Xormon::create_hash_key($cfg->{'hostcfg_id'}."CPU-CORE".$var->{'Node-CPU Core'}) );
          my %arch = (
              "item_id"   => $id,
              "label"     => $var->{'Node-CPU Core'},
              "hw_type"   => "3par",
              "subsystem" => "cpu",
              "hostcfg_id" => "$cfg->{'hostcfg_id'}",
              "parents"   => [$cfg->{'hostcfg_id'}],
            );
            push(@architecture, \%arch);

          if ( exists $var->{'CPU Utilization - System'}) { $data_out{'3PAR'}{'CPU'}{$id}{$timestamp}{sys}    = Xormon::formatNumberByType("float4",$var->{'CPU Utilization - System'}); }
          if ( exists $var->{'CPU Utilization - User'})  { $data_out{'3PAR'}{'CPU'}{$id}{$timestamp}{user}    = Xormon::formatNumberByType("float4",$var->{'CPU Utilization - User'}); }

        }
      }
    }

  }

  #save configuration
  $xormon->saveConfJSON(\%conf);

  #save architecture
  #print "ARCHITECTURE 1\n";
  #print Dumper \@architecture;
  $xormon->saveArchitecture(\@architecture);

  return \%data_out;

}
sub parse_perf {
  my $perf_file = shift;

  open( IN, "<$perf_file" ) || $logger->error( "Couldn't open file $perf_file $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <IN>;
  close(IN);

  my %data_in;
  my $section    = "";
  my $act_header = "";
  my $stat_time  = "";

  my $line_idx = 0;
  foreach my $line (@lines) {
    chomp $line;
    $line_idx++;



    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    if ( $line =~ m/^----------/ )              { next; }
    if ( $line =~ m/^Interval Length/ )          { next; }
    if ( $line =~ m/^$/ && $section ne "DEVICE" )        { $section = "NA";        next; }

    if ( $line =~ m/Pool Capacity Statistics/ )          { $section = "POOL-CAP";      next; }
    if ( $line =~ m/Volume Cache Level Statistics/ )     { $section = "VOLUME-CACHE";  next; }
    if ( $line =~ m/Physical Disk Level Statistics/ )    { $section = "DRIVE";         next; }
    if ( $line =~ m/Node Cache Level Statistics/ )       { $section = "NODE";          next; }
    if ( $line =~ m/Port Level Statistics/ )             { $section = "PORT";          next; }
    if ( $line =~ m/Volume Level Statistics/ )           { $section = "VOLUME";        next; }
    if ( $line =~ m/Drive Level Statistics/ )            { $section = "DRIVE";         next; }
    if ( $line =~ m/CPU-Node Level Statistics/ )         { $section = "CPU-NODE";      next; }
    if ( $line =~ m/CPU-Core Level Statistics/ )         { $section = "CPU-CORE";      next; }
    if ( $line =~ m/Host Level Statistics/ )             { $section = "HOST";          next; }

    if ( $line =~ m/^Node/ )                                                                        { $act_header = $line; next; }
    if ( $line =~ m/^Node,Time,Interval (s),CPU Utilization - User,CPU Utilization - System/ )      { $act_header = $line; next; }
    if ( $line =~ m/^Node-CPU Core,Time,Interval (s),Node,CPU Core/ )                               { $act_header = $line; next; }
    if ( $line =~ m/^host_id,Time,/ )                     { $act_header = $line; next; }
    if ( $line =~ m/^Physical Disk ID,Time,Interval/ )    { $act_header = $line; next; }
    if ( $line =~ m/^N\:S\:P,Time,Interval/ )             { $act_header = $line; next; }
    if ( $line =~ m/^Volume ID,Time,Interval/ )           { $act_header = $line; next; }


    if ( $line =~ m/Interval End:/ || $line =~ m/Interval Start:/ ) {
      $line =~ s/>//g;
      $line =~ s/Interval End:     //g;
      $line =~ s/GMT.*//g;
      my $line_conv = $line;    # temp fix when data is in wrong format
      if ( $line =~ m/_/ ) {
        $line =~ s/_/ /g;
        $line =~ s/Interval Start:   //g;
        $line =~ s/\(STOR2RRD Server time\)//g;
        $line_conv = "20" . substr( $line, 0, 2 ) . "-" . substr( $line, 2, 2 ) . "-" . substr( $line, 4, 2 ) . "T" . substr( $line, 7, 2 ) . ":" . substr( $line, 9, 2 ) . ":" . substr( $line, 11, 2 );
        #$line_conv = $line;
        #2015:01:27T03:30:03.000000 --> right one
        #Next Gen => 2015-01-27T03:30:03.000000 --> right one (str2time does not work with : in date ( NG ))
        #160128_052000
      } ## end if ( $line =~ m/_/ )
      ( my $trash1, my $trash2, my $date, my $time_only ) = split( / /, $line );
      $stat_time = str2time($line_conv);
      if ( !defined($stat_time) || Xormon::isdigit($stat_time) == 0 ) {
        $logger->info( "no correct time format: $line : $line_conv " . __FILE__ . ":" . __LINE__ );
      }
      #print "003 $line - $stat_time\n";
      next;
    } ## end if ( $line =~ m/Interval End:/...)

    if ( $section eq "HOST" || $section eq "PORT" || $section eq "NODE" || $section eq "VOLUME" || $section eq "DRIVE"|| $section eq "CPU-NODE" || $section eq "CPU-CORE" || $section eq "POOL-CAP" || $section eq "VOLUME-CACHE" ) {
      if ( defined $stat_time && $stat_time ne '' ) {
        $data_in{$stat_time}{$section}{$line_idx} = Xormon::parse_CSV_line_by_header(",",$act_header,$line);
      }
    }
  }

  #/print Dumper \%data_in;

  return \%data_in;
}
sub create_password_file {
  my $user = shift;
  my $pass = shift;
  my $host = shift;
  my $basedir = shift;
  if ( not -d "$basedir/etc" ) {
    mkdir("$basedir/etc") or die("ERROR: Cannot create directory '$basedir/etc'. Exitting.");
  }
  if ( not -d "$basedir/etc/$host" ) {
    mkdir("$basedir/etc/$host") or die("ERROR: Cannot create directory '$basedir/etc/$host'. Exitting.");
  }

  #print "basedir $basedir\n";
  my $result = `expect -d -f $basedir/script.exp $user $pass $host $basedir 2>&1`;
  return $result;
}
1;
